// Koka generated module: "std/text/regex", koka version: 2.2.1
#include "std_text_regex.h"
/*---------------------------------------------------------------------------
  Copyright 2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/

#pragma clang diagnostic ignored "-Wdisabled-macro-expansion"
#define PCRE2_CODE_UNIT_WIDTH 8
#include <pcre2.h>

/* -----------------------------------------------------------------------
  Init/Done
------------------------------------------------------------------------*/
#define KK_CUSTOM_INIT  kk_regex_custom_init
#define KK_CUSTOM_DONE  kk_regex_custom_done

static pcre2_general_context* gen_ctx;
static pcre2_compile_context* cmp_ctx;
static pcre2_match_context*   match_ctx; 

static void* kk_pcre2_malloc( PCRE2_SIZE size, void* data ) {
  return kk_malloc( kk_to_ssize_t(size), kk_get_context() );  
}
static void kk_pcre2_free( void* p, void* data ) {
  kk_free(p);
}

static void kk_regex_custom_init( kk_context_t* ctx ) {
  gen_ctx = pcre2_general_context_create( &kk_pcre2_malloc, &kk_pcre2_free, NULL );
  if (gen_ctx != NULL) {
    match_ctx = pcre2_match_context_create( gen_ctx );
    cmp_ctx = pcre2_compile_context_create( gen_ctx );
    if (cmp_ctx != NULL) {
      pcre2_set_newline( cmp_ctx, PCRE2_NEWLINE_ANYCRLF );
      pcre2_set_bsr( cmp_ctx, PCRE2_BSR_ANYCRLF );
    }
  }
}

static void kk_regex_custom_done( kk_context_t* ctx ) {
  if (cmp_ctx != NULL) {
    pcre2_compile_context_free(cmp_ctx);
    cmp_ctx = NULL;
  }
  if (match_ctx != NULL) {
    pcre2_match_context_free(match_ctx);
    match_ctx = NULL;
  }
  if (gen_ctx != NULL) {
    pcre2_general_context_free(gen_ctx);
    gen_ctx = NULL;
  }
}


/* -----------------------------------------------------------------------
  Compile
------------------------------------------------------------------------*/

static void kk_regex_free( void* pre, kk_block_t* b, kk_context_t* ctx ) {
  KK_UNUSED(ctx);
  pcre2_code* re = (pcre2_code*)pre;
  //kk_info_message( "free regex at %p\n", re );
  if (re != NULL) pcre2_code_free(re);
}

#define KK_REGEX_OPTIONS  (PCRE2_ALT_BSUX | PCRE2_EXTRA_ALT_BSUX | PCRE2_MATCH_UNSET_BACKREF /* javascript compat */ \
                          | PCRE2_NEVER_BACKSLASH_C | PCRE2_NEVER_UCP | PCRE2_UTF /* utf-8 safety */ \
                          )

static kk_box_t kk_regex_create( kk_string_t pat, bool ignore_case, bool multi_line, kk_context_t* ctx ) {
  kk_ssize_t len;
  const uint8_t* cpat = kk_string_buf_borrow( pat, &len );
  PCRE2_SIZE errofs = 0;
  int        errnum = 0;
  uint32_t   options = KK_REGEX_OPTIONS;
  if (ignore_case) options |= PCRE2_CASELESS;
  if (multi_line)  options |= PCRE2_MULTILINE;
  pcre2_code* re = pcre2_compile( cpat, PCRE2_ZERO_TERMINATED, options, &errnum, &errofs, cmp_ctx);
  //kk_info_message( "create regex: err:%i, at %p\n", (re==NULL ? 0 : errnum), re );
  kk_string_drop(pat,ctx);
  return kk_cptr_raw_box( &kk_regex_free, re, ctx );
}


/* -----------------------------------------------------------------------
  Match
------------------------------------------------------------------------*/

/*
static void kk_match_data_free( void* pmd, kk_block_t* b ) {
  kk_info_message( "free match_data\n" );
  pcre2_match_data* md = (pcre2_match_data*)pmd;
  if (md != NULL) pcre2_match_data_free(md);
}
*/

static kk_std_core__list kk_regex_exec_ex( pcre2_code* re, pcre2_match_data* match_data, 
                                           kk_string_t str_borrow, const uint8_t* cstr, kk_ssize_t len, bool allow_empty, 
                                           kk_ssize_t start, kk_ssize_t* mstart, kk_ssize_t* end, int* res, kk_context_t* ctx ) 
{
  // match
  kk_std_core__list hd  = kk_std_core__new_Nil(ctx);
  uint32_t options = 0;
  if (!allow_empty) options |= (PCRE2_NOTEMPTY_ATSTART | PCRE2_ANCHORED);
  int rc = pcre2_match( re, cstr, len, start, options, match_data, match_ctx );
  if (res != NULL) *res = rc;    
  if (rc > 0) {    
    // extract captures
    uint32_t    gcount = pcre2_get_ovector_count(match_data);
    PCRE2_SIZE* groups = pcre2_get_ovector_pointer(match_data);
    for( uint32_t i = gcount; i > 0; ) {
      i--;
      kk_ssize_t sstart = groups[i*2];       // on no-match, sstart and send == -1.
      kk_ssize_t send   = groups[i*2 + 1];
      kk_assert(send >= sstart);
      kk_std_core__sslice sslice = kk_std_core__new_Sslice( kk_string_dup(str_borrow), sstart, send - sstart, ctx ); 
      hd = kk_std_core__new_Cons(kk_reuse_null,kk_std_core__sslice_box(sslice,ctx), hd, ctx);
      if (i == 0) {
        if (mstart != NULL) { *mstart = sstart; }
        if (end    != NULL) { *end = send; }
      }
    }
  }
  return hd;
}


static kk_std_core__list kk_regex_exec( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_context_t* ctx ) 
{
  // unpack
  pcre2_match_data* match_data = NULL;
  kk_std_core__list res = kk_std_core__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox(bre);
  if (re == NULL) goto done;    
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;  
  kk_ssize_t len;
  const uint8_t* cstr = kk_string_buf_borrow(str, &len );  

  // and match
  res = kk_regex_exec_ex( re, match_data, str, cstr, len, true, start, NULL, NULL, NULL, ctx );

done:  
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

static kk_std_core__list kk_regex_exec_all( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* ctx ) {
  // unpack
  if (atmost < 0) atmost = KK_SSIZE_MAX;
  pcre2_match_data* match_data = NULL;
  kk_std_core__list res = kk_std_core__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox(bre);
  if (re == NULL) goto done;    
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;  
  kk_ssize_t len;
  const uint8_t* cstr = kk_string_buf_borrow(str, &len );  

  // and match
  kk_std_core__list* tail = NULL;
  bool allow_empty = true;
  int rc = 1;    
  kk_ssize_t next = start;
  while( rc > 0 && start < len && atmost > 0) {
    atmost--;
    rc = 0;
    kk_ssize_t mstart = start;
    kk_std_core__list cap = kk_regex_exec_ex( re, match_data, str, cstr, len, allow_empty, start, &mstart, &next, &rc, ctx );
    if (rc > 0) {
      // found a match; 
      // push string up to match, and the actual matched regex
      kk_std_core__sslice pre = kk_std_core__new_Sslice( kk_string_dup(str), start, mstart - start, ctx ); 
      kk_std_core__list   prelist = kk_std_core__new_Cons( kk_reuse_null, kk_std_core__sslice_box(pre,ctx), kk_std_core__new_Nil(ctx), ctx );
      kk_std_core__list   capcons = kk_std_core__new_Cons( kk_reuse_null, kk_std_core__list_box(cap,ctx), kk_std_core__new_Nil(ctx) /*tail*/, ctx );
      kk_std_core__list   cons = kk_std_core__new_Cons( kk_reuse_null, kk_std_core__list_box(prelist,ctx), capcons, ctx );
      if (tail==NULL) res = cons;
                 else *tail = cons;
      tail = &kk_std_core__as_Cons(capcons)->tail;
      allow_empty = (next > start);
      start = next;
    }
    else if (rc <= 0 && !allow_empty) {
      // skip one character and try again
      // todo: handle cr/lf pairs better?
      const uint8_t* p = kk_utf8_next( cstr + start );
      start = (p - cstr);
      allow_empty = true;
      rc = 1;
    }
  }
  
  // push final string part as well and end the list
  kk_std_core__sslice post    = kk_std_core__new_Sslice( kk_string_dup(str), next, len - next, ctx ); 
  kk_std_core__list   postlist= kk_std_core__new_Cons( kk_reuse_null, kk_std_core__sslice_box(post,ctx), kk_std_core__new_Nil(ctx), ctx );
  kk_std_core__list   cons    = kk_std_core__new_Cons( kk_reuse_null, kk_std_core__list_box(postlist,ctx), kk_std_core__new_Nil(ctx), ctx );
  if (tail==NULL) res = cons;
            else *tail = cons;  

done:  
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

/*
kk_std_core__sslice kk_slice_upto( struct kk_std_core_Sslice slice1, struct kk_std_core_Sslice slice2, kk_context_t* ctx ) {
  kk_ssize_t start = slice1.start;
  kk_ssize_t len   = (slice1.start <= slice2.start ? slice2.start - slice1.start : -1);
  return kk_std_core__new_Sslice(slice1.str, start, len, ctx);
}
*/
 
// monadic lift

kk_std_text_regex__regex kk_std_text_regex__mlift999_op(kk_std_text_regex__regex _this, kk_std_core_types__optional obj0, kk_string_t _c_985, kk_context_t* _ctx) { /* (regex, obj0 : optional<any>, string) -> regex */ 
  kk_box_t _x1242;
  if (kk_std_core_types__is_Optional(obj0)) {
    kk_box_t _box_x1056 = obj0._cons.Optional.value;
    kk_box_t _obj_104 = kk_box_unbox(_box_x1056, NULL);
    kk_box_dup(_obj_104);
    kk_std_core_types__optional_drop(obj0, _ctx);
    kk_Unit;
    kk_std_text_regex__regex_drop(_this, _ctx);
    kk_Unit;
    _x1242 = _obj_104; /*any*/
    goto _match1243;
  }
  kk_box_t _x = _this.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(_this, _ctx);
  kk_Unit;
  _x1242 = _x; /*any*/
  _match1243: ;
  return kk_std_text_regex__new_Regex(_x1242, _c_985, _ctx);
}


// lift anonymous function
struct kk_std_text_regex__copy_fun1245__t {
  struct kk_function_s _base;
  kk_std_text_regex__regex _this;
  kk_std_core_types__optional obj0;
};
static kk_std_text_regex__regex kk_std_text_regex__copy_fun1245(kk_function_t _fself, kk_string_t _c_985, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex__new_copy_fun1245(kk_std_text_regex__regex _this, kk_std_core_types__optional obj0, kk_context_t* _ctx) {
  struct kk_std_text_regex__copy_fun1245__t* _self = kk_function_alloc_as(struct kk_std_text_regex__copy_fun1245__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex__copy_fun1245, kk_context());
  _self->_this = _this;
  _self->obj0 = obj0;
  return &_self->_base;
}

static kk_std_text_regex__regex kk_std_text_regex__copy_fun1245(kk_function_t _fself, kk_string_t _c_985, kk_context_t* _ctx) {
  struct kk_std_text_regex__copy_fun1245__t* _self = kk_function_as(struct kk_std_text_regex__copy_fun1245__t*, _fself);
  kk_std_text_regex__regex _this = _self->_this; /* std/text/regex/regex */
  kk_std_core_types__optional obj0 = _self->obj0; /* optional<any> */
  kk_drop_match(_self, {kk_std_text_regex__regex_dup(_this);kk_std_core_types__optional_dup(obj0);}, {}, _ctx)
  kk_box_t _x1246;
  if (kk_std_core_types__is_Optional(obj0)) {
    kk_box_t _box_x1057 = obj0._cons.Optional.value;
    kk_box_t _obj_104 = kk_box_unbox(_box_x1057, NULL);
    kk_box_dup(_obj_104);
    kk_std_core_types__optional_drop(obj0, _ctx);
    kk_Unit;
    kk_std_text_regex__regex_drop(_this, _ctx);
    kk_Unit;
    _x1246 = _obj_104; /*any*/
    goto _match1247;
  }
  kk_box_t _x = _this.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(_this, _ctx);
  kk_Unit;
  _x1246 = _x; /*any*/
  _match1247: ;
  return kk_std_text_regex__new_Regex(_x1246, _c_985, _ctx);
}


// lift anonymous function
struct kk_std_text_regex__copy_fun1250__t {
  struct kk_function_s _base;
  kk_function_t _b_1062_1058;
};
static kk_box_t kk_std_text_regex__copy_fun1250(kk_function_t _fself, kk_box_t _b_1059, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex__new_copy_fun1250(kk_function_t _b_1062_1058, kk_context_t* _ctx) {
  struct kk_std_text_regex__copy_fun1250__t* _self = kk_function_alloc_as(struct kk_std_text_regex__copy_fun1250__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex__copy_fun1250, kk_context());
  _self->_b_1062_1058 = _b_1062_1058;
  return &_self->_base;
}

static kk_box_t kk_std_text_regex__copy_fun1250(kk_function_t _fself, kk_box_t _b_1059, kk_context_t* _ctx) {
  struct kk_std_text_regex__copy_fun1250__t* _self = kk_function_as(struct kk_std_text_regex__copy_fun1250__t*, _fself);
  kk_function_t _b_1062_1058 = _self->_b_1062_1058; /* (string) -> std/text/regex/regex */
  kk_drop_match(_self, {kk_function_dup(_b_1062_1058);}, {}, _ctx)
  kk_std_text_regex__regex _x1251;
  kk_string_t _x1252 = kk_string_unbox(_b_1059); /*string*/
  _x1251 = kk_function_call(kk_std_text_regex__regex, (kk_function_t, kk_string_t, kk_context_t*), _b_1062_1058, (_b_1062_1058, _x1252, _ctx)); /*std/text/regex/regex*/
  return kk_std_text_regex__regex_box(_x1251, _ctx);
}

kk_std_text_regex__regex kk_std_text_regex__copy(kk_std_text_regex__regex _this, kk_std_core_types__optional obj0, kk_std_core_types__optional src0, kk_context_t* _ctx) { /* (regex, obj : optional<any>, src : optional<string>) -> regex */ 
  if (kk_yielding(kk_context())) {
    kk_std_core_types__optional_drop(src0, _ctx);
    kk_function_t _b_1062_1058 = kk_std_text_regex__new_copy_fun1245(_this, obj0, _ctx); /*(string) -> std/text/regex/regex*/;
    kk_box_t _x1249 = kk_std_core_hnd_yield_extend(kk_std_text_regex__new_copy_fun1250(_b_1062_1058, _ctx), _ctx); /*3858*/
    return kk_std_text_regex__regex_unbox(_x1249, _ctx);
  }
  kk_box_t _x1253;
  if (kk_std_core_types__is_Optional(obj0)) {
    kk_box_t _box_x1060 = obj0._cons.Optional.value;
    kk_box_t _obj_1040 = kk_box_unbox(_box_x1060, NULL);
    kk_box_dup(_obj_1040);
    kk_std_core_types__optional_drop(obj0, _ctx);
    kk_Unit;
    _x1253 = _obj_1040; /*any*/
    goto _match1254;
  }
  kk_box_t _x1 = _this.obj;
  kk_box_dup(_x1);
  _x1253 = _x1; /*any*/
  _match1254: ;
  kk_string_t _x1256;
  if (kk_std_core_types__is_Optional(src0)) {
    kk_box_t _box_x1061 = src0._cons.Optional.value;
    kk_string_t _src_110 = kk_string_unbox(_box_x1061);
    kk_string_dup(_src_110);
    kk_std_core_types__optional_drop(src0, _ctx);
    kk_Unit;
    kk_std_text_regex__regex_drop(_this, _ctx);
    kk_Unit;
    _x1256 = _src_110; /*string*/
    goto _match1257;
  }
  kk_string_t _x0 = _this.src;
  kk_string_dup(_x0);
  kk_std_text_regex__regex_drop(_this, _ctx);
  kk_Unit;
  _x1256 = _x0; /*string*/
  _match1257: ;
  return kk_std_text_regex__new_Regex(_x1253, _x1256, _ctx);
}
 
// Return the full matched string of a capture group

kk_string_t kk_std_text_regex_captured(kk_std_core__list matched0, kk_context_t* _ctx) { /* (matched : list<sslice>) -> string */ 
  if (kk_std_core__is_Cons(matched0)) {
    struct kk_std_core_Cons* _con1259 = kk_std_core__as_Cons(matched0);
    kk_box_t _box_x1063 = _con1259->head;
    kk_std_core__list _pat3 = _con1259->tail;
    kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1063, NULL);
    if (kk_likely(kk_std_core__list_is_unique(matched0))) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(s);
      kk_box_drop(_box_x1063, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(matched0);
    }
    else {
      kk_std_core__sslice_dup(s);
      kk_Unit;
      kk_std_core__list_decref(matched0, _ctx);
      kk_Unit;
    }
    return kk_std_core_string_3(s, _ctx);
  }
  return kk_string_empty();
}

kk_std_core__list kk_std_text_regex_regex_exec(kk_box_t regex0, kk_string_t str, kk_ssize_t start, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t) -> list<sslice> */ 
  return kk_regex_exec(regex0,str,start,kk_context());
}

kk_std_core__list kk_std_text_regex_regex_exec_all(kk_box_t regex0, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t, atmost : ssize_t) -> list<list<sslice>> */ 
  return kk_regex_exec_all(regex0,str,start,atmost,kk_context());
}

kk_box_t kk_std_text_regex_regex_create(kk_string_t _arg1, bool _arg2, bool _arg3, kk_context_t* _ctx) { /* (string, bool, bool) -> any */ 
  return kk_regex_create(_arg1,_arg2,_arg3,kk_context());
}
extern kk_string_t kk_std_text_regex_captures_fun1265(kk_function_t _fself, kk_std_core__list _x11264, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_text_regex_captured(_x11264, _ctx);
}
extern kk_box_t kk_std_text_regex_captures_fun1266(kk_function_t _fself, kk_box_t _b_1066, kk_context_t* _ctx) {
  struct kk_std_text_regex_captures_fun1266__t* _self = kk_function_as(struct kk_std_text_regex_captures_fun1266__t*, _fself);
  kk_function_t _b_1068_1065 = _self->_b_1068_1065; /* (matched : list<sslice>) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_1068_1065);}, {}, _ctx)
  kk_string_t _x1267;
  kk_std_core__list _x1268 = kk_std_core__list_unbox(_b_1066, _ctx); /*list<sslice>*/
  _x1267 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _b_1068_1065, (_b_1068_1065, _x1268, _ctx)); /*string*/
  return kk_string_box(_x1267);
}
 
// monadic lift

kk_string_t kk_std_text_regex__mlift1000_concat_replace(kk_std_core__list acc, kk_std_core__list mm, kk_std_core__list pre, kk_function_t repl, kk_string_t _y_986, kk_context_t* _ctx) { /* forall<e> (acc : list<string>, mm : list<list<sslice>>, pre : list<sslice>, repl : (list<sslice>) -> e string, string) -> e string */ 
  kk_std_core__list _x1269;
  kk_string_t _b_1074_1072 = _y_986; /*string*/;
  kk_std_core__list _b_1075_1073;
  kk_string_t _b_1076_1070;
  if (kk_std_core__is_Cons(pre)) {
    struct kk_std_core_Cons* _con1271 = kk_std_core__as_Cons(pre);
    kk_box_t _box_x1069 = _con1271->head;
    kk_std_core__list _pat3 = _con1271->tail;
    kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1069, NULL);
    if (kk_likely(kk_std_core__list_is_unique(pre))) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(s);
      kk_box_drop(_box_x1069, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(pre);
    }
    else {
      kk_std_core__sslice_dup(s);
      kk_Unit;
      kk_std_core__list_decref(pre, _ctx);
      kk_Unit;
    }
    _b_1076_1070 = kk_std_core_string_3(s, _ctx); /*string*/
    goto _match1270;
  }
  _b_1076_1070 = kk_string_empty(); /*string*/
  _match1270: ;
  kk_std_core__list _b_1077_1071 = acc; /*list<string>*/;
  _b_1075_1073 = kk_std_core__new_Cons(kk_reuse_null, kk_string_box(_b_1076_1070), _b_1077_1071, _ctx); /*list<string>*/
  _x1269 = kk_std_core__new_Cons(kk_reuse_null, kk_string_box(_b_1074_1072), _b_1075_1073, _ctx); /*list<61>*/
  return kk_std_text_regex_concat_replace(mm, repl, _x1269, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_concat_replace_fun1279__t {
  struct kk_function_s _base;
  kk_std_core__list acc0;
  kk_std_core__list mm0;
  kk_std_core__list pre0;
  kk_function_t repl0;
};
static kk_string_t kk_std_text_regex_concat_replace_fun1279(kk_function_t _fself, kk_string_t _y_9860, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_concat_replace_fun1279(kk_std_core__list acc0, kk_std_core__list mm0, kk_std_core__list pre0, kk_function_t repl0, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1279__t* _self = kk_function_alloc_as(struct kk_std_text_regex_concat_replace_fun1279__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_concat_replace_fun1279, kk_context());
  _self->acc0 = acc0;
  _self->mm0 = mm0;
  _self->pre0 = pre0;
  _self->repl0 = repl0;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_concat_replace_fun1279(kk_function_t _fself, kk_string_t _y_9860, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1279__t* _self = kk_function_as(struct kk_std_text_regex_concat_replace_fun1279__t*, _fself);
  kk_std_core__list acc0 = _self->acc0; /* list<string> */
  kk_std_core__list mm0 = _self->mm0; /* list<list<sslice>> */
  kk_std_core__list pre0 = _self->pre0; /* list<sslice> */
  kk_function_t repl0 = _self->repl0; /* (list<sslice>) -> 321 string */
  kk_drop_match(_self, {kk_std_core__list_dup(acc0);kk_std_core__list_dup(mm0);kk_std_core__list_dup(pre0);kk_function_dup(repl0);}, {}, _ctx)
  return kk_std_text_regex__mlift1000_concat_replace(acc0, mm0, pre0, repl0, _y_9860, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_concat_replace_fun1281__t {
  struct kk_function_s _base;
  kk_function_t _b_1095_1080;
};
static kk_box_t kk_std_text_regex_concat_replace_fun1281(kk_function_t _fself, kk_box_t _b_1081, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_concat_replace_fun1281(kk_function_t _b_1095_1080, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1281__t* _self = kk_function_alloc_as(struct kk_std_text_regex_concat_replace_fun1281__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_concat_replace_fun1281, kk_context());
  _self->_b_1095_1080 = _b_1095_1080;
  return &_self->_base;
}

static kk_box_t kk_std_text_regex_concat_replace_fun1281(kk_function_t _fself, kk_box_t _b_1081, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1281__t* _self = kk_function_as(struct kk_std_text_regex_concat_replace_fun1281__t*, _fself);
  kk_function_t _b_1095_1080 = _self->_b_1095_1080; /* (string) -> 321 string */
  kk_drop_match(_self, {kk_function_dup(_b_1095_1080);}, {}, _ctx)
  kk_string_t _x1282;
  kk_string_t _x1283 = kk_string_unbox(_b_1081); /*string*/
  _x1282 = kk_function_call(kk_string_t, (kk_function_t, kk_string_t, kk_context_t*), _b_1095_1080, (_b_1095_1080, _x1283, _ctx)); /*string*/
  return kk_string_box(_x1282);
}

kk_string_t kk_std_text_regex_concat_replace(kk_std_core__list matches, kk_function_t repl0, kk_std_core__list acc0, kk_context_t* _ctx) { /* forall<e> (matches : list<list<sslice>>, repl : (list<sslice>) -> e string, acc : list<string>) -> e string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches)) {
    struct kk_std_core_Cons* _con1274 = kk_std_core__as_Cons(matches);
    kk_box_t _box_x1078 = _con1274->head;
    kk_std_core__list _pat0 = _con1274->tail;
    if (kk_std_core__is_Cons(_pat0)) {
      struct kk_std_core_Cons* _con1276 = kk_std_core__as_Cons(_pat0);
      kk_std_core__list pre0 = kk_std_core__list_unbox(_box_x1078, NULL);
      kk_box_t _box_x1079 = _con1276->head;
      kk_std_core__list mm0 = _con1276->tail;
      kk_std_core__list m = kk_std_core__list_unbox(_box_x1079, NULL);
      kk_reuse_t _ru_1224 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1225 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(matches))) {
        if (kk_likely(kk_std_core__list_is_unique(_pat0))) {
          _ru_1225 = (kk_std_core__list_reuse(_pat0));
        }
        else {
          kk_std_core__list_dup(m);
          kk_std_core__list_dup(mm0);
          kk_Unit;
          kk_std_core__list_decref(_pat0, _ctx);
          kk_Unit;
        }
        kk_Unit;
        _ru_1224 = (kk_std_core__list_reuse(matches));
      }
      else {
        kk_std_core__list_dup(m);
        kk_std_core__list_dup(mm0);
        kk_std_core__list_dup(pre0);
        kk_Unit;
        kk_std_core__list_decref(matches, _ctx);
        kk_Unit;
      }
      kk_string_t x_1007;
      kk_function_t _x1278 = kk_function_dup(repl0); /*(list<sslice>) -> 321 string*/
      x_1007 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _x1278, (_x1278, m, _ctx)); /*string*/
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_1224);
        kk_reuse_drop(_ru_1225);
        kk_string_drop(x_1007, _ctx);
        kk_function_t _b_1095_1080 = kk_std_text_regex_new_concat_replace_fun1279(acc0, mm0, pre0, repl0, _ctx); /*(string) -> 321 string*/;
        kk_box_t _x1280 = kk_std_core_hnd_yield_extend(kk_std_text_regex_new_concat_replace_fun1281(_b_1095_1080, _ctx), _ctx); /*3858*/
        return kk_string_unbox(_x1280);
      }
      { // tailcall
        kk_std_core__list _x1284;
        kk_string_t _b_1096_1085 = x_1007; /*string*/;
        kk_std_core__list _b_1097_1086;
        kk_string_t _b_1098_1083;
        if (kk_std_core__is_Cons(pre0)) {
          struct kk_std_core_Cons* _con1286 = kk_std_core__as_Cons(pre0);
          kk_box_t _box_x1082 = _con1286->head;
          kk_std_core__list _pat30 = _con1286->tail;
          kk_std_core__sslice s0 = kk_std_core__sslice_unbox(_box_x1082, NULL);
          if (kk_likely(kk_std_core__list_is_unique(pre0))) {
            kk_std_core__list_drop(_pat30, _ctx);
            kk_std_core__sslice_dup(s0);
            kk_box_drop(_box_x1082, _ctx);
            kk_Unit;
            kk_Unit;
            kk_std_core__list_free(pre0);
          }
          else {
            kk_std_core__sslice_dup(s0);
            kk_Unit;
            kk_std_core__list_decref(pre0, _ctx);
            kk_Unit;
          }
          _b_1098_1083 = kk_std_core_string_3(s0, _ctx); /*string*/
          goto _match1285;
        }
        _b_1098_1083 = kk_string_empty(); /*string*/
        _match1285: ;
        kk_std_core__list _b_1099_1084 = acc0; /*list<string>*/;
        _b_1097_1086 = kk_std_core__new_Cons(_ru_1225, kk_string_box(_b_1098_1083), _b_1099_1084, _ctx); /*list<string>*/
        _x1284 = kk_std_core__new_Cons(_ru_1224, kk_string_box(_b_1096_1085), _b_1097_1086, _ctx); /*list<61>*/
        matches = mm0;
        acc0 = _x1284;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(matches)) {
    struct kk_std_core_Cons* _con1289 = kk_std_core__as_Cons(matches);
    kk_box_t _box_x1087 = _con1289->head;
    kk_std_core__list _pat200 = _con1289->tail;
    kk_std_core__list post = kk_std_core__list_unbox(_box_x1087, NULL);
    kk_function_drop(repl0, _ctx);
    kk_reuse_t _ru_1227 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(matches))) {
      _ru_1227 = (kk_std_core__list_reuse(matches));
    }
    else {
      kk_std_core__list_dup(post);
      kk_Unit;
      kk_std_core__list_decref(matches, _ctx);
      kk_Unit;
    }
    kk_string_t _b_1091_1089;
    if (kk_std_core__is_Cons(post)) {
      struct kk_std_core_Cons* _con1292 = kk_std_core__as_Cons(post);
      kk_box_t _box_x1088 = _con1292->head;
      kk_std_core__list _pat300 = _con1292->tail;
      kk_std_core__sslice s00 = kk_std_core__sslice_unbox(_box_x1088, NULL);
      if (kk_likely(kk_std_core__list_is_unique(post))) {
        kk_std_core__list_drop(_pat300, _ctx);
        kk_std_core__sslice_dup(s00);
        kk_box_drop(_box_x1088, _ctx);
        kk_Unit;
        kk_Unit;
        kk_std_core__list_free(post);
      }
      else {
        kk_std_core__sslice_dup(s00);
        kk_Unit;
        kk_std_core__list_decref(post, _ctx);
        kk_Unit;
      }
      _b_1091_1089 = kk_std_core_string_3(s00, _ctx); /*string*/
      goto _match1291;
    }
    _b_1091_1089 = kk_string_empty(); /*string*/
    _match1291: ;
    kk_std_core__list xs0_17047;
    kk_std_core__list _x1295 = kk_std_core__new_Cons(_ru_1227, kk_string_box(_b_1091_1089), acc0, _ctx); /*list<61>*/
    xs0_17047 = kk_std_core__lift17083_reverse_join(kk_std_core__new_Nil(_ctx), _x1295, _ctx); /*list<string>*/
    if (kk_std_core__is_Nil(xs0_17047)) {
      return kk_string_empty();
    }
    struct kk_std_core_Cons* _con1297 = kk_std_core__as_Cons(xs0_17047);
    kk_box_t _box_x1093 = _con1297->head;
    kk_std_core__list xx0 = _con1297->tail;
    kk_string_t x0 = kk_string_unbox(_box_x1093);
    if (kk_likely(kk_std_core__list_is_unique(xs0_17047))) {
      kk_std_core__list_free(xs0_17047);
    }
    else {
      kk_string_dup(x0);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs0_17047, _ctx);
      kk_Unit;
    }
    kk_string_t _x1299 = kk_string_empty(); /*string*/
    return kk_std_core__lift17084_reverse_join(_x1299, xx0, x0, _ctx);
  }
  kk_function_drop(repl0, _ctx);
  kk_std_core__list xs0_170470 = kk_std_core__lift17083_reverse_join(kk_std_core__new_Nil(_ctx), acc0, _ctx); /*list<string>*/;
  if (kk_std_core__is_Nil(xs0_170470)) {
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con1302 = kk_std_core__as_Cons(xs0_170470);
  kk_box_t _box_x1094 = _con1302->head;
  kk_std_core__list xx00 = _con1302->tail;
  kk_string_t x00 = kk_string_unbox(_box_x1094);
  if (kk_likely(kk_std_core__list_is_unique(xs0_170470))) {
    kk_std_core__list_free(xs0_170470);
  }
  else {
    kk_string_dup(x00);
    kk_std_core__list_dup(xx00);
    kk_Unit;
    kk_std_core__list_decref(xs0_170470, _ctx);
    kk_Unit;
  }
  kk_string_t _x1304 = kk_string_empty(); /*string*/
  return kk_std_core__lift17084_reverse_join(_x1304, xx00, x00, _ctx);
}
 
// Does a regular expression pattern occur in a string `s`?
// (note: called `test` in javascript)

bool kk_std_text_regex_contains(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> bool */ 
  kk_std_core__list list0_944;
  kk_box_t _x1308;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1308 = _x; /*any*/
  kk_ssize_t _x1309 = ((kk_ssize_t)0); /*ssize_t*/
  list0_944 = kk_std_text_regex_regex_exec(_x1308, s, _x1309, _ctx); /*list<sslice>*/
  if (kk_std_core__is_Cons(list0_944)) {
    struct kk_std_core_Cons* _con1310 = kk_std_core__as_Cons(list0_944);
    kk_box_t _box_x1100 = _con1310->head;
    kk_std_core__list_dropn(list0_944, ((int32_t)2), _ctx);
    return true;
  }
  return false;
}
 
// Match a regular expression `regex` over a string `s`.
// Matches at most `atmost` times (and matches all by default).
// Returns always an odd number of elements where every even
// element is a match and the odd ones the string parts between the
// matches.
// See also `find-all` and `strings`.

kk_std_core__list kk_std_text_regex_exec_all(kk_std_text_regex__regex regex0, kk_string_t s, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (regex : regex, s : string, atmost : optional<int>) -> list<list<sslice>> */ 
  kk_box_t _x1312;
  kk_box_t _x = regex0.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(regex0, _ctx);
  kk_Unit;
  _x1312 = _x; /*any*/
  kk_ssize_t _x1313 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1314;
  kk_integer_t _x1315;
  if (kk_std_core_types__is_Optional(atmost)) {
    kk_box_t _box_x1101 = atmost._cons.Optional.value;
    kk_integer_t _atmost_346 = kk_integer_unbox(_box_x1101);
    kk_integer_dup(_atmost_346);
    kk_std_core_types__optional_drop(atmost, _ctx);
    kk_Unit;
    _x1315 = _atmost_346; /*int*/
    goto _match1316;
  }
  _x1315 = kk_integer_from_small(-1); /*int*/
  _match1316: ;
  _x1314 = kk_std_core_ssize__t(_x1315, _ctx); /*ssize_t*/
  return kk_std_text_regex_regex_exec_all(_x1312, s, _x1313, _x1314, _ctx);
}
 
// Filter only for the matched parts.

kk_std_core__list kk_std_text_regex__ctail_filter_matches(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice>>, ctail<list<list<sslice>>>) -> list<list<sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con1318 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x1102 = _con1318->head;
    kk_std_core__list _pat1 = _con1318->tail;
    if (kk_std_core__is_Cons(_pat1)) {
      struct kk_std_core_Cons* _con1320 = kk_std_core__as_Cons(_pat1);
      kk_box_t _box_x1103 = _con1320->head;
      kk_std_core__list mm = _con1320->tail;
      kk_std_core__list m = kk_std_core__list_unbox(_box_x1103, NULL);
      kk_reuse_t _ru_1233 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        if (kk_likely(kk_std_core__list_is_unique(_pat1))) {
          _ru_1233 = (kk_std_core__list_reuse(_pat1));
        }
        else {
          kk_std_core__list_dup(m);
          kk_std_core__list_dup(mm);
          kk_Unit;
          kk_std_core__list_decref(_pat1, _ctx);
          kk_Unit;
        }
        kk_box_drop(_box_x1102, _ctx);
        kk_Unit;
        kk_Unit;
        kk_std_core__list_free(xs);
      }
      else {
        kk_std_core__list_dup(m);
        kk_std_core__list_dup(mm);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      kk_std_core__list _ctail_981 = kk_std_core__list_hole(); /*list<list<sslice>>*/;
      kk_std_core__list _ctail_982 = kk_std_core__new_Cons(_ru_1233, kk_std_core__list_box(m, _ctx), _ctail_981, _ctx); /*list<list<sslice>>*/;
      { // tailcall
        kk_std_core_types__ctail _x1322;
        kk_box_t* field_1012 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_982)->tail)); /*cfield<list<list<sslice>>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x1114 = _acc._cons.CTail._field1;
          kk_box_t* last = _acc._cons.CTail._field2;
          kk_std_core__list head = kk_std_core__list_unbox(_box_x1114, NULL);
          kk_std_core__list_dup(head);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last) = kk_std_core__list_box(_ctail_982, _ctx);
          kk_std_core__list _b_1128_1119 = head; /*list<list<sslice>>*/;
          kk_box_t* _b_1129_1120 = field_1012; /*cfield<list<list<sslice>>>*/;
          _x1322 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1128_1119, _ctx), _b_1129_1120, _ctx); /*ctail<58>*/
          goto _match1323;
        }
        kk_std_core__list _b_1130_1121 = _ctail_982; /*list<list<sslice>>*/;
        kk_box_t* _b_1131_1122 = field_1012; /*cfield<list<list<sslice>>>*/;
        _x1322 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1130_1121, _ctx), _b_1131_1122, _ctx); /*ctail<58>*/
        _match1323: ;
        xs = mm;
        _acc = _x1322;
        goto kk__tailcall;
      }
    }
  }
  kk_std_core__list_drop(xs, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x1123 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x1123, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last0) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head0;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Filter only for the matched parts.

kk_std_core__list kk_std_text_regex_filter_matches(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<list<sslice>>) -> list<list<sslice>> */ 
  return kk_std_text_regex__ctail_filter_matches(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Filter only for the non-matched parts.

kk_std_core__list kk_std_text_regex__ctail_filter_non_matches(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice>>, ctail<list<list<sslice>>>) -> list<list<sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con1326 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x1132 = _con1326->head;
    kk_std_core__list _pat0 = _con1326->tail;
    if (kk_std_core__is_Cons(_pat0)) {
      struct kk_std_core_Cons* _con1328 = kk_std_core__as_Cons(_pat0);
      kk_std_core__list s = kk_std_core__list_unbox(_box_x1132, NULL);
      kk_box_t _box_x1133 = _con1328->head;
      kk_std_core__list xx = _con1328->tail;
      kk_reuse_t _ru_1235 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        if (kk_likely(kk_std_core__list_is_unique(_pat0))) {
          kk_box_drop(_box_x1133, _ctx);
          kk_Unit;
          kk_Unit;
          _ru_1235 = (kk_std_core__list_reuse(_pat0));
        }
        else {
          kk_std_core__list_dup(xx);
          kk_Unit;
          kk_std_core__list_decref(_pat0, _ctx);
          kk_Unit;
        }
        kk_Unit;
        kk_std_core__list_free(xs);
      }
      else {
        kk_std_core__list_dup(s);
        kk_std_core__list_dup(xx);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      kk_std_core__list _ctail_983 = kk_std_core__list_hole(); /*list<list<sslice>>*/;
      kk_std_core__list _ctail_984 = kk_std_core__new_Cons(_ru_1235, kk_std_core__list_box(s, _ctx), _ctail_983, _ctx); /*list<list<sslice>>*/;
      { // tailcall
        kk_std_core_types__ctail _x1330;
        kk_box_t* field_1017 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_984)->tail)); /*cfield<list<list<sslice>>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x1144 = _acc._cons.CTail._field1;
          kk_box_t* last = _acc._cons.CTail._field2;
          kk_std_core__list head = kk_std_core__list_unbox(_box_x1144, NULL);
          kk_std_core__list_dup(head);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last) = kk_std_core__list_box(_ctail_984, _ctx);
          kk_std_core__list _b_1168_1149 = head; /*list<list<sslice>>*/;
          kk_box_t* _b_1169_1150 = field_1017; /*cfield<list<list<sslice>>>*/;
          _x1330 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1168_1149, _ctx), _b_1169_1150, _ctx); /*ctail<58>*/
          goto _match1331;
        }
        kk_std_core__list _b_1170_1151 = _ctail_984; /*list<list<sslice>>*/;
        kk_box_t* _b_1171_1152 = field_1017; /*cfield<list<list<sslice>>>*/;
        _x1330 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1170_1151, _ctx), _b_1171_1152, _ctx); /*ctail<58>*/
        _match1331: ;
        xs = xx;
        _acc = _x1330;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con1333 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x1153 = _con1333->head;
    kk_std_core__list _pat3 = _con1333->tail;
    kk_std_core__list s0 = kk_std_core__list_unbox(_box_x1153, NULL);
    kk_reuse_t _ru_1236 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_1236 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_std_core__list_dup(s0);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_std_core__list rest_1019;
    if (kk_likely(_ru_1236!=NULL)) {
      struct kk_std_core_Cons* _con1335 = (struct kk_std_core_Cons*)_ru_1236;
      _con1335->head = kk_std_core__list_box(s0, _ctx);
      rest_1019 = kk_std_core__base_Cons(_con1335); /*list<list<sslice>>*/
    }
    else {
      rest_1019 = kk_std_core__new_Cons(kk_reuse_null, kk_std_core__list_box(s0, _ctx), _pat3, _ctx); /*list<list<sslice>>*/
    }
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x1158 = _acc._cons.CTail._field1;
      kk_box_t* last0 = _acc._cons.CTail._field2;
      kk_std_core__list head0 = kk_std_core__list_unbox(_box_x1158, NULL);
      kk_std_core__list_dup(head0);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __0 = kk_Unit;
      *(last0) = kk_std_core__list_box(rest_1019, _ctx);
      return head0;
    }
    return rest_1019;
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x1163 = _acc._cons.CTail._field1;
    kk_box_t* last1 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x1163, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last1) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Filter only for the non-matched parts.

kk_std_core__list kk_std_text_regex_filter_non_matches(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<list<sslice>>) -> list<list<sslice>> */ 
  return kk_std_text_regex__ctail_filter_non_matches(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Find a match for a regular expression.
// See also `exec`

kk_std_core_types__maybe kk_std_text_regex_find(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> maybe<string> */ 
  kk_std_core__list _match_1219;
  kk_box_t _x1338;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1338 = _x; /*any*/
  kk_ssize_t _x1339 = ((kk_ssize_t)0); /*ssize_t*/
  _match_1219 = kk_std_text_regex_regex_exec(_x1338, s, _x1339, _ctx); /*list<sslice>*/
  if (kk_std_core__is_Cons(_match_1219)) {
    struct kk_std_core_Cons* _con1340 = kk_std_core__as_Cons(_match_1219);
    kk_box_t _box_x1172 = _con1340->head;
    kk_std_core__list _pat3 = _con1340->tail;
    kk_std_core__sslice m = kk_std_core__sslice_unbox(_box_x1172, NULL);
    if (kk_likely(kk_std_core__list_is_unique(_match_1219))) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(m);
      kk_box_drop(_box_x1172, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(_match_1219);
    }
    else {
      kk_std_core__sslice_dup(m);
      kk_Unit;
      kk_std_core__list_decref(_match_1219, _ctx);
      kk_Unit;
    }
    kk_string_t _b_1174_1173 = kk_std_core_string_3(m, _ctx); /*string*/;
    return kk_std_core_types__new_Just(kk_string_box(_b_1174_1173), _ctx);
  }
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Find all matches for a regular expression in a string.


// lift anonymous function
struct kk_std_text_regex_find_all_fun1350__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_text_regex_find_all_fun1350(kk_function_t _fself, kk_std_core__list _x11349, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_find_all_fun1350(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_find_all_fun1350, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_text_regex_find_all_fun1350(kk_function_t _fself, kk_std_core__list _x11349, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_text_regex_captured(_x11349, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_find_all_fun1351__t {
  struct kk_function_s _base;
  kk_function_t _b_1180_1177;
};
static kk_box_t kk_std_text_regex_find_all_fun1351(kk_function_t _fself, kk_box_t _b_1178, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_find_all_fun1351(kk_function_t _b_1180_1177, kk_context_t* _ctx) {
  struct kk_std_text_regex_find_all_fun1351__t* _self = kk_function_alloc_as(struct kk_std_text_regex_find_all_fun1351__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_find_all_fun1351, kk_context());
  _self->_b_1180_1177 = _b_1180_1177;
  return &_self->_base;
}

static kk_box_t kk_std_text_regex_find_all_fun1351(kk_function_t _fself, kk_box_t _b_1178, kk_context_t* _ctx) {
  struct kk_std_text_regex_find_all_fun1351__t* _self = kk_function_as(struct kk_std_text_regex_find_all_fun1351__t*, _fself);
  kk_function_t _b_1180_1177 = _self->_b_1180_1177; /* (matched : list<sslice>) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_1180_1177);}, {}, _ctx)
  kk_string_t _x1352;
  kk_std_core__list _x1353 = kk_std_core__list_unbox(_b_1178, _ctx); /*list<sslice>*/
  _x1352 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _b_1180_1177, (_b_1180_1177, _x1353, _ctx)); /*string*/
  return kk_string_box(_x1352);
}

kk_std_core__list kk_std_text_regex_find_all(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : optional<int>) -> list<string> */ 
  kk_std_core__list xs_952;
  kk_std_core__list _x1342;
  kk_box_t _x1343;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1343 = _x; /*any*/
  kk_ssize_t _x1344 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1345;
  kk_integer_t _x1346;
  if (kk_std_core_types__is_Optional(atmost)) {
    kk_box_t _box_x1175 = atmost._cons.Optional.value;
    kk_integer_t _atmost_439 = kk_integer_unbox(_box_x1175);
    kk_integer_dup(_atmost_439);
    kk_std_core_types__optional_drop(atmost, _ctx);
    kk_Unit;
    _x1346 = _atmost_439; /*int*/
    goto _match1347;
  }
  _x1346 = kk_integer_from_small(-1); /*int*/
  _match1347: ;
  _x1345 = kk_std_core_ssize__t(_x1346, _ctx); /*ssize_t*/
  _x1342 = kk_std_text_regex_regex_exec_all(_x1343, s, _x1344, _x1345, _ctx); /*list<list<sslice>>*/
  xs_952 = kk_std_text_regex_filter_matches(_x1342, _ctx); /*list<list<sslice>>*/
  kk_std_core__list _b_1179_1176 = xs_952; /*list<list<sslice>>*/;
  kk_function_t _b_1180_1177 = kk_std_text_regex_new_find_all_fun1350(_ctx); /*(matched : list<sslice>) -> string*/;
  return kk_std_core_map_5(_b_1179_1176, kk_std_text_regex_new_find_all_fun1351(_b_1180_1177, _ctx), _ctx);
}
 
// Create a new regular expression. Takes two optional parameters. Set `ignoreCase` to `True`
// to ignore uppercase/lowercase distinction. If  `multiline` is set to `True`, then `^` and `$`
// match also the beginning and end of every line (instead of the entire input).

kk_std_text_regex__regex kk_std_text_regex_regex(kk_string_t regex0, kk_std_core_types__optional ignorecase, kk_std_core_types__optional multiline, kk_context_t* _ctx) { /* (regex : string, ignorecase : optional<bool>, multiline : optional<bool>) -> regex */ 
  kk_box_t _x1354;
  kk_string_t _x1355 = kk_string_dup(regex0); /*string*/
  bool _x1356;
  if (kk_std_core_types__is_Optional(ignorecase)) {
    kk_box_t _box_x1181 = ignorecase._cons.Optional.value;
    bool _ignorecase_458 = kk_bool_unbox(_box_x1181);
    kk_std_core_types__optional_drop(ignorecase, _ctx);
    kk_Unit;
    _x1356 = _ignorecase_458; /*bool*/
    goto _match1357;
  }
  _x1356 = false; /*bool*/
  _match1357: ;
  bool _x1359;
  if (kk_std_core_types__is_Optional(multiline)) {
    kk_box_t _box_x1182 = multiline._cons.Optional.value;
    bool _multiline_462 = kk_bool_unbox(_box_x1182);
    kk_std_core_types__optional_drop(multiline, _ctx);
    kk_Unit;
    _x1359 = _multiline_462; /*bool*/
    goto _match1360;
  }
  _x1359 = false; /*bool*/
  _match1360: ;
  _x1354 = kk_std_text_regex_regex_create(_x1355, _x1356, _x1359, _ctx); /*any*/
  return kk_std_text_regex__new_Regex(_x1354, regex0, _ctx);
}
 
// Replace the all occurrences of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace_all(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice>) -> e string, atmost : optional<int>) -> e string */ 
  kk_std_core__list _x1362;
  kk_box_t _x1363;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1363 = _x; /*any*/
  kk_ssize_t _x1364 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1365;
  kk_integer_t _x1366;
  if (kk_std_core_types__is_Optional(atmost)) {
    kk_box_t _box_x1183 = atmost._cons.Optional.value;
    kk_integer_t _atmost_478 = kk_integer_unbox(_box_x1183);
    kk_integer_dup(_atmost_478);
    kk_std_core_types__optional_drop(atmost, _ctx);
    kk_Unit;
    _x1366 = _atmost_478; /*int*/
    goto _match1367;
  }
  _x1366 = kk_integer_from_small(-1); /*int*/
  _match1367: ;
  _x1365 = kk_std_core_ssize__t(_x1366, _ctx); /*ssize_t*/
  _x1362 = kk_std_text_regex_regex_exec_all(_x1363, s, _x1364, _x1365, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1362, repl, kk_std_core__new_Nil(_ctx), _ctx);
}
 
// Replace using a replacement string that can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match (`==$0`).


// lift anonymous function
struct kk_std_text_regex_replace_captures_fun1387__t {
  struct kk_function_s _base;
  kk_std_core__list caps;
};
static kk_string_t kk_std_text_regex_replace_captures_fun1387(kk_function_t _fself, kk_std_core__list cap, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_captures_fun1387(kk_std_core__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_captures_fun1387__t* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_captures_fun1387__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_replace_captures_fun1387, kk_context());
  _self->caps = caps;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_replace_captures_fun1387(kk_function_t _fself, kk_std_core__list cap, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_captures_fun1387__t* _self = kk_function_as(struct kk_std_text_regex_replace_captures_fun1387__t*, _fself);
  kk_std_core__list caps = _self->caps; /* list<sslice> */
  kk_drop_match(_self, {kk_std_core__list_dup(caps);}, {}, _ctx)
  if (kk_std_core__is_Cons(cap)) {
    struct kk_std_core_Cons* _con1388 = kk_std_core__as_Cons(cap);
    kk_box_t _box_x1187 = _con1388->head;
    kk_std_core__list _pat4 = _con1388->tail;
    if (kk_std_core__is_Cons(_pat4)) {
      struct kk_std_core_Cons* _con1390 = kk_std_core__as_Cons(_pat4);
      kk_std_core__sslice _pat000 = kk_std_core__sslice_unbox(_box_x1187, NULL);
      kk_box_t _box_x1188 = _con1390->head;
      kk_std_core__list _pat8 = _con1390->tail;
      if (kk_std_core__is_Cons(_pat8)) {
        struct kk_std_core_Cons* _con1392 = kk_std_core__as_Cons(_pat8);
        kk_std_core__sslice digit = kk_std_core__sslice_unbox(_box_x1188, NULL);
        kk_box_t _box_x1189 = _con1392->head;
        kk_std_core__list _pat12 = _con1392->tail;
        if (kk_std_core__is_Cons(_pat12)) {
          struct kk_std_core_Cons* _con1394 = kk_std_core__as_Cons(_pat12);
          kk_std_core__sslice amp = kk_std_core__sslice_unbox(_box_x1189, NULL);
          kk_box_t _box_x1190 = _con1394->head;
          kk_std_core__list _pat16 = _con1394->tail;
          if (kk_std_core__is_Nil(_pat16)) {
            kk_std_core__sslice dollar = kk_std_core__sslice_unbox(_box_x1190, NULL);
            if (kk_likely(kk_std_core__list_is_unique(cap))) {
              if (kk_likely(kk_std_core__list_is_unique(_pat4))) {
                if (kk_likely(kk_std_core__list_is_unique(_pat8))) {
                  if (kk_likely(kk_std_core__list_is_unique(_pat12))) {
                    kk_std_core__sslice_dup(dollar);
                    kk_box_drop(_box_x1190, _ctx);
                    kk_Unit;
                    kk_Unit;
                    kk_std_core__list_free(_pat12);
                  }
                  else {
                    kk_std_core__sslice_dup(dollar);
                    kk_Unit;
                    kk_std_core__list_decref(_pat12, _ctx);
                    kk_Unit;
                  }
                  kk_std_core__sslice_dup(amp);
                  kk_box_drop(_box_x1189, _ctx);
                  kk_Unit;
                  kk_Unit;
                  kk_std_core__list_free(_pat8);
                }
                else {
                  kk_std_core__sslice_dup(amp);
                  kk_std_core__sslice_dup(dollar);
                  kk_Unit;
                  kk_std_core__list_decref(_pat8, _ctx);
                  kk_Unit;
                }
                kk_std_core__sslice_dup(digit);
                kk_box_drop(_box_x1188, _ctx);
                kk_Unit;
                kk_Unit;
                kk_std_core__list_free(_pat4);
              }
              else {
                kk_std_core__sslice_dup(amp);
                kk_std_core__sslice_dup(digit);
                kk_std_core__sslice_dup(dollar);
                kk_Unit;
                kk_std_core__list_decref(_pat4, _ctx);
                kk_Unit;
              }
              kk_box_drop(_box_x1187, _ctx);
              kk_Unit;
              kk_Unit;
              kk_std_core__list_free(cap);
            }
            else {
              kk_std_core__sslice_dup(amp);
              kk_std_core__sslice_dup(digit);
              kk_std_core__sslice_dup(dollar);
              kk_Unit;
              kk_std_core__list_decref(cap, _ctx);
              kk_Unit;
            }
            bool _match_1213;
            kk_ssize_t _x1396;
            kk_ssize_t _x0 = dollar.start;
            kk_std_core__sslice_drop(dollar, _ctx);
            kk_Unit;
            _x1396 = _x0; /*ssize_t*/
            kk_ssize_t _x1397 = ((kk_ssize_t)0); /*ssize_t*/
            _match_1213 = (_x1396 >= _x1397); /*bool*/
            if (_match_1213) {
              kk_std_core__sslice_drop(amp, _ctx);
              kk_std_core__list_drop(caps, _ctx);
              kk_std_core__sslice_drop(digit, _ctx);
              kk_define_string_literal(, _s1398, 1, "$")
              return kk_string_dup(_s1398);
            }
            kk_integer_t grp;
            bool _match_1215;
            kk_ssize_t _x1399;
            kk_ssize_t _x00 = amp.start;
            kk_std_core__sslice_drop(amp, _ctx);
            kk_Unit;
            _x1399 = _x00; /*ssize_t*/
            kk_ssize_t _x1400 = ((kk_ssize_t)0); /*ssize_t*/
            _match_1215 = (_x1399 >= _x1400); /*bool*/
            if (_match_1215) {
              kk_std_core__sslice_drop(digit, _ctx);
              grp = kk_integer_from_small(0); /*int*/
            }
            else {
              kk_string_t _x1401 = kk_std_core_string_3(digit, _ctx); /*string*/
              kk_std_core_types__optional _x1402 = kk_std_core_types__new_Optional(kk_integer_box(kk_integer_from_small(0)), _ctx); /*optional<106>*/
              grp = kk_std_core_parse_int_default(_x1401, _x1402, kk_std_core_types__new_None(_ctx), _ctx); /*int*/
            }
            kk_std_core_types__maybe _match_1214 = kk_std_core__lp__lb__rb__2_rp_(caps, grp, _ctx); /*maybe<9772>*/;
            if (kk_std_core_types__is_Nothing(_match_1214)) {
              return kk_string_empty();
            }
            kk_box_t _box_x1193 = _match_1214._cons.Just.value;
            kk_std_core__sslice s0 = kk_std_core__sslice_unbox(_box_x1193, NULL);
            kk_std_core__sslice_dup(s0);
            kk_std_core_types__maybe_drop(_match_1214, _ctx);
            kk_Unit;
            return kk_std_core_string_3(s0, _ctx);
          }
        }
      }
    }
  }
  kk_std_core__list_drop(cap, _ctx);
  kk_std_core__list_drop(caps, _ctx);
  kk_define_string_literal(, _s1405, 1, "$")
  return kk_string_dup(_s1405);
}

kk_string_t kk_std_text_regex_replace_captures(kk_std_core__list caps, kk_string_t repl, kk_context_t* _ctx) { /* (caps : list<sslice>, repl : string) -> string */ 
  kk_std_text_regex__regex r_1023;
  kk_box_t _x1369;
  kk_string_t _x1370;
  kk_define_string_literal(, _s1371, 20, "\\$(\?:(\\d)|(\\&)|(\\$))")
  _x1370 = kk_string_dup(_s1371); /*string*/
  bool _x1372;
  kk_std_core_types__optional _match_1218 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1218)) {
    kk_box_t _box_x1184 = _match_1218._cons.Optional.value;
    bool _ignorecase_458 = kk_bool_unbox(_box_x1184);
    kk_std_core_types__optional_drop(_match_1218, _ctx);
    kk_Unit;
    _x1372 = _ignorecase_458; /*bool*/
    goto _match1373;
  }
  _x1372 = false; /*bool*/
  _match1373: ;
  bool _x1375;
  kk_std_core_types__optional _match_1217 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1217)) {
    kk_box_t _box_x1185 = _match_1217._cons.Optional.value;
    bool _multiline_462 = kk_bool_unbox(_box_x1185);
    kk_std_core_types__optional_drop(_match_1217, _ctx);
    kk_Unit;
    _x1375 = _multiline_462; /*bool*/
    goto _match1376;
  }
  _x1375 = false; /*bool*/
  _match1376: ;
  _x1369 = kk_std_text_regex_regex_create(_x1370, _x1372, _x1375, _ctx); /*any*/
  kk_string_t _x1378;
  kk_define_string_literal(, _s1379, 20, "\\$(\?:(\\d)|(\\&)|(\\$))")
  _x1378 = kk_string_dup(_s1379); /*string*/
  r_1023 = kk_std_text_regex__new_Regex(_x1369, _x1378, _ctx); /*std/text/regex/regex*/
  kk_std_core__list _x1380;
  kk_box_t _x1381;
  kk_box_t _x = r_1023.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r_1023, _ctx);
  kk_Unit;
  _x1381 = _x; /*any*/
  kk_ssize_t _x1382 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1383;
  kk_integer_t _x1384;
  kk_std_core_types__optional _match_1216 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1216)) {
    kk_box_t _box_x1186 = _match_1216._cons.Optional.value;
    kk_integer_t _atmost_478 = kk_integer_unbox(_box_x1186);
    kk_integer_dup(_atmost_478);
    kk_std_core_types__optional_drop(_match_1216, _ctx);
    kk_Unit;
    _x1384 = _atmost_478; /*int*/
    goto _match1385;
  }
  _x1384 = kk_integer_from_small(-1); /*int*/
  _match1385: ;
  _x1383 = kk_std_core_ssize__t(_x1384, _ctx); /*ssize_t*/
  _x1380 = kk_std_text_regex_regex_exec_all(_x1381, repl, _x1382, _x1383, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1380, kk_std_text_regex_new_replace_captures_fun1387(caps, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
}
 
// Replace all occurrences of `regex` with the replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match (`==$0`).


// lift anonymous function
struct kk_std_text_regex_replace_all_fun1416__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_all_fun1416_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_all_fun1416_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1416__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_all_fun1416__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_replace_all_fun1416_1, kk_context());
  _self->repl = repl;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_replace_all_fun1416_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1416__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_all_fun1416__t_1*, _fself);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl);}, {}, _ctx)
  return kk_std_text_regex_replace_captures(caps, repl, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_replace_all_fun1424__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_all_fun1424_1(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_all_fun1424_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1424__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_all_fun1424__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_replace_all_fun1424_1, kk_context());
  _self->repl = repl;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_replace_all_fun1424_1(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1424__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_all_fun1424__t_1*, _fself);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl);}, {}, _ctx)
  kk_std_core__list_drop(___wildcard__183__35, _ctx);
  return repl;
}

kk_string_t kk_std_text_regex_replace_all_1(kk_string_t s, kk_std_text_regex__regex regex0, kk_string_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string, atmost : optional<int>) -> string */ 
  bool _match_1212;
  kk_string_t _x1406 = kk_string_dup(repl); /*string*/
  kk_string_t _x1407;
  kk_define_string_literal(, _s1408, 1, "$")
  _x1407 = kk_string_dup(_s1408); /*string*/
  _match_1212 = kk_string_contains(_x1406,_x1407,kk_context()); /*bool*/
  if (_match_1212) {
    kk_std_core__list _x1409;
    kk_box_t _x1410;
    kk_box_t _x = regex0.obj;
    kk_box_dup(_x);
    kk_std_text_regex__regex_drop(regex0, _ctx);
    kk_Unit;
    _x1410 = _x; /*any*/
    kk_ssize_t _x1411 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x1412;
    kk_integer_t _x1413;
    if (kk_std_core_types__is_Optional(atmost)) {
      kk_box_t _box_x1194 = atmost._cons.Optional.value;
      kk_integer_t _atmost_499 = kk_integer_unbox(_box_x1194);
      kk_integer_dup(_atmost_499);
      kk_std_core_types__optional_drop(atmost, _ctx);
      kk_Unit;
      _x1413 = _atmost_499; /*int*/
      goto _match1414;
    }
    _x1413 = kk_integer_from_small(-1); /*int*/
    _match1414: ;
    _x1412 = kk_std_core_ssize__t(_x1413, _ctx); /*ssize_t*/
    _x1409 = kk_std_text_regex_regex_exec_all(_x1410, s, _x1411, _x1412, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex_concat_replace(_x1409, kk_std_text_regex_new_replace_all_fun1416_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
  }
  kk_std_core__list _x1417;
  kk_box_t _x1418;
  kk_box_t _x0 = regex0.obj;
  kk_box_dup(_x0);
  kk_std_text_regex__regex_drop(regex0, _ctx);
  kk_Unit;
  _x1418 = _x0; /*any*/
  kk_ssize_t _x1419 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1420;
  kk_integer_t _x1421;
  if (kk_std_core_types__is_Optional(atmost)) {
    kk_box_t _box_x1195 = atmost._cons.Optional.value;
    kk_integer_t _atmost_4990 = kk_integer_unbox(_box_x1195);
    kk_integer_dup(_atmost_4990);
    kk_std_core_types__optional_drop(atmost, _ctx);
    kk_Unit;
    _x1421 = _atmost_4990; /*int*/
    goto _match1422;
  }
  _x1421 = kk_integer_from_small(-1); /*int*/
  _match1422: ;
  _x1420 = kk_std_core_ssize__t(_x1421, _ctx); /*ssize_t*/
  _x1417 = kk_std_text_regex_regex_exec_all(_x1418, s, _x1419, _x1420, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1417, kk_std_text_regex_new_replace_all_fun1424_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
}

kk_std_text_regex__regex kk_std_text_regex_rx_nongroup;
 
// How many groups are captured by this regex?


// lift anonymous function
struct kk_std_text_regex_groups_count_fun1450__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_text_regex_groups_count_fun1450(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_groups_count_fun1450(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_groups_count_fun1450, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_text_regex_groups_count_fun1450(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_string_t _x1451 = kk_string_empty(); /*string*/
  return kk_std_text_regex_replace_captures(caps, _x1451, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_groups_count_fun1462__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_text_regex_groups_count_fun1462(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_groups_count_fun1462(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_groups_count_fun1462, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_text_regex_groups_count_fun1462(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core__list_drop(___wildcard__183__35, _ctx);
  return kk_string_empty();
}

kk_integer_t kk_std_text_regex_groups_count(kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (r : regex) -> int */ 
  kk_string_t _x1436;
  bool _match_1207;
  kk_string_t _x1437 = kk_string_empty(); /*string*/
  kk_string_t _x1439;
  kk_define_string_literal(, _s1440, 1, "$")
  _x1439 = kk_string_dup(_s1440); /*string*/
  _match_1207 = kk_string_contains(_x1437,_x1439,kk_context()); /*bool*/
  if (_match_1207) {
    kk_std_core__list _x1441;
    kk_box_t _x1442;
    kk_std_text_regex__regex _x1443 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    kk_box_t _x = _x1443.obj;
    _x1442 = kk_box_dup(_x); /*any*/
    kk_string_t _x1444;
    kk_string_t _x0 = r.src;
    kk_string_dup(_x0);
    kk_std_text_regex__regex_drop(r, _ctx);
    kk_Unit;
    _x1444 = _x0; /*string*/
    kk_ssize_t _x1445 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x1446;
    kk_integer_t _x1447;
    kk_std_core_types__optional _match_1209 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1209)) {
      kk_box_t _box_x1198 = _match_1209._cons.Optional.value;
      kk_integer_t _atmost_499 = kk_integer_unbox(_box_x1198);
      kk_integer_dup(_atmost_499);
      kk_std_core_types__optional_drop(_match_1209, _ctx);
      kk_Unit;
      _x1447 = _atmost_499; /*int*/
      goto _match1448;
    }
    _x1447 = kk_integer_from_small(-1); /*int*/
    _match1448: ;
    _x1446 = kk_std_core_ssize__t(_x1447, _ctx); /*ssize_t*/
    _x1441 = kk_std_text_regex_regex_exec_all(_x1442, _x1444, _x1445, _x1446, _ctx); /*list<list<sslice>>*/
    _x1436 = kk_std_text_regex_concat_replace(_x1441, kk_std_text_regex_new_groups_count_fun1450(_ctx), kk_std_core__new_Nil(_ctx), _ctx); /*string*/
  }
  else {
    kk_std_core__list _x1453;
    kk_box_t _x1454;
    kk_std_text_regex__regex _x1455 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    kk_box_t _x1 = _x1455.obj;
    _x1454 = kk_box_dup(_x1); /*any*/
    kk_string_t _x1456;
    kk_string_t _x00 = r.src;
    kk_string_dup(_x00);
    kk_std_text_regex__regex_drop(r, _ctx);
    kk_Unit;
    _x1456 = _x00; /*string*/
    kk_ssize_t _x1457 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x1458;
    kk_integer_t _x1459;
    kk_std_core_types__optional _match_1208 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1208)) {
      kk_box_t _box_x1199 = _match_1208._cons.Optional.value;
      kk_integer_t _atmost_4990 = kk_integer_unbox(_box_x1199);
      kk_integer_dup(_atmost_4990);
      kk_std_core_types__optional_drop(_match_1208, _ctx);
      kk_Unit;
      _x1459 = _atmost_4990; /*int*/
      goto _match1460;
    }
    _x1459 = kk_integer_from_small(-1); /*int*/
    _match1460: ;
    _x1458 = kk_std_core_ssize__t(_x1459, _ctx); /*ssize_t*/
    _x1453 = kk_std_text_regex_regex_exec_all(_x1454, _x1456, _x1457, _x1458, _ctx); /*list<list<sslice>>*/
    _x1436 = kk_std_text_regex_concat_replace(_x1453, kk_std_text_regex_new_groups_count_fun1462(_ctx), kk_std_core__new_Nil(_ctx), _ctx); /*string*/
  }
  return kk_std_core_count_1(_x1436, _ctx);
}
 
// Replace the first occurrence of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice>) -> e string) -> e string */ 
  kk_std_core__list _x1464;
  kk_box_t _x1465;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1465 = _x; /*any*/
  kk_ssize_t _x1466 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1467 = ((kk_ssize_t)1); /*ssize_t*/
  _x1464 = kk_std_text_regex_regex_exec_all(_x1465, s, _x1466, _x1467, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1464, repl, kk_std_core__new_Nil(_ctx), _ctx);
}
 
// Replace the first occurrence of `regex` with a replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match (`==$0`).


// lift anonymous function
struct kk_std_text_regex_replace_fun1475__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_fun1475_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_fun1475_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1475__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_fun1475__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_replace_fun1475_1, kk_context());
  _self->repl = repl;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_replace_fun1475_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1475__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_fun1475__t_1*, _fself);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl);}, {}, _ctx)
  return kk_std_text_regex_replace_captures(caps, repl, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_replace_fun1480__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_fun1480_1(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_fun1480_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1480__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_fun1480__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_replace_fun1480_1, kk_context());
  _self->repl = repl;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_replace_fun1480_1(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1480__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_fun1480__t_1*, _fself);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl);}, {}, _ctx)
  kk_std_core__list_drop(___wildcard__183__35, _ctx);
  return repl;
}

kk_string_t kk_std_text_regex_replace_1(kk_string_t s, kk_std_text_regex__regex regex0, kk_string_t repl, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string) -> string */ 
  bool _match_1206;
  kk_string_t _x1468 = kk_string_dup(repl); /*string*/
  kk_string_t _x1469;
  kk_define_string_literal(, _s1470, 1, "$")
  _x1469 = kk_string_dup(_s1470); /*string*/
  _match_1206 = kk_string_contains(_x1468,_x1469,kk_context()); /*bool*/
  if (_match_1206) {
    kk_std_core__list _x1471;
    kk_box_t _x1472;
    kk_box_t _x = regex0.obj;
    kk_box_dup(_x);
    kk_std_text_regex__regex_drop(regex0, _ctx);
    kk_Unit;
    _x1472 = _x; /*any*/
    kk_ssize_t _x1473 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x1474 = ((kk_ssize_t)1); /*ssize_t*/
    _x1471 = kk_std_text_regex_regex_exec_all(_x1472, s, _x1473, _x1474, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex_concat_replace(_x1471, kk_std_text_regex_new_replace_fun1475_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
  }
  kk_std_core__list _x1476;
  kk_box_t _x1477;
  kk_box_t _x0 = regex0.obj;
  kk_box_dup(_x0);
  kk_std_text_regex__regex_drop(regex0, _ctx);
  kk_Unit;
  _x1477 = _x0; /*any*/
  kk_ssize_t _x1478 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1479 = ((kk_ssize_t)1); /*ssize_t*/
  _x1476 = kk_std_text_regex_regex_exec_all(_x1477, s, _x1478, _x1479, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1476, kk_std_text_regex_new_replace_fun1480_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
}
 
// Split a string `s` in at most `atmost` parts using a regular expression `r` as separator.


// lift anonymous function
struct kk_std_text_regex_split_fun1489__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_text_regex_split_fun1489(kk_function_t _fself, kk_std_core__list _x11488, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_split_fun1489(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_split_fun1489, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_text_regex_split_fun1489(kk_function_t _fself, kk_std_core__list _x11488, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_text_regex_captured(_x11488, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_split_fun1490__t {
  struct kk_function_s _base;
  kk_function_t _b_1205_1202;
};
static kk_box_t kk_std_text_regex_split_fun1490(kk_function_t _fself, kk_box_t _b_1203, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_split_fun1490(kk_function_t _b_1205_1202, kk_context_t* _ctx) {
  struct kk_std_text_regex_split_fun1490__t* _self = kk_function_alloc_as(struct kk_std_text_regex_split_fun1490__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_split_fun1490, kk_context());
  _self->_b_1205_1202 = _b_1205_1202;
  return &_self->_base;
}

static kk_box_t kk_std_text_regex_split_fun1490(kk_function_t _fself, kk_box_t _b_1203, kk_context_t* _ctx) {
  struct kk_std_text_regex_split_fun1490__t* _self = kk_function_as(struct kk_std_text_regex_split_fun1490__t*, _fself);
  kk_function_t _b_1205_1202 = _self->_b_1205_1202; /* (matched : list<sslice>) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_1205_1202);}, {}, _ctx)
  kk_string_t _x1491;
  kk_std_core__list _x1492 = kk_std_core__list_unbox(_b_1203, _ctx); /*list<sslice>*/
  _x1491 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _b_1205_1202, (_b_1205_1202, _x1492, _ctx)); /*string*/
  return kk_string_box(_x1491);
}

kk_std_core__list kk_std_text_regex_split(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : optional<int>) -> list<string> */ 
  kk_std_core__list xs_976;
  kk_std_core__list _x1481;
  kk_box_t _x1482;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1482 = _x; /*any*/
  kk_ssize_t _x1483 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1484;
  kk_integer_t _x1485;
  if (kk_std_core_types__is_Optional(atmost)) {
    kk_box_t _box_x1200 = atmost._cons.Optional.value;
    kk_integer_t _atmost_925 = kk_integer_unbox(_box_x1200);
    kk_integer_dup(_atmost_925);
    kk_std_core_types__optional_drop(atmost, _ctx);
    kk_Unit;
    _x1485 = _atmost_925; /*int*/
    goto _match1486;
  }
  _x1485 = kk_integer_from_small(-1); /*int*/
  _match1486: ;
  _x1484 = kk_std_core_ssize__t(_x1485, _ctx); /*ssize_t*/
  _x1481 = kk_std_text_regex_regex_exec_all(_x1482, s, _x1483, _x1484, _ctx); /*list<list<sslice>>*/
  xs_976 = kk_std_text_regex_filter_non_matches(_x1481, _ctx); /*list<list<sslice>>*/
  kk_std_core__list _b_1204_1201 = xs_976; /*list<list<sslice>>*/;
  kk_function_t _b_1205_1202 = kk_std_text_regex_new_split_fun1489(_ctx); /*(matched : list<sslice>) -> string*/;
  return kk_std_core_map_5(_b_1204_1201, kk_std_text_regex_new_split_fun1490(_b_1205_1202, _ctx), _ctx);
}

// initialization
void kk_std_text_regex__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  kk_std_core__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_box_t _x1425;
    kk_string_t _x1426;
    kk_define_string_literal(, _s1427, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]")
    _x1426 = kk_string_dup(_s1427); /*string*/
    bool _x1428;
    kk_std_core_types__optional _match_1211 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1211)) {
      kk_box_t _box_x1196 = _match_1211._cons.Optional.value;
      bool _ignorecase_458 = kk_bool_unbox(_box_x1196);
      kk_std_core_types__optional_drop(_match_1211, _ctx);
      kk_Unit;
      _x1428 = _ignorecase_458; /*bool*/
      goto _match1429;
    }
    _x1428 = false; /*bool*/
    _match1429: ;
    bool _x1431;
    kk_std_core_types__optional _match_1210 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1210)) {
      kk_box_t _box_x1197 = _match_1210._cons.Optional.value;
      bool _multiline_462 = kk_bool_unbox(_box_x1197);
      kk_std_core_types__optional_drop(_match_1210, _ctx);
      kk_Unit;
      _x1431 = _multiline_462; /*bool*/
      goto _match1432;
    }
    _x1431 = false; /*bool*/
    _match1432: ;
    _x1425 = kk_std_text_regex_regex_create(_x1426, _x1428, _x1431, _ctx); /*any*/
    kk_string_t _x1434;
    kk_define_string_literal(, _s1435, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]")
    _x1434 = kk_string_dup(_s1435); /*string*/
    kk_std_text_regex_rx_nongroup = kk_std_text_regex__new_Regex(_x1425, _x1434, _ctx); /*std/text/regex/regex*/
  }
}

// termination
void kk_std_text_regex__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_text_regex__regex_drop(kk_std_text_regex_rx_nongroup, _ctx);
  kk_std_core__done(_ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
