// Koka generated module: "std/text/regex", koka version: 2.2.1
#include "std_text_regex.h"
/*---------------------------------------------------------------------------
  Copyright 2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/

#pragma clang diagnostic ignored "-Wdisabled-macro-expansion"
#define PCRE2_CODE_UNIT_WIDTH 8
#include <pcre2.h>

/* -----------------------------------------------------------------------
  Init/Done
------------------------------------------------------------------------*/
#define KK_CUSTOM_INIT  kk_regex_custom_init
#define KK_CUSTOM_DONE  kk_regex_custom_done

static pcre2_general_context* gen_ctx;
static pcre2_compile_context* cmp_ctx;
static pcre2_match_context*   match_ctx; 

static void* kk_pcre2_malloc( PCRE2_SIZE size, void* data ) {
  return kk_malloc( kk_to_ssize_t(size), kk_get_context() );  
}
static void kk_pcre2_free( void* p, void* data ) {
  kk_free(p);
}

static void kk_regex_custom_init( kk_context_t* ctx ) {
  gen_ctx = pcre2_general_context_create( &kk_pcre2_malloc, &kk_pcre2_free, NULL );
  if (gen_ctx != NULL) {
    match_ctx = pcre2_match_context_create( gen_ctx );
    cmp_ctx = pcre2_compile_context_create( gen_ctx );
    if (cmp_ctx != NULL) {
      pcre2_set_newline( cmp_ctx, PCRE2_NEWLINE_ANYCRLF );
      pcre2_set_bsr( cmp_ctx, PCRE2_BSR_ANYCRLF );
    }
  }
}

static void kk_regex_custom_done( kk_context_t* ctx ) {
  if (cmp_ctx != NULL) {
    pcre2_compile_context_free(cmp_ctx);
    cmp_ctx = NULL;
  }
  if (match_ctx != NULL) {
    pcre2_match_context_free(match_ctx);
    match_ctx = NULL;
  }
  if (gen_ctx != NULL) {
    pcre2_general_context_free(gen_ctx);
    gen_ctx = NULL;
  }
}


/* -----------------------------------------------------------------------
  Compile
------------------------------------------------------------------------*/

static void kk_regex_free( void* pre, kk_block_t* b, kk_context_t* ctx ) {
  KK_UNUSED(ctx);
  pcre2_code* re = (pcre2_code*)pre;
  //kk_info_message( "free regex at %p\n", re );
  if (re != NULL) pcre2_code_free(re);
}

#define KK_REGEX_OPTIONS  (PCRE2_ALT_BSUX | PCRE2_EXTRA_ALT_BSUX | PCRE2_MATCH_UNSET_BACKREF /* javascript compat */ \
                          | PCRE2_NEVER_BACKSLASH_C | PCRE2_NEVER_UCP | PCRE2_UTF /* utf-8 safety */ \
                          )

static kk_box_t kk_regex_create( kk_string_t pat, bool ignore_case, bool multi_line, kk_context_t* ctx ) {
  kk_ssize_t len;
  const uint8_t* cpat = kk_string_buf_borrow( pat, &len );
  PCRE2_SIZE errofs = 0;
  int        errnum = 0;
  uint32_t   options = KK_REGEX_OPTIONS;
  if (ignore_case) options |= PCRE2_CASELESS;
  if (multi_line)  options |= PCRE2_MULTILINE;
  pcre2_code* re = pcre2_compile( cpat, PCRE2_ZERO_TERMINATED, options, &errnum, &errofs, cmp_ctx);
  //kk_info_message( "create regex: err:%i, at %p\n", (re==NULL ? 0 : errnum), re );
  kk_string_drop(pat,ctx);
  return kk_cptr_raw_box( &kk_regex_free, re, ctx );
}


/* -----------------------------------------------------------------------
  Match
------------------------------------------------------------------------*/

/*
static void kk_match_data_free( void* pmd, kk_block_t* b ) {
  kk_info_message( "free match_data\n" );
  pcre2_match_data* md = (pcre2_match_data*)pmd;
  if (md != NULL) pcre2_match_data_free(md);
}
*/

static kk_std_core__list kk_regex_exec_ex( pcre2_code* re, pcre2_match_data* match_data, 
                                           kk_string_t str_borrow, const uint8_t* cstr, kk_ssize_t len, bool allow_empty, 
                                           kk_ssize_t start, kk_ssize_t* mstart, kk_ssize_t* end, int* res, kk_context_t* ctx ) 
{
  // match
  kk_std_core__list hd  = kk_std_core__new_Nil(ctx);
  uint32_t options = 0;
  if (!allow_empty) options |= (PCRE2_NOTEMPTY_ATSTART | PCRE2_ANCHORED);
  int rc = pcre2_match( re, cstr, len, start, options, match_data, match_ctx );
  if (res != NULL) *res = rc;    
  if (rc > 0) {    
    // extract captures
    uint32_t    gcount = pcre2_get_ovector_count(match_data);
    PCRE2_SIZE* groups = pcre2_get_ovector_pointer(match_data);
    for( uint32_t i = gcount; i > 0; ) {
      i--;
      kk_ssize_t sstart = groups[i*2];       // on no-match, sstart and send == -1.
      kk_ssize_t send   = groups[i*2 + 1];
      kk_assert(send >= sstart);
      kk_std_core__sslice sslice = kk_std_core__new_Sslice( kk_string_dup(str_borrow), sstart, send - sstart, ctx ); 
      hd = kk_std_core__new_Cons(kk_reuse_null,kk_std_core__sslice_box(sslice,ctx), hd, ctx);
      if (i == 0) {
        if (mstart != NULL) { *mstart = sstart; }
        if (end    != NULL) { *end = send; }
      }
    }
  }
  return hd;
}


static kk_std_core__list kk_regex_exec( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_context_t* ctx ) 
{
  // unpack
  pcre2_match_data* match_data = NULL;
  kk_std_core__list res = kk_std_core__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox(bre);
  if (re == NULL) goto done;    
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;  
  kk_ssize_t len;
  const uint8_t* cstr = kk_string_buf_borrow(str, &len );  

  // and match
  res = kk_regex_exec_ex( re, match_data, str, cstr, len, true, start, NULL, NULL, NULL, ctx );

done:  
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

static kk_std_core__list kk_regex_exec_all( kk_box_t bre, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* ctx ) {
  // unpack
  if (atmost < 0) atmost = KK_SSIZE_MAX;
  pcre2_match_data* match_data = NULL;
  kk_std_core__list res = kk_std_core__new_Nil(ctx);
  pcre2_code* re = (pcre2_code*)kk_cptr_raw_unbox(bre);
  if (re == NULL) goto done;    
  match_data = pcre2_match_data_create_from_pattern(re, gen_ctx);
  if (match_data==NULL) goto done;  
  kk_ssize_t len;
  const uint8_t* cstr = kk_string_buf_borrow(str, &len );  

  // and match
  kk_std_core__list* tail = NULL;
  bool allow_empty = true;
  int rc = 1;    
  kk_ssize_t next = start;
  while( rc > 0 && start < len && atmost > 0) {
    atmost--;
    rc = 0;
    kk_ssize_t mstart = start;
    kk_std_core__list cap = kk_regex_exec_ex( re, match_data, str, cstr, len, allow_empty, start, &mstart, &next, &rc, ctx );
    if (rc > 0) {
      // found a match; 
      // push string up to match, and the actual matched regex
      kk_std_core__sslice pre = kk_std_core__new_Sslice( kk_string_dup(str), start, mstart - start, ctx ); 
      kk_std_core__list   prelist = kk_std_core__new_Cons( kk_reuse_null, kk_std_core__sslice_box(pre,ctx), kk_std_core__new_Nil(ctx), ctx );
      kk_std_core__list   capcons = kk_std_core__new_Cons( kk_reuse_null, kk_std_core__list_box(cap,ctx), kk_std_core__new_Nil(ctx) /*tail*/, ctx );
      kk_std_core__list   cons = kk_std_core__new_Cons( kk_reuse_null, kk_std_core__list_box(prelist,ctx), capcons, ctx );
      if (tail==NULL) res = cons;
                 else *tail = cons;
      tail = &kk_std_core__as_Cons(capcons)->tail;
      allow_empty = (next > start);
      start = next;
    }
    else if (rc <= 0 && !allow_empty) {
      // skip one character and try again
      // todo: handle cr/lf pairs better?
      const uint8_t* p = kk_utf8_next( cstr + start );
      start = (p - cstr);
      allow_empty = true;
      rc = 1;
    }
  }
  
  // push final string part as well and end the list
  kk_std_core__sslice post    = kk_std_core__new_Sslice( kk_string_dup(str), next, len - next, ctx ); 
  kk_std_core__list   postlist= kk_std_core__new_Cons( kk_reuse_null, kk_std_core__sslice_box(post,ctx), kk_std_core__new_Nil(ctx), ctx );
  kk_std_core__list   cons    = kk_std_core__new_Cons( kk_reuse_null, kk_std_core__list_box(postlist,ctx), kk_std_core__new_Nil(ctx), ctx );
  if (tail==NULL) res = cons;
            else *tail = cons;  

done:  
  if (match_data != NULL) {
    //kk_info_message( "free match data: %p\n", match_data );
    pcre2_match_data_free(match_data);
  }
  kk_string_drop(str,ctx);
  kk_box_drop(bre,ctx);
  return res;
}

/*
kk_std_core__sslice kk_slice_upto( struct kk_std_core_Sslice slice1, struct kk_std_core_Sslice slice2, kk_context_t* ctx ) {
  kk_ssize_t start = slice1.start;
  kk_ssize_t len   = (slice1.start <= slice2.start ? slice2.start - slice1.start : -1);
  return kk_std_core__new_Sslice(slice1.str, start, len, ctx);
}
*/
 
// monadic lift

kk_std_text_regex__regex kk_std_text_regex__mlift1037_op(kk_std_text_regex__regex _this, kk_std_core_types__optional obj0, kk_string_t _c_1020, kk_context_t* _ctx) { /* (regex, obj0 : optional<any>, string) -> regex */ 
  kk_box_t _x1353;
  if (kk_std_core_types__is_Optional(obj0)) {
    kk_box_t _box_x1080 = obj0._cons.Optional.value;
    kk_box_t _obj_104 = kk_box_unbox(_box_x1080, NULL);
    kk_box_dup(_obj_104);
    kk_std_core_types__optional_drop(obj0, _ctx);
    kk_Unit;
    kk_std_text_regex__regex_drop(_this, _ctx);
    kk_Unit;
    _x1353 = _obj_104; /*any*/
    goto _match1354;
  }
  kk_box_t _x = _this.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(_this, _ctx);
  kk_Unit;
  _x1353 = _x; /*any*/
  _match1354: ;
  return kk_std_text_regex__new_Regex(_x1353, _c_1020, _ctx);
}


// lift anonymous function
struct kk_std_text_regex__copy_fun1356__t {
  struct kk_function_s _base;
  kk_std_text_regex__regex _this;
  kk_std_core_types__optional obj0;
};
static kk_std_text_regex__regex kk_std_text_regex__copy_fun1356(kk_function_t _fself, kk_string_t _c_1020, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex__new_copy_fun1356(kk_std_text_regex__regex _this, kk_std_core_types__optional obj0, kk_context_t* _ctx) {
  struct kk_std_text_regex__copy_fun1356__t* _self = kk_function_alloc_as(struct kk_std_text_regex__copy_fun1356__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex__copy_fun1356, kk_context());
  _self->_this = _this;
  _self->obj0 = obj0;
  return &_self->_base;
}

static kk_std_text_regex__regex kk_std_text_regex__copy_fun1356(kk_function_t _fself, kk_string_t _c_1020, kk_context_t* _ctx) {
  struct kk_std_text_regex__copy_fun1356__t* _self = kk_function_as(struct kk_std_text_regex__copy_fun1356__t*, _fself);
  kk_std_text_regex__regex _this = _self->_this; /* std/text/regex/regex */
  kk_std_core_types__optional obj0 = _self->obj0; /* optional<any> */
  kk_drop_match(_self, {kk_std_text_regex__regex_dup(_this);kk_std_core_types__optional_dup(obj0);}, {}, _ctx)
  kk_box_t _x1357;
  if (kk_std_core_types__is_Optional(obj0)) {
    kk_box_t _box_x1081 = obj0._cons.Optional.value;
    kk_box_t _obj_104 = kk_box_unbox(_box_x1081, NULL);
    kk_box_dup(_obj_104);
    kk_std_core_types__optional_drop(obj0, _ctx);
    kk_Unit;
    kk_std_text_regex__regex_drop(_this, _ctx);
    kk_Unit;
    _x1357 = _obj_104; /*any*/
    goto _match1358;
  }
  kk_box_t _x = _this.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(_this, _ctx);
  kk_Unit;
  _x1357 = _x; /*any*/
  _match1358: ;
  return kk_std_text_regex__new_Regex(_x1357, _c_1020, _ctx);
}


// lift anonymous function
struct kk_std_text_regex__copy_fun1361__t {
  struct kk_function_s _base;
  kk_function_t _b_1086_1082;
};
static kk_box_t kk_std_text_regex__copy_fun1361(kk_function_t _fself, kk_box_t _b_1083, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex__new_copy_fun1361(kk_function_t _b_1086_1082, kk_context_t* _ctx) {
  struct kk_std_text_regex__copy_fun1361__t* _self = kk_function_alloc_as(struct kk_std_text_regex__copy_fun1361__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex__copy_fun1361, kk_context());
  _self->_b_1086_1082 = _b_1086_1082;
  return &_self->_base;
}

static kk_box_t kk_std_text_regex__copy_fun1361(kk_function_t _fself, kk_box_t _b_1083, kk_context_t* _ctx) {
  struct kk_std_text_regex__copy_fun1361__t* _self = kk_function_as(struct kk_std_text_regex__copy_fun1361__t*, _fself);
  kk_function_t _b_1086_1082 = _self->_b_1086_1082; /* (string) -> std/text/regex/regex */
  kk_drop_match(_self, {kk_function_dup(_b_1086_1082);}, {}, _ctx)
  kk_std_text_regex__regex _x1362;
  kk_string_t _x1363 = kk_string_unbox(_b_1083); /*string*/
  _x1362 = kk_function_call(kk_std_text_regex__regex, (kk_function_t, kk_string_t, kk_context_t*), _b_1086_1082, (_b_1086_1082, _x1363, _ctx)); /*std/text/regex/regex*/
  return kk_std_text_regex__regex_box(_x1362, _ctx);
}

kk_std_text_regex__regex kk_std_text_regex__copy(kk_std_text_regex__regex _this, kk_std_core_types__optional obj0, kk_std_core_types__optional src0, kk_context_t* _ctx) { /* (regex, obj : optional<any>, src : optional<string>) -> regex */ 
  if (kk_yielding(kk_context())) {
    kk_std_core_types__optional_drop(src0, _ctx);
    kk_function_t _b_1086_1082 = kk_std_text_regex__new_copy_fun1356(_this, obj0, _ctx); /*(string) -> std/text/regex/regex*/;
    kk_box_t _x1360 = kk_std_core_hnd_yield_extend(kk_std_text_regex__new_copy_fun1361(_b_1086_1082, _ctx), _ctx); /*3858*/
    return kk_std_text_regex__regex_unbox(_x1360, _ctx);
  }
  kk_box_t _x1364;
  if (kk_std_core_types__is_Optional(obj0)) {
    kk_box_t _box_x1084 = obj0._cons.Optional.value;
    kk_box_t _obj_1040 = kk_box_unbox(_box_x1084, NULL);
    kk_box_dup(_obj_1040);
    kk_std_core_types__optional_drop(obj0, _ctx);
    kk_Unit;
    _x1364 = _obj_1040; /*any*/
    goto _match1365;
  }
  kk_box_t _x1 = _this.obj;
  kk_box_dup(_x1);
  _x1364 = _x1; /*any*/
  _match1365: ;
  kk_string_t _x1367;
  if (kk_std_core_types__is_Optional(src0)) {
    kk_box_t _box_x1085 = src0._cons.Optional.value;
    kk_string_t _src_110 = kk_string_unbox(_box_x1085);
    kk_string_dup(_src_110);
    kk_std_core_types__optional_drop(src0, _ctx);
    kk_Unit;
    kk_std_text_regex__regex_drop(_this, _ctx);
    kk_Unit;
    _x1367 = _src_110; /*string*/
    goto _match1368;
  }
  kk_string_t _x0 = _this.src;
  kk_string_dup(_x0);
  kk_std_text_regex__regex_drop(_this, _ctx);
  kk_Unit;
  _x1367 = _x0; /*string*/
  _match1368: ;
  return kk_std_text_regex__new_Regex(_x1364, _x1367, _ctx);
}
 
// Return the full matched string of a capture group

kk_string_t kk_std_text_regex_captured(kk_std_core__list matched0, kk_context_t* _ctx) { /* (matched : list<sslice>) -> string */ 
  if (kk_std_core__is_Cons(matched0)) {
    struct kk_std_core_Cons* _con1370 = kk_std_core__as_Cons(matched0);
    kk_box_t _box_x1087 = _con1370->head;
    kk_std_core__list _pat3 = _con1370->tail;
    kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1087, NULL);
    if (kk_likely(kk_std_core__list_is_unique(matched0))) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(s);
      kk_box_drop(_box_x1087, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(matched0);
    }
    else {
      kk_std_core__sslice_dup(s);
      kk_Unit;
      kk_std_core__list_decref(matched0, _ctx);
      kk_Unit;
    }
    return kk_std_core_string_3(s, _ctx);
  }
  return kk_string_empty();
}

kk_std_core__list kk_std_text_regex_regex_exec(kk_box_t regex0, kk_string_t str, kk_ssize_t start, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t) -> list<sslice> */ 
  return kk_regex_exec(regex0,str,start,kk_context());
}

kk_std_core__list kk_std_text_regex_regex_exec_all(kk_box_t regex0, kk_string_t str, kk_ssize_t start, kk_ssize_t atmost, kk_context_t* _ctx) { /* (regex : any, str : string, start : ssize_t, atmost : ssize_t) -> list<list<sslice>> */ 
  return kk_regex_exec_all(regex0,str,start,atmost,kk_context());
}

kk_box_t kk_std_text_regex_regex_create(kk_string_t _arg1, bool _arg2, bool _arg3, kk_context_t* _ctx) { /* (string, bool, bool) -> any */ 
  return kk_regex_create(_arg1,_arg2,_arg3,kk_context());
}
 
// lifted
// specialized std/core/map.5 to parameters [f]

kk_std_core__list kk_std_text_regex__ctail_lift996_captures(kk_std_core__list xs0, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (xs0 : list<list<sslice>>, ctail<list<string>>) -> list<string> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con1375 = kk_std_core__as_Cons(xs0);
    kk_box_t _box_x1088 = _con1375->head;
    kk_std_core__list xx = _con1375->tail;
    kk_std_core__list x = kk_std_core__list_unbox(_box_x1088, NULL);
    kk_reuse_t _ru_1328 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      _ru_1328 = (kk_std_core__list_reuse(xs0));
    }
    else {
      kk_std_core__list_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    kk_string_t _ctail_1007;
    if (kk_std_core__is_Cons(x)) {
      struct kk_std_core_Cons* _con1378 = kk_std_core__as_Cons(x);
      kk_box_t _box_x1089 = _con1378->head;
      kk_std_core__list _pat3 = _con1378->tail;
      kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1089, NULL);
      if (kk_likely(kk_std_core__list_is_unique(x))) {
        kk_std_core__list_drop(_pat3, _ctx);
        kk_std_core__sslice_dup(s);
        kk_box_drop(_box_x1089, _ctx);
        kk_Unit;
        kk_Unit;
        kk_std_core__list_free(x);
      }
      else {
        kk_std_core__sslice_dup(s);
        kk_Unit;
        kk_std_core__list_decref(x, _ctx);
        kk_Unit;
      }
      _ctail_1007 = kk_std_core_string_3(s, _ctx); /*string*/
      goto _match1377;
    }
    _ctail_1007 = kk_string_empty(); /*string*/
    _match1377: ;
    kk_std_core__list _ctail_1008 = kk_std_core__list_hole(); /*list<string>*/;
    kk_std_core__list _ctail_1009 = kk_std_core__new_Cons(_ru_1328, kk_string_box(_ctail_1007), _ctail_1008, _ctx); /*list<string>*/;
    { // tailcall
      kk_std_core_types__ctail _x1381;
      kk_box_t* field_1048 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_1009)->tail)); /*cfield<list<string>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x1100 = _acc._cons.CTail._field1;
        kk_box_t* last = _acc._cons.CTail._field2;
        kk_std_core__list head = kk_std_core__list_unbox(_box_x1100, NULL);
        kk_std_core__list_dup(head);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last) = kk_std_core__list_box(_ctail_1009, _ctx);
        kk_std_core__list _b_1114_1105 = head; /*list<string>*/;
        kk_box_t* _b_1115_1106 = field_1048; /*cfield<list<string>>*/;
        _x1381 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1114_1105, _ctx), _b_1115_1106, _ctx); /*ctail<58>*/
        goto _match1382;
      }
      kk_std_core__list _b_1116_1107 = _ctail_1009; /*list<string>*/;
      kk_box_t* _b_1117_1108 = field_1048; /*cfield<list<string>>*/;
      _x1381 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1116_1107, _ctx), _b_1117_1108, _ctx); /*ctail<58>*/
      _match1382: ;
      xs0 = xx;
      _acc = _x1381;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x1109 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x1109, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last0) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head0;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted
// specialized std/core/map.5 to parameters [f]

kk_std_core__list kk_std_text_regex__lift996_captures(kk_std_core__list xs00, kk_context_t* _ctx) { /* (xs0 : list<list<sslice>>) -> list<string> */ 
  return kk_std_text_regex__ctail_lift996_captures(xs00, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// monadic lift

kk_string_t kk_std_text_regex__mlift1038_concat_replace(kk_std_core__list acc, kk_std_core__list mm, kk_std_core__list pre, kk_function_t repl, kk_string_t _y_1021, kk_context_t* _ctx) { /* forall<e> (acc : list<string>, mm : list<list<sslice>>, pre : list<sslice>, repl : (list<sslice>) -> e string, string) -> e string */ 
  kk_std_core__list _x1385;
  kk_string_t _b_1123_1121 = _y_1021; /*string*/;
  kk_std_core__list _b_1124_1122;
  kk_string_t _b_1125_1119;
  if (kk_std_core__is_Cons(pre)) {
    struct kk_std_core_Cons* _con1387 = kk_std_core__as_Cons(pre);
    kk_box_t _box_x1118 = _con1387->head;
    kk_std_core__list _pat3 = _con1387->tail;
    kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1118, NULL);
    if (kk_likely(kk_std_core__list_is_unique(pre))) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(s);
      kk_box_drop(_box_x1118, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(pre);
    }
    else {
      kk_std_core__sslice_dup(s);
      kk_Unit;
      kk_std_core__list_decref(pre, _ctx);
      kk_Unit;
    }
    _b_1125_1119 = kk_std_core_string_3(s, _ctx); /*string*/
    goto _match1386;
  }
  _b_1125_1119 = kk_string_empty(); /*string*/
  _match1386: ;
  kk_std_core__list _b_1126_1120 = acc; /*list<string>*/;
  _b_1124_1122 = kk_std_core__new_Cons(kk_reuse_null, kk_string_box(_b_1125_1119), _b_1126_1120, _ctx); /*list<string>*/
  _x1385 = kk_std_core__new_Cons(kk_reuse_null, kk_string_box(_b_1123_1121), _b_1124_1122, _ctx); /*list<61>*/
  return kk_std_text_regex_concat_replace(mm, repl, _x1385, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_concat_replace_fun1395__t {
  struct kk_function_s _base;
  kk_std_core__list acc0;
  kk_std_core__list mm0;
  kk_std_core__list pre0;
  kk_function_t repl0;
};
static kk_string_t kk_std_text_regex_concat_replace_fun1395(kk_function_t _fself, kk_string_t _y_10210, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_concat_replace_fun1395(kk_std_core__list acc0, kk_std_core__list mm0, kk_std_core__list pre0, kk_function_t repl0, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1395__t* _self = kk_function_alloc_as(struct kk_std_text_regex_concat_replace_fun1395__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_concat_replace_fun1395, kk_context());
  _self->acc0 = acc0;
  _self->mm0 = mm0;
  _self->pre0 = pre0;
  _self->repl0 = repl0;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_concat_replace_fun1395(kk_function_t _fself, kk_string_t _y_10210, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1395__t* _self = kk_function_as(struct kk_std_text_regex_concat_replace_fun1395__t*, _fself);
  kk_std_core__list acc0 = _self->acc0; /* list<string> */
  kk_std_core__list mm0 = _self->mm0; /* list<list<sslice>> */
  kk_std_core__list pre0 = _self->pre0; /* list<sslice> */
  kk_function_t repl0 = _self->repl0; /* (list<sslice>) -> 321 string */
  kk_drop_match(_self, {kk_std_core__list_dup(acc0);kk_std_core__list_dup(mm0);kk_std_core__list_dup(pre0);kk_function_dup(repl0);}, {}, _ctx)
  return kk_std_text_regex__mlift1038_concat_replace(acc0, mm0, pre0, repl0, _y_10210, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_concat_replace_fun1397__t {
  struct kk_function_s _base;
  kk_function_t _b_1144_1129;
};
static kk_box_t kk_std_text_regex_concat_replace_fun1397(kk_function_t _fself, kk_box_t _b_1130, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_concat_replace_fun1397(kk_function_t _b_1144_1129, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1397__t* _self = kk_function_alloc_as(struct kk_std_text_regex_concat_replace_fun1397__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_concat_replace_fun1397, kk_context());
  _self->_b_1144_1129 = _b_1144_1129;
  return &_self->_base;
}

static kk_box_t kk_std_text_regex_concat_replace_fun1397(kk_function_t _fself, kk_box_t _b_1130, kk_context_t* _ctx) {
  struct kk_std_text_regex_concat_replace_fun1397__t* _self = kk_function_as(struct kk_std_text_regex_concat_replace_fun1397__t*, _fself);
  kk_function_t _b_1144_1129 = _self->_b_1144_1129; /* (string) -> 321 string */
  kk_drop_match(_self, {kk_function_dup(_b_1144_1129);}, {}, _ctx)
  kk_string_t _x1398;
  kk_string_t _x1399 = kk_string_unbox(_b_1130); /*string*/
  _x1398 = kk_function_call(kk_string_t, (kk_function_t, kk_string_t, kk_context_t*), _b_1144_1129, (_b_1144_1129, _x1399, _ctx)); /*string*/
  return kk_string_box(_x1398);
}

kk_string_t kk_std_text_regex_concat_replace(kk_std_core__list matches, kk_function_t repl0, kk_std_core__list acc0, kk_context_t* _ctx) { /* forall<e> (matches : list<list<sslice>>, repl : (list<sslice>) -> e string, acc : list<string>) -> e string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(matches)) {
    struct kk_std_core_Cons* _con1390 = kk_std_core__as_Cons(matches);
    kk_box_t _box_x1127 = _con1390->head;
    kk_std_core__list _pat0 = _con1390->tail;
    if (kk_std_core__is_Cons(_pat0)) {
      struct kk_std_core_Cons* _con1392 = kk_std_core__as_Cons(_pat0);
      kk_std_core__list pre0 = kk_std_core__list_unbox(_box_x1127, NULL);
      kk_box_t _box_x1128 = _con1392->head;
      kk_std_core__list mm0 = _con1392->tail;
      kk_std_core__list m = kk_std_core__list_unbox(_box_x1128, NULL);
      kk_reuse_t _ru_1331 = kk_reuse_null; /*reuse*/;
      kk_reuse_t _ru_1332 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(matches))) {
        if (kk_likely(kk_std_core__list_is_unique(_pat0))) {
          _ru_1332 = (kk_std_core__list_reuse(_pat0));
        }
        else {
          kk_std_core__list_dup(m);
          kk_std_core__list_dup(mm0);
          kk_Unit;
          kk_std_core__list_decref(_pat0, _ctx);
          kk_Unit;
        }
        kk_Unit;
        _ru_1331 = (kk_std_core__list_reuse(matches));
      }
      else {
        kk_std_core__list_dup(m);
        kk_std_core__list_dup(mm0);
        kk_std_core__list_dup(pre0);
        kk_Unit;
        kk_std_core__list_decref(matches, _ctx);
        kk_Unit;
      }
      kk_string_t x_1051;
      kk_function_t _x1394 = kk_function_dup(repl0); /*(list<sslice>) -> 321 string*/
      x_1051 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _x1394, (_x1394, m, _ctx)); /*string*/
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_1331);
        kk_reuse_drop(_ru_1332);
        kk_string_drop(x_1051, _ctx);
        kk_function_t _b_1144_1129 = kk_std_text_regex_new_concat_replace_fun1395(acc0, mm0, pre0, repl0, _ctx); /*(string) -> 321 string*/;
        kk_box_t _x1396 = kk_std_core_hnd_yield_extend(kk_std_text_regex_new_concat_replace_fun1397(_b_1144_1129, _ctx), _ctx); /*3858*/
        return kk_string_unbox(_x1396);
      }
      { // tailcall
        kk_std_core__list _x1400;
        kk_string_t _b_1145_1134 = x_1051; /*string*/;
        kk_std_core__list _b_1146_1135;
        kk_string_t _b_1147_1132;
        if (kk_std_core__is_Cons(pre0)) {
          struct kk_std_core_Cons* _con1402 = kk_std_core__as_Cons(pre0);
          kk_box_t _box_x1131 = _con1402->head;
          kk_std_core__list _pat30 = _con1402->tail;
          kk_std_core__sslice s0 = kk_std_core__sslice_unbox(_box_x1131, NULL);
          if (kk_likely(kk_std_core__list_is_unique(pre0))) {
            kk_std_core__list_drop(_pat30, _ctx);
            kk_std_core__sslice_dup(s0);
            kk_box_drop(_box_x1131, _ctx);
            kk_Unit;
            kk_Unit;
            kk_std_core__list_free(pre0);
          }
          else {
            kk_std_core__sslice_dup(s0);
            kk_Unit;
            kk_std_core__list_decref(pre0, _ctx);
            kk_Unit;
          }
          _b_1147_1132 = kk_std_core_string_3(s0, _ctx); /*string*/
          goto _match1401;
        }
        _b_1147_1132 = kk_string_empty(); /*string*/
        _match1401: ;
        kk_std_core__list _b_1148_1133 = acc0; /*list<string>*/;
        _b_1146_1135 = kk_std_core__new_Cons(_ru_1332, kk_string_box(_b_1147_1132), _b_1148_1133, _ctx); /*list<string>*/
        _x1400 = kk_std_core__new_Cons(_ru_1331, kk_string_box(_b_1145_1134), _b_1146_1135, _ctx); /*list<61>*/
        matches = mm0;
        acc0 = _x1400;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(matches)) {
    struct kk_std_core_Cons* _con1405 = kk_std_core__as_Cons(matches);
    kk_box_t _box_x1136 = _con1405->head;
    kk_std_core__list _pat200 = _con1405->tail;
    kk_std_core__list post = kk_std_core__list_unbox(_box_x1136, NULL);
    kk_function_drop(repl0, _ctx);
    kk_reuse_t _ru_1334 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(matches))) {
      _ru_1334 = (kk_std_core__list_reuse(matches));
    }
    else {
      kk_std_core__list_dup(post);
      kk_Unit;
      kk_std_core__list_decref(matches, _ctx);
      kk_Unit;
    }
    kk_string_t _b_1140_1138;
    if (kk_std_core__is_Cons(post)) {
      struct kk_std_core_Cons* _con1408 = kk_std_core__as_Cons(post);
      kk_box_t _box_x1137 = _con1408->head;
      kk_std_core__list _pat300 = _con1408->tail;
      kk_std_core__sslice s00 = kk_std_core__sslice_unbox(_box_x1137, NULL);
      if (kk_likely(kk_std_core__list_is_unique(post))) {
        kk_std_core__list_drop(_pat300, _ctx);
        kk_std_core__sslice_dup(s00);
        kk_box_drop(_box_x1137, _ctx);
        kk_Unit;
        kk_Unit;
        kk_std_core__list_free(post);
      }
      else {
        kk_std_core__sslice_dup(s00);
        kk_Unit;
        kk_std_core__list_decref(post, _ctx);
        kk_Unit;
      }
      _b_1140_1138 = kk_std_core_string_3(s00, _ctx); /*string*/
      goto _match1407;
    }
    _b_1140_1138 = kk_string_empty(); /*string*/
    _match1407: ;
    kk_std_core__list xs0_17145;
    kk_std_core__list _x1411 = kk_std_core__new_Cons(_ru_1334, kk_string_box(_b_1140_1138), acc0, _ctx); /*list<61>*/
    xs0_17145 = kk_std_core__lift17201_reverse_join(kk_std_core__new_Nil(_ctx), _x1411, _ctx); /*list<string>*/
    if (kk_std_core__is_Nil(xs0_17145)) {
      return kk_string_empty();
    }
    struct kk_std_core_Cons* _con1413 = kk_std_core__as_Cons(xs0_17145);
    kk_box_t _box_x1142 = _con1413->head;
    kk_std_core__list xx0 = _con1413->tail;
    kk_string_t x0 = kk_string_unbox(_box_x1142);
    if (kk_likely(kk_std_core__list_is_unique(xs0_17145))) {
      kk_std_core__list_free(xs0_17145);
    }
    else {
      kk_string_dup(x0);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs0_17145, _ctx);
      kk_Unit;
    }
    kk_string_t _x1415 = kk_string_empty(); /*string*/
    return kk_std_core__lift17202_reverse_join(_x1415, xx0, x0, _ctx);
  }
  kk_function_drop(repl0, _ctx);
  kk_std_core__list xs0_171450 = kk_std_core__lift17201_reverse_join(kk_std_core__new_Nil(_ctx), acc0, _ctx); /*list<string>*/;
  if (kk_std_core__is_Nil(xs0_171450)) {
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con1418 = kk_std_core__as_Cons(xs0_171450);
  kk_box_t _box_x1143 = _con1418->head;
  kk_std_core__list xx00 = _con1418->tail;
  kk_string_t x00 = kk_string_unbox(_box_x1143);
  if (kk_likely(kk_std_core__list_is_unique(xs0_171450))) {
    kk_std_core__list_free(xs0_171450);
  }
  else {
    kk_string_dup(x00);
    kk_std_core__list_dup(xx00);
    kk_Unit;
    kk_std_core__list_decref(xs0_171450, _ctx);
    kk_Unit;
  }
  kk_string_t _x1420 = kk_string_empty(); /*string*/
  return kk_std_core__lift17202_reverse_join(_x1420, xx00, x00, _ctx);
}
 
// Does a regular expression pattern occur in a string `s`?
// (note: called `test` in javascript)

bool kk_std_text_regex_contains(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> bool */ 
  kk_std_core__list list0_944;
  kk_box_t _x1424;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1424 = _x; /*any*/
  kk_ssize_t _x1425 = ((kk_ssize_t)0); /*ssize_t*/
  list0_944 = kk_std_text_regex_regex_exec(_x1424, s, _x1425, _ctx); /*list<sslice>*/
  if (kk_std_core__is_Cons(list0_944)) {
    struct kk_std_core_Cons* _con1426 = kk_std_core__as_Cons(list0_944);
    kk_box_t _box_x1149 = _con1426->head;
    kk_std_core__list_dropn(list0_944, ((int32_t)2), _ctx);
    return true;
  }
  return false;
}
 
// Match a regular expression `regex` over a string `s`.
// Matches at most `atmost` times (and matches all by default).
// Returns always an odd number of elements where every even
// element is a match and the odd ones the string parts between the
// matches.
// See also `find-all` and `strings`.

kk_std_core__list kk_std_text_regex_exec_all(kk_std_text_regex__regex regex0, kk_string_t s, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (regex : regex, s : string, atmost : optional<int>) -> list<list<sslice>> */ 
  kk_box_t _x1428;
  kk_box_t _x = regex0.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(regex0, _ctx);
  kk_Unit;
  _x1428 = _x; /*any*/
  kk_ssize_t _x1429 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1430;
  kk_integer_t _x1431;
  if (kk_std_core_types__is_Optional(atmost)) {
    kk_box_t _box_x1150 = atmost._cons.Optional.value;
    kk_integer_t _atmost_346 = kk_integer_unbox(_box_x1150);
    kk_integer_dup(_atmost_346);
    kk_std_core_types__optional_drop(atmost, _ctx);
    kk_Unit;
    _x1431 = _atmost_346; /*int*/
    goto _match1432;
  }
  _x1431 = kk_integer_from_small(-1); /*int*/
  _match1432: ;
  _x1430 = kk_std_core_ssize__t(_x1431, _ctx); /*ssize_t*/
  return kk_std_text_regex_regex_exec_all(_x1428, s, _x1429, _x1430, _ctx);
}
 
// Filter only for the matched parts.

kk_std_core__list kk_std_text_regex__ctail_filter_matches(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice>>, ctail<list<list<sslice>>>) -> list<list<sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con1434 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x1151 = _con1434->head;
    kk_std_core__list _pat1 = _con1434->tail;
    if (kk_std_core__is_Cons(_pat1)) {
      struct kk_std_core_Cons* _con1436 = kk_std_core__as_Cons(_pat1);
      kk_box_t _box_x1152 = _con1436->head;
      kk_std_core__list mm = _con1436->tail;
      kk_std_core__list m = kk_std_core__list_unbox(_box_x1152, NULL);
      kk_reuse_t _ru_1340 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        if (kk_likely(kk_std_core__list_is_unique(_pat1))) {
          _ru_1340 = (kk_std_core__list_reuse(_pat1));
        }
        else {
          kk_std_core__list_dup(m);
          kk_std_core__list_dup(mm);
          kk_Unit;
          kk_std_core__list_decref(_pat1, _ctx);
          kk_Unit;
        }
        kk_box_drop(_box_x1151, _ctx);
        kk_Unit;
        kk_Unit;
        kk_std_core__list_free(xs);
      }
      else {
        kk_std_core__list_dup(m);
        kk_std_core__list_dup(mm);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      kk_std_core__list _ctail_1010 = kk_std_core__list_hole(); /*list<list<sslice>>*/;
      kk_std_core__list _ctail_1011 = kk_std_core__new_Cons(_ru_1340, kk_std_core__list_box(m, _ctx), _ctail_1010, _ctx); /*list<list<sslice>>*/;
      { // tailcall
        kk_std_core_types__ctail _x1438;
        kk_box_t* field_1056 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_1011)->tail)); /*cfield<list<list<sslice>>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x1163 = _acc._cons.CTail._field1;
          kk_box_t* last = _acc._cons.CTail._field2;
          kk_std_core__list head = kk_std_core__list_unbox(_box_x1163, NULL);
          kk_std_core__list_dup(head);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last) = kk_std_core__list_box(_ctail_1011, _ctx);
          kk_std_core__list _b_1177_1168 = head; /*list<list<sslice>>*/;
          kk_box_t* _b_1178_1169 = field_1056; /*cfield<list<list<sslice>>>*/;
          _x1438 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1177_1168, _ctx), _b_1178_1169, _ctx); /*ctail<58>*/
          goto _match1439;
        }
        kk_std_core__list _b_1179_1170 = _ctail_1011; /*list<list<sslice>>*/;
        kk_box_t* _b_1180_1171 = field_1056; /*cfield<list<list<sslice>>>*/;
        _x1438 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1179_1170, _ctx), _b_1180_1171, _ctx); /*ctail<58>*/
        _match1439: ;
        xs = mm;
        _acc = _x1438;
        goto kk__tailcall;
      }
    }
  }
  kk_std_core__list_drop(xs, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x1172 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x1172, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last0) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head0;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Filter only for the matched parts.

kk_std_core__list kk_std_text_regex_filter_matches(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<list<sslice>>) -> list<list<sslice>> */ 
  return kk_std_text_regex__ctail_filter_matches(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Filter only for the non-matched parts.

kk_std_core__list kk_std_text_regex__ctail_filter_non_matches(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice>>, ctail<list<list<sslice>>>) -> list<list<sslice>> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con1442 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x1181 = _con1442->head;
    kk_std_core__list _pat0 = _con1442->tail;
    if (kk_std_core__is_Cons(_pat0)) {
      struct kk_std_core_Cons* _con1444 = kk_std_core__as_Cons(_pat0);
      kk_std_core__list s = kk_std_core__list_unbox(_box_x1181, NULL);
      kk_box_t _box_x1182 = _con1444->head;
      kk_std_core__list xx = _con1444->tail;
      kk_reuse_t _ru_1342 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        if (kk_likely(kk_std_core__list_is_unique(_pat0))) {
          kk_box_drop(_box_x1182, _ctx);
          kk_Unit;
          kk_Unit;
          _ru_1342 = (kk_std_core__list_reuse(_pat0));
        }
        else {
          kk_std_core__list_dup(xx);
          kk_Unit;
          kk_std_core__list_decref(_pat0, _ctx);
          kk_Unit;
        }
        kk_Unit;
        kk_std_core__list_free(xs);
      }
      else {
        kk_std_core__list_dup(s);
        kk_std_core__list_dup(xx);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      kk_std_core__list _ctail_1012 = kk_std_core__list_hole(); /*list<list<sslice>>*/;
      kk_std_core__list _ctail_1013 = kk_std_core__new_Cons(_ru_1342, kk_std_core__list_box(s, _ctx), _ctail_1012, _ctx); /*list<list<sslice>>*/;
      { // tailcall
        kk_std_core_types__ctail _x1446;
        kk_box_t* field_1061 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_1013)->tail)); /*cfield<list<list<sslice>>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x1193 = _acc._cons.CTail._field1;
          kk_box_t* last = _acc._cons.CTail._field2;
          kk_std_core__list head = kk_std_core__list_unbox(_box_x1193, NULL);
          kk_std_core__list_dup(head);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last) = kk_std_core__list_box(_ctail_1013, _ctx);
          kk_std_core__list _b_1217_1198 = head; /*list<list<sslice>>*/;
          kk_box_t* _b_1218_1199 = field_1061; /*cfield<list<list<sslice>>>*/;
          _x1446 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1217_1198, _ctx), _b_1218_1199, _ctx); /*ctail<58>*/
          goto _match1447;
        }
        kk_std_core__list _b_1219_1200 = _ctail_1013; /*list<list<sslice>>*/;
        kk_box_t* _b_1220_1201 = field_1061; /*cfield<list<list<sslice>>>*/;
        _x1446 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1219_1200, _ctx), _b_1220_1201, _ctx); /*ctail<58>*/
        _match1447: ;
        xs = xx;
        _acc = _x1446;
        goto kk__tailcall;
      }
    }
  }
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con1449 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x1202 = _con1449->head;
    kk_std_core__list _pat3 = _con1449->tail;
    kk_std_core__list s0 = kk_std_core__list_unbox(_box_x1202, NULL);
    kk_reuse_t _ru_1343 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_1343 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_std_core__list_dup(s0);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_std_core__list rest_1063;
    if (kk_likely(_ru_1343!=NULL)) {
      struct kk_std_core_Cons* _con1451 = (struct kk_std_core_Cons*)_ru_1343;
      _con1451->head = kk_std_core__list_box(s0, _ctx);
      rest_1063 = kk_std_core__base_Cons(_con1451); /*list<list<sslice>>*/
    }
    else {
      rest_1063 = kk_std_core__new_Cons(kk_reuse_null, kk_std_core__list_box(s0, _ctx), _pat3, _ctx); /*list<list<sslice>>*/
    }
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x1207 = _acc._cons.CTail._field1;
      kk_box_t* last0 = _acc._cons.CTail._field2;
      kk_std_core__list head0 = kk_std_core__list_unbox(_box_x1207, NULL);
      kk_std_core__list_dup(head0);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __0 = kk_Unit;
      *(last0) = kk_std_core__list_box(rest_1063, _ctx);
      return head0;
    }
    return rest_1063;
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x1212 = _acc._cons.CTail._field1;
    kk_box_t* last1 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x1212, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last1) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Filter only for the non-matched parts.

kk_std_core__list kk_std_text_regex_filter_non_matches(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<list<sslice>>) -> list<list<sslice>> */ 
  return kk_std_text_regex__ctail_filter_non_matches(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Find a match for a regular expression.
// See also `exec`

kk_std_core_types__maybe kk_std_text_regex_find(kk_string_t s, kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (s : string, r : regex) -> maybe<string> */ 
  kk_std_core__list _match_1324;
  kk_box_t _x1454;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1454 = _x; /*any*/
  kk_ssize_t _x1455 = ((kk_ssize_t)0); /*ssize_t*/
  _match_1324 = kk_std_text_regex_regex_exec(_x1454, s, _x1455, _ctx); /*list<sslice>*/
  if (kk_std_core__is_Cons(_match_1324)) {
    struct kk_std_core_Cons* _con1456 = kk_std_core__as_Cons(_match_1324);
    kk_box_t _box_x1221 = _con1456->head;
    kk_std_core__list _pat3 = _con1456->tail;
    kk_std_core__sslice m = kk_std_core__sslice_unbox(_box_x1221, NULL);
    if (kk_likely(kk_std_core__list_is_unique(_match_1324))) {
      kk_std_core__list_drop(_pat3, _ctx);
      kk_std_core__sslice_dup(m);
      kk_box_drop(_box_x1221, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(_match_1324);
    }
    else {
      kk_std_core__sslice_dup(m);
      kk_Unit;
      kk_std_core__list_decref(_match_1324, _ctx);
      kk_Unit;
    }
    kk_string_t _b_1223_1222 = kk_std_core_string_3(m, _ctx); /*string*/;
    return kk_std_core_types__new_Just(kk_string_box(_b_1223_1222), _ctx);
  }
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// lifted
// specialized std/core/map.5 to parameters [f]

kk_std_core__list kk_std_text_regex__ctail_lift997_find_all(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice>>, ctail<list<string>>) -> list<string> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con1458 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x1224 = _con1458->head;
    kk_std_core__list xx = _con1458->tail;
    kk_std_core__list x = kk_std_core__list_unbox(_box_x1224, NULL);
    kk_reuse_t _ru_1345 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_1345 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_std_core__list_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_string_t _ctail_1014;
    if (kk_std_core__is_Cons(x)) {
      struct kk_std_core_Cons* _con1461 = kk_std_core__as_Cons(x);
      kk_box_t _box_x1225 = _con1461->head;
      kk_std_core__list _pat30 = _con1461->tail;
      kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1225, NULL);
      if (kk_likely(kk_std_core__list_is_unique(x))) {
        kk_std_core__list_drop(_pat30, _ctx);
        kk_std_core__sslice_dup(s);
        kk_box_drop(_box_x1225, _ctx);
        kk_Unit;
        kk_Unit;
        kk_std_core__list_free(x);
      }
      else {
        kk_std_core__sslice_dup(s);
        kk_Unit;
        kk_std_core__list_decref(x, _ctx);
        kk_Unit;
      }
      _ctail_1014 = kk_std_core_string_3(s, _ctx); /*string*/
      goto _match1460;
    }
    _ctail_1014 = kk_string_empty(); /*string*/
    _match1460: ;
    kk_std_core__list _ctail_1015 = kk_std_core__list_hole(); /*list<string>*/;
    kk_std_core__list _ctail_1016 = kk_std_core__new_Cons(_ru_1345, kk_string_box(_ctail_1014), _ctail_1015, _ctx); /*list<string>*/;
    { // tailcall
      kk_std_core_types__ctail _x1464;
      kk_box_t* field_1069 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_1016)->tail)); /*cfield<list<string>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x1236 = _acc._cons.CTail._field1;
        kk_box_t* last = _acc._cons.CTail._field2;
        kk_std_core__list head = kk_std_core__list_unbox(_box_x1236, NULL);
        kk_std_core__list_dup(head);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last) = kk_std_core__list_box(_ctail_1016, _ctx);
        kk_std_core__list _b_1250_1241 = head; /*list<string>*/;
        kk_box_t* _b_1251_1242 = field_1069; /*cfield<list<string>>*/;
        _x1464 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1250_1241, _ctx), _b_1251_1242, _ctx); /*ctail<58>*/
        goto _match1465;
      }
      kk_std_core__list _b_1252_1243 = _ctail_1016; /*list<string>*/;
      kk_box_t* _b_1253_1244 = field_1069; /*cfield<list<string>>*/;
      _x1464 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1252_1243, _ctx), _b_1253_1244, _ctx); /*ctail<58>*/
      _match1465: ;
      xs = xx;
      _acc = _x1464;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x1245 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x1245, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last0) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head0;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted
// specialized std/core/map.5 to parameters [f]

kk_std_core__list kk_std_text_regex__lift997_find_all(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<list<sslice>>) -> list<string> */ 
  return kk_std_text_regex__ctail_lift997_find_all(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Find all matches for a regular expression in a string.

kk_std_core__list kk_std_text_regex_find_all(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : optional<int>) -> list<string> */ 
  kk_std_core__list xs_952;
  kk_std_core__list _x1468;
  kk_box_t _x1469;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1469 = _x; /*any*/
  kk_ssize_t _x1470 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1471;
  kk_integer_t _x1472;
  if (kk_std_core_types__is_Optional(atmost)) {
    kk_box_t _box_x1254 = atmost._cons.Optional.value;
    kk_integer_t _atmost_439 = kk_integer_unbox(_box_x1254);
    kk_integer_dup(_atmost_439);
    kk_std_core_types__optional_drop(atmost, _ctx);
    kk_Unit;
    _x1472 = _atmost_439; /*int*/
    goto _match1473;
  }
  _x1472 = kk_integer_from_small(-1); /*int*/
  _match1473: ;
  _x1471 = kk_std_core_ssize__t(_x1472, _ctx); /*ssize_t*/
  _x1468 = kk_std_text_regex_regex_exec_all(_x1469, s, _x1470, _x1471, _ctx); /*list<list<sslice>>*/
  xs_952 = kk_std_text_regex_filter_matches(_x1468, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex__lift997_find_all(xs_952, _ctx);
}
 
// Create a new regular expression. Takes two optional parameters. Set `ignoreCase` to `True`
// to ignore uppercase/lowercase distinction. If  `multiline` is set to `True`, then `^` and `$`
// match also the beginning and end of every line (instead of the entire input).

kk_std_text_regex__regex kk_std_text_regex_regex(kk_string_t regex0, kk_std_core_types__optional ignorecase, kk_std_core_types__optional multiline, kk_context_t* _ctx) { /* (regex : string, ignorecase : optional<bool>, multiline : optional<bool>) -> regex */ 
  kk_box_t _x1475;
  kk_string_t _x1476 = kk_string_dup(regex0); /*string*/
  bool _x1477;
  if (kk_std_core_types__is_Optional(ignorecase)) {
    kk_box_t _box_x1255 = ignorecase._cons.Optional.value;
    bool _ignorecase_458 = kk_bool_unbox(_box_x1255);
    kk_std_core_types__optional_drop(ignorecase, _ctx);
    kk_Unit;
    _x1477 = _ignorecase_458; /*bool*/
    goto _match1478;
  }
  _x1477 = false; /*bool*/
  _match1478: ;
  bool _x1480;
  if (kk_std_core_types__is_Optional(multiline)) {
    kk_box_t _box_x1256 = multiline._cons.Optional.value;
    bool _multiline_462 = kk_bool_unbox(_box_x1256);
    kk_std_core_types__optional_drop(multiline, _ctx);
    kk_Unit;
    _x1480 = _multiline_462; /*bool*/
    goto _match1481;
  }
  _x1480 = false; /*bool*/
  _match1481: ;
  _x1475 = kk_std_text_regex_regex_create(_x1476, _x1477, _x1480, _ctx); /*any*/
  return kk_std_text_regex__new_Regex(_x1475, regex0, _ctx);
}
 
// Replace the all occurrences of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace_all(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice>) -> e string, atmost : optional<int>) -> e string */ 
  kk_std_core__list _x1483;
  kk_box_t _x1484;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1484 = _x; /*any*/
  kk_ssize_t _x1485 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1486;
  kk_integer_t _x1487;
  if (kk_std_core_types__is_Optional(atmost)) {
    kk_box_t _box_x1257 = atmost._cons.Optional.value;
    kk_integer_t _atmost_478 = kk_integer_unbox(_box_x1257);
    kk_integer_dup(_atmost_478);
    kk_std_core_types__optional_drop(atmost, _ctx);
    kk_Unit;
    _x1487 = _atmost_478; /*int*/
    goto _match1488;
  }
  _x1487 = kk_integer_from_small(-1); /*int*/
  _match1488: ;
  _x1486 = kk_std_core_ssize__t(_x1487, _ctx); /*ssize_t*/
  _x1483 = kk_std_text_regex_regex_exec_all(_x1484, s, _x1485, _x1486, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1483, repl, kk_std_core__new_Nil(_ctx), _ctx);
}
 
// Replace using a replacement string that can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match (`==$0`).


// lift anonymous function
struct kk_std_text_regex_replace_captures_fun1508__t {
  struct kk_function_s _base;
  kk_std_core__list caps;
};
static kk_string_t kk_std_text_regex_replace_captures_fun1508(kk_function_t _fself, kk_std_core__list cap, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_captures_fun1508(kk_std_core__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_captures_fun1508__t* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_captures_fun1508__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_replace_captures_fun1508, kk_context());
  _self->caps = caps;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_replace_captures_fun1508(kk_function_t _fself, kk_std_core__list cap, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_captures_fun1508__t* _self = kk_function_as(struct kk_std_text_regex_replace_captures_fun1508__t*, _fself);
  kk_std_core__list caps = _self->caps; /* list<sslice> */
  kk_drop_match(_self, {kk_std_core__list_dup(caps);}, {}, _ctx)
  if (kk_std_core__is_Cons(cap)) {
    struct kk_std_core_Cons* _con1509 = kk_std_core__as_Cons(cap);
    kk_box_t _box_x1261 = _con1509->head;
    kk_std_core__list _pat40 = _con1509->tail;
    if (kk_std_core__is_Cons(_pat40)) {
      struct kk_std_core_Cons* _con1511 = kk_std_core__as_Cons(_pat40);
      kk_std_core__sslice _pat03 = kk_std_core__sslice_unbox(_box_x1261, NULL);
      kk_box_t _box_x1262 = _con1511->head;
      kk_std_core__list _pat8 = _con1511->tail;
      if (kk_std_core__is_Cons(_pat8)) {
        struct kk_std_core_Cons* _con1513 = kk_std_core__as_Cons(_pat8);
        kk_std_core__sslice digit = kk_std_core__sslice_unbox(_box_x1262, NULL);
        kk_box_t _box_x1263 = _con1513->head;
        kk_std_core__list _pat12 = _con1513->tail;
        if (kk_std_core__is_Cons(_pat12)) {
          struct kk_std_core_Cons* _con1515 = kk_std_core__as_Cons(_pat12);
          kk_std_core__sslice amp = kk_std_core__sslice_unbox(_box_x1263, NULL);
          kk_box_t _box_x1264 = _con1515->head;
          kk_std_core__list _pat16 = _con1515->tail;
          if (kk_std_core__is_Nil(_pat16)) {
            kk_std_core__sslice dollar = kk_std_core__sslice_unbox(_box_x1264, NULL);
            if (kk_likely(kk_std_core__list_is_unique(cap))) {
              if (kk_likely(kk_std_core__list_is_unique(_pat40))) {
                if (kk_likely(kk_std_core__list_is_unique(_pat8))) {
                  if (kk_likely(kk_std_core__list_is_unique(_pat12))) {
                    kk_std_core__sslice_dup(dollar);
                    kk_box_drop(_box_x1264, _ctx);
                    kk_Unit;
                    kk_Unit;
                    kk_std_core__list_free(_pat12);
                  }
                  else {
                    kk_std_core__sslice_dup(dollar);
                    kk_Unit;
                    kk_std_core__list_decref(_pat12, _ctx);
                    kk_Unit;
                  }
                  kk_std_core__sslice_dup(amp);
                  kk_box_drop(_box_x1263, _ctx);
                  kk_Unit;
                  kk_Unit;
                  kk_std_core__list_free(_pat8);
                }
                else {
                  kk_std_core__sslice_dup(amp);
                  kk_std_core__sslice_dup(dollar);
                  kk_Unit;
                  kk_std_core__list_decref(_pat8, _ctx);
                  kk_Unit;
                }
                kk_std_core__sslice_dup(digit);
                kk_box_drop(_box_x1262, _ctx);
                kk_Unit;
                kk_Unit;
                kk_std_core__list_free(_pat40);
              }
              else {
                kk_std_core__sslice_dup(amp);
                kk_std_core__sslice_dup(digit);
                kk_std_core__sslice_dup(dollar);
                kk_Unit;
                kk_std_core__list_decref(_pat40, _ctx);
                kk_Unit;
              }
              kk_box_drop(_box_x1261, _ctx);
              kk_Unit;
              kk_Unit;
              kk_std_core__list_free(cap);
            }
            else {
              kk_std_core__sslice_dup(amp);
              kk_std_core__sslice_dup(digit);
              kk_std_core__sslice_dup(dollar);
              kk_Unit;
              kk_std_core__list_decref(cap, _ctx);
              kk_Unit;
            }
            bool _match_1316;
            kk_ssize_t _x1517;
            kk_ssize_t _x0 = dollar.start;
            kk_std_core__sslice_drop(dollar, _ctx);
            kk_Unit;
            _x1517 = _x0; /*ssize_t*/
            kk_ssize_t _x1518 = ((kk_ssize_t)0); /*ssize_t*/
            _match_1316 = (_x1517 >= _x1518); /*bool*/
            if (_match_1316) {
              kk_std_core__sslice_drop(amp, _ctx);
              kk_std_core__list_drop(caps, _ctx);
              kk_std_core__sslice_drop(digit, _ctx);
              kk_define_string_literal(, _s1519, 1, "$")
              return kk_string_dup(_s1519);
            }
            kk_integer_t grp;
            bool _match_1318;
            kk_ssize_t _x1520;
            kk_ssize_t _x1 = amp.start;
            kk_std_core__sslice_drop(amp, _ctx);
            kk_Unit;
            _x1520 = _x1; /*ssize_t*/
            kk_ssize_t _x1521 = ((kk_ssize_t)0); /*ssize_t*/
            _match_1318 = (_x1520 >= _x1521); /*bool*/
            if (_match_1318) {
              kk_std_core__sslice_drop(digit, _ctx);
              grp = kk_integer_from_small(0); /*int*/
            }
            else {
              kk_string_t s1_967 = kk_std_core_string_3(digit, _ctx); /*string*/;
              kk_std_core_types__optional default0_968 = kk_std_core_types__new_Optional(kk_integer_box(kk_integer_from_small(0)), _ctx); /*optional<int>*/;
              bool _match_1319;
              kk_string_t _x1522 = kk_string_dup(s1_967); /*string*/
              kk_string_t _x1523 = kk_string_empty(); /*string*/
              _match_1319 = kk_string_is_eq(_x1522,_x1523,kk_context()); /*bool*/
              if (_match_1319) {
                kk_string_drop(s1_967, _ctx);
                if (kk_std_core_types__is_Optional(default0_968)) {
                  kk_box_t _box_x1267 = default0_968._cons.Optional.value;
                  kk_integer_t _default_15449 = kk_integer_unbox(_box_x1267);
                  kk_integer_dup(_default_15449);
                  kk_std_core_types__optional_drop(default0_968, _ctx);
                  kk_Unit;
                  grp = _default_15449; /*int*/
                  goto _match1525;
                }
                grp = kk_integer_from_small(0); /*int*/
                _match1525: ;
              }
              else {
                kk_std_core_types__maybe m_17133;
                kk_string_t _x1527;
                kk_string_t _x1528 = kk_string_trim_left(s1_967,kk_context()); /*string*/
                _x1527 = kk_string_trim_right(_x1528,kk_context()); /*string*/
                bool _x1529;
                kk_std_core_types__optional _match_1320 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
                if (kk_std_core_types__is_Optional(_match_1320)) {
                  kk_box_t _box_x1268 = _match_1320._cons.Optional.value;
                  bool _hex_15453 = kk_bool_unbox(_box_x1268);
                  kk_std_core_types__optional_drop(_match_1320, _ctx);
                  kk_Unit;
                  _x1529 = _hex_15453; /*bool*/
                  goto _match1530;
                }
                _x1529 = false; /*bool*/
                _match1530: ;
                m_17133 = kk_std_core_xparse_int(_x1527, _x1529, _ctx); /*maybe<int>*/
                if (kk_std_core_types__is_Nothing(m_17133)) {
                  if (kk_std_core_types__is_Optional(default0_968)) {
                    kk_box_t _box_x1269 = default0_968._cons.Optional.value;
                    kk_integer_t _default_154490 = kk_integer_unbox(_box_x1269);
                    kk_integer_dup(_default_154490);
                    kk_std_core_types__optional_drop(default0_968, _ctx);
                    kk_Unit;
                    grp = _default_154490; /*int*/
                    goto _match1532;
                  }
                  grp = kk_integer_from_small(0); /*int*/
                  _match1532: ;
                }
                else {
                  kk_box_t _box_x1270 = m_17133._cons.Just.value;
                  kk_integer_t x = kk_integer_unbox(_box_x1270);
                  kk_integer_dup(x);
                  kk_std_core_types__maybe_drop(m_17133, _ctx);
                  kk_Unit;
                  kk_std_core_types__optional_drop(default0_968, _ctx);
                  kk_Unit;
                  grp = x; /*int*/
                }
              }
            }
            kk_std_core_types__maybe _match_1317 = kk_std_core__lp__lb__rb__2_rp_(caps, grp, _ctx); /*maybe<9772>*/;
            if (kk_std_core_types__is_Nothing(_match_1317)) {
              return kk_string_empty();
            }
            kk_box_t _box_x1271 = _match_1317._cons.Just.value;
            kk_std_core__sslice s2 = kk_std_core__sslice_unbox(_box_x1271, NULL);
            kk_std_core__sslice_dup(s2);
            kk_std_core_types__maybe_drop(_match_1317, _ctx);
            kk_Unit;
            return kk_std_core_string_3(s2, _ctx);
          }
        }
      }
    }
  }
  kk_std_core__list_drop(cap, _ctx);
  kk_std_core__list_drop(caps, _ctx);
  kk_define_string_literal(, _s1537, 1, "$")
  return kk_string_dup(_s1537);
}

kk_string_t kk_std_text_regex_replace_captures(kk_std_core__list caps, kk_string_t repl, kk_context_t* _ctx) { /* (caps : list<sslice>, repl : string) -> string */ 
  kk_std_text_regex__regex r_959;
  kk_box_t _x1490;
  kk_string_t _x1491;
  kk_define_string_literal(, _s1492, 20, "\\$(\?:(\\d)|(\\&)|(\\$))")
  _x1491 = kk_string_dup(_s1492); /*string*/
  bool _x1493;
  kk_std_core_types__optional _match_1323 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1323)) {
    kk_box_t _box_x1258 = _match_1323._cons.Optional.value;
    bool _ignorecase_458 = kk_bool_unbox(_box_x1258);
    kk_std_core_types__optional_drop(_match_1323, _ctx);
    kk_Unit;
    _x1493 = _ignorecase_458; /*bool*/
    goto _match1494;
  }
  _x1493 = false; /*bool*/
  _match1494: ;
  bool _x1496;
  kk_std_core_types__optional _match_1322 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1322)) {
    kk_box_t _box_x1259 = _match_1322._cons.Optional.value;
    bool _multiline_462 = kk_bool_unbox(_box_x1259);
    kk_std_core_types__optional_drop(_match_1322, _ctx);
    kk_Unit;
    _x1496 = _multiline_462; /*bool*/
    goto _match1497;
  }
  _x1496 = false; /*bool*/
  _match1497: ;
  _x1490 = kk_std_text_regex_regex_create(_x1491, _x1493, _x1496, _ctx); /*any*/
  kk_string_t _x1499;
  kk_define_string_literal(, _s1500, 20, "\\$(\?:(\\d)|(\\&)|(\\$))")
  _x1499 = kk_string_dup(_s1500); /*string*/
  r_959 = kk_std_text_regex__new_Regex(_x1490, _x1499, _ctx); /*std/text/regex/regex*/
  kk_std_core__list _x1501;
  kk_box_t _x1502;
  kk_box_t _x = r_959.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r_959, _ctx);
  kk_Unit;
  _x1502 = _x; /*any*/
  kk_ssize_t _x1503 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1504;
  kk_integer_t _x1505;
  kk_std_core_types__optional _match_1321 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_1321)) {
    kk_box_t _box_x1260 = _match_1321._cons.Optional.value;
    kk_integer_t _atmost_478 = kk_integer_unbox(_box_x1260);
    kk_integer_dup(_atmost_478);
    kk_std_core_types__optional_drop(_match_1321, _ctx);
    kk_Unit;
    _x1505 = _atmost_478; /*int*/
    goto _match1506;
  }
  _x1505 = kk_integer_from_small(-1); /*int*/
  _match1506: ;
  _x1504 = kk_std_core_ssize__t(_x1505, _ctx); /*ssize_t*/
  _x1501 = kk_std_text_regex_regex_exec_all(_x1502, repl, _x1503, _x1504, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1501, kk_std_text_regex_new_replace_captures_fun1508(caps, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
}
 
// Replace all occurrences of `regex` with the replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match (`==$0`).


// lift anonymous function
struct kk_std_text_regex_replace_all_fun1548__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_all_fun1548_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_all_fun1548_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1548__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_all_fun1548__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_replace_all_fun1548_1, kk_context());
  _self->repl = repl;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_replace_all_fun1548_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1548__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_all_fun1548__t_1*, _fself);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl);}, {}, _ctx)
  return kk_std_text_regex_replace_captures(caps, repl, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_replace_all_fun1556__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_all_fun1556_1(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_all_fun1556_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1556__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_all_fun1556__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_replace_all_fun1556_1, kk_context());
  _self->repl = repl;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_replace_all_fun1556_1(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_all_fun1556__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_all_fun1556__t_1*, _fself);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl);}, {}, _ctx)
  kk_std_core__list_drop(___wildcard__183__35, _ctx);
  return repl;
}

kk_string_t kk_std_text_regex_replace_all_1(kk_string_t s, kk_std_text_regex__regex regex0, kk_string_t repl, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string, atmost : optional<int>) -> string */ 
  bool _match_1315;
  kk_string_t _x1538 = kk_string_dup(repl); /*string*/
  kk_string_t _x1539;
  kk_define_string_literal(, _s1540, 1, "$")
  _x1539 = kk_string_dup(_s1540); /*string*/
  _match_1315 = kk_string_contains(_x1538,_x1539,kk_context()); /*bool*/
  if (_match_1315) {
    kk_std_core__list _x1541;
    kk_box_t _x1542;
    kk_box_t _x = regex0.obj;
    kk_box_dup(_x);
    kk_std_text_regex__regex_drop(regex0, _ctx);
    kk_Unit;
    _x1542 = _x; /*any*/
    kk_ssize_t _x1543 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x1544;
    kk_integer_t _x1545;
    if (kk_std_core_types__is_Optional(atmost)) {
      kk_box_t _box_x1272 = atmost._cons.Optional.value;
      kk_integer_t _atmost_499 = kk_integer_unbox(_box_x1272);
      kk_integer_dup(_atmost_499);
      kk_std_core_types__optional_drop(atmost, _ctx);
      kk_Unit;
      _x1545 = _atmost_499; /*int*/
      goto _match1546;
    }
    _x1545 = kk_integer_from_small(-1); /*int*/
    _match1546: ;
    _x1544 = kk_std_core_ssize__t(_x1545, _ctx); /*ssize_t*/
    _x1541 = kk_std_text_regex_regex_exec_all(_x1542, s, _x1543, _x1544, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex_concat_replace(_x1541, kk_std_text_regex_new_replace_all_fun1548_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
  }
  kk_std_core__list _x1549;
  kk_box_t _x1550;
  kk_box_t _x0 = regex0.obj;
  kk_box_dup(_x0);
  kk_std_text_regex__regex_drop(regex0, _ctx);
  kk_Unit;
  _x1550 = _x0; /*any*/
  kk_ssize_t _x1551 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1552;
  kk_integer_t _x1553;
  if (kk_std_core_types__is_Optional(atmost)) {
    kk_box_t _box_x1273 = atmost._cons.Optional.value;
    kk_integer_t _atmost_4990 = kk_integer_unbox(_box_x1273);
    kk_integer_dup(_atmost_4990);
    kk_std_core_types__optional_drop(atmost, _ctx);
    kk_Unit;
    _x1553 = _atmost_4990; /*int*/
    goto _match1554;
  }
  _x1553 = kk_integer_from_small(-1); /*int*/
  _match1554: ;
  _x1552 = kk_std_core_ssize__t(_x1553, _ctx); /*ssize_t*/
  _x1549 = kk_std_text_regex_regex_exec_all(_x1550, s, _x1551, _x1552, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1549, kk_std_text_regex_new_replace_all_fun1556_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
}

kk_std_text_regex__regex kk_std_text_regex_rx_nongroup;
 
// How many groups are captured by this regex?


// lift anonymous function
struct kk_std_text_regex_groups_count_fun1582__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_text_regex_groups_count_fun1582(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_groups_count_fun1582(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_groups_count_fun1582, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_text_regex_groups_count_fun1582(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_string_t _x1583 = kk_string_empty(); /*string*/
  return kk_std_text_regex_replace_captures(caps, _x1583, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_groups_count_fun1594__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_text_regex_groups_count_fun1594(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_groups_count_fun1594(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_text_regex_groups_count_fun1594, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_text_regex_groups_count_fun1594(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core__list_drop(___wildcard__183__35, _ctx);
  return kk_string_empty();
}

kk_integer_t kk_std_text_regex_groups_count(kk_std_text_regex__regex r, kk_context_t* _ctx) { /* (r : regex) -> int */ 
  kk_string_t _x1568;
  bool _match_1310;
  kk_string_t _x1569 = kk_string_empty(); /*string*/
  kk_string_t _x1571;
  kk_define_string_literal(, _s1572, 1, "$")
  _x1571 = kk_string_dup(_s1572); /*string*/
  _match_1310 = kk_string_contains(_x1569,_x1571,kk_context()); /*bool*/
  if (_match_1310) {
    kk_std_core__list _x1573;
    kk_box_t _x1574;
    kk_std_text_regex__regex _x1575 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    kk_box_t _x = _x1575.obj;
    _x1574 = kk_box_dup(_x); /*any*/
    kk_string_t _x1576;
    kk_string_t _x1 = r.src;
    kk_string_dup(_x1);
    kk_std_text_regex__regex_drop(r, _ctx);
    kk_Unit;
    _x1576 = _x1; /*string*/
    kk_ssize_t _x1577 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x1578;
    kk_integer_t _x1579;
    kk_std_core_types__optional _match_1312 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1312)) {
      kk_box_t _box_x1276 = _match_1312._cons.Optional.value;
      kk_integer_t _atmost_499 = kk_integer_unbox(_box_x1276);
      kk_integer_dup(_atmost_499);
      kk_std_core_types__optional_drop(_match_1312, _ctx);
      kk_Unit;
      _x1579 = _atmost_499; /*int*/
      goto _match1580;
    }
    _x1579 = kk_integer_from_small(-1); /*int*/
    _match1580: ;
    _x1578 = kk_std_core_ssize__t(_x1579, _ctx); /*ssize_t*/
    _x1573 = kk_std_text_regex_regex_exec_all(_x1574, _x1576, _x1577, _x1578, _ctx); /*list<list<sslice>>*/
    _x1568 = kk_std_text_regex_concat_replace(_x1573, kk_std_text_regex_new_groups_count_fun1582(_ctx), kk_std_core__new_Nil(_ctx), _ctx); /*string*/
  }
  else {
    kk_std_core__list _x1585;
    kk_box_t _x1586;
    kk_std_text_regex__regex _x1587 = kk_std_text_regex_rx_nongroup; /*std/text/regex/regex*/
    kk_box_t _x0 = _x1587.obj;
    _x1586 = kk_box_dup(_x0); /*any*/
    kk_string_t _x1588;
    kk_string_t _x10 = r.src;
    kk_string_dup(_x10);
    kk_std_text_regex__regex_drop(r, _ctx);
    kk_Unit;
    _x1588 = _x10; /*string*/
    kk_ssize_t _x1589 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x1590;
    kk_integer_t _x1591;
    kk_std_core_types__optional _match_1311 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1311)) {
      kk_box_t _box_x1277 = _match_1311._cons.Optional.value;
      kk_integer_t _atmost_4990 = kk_integer_unbox(_box_x1277);
      kk_integer_dup(_atmost_4990);
      kk_std_core_types__optional_drop(_match_1311, _ctx);
      kk_Unit;
      _x1591 = _atmost_4990; /*int*/
      goto _match1592;
    }
    _x1591 = kk_integer_from_small(-1); /*int*/
    _match1592: ;
    _x1590 = kk_std_core_ssize__t(_x1591, _ctx); /*ssize_t*/
    _x1585 = kk_std_text_regex_regex_exec_all(_x1586, _x1588, _x1589, _x1590, _ctx); /*list<list<sslice>>*/
    _x1568 = kk_std_text_regex_concat_replace(_x1585, kk_std_text_regex_new_groups_count_fun1594(_ctx), kk_std_core__new_Nil(_ctx), _ctx); /*string*/
  }
  return kk_std_core_count_1(_x1568, _ctx);
}
 
// Replace the first occurrence of `regex` by the result of the replacement fun `repl` in a string `s`.

kk_string_t kk_std_text_regex_replace(kk_string_t s, kk_std_text_regex__regex r, kk_function_t repl, kk_context_t* _ctx) { /* forall<e> (s : string, r : regex, repl : (list<sslice>) -> e string) -> e string */ 
  kk_std_core__list _x1596;
  kk_box_t _x1597;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1597 = _x; /*any*/
  kk_ssize_t _x1598 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1599 = ((kk_ssize_t)1); /*ssize_t*/
  _x1596 = kk_std_text_regex_regex_exec_all(_x1597, s, _x1598, _x1599, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1596, repl, kk_std_core__new_Nil(_ctx), _ctx);
}
 
// Replace the first occurrence of `regex` with a replacement string `repl` in a string `s`.
// The replacement string can contain `$$` for a `$` sign, `$n` for a capture group,
// `$&` for the entire match (`==$0`).


// lift anonymous function
struct kk_std_text_regex_replace_fun1607__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_fun1607_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_fun1607_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1607__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_fun1607__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_replace_fun1607_1, kk_context());
  _self->repl = repl;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_replace_fun1607_1(kk_function_t _fself, kk_std_core__list caps, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1607__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_fun1607__t_1*, _fself);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl);}, {}, _ctx)
  return kk_std_text_regex_replace_captures(caps, repl, _ctx);
}


// lift anonymous function
struct kk_std_text_regex_replace_fun1612__t_1 {
  struct kk_function_s _base;
  kk_string_t repl;
};
static kk_string_t kk_std_text_regex_replace_fun1612_1(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx);
static kk_function_t kk_std_text_regex_new_replace_fun1612_1(kk_string_t repl, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1612__t_1* _self = kk_function_alloc_as(struct kk_std_text_regex_replace_fun1612__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_text_regex_replace_fun1612_1, kk_context());
  _self->repl = repl;
  return &_self->_base;
}

static kk_string_t kk_std_text_regex_replace_fun1612_1(kk_function_t _fself, kk_std_core__list ___wildcard__183__35, kk_context_t* _ctx) {
  struct kk_std_text_regex_replace_fun1612__t_1* _self = kk_function_as(struct kk_std_text_regex_replace_fun1612__t_1*, _fself);
  kk_string_t repl = _self->repl; /* string */
  kk_drop_match(_self, {kk_string_dup(repl);}, {}, _ctx)
  kk_std_core__list_drop(___wildcard__183__35, _ctx);
  return repl;
}

kk_string_t kk_std_text_regex_replace_1(kk_string_t s, kk_std_text_regex__regex regex0, kk_string_t repl, kk_context_t* _ctx) { /* (s : string, regex : regex, repl : string) -> string */ 
  bool _match_1309;
  kk_string_t _x1600 = kk_string_dup(repl); /*string*/
  kk_string_t _x1601;
  kk_define_string_literal(, _s1602, 1, "$")
  _x1601 = kk_string_dup(_s1602); /*string*/
  _match_1309 = kk_string_contains(_x1600,_x1601,kk_context()); /*bool*/
  if (_match_1309) {
    kk_std_core__list _x1603;
    kk_box_t _x1604;
    kk_box_t _x = regex0.obj;
    kk_box_dup(_x);
    kk_std_text_regex__regex_drop(regex0, _ctx);
    kk_Unit;
    _x1604 = _x; /*any*/
    kk_ssize_t _x1605 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x1606 = ((kk_ssize_t)1); /*ssize_t*/
    _x1603 = kk_std_text_regex_regex_exec_all(_x1604, s, _x1605, _x1606, _ctx); /*list<list<sslice>>*/
    return kk_std_text_regex_concat_replace(_x1603, kk_std_text_regex_new_replace_fun1607_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
  }
  kk_std_core__list _x1608;
  kk_box_t _x1609;
  kk_box_t _x0 = regex0.obj;
  kk_box_dup(_x0);
  kk_std_text_regex__regex_drop(regex0, _ctx);
  kk_Unit;
  _x1609 = _x0; /*any*/
  kk_ssize_t _x1610 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1611 = ((kk_ssize_t)1); /*ssize_t*/
  _x1608 = kk_std_text_regex_regex_exec_all(_x1609, s, _x1610, _x1611, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex_concat_replace(_x1608, kk_std_text_regex_new_replace_fun1612_1(repl, _ctx), kk_std_core__new_Nil(_ctx), _ctx);
}
 
// lifted
// specialized std/core/map.5 to parameters [f]

kk_std_core__list kk_std_text_regex__ctail_lift998_split(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (xs : list<list<sslice>>, ctail<list<string>>) -> list<string> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con1613 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x1278 = _con1613->head;
    kk_std_core__list xx = _con1613->tail;
    kk_std_core__list x = kk_std_core__list_unbox(_box_x1278, NULL);
    kk_reuse_t _ru_1351 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_1351 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_std_core__list_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_string_t _ctail_1017;
    if (kk_std_core__is_Cons(x)) {
      struct kk_std_core_Cons* _con1616 = kk_std_core__as_Cons(x);
      kk_box_t _box_x1279 = _con1616->head;
      kk_std_core__list _pat30 = _con1616->tail;
      kk_std_core__sslice s = kk_std_core__sslice_unbox(_box_x1279, NULL);
      if (kk_likely(kk_std_core__list_is_unique(x))) {
        kk_std_core__list_drop(_pat30, _ctx);
        kk_std_core__sslice_dup(s);
        kk_box_drop(_box_x1279, _ctx);
        kk_Unit;
        kk_Unit;
        kk_std_core__list_free(x);
      }
      else {
        kk_std_core__sslice_dup(s);
        kk_Unit;
        kk_std_core__list_decref(x, _ctx);
        kk_Unit;
      }
      _ctail_1017 = kk_std_core_string_3(s, _ctx); /*string*/
      goto _match1615;
    }
    _ctail_1017 = kk_string_empty(); /*string*/
    _match1615: ;
    kk_std_core__list _ctail_1018 = kk_std_core__list_hole(); /*list<string>*/;
    kk_std_core__list _ctail_1019 = kk_std_core__new_Cons(_ru_1351, kk_string_box(_ctail_1017), _ctail_1018, _ctx); /*list<string>*/;
    { // tailcall
      kk_std_core_types__ctail _x1619;
      kk_box_t* field_1075 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_1019)->tail)); /*cfield<list<string>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x1290 = _acc._cons.CTail._field1;
        kk_box_t* last = _acc._cons.CTail._field2;
        kk_std_core__list head = kk_std_core__list_unbox(_box_x1290, NULL);
        kk_std_core__list_dup(head);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last) = kk_std_core__list_box(_ctail_1019, _ctx);
        kk_std_core__list _b_1304_1295 = head; /*list<string>*/;
        kk_box_t* _b_1305_1296 = field_1075; /*cfield<list<string>>*/;
        _x1619 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1304_1295, _ctx), _b_1305_1296, _ctx); /*ctail<58>*/
        goto _match1620;
      }
      kk_std_core__list _b_1306_1297 = _ctail_1019; /*list<string>*/;
      kk_box_t* _b_1307_1298 = field_1075; /*cfield<list<string>>*/;
      _x1619 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_1306_1297, _ctx), _b_1307_1298, _ctx); /*ctail<58>*/
      _match1620: ;
      xs = xx;
      _acc = _x1619;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x1299 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x1299, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last0) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head0;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted
// specialized std/core/map.5 to parameters [f]

kk_std_core__list kk_std_text_regex__lift998_split(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<list<sslice>>) -> list<string> */ 
  return kk_std_text_regex__ctail_lift998_split(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Split a string `s` in at most `atmost` parts using a regular expression `r` as separator.

kk_std_core__list kk_std_text_regex_split(kk_string_t s, kk_std_text_regex__regex r, kk_std_core_types__optional atmost, kk_context_t* _ctx) { /* (s : string, r : regex, atmost : optional<int>) -> list<string> */ 
  kk_std_core__list xs_991;
  kk_std_core__list _x1623;
  kk_box_t _x1624;
  kk_box_t _x = r.obj;
  kk_box_dup(_x);
  kk_std_text_regex__regex_drop(r, _ctx);
  kk_Unit;
  _x1624 = _x; /*any*/
  kk_ssize_t _x1625 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x1626;
  kk_integer_t _x1627;
  if (kk_std_core_types__is_Optional(atmost)) {
    kk_box_t _box_x1308 = atmost._cons.Optional.value;
    kk_integer_t _atmost_925 = kk_integer_unbox(_box_x1308);
    kk_integer_dup(_atmost_925);
    kk_std_core_types__optional_drop(atmost, _ctx);
    kk_Unit;
    _x1627 = _atmost_925; /*int*/
    goto _match1628;
  }
  _x1627 = kk_integer_from_small(-1); /*int*/
  _match1628: ;
  _x1626 = kk_std_core_ssize__t(_x1627, _ctx); /*ssize_t*/
  _x1623 = kk_std_text_regex_regex_exec_all(_x1624, s, _x1625, _x1626, _ctx); /*list<list<sslice>>*/
  xs_991 = kk_std_text_regex_filter_non_matches(_x1623, _ctx); /*list<list<sslice>>*/
  return kk_std_text_regex__lift998_split(xs_991, _ctx);
}

// initialization
void kk_std_text_regex__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  kk_std_core__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_box_t _x1557;
    kk_string_t _x1558;
    kk_define_string_literal(, _s1559, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]")
    _x1558 = kk_string_dup(_s1559); /*string*/
    bool _x1560;
    kk_std_core_types__optional _match_1314 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1314)) {
      kk_box_t _box_x1274 = _match_1314._cons.Optional.value;
      bool _ignorecase_458 = kk_bool_unbox(_box_x1274);
      kk_std_core_types__optional_drop(_match_1314, _ctx);
      kk_Unit;
      _x1560 = _ignorecase_458; /*bool*/
      goto _match1561;
    }
    _x1560 = false; /*bool*/
    _match1561: ;
    bool _x1563;
    kk_std_core_types__optional _match_1313 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_1313)) {
      kk_box_t _box_x1275 = _match_1313._cons.Optional.value;
      bool _multiline_462 = kk_bool_unbox(_box_x1275);
      kk_std_core_types__optional_drop(_match_1313, _ctx);
      kk_Unit;
      _x1563 = _multiline_462; /*bool*/
      goto _match1564;
    }
    _x1563 = false; /*bool*/
    _match1564: ;
    _x1557 = kk_std_text_regex_regex_create(_x1558, _x1560, _x1563, _ctx); /*any*/
    kk_string_t _x1566;
    kk_define_string_literal(, _s1567, 45, "[^\\\\\\[(]+|\\\\[\\s\\S]\?|\\(\\\?|\\[(\?:[^\\\\\\]]|\\\\.)*\\]")
    _x1566 = kk_string_dup(_s1567); /*string*/
    kk_std_text_regex_rx_nongroup = kk_std_text_regex__new_Regex(_x1557, _x1566, _ctx); /*std/text/regex/regex*/
  }
}

// termination
void kk_std_text_regex__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_text_regex__regex_drop(kk_std_text_regex_rx_nongroup, _ctx);
  kk_std_core__done(_ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
