// Koka generated module: "std/core", koka version: 2.2.1
#include "std_core.h"
/*---------------------------------------------------------------------------
  Copyright 2020-2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/

kk_std_core__list kk_vector_to_list(kk_vector_t v, kk_std_core__list tail, kk_context_t* ctx) {
  // todo: avoid boxed_dup if v is unique
  kk_ssize_t n;
  kk_box_t* p = kk_vector_buf_borrow(v, &n);
  if (n <= 0) {
    kk_vector_drop(v,ctx);
    return tail;
  }
  kk_std_core__list nil  = kk_std_core__new_Nil(ctx);
  struct kk_std_core_Cons* cons = NULL;
  kk_std_core__list list = kk_std_core__new_Nil(ctx);
  for( kk_ssize_t i = 0; i < n; i++ ) {
    kk_std_core__list hd = kk_std_core__new_Cons(kk_reuse_null,kk_box_dup(p[i]), nil, ctx);
    if (cons==NULL) {
      list = hd;
    }
    else {
      cons->tail = hd;
    }
    cons = kk_std_core__as_Cons(hd);
  }
  cons->tail = tail;
  kk_vector_drop(v,ctx);
  return list;
}

kk_vector_t kk_list_to_vector(kk_std_core__list xs, kk_context_t* ctx) {
  // todo: avoid boxed_dup if xs is unique
  // find the length
  kk_ssize_t len = 0;
  kk_std_core__list ys = xs;
  while (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* cons = kk_std_core__as_Cons(ys);
    len++;
    ys = cons->tail;
  }
  // alloc the vector and copy
  kk_box_t* p;
  kk_vector_t v = kk_vector_alloc_uninit(len, &p, ctx);  
  ys = xs;
  for( kk_ssize_t i = 0; i < len; i++) {
    struct kk_std_core_Cons* cons = kk_std_core__as_Cons(ys);
    ys = cons->tail;
    p[i] = kk_box_dup(cons->head);
  }
  kk_std_core__list_drop(xs,ctx);  // todo: drop while visiting?
  return v;
}

kk_vector_t kk_vector_init( kk_ssize_t n, kk_function_t init, kk_context_t* ctx) {
  kk_box_t* p;
  kk_vector_t v = kk_vector_alloc_uninit(n, &p, ctx);  
  for(kk_ssize_t i = 0; i < n; i++) {
    kk_function_dup(init);
    p[i] = kk_function_call(kk_box_t,(kk_function_t,kk_ssize_t,kk_context_t*),init,(init,i,ctx));
  }
  kk_function_drop(init,ctx);
  return v;
}

kk_box_t kk_main_console( kk_function_t action, kk_context_t* ctx ) {
  return kk_function_call(kk_box_t,(kk_function_t,kk_unit_t,kk_context_t*),action,(action,kk_Unit,ctx));
}


kk_std_core__list kk_string_to_list(kk_string_t s, kk_context_t* ctx) {
  kk_ssize_t len;
  const uint8_t* p = kk_string_buf_borrow(s,&len);
  const uint8_t* const end = p + len;
  kk_std_core__list nil  = kk_std_core__new_Nil(ctx);
  kk_std_core__list list = nil;
  struct kk_std_core_Cons* tl = NULL;
  kk_ssize_t count;
  while( p < end ) {
    kk_char_t c = kk_utf8_read(p,&count);
    p += count;
    kk_std_core__list cons = kk_std_core__new_Cons(kk_reuse_null,kk_char_box(c,ctx), nil, ctx);
    if (tl!=NULL) {
      tl->tail = cons;
    }
    else {
      list = cons;
    }
    tl = kk_std_core__as_Cons(cons);
  }
  kk_string_drop(s,ctx);
  return list;
}

kk_string_t kk_string_from_list(kk_std_core__list cs, kk_context_t* ctx) {
  // TODO: optimize for short strings to write directly into a local buffer?
  // find total UTF8 length
  kk_ssize_t len = 0;
  kk_std_core__list xs = cs;
  while (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* cons = kk_std_core__as_Cons(xs);
    len += kk_utf8_len(kk_char_unbox(cons->head,ctx));
    xs = cons->tail;
  }
  // allocate and copy the characters
  uint8_t* p;
  kk_string_t s = kk_unsafe_string_alloc_buf(len,&p,ctx);  // must be initialized
  xs = cs;
  while (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* cons = kk_std_core__as_Cons(xs);
    kk_ssize_t count;
    kk_utf8_write( kk_char_unbox(cons->head,ctx), p, &count );
    p += count;
    xs = cons->tail;
  }
  kk_assert_internal(*p == 0 && (p - kk_string_buf_borrow(s,NULL)) == len);
  kk_std_core__list_drop(cs,ctx);  // todo: drop while visiting?
  return s;
}

static inline void kk_sslice_start_end_borrowx( kk_std_core__sslice sslice, const uint8_t** start, const uint8_t** end, const uint8_t** sstart, const uint8_t** send) {
  kk_ssize_t slen;
  const uint8_t* s = kk_string_buf_borrow(sslice.str,&slen);
  *start = s + sslice.start;
  *end = s + sslice.start + sslice.len;
  if (sstart != NULL) *sstart = s;
  if (send != NULL) *send = s + slen;
  kk_assert_internal(*start >= s && *start <= *end);
  kk_assert_internal(*end >= *start && *end <= s + slen);
}

static inline void kk_sslice_start_end_borrow( kk_std_core__sslice sslice, const uint8_t** start, const uint8_t** end) {
  kk_sslice_start_end_borrowx(sslice,start,end,NULL,NULL);
}

kk_integer_t kk_slice_count( kk_std_core__sslice sslice, kk_context_t* ctx ) {
  // TODO: optimize this by extending kk_string_count
  const uint8_t* start;
  const uint8_t* end;
  kk_sslice_start_end_borrow(sslice, &start, &end);
  kk_ssize_t count = 0;
  while( start < end && *start != 0 ) {
    const uint8_t* next = kk_utf8_next(start);
    count++;
    start = next;
  }
  kk_std_core__sslice_drop(sslice,ctx);
  return kk_integer_from_ssize_t(count,ctx);
}

kk_string_t kk_slice_to_string( kk_std_core__sslice  sslice, kk_context_t* ctx ) {
  const uint8_t* start;
  const uint8_t* end;
  kk_sslice_start_end_borrow(sslice, &start, &end);
  // is it the full string?
  if (sslice.start == 0 && sslice.len == kk_string_len_borrow(sslice.str)) {
    // TODO: drop sslice and dup sslice.str?
    return sslice.str;
  }
  else {
    // if not, we copy len bytes
    kk_string_t s = kk_string_alloc_dupn_valid_utf8(sslice.len, start, ctx);
    kk_std_core__sslice_drop(sslice,ctx);
    return s;
  }
}

kk_std_core__sslice kk_slice_first( kk_string_t str, kk_context_t* ctx ) {
  kk_ssize_t slen;
  const uint8_t* s = kk_string_buf_borrow(str,&slen);
  const uint8_t* next = (slen > 0 ? kk_utf8_next(s) : s);
  return kk_std_core__new_Sslice(str, 0, (next - s), ctx);
}

kk_std_core__sslice kk_slice_last( kk_string_t str, kk_context_t* ctx ) {
  kk_ssize_t slen;
  const uint8_t* s = kk_string_buf_borrow(str,&slen);
  const uint8_t* end = s + slen;
  const uint8_t* prev = (s==end ? s : kk_utf8_prev(end));
  return kk_std_core__new_Sslice(str, (prev - s), (end - prev), ctx);
}

kk_std_core__sslice kk_slice_between( struct kk_std_core_Sslice slice1, struct kk_std_core_Sslice slice2, kk_context_t* ctx ) {
  const uint8_t* s1 = kk_string_buf_borrow( slice1.str, NULL );
  const uint8_t* s2 = kk_string_buf_borrow( slice2.str, NULL );
  if (s1 != s2) {
    kk_info_message("between: not equal slices: %p vs. %p\n", s1, s2);
    return kk_std_core__new_Sslice(kk_string_empty(), 0, -1, ctx); // invalid slice
  }
  kk_ssize_t start = (slice1.start <= slice2.start ? slice1.start : slice2.start);
  kk_ssize_t len   = (slice1.start <= slice2.start ? slice2.start - slice1.start : slice1.start - slice2.start);
  return kk_std_core__new_Sslice(slice1.str, start, len, ctx);
}

kk_std_core_types__maybe kk_slice_next( struct kk_std_core_Sslice slice, kk_context_t* ctx ) {
  if (slice.len <= 0) {
    kk_std_core__sslice_drop(slice,ctx);
    return kk_std_core_types__new_Nothing(ctx);
  }
  const uint8_t* start;
  const uint8_t* end;
  kk_sslice_start_end_borrow(slice, &start, &end);
  kk_ssize_t clen;
  const kk_char_t c = kk_utf8_read(start,&clen);
  kk_assert_internal(clen > 0 && clen <= slice.len);
  if (clen > slice.len) clen = slice.len;
  // TODO: specialize type to avoid boxing
  kk_std_core__sslice snext = kk_std_core__new_Sslice(slice.str, slice.start + clen, slice.len - clen, ctx);
  kk_std_core_types__tuple2_ res = kk_std_core_types__new_dash__lp__comma__rp_( kk_char_box(c,ctx), kk_std_core__sslice_box(snext,ctx), ctx);
  return kk_std_core_types__new_Just( kk_std_core_types__tuple2__box(res,ctx), ctx );
}

/* Borrow count */
struct kk_std_core_Sslice kk_slice_extend_borrow( struct kk_std_core_Sslice slice, kk_integer_t count, kk_context_t* ctx ) {
  kk_ssize_t cnt = kk_integer_clamp_borrow(count);
  if (cnt==0 || (slice.len <= 0 && cnt<0)) return slice;
  const uint8_t* s0;
  const uint8_t* s1;
  kk_sslice_start_end_borrow(slice,&s0,&s1);
  const uint8_t* t  = s1;
  if (cnt >= 0) {
    do {
      t = kk_utf8_next(t);
      cnt--;
    } while (cnt > 0 && *t != 0);
  }
  else {  // cnt < 0
    const uint8_t* sstart = s0 - slice.start;
    do {
      t = kk_utf8_prev(t);
      cnt++;
    } while (cnt < 0 && t > sstart);
  }
  if (t == s1) return slice;  // length is unchanged
  return kk_std_core__new_Sslice(slice.str, slice.start, (t < s0 ? 0 : (t - s0)), ctx);
}

/* Borrow count */
struct kk_std_core_Sslice kk_slice_advance_borrow( struct kk_std_core_Sslice slice, kk_integer_t count, kk_context_t* ctx ) {
  const kk_ssize_t cnt0 = kk_integer_clamp_borrow(count);
  kk_ssize_t cnt = cnt0;
  if (cnt==0 || (slice.start == 0 && cnt<0)) return slice;
  const uint8_t* sstart;
  const uint8_t* s0;
  const uint8_t* s1;
  const uint8_t* send;
  kk_sslice_start_end_borrowx(slice,&s0,&s1,&sstart,&send);
  // advance the start
  const uint8_t* t0  = s0;
  if (cnt >= 0) {
    do {
      t0 = kk_utf8_next(t0);
      cnt--;
    } while (cnt > 0 && t0 < send);
  }
  else {  // cnt < 0
    do {
      t0 = kk_utf8_prev(t0);
      cnt++;
    } while (cnt < 0 && t0 > sstart);
  }
  if (t0 == s0) return slice;  // start is unchanged
  // "t0" points to the new start, now advance the end by the same amount of codepoints
  const uint8_t* t1 = s1;
  cnt = cnt0;
  if (cnt >= 0) {
    do {
      t1 = kk_utf8_next(t1);
      cnt--;
    } while (cnt > 0 && t1 < send);
  }
  else {  // cnt < 0
    do {
      t1 = kk_utf8_prev(t1);
      cnt++;
    } while (cnt < 0 && t1 > sstart);
  }
  // t1 points to the new end
  kk_assert_internal(t1 >= t0);
  return kk_std_core__new_Sslice(slice.str, (t0 - sstart), (t1 - t0), ctx);
}

/* Borrow iupto */
struct kk_std_core_Sslice kk_slice_common_prefix_borrow( kk_string_t str1, kk_string_t str2, kk_integer_t iupto, kk_context_t* ctx ) {
  const uint8_t* s1 = kk_string_buf_borrow(str1,NULL);
  const uint8_t* s2 = kk_string_buf_borrow(str2,NULL);
  kk_ssize_t upto = kk_integer_clamp_ssize_t_borrow(iupto);
  kk_ssize_t count;
  for(count = 0; count < upto && *s1 != 0 && *s2 != 0; count++, s1++, s2++ ) {
    if (*s1 != *s2) break;
  }
  kk_string_drop(str2,ctx);
  return kk_std_core__new_Sslice(str1, 0, count, ctx);
}


kk_std_core__error kk_error_ok( kk_box_t result, kk_context_t* ctx ) {
  return kk_std_core__new_Ok( result, ctx );
}

kk_std_core__error kk_error_from_errno( int err, kk_context_t* ctx ) {  
  kk_string_t msg;
  #if defined(_GNU_SOURCE) && !defined(WIN32) && !defined(__APPLE__) && !defined(__FreeBSD__)
    // GNU version of strerror_r
    char buf[256];
    char* serr = strerror_r(err, buf, 255); buf[255] = 0;
    msg = kk_string_alloc_from_qutf8( serr, ctx );
  #elif (/* _POSIX_C_SOURCE >= 200112L ||*/ !defined(WIN32) && (_XOPEN_SOURCE >= 600 || defined(__APPLE__) || defined(__FreeBSD__)))
    // XSI version of strerror_r
    char buf[256];
    strerror_r(err, buf, 255); buf[255] = 0;
    msg = kk_string_alloc_from_qutf8( buf, ctx );
  #elif defined(_MSC_VER) || (__STDC_VERSION__ >= 201112L || __cplusplus >= 201103L)
    // MSVC, or C/C++ 11
    char buf[256];
    strerror_s(buf, 255, err); buf[255] = 0;
    msg = kk_string_alloc_from_qutf8( buf, ctx );
  #else
    // Old style
    msg = kk_string_alloc_from_qutf8( strerror(err), ctx );
  #endif
  return kk_std_core__new_Error( kk_std_core__new_Exception( msg, kk_std_core__new_ExnSystem(kk_reuse_null, kk_integer_from_int(err,ctx), ctx), ctx), ctx );  
}


kk_unit_t kk_assert_fail( kk_string_t msg, kk_context_t* ctx ) {
  kk_fatal_error(EINVAL, "assertion failed: %s\n", kk_string_cbuf_borrow(msg,NULL));
  kk_string_drop(msg,ctx);
  return kk_Unit;
}
kk_define_string_literal(, kk_std_core__tag_ExnError, 17, "std/core/ExnError")
kk_define_string_literal(, kk_std_core__tag_ExnAssert, 18, "std/core/ExnAssert")
kk_define_string_literal(, kk_std_core__tag_ExnTodo, 16, "std/core/ExnTodo")
kk_define_string_literal(, kk_std_core__tag_ExnRange, 17, "std/core/ExnRange")
kk_define_string_literal(, kk_std_core__tag_ExnPattern, 19, "std/core/ExnPattern")
kk_define_string_literal(, kk_std_core__tag_ExnSystem, 18, "std/core/ExnSystem")
kk_define_string_literal(, kk_std_core__tag_ExnInternal, 20, "std/core/ExnInternal")

kk_std_core__exception kk_std_core__copy(kk_std_core__exception _this, kk_std_core_types__optional message0, kk_std_core_types__optional info0, kk_context_t* _ctx) { /* (exception, message : optional<string>, info : optional<exception-info>) -> exception */ 
  kk_string_t _x20118;
  if (kk_std_core_types__is_Optional(message0)) {
    kk_box_t _box_x18175 = message0._cons.Optional.value;
    kk_string_t _message_1676 = kk_string_unbox(_box_x18175);
    kk_string_dup(_message_1676);
    kk_std_core_types__optional_drop(message0, _ctx);
    kk_Unit;
    _x20118 = _message_1676; /*string*/
    goto _match20119;
  }
  kk_string_t _x = _this.message;
  kk_string_dup(_x);
  _x20118 = _x; /*string*/
  _match20119: ;
  kk_std_core__exception_info _x20121;
  if (kk_std_core_types__is_Optional(info0)) {
    kk_box_t _box_x18176 = info0._cons.Optional.value;
    kk_std_core__exception_info _info_1682 = kk_std_core__exception_info_unbox(_box_x18176, NULL);
    kk_std_core__exception_info_dup(_info_1682);
    kk_std_core_types__optional_drop(info0, _ctx);
    kk_Unit;
    kk_std_core__exception_drop(_this, _ctx);
    kk_Unit;
    _x20121 = _info_1682; /*exception-info*/
    goto _match20122;
  }
  kk_std_core__exception_info _x0 = _this.info;
  kk_std_core__exception_info_dup(_x0);
  kk_std_core__exception_drop(_this, _ctx);
  kk_Unit;
  _x20121 = _x0; /*exception-info*/
  _match20122: ;
  return kk_std_core__new_Exception(_x20118, _x20121, _ctx);
}

kk_std_core__sslice kk_std_core__copy_2(kk_std_core__sslice _this, kk_std_core_types__optional str0, kk_std_core_types__optional start0, kk_std_core_types__optional len0, kk_context_t* _ctx) { /* (sslice, str : optional<string>, start : optional<ssize_t>, len : optional<ssize_t>) -> sslice */ 
  kk_string_t _x20128;
  if (kk_std_core_types__is_Optional(str0)) {
    kk_box_t _box_x18178 = str0._cons.Optional.value;
    kk_string_t _str_1814 = kk_string_unbox(_box_x18178);
    kk_string_dup(_str_1814);
    kk_std_core_types__optional_drop(str0, _ctx);
    kk_Unit;
    _x20128 = _str_1814; /*string*/
    goto _match20129;
  }
  kk_string_t _x = _this.str;
  kk_string_dup(_x);
  _x20128 = _x; /*string*/
  _match20129: ;
  kk_ssize_t _x20131;
  if (kk_std_core_types__is_Optional(start0)) {
    kk_box_t _box_x18179 = start0._cons.Optional.value;
    kk_ssize_t _start_1820 = kk_ssize_unbox(_box_x18179, NULL);
    kk_std_core_types__optional_drop(start0, _ctx);
    kk_Unit;
    _x20131 = _start_1820; /*ssize_t*/
    goto _match20132;
  }
  kk_ssize_t _x0 = _this.start;
  _x20131 = _x0; /*ssize_t*/
  _match20132: ;
  kk_ssize_t _x20134;
  if (kk_std_core_types__is_Optional(len0)) {
    kk_box_t _box_x18180 = len0._cons.Optional.value;
    kk_ssize_t _len_1826 = kk_ssize_unbox(_box_x18180, NULL);
    kk_std_core_types__optional_drop(len0, _ctx);
    kk_Unit;
    kk_std_core__sslice_drop(_this, _ctx);
    kk_Unit;
    _x20134 = _len_1826; /*ssize_t*/
    goto _match20135;
  }
  kk_ssize_t _x1 = _this.len;
  kk_std_core__sslice_drop(_this, _ctx);
  kk_Unit;
  _x20134 = _x1; /*ssize_t*/
  _match20135: ;
  return kk_std_core__new_Sslice(_x20128, _x20131, _x20134, _ctx);
}

kk_std_core__stream kk_std_core__copy_3(kk_std_core__stream _this, kk_std_core_types__optional head0, kk_std_core_types__optional tail0, kk_context_t* _ctx) { /* forall<a> (stream<a>, head : optional<a>, tail : optional<stream<a>>) -> stream<a> */ 
  kk_box_t _x20139;
  if (kk_std_core_types__is_Optional(head0)) {
    kk_box_t _head_1872 = head0._cons.Optional.value;
    _x20139 = _head_1872; /*1889*/
  }
  else {
    struct kk_std_core_Next* _con20140 = kk_std_core__as_Next(_this);
    kk_box_t _x = _con20140->head;
    kk_box_dup(_x);
    _x20139 = _x; /*1889*/
  }
  kk_std_core__stream _x20141;
  if (kk_std_core_types__is_Optional(tail0)) {
    kk_box_t _box_x18181 = tail0._cons.Optional.value;
    kk_std_core__stream _tail_1879 = kk_std_core__stream_unbox(_box_x18181, NULL);
    kk_std_core__stream_dup(_tail_1879);
    kk_std_core_types__optional_drop(tail0, _ctx);
    kk_Unit;
    kk_std_core__stream_drop(_this, _ctx);
    kk_Unit;
    _x20141 = _tail_1879; /*stream<1889>*/
    goto _match20142;
  }
  struct kk_std_core_Next* _con20144 = kk_std_core__as_Next(_this);
  kk_box_t _pat01 = _con20144->head;
  kk_std_core__stream _x0 = _con20144->tail;
  if (kk_likely(kk_std_core__stream_is_unique(_this))) {
    kk_box_drop(_pat01, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core__stream_free(_this);
  }
  else {
    kk_std_core__stream_dup(_x0);
    kk_Unit;
    kk_std_core__stream_decref(_this, _ctx);
    kk_Unit;
  }
  _x20141 = _x0; /*stream<1889>*/
  _match20142: ;
  return kk_std_core__new_Next(kk_reuse_null, _x20139, _x20141, _ctx);
}
 
// runtime tag for the `:exn` effect

kk_std_core_hnd__htag kk_std_core__tag_exn;
 
// handler for the `:exn` effect

kk_box_t kk_std_core__handle_exn(int32_t cfc, kk_std_core__hnd_exn hnd, kk_function_t ret, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e,b> (cfc : int32, hnd : .hnd-exn<e,b>, ret : (res : a) -> e b, action : () -> <exn|e> a) -> e b */ 
  kk_std_core_hnd__htag _b_18187_18182 = kk_std_core_hnd__htag_dup(kk_std_core__tag_exn); /*std/core/hnd/htag<.hnd-exn>*/;
  int32_t _b_18188_18183 = cfc; /*int32*/;
  kk_std_core__hnd_exn _b_18189_18184 = hnd; /*.hnd-exn<1956,1957>*/;
  kk_function_t _b_18190_18185 = ret; /*(res : 1955) -> 1956 1957*/;
  kk_function_t _b_18191_18186 = action; /*() -> <exn|1956> 1955*/;
  return kk_std_core_hnd__hhandle(_b_18187_18182, _b_18188_18183, kk_std_core__hnd_exn_box(_b_18189_18184, _ctx), _b_18190_18185, _b_18191_18186, _ctx);
}
 
// Unsafe: transform any type to a `null` type; used internally by the compiler.

kk_std_core__null kk_std_core__null_any(kk_box_t x, kk_context_t* _ctx) { /* forall<a> (x : a) -> null<a> */ 
  return ((x).box == kk_box_null.box ? kk_datatype_from_ptr(NULL) : kk_datatype_unbox(x));
}

kk_ref_t kk_std_core_redirect;
 
// Print a string to the console, including a final newline character.

kk_unit_t kk_std_core_xprintsln(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> console () */ 
  kk_println(s,kk_context()); return kk_Unit;
}

kk_integer_t kk_std_core_string_compare(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> int */ 
  return kk_string_cmp_int(x,y,kk_context());
}
 
// Convert an integer to an `:int32`. The number is _clamped_ to the maximal or minimum `:int32`
// value if it is outside the range of an `:int32`.

int32_t kk_std_core_int32(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int32 */ 
  return kk_integer_clamp32(i,kk_context());
}
 
// Convert an integer to an `:ssize_t`. The number is _clamped_ to the maximal or minimum `:ssize_t`
// value if it is outside the range of an `:ssize_t`.

kk_ssize_t kk_std_core_ssize__t(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> ssize_t */ 
  return kk_integer_clamp_ssize_t(i,kk_context());
}
 
// Convert a character to a string

kk_string_t kk_std_core_string(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> string */ 
  return kk_string_from_char(c,kk_context());
}
 
// Convert a vector of characters to a string.

kk_string_t kk_std_core_string_1(kk_vector_t _arg1, kk_context_t* _ctx) { /* (vector<char>) -> string */ 
  return kk_string_from_chars(_arg1,kk_context());
}
 
// Convert a list of characters to a string

kk_string_t kk_std_core_string_2(kk_std_core__list cs, kk_context_t* _ctx) { /* (cs : list<char>) -> total string */ 
  return kk_string_from_list(cs,kk_context());
}
 
// O(n). Copy the `slice` argument into a fresh string.
// Takes O(1) time if the slice covers the entire string.

kk_string_t kk_std_core_string_3(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> string */ 
  return kk_slice_to_string(slice0,kk_context());
}
 
// Convert a vector to a list with an optional tail.

kk_std_core__list kk_std_core_vlist(kk_vector_t v, kk_std_core_types__optional tail0, kk_context_t* _ctx) { /* forall<a> (v : vector<a>, tail : optional<list<a>>) -> list<a> */ 
  kk_std_core__list _x20150;
  if (kk_std_core_types__is_Optional(tail0)) {
    kk_box_t _box_x18195 = tail0._cons.Optional.value;
    kk_std_core__list _tail_2083 = kk_std_core__list_unbox(_box_x18195, NULL);
    kk_std_core__list_dup(_tail_2083);
    kk_std_core_types__optional_drop(tail0, _ctx);
    kk_Unit;
    _x20150 = _tail_2083; /*list<2093>*/
    goto _match20151;
  }
  _x20150 = kk_std_core__new_Nil(_ctx); /*list<2093>*/
  _match20151: ;
  return kk_vector_to_list(v,_x20150,kk_context());
}

kk_string_t kk_std_core_int_show_hex(kk_integer_t i, bool use_capitals, kk_context_t* _ctx) { /* (i : int, use-capitals : bool) -> string */ 
  return kk_integer_to_hex_string(i,use_capitals,kk_context());
}

kk_string_t kk_std_core_repeatz(kk_string_t s, kk_ssize_t n, kk_context_t* _ctx) { /* (s : string, n : ssize_t) -> string */ 
  return kk_string_repeat(s,n,kk_context());
}

kk_string_t kk_std_core_show_expx(double d, int32_t prec, kk_context_t* _ctx) { /* (d : double, prec : int32) -> string */ 
  return kk_double_show_exp(d,prec,kk_context());
}

kk_string_t kk_std_core_show_fixedx(double d, int32_t prec, kk_context_t* _ctx) { /* (d : double, prec : int32) -> string */ 
  return kk_double_show_fixed(d,prec,kk_context());
}
 
// Print a string to the console

kk_unit_t kk_std_core_xprints(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> console () */ 
  kk_print(s,kk_context()); return kk_Unit;
}
 
// Raise an integer `i` to the power of `exp`.

kk_integer_t kk_std_core_pow(kk_integer_t i, kk_integer_t exp, kk_context_t* _ctx) { /* (i : int, exp : int) -> int */ 
  return kk_integer_pow(i,exp,kk_context());
}
 
// O(`count`). Advance the start position of a string slice by `count` characters
// up to the end of the string.
// A negative `count` advances the start position backwards upto the first position
// in a string.
// Maintains the character count of the original slice upto the end of the string.
// For example:
//
// * `"abc".first.advance(1).string == "b"`,
// * `"abc".first.advance(3).string == ""`,
// * `"abc".last.advance(-1).string == "b"`.
//

kk_std_core__sslice kk_std_core_advance(kk_std_core__sslice slice0, kk_integer_t count, kk_context_t* _ctx) { /* (slice : sslice, count : int) -> sslice */ 
  return kk_slice_advance_borrow(slice0,count,kk_context());
}

kk_unit_t kk_std_core_unsafe_assert_fail(kk_string_t msg, kk_context_t* _ctx) { /* (msg : string) -> () */ 
  kk_assert_fail(msg,kk_context()); return kk_Unit;
}
 
// O(`count`). Extend a string slice by `count` characters up to the end of the string.
// A negative `count` shrinks the slice up to the empty slice.
// For example:
//
// * `"abc".first.extend(1).string == "ab"`
// * `"abc".last.extend(-1).string == ""`
//

kk_std_core__sslice kk_std_core_extend(kk_std_core__sslice slice0, kk_integer_t count, kk_context_t* _ctx) { /* (slice : sslice, count : int) -> sslice */ 
  return kk_slice_extend_borrow(slice0,count,kk_context());
}

kk_std_core__sslice kk_std_core_first1(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> sslice */ 
  return kk_slice_first(s,kk_context());
}
 
// Convert a string to upper-case

kk_string_t kk_std_core_to_upper(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  return kk_string_to_upper(s,kk_context());
}

kk_integer_t kk_std_core_cdiv_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx) { /* (i : int, n : int) -> int */ 
  return kk_integer_cdiv_pow10(i,n,kk_context());
}

kk_integer_t kk_std_core_mul_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx) { /* (i : int, n : int) -> int */ 
  return kk_integer_mul_pow10(i,n,kk_context());
}
 
// Return the common prefix of two strings (upto `upto` characters (default is minimum length of the two strings))

kk_std_core__sslice kk_std_core_common_prefix(kk_string_t s, kk_string_t t, kk_std_core_types__optional upto, kk_context_t* _ctx) { /* (s : string, t : string, upto : optional<int>) -> sslice */ 
  kk_integer_t _borrowedArg_19999;
  if (kk_std_core_types__is_Optional(upto)) {
    kk_box_t _box_x18196 = upto._cons.Optional.value;
    kk_integer_t _upto_2182 = kk_integer_unbox(_box_x18196);
    _borrowedArg_19999 = kk_integer_dup(_upto_2182); /*int*/
    goto _match20153;
  }
  _borrowedArg_19999 = kk_integer_from_small(-1); /*int*/
  _match20153: ;
  kk_std_core__sslice _borrowApp_20000 = kk_slice_common_prefix_borrow(s,t,_borrowedArg_19999,kk_context()); /*sslice*/;
  kk_integer_drop(_borrowedArg_19999, _ctx);
  return _borrowApp_20000;
}
 
// lifted

kk_std_core__list kk_std_core__ctail_lift17066_concat(kk_std_core__list ys, kk_std_core__list zss, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, zss : list<list<a>>, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20155 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con20155->head;
    kk_std_core__list yy = _con20155->tail;
    kk_reuse_t _ru_20010 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20010 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_std_core__list _ctail_17144 = kk_std_core__list_hole(); /*list<2230>*/;
    kk_std_core__list _ctail_17145;
    if (kk_likely(_ru_20010!=NULL)) {
      struct kk_std_core_Cons* _con20156 = (struct kk_std_core_Cons*)_ru_20010;
      _con20156->tail = _ctail_17144;
      _ctail_17145 = kk_std_core__base_Cons(_con20156); /*list<2230>*/
    }
    else {
      _ctail_17145 = kk_std_core__new_Cons(kk_reuse_null, y, _ctail_17144, _ctx); /*list<2230>*/
    }
    { // tailcall
      kk_std_core_types__ctail _x20157;
      kk_box_t* field_17587 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17145)->tail)); /*cfield<list<2230>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18203 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18203, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17145, _ctx);
        kk_std_core__list _b_18218_18208 = head0; /*list<2230>*/;
        kk_box_t* _b_18219_18209 = field_17587; /*cfield<list<2230>>*/;
        _x20157 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18218_18208, _ctx), _b_18219_18209, _ctx); /*ctail<58>*/
        goto _match20158;
      }
      kk_std_core__list _b_18220_18210 = _ctail_17145; /*list<2230>*/;
      kk_box_t* _b_18221_18211 = field_17587; /*cfield<list<2230>>*/;
      _x20157 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18220_18210, _ctx), _b_18221_18211, _ctx); /*ctail<58>*/
      _match20158: ;
      ys = yy;
      _acc = _x20157;
      goto kk__tailcall;
    }
  }
  if (kk_std_core__is_Cons(zss)) {
    struct kk_std_core_Cons* _con20160 = kk_std_core__as_Cons(zss);
    kk_box_t _box_x18212 = _con20160->head;
    kk_std_core__list zzs = _con20160->tail;
    kk_std_core__list zs = kk_std_core__list_unbox(_box_x18212, NULL);
    if (kk_likely(kk_std_core__list_is_unique(zss))) {
      kk_std_core__list_free(zss);
    }
    else {
      kk_std_core__list_dup(zs);
      kk_std_core__list_dup(zzs);
      kk_Unit;
      kk_std_core__list_decref(zss, _ctx);
      kk_Unit;
    }
    { // tailcall
      ys = zs;
      zss = zzs;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18213 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18213, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17066_concat(kk_std_core__list ys0, kk_std_core__list zss0, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, zss : list<list<a>>) -> list<a> */ 
  return kk_std_core__ctail_lift17066_concat(ys0, zss0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
extern kk_box_t kk_std_core_const_fun20163_1(kk_function_t _fself, kk_box_t ___wildcard__123__7, kk_context_t* _ctx) {
  struct kk_std_core_const_fun20163__t_1* _self = kk_function_as(struct kk_std_core_const_fun20163__t_1*, _fself);
  kk_box_t default0 = _self->default0; /* 2258 */
  kk_drop_match(_self, {kk_box_dup(default0);}, {}, _ctx)
  kk_box_drop(___wildcard__123__7, _ctx);
  return default0;
}
 
// If the slice is not empty,
// return the first character, and a new slice that is advanced by 1.

kk_std_core_types__maybe kk_std_core_next(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> maybe<(char, sslice)> */ 
  return kk_slice_next(slice0,kk_context());
}
 
// Return the number of decimal digits of `i`. Return `0` when `i==0`.

kk_integer_t kk_std_core_count_digits(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int */ 
  return kk_integer_count_digits(i,kk_context());
}
 
// Transform an `:error` type to an `:either` value.

kk_std_core_types__either kk_std_core_either(kk_std_core__error t, kk_context_t* _ctx) { /* forall<a> (t : error<a>) -> either<exception,a> */ 
  if (kk_std_core__is_Error(t)) {
    kk_std_core__exception exn0 = t._cons.Error.exception;
    kk_std_core__exception_dup(exn0);
    kk_std_core__error_drop(t, _ctx);
    kk_Unit;
    kk_std_core__exception _b_18223_18222 = exn0; /*exception*/;
    return kk_std_core_types__new_Left(kk_std_core__exception_box(_b_18223_18222, _ctx), _ctx);
  }
  kk_box_t x = t._cons.Ok.result;
  kk_box_dup(x);
  kk_std_core__error_drop(t, _ctx);
  kk_Unit;
  return kk_std_core_types__new_Right(x, _ctx);
}

bool kk_std_core_xends_with(kk_string_t s, kk_string_t post, kk_context_t* _ctx) { /* (s : string, post : string) -> bool */ 
  return kk_string_ends_with(s,post,kk_context());
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17511_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_std_core__list zz, kk_std_core__list ys1_17092, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, f : (a) -> e list<b>, zz : list<a>, ys1.17092 : list<b>) -> e list<b> */ 
  return kk_std_core__ctail_lift17067_flatmap(f, ys1_17092, zz, _acc, _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17512_op(kk_function_t _accm, kk_function_t f0, kk_std_core__list zz0, kk_std_core__list ys1_170920, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, f : (a) -> e list<b>, zz : list<a>, ys1.17092 : list<b>) -> e list<b> */ 
  return kk_std_core__ctailm_lift17067_flatmap(f0, ys1_170920, zz0, _accm, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__ctail_lift17067_flatmap_fun20171__t {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_std_core__list zz1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_lift17067_flatmap_fun20171(kk_function_t _fself, kk_std_core__list ys1_170921, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_lift17067_flatmap_fun20171(kk_function_t f1, kk_std_core__list zz1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17067_flatmap_fun20171__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_lift17067_flatmap_fun20171__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_lift17067_flatmap_fun20171, kk_context());
  _self->f1 = f1;
  _self->zz1 = zz1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_lift17067_flatmap_fun20171(kk_function_t _fself, kk_std_core__list ys1_170921, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17067_flatmap_fun20171__t* _self = kk_function_as(struct kk_std_core__ctail_lift17067_flatmap_fun20171__t*, _fself);
  kk_function_t f1 = _self->f1; /* (2407) -> 2409 list<2408> */
  kk_std_core__list zz1 = _self->zz1; /* list<2407> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<2408>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_std_core__list_dup(zz1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17511_op(_acc0, f1, zz1, ys1_170921, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_lift17067_flatmap_fun20173__t {
  struct kk_function_s _base;
  kk_function_t _b_18250_18239;
};
static kk_box_t kk_std_core__ctail_lift17067_flatmap_fun20173(kk_function_t _fself, kk_box_t _b_18240, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_lift17067_flatmap_fun20173(kk_function_t _b_18250_18239, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17067_flatmap_fun20173__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_lift17067_flatmap_fun20173__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_lift17067_flatmap_fun20173, kk_context());
  _self->_b_18250_18239 = _b_18250_18239;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_lift17067_flatmap_fun20173(kk_function_t _fself, kk_box_t _b_18240, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17067_flatmap_fun20173__t* _self = kk_function_as(struct kk_std_core__ctail_lift17067_flatmap_fun20173__t*, _fself);
  kk_function_t _b_18250_18239 = _self->_b_18250_18239; /* (ys1.170921 : list<2408>) -> 2409 list<2408> */
  kk_drop_match(_self, {kk_function_dup(_b_18250_18239);}, {}, _ctx)
  kk_std_core__list _x20174;
  kk_std_core__list _x20175 = kk_std_core__list_unbox(_b_18240, _ctx); /*list<2408>*/
  _x20174 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_18250_18239, (_b_18250_18239, _x20175, _ctx)); /*list<2408>*/
  return kk_std_core__list_box(_x20174, _ctx);
}

kk_std_core__list kk_std_core__ctail_lift17067_flatmap(kk_function_t f1, kk_std_core__list ys, kk_std_core__list zs, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e list<b>, ys : list<b>, zs : list<a>, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20164 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con20164->head;
    kk_std_core__list yy = _con20164->tail;
    kk_reuse_t _ru_20012 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20012 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_std_core__list _ctail_17146 = kk_std_core__list_hole(); /*list<2408>*/;
    kk_std_core__list _ctail_17147;
    if (kk_likely(_ru_20012!=NULL)) {
      struct kk_std_core_Cons* _con20165 = (struct kk_std_core_Cons*)_ru_20012;
      _con20165->tail = _ctail_17146;
      _ctail_17147 = kk_std_core__base_Cons(_con20165); /*list<2408>*/
    }
    else {
      _ctail_17147 = kk_std_core__new_Cons(kk_reuse_null, y, _ctail_17146, _ctx); /*list<2408>*/
    }
    { // tailcall
      kk_std_core_types__ctail _x20166;
      kk_box_t* field_17592 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17147)->tail)); /*cfield<list<2408>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x18230 = _acc0._cons.CTail._field1;
        kk_box_t* last0 = _acc0._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18230, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17147, _ctx);
        kk_std_core__list _b_18246_18235 = head0; /*list<2408>*/;
        kk_box_t* _b_18247_18236 = field_17592; /*cfield<list<2408>>*/;
        _x20166 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18246_18235, _ctx), _b_18247_18236, _ctx); /*ctail<58>*/
        goto _match20167;
      }
      kk_std_core__list _b_18248_18237 = _ctail_17147; /*list<2408>*/;
      kk_box_t* _b_18249_18238 = field_17592; /*cfield<list<2408>>*/;
      _x20166 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18248_18237, _ctx), _b_18249_18238, _ctx); /*ctail<58>*/
      _match20167: ;
      ys = yy;
      _acc0 = _x20166;
      goto kk__tailcall;
    }
  }
  if (kk_std_core__is_Cons(zs)) {
    struct kk_std_core_Cons* _con20169 = kk_std_core__as_Cons(zs);
    kk_box_t z = _con20169->head;
    kk_std_core__list zz1 = _con20169->tail;
    if (kk_likely(kk_std_core__list_is_unique(zs))) {
      kk_std_core__list_free(zs);
    }
    else {
      kk_box_dup(z);
      kk_std_core__list_dup(zz1);
      kk_Unit;
      kk_std_core__list_decref(zs, _ctx);
      kk_Unit;
    }
    kk_std_core__list x_17593;
    kk_function_t _x20170 = kk_function_dup(f1); /*(2407) -> 2409 list<2408>*/
    x_17593 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _x20170, (_x20170, z, _ctx)); /*list<2408>*/
    if (kk_yielding(kk_context())) {
      kk_std_core__list_drop(x_17593, _ctx);
      kk_function_t _b_18250_18239 = kk_std_core__new_ctail_lift17067_flatmap_fun20171(f1, zz1, _acc0, _ctx); /*(ys1.170921 : list<2408>) -> 2409 list<2408>*/;
      kk_box_t _x20172 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_lift17067_flatmap_fun20173(_b_18250_18239, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20172, _ctx);
    }
    { // tailcall
      ys = x_17593;
      zs = zz1;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18241 = _acc0._cons.CTail._field1;
    kk_box_t* last2 = _acc0._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18241, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__ctailm_lift17067_flatmap_fun20179__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t y0;
};
static kk_std_core__list kk_std_core__ctailm_lift17067_flatmap_fun20179(kk_function_t _fself, kk_std_core__list _ctail_17149, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17067_flatmap_fun20179(kk_function_t _accm0, kk_box_t y0, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17067_flatmap_fun20179__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17067_flatmap_fun20179__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17067_flatmap_fun20179, kk_context());
  _self->_accm0 = _accm0;
  _self->y0 = y0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_lift17067_flatmap_fun20179(kk_function_t _fself, kk_std_core__list _ctail_17149, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17067_flatmap_fun20179__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17067_flatmap_fun20179__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<2408>) -> list<2408> */
  kk_box_t y0 = _self->y0; /* 2408 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(y0);}, {}, _ctx)
  kk_std_core__list _x20180 = kk_std_core__new_Cons(kk_reuse_null, y0, _ctail_17149, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x20180, _ctx));
}


// lift anonymous function
struct kk_std_core__ctailm_lift17067_flatmap_fun20183__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_std_core__list zz2;
};
static kk_std_core__list kk_std_core__ctailm_lift17067_flatmap_fun20183(kk_function_t _fself, kk_std_core__list ys1_170923, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17067_flatmap_fun20183(kk_function_t _accm0, kk_function_t f2, kk_std_core__list zz2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17067_flatmap_fun20183__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17067_flatmap_fun20183__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17067_flatmap_fun20183, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->zz2 = zz2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_lift17067_flatmap_fun20183(kk_function_t _fself, kk_std_core__list ys1_170923, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17067_flatmap_fun20183__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17067_flatmap_fun20183__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<2408>) -> list<2408> */
  kk_function_t f2 = _self->f2; /* (2407) -> 2409 list<2408> */
  kk_std_core__list zz2 = _self->zz2; /* list<2407> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_std_core__list_dup(zz2);}, {}, _ctx)
  return kk_std_core__mlift17512_op(_accm0, f2, zz2, ys1_170923, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_lift17067_flatmap_fun20185__t {
  struct kk_function_s _base;
  kk_function_t _b_18253_18251;
};
static kk_box_t kk_std_core__ctailm_lift17067_flatmap_fun20185(kk_function_t _fself, kk_box_t _b_18252, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17067_flatmap_fun20185(kk_function_t _b_18253_18251, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17067_flatmap_fun20185__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17067_flatmap_fun20185__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17067_flatmap_fun20185, kk_context());
  _self->_b_18253_18251 = _b_18253_18251;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_lift17067_flatmap_fun20185(kk_function_t _fself, kk_box_t _b_18252, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17067_flatmap_fun20185__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17067_flatmap_fun20185__t*, _fself);
  kk_function_t _b_18253_18251 = _self->_b_18253_18251; /* (ys1.170923 : list<2408>) -> 2409 list<2408> */
  kk_drop_match(_self, {kk_function_dup(_b_18253_18251);}, {}, _ctx)
  kk_std_core__list _x20186;
  kk_std_core__list _x20187 = kk_std_core__list_unbox(_b_18252, _ctx); /*list<2408>*/
  _x20186 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_18253_18251, (_b_18253_18251, _x20187, _ctx)); /*list<2408>*/
  return kk_std_core__list_box(_x20186, _ctx);
}

kk_std_core__list kk_std_core__ctailm_lift17067_flatmap(kk_function_t f2, kk_std_core__list ys0, kk_std_core__list zs0, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e list<b>, ys : list<b>, zs : list<a>, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys0)) {
    struct kk_std_core_Cons* _con20177 = kk_std_core__as_Cons(ys0);
    kk_box_t y0 = _con20177->head;
    kk_std_core__list yy0 = _con20177->tail;
    if (kk_likely(kk_std_core__list_is_unique(ys0))) {
      kk_std_core__list_free(ys0);
    }
    else {
      kk_box_dup(y0);
      kk_std_core__list_dup(yy0);
      kk_Unit;
      kk_std_core__list_decref(ys0, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_function_t _x20178 = kk_std_core__new_ctailm_lift17067_flatmap_fun20179(_accm0, y0, _ctx); /*(list<2408>) -> list<2408>*/
      ys0 = yy0;
      _accm0 = _x20178;
      goto kk__tailcall;
    }
  }
  if (kk_std_core__is_Cons(zs0)) {
    struct kk_std_core_Cons* _con20181 = kk_std_core__as_Cons(zs0);
    kk_box_t z0 = _con20181->head;
    kk_std_core__list zz2 = _con20181->tail;
    if (kk_likely(kk_std_core__list_is_unique(zs0))) {
      kk_std_core__list_free(zs0);
    }
    else {
      kk_box_dup(z0);
      kk_std_core__list_dup(zz2);
      kk_Unit;
      kk_std_core__list_decref(zs0, _ctx);
      kk_Unit;
    }
    kk_std_core__list x0_17598;
    kk_function_t _x20182 = kk_function_dup(f2); /*(2407) -> 2409 list<2408>*/
    x0_17598 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _x20182, (_x20182, z0, _ctx)); /*list<2408>*/
    if (kk_yielding(kk_context())) {
      kk_std_core__list_drop(x0_17598, _ctx);
      kk_function_t _b_18253_18251 = kk_std_core__new_ctailm_lift17067_flatmap_fun20183(_accm0, f2, zz2, _ctx); /*(ys1.170923 : list<2408>) -> 2409 list<2408>*/;
      kk_box_t _x20184 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_lift17067_flatmap_fun20185(_b_18253_18251, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20184, _ctx);
    }
    { // tailcall
      ys0 = x0_17598;
      zs0 = zz2;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17067_flatmap_fun20188__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core__lift17067_flatmap_fun20188(kk_function_t _fself, kk_std_core__list _ctail_17148, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17067_flatmap_fun20188(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core__lift17067_flatmap_fun20188, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core__lift17067_flatmap_fun20188(kk_function_t _fself, kk_std_core__list _ctail_17148, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17148;
}

kk_std_core__list kk_std_core__lift17067_flatmap(kk_function_t f3, kk_std_core__list ys1, kk_std_core__list zs1, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e list<b>, ys : list<b>, zs : list<a>) -> e list<b> */ 
  bool _match_19996 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19996) {
    return kk_std_core__ctail_lift17067_flatmap(f3, ys1, zs1, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_lift17067_flatmap(f3, ys1, zs1, kk_std_core__new_lift17067_flatmap_fun20188(_ctx), _ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17068_reverse_append(kk_std_core__list acc, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a> (acc : list<a>, ys : list<a>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20189 = kk_std_core__as_Cons(ys);
    kk_box_t x = _con20189->head;
    kk_std_core__list xx = _con20189->tail;
    kk_reuse_t _ru_20016 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20016 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x20190;
      if (kk_likely(_ru_20016!=NULL)) {
        struct kk_std_core_Cons* _con20191 = (struct kk_std_core_Cons*)_ru_20016;
        _con20191->tail = acc;
        _x20190 = kk_std_core__base_Cons(_con20191); /*list<61>*/
      }
      else {
        _x20190 = kk_std_core__new_Cons(kk_reuse_null, x, acc, _ctx); /*list<61>*/
      }
      acc = _x20190;
      ys = xx;
      goto kk__tailcall;
    }
  }
  return acc;
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17514_force_fun20196__t {
  struct kk_function_s _base;
  kk_ref_t r;
};
static kk_box_t kk_std_core__mlift17514_force_fun20196(kk_function_t _fself, kk_box_t x00, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17514_force_fun20196(kk_ref_t r, kk_context_t* _ctx) {
  struct kk_std_core__mlift17514_force_fun20196__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17514_force_fun20196__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17514_force_fun20196, kk_context());
  _self->r = r;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17514_force_fun20196(kk_function_t _fself, kk_box_t x00, kk_context_t* _ctx) {
  struct kk_std_core__mlift17514_force_fun20196__t* _self = kk_function_as(struct kk_std_core__mlift17514_force_fun20196__t*, _fself);
  kk_ref_t r = _self->r; /* ref<global,either<() -> 2544 2543,2543>> */
  kk_drop_match(_self, {kk_ref_dup(r);}, {}, _ctx)
  kk_unit_t __ = kk_Unit;
  kk_unit_t _borrowApp_19994 = kk_Unit;
  kk_box_t _x20197;
  kk_std_core_types__either _x20198;
  kk_box_t _x20199 = kk_box_dup(x00); /*2543*/
  _x20198 = kk_std_core_types__new_Right(_x20199, _ctx); /*either<70,71>*/
  _x20197 = kk_std_core_types__either_box(_x20198, _ctx); /*168*/
  kk_ref_set_borrow(r,_x20197,kk_context());
  kk_ref_drop(r, _ctx);
  _borrowApp_19994;
  return x00;
}

kk_box_t kk_std_core__mlift17514_force(kk_ref_t r, kk_std_core_types__either _y_17234, kk_context_t* _ctx) { /* forall<a,e> (r : ref<global,either<() -> e a,a>>, either<() -> e a,a>) -> <read<global>,div,alloc<global>,write<global>|e> a */ 
  if (kk_std_core_types__is_Right(_y_17234)) {
    kk_box_t x = _y_17234._cons.Right.right;
    kk_ref_drop(r, _ctx);
    return x;
  }
  kk_box_t _fun_unbox_x18259 = _y_17234._cons.Left.left;
  kk_box_t x0_17601;
  kk_function_t _x20195 = kk_function_unbox(_fun_unbox_x18259); /*() -> 2544 18260*/
  x0_17601 = kk_function_call(kk_box_t, (kk_function_t, kk_context_t*), _x20195, (_x20195, _ctx)); /*2543*/
  kk_function_t next0_17602 = kk_std_core__new_mlift17514_force_fun20196(r, _ctx); /*(2543) -> <st<global>,div|2544> 2543*/;
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17601, _ctx);
    return kk_std_core_hnd_yield_extend(next0_17602, _ctx);
  }
  return kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), next0_17602, (next0_17602, x0_17601, _ctx));
}
 
// Force a delayed value; the value is computed only on the first
// call to `force` and cached afterwards.


// lift anonymous function
struct kk_std_core_force_fun20202__t {
  struct kk_function_s _base;
  kk_std_core__delayed delayed;
};
static kk_box_t kk_std_core_force_fun20202(kk_function_t _fself, kk_std_core_types__either _y_17234, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_force_fun20202(kk_std_core__delayed delayed, kk_context_t* _ctx) {
  struct kk_std_core_force_fun20202__t* _self = kk_function_alloc_as(struct kk_std_core_force_fun20202__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_force_fun20202, kk_context());
  _self->delayed = delayed;
  return &_self->_base;
}

static kk_box_t kk_std_core_force_fun20202(kk_function_t _fself, kk_std_core_types__either _y_17234, kk_context_t* _ctx) {
  struct kk_std_core_force_fun20202__t* _self = kk_function_as(struct kk_std_core_force_fun20202__t*, _fself);
  kk_std_core__delayed delayed = _self->delayed; /* delayed<2544,2543> */
  kk_drop_match(_self, {kk_std_core__delayed_dup(delayed);}, {}, _ctx)
  kk_ref_t _x20203;
  kk_ref_t _x0 = delayed.dref;
  _x20203 = _x0; /*ref<global,either<() -> 2544 2543,2543>>*/
  return kk_std_core__mlift17514_force(_x20203, _y_17234, _ctx);
}


// lift anonymous function
struct kk_std_core_force_fun20204__t {
  struct kk_function_s _base;
  kk_function_t _b_18269_18267;
};
static kk_box_t kk_std_core_force_fun20204(kk_function_t _fself, kk_box_t _b_18268, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_force_fun20204(kk_function_t _b_18269_18267, kk_context_t* _ctx) {
  struct kk_std_core_force_fun20204__t* _self = kk_function_alloc_as(struct kk_std_core_force_fun20204__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_force_fun20204, kk_context());
  _self->_b_18269_18267 = _b_18269_18267;
  return &_self->_base;
}

static kk_box_t kk_std_core_force_fun20204(kk_function_t _fself, kk_box_t _b_18268, kk_context_t* _ctx) {
  struct kk_std_core_force_fun20204__t* _self = kk_function_as(struct kk_std_core_force_fun20204__t*, _fself);
  kk_function_t _b_18269_18267 = _self->_b_18269_18267; /* (either<() -> 2544 2543,2543>) -> <read<global>,div,alloc<global>,write<global>|2544> 2543 */
  kk_drop_match(_self, {kk_function_dup(_b_18269_18267);}, {}, _ctx)
  kk_std_core_types__either _x20205 = kk_std_core_types__either_unbox(_b_18268, _ctx); /*either<() -> 2544 2543,2543>*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_types__either, kk_context_t*), _b_18269_18267, (_b_18269_18267, _x20205, _ctx));
}

kk_box_t kk_std_core_force(kk_std_core__delayed delayed, kk_context_t* _ctx) { /* forall<a,e> (delayed : delayed<e,a>) -> e a */ 
  kk_std_core_types__either x_17608;
  kk_box_t _x20200;
  kk_ref_t _x20201;
  kk_ref_t _x = delayed.dref;
  kk_ref_dup(_x);
  _x20201 = _x; /*ref<global,either<() -> 2544 2543,2543>>*/
  _x20200 = kk_ref_get(_x20201,kk_context()); /*182*/
  x_17608 = kk_std_core_types__either_unbox(_x20200, _ctx); /*either<() -> 2544 2543,2543>*/
  kk_function_t next0_17609 = kk_std_core_new_force_fun20202(delayed, _ctx); /*(either<() -> 2544 2543,2543>) -> <read<global>,div,alloc<global>,write<global>|2544> 2543*/;
  if (kk_yielding(kk_context())) {
    kk_std_core_types__either_drop(x_17608, _ctx);
    kk_function_t _b_18269_18267 = next0_17609; /*(either<() -> 2544 2543,2543>) -> <read<global>,div,alloc<global>,write<global>|2544> 2543*/;
    return kk_std_core_hnd_yield_extend(kk_std_core_new_force_fun20204(_b_18269_18267, _ctx), _ctx);
  }
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_types__either, kk_context_t*), next0_17609, (next0_17609, x_17608, _ctx));
}
 
// Generic show: shows the internal representation of an object as a string
// Note: this breaks parametricity so it should not be public

kk_string_t kk_std_core_gshow(kk_box_t _arg1, kk_context_t* _ctx) { /* forall<a> (a) -> string */ 
  return kk_show_any(_arg1,kk_context());
}
 
// Return the host environment: `dotnet`, `browser`, `webworker`, `node`, or `libc`.

kk_string_t kk_std_core_host(kk_context_t* _ctx) { /* () -> ndet string */ 
  return kk_get_host(kk_context());
}

int64_t kk_std_core_int64(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int64 */ 
  return kk_integer_clamp64_borrow(i);
}
 
// lifted

kk_std_core__list kk_std_core__ctail_lift17069_intersperse(kk_std_core__list ys, kk_box_t s, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, s : a, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20206 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con20206->head;
    kk_std_core__list yy = _con20206->tail;
    kk_reuse_t _ru_20017 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20017 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_std_core__list _ctail_17150;
    kk_std_core__list _ru_20109 = kk_std_core__list_hole(); /*list<2575>*/;
    if (kk_likely(_ru_20017!=NULL)) {
      struct kk_std_core_Cons* _con20207 = (struct kk_std_core_Cons*)_ru_20017;
      _con20207->tail = _ru_20109;
      _ctail_17150 = kk_std_core__base_Cons(_con20207); /*list<2575>*/
    }
    else {
      _ctail_17150 = kk_std_core__new_Cons(kk_reuse_null, y, _ru_20109, _ctx); /*list<2575>*/
    }
    { // tailcall
      kk_box_t _x20208 = kk_box_dup(s); /*2575*/
      kk_std_core_types__ctail _x20209;
      kk_std_core__list res_17611;
      kk_std_core__list _x20210 = kk_std_core__list_dup(_ctail_17150); /*list<2575>*/
      res_17611 = kk_std_core__new_Cons(kk_reuse_null, s, _x20210, _ctx); /*list<2575>*/
      kk_box_t* field_17612;
      kk_std_core__list_drop(_ctail_17150, _ctx);
      field_17612 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17150)->tail)); /*cfield<list<2575>>*/
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18276 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18276, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(res_17611, _ctx);
        kk_std_core__list _b_18290_18281 = head0; /*list<2575>*/;
        kk_box_t* _b_18291_18282 = field_17612; /*cfield<list<2575>>*/;
        _x20209 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18290_18281, _ctx), _b_18291_18282, _ctx); /*ctail<58>*/
        goto _match20211;
      }
      kk_std_core__list _b_18292_18283 = res_17611; /*list<2575>*/;
      kk_box_t* _b_18293_18284 = field_17612; /*cfield<list<2575>>*/;
      _x20209 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18292_18283, _ctx), _b_18293_18284, _ctx); /*ctail<58>*/
      _match20211: ;
      ys = yy;
      s = _x20208;
      _acc = _x20209;
      goto kk__tailcall;
    }
  }
  kk_box_drop(s, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18285 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18285, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17069_intersperse(kk_std_core__list ys0, kk_box_t s0, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, s : a) -> list<a> */ 
  return kk_std_core__ctail_lift17069_intersperse(ys0, s0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Insert a separator `sep`  between all elements of a list `xs` .

kk_std_core__list kk_std_core_intersperse(kk_std_core__list xs, kk_box_t sep, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, sep : a) -> list<a> */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20214 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con20214->head;
    kk_std_core__list xx = _con20214->tail;
    kk_reuse_t _ru_20018 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20018 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_std_core__list _ru_20110 = kk_std_core__lift17069_intersperse(xx, sep, _ctx); /*list<2575>*/;
    if (kk_likely(_ru_20018!=NULL)) {
      struct kk_std_core_Cons* _con20215 = (struct kk_std_core_Cons*)_ru_20018;
      _con20215->tail = _ru_20110;
      return kk_std_core__base_Cons(_con20215);
    }
    return kk_std_core__new_Cons(kk_reuse_null, x, _ru_20110, _ctx);
  }
  kk_box_drop(sep, _ctx);
  return kk_std_core__new_Nil(_ctx);
}
 
// Return the number of ending `0` digits of `i`. Return `0` when `i==0`.

kk_integer_t kk_std_core_is_exp10(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int */ 
  return kk_integer_ctz(i,kk_context());
}

kk_std_core__sslice kk_std_core_last1(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> sslice */ 
  return kk_slice_last(s,kk_context());
}
 
// Used by the compiler to wrap main console applications

kk_box_t kk_std_core_main_console(kk_function_t main, kk_context_t* _ctx) { /* forall<a,e> (main : () -> e a) -> e a */ 
  return kk_main_console(main,kk_context());
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17515_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_std_core__list yy, kk_box_t _ctail_17152, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, f : (value : a, rest : list<a>) -> e b, yy : list<a>, b) -> e list<b> */ 
  kk_std_core__list _ctail_17153 = kk_std_core__list_hole(); /*list<2711>*/;
  kk_std_core__list _ctail_17154 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17152, _ctail_17153, _ctx); /*list<2711>*/;
  kk_std_core_types__ctail _x20216;
  kk_box_t* field_17617 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17154)->tail)); /*cfield<list<2711>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18300 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18300, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17154, _ctx);
    kk_std_core__list _b_18309_18305 = head0; /*list<2711>*/;
    kk_box_t* _b_18310_18306 = field_17617; /*cfield<list<2711>>*/;
    _x20216 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18309_18305, _ctx), _b_18310_18306, _ctx); /*ctail<58>*/
    goto _match20217;
  }
  kk_std_core__list _b_18311_18307 = _ctail_17154; /*list<2711>*/;
  kk_box_t* _b_18312_18308 = field_17617; /*cfield<list<2711>>*/;
  _x20216 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18311_18307, _ctx), _b_18312_18308, _ctx); /*ctail<58>*/
  _match20217: ;
  return kk_std_core__ctail_lift17070_map_peek(f, yy, _x20216, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17516_op_fun20219__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t _ctail_17157;
};
static kk_std_core__list kk_std_core__mlift17516_op_fun20219(kk_function_t _fself, kk_std_core__list _ctail_17156, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17516_op_fun20219(kk_function_t _accm, kk_box_t _ctail_17157, kk_context_t* _ctx) {
  struct kk_std_core__mlift17516_op_fun20219__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17516_op_fun20219__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17516_op_fun20219, kk_context());
  _self->_accm = _accm;
  _self->_ctail_17157 = _ctail_17157;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17516_op_fun20219(kk_function_t _fself, kk_std_core__list _ctail_17156, kk_context_t* _ctx) {
  struct kk_std_core__mlift17516_op_fun20219__t* _self = kk_function_as(struct kk_std_core__mlift17516_op_fun20219__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<2711>) -> list<2711> */
  kk_box_t _ctail_17157 = _self->_ctail_17157; /* 2711 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(_ctail_17157);}, {}, _ctx)
  kk_std_core__list _x20220 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17157, _ctail_17156, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x20220, _ctx));
}

kk_std_core__list kk_std_core__mlift17516_op(kk_function_t _accm, kk_function_t f0, kk_std_core__list yy0, kk_box_t _ctail_17157, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, f : (value : a, rest : list<a>) -> e b, yy : list<a>, b) -> e list<b> */ 
  return kk_std_core__ctailm_lift17070_map_peek(f0, yy0, kk_std_core__new_mlift17516_op_fun20219(_accm, _ctail_17157, _ctx), _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__ctail_lift17070_map_peek_fun20224__t {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_std_core__list yy1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_lift17070_map_peek_fun20224(kk_function_t _fself, kk_box_t _ctail_171520, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_lift17070_map_peek_fun20224(kk_function_t f1, kk_std_core__list yy1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17070_map_peek_fun20224__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_lift17070_map_peek_fun20224__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_lift17070_map_peek_fun20224, kk_context());
  _self->f1 = f1;
  _self->yy1 = yy1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_lift17070_map_peek_fun20224(kk_function_t _fself, kk_box_t _ctail_171520, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17070_map_peek_fun20224__t* _self = kk_function_as(struct kk_std_core__ctail_lift17070_map_peek_fun20224__t*, _fself);
  kk_function_t f1 = _self->f1; /* (value : 2710, rest : list<2710>) -> 2712 2711 */
  kk_std_core__list yy1 = _self->yy1; /* list<2710> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<2711>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_std_core__list_dup(yy1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17515_op(_acc0, f1, yy1, _ctail_171520, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_lift17070_map_peek_fun20226__t {
  struct kk_function_s _base;
  kk_function_t _b_18335_18313;
};
static kk_box_t kk_std_core__ctail_lift17070_map_peek_fun20226(kk_function_t _fself, kk_box_t _b_18314, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_lift17070_map_peek_fun20226(kk_function_t _b_18335_18313, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17070_map_peek_fun20226__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_lift17070_map_peek_fun20226__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_lift17070_map_peek_fun20226, kk_context());
  _self->_b_18335_18313 = _b_18335_18313;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_lift17070_map_peek_fun20226(kk_function_t _fself, kk_box_t _b_18314, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17070_map_peek_fun20226__t* _self = kk_function_as(struct kk_std_core__ctail_lift17070_map_peek_fun20226__t*, _fself);
  kk_function_t _b_18335_18313 = _self->_b_18335_18313; /* (2711) -> 2712 list<2711> */
  kk_drop_match(_self, {kk_function_dup(_b_18335_18313);}, {}, _ctx)
  kk_std_core__list _x20227 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18335_18313, (_b_18335_18313, _b_18314, _ctx)); /*list<2711>*/
  return kk_std_core__list_box(_x20227, _ctx);
}

kk_std_core__list kk_std_core__ctail_lift17070_map_peek(kk_function_t f1, kk_std_core__list ys, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (value : a, rest : list<a>) -> e b, ys : list<a>, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20221 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con20221->head;
    kk_std_core__list yy1 = _con20221->tail;
    kk_reuse_t _ru_20019 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20019 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy1);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_box_t x_17618;
    kk_function_t _x20223 = kk_function_dup(f1); /*(value : 2710, rest : list<2710>) -> 2712 2711*/
    kk_std_core__list _x20222 = kk_std_core__list_dup(yy1); /*list<2710>*/
    x_17618 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_std_core__list, kk_context_t*), _x20223, (_x20223, y, _x20222, _ctx)); /*2711*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20019);
      kk_box_drop(x_17618, _ctx);
      kk_function_t _b_18335_18313 = kk_std_core__new_ctail_lift17070_map_peek_fun20224(f1, yy1, _acc0, _ctx); /*(2711) -> 2712 list<2711>*/;
      kk_box_t _x20225 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_lift17070_map_peek_fun20226(_b_18335_18313, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20225, _ctx);
    }
    kk_std_core__list _ctail_171530 = kk_std_core__list_hole(); /*list<2711>*/;
    kk_std_core__list _ctail_171540 = kk_std_core__new_Cons(_ru_20019, x_17618, _ctail_171530, _ctx); /*list<2711>*/;
    { // tailcall
      kk_std_core_types__ctail _x20228;
      kk_box_t* field0_17623 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_171540)->tail)); /*cfield<list<2711>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x18321 = _acc0._cons.CTail._field1;
        kk_box_t* last2 = _acc0._cons.CTail._field2;
        kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18321, NULL);
        kk_std_core__list_dup(head1);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __0 = kk_Unit;
        *(last2) = kk_std_core__list_box(_ctail_171540, _ctx);
        kk_std_core__list _b_18336_18326 = head1; /*list<2711>*/;
        kk_box_t* _b_18337_18327 = field0_17623; /*cfield<list<2711>>*/;
        _x20228 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18336_18326, _ctx), _b_18337_18327, _ctx); /*ctail<58>*/
        goto _match20229;
      }
      kk_std_core__list _b_18338_18328 = _ctail_171540; /*list<2711>*/;
      kk_box_t* _b_18339_18329 = field0_17623; /*cfield<list<2711>>*/;
      _x20228 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18338_18328, _ctx), _b_18339_18329, _ctx); /*ctail<58>*/
      _match20229: ;
      ys = yy1;
      _acc0 = _x20228;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18330 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x18330, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__ctailm_lift17070_map_peek_fun20235__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_std_core__list yy2;
};
static kk_std_core__list kk_std_core__ctailm_lift17070_map_peek_fun20235(kk_function_t _fself, kk_box_t _ctail_171570, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17070_map_peek_fun20235(kk_function_t _accm0, kk_function_t f2, kk_std_core__list yy2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17070_map_peek_fun20235__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17070_map_peek_fun20235__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17070_map_peek_fun20235, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->yy2 = yy2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_lift17070_map_peek_fun20235(kk_function_t _fself, kk_box_t _ctail_171570, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17070_map_peek_fun20235__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17070_map_peek_fun20235__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<2711>) -> list<2711> */
  kk_function_t f2 = _self->f2; /* (value : 2710, rest : list<2710>) -> 2712 2711 */
  kk_std_core__list yy2 = _self->yy2; /* list<2710> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_std_core__list_dup(yy2);}, {}, _ctx)
  return kk_std_core__mlift17516_op(_accm0, f2, yy2, _ctail_171570, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_lift17070_map_peek_fun20237__t {
  struct kk_function_s _base;
  kk_function_t _b_18342_18340;
};
static kk_box_t kk_std_core__ctailm_lift17070_map_peek_fun20237(kk_function_t _fself, kk_box_t _b_18341, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17070_map_peek_fun20237(kk_function_t _b_18342_18340, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17070_map_peek_fun20237__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17070_map_peek_fun20237__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17070_map_peek_fun20237, kk_context());
  _self->_b_18342_18340 = _b_18342_18340;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_lift17070_map_peek_fun20237(kk_function_t _fself, kk_box_t _b_18341, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17070_map_peek_fun20237__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17070_map_peek_fun20237__t*, _fself);
  kk_function_t _b_18342_18340 = _self->_b_18342_18340; /* (2711) -> 2712 list<2711> */
  kk_drop_match(_self, {kk_function_dup(_b_18342_18340);}, {}, _ctx)
  kk_std_core__list _x20238 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18342_18340, (_b_18342_18340, _b_18341, _ctx)); /*list<2711>*/
  return kk_std_core__list_box(_x20238, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_lift17070_map_peek_fun20240__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x0_17626;
};
static kk_std_core__list kk_std_core__ctailm_lift17070_map_peek_fun20240(kk_function_t _fself, kk_std_core__list _ctail_171560, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17070_map_peek_fun20240(kk_function_t _accm0, kk_box_t x0_17626, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17070_map_peek_fun20240__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17070_map_peek_fun20240__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17070_map_peek_fun20240, kk_context());
  _self->_accm0 = _accm0;
  _self->x0_17626 = x0_17626;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_lift17070_map_peek_fun20240(kk_function_t _fself, kk_std_core__list _ctail_171560, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17070_map_peek_fun20240__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17070_map_peek_fun20240__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<2711>) -> list<2711> */
  kk_box_t x0_17626 = _self->x0_17626; /* 2711 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x0_17626);}, {}, _ctx)
  kk_std_core__list _x20241 = kk_std_core__new_Cons(kk_reuse_null, x0_17626, _ctail_171560, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x20241, _ctx));
}

kk_std_core__list kk_std_core__ctailm_lift17070_map_peek(kk_function_t f2, kk_std_core__list ys0, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (value : a, rest : list<a>) -> e b, ys : list<a>, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys0)) {
    struct kk_std_core_Cons* _con20232 = kk_std_core__as_Cons(ys0);
    kk_box_t y0 = _con20232->head;
    kk_std_core__list yy2 = _con20232->tail;
    if (kk_likely(kk_std_core__list_is_unique(ys0))) {
      kk_std_core__list_free(ys0);
    }
    else {
      kk_box_dup(y0);
      kk_std_core__list_dup(yy2);
      kk_Unit;
      kk_std_core__list_decref(ys0, _ctx);
      kk_Unit;
    }
    kk_box_t x0_17626;
    kk_function_t _x20234 = kk_function_dup(f2); /*(value : 2710, rest : list<2710>) -> 2712 2711*/
    kk_std_core__list _x20233 = kk_std_core__list_dup(yy2); /*list<2710>*/
    x0_17626 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_std_core__list, kk_context_t*), _x20234, (_x20234, y0, _x20233, _ctx)); /*2711*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x0_17626, _ctx);
      kk_function_t _b_18342_18340 = kk_std_core__new_ctailm_lift17070_map_peek_fun20235(_accm0, f2, yy2, _ctx); /*(2711) -> 2712 list<2711>*/;
      kk_box_t _x20236 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_lift17070_map_peek_fun20237(_b_18342_18340, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20236, _ctx);
    }
    { // tailcall
      kk_function_t _x20239 = kk_std_core__new_ctailm_lift17070_map_peek_fun20240(_accm0, x0_17626, _ctx); /*(list<2711>) -> list<2711>*/
      ys0 = yy2;
      _accm0 = _x20239;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17070_map_peek_fun20242__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core__lift17070_map_peek_fun20242(kk_function_t _fself, kk_std_core__list _ctail_17155, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17070_map_peek_fun20242(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core__lift17070_map_peek_fun20242, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core__lift17070_map_peek_fun20242(kk_function_t _fself, kk_std_core__list _ctail_17155, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17155;
}

kk_std_core__list kk_std_core__lift17070_map_peek(kk_function_t f3, kk_std_core__list ys1, kk_context_t* _ctx) { /* forall<a,b,e> (f : (value : a, rest : list<a>) -> e b, ys : list<a>) -> e list<b> */ 
  bool _match_19989 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19989) {
    return kk_std_core__ctail_lift17070_map_peek(f3, ys1, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_lift17070_map_peek(f3, ys1, kk_std_core__new_lift17070_map_peek_fun20242(_ctx), _ctx);
}

kk_integer_t kk_std_core_maxListStack;

kk_ref_t kk_std_core_trace_enabled;
 
// Compose two funs `f` and `g`.


// lift anonymous function
struct kk_std_core_o_fun20244__t {
  struct kk_function_s _base;
  kk_function_t f;
  kk_function_t g;
};
static kk_box_t kk_std_core_o_fun20244(kk_function_t _fself, kk_box_t x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_o_fun20244(kk_function_t f, kk_function_t g, kk_context_t* _ctx) {
  struct kk_std_core_o_fun20244__t* _self = kk_function_alloc_as(struct kk_std_core_o_fun20244__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_o_fun20244, kk_context());
  _self->f = f;
  _self->g = g;
  return &_self->_base;
}

static kk_box_t kk_std_core_o_fun20244(kk_function_t _fself, kk_box_t x, kk_context_t* _ctx) {
  struct kk_std_core_o_fun20244__t* _self = kk_function_as(struct kk_std_core_o_fun20244__t*, _fself);
  kk_function_t f = _self->f; /* (2754) -> 2751 2752 */
  kk_function_t g = _self->g; /* (2750) -> 2751 2754 */
  kk_drop_match(_self, {kk_function_dup(f);kk_function_dup(g);}, {}, _ctx)
  kk_box_t x0_17629 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), g, (g, x, _ctx)); /*2754*/;
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17629, _ctx);
    return kk_std_core_hnd_yield_extend(f, _ctx);
  }
  return kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x0_17629, _ctx));
}

kk_function_t kk_std_core_o(kk_function_t f, kk_function_t g, kk_context_t* _ctx) { /* forall<a,b,c,e> (f : (a) -> e b, g : (c) -> e a) -> ((x : c) -> e b) */ 
  return kk_std_core_new_o_fun20244(f, g, _ctx);
}
 
// monadic lift

kk_box_t kk_std_core__mlift17517_once(kk_function_t calc, kk_ref_t r, kk_std_core_types__maybe _y_17252, kk_context_t* _ctx) { /* forall<_h,_e,a> (calc : () -> a, r : ref<_h,maybe<a>>, maybe<a>) -> <read<_h>,write<_h>,div|_e> a */ 
  if (kk_std_core_types__is_Just(_y_17252)) {
    kk_box_t x = _y_17252._cons.Just.value;
    kk_function_drop(calc, _ctx);
    kk_ref_drop(r, _ctx);
    return x;
  }
  kk_box_t x0 = kk_function_call(kk_box_t, (kk_function_t, kk_context_t*), calc, (calc, _ctx)); /*2875*/;
  kk_unit_t __ = kk_Unit;
  kk_unit_t _borrowApp_19987 = kk_Unit;
  kk_box_t _x20246;
  kk_std_core_types__maybe _x20247;
  kk_box_t _x20248 = kk_box_dup(x0); /*2875*/
  _x20247 = kk_std_core_types__new_Just(_x20248, _ctx); /*maybe<101>*/
  _x20246 = kk_std_core_types__maybe_box(_x20247, _ctx); /*168*/
  kk_ref_set_borrow(r,_x20246,kk_context());
  kk_ref_drop(r, _ctx);
  _borrowApp_19987;
  return x0;
}
 
// Given a total function to calculate a value `:a`, return
// a total function that only calculates the value once and then
// returns the cached result.


// lift anonymous function
struct kk_std_core_once_fun20249__t {
  struct kk_function_s _base;
  kk_function_t calc;
  kk_ref_t r;
};
static kk_box_t kk_std_core_once_fun20249(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_once_fun20249(kk_function_t calc, kk_ref_t r, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20249__t* _self = kk_function_alloc_as(struct kk_std_core_once_fun20249__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_once_fun20249, kk_context());
  _self->calc = calc;
  _self->r = r;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_core_once_fun20252__t {
  struct kk_function_s _base;
  kk_function_t calc;
  kk_ref_t r;
};
static kk_box_t kk_std_core_once_fun20252(kk_function_t _fself, kk_std_core_types__maybe _y_17252, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_once_fun20252(kk_function_t calc, kk_ref_t r, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20252__t* _self = kk_function_alloc_as(struct kk_std_core_once_fun20252__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_once_fun20252, kk_context());
  _self->calc = calc;
  _self->r = r;
  return &_self->_base;
}

static kk_box_t kk_std_core_once_fun20252(kk_function_t _fself, kk_std_core_types__maybe _y_17252, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20252__t* _self = kk_function_as(struct kk_std_core_once_fun20252__t*, _fself);
  kk_function_t calc = _self->calc; /* () -> 2875 */
  kk_ref_t r = _self->r; /* ref<_2784,maybe<2875>> */
  kk_drop_match(_self, {kk_function_dup(calc);kk_ref_dup(r);}, {}, _ctx)
  if (kk_std_core_types__is_Just(_y_17252)) {
    kk_box_t x0 = _y_17252._cons.Just.value;
    kk_function_drop(calc, _ctx);
    kk_ref_drop(r, _ctx);
    return x0;
  }
  kk_box_t x00 = kk_function_call(kk_box_t, (kk_function_t, kk_context_t*), calc, (calc, _ctx)); /*2875*/;
  kk_unit_t __ = kk_Unit;
  kk_unit_t _borrowApp_19986 = kk_Unit;
  kk_box_t _x20253;
  kk_std_core_types__maybe _x20254;
  kk_box_t _x20255 = kk_box_dup(x00); /*2875*/
  _x20254 = kk_std_core_types__new_Just(_x20255, _ctx); /*maybe<101>*/
  _x20253 = kk_std_core_types__maybe_box(_x20254, _ctx); /*168*/
  kk_ref_set_borrow(r,_x20253,kk_context());
  kk_ref_drop(r, _ctx);
  _borrowApp_19986;
  return x00;
}


// lift anonymous function
struct kk_std_core_once_fun20256__t {
  struct kk_function_s _base;
  kk_function_t _b_18360_18358;
};
static kk_box_t kk_std_core_once_fun20256(kk_function_t _fself, kk_box_t _b_18359, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_once_fun20256(kk_function_t _b_18360_18358, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20256__t* _self = kk_function_alloc_as(struct kk_std_core_once_fun20256__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_once_fun20256, kk_context());
  _self->_b_18360_18358 = _b_18360_18358;
  return &_self->_base;
}

static kk_box_t kk_std_core_once_fun20256(kk_function_t _fself, kk_box_t _b_18359, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20256__t* _self = kk_function_as(struct kk_std_core_once_fun20256__t*, _fself);
  kk_function_t _b_18360_18358 = _self->_b_18360_18358; /* (maybe<2875>) -> <read<_2784>,write<_2784>,div|_2867> 2875 */
  kk_drop_match(_self, {kk_function_dup(_b_18360_18358);}, {}, _ctx)
  kk_std_core_types__maybe _x20257 = kk_std_core_types__maybe_unbox(_b_18359, _ctx); /*maybe<2875>*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_18360_18358, (_b_18360_18358, _x20257, _ctx));
}
static kk_box_t kk_std_core_once_fun20249(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20249__t* _self = kk_function_as(struct kk_std_core_once_fun20249__t*, _fself);
  kk_function_t calc = _self->calc; /* () -> 2875 */
  kk_ref_t r = _self->r; /* ref<_2784,maybe<2875>> */
  kk_drop_match(_self, {kk_function_dup(calc);kk_ref_dup(r);}, {}, _ctx)
  kk_std_core_types__maybe x_17633;
  kk_box_t _x20250;
  kk_ref_t _x20251 = kk_ref_dup(r); /*ref<_2784,maybe<2875>>*/
  _x20250 = kk_ref_get(_x20251,kk_context()); /*182*/
  x_17633 = kk_std_core_types__maybe_unbox(_x20250, _ctx); /*maybe<2875>*/
  kk_function_t next0_17634 = kk_std_core_new_once_fun20252(calc, r, _ctx); /*(maybe<2875>) -> <read<_2784>,write<_2784>,div|_2867> 2875*/;
  if (kk_yielding(kk_context())) {
    kk_std_core_types__maybe_drop(x_17633, _ctx);
    kk_function_t _b_18360_18358 = next0_17634; /*(maybe<2875>) -> <read<_2784>,write<_2784>,div|_2867> 2875*/;
    return kk_std_core_hnd_yield_extend(kk_std_core_new_once_fun20256(_b_18360_18358, _ctx), _ctx);
  }
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), next0_17634, (next0_17634, x_17633, _ctx));
}

kk_function_t kk_std_core_once(kk_function_t calc, kk_context_t* _ctx) { /* forall<a> (calc : () -> a) -> (() -> a) */ 
  kk_ref_t r = kk_ref_alloc((kk_std_core_types__maybe_box(kk_std_core_types__new_Nothing(_ctx), _ctx)),kk_context()); /*ref<_2784,maybe<2875>>*/;
  return kk_std_core_new_once_fun20249(calc, r, _ctx);
}

kk_std_core_types__maybe kk_std_core_xparse_int(kk_string_t s, bool hex, kk_context_t* _ctx) { /* (s : string, hex : bool) -> maybe<int> */ 
  return kk_integer_xparse(s,hex,kk_context());
}

kk_box_t kk_std_core_phantom(kk_context_t* _ctx) { /* forall<a> () -> a */ 
  return kk_box_null;
}
 
// Convert a string to lower-case

kk_string_t kk_std_core_to_lower(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  return kk_string_to_lower(s,kk_context());
}

kk_unit_t kk_std_core_xtrace(kk_string_t message0, kk_context_t* _ctx) { /* (message : string) -> () */ 
  kk_trace(message0,kk_context()); return kk_Unit;
}

kk_unit_t kk_std_core_xtrace_any(kk_string_t message0, kk_box_t x, kk_context_t* _ctx) { /* forall<a> (message : string, x : a) -> () */ 
  kk_trace_any(message0,x,kk_context()); return kk_Unit;
}

kk_ref_t kk_std_core_unique_count;

kk_vector_t kk_std_core_unvlist(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> vector<a> */ 
  return kk_list_to_vector(xs,kk_context());
}
 
// Create a new vector of length `n`  with initial elements given by function `f` .

kk_vector_t kk_std_core_vector_initz(kk_ssize_t n, kk_function_t f, kk_context_t* _ctx) { /* forall<a> (n : ssize_t, f : (ssize_t) -> a) -> vector<a> */ 
  return kk_vector_init(n,f,kk_context());
}

bool kk_std_core__lp__excl__eq__4_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_19981;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_19981 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20260;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_19981 = kk_integer_from_small(0); /*int*/
    goto _match20260;
  }
  _borrowedArg_19981 = kk_integer_from_small(1); /*int*/
  _match20260: ;
  kk_integer_t _borrowedArg_19980;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_19980 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20261;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_19980 = kk_integer_from_small(0); /*int*/
    goto _match20261;
  }
  _borrowedArg_19980 = kk_integer_from_small(1); /*int*/
  _match20261: ;
  bool _borrowApp_19982 = kk_integer_neq_borrow(_borrowedArg_19981,_borrowedArg_19980,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19981, _ctx);
  kk_integer_drop(_borrowedArg_19980, _ctx);
  return _borrowApp_19982;
}
 
// Append two lists.

kk_std_core__list kk_std_core__ctail_append(kk_std_core__list xs, kk_std_core__list ys, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, ys : list<a>, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20264 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con20264->head;
    kk_std_core__list xx = _con20264->tail;
    kk_reuse_t _ru_20021 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20021 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_std_core__list _ctail_17158 = kk_std_core__list_hole(); /*list<3245>*/;
    kk_std_core__list _ctail_17159;
    if (kk_likely(_ru_20021!=NULL)) {
      struct kk_std_core_Cons* _con20265 = (struct kk_std_core_Cons*)_ru_20021;
      _con20265->tail = _ctail_17158;
      _ctail_17159 = kk_std_core__base_Cons(_con20265); /*list<3245>*/
    }
    else {
      _ctail_17159 = kk_std_core__new_Cons(kk_reuse_null, x, _ctail_17158, _ctx); /*list<3245>*/
    }
    { // tailcall
      kk_std_core_types__ctail _x20266;
      kk_box_t* field_17642 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17159)->tail)); /*cfield<list<3245>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18369 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18369, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17159, _ctx);
        kk_std_core__list _b_18383_18374 = head0; /*list<3245>*/;
        kk_box_t* _b_18384_18375 = field_17642; /*cfield<list<3245>>*/;
        _x20266 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18383_18374, _ctx), _b_18384_18375, _ctx); /*ctail<58>*/
        goto _match20267;
      }
      kk_std_core__list _b_18385_18376 = _ctail_17159; /*list<3245>*/;
      kk_box_t* _b_18386_18377 = field_17642; /*cfield<list<3245>>*/;
      _x20266 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18385_18376, _ctx), _b_18386_18377, _ctx); /*ctail<58>*/
      _match20267: ;
      xs = xx;
      _acc = _x20266;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18378 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18378, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(ys, _ctx);
    return head1;
  }
  return ys;
}
 
// Append two lists.

kk_std_core__list kk_std_core_append(kk_std_core__list xs0, kk_std_core__list ys0, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, ys : list<a>) -> list<a> */ 
  return kk_std_core__ctail_append(xs0, ys0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Append two strings.

kk_string_t kk_std_core__lp__plus__plus__1_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> string */ 
  return kk_string_cat(x,y,kk_context());
}

kk_unit_t kk_std_core_printsln(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> console () */ 
  kk_std_core_types__maybe _match_19977;
  kk_ref_t _b_18395_18387 = kk_ref_dup(kk_std_core_redirect); /*ref<global,maybe<(string) -> console ()>>*/;
  kk_box_t _x20270 = kk_ref_get(_b_18395_18387,kk_context()); /*182*/
  _match_19977 = kk_std_core_types__maybe_unbox(_x20270, _ctx); /*maybe<(string) -> console ()>*/
  if (kk_std_core_types__is_Nothing(_match_19977)) {
    kk_std_core_xprintsln(s, _ctx); return kk_Unit;
  }
  kk_box_t _fun_unbox_x18391 = _match_19977._cons.Just.value;
  kk_string_t _b_18394;
  kk_string_t _x20271;
  kk_define_string_literal(, _s20272, 1, "\n")
  _x20271 = kk_string_dup(_s20272); /*string*/
  _b_18394 = kk_std_core__lp__plus__plus__1_rp_(s, _x20271, _ctx); /*string*/
  kk_box_t _x20273;
  kk_function_t _x20274 = kk_function_unbox(_fun_unbox_x18391); /*(18392) -> console 18393*/
  _x20273 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20274, (_x20274, kk_string_box(_b_18394), _ctx)); /*18393*/
  kk_unit_unbox(_x20273); return kk_Unit;
}

bool kk_std_core__lp__eq__eq__4_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_19975;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_19975 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20275;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_19975 = kk_integer_from_small(0); /*int*/
    goto _match20275;
  }
  _borrowedArg_19975 = kk_integer_from_small(1); /*int*/
  _match20275: ;
  kk_integer_t _borrowedArg_19974;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_19974 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20276;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_19974 = kk_integer_from_small(0); /*int*/
    goto _match20276;
  }
  _borrowedArg_19974 = kk_integer_from_small(1); /*int*/
  _match20276: ;
  bool _borrowApp_19976 = kk_integer_eq_borrow(_borrowedArg_19975,_borrowedArg_19974,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19975, _ctx);
  kk_integer_drop(_borrowedArg_19974, _ctx);
  return _borrowApp_19976;
}

bool kk_std_core__lp__lt__5_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_19972;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_19972 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20277;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_19972 = kk_integer_from_small(0); /*int*/
    goto _match20277;
  }
  _borrowedArg_19972 = kk_integer_from_small(1); /*int*/
  _match20277: ;
  kk_integer_t _borrowedArg_19971;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_19971 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20278;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_19971 = kk_integer_from_small(0); /*int*/
    goto _match20278;
  }
  _borrowedArg_19971 = kk_integer_from_small(1); /*int*/
  _match20278: ;
  bool _borrowApp_19973 = kk_integer_lt_borrow(_borrowedArg_19972,_borrowedArg_19971,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19972, _ctx);
  kk_integer_drop(_borrowedArg_19971, _ctx);
  return _borrowApp_19973;
}

kk_std_core_types__order kk_std_core_order(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> order */ 
  bool _match_19965;
  kk_integer_t _borrowedArg_19969 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19970 = kk_integer_lt_borrow(i,_borrowedArg_19969,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19969, _ctx);
  _match_19965 = _borrowApp_19970; /*bool*/
  if (_match_19965) {
    kk_integer_drop(i, _ctx);
    return kk_std_core_types__new_Lt(_ctx);
  }
  bool _match_19966;
  kk_integer_t _borrowedArg_19967 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19968 = kk_integer_gt_borrow(i,_borrowedArg_19967,kk_context()); /*bool*/;
  kk_integer_drop(i, _ctx);
  kk_integer_drop(_borrowedArg_19967, _ctx);
  _match_19966 = _borrowApp_19968; /*bool*/
  if (_match_19966) {
    return kk_std_core_types__new_Gt(_ctx);
  }
  return kk_std_core_types__new_Eq(_ctx);
}
 
// Compare two strings.
// Uses the character codes directly for comparison

kk_std_core_types__order kk_std_core_compare_4(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> order */ 
  kk_integer_t i_16932 = kk_std_core_string_compare(x, y, _ctx); /*int*/;
  bool _match_19959;
  kk_integer_t _borrowedArg_19963 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19964 = kk_integer_lt_borrow(i_16932,_borrowedArg_19963,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19963, _ctx);
  _match_19959 = _borrowApp_19964; /*bool*/
  if (_match_19959) {
    kk_integer_drop(i_16932, _ctx);
    return kk_std_core_types__new_Lt(_ctx);
  }
  bool _match_19960;
  kk_integer_t _borrowedArg_19961 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19962 = kk_integer_gt_borrow(i_16932,_borrowedArg_19961,kk_context()); /*bool*/;
  kk_integer_drop(i_16932, _ctx);
  kk_integer_drop(_borrowedArg_19961, _ctx);
  _match_19960 = _borrowApp_19962; /*bool*/
  if (_match_19960) {
    return kk_std_core_types__new_Gt(_ctx);
  }
  return kk_std_core_types__new_Eq(_ctx);
}

bool kk_std_core__lp__lt__7_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_17095 = kk_std_core_compare_4(x, y, _ctx); /*order*/;
  kk_integer_t _borrowedArg_19957;
  if (kk_std_core_types__is_Lt(x0_17095)) {
    _borrowedArg_19957 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20279;
  }
  if (kk_std_core_types__is_Eq(x0_17095)) {
    _borrowedArg_19957 = kk_integer_from_small(0); /*int*/
    goto _match20279;
  }
  _borrowedArg_19957 = kk_integer_from_small(1); /*int*/
  _match20279: ;
  kk_integer_t _borrowedArg_19956 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/;
  bool _borrowApp_19958 = kk_integer_eq_borrow(_borrowedArg_19957,_borrowedArg_19956,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19957, _ctx);
  kk_integer_drop(_borrowedArg_19956, _ctx);
  return _borrowApp_19958;
}

bool kk_std_core__lp__gt__3_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_19954;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_19954 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20280;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_19954 = kk_integer_from_small(0); /*int*/
    goto _match20280;
  }
  _borrowedArg_19954 = kk_integer_from_small(1); /*int*/
  _match20280: ;
  kk_integer_t _borrowedArg_19953;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_19953 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20281;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_19953 = kk_integer_from_small(0); /*int*/
    goto _match20281;
  }
  _borrowedArg_19953 = kk_integer_from_small(1); /*int*/
  _match20281: ;
  bool _borrowApp_19955 = kk_integer_gt_borrow(_borrowedArg_19954,_borrowedArg_19953,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19954, _ctx);
  kk_integer_drop(_borrowedArg_19953, _ctx);
  return _borrowApp_19955;
}

bool kk_std_core__lp__gt__5_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_17097 = kk_std_core_compare_4(x, y, _ctx); /*order*/;
  kk_integer_t _borrowedArg_19951;
  if (kk_std_core_types__is_Lt(x0_17097)) {
    _borrowedArg_19951 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20282;
  }
  if (kk_std_core_types__is_Eq(x0_17097)) {
    _borrowedArg_19951 = kk_integer_from_small(0); /*int*/
    goto _match20282;
  }
  _borrowedArg_19951 = kk_integer_from_small(1); /*int*/
  _match20282: ;
  kk_integer_t _borrowedArg_19950 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19952 = kk_integer_eq_borrow(_borrowedArg_19951,_borrowedArg_19950,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19951, _ctx);
  kk_integer_drop(_borrowedArg_19950, _ctx);
  return _borrowApp_19952;
}

kk_std_core_types__order kk_std_core_compare_2(bool x, bool y, kk_context_t* _ctx) { /* (x : bool, y : bool) -> order */ 
  if (x) {
    if (x) {
      if (y) {
        return kk_std_core_types__new_Eq(_ctx);
      }
      return kk_std_core_types__new_Gt(_ctx);
    }
    return kk_std_core_types__new_Eq(_ctx);
  }
  if (y) {
    return kk_std_core_types__new_Lt(_ctx);
  }
  if (x) {
    if (y) {
      return kk_std_core_types__new_Eq(_ctx);
    }
    return kk_std_core_types__new_Gt(_ctx);
  }
  return kk_std_core_types__new_Eq(_ctx);
}

bool kk_std_core__lp__gt__eq__4_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_19944;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_19944 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20283;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_19944 = kk_integer_from_small(0); /*int*/
    goto _match20283;
  }
  _borrowedArg_19944 = kk_integer_from_small(1); /*int*/
  _match20283: ;
  kk_integer_t _borrowedArg_19943;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_19943 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20284;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_19943 = kk_integer_from_small(0); /*int*/
    goto _match20284;
  }
  _borrowedArg_19943 = kk_integer_from_small(1); /*int*/
  _match20284: ;
  bool _borrowApp_19945 = kk_integer_gte_borrow(_borrowedArg_19944,_borrowedArg_19943,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19944, _ctx);
  kk_integer_drop(_borrowedArg_19943, _ctx);
  return _borrowApp_19945;
}

bool kk_std_core__lp__gt__eq__6_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_17099 = kk_std_core_compare_4(x, y, _ctx); /*order*/;
  kk_integer_t _borrowedArg_19941;
  if (kk_std_core_types__is_Lt(x0_17099)) {
    _borrowedArg_19941 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20285;
  }
  if (kk_std_core_types__is_Eq(x0_17099)) {
    _borrowedArg_19941 = kk_integer_from_small(0); /*int*/
    goto _match20285;
  }
  _borrowedArg_19941 = kk_integer_from_small(1); /*int*/
  _match20285: ;
  kk_integer_t _borrowedArg_19940 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/;
  bool _borrowApp_19942 = kk_integer_gt_borrow(_borrowedArg_19941,_borrowedArg_19940,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19941, _ctx);
  kk_integer_drop(_borrowedArg_19940, _ctx);
  return _borrowApp_19942;
}
 
// lifted

kk_string_t kk_std_core__lift17071_joinsep(kk_string_t sep, kk_std_core__list ys, kk_string_t acc, kk_context_t* _ctx) { /* (sep : string, ys : list<string>, acc : string) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20286 = kk_std_core__as_Cons(ys);
    kk_box_t _box_x18396 = _con20286->head;
    kk_std_core__list yy = _con20286->tail;
    kk_string_t y = kk_string_unbox(_box_x18396);
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_std_core__list_free(ys);
    }
    else {
      kk_string_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_string_t acc0_17102;
    kk_string_t _x20288;
    kk_string_t _x20289 = kk_string_dup(sep); /*string*/
    _x20288 = kk_std_core__lp__plus__plus__1_rp_(_x20289, y, _ctx); /*string*/
    acc0_17102 = kk_std_core__lp__plus__plus__1_rp_(acc, _x20288, _ctx); /*string*/
    { // tailcall
      ys = yy;
      acc = acc0_17102;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sep, _ctx);
  return acc;
}
 
// Concatenate all strings in a list

kk_string_t kk_std_core_joinsep(kk_std_core__list xs, kk_string_t sep, kk_context_t* _ctx) { /* (xs : list<string>, sep : string) -> string */ 
  if (kk_std_core__is_Nil(xs)) {
    kk_string_drop(sep, _ctx);
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con20291 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x18397 = _con20291->head;
  kk_std_core__list xx = _con20291->tail;
  kk_string_t x = kk_string_unbox(_box_x18397);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_string_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  return kk_std_core__lift17071_joinsep(sep, xx, x, _ctx);
}
 
// lifted

kk_string_t kk_std_core__lift17072_join_2(kk_string_t sep_16949, kk_std_core__list ys, kk_string_t acc, kk_context_t* _ctx) { /* (sep.16949 : string, ys : list<string>, acc : string) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20293 = kk_std_core__as_Cons(ys);
    kk_box_t _box_x18398 = _con20293->head;
    kk_std_core__list yy = _con20293->tail;
    kk_string_t y = kk_string_unbox(_box_x18398);
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_std_core__list_free(ys);
    }
    else {
      kk_string_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_string_t acc0_17106;
    kk_string_t _x20295;
    kk_string_t _x20296 = kk_string_dup(sep_16949); /*string*/
    _x20295 = kk_std_core__lp__plus__plus__1_rp_(_x20296, y, _ctx); /*string*/
    acc0_17106 = kk_std_core__lp__plus__plus__1_rp_(acc, _x20295, _ctx); /*string*/
    { // tailcall
      ys = yy;
      acc = acc0_17106;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sep_16949, _ctx);
  return acc;
}
 
// Concatenate all strings in a list

kk_string_t kk_std_core_join_2(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<string>) -> string */ 
  if (kk_std_core__is_Nil(xs)) {
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con20298 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x18399 = _con20298->head;
  kk_std_core__list xx = _con20298->tail;
  kk_string_t x = kk_string_unbox(_box_x18399);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_string_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_string_t _x20300 = kk_string_empty(); /*string*/
  return kk_std_core__lift17072_join_2(_x20300, xx, x, _ctx);
}
 
// Concatenate all strings in a list using a specific separator

kk_string_t kk_std_core_join_3(kk_std_core__list xs, kk_string_t sep, kk_context_t* _ctx) { /* (xs : list<string>, sep : string) -> string */ 
  if (kk_std_core__is_Nil(xs)) {
    kk_string_drop(sep, _ctx);
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con20303 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x18400 = _con20303->head;
  kk_std_core__list xx = _con20303->tail;
  kk_string_t x = kk_string_unbox(_box_x18400);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_string_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  return kk_std_core__lift17071_joinsep(sep, xx, x, _ctx);
}

bool kk_std_core__lp__lt__eq__5_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_19938;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_19938 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20305;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_19938 = kk_integer_from_small(0); /*int*/
    goto _match20305;
  }
  _borrowedArg_19938 = kk_integer_from_small(1); /*int*/
  _match20305: ;
  kk_integer_t _borrowedArg_19937;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_19937 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20306;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_19937 = kk_integer_from_small(0); /*int*/
    goto _match20306;
  }
  _borrowedArg_19937 = kk_integer_from_small(1); /*int*/
  _match20306: ;
  bool _borrowApp_19939 = kk_integer_lte_borrow(_borrowedArg_19938,_borrowedArg_19937,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19938, _ctx);
  kk_integer_drop(_borrowedArg_19937, _ctx);
  return _borrowApp_19939;
}

bool kk_std_core__lp__lt__eq__7_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_17109 = kk_std_core_compare_4(x, y, _ctx); /*order*/;
  kk_integer_t _borrowedArg_19935;
  if (kk_std_core_types__is_Lt(x0_17109)) {
    _borrowedArg_19935 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20307;
  }
  if (kk_std_core_types__is_Eq(x0_17109)) {
    _borrowedArg_19935 = kk_integer_from_small(0); /*int*/
    goto _match20307;
  }
  _borrowedArg_19935 = kk_integer_from_small(1); /*int*/
  _match20307: ;
  kk_integer_t _borrowedArg_19934 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19936 = kk_integer_lt_borrow(_borrowedArg_19935,_borrowedArg_19934,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19935, _ctx);
  kk_integer_drop(_borrowedArg_19934, _ctx);
  return _borrowApp_19936;
}

kk_ssize_t kk_std_core_decr_1(kk_ssize_t i, kk_context_t* _ctx) { /* (i : ssize_t) -> ssize_t */ 
  return (i - 1);
}

kk_ssize_t kk_std_core_incr_1(kk_ssize_t i, kk_context_t* _ctx) { /* (i : ssize_t) -> ssize_t */ 
  return (i + 1);
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17518_op(kk_function_t action, kk_ssize_t end, kk_ssize_t i, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action : (ssize_t) -> e (), end : ssize_t, i : ssize_t, wild_ : ()) -> e () */ 
  kk_ssize_t i0_17111 = kk_std_core_incr_1(i, _ctx); /*ssize_t*/;
  kk_std_core__lift17073_forz(action, end, i0_17111, _ctx); return kk_Unit;
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17073_forz_fun20309__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_ssize_t end0;
  kk_ssize_t i0;
};
static kk_unit_t kk_std_core__lift17073_forz_fun20309(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17073_forz_fun20309(kk_function_t action0, kk_ssize_t end0, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17073_forz_fun20309__t* _self = kk_function_alloc_as(struct kk_std_core__lift17073_forz_fun20309__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17073_forz_fun20309, kk_context());
  _self->action0 = action0;
  _self->end0 = end0;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17073_forz_fun20309(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17073_forz_fun20309__t* _self = kk_function_as(struct kk_std_core__lift17073_forz_fun20309__t*, _fself);
  kk_function_t action0 = _self->action0; /* (ssize_t) -> 4353 () */
  kk_ssize_t end0 = _self->end0; /* ssize_t */
  kk_ssize_t i0 = _self->i0; /* ssize_t */
  kk_drop_match(_self, {kk_function_dup(action0);;;}, {}, _ctx)
  return kk_std_core__mlift17518_op(action0, end0, i0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17073_forz_fun20311__t {
  struct kk_function_s _base;
  kk_function_t _b_18403_18401;
};
static kk_box_t kk_std_core__lift17073_forz_fun20311(kk_function_t _fself, kk_box_t _b_18402, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17073_forz_fun20311(kk_function_t _b_18403_18401, kk_context_t* _ctx) {
  struct kk_std_core__lift17073_forz_fun20311__t* _self = kk_function_alloc_as(struct kk_std_core__lift17073_forz_fun20311__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17073_forz_fun20311, kk_context());
  _self->_b_18403_18401 = _b_18403_18401;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17073_forz_fun20311(kk_function_t _fself, kk_box_t _b_18402, kk_context_t* _ctx) {
  struct kk_std_core__lift17073_forz_fun20311__t* _self = kk_function_as(struct kk_std_core__lift17073_forz_fun20311__t*, _fself);
  kk_function_t _b_18403_18401 = _self->_b_18403_18401; /* (wild_0 : ()) -> 4353 () */
  kk_drop_match(_self, {kk_function_dup(_b_18403_18401);}, {}, _ctx)
  kk_unit_t _x20312 = kk_Unit;
  kk_unit_t _x20313 = kk_Unit;
  kk_unit_unbox(_b_18402);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_18403_18401, (_b_18403_18401, _x20313, _ctx));
  return kk_unit_box(_x20312);
}

kk_unit_t kk_std_core__lift17073_forz(kk_function_t action0, kk_ssize_t end0, kk_ssize_t i0, kk_context_t* _ctx) { /* forall<e> (action : (ssize_t) -> e (), end : ssize_t, i : ssize_t) -> e () */ 
  kk__tailcall: ;
  bool _match_19932 = (i0 <= end0); /*bool*/;
  if (_match_19932) {
    kk_unit_t x_17647 = kk_Unit;
    kk_function_t _x20308 = kk_function_dup(action0); /*(ssize_t) -> 4353 ()*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_ssize_t, kk_context_t*), _x20308, (_x20308, i0, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_18403_18401 = kk_std_core__new_lift17073_forz_fun20309(action0, end0, i0, _ctx); /*(wild_0 : ()) -> 4353 ()*/;
      kk_box_t _x20310 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17073_forz_fun20311(_b_18403_18401, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x20310); return kk_Unit;
    }
    kk_ssize_t i0_171110 = kk_std_core_incr_1(i0, _ctx); /*ssize_t*/;
    { // tailcall
      i0 = i0_171110;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action0, _ctx);
  kk_Unit; return kk_Unit;
}


// lift anonymous function
struct kk_std_core_foreach_indexedz_fun20315__t {
  struct kk_function_s _base;
  kk_function_t f;
  kk_vector_t v;
};
static kk_unit_t kk_std_core_foreach_indexedz_fun20315(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_indexedz_fun20315(kk_function_t f, kk_vector_t v, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexedz_fun20315__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_indexedz_fun20315__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_indexedz_fun20315, kk_context());
  _self->f = f;
  _self->v = v;
  return &_self->_base;
}

static kk_unit_t kk_std_core_foreach_indexedz_fun20315(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexedz_fun20315__t* _self = kk_function_as(struct kk_std_core_foreach_indexedz_fun20315__t*, _fself);
  kk_function_t f = _self->f; /* (4387, ssize_t) -> 4388 () */
  kk_vector_t v = _self->v; /* vector<4387> */
  kk_drop_match(_self, {kk_function_dup(f);kk_vector_dup(v);}, {}, _ctx)
  kk_box_t _x20316;
  kk_box_t _borrowApp_19931 = kk_vector_at_borrow(v, i); /*219*/;
  kk_vector_drop(v, _ctx);
  _x20316 = _borrowApp_19931; /*219*/
  return kk_function_call(kk_unit_t, (kk_function_t, kk_box_t, kk_ssize_t, kk_context_t*), f, (f, _x20316, i, _ctx));
}

kk_unit_t kk_std_core_foreach_indexedz(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (v : vector<a>, f : (a, ssize_t) -> e ()) -> e () */ 
  kk_ssize_t start0_17650 = ((kk_ssize_t)0); /*ssize_t*/;
  kk_ssize_t end_17651;
  kk_ssize_t _x20314 = kk_vector_len_borrow(v); /*ssize_t*/
  end_17651 = kk_std_core_decr_1(_x20314, _ctx); /*ssize_t*/
  kk_std_core__lift17073_forz(kk_std_core_new_foreach_indexedz_fun20315(f, v, _ctx), end_17651, start0_17650, _ctx); return kk_Unit;
}
 
// lifted

kk_integer_t kk_std_core__lift17074_length_1(kk_std_core__list ys, kk_integer_t acc, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, acc : int) -> int */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20317 = kk_std_core__as_Cons(ys);
    kk_box_t _pat0 = _con20317->head;
    kk_std_core__list yy = _con20317->tail;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_box_drop(_pat0, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(ys);
    }
    else {
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_integer_t _x20318 = kk_integer_add(acc,(kk_integer_from_small(1)),kk_context()); /*int*/
      ys = yy;
      acc = _x20318;
      goto kk__tailcall;
    }
  }
  return acc;
}
 
// Returns an integer list of increasing elements from `lo`  to `hi`
// (including both `lo`  and `hi` ).
// If `lo > hi`  the function returns the empty list.

kk_std_core__list kk_std_core__ctail_list(kk_integer_t lo, kk_integer_t hi, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (lo : int, hi : int, ctail<list<int>>) -> total list<int> */ 
  kk__tailcall: ;
  bool _match_19930 = kk_integer_lte_borrow(lo,hi,kk_context()); /*bool*/;
  if (_match_19930) {
    kk_std_core__list _ctail_17160 = kk_std_core__list_hole(); /*list<int>*/;
    kk_std_core__list _ctail_17161;
    kk_box_t _x20320;
    kk_integer_t _x20321 = kk_integer_dup(lo); /*int*/
    _x20320 = kk_integer_box(_x20321); /*61*/
    _ctail_17161 = kk_std_core__new_Cons(kk_reuse_null, _x20320, _ctail_17160, _ctx); /*list<int>*/
    { // tailcall
      kk_integer_t _x20322 = kk_integer_add(lo,(kk_integer_from_small(1)),kk_context()); /*int*/
      kk_std_core_types__ctail _x20323;
      kk_box_t* field_17655 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17161)->tail)); /*cfield<list<int>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18414 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18414, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17161, _ctx);
        kk_std_core__list _b_18428_18419 = head0; /*list<int>*/;
        kk_box_t* _b_18429_18420 = field_17655; /*cfield<list<int>>*/;
        _x20323 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18428_18419, _ctx), _b_18429_18420, _ctx); /*ctail<58>*/
        goto _match20324;
      }
      kk_std_core__list _b_18430_18421 = _ctail_17161; /*list<int>*/;
      kk_box_t* _b_18431_18422 = field_17655; /*cfield<list<int>>*/;
      _x20323 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18430_18421, _ctx), _b_18431_18422, _ctx); /*ctail<58>*/
      _match20324: ;
      lo = _x20322;
      _acc = _x20323;
      goto kk__tailcall;
    }
  }
  kk_integer_drop(hi, _ctx);
  kk_integer_drop(lo, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18423 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18423, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Returns an integer list of increasing elements from `lo`  to `hi`
// (including both `lo`  and `hi` ).
// If `lo > hi`  the function returns the empty list.

kk_std_core__list kk_std_core_list(kk_integer_t lo0, kk_integer_t hi0, kk_context_t* _ctx) { /* (lo : int, hi : int) -> total list<int> */ 
  return kk_std_core__ctail_list(lo0, hi0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Returns an integer list of increasing elements from `lo`  to `hi` with stride `stride`.
// If `lo > hi`  the function returns the empty list.

kk_std_core__list kk_std_core__ctail_list_1(kk_integer_t lo, kk_integer_t hi, kk_integer_t stride, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (lo : int, hi : int, stride : int, ctail<list<int>>) -> total list<int> */ 
  kk__tailcall: ;
  bool _match_19929 = kk_integer_lte_borrow(lo,hi,kk_context()); /*bool*/;
  if (_match_19929) {
    kk_std_core__list _ctail_17162 = kk_std_core__list_hole(); /*list<int>*/;
    kk_std_core__list _ctail_17163;
    kk_box_t _x20327;
    kk_integer_t _x20328 = kk_integer_dup(lo); /*int*/
    _x20327 = kk_integer_box(_x20328); /*61*/
    _ctail_17163 = kk_std_core__new_Cons(kk_reuse_null, _x20327, _ctail_17162, _ctx); /*list<int>*/
    { // tailcall
      kk_integer_t _x20329;
      kk_integer_t _x20330 = kk_integer_dup(stride); /*int*/
      _x20329 = kk_integer_add(lo,_x20330,kk_context()); /*int*/
      kk_std_core_types__ctail _x20331;
      kk_box_t* field_17660 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17163)->tail)); /*cfield<list<int>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18442 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18442, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17163, _ctx);
        kk_std_core__list _b_18456_18447 = head0; /*list<int>*/;
        kk_box_t* _b_18457_18448 = field_17660; /*cfield<list<int>>*/;
        _x20331 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18456_18447, _ctx), _b_18457_18448, _ctx); /*ctail<58>*/
        goto _match20332;
      }
      kk_std_core__list _b_18458_18449 = _ctail_17163; /*list<int>*/;
      kk_box_t* _b_18459_18450 = field_17660; /*cfield<list<int>>*/;
      _x20331 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18458_18449, _ctx), _b_18459_18450, _ctx); /*ctail<58>*/
      _match20332: ;
      lo = _x20329;
      _acc = _x20331;
      goto kk__tailcall;
    }
  }
  kk_integer_drop(hi, _ctx);
  kk_integer_drop(lo, _ctx);
  kk_integer_drop(stride, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18451 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18451, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Returns an integer list of increasing elements from `lo`  to `hi` with stride `stride`.
// If `lo > hi`  the function returns the empty list.

kk_std_core__list kk_std_core_list_1(kk_integer_t lo0, kk_integer_t hi0, kk_integer_t stride0, kk_context_t* _ctx) { /* (lo : int, hi : int, stride : int) -> total list<int> */ 
  return kk_std_core__ctail_list_1(lo0, hi0, stride0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17519_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_integer_t hi, kk_integer_t lo, kk_box_t _ctail_17164, kk_context_t* _ctx) { /* forall<a,e> (ctail<list<a>>, f : (int) -> e a, hi : int, lo : int, a) -> e list<a> */ 
  kk_std_core__list _ctail_17165 = kk_std_core__list_hole(); /*list<4938>*/;
  kk_std_core__list _ctail_17166 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17164, _ctail_17165, _ctx); /*list<4938>*/;
  kk_integer_t _x20335 = kk_integer_add(lo,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_std_core_types__ctail _x20336;
  kk_box_t* field_17665 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17166)->tail)); /*cfield<list<4938>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18466 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18466, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17166, _ctx);
    kk_std_core__list _b_18475_18471 = head0; /*list<4938>*/;
    kk_box_t* _b_18476_18472 = field_17665; /*cfield<list<4938>>*/;
    _x20336 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18475_18471, _ctx), _b_18476_18472, _ctx); /*ctail<58>*/
    goto _match20337;
  }
  kk_std_core__list _b_18477_18473 = _ctail_17166; /*list<4938>*/;
  kk_box_t* _b_18478_18474 = field_17665; /*cfield<list<4938>>*/;
  _x20336 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18477_18473, _ctx), _b_18478_18474, _ctx); /*ctail<58>*/
  _match20337: ;
  return kk_std_core__ctail_list_2(_x20335, hi, f, _x20336, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17520_op_fun20340__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t _ctail_17169;
};
static kk_std_core__list kk_std_core__mlift17520_op_fun20340(kk_function_t _fself, kk_std_core__list _ctail_17168, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17520_op_fun20340(kk_function_t _accm, kk_box_t _ctail_17169, kk_context_t* _ctx) {
  struct kk_std_core__mlift17520_op_fun20340__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17520_op_fun20340__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17520_op_fun20340, kk_context());
  _self->_accm = _accm;
  _self->_ctail_17169 = _ctail_17169;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17520_op_fun20340(kk_function_t _fself, kk_std_core__list _ctail_17168, kk_context_t* _ctx) {
  struct kk_std_core__mlift17520_op_fun20340__t* _self = kk_function_as(struct kk_std_core__mlift17520_op_fun20340__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<4938>) -> list<4938> */
  kk_box_t _ctail_17169 = _self->_ctail_17169; /* 4938 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(_ctail_17169);}, {}, _ctx)
  kk_std_core__list _x20341 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17169, _ctail_17168, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x20341, _ctx));
}

kk_std_core__list kk_std_core__mlift17520_op(kk_function_t _accm, kk_function_t f0, kk_integer_t hi0, kk_integer_t lo0, kk_box_t _ctail_17169, kk_context_t* _ctx) { /* forall<a,e> ((list<a>) -> list<a>, f : (int) -> e a, hi : int, lo : int, a) -> e list<a> */ 
  kk_integer_t _x20339 = kk_integer_add(lo0,(kk_integer_from_small(1)),kk_context()); /*int*/
  return kk_std_core__ctailm_list_2(_x20339, hi0, f0, kk_std_core__new_mlift17520_op_fun20340(_accm, _ctail_17169, _ctx), _ctx);
}
 
// Applies a function `f` to list of increasing elements from `lo`  to `hi`
// (including both `lo`  and `hi` ).
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core__ctail_list_fun20344__t_2 {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_integer_t hi1;
  kk_integer_t lo1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_list_fun20344_2(kk_function_t _fself, kk_box_t _ctail_171640, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_list_fun20344_2(kk_function_t f1, kk_integer_t hi1, kk_integer_t lo1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20344__t_2* _self = kk_function_alloc_as(struct kk_std_core__ctail_list_fun20344__t_2, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_list_fun20344_2, kk_context());
  _self->f1 = f1;
  _self->hi1 = hi1;
  _self->lo1 = lo1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_list_fun20344_2(kk_function_t _fself, kk_box_t _ctail_171640, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20344__t_2* _self = kk_function_as(struct kk_std_core__ctail_list_fun20344__t_2*, _fself);
  kk_function_t f1 = _self->f1; /* (int) -> 4939 4938 */
  kk_integer_t hi1 = _self->hi1; /* int */
  kk_integer_t lo1 = _self->lo1; /* int */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<4938>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_integer_dup(hi1);kk_integer_dup(lo1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17519_op(_acc0, f1, hi1, lo1, _ctail_171640, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_list_fun20346__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18501_18479;
};
static kk_box_t kk_std_core__ctail_list_fun20346_2(kk_function_t _fself, kk_box_t _b_18480, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_list_fun20346_2(kk_function_t _b_18501_18479, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20346__t_2* _self = kk_function_alloc_as(struct kk_std_core__ctail_list_fun20346__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_list_fun20346_2, kk_context());
  _self->_b_18501_18479 = _b_18501_18479;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_list_fun20346_2(kk_function_t _fself, kk_box_t _b_18480, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20346__t_2* _self = kk_function_as(struct kk_std_core__ctail_list_fun20346__t_2*, _fself);
  kk_function_t _b_18501_18479 = _self->_b_18501_18479; /* (4938) -> 4939 list<4938> */
  kk_drop_match(_self, {kk_function_dup(_b_18501_18479);}, {}, _ctx)
  kk_std_core__list _x20347 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18501_18479, (_b_18501_18479, _b_18480, _ctx)); /*list<4938>*/
  return kk_std_core__list_box(_x20347, _ctx);
}

kk_std_core__list kk_std_core__ctail_list_2(kk_integer_t lo1, kk_integer_t hi1, kk_function_t f1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, f : (int) -> e a, ctail<list<a>>) -> e list<a> */ 
  kk__tailcall: ;
  bool _match_19927 = kk_integer_lte_borrow(lo1,hi1,kk_context()); /*bool*/;
  if (_match_19927) {
    kk_box_t x_17666;
    kk_function_t _x20343 = kk_function_dup(f1); /*(int) -> 4939 4938*/
    kk_integer_t _x20342 = kk_integer_dup(lo1); /*int*/
    x_17666 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_context_t*), _x20343, (_x20343, _x20342, _ctx)); /*4938*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x_17666, _ctx);
      kk_function_t _b_18501_18479 = kk_std_core__new_ctail_list_fun20344_2(f1, hi1, lo1, _acc0, _ctx); /*(4938) -> 4939 list<4938>*/;
      kk_box_t _x20345 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_list_fun20346_2(_b_18501_18479, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20345, _ctx);
    }
    kk_std_core__list _ctail_171650 = kk_std_core__list_hole(); /*list<4938>*/;
    kk_std_core__list _ctail_171660 = kk_std_core__new_Cons(kk_reuse_null, x_17666, _ctail_171650, _ctx); /*list<4938>*/;
    { // tailcall
      kk_integer_t _x20348 = kk_integer_add(lo1,(kk_integer_from_small(1)),kk_context()); /*int*/
      kk_std_core_types__ctail _x20349;
      kk_box_t* field0_17671 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_171660)->tail)); /*cfield<list<4938>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x18487 = _acc0._cons.CTail._field1;
        kk_box_t* last2 = _acc0._cons.CTail._field2;
        kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18487, NULL);
        kk_std_core__list_dup(head1);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __0 = kk_Unit;
        *(last2) = kk_std_core__list_box(_ctail_171660, _ctx);
        kk_std_core__list _b_18502_18492 = head1; /*list<4938>*/;
        kk_box_t* _b_18503_18493 = field0_17671; /*cfield<list<4938>>*/;
        _x20349 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18502_18492, _ctx), _b_18503_18493, _ctx); /*ctail<58>*/
        goto _match20350;
      }
      kk_std_core__list _b_18504_18494 = _ctail_171660; /*list<4938>*/;
      kk_box_t* _b_18505_18495 = field0_17671; /*cfield<list<4938>>*/;
      _x20349 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18504_18494, _ctx), _b_18505_18495, _ctx); /*ctail<58>*/
      _match20350: ;
      lo1 = _x20348;
      _acc0 = _x20349;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  kk_integer_drop(hi1, _ctx);
  kk_integer_drop(lo1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18496 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x18496, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Applies a function `f` to list of increasing elements from `lo`  to `hi`
// (including both `lo`  and `hi` ).
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core__ctailm_list_fun20355__t_2 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_integer_t hi2;
  kk_integer_t lo2;
};
static kk_std_core__list kk_std_core__ctailm_list_fun20355_2(kk_function_t _fself, kk_box_t _ctail_171690, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20355_2(kk_function_t _accm0, kk_function_t f2, kk_integer_t hi2, kk_integer_t lo2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20355__t_2* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20355__t_2, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20355_2, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->hi2 = hi2;
  _self->lo2 = lo2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_list_fun20355_2(kk_function_t _fself, kk_box_t _ctail_171690, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20355__t_2* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20355__t_2*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<4938>) -> list<4938> */
  kk_function_t f2 = _self->f2; /* (int) -> 4939 4938 */
  kk_integer_t hi2 = _self->hi2; /* int */
  kk_integer_t lo2 = _self->lo2; /* int */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_integer_dup(hi2);kk_integer_dup(lo2);}, {}, _ctx)
  return kk_std_core__mlift17520_op(_accm0, f2, hi2, lo2, _ctail_171690, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_list_fun20357__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18508_18506;
};
static kk_box_t kk_std_core__ctailm_list_fun20357_2(kk_function_t _fself, kk_box_t _b_18507, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20357_2(kk_function_t _b_18508_18506, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20357__t_2* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20357__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20357_2, kk_context());
  _self->_b_18508_18506 = _b_18508_18506;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_list_fun20357_2(kk_function_t _fself, kk_box_t _b_18507, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20357__t_2* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20357__t_2*, _fself);
  kk_function_t _b_18508_18506 = _self->_b_18508_18506; /* (4938) -> 4939 list<4938> */
  kk_drop_match(_self, {kk_function_dup(_b_18508_18506);}, {}, _ctx)
  kk_std_core__list _x20358 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18508_18506, (_b_18508_18506, _b_18507, _ctx)); /*list<4938>*/
  return kk_std_core__list_box(_x20358, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_list_fun20361__t_2 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x0_17674;
};
static kk_std_core__list kk_std_core__ctailm_list_fun20361_2(kk_function_t _fself, kk_std_core__list _ctail_171680, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20361_2(kk_function_t _accm0, kk_box_t x0_17674, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20361__t_2* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20361__t_2, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20361_2, kk_context());
  _self->_accm0 = _accm0;
  _self->x0_17674 = x0_17674;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_list_fun20361_2(kk_function_t _fself, kk_std_core__list _ctail_171680, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20361__t_2* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20361__t_2*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<4938>) -> list<4938> */
  kk_box_t x0_17674 = _self->x0_17674; /* 4938 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x0_17674);}, {}, _ctx)
  kk_std_core__list _x20362 = kk_std_core__new_Cons(kk_reuse_null, x0_17674, _ctail_171680, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x20362, _ctx));
}

kk_std_core__list kk_std_core__ctailm_list_2(kk_integer_t lo2, kk_integer_t hi2, kk_function_t f2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, f : (int) -> e a, (list<a>) -> list<a>) -> e list<a> */ 
  kk__tailcall: ;
  bool _match_19925 = kk_integer_lte_borrow(lo2,hi2,kk_context()); /*bool*/;
  if (_match_19925) {
    kk_box_t x0_17674;
    kk_function_t _x20354 = kk_function_dup(f2); /*(int) -> 4939 4938*/
    kk_integer_t _x20353 = kk_integer_dup(lo2); /*int*/
    x0_17674 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_context_t*), _x20354, (_x20354, _x20353, _ctx)); /*4938*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x0_17674, _ctx);
      kk_function_t _b_18508_18506 = kk_std_core__new_ctailm_list_fun20355_2(_accm0, f2, hi2, lo2, _ctx); /*(4938) -> 4939 list<4938>*/;
      kk_box_t _x20356 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_list_fun20357_2(_b_18508_18506, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20356, _ctx);
    }
    { // tailcall
      kk_integer_t _x20359 = kk_integer_add(lo2,(kk_integer_from_small(1)),kk_context()); /*int*/
      kk_function_t _x20360 = kk_std_core__new_ctailm_list_fun20361_2(_accm0, x0_17674, _ctx); /*(list<4938>) -> list<4938>*/
      lo2 = _x20359;
      _accm0 = _x20360;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  kk_integer_drop(hi2, _ctx);
  kk_integer_drop(lo2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Applies a function `f` to list of increasing elements from `lo`  to `hi`
// (including both `lo`  and `hi` ).
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core_list_fun20363__t_2 {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_list_fun20363_2(kk_function_t _fself, kk_std_core__list _ctail_17167, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_list_fun20363_2(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_list_fun20363_2, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_list_fun20363_2(kk_function_t _fself, kk_std_core__list _ctail_17167, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17167;
}

kk_std_core__list kk_std_core_list_2(kk_integer_t lo3, kk_integer_t hi3, kk_function_t f3, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, f : (int) -> e a) -> e list<a> */ 
  bool _match_19924 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19924) {
    return kk_std_core__ctail_list_2(lo3, hi3, f3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_list_2(lo3, hi3, f3, kk_std_core_new_list_fun20363_2(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17521_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_integer_t hi, kk_integer_t lo, kk_integer_t stride, kk_box_t _ctail_17170, kk_context_t* _ctx) { /* forall<a,e> (ctail<list<a>>, f : (int) -> e a, hi : int, lo : int, stride : int, a) -> e list<a> */ 
  kk_std_core__list _ctail_17171 = kk_std_core__list_hole(); /*list<5029>*/;
  kk_std_core__list _ctail_17172 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17170, _ctail_17171, _ctx); /*list<5029>*/;
  kk_integer_t _x20364;
  kk_integer_t _x20365 = kk_integer_dup(stride); /*int*/
  _x20364 = kk_integer_add(lo,_x20365,kk_context()); /*int*/
  kk_std_core_types__ctail _x20366;
  kk_box_t* field_17679 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17172)->tail)); /*cfield<list<5029>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18515 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18515, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17172, _ctx);
    kk_std_core__list _b_18524_18520 = head0; /*list<5029>*/;
    kk_box_t* _b_18525_18521 = field_17679; /*cfield<list<5029>>*/;
    _x20366 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18524_18520, _ctx), _b_18525_18521, _ctx); /*ctail<58>*/
    goto _match20367;
  }
  kk_std_core__list _b_18526_18522 = _ctail_17172; /*list<5029>*/;
  kk_box_t* _b_18527_18523 = field_17679; /*cfield<list<5029>>*/;
  _x20366 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18526_18522, _ctx), _b_18527_18523, _ctx); /*ctail<58>*/
  _match20367: ;
  return kk_std_core__ctail_list_3(_x20364, hi, stride, f, _x20366, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17522_op_fun20371__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t _ctail_17175;
};
static kk_std_core__list kk_std_core__mlift17522_op_fun20371(kk_function_t _fself, kk_std_core__list _ctail_17174, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17522_op_fun20371(kk_function_t _accm, kk_box_t _ctail_17175, kk_context_t* _ctx) {
  struct kk_std_core__mlift17522_op_fun20371__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17522_op_fun20371__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17522_op_fun20371, kk_context());
  _self->_accm = _accm;
  _self->_ctail_17175 = _ctail_17175;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17522_op_fun20371(kk_function_t _fself, kk_std_core__list _ctail_17174, kk_context_t* _ctx) {
  struct kk_std_core__mlift17522_op_fun20371__t* _self = kk_function_as(struct kk_std_core__mlift17522_op_fun20371__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<5029>) -> list<5029> */
  kk_box_t _ctail_17175 = _self->_ctail_17175; /* 5029 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(_ctail_17175);}, {}, _ctx)
  kk_std_core__list _x20372 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17175, _ctail_17174, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x20372, _ctx));
}

kk_std_core__list kk_std_core__mlift17522_op(kk_function_t _accm, kk_function_t f0, kk_integer_t hi0, kk_integer_t lo0, kk_integer_t stride0, kk_box_t _ctail_17175, kk_context_t* _ctx) { /* forall<a,e> ((list<a>) -> list<a>, f : (int) -> e a, hi : int, lo : int, stride : int, a) -> e list<a> */ 
  kk_integer_t _x20369;
  kk_integer_t _x20370 = kk_integer_dup(stride0); /*int*/
  _x20369 = kk_integer_add(lo0,_x20370,kk_context()); /*int*/
  return kk_std_core__ctailm_list_3(_x20369, hi0, stride0, f0, kk_std_core__new_mlift17522_op_fun20371(_accm, _ctail_17175, _ctx), _ctx);
}
 
// Returns an integer list of increasing elements from `lo`  to `hi` with stride `stride`.
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core__ctail_list_fun20375__t_3 {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_integer_t hi1;
  kk_integer_t lo1;
  kk_integer_t stride1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_list_fun20375_3(kk_function_t _fself, kk_box_t _ctail_171700, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_list_fun20375_3(kk_function_t f1, kk_integer_t hi1, kk_integer_t lo1, kk_integer_t stride1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20375__t_3* _self = kk_function_alloc_as(struct kk_std_core__ctail_list_fun20375__t_3, 7, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_list_fun20375_3, kk_context());
  _self->f1 = f1;
  _self->hi1 = hi1;
  _self->lo1 = lo1;
  _self->stride1 = stride1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_list_fun20375_3(kk_function_t _fself, kk_box_t _ctail_171700, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20375__t_3* _self = kk_function_as(struct kk_std_core__ctail_list_fun20375__t_3*, _fself);
  kk_function_t f1 = _self->f1; /* (int) -> 5030 5029 */
  kk_integer_t hi1 = _self->hi1; /* int */
  kk_integer_t lo1 = _self->lo1; /* int */
  kk_integer_t stride1 = _self->stride1; /* int */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<5029>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_integer_dup(hi1);kk_integer_dup(lo1);kk_integer_dup(stride1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17521_op(_acc0, f1, hi1, lo1, stride1, _ctail_171700, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_list_fun20377__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18550_18528;
};
static kk_box_t kk_std_core__ctail_list_fun20377_3(kk_function_t _fself, kk_box_t _b_18529, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_list_fun20377_3(kk_function_t _b_18550_18528, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20377__t_3* _self = kk_function_alloc_as(struct kk_std_core__ctail_list_fun20377__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_list_fun20377_3, kk_context());
  _self->_b_18550_18528 = _b_18550_18528;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_list_fun20377_3(kk_function_t _fself, kk_box_t _b_18529, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20377__t_3* _self = kk_function_as(struct kk_std_core__ctail_list_fun20377__t_3*, _fself);
  kk_function_t _b_18550_18528 = _self->_b_18550_18528; /* (5029) -> 5030 list<5029> */
  kk_drop_match(_self, {kk_function_dup(_b_18550_18528);}, {}, _ctx)
  kk_std_core__list _x20378 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18550_18528, (_b_18550_18528, _b_18529, _ctx)); /*list<5029>*/
  return kk_std_core__list_box(_x20378, _ctx);
}

kk_std_core__list kk_std_core__ctail_list_3(kk_integer_t lo1, kk_integer_t hi1, kk_integer_t stride1, kk_function_t f1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, stride : int, f : (int) -> e a, ctail<list<a>>) -> e list<a> */ 
  kk__tailcall: ;
  bool _match_19922 = kk_integer_lte_borrow(lo1,hi1,kk_context()); /*bool*/;
  if (_match_19922) {
    kk_box_t x_17680;
    kk_function_t _x20374 = kk_function_dup(f1); /*(int) -> 5030 5029*/
    kk_integer_t _x20373 = kk_integer_dup(lo1); /*int*/
    x_17680 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_context_t*), _x20374, (_x20374, _x20373, _ctx)); /*5029*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x_17680, _ctx);
      kk_function_t _b_18550_18528 = kk_std_core__new_ctail_list_fun20375_3(f1, hi1, lo1, stride1, _acc0, _ctx); /*(5029) -> 5030 list<5029>*/;
      kk_box_t _x20376 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_list_fun20377_3(_b_18550_18528, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20376, _ctx);
    }
    kk_std_core__list _ctail_171710 = kk_std_core__list_hole(); /*list<5029>*/;
    kk_std_core__list _ctail_171720 = kk_std_core__new_Cons(kk_reuse_null, x_17680, _ctail_171710, _ctx); /*list<5029>*/;
    { // tailcall
      kk_integer_t _x20379;
      kk_integer_t _x20380 = kk_integer_dup(stride1); /*int*/
      _x20379 = kk_integer_add(lo1,_x20380,kk_context()); /*int*/
      kk_std_core_types__ctail _x20381;
      kk_box_t* field0_17685 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_171720)->tail)); /*cfield<list<5029>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x18536 = _acc0._cons.CTail._field1;
        kk_box_t* last2 = _acc0._cons.CTail._field2;
        kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18536, NULL);
        kk_std_core__list_dup(head1);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __0 = kk_Unit;
        *(last2) = kk_std_core__list_box(_ctail_171720, _ctx);
        kk_std_core__list _b_18551_18541 = head1; /*list<5029>*/;
        kk_box_t* _b_18552_18542 = field0_17685; /*cfield<list<5029>>*/;
        _x20381 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18551_18541, _ctx), _b_18552_18542, _ctx); /*ctail<58>*/
        goto _match20382;
      }
      kk_std_core__list _b_18553_18543 = _ctail_171720; /*list<5029>*/;
      kk_box_t* _b_18554_18544 = field0_17685; /*cfield<list<5029>>*/;
      _x20381 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18553_18543, _ctx), _b_18554_18544, _ctx); /*ctail<58>*/
      _match20382: ;
      lo1 = _x20379;
      _acc0 = _x20381;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  kk_integer_drop(hi1, _ctx);
  kk_integer_drop(lo1, _ctx);
  kk_integer_drop(stride1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18545 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x18545, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Returns an integer list of increasing elements from `lo`  to `hi` with stride `stride`.
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core__ctailm_list_fun20387__t_3 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_integer_t hi2;
  kk_integer_t lo2;
  kk_integer_t stride2;
};
static kk_std_core__list kk_std_core__ctailm_list_fun20387_3(kk_function_t _fself, kk_box_t _ctail_171750, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20387_3(kk_function_t _accm0, kk_function_t f2, kk_integer_t hi2, kk_integer_t lo2, kk_integer_t stride2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20387__t_3* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20387__t_3, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20387_3, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->hi2 = hi2;
  _self->lo2 = lo2;
  _self->stride2 = stride2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_list_fun20387_3(kk_function_t _fself, kk_box_t _ctail_171750, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20387__t_3* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20387__t_3*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<5029>) -> list<5029> */
  kk_function_t f2 = _self->f2; /* (int) -> 5030 5029 */
  kk_integer_t hi2 = _self->hi2; /* int */
  kk_integer_t lo2 = _self->lo2; /* int */
  kk_integer_t stride2 = _self->stride2; /* int */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_integer_dup(hi2);kk_integer_dup(lo2);kk_integer_dup(stride2);}, {}, _ctx)
  return kk_std_core__mlift17522_op(_accm0, f2, hi2, lo2, stride2, _ctail_171750, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_list_fun20389__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18557_18555;
};
static kk_box_t kk_std_core__ctailm_list_fun20389_3(kk_function_t _fself, kk_box_t _b_18556, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20389_3(kk_function_t _b_18557_18555, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20389__t_3* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20389__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20389_3, kk_context());
  _self->_b_18557_18555 = _b_18557_18555;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_list_fun20389_3(kk_function_t _fself, kk_box_t _b_18556, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20389__t_3* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20389__t_3*, _fself);
  kk_function_t _b_18557_18555 = _self->_b_18557_18555; /* (5029) -> 5030 list<5029> */
  kk_drop_match(_self, {kk_function_dup(_b_18557_18555);}, {}, _ctx)
  kk_std_core__list _x20390 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18557_18555, (_b_18557_18555, _b_18556, _ctx)); /*list<5029>*/
  return kk_std_core__list_box(_x20390, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_list_fun20394__t_3 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x0_17688;
};
static kk_std_core__list kk_std_core__ctailm_list_fun20394_3(kk_function_t _fself, kk_std_core__list _ctail_171740, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20394_3(kk_function_t _accm0, kk_box_t x0_17688, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20394__t_3* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20394__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20394_3, kk_context());
  _self->_accm0 = _accm0;
  _self->x0_17688 = x0_17688;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_list_fun20394_3(kk_function_t _fself, kk_std_core__list _ctail_171740, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20394__t_3* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20394__t_3*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<5029>) -> list<5029> */
  kk_box_t x0_17688 = _self->x0_17688; /* 5029 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x0_17688);}, {}, _ctx)
  kk_std_core__list _x20395 = kk_std_core__new_Cons(kk_reuse_null, x0_17688, _ctail_171740, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x20395, _ctx));
}

kk_std_core__list kk_std_core__ctailm_list_3(kk_integer_t lo2, kk_integer_t hi2, kk_integer_t stride2, kk_function_t f2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, stride : int, f : (int) -> e a, (list<a>) -> list<a>) -> e list<a> */ 
  kk__tailcall: ;
  bool _match_19920 = kk_integer_lte_borrow(lo2,hi2,kk_context()); /*bool*/;
  if (_match_19920) {
    kk_box_t x0_17688;
    kk_function_t _x20386 = kk_function_dup(f2); /*(int) -> 5030 5029*/
    kk_integer_t _x20385 = kk_integer_dup(lo2); /*int*/
    x0_17688 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_context_t*), _x20386, (_x20386, _x20385, _ctx)); /*5029*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x0_17688, _ctx);
      kk_function_t _b_18557_18555 = kk_std_core__new_ctailm_list_fun20387_3(_accm0, f2, hi2, lo2, stride2, _ctx); /*(5029) -> 5030 list<5029>*/;
      kk_box_t _x20388 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_list_fun20389_3(_b_18557_18555, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20388, _ctx);
    }
    { // tailcall
      kk_integer_t _x20391;
      kk_integer_t _x20392 = kk_integer_dup(stride2); /*int*/
      _x20391 = kk_integer_add(lo2,_x20392,kk_context()); /*int*/
      kk_function_t _x20393 = kk_std_core__new_ctailm_list_fun20394_3(_accm0, x0_17688, _ctx); /*(list<5029>) -> list<5029>*/
      lo2 = _x20391;
      _accm0 = _x20393;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  kk_integer_drop(hi2, _ctx);
  kk_integer_drop(lo2, _ctx);
  kk_integer_drop(stride2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Returns an integer list of increasing elements from `lo`  to `hi` with stride `stride`.
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core_list_fun20396__t_3 {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_list_fun20396_3(kk_function_t _fself, kk_std_core__list _ctail_17173, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_list_fun20396_3(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_list_fun20396_3, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_list_fun20396_3(kk_function_t _fself, kk_std_core__list _ctail_17173, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17173;
}

kk_std_core__list kk_std_core_list_3(kk_integer_t lo3, kk_integer_t hi3, kk_integer_t stride3, kk_function_t f3, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, stride : int, f : (int) -> e a) -> e list<a> */ 
  bool _match_19919 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19919) {
    return kk_std_core__ctail_list_3(lo3, hi3, stride3, f3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_list_3(lo3, hi3, stride3, f3, kk_std_core_new_list_fun20396_3(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17523_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_std_core__list xx, kk_box_t _ctail_17176, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, f : (a) -> e b, xx : list<a>, b) -> e list<b> */ 
  kk_std_core__list _ctail_17177 = kk_std_core__list_hole(); /*list<6139>*/;
  kk_std_core__list _ctail_17178 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17176, _ctail_17177, _ctx); /*list<6139>*/;
  kk_std_core_types__ctail _x20397;
  kk_box_t* field_17693 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17178)->tail)); /*cfield<list<6139>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18564 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18564, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17178, _ctx);
    kk_std_core__list _b_18573_18569 = head0; /*list<6139>*/;
    kk_box_t* _b_18574_18570 = field_17693; /*cfield<list<6139>>*/;
    _x20397 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18573_18569, _ctx), _b_18574_18570, _ctx); /*ctail<58>*/
    goto _match20398;
  }
  kk_std_core__list _b_18575_18571 = _ctail_17178; /*list<6139>*/;
  kk_box_t* _b_18576_18572 = field_17693; /*cfield<list<6139>>*/;
  _x20397 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18575_18571, _ctx), _b_18576_18572, _ctx); /*ctail<58>*/
  _match20398: ;
  return kk_std_core__ctail_map_5(xx, f, _x20397, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17524_op_fun20400__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t _ctail_17181;
};
static kk_std_core__list kk_std_core__mlift17524_op_fun20400(kk_function_t _fself, kk_std_core__list _ctail_17180, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17524_op_fun20400(kk_function_t _accm, kk_box_t _ctail_17181, kk_context_t* _ctx) {
  struct kk_std_core__mlift17524_op_fun20400__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17524_op_fun20400__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17524_op_fun20400, kk_context());
  _self->_accm = _accm;
  _self->_ctail_17181 = _ctail_17181;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17524_op_fun20400(kk_function_t _fself, kk_std_core__list _ctail_17180, kk_context_t* _ctx) {
  struct kk_std_core__mlift17524_op_fun20400__t* _self = kk_function_as(struct kk_std_core__mlift17524_op_fun20400__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<6139>) -> list<6139> */
  kk_box_t _ctail_17181 = _self->_ctail_17181; /* 6139 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(_ctail_17181);}, {}, _ctx)
  kk_std_core__list _x20401 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17181, _ctail_17180, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x20401, _ctx));
}

kk_std_core__list kk_std_core__mlift17524_op(kk_function_t _accm, kk_function_t f0, kk_std_core__list xx0, kk_box_t _ctail_17181, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, f : (a) -> e b, xx : list<a>, b) -> e list<b> */ 
  return kk_std_core__ctailm_map_5(xx0, f0, kk_std_core__new_mlift17524_op_fun20400(_accm, _ctail_17181, _ctx), _ctx);
}
 
// Apply a function `f`  to each element of the input list in sequence.


// lift anonymous function
struct kk_std_core__ctail_map_fun20404__t_5 {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_map_fun20404_5(kk_function_t _fself, kk_box_t _ctail_171760, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_map_fun20404_5(kk_function_t f1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_fun20404__t_5* _self = kk_function_alloc_as(struct kk_std_core__ctail_map_fun20404__t_5, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_map_fun20404_5, kk_context());
  _self->f1 = f1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_map_fun20404_5(kk_function_t _fself, kk_box_t _ctail_171760, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_fun20404__t_5* _self = kk_function_as(struct kk_std_core__ctail_map_fun20404__t_5*, _fself);
  kk_function_t f1 = _self->f1; /* (6138) -> 6140 6139 */
  kk_std_core__list xx1 = _self->xx1; /* list<6138> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<6139>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17523_op(_acc0, f1, xx1, _ctail_171760, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_map_fun20406__t_5 {
  struct kk_function_s _base;
  kk_function_t _b_18599_18577;
};
static kk_box_t kk_std_core__ctail_map_fun20406_5(kk_function_t _fself, kk_box_t _b_18578, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_map_fun20406_5(kk_function_t _b_18599_18577, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_fun20406__t_5* _self = kk_function_alloc_as(struct kk_std_core__ctail_map_fun20406__t_5, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_map_fun20406_5, kk_context());
  _self->_b_18599_18577 = _b_18599_18577;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_map_fun20406_5(kk_function_t _fself, kk_box_t _b_18578, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_fun20406__t_5* _self = kk_function_as(struct kk_std_core__ctail_map_fun20406__t_5*, _fself);
  kk_function_t _b_18599_18577 = _self->_b_18599_18577; /* (6139) -> 6140 list<6139> */
  kk_drop_match(_self, {kk_function_dup(_b_18599_18577);}, {}, _ctx)
  kk_std_core__list _x20407 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18599_18577, (_b_18599_18577, _b_18578, _ctx)); /*list<6139>*/
  return kk_std_core__list_box(_x20407, _ctx);
}

kk_std_core__list kk_std_core__ctail_map_5(kk_std_core__list xs, kk_function_t f1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e b, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20402 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con20402->head;
    kk_std_core__list xx1 = _con20402->tail;
    kk_reuse_t _ru_20028 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20028 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_box_t x0_17694;
    kk_function_t _x20403 = kk_function_dup(f1); /*(6138) -> 6140 6139*/
    x0_17694 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20403, (_x20403, x, _ctx)); /*6139*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20028);
      kk_box_drop(x0_17694, _ctx);
      kk_function_t _b_18599_18577 = kk_std_core__new_ctail_map_fun20404_5(f1, xx1, _acc0, _ctx); /*(6139) -> 6140 list<6139>*/;
      kk_box_t _x20405 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_map_fun20406_5(_b_18599_18577, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20405, _ctx);
    }
    kk_std_core__list _ctail_171770 = kk_std_core__list_hole(); /*list<6139>*/;
    kk_std_core__list _ctail_171780 = kk_std_core__new_Cons(_ru_20028, x0_17694, _ctail_171770, _ctx); /*list<6139>*/;
    { // tailcall
      kk_std_core_types__ctail _x20408;
      kk_box_t* field0_17699 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_171780)->tail)); /*cfield<list<6139>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x18585 = _acc0._cons.CTail._field1;
        kk_box_t* last2 = _acc0._cons.CTail._field2;
        kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18585, NULL);
        kk_std_core__list_dup(head1);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __0 = kk_Unit;
        *(last2) = kk_std_core__list_box(_ctail_171780, _ctx);
        kk_std_core__list _b_18600_18590 = head1; /*list<6139>*/;
        kk_box_t* _b_18601_18591 = field0_17699; /*cfield<list<6139>>*/;
        _x20408 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18600_18590, _ctx), _b_18601_18591, _ctx); /*ctail<58>*/
        goto _match20409;
      }
      kk_std_core__list _b_18602_18592 = _ctail_171780; /*list<6139>*/;
      kk_box_t* _b_18603_18593 = field0_17699; /*cfield<list<6139>>*/;
      _x20408 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18602_18592, _ctx), _b_18603_18593, _ctx); /*ctail<58>*/
      _match20409: ;
      xs = xx1;
      _acc0 = _x20408;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18594 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x18594, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Apply a function `f`  to each element of the input list in sequence.


// lift anonymous function
struct kk_std_core__ctailm_map_fun20414__t_5 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_map_fun20414_5(kk_function_t _fself, kk_box_t _ctail_171810, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_fun20414_5(kk_function_t _accm0, kk_function_t f2, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20414__t_5* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_fun20414__t_5, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_fun20414_5, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_map_fun20414_5(kk_function_t _fself, kk_box_t _ctail_171810, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20414__t_5* _self = kk_function_as(struct kk_std_core__ctailm_map_fun20414__t_5*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<6139>) -> list<6139> */
  kk_function_t f2 = _self->f2; /* (6138) -> 6140 6139 */
  kk_std_core__list xx2 = _self->xx2; /* list<6138> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17524_op(_accm0, f2, xx2, _ctail_171810, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_map_fun20416__t_5 {
  struct kk_function_s _base;
  kk_function_t _b_18606_18604;
};
static kk_box_t kk_std_core__ctailm_map_fun20416_5(kk_function_t _fself, kk_box_t _b_18605, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_fun20416_5(kk_function_t _b_18606_18604, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20416__t_5* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_fun20416__t_5, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_fun20416_5, kk_context());
  _self->_b_18606_18604 = _b_18606_18604;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_map_fun20416_5(kk_function_t _fself, kk_box_t _b_18605, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20416__t_5* _self = kk_function_as(struct kk_std_core__ctailm_map_fun20416__t_5*, _fself);
  kk_function_t _b_18606_18604 = _self->_b_18606_18604; /* (6139) -> 6140 list<6139> */
  kk_drop_match(_self, {kk_function_dup(_b_18606_18604);}, {}, _ctx)
  kk_std_core__list _x20417 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18606_18604, (_b_18606_18604, _b_18605, _ctx)); /*list<6139>*/
  return kk_std_core__list_box(_x20417, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_map_fun20419__t_5 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x2_17702;
};
static kk_std_core__list kk_std_core__ctailm_map_fun20419_5(kk_function_t _fself, kk_std_core__list _ctail_171800, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_fun20419_5(kk_function_t _accm0, kk_box_t x2_17702, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20419__t_5* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_fun20419__t_5, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_fun20419_5, kk_context());
  _self->_accm0 = _accm0;
  _self->x2_17702 = x2_17702;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_map_fun20419_5(kk_function_t _fself, kk_std_core__list _ctail_171800, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20419__t_5* _self = kk_function_as(struct kk_std_core__ctailm_map_fun20419__t_5*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<6139>) -> list<6139> */
  kk_box_t x2_17702 = _self->x2_17702; /* 6139 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x2_17702);}, {}, _ctx)
  kk_std_core__list _x20420 = kk_std_core__new_Cons(kk_reuse_null, x2_17702, _ctail_171800, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x20420, _ctx));
}

kk_std_core__list kk_std_core__ctailm_map_5(kk_std_core__list xs0, kk_function_t f2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e b, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con20412 = kk_std_core__as_Cons(xs0);
    kk_box_t x1 = _con20412->head;
    kk_std_core__list xx2 = _con20412->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx2);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    kk_box_t x2_17702;
    kk_function_t _x20413 = kk_function_dup(f2); /*(6138) -> 6140 6139*/
    x2_17702 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20413, (_x20413, x1, _ctx)); /*6139*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x2_17702, _ctx);
      kk_function_t _b_18606_18604 = kk_std_core__new_ctailm_map_fun20414_5(_accm0, f2, xx2, _ctx); /*(6139) -> 6140 list<6139>*/;
      kk_box_t _x20415 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_map_fun20416_5(_b_18606_18604, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20415, _ctx);
    }
    { // tailcall
      kk_function_t _x20418 = kk_std_core__new_ctailm_map_fun20419_5(_accm0, x2_17702, _ctx); /*(list<6139>) -> list<6139>*/
      xs0 = xx2;
      _accm0 = _x20418;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Apply a function `f`  to each element of the input list in sequence.


// lift anonymous function
struct kk_std_core_map_fun20421__t_5 {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_map_fun20421_5(kk_function_t _fself, kk_std_core__list _ctail_17179, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20421_5(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_map_fun20421_5, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_map_fun20421_5(kk_function_t _fself, kk_std_core__list _ctail_17179, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17179;
}

kk_std_core__list kk_std_core_map_5(kk_std_core__list xs1, kk_function_t f3, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e b) -> e list<b> */ 
  bool _match_19916 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19916) {
    return kk_std_core__ctail_map_5(xs1, f3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_map_5(xs1, f3, kk_std_core_new_map_fun20421_5(_ctx), _ctx);
}
 
// Create a list of characters from `lo`  to `hi`  (inclusive).


// lift anonymous function
struct kk_std_core_list_fun20424__t_4 {
  struct kk_function_s _base;
};
static kk_char_t kk_std_core_list_fun20424_4(kk_function_t _fself, kk_integer_t i, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_list_fun20424_4(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_list_fun20424_4, _ctx)
  return kk_function_dup(_fself);
}

static kk_char_t kk_std_core_list_fun20424_4(kk_function_t _fself, kk_integer_t i, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_char_t _borrowApp_19915 = kk_integer_clamp32_borrow(i); /*char*/;
  kk_integer_drop(i, _ctx);
  return _borrowApp_19915;
}


// lift anonymous function
struct kk_std_core_list_fun20425__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18611_18608;
};
static kk_box_t kk_std_core_list_fun20425_4(kk_function_t _fself, kk_box_t _b_18609, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_list_fun20425_4(kk_function_t _b_18611_18608, kk_context_t* _ctx) {
  struct kk_std_core_list_fun20425__t_4* _self = kk_function_alloc_as(struct kk_std_core_list_fun20425__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_list_fun20425_4, kk_context());
  _self->_b_18611_18608 = _b_18611_18608;
  return &_self->_base;
}

static kk_box_t kk_std_core_list_fun20425_4(kk_function_t _fself, kk_box_t _b_18609, kk_context_t* _ctx) {
  struct kk_std_core_list_fun20425__t_4* _self = kk_function_as(struct kk_std_core_list_fun20425__t_4*, _fself);
  kk_function_t _b_18611_18608 = _self->_b_18611_18608; /* (i : int) -> char */
  kk_drop_match(_self, {kk_function_dup(_b_18611_18608);}, {}, _ctx)
  kk_char_t _x20426;
  kk_integer_t _x20427 = kk_integer_unbox(_b_18609); /*int*/
  _x20426 = kk_function_call(kk_char_t, (kk_function_t, kk_integer_t, kk_context_t*), _b_18611_18608, (_b_18611_18608, _x20427, _ctx)); /*char*/
  return kk_char_box(_x20426, _ctx);
}

kk_std_core__list kk_std_core_list_4(kk_char_t lo, kk_char_t hi, kk_context_t* _ctx) { /* (lo : char, hi : char) -> total list<char> */ 
  kk_std_core__list _b_18610_18607;
  kk_integer_t _x20422 = kk_integer_from_int(lo,kk_context()); /*int*/
  kk_integer_t _x20423 = kk_integer_from_int(hi,kk_context()); /*int*/
  _b_18610_18607 = kk_std_core_list(_x20422, _x20423, _ctx); /*list<int>*/
  kk_function_t _b_18611_18608 = kk_std_core_new_list_fun20424_4(_ctx); /*(i : int) -> char*/;
  return kk_std_core_map_5(_b_18610_18607, kk_std_core_new_list_fun20425_4(_b_18611_18608, _ctx), _ctx);
}
 
// Convert a string to a list of characters

kk_std_core__list kk_std_core_list_6(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> total list<char> */ 
  return kk_string_to_list(s,kk_context());
}


// lift anonymous function
struct kk_std_core_map_fun20428__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_core_map_fun20428(kk_function_t _fself, kk_box_t _y_17287, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20428(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_map_fun20428, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_core_map_fun20428(kk_function_t _fself, kk_box_t _y_17287, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_types__new_Just(_y_17287, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20430__t {
  struct kk_function_s _base;
  kk_function_t _b_18614_18612;
};
static kk_box_t kk_std_core_map_fun20430(kk_function_t _fself, kk_box_t _b_18613, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20430(kk_function_t _b_18614_18612, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20430__t* _self = kk_function_alloc_as(struct kk_std_core_map_fun20430__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20430, kk_context());
  _self->_b_18614_18612 = _b_18614_18612;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20430(kk_function_t _fself, kk_box_t _b_18613, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20430__t* _self = kk_function_as(struct kk_std_core_map_fun20430__t*, _fself);
  kk_function_t _b_18614_18612 = _self->_b_18614_18612; /* (5319) -> 5320 maybe<5319> */
  kk_drop_match(_self, {kk_function_dup(_b_18614_18612);}, {}, _ctx)
  kk_std_core_types__maybe _x20431 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _b_18614_18612, (_b_18614_18612, _b_18613, _ctx)); /*maybe<5319>*/
  return kk_std_core_types__maybe_box(_x20431, _ctx);
}

kk_std_core_types__maybe kk_std_core_map(kk_std_core_types__maybe m, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (m : maybe<a>, f : (a) -> e b) -> e maybe<b> */ 
  if (kk_std_core_types__is_Nothing(m)) {
    kk_function_drop(f, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_box_t x = m._cons.Just.value;
  kk_box_t x0_17705 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x, _ctx)); /*5319*/;
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17705, _ctx);
    kk_function_t _b_18614_18612 = kk_std_core_new_map_fun20428(_ctx); /*(5319) -> 5320 maybe<5319>*/;
    kk_box_t _x20429 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20430(_b_18614_18612, _ctx), _ctx); /*3858*/
    return kk_std_core_types__maybe_unbox(_x20429, _ctx);
  }
  return kk_std_core_types__new_Just(x0_17705, _ctx);
}
 
// Map over the `Right` component of an `:either` type.


// lift anonymous function
struct kk_std_core_map_fun20432__t_1 {
  struct kk_function_s _base;
};
static kk_std_core_types__either kk_std_core_map_fun20432_1(kk_function_t _fself, kk_box_t _y_17289, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20432_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_map_fun20432_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__either kk_std_core_map_fun20432_1(kk_function_t _fself, kk_box_t _y_17289, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_types__new_Right(_y_17289, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20434__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_18617_18615;
};
static kk_box_t kk_std_core_map_fun20434_1(kk_function_t _fself, kk_box_t _b_18616, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20434_1(kk_function_t _b_18617_18615, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20434__t_1* _self = kk_function_alloc_as(struct kk_std_core_map_fun20434__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20434_1, kk_context());
  _self->_b_18617_18615 = _b_18617_18615;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20434_1(kk_function_t _fself, kk_box_t _b_18616, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20434__t_1* _self = kk_function_as(struct kk_std_core_map_fun20434__t_1*, _fself);
  kk_function_t _b_18617_18615 = _self->_b_18617_18615; /* (5357) -> 5358 either<5355,5357> */
  kk_drop_match(_self, {kk_function_dup(_b_18617_18615);}, {}, _ctx)
  kk_std_core_types__either _x20435 = kk_function_call(kk_std_core_types__either, (kk_function_t, kk_box_t, kk_context_t*), _b_18617_18615, (_b_18617_18615, _b_18616, _ctx)); /*either<5355,5357>*/
  return kk_std_core_types__either_box(_x20435, _ctx);
}

kk_std_core_types__either kk_std_core_map_1(kk_std_core_types__either e, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,c,e> (e : either<a,b>, f : (b) -> e c) -> e either<a,c> */ 
  if (kk_std_core_types__is_Right(e)) {
    kk_box_t x = e._cons.Right.right;
    kk_box_t x0_17709 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x, _ctx)); /*5357*/;
    if (kk_yielding(kk_context())) {
      kk_box_drop(x0_17709, _ctx);
      kk_function_t _b_18617_18615 = kk_std_core_new_map_fun20432_1(_ctx); /*(5357) -> 5358 either<5355,5357>*/;
      kk_box_t _x20433 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20434_1(_b_18617_18615, _ctx), _ctx); /*3858*/
      return kk_std_core_types__either_unbox(_x20433, _ctx);
    }
    return kk_std_core_types__new_Right(x0_17709, _ctx);
  }
  kk_box_t x00 = e._cons.Left.left;
  kk_function_drop(f, _ctx);
  return kk_std_core_types__new_Left(x00, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17528_map_fun20437__t_2 {
  struct kk_function_s _base;
  kk_box_t _y_17291;
};
static kk_std_core_types__tuple2_ kk_std_core__mlift17528_map_fun20437_2(kk_function_t _fself, kk_box_t _y_17292, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17528_map_fun20437_2(kk_box_t _y_17291, kk_context_t* _ctx) {
  struct kk_std_core__mlift17528_map_fun20437__t_2* _self = kk_function_alloc_as(struct kk_std_core__mlift17528_map_fun20437__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17528_map_fun20437_2, kk_context());
  _self->_y_17291 = _y_17291;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_core__mlift17528_map_fun20437_2(kk_function_t _fself, kk_box_t _y_17292, kk_context_t* _ctx) {
  struct kk_std_core__mlift17528_map_fun20437__t_2* _self = kk_function_as(struct kk_std_core__mlift17528_map_fun20437__t_2*, _fself);
  kk_box_t _y_17291 = _self->_y_17291; /* 5502 */
  kk_drop_match(_self, {kk_box_dup(_y_17291);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__rp_(_y_17291, _y_17292, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17528_map_fun20439__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18620_18618;
};
static kk_box_t kk_std_core__mlift17528_map_fun20439_2(kk_function_t _fself, kk_box_t _b_18619, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17528_map_fun20439_2(kk_function_t _b_18620_18618, kk_context_t* _ctx) {
  struct kk_std_core__mlift17528_map_fun20439__t_2* _self = kk_function_alloc_as(struct kk_std_core__mlift17528_map_fun20439__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17528_map_fun20439_2, kk_context());
  _self->_b_18620_18618 = _b_18620_18618;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17528_map_fun20439_2(kk_function_t _fself, kk_box_t _b_18619, kk_context_t* _ctx) {
  struct kk_std_core__mlift17528_map_fun20439__t_2* _self = kk_function_as(struct kk_std_core__mlift17528_map_fun20439__t_2*, _fself);
  kk_function_t _b_18620_18618 = _self->_b_18620_18618; /* (5502) -> 5503 (5502, 5502) */
  kk_drop_match(_self, {kk_function_dup(_b_18620_18618);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x20440 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_box_t, kk_context_t*), _b_18620_18618, (_b_18620_18618, _b_18619, _ctx)); /*(5502, 5502)*/
  return kk_std_core_types__tuple2__box(_x20440, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core__mlift17528_map_2(kk_function_t f, kk_std_core_types__tuple2_ t, kk_box_t _y_17291, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e b, t : (a, a), b) -> e (b, b) */ 
  kk_box_t x_17713;
  kk_box_t _x20436;
  kk_box_t _x = t.snd;
  kk_box_dup(_x);
  kk_std_core_types__tuple2__drop(t, _ctx);
  kk_Unit;
  _x20436 = _x; /*5501*/
  x_17713 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20436, _ctx)); /*5502*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17713, _ctx);
    kk_function_t _b_18620_18618 = kk_std_core__new_mlift17528_map_fun20437_2(_y_17291, _ctx); /*(5502) -> 5503 (5502, 5502)*/;
    kk_box_t _x20438 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17528_map_fun20439_2(_b_18620_18618, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple2__unbox(_x20438, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__rp_(_y_17291, x_17713, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20443__t_2 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple2_ t;
};
static kk_std_core_types__tuple2_ kk_std_core_map_fun20443_2(kk_function_t _fself, kk_box_t _y_17291, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20443_2(kk_function_t f, kk_std_core_types__tuple2_ t, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20443__t_2* _self = kk_function_alloc_as(struct kk_std_core_map_fun20443__t_2, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20443_2, kk_context());
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_core_map_fun20443_2(kk_function_t _fself, kk_box_t _y_17291, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20443__t_2* _self = kk_function_as(struct kk_std_core_map_fun20443__t_2*, _fself);
  kk_function_t f = _self->f; /* (5501) -> 5503 5502 */
  kk_std_core_types__tuple2_ t = _self->t; /* (5501, 5501) */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple2__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17528_map_2(f, t, _y_17291, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20445__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18625_18621;
};
static kk_box_t kk_std_core_map_fun20445_2(kk_function_t _fself, kk_box_t _b_18622, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20445_2(kk_function_t _b_18625_18621, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20445__t_2* _self = kk_function_alloc_as(struct kk_std_core_map_fun20445__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20445_2, kk_context());
  _self->_b_18625_18621 = _b_18625_18621;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20445_2(kk_function_t _fself, kk_box_t _b_18622, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20445__t_2* _self = kk_function_as(struct kk_std_core_map_fun20445__t_2*, _fself);
  kk_function_t _b_18625_18621 = _self->_b_18625_18621; /* (5502) -> 5503 (5502, 5502) */
  kk_drop_match(_self, {kk_function_dup(_b_18625_18621);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x20446 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_box_t, kk_context_t*), _b_18625_18621, (_b_18625_18621, _b_18622, _ctx)); /*(5502, 5502)*/
  return kk_std_core_types__tuple2__box(_x20446, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20448__t_2 {
  struct kk_function_s _base;
  kk_box_t x_17718;
};
static kk_std_core_types__tuple2_ kk_std_core_map_fun20448_2(kk_function_t _fself, kk_box_t _y_17292, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20448_2(kk_box_t x_17718, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20448__t_2* _self = kk_function_alloc_as(struct kk_std_core_map_fun20448__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20448_2, kk_context());
  _self->x_17718 = x_17718;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_core_map_fun20448_2(kk_function_t _fself, kk_box_t _y_17292, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20448__t_2* _self = kk_function_as(struct kk_std_core_map_fun20448__t_2*, _fself);
  kk_box_t x_17718 = _self->x_17718; /* 5502 */
  kk_drop_match(_self, {kk_box_dup(x_17718);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__rp_(x_17718, _y_17292, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20450__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18626_18623;
};
static kk_box_t kk_std_core_map_fun20450_2(kk_function_t _fself, kk_box_t _b_18624, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20450_2(kk_function_t _b_18626_18623, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20450__t_2* _self = kk_function_alloc_as(struct kk_std_core_map_fun20450__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20450_2, kk_context());
  _self->_b_18626_18623 = _b_18626_18623;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20450_2(kk_function_t _fself, kk_box_t _b_18624, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20450__t_2* _self = kk_function_as(struct kk_std_core_map_fun20450__t_2*, _fself);
  kk_function_t _b_18626_18623 = _self->_b_18626_18623; /* (5502) -> 5503 (5502, 5502) */
  kk_drop_match(_self, {kk_function_dup(_b_18626_18623);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x20451 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_box_t, kk_context_t*), _b_18626_18623, (_b_18626_18623, _b_18624, _ctx)); /*(5502, 5502)*/
  return kk_std_core_types__tuple2__box(_x20451, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core_map_2(kk_std_core_types__tuple2_ t, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (t : (a, a), f : (a) -> e b) -> e (b, b) */ 
  kk_box_t x_17718;
  kk_function_t _x20442 = kk_function_dup(f); /*(5501) -> 5503 5502*/
  kk_box_t _x20441;
  kk_box_t _x = t.fst;
  kk_box_dup(_x);
  _x20441 = _x; /*5501*/
  x_17718 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20442, (_x20442, _x20441, _ctx)); /*5502*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17718, _ctx);
    kk_function_t _b_18625_18621 = kk_std_core_new_map_fun20443_2(f, t, _ctx); /*(5502) -> 5503 (5502, 5502)*/;
    kk_box_t _x20444 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20445_2(_b_18625_18621, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple2__unbox(_x20444, _ctx);
  }
  kk_box_t x0_17722;
  kk_box_t _x20447;
  kk_box_t _x0 = t.snd;
  kk_box_dup(_x0);
  kk_std_core_types__tuple2__drop(t, _ctx);
  kk_Unit;
  _x20447 = _x0; /*5501*/
  x0_17722 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20447, _ctx)); /*5502*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17722, _ctx);
    kk_function_t _b_18626_18623 = kk_std_core_new_map_fun20448_2(x_17718, _ctx); /*(5502) -> 5503 (5502, 5502)*/;
    kk_box_t _x20449 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20450_2(_b_18626_18623, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple2__unbox(_x20449, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__rp_(x_17718, x0_17722, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17530_map_fun20453__t_3 {
  struct kk_function_s _base;
  kk_box_t _y_17293;
  kk_box_t _y_17294;
};
static kk_std_core_types__tuple3_ kk_std_core__mlift17530_map_fun20453_3(kk_function_t _fself, kk_box_t _y_17295, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17530_map_fun20453_3(kk_box_t _y_17293, kk_box_t _y_17294, kk_context_t* _ctx) {
  struct kk_std_core__mlift17530_map_fun20453__t_3* _self = kk_function_alloc_as(struct kk_std_core__mlift17530_map_fun20453__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17530_map_fun20453_3, kk_context());
  _self->_y_17293 = _y_17293;
  _self->_y_17294 = _y_17294;
  return &_self->_base;
}

static kk_std_core_types__tuple3_ kk_std_core__mlift17530_map_fun20453_3(kk_function_t _fself, kk_box_t _y_17295, kk_context_t* _ctx) {
  struct kk_std_core__mlift17530_map_fun20453__t_3* _self = kk_function_as(struct kk_std_core__mlift17530_map_fun20453__t_3*, _fself);
  kk_box_t _y_17293 = _self->_y_17293; /* 5720 */
  kk_box_t _y_17294 = _self->_y_17294; /* 5720 */
  kk_drop_match(_self, {kk_box_dup(_y_17293);kk_box_dup(_y_17294);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__comma__rp_(_y_17293, _y_17294, _y_17295, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17530_map_fun20455__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18629_18627;
};
static kk_box_t kk_std_core__mlift17530_map_fun20455_3(kk_function_t _fself, kk_box_t _b_18628, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17530_map_fun20455_3(kk_function_t _b_18629_18627, kk_context_t* _ctx) {
  struct kk_std_core__mlift17530_map_fun20455__t_3* _self = kk_function_alloc_as(struct kk_std_core__mlift17530_map_fun20455__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17530_map_fun20455_3, kk_context());
  _self->_b_18629_18627 = _b_18629_18627;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17530_map_fun20455_3(kk_function_t _fself, kk_box_t _b_18628, kk_context_t* _ctx) {
  struct kk_std_core__mlift17530_map_fun20455__t_3* _self = kk_function_as(struct kk_std_core__mlift17530_map_fun20455__t_3*, _fself);
  kk_function_t _b_18629_18627 = _self->_b_18629_18627; /* (5720) -> 5721 (5720, 5720, 5720) */
  kk_drop_match(_self, {kk_function_dup(_b_18629_18627);}, {}, _ctx)
  kk_std_core_types__tuple3_ _x20456 = kk_function_call(kk_std_core_types__tuple3_, (kk_function_t, kk_box_t, kk_context_t*), _b_18629_18627, (_b_18629_18627, _b_18628, _ctx)); /*(5720, 5720, 5720)*/
  return kk_std_core_types__tuple3__box(_x20456, _ctx);
}

kk_std_core_types__tuple3_ kk_std_core__mlift17530_map_3(kk_box_t _y_17293, kk_function_t f, kk_std_core_types__tuple3_ t, kk_box_t _y_17294, kk_context_t* _ctx) { /* forall<a,b,e> (b, f : (a) -> e b, t : (a, a, a), b) -> e (b, b, b) */ 
  kk_box_t x_17728;
  kk_box_t _x20452;
  kk_box_t _x = t.thd;
  kk_box_dup(_x);
  kk_std_core_types__tuple3__drop(t, _ctx);
  kk_Unit;
  _x20452 = _x; /*5719*/
  x_17728 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20452, _ctx)); /*5720*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17728, _ctx);
    kk_function_t _b_18629_18627 = kk_std_core__new_mlift17530_map_fun20453_3(_y_17293, _y_17294, _ctx); /*(5720) -> 5721 (5720, 5720, 5720)*/;
    kk_box_t _x20454 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17530_map_fun20455_3(_b_18629_18627, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple3__unbox(_x20454, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__comma__rp_(_y_17293, _y_17294, x_17728, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17531_map_fun20459__t_3 {
  struct kk_function_s _base;
  kk_box_t _y_17293;
  kk_function_t f;
  kk_std_core_types__tuple3_ t;
};
static kk_std_core_types__tuple3_ kk_std_core__mlift17531_map_fun20459_3(kk_function_t _fself, kk_box_t _y_17294, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17531_map_fun20459_3(kk_box_t _y_17293, kk_function_t f, kk_std_core_types__tuple3_ t, kk_context_t* _ctx) {
  struct kk_std_core__mlift17531_map_fun20459__t_3* _self = kk_function_alloc_as(struct kk_std_core__mlift17531_map_fun20459__t_3, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17531_map_fun20459_3, kk_context());
  _self->_y_17293 = _y_17293;
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple3_ kk_std_core__mlift17531_map_fun20459_3(kk_function_t _fself, kk_box_t _y_17294, kk_context_t* _ctx) {
  struct kk_std_core__mlift17531_map_fun20459__t_3* _self = kk_function_as(struct kk_std_core__mlift17531_map_fun20459__t_3*, _fself);
  kk_box_t _y_17293 = _self->_y_17293; /* 5720 */
  kk_function_t f = _self->f; /* (5719) -> 5721 5720 */
  kk_std_core_types__tuple3_ t = _self->t; /* (5719, 5719, 5719) */
  kk_drop_match(_self, {kk_box_dup(_y_17293);kk_function_dup(f);kk_std_core_types__tuple3__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17530_map_3(_y_17293, f, t, _y_17294, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17531_map_fun20461__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18632_18630;
};
static kk_box_t kk_std_core__mlift17531_map_fun20461_3(kk_function_t _fself, kk_box_t _b_18631, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17531_map_fun20461_3(kk_function_t _b_18632_18630, kk_context_t* _ctx) {
  struct kk_std_core__mlift17531_map_fun20461__t_3* _self = kk_function_alloc_as(struct kk_std_core__mlift17531_map_fun20461__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17531_map_fun20461_3, kk_context());
  _self->_b_18632_18630 = _b_18632_18630;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17531_map_fun20461_3(kk_function_t _fself, kk_box_t _b_18631, kk_context_t* _ctx) {
  struct kk_std_core__mlift17531_map_fun20461__t_3* _self = kk_function_as(struct kk_std_core__mlift17531_map_fun20461__t_3*, _fself);
  kk_function_t _b_18632_18630 = _self->_b_18632_18630; /* (5720) -> 5721 (5720, 5720, 5720) */
  kk_drop_match(_self, {kk_function_dup(_b_18632_18630);}, {}, _ctx)
  kk_std_core_types__tuple3_ _x20462 = kk_function_call(kk_std_core_types__tuple3_, (kk_function_t, kk_box_t, kk_context_t*), _b_18632_18630, (_b_18632_18630, _b_18631, _ctx)); /*(5720, 5720, 5720)*/
  return kk_std_core_types__tuple3__box(_x20462, _ctx);
}

kk_std_core_types__tuple3_ kk_std_core__mlift17531_map_3(kk_function_t f, kk_std_core_types__tuple3_ t, kk_box_t _y_17293, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e b, t : (a, a, a), b) -> e (b, b, b) */ 
  kk_box_t x_17734;
  kk_function_t _x20458 = kk_function_dup(f); /*(5719) -> 5721 5720*/
  kk_box_t _x20457;
  kk_box_t _x = t.snd;
  kk_box_dup(_x);
  _x20457 = _x; /*5719*/
  x_17734 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20458, (_x20458, _x20457, _ctx)); /*5720*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17734, _ctx);
    kk_function_t _b_18632_18630 = kk_std_core__new_mlift17531_map_fun20459_3(_y_17293, f, t, _ctx); /*(5720) -> 5721 (5720, 5720, 5720)*/;
    kk_box_t _x20460 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17531_map_fun20461_3(_b_18632_18630, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple3__unbox(_x20460, _ctx);
  }
  return kk_std_core__mlift17530_map_3(_y_17293, f, t, x_17734, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20465__t_3 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple3_ t;
};
static kk_std_core_types__tuple3_ kk_std_core_map_fun20465_3(kk_function_t _fself, kk_box_t _y_17293, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20465_3(kk_function_t f, kk_std_core_types__tuple3_ t, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20465__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20465__t_3, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20465_3, kk_context());
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple3_ kk_std_core_map_fun20465_3(kk_function_t _fself, kk_box_t _y_17293, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20465__t_3* _self = kk_function_as(struct kk_std_core_map_fun20465__t_3*, _fself);
  kk_function_t f = _self->f; /* (5719) -> 5721 5720 */
  kk_std_core_types__tuple3_ t = _self->t; /* (5719, 5719, 5719) */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple3__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17531_map_3(f, t, _y_17293, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20467__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18639_18633;
};
static kk_box_t kk_std_core_map_fun20467_3(kk_function_t _fself, kk_box_t _b_18634, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20467_3(kk_function_t _b_18639_18633, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20467__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20467__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20467_3, kk_context());
  _self->_b_18639_18633 = _b_18639_18633;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20467_3(kk_function_t _fself, kk_box_t _b_18634, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20467__t_3* _self = kk_function_as(struct kk_std_core_map_fun20467__t_3*, _fself);
  kk_function_t _b_18639_18633 = _self->_b_18639_18633; /* (5720) -> 5721 (5720, 5720, 5720) */
  kk_drop_match(_self, {kk_function_dup(_b_18639_18633);}, {}, _ctx)
  kk_std_core_types__tuple3_ _x20468 = kk_function_call(kk_std_core_types__tuple3_, (kk_function_t, kk_box_t, kk_context_t*), _b_18639_18633, (_b_18639_18633, _b_18634, _ctx)); /*(5720, 5720, 5720)*/
  return kk_std_core_types__tuple3__box(_x20468, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20471__t_3 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple3_ t;
  kk_box_t x_17737;
};
static kk_std_core_types__tuple3_ kk_std_core_map_fun20471_3(kk_function_t _fself, kk_box_t _y_17294, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20471_3(kk_function_t f, kk_std_core_types__tuple3_ t, kk_box_t x_17737, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20471__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20471__t_3, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20471_3, kk_context());
  _self->f = f;
  _self->t = t;
  _self->x_17737 = x_17737;
  return &_self->_base;
}

static kk_std_core_types__tuple3_ kk_std_core_map_fun20471_3(kk_function_t _fself, kk_box_t _y_17294, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20471__t_3* _self = kk_function_as(struct kk_std_core_map_fun20471__t_3*, _fself);
  kk_function_t f = _self->f; /* (5719) -> 5721 5720 */
  kk_std_core_types__tuple3_ t = _self->t; /* (5719, 5719, 5719) */
  kk_box_t x_17737 = _self->x_17737; /* 5720 */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple3__dup(t);kk_box_dup(x_17737);}, {}, _ctx)
  return kk_std_core__mlift17530_map_3(x_17737, f, t, _y_17294, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20473__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18640_18635;
};
static kk_box_t kk_std_core_map_fun20473_3(kk_function_t _fself, kk_box_t _b_18636, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20473_3(kk_function_t _b_18640_18635, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20473__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20473__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20473_3, kk_context());
  _self->_b_18640_18635 = _b_18640_18635;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20473_3(kk_function_t _fself, kk_box_t _b_18636, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20473__t_3* _self = kk_function_as(struct kk_std_core_map_fun20473__t_3*, _fself);
  kk_function_t _b_18640_18635 = _self->_b_18640_18635; /* (5720) -> 5721 (5720, 5720, 5720) */
  kk_drop_match(_self, {kk_function_dup(_b_18640_18635);}, {}, _ctx)
  kk_std_core_types__tuple3_ _x20474 = kk_function_call(kk_std_core_types__tuple3_, (kk_function_t, kk_box_t, kk_context_t*), _b_18640_18635, (_b_18640_18635, _b_18636, _ctx)); /*(5720, 5720, 5720)*/
  return kk_std_core_types__tuple3__box(_x20474, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20476__t_3 {
  struct kk_function_s _base;
  kk_box_t x_17737;
  kk_box_t x0_17741;
};
static kk_std_core_types__tuple3_ kk_std_core_map_fun20476_3(kk_function_t _fself, kk_box_t _y_17295, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20476_3(kk_box_t x_17737, kk_box_t x0_17741, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20476__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20476__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20476_3, kk_context());
  _self->x_17737 = x_17737;
  _self->x0_17741 = x0_17741;
  return &_self->_base;
}

static kk_std_core_types__tuple3_ kk_std_core_map_fun20476_3(kk_function_t _fself, kk_box_t _y_17295, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20476__t_3* _self = kk_function_as(struct kk_std_core_map_fun20476__t_3*, _fself);
  kk_box_t x_17737 = _self->x_17737; /* 5720 */
  kk_box_t x0_17741 = _self->x0_17741; /* 5720 */
  kk_drop_match(_self, {kk_box_dup(x_17737);kk_box_dup(x0_17741);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__comma__rp_(x_17737, x0_17741, _y_17295, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20478__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18641_18637;
};
static kk_box_t kk_std_core_map_fun20478_3(kk_function_t _fself, kk_box_t _b_18638, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20478_3(kk_function_t _b_18641_18637, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20478__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20478__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20478_3, kk_context());
  _self->_b_18641_18637 = _b_18641_18637;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20478_3(kk_function_t _fself, kk_box_t _b_18638, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20478__t_3* _self = kk_function_as(struct kk_std_core_map_fun20478__t_3*, _fself);
  kk_function_t _b_18641_18637 = _self->_b_18641_18637; /* (5720) -> 5721 (5720, 5720, 5720) */
  kk_drop_match(_self, {kk_function_dup(_b_18641_18637);}, {}, _ctx)
  kk_std_core_types__tuple3_ _x20479 = kk_function_call(kk_std_core_types__tuple3_, (kk_function_t, kk_box_t, kk_context_t*), _b_18641_18637, (_b_18641_18637, _b_18638, _ctx)); /*(5720, 5720, 5720)*/
  return kk_std_core_types__tuple3__box(_x20479, _ctx);
}

kk_std_core_types__tuple3_ kk_std_core_map_3(kk_std_core_types__tuple3_ t, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (t : (a, a, a), f : (a) -> e b) -> e (b, b, b) */ 
  kk_box_t x_17737;
  kk_function_t _x20464 = kk_function_dup(f); /*(5719) -> 5721 5720*/
  kk_box_t _x20463;
  kk_box_t _x = t.fst;
  kk_box_dup(_x);
  _x20463 = _x; /*5719*/
  x_17737 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20464, (_x20464, _x20463, _ctx)); /*5720*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17737, _ctx);
    kk_function_t _b_18639_18633 = kk_std_core_new_map_fun20465_3(f, t, _ctx); /*(5720) -> 5721 (5720, 5720, 5720)*/;
    kk_box_t _x20466 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20467_3(_b_18639_18633, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple3__unbox(_x20466, _ctx);
  }
  kk_box_t x0_17741;
  kk_function_t _x20470 = kk_function_dup(f); /*(5719) -> 5721 5720*/
  kk_box_t _x20469;
  kk_box_t _x0 = t.snd;
  kk_box_dup(_x0);
  _x20469 = _x0; /*5719*/
  x0_17741 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20470, (_x20470, _x20469, _ctx)); /*5720*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17741, _ctx);
    kk_function_t _b_18640_18635 = kk_std_core_new_map_fun20471_3(f, t, x_17737, _ctx); /*(5720) -> 5721 (5720, 5720, 5720)*/;
    kk_box_t _x20472 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20473_3(_b_18640_18635, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple3__unbox(_x20472, _ctx);
  }
  kk_box_t x1_17745;
  kk_box_t _x20475;
  kk_box_t _x1 = t.thd;
  kk_box_dup(_x1);
  kk_std_core_types__tuple3__drop(t, _ctx);
  kk_Unit;
  _x20475 = _x1; /*5719*/
  x1_17745 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20475, _ctx)); /*5720*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x1_17745, _ctx);
    kk_function_t _b_18641_18637 = kk_std_core_new_map_fun20476_3(x_17737, x0_17741, _ctx); /*(5720) -> 5721 (5720, 5720, 5720)*/;
    kk_box_t _x20477 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20478_3(_b_18641_18637, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple3__unbox(_x20477, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__comma__rp_(x_17737, x0_17741, x1_17745, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17533_map_fun20482__t_4 {
  struct kk_function_s _base;
  kk_box_t _y_17296;
  kk_box_t _y_17297;
  kk_box_t _y_17298;
};
static kk_std_core_types__tuple4_ kk_std_core__mlift17533_map_fun20482_4(kk_function_t _fself, kk_box_t _y_17299, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17533_map_fun20482_4(kk_box_t _y_17296, kk_box_t _y_17297, kk_box_t _y_17298, kk_context_t* _ctx) {
  struct kk_std_core__mlift17533_map_fun20482__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17533_map_fun20482__t_4, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17533_map_fun20482_4, kk_context());
  _self->_y_17296 = _y_17296;
  _self->_y_17297 = _y_17297;
  _self->_y_17298 = _y_17298;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core__mlift17533_map_fun20482_4(kk_function_t _fself, kk_box_t _y_17299, kk_context_t* _ctx) {
  struct kk_std_core__mlift17533_map_fun20482__t_4* _self = kk_function_as(struct kk_std_core__mlift17533_map_fun20482__t_4*, _fself);
  kk_box_t _y_17296 = _self->_y_17296; /* 6007 */
  kk_box_t _y_17297 = _self->_y_17297; /* 6007 */
  kk_box_t _y_17298 = _self->_y_17298; /* 6007 */
  kk_drop_match(_self, {kk_box_dup(_y_17296);kk_box_dup(_y_17297);kk_box_dup(_y_17298);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, _y_17296, _y_17297, _y_17298, _y_17299, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17533_map_fun20484__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18644_18642;
};
static kk_box_t kk_std_core__mlift17533_map_fun20484_4(kk_function_t _fself, kk_box_t _b_18643, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17533_map_fun20484_4(kk_function_t _b_18644_18642, kk_context_t* _ctx) {
  struct kk_std_core__mlift17533_map_fun20484__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17533_map_fun20484__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17533_map_fun20484_4, kk_context());
  _self->_b_18644_18642 = _b_18644_18642;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17533_map_fun20484_4(kk_function_t _fself, kk_box_t _b_18643, kk_context_t* _ctx) {
  struct kk_std_core__mlift17533_map_fun20484__t_4* _self = kk_function_as(struct kk_std_core__mlift17533_map_fun20484__t_4*, _fself);
  kk_function_t _b_18644_18642 = _self->_b_18644_18642; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18644_18642);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20485 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18644_18642, (_b_18644_18642, _b_18643, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20485, _ctx);
}

kk_std_core_types__tuple4_ kk_std_core__mlift17533_map_4(kk_box_t _y_17296, kk_box_t _y_17297, kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t _y_17298, kk_context_t* _ctx) { /* forall<a,b,e> (b, b, f : (a) -> e b, t : (a, a, a, a), b) -> e (b, b, b, b) */ 
  kk_box_t x_17752;
  kk_box_t _x20480;
  struct kk_std_core_types__Tuple4_* _con20481 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _pat00 = _con20481->fst;
  kk_box_t _pat10 = _con20481->snd;
  kk_box_t _pat2 = _con20481->thd;
  kk_box_t _x = _con20481->field4;
  if (kk_likely(kk_std_core_types__tuple4__is_unique(t))) {
    kk_box_drop(_pat2, _ctx);
    kk_Unit;
    kk_box_drop(_pat10, _ctx);
    kk_Unit;
    kk_box_drop(_pat00, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core_types__tuple4__free(t);
  }
  else {
    kk_box_dup(_x);
    kk_Unit;
    kk_std_core_types__tuple4__decref(t, _ctx);
    kk_Unit;
  }
  _x20480 = _x; /*6006*/
  x_17752 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20480, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17752, _ctx);
    kk_function_t _b_18644_18642 = kk_std_core__new_mlift17533_map_fun20482_4(_y_17296, _y_17297, _y_17298, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20483 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17533_map_fun20484_4(_b_18644_18642, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20483, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, _y_17296, _y_17297, _y_17298, x_17752, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17534_map_fun20489__t_4 {
  struct kk_function_s _base;
  kk_box_t _y_17296;
  kk_box_t _y_17297;
  kk_function_t f;
  kk_std_core_types__tuple4_ t;
};
static kk_std_core_types__tuple4_ kk_std_core__mlift17534_map_fun20489_4(kk_function_t _fself, kk_box_t _y_17298, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17534_map_fun20489_4(kk_box_t _y_17296, kk_box_t _y_17297, kk_function_t f, kk_std_core_types__tuple4_ t, kk_context_t* _ctx) {
  struct kk_std_core__mlift17534_map_fun20489__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17534_map_fun20489__t_4, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17534_map_fun20489_4, kk_context());
  _self->_y_17296 = _y_17296;
  _self->_y_17297 = _y_17297;
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core__mlift17534_map_fun20489_4(kk_function_t _fself, kk_box_t _y_17298, kk_context_t* _ctx) {
  struct kk_std_core__mlift17534_map_fun20489__t_4* _self = kk_function_as(struct kk_std_core__mlift17534_map_fun20489__t_4*, _fself);
  kk_box_t _y_17296 = _self->_y_17296; /* 6007 */
  kk_box_t _y_17297 = _self->_y_17297; /* 6007 */
  kk_function_t f = _self->f; /* (6006) -> 6008 6007 */
  kk_std_core_types__tuple4_ t = _self->t; /* (6006, 6006, 6006, 6006) */
  kk_drop_match(_self, {kk_box_dup(_y_17296);kk_box_dup(_y_17297);kk_function_dup(f);kk_std_core_types__tuple4__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17533_map_4(_y_17296, _y_17297, f, t, _y_17298, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17534_map_fun20491__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18647_18645;
};
static kk_box_t kk_std_core__mlift17534_map_fun20491_4(kk_function_t _fself, kk_box_t _b_18646, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17534_map_fun20491_4(kk_function_t _b_18647_18645, kk_context_t* _ctx) {
  struct kk_std_core__mlift17534_map_fun20491__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17534_map_fun20491__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17534_map_fun20491_4, kk_context());
  _self->_b_18647_18645 = _b_18647_18645;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17534_map_fun20491_4(kk_function_t _fself, kk_box_t _b_18646, kk_context_t* _ctx) {
  struct kk_std_core__mlift17534_map_fun20491__t_4* _self = kk_function_as(struct kk_std_core__mlift17534_map_fun20491__t_4*, _fself);
  kk_function_t _b_18647_18645 = _self->_b_18647_18645; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18647_18645);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20492 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18647_18645, (_b_18647_18645, _b_18646, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20492, _ctx);
}

kk_std_core_types__tuple4_ kk_std_core__mlift17534_map_4(kk_box_t _y_17296, kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t _y_17297, kk_context_t* _ctx) { /* forall<a,b,e> (b, f : (a) -> e b, t : (a, a, a, a), b) -> e (b, b, b, b) */ 
  kk_box_t x_17759;
  kk_function_t _x20488 = kk_function_dup(f); /*(6006) -> 6008 6007*/
  kk_box_t _x20486;
  struct kk_std_core_types__Tuple4_* _con20487 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _x = _con20487->thd;
  kk_box_dup(_x);
  _x20486 = _x; /*6006*/
  x_17759 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20488, (_x20488, _x20486, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17759, _ctx);
    kk_function_t _b_18647_18645 = kk_std_core__new_mlift17534_map_fun20489_4(_y_17296, _y_17297, f, t, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20490 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17534_map_fun20491_4(_b_18647_18645, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20490, _ctx);
  }
  return kk_std_core__mlift17533_map_4(_y_17296, _y_17297, f, t, x_17759, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17535_map_fun20496__t_4 {
  struct kk_function_s _base;
  kk_box_t _y_17296;
  kk_function_t f;
  kk_std_core_types__tuple4_ t;
};
static kk_std_core_types__tuple4_ kk_std_core__mlift17535_map_fun20496_4(kk_function_t _fself, kk_box_t _y_17297, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17535_map_fun20496_4(kk_box_t _y_17296, kk_function_t f, kk_std_core_types__tuple4_ t, kk_context_t* _ctx) {
  struct kk_std_core__mlift17535_map_fun20496__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17535_map_fun20496__t_4, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17535_map_fun20496_4, kk_context());
  _self->_y_17296 = _y_17296;
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core__mlift17535_map_fun20496_4(kk_function_t _fself, kk_box_t _y_17297, kk_context_t* _ctx) {
  struct kk_std_core__mlift17535_map_fun20496__t_4* _self = kk_function_as(struct kk_std_core__mlift17535_map_fun20496__t_4*, _fself);
  kk_box_t _y_17296 = _self->_y_17296; /* 6007 */
  kk_function_t f = _self->f; /* (6006) -> 6008 6007 */
  kk_std_core_types__tuple4_ t = _self->t; /* (6006, 6006, 6006, 6006) */
  kk_drop_match(_self, {kk_box_dup(_y_17296);kk_function_dup(f);kk_std_core_types__tuple4__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17534_map_4(_y_17296, f, t, _y_17297, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17535_map_fun20498__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18650_18648;
};
static kk_box_t kk_std_core__mlift17535_map_fun20498_4(kk_function_t _fself, kk_box_t _b_18649, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17535_map_fun20498_4(kk_function_t _b_18650_18648, kk_context_t* _ctx) {
  struct kk_std_core__mlift17535_map_fun20498__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17535_map_fun20498__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17535_map_fun20498_4, kk_context());
  _self->_b_18650_18648 = _b_18650_18648;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17535_map_fun20498_4(kk_function_t _fself, kk_box_t _b_18649, kk_context_t* _ctx) {
  struct kk_std_core__mlift17535_map_fun20498__t_4* _self = kk_function_as(struct kk_std_core__mlift17535_map_fun20498__t_4*, _fself);
  kk_function_t _b_18650_18648 = _self->_b_18650_18648; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18650_18648);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20499 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18650_18648, (_b_18650_18648, _b_18649, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20499, _ctx);
}

kk_std_core_types__tuple4_ kk_std_core__mlift17535_map_4(kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t _y_17296, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e b, t : (a, a, a, a), b) -> e (b, b, b, b) */ 
  kk_box_t x_17762;
  kk_function_t _x20495 = kk_function_dup(f); /*(6006) -> 6008 6007*/
  kk_box_t _x20493;
  struct kk_std_core_types__Tuple4_* _con20494 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _x = _con20494->snd;
  kk_box_dup(_x);
  _x20493 = _x; /*6006*/
  x_17762 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20495, (_x20495, _x20493, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17762, _ctx);
    kk_function_t _b_18650_18648 = kk_std_core__new_mlift17535_map_fun20496_4(_y_17296, f, t, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20497 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17535_map_fun20498_4(_b_18650_18648, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20497, _ctx);
  }
  return kk_std_core__mlift17534_map_4(_y_17296, f, t, x_17762, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20503__t_4 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple4_ t;
};
static kk_std_core_types__tuple4_ kk_std_core_map_fun20503_4(kk_function_t _fself, kk_box_t _y_17296, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20503_4(kk_function_t f, kk_std_core_types__tuple4_ t, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20503__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20503__t_4, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20503_4, kk_context());
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core_map_fun20503_4(kk_function_t _fself, kk_box_t _y_17296, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20503__t_4* _self = kk_function_as(struct kk_std_core_map_fun20503__t_4*, _fself);
  kk_function_t f = _self->f; /* (6006) -> 6008 6007 */
  kk_std_core_types__tuple4_ t = _self->t; /* (6006, 6006, 6006, 6006) */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple4__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17535_map_4(f, t, _y_17296, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20505__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18659_18651;
};
static kk_box_t kk_std_core_map_fun20505_4(kk_function_t _fself, kk_box_t _b_18652, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20505_4(kk_function_t _b_18659_18651, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20505__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20505__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20505_4, kk_context());
  _self->_b_18659_18651 = _b_18659_18651;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20505_4(kk_function_t _fself, kk_box_t _b_18652, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20505__t_4* _self = kk_function_as(struct kk_std_core_map_fun20505__t_4*, _fself);
  kk_function_t _b_18659_18651 = _self->_b_18659_18651; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18659_18651);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20506 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18659_18651, (_b_18659_18651, _b_18652, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20506, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20510__t_4 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple4_ t;
  kk_box_t x_17765;
};
static kk_std_core_types__tuple4_ kk_std_core_map_fun20510_4(kk_function_t _fself, kk_box_t _y_17297, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20510_4(kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t x_17765, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20510__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20510__t_4, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20510_4, kk_context());
  _self->f = f;
  _self->t = t;
  _self->x_17765 = x_17765;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core_map_fun20510_4(kk_function_t _fself, kk_box_t _y_17297, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20510__t_4* _self = kk_function_as(struct kk_std_core_map_fun20510__t_4*, _fself);
  kk_function_t f = _self->f; /* (6006) -> 6008 6007 */
  kk_std_core_types__tuple4_ t = _self->t; /* (6006, 6006, 6006, 6006) */
  kk_box_t x_17765 = _self->x_17765; /* 6007 */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple4__dup(t);kk_box_dup(x_17765);}, {}, _ctx)
  return kk_std_core__mlift17534_map_4(x_17765, f, t, _y_17297, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20512__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18660_18653;
};
static kk_box_t kk_std_core_map_fun20512_4(kk_function_t _fself, kk_box_t _b_18654, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20512_4(kk_function_t _b_18660_18653, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20512__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20512__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20512_4, kk_context());
  _self->_b_18660_18653 = _b_18660_18653;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20512_4(kk_function_t _fself, kk_box_t _b_18654, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20512__t_4* _self = kk_function_as(struct kk_std_core_map_fun20512__t_4*, _fself);
  kk_function_t _b_18660_18653 = _self->_b_18660_18653; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18660_18653);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20513 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18660_18653, (_b_18660_18653, _b_18654, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20513, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20517__t_4 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple4_ t;
  kk_box_t x_17765;
  kk_box_t x0_17769;
};
static kk_std_core_types__tuple4_ kk_std_core_map_fun20517_4(kk_function_t _fself, kk_box_t _y_17298, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20517_4(kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t x_17765, kk_box_t x0_17769, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20517__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20517__t_4, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20517_4, kk_context());
  _self->f = f;
  _self->t = t;
  _self->x_17765 = x_17765;
  _self->x0_17769 = x0_17769;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core_map_fun20517_4(kk_function_t _fself, kk_box_t _y_17298, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20517__t_4* _self = kk_function_as(struct kk_std_core_map_fun20517__t_4*, _fself);
  kk_function_t f = _self->f; /* (6006) -> 6008 6007 */
  kk_std_core_types__tuple4_ t = _self->t; /* (6006, 6006, 6006, 6006) */
  kk_box_t x_17765 = _self->x_17765; /* 6007 */
  kk_box_t x0_17769 = _self->x0_17769; /* 6007 */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple4__dup(t);kk_box_dup(x_17765);kk_box_dup(x0_17769);}, {}, _ctx)
  return kk_std_core__mlift17533_map_4(x_17765, x0_17769, f, t, _y_17298, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20519__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18661_18655;
};
static kk_box_t kk_std_core_map_fun20519_4(kk_function_t _fself, kk_box_t _b_18656, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20519_4(kk_function_t _b_18661_18655, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20519__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20519__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20519_4, kk_context());
  _self->_b_18661_18655 = _b_18661_18655;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20519_4(kk_function_t _fself, kk_box_t _b_18656, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20519__t_4* _self = kk_function_as(struct kk_std_core_map_fun20519__t_4*, _fself);
  kk_function_t _b_18661_18655 = _self->_b_18661_18655; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18661_18655);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20520 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18661_18655, (_b_18661_18655, _b_18656, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20520, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20523__t_4 {
  struct kk_function_s _base;
  kk_box_t x_17765;
  kk_box_t x0_17769;
  kk_box_t x1_17773;
};
static kk_std_core_types__tuple4_ kk_std_core_map_fun20523_4(kk_function_t _fself, kk_box_t _y_17299, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20523_4(kk_box_t x_17765, kk_box_t x0_17769, kk_box_t x1_17773, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20523__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20523__t_4, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20523_4, kk_context());
  _self->x_17765 = x_17765;
  _self->x0_17769 = x0_17769;
  _self->x1_17773 = x1_17773;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core_map_fun20523_4(kk_function_t _fself, kk_box_t _y_17299, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20523__t_4* _self = kk_function_as(struct kk_std_core_map_fun20523__t_4*, _fself);
  kk_box_t x_17765 = _self->x_17765; /* 6007 */
  kk_box_t x0_17769 = _self->x0_17769; /* 6007 */
  kk_box_t x1_17773 = _self->x1_17773; /* 6007 */
  kk_drop_match(_self, {kk_box_dup(x_17765);kk_box_dup(x0_17769);kk_box_dup(x1_17773);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, x_17765, x0_17769, x1_17773, _y_17299, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20525__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18662_18657;
};
static kk_box_t kk_std_core_map_fun20525_4(kk_function_t _fself, kk_box_t _b_18658, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20525_4(kk_function_t _b_18662_18657, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20525__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20525__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20525_4, kk_context());
  _self->_b_18662_18657 = _b_18662_18657;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20525_4(kk_function_t _fself, kk_box_t _b_18658, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20525__t_4* _self = kk_function_as(struct kk_std_core_map_fun20525__t_4*, _fself);
  kk_function_t _b_18662_18657 = _self->_b_18662_18657; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18662_18657);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20526 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18662_18657, (_b_18662_18657, _b_18658, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20526, _ctx);
}

kk_std_core_types__tuple4_ kk_std_core_map_4(kk_std_core_types__tuple4_ t, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (t : (a, a, a, a), f : (a) -> e b) -> e (b, b, b, b) */ 
  kk_box_t x_17765;
  kk_function_t _x20502 = kk_function_dup(f); /*(6006) -> 6008 6007*/
  kk_box_t _x20500;
  struct kk_std_core_types__Tuple4_* _con20501 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _x = _con20501->fst;
  kk_box_dup(_x);
  _x20500 = _x; /*6006*/
  x_17765 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20502, (_x20502, _x20500, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17765, _ctx);
    kk_function_t _b_18659_18651 = kk_std_core_new_map_fun20503_4(f, t, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20504 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20505_4(_b_18659_18651, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20504, _ctx);
  }
  kk_box_t x0_17769;
  kk_function_t _x20509 = kk_function_dup(f); /*(6006) -> 6008 6007*/
  kk_box_t _x20507;
  struct kk_std_core_types__Tuple4_* _con20508 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _x0 = _con20508->snd;
  kk_box_dup(_x0);
  _x20507 = _x0; /*6006*/
  x0_17769 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20509, (_x20509, _x20507, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17769, _ctx);
    kk_function_t _b_18660_18653 = kk_std_core_new_map_fun20510_4(f, t, x_17765, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20511 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20512_4(_b_18660_18653, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20511, _ctx);
  }
  kk_box_t x1_17773;
  kk_function_t _x20516 = kk_function_dup(f); /*(6006) -> 6008 6007*/
  kk_box_t _x20514;
  struct kk_std_core_types__Tuple4_* _con20515 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _x1 = _con20515->thd;
  kk_box_dup(_x1);
  _x20514 = _x1; /*6006*/
  x1_17773 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20516, (_x20516, _x20514, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x1_17773, _ctx);
    kk_function_t _b_18661_18655 = kk_std_core_new_map_fun20517_4(f, t, x_17765, x0_17769, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20518 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20519_4(_b_18661_18655, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20518, _ctx);
  }
  kk_box_t x2_17777;
  kk_box_t _x20521;
  struct kk_std_core_types__Tuple4_* _con20522 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _pat06 = _con20522->fst;
  kk_box_t _pat13 = _con20522->snd;
  kk_box_t _pat22 = _con20522->thd;
  kk_box_t _x2 = _con20522->field4;
  if (kk_likely(kk_std_core_types__tuple4__is_unique(t))) {
    kk_box_drop(_pat22, _ctx);
    kk_Unit;
    kk_box_drop(_pat13, _ctx);
    kk_Unit;
    kk_box_drop(_pat06, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core_types__tuple4__free(t);
  }
  else {
    kk_box_dup(_x2);
    kk_Unit;
    kk_std_core_types__tuple4__decref(t, _ctx);
    kk_Unit;
  }
  _x20521 = _x2; /*6006*/
  x2_17777 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20521, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x2_17777, _ctx);
    kk_function_t _b_18662_18657 = kk_std_core_new_map_fun20523_4(x_17765, x0_17769, x1_17773, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20524 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20525_4(_b_18662_18657, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20524, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, x_17765, x0_17769, x1_17773, x2_17777, _ctx);
}
 
// Apply a function `f` to each character in a string


// lift anonymous function
struct kk_std_core_map_fun20527__t_6 {
  struct kk_function_s _base;
  kk_function_t f;
};
static kk_box_t kk_std_core_map_fun20527_6(kk_function_t _fself, kk_box_t _b_18665, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20527_6(kk_function_t f, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20527__t_6* _self = kk_function_alloc_as(struct kk_std_core_map_fun20527__t_6, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20527_6, kk_context());
  _self->f = f;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20527_6(kk_function_t _fself, kk_box_t _b_18665, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20527__t_6* _self = kk_function_as(struct kk_std_core_map_fun20527__t_6*, _fself);
  kk_function_t f = _self->f; /* (char) -> 6475 char */
  kk_drop_match(_self, {kk_function_dup(f);}, {}, _ctx)
  kk_char_t _x20528;
  kk_char_t _x20529 = kk_char_unbox(_b_18665, _ctx); /*char*/
  _x20528 = kk_function_call(kk_char_t, (kk_function_t, kk_char_t, kk_context_t*), f, (f, _x20529, _ctx)); /*char*/
  return kk_char_box(_x20528, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20531__t_6 {
  struct kk_function_s _base;
};
static kk_string_t kk_std_core_map_fun20531_6(kk_function_t _fself, kk_std_core__list _x120530, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20531_6(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_map_fun20531_6, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_core_map_fun20531_6(kk_function_t _fself, kk_std_core__list _x120530, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_string_2(_x120530, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20533__t_6 {
  struct kk_function_s _base;
  kk_function_t _b_18670_18668;
};
static kk_box_t kk_std_core_map_fun20533_6(kk_function_t _fself, kk_box_t _b_18669, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20533_6(kk_function_t _b_18670_18668, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20533__t_6* _self = kk_function_alloc_as(struct kk_std_core_map_fun20533__t_6, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20533_6, kk_context());
  _self->_b_18670_18668 = _b_18670_18668;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20533_6(kk_function_t _fself, kk_box_t _b_18669, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20533__t_6* _self = kk_function_as(struct kk_std_core_map_fun20533__t_6*, _fself);
  kk_function_t _b_18670_18668 = _self->_b_18670_18668; /* (cs : list<char>) -> total string */
  kk_drop_match(_self, {kk_function_dup(_b_18670_18668);}, {}, _ctx)
  kk_string_t _x20534;
  kk_std_core__list _x20535 = kk_std_core__list_unbox(_b_18669, _ctx); /*list<char>*/
  _x20534 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _b_18670_18668, (_b_18670_18668, _x20535, _ctx)); /*string*/
  return kk_string_box(_x20534);
}

kk_string_t kk_std_core_map_6(kk_string_t s, kk_function_t f, kk_context_t* _ctx) { /* forall<e> (s : string, f : (char) -> e char) -> e string */ 
  kk_std_core__list _b_18666_18663 = kk_std_core_list_6(s, _ctx); /*list<char>*/;
  kk_std_core__list x_17785 = kk_std_core_map_5(_b_18666_18663, kk_std_core_new_map_fun20527_6(f, _ctx), _ctx); /*list<char>*/;
  if (kk_yielding(kk_context())) {
    kk_std_core__list_drop(x_17785, _ctx);
    kk_function_t _b_18670_18668 = kk_std_core_new_map_fun20531_6(_ctx); /*(cs : list<char>) -> total string*/;
    kk_box_t _x20532 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20533_6(_b_18670_18668, _ctx), _ctx); /*3858*/
    return kk_string_unbox(_x20532);
  }
  return kk_std_core_string_2(x_17785, _ctx);
}
 
// Apply a total function `f` to each element in a vector `v`


// lift anonymous function
struct kk_std_core_map_fun20541__t_7 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_vector_t v;
  kk_vector_t w;
};
static kk_unit_t kk_std_core_map_fun20541_7(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20541_7(kk_function_t f, kk_vector_t v, kk_vector_t w, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20541__t_7* _self = kk_function_alloc_as(struct kk_std_core_map_fun20541__t_7, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20541_7, kk_context());
  _self->f = f;
  _self->v = v;
  _self->w = w;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_core_map_fun20543__t_7 {
  struct kk_function_s _base;
  kk_vector_t w;
  kk_ssize_t i;
};
static kk_box_t kk_std_core_map_fun20543_7(kk_function_t _fself, kk_box_t _b_18672, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20543_7(kk_vector_t w, kk_ssize_t i, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20543__t_7* _self = kk_function_alloc_as(struct kk_std_core_map_fun20543__t_7, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20543_7, kk_context());
  _self->w = w;
  _self->i = i;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20543_7(kk_function_t _fself, kk_box_t _b_18672, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20543__t_7* _self = kk_function_as(struct kk_std_core_map_fun20543__t_7*, _fself);
  kk_vector_t w = _self->w; /* vector<6524> */
  kk_ssize_t i = _self->i; /* ssize_t */
  kk_drop_match(_self, {kk_vector_dup(w);;}, {}, _ctx)
  kk_unit_t _x20544 = kk_Unit;
  kk_box_t _y_18676_17301 = _b_18672; /*6524*/;
  kk_vector_unsafe_assign(w,i,_y_18676_17301,kk_context());
  return kk_unit_box(_x20544);
}
static kk_unit_t kk_std_core_map_fun20541_7(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20541__t_7* _self = kk_function_as(struct kk_std_core_map_fun20541__t_7*, _fself);
  kk_function_t f = _self->f; /* (6523) -> 6525 6524 */
  kk_vector_t v = _self->v; /* vector<6523> */
  kk_vector_t w = _self->w; /* vector<6524> */
  kk_drop_match(_self, {kk_function_dup(f);kk_vector_dup(v);kk_vector_dup(w);}, {}, _ctx)
  kk_box_t x0_18111;
  kk_box_t _borrowApp_19896 = kk_vector_at_borrow(v, i); /*219*/;
  kk_vector_drop(v, _ctx);
  x0_18111 = _borrowApp_19896; /*6523*/
  kk_box_t x1_17793 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x0_18111, _ctx)); /*6524*/;
  if (kk_yielding(kk_context())) {
    kk_box_drop(x1_17793, _ctx);
    kk_box_t _x20542 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20543_7(w, i, _ctx), _ctx); /*3858*/
    return kk_unit_unbox(_x20542);
  }
  return kk_vector_unsafe_assign(w,i,x1_17793,kk_context());
}


// lift anonymous function
struct kk_std_core_map_fun20545__t_7 {
  struct kk_function_s _base;
  kk_vector_t w;
};
static kk_vector_t kk_std_core_map_fun20545_7(kk_function_t _fself, kk_unit_t wild__, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20545_7(kk_vector_t w, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20545__t_7* _self = kk_function_alloc_as(struct kk_std_core_map_fun20545__t_7, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20545_7, kk_context());
  _self->w = w;
  return &_self->_base;
}

static kk_vector_t kk_std_core_map_fun20545_7(kk_function_t _fself, kk_unit_t wild__, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20545__t_7* _self = kk_function_as(struct kk_std_core_map_fun20545__t_7*, _fself);
  kk_vector_t w = _self->w; /* vector<6524> */
  kk_drop_match(_self, {kk_vector_dup(w);}, {}, _ctx)
  return w;
}


// lift anonymous function
struct kk_std_core_map_fun20547__t_7 {
  struct kk_function_s _base;
  kk_function_t _b_18677_18674;
};
static kk_box_t kk_std_core_map_fun20547_7(kk_function_t _fself, kk_box_t _b_18675, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20547_7(kk_function_t _b_18677_18674, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20547__t_7* _self = kk_function_alloc_as(struct kk_std_core_map_fun20547__t_7, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20547_7, kk_context());
  _self->_b_18677_18674 = _b_18677_18674;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20547_7(kk_function_t _fself, kk_box_t _b_18675, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20547__t_7* _self = kk_function_as(struct kk_std_core_map_fun20547__t_7*, _fself);
  kk_function_t _b_18677_18674 = _self->_b_18677_18674; /* (wild_ : ()) -> 6525 vector<6524> */
  kk_drop_match(_self, {kk_function_dup(_b_18677_18674);}, {}, _ctx)
  kk_vector_t _x20548;
  kk_unit_t _x20549 = kk_Unit;
  kk_unit_unbox(_b_18675);
  _x20548 = kk_function_call(kk_vector_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_18677_18674, (_b_18677_18674, _x20549, _ctx)); /*vector<6524>*/
  return kk_vector_box(_x20548, _ctx);
}

kk_vector_t kk_std_core_map_7(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (v : vector<a>, f : (a) -> e b) -> e vector<b> */ 
  kk_vector_t w;
  kk_ssize_t _x20536;
  kk_integer_t _x20537;
  kk_ssize_t _x20538 = kk_vector_len_borrow(v); /*ssize_t*/
  _x20537 = kk_integer_from_ssize_t(_x20538,kk_context()); /*int*/
  _x20536 = kk_std_core_ssize__t(_x20537, _ctx); /*ssize_t*/
  w = kk_vector_alloc_uninit(_x20536,NULL,kk_context()); /*vector<6524>*/
  kk_ssize_t start0_17798 = ((kk_ssize_t)0); /*ssize_t*/;
  kk_ssize_t end_17799;
  kk_ssize_t _x20539 = kk_vector_len_borrow(v); /*ssize_t*/
  end_17799 = kk_std_core_decr_1(_x20539, _ctx); /*ssize_t*/
  kk_unit_t x_17788 = kk_Unit;
  kk_function_t _x20540;
  kk_vector_dup(w);
  _x20540 = kk_std_core_new_map_fun20541_7(f, v, w, _ctx); /*(i : ssize_t) -> 6525 ()*/
  kk_std_core__lift17073_forz(_x20540, end_17799, start0_17798, _ctx);
  if (kk_yielding(kk_context())) {
    kk_function_t _b_18677_18674 = kk_std_core_new_map_fun20545_7(w, _ctx); /*(wild_ : ()) -> 6525 vector<6524>*/;
    kk_box_t _x20546 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20547_7(_b_18677_18674, _ctx), _ctx); /*3858*/
    return kk_vector_unbox(_x20546, _ctx);
  }
  return w;
}
 
// Right-align a string to width `width`  using `fill`  (default is a space) to fill from the left.

kk_string_t kk_std_core_pad_left(kk_string_t s, kk_integer_t width, kk_std_core_types__optional fill, kk_context_t* _ctx) { /* (s : string, width : int, fill : optional<char>) -> string */ 
  kk_ssize_t w;
  kk_integer_t _x20550 = kk_integer_dup(width); /*int*/
  w = kk_std_core_ssize__t(_x20550, _ctx); /*ssize_t*/
  kk_ssize_t n;
  kk_string_t _x20551 = kk_string_dup(s); /*string*/
  n = kk_string_len(_x20551,kk_context()); /*ssize_t*/
  bool _match_19893 = (w <= n); /*bool*/;
  if (_match_19893) {
    kk_std_core_types__optional_drop(fill, _ctx);
    return s;
  }
  kk_string_t _x20552;
  kk_string_t _x20553;
  kk_char_t _x20554;
  if (kk_std_core_types__is_Optional(fill)) {
    kk_box_t _box_x18678 = fill._cons.Optional.value;
    kk_char_t _fill_7273 = kk_char_unbox(_box_x18678, NULL);
    kk_std_core_types__optional_drop(fill, _ctx);
    kk_Unit;
    _x20554 = _fill_7273; /*char*/
    goto _match20555;
  }
  _x20554 = ' '; /*char*/
  _match20555: ;
  _x20553 = kk_std_core_string(_x20554, _ctx); /*string*/
  kk_ssize_t _x20557 = (w - n); /*ssize_t*/
  _x20552 = kk_std_core_repeatz(_x20553, _x20557, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x20552, s, _ctx);
}
 
// Show an `:int` as a hexadecimal value.
// The `width`  parameter specifies how wide the hex value is where `"0"`  is used to align.
// The `use-capitals` parameter (= `True`) determines if captical letters should be used to display the hexadecimal digits.
// The `pre` (=`"0x"`) is an optional prefix for the number (goes between the sign and the number).

kk_string_t kk_std_core_show_hex(kk_integer_t i, kk_std_core_types__optional width, kk_std_core_types__optional use_capitals, kk_std_core_types__optional pre, kk_context_t* _ctx) { /* (i : int, width : optional<int>, use-capitals : optional<bool>, pre : optional<string>) -> string */ 
  kk_string_t _x20558;
  bool _match_19890;
  kk_integer_t _borrowedArg_19891 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19892 = kk_integer_lt_borrow(i,_borrowedArg_19891,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19891, _ctx);
  _match_19890 = _borrowApp_19892; /*bool*/
  if (_match_19890) {
    kk_define_string_literal(, _s20559, 1, "-")
    _x20558 = kk_string_dup(_s20559); /*string*/
  }
  else {
    _x20558 = kk_string_empty(); /*string*/
  }
  kk_string_t _x20561;
  kk_string_t _x20562;
  if (kk_std_core_types__is_Optional(pre)) {
    kk_box_t _box_x18679 = pre._cons.Optional.value;
    kk_string_t _pre_7381 = kk_string_unbox(_box_x18679);
    kk_string_dup(_pre_7381);
    kk_std_core_types__optional_drop(pre, _ctx);
    kk_Unit;
    _x20562 = _pre_7381; /*string*/
    goto _match20563;
  }
  kk_define_string_literal(, _s20565, 2, "0x")
  _x20562 = kk_string_dup(_s20565); /*string*/
  _match20563: ;
  kk_string_t _x20566;
  kk_string_t _ownedArg_19888;
  kk_integer_t _x20567 = kk_integer_abs(i,kk_context()); /*int*/
  bool _x20568;
  if (kk_std_core_types__is_Optional(use_capitals)) {
    kk_box_t _box_x18680 = use_capitals._cons.Optional.value;
    bool _use_capitals_7377 = kk_bool_unbox(_box_x18680);
    kk_std_core_types__optional_drop(use_capitals, _ctx);
    kk_Unit;
    _x20568 = _use_capitals_7377; /*bool*/
    goto _match20569;
  }
  _x20568 = true; /*bool*/
  _match20569: ;
  _ownedArg_19888 = kk_std_core_int_show_hex(_x20567, _x20568, _ctx); /*string*/
  kk_integer_t _borrowedArg_19887;
  if (kk_std_core_types__is_Optional(width)) {
    kk_box_t _box_x18681 = width._cons.Optional.value;
    kk_integer_t _width_7373 = kk_integer_unbox(_box_x18681);
    kk_integer_dup(_width_7373);
    kk_std_core_types__optional_drop(width, _ctx);
    kk_Unit;
    _borrowedArg_19887 = _width_7373; /*int*/
    goto _match20571;
  }
  _borrowedArg_19887 = kk_integer_from_small(1); /*int*/
  _match20571: ;
  kk_string_t _borrowApp_19889;
  kk_std_core_types__optional _x20573;
  kk_char_t _b_18683_18682 = '0'; /*char*/;
  _x20573 = kk_std_core_types__new_Optional(kk_char_box(_b_18683_18682, _ctx), _ctx); /*optional<106>*/
  _borrowApp_19889 = kk_std_core_pad_left(_ownedArg_19888, _borrowedArg_19887, _x20573, _ctx); /*string*/
  kk_integer_drop(_borrowedArg_19887, _ctx);
  _x20566 = _borrowApp_19889; /*string*/
  _x20561 = kk_std_core__lp__plus__plus__1_rp_(_x20562, _x20566, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x20558, _x20561, _ctx);
}
 
// Is the integer positive (stricly greater than zero)

bool kk_std_core_is_pos_2(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> bool */ 
  kk_std_core_types__order x_17112;
  kk_std_core_types__order _borrowApp_19884 = kk_int_as_order(kk_integer_signum_borrow(i),kk_context()); /*order*/;
  kk_integer_drop(i, _ctx);
  x_17112 = _borrowApp_19884; /*order*/
  kk_integer_t _borrowedArg_19882;
  if (kk_std_core_types__is_Lt(x_17112)) {
    _borrowedArg_19882 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20574;
  }
  if (kk_std_core_types__is_Eq(x_17112)) {
    _borrowedArg_19882 = kk_integer_from_small(0); /*int*/
    goto _match20574;
  }
  _borrowedArg_19882 = kk_integer_from_small(1); /*int*/
  _match20574: ;
  kk_integer_t _borrowedArg_19881 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19883 = kk_integer_eq_borrow(_borrowedArg_19882,_borrowedArg_19881,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19882, _ctx);
  kk_integer_drop(_borrowedArg_19881, _ctx);
  return _borrowApp_19883;
}
 
// Show a character as a string

kk_string_t kk_std_core_show_char(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> string */ 
  bool _match_19853 = (c < (' ')); /*bool*/;
  if (_match_19853) {
    bool _match_19869 = (c == 0x000A); /*bool*/;
    if (_match_19869) {
      kk_define_string_literal(, _s20581, 2, "\\n")
      return kk_string_dup(_s20581);
    }
    bool _match_19870 = (c == 0x000D); /*bool*/;
    if (_match_19870) {
      kk_define_string_literal(, _s20582, 2, "\\r")
      return kk_string_dup(_s20582);
    }
    bool _match_19871 = (c == 0x0009); /*bool*/;
    if (_match_19871) {
      kk_define_string_literal(, _s20583, 2, "\\t")
      return kk_string_dup(_s20583);
    }
    bool _match_19872;
    kk_integer_t _borrowedArg_19878 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_integer_t _borrowedArg_19877 = kk_integer_from_small(255); /*int*/;
    bool _borrowApp_19879 = kk_integer_lte_borrow(_borrowedArg_19878,_borrowedArg_19877,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_19878, _ctx);
    kk_integer_drop(_borrowedArg_19877, _ctx);
    _match_19872 = _borrowApp_19879; /*bool*/
    if (_match_19872) {
      kk_string_t _x20584;
      kk_define_string_literal(, _s20585, 2, "\\x")
      _x20584 = kk_string_dup(_s20585); /*string*/
      kk_string_t _x20586;
      kk_integer_t _arg_7841 = kk_integer_from_int(c,kk_context()); /*int*/;
      kk_std_core_types__optional _x20587;
      kk_integer_t _b_18696_18684 = kk_integer_from_small(2); /*int*/;
      _x20587 = kk_std_core_types__new_Optional(kk_integer_box(_b_18696_18684), _ctx); /*optional<106>*/
      kk_std_core_types__optional _x20588;
      kk_string_t _b_18697_18685 = kk_string_empty(); /*string*/;
      _x20588 = kk_std_core_types__new_Optional(kk_string_box(_b_18697_18685), _ctx); /*optional<106>*/
      _x20586 = kk_std_core_show_hex(_arg_7841, _x20587, kk_std_core_types__new_None(_ctx), _x20588, _ctx); /*string*/
      return kk_std_core__lp__plus__plus__1_rp_(_x20584, _x20586, _ctx);
    }
    bool _match_19873;
    kk_integer_t _borrowedArg_19875 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_integer_t _borrowedArg_19874 = kk_integer_from_int(65535, _ctx); /*int*/;
    bool _borrowApp_19876 = kk_integer_lte_borrow(_borrowedArg_19875,_borrowedArg_19874,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_19875, _ctx);
    kk_integer_drop(_borrowedArg_19874, _ctx);
    _match_19873 = _borrowApp_19876; /*bool*/
    if (_match_19873) {
      kk_string_t _x20590;
      kk_define_string_literal(, _s20591, 2, "\\u")
      _x20590 = kk_string_dup(_s20591); /*string*/
      kk_string_t _x20592;
      kk_integer_t _arg_7920 = kk_integer_from_int(c,kk_context()); /*int*/;
      kk_std_core_types__optional _x20593;
      kk_integer_t _b_18698_18686 = kk_integer_from_small(4); /*int*/;
      _x20593 = kk_std_core_types__new_Optional(kk_integer_box(_b_18698_18686), _ctx); /*optional<106>*/
      kk_std_core_types__optional _x20594;
      kk_string_t _b_18699_18687 = kk_string_empty(); /*string*/;
      _x20594 = kk_std_core_types__new_Optional(kk_string_box(_b_18699_18687), _ctx); /*optional<106>*/
      _x20592 = kk_std_core_show_hex(_arg_7920, _x20593, kk_std_core_types__new_None(_ctx), _x20594, _ctx); /*string*/
      return kk_std_core__lp__plus__plus__1_rp_(_x20590, _x20592, _ctx);
    }
    kk_string_t _x20596;
    kk_define_string_literal(, _s20597, 2, "\\U")
    _x20596 = kk_string_dup(_s20597); /*string*/
    kk_string_t _x20598;
    kk_integer_t _arg_7961 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_std_core_types__optional _x20599;
    kk_integer_t _b_18700_18688 = kk_integer_from_small(6); /*int*/;
    _x20599 = kk_std_core_types__new_Optional(kk_integer_box(_b_18700_18688), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x20600;
    kk_string_t _b_18701_18689 = kk_string_empty(); /*string*/;
    _x20600 = kk_std_core_types__new_Optional(kk_string_box(_b_18701_18689), _ctx); /*optional<106>*/
    _x20598 = kk_std_core_show_hex(_arg_7961, _x20599, kk_std_core_types__new_None(_ctx), _x20600, _ctx); /*string*/
    return kk_std_core__lp__plus__plus__1_rp_(_x20596, _x20598, _ctx);
  }
  bool _match_19854 = (c > ('~')); /*bool*/;
  if (_match_19854) {
    bool _match_19858 = (c == 0x000A); /*bool*/;
    if (_match_19858) {
      kk_define_string_literal(, _s20602, 2, "\\n")
      return kk_string_dup(_s20602);
    }
    bool _match_19859 = (c == 0x000D); /*bool*/;
    if (_match_19859) {
      kk_define_string_literal(, _s20603, 2, "\\r")
      return kk_string_dup(_s20603);
    }
    bool _match_19860 = (c == 0x0009); /*bool*/;
    if (_match_19860) {
      kk_define_string_literal(, _s20604, 2, "\\t")
      return kk_string_dup(_s20604);
    }
    bool _match_19861;
    kk_integer_t _borrowedArg_19867 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_integer_t _borrowedArg_19866 = kk_integer_from_small(255); /*int*/;
    bool _borrowApp_19868 = kk_integer_lte_borrow(_borrowedArg_19867,_borrowedArg_19866,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_19867, _ctx);
    kk_integer_drop(_borrowedArg_19866, _ctx);
    _match_19861 = _borrowApp_19868; /*bool*/
    if (_match_19861) {
      kk_string_t _x20605;
      kk_define_string_literal(, _s20606, 2, "\\x")
      _x20605 = kk_string_dup(_s20606); /*string*/
      kk_string_t _x20607;
      kk_integer_t _arg_78410 = kk_integer_from_int(c,kk_context()); /*int*/;
      kk_std_core_types__optional _x20608;
      kk_integer_t _b_18702_18690 = kk_integer_from_small(2); /*int*/;
      _x20608 = kk_std_core_types__new_Optional(kk_integer_box(_b_18702_18690), _ctx); /*optional<106>*/
      kk_std_core_types__optional _x20609;
      kk_string_t _b_18703_18691 = kk_string_empty(); /*string*/;
      _x20609 = kk_std_core_types__new_Optional(kk_string_box(_b_18703_18691), _ctx); /*optional<106>*/
      _x20607 = kk_std_core_show_hex(_arg_78410, _x20608, kk_std_core_types__new_None(_ctx), _x20609, _ctx); /*string*/
      return kk_std_core__lp__plus__plus__1_rp_(_x20605, _x20607, _ctx);
    }
    bool _match_19862;
    kk_integer_t _borrowedArg_19864 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_integer_t _borrowedArg_19863 = kk_integer_from_int(65535, _ctx); /*int*/;
    bool _borrowApp_19865 = kk_integer_lte_borrow(_borrowedArg_19864,_borrowedArg_19863,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_19864, _ctx);
    kk_integer_drop(_borrowedArg_19863, _ctx);
    _match_19862 = _borrowApp_19865; /*bool*/
    if (_match_19862) {
      kk_string_t _x20611;
      kk_define_string_literal(, _s20612, 2, "\\u")
      _x20611 = kk_string_dup(_s20612); /*string*/
      kk_string_t _x20613;
      kk_integer_t _arg_79200 = kk_integer_from_int(c,kk_context()); /*int*/;
      kk_std_core_types__optional _x20614;
      kk_integer_t _b_18704_18692 = kk_integer_from_small(4); /*int*/;
      _x20614 = kk_std_core_types__new_Optional(kk_integer_box(_b_18704_18692), _ctx); /*optional<106>*/
      kk_std_core_types__optional _x20615;
      kk_string_t _b_18705_18693 = kk_string_empty(); /*string*/;
      _x20615 = kk_std_core_types__new_Optional(kk_string_box(_b_18705_18693), _ctx); /*optional<106>*/
      _x20613 = kk_std_core_show_hex(_arg_79200, _x20614, kk_std_core_types__new_None(_ctx), _x20615, _ctx); /*string*/
      return kk_std_core__lp__plus__plus__1_rp_(_x20611, _x20613, _ctx);
    }
    kk_string_t _x20617;
    kk_define_string_literal(, _s20618, 2, "\\U")
    _x20617 = kk_string_dup(_s20618); /*string*/
    kk_string_t _x20619;
    kk_integer_t _arg_79610 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_std_core_types__optional _x20620;
    kk_integer_t _b_18706_18694 = kk_integer_from_small(6); /*int*/;
    _x20620 = kk_std_core_types__new_Optional(kk_integer_box(_b_18706_18694), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x20621;
    kk_string_t _b_18707_18695 = kk_string_empty(); /*string*/;
    _x20621 = kk_std_core_types__new_Optional(kk_string_box(_b_18707_18695), _ctx); /*optional<106>*/
    _x20619 = kk_std_core_show_hex(_arg_79610, _x20620, kk_std_core_types__new_None(_ctx), _x20621, _ctx); /*string*/
    return kk_std_core__lp__plus__plus__1_rp_(_x20617, _x20619, _ctx);
  }
  bool _match_19855 = (c == ('\'')); /*bool*/;
  if (_match_19855) {
    kk_define_string_literal(, _s20623, 2, "\\\'")
    return kk_string_dup(_s20623);
  }
  bool _match_19856 = (c == ('"')); /*bool*/;
  if (_match_19856) {
    kk_define_string_literal(, _s20624, 2, "\\\"")
    return kk_string_dup(_s20624);
  }
  bool _match_19857 = (c == ('\\')); /*bool*/;
  if (_match_19857) {
    kk_define_string_literal(, _s20625, 2, "\\\\")
    return kk_string_dup(_s20625);
  }
  return kk_std_core_string(c, _ctx);
}
 
// Show a `:double` fixed-point notation.
// The optional `precision` (= `-2`) specifies the maximum precision.
// If `>=0` it specifies the number of digits behind the dot (up to `20` max).
// If negative, then at most the absolute value of `precision` digits behind the dot are used.
// This may still show a number in exponential notation if the it is too small or large,
// in particular, for  a `d` where `d > 1.0e21` or `d < 1.0e-15`, or if
// `precision.abs > 17`, the `show-exp` routine is used.

kk_string_t kk_std_core_show_fixed(double d, kk_std_core_types__optional precision, kk_context_t* _ctx) { /* (d : double, precision : optional<int>) -> string */ 
  double dabs = kk_double_abs(d); /*double*/;
  bool _match_19851 = (dabs < (1.0e-15)); /*bool*/;
  if (_match_19851) {
    int32_t _x20630;
    kk_integer_t _x20631;
    if (kk_std_core_types__is_Optional(precision)) {
      kk_box_t _box_x18709 = precision._cons.Optional.value;
      kk_integer_t _precision_8050 = kk_integer_unbox(_box_x18709);
      kk_integer_dup(_precision_8050);
      kk_std_core_types__optional_drop(precision, _ctx);
      kk_Unit;
      _x20631 = _precision_8050; /*int*/
      goto _match20632;
    }
    _x20631 = kk_integer_from_small(-2); /*int*/
    _match20632: ;
    _x20630 = kk_std_core_int32(_x20631, _ctx); /*int32*/
    return kk_std_core_show_expx(d, _x20630, _ctx);
  }
  bool _match_19852 = (dabs > (1.0e21)); /*bool*/;
  if (_match_19852) {
    int32_t _x20634;
    kk_integer_t _x20635;
    if (kk_std_core_types__is_Optional(precision)) {
      kk_box_t _box_x18710 = precision._cons.Optional.value;
      kk_integer_t _precision_80500 = kk_integer_unbox(_box_x18710);
      kk_integer_dup(_precision_80500);
      kk_std_core_types__optional_drop(precision, _ctx);
      kk_Unit;
      _x20635 = _precision_80500; /*int*/
      goto _match20636;
    }
    _x20635 = kk_integer_from_small(-2); /*int*/
    _match20636: ;
    _x20634 = kk_std_core_int32(_x20635, _ctx); /*int32*/
    return kk_std_core_show_expx(d, _x20634, _ctx);
  }
  int32_t _x20638;
  kk_integer_t _x20639;
  if (kk_std_core_types__is_Optional(precision)) {
    kk_box_t _box_x18711 = precision._cons.Optional.value;
    kk_integer_t _precision_80501 = kk_integer_unbox(_box_x18711);
    kk_integer_dup(_precision_80501);
    kk_std_core_types__optional_drop(precision, _ctx);
    kk_Unit;
    _x20639 = _precision_80501; /*int*/
    goto _match20640;
  }
  _x20639 = kk_integer_from_small(-2); /*int*/
  _match20640: ;
  _x20638 = kk_std_core_int32(_x20639, _ctx); /*int32*/
  return kk_std_core_show_fixedx(d, _x20638, _ctx);
}
 
// lifted

kk_string_t kk_std_core__lift17075_show_list(kk_string_t sep, kk_std_core__list ys, kk_string_t acc, kk_context_t* _ctx) { /* (sep : string, ys : list<string>, acc : string) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20642 = kk_std_core__as_Cons(ys);
    kk_box_t _box_x18712 = _con20642->head;
    kk_std_core__list yy = _con20642->tail;
    kk_string_t y = kk_string_unbox(_box_x18712);
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_std_core__list_free(ys);
    }
    else {
      kk_string_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_string_t acc0_17115;
    kk_string_t _x20644;
    kk_string_t _x20645 = kk_string_dup(sep); /*string*/
    _x20644 = kk_std_core__lp__plus__plus__1_rp_(_x20645, y, _ctx); /*string*/
    acc0_17115 = kk_std_core__lp__plus__plus__1_rp_(acc, _x20644, _ctx); /*string*/
    { // tailcall
      ys = yy;
      acc = acc0_17115;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sep, _ctx);
  return acc;
}
 
// monadic lift

kk_string_t kk_std_core__mlift17538_show_list(kk_std_core__list _y_17303, kk_context_t* _ctx) { /* forall<e> (list<string>) -> e string */ 
  kk_string_t _x20646;
  kk_define_string_literal(, _s20647, 1, "[")
  _x20646 = kk_string_dup(_s20647); /*string*/
  kk_string_t _x20648;
  kk_string_t _x20649;
  if (kk_std_core__is_Nil(_y_17303)) {
    _x20649 = kk_string_empty(); /*string*/
  }
  else {
    struct kk_std_core_Cons* _con20651 = kk_std_core__as_Cons(_y_17303);
    kk_box_t _box_x18713 = _con20651->head;
    kk_std_core__list xx = _con20651->tail;
    kk_string_t x = kk_string_unbox(_box_x18713);
    if (kk_likely(kk_std_core__list_is_unique(_y_17303))) {
      kk_std_core__list_free(_y_17303);
    }
    else {
      kk_string_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(_y_17303, _ctx);
      kk_Unit;
    }
    kk_string_t _x20653;
    kk_define_string_literal(, _s20654, 1, ",")
    _x20653 = kk_string_dup(_s20654); /*string*/
    _x20649 = kk_std_core__lift17075_show_list(_x20653, xx, x, _ctx); /*string*/
  }
  kk_string_t _x20655;
  kk_define_string_literal(, _s20656, 1, "]")
  _x20655 = kk_string_dup(_s20656); /*string*/
  _x20648 = kk_std_core__lp__plus__plus__1_rp_(_x20649, _x20655, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x20646, _x20648, _ctx);
}
 
// Convert a list to a string


// lift anonymous function
struct kk_std_core_show_list_fun20657__t {
  struct kk_function_s _base;
  kk_function_t show_elem;
};
static kk_box_t kk_std_core_show_list_fun20657(kk_function_t _fself, kk_box_t _b_18716, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_show_list_fun20657(kk_function_t show_elem, kk_context_t* _ctx) {
  struct kk_std_core_show_list_fun20657__t* _self = kk_function_alloc_as(struct kk_std_core_show_list_fun20657__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_show_list_fun20657, kk_context());
  _self->show_elem = show_elem;
  return &_self->_base;
}

static kk_box_t kk_std_core_show_list_fun20657(kk_function_t _fself, kk_box_t _b_18716, kk_context_t* _ctx) {
  struct kk_std_core_show_list_fun20657__t* _self = kk_function_as(struct kk_std_core_show_list_fun20657__t*, _fself);
  kk_function_t show_elem = _self->show_elem; /* (8595) -> 8596 string */
  kk_drop_match(_self, {kk_function_dup(show_elem);}, {}, _ctx)
  kk_string_t _x20658 = kk_function_call(kk_string_t, (kk_function_t, kk_box_t, kk_context_t*), show_elem, (show_elem, _b_18716, _ctx)); /*string*/
  return kk_string_box(_x20658);
}


// lift anonymous function
struct kk_std_core_show_list_fun20659__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_core_show_list_fun20659(kk_function_t _fself, kk_std_core__list _y_17303, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_show_list_fun20659(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_show_list_fun20659, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_core_show_list_fun20659(kk_function_t _fself, kk_std_core__list _y_17303, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_string_t _x20660;
  kk_define_string_literal(, _s20661, 1, "[")
  _x20660 = kk_string_dup(_s20661); /*string*/
  kk_string_t _x20662;
  kk_string_t _x20663;
  if (kk_std_core__is_Nil(_y_17303)) {
    _x20663 = kk_string_empty(); /*string*/
  }
  else {
    struct kk_std_core_Cons* _con20665 = kk_std_core__as_Cons(_y_17303);
    kk_box_t _box_x18719 = _con20665->head;
    kk_std_core__list xx = _con20665->tail;
    kk_string_t x0 = kk_string_unbox(_box_x18719);
    if (kk_likely(kk_std_core__list_is_unique(_y_17303))) {
      kk_std_core__list_free(_y_17303);
    }
    else {
      kk_string_dup(x0);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(_y_17303, _ctx);
      kk_Unit;
    }
    kk_string_t _x20667;
    kk_define_string_literal(, _s20668, 1, ",")
    _x20667 = kk_string_dup(_s20668); /*string*/
    _x20663 = kk_std_core__lift17075_show_list(_x20667, xx, x0, _ctx); /*string*/
  }
  kk_string_t _x20669;
  kk_define_string_literal(, _s20670, 1, "]")
  _x20669 = kk_string_dup(_s20670); /*string*/
  _x20662 = kk_std_core__lp__plus__plus__1_rp_(_x20663, _x20669, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x20660, _x20662, _ctx);
}


// lift anonymous function
struct kk_std_core_show_list_fun20672__t {
  struct kk_function_s _base;
  kk_function_t _b_18723_18720;
};
static kk_box_t kk_std_core_show_list_fun20672(kk_function_t _fself, kk_box_t _b_18721, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_show_list_fun20672(kk_function_t _b_18723_18720, kk_context_t* _ctx) {
  struct kk_std_core_show_list_fun20672__t* _self = kk_function_alloc_as(struct kk_std_core_show_list_fun20672__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_show_list_fun20672, kk_context());
  _self->_b_18723_18720 = _b_18723_18720;
  return &_self->_base;
}

static kk_box_t kk_std_core_show_list_fun20672(kk_function_t _fself, kk_box_t _b_18721, kk_context_t* _ctx) {
  struct kk_std_core_show_list_fun20672__t* _self = kk_function_as(struct kk_std_core_show_list_fun20672__t*, _fself);
  kk_function_t _b_18723_18720 = _self->_b_18723_18720; /* (list<string>) -> 8596 string */
  kk_drop_match(_self, {kk_function_dup(_b_18723_18720);}, {}, _ctx)
  kk_string_t _x20673;
  kk_std_core__list _x20674 = kk_std_core__list_unbox(_b_18721, _ctx); /*list<string>*/
  _x20673 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _b_18723_18720, (_b_18723_18720, _x20674, _ctx)); /*string*/
  return kk_string_box(_x20673);
}

kk_string_t kk_std_core_show_list(kk_std_core__list xs, kk_function_t show_elem, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, show-elem : (a) -> e string) -> e string */ 
  kk_std_core__list x_17803 = kk_std_core_map_5(xs, kk_std_core_new_show_list_fun20657(show_elem, _ctx), _ctx); /*list<string>*/;
  if (kk_yielding(kk_context())) {
    kk_std_core__list_drop(x_17803, _ctx);
    kk_function_t _b_18723_18720 = kk_std_core_new_show_list_fun20659(_ctx); /*(list<string>) -> 8596 string*/;
    kk_box_t _x20671 = kk_std_core_hnd_yield_extend(kk_std_core_new_show_list_fun20672(_b_18723_18720, _ctx), _ctx); /*3858*/
    return kk_string_unbox(_x20671);
  }
  kk_string_t _x20675;
  kk_define_string_literal(, _s20676, 1, "[")
  _x20675 = kk_string_dup(_s20676); /*string*/
  kk_string_t _x20677;
  kk_string_t _x20678;
  if (kk_std_core__is_Nil(x_17803)) {
    _x20678 = kk_string_empty(); /*string*/
  }
  else {
    struct kk_std_core_Cons* _con20680 = kk_std_core__as_Cons(x_17803);
    kk_box_t _box_x18722 = _con20680->head;
    kk_std_core__list xx0 = _con20680->tail;
    kk_string_t x1 = kk_string_unbox(_box_x18722);
    if (kk_likely(kk_std_core__list_is_unique(x_17803))) {
      kk_std_core__list_free(x_17803);
    }
    else {
      kk_string_dup(x1);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(x_17803, _ctx);
      kk_Unit;
    }
    kk_string_t _x20682;
    kk_define_string_literal(, _s20683, 1, ",")
    _x20682 = kk_string_dup(_s20683); /*string*/
    _x20678 = kk_std_core__lift17075_show_list(_x20682, xx0, x1, _ctx); /*string*/
  }
  kk_string_t _x20684;
  kk_define_string_literal(, _s20685, 1, "]")
  _x20684 = kk_string_dup(_s20685); /*string*/
  _x20677 = kk_std_core__lp__plus__plus__1_rp_(_x20678, _x20684, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x20675, _x20677, _ctx);
}
 
// Convert an `:int` to a string

kk_string_t kk_std_core_show(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> string */ 
  return kk_integer_to_string(i,kk_context());
}
 
// Show a `:double` as a string.
// If `d >= 1.0e-5` and `d < 1.0e+21`, `show-fixed` is used and otherwise `show-exp`.
// Default `precision` is `-17`.

kk_string_t kk_std_core_show_1(double d, kk_std_core_types__optional precision, kk_context_t* _ctx) { /* (d : double, precision : optional<int>) -> string */ 
  double dabs = kk_double_abs(d); /*double*/;
  bool _match_19848 = (dabs >= (1.0e-5)); /*bool*/;
  if (_match_19848) {
    bool _match_19849 = (dabs < (1.0e21)); /*bool*/;
    if (_match_19849) {
      kk_std_core_types__optional _x20686;
      kk_integer_t _b_18728_18725;
      if (kk_std_core_types__is_Optional(precision)) {
        kk_box_t _box_x18724 = precision._cons.Optional.value;
        kk_integer_t _precision_8605 = kk_integer_unbox(_box_x18724);
        kk_integer_dup(_precision_8605);
        kk_std_core_types__optional_drop(precision, _ctx);
        kk_Unit;
        _b_18728_18725 = _precision_8605; /*int*/
        goto _match20687;
      }
      _b_18728_18725 = kk_integer_from_small(-17); /*int*/
      _match20687: ;
      _x20686 = kk_std_core_types__new_Optional(kk_integer_box(_b_18728_18725), _ctx); /*optional<106>*/
      return kk_std_core_show_fixed(d, _x20686, _ctx);
    }
    int32_t _x20689;
    kk_integer_t _x20690;
    if (kk_std_core_types__is_Optional(precision)) {
      kk_box_t _box_x18726 = precision._cons.Optional.value;
      kk_integer_t _precision_86050 = kk_integer_unbox(_box_x18726);
      kk_integer_dup(_precision_86050);
      kk_std_core_types__optional_drop(precision, _ctx);
      kk_Unit;
      _x20690 = _precision_86050; /*int*/
      goto _match20691;
    }
    _x20690 = kk_integer_from_small(-17); /*int*/
    _match20691: ;
    _x20689 = kk_std_core_int32(_x20690, _ctx); /*int32*/
    return kk_std_core_show_expx(d, _x20689, _ctx);
  }
  int32_t _x20693;
  kk_integer_t _x20694;
  if (kk_std_core_types__is_Optional(precision)) {
    kk_box_t _box_x18727 = precision._cons.Optional.value;
    kk_integer_t _precision_86051 = kk_integer_unbox(_box_x18727);
    kk_integer_dup(_precision_86051);
    kk_std_core_types__optional_drop(precision, _ctx);
    kk_Unit;
    _x20694 = _precision_86051; /*int*/
    goto _match20695;
  }
  _x20694 = kk_integer_from_small(-17); /*int*/
  _match20695: ;
  _x20693 = kk_std_core_int32(_x20694, _ctx); /*int32*/
  return kk_std_core_show_expx(d, _x20693, _ctx);
}
 
// lifted

kk_string_t kk_std_core__lift17076_show_3(kk_string_t sep_16971, kk_std_core__list ys, kk_string_t acc, kk_context_t* _ctx) { /* (sep.16971 : string, ys : list<string>, acc : string) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20703 = kk_std_core__as_Cons(ys);
    kk_box_t _box_x18729 = _con20703->head;
    kk_std_core__list yy = _con20703->tail;
    kk_string_t y = kk_string_unbox(_box_x18729);
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_std_core__list_free(ys);
    }
    else {
      kk_string_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_string_t acc0_17119;
    kk_string_t _x20705;
    kk_string_t _x20706 = kk_string_dup(sep_16971); /*string*/
    _x20705 = kk_std_core__lp__plus__plus__1_rp_(_x20706, y, _ctx); /*string*/
    acc0_17119 = kk_std_core__lp__plus__plus__1_rp_(acc, _x20705, _ctx); /*string*/
    { // tailcall
      ys = yy;
      acc = acc0_17119;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sep_16971, _ctx);
  return acc;
}
 
// Show a string as a string literal


// lift anonymous function
struct kk_std_core_show_fun20711__t_3 {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_show_fun20711_3(kk_function_t _fself, kk_box_t _b_18732, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_show_fun20711_3(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_show_fun20711_3, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_show_fun20711_3(kk_function_t _fself, kk_box_t _b_18732, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_string_t _x20712;
  kk_char_t _x20713 = kk_char_unbox(_b_18732, _ctx); /*char*/
  _x20712 = kk_std_core_show_char(_x20713, _ctx); /*string*/
  return kk_string_box(_x20712);
}

kk_string_t kk_std_core_show_3(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  kk_string_t _x20707;
  kk_define_string_literal(, _s20708, 1, "\"")
  _x20707 = kk_string_dup(_s20708); /*string*/
  kk_string_t _x20709;
  kk_string_t _x20710;
  kk_std_core__list _b_18733_18730 = kk_std_core_list_6(s, _ctx); /*list<char>*/;
  kk_std_core__list xs_16969 = kk_std_core_map_5(_b_18733_18730, kk_std_core_new_show_fun20711_3(_ctx), _ctx); /*list<string>*/;
  if (kk_std_core__is_Nil(xs_16969)) {
    _x20710 = kk_string_empty(); /*string*/
  }
  else {
    struct kk_std_core_Cons* _con20715 = kk_std_core__as_Cons(xs_16969);
    kk_box_t _box_x18735 = _con20715->head;
    kk_std_core__list xx = _con20715->tail;
    kk_string_t x = kk_string_unbox(_box_x18735);
    if (kk_likely(kk_std_core__list_is_unique(xs_16969))) {
      kk_std_core__list_free(xs_16969);
    }
    else {
      kk_string_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs_16969, _ctx);
      kk_Unit;
    }
    kk_string_t _x20717 = kk_string_empty(); /*string*/
    _x20710 = kk_std_core__lift17076_show_3(_x20717, xx, x, _ctx); /*string*/
  }
  kk_string_t _x20719;
  kk_define_string_literal(, _s20720, 1, "\"")
  _x20719 = kk_string_dup(_s20720); /*string*/
  _x20709 = kk_std_core__lp__plus__plus__1_rp_(_x20710, _x20719, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x20707, _x20709, _ctx);
}
extern kk_string_t kk_std_core_show_fun20726_7(kk_function_t _fself, kk_string_t _x120725, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_show_3(_x120725, _ctx);
}
extern kk_string_t kk_std_core_show_fun20727_7(kk_function_t _fself, kk_box_t _b_18738, kk_context_t* _ctx) {
  struct kk_std_core_show_fun20727__t_7* _self = kk_function_as(struct kk_std_core_show_fun20727__t_7*, _fself);
  kk_function_t _b_18740_18737 = _self->_b_18740_18737; /* (s : string) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_18740_18737);}, {}, _ctx)
  kk_string_t _x20728 = kk_string_unbox(_b_18738); /*string*/
  return kk_function_call(kk_string_t, (kk_function_t, kk_string_t, kk_context_t*), _b_18740_18737, (_b_18740_18737, _x20728, _ctx));
}
extern kk_string_t kk_std_core_show_fun20730_8(kk_function_t _fself, kk_integer_t _x120729, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_show(_x120729, _ctx);
}
extern kk_string_t kk_std_core_show_fun20731_8(kk_function_t _fself, kk_box_t _b_18743, kk_context_t* _ctx) {
  struct kk_std_core_show_fun20731__t_8* _self = kk_function_as(struct kk_std_core_show_fun20731__t_8*, _fself);
  kk_function_t _b_18745_18742 = _self->_b_18745_18742; /* (i : int) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_18745_18742);}, {}, _ctx)
  kk_integer_t _x20732 = kk_integer_unbox(_b_18743); /*int*/
  return kk_function_call(kk_string_t, (kk_function_t, kk_integer_t, kk_context_t*), _b_18745_18742, (_b_18745_18742, _x20732, _ctx));
}
extern kk_string_t kk_std_core_show_fun20734_9(kk_function_t _fself, bool _x120733, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_show_4(_x120733, _ctx);
}
extern kk_string_t kk_std_core_show_fun20735_9(kk_function_t _fself, kk_box_t _b_18748, kk_context_t* _ctx) {
  struct kk_std_core_show_fun20735__t_9* _self = kk_function_as(struct kk_std_core_show_fun20735__t_9*, _fself);
  kk_function_t _b_18750_18747 = _self->_b_18750_18747; /* (b : bool) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_18750_18747);}, {}, _ctx)
  bool _x20736 = kk_bool_unbox(_b_18748); /*bool*/
  return kk_function_call(kk_string_t, (kk_function_t, bool, kk_context_t*), _b_18750_18747, (_b_18750_18747, _x20736, _ctx));
}

kk_unit_t kk_std_core_prints(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> console () */ 
  kk_std_core_types__maybe _match_19847;
  kk_ref_t _b_18759_18751 = kk_ref_dup(kk_std_core_redirect); /*ref<global,maybe<(string) -> console ()>>*/;
  kk_box_t _x20745 = kk_ref_get(_b_18759_18751,kk_context()); /*182*/
  _match_19847 = kk_std_core_types__maybe_unbox(_x20745, _ctx); /*maybe<(string) -> console ()>*/
  if (kk_std_core_types__is_Nothing(_match_19847)) {
    kk_std_core_xprints(s, _ctx); return kk_Unit;
  }
  kk_box_t _fun_unbox_x18755 = _match_19847._cons.Just.value;
  kk_string_t _b_18758 = s; /*string*/;
  kk_box_t _x20746;
  kk_function_t _x20747 = kk_function_unbox(_fun_unbox_x18755); /*(18756) -> console 18757*/
  _x20746 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20747, (_x20747, kk_string_box(_b_18758), _ctx)); /*18757*/
  kk_unit_unbox(_x20746); return kk_Unit;
}


// lift anonymous function
struct kk_std_core__default_exn_fun20752__t {
  struct kk_function_s _base;
};
static kk_unit_t kk_std_core__default_exn_fun20752(kk_function_t _fself, kk_std_core__exception exn0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_default_exn_fun20752(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core__default_exn_fun20752, _ctx)
  return kk_function_dup(_fself);
}

static kk_unit_t kk_std_core__default_exn_fun20752(kk_function_t _fself, kk_std_core__exception exn0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_unit_t __ = kk_Unit;
  kk_string_t _x20753;
  kk_define_string_literal(, _s20754, 20, "uncaught exception: ")
  _x20753 = kk_string_dup(_s20754); /*string*/
  kk_std_core_prints(_x20753, _ctx);
  kk_string_t _x20755;
  kk_string_t _x = exn0.message;
  kk_string_dup(_x);
  kk_std_core__exception_drop(exn0, _ctx);
  kk_Unit;
  _x20755 = _x; /*string*/
  return kk_std_core_printsln(_x20755, _ctx);
}


// lift anonymous function
struct kk_std_core__default_exn_fun20756__t {
  struct kk_function_s _base;
  kk_function_t _b_18779_18768;
};
static kk_box_t kk_std_core__default_exn_fun20756(kk_function_t _fself, kk_box_t _b_18769, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_default_exn_fun20756(kk_function_t _b_18779_18768, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun20756__t* _self = kk_function_alloc_as(struct kk_std_core__default_exn_fun20756__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__default_exn_fun20756, kk_context());
  _self->_b_18779_18768 = _b_18779_18768;
  return &_self->_base;
}

static kk_box_t kk_std_core__default_exn_fun20756(kk_function_t _fself, kk_box_t _b_18769, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun20756__t* _self = kk_function_as(struct kk_std_core__default_exn_fun20756__t*, _fself);
  kk_function_t _b_18779_18768 = _self->_b_18779_18768; /* (exn0 : exception) -> <console|9657> () */
  kk_drop_match(_self, {kk_function_dup(_b_18779_18768);}, {}, _ctx)
  kk_unit_t _x20757 = kk_Unit;
  kk_std_core__exception _x20758 = kk_std_core__exception_unbox(_b_18769, _ctx); /*exception*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_std_core__exception, kk_context_t*), _b_18779_18768, (_b_18779_18768, _x20758, _ctx));
  return kk_unit_box(_x20757);
}


// lift anonymous function
struct kk_std_core__default_exn_fun20759__t {
  struct kk_function_s _base;
};
static kk_unit_t kk_std_core__default_exn_fun20759(kk_function_t _fself, kk_unit_t _x0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_default_exn_fun20759(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core__default_exn_fun20759, _ctx)
  return kk_function_dup(_fself);
}

static kk_unit_t kk_std_core__default_exn_fun20759(kk_function_t _fself, kk_unit_t _x0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _x0;
}


// lift anonymous function
struct kk_std_core__default_exn_fun20761__t {
  struct kk_function_s _base;
  kk_function_t _b_18777_18772;
};
static kk_box_t kk_std_core__default_exn_fun20761(kk_function_t _fself, kk_box_t _b_18774, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_default_exn_fun20761(kk_function_t _b_18777_18772, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun20761__t* _self = kk_function_alloc_as(struct kk_std_core__default_exn_fun20761__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__default_exn_fun20761, kk_context());
  _self->_b_18777_18772 = _b_18777_18772;
  return &_self->_base;
}

static kk_box_t kk_std_core__default_exn_fun20761(kk_function_t _fself, kk_box_t _b_18774, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun20761__t* _self = kk_function_as(struct kk_std_core__default_exn_fun20761__t*, _fself);
  kk_function_t _b_18777_18772 = _self->_b_18777_18772; /* (()) -> <console|9657> () */
  kk_drop_match(_self, {kk_function_dup(_b_18777_18772);}, {}, _ctx)
  kk_unit_t _x20762 = kk_Unit;
  kk_unit_t _x20763 = kk_Unit;
  kk_unit_unbox(_b_18774);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_18777_18772, (_b_18777_18772, _x20763, _ctx));
  return kk_unit_box(_x20762);
}


// lift anonymous function
struct kk_std_core__default_exn_fun20764__t {
  struct kk_function_s _base;
  kk_function_t _b_18778_18773;
};
static kk_box_t kk_std_core__default_exn_fun20764(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_default_exn_fun20764(kk_function_t _b_18778_18773, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun20764__t* _self = kk_function_alloc_as(struct kk_std_core__default_exn_fun20764__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__default_exn_fun20764, kk_context());
  _self->_b_18778_18773 = _b_18778_18773;
  return &_self->_base;
}

static kk_box_t kk_std_core__default_exn_fun20764(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun20764__t* _self = kk_function_as(struct kk_std_core__default_exn_fun20764__t*, _fself);
  kk_function_t _b_18778_18773 = _self->_b_18778_18773; /* () -> <exn,console|9657> () */
  kk_drop_match(_self, {kk_function_dup(_b_18778_18773);}, {}, _ctx)
  kk_unit_t _x20765 = kk_Unit;
  kk_function_call(kk_unit_t, (kk_function_t, kk_context_t*), _b_18778_18773, (_b_18778_18773, _ctx));
  return kk_unit_box(_x20765);
}

kk_unit_t kk_std_core__default_exn(kk_function_t action, kk_context_t* _ctx) { /* forall<e> (action : () -> <console,exn|e> ()) -> <console|e> () */ 
  int32_t _b_18775_18770 = ((int32_t)0); /*int32*/;
  kk_std_core__hnd_exn _b_18776_18771;
  kk_std_core_hnd__clause1 _x20751;
  kk_function_t _b_18779_18768 = kk_std_core__new_default_exn_fun20752(_ctx); /*(exn0 : exception) -> <console|9657> ()*/;
  _x20751 = kk_std_core_hnd_clause_never1(kk_std_core__new_default_exn_fun20756(_b_18779_18768, _ctx), _ctx); /*std/core/hnd/clause1<8108,8109,8111,8110,8112>*/
  _b_18776_18771 = kk_std_core__new_Hnd_exn(kk_reuse_null, _x20751, _ctx); /*.hnd-exn<<console|9657>,()>*/
  kk_function_t _b_18777_18772 = kk_std_core__new_default_exn_fun20759(_ctx); /*(()) -> <console|9657> ()*/;
  kk_function_t _b_18778_18773 = action; /*() -> <exn,console|9657> ()*/;
  kk_box_t _x20760 = kk_std_core__handle_exn(_b_18775_18770, _b_18776_18771, kk_std_core__new_default_exn_fun20761(_b_18777_18772, _ctx), kk_std_core__new_default_exn_fun20764(_b_18778_18773, _ctx), _ctx); /*1957*/
  kk_unit_unbox(_x20760); return kk_Unit;
}
 
// Get (zero-based) element `n`  of a list. Return a `:maybe` type.

kk_std_core_types__maybe kk_std_core__lp__lb__rb__2_rp_(kk_std_core__list xs, kk_integer_t n, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int) -> maybe<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20766 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con20766->head;
    kk_std_core__list xx = _con20766->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    bool _match_19839;
    kk_integer_t _borrowedArg_19843 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_19844 = kk_integer_gt_borrow(n,_borrowedArg_19843,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_19843, _ctx);
    _match_19839 = _borrowApp_19844; /*bool*/
    if (_match_19839) {
      kk_box_drop(x, _ctx);
      { // tailcall
        kk_integer_t _x20767 = kk_integer_sub(n,(kk_integer_from_small(1)),kk_context()); /*int*/
        xs = xx;
        n = _x20767;
        goto kk__tailcall;
      }
    }
    kk_std_core__list_drop(xx, _ctx);
    bool _match_19840;
    kk_integer_t _borrowedArg_19841 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_19842 = kk_integer_eq_borrow(n,_borrowedArg_19841,kk_context()); /*bool*/;
    kk_integer_drop(n, _ctx);
    kk_integer_drop(_borrowedArg_19841, _ctx);
    _match_19840 = _borrowApp_19842; /*bool*/
    if (_match_19840) {
      return kk_std_core_types__new_Just(x, _ctx);
    }
    kk_box_drop(x, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_integer_drop(n, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// O(1). Return the string slice from the end of `slice` argument
// to the end of the string.

kk_std_core__sslice kk_std_core_after(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> sslice */ 
  kk_string_t s = slice0.str;
  kk_ssize_t start0 = slice0.start;
  kk_ssize_t len0 = slice0.len;
  kk_string_dup(s);
  kk_std_core__sslice_drop(slice0, _ctx);
  kk_Unit;
  kk_string_t _x20768 = kk_string_dup(s); /*string*/
  kk_ssize_t _x20769 = (start0 + len0); /*ssize_t*/
  kk_ssize_t _x20770;
  kk_ssize_t _x20771 = kk_string_len(s,kk_context()); /*ssize_t*/
  kk_ssize_t _x20772 = (start0 + len0); /*ssize_t*/
  _x20770 = (_x20771 - _x20772); /*ssize_t*/
  return kk_std_core__new_Sslice(_x20768, _x20769, _x20770, _ctx);
}
 
// monadic lift

bool kk_std_core__mlift17539_all(kk_function_t predicate, kk_std_core__list xx, bool _y_17306, kk_context_t* _ctx) { /* forall<a,e> (predicate : (a) -> e bool, xx : list<a>, bool) -> e bool */ 
  if (_y_17306) {
    return kk_std_core_all(xx, predicate, _ctx);
  }
  kk_function_drop(predicate, _ctx);
  kk_std_core__list_drop(xx, _ctx);
  return false;
}
 
// Do all elements satisfy a predicate ?


// lift anonymous function
struct kk_std_core_all_fun20775__t {
  struct kk_function_s _base;
  kk_function_t predicate0;
  kk_std_core__list xx0;
};
static bool kk_std_core_all_fun20775(kk_function_t _fself, bool _y_173060, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_all_fun20775(kk_function_t predicate0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_all_fun20775__t* _self = kk_function_alloc_as(struct kk_std_core_all_fun20775__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_all_fun20775, kk_context());
  _self->predicate0 = predicate0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static bool kk_std_core_all_fun20775(kk_function_t _fself, bool _y_173060, kk_context_t* _ctx) {
  struct kk_std_core_all_fun20775__t* _self = kk_function_as(struct kk_std_core_all_fun20775__t*, _fself);
  kk_function_t predicate0 = _self->predicate0; /* (9894) -> 9895 bool */
  kk_std_core__list xx0 = _self->xx0; /* list<9894> */
  kk_drop_match(_self, {kk_function_dup(predicate0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17539_all(predicate0, xx0, _y_173060, _ctx);
}


// lift anonymous function
struct kk_std_core_all_fun20777__t {
  struct kk_function_s _base;
  kk_function_t _b_18782_18780;
};
static kk_box_t kk_std_core_all_fun20777(kk_function_t _fself, kk_box_t _b_18781, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_all_fun20777(kk_function_t _b_18782_18780, kk_context_t* _ctx) {
  struct kk_std_core_all_fun20777__t* _self = kk_function_alloc_as(struct kk_std_core_all_fun20777__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_all_fun20777, kk_context());
  _self->_b_18782_18780 = _b_18782_18780;
  return &_self->_base;
}

static kk_box_t kk_std_core_all_fun20777(kk_function_t _fself, kk_box_t _b_18781, kk_context_t* _ctx) {
  struct kk_std_core_all_fun20777__t* _self = kk_function_as(struct kk_std_core_all_fun20777__t*, _fself);
  kk_function_t _b_18782_18780 = _self->_b_18782_18780; /* (bool) -> 9895 bool */
  kk_drop_match(_self, {kk_function_dup(_b_18782_18780);}, {}, _ctx)
  bool _x20778;
  bool _x20779 = kk_bool_unbox(_b_18781); /*bool*/
  _x20778 = kk_function_call(bool, (kk_function_t, bool, kk_context_t*), _b_18782_18780, (_b_18782_18780, _x20779, _ctx)); /*bool*/
  return kk_bool_box(_x20778);
}

bool kk_std_core_all(kk_std_core__list xs, kk_function_t predicate0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e bool */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20773 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con20773->head;
    kk_std_core__list xx0 = _con20773->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    bool x0_17810;
    kk_function_t _x20774 = kk_function_dup(predicate0); /*(9894) -> 9895 bool*/
    x0_17810 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x20774, (_x20774, x, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_18782_18780 = kk_std_core_new_all_fun20775(predicate0, xx0, _ctx); /*(bool) -> 9895 bool*/;
      kk_box_t _x20776 = kk_std_core_hnd_yield_extend(kk_std_core_new_all_fun20777(_b_18782_18780, _ctx), _ctx); /*3858*/
      return kk_bool_unbox(_x20776);
    }
    if (x0_17810) {
      { // tailcall
        xs = xx0;
        goto kk__tailcall;
      }
    }
    kk_function_drop(predicate0, _ctx);
    kk_std_core__list_drop(xx0, _ctx);
    return false;
  }
  kk_function_drop(predicate0, _ctx);
  return true;
}
 
// monadic lift

bool kk_std_core__mlift17540_any(kk_function_t predicate, kk_std_core__list xx, bool _y_17310, kk_context_t* _ctx) { /* forall<a,e> (predicate : (a) -> e bool, xx : list<a>, bool) -> e bool */ 
  if (_y_17310) {
    kk_function_drop(predicate, _ctx);
    kk_std_core__list_drop(xx, _ctx);
    return true;
  }
  return kk_std_core_any(xx, predicate, _ctx);
}
 
// Are there any elements in a list that satisfy a predicate ?


// lift anonymous function
struct kk_std_core_any_fun20782__t {
  struct kk_function_s _base;
  kk_function_t predicate0;
  kk_std_core__list xx0;
};
static bool kk_std_core_any_fun20782(kk_function_t _fself, bool _y_173100, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_any_fun20782(kk_function_t predicate0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_any_fun20782__t* _self = kk_function_alloc_as(struct kk_std_core_any_fun20782__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_any_fun20782, kk_context());
  _self->predicate0 = predicate0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static bool kk_std_core_any_fun20782(kk_function_t _fself, bool _y_173100, kk_context_t* _ctx) {
  struct kk_std_core_any_fun20782__t* _self = kk_function_as(struct kk_std_core_any_fun20782__t*, _fself);
  kk_function_t predicate0 = _self->predicate0; /* (9922) -> 9923 bool */
  kk_std_core__list xx0 = _self->xx0; /* list<9922> */
  kk_drop_match(_self, {kk_function_dup(predicate0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17540_any(predicate0, xx0, _y_173100, _ctx);
}


// lift anonymous function
struct kk_std_core_any_fun20784__t {
  struct kk_function_s _base;
  kk_function_t _b_18785_18783;
};
static kk_box_t kk_std_core_any_fun20784(kk_function_t _fself, kk_box_t _b_18784, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_any_fun20784(kk_function_t _b_18785_18783, kk_context_t* _ctx) {
  struct kk_std_core_any_fun20784__t* _self = kk_function_alloc_as(struct kk_std_core_any_fun20784__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_any_fun20784, kk_context());
  _self->_b_18785_18783 = _b_18785_18783;
  return &_self->_base;
}

static kk_box_t kk_std_core_any_fun20784(kk_function_t _fself, kk_box_t _b_18784, kk_context_t* _ctx) {
  struct kk_std_core_any_fun20784__t* _self = kk_function_as(struct kk_std_core_any_fun20784__t*, _fself);
  kk_function_t _b_18785_18783 = _self->_b_18785_18783; /* (bool) -> 9923 bool */
  kk_drop_match(_self, {kk_function_dup(_b_18785_18783);}, {}, _ctx)
  bool _x20785;
  bool _x20786 = kk_bool_unbox(_b_18784); /*bool*/
  _x20785 = kk_function_call(bool, (kk_function_t, bool, kk_context_t*), _b_18785_18783, (_b_18785_18783, _x20786, _ctx)); /*bool*/
  return kk_bool_box(_x20785);
}

bool kk_std_core_any(kk_std_core__list xs, kk_function_t predicate0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e bool */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20780 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con20780->head;
    kk_std_core__list xx0 = _con20780->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    bool x0_17813;
    kk_function_t _x20781 = kk_function_dup(predicate0); /*(9922) -> 9923 bool*/
    x0_17813 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x20781, (_x20781, x, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_18785_18783 = kk_std_core_new_any_fun20782(predicate0, xx0, _ctx); /*(bool) -> 9923 bool*/;
      kk_box_t _x20783 = kk_std_core_hnd_yield_extend(kk_std_core_new_any_fun20784(_b_18785_18783, _ctx), _ctx); /*3858*/
      return kk_bool_unbox(_x20783);
    }
    if (x0_17813) {
      kk_function_drop(predicate0, _ctx);
      kk_std_core__list_drop(xx0, _ctx);
      return true;
    }
    { // tailcall
      xs = xx0;
      goto kk__tailcall;
    }
  }
  kk_function_drop(predicate0, _ctx);
  return false;
}
 
// Return the element at position `index` in vector `v`, or `Nothing` if out of bounds

kk_std_core_types__maybe kk_std_core_at(kk_vector_t v, kk_integer_t index, kk_context_t* _ctx) { /* forall<a> (v : vector<a>, index : int) -> maybe<a> */ 
  kk_ssize_t idx;
  kk_integer_t _x20787 = kk_integer_dup(index); /*int*/
  idx = kk_std_core_ssize__t(_x20787, _ctx); /*ssize_t*/
  bool _match_19836;
  kk_ssize_t _x20788 = kk_vector_len_borrow(v); /*ssize_t*/
  _match_19836 = (_x20788 <= idx); /*bool*/
  if (_match_19836) {
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_box_t _x20789 = kk_vector_at_borrow(v, idx); /*219*/
  return kk_std_core_types__new_Just(_x20789, _ctx);
}
 
// O(`n`). The first `n` (default = `1`) characters in a string.

kk_std_core__sslice kk_std_core_first(kk_string_t s, kk_std_core_types__optional n, kk_context_t* _ctx) { /* (s : string, n : optional<int>) -> sslice */ 
  kk_std_core__sslice slice0 = kk_std_core_first1(s, _ctx); /*sslice*/;
  bool _match_19828;
  kk_integer_t _borrowedArg_19832;
  if (kk_std_core_types__is_Optional(n)) {
    kk_box_t _box_x18786 = n._cons.Optional.value;
    kk_integer_t _n_10065 = kk_integer_unbox(_box_x18786);
    kk_integer_dup(_n_10065);
    _borrowedArg_19832 = _n_10065; /*int*/
    goto _match20793;
  }
  _borrowedArg_19832 = kk_integer_from_small(1); /*int*/
  _match20793: ;
  kk_integer_t _borrowedArg_19831 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19833 = kk_integer_eq_borrow(_borrowedArg_19832,_borrowedArg_19831,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19832, _ctx);
  kk_integer_drop(_borrowedArg_19831, _ctx);
  _match_19828 = _borrowApp_19833; /*bool*/
  if (_match_19828) {
    kk_std_core_types__optional_drop(n, _ctx);
    return slice0;
  }
  kk_integer_t _borrowedArg_19829;
  kk_integer_t _x20795;
  if (kk_std_core_types__is_Optional(n)) {
    kk_box_t _box_x18787 = n._cons.Optional.value;
    kk_integer_t _n_100650 = kk_integer_unbox(_box_x18787);
    kk_integer_dup(_n_100650);
    kk_std_core_types__optional_drop(n, _ctx);
    kk_Unit;
    _x20795 = _n_100650; /*int*/
    goto _match20796;
  }
  _x20795 = kk_integer_from_small(1); /*int*/
  _match20796: ;
  _borrowedArg_19829 = kk_integer_sub(_x20795,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_std_core__sslice _borrowApp_19830 = kk_std_core_extend(slice0, _borrowedArg_19829, _ctx); /*sslice*/;
  kk_integer_drop(_borrowedArg_19829, _ctx);
  return _borrowApp_19830;
}
 
// Convert the first character of a string to uppercase.

kk_string_t kk_std_core_capitalize(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  kk_string_t _x20798;
  kk_string_t _x20799;
  kk_std_core__sslice _x20800;
  kk_std_core__sslice slice0;
  kk_string_t _x20801 = kk_string_dup(s); /*string*/
  slice0 = kk_std_core_first1(_x20801, _ctx); /*sslice*/
  bool _match_19820;
  kk_integer_t _borrowedArg_19825;
  kk_std_core_types__optional _match_19826 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_19826)) {
    kk_box_t _box_x18788 = _match_19826._cons.Optional.value;
    kk_integer_t _n_10065 = kk_integer_unbox(_box_x18788);
    kk_integer_dup(_n_10065);
    kk_std_core_types__optional_drop(_match_19826, _ctx);
    kk_Unit;
    _borrowedArg_19825 = _n_10065; /*int*/
    goto _match20802;
  }
  _borrowedArg_19825 = kk_integer_from_small(1); /*int*/
  _match20802: ;
  kk_integer_t _borrowedArg_19824 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19827 = kk_integer_eq_borrow(_borrowedArg_19825,_borrowedArg_19824,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19825, _ctx);
  kk_integer_drop(_borrowedArg_19824, _ctx);
  _match_19820 = _borrowApp_19827; /*bool*/
  if (_match_19820) {
    _x20800 = slice0; /*sslice*/
  }
  else {
    kk_integer_t _borrowedArg_19821;
    kk_integer_t _x20804;
    kk_std_core_types__optional _match_19822 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_19822)) {
      kk_box_t _box_x18789 = _match_19822._cons.Optional.value;
      kk_integer_t _n_100650 = kk_integer_unbox(_box_x18789);
      kk_integer_dup(_n_100650);
      kk_std_core_types__optional_drop(_match_19822, _ctx);
      kk_Unit;
      _x20804 = _n_100650; /*int*/
      goto _match20805;
    }
    _x20804 = kk_integer_from_small(1); /*int*/
    _match20805: ;
    _borrowedArg_19821 = kk_integer_sub(_x20804,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__sslice _borrowApp_19823 = kk_std_core_extend(slice0, _borrowedArg_19821, _ctx); /*sslice*/;
    kk_integer_drop(_borrowedArg_19821, _ctx);
    _x20800 = _borrowApp_19823; /*sslice*/
  }
  _x20799 = kk_std_core_string_3(_x20800, _ctx); /*string*/
  _x20798 = kk_std_core_to_upper(_x20799, _ctx); /*string*/
  kk_string_t _x20807;
  kk_std_core__sslice _x20808;
  kk_std_core__sslice slice2 = kk_std_core_first1(s, _ctx); /*sslice*/;
  kk_std_core__sslice slice1_16978;
  bool _match_19812;
  kk_integer_t _borrowedArg_19817;
  kk_std_core_types__optional _match_19818 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_19818)) {
    kk_box_t _box_x18790 = _match_19818._cons.Optional.value;
    kk_integer_t _n_1006500 = kk_integer_unbox(_box_x18790);
    kk_integer_dup(_n_1006500);
    kk_std_core_types__optional_drop(_match_19818, _ctx);
    kk_Unit;
    _borrowedArg_19817 = _n_1006500; /*int*/
    goto _match20809;
  }
  _borrowedArg_19817 = kk_integer_from_small(1); /*int*/
  _match20809: ;
  kk_integer_t _borrowedArg_19816 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19819 = kk_integer_eq_borrow(_borrowedArg_19817,_borrowedArg_19816,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19817, _ctx);
  kk_integer_drop(_borrowedArg_19816, _ctx);
  _match_19812 = _borrowApp_19819; /*bool*/
  if (_match_19812) {
    slice1_16978 = slice2; /*sslice*/
  }
  else {
    kk_integer_t _borrowedArg_19813;
    kk_integer_t _x20811;
    kk_std_core_types__optional _match_19814 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_19814)) {
      kk_box_t _box_x18791 = _match_19814._cons.Optional.value;
      kk_integer_t _n_1006501 = kk_integer_unbox(_box_x18791);
      kk_integer_dup(_n_1006501);
      kk_std_core_types__optional_drop(_match_19814, _ctx);
      kk_Unit;
      _x20811 = _n_1006501; /*int*/
      goto _match20812;
    }
    _x20811 = kk_integer_from_small(1); /*int*/
    _match20812: ;
    _borrowedArg_19813 = kk_integer_sub(_x20811,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__sslice _borrowApp_19815 = kk_std_core_extend(slice2, _borrowedArg_19813, _ctx); /*sslice*/;
    kk_integer_drop(_borrowedArg_19813, _ctx);
    slice1_16978 = _borrowApp_19815; /*sslice*/
  }
  kk_string_t s1 = slice1_16978.str;
  kk_ssize_t start0 = slice1_16978.start;
  kk_ssize_t len0 = slice1_16978.len;
  kk_string_dup(s1);
  kk_std_core__sslice_drop(slice1_16978, _ctx);
  kk_Unit;
  kk_string_t _x20814 = kk_string_dup(s1); /*string*/
  kk_ssize_t _x20815 = (start0 + len0); /*ssize_t*/
  kk_ssize_t _x20816;
  kk_ssize_t _x20817 = kk_string_len(s1,kk_context()); /*ssize_t*/
  kk_ssize_t _x20818 = (start0 + len0); /*ssize_t*/
  _x20816 = (_x20817 - _x20818); /*ssize_t*/
  _x20808 = kk_std_core__new_Sslice(_x20814, _x20815, _x20816, _ctx); /*sslice*/
  _x20807 = kk_std_core_string_3(_x20808, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x20798, _x20807, _ctx);
}
 
// Catch any exception raised in `action` and handle it.
// Use `on-exn` or `on-exit` when appropiate.


// lift anonymous function
struct kk_std_core_try_fun20821__t {
  struct kk_function_s _base;
  kk_function_t _b_18794_18792;
};
static kk_box_t kk_std_core_try_fun20821(kk_function_t _fself, kk_box_t _b_18793, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun20821(kk_function_t _b_18794_18792, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20821__t* _self = kk_function_alloc_as(struct kk_std_core_try_fun20821__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun20821, kk_context());
  _self->_b_18794_18792 = _b_18794_18792;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_fun20821(kk_function_t _fself, kk_box_t _b_18793, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20821__t* _self = kk_function_as(struct kk_std_core_try_fun20821__t*, _fself);
  kk_function_t _b_18794_18792 = _self->_b_18794_18792; /* (exception) -> 10232 10231 */
  kk_drop_match(_self, {kk_function_dup(_b_18794_18792);}, {}, _ctx)
  kk_std_core__exception _x20822 = kk_std_core__exception_unbox(_b_18793, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core__exception, kk_context_t*), _b_18794_18792, (_b_18794_18792, _x20822, _ctx));
}


// lift anonymous function
struct kk_std_core_try_fun20823__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_try_fun20823(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun20823(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_try_fun20823, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_try_fun20823(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _x;
}

kk_box_t kk_std_core_try(kk_function_t action, kk_function_t hndl, kk_context_t* _ctx) { /* forall<a,e> (action : () -> <exn|e> a, hndl : (exception) -> e a) -> e a */ 
  kk_std_core__hnd_exn _x20819;
  kk_std_core_hnd__clause1 _x20820;
  kk_function_t _b_18794_18792 = hndl; /*(exception) -> 10232 10231*/;
  _x20820 = kk_std_core_hnd_clause_never1(kk_std_core_new_try_fun20821(_b_18794_18792, _ctx), _ctx); /*std/core/hnd/clause1<8108,8109,8111,8110,8112>*/
  _x20819 = kk_std_core__new_Hnd_exn(kk_reuse_null, _x20820, _ctx); /*.hnd-exn<11,12>*/
  return kk_std_core__handle_exn(((int32_t)0), _x20819, kk_std_core_new_try_fun20823(_ctx), action, _ctx);
}
 
// Transform an exception effect to an  `:error` type.


// lift anonymous function
struct kk_std_core_try_fun20825__t_1 {
  struct kk_function_s _base;
};
static kk_std_core__error kk_std_core_try_fun20825_1(kk_function_t _fself, kk_std_core__exception exn0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun20825_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_try_fun20825_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__error kk_std_core_try_fun20825_1(kk_function_t _fself, kk_std_core__exception exn0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core__new_Error(exn0, _ctx);
}


// lift anonymous function
struct kk_std_core_try_fun20826__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_18808_18795;
};
static kk_box_t kk_std_core_try_fun20826_1(kk_function_t _fself, kk_box_t _b_18796, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun20826_1(kk_function_t _b_18808_18795, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20826__t_1* _self = kk_function_alloc_as(struct kk_std_core_try_fun20826__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun20826_1, kk_context());
  _self->_b_18808_18795 = _b_18808_18795;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_fun20826_1(kk_function_t _fself, kk_box_t _b_18796, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20826__t_1* _self = kk_function_as(struct kk_std_core_try_fun20826__t_1*, _fself);
  kk_function_t _b_18808_18795 = _self->_b_18808_18795; /* (exn0 : exception) -> 10271 error<10270> */
  kk_drop_match(_self, {kk_function_dup(_b_18808_18795);}, {}, _ctx)
  kk_std_core__error _x20827;
  kk_std_core__exception _x20828 = kk_std_core__exception_unbox(_b_18796, _ctx); /*exception*/
  _x20827 = kk_function_call(kk_std_core__error, (kk_function_t, kk_std_core__exception, kk_context_t*), _b_18808_18795, (_b_18808_18795, _x20828, _ctx)); /*error<10270>*/
  return kk_std_core__error_box(_x20827, _ctx);
}


// lift anonymous function
struct kk_std_core_try_fun20829__t_1 {
  struct kk_function_s _base;
};
static kk_std_core__error kk_std_core_try_fun20829_1(kk_function_t _fself, kk_std_core__error _x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun20829_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_try_fun20829_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__error kk_std_core_try_fun20829_1(kk_function_t _fself, kk_std_core__error _x, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _x;
}


// lift anonymous function
struct kk_std_core_try_fun20830__t_1 {
  struct kk_function_s _base;
  kk_function_t action;
};
static kk_std_core__error kk_std_core_try_fun20830_1(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun20830_1(kk_function_t action, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20830__t_1* _self = kk_function_alloc_as(struct kk_std_core_try_fun20830__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun20830_1, kk_context());
  _self->action = action;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_core_try_fun20831__t_1 {
  struct kk_function_s _base;
};
static kk_std_core__error kk_std_core_try_fun20831_1(kk_function_t _fself, kk_box_t _y_17315, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun20831_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_try_fun20831_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__error kk_std_core_try_fun20831_1(kk_function_t _fself, kk_box_t _y_17315, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core__new_Ok(_y_17315, _ctx);
}


// lift anonymous function
struct kk_std_core_try_fun20833__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_18809_18797;
};
static kk_box_t kk_std_core_try_fun20833_1(kk_function_t _fself, kk_box_t _b_18798, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun20833_1(kk_function_t _b_18809_18797, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20833__t_1* _self = kk_function_alloc_as(struct kk_std_core_try_fun20833__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun20833_1, kk_context());
  _self->_b_18809_18797 = _b_18809_18797;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_fun20833_1(kk_function_t _fself, kk_box_t _b_18798, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20833__t_1* _self = kk_function_as(struct kk_std_core_try_fun20833__t_1*, _fself);
  kk_function_t _b_18809_18797 = _self->_b_18809_18797; /* (10270) -> <exn|10271> error<10270> */
  kk_drop_match(_self, {kk_function_dup(_b_18809_18797);}, {}, _ctx)
  kk_std_core__error _x20834 = kk_function_call(kk_std_core__error, (kk_function_t, kk_box_t, kk_context_t*), _b_18809_18797, (_b_18809_18797, _b_18798, _ctx)); /*error<10270>*/
  return kk_std_core__error_box(_x20834, _ctx);
}
static kk_std_core__error kk_std_core_try_fun20830_1(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20830__t_1* _self = kk_function_as(struct kk_std_core_try_fun20830__t_1*, _fself);
  kk_function_t action = _self->action; /* () -> <exn|10271> 10270 */
  kk_drop_match(_self, {kk_function_dup(action);}, {}, _ctx)
  kk_box_t x_17816 = kk_function_call(kk_box_t, (kk_function_t, kk_context_t*), action, (action, _ctx)); /*10270*/;
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17816, _ctx);
    kk_function_t _b_18809_18797 = kk_std_core_new_try_fun20831_1(_ctx); /*(10270) -> <exn|10271> error<10270>*/;
    kk_box_t _x20832 = kk_std_core_hnd_yield_extend(kk_std_core_new_try_fun20833_1(_b_18809_18797, _ctx), _ctx); /*3858*/
    return kk_std_core__error_unbox(_x20832, _ctx);
  }
  return kk_std_core__new_Ok(x_17816, _ctx);
}


// lift anonymous function
struct kk_std_core_try_fun20836__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_18806_18801;
};
static kk_box_t kk_std_core_try_fun20836_1(kk_function_t _fself, kk_box_t _b_18803, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun20836_1(kk_function_t _b_18806_18801, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20836__t_1* _self = kk_function_alloc_as(struct kk_std_core_try_fun20836__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun20836_1, kk_context());
  _self->_b_18806_18801 = _b_18806_18801;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_fun20836_1(kk_function_t _fself, kk_box_t _b_18803, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20836__t_1* _self = kk_function_as(struct kk_std_core_try_fun20836__t_1*, _fself);
  kk_function_t _b_18806_18801 = _self->_b_18806_18801; /* (error<10270>) -> 10271 error<10270> */
  kk_drop_match(_self, {kk_function_dup(_b_18806_18801);}, {}, _ctx)
  kk_std_core__error _x20837;
  kk_std_core__error _x20838 = kk_std_core__error_unbox(_b_18803, _ctx); /*error<10270>*/
  _x20837 = kk_function_call(kk_std_core__error, (kk_function_t, kk_std_core__error, kk_context_t*), _b_18806_18801, (_b_18806_18801, _x20838, _ctx)); /*error<10270>*/
  return kk_std_core__error_box(_x20837, _ctx);
}


// lift anonymous function
struct kk_std_core_try_fun20839__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_18807_18802;
};
static kk_box_t kk_std_core_try_fun20839_1(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun20839_1(kk_function_t _b_18807_18802, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20839__t_1* _self = kk_function_alloc_as(struct kk_std_core_try_fun20839__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun20839_1, kk_context());
  _self->_b_18807_18802 = _b_18807_18802;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_fun20839_1(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_core_try_fun20839__t_1* _self = kk_function_as(struct kk_std_core_try_fun20839__t_1*, _fself);
  kk_function_t _b_18807_18802 = _self->_b_18807_18802; /* () -> <exn|10271> error<10270> */
  kk_drop_match(_self, {kk_function_dup(_b_18807_18802);}, {}, _ctx)
  kk_std_core__error _x20840 = kk_function_call(kk_std_core__error, (kk_function_t, kk_context_t*), _b_18807_18802, (_b_18807_18802, _ctx)); /*error<10270>*/
  return kk_std_core__error_box(_x20840, _ctx);
}

kk_std_core__error kk_std_core_try_1(kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (action : () -> <exn|e> a) -> e error<a> */ 
  int32_t _b_18804_18799 = ((int32_t)0); /*int32*/;
  kk_std_core__hnd_exn _b_18805_18800;
  kk_std_core_hnd__clause1 _x20824;
  kk_function_t _b_18808_18795 = kk_std_core_new_try_fun20825_1(_ctx); /*(exn0 : exception) -> 10271 error<10270>*/;
  _x20824 = kk_std_core_hnd_clause_never1(kk_std_core_new_try_fun20826_1(_b_18808_18795, _ctx), _ctx); /*std/core/hnd/clause1<8108,8109,8111,8110,8112>*/
  _b_18805_18800 = kk_std_core__new_Hnd_exn(kk_reuse_null, _x20824, _ctx); /*.hnd-exn<10271,error<10270>>*/
  kk_function_t _b_18806_18801 = kk_std_core_new_try_fun20829_1(_ctx); /*(error<10270>) -> 10271 error<10270>*/;
  kk_function_t _b_18807_18802 = kk_std_core_new_try_fun20830_1(action, _ctx); /*() -> <exn|10271> error<10270>*/;
  kk_box_t _x20835 = kk_std_core__handle_exn(_b_18804_18799, _b_18805_18800, kk_std_core_new_try_fun20836_1(_b_18806_18801, _ctx), kk_std_core_new_try_fun20839_1(_b_18807_18802, _ctx), _ctx); /*1957*/
  return kk_std_core__error_unbox(_x20835, _ctx);
}
 
// _Deprecated_; use `try` instead. Catch an exception raised by `throw` and handle it.
// Use `on-exn` or `on-exit` when appropiate.


// lift anonymous function
struct kk_std_core_catch_fun20843__t {
  struct kk_function_s _base;
  kk_function_t _b_18812_18810;
};
static kk_box_t kk_std_core_catch_fun20843(kk_function_t _fself, kk_box_t _b_18811, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_catch_fun20843(kk_function_t _b_18812_18810, kk_context_t* _ctx) {
  struct kk_std_core_catch_fun20843__t* _self = kk_function_alloc_as(struct kk_std_core_catch_fun20843__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_catch_fun20843, kk_context());
  _self->_b_18812_18810 = _b_18812_18810;
  return &_self->_base;
}

static kk_box_t kk_std_core_catch_fun20843(kk_function_t _fself, kk_box_t _b_18811, kk_context_t* _ctx) {
  struct kk_std_core_catch_fun20843__t* _self = kk_function_as(struct kk_std_core_catch_fun20843__t*, _fself);
  kk_function_t _b_18812_18810 = _self->_b_18812_18810; /* (exception) -> 10323 10322 */
  kk_drop_match(_self, {kk_function_dup(_b_18812_18810);}, {}, _ctx)
  kk_std_core__exception _x20844 = kk_std_core__exception_unbox(_b_18811, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core__exception, kk_context_t*), _b_18812_18810, (_b_18812_18810, _x20844, _ctx));
}


// lift anonymous function
struct kk_std_core_catch_fun20845__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_catch_fun20845(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_catch_fun20845(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_catch_fun20845, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_catch_fun20845(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _x;
}

kk_box_t kk_std_core_catch(kk_function_t action, kk_function_t hndl, kk_context_t* _ctx) { /* forall<a,e> (action : () -> <exn|e> a, hndl : (exception) -> e a) -> e a */ 
  kk_std_core__hnd_exn _x20841;
  kk_std_core_hnd__clause1 _x20842;
  kk_function_t _b_18812_18810 = hndl; /*(exception) -> 10323 10322*/;
  _x20842 = kk_std_core_hnd_clause_never1(kk_std_core_new_catch_fun20843(_b_18812_18810, _ctx), _ctx); /*std/core/hnd/clause1<8108,8109,8111,8110,8112>*/
  _x20841 = kk_std_core__new_Hnd_exn(kk_reuse_null, _x20842, _ctx); /*.hnd-exn<11,12>*/
  return kk_std_core__handle_exn(((int32_t)0), _x20841, kk_std_core_new_catch_fun20845(_ctx), action, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core_cdivmod_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx) { /* (i : int, n : int) -> (int, int) */ 
  bool _match_19808;
  kk_integer_t _borrowedArg_19809 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19810 = kk_integer_lte_borrow(n,_borrowedArg_19809,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19809, _ctx);
  _match_19808 = _borrowApp_19810; /*bool*/
  if (_match_19808) {
    kk_integer_drop(n, _ctx);
    kk_integer_t _b_18817_18813 = i; /*int*/;
    kk_integer_t _b_18818_18814 = kk_integer_from_small(0); /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_18817_18813), kk_integer_box(_b_18818_18814), _ctx);
  }
  kk_integer_t cq;
  kk_integer_t _x20846 = kk_integer_dup(i); /*int*/
  kk_integer_t _x20847 = kk_integer_dup(n); /*int*/
  cq = kk_std_core_cdiv_exp10(_x20846, _x20847, _ctx); /*int*/
  kk_integer_t cr;
  kk_integer_t _x20848;
  kk_integer_t _x20849 = kk_integer_dup(cq); /*int*/
  _x20848 = kk_std_core_mul_exp10(_x20849, n, _ctx); /*int*/
  cr = kk_integer_sub(i,_x20848,kk_context()); /*int*/
  kk_integer_t _b_18819_18815 = cq; /*int*/;
  kk_integer_t _b_18820_18816 = cr; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_18819_18815), kk_integer_box(_b_18820_18816), _ctx);
}
 
// Concatenate a list of `:maybe` values

kk_std_core__list kk_std_core__ctail_concat_maybe(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (xs : list<maybe<a>>, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20850 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x18821 = _con20850->head;
    kk_std_core__list xx = _con20850->tail;
    kk_std_core_types__maybe x = kk_std_core_types__maybe_unbox(_box_x18821, NULL);
    kk_reuse_t _ru_20047 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core_types__maybe_dup(x);
      kk_box_drop(_box_x18821, _ctx);
      kk_Unit;
      kk_Unit;
      _ru_20047 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_std_core_types__maybe_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    if (kk_std_core_types__is_Just(x)) {
      kk_box_t y = x._cons.Just.value;
      kk_std_core__list _ctail_17182 = kk_std_core__list_hole(); /*list<10410>*/;
      kk_std_core__list _ctail_17183 = kk_std_core__new_Cons(_ru_20047, y, _ctail_17182, _ctx); /*list<10410>*/;
      { // tailcall
        kk_std_core_types__ctail _x20852;
        kk_box_t* field_17823 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17183)->tail)); /*cfield<list<10410>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x18828 = _acc._cons.CTail._field1;
          kk_box_t* last0 = _acc._cons.CTail._field2;
          kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18828, NULL);
          kk_std_core__list_dup(head0);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last0) = kk_std_core__list_box(_ctail_17183, _ctx);
          kk_std_core__list _b_18842_18833 = head0; /*list<10410>*/;
          kk_box_t* _b_18843_18834 = field_17823; /*cfield<list<10410>>*/;
          _x20852 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18842_18833, _ctx), _b_18843_18834, _ctx); /*ctail<58>*/
          goto _match20853;
        }
        kk_std_core__list _b_18844_18835 = _ctail_17183; /*list<10410>*/;
        kk_box_t* _b_18845_18836 = field_17823; /*cfield<list<10410>>*/;
        _x20852 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18844_18835, _ctx), _b_18845_18836, _ctx); /*ctail<58>*/
        _match20853: ;
        xs = xx;
        _acc = _x20852;
        goto kk__tailcall;
      }
    }
    kk_reuse_drop(_ru_20047);
    { // tailcall
      xs = xx;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18837 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18837, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Concatenate a list of `:maybe` values

kk_std_core__list kk_std_core_concat_maybe(kk_std_core__list xs0, kk_context_t* _ctx) { /* forall<a> (xs : list<maybe<a>>) -> list<a> */ 
  return kk_std_core__ctail_concat_maybe(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17542_op(kk_function_t action, kk_ssize_t end, kk_ssize_t i, kk_std_core_types__maybe _y_17318, kk_context_t* _ctx) { /* forall<a,e> (action : (ssize_t) -> e maybe<a>, end : ssize_t, i : ssize_t, maybe<a>) -> e maybe<a> */ 
  if (kk_std_core_types__is_Nothing(_y_17318)) {
    kk_ssize_t i0_17122 = kk_std_core_incr_1(i, _ctx); /*ssize_t*/;
    return kk_std_core__lift17077_for_whilez(action, end, i0_17122, _ctx);
  }
  kk_box_t x = _y_17318._cons.Just.value;
  kk_function_drop(action, _ctx);
  return kk_std_core_types__new_Just(x, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17077_for_whilez_fun20857__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_ssize_t end0;
  kk_ssize_t i0;
};
static kk_std_core_types__maybe kk_std_core__lift17077_for_whilez_fun20857(kk_function_t _fself, kk_std_core_types__maybe _y_173180, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17077_for_whilez_fun20857(kk_function_t action0, kk_ssize_t end0, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17077_for_whilez_fun20857__t* _self = kk_function_alloc_as(struct kk_std_core__lift17077_for_whilez_fun20857__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17077_for_whilez_fun20857, kk_context());
  _self->action0 = action0;
  _self->end0 = end0;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core__lift17077_for_whilez_fun20857(kk_function_t _fself, kk_std_core_types__maybe _y_173180, kk_context_t* _ctx) {
  struct kk_std_core__lift17077_for_whilez_fun20857__t* _self = kk_function_as(struct kk_std_core__lift17077_for_whilez_fun20857__t*, _fself);
  kk_function_t action0 = _self->action0; /* (ssize_t) -> 10465 maybe<10464> */
  kk_ssize_t end0 = _self->end0; /* ssize_t */
  kk_ssize_t i0 = _self->i0; /* ssize_t */
  kk_drop_match(_self, {kk_function_dup(action0);;;}, {}, _ctx)
  return kk_std_core__mlift17542_op(action0, end0, i0, _y_173180, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17077_for_whilez_fun20859__t {
  struct kk_function_s _base;
  kk_function_t _b_18848_18846;
};
static kk_box_t kk_std_core__lift17077_for_whilez_fun20859(kk_function_t _fself, kk_box_t _b_18847, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17077_for_whilez_fun20859(kk_function_t _b_18848_18846, kk_context_t* _ctx) {
  struct kk_std_core__lift17077_for_whilez_fun20859__t* _self = kk_function_alloc_as(struct kk_std_core__lift17077_for_whilez_fun20859__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17077_for_whilez_fun20859, kk_context());
  _self->_b_18848_18846 = _b_18848_18846;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17077_for_whilez_fun20859(kk_function_t _fself, kk_box_t _b_18847, kk_context_t* _ctx) {
  struct kk_std_core__lift17077_for_whilez_fun20859__t* _self = kk_function_as(struct kk_std_core__lift17077_for_whilez_fun20859__t*, _fself);
  kk_function_t _b_18848_18846 = _self->_b_18848_18846; /* (maybe<10464>) -> 10465 maybe<10464> */
  kk_drop_match(_self, {kk_function_dup(_b_18848_18846);}, {}, _ctx)
  kk_std_core_types__maybe _x20860;
  kk_std_core_types__maybe _x20861 = kk_std_core_types__maybe_unbox(_b_18847, _ctx); /*maybe<10464>*/
  _x20860 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_18848_18846, (_b_18848_18846, _x20861, _ctx)); /*maybe<10464>*/
  return kk_std_core_types__maybe_box(_x20860, _ctx);
}

kk_std_core_types__maybe kk_std_core__lift17077_for_whilez(kk_function_t action0, kk_ssize_t end0, kk_ssize_t i0, kk_context_t* _ctx) { /* forall<a,e> (action : (ssize_t) -> e maybe<a>, end : ssize_t, i : ssize_t) -> e maybe<a> */ 
  kk__tailcall: ;
  bool _match_19806 = (i0 <= end0); /*bool*/;
  if (_match_19806) {
    kk_std_core_types__maybe x0_17826;
    kk_function_t _x20856 = kk_function_dup(action0); /*(ssize_t) -> 10465 maybe<10464>*/
    x0_17826 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_ssize_t, kk_context_t*), _x20856, (_x20856, i0, _ctx)); /*maybe<10464>*/
    if (kk_yielding(kk_context())) {
      kk_std_core_types__maybe_drop(x0_17826, _ctx);
      kk_function_t _b_18848_18846 = kk_std_core__new_lift17077_for_whilez_fun20857(action0, end0, i0, _ctx); /*(maybe<10464>) -> 10465 maybe<10464>*/;
      kk_box_t _x20858 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17077_for_whilez_fun20859(_b_18848_18846, _ctx), _ctx); /*3858*/
      return kk_std_core_types__maybe_unbox(_x20858, _ctx);
    }
    if (kk_std_core_types__is_Nothing(x0_17826)) {
      kk_ssize_t i0_171220 = kk_std_core_incr_1(i0, _ctx); /*ssize_t*/;
      { // tailcall
        i0 = i0_171220;
        goto kk__tailcall;
      }
    }
    kk_box_t x1 = x0_17826._cons.Just.value;
    kk_function_drop(action0, _ctx);
    kk_Unit;
    return kk_std_core_types__new_Just(x1, _ctx);
  }
  kk_function_drop(action0, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17543_foreach_while(kk_function_t action, kk_std_core__list xx, kk_std_core_types__maybe _y_17323, kk_context_t* _ctx) { /* forall<a,b,e> (action : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e maybe<b> */ 
  if (kk_std_core_types__is_Nothing(_y_17323)) {
    return kk_std_core_foreach_while(xx, action, _ctx);
  }
  kk_function_drop(action, _ctx);
  kk_std_core__list_drop(xx, _ctx);
  return _y_17323;
}
 
// Invoke `action` for each element of a list while `action` return `Nothing`


// lift anonymous function
struct kk_std_core_foreach_while_fun20867__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_std_core__list xx0;
};
static kk_std_core_types__maybe kk_std_core_foreach_while_fun20867(kk_function_t _fself, kk_std_core_types__maybe _y_173230, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_while_fun20867(kk_function_t action0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun20867__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_while_fun20867__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_while_fun20867, kk_context());
  _self->action0 = action0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core_foreach_while_fun20867(kk_function_t _fself, kk_std_core_types__maybe _y_173230, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun20867__t* _self = kk_function_as(struct kk_std_core_foreach_while_fun20867__t*, _fself);
  kk_function_t action0 = _self->action0; /* (10598) -> 10600 maybe<10599> */
  kk_std_core__list xx0 = _self->xx0; /* list<10598> */
  kk_drop_match(_self, {kk_function_dup(action0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17543_foreach_while(action0, xx0, _y_173230, _ctx);
}


// lift anonymous function
struct kk_std_core_foreach_while_fun20869__t {
  struct kk_function_s _base;
  kk_function_t _b_18851_18849;
};
static kk_box_t kk_std_core_foreach_while_fun20869(kk_function_t _fself, kk_box_t _b_18850, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_while_fun20869(kk_function_t _b_18851_18849, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun20869__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_while_fun20869__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_while_fun20869, kk_context());
  _self->_b_18851_18849 = _b_18851_18849;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_while_fun20869(kk_function_t _fself, kk_box_t _b_18850, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun20869__t* _self = kk_function_as(struct kk_std_core_foreach_while_fun20869__t*, _fself);
  kk_function_t _b_18851_18849 = _self->_b_18851_18849; /* (maybe<10599>) -> 10600 maybe<10599> */
  kk_drop_match(_self, {kk_function_dup(_b_18851_18849);}, {}, _ctx)
  kk_std_core_types__maybe _x20870;
  kk_std_core_types__maybe _x20871 = kk_std_core_types__maybe_unbox(_b_18850, _ctx); /*maybe<10599>*/
  _x20870 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_18851_18849, (_b_18851_18849, _x20871, _ctx)); /*maybe<10599>*/
  return kk_std_core_types__maybe_box(_x20870, _ctx);
}

kk_std_core_types__maybe kk_std_core_foreach_while(kk_std_core__list xs, kk_function_t action0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>) -> e maybe<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(action0, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  struct kk_std_core_Cons* _con20865 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con20865->head;
  kk_std_core__list xx0 = _con20865->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx0);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core_types__maybe x0_17829;
  kk_function_t _x20866 = kk_function_dup(action0); /*(10598) -> 10600 maybe<10599>*/
  x0_17829 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x20866, (_x20866, x, _ctx)); /*maybe<10599>*/
  if (kk_yielding(kk_context())) {
    kk_std_core_types__maybe_drop(x0_17829, _ctx);
    kk_function_t _b_18851_18849 = kk_std_core_new_foreach_while_fun20867(action0, xx0, _ctx); /*(maybe<10599>) -> 10600 maybe<10599>*/;
    kk_box_t _x20868 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_while_fun20869(_b_18851_18849, _ctx), _ctx); /*3858*/
    return kk_std_core_types__maybe_unbox(_x20868, _ctx);
  }
  if (kk_std_core_types__is_Nothing(x0_17829)) {
    { // tailcall
      xs = xx0;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action0, _ctx);
  kk_std_core__list_drop(xx0, _ctx);
  return x0_17829;
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17544_foreach_while_1(kk_function_t action, kk_std_core__sslice rest, kk_std_core_types__maybe _y_17327, kk_context_t* _ctx) { /* forall<a,e> (action : (c : char) -> e maybe<a>, rest : sslice, maybe<a>) -> e maybe<a> */ 
  if (kk_std_core_types__is_Nothing(_y_17327)) {
    return kk_std_core_foreach_while_1(rest, action, _ctx);
  }
  kk_function_drop(action, _ctx);
  kk_std_core__sslice_drop(rest, _ctx);
  return _y_17327;
}
 
// Apply a function for each character in a string slice.
// If `action` returns `Just`, the function returns immediately with that result.


// lift anonymous function
struct kk_std_core_foreach_while_fun20876__t_1 {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_std_core__sslice rest0;
};
static kk_std_core_types__maybe kk_std_core_foreach_while_fun20876_1(kk_function_t _fself, kk_std_core_types__maybe _y_173270, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_while_fun20876_1(kk_function_t action0, kk_std_core__sslice rest0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun20876__t_1* _self = kk_function_alloc_as(struct kk_std_core_foreach_while_fun20876__t_1, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_while_fun20876_1, kk_context());
  _self->action0 = action0;
  _self->rest0 = rest0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core_foreach_while_fun20876_1(kk_function_t _fself, kk_std_core_types__maybe _y_173270, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun20876__t_1* _self = kk_function_as(struct kk_std_core_foreach_while_fun20876__t_1*, _fself);
  kk_function_t action0 = _self->action0; /* (c : char) -> 10685 maybe<10684> */
  kk_std_core__sslice rest0 = _self->rest0; /* sslice */
  kk_drop_match(_self, {kk_function_dup(action0);kk_std_core__sslice_dup(rest0);}, {}, _ctx)
  return kk_std_core__mlift17544_foreach_while_1(action0, rest0, _y_173270, _ctx);
}


// lift anonymous function
struct kk_std_core_foreach_while_fun20878__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_18857_18855;
};
static kk_box_t kk_std_core_foreach_while_fun20878_1(kk_function_t _fself, kk_box_t _b_18856, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_while_fun20878_1(kk_function_t _b_18857_18855, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun20878__t_1* _self = kk_function_alloc_as(struct kk_std_core_foreach_while_fun20878__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_while_fun20878_1, kk_context());
  _self->_b_18857_18855 = _b_18857_18855;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_while_fun20878_1(kk_function_t _fself, kk_box_t _b_18856, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun20878__t_1* _self = kk_function_as(struct kk_std_core_foreach_while_fun20878__t_1*, _fself);
  kk_function_t _b_18857_18855 = _self->_b_18857_18855; /* (maybe<10684>) -> 10685 maybe<10684> */
  kk_drop_match(_self, {kk_function_dup(_b_18857_18855);}, {}, _ctx)
  kk_std_core_types__maybe _x20879;
  kk_std_core_types__maybe _x20880 = kk_std_core_types__maybe_unbox(_b_18856, _ctx); /*maybe<10684>*/
  _x20879 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_18857_18855, (_b_18857_18855, _x20880, _ctx)); /*maybe<10684>*/
  return kk_std_core_types__maybe_box(_x20879, _ctx);
}

kk_std_core_types__maybe kk_std_core_foreach_while_1(kk_std_core__sslice slice0, kk_function_t action0, kk_context_t* _ctx) { /* forall<a,e> (slice : sslice, action : (c : char) -> e maybe<a>) -> e maybe<a> */ 
  kk__tailcall: ;
  kk_std_core_types__maybe _match_19803 = kk_std_core_next(slice0, _ctx); /*maybe<(char, sslice)>*/;
  if (kk_std_core_types__is_Nothing(_match_19803)) {
    kk_function_drop(action0, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_box_t _box_x18852 = _match_19803._cons.Just.value;
  kk_std_core_types__tuple2_ _pat1 = kk_std_core_types__tuple2__unbox(_box_x18852, NULL);
  kk_box_t _box_x18853 = _pat1.fst;
  kk_box_t _box_x18854 = _pat1.snd;
  kk_char_t c = kk_char_unbox(_box_x18853, NULL);
  kk_std_core__sslice rest0 = kk_std_core__sslice_unbox(_box_x18854, NULL);
  kk_std_core__sslice_dup(rest0);
  kk_std_core_types__maybe_drop(_match_19803, _ctx);
  kk_Unit;
  kk_std_core_types__maybe x_17832;
  kk_function_t _x20875 = kk_function_dup(action0); /*(c : char) -> 10685 maybe<10684>*/
  x_17832 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_char_t, kk_context_t*), _x20875, (_x20875, c, _ctx)); /*maybe<10684>*/
  if (kk_yielding(kk_context())) {
    kk_std_core_types__maybe_drop(x_17832, _ctx);
    kk_function_t _b_18857_18855 = kk_std_core_new_foreach_while_fun20876_1(action0, rest0, _ctx); /*(maybe<10684>) -> 10685 maybe<10684>*/;
    kk_box_t _x20877 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_while_fun20878_1(_b_18857_18855, _ctx), _ctx); /*3858*/
    return kk_std_core_types__maybe_unbox(_x20877, _ctx);
  }
  if (kk_std_core_types__is_Nothing(x_17832)) {
    { // tailcall
      slice0 = rest0;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action0, _ctx);
  kk_std_core__sslice_drop(rest0, _ctx);
  return x_17832;
}
 
// Invoke a function `f` for each element in a vector `v`.
// If `f` returns `Just`, the iteration is stopped early and the result is returned.


// lift anonymous function
struct kk_std_core_foreach_while_fun20886__t_3 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_vector_t v;
};
static kk_std_core_types__maybe kk_std_core_foreach_while_fun20886_3(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_while_fun20886_3(kk_function_t f, kk_vector_t v, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun20886__t_3* _self = kk_function_alloc_as(struct kk_std_core_foreach_while_fun20886__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_while_fun20886_3, kk_context());
  _self->f = f;
  _self->v = v;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core_foreach_while_fun20886_3(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun20886__t_3* _self = kk_function_as(struct kk_std_core_foreach_while_fun20886__t_3*, _fself);
  kk_function_t f = _self->f; /* (10783) -> 10785 maybe<10784> */
  kk_vector_t v = _self->v; /* vector<10783> */
  kk_drop_match(_self, {kk_function_dup(f);kk_vector_dup(v);}, {}, _ctx)
  kk_box_t _x20887;
  kk_box_t _borrowApp_19802 = kk_vector_at_borrow(v, i); /*219*/;
  kk_vector_drop(v, _ctx);
  _x20887 = _borrowApp_19802; /*219*/
  return kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20887, _ctx));
}

kk_std_core_types__maybe kk_std_core_foreach_while_3(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (v : vector<a>, f : (a) -> e maybe<b>) -> e maybe<b> */ 
  kk_ssize_t start0_17835 = ((kk_ssize_t)0); /*ssize_t*/;
  kk_ssize_t end_17836;
  kk_ssize_t _x20885 = kk_vector_len_borrow(v); /*ssize_t*/
  end_17836 = kk_std_core_decr_1(_x20885, _ctx); /*ssize_t*/
  return kk_std_core__lift17077_for_whilez(kk_std_core_new_foreach_while_fun20886_3(f, v, _ctx), end_17836, start0_17835, _ctx);
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17545_foreach(kk_function_t action, kk_std_core__list xx, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<a,e> (action : (a) -> e (), xx : list<a>, wild_ : ()) -> e () */ 
  kk_std_core_foreach(xx, action, _ctx); return kk_Unit;
}
 
// Invoke `action` for each element of a list


// lift anonymous function
struct kk_std_core_foreach_fun20890__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_std_core__list xx0;
};
static kk_unit_t kk_std_core_foreach_fun20890(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20890(kk_function_t action0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20890__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun20890__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun20890, kk_context());
  _self->action0 = action0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_unit_t kk_std_core_foreach_fun20890(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20890__t* _self = kk_function_as(struct kk_std_core_foreach_fun20890__t*, _fself);
  kk_function_t action0 = _self->action0; /* (10978) -> 10979 () */
  kk_std_core__list xx0 = _self->xx0; /* list<10978> */
  kk_drop_match(_self, {kk_function_dup(action0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17545_foreach(action0, xx0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core_foreach_fun20892__t {
  struct kk_function_s _base;
  kk_function_t _b_18860_18858;
};
static kk_box_t kk_std_core_foreach_fun20892(kk_function_t _fself, kk_box_t _b_18859, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20892(kk_function_t _b_18860_18858, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20892__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun20892__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun20892, kk_context());
  _self->_b_18860_18858 = _b_18860_18858;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_fun20892(kk_function_t _fself, kk_box_t _b_18859, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20892__t* _self = kk_function_as(struct kk_std_core_foreach_fun20892__t*, _fself);
  kk_function_t _b_18860_18858 = _self->_b_18860_18858; /* (wild_0 : ()) -> 10979 () */
  kk_drop_match(_self, {kk_function_dup(_b_18860_18858);}, {}, _ctx)
  kk_unit_t _x20893 = kk_Unit;
  kk_unit_t _x20894 = kk_Unit;
  kk_unit_unbox(_b_18859);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_18860_18858, (_b_18860_18858, _x20894, _ctx));
  return kk_unit_box(_x20893);
}

kk_unit_t kk_std_core_foreach(kk_std_core__list xs, kk_function_t action0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, action : (a) -> e ()) -> e () */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(action0, _ctx);
    kk_Unit; return kk_Unit;
  }
  struct kk_std_core_Cons* _con20888 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con20888->head;
  kk_std_core__list xx0 = _con20888->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx0);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_unit_t x0_17838 = kk_Unit;
  kk_function_t _x20889 = kk_function_dup(action0); /*(10978) -> 10979 ()*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_box_t, kk_context_t*), _x20889, (_x20889, x, _ctx));
  if (kk_yielding(kk_context())) {
    kk_function_t _b_18860_18858 = kk_std_core_new_foreach_fun20890(action0, xx0, _ctx); /*(wild_0 : ()) -> 10979 ()*/;
    kk_box_t _x20891 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_fun20892(_b_18860_18858, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x20891); return kk_Unit;
  }
  { // tailcall
    xs = xx0;
    goto kk__tailcall;
  }
}
 
// Apply a function for each character in a string slice.


// lift anonymous function
struct kk_std_core_foreach_fun20895__t_1 {
  struct kk_function_s _base;
  kk_function_t action;
};
static kk_std_core_types__maybe kk_std_core_foreach_fun20895_1(kk_function_t _fself, kk_char_t c, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20895_1(kk_function_t action, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20895__t_1* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun20895__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun20895_1, kk_context());
  _self->action = action;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_core_foreach_fun20897__t_1 {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_foreach_fun20897_1(kk_function_t _fself, kk_box_t _b_18862, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20897_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_foreach_fun20897_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_foreach_fun20897_1(kk_function_t _fself, kk_box_t _b_18862, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__maybe _x20898;
  kk_unit_t wild___18866 = kk_Unit;
  kk_unit_unbox(_b_18862);
  _x20898 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/
  return kk_std_core_types__maybe_box(_x20898, _ctx);
}
static kk_std_core_types__maybe kk_std_core_foreach_fun20895_1(kk_function_t _fself, kk_char_t c, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20895__t_1* _self = kk_function_as(struct kk_std_core_foreach_fun20895__t_1*, _fself);
  kk_function_t action = _self->action; /* (c : char) -> 11037 () */
  kk_drop_match(_self, {kk_function_dup(action);}, {}, _ctx)
  kk_unit_t x0_17844 = kk_Unit;
  kk_function_call(kk_unit_t, (kk_function_t, kk_char_t, kk_context_t*), action, (action, c, _ctx));
  if (kk_yielding(kk_context())) {
    kk_box_t _x20896 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_fun20897_1(_ctx), _ctx); /*3858*/
    return kk_std_core_types__maybe_unbox(_x20896, _ctx);
  }
  return kk_std_core_types__new_Nothing(_ctx);
}


// lift anonymous function
struct kk_std_core_foreach_fun20899__t_1 {
  struct kk_function_s _base;
};
static kk_unit_t kk_std_core_foreach_fun20899_1(kk_function_t _fself, kk_std_core_types__maybe wild__00, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20899_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_foreach_fun20899_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_unit_t kk_std_core_foreach_fun20899_1(kk_function_t _fself, kk_std_core_types__maybe wild__00, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__maybe_drop(wild__00, _ctx);
  return kk_Unit;
}


// lift anonymous function
struct kk_std_core_foreach_fun20901__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_18867_18864;
};
static kk_box_t kk_std_core_foreach_fun20901_1(kk_function_t _fself, kk_box_t _b_18865, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20901_1(kk_function_t _b_18867_18864, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20901__t_1* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun20901__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun20901_1, kk_context());
  _self->_b_18867_18864 = _b_18867_18864;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_fun20901_1(kk_function_t _fself, kk_box_t _b_18865, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20901__t_1* _self = kk_function_as(struct kk_std_core_foreach_fun20901__t_1*, _fself);
  kk_function_t _b_18867_18864 = _self->_b_18867_18864; /* (wild_00 : maybe<_11024>) -> 11037 () */
  kk_drop_match(_self, {kk_function_dup(_b_18867_18864);}, {}, _ctx)
  kk_unit_t _x20902 = kk_Unit;
  kk_std_core_types__maybe _x20903 = kk_std_core_types__maybe_unbox(_b_18865, _ctx); /*maybe<_11024>*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_18867_18864, (_b_18867_18864, _x20903, _ctx));
  return kk_unit_box(_x20902);
}

kk_unit_t kk_std_core_foreach_1(kk_std_core__sslice slice0, kk_function_t action, kk_context_t* _ctx) { /* forall<e> (slice : sslice, action : (c : char) -> e ()) -> e () */ 
  kk_std_core_types__maybe x_17841 = kk_std_core_foreach_while_1(slice0, kk_std_core_new_foreach_fun20895_1(action, _ctx), _ctx); /*maybe<_11024>*/;
  kk_std_core_types__maybe_drop(x_17841, _ctx);
  if (kk_yielding(kk_context())) {
    kk_function_t _b_18867_18864 = kk_std_core_new_foreach_fun20899_1(_ctx); /*(wild_00 : maybe<_11024>) -> 11037 ()*/;
    kk_box_t _x20900 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_fun20901_1(_b_18867_18864, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x20900); return kk_Unit;
  }
  kk_Unit; return kk_Unit;
}
 
// Invoke a function for each character in a string


// lift anonymous function
struct kk_std_core_foreach_fun20907__t_2 {
  struct kk_function_s _base;
  kk_function_t action;
};
static kk_std_core_types__maybe kk_std_core_foreach_fun20907_2(kk_function_t _fself, kk_char_t c, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20907_2(kk_function_t action, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20907__t_2* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun20907__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun20907_2, kk_context());
  _self->action = action;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_core_foreach_fun20909__t_2 {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_foreach_fun20909_2(kk_function_t _fself, kk_box_t _b_18869, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20909_2(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_foreach_fun20909_2, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_foreach_fun20909_2(kk_function_t _fself, kk_box_t _b_18869, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__maybe _x20910;
  kk_unit_t wild___18873 = kk_Unit;
  kk_unit_unbox(_b_18869);
  _x20910 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/
  return kk_std_core_types__maybe_box(_x20910, _ctx);
}
static kk_std_core_types__maybe kk_std_core_foreach_fun20907_2(kk_function_t _fself, kk_char_t c, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20907__t_2* _self = kk_function_as(struct kk_std_core_foreach_fun20907__t_2*, _fself);
  kk_function_t action = _self->action; /* (c : char) -> 11076 () */
  kk_drop_match(_self, {kk_function_dup(action);}, {}, _ctx)
  kk_unit_t x0_17851 = kk_Unit;
  kk_function_call(kk_unit_t, (kk_function_t, kk_char_t, kk_context_t*), action, (action, c, _ctx));
  if (kk_yielding(kk_context())) {
    kk_box_t _x20908 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_fun20909_2(_ctx), _ctx); /*3858*/
    return kk_std_core_types__maybe_unbox(_x20908, _ctx);
  }
  return kk_std_core_types__new_Nothing(_ctx);
}


// lift anonymous function
struct kk_std_core_foreach_fun20911__t_2 {
  struct kk_function_s _base;
};
static kk_unit_t kk_std_core_foreach_fun20911_2(kk_function_t _fself, kk_std_core_types__maybe wild__00, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20911_2(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_foreach_fun20911_2, _ctx)
  return kk_function_dup(_fself);
}

static kk_unit_t kk_std_core_foreach_fun20911_2(kk_function_t _fself, kk_std_core_types__maybe wild__00, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__maybe_drop(wild__00, _ctx);
  return kk_Unit;
}


// lift anonymous function
struct kk_std_core_foreach_fun20913__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18874_18871;
};
static kk_box_t kk_std_core_foreach_fun20913_2(kk_function_t _fself, kk_box_t _b_18872, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20913_2(kk_function_t _b_18874_18871, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20913__t_2* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun20913__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun20913_2, kk_context());
  _self->_b_18874_18871 = _b_18874_18871;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_fun20913_2(kk_function_t _fself, kk_box_t _b_18872, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20913__t_2* _self = kk_function_as(struct kk_std_core_foreach_fun20913__t_2*, _fself);
  kk_function_t _b_18874_18871 = _self->_b_18874_18871; /* (wild_00 : maybe<_11024>) -> 11076 () */
  kk_drop_match(_self, {kk_function_dup(_b_18874_18871);}, {}, _ctx)
  kk_unit_t _x20914 = kk_Unit;
  kk_std_core_types__maybe _x20915 = kk_std_core_types__maybe_unbox(_b_18872, _ctx); /*maybe<_11024>*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_18874_18871, (_b_18874_18871, _x20915, _ctx));
  return kk_unit_box(_x20914);
}

kk_unit_t kk_std_core_foreach_2(kk_string_t s, kk_function_t action, kk_context_t* _ctx) { /* forall<e> (s : string, action : (c : char) -> e ()) -> e () */ 
  kk_std_core__sslice slice0_16983;
  kk_string_t _x20904 = kk_string_dup(s); /*string*/
  kk_ssize_t _x20905 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x20906 = kk_string_len(s,kk_context()); /*ssize_t*/
  slice0_16983 = kk_std_core__new_Sslice(_x20904, _x20905, _x20906, _ctx); /*sslice*/
  kk_std_core_types__maybe x_17848 = kk_std_core_foreach_while_1(slice0_16983, kk_std_core_new_foreach_fun20907_2(action, _ctx), _ctx); /*maybe<_11024>*/;
  kk_std_core_types__maybe_drop(x_17848, _ctx);
  if (kk_yielding(kk_context())) {
    kk_function_t _b_18874_18871 = kk_std_core_new_foreach_fun20911_2(_ctx); /*(wild_00 : maybe<_11024>) -> 11076 ()*/;
    kk_box_t _x20912 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_fun20913_2(_b_18874_18871, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x20912); return kk_Unit;
  }
  kk_Unit; return kk_Unit;
}
 
// Invoke a function `f` for each element in a vector `v`


// lift anonymous function
struct kk_std_core_foreach_fun20917__t_3 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_vector_t v;
};
static kk_unit_t kk_std_core_foreach_fun20917_3(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun20917_3(kk_function_t f, kk_vector_t v, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20917__t_3* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun20917__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun20917_3, kk_context());
  _self->f = f;
  _self->v = v;
  return &_self->_base;
}

static kk_unit_t kk_std_core_foreach_fun20917_3(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun20917__t_3* _self = kk_function_as(struct kk_std_core_foreach_fun20917__t_3*, _fself);
  kk_function_t f = _self->f; /* (11091) -> 11092 () */
  kk_vector_t v = _self->v; /* vector<11091> */
  kk_drop_match(_self, {kk_function_dup(f);kk_vector_dup(v);}, {}, _ctx)
  kk_box_t x_18123;
  kk_box_t _borrowApp_19796 = kk_vector_at_borrow(v, i); /*219*/;
  kk_vector_drop(v, _ctx);
  x_18123 = _borrowApp_19796; /*11091*/
  return kk_function_call(kk_unit_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x_18123, _ctx));
}

kk_unit_t kk_std_core_foreach_3(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (v : vector<a>, f : (a) -> e ()) -> e () */ 
  kk_ssize_t start0_17857 = ((kk_ssize_t)0); /*ssize_t*/;
  kk_ssize_t end_17858;
  kk_ssize_t _x20916 = kk_vector_len_borrow(v); /*ssize_t*/
  end_17858 = kk_std_core_decr_1(_x20916, _ctx); /*ssize_t*/
  kk_std_core__lift17073_forz(kk_std_core_new_foreach_fun20917_3(f, v, _ctx), end_17858, start0_17857, _ctx); return kk_Unit;
}
 
// O(n). Return the number of characters in a string.

kk_integer_t kk_std_core_count_1(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> int */ 
  return kk_string_count_int(s,kk_context());
}
 
// O(n). Return the number of characters in a string slice

kk_integer_t kk_std_core_count_2(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> int */ 
  return kk_slice_count(slice0,kk_context());
}
 
// Count the number of times a predicate is true for each character in a string


// lift anonymous function
struct kk_std_core_count_fun20922__t_3 {
  struct kk_function_s _base;
  kk_ref_t loc;
  kk_function_t pred;
};
static kk_std_core_types__maybe kk_std_core_count_fun20922_3(kk_function_t _fself, kk_char_t c, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_count_fun20922_3(kk_ref_t loc, kk_function_t pred, kk_context_t* _ctx) {
  struct kk_std_core_count_fun20922__t_3* _self = kk_function_alloc_as(struct kk_std_core_count_fun20922__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_count_fun20922_3, kk_context());
  _self->loc = loc;
  _self->pred = pred;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core_count_fun20922_3(kk_function_t _fself, kk_char_t c, kk_context_t* _ctx) {
  struct kk_std_core_count_fun20922__t_3* _self = kk_function_as(struct kk_std_core_count_fun20922__t_3*, _fself);
  kk_ref_t loc = _self->loc; /* local-var<global,int> */
  kk_function_t pred = _self->pred; /* (char) -> bool */
  kk_drop_match(_self, {kk_ref_dup(loc);kk_function_dup(pred);}, {}, _ctx)
  kk_unit_t __ = kk_Unit;
  bool _match_19794 = kk_function_call(bool, (kk_function_t, kk_char_t, kk_context_t*), pred, (pred, c, _ctx)); /*bool*/;
  if (_match_19794) {
    kk_integer_t _b_18881_18879;
    kk_integer_t _x20923;
    kk_box_t _x20924;
    kk_ref_t _x20925 = kk_ref_dup(loc); /*local-var<global,int>*/
    _x20924 = (kk_ref_get(_x20925,kk_context())); /*256*/
    _x20923 = kk_integer_unbox(_x20924); /*int*/
    _b_18881_18879 = kk_integer_add(_x20923,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_unit_t _borrowApp_19795 = kk_Unit;
    (kk_ref_set_borrow(loc,(kk_integer_box(_b_18881_18879)),kk_context()));
    kk_ref_drop(loc, _ctx);
    _borrowApp_19795;
  }
  else {
    kk_ref_drop(loc, _ctx);
    kk_Unit;
  }
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_integer_t kk_std_core_count_3(kk_string_t s, kk_function_t pred, kk_context_t* _ctx) { /* (s : string, pred : (char) -> bool) -> int */ 
  kk_ref_t loc = kk_ref_alloc((kk_integer_box(kk_integer_from_small(0))),kk_context()); /*local-var<global,int>*/;
  kk_std_core__sslice slice0_16988;
  kk_string_t _x20918 = kk_string_dup(s); /*string*/
  kk_ssize_t _x20919 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x20920 = kk_string_len(s,kk_context()); /*ssize_t*/
  slice0_16988 = kk_std_core__new_Sslice(_x20918, _x20919, _x20920, _ctx); /*sslice*/
  kk_std_core_types__maybe __0;
  kk_function_t _x20921;
  kk_ref_dup(loc);
  _x20921 = kk_std_core_new_count_fun20922_3(loc, pred, _ctx); /*(c : char) -> (local<global>) (forall<a> maybe<a>)*/
  __0 = kk_std_core_foreach_while_1(slice0_16988, _x20921, _ctx); /*maybe<_11024>*/
  kk_std_core_types__maybe_drop(__0, _ctx);
  kk_integer_t res;
  kk_box_t _x20926;
  kk_ref_t _x20927 = kk_ref_dup(loc); /*local-var<global,int>*/
  _x20926 = (kk_ref_get(_x20927,kk_context())); /*256*/
  res = kk_integer_unbox(_x20926); /*int*/
  kk_ref_t _b_18887_18885 = loc; /*local-var<global,int>*/;
  kk_integer_t _b_18888_18886 = res; /*int*/;
  kk_box_t _x20928 = kk_std_core_hnd_prompt_local_var(_b_18887_18885, kk_integer_box(_b_18888_18886), _ctx); /*9895*/
  return kk_integer_unbox(_x20928);
}
 
// Is the integer negative (stricly smaller than zero)

bool kk_std_core_is_neg_2(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> bool */ 
  kk_std_core_types__order x_17124;
  kk_std_core_types__order _borrowApp_19793 = kk_int_as_order(kk_integer_signum_borrow(i),kk_context()); /*order*/;
  kk_integer_drop(i, _ctx);
  x_17124 = _borrowApp_19793; /*order*/
  kk_integer_t _borrowedArg_19791;
  if (kk_std_core_types__is_Lt(x_17124)) {
    _borrowedArg_19791 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20930;
  }
  if (kk_std_core_types__is_Eq(x_17124)) {
    _borrowedArg_19791 = kk_integer_from_small(0); /*int*/
    goto _match20930;
  }
  _borrowedArg_19791 = kk_integer_from_small(1); /*int*/
  _match20930: ;
  kk_integer_t _borrowedArg_19790 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/;
  bool _borrowApp_19792 = kk_integer_eq_borrow(_borrowedArg_19791,_borrowedArg_19790,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19791, _ctx);
  kk_integer_drop(_borrowedArg_19790, _ctx);
  return _borrowApp_19792;
}

kk_std_core_types__tuple2_ kk_std_core_divmod_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx) { /* (i : int, n : int) -> (int, int) */ 
  kk_std_core_types__tuple2_ _match_19786;
  kk_integer_t _x20931 = kk_integer_dup(n); /*int*/
  _match_19786 = kk_std_core_cdivmod_exp10(i, _x20931, _ctx); /*(int, int)*/
  kk_box_t _box_x18894 = _match_19786.fst;
  kk_box_t _box_x18895 = _match_19786.snd;
  kk_integer_t cq = kk_integer_unbox(_box_x18894);
  kk_integer_t cr = kk_integer_unbox(_box_x18895);
  kk_integer_dup(cq);
  kk_integer_dup(cr);
  kk_std_core_types__tuple2__drop(_match_19786, _ctx);
  kk_Unit;
  kk_std_core_types__order x = kk_int_as_order(kk_integer_signum_borrow(cr),kk_context()); /*order*/;
  bool b_16991;
  kk_integer_t _borrowedArg_19788;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_19788 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20934;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_19788 = kk_integer_from_small(0); /*int*/
    goto _match20934;
  }
  _borrowedArg_19788 = kk_integer_from_small(1); /*int*/
  _match20934: ;
  kk_integer_t _borrowedArg_19787 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/;
  bool _borrowApp_19789 = kk_integer_eq_borrow(_borrowedArg_19788,_borrowedArg_19787,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19788, _ctx);
  kk_integer_drop(_borrowedArg_19787, _ctx);
  b_16991 = _borrowApp_19789; /*bool*/
  if (b_16991) {
    kk_integer_t _b_18900_18896 = kk_integer_sub(cq,(kk_integer_from_small(1)),kk_context()); /*int*/;
    kk_integer_t _b_18901_18897;
    kk_integer_t _x20935 = kk_std_core_mul_exp10(kk_integer_from_small(1), n, _ctx); /*int*/
    _b_18901_18897 = kk_integer_add(cr,_x20935,kk_context()); /*int*/
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_18900_18896), kk_integer_box(_b_18901_18897), _ctx);
  }
  kk_integer_drop(n, _ctx);
  kk_integer_t _b_18902_18898 = cq; /*int*/;
  kk_integer_t _b_18903_18899 = cr; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_18902_18898), kk_integer_box(_b_18903_18899), _ctx);
}
 
// Drop the first `n` elements of a list (or fewer if the list is shorter than `n`)

kk_std_core__list kk_std_core_drop(kk_std_core__list xs, kk_integer_t n, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20936 = kk_std_core__as_Cons(xs);
    kk_box_t _pat0 = _con20936->head;
    kk_std_core__list xx = _con20936->tail;
    kk_integer_t _borrowedArg_19784 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_19785 = kk_integer_gt_borrow(n,_borrowedArg_19784,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_19784, _ctx);
    if (_borrowApp_19785) {
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        kk_box_drop(_pat0, _ctx);
        kk_Unit;
        kk_Unit;
        kk_std_core__list_free(xs);
      }
      else {
        kk_std_core__list_dup(xx);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      { // tailcall
        kk_integer_t _x20937 = kk_integer_sub(n,(kk_integer_from_small(1)),kk_context()); /*int*/
        xs = xx;
        n = _x20937;
        goto kk__tailcall;
      }
    }
  }
  kk_integer_drop(n, _ctx);
  return xs;
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17550_drop_while(kk_function_t predicate, kk_std_core__list xs, kk_std_core__list xx, bool _y_17343, kk_context_t* _ctx) { /* forall<a,e> (predicate : (a) -> e bool, xs : list<a>, xx : list<a>, bool) -> e list<a> */ 
  if (_y_17343) {
    kk_std_core__list_drop(xs, _ctx);
    return kk_std_core_drop_while(xx, predicate, _ctx);
  }
  kk_function_drop(predicate, _ctx);
  kk_std_core__list_drop(xx, _ctx);
  return xs;
}
 
// Drop all initial elements that satisfy `predicate`


// lift anonymous function
struct kk_std_core_drop_while_fun20940__t {
  struct kk_function_s _base;
  kk_function_t predicate0;
  kk_std_core__list xs0;
  kk_std_core__list xx0;
};
static kk_std_core__list kk_std_core_drop_while_fun20940(kk_function_t _fself, bool _y_173430, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_drop_while_fun20940(kk_function_t predicate0, kk_std_core__list xs0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_drop_while_fun20940__t* _self = kk_function_alloc_as(struct kk_std_core_drop_while_fun20940__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_drop_while_fun20940, kk_context());
  _self->predicate0 = predicate0;
  _self->xs0 = xs0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core_drop_while_fun20940(kk_function_t _fself, bool _y_173430, kk_context_t* _ctx) {
  struct kk_std_core_drop_while_fun20940__t* _self = kk_function_as(struct kk_std_core_drop_while_fun20940__t*, _fself);
  kk_function_t predicate0 = _self->predicate0; /* (11553) -> 11554 bool */
  kk_std_core__list xs0 = _self->xs0; /* list<11553> */
  kk_std_core__list xx0 = _self->xx0; /* list<11553> */
  kk_drop_match(_self, {kk_function_dup(predicate0);kk_std_core__list_dup(xs0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17550_drop_while(predicate0, xs0, xx0, _y_173430, _ctx);
}


// lift anonymous function
struct kk_std_core_drop_while_fun20942__t {
  struct kk_function_s _base;
  kk_function_t _b_18906_18904;
};
static kk_box_t kk_std_core_drop_while_fun20942(kk_function_t _fself, kk_box_t _b_18905, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_drop_while_fun20942(kk_function_t _b_18906_18904, kk_context_t* _ctx) {
  struct kk_std_core_drop_while_fun20942__t* _self = kk_function_alloc_as(struct kk_std_core_drop_while_fun20942__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_drop_while_fun20942, kk_context());
  _self->_b_18906_18904 = _b_18906_18904;
  return &_self->_base;
}

static kk_box_t kk_std_core_drop_while_fun20942(kk_function_t _fself, kk_box_t _b_18905, kk_context_t* _ctx) {
  struct kk_std_core_drop_while_fun20942__t* _self = kk_function_as(struct kk_std_core_drop_while_fun20942__t*, _fself);
  kk_function_t _b_18906_18904 = _self->_b_18906_18904; /* (bool) -> 11554 list<11553> */
  kk_drop_match(_self, {kk_function_dup(_b_18906_18904);}, {}, _ctx)
  kk_std_core__list _x20943;
  bool _x20944 = kk_bool_unbox(_b_18905); /*bool*/
  _x20943 = kk_function_call(kk_std_core__list, (kk_function_t, bool, kk_context_t*), _b_18906_18904, (_b_18906_18904, _x20944, _ctx)); /*list<11553>*/
  return kk_std_core__list_box(_x20943, _ctx);
}

kk_std_core__list kk_std_core_drop_while(kk_std_core__list xs0, kk_function_t predicate0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con20938 = kk_std_core__as_Cons(xs0);
    kk_box_t x = _con20938->head;
    kk_std_core__list xx0 = _con20938->tail;
    kk_box_dup(x);
    kk_std_core__list_dup(xx0);
    bool x0_17862;
    kk_function_t _x20939 = kk_function_dup(predicate0); /*(11553) -> 11554 bool*/
    x0_17862 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x20939, (_x20939, x, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_18906_18904 = kk_std_core_new_drop_while_fun20940(predicate0, xs0, xx0, _ctx); /*(bool) -> 11554 list<11553>*/;
      kk_box_t _x20941 = kk_std_core_hnd_yield_extend(kk_std_core_new_drop_while_fun20942(_b_18906_18904, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20941, _ctx);
    }
    if (x0_17862) {
      kk_std_core__list_dropn(xs0, ((int32_t)2), _ctx);
      { // tailcall
        xs0 = xx0;
        goto kk__tailcall;
      }
    }
    kk_function_drop(predicate0, _ctx);
    kk_std_core__list_drop(xx0, _ctx);
    return xs0;
  }
  kk_function_drop(predicate0, _ctx);
  return kk_std_core__new_Nil(_ctx);
}
 
// An empty slice

kk_std_core__sslice kk_std_core_empty;
 
// Does string `s`  end with `post`?
// If so, returns a slice of `s` from the start up to the `post` string at the end.

kk_std_core_types__maybe kk_std_core_ends_with(kk_string_t s, kk_string_t post, kk_context_t* _ctx) { /* (s : string, post : string) -> maybe<sslice> */ 
  bool _match_19782;
  kk_string_t _x20949 = kk_string_dup(s); /*string*/
  kk_string_t _x20950 = kk_string_dup(post); /*string*/
  _match_19782 = kk_std_core_xends_with(_x20949, _x20950, _ctx); /*bool*/
  if (_match_19782) {
    kk_std_core__sslice _b_18908_18907;
    kk_string_t _x20951 = kk_string_dup(s); /*string*/
    kk_ssize_t _x20952 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x20953;
    kk_ssize_t _x20954 = kk_string_len(s,kk_context()); /*ssize_t*/
    kk_ssize_t _x20955 = kk_string_len(post,kk_context()); /*ssize_t*/
    _x20953 = (_x20954 - _x20955); /*ssize_t*/
    _b_18908_18907 = kk_std_core__new_Sslice(_x20951, _x20952, _x20953, _ctx); /*sslice*/
    return kk_std_core_types__new_Just(kk_std_core__sslice_box(_b_18908_18907, _ctx), _ctx);
  }
  kk_string_drop(post, _ctx);
  kk_string_drop(s, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Throw an exception with a specified message.

kk_box_t kk_std_core_throw(kk_string_t message0, kk_std_core_types__optional info0, kk_context_t* _ctx) { /* forall<a> (message : string, info : optional<exception-info>) -> exn a */ 
  kk_std_core_hnd__ev ev_17865;
  kk_ssize_t _x20956 = ((kk_ssize_t)0); /*ssize_t*/
  ev_17865 = kk_evv_at(_x20956,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
  struct kk_std_core_hnd_Ev* _con20957 = kk_std_core_hnd__as_Ev(ev_17865);
  kk_std_core_hnd__marker m0 = _con20957->marker;
  kk_box_t _box_x18909 = _con20957->hnd;
  kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x18909, NULL);
  kk_std_core__hnd_exn_dup(h);
  kk_std_core_hnd__clause1 _match_19780;
  kk_std_core_hnd__clause1 _borrowApp_19781 = kk_std_core__select_throw_exn(h, _ctx); /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/;
  kk_std_core__hnd_exn_drop(h, _ctx);
  _match_19780 = _borrowApp_19781; /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/
  kk_function_t _fun_unbox_x18913 = _match_19780.clause;
  kk_std_core_hnd__marker _b_18914 = m0; /*std/core/hnd/marker<3573,3574>*/;
  kk_std_core_hnd__ev _b_18915 = ev_17865; /*std/core/hnd/ev<.hnd-exn>*/;
  kk_std_core__exception _b_18916;
  kk_std_core__exception_info _x20959;
  if (kk_std_core_types__is_Optional(info0)) {
    kk_box_t _box_x18917 = info0._cons.Optional.value;
    kk_std_core__exception_info _info_11639 = kk_std_core__exception_info_unbox(_box_x18917, NULL);
    kk_std_core__exception_info_dup(_info_11639);
    kk_std_core_types__optional_drop(info0, _ctx);
    kk_Unit;
    _x20959 = _info_11639; /*exception-info*/
    goto _match20960;
  }
  _x20959 = kk_std_core__new_ExnError(_ctx); /*exception-info*/
  _match20960: ;
  _b_18916 = kk_std_core__new_Exception(message0, _x20959, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x18913, (_fun_unbox_x18913, _b_18914, _b_18915, kk_std_core__exception_box(_b_18916, _ctx), _ctx));
}
 
// Transform an `:error` type back to an `exn` effect.

kk_box_t kk_std_core_throw_1(kk_std_core__error err, kk_context_t* _ctx) { /* forall<a> (err : error<a>) -> exn a */ 
  if (kk_std_core__is_Error(err)) {
    kk_std_core__exception exn0 = err._cons.Error.exception;
    kk_std_core__exception_dup(exn0);
    kk_std_core__error_drop(err, _ctx);
    kk_Unit;
    kk_std_core_hnd__ev ev_17868;
    kk_ssize_t _x20962 = ((kk_ssize_t)0); /*ssize_t*/
    ev_17868 = kk_evv_at(_x20962,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
    struct kk_std_core_hnd_Ev* _con20963 = kk_std_core_hnd__as_Ev(ev_17868);
    kk_std_core_hnd__marker m0 = _con20963->marker;
    kk_box_t _box_x18918 = _con20963->hnd;
    kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x18918, NULL);
    kk_std_core__hnd_exn_dup(h);
    kk_std_core_hnd__clause1 _match_19778;
    kk_std_core_hnd__clause1 _borrowApp_19779 = kk_std_core__select_throw_exn(h, _ctx); /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/;
    kk_std_core__hnd_exn_drop(h, _ctx);
    _match_19778 = _borrowApp_19779; /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/
    kk_function_t _fun_unbox_x18922 = _match_19778.clause;
    kk_std_core_hnd__marker _b_18923 = m0; /*std/core/hnd/marker<3573,3574>*/;
    kk_std_core_hnd__ev _b_18924 = ev_17868; /*std/core/hnd/ev<.hnd-exn>*/;
    kk_std_core__exception _b_18925 = exn0; /*exception*/;
    return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x18922, (_fun_unbox_x18922, _b_18923, _b_18924, kk_std_core__exception_box(_b_18925, _ctx), _ctx));
  }
  kk_box_t x0 = err._cons.Ok.result;
  kk_box_dup(x0);
  kk_std_core__error_drop(err, _ctx);
  kk_Unit;
  return x0;
}
 
// monadic lift

kk_box_t kk_std_core__mlift17551_error_pattern(kk_string_t definition, kk_string_t location, kk_string_t _c_17350, kk_context_t* _ctx) { /* forall<a> (definition : string, location : string, string) -> a */ 
  kk_string_t message0_16997;
  kk_string_t _x20965 = kk_string_dup(location); /*string*/
  kk_string_t _x20966;
  kk_string_t _x20967;
  kk_define_string_literal(, _s20968, 23, ": pattern match failure")
  _x20967 = kk_string_dup(_s20968); /*string*/
  _x20966 = kk_std_core__lp__plus__plus__1_rp_(_c_17350, _x20967, _ctx); /*string*/
  message0_16997 = kk_std_core__lp__plus__plus__1_rp_(_x20965, _x20966, _ctx); /*string*/
  kk_std_core_hnd__ev ev_17871;
  kk_ssize_t _x20969 = ((kk_ssize_t)0); /*ssize_t*/
  ev_17871 = kk_evv_at(_x20969,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
  struct kk_std_core_hnd_Ev* _con20970 = kk_std_core_hnd__as_Ev(ev_17871);
  kk_std_core_hnd__marker m0 = _con20970->marker;
  kk_box_t _box_x18926 = _con20970->hnd;
  kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x18926, NULL);
  kk_std_core__hnd_exn_dup(h);
  kk_std_core_hnd__clause1 _match_19776;
  kk_std_core_hnd__clause1 _borrowApp_19777 = kk_std_core__select_throw_exn(h, _ctx); /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/;
  kk_std_core__hnd_exn_drop(h, _ctx);
  _match_19776 = _borrowApp_19777; /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/
  kk_function_t _fun_unbox_x18930 = _match_19776.clause;
  kk_std_core_hnd__marker _b_18931 = m0; /*std/core/hnd/marker<3573,3574>*/;
  kk_std_core_hnd__ev _b_18932 = ev_17871; /*std/core/hnd/ev<.hnd-exn>*/;
  kk_std_core__exception _b_18933;
  kk_std_core__exception_info _x20972 = kk_std_core__new_ExnPattern(kk_reuse_null, location, definition, _ctx); /*exception-info*/
  _b_18933 = kk_std_core__new_Exception(message0_16997, _x20972, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x18930, (_fun_unbox_x18930, _b_18931, _b_18932, kk_std_core__exception_box(_b_18933, _ctx), _ctx));
}
 
// Raise a pattern match exception. This is function is used internally by the
// compiler to generate error messages on pattern match failures.


// lift anonymous function
struct kk_std_core_error_pattern_fun20980__t {
  struct kk_function_s _base;
  kk_string_t definition;
  kk_string_t location;
};
static kk_box_t kk_std_core_error_pattern_fun20980(kk_function_t _fself, kk_string_t _c_17350, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_error_pattern_fun20980(kk_string_t definition, kk_string_t location, kk_context_t* _ctx) {
  struct kk_std_core_error_pattern_fun20980__t* _self = kk_function_alloc_as(struct kk_std_core_error_pattern_fun20980__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_error_pattern_fun20980, kk_context());
  _self->definition = definition;
  _self->location = location;
  return &_self->_base;
}

static kk_box_t kk_std_core_error_pattern_fun20980(kk_function_t _fself, kk_string_t _c_17350, kk_context_t* _ctx) {
  struct kk_std_core_error_pattern_fun20980__t* _self = kk_function_as(struct kk_std_core_error_pattern_fun20980__t*, _fself);
  kk_string_t definition = _self->definition; /* string */
  kk_string_t location = _self->location; /* string */
  kk_drop_match(_self, {kk_string_dup(definition);kk_string_dup(location);}, {}, _ctx)
  return kk_std_core__mlift17551_error_pattern(definition, location, _c_17350, _ctx);
}


// lift anonymous function
struct kk_std_core_error_pattern_fun20981__t {
  struct kk_function_s _base;
  kk_function_t _b_18944_18934;
};
static kk_box_t kk_std_core_error_pattern_fun20981(kk_function_t _fself, kk_box_t _b_18935, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_error_pattern_fun20981(kk_function_t _b_18944_18934, kk_context_t* _ctx) {
  struct kk_std_core_error_pattern_fun20981__t* _self = kk_function_alloc_as(struct kk_std_core_error_pattern_fun20981__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_error_pattern_fun20981, kk_context());
  _self->_b_18944_18934 = _b_18944_18934;
  return &_self->_base;
}

static kk_box_t kk_std_core_error_pattern_fun20981(kk_function_t _fself, kk_box_t _b_18935, kk_context_t* _ctx) {
  struct kk_std_core_error_pattern_fun20981__t* _self = kk_function_as(struct kk_std_core_error_pattern_fun20981__t*, _fself);
  kk_function_t _b_18944_18934 = _self->_b_18944_18934; /* (string) -> 11784 */
  kk_drop_match(_self, {kk_function_dup(_b_18944_18934);}, {}, _ctx)
  kk_string_t _x20982 = kk_string_unbox(_b_18935); /*string*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_string_t, kk_context_t*), _b_18944_18934, (_b_18944_18934, _x20982, _ctx));
}

kk_box_t kk_std_core_error_pattern(kk_string_t location, kk_string_t definition, kk_context_t* _ctx) { /* forall<a> (location : string, definition : string) -> exn a */ 
  kk_string_t x_17874;
  bool _match_19775;
  kk_string_t _x20973 = kk_string_dup(definition); /*string*/
  kk_string_t _x20974 = kk_string_empty(); /*string*/
  _match_19775 = kk_string_is_eq(_x20973,_x20974,kk_context()); /*bool*/
  if (_match_19775) {
    x_17874 = kk_string_empty(); /*string*/
  }
  else {
    kk_string_t _x20977;
    kk_define_string_literal(, _s20978, 2, ": ")
    _x20977 = kk_string_dup(_s20978); /*string*/
    kk_string_t _x20979 = kk_string_dup(definition); /*string*/
    x_17874 = kk_std_core__lp__plus__plus__1_rp_(_x20977, _x20979, _ctx); /*string*/
  }
  if (kk_yielding(kk_context())) {
    kk_string_drop(x_17874, _ctx);
    kk_function_t _b_18944_18934 = kk_std_core_new_error_pattern_fun20980(definition, location, _ctx); /*(string) -> 11784*/;
    return kk_std_core_hnd_yield_extend(kk_std_core_new_error_pattern_fun20981(_b_18944_18934, _ctx), _ctx);
  }
  kk_string_t message0_16997;
  kk_string_t _x20983 = kk_string_dup(location); /*string*/
  kk_string_t _x20984;
  kk_string_t _x20985;
  kk_define_string_literal(, _s20986, 23, ": pattern match failure")
  _x20985 = kk_string_dup(_s20986); /*string*/
  _x20984 = kk_std_core__lp__plus__plus__1_rp_(x_17874, _x20985, _ctx); /*string*/
  message0_16997 = kk_std_core__lp__plus__plus__1_rp_(_x20983, _x20984, _ctx); /*string*/
  kk_std_core_hnd__ev ev_17877;
  kk_ssize_t _x20987 = ((kk_ssize_t)0); /*ssize_t*/
  ev_17877 = kk_evv_at(_x20987,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
  struct kk_std_core_hnd_Ev* _con20988 = kk_std_core_hnd__as_Ev(ev_17877);
  kk_std_core_hnd__marker m0 = _con20988->marker;
  kk_box_t _box_x18936 = _con20988->hnd;
  kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x18936, NULL);
  kk_std_core__hnd_exn_dup(h);
  kk_std_core_hnd__clause1 _match_19773;
  kk_std_core_hnd__clause1 _borrowApp_19774 = kk_std_core__select_throw_exn(h, _ctx); /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/;
  kk_std_core__hnd_exn_drop(h, _ctx);
  _match_19773 = _borrowApp_19774; /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/
  kk_function_t _fun_unbox_x18940 = _match_19773.clause;
  kk_std_core_hnd__marker _b_18941 = m0; /*std/core/hnd/marker<3573,3574>*/;
  kk_std_core_hnd__ev _b_18942 = ev_17877; /*std/core/hnd/ev<.hnd-exn>*/;
  kk_std_core__exception _b_18943;
  kk_std_core__exception_info _x20990 = kk_std_core__new_ExnPattern(kk_reuse_null, location, definition, _ctx); /*exception-info*/
  _b_18943 = kk_std_core__new_Exception(message0_16997, _x20990, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x18940, (_fun_unbox_x18940, _b_18941, _b_18942, kk_std_core__exception_box(_b_18943, _ctx), _ctx));
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17552_op(kk_std_core_types__ctail _acc, kk_function_t pred, kk_box_t x, kk_std_core__list xx, bool _y_17353, kk_context_t* _ctx) { /* forall<a,e> (ctail<list<a>>, pred : (a) -> e bool, x : a, xx : list<a>, bool) -> e list<a> */ 
  if (_y_17353) {
    kk_std_core__list _ctail_17184 = kk_std_core__list_hole(); /*list<11850>*/;
    kk_std_core__list _ctail_17185 = kk_std_core__new_Cons(kk_reuse_null, x, _ctail_17184, _ctx); /*list<11850>*/;
    kk_std_core_types__ctail _x20991;
    kk_box_t* field_17882 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17185)->tail)); /*cfield<list<11850>>*/;
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x18951 = _acc._cons.CTail._field1;
      kk_box_t* last0 = _acc._cons.CTail._field2;
      kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18951, NULL);
      kk_std_core__list_dup(head0);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __ = kk_Unit;
      *(last0) = kk_std_core__list_box(_ctail_17185, _ctx);
      kk_std_core__list _b_18960_18956 = head0; /*list<11850>*/;
      kk_box_t* _b_18961_18957 = field_17882; /*cfield<list<11850>>*/;
      _x20991 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18960_18956, _ctx), _b_18961_18957, _ctx); /*ctail<58>*/
      goto _match20992;
    }
    kk_std_core__list _b_18962_18958 = _ctail_17185; /*list<11850>*/;
    kk_box_t* _b_18963_18959 = field_17882; /*cfield<list<11850>>*/;
    _x20991 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18962_18958, _ctx), _b_18963_18959, _ctx); /*ctail<58>*/
    _match20992: ;
    return kk_std_core__ctail_filter(xx, pred, _x20991, _ctx);
  }
  kk_box_drop(x, _ctx);
  return kk_std_core__ctail_filter(xx, pred, _acc, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17553_op_fun20994__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t x0;
};
static kk_std_core__list kk_std_core__mlift17553_op_fun20994(kk_function_t _fself, kk_std_core__list _ctail_17187, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17553_op_fun20994(kk_function_t _accm, kk_box_t x0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17553_op_fun20994__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17553_op_fun20994__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17553_op_fun20994, kk_context());
  _self->_accm = _accm;
  _self->x0 = x0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17553_op_fun20994(kk_function_t _fself, kk_std_core__list _ctail_17187, kk_context_t* _ctx) {
  struct kk_std_core__mlift17553_op_fun20994__t* _self = kk_function_as(struct kk_std_core__mlift17553_op_fun20994__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<11850>) -> list<11850> */
  kk_box_t x0 = _self->x0; /* 11850 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(x0);}, {}, _ctx)
  kk_std_core__list _x20995 = kk_std_core__new_Cons(kk_reuse_null, x0, _ctail_17187, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x20995, _ctx));
}

kk_std_core__list kk_std_core__mlift17553_op(kk_function_t _accm, kk_function_t pred0, kk_box_t x0, kk_std_core__list xx0, bool _y_17358, kk_context_t* _ctx) { /* forall<a,e> ((list<a>) -> list<a>, pred : (a) -> e bool, x : a, xx : list<a>, bool) -> e list<a> */ 
  if (_y_17358) {
    return kk_std_core__ctailm_filter(xx0, pred0, kk_std_core__new_mlift17553_op_fun20994(_accm, x0, _ctx), _ctx);
  }
  kk_box_drop(x0, _ctx);
  return kk_std_core__ctailm_filter(xx0, pred0, _accm, _ctx);
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filter([1,2,3],odd?) == [1,3]`


// lift anonymous function
struct kk_std_core__ctail_filter_fun20999__t {
  struct kk_function_s _base;
  kk_function_t pred1;
  kk_box_t x1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_filter_fun20999(kk_function_t _fself, bool _y_173530, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_filter_fun20999(kk_function_t pred1, kk_box_t x1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_fun20999__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_filter_fun20999__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_filter_fun20999, kk_context());
  _self->pred1 = pred1;
  _self->x1 = x1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_filter_fun20999(kk_function_t _fself, bool _y_173530, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_fun20999__t* _self = kk_function_as(struct kk_std_core__ctail_filter_fun20999__t*, _fself);
  kk_function_t pred1 = _self->pred1; /* (11850) -> 11851 bool */
  kk_box_t x1 = _self->x1; /* 11850 */
  kk_std_core__list xx1 = _self->xx1; /* list<11850> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<11850>> */
  kk_drop_match(_self, {kk_function_dup(pred1);kk_box_dup(x1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17552_op(_acc0, pred1, x1, xx1, _y_173530, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_filter_fun21001__t {
  struct kk_function_s _base;
  kk_function_t _b_18986_18964;
};
static kk_box_t kk_std_core__ctail_filter_fun21001(kk_function_t _fself, kk_box_t _b_18965, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_filter_fun21001(kk_function_t _b_18986_18964, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_fun21001__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_filter_fun21001__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_filter_fun21001, kk_context());
  _self->_b_18986_18964 = _b_18986_18964;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_filter_fun21001(kk_function_t _fself, kk_box_t _b_18965, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_fun21001__t* _self = kk_function_as(struct kk_std_core__ctail_filter_fun21001__t*, _fself);
  kk_function_t _b_18986_18964 = _self->_b_18986_18964; /* (bool) -> 11851 list<11850> */
  kk_drop_match(_self, {kk_function_dup(_b_18986_18964);}, {}, _ctx)
  kk_std_core__list _x21002;
  bool _x21003 = kk_bool_unbox(_b_18965); /*bool*/
  _x21002 = kk_function_call(kk_std_core__list, (kk_function_t, bool, kk_context_t*), _b_18986_18964, (_b_18986_18964, _x21003, _ctx)); /*list<11850>*/
  return kk_std_core__list_box(_x21002, _ctx);
}

kk_std_core__list kk_std_core__ctail_filter(kk_std_core__list xs, kk_function_t pred1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, pred : (a) -> e bool, ctail<list<a>>) -> e list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20996 = kk_std_core__as_Cons(xs);
    kk_box_t x1 = _con20996->head;
    kk_std_core__list xx1 = _con20996->tail;
    kk_reuse_t _ru_20056 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20056 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    bool x2_17883;
    kk_function_t _x20998 = kk_function_dup(pred1); /*(11850) -> 11851 bool*/
    kk_box_t _x20997 = kk_box_dup(x1); /*11850*/
    x2_17883 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x20998, (_x20998, _x20997, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20056);
      kk_function_t _b_18986_18964 = kk_std_core__new_ctail_filter_fun20999(pred1, x1, xx1, _acc0, _ctx); /*(bool) -> 11851 list<11850>*/;
      kk_box_t _x21000 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_filter_fun21001(_b_18986_18964, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21000, _ctx);
    }
    if (x2_17883) {
      kk_std_core__list _ctail_171840 = kk_std_core__list_hole(); /*list<11850>*/;
      kk_std_core__list _ctail_171850;
      if (kk_likely(_ru_20056!=NULL)) {
        struct kk_std_core_Cons* _con21004 = (struct kk_std_core_Cons*)_ru_20056;
        _con21004->tail = _ctail_171840;
        _ctail_171850 = kk_std_core__base_Cons(_con21004); /*list<11850>*/
      }
      else {
        _ctail_171850 = kk_std_core__new_Cons(kk_reuse_null, x1, _ctail_171840, _ctx); /*list<11850>*/
      }
      { // tailcall
        kk_std_core_types__ctail _x21005;
        kk_box_t* field0_17888 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_171850)->tail)); /*cfield<list<11850>>*/;
        if (kk_std_core_types__is_CTail(_acc0)) {
          kk_box_t _box_x18972 = _acc0._cons.CTail._field1;
          kk_box_t* last2 = _acc0._cons.CTail._field2;
          kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18972, NULL);
          kk_std_core__list_dup(head1);
          kk_std_core_types__ctail_drop(_acc0, _ctx);
          kk_Unit;
          kk_unit_t __0 = kk_Unit;
          *(last2) = kk_std_core__list_box(_ctail_171850, _ctx);
          kk_std_core__list _b_18987_18977 = head1; /*list<11850>*/;
          kk_box_t* _b_18988_18978 = field0_17888; /*cfield<list<11850>>*/;
          _x21005 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18987_18977, _ctx), _b_18988_18978, _ctx); /*ctail<58>*/
          goto _match21006;
        }
        kk_std_core__list _b_18989_18979 = _ctail_171850; /*list<11850>*/;
        kk_box_t* _b_18990_18980 = field0_17888; /*cfield<list<11850>>*/;
        _x21005 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18989_18979, _ctx), _b_18990_18980, _ctx); /*ctail<58>*/
        _match21006: ;
        xs = xx1;
        _acc0 = _x21005;
        goto kk__tailcall;
      }
    }
    kk_reuse_drop(_ru_20056);
    kk_box_drop(x1, _ctx);
    { // tailcall
      xs = xx1;
      goto kk__tailcall;
    }
  }
  kk_function_drop(pred1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18981 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x18981, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filter([1,2,3],odd?) == [1,3]`


// lift anonymous function
struct kk_std_core__ctailm_filter_fun21012__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t pred2;
  kk_box_t x3;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_filter_fun21012(kk_function_t _fself, bool _y_173580, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_fun21012(kk_function_t _accm0, kk_function_t pred2, kk_box_t x3, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21012__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_fun21012__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_fun21012, kk_context());
  _self->_accm0 = _accm0;
  _self->pred2 = pred2;
  _self->x3 = x3;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_filter_fun21012(kk_function_t _fself, bool _y_173580, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21012__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_fun21012__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<11850>) -> list<11850> */
  kk_function_t pred2 = _self->pred2; /* (11850) -> 11851 bool */
  kk_box_t x3 = _self->x3; /* 11850 */
  kk_std_core__list xx2 = _self->xx2; /* list<11850> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(pred2);kk_box_dup(x3);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17553_op(_accm0, pred2, x3, xx2, _y_173580, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_filter_fun21014__t {
  struct kk_function_s _base;
  kk_function_t _b_18993_18991;
};
static kk_box_t kk_std_core__ctailm_filter_fun21014(kk_function_t _fself, kk_box_t _b_18992, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_fun21014(kk_function_t _b_18993_18991, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21014__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_fun21014__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_fun21014, kk_context());
  _self->_b_18993_18991 = _b_18993_18991;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_filter_fun21014(kk_function_t _fself, kk_box_t _b_18992, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21014__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_fun21014__t*, _fself);
  kk_function_t _b_18993_18991 = _self->_b_18993_18991; /* (bool) -> 11851 list<11850> */
  kk_drop_match(_self, {kk_function_dup(_b_18993_18991);}, {}, _ctx)
  kk_std_core__list _x21015;
  bool _x21016 = kk_bool_unbox(_b_18992); /*bool*/
  _x21015 = kk_function_call(kk_std_core__list, (kk_function_t, bool, kk_context_t*), _b_18993_18991, (_b_18993_18991, _x21016, _ctx)); /*list<11850>*/
  return kk_std_core__list_box(_x21015, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_filter_fun21018__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x3;
};
static kk_std_core__list kk_std_core__ctailm_filter_fun21018(kk_function_t _fself, kk_std_core__list _ctail_171870, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_fun21018(kk_function_t _accm0, kk_box_t x3, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21018__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_fun21018__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_fun21018, kk_context());
  _self->_accm0 = _accm0;
  _self->x3 = x3;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_filter_fun21018(kk_function_t _fself, kk_std_core__list _ctail_171870, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21018__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_fun21018__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<11850>) -> list<11850> */
  kk_box_t x3 = _self->x3; /* 11850 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x3);}, {}, _ctx)
  kk_std_core__list _x21019 = kk_std_core__new_Cons(kk_reuse_null, x3, _ctail_171870, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21019, _ctx));
}

kk_std_core__list kk_std_core__ctailm_filter(kk_std_core__list xs0, kk_function_t pred2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, pred : (a) -> e bool, (list<a>) -> list<a>) -> e list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21009 = kk_std_core__as_Cons(xs0);
    kk_box_t x3 = _con21009->head;
    kk_std_core__list xx2 = _con21009->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x3);
      kk_std_core__list_dup(xx2);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    bool x4_17891;
    kk_function_t _x21011 = kk_function_dup(pred2); /*(11850) -> 11851 bool*/
    kk_box_t _x21010 = kk_box_dup(x3); /*11850*/
    x4_17891 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21011, (_x21011, _x21010, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_18993_18991 = kk_std_core__new_ctailm_filter_fun21012(_accm0, pred2, x3, xx2, _ctx); /*(bool) -> 11851 list<11850>*/;
      kk_box_t _x21013 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_filter_fun21014(_b_18993_18991, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21013, _ctx);
    }
    if (x4_17891) {
      { // tailcall
        kk_function_t _x21017 = kk_std_core__new_ctailm_filter_fun21018(_accm0, x3, _ctx); /*(list<11850>) -> list<11850>*/
        xs0 = xx2;
        _accm0 = _x21017;
        goto kk__tailcall;
      }
    }
    kk_box_drop(x3, _ctx);
    { // tailcall
      xs0 = xx2;
      goto kk__tailcall;
    }
  }
  kk_function_drop(pred2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filter([1,2,3],odd?) == [1,3]`


// lift anonymous function
struct kk_std_core_filter_fun21020__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_filter_fun21020(kk_function_t _fself, kk_std_core__list _ctail_17186, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_filter_fun21020(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_filter_fun21020, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_filter_fun21020(kk_function_t _fself, kk_std_core__list _ctail_17186, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17186;
}

kk_std_core__list kk_std_core_filter(kk_std_core__list xs1, kk_function_t pred3, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, pred : (a) -> e bool) -> e list<a> */ 
  bool _match_19769 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19769) {
    return kk_std_core__ctail_filter(xs1, pred3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_filter(xs1, pred3, kk_std_core_new_filter_fun21020(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17554_op(kk_std_core_types__ctail _acc, kk_function_t pred, kk_std_core__list xx, kk_std_core_types__maybe _y_17366, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, pred : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Nothing(_y_17366)) {
    return kk_std_core__ctail_filter_map(xx, pred, _acc, _ctx);
  }
  kk_box_t y = _y_17366._cons.Just.value;
  kk_std_core__list _ctail_17188 = kk_std_core__list_hole(); /*list<11896>*/;
  kk_std_core__list _ctail_17189 = kk_std_core__new_Cons(kk_reuse_null, y, _ctail_17188, _ctx); /*list<11896>*/;
  kk_std_core_types__ctail _x21021;
  kk_box_t* field_17896 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17189)->tail)); /*cfield<list<11896>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19000 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19000, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17189, _ctx);
    kk_std_core__list _b_19009_19005 = head0; /*list<11896>*/;
    kk_box_t* _b_19010_19006 = field_17896; /*cfield<list<11896>>*/;
    _x21021 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19009_19005, _ctx), _b_19010_19006, _ctx); /*ctail<58>*/
    goto _match21022;
  }
  kk_std_core__list _b_19011_19007 = _ctail_17189; /*list<11896>*/;
  kk_box_t* _b_19012_19008 = field_17896; /*cfield<list<11896>>*/;
  _x21021 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19011_19007, _ctx), _b_19012_19008, _ctx); /*ctail<58>*/
  _match21022: ;
  return kk_std_core__ctail_filter_map(xx, pred, _x21021, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17555_op_fun21024__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t y0;
};
static kk_std_core__list kk_std_core__mlift17555_op_fun21024(kk_function_t _fself, kk_std_core__list _ctail_17191, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17555_op_fun21024(kk_function_t _accm, kk_box_t y0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17555_op_fun21024__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17555_op_fun21024__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17555_op_fun21024, kk_context());
  _self->_accm = _accm;
  _self->y0 = y0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17555_op_fun21024(kk_function_t _fself, kk_std_core__list _ctail_17191, kk_context_t* _ctx) {
  struct kk_std_core__mlift17555_op_fun21024__t* _self = kk_function_as(struct kk_std_core__mlift17555_op_fun21024__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<11896>) -> list<11896> */
  kk_box_t y0 = _self->y0; /* 11896 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(y0);}, {}, _ctx)
  kk_std_core__list _x21025 = kk_std_core__new_Cons(kk_reuse_null, y0, _ctail_17191, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x21025, _ctx));
}

kk_std_core__list kk_std_core__mlift17555_op(kk_function_t _accm, kk_function_t pred0, kk_std_core__list xx0, kk_std_core_types__maybe _y_17371, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, pred : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Nothing(_y_17371)) {
    return kk_std_core__ctailm_filter_map(xx0, pred0, _accm, _ctx);
  }
  kk_box_t y0 = _y_17371._cons.Just.value;
  return kk_std_core__ctailm_filter_map(xx0, pred0, kk_std_core__new_mlift17555_op_fun21024(_accm, y0, _ctx), _ctx);
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filterMap([1,2,3],fn(i) { if (i.odd?) then Nothing else Just(i*i) }) == [4]`


// lift anonymous function
struct kk_std_core__ctail_filter_map_fun21029__t {
  struct kk_function_s _base;
  kk_function_t pred1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_filter_map_fun21029(kk_function_t _fself, kk_std_core_types__maybe _y_173660, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_filter_map_fun21029(kk_function_t pred1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_map_fun21029__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_filter_map_fun21029__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_filter_map_fun21029, kk_context());
  _self->pred1 = pred1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_filter_map_fun21029(kk_function_t _fself, kk_std_core_types__maybe _y_173660, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_map_fun21029__t* _self = kk_function_as(struct kk_std_core__ctail_filter_map_fun21029__t*, _fself);
  kk_function_t pred1 = _self->pred1; /* (11895) -> 11897 maybe<11896> */
  kk_std_core__list xx1 = _self->xx1; /* list<11895> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<11896>> */
  kk_drop_match(_self, {kk_function_dup(pred1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17554_op(_acc0, pred1, xx1, _y_173660, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_filter_map_fun21031__t {
  struct kk_function_s _base;
  kk_function_t _b_19035_19018;
};
static kk_box_t kk_std_core__ctail_filter_map_fun21031(kk_function_t _fself, kk_box_t _b_19019, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_filter_map_fun21031(kk_function_t _b_19035_19018, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_map_fun21031__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_filter_map_fun21031__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_filter_map_fun21031, kk_context());
  _self->_b_19035_19018 = _b_19035_19018;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_filter_map_fun21031(kk_function_t _fself, kk_box_t _b_19019, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_map_fun21031__t* _self = kk_function_as(struct kk_std_core__ctail_filter_map_fun21031__t*, _fself);
  kk_function_t _b_19035_19018 = _self->_b_19035_19018; /* (maybe<11896>) -> 11897 list<11896> */
  kk_drop_match(_self, {kk_function_dup(_b_19035_19018);}, {}, _ctx)
  kk_std_core__list _x21032;
  kk_std_core_types__maybe _x21033 = kk_std_core_types__maybe_unbox(_b_19019, _ctx); /*maybe<11896>*/
  _x21032 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19035_19018, (_b_19035_19018, _x21033, _ctx)); /*list<11896>*/
  return kk_std_core__list_box(_x21032, _ctx);
}

kk_std_core__list kk_std_core__ctail_filter_map(kk_std_core__list xs, kk_function_t pred1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, pred : (a) -> e maybe<b>, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(pred1, _ctx);
    if (kk_std_core_types__is_CTail(_acc0)) {
      kk_box_t _box_x19013 = _acc0._cons.CTail._field1;
      kk_box_t* last2 = _acc0._cons.CTail._field2;
      kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19013, NULL);
      kk_std_core__list_dup(head1);
      kk_std_core_types__ctail_drop(_acc0, _ctx);
      kk_Unit;
      kk_unit_t __0 = kk_Unit;
      *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
      return head1;
    }
    return kk_std_core__new_Nil(_ctx);
  }
  struct kk_std_core_Cons* _con21027 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21027->head;
  kk_std_core__list xx1 = _con21027->tail;
  kk_reuse_t _ru_20058 = kk_reuse_null; /*reuse*/;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    _ru_20058 = (kk_std_core__list_reuse(xs));
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx1);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core_types__maybe x0_17899;
  kk_function_t _x21028 = kk_function_dup(pred1); /*(11895) -> 11897 maybe<11896>*/
  x0_17899 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21028, (_x21028, x, _ctx)); /*maybe<11896>*/
  if (kk_yielding(kk_context())) {
    kk_reuse_drop(_ru_20058);
    kk_std_core_types__maybe_drop(x0_17899, _ctx);
    kk_function_t _b_19035_19018 = kk_std_core__new_ctail_filter_map_fun21029(pred1, xx1, _acc0, _ctx); /*(maybe<11896>) -> 11897 list<11896>*/;
    kk_box_t _x21030 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_filter_map_fun21031(_b_19035_19018, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21030, _ctx);
  }
  if (kk_std_core_types__is_Nothing(x0_17899)) {
    kk_reuse_drop(_ru_20058);
    { // tailcall
      xs = xx1;
      goto kk__tailcall;
    }
  }
  kk_box_t y1 = x0_17899._cons.Just.value;
  kk_std_core__list _ctail_171880 = kk_std_core__list_hole(); /*list<11896>*/;
  kk_std_core__list _ctail_171890 = kk_std_core__new_Cons(_ru_20058, y1, _ctail_171880, _ctx); /*list<11896>*/;
  { // tailcall
    kk_std_core_types__ctail _x21034;
    kk_box_t* field0_17904 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_171890)->tail)); /*cfield<list<11896>>*/;
    if (kk_std_core_types__is_CTail(_acc0)) {
      kk_box_t _box_x19026 = _acc0._cons.CTail._field1;
      kk_box_t* last3 = _acc0._cons.CTail._field2;
      kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19026, NULL);
      kk_std_core__list_dup(head2);
      kk_std_core_types__ctail_drop(_acc0, _ctx);
      kk_Unit;
      kk_unit_t __1 = kk_Unit;
      *(last3) = kk_std_core__list_box(_ctail_171890, _ctx);
      kk_std_core__list _b_19036_19031 = head2; /*list<11896>*/;
      kk_box_t* _b_19037_19032 = field0_17904; /*cfield<list<11896>>*/;
      _x21034 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19036_19031, _ctx), _b_19037_19032, _ctx); /*ctail<58>*/
      goto _match21035;
    }
    kk_std_core__list _b_19038_19033 = _ctail_171890; /*list<11896>*/;
    kk_box_t* _b_19039_19034 = field0_17904; /*cfield<list<11896>>*/;
    _x21034 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19038_19033, _ctx), _b_19039_19034, _ctx); /*ctail<58>*/
    _match21035: ;
    xs = xx1;
    _acc0 = _x21034;
    goto kk__tailcall;
  }
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filterMap([1,2,3],fn(i) { if (i.odd?) then Nothing else Just(i*i) }) == [4]`


// lift anonymous function
struct kk_std_core__ctailm_filter_map_fun21039__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t pred2;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_filter_map_fun21039(kk_function_t _fself, kk_std_core_types__maybe _y_173710, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_map_fun21039(kk_function_t _accm0, kk_function_t pred2, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21039__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_map_fun21039__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_map_fun21039, kk_context());
  _self->_accm0 = _accm0;
  _self->pred2 = pred2;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_filter_map_fun21039(kk_function_t _fself, kk_std_core_types__maybe _y_173710, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21039__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_map_fun21039__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<11896>) -> list<11896> */
  kk_function_t pred2 = _self->pred2; /* (11895) -> 11897 maybe<11896> */
  kk_std_core__list xx2 = _self->xx2; /* list<11895> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(pred2);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17555_op(_accm0, pred2, xx2, _y_173710, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_filter_map_fun21041__t {
  struct kk_function_s _base;
  kk_function_t _b_19042_19040;
};
static kk_box_t kk_std_core__ctailm_filter_map_fun21041(kk_function_t _fself, kk_box_t _b_19041, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_map_fun21041(kk_function_t _b_19042_19040, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21041__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_map_fun21041__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_map_fun21041, kk_context());
  _self->_b_19042_19040 = _b_19042_19040;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_filter_map_fun21041(kk_function_t _fself, kk_box_t _b_19041, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21041__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_map_fun21041__t*, _fself);
  kk_function_t _b_19042_19040 = _self->_b_19042_19040; /* (maybe<11896>) -> 11897 list<11896> */
  kk_drop_match(_self, {kk_function_dup(_b_19042_19040);}, {}, _ctx)
  kk_std_core__list _x21042;
  kk_std_core_types__maybe _x21043 = kk_std_core_types__maybe_unbox(_b_19041, _ctx); /*maybe<11896>*/
  _x21042 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19042_19040, (_b_19042_19040, _x21043, _ctx)); /*list<11896>*/
  return kk_std_core__list_box(_x21042, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_filter_map_fun21045__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t y2;
};
static kk_std_core__list kk_std_core__ctailm_filter_map_fun21045(kk_function_t _fself, kk_std_core__list _ctail_171910, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_map_fun21045(kk_function_t _accm0, kk_box_t y2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21045__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_map_fun21045__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_map_fun21045, kk_context());
  _self->_accm0 = _accm0;
  _self->y2 = y2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_filter_map_fun21045(kk_function_t _fself, kk_std_core__list _ctail_171910, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21045__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_map_fun21045__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<11896>) -> list<11896> */
  kk_box_t y2 = _self->y2; /* 11896 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(y2);}, {}, _ctx)
  kk_std_core__list _x21046 = kk_std_core__new_Cons(kk_reuse_null, y2, _ctail_171910, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21046, _ctx));
}

kk_std_core__list kk_std_core__ctailm_filter_map(kk_std_core__list xs0, kk_function_t pred2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, pred : (a) -> e maybe<b>, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs0)) {
    kk_function_drop(pred2, _ctx);
    return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
  }
  struct kk_std_core_Cons* _con21037 = kk_std_core__as_Cons(xs0);
  kk_box_t x1 = _con21037->head;
  kk_std_core__list xx2 = _con21037->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs0))) {
    kk_std_core__list_free(xs0);
  }
  else {
    kk_box_dup(x1);
    kk_std_core__list_dup(xx2);
    kk_Unit;
    kk_std_core__list_decref(xs0, _ctx);
    kk_Unit;
  }
  kk_std_core_types__maybe x2_17905;
  kk_function_t _x21038 = kk_function_dup(pred2); /*(11895) -> 11897 maybe<11896>*/
  x2_17905 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21038, (_x21038, x1, _ctx)); /*maybe<11896>*/
  if (kk_yielding(kk_context())) {
    kk_std_core_types__maybe_drop(x2_17905, _ctx);
    kk_function_t _b_19042_19040 = kk_std_core__new_ctailm_filter_map_fun21039(_accm0, pred2, xx2, _ctx); /*(maybe<11896>) -> 11897 list<11896>*/;
    kk_box_t _x21040 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_filter_map_fun21041(_b_19042_19040, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21040, _ctx);
  }
  if (kk_std_core_types__is_Nothing(x2_17905)) {
    { // tailcall
      xs0 = xx2;
      goto kk__tailcall;
    }
  }
  kk_box_t y2 = x2_17905._cons.Just.value;
  { // tailcall
    kk_function_t _x21044 = kk_std_core__new_ctailm_filter_map_fun21045(_accm0, y2, _ctx); /*(list<11896>) -> list<11896>*/
    xs0 = xx2;
    _accm0 = _x21044;
    goto kk__tailcall;
  }
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filterMap([1,2,3],fn(i) { if (i.odd?) then Nothing else Just(i*i) }) == [4]`


// lift anonymous function
struct kk_std_core_filter_map_fun21047__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_filter_map_fun21047(kk_function_t _fself, kk_std_core__list _ctail_17190, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_filter_map_fun21047(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_filter_map_fun21047, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_filter_map_fun21047(kk_function_t _fself, kk_std_core__list _ctail_17190, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17190;
}

kk_std_core__list kk_std_core_filter_map(kk_std_core__list xs1, kk_function_t pred3, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, pred : (a) -> e maybe<b>) -> e list<b> */ 
  bool _match_19766 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19766) {
    return kk_std_core__ctail_filter_map(xs1, pred3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_filter_map(xs1, pred3, kk_std_core_new_filter_map_fun21047(_ctx), _ctx);
}

bool kk_std_core_is_zero_1(kk_ssize_t i, kk_context_t* _ctx) { /* (i : ssize_t) -> bool */ 
  return (i == 0);
}
 
// Find the first element satisfying some predicate


// lift anonymous function
struct kk_std_core_find_fun21048__t {
  struct kk_function_s _base;
  kk_function_t pred;
};
static kk_std_core_types__maybe kk_std_core_find_fun21048(kk_function_t _fself, kk_box_t x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_find_fun21048(kk_function_t pred, kk_context_t* _ctx) {
  struct kk_std_core_find_fun21048__t* _self = kk_function_alloc_as(struct kk_std_core_find_fun21048__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_find_fun21048, kk_context());
  _self->pred = pred;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core_find_fun21048(kk_function_t _fself, kk_box_t x, kk_context_t* _ctx) {
  struct kk_std_core_find_fun21048__t* _self = kk_function_as(struct kk_std_core_find_fun21048__t*, _fself);
  kk_function_t pred = _self->pred; /* (11994) -> bool */
  kk_drop_match(_self, {kk_function_dup(pred);}, {}, _ctx)
  bool _match_19765;
  kk_box_t _x21049 = kk_box_dup(x); /*11994*/
  _match_19765 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), pred, (pred, _x21049, _ctx)); /*bool*/
  if (_match_19765) {
    return kk_std_core_types__new_Just(x, _ctx);
  }
  kk_box_drop(x, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_core_types__maybe kk_std_core_find(kk_std_core__list xs, kk_function_t pred, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, pred : (a) -> bool) -> maybe<a> */ 
  return kk_std_core_foreach_while(xs, kk_std_core_new_find_fun21048(pred, _ctx), _ctx);
}
 
// O(n). If it occurs, return the position of substring `sub` in `s`, tupled with
// the position just following the substring `sub`.

kk_std_core_types__maybe kk_std_core_find_1(kk_string_t s, kk_string_t sub, kk_context_t* _ctx) { /* (s : string, sub : string) -> maybe<sslice> */ 
  kk_ssize_t i;
  kk_string_t _x21050 = kk_string_dup(s); /*string*/
  kk_string_t _x21051 = kk_string_dup(sub); /*string*/
  i = kk_string_index_of1(_x21050,_x21051,kk_context()); /*ssize_t*/
  bool _match_19764 = kk_std_core_is_zero_1(i, _ctx); /*bool*/;
  if (_match_19764) {
    kk_string_drop(s, _ctx);
    kk_string_drop(sub, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_std_core__sslice _b_19044_19043;
  kk_ssize_t _x21052 = kk_std_core_decr_1(i, _ctx); /*ssize_t*/
  kk_ssize_t _x21053 = kk_string_len(sub,kk_context()); /*ssize_t*/
  _b_19044_19043 = kk_std_core__new_Sslice(s, _x21052, _x21053, _ctx); /*sslice*/
  return kk_std_core_types__new_Just(kk_std_core__sslice_box(_b_19044_19043, _ctx), _ctx);
}
 
// Return the last index of substring `sub` in `s` if it occurs.

kk_std_core_types__maybe kk_std_core_find_last(kk_string_t s, kk_string_t sub, kk_context_t* _ctx) { /* (s : string, sub : string) -> maybe<sslice> */ 
  kk_ssize_t i;
  kk_string_t _x21054 = kk_string_dup(s); /*string*/
  kk_string_t _x21055 = kk_string_dup(sub); /*string*/
  i = kk_string_last_index_of1(_x21054,_x21055,kk_context()); /*ssize_t*/
  bool _match_19763 = kk_std_core_is_zero_1(i, _ctx); /*bool*/;
  if (_match_19763) {
    kk_string_drop(s, _ctx);
    kk_string_drop(sub, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_std_core__sslice _b_19046_19045;
  kk_ssize_t _x21056 = kk_std_core_decr_1(i, _ctx); /*ssize_t*/
  kk_ssize_t _x21057 = kk_string_len(sub,kk_context()); /*ssize_t*/
  _b_19046_19045 = kk_std_core__new_Sslice(s, _x21056, _x21057, _ctx); /*sslice*/
  return kk_std_core_types__new_Just(kk_std_core__sslice_box(_b_19046_19045, _ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17556_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_std_core__list xx, kk_std_core_types__maybe _y_17379, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, f : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Just(_y_17379)) {
    kk_box_t y = _y_17379._cons.Just.value;
    kk_std_core__list _ctail_17192 = kk_std_core__list_hole(); /*list<12200>*/;
    kk_std_core__list _ctail_17193 = kk_std_core__new_Cons(kk_reuse_null, y, _ctail_17192, _ctx); /*list<12200>*/;
    kk_std_core_types__ctail _x21058;
    kk_box_t* field_17910 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17193)->tail)); /*cfield<list<12200>>*/;
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x19053 = _acc._cons.CTail._field1;
      kk_box_t* last0 = _acc._cons.CTail._field2;
      kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19053, NULL);
      kk_std_core__list_dup(head0);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __ = kk_Unit;
      *(last0) = kk_std_core__list_box(_ctail_17193, _ctx);
      kk_std_core__list _b_19062_19058 = head0; /*list<12200>*/;
      kk_box_t* _b_19063_19059 = field_17910; /*cfield<list<12200>>*/;
      _x21058 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19062_19058, _ctx), _b_19063_19059, _ctx); /*ctail<58>*/
      goto _match21059;
    }
    kk_std_core__list _b_19064_19060 = _ctail_17193; /*list<12200>*/;
    kk_box_t* _b_19065_19061 = field_17910; /*cfield<list<12200>>*/;
    _x21058 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19064_19060, _ctx), _b_19065_19061, _ctx); /*ctail<58>*/
    _match21059: ;
    return kk_std_core__ctail_flatmap_maybe(xx, f, _x21058, _ctx);
  }
  return kk_std_core__ctail_flatmap_maybe(xx, f, _acc, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17557_op_fun21061__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t y0;
};
static kk_std_core__list kk_std_core__mlift17557_op_fun21061(kk_function_t _fself, kk_std_core__list _ctail_17195, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17557_op_fun21061(kk_function_t _accm, kk_box_t y0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17557_op_fun21061__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17557_op_fun21061__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17557_op_fun21061, kk_context());
  _self->_accm = _accm;
  _self->y0 = y0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17557_op_fun21061(kk_function_t _fself, kk_std_core__list _ctail_17195, kk_context_t* _ctx) {
  struct kk_std_core__mlift17557_op_fun21061__t* _self = kk_function_as(struct kk_std_core__mlift17557_op_fun21061__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<12200>) -> list<12200> */
  kk_box_t y0 = _self->y0; /* 12200 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(y0);}, {}, _ctx)
  kk_std_core__list _x21062 = kk_std_core__new_Cons(kk_reuse_null, y0, _ctail_17195, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x21062, _ctx));
}

kk_std_core__list kk_std_core__mlift17557_op(kk_function_t _accm, kk_function_t f0, kk_std_core__list xx0, kk_std_core_types__maybe _y_17384, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, f : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Just(_y_17384)) {
    kk_box_t y0 = _y_17384._cons.Just.value;
    return kk_std_core__ctailm_flatmap_maybe(xx0, f0, kk_std_core__new_mlift17557_op_fun21061(_accm, y0, _ctx), _ctx);
  }
  return kk_std_core__ctailm_flatmap_maybe(xx0, f0, _accm, _ctx);
}
 
// Concatenate the `Just` result elements from applying a function to all elements.


// lift anonymous function
struct kk_std_core__ctail_flatmap_maybe_fun21065__t {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_flatmap_maybe_fun21065(kk_function_t _fself, kk_std_core_types__maybe _y_173790, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_flatmap_maybe_fun21065(kk_function_t f1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_flatmap_maybe_fun21065__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_flatmap_maybe_fun21065__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_flatmap_maybe_fun21065, kk_context());
  _self->f1 = f1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_flatmap_maybe_fun21065(kk_function_t _fself, kk_std_core_types__maybe _y_173790, kk_context_t* _ctx) {
  struct kk_std_core__ctail_flatmap_maybe_fun21065__t* _self = kk_function_as(struct kk_std_core__ctail_flatmap_maybe_fun21065__t*, _fself);
  kk_function_t f1 = _self->f1; /* (12199) -> 12201 maybe<12200> */
  kk_std_core__list xx1 = _self->xx1; /* list<12199> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<12200>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17556_op(_acc0, f1, xx1, _y_173790, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_flatmap_maybe_fun21067__t {
  struct kk_function_s _base;
  kk_function_t _b_19088_19066;
};
static kk_box_t kk_std_core__ctail_flatmap_maybe_fun21067(kk_function_t _fself, kk_box_t _b_19067, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_flatmap_maybe_fun21067(kk_function_t _b_19088_19066, kk_context_t* _ctx) {
  struct kk_std_core__ctail_flatmap_maybe_fun21067__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_flatmap_maybe_fun21067__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_flatmap_maybe_fun21067, kk_context());
  _self->_b_19088_19066 = _b_19088_19066;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_flatmap_maybe_fun21067(kk_function_t _fself, kk_box_t _b_19067, kk_context_t* _ctx) {
  struct kk_std_core__ctail_flatmap_maybe_fun21067__t* _self = kk_function_as(struct kk_std_core__ctail_flatmap_maybe_fun21067__t*, _fself);
  kk_function_t _b_19088_19066 = _self->_b_19088_19066; /* (maybe<12200>) -> 12201 list<12200> */
  kk_drop_match(_self, {kk_function_dup(_b_19088_19066);}, {}, _ctx)
  kk_std_core__list _x21068;
  kk_std_core_types__maybe _x21069 = kk_std_core_types__maybe_unbox(_b_19067, _ctx); /*maybe<12200>*/
  _x21068 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19088_19066, (_b_19088_19066, _x21069, _ctx)); /*list<12200>*/
  return kk_std_core__list_box(_x21068, _ctx);
}

kk_std_core__list kk_std_core__ctail_flatmap_maybe(kk_std_core__list xs, kk_function_t f1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e maybe<b>, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21063 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21063->head;
    kk_std_core__list xx1 = _con21063->tail;
    kk_reuse_t _ru_20060 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20060 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_std_core_types__maybe x0_17911;
    kk_function_t _x21064 = kk_function_dup(f1); /*(12199) -> 12201 maybe<12200>*/
    x0_17911 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21064, (_x21064, x, _ctx)); /*maybe<12200>*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20060);
      kk_std_core_types__maybe_drop(x0_17911, _ctx);
      kk_function_t _b_19088_19066 = kk_std_core__new_ctail_flatmap_maybe_fun21065(f1, xx1, _acc0, _ctx); /*(maybe<12200>) -> 12201 list<12200>*/;
      kk_box_t _x21066 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_flatmap_maybe_fun21067(_b_19088_19066, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21066, _ctx);
    }
    if (kk_std_core_types__is_Just(x0_17911)) {
      kk_box_t y1 = x0_17911._cons.Just.value;
      kk_std_core__list _ctail_171920 = kk_std_core__list_hole(); /*list<12200>*/;
      kk_std_core__list _ctail_171930 = kk_std_core__new_Cons(_ru_20060, y1, _ctail_171920, _ctx); /*list<12200>*/;
      { // tailcall
        kk_std_core_types__ctail _x21070;
        kk_box_t* field0_17916 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_171930)->tail)); /*cfield<list<12200>>*/;
        if (kk_std_core_types__is_CTail(_acc0)) {
          kk_box_t _box_x19074 = _acc0._cons.CTail._field1;
          kk_box_t* last2 = _acc0._cons.CTail._field2;
          kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19074, NULL);
          kk_std_core__list_dup(head1);
          kk_std_core_types__ctail_drop(_acc0, _ctx);
          kk_Unit;
          kk_unit_t __0 = kk_Unit;
          *(last2) = kk_std_core__list_box(_ctail_171930, _ctx);
          kk_std_core__list _b_19089_19079 = head1; /*list<12200>*/;
          kk_box_t* _b_19090_19080 = field0_17916; /*cfield<list<12200>>*/;
          _x21070 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19089_19079, _ctx), _b_19090_19080, _ctx); /*ctail<58>*/
          goto _match21071;
        }
        kk_std_core__list _b_19091_19081 = _ctail_171930; /*list<12200>*/;
        kk_box_t* _b_19092_19082 = field0_17916; /*cfield<list<12200>>*/;
        _x21070 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19091_19081, _ctx), _b_19092_19082, _ctx); /*ctail<58>*/
        _match21071: ;
        xs = xx1;
        _acc0 = _x21070;
        goto kk__tailcall;
      }
    }
    kk_reuse_drop(_ru_20060);
    { // tailcall
      xs = xx1;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x19083 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19083, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Concatenate the `Just` result elements from applying a function to all elements.


// lift anonymous function
struct kk_std_core__ctailm_flatmap_maybe_fun21076__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_flatmap_maybe_fun21076(kk_function_t _fself, kk_std_core_types__maybe _y_173840, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_flatmap_maybe_fun21076(kk_function_t _accm0, kk_function_t f2, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21076__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_flatmap_maybe_fun21076__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_flatmap_maybe_fun21076, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_flatmap_maybe_fun21076(kk_function_t _fself, kk_std_core_types__maybe _y_173840, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21076__t* _self = kk_function_as(struct kk_std_core__ctailm_flatmap_maybe_fun21076__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<12200>) -> list<12200> */
  kk_function_t f2 = _self->f2; /* (12199) -> 12201 maybe<12200> */
  kk_std_core__list xx2 = _self->xx2; /* list<12199> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17557_op(_accm0, f2, xx2, _y_173840, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_flatmap_maybe_fun21078__t {
  struct kk_function_s _base;
  kk_function_t _b_19095_19093;
};
static kk_box_t kk_std_core__ctailm_flatmap_maybe_fun21078(kk_function_t _fself, kk_box_t _b_19094, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_flatmap_maybe_fun21078(kk_function_t _b_19095_19093, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21078__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_flatmap_maybe_fun21078__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_flatmap_maybe_fun21078, kk_context());
  _self->_b_19095_19093 = _b_19095_19093;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_flatmap_maybe_fun21078(kk_function_t _fself, kk_box_t _b_19094, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21078__t* _self = kk_function_as(struct kk_std_core__ctailm_flatmap_maybe_fun21078__t*, _fself);
  kk_function_t _b_19095_19093 = _self->_b_19095_19093; /* (maybe<12200>) -> 12201 list<12200> */
  kk_drop_match(_self, {kk_function_dup(_b_19095_19093);}, {}, _ctx)
  kk_std_core__list _x21079;
  kk_std_core_types__maybe _x21080 = kk_std_core_types__maybe_unbox(_b_19094, _ctx); /*maybe<12200>*/
  _x21079 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19095_19093, (_b_19095_19093, _x21080, _ctx)); /*list<12200>*/
  return kk_std_core__list_box(_x21079, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_flatmap_maybe_fun21082__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t y2;
};
static kk_std_core__list kk_std_core__ctailm_flatmap_maybe_fun21082(kk_function_t _fself, kk_std_core__list _ctail_171950, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_flatmap_maybe_fun21082(kk_function_t _accm0, kk_box_t y2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21082__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_flatmap_maybe_fun21082__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_flatmap_maybe_fun21082, kk_context());
  _self->_accm0 = _accm0;
  _self->y2 = y2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_flatmap_maybe_fun21082(kk_function_t _fself, kk_std_core__list _ctail_171950, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21082__t* _self = kk_function_as(struct kk_std_core__ctailm_flatmap_maybe_fun21082__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<12200>) -> list<12200> */
  kk_box_t y2 = _self->y2; /* 12200 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(y2);}, {}, _ctx)
  kk_std_core__list _x21083 = kk_std_core__new_Cons(kk_reuse_null, y2, _ctail_171950, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21083, _ctx));
}

kk_std_core__list kk_std_core__ctailm_flatmap_maybe(kk_std_core__list xs0, kk_function_t f2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e maybe<b>, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21074 = kk_std_core__as_Cons(xs0);
    kk_box_t x1 = _con21074->head;
    kk_std_core__list xx2 = _con21074->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx2);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    kk_std_core_types__maybe x2_17919;
    kk_function_t _x21075 = kk_function_dup(f2); /*(12199) -> 12201 maybe<12200>*/
    x2_17919 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21075, (_x21075, x1, _ctx)); /*maybe<12200>*/
    if (kk_yielding(kk_context())) {
      kk_std_core_types__maybe_drop(x2_17919, _ctx);
      kk_function_t _b_19095_19093 = kk_std_core__new_ctailm_flatmap_maybe_fun21076(_accm0, f2, xx2, _ctx); /*(maybe<12200>) -> 12201 list<12200>*/;
      kk_box_t _x21077 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_flatmap_maybe_fun21078(_b_19095_19093, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21077, _ctx);
    }
    if (kk_std_core_types__is_Just(x2_17919)) {
      kk_box_t y2 = x2_17919._cons.Just.value;
      { // tailcall
        kk_function_t _x21081 = kk_std_core__new_ctailm_flatmap_maybe_fun21082(_accm0, y2, _ctx); /*(list<12200>) -> list<12200>*/
        xs0 = xx2;
        _accm0 = _x21081;
        goto kk__tailcall;
      }
    }
    { // tailcall
      xs0 = xx2;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Concatenate the `Just` result elements from applying a function to all elements.


// lift anonymous function
struct kk_std_core_flatmap_maybe_fun21084__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_flatmap_maybe_fun21084(kk_function_t _fself, kk_std_core__list _ctail_17194, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_flatmap_maybe_fun21084(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_flatmap_maybe_fun21084, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_flatmap_maybe_fun21084(kk_function_t _fself, kk_std_core__list _ctail_17194, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17194;
}

kk_std_core__list kk_std_core_flatmap_maybe(kk_std_core__list xs1, kk_function_t f3, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e maybe<b>) -> e list<b> */ 
  bool _match_19760 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19760) {
    return kk_std_core__ctail_flatmap_maybe(xs1, f3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_flatmap_maybe(xs1, f3, kk_std_core_new_flatmap_maybe_fun21084(_ctx), _ctx);
}

kk_box_t kk_std_core_fold_int(kk_integer_t start0, kk_integer_t end, kk_box_t init0, kk_function_t f, kk_context_t* _ctx) { /* forall<a> (start : int, end : int, init : a, f : (int, a) -> a) -> a */ 
  kk__tailcall: ;
  bool _match_19759 = kk_integer_gte_borrow(start0,end,kk_context()); /*bool*/;
  if (_match_19759) {
    kk_integer_drop(end, _ctx);
    kk_function_drop(f, _ctx);
    kk_integer_drop(start0, _ctx);
    return init0;
  }
  kk_box_t x;
  kk_function_t _x21086 = kk_function_dup(f); /*(int, 12244) -> 12244*/
  kk_integer_t _x21085 = kk_integer_dup(start0); /*int*/
  x = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_box_t, kk_context_t*), _x21086, (_x21086, _x21085, init0, _ctx)); /*12244*/
  { // tailcall
    kk_integer_t _x21087 = kk_integer_add(start0,(kk_integer_from_small(1)),kk_context()); /*int*/
    start0 = _x21087;
    init0 = x;
    goto kk__tailcall;
  }
}
 
// monadic lift

kk_box_t kk_std_core__mlift17558_foldl(kk_function_t f, kk_std_core__list xx, kk_box_t _y_17392, kk_context_t* _ctx) { /* forall<a,e,b> (f : (a, b) -> e a, xx : list<b>, a) -> e a */ 
  return kk_std_core_foldl(xx, _y_17392, f, _ctx);
}
 
// Fold a list from the left, i.e. `foldl([1,2],0,(+)) == (0+1)+2`
// Since `foldl` is tail recursive, it is preferred over `foldr` when using an associative function `f`


// lift anonymous function
struct kk_std_core_foldl_fun21090__t {
  struct kk_function_s _base;
  kk_function_t f0;
  kk_std_core__list xx0;
};
static kk_box_t kk_std_core_foldl_fun21090(kk_function_t _fself, kk_box_t _y_173920, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foldl_fun21090(kk_function_t f0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_foldl_fun21090__t* _self = kk_function_alloc_as(struct kk_std_core_foldl_fun21090__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foldl_fun21090, kk_context());
  _self->f0 = f0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_box_t kk_std_core_foldl_fun21090(kk_function_t _fself, kk_box_t _y_173920, kk_context_t* _ctx) {
  struct kk_std_core_foldl_fun21090__t* _self = kk_function_as(struct kk_std_core_foldl_fun21090__t*, _fself);
  kk_function_t f0 = _self->f0; /* (12247, 12252) -> 12249 12247 */
  kk_std_core__list xx0 = _self->xx0; /* list<12252> */
  kk_drop_match(_self, {kk_function_dup(f0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17558_foldl(f0, xx0, _y_173920, _ctx);
}

kk_box_t kk_std_core_foldl(kk_std_core__list xs, kk_box_t z, kk_function_t f0, kk_context_t* _ctx) { /* forall<a,b,e> (list<a>, b, (b, a) -> e b) -> e b */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21088 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21088->head;
    kk_std_core__list xx0 = _con21088->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_box_t x0_17922;
    kk_function_t _x21089 = kk_function_dup(f0); /*(12247, 12252) -> 12249 12247*/
    x0_17922 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x21089, (_x21089, z, x, _ctx)); /*12247*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x0_17922, _ctx);
      return kk_std_core_hnd_yield_extend(kk_std_core_new_foldl_fun21090(f0, xx0, _ctx), _ctx);
    }
    { // tailcall
      xs = xx0;
      z = x0_17922;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f0, _ctx);
  return z;
}


// lift anonymous function
struct kk_std_core_foldl1_fun21095__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_foldl1_fun21095(kk_function_t _fself, kk_string_t _x121093, kk_std_core_types__optional _x221094, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foldl1_fun21095(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_foldl1_fun21095, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_foldl1_fun21095(kk_function_t _fself, kk_string_t _x121093, kk_std_core_types__optional _x221094, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_throw(_x121093, _x221094, _ctx);
}


// lift anonymous function
struct kk_std_core_foldl1_fun21097__t {
  struct kk_function_s _base;
  kk_function_t _b_19103_19097;
};
static kk_box_t kk_std_core_foldl1_fun21097(kk_function_t _fself, kk_box_t _b_19100, kk_box_t _b_19101, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foldl1_fun21097(kk_function_t _b_19103_19097, kk_context_t* _ctx) {
  struct kk_std_core_foldl1_fun21097__t* _self = kk_function_alloc_as(struct kk_std_core_foldl1_fun21097__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foldl1_fun21097, kk_context());
  _self->_b_19103_19097 = _b_19103_19097;
  return &_self->_base;
}

static kk_box_t kk_std_core_foldl1_fun21097(kk_function_t _fself, kk_box_t _b_19100, kk_box_t _b_19101, kk_context_t* _ctx) {
  struct kk_std_core_foldl1_fun21097__t* _self = kk_function_as(struct kk_std_core_foldl1_fun21097__t*, _fself);
  kk_function_t _b_19103_19097 = _self->_b_19103_19097; /* (message : string, info : optional<exception-info>) -> exn 12319 */
  kk_drop_match(_self, {kk_function_dup(_b_19103_19097);}, {}, _ctx)
  kk_string_t _x21098 = kk_string_unbox(_b_19100); /*string*/
  kk_std_core_types__optional _x21099 = kk_std_core_types__optional_unbox(_b_19101, _ctx); /*optional<exception-info>*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_string_t, kk_std_core_types__optional, kk_context_t*), _b_19103_19097, (_b_19103_19097, _x21098, _x21099, _ctx));
}

kk_box_t kk_std_core_foldl1(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, f : (a, a) -> <exn|e> a) -> <exn|e> a */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21091 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21091->head;
    kk_std_core__list xx = _con21091->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    return kk_std_core_foldl(xx, x, f, _ctx);
  }
  kk_function_drop(f, _ctx);
  kk_ssize_t _b_19102_19096;
  kk_std_core_hnd__htag _x21092 = kk_std_core_hnd__htag_dup(kk_std_core__tag_exn); /*std/core/hnd/htag<.hnd-exn>*/
  _b_19102_19096 = kk_std_core_hnd__evv_index(_x21092, _ctx); /*std/core/hnd/ev-index*/
  kk_function_t _b_19103_19097 = kk_std_core_new_foldl1_fun21095(_ctx); /*(message : string, info : optional<exception-info>) -> exn 12319*/;
  kk_string_t _b_19104_19098;
  kk_define_string_literal(, _s21096, 33, "unexpected Nil in std/core/foldl1")
  _b_19104_19098 = kk_string_dup(_s21096); /*string*/
  kk_std_core_types__optional _b_19105_19099 = kk_std_core_types__new_None(_ctx); /*optional<exception-info>*/;
  return kk_std_core_hnd__open_at2(_b_19102_19096, kk_std_core_new_foldl1_fun21097(_b_19103_19097, _ctx), kk_string_box(_b_19104_19098), kk_std_core_types__optional_box(_b_19105_19099, _ctx), _ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17078_reverse(kk_std_core__list acc, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a> (acc : list<a>, ys : list<a>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21100 = kk_std_core__as_Cons(ys);
    kk_box_t x = _con21100->head;
    kk_std_core__list xx = _con21100->tail;
    kk_reuse_t _ru_20064 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20064 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x21101;
      if (kk_likely(_ru_20064!=NULL)) {
        struct kk_std_core_Cons* _con21102 = (struct kk_std_core_Cons*)_ru_20064;
        _con21102->tail = acc;
        _x21101 = kk_std_core__base_Cons(_con21102); /*list<61>*/
      }
      else {
        _x21101 = kk_std_core__new_Cons(kk_reuse_null, x, acc, _ctx); /*list<61>*/
      }
      acc = _x21101;
      ys = xx;
      goto kk__tailcall;
    }
  }
  return acc;
}
extern kk_box_t kk_std_core_foldr_fun21104(kk_function_t _fself, kk_box_t x, kk_box_t y, kk_context_t* _ctx) {
  struct kk_std_core_foldr_fun21104__t* _self = kk_function_as(struct kk_std_core_foldr_fun21104__t*, _fself);
  kk_function_t f = _self->f; /* (12338, 12334) -> 12336 12334 */
  kk_drop_match(_self, {kk_function_dup(f);}, {}, _ctx)
  return kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), f, (f, y, x, _ctx));
}


// lift anonymous function
struct kk_std_core_foldr1_fun21109__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_foldr1_fun21109(kk_function_t _fself, kk_string_t _x121107, kk_std_core_types__optional _x221108, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foldr1_fun21109(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_foldr1_fun21109, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_foldr1_fun21109(kk_function_t _fself, kk_string_t _x121107, kk_std_core_types__optional _x221108, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_throw(_x121107, _x221108, _ctx);
}


// lift anonymous function
struct kk_std_core_foldr1_fun21111__t {
  struct kk_function_s _base;
  kk_function_t _b_19113_19107;
};
static kk_box_t kk_std_core_foldr1_fun21111(kk_function_t _fself, kk_box_t _b_19110, kk_box_t _b_19111, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foldr1_fun21111(kk_function_t _b_19113_19107, kk_context_t* _ctx) {
  struct kk_std_core_foldr1_fun21111__t* _self = kk_function_alloc_as(struct kk_std_core_foldr1_fun21111__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foldr1_fun21111, kk_context());
  _self->_b_19113_19107 = _b_19113_19107;
  return &_self->_base;
}

static kk_box_t kk_std_core_foldr1_fun21111(kk_function_t _fself, kk_box_t _b_19110, kk_box_t _b_19111, kk_context_t* _ctx) {
  struct kk_std_core_foldr1_fun21111__t* _self = kk_function_as(struct kk_std_core_foldr1_fun21111__t*, _fself);
  kk_function_t _b_19113_19107 = _self->_b_19113_19107; /* (message : string, info : optional<exception-info>) -> exn 12372 */
  kk_drop_match(_self, {kk_function_dup(_b_19113_19107);}, {}, _ctx)
  kk_string_t _x21112 = kk_string_unbox(_b_19110); /*string*/
  kk_std_core_types__optional _x21113 = kk_std_core_types__optional_unbox(_b_19111, _ctx); /*optional<exception-info>*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_string_t, kk_std_core_types__optional, kk_context_t*), _b_19113_19107, (_b_19113_19107, _x21112, _x21113, _ctx));
}

kk_box_t kk_std_core_foldr1(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, f : (a, a) -> <exn|e> a) -> <exn|e> a */ 
  kk_std_core__list xs0_17003 = kk_std_core__lift17078_reverse(kk_std_core__new_Nil(_ctx), xs, _ctx); /*list<12372>*/;
  if (kk_std_core__is_Cons(xs0_17003)) {
    struct kk_std_core_Cons* _con21105 = kk_std_core__as_Cons(xs0_17003);
    kk_box_t x = _con21105->head;
    kk_std_core__list xx = _con21105->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0_17003))) {
      kk_std_core__list_free(xs0_17003);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs0_17003, _ctx);
      kk_Unit;
    }
    return kk_std_core_foldl(xx, x, f, _ctx);
  }
  kk_function_drop(f, _ctx);
  kk_ssize_t _b_19112_19106;
  kk_std_core_hnd__htag _x21106 = kk_std_core_hnd__htag_dup(kk_std_core__tag_exn); /*std/core/hnd/htag<.hnd-exn>*/
  _b_19112_19106 = kk_std_core_hnd__evv_index(_x21106, _ctx); /*std/core/hnd/ev-index*/
  kk_function_t _b_19113_19107 = kk_std_core_new_foldr1_fun21109(_ctx); /*(message : string, info : optional<exception-info>) -> exn 12372*/;
  kk_string_t _b_19114_19108;
  kk_define_string_literal(, _s21110, 33, "unexpected Nil in std/core/foldl1")
  _b_19114_19108 = kk_string_dup(_s21110); /*string*/
  kk_std_core_types__optional _b_19115_19109 = kk_std_core_types__new_None(_ctx); /*optional<exception-info>*/;
  return kk_std_core_hnd__open_at2(_b_19112_19106, kk_std_core_new_foldr1_fun21111(_b_19113_19107, _ctx), kk_string_box(_b_19114_19108), kk_std_core_types__optional_box(_b_19115_19109, _ctx), _ctx);
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17559_op(kk_function_t action, kk_integer_t end, kk_integer_t i, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action : (int) -> e (), end : int, i : int, wild_ : ()) -> e () */ 
  kk_integer_t i0_17126 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__lift17079_for(action, end, i0_17126, _ctx); return kk_Unit;
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17079_for_fun21116__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_integer_t end0;
  kk_integer_t i0;
};
static kk_unit_t kk_std_core__lift17079_for_fun21116(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17079_for_fun21116(kk_function_t action0, kk_integer_t end0, kk_integer_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17079_for_fun21116__t* _self = kk_function_alloc_as(struct kk_std_core__lift17079_for_fun21116__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17079_for_fun21116, kk_context());
  _self->action0 = action0;
  _self->end0 = end0;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17079_for_fun21116(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17079_for_fun21116__t* _self = kk_function_as(struct kk_std_core__lift17079_for_fun21116__t*, _fself);
  kk_function_t action0 = _self->action0; /* (int) -> 12409 () */
  kk_integer_t end0 = _self->end0; /* int */
  kk_integer_t i0 = _self->i0; /* int */
  kk_drop_match(_self, {kk_function_dup(action0);kk_integer_dup(end0);kk_integer_dup(i0);}, {}, _ctx)
  return kk_std_core__mlift17559_op(action0, end0, i0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17079_for_fun21118__t {
  struct kk_function_s _base;
  kk_function_t _b_19118_19116;
};
static kk_box_t kk_std_core__lift17079_for_fun21118(kk_function_t _fself, kk_box_t _b_19117, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17079_for_fun21118(kk_function_t _b_19118_19116, kk_context_t* _ctx) {
  struct kk_std_core__lift17079_for_fun21118__t* _self = kk_function_alloc_as(struct kk_std_core__lift17079_for_fun21118__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17079_for_fun21118, kk_context());
  _self->_b_19118_19116 = _b_19118_19116;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17079_for_fun21118(kk_function_t _fself, kk_box_t _b_19117, kk_context_t* _ctx) {
  struct kk_std_core__lift17079_for_fun21118__t* _self = kk_function_as(struct kk_std_core__lift17079_for_fun21118__t*, _fself);
  kk_function_t _b_19118_19116 = _self->_b_19118_19116; /* (wild_0 : ()) -> 12409 () */
  kk_drop_match(_self, {kk_function_dup(_b_19118_19116);}, {}, _ctx)
  kk_unit_t _x21119 = kk_Unit;
  kk_unit_t _x21120 = kk_Unit;
  kk_unit_unbox(_b_19117);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19118_19116, (_b_19118_19116, _x21120, _ctx));
  return kk_unit_box(_x21119);
}

kk_unit_t kk_std_core__lift17079_for(kk_function_t action0, kk_integer_t end0, kk_integer_t i0, kk_context_t* _ctx) { /* forall<e> (action : (int) -> e (), end : int, i : int) -> e () */ 
  kk__tailcall: ;
  bool _match_19756 = kk_integer_lte_borrow(i0,end0,kk_context()); /*bool*/;
  if (_match_19756) {
    kk_unit_t x_17927 = kk_Unit;
    kk_function_t _x21115 = kk_function_dup(action0); /*(int) -> 12409 ()*/
    kk_integer_t _x21114 = kk_integer_dup(i0); /*int*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_context_t*), _x21115, (_x21115, _x21114, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19118_19116 = kk_std_core__new_lift17079_for_fun21116(action0, end0, i0, _ctx); /*(wild_0 : ()) -> 12409 ()*/;
      kk_box_t _x21117 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17079_for_fun21118(_b_19118_19116, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x21117); return kk_Unit;
    }
    kk_integer_t i0_171260 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
    { // tailcall
      i0 = i0_171260;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action0, _ctx);
  kk_integer_drop(end0, _ctx);
  kk_integer_drop(i0, _ctx);
  kk_Unit; return kk_Unit;
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17560_op(kk_function_t action, kk_integer_t end, kk_integer_t i, kk_std_core_types__maybe _y_17407, kk_context_t* _ctx) { /* forall<a,e> (action : (int) -> e maybe<a>, end : int, i : int, maybe<a>) -> e maybe<a> */ 
  if (kk_std_core_types__is_Nothing(_y_17407)) {
    kk_integer_t i0_17127 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
    return kk_std_core__lift17080_for_while(action, end, i0_17127, _ctx);
  }
  kk_box_t x = _y_17407._cons.Just.value;
  kk_function_drop(action, _ctx);
  kk_integer_drop(end, _ctx);
  kk_integer_drop(i, _ctx);
  return kk_std_core_types__new_Just(x, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17080_for_while_fun21124__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_integer_t end0;
  kk_integer_t i0;
};
static kk_std_core_types__maybe kk_std_core__lift17080_for_while_fun21124(kk_function_t _fself, kk_std_core_types__maybe _y_174070, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17080_for_while_fun21124(kk_function_t action0, kk_integer_t end0, kk_integer_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17080_for_while_fun21124__t* _self = kk_function_alloc_as(struct kk_std_core__lift17080_for_while_fun21124__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17080_for_while_fun21124, kk_context());
  _self->action0 = action0;
  _self->end0 = end0;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core__lift17080_for_while_fun21124(kk_function_t _fself, kk_std_core_types__maybe _y_174070, kk_context_t* _ctx) {
  struct kk_std_core__lift17080_for_while_fun21124__t* _self = kk_function_as(struct kk_std_core__lift17080_for_while_fun21124__t*, _fself);
  kk_function_t action0 = _self->action0; /* (int) -> 12460 maybe<12459> */
  kk_integer_t end0 = _self->end0; /* int */
  kk_integer_t i0 = _self->i0; /* int */
  kk_drop_match(_self, {kk_function_dup(action0);kk_integer_dup(end0);kk_integer_dup(i0);}, {}, _ctx)
  return kk_std_core__mlift17560_op(action0, end0, i0, _y_174070, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17080_for_while_fun21126__t {
  struct kk_function_s _base;
  kk_function_t _b_19121_19119;
};
static kk_box_t kk_std_core__lift17080_for_while_fun21126(kk_function_t _fself, kk_box_t _b_19120, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17080_for_while_fun21126(kk_function_t _b_19121_19119, kk_context_t* _ctx) {
  struct kk_std_core__lift17080_for_while_fun21126__t* _self = kk_function_alloc_as(struct kk_std_core__lift17080_for_while_fun21126__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17080_for_while_fun21126, kk_context());
  _self->_b_19121_19119 = _b_19121_19119;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17080_for_while_fun21126(kk_function_t _fself, kk_box_t _b_19120, kk_context_t* _ctx) {
  struct kk_std_core__lift17080_for_while_fun21126__t* _self = kk_function_as(struct kk_std_core__lift17080_for_while_fun21126__t*, _fself);
  kk_function_t _b_19121_19119 = _self->_b_19121_19119; /* (maybe<12459>) -> 12460 maybe<12459> */
  kk_drop_match(_self, {kk_function_dup(_b_19121_19119);}, {}, _ctx)
  kk_std_core_types__maybe _x21127;
  kk_std_core_types__maybe _x21128 = kk_std_core_types__maybe_unbox(_b_19120, _ctx); /*maybe<12459>*/
  _x21127 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19121_19119, (_b_19121_19119, _x21128, _ctx)); /*maybe<12459>*/
  return kk_std_core_types__maybe_box(_x21127, _ctx);
}

kk_std_core_types__maybe kk_std_core__lift17080_for_while(kk_function_t action0, kk_integer_t end0, kk_integer_t i0, kk_context_t* _ctx) { /* forall<a,e> (action : (int) -> e maybe<a>, end : int, i : int) -> e maybe<a> */ 
  kk__tailcall: ;
  bool _match_19754 = kk_integer_lte_borrow(i0,end0,kk_context()); /*bool*/;
  if (_match_19754) {
    kk_std_core_types__maybe x0_17930;
    kk_function_t _x21123 = kk_function_dup(action0); /*(int) -> 12460 maybe<12459>*/
    kk_integer_t _x21122 = kk_integer_dup(i0); /*int*/
    x0_17930 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_integer_t, kk_context_t*), _x21123, (_x21123, _x21122, _ctx)); /*maybe<12459>*/
    if (kk_yielding(kk_context())) {
      kk_std_core_types__maybe_drop(x0_17930, _ctx);
      kk_function_t _b_19121_19119 = kk_std_core__new_lift17080_for_while_fun21124(action0, end0, i0, _ctx); /*(maybe<12459>) -> 12460 maybe<12459>*/;
      kk_box_t _x21125 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17080_for_while_fun21126(_b_19121_19119, _ctx), _ctx); /*3858*/
      return kk_std_core_types__maybe_unbox(_x21125, _ctx);
    }
    if (kk_std_core_types__is_Nothing(x0_17930)) {
      kk_integer_t i0_171270 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
      { // tailcall
        i0 = i0_171270;
        goto kk__tailcall;
      }
    }
    kk_box_t x1 = x0_17930._cons.Just.value;
    kk_integer_drop(i0, _ctx);
    kk_Unit;
    kk_integer_drop(end0, _ctx);
    kk_Unit;
    kk_function_drop(action0, _ctx);
    kk_Unit;
    return kk_std_core_types__new_Just(x1, _ctx);
  }
  kk_function_drop(action0, _ctx);
  kk_integer_drop(end0, _ctx);
  kk_integer_drop(i0, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17562_foreach_indexed_fun21131__t {
  struct kk_function_s _base;
  kk_ref_t i;
};
static kk_unit_t kk_std_core__mlift17562_foreach_indexed_fun21131(kk_function_t _fself, kk_integer_t _y_17415, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17562_foreach_indexed_fun21131(kk_ref_t i, kk_context_t* _ctx) {
  struct kk_std_core__mlift17562_foreach_indexed_fun21131__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17562_foreach_indexed_fun21131__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17562_foreach_indexed_fun21131, kk_context());
  _self->i = i;
  return &_self->_base;
}

static kk_unit_t kk_std_core__mlift17562_foreach_indexed_fun21131(kk_function_t _fself, kk_integer_t _y_17415, kk_context_t* _ctx) {
  struct kk_std_core__mlift17562_foreach_indexed_fun21131__t* _self = kk_function_as(struct kk_std_core__mlift17562_foreach_indexed_fun21131__t*, _fself);
  kk_ref_t i = _self->i; /* local-var<global,int> */
  kk_drop_match(_self, {kk_ref_dup(i);}, {}, _ctx)
  kk_integer_t _b_19131_19129 = kk_integer_add(_y_17415,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_unit_t _borrowApp_19752 = kk_Unit;
  (kk_ref_set_borrow(i,(kk_integer_box(_b_19131_19129)),kk_context()));
  kk_ref_drop(i, _ctx);
  return _borrowApp_19752;
}


// lift anonymous function
struct kk_std_core__mlift17562_foreach_indexed_fun21133__t {
  struct kk_function_s _base;
  kk_function_t _b_19134_19132;
};
static kk_box_t kk_std_core__mlift17562_foreach_indexed_fun21133(kk_function_t _fself, kk_box_t _b_19133, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17562_foreach_indexed_fun21133(kk_function_t _b_19134_19132, kk_context_t* _ctx) {
  struct kk_std_core__mlift17562_foreach_indexed_fun21133__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17562_foreach_indexed_fun21133__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17562_foreach_indexed_fun21133, kk_context());
  _self->_b_19134_19132 = _b_19134_19132;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17562_foreach_indexed_fun21133(kk_function_t _fself, kk_box_t _b_19133, kk_context_t* _ctx) {
  struct kk_std_core__mlift17562_foreach_indexed_fun21133__t* _self = kk_function_as(struct kk_std_core__mlift17562_foreach_indexed_fun21133__t*, _fself);
  kk_function_t _b_19134_19132 = _self->_b_19134_19132; /* (int) -> <local<global>|12571> () */
  kk_drop_match(_self, {kk_function_dup(_b_19134_19132);}, {}, _ctx)
  kk_unit_t _x21134 = kk_Unit;
  kk_integer_t _x21135 = kk_integer_unbox(_b_19133); /*int*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_context_t*), _b_19134_19132, (_b_19134_19132, _x21135, _ctx));
  return kk_unit_box(_x21134);
}

kk_unit_t kk_std_core__mlift17562_foreach_indexed(kk_ref_t i, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (i : local-var<global,int>, wild_ : ()) -> <local<global>|e> () */ 
  kk_integer_t x_17933;
  kk_box_t _x21129;
  kk_ref_t _x21130 = kk_ref_dup(i); /*local-var<global,int>*/
  _x21129 = (kk_ref_get(_x21130,kk_context())); /*256*/
  x_17933 = kk_integer_unbox(_x21129); /*int*/
  kk_function_t next0_17934 = kk_std_core__new_mlift17562_foreach_indexed_fun21131(i, _ctx); /*(int) -> <local<global>|12571> ()*/;
  if (kk_yielding(kk_context())) {
    kk_integer_drop(x_17933, _ctx);
    kk_function_t _b_19134_19132 = next0_17934; /*(int) -> <local<global>|12571> ()*/;
    kk_box_t _x21132 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17562_foreach_indexed_fun21133(_b_19134_19132, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21132); return kk_Unit;
  }
  kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_context_t*), next0_17934, (next0_17934, x_17933, _ctx)); return kk_Unit;
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17563_foreach_indexed_fun21136__t {
  struct kk_function_s _base;
  kk_ref_t i;
};
static kk_unit_t kk_std_core__mlift17563_foreach_indexed_fun21136(kk_function_t _fself, kk_unit_t wild__, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17563_foreach_indexed_fun21136(kk_ref_t i, kk_context_t* _ctx) {
  struct kk_std_core__mlift17563_foreach_indexed_fun21136__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17563_foreach_indexed_fun21136__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17563_foreach_indexed_fun21136, kk_context());
  _self->i = i;
  return &_self->_base;
}

static kk_unit_t kk_std_core__mlift17563_foreach_indexed_fun21136(kk_function_t _fself, kk_unit_t wild__, kk_context_t* _ctx) {
  struct kk_std_core__mlift17563_foreach_indexed_fun21136__t* _self = kk_function_as(struct kk_std_core__mlift17563_foreach_indexed_fun21136__t*, _fself);
  kk_ref_t i = _self->i; /* local-var<global,int> */
  kk_drop_match(_self, {kk_ref_dup(i);}, {}, _ctx)
  return kk_std_core__mlift17562_foreach_indexed(i, wild__, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17563_foreach_indexed_fun21138__t {
  struct kk_function_s _base;
  kk_function_t _b_19137_19135;
};
static kk_box_t kk_std_core__mlift17563_foreach_indexed_fun21138(kk_function_t _fself, kk_box_t _b_19136, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17563_foreach_indexed_fun21138(kk_function_t _b_19137_19135, kk_context_t* _ctx) {
  struct kk_std_core__mlift17563_foreach_indexed_fun21138__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17563_foreach_indexed_fun21138__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17563_foreach_indexed_fun21138, kk_context());
  _self->_b_19137_19135 = _b_19137_19135;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17563_foreach_indexed_fun21138(kk_function_t _fself, kk_box_t _b_19136, kk_context_t* _ctx) {
  struct kk_std_core__mlift17563_foreach_indexed_fun21138__t* _self = kk_function_as(struct kk_std_core__mlift17563_foreach_indexed_fun21138__t*, _fself);
  kk_function_t _b_19137_19135 = _self->_b_19137_19135; /* (wild_ : ()) -> <local<global>|12571> () */
  kk_drop_match(_self, {kk_function_dup(_b_19137_19135);}, {}, _ctx)
  kk_unit_t _x21139 = kk_Unit;
  kk_unit_t _x21140 = kk_Unit;
  kk_unit_unbox(_b_19136);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19137_19135, (_b_19137_19135, _x21140, _ctx));
  return kk_unit_box(_x21139);
}

kk_unit_t kk_std_core__mlift17563_foreach_indexed(kk_function_t action, kk_ref_t i, kk_box_t x, kk_integer_t j, kk_context_t* _ctx) { /* forall<a,e> (action : (int, a) -> e (), i : local-var<global,int>, x : a, j : int) -> <local<global>|e> () */ 
  kk_unit_t x0_17937 = kk_Unit;
  kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_box_t, kk_context_t*), action, (action, j, x, _ctx));
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19137_19135 = kk_std_core__new_mlift17563_foreach_indexed_fun21136(i, _ctx); /*(wild_ : ()) -> <local<global>|12571> ()*/;
    kk_box_t _x21137 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17563_foreach_indexed_fun21138(_b_19137_19135, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21137); return kk_Unit;
  }
  kk_std_core__mlift17562_foreach_indexed(i, x0_17937, _ctx); return kk_Unit;
}
 
// Invoke `action` for each element of a list, passing also the position of the element.


// lift anonymous function
struct kk_std_core_foreach_indexed_fun21142__t {
  struct kk_function_s _base;
  kk_function_t action;
  kk_ref_t loc;
};
static kk_unit_t kk_std_core_foreach_indexed_fun21142(kk_function_t _fself, kk_box_t x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_indexed_fun21142(kk_function_t action, kk_ref_t loc, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexed_fun21142__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_indexed_fun21142__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_indexed_fun21142, kk_context());
  _self->action = action;
  _self->loc = loc;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_core_foreach_indexed_fun21146__t {
  struct kk_function_s _base;
  kk_function_t action;
  kk_ref_t loc;
  kk_box_t x;
};
static kk_box_t kk_std_core_foreach_indexed_fun21146(kk_function_t _fself, kk_box_t _b_19143, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_indexed_fun21146(kk_function_t action, kk_ref_t loc, kk_box_t x, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexed_fun21146__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_indexed_fun21146__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_indexed_fun21146, kk_context());
  _self->action = action;
  _self->loc = loc;
  _self->x = x;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_indexed_fun21146(kk_function_t _fself, kk_box_t _b_19143, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexed_fun21146__t* _self = kk_function_as(struct kk_std_core_foreach_indexed_fun21146__t*, _fself);
  kk_function_t action = _self->action; /* (int, 12570) -> 12571 () */
  kk_ref_t loc = _self->loc; /* local-var<global,int> */
  kk_box_t x = _self->x; /* 12570 */
  kk_drop_match(_self, {kk_function_dup(action);kk_ref_dup(loc);kk_box_dup(x);}, {}, _ctx)
  kk_unit_t _x21147 = kk_Unit;
  kk_integer_t j_19147 = kk_integer_unbox(_b_19143); /*int*/;
  kk_std_core__mlift17563_foreach_indexed(action, loc, x, j_19147, _ctx);
  return kk_unit_box(_x21147);
}
static kk_unit_t kk_std_core_foreach_indexed_fun21142(kk_function_t _fself, kk_box_t x, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexed_fun21142__t* _self = kk_function_as(struct kk_std_core_foreach_indexed_fun21142__t*, _fself);
  kk_function_t action = _self->action; /* (int, 12570) -> 12571 () */
  kk_ref_t loc = _self->loc; /* local-var<global,int> */
  kk_drop_match(_self, {kk_function_dup(action);kk_ref_dup(loc);}, {}, _ctx)
  kk_integer_t x0_17942;
  kk_box_t _x21143;
  kk_ref_t _x21144 = kk_ref_dup(loc); /*local-var<global,int>*/
  _x21143 = (kk_ref_get(_x21144,kk_context())); /*256*/
  x0_17942 = kk_integer_unbox(_x21143); /*int*/
  if (kk_yielding(kk_context())) {
    kk_integer_drop(x0_17942, _ctx);
    kk_box_t _x21145 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_indexed_fun21146(action, loc, x, _ctx), _ctx); /*3858*/
    return kk_unit_unbox(_x21145);
  }
  return kk_std_core__mlift17563_foreach_indexed(action, loc, x, x0_17942, _ctx);
}

kk_unit_t kk_std_core_foreach_indexed(kk_std_core__list xs, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, action : (int, a) -> e ()) -> e () */ 
  kk_ref_t loc = kk_ref_alloc((kk_integer_box(kk_integer_from_small(0))),kk_context()); /*local-var<global,int>*/;
  kk_unit_t res = kk_Unit;
  kk_function_t _x21141;
  kk_ref_dup(loc);
  _x21141 = kk_std_core_new_foreach_indexed_fun21142(action, loc, _ctx); /*(x : 12570) -> <local<global>|12571> ()*/
  kk_std_core_foreach(xs, _x21141, _ctx);
  kk_ref_t _b_19148_19145 = loc; /*local-var<global,int>*/;
  kk_unit_t _b_19149_19146 = kk_Unit;
  res;
  kk_box_t _x21148 = kk_std_core_hnd_prompt_local_var(_b_19148_19145, kk_unit_box(_b_19149_19146), _ctx); /*9895*/
  kk_unit_unbox(_x21148); return kk_Unit;
}
 
// Invoke a function `f` for each element in a vector `v`


// lift anonymous function
struct kk_std_core_foreach_indexed_fun21150__t_1 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_vector_t v;
};
static kk_unit_t kk_std_core_foreach_indexed_fun21150_1(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_indexed_fun21150_1(kk_function_t f, kk_vector_t v, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexed_fun21150__t_1* _self = kk_function_alloc_as(struct kk_std_core_foreach_indexed_fun21150__t_1, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_indexed_fun21150_1, kk_context());
  _self->f = f;
  _self->v = v;
  return &_self->_base;
}

static kk_unit_t kk_std_core_foreach_indexed_fun21150_1(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexed_fun21150__t_1* _self = kk_function_as(struct kk_std_core_foreach_indexed_fun21150__t_1*, _fself);
  kk_function_t f = _self->f; /* (12598, int) -> 12599 () */
  kk_vector_t v = _self->v; /* vector<12598> */
  kk_drop_match(_self, {kk_function_dup(f);kk_vector_dup(v);}, {}, _ctx)
  kk_box_t x_18136;
  kk_box_t _borrowApp_19748 = kk_vector_at_borrow(v, i); /*219*/;
  kk_vector_drop(v, _ctx);
  x_18136 = _borrowApp_19748; /*12598*/
  kk_integer_t _x21151 = kk_integer_from_ssize_t(i,kk_context()); /*int*/
  return kk_function_call(kk_unit_t, (kk_function_t, kk_box_t, kk_integer_t, kk_context_t*), f, (f, x_18136, _x21151, _ctx));
}

kk_unit_t kk_std_core_foreach_indexed_1(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (v : vector<a>, f : (a, int) -> e ()) -> e () */ 
  kk_ssize_t start0_17946 = ((kk_ssize_t)0); /*ssize_t*/;
  kk_ssize_t end_17947;
  kk_ssize_t _x21149 = kk_vector_len_borrow(v); /*ssize_t*/
  end_17947 = kk_std_core_decr_1(_x21149, _ctx); /*ssize_t*/
  kk_std_core__lift17073_forz(kk_std_core_new_foreach_indexed_fun21150_1(f, v, _ctx), end_17947, start0_17946, _ctx); return kk_Unit;
}
 
// Return the head of list if the list is not empty.

kk_std_core_types__maybe kk_std_core_head_1(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> maybe<a> */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21160 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21160->head;
    kk_std_core__list _pat0 = _con21160->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_drop(_pat0, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    return kk_std_core_types__new_Just(x, _ctx);
  }
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Return the head of list if the list is not empty, or use `default` otherwise

kk_box_t kk_std_core_head_2(kk_std_core__list xs, kk_box_t default0, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, default : a) -> a */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21161 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21161->head;
    kk_std_core__list _pat0 = _con21161->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_drop(_pat0, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_box_drop(default0, _ctx);
    kk_Unit;
    return x;
  }
  return default0;
}
 
// Return the first character of a string as a string (or the empty string)

kk_string_t kk_std_core_head_3(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  kk_std_core__sslice _x21162;
  kk_std_core__sslice slice0 = kk_std_core_first1(s, _ctx); /*sslice*/;
  bool _match_19740;
  kk_integer_t _borrowedArg_19745;
  kk_std_core_types__optional _match_19746 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_19746)) {
    kk_box_t _box_x19150 = _match_19746._cons.Optional.value;
    kk_integer_t _n_10065 = kk_integer_unbox(_box_x19150);
    kk_integer_dup(_n_10065);
    kk_std_core_types__optional_drop(_match_19746, _ctx);
    kk_Unit;
    _borrowedArg_19745 = _n_10065; /*int*/
    goto _match21163;
  }
  _borrowedArg_19745 = kk_integer_from_small(1); /*int*/
  _match21163: ;
  kk_integer_t _borrowedArg_19744 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19747 = kk_integer_eq_borrow(_borrowedArg_19745,_borrowedArg_19744,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19745, _ctx);
  kk_integer_drop(_borrowedArg_19744, _ctx);
  _match_19740 = _borrowApp_19747; /*bool*/
  if (_match_19740) {
    _x21162 = slice0; /*sslice*/
  }
  else {
    kk_integer_t _borrowedArg_19741;
    kk_integer_t _x21165;
    kk_std_core_types__optional _match_19742 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_19742)) {
      kk_box_t _box_x19151 = _match_19742._cons.Optional.value;
      kk_integer_t _n_100650 = kk_integer_unbox(_box_x19151);
      kk_integer_dup(_n_100650);
      kk_std_core_types__optional_drop(_match_19742, _ctx);
      kk_Unit;
      _x21165 = _n_100650; /*int*/
      goto _match21166;
    }
    _x21165 = kk_integer_from_small(1); /*int*/
    _match21166: ;
    _borrowedArg_19741 = kk_integer_sub(_x21165,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__sslice _borrowApp_19743 = kk_std_core_extend(slice0, _borrowedArg_19741, _ctx); /*sslice*/;
    kk_integer_drop(_borrowedArg_19741, _ctx);
    _x21162 = _borrowApp_19743; /*sslice*/
  }
  return kk_std_core_string_3(_x21162, _ctx);
}
 
// Return the first character of a string (or `Nothing` for the empty string).


// lift anonymous function
struct kk_std_core_head_char_fun21172__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_core_head_char_fun21172(kk_function_t _fself, kk_char_t _b_19152, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_head_char_fun21172(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_head_char_fun21172, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_core_head_char_fun21172(kk_function_t _fself, kk_char_t _b_19152, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_types__new_Just(kk_char_box(_b_19152, _ctx), _ctx);
}

kk_std_core_types__maybe kk_std_core_head_char(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> maybe<char> */ 
  kk_std_core__sslice _x21168;
  kk_string_t _x21169 = kk_string_dup(s); /*string*/
  kk_ssize_t _x21170 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x21171 = kk_string_len(s,kk_context()); /*ssize_t*/
  _x21168 = kk_std_core__new_Sslice(_x21169, _x21170, _x21171, _ctx); /*sslice*/
  return kk_std_core_foreach_while_1(_x21168, kk_std_core_new_head_char_fun21172(_ctx), _ctx);
}

kk_integer_t kk_std_core_index_of_acc(kk_std_core__list xs, kk_function_t pred, kk_integer_t idx, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, pred : (a) -> bool, idx : int) -> int */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_integer_drop(idx, _ctx);
    kk_function_drop(pred, _ctx);
    return kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context());
  }
  struct kk_std_core_Cons* _con21173 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21173->head;
  kk_std_core__list xx = _con21173->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  bool _match_19739;
  kk_function_t _x21174 = kk_function_dup(pred); /*(12975) -> bool*/
  _match_19739 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21174, (_x21174, x, _ctx)); /*bool*/
  if (_match_19739) {
    kk_function_drop(pred, _ctx);
    kk_std_core__list_drop(xx, _ctx);
    return idx;
  }
  { // tailcall
    kk_integer_t _x21175 = kk_integer_add(idx,(kk_integer_from_small(1)),kk_context()); /*int*/
    xs = xx;
    idx = _x21175;
    goto kk__tailcall;
  }
}
 
// Return the list without its last element.
// Return an empty list for an empty list.

kk_std_core__list kk_std_core__ctail_init(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21176 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21176->head;
    kk_std_core__list xx = _con21176->tail;
    if (kk_std_core__is_Cons(xx)) {
      struct kk_std_core_Cons* _con21177 = kk_std_core__as_Cons(xx);
      kk_reuse_t _ru_20069 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        _ru_20069 = (kk_std_core__list_reuse(xs));
      }
      else {
        kk_box_dup(x);
        kk_std_core__list_dup(xx);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      kk_std_core__list _ctail_17196 = kk_std_core__list_hole(); /*list<13012>*/;
      kk_std_core__list _ctail_17197;
      if (kk_likely(_ru_20069!=NULL)) {
        struct kk_std_core_Cons* _con21178 = (struct kk_std_core_Cons*)_ru_20069;
        _con21178->tail = _ctail_17196;
        _ctail_17197 = kk_std_core__base_Cons(_con21178); /*list<13012>*/
      }
      else {
        _ctail_17197 = kk_std_core__new_Cons(kk_reuse_null, x, _ctail_17196, _ctx); /*list<13012>*/
      }
      { // tailcall
        kk_std_core_types__ctail _x21179;
        kk_box_t* field_17951 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17197)->tail)); /*cfield<list<13012>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x19159 = _acc._cons.CTail._field1;
          kk_box_t* last0 = _acc._cons.CTail._field2;
          kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19159, NULL);
          kk_std_core__list_dup(head0);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last0) = kk_std_core__list_box(_ctail_17197, _ctx);
          kk_std_core__list _b_19173_19164 = head0; /*list<13012>*/;
          kk_box_t* _b_19174_19165 = field_17951; /*cfield<list<13012>>*/;
          _x21179 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19173_19164, _ctx), _b_19174_19165, _ctx); /*ctail<58>*/
          goto _match21180;
        }
        kk_std_core__list _b_19175_19166 = _ctail_17197; /*list<13012>*/;
        kk_box_t* _b_19176_19167 = field_17951; /*cfield<list<13012>>*/;
        _x21179 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19175_19166, _ctx), _b_19176_19167, _ctx); /*ctail<58>*/
        _match21180: ;
        xs = xx;
        _acc = _x21179;
        goto kk__tailcall;
      }
    }
  }
  kk_std_core__list_drop(xs, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19168 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19168, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Return the list without its last element.
// Return an empty list for an empty list.

kk_std_core__list kk_std_core_init(kk_std_core__list xs0, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> list<a> */ 
  return kk_std_core__ctail_init(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// An invalid slice

kk_std_core__sslice kk_std_core_invalid;
 
// Is the character an ASCII letter is-

bool kk_std_core_is_alpha(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  bool _match_19733 = (c >= ('a')); /*bool*/;
  if (_match_19733) {
    bool _match_19735 = (c <= ('z')); /*bool*/;
    if (_match_19735) {
      return true;
    }
    bool _match_19736 = (c >= ('A')); /*bool*/;
    if (_match_19736) {
      return (c <= ('Z'));
    }
    return false;
  }
  bool _match_19734 = (c >= ('A')); /*bool*/;
  if (_match_19734) {
    return (c <= ('Z'));
  }
  return false;
}
 
// Is the character an ASCII hexa-decimal digit ?

bool kk_std_core_is_hex_digit(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  bool _match_19720 = (c >= ('0')); /*bool*/;
  if (_match_19720) {
    bool _match_19725 = (c <= ('9')); /*bool*/;
    if (_match_19725) {
      return true;
    }
    bool _match_19726 = (c >= ('a')); /*bool*/;
    if (_match_19726) {
      bool _match_19728 = (c <= ('f')); /*bool*/;
      if (_match_19728) {
        return true;
      }
      bool _match_19729 = (c >= ('A')); /*bool*/;
      if (_match_19729) {
        return (c <= ('F'));
      }
      return false;
    }
    bool _match_19727 = (c >= ('A')); /*bool*/;
    if (_match_19727) {
      return (c <= ('F'));
    }
    return false;
  }
  bool _match_19721 = (c >= ('a')); /*bool*/;
  if (_match_19721) {
    bool _match_19723 = (c <= ('f')); /*bool*/;
    if (_match_19723) {
      return true;
    }
    bool _match_19724 = (c >= ('A')); /*bool*/;
    if (_match_19724) {
      return (c <= ('F'));
    }
    return false;
  }
  bool _match_19722 = (c >= ('A')); /*bool*/;
  if (_match_19722) {
    return (c <= ('F'));
  }
  return false;
}
 
// Tests if a character is an element of `" \t\n\r"`

bool kk_std_core_is_white(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  bool _match_19717 = (c == (' ')); /*bool*/;
  if (_match_19717) {
    return true;
  }
  bool _match_19718 = (c == 0x0009); /*bool*/;
  if (_match_19718) {
    return true;
  }
  bool _match_19719 = (c == 0x000A); /*bool*/;
  if (_match_19719) {
    return true;
  }
  return (c == 0x000D);
}
 
// Append `end` to each string in the list `xs` and join them all together.
// `join-end([],end) === ""`
// `join-end(["a","b"],"/") === "a/b/"`

kk_string_t kk_std_core_join_end(kk_std_core__list xs, kk_string_t end, kk_context_t* _ctx) { /* (xs : list<string>, end : string) -> string */ 
  if (kk_std_core__is_Nil(xs)) {
    kk_string_drop(end, _ctx);
    return kk_string_empty();
  }
  kk_string_t _x21193;
  if (kk_std_core__is_Nil(xs)) {
    _x21193 = kk_string_empty(); /*string*/
  }
  else {
    struct kk_std_core_Cons* _con21195 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x19177 = _con21195->head;
    kk_std_core__list xx = _con21195->tail;
    kk_string_t x = kk_string_unbox(_box_x19177);
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_string_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_string_t _x21197 = kk_string_dup(end); /*string*/
    _x21193 = kk_std_core__lift17071_joinsep(_x21197, xx, x, _ctx); /*string*/
  }
  return kk_std_core__lp__plus__plus__1_rp_(_x21193, end, _ctx);
}
 
// Return the last element of a list (or `Nothing` for the empty list)

kk_std_core_types__maybe kk_std_core_last(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> maybe<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21198 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21198->head;
    kk_std_core__list _pat0 = _con21198->tail;
    if (kk_std_core__is_Nil(_pat0)) {
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        if (kk_likely(kk_std_core__list_is_unique(_pat0))) {
          kk_std_core__list_free(_pat0);
        }
        else {
          kk_std_core__list_decref(_pat0, _ctx);
          kk_Unit;
        }
        kk_Unit;
        kk_std_core__list_free(xs);
      }
      else {
        kk_box_dup(x);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      return kk_std_core_types__new_Just(x, _ctx);
    }
  }
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21199 = kk_std_core__as_Cons(xs);
    kk_box_t _pat2 = _con21199->head;
    kk_std_core__list xx = _con21199->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_box_drop(_pat2, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs);
    }
    else {
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    { // tailcall
      xs = xx;
      goto kk__tailcall;
    }
  }
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Return the last element of a list (or `default` for the empty list)

kk_box_t kk_std_core_last_1(kk_std_core__list xs, kk_box_t default0, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, default : a) -> a */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21200 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21200->head;
    kk_std_core__list _pat0 = _con21200->tail;
    if (kk_std_core__is_Nil(_pat0)) {
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        if (kk_likely(kk_std_core__list_is_unique(_pat0))) {
          kk_std_core__list_free(_pat0);
        }
        else {
          kk_std_core__list_decref(_pat0, _ctx);
          kk_Unit;
        }
        kk_Unit;
        kk_std_core__list_free(xs);
      }
      else {
        kk_box_dup(x);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      kk_box_drop(default0, _ctx);
      kk_Unit;
      return x;
    }
  }
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21201 = kk_std_core__as_Cons(xs);
    kk_box_t _pat2 = _con21201->head;
    kk_std_core__list xx = _con21201->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_box_drop(_pat2, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs);
    }
    else {
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    { // tailcall
      xs = xx;
      goto kk__tailcall;
    }
  }
  return default0;
}
 
// O(`n`). The last `n` (default = `1`) characters in a string

kk_std_core__sslice kk_std_core_last_2(kk_string_t s, kk_std_core_types__optional n, kk_context_t* _ctx) { /* (s : string, n : optional<int>) -> sslice */ 
  kk_std_core__sslice slice0 = kk_std_core_last1(s, _ctx); /*sslice*/;
  bool _match_19708;
  kk_integer_t _borrowedArg_19715;
  if (kk_std_core_types__is_Optional(n)) {
    kk_box_t _box_x19178 = n._cons.Optional.value;
    kk_integer_t _n_13606 = kk_integer_unbox(_box_x19178);
    kk_integer_dup(_n_13606);
    _borrowedArg_19715 = _n_13606; /*int*/
    goto _match21202;
  }
  _borrowedArg_19715 = kk_integer_from_small(1); /*int*/
  _match21202: ;
  kk_integer_t _borrowedArg_19714 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19716 = kk_integer_eq_borrow(_borrowedArg_19715,_borrowedArg_19714,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19715, _ctx);
  kk_integer_drop(_borrowedArg_19714, _ctx);
  _match_19708 = _borrowApp_19716; /*bool*/
  if (_match_19708) {
    kk_std_core_types__optional_drop(n, _ctx);
    return slice0;
  }
  kk_std_core__sslice _ownedArg_19710;
  kk_integer_t _borrowedArg_19711;
  kk_integer_t _x21204;
  if (kk_std_core_types__is_Optional(n)) {
    kk_box_t _box_x19179 = n._cons.Optional.value;
    kk_integer_t _n_136060 = kk_integer_unbox(_box_x19179);
    kk_integer_dup(_n_136060);
    _x21204 = _n_136060; /*int*/
    goto _match21205;
  }
  _x21204 = kk_integer_from_small(1); /*int*/
  _match21205: ;
  _borrowedArg_19711 = kk_integer_sub((kk_integer_from_small(1)),_x21204,kk_context()); /*int*/
  kk_std_core__sslice _borrowApp_19712 = kk_std_core_advance(slice0, _borrowedArg_19711, _ctx); /*sslice*/;
  kk_integer_drop(_borrowedArg_19711, _ctx);
  _ownedArg_19710 = _borrowApp_19712; /*sslice*/
  kk_integer_t _borrowedArg_19709;
  kk_integer_t _x21207;
  if (kk_std_core_types__is_Optional(n)) {
    kk_box_t _box_x19180 = n._cons.Optional.value;
    kk_integer_t _n_136061 = kk_integer_unbox(_box_x19180);
    kk_integer_dup(_n_136061);
    kk_std_core_types__optional_drop(n, _ctx);
    kk_Unit;
    _x21207 = _n_136061; /*int*/
    goto _match21208;
  }
  _x21207 = kk_integer_from_small(1); /*int*/
  _match21208: ;
  _borrowedArg_19709 = kk_integer_sub(_x21207,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_std_core__sslice _borrowApp_19713 = kk_std_core_extend(_ownedArg_19710, _borrowedArg_19709, _ctx); /*sslice*/;
  kk_integer_drop(_borrowedArg_19709, _ctx);
  return _borrowApp_19713;
}
 
// Take the first `n` elements of a list (or fewer if the list is shorter than `n`)

kk_std_core__list kk_std_core__ctail_take(kk_std_core__list xs, kk_integer_t n, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21210 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21210->head;
    kk_std_core__list xx = _con21210->tail;
    kk_integer_t _borrowedArg_19706 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_19707 = kk_integer_gt_borrow(n,_borrowedArg_19706,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_19706, _ctx);
    if (_borrowApp_19707) {
      kk_reuse_t _ru_20076 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        _ru_20076 = (kk_std_core__list_reuse(xs));
      }
      else {
        kk_box_dup(x);
        kk_std_core__list_dup(xx);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      kk_std_core__list _ctail_17198 = kk_std_core__list_hole(); /*list<13730>*/;
      kk_std_core__list _ctail_17199;
      if (kk_likely(_ru_20076!=NULL)) {
        struct kk_std_core_Cons* _con21211 = (struct kk_std_core_Cons*)_ru_20076;
        _con21211->tail = _ctail_17198;
        _ctail_17199 = kk_std_core__base_Cons(_con21211); /*list<13730>*/
      }
      else {
        _ctail_17199 = kk_std_core__new_Cons(kk_reuse_null, x, _ctail_17198, _ctx); /*list<13730>*/
      }
      { // tailcall
        kk_integer_t _x21212 = kk_integer_sub(n,(kk_integer_from_small(1)),kk_context()); /*int*/
        kk_std_core_types__ctail _x21213;
        kk_box_t* field_17958 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17199)->tail)); /*cfield<list<13730>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x19187 = _acc._cons.CTail._field1;
          kk_box_t* last0 = _acc._cons.CTail._field2;
          kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19187, NULL);
          kk_std_core__list_dup(head0);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last0) = kk_std_core__list_box(_ctail_17199, _ctx);
          kk_std_core__list _b_19201_19192 = head0; /*list<13730>*/;
          kk_box_t* _b_19202_19193 = field_17958; /*cfield<list<13730>>*/;
          _x21213 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19201_19192, _ctx), _b_19202_19193, _ctx); /*ctail<58>*/
          goto _match21214;
        }
        kk_std_core__list _b_19203_19194 = _ctail_17199; /*list<13730>*/;
        kk_box_t* _b_19204_19195 = field_17958; /*cfield<list<13730>>*/;
        _x21213 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19203_19194, _ctx), _b_19204_19195, _ctx); /*ctail<58>*/
        _match21214: ;
        xs = xx;
        n = _x21212;
        _acc = _x21213;
        goto kk__tailcall;
      }
    }
  }
  kk_integer_drop(n, _ctx);
  kk_std_core__list_drop(xs, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19196 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19196, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Take the first `n` elements of a list (or fewer if the list is shorter than `n`)

kk_std_core__list kk_std_core_take(kk_std_core__list xs0, kk_integer_t n0, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int) -> list<a> */ 
  return kk_std_core__ctail_take(xs0, n0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// split a list at position `n`

kk_std_core_types__tuple2_ kk_std_core_split(kk_std_core__list xs, kk_integer_t n, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int) -> (list<a>, list<a>) */ 
  kk_std_core__list _b_19207_19205;
  kk_std_core__list _x21217 = kk_std_core__list_dup(xs); /*list<13750>*/
  kk_integer_t _x21218 = kk_integer_dup(n); /*int*/
  _b_19207_19205 = kk_std_core_take(_x21217, _x21218, _ctx); /*list<13750>*/
  kk_std_core__list _b_19208_19206 = kk_std_core_drop(xs, n, _ctx); /*list<13750>*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19207_19205, _ctx), kk_std_core__list_box(_b_19208_19206, _ctx), _ctx);
}
 
// Lookup the first element satisfying some predicate


// lift anonymous function
struct kk_std_core_lookup_fun21223__t {
  struct kk_function_s _base;
  kk_function_t pred;
};
static kk_std_core_types__maybe kk_std_core_lookup_fun21223(kk_function_t _fself, kk_std_core_types__tuple2_ kv, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_lookup_fun21223(kk_function_t pred, kk_context_t* _ctx) {
  struct kk_std_core_lookup_fun21223__t* _self = kk_function_alloc_as(struct kk_std_core_lookup_fun21223__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_lookup_fun21223, kk_context());
  _self->pred = pred;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core_lookup_fun21223(kk_function_t _fself, kk_std_core_types__tuple2_ kv, kk_context_t* _ctx) {
  struct kk_std_core_lookup_fun21223__t* _self = kk_function_as(struct kk_std_core_lookup_fun21223__t*, _fself);
  kk_function_t pred = _self->pred; /* (14082) -> bool */
  kk_drop_match(_self, {kk_function_dup(pred);}, {}, _ctx)
  bool _match_19705;
  kk_box_t _x21224;
  kk_box_t _x = kv.fst;
  kk_box_dup(_x);
  _x21224 = _x; /*14082*/
  _match_19705 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), pred, (pred, _x21224, _ctx)); /*bool*/
  if (_match_19705) {
    kk_box_t _x21225;
    kk_box_t _x0 = kv.snd;
    kk_box_dup(_x0);
    kk_std_core_types__tuple2__drop(kv, _ctx);
    kk_Unit;
    _x21225 = _x0; /*14083*/
    return kk_std_core_types__new_Just(_x21225, _ctx);
  }
  kk_std_core_types__tuple2__drop(kv, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}


// lift anonymous function
struct kk_std_core_lookup_fun21226__t {
  struct kk_function_s _base;
  kk_function_t _b_19213_19210;
};
static kk_std_core_types__maybe kk_std_core_lookup_fun21226(kk_function_t _fself, kk_box_t _b_19211, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_lookup_fun21226(kk_function_t _b_19213_19210, kk_context_t* _ctx) {
  struct kk_std_core_lookup_fun21226__t* _self = kk_function_alloc_as(struct kk_std_core_lookup_fun21226__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_lookup_fun21226, kk_context());
  _self->_b_19213_19210 = _b_19213_19210;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core_lookup_fun21226(kk_function_t _fself, kk_box_t _b_19211, kk_context_t* _ctx) {
  struct kk_std_core_lookup_fun21226__t* _self = kk_function_as(struct kk_std_core_lookup_fun21226__t*, _fself);
  kk_function_t _b_19213_19210 = _self->_b_19213_19210; /* (kv : (14082, 14083)) -> maybe<14083> */
  kk_drop_match(_self, {kk_function_dup(_b_19213_19210);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x21227 = kk_std_core_types__tuple2__unbox(_b_19211, _ctx); /*(14082, 14083)*/
  return kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_core_types__tuple2_, kk_context_t*), _b_19213_19210, (_b_19213_19210, _x21227, _ctx));
}

kk_std_core_types__maybe kk_std_core_lookup(kk_std_core__list xs, kk_function_t pred, kk_context_t* _ctx) { /* forall<a,b> (xs : list<(a, b)>, pred : (a) -> bool) -> maybe<b> */ 
  kk_std_core__list _b_19212_19209 = xs; /*list<(14082, 14083)>*/;
  kk_function_t _b_19213_19210 = kk_std_core_new_lookup_fun21223(pred, _ctx); /*(kv : (14082, 14083)) -> maybe<14083>*/;
  return kk_std_core_foreach_while(_b_19212_19209, kk_std_core_new_lookup_fun21226(_b_19213_19210, _ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17564_op(kk_box_t _y_17422, kk_std_core__list _y_17423, kk_context_t* _ctx) { /* forall<a,e> (a, list<a>) -> e list<a> */ 
  return kk_std_core__new_Cons(kk_reuse_null, _y_17422, _y_17423, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17565_op_fun21228__t {
  struct kk_function_s _base;
  kk_box_t _y_174220;
};
static kk_std_core__list kk_std_core__mlift17565_op_fun21228(kk_function_t _fself, kk_std_core__list _y_174230, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17565_op_fun21228(kk_box_t _y_174220, kk_context_t* _ctx) {
  struct kk_std_core__mlift17565_op_fun21228__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17565_op_fun21228__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17565_op_fun21228, kk_context());
  _self->_y_174220 = _y_174220;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17565_op_fun21228(kk_function_t _fself, kk_std_core__list _y_174230, kk_context_t* _ctx) {
  struct kk_std_core__mlift17565_op_fun21228__t* _self = kk_function_as(struct kk_std_core__mlift17565_op_fun21228__t*, _fself);
  kk_box_t _y_174220 = _self->_y_174220; /* 14138 */
  kk_drop_match(_self, {kk_box_dup(_y_174220);}, {}, _ctx)
  return kk_std_core__mlift17564_op(_y_174220, _y_174230, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17565_op_fun21230__t {
  struct kk_function_s _base;
  kk_function_t _b_19216_19214;
};
static kk_box_t kk_std_core__mlift17565_op_fun21230(kk_function_t _fself, kk_box_t _b_19215, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17565_op_fun21230(kk_function_t _b_19216_19214, kk_context_t* _ctx) {
  struct kk_std_core__mlift17565_op_fun21230__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17565_op_fun21230__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17565_op_fun21230, kk_context());
  _self->_b_19216_19214 = _b_19216_19214;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17565_op_fun21230(kk_function_t _fself, kk_box_t _b_19215, kk_context_t* _ctx) {
  struct kk_std_core__mlift17565_op_fun21230__t* _self = kk_function_as(struct kk_std_core__mlift17565_op_fun21230__t*, _fself);
  kk_function_t _b_19216_19214 = _self->_b_19216_19214; /* (list<14138>) -> 14139 list<14138> */
  kk_drop_match(_self, {kk_function_dup(_b_19216_19214);}, {}, _ctx)
  kk_std_core__list _x21231;
  kk_std_core__list _x21232 = kk_std_core__list_unbox(_b_19215, _ctx); /*list<14138>*/
  _x21231 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19216_19214, (_b_19216_19214, _x21232, _ctx)); /*list<14138>*/
  return kk_std_core__list_box(_x21231, _ctx);
}

kk_std_core__list kk_std_core__mlift17565_op(kk_function_t f, kk_integer_t i, kk_std_core__list yy, kk_box_t _y_174220, kk_context_t* _ctx) { /* forall<a,b,e> (f : (idx : int, value : a) -> e b, i : int, yy : list<a>, b) -> e list<b> */ 
  kk_integer_t i0_17129 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__list x_17961 = kk_std_core__lift17081_map_indexed(f, yy, i0_17129, _ctx); /*list<14138>*/;
  if (kk_yielding(kk_context())) {
    kk_std_core__list_drop(x_17961, _ctx);
    kk_function_t _b_19216_19214 = kk_std_core__new_mlift17565_op_fun21228(_y_174220, _ctx); /*(list<14138>) -> 14139 list<14138>*/;
    kk_box_t _x21229 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17565_op_fun21230(_b_19216_19214, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21229, _ctx);
  }
  return kk_std_core__mlift17564_op(_y_174220, x_17961, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17081_map_indexed_fun21236__t {
  struct kk_function_s _base;
  kk_function_t f0;
  kk_integer_t i0;
  kk_std_core__list yy0;
};
static kk_std_core__list kk_std_core__lift17081_map_indexed_fun21236(kk_function_t _fself, kk_box_t _y_174221, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17081_map_indexed_fun21236(kk_function_t f0, kk_integer_t i0, kk_std_core__list yy0, kk_context_t* _ctx) {
  struct kk_std_core__lift17081_map_indexed_fun21236__t* _self = kk_function_alloc_as(struct kk_std_core__lift17081_map_indexed_fun21236__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17081_map_indexed_fun21236, kk_context());
  _self->f0 = f0;
  _self->i0 = i0;
  _self->yy0 = yy0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17081_map_indexed_fun21236(kk_function_t _fself, kk_box_t _y_174221, kk_context_t* _ctx) {
  struct kk_std_core__lift17081_map_indexed_fun21236__t* _self = kk_function_as(struct kk_std_core__lift17081_map_indexed_fun21236__t*, _fself);
  kk_function_t f0 = _self->f0; /* (idx : int, value : 14137) -> 14139 14138 */
  kk_integer_t i0 = _self->i0; /* int */
  kk_std_core__list yy0 = _self->yy0; /* list<14137> */
  kk_drop_match(_self, {kk_function_dup(f0);kk_integer_dup(i0);kk_std_core__list_dup(yy0);}, {}, _ctx)
  return kk_std_core__mlift17565_op(f0, i0, yy0, _y_174221, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17081_map_indexed_fun21238__t {
  struct kk_function_s _base;
  kk_function_t _b_19221_19217;
};
static kk_box_t kk_std_core__lift17081_map_indexed_fun21238(kk_function_t _fself, kk_box_t _b_19218, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17081_map_indexed_fun21238(kk_function_t _b_19221_19217, kk_context_t* _ctx) {
  struct kk_std_core__lift17081_map_indexed_fun21238__t* _self = kk_function_alloc_as(struct kk_std_core__lift17081_map_indexed_fun21238__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17081_map_indexed_fun21238, kk_context());
  _self->_b_19221_19217 = _b_19221_19217;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17081_map_indexed_fun21238(kk_function_t _fself, kk_box_t _b_19218, kk_context_t* _ctx) {
  struct kk_std_core__lift17081_map_indexed_fun21238__t* _self = kk_function_as(struct kk_std_core__lift17081_map_indexed_fun21238__t*, _fself);
  kk_function_t _b_19221_19217 = _self->_b_19221_19217; /* (14138) -> 14139 list<14138> */
  kk_drop_match(_self, {kk_function_dup(_b_19221_19217);}, {}, _ctx)
  kk_std_core__list _x21239 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19221_19217, (_b_19221_19217, _b_19218, _ctx)); /*list<14138>*/
  return kk_std_core__list_box(_x21239, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17081_map_indexed_fun21240__t {
  struct kk_function_s _base;
  kk_box_t x0_17963;
};
static kk_std_core__list kk_std_core__lift17081_map_indexed_fun21240(kk_function_t _fself, kk_std_core__list _y_174231, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17081_map_indexed_fun21240(kk_box_t x0_17963, kk_context_t* _ctx) {
  struct kk_std_core__lift17081_map_indexed_fun21240__t* _self = kk_function_alloc_as(struct kk_std_core__lift17081_map_indexed_fun21240__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17081_map_indexed_fun21240, kk_context());
  _self->x0_17963 = x0_17963;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17081_map_indexed_fun21240(kk_function_t _fself, kk_std_core__list _y_174231, kk_context_t* _ctx) {
  struct kk_std_core__lift17081_map_indexed_fun21240__t* _self = kk_function_as(struct kk_std_core__lift17081_map_indexed_fun21240__t*, _fself);
  kk_box_t x0_17963 = _self->x0_17963; /* 14138 */
  kk_drop_match(_self, {kk_box_dup(x0_17963);}, {}, _ctx)
  return kk_std_core__mlift17564_op(x0_17963, _y_174231, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17081_map_indexed_fun21242__t {
  struct kk_function_s _base;
  kk_function_t _b_19222_19219;
};
static kk_box_t kk_std_core__lift17081_map_indexed_fun21242(kk_function_t _fself, kk_box_t _b_19220, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17081_map_indexed_fun21242(kk_function_t _b_19222_19219, kk_context_t* _ctx) {
  struct kk_std_core__lift17081_map_indexed_fun21242__t* _self = kk_function_alloc_as(struct kk_std_core__lift17081_map_indexed_fun21242__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17081_map_indexed_fun21242, kk_context());
  _self->_b_19222_19219 = _b_19222_19219;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17081_map_indexed_fun21242(kk_function_t _fself, kk_box_t _b_19220, kk_context_t* _ctx) {
  struct kk_std_core__lift17081_map_indexed_fun21242__t* _self = kk_function_as(struct kk_std_core__lift17081_map_indexed_fun21242__t*, _fself);
  kk_function_t _b_19222_19219 = _self->_b_19222_19219; /* (list<14138>) -> 14139 list<14138> */
  kk_drop_match(_self, {kk_function_dup(_b_19222_19219);}, {}, _ctx)
  kk_std_core__list _x21243;
  kk_std_core__list _x21244 = kk_std_core__list_unbox(_b_19220, _ctx); /*list<14138>*/
  _x21243 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19222_19219, (_b_19222_19219, _x21244, _ctx)); /*list<14138>*/
  return kk_std_core__list_box(_x21243, _ctx);
}

kk_std_core__list kk_std_core__lift17081_map_indexed(kk_function_t f0, kk_std_core__list ys, kk_integer_t i0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (idx : int, value : a) -> e b, ys : list<a>, i : int) -> e list<b> */ 
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21233 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con21233->head;
    kk_std_core__list yy0 = _con21233->tail;
    kk_reuse_t _ru_20077 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20077 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy0);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_box_t x0_17963;
    kk_function_t _x21235 = kk_function_dup(f0); /*(idx : int, value : 14137) -> 14139 14138*/
    kk_integer_t _x21234 = kk_integer_dup(i0); /*int*/
    x0_17963 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_box_t, kk_context_t*), _x21235, (_x21235, _x21234, y, _ctx)); /*14138*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20077);
      kk_box_drop(x0_17963, _ctx);
      kk_function_t _b_19221_19217 = kk_std_core__new_lift17081_map_indexed_fun21236(f0, i0, yy0, _ctx); /*(14138) -> 14139 list<14138>*/;
      kk_box_t _x21237 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17081_map_indexed_fun21238(_b_19221_19217, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21237, _ctx);
    }
    kk_integer_t i0_171290 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
    kk_std_core__list x1_17966 = kk_std_core__lift17081_map_indexed(f0, yy0, i0_171290, _ctx); /*list<14138>*/;
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20077);
      kk_std_core__list_drop(x1_17966, _ctx);
      kk_function_t _b_19222_19219 = kk_std_core__new_lift17081_map_indexed_fun21240(x0_17963, _ctx); /*(list<14138>) -> 14139 list<14138>*/;
      kk_box_t _x21241 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17081_map_indexed_fun21242(_b_19222_19219, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21241, _ctx);
    }
    return kk_std_core__new_Cons(_ru_20077, x0_17963, x1_17966, _ctx);
  }
  kk_function_drop(f0, _ctx);
  kk_integer_drop(i0, _ctx);
  return kk_std_core__new_Nil(_ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17566_op(kk_box_t _y_17426, kk_std_core__list _y_17427, kk_context_t* _ctx) { /* forall<a,e> (a, list<a>) -> e list<a> */ 
  return kk_std_core__new_Cons(kk_reuse_null, _y_17426, _y_17427, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17567_op_fun21245__t {
  struct kk_function_s _base;
  kk_box_t _y_174260;
};
static kk_std_core__list kk_std_core__mlift17567_op_fun21245(kk_function_t _fself, kk_std_core__list _y_174270, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17567_op_fun21245(kk_box_t _y_174260, kk_context_t* _ctx) {
  struct kk_std_core__mlift17567_op_fun21245__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17567_op_fun21245__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17567_op_fun21245, kk_context());
  _self->_y_174260 = _y_174260;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17567_op_fun21245(kk_function_t _fself, kk_std_core__list _y_174270, kk_context_t* _ctx) {
  struct kk_std_core__mlift17567_op_fun21245__t* _self = kk_function_as(struct kk_std_core__mlift17567_op_fun21245__t*, _fself);
  kk_box_t _y_174260 = _self->_y_174260; /* 14195 */
  kk_drop_match(_self, {kk_box_dup(_y_174260);}, {}, _ctx)
  return kk_std_core__mlift17566_op(_y_174260, _y_174270, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17567_op_fun21247__t {
  struct kk_function_s _base;
  kk_function_t _b_19225_19223;
};
static kk_box_t kk_std_core__mlift17567_op_fun21247(kk_function_t _fself, kk_box_t _b_19224, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17567_op_fun21247(kk_function_t _b_19225_19223, kk_context_t* _ctx) {
  struct kk_std_core__mlift17567_op_fun21247__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17567_op_fun21247__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17567_op_fun21247, kk_context());
  _self->_b_19225_19223 = _b_19225_19223;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17567_op_fun21247(kk_function_t _fself, kk_box_t _b_19224, kk_context_t* _ctx) {
  struct kk_std_core__mlift17567_op_fun21247__t* _self = kk_function_as(struct kk_std_core__mlift17567_op_fun21247__t*, _fself);
  kk_function_t _b_19225_19223 = _self->_b_19225_19223; /* (list<14195>) -> 14196 list<14195> */
  kk_drop_match(_self, {kk_function_dup(_b_19225_19223);}, {}, _ctx)
  kk_std_core__list _x21248;
  kk_std_core__list _x21249 = kk_std_core__list_unbox(_b_19224, _ctx); /*list<14195>*/
  _x21248 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19225_19223, (_b_19225_19223, _x21249, _ctx)); /*list<14195>*/
  return kk_std_core__list_box(_x21248, _ctx);
}

kk_std_core__list kk_std_core__mlift17567_op(kk_function_t f, kk_integer_t i, kk_std_core__list yy, kk_box_t _y_174260, kk_context_t* _ctx) { /* forall<a,b,e> (f : (idx : int, value : a, rest : list<a>) -> e b, i : int, yy : list<a>, b) -> e list<b> */ 
  kk_integer_t i0_17131 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__list x_17969 = kk_std_core__lift17082_map_indexed_peek(f, yy, i0_17131, _ctx); /*list<14195>*/;
  if (kk_yielding(kk_context())) {
    kk_std_core__list_drop(x_17969, _ctx);
    kk_function_t _b_19225_19223 = kk_std_core__new_mlift17567_op_fun21245(_y_174260, _ctx); /*(list<14195>) -> 14196 list<14195>*/;
    kk_box_t _x21246 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17567_op_fun21247(_b_19225_19223, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21246, _ctx);
  }
  return kk_std_core__mlift17566_op(_y_174260, x_17969, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17082_map_indexed_peek_fun21254__t {
  struct kk_function_s _base;
  kk_function_t f0;
  kk_integer_t i0;
  kk_std_core__list yy0;
};
static kk_std_core__list kk_std_core__lift17082_map_indexed_peek_fun21254(kk_function_t _fself, kk_box_t _y_174261, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17082_map_indexed_peek_fun21254(kk_function_t f0, kk_integer_t i0, kk_std_core__list yy0, kk_context_t* _ctx) {
  struct kk_std_core__lift17082_map_indexed_peek_fun21254__t* _self = kk_function_alloc_as(struct kk_std_core__lift17082_map_indexed_peek_fun21254__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17082_map_indexed_peek_fun21254, kk_context());
  _self->f0 = f0;
  _self->i0 = i0;
  _self->yy0 = yy0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17082_map_indexed_peek_fun21254(kk_function_t _fself, kk_box_t _y_174261, kk_context_t* _ctx) {
  struct kk_std_core__lift17082_map_indexed_peek_fun21254__t* _self = kk_function_as(struct kk_std_core__lift17082_map_indexed_peek_fun21254__t*, _fself);
  kk_function_t f0 = _self->f0; /* (idx : int, value : 14194, rest : list<14194>) -> 14196 14195 */
  kk_integer_t i0 = _self->i0; /* int */
  kk_std_core__list yy0 = _self->yy0; /* list<14194> */
  kk_drop_match(_self, {kk_function_dup(f0);kk_integer_dup(i0);kk_std_core__list_dup(yy0);}, {}, _ctx)
  return kk_std_core__mlift17567_op(f0, i0, yy0, _y_174261, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17082_map_indexed_peek_fun21256__t {
  struct kk_function_s _base;
  kk_function_t _b_19230_19226;
};
static kk_box_t kk_std_core__lift17082_map_indexed_peek_fun21256(kk_function_t _fself, kk_box_t _b_19227, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17082_map_indexed_peek_fun21256(kk_function_t _b_19230_19226, kk_context_t* _ctx) {
  struct kk_std_core__lift17082_map_indexed_peek_fun21256__t* _self = kk_function_alloc_as(struct kk_std_core__lift17082_map_indexed_peek_fun21256__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17082_map_indexed_peek_fun21256, kk_context());
  _self->_b_19230_19226 = _b_19230_19226;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17082_map_indexed_peek_fun21256(kk_function_t _fself, kk_box_t _b_19227, kk_context_t* _ctx) {
  struct kk_std_core__lift17082_map_indexed_peek_fun21256__t* _self = kk_function_as(struct kk_std_core__lift17082_map_indexed_peek_fun21256__t*, _fself);
  kk_function_t _b_19230_19226 = _self->_b_19230_19226; /* (14195) -> 14196 list<14195> */
  kk_drop_match(_self, {kk_function_dup(_b_19230_19226);}, {}, _ctx)
  kk_std_core__list _x21257 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19230_19226, (_b_19230_19226, _b_19227, _ctx)); /*list<14195>*/
  return kk_std_core__list_box(_x21257, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17082_map_indexed_peek_fun21258__t {
  struct kk_function_s _base;
  kk_box_t x0_17971;
};
static kk_std_core__list kk_std_core__lift17082_map_indexed_peek_fun21258(kk_function_t _fself, kk_std_core__list _y_174271, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17082_map_indexed_peek_fun21258(kk_box_t x0_17971, kk_context_t* _ctx) {
  struct kk_std_core__lift17082_map_indexed_peek_fun21258__t* _self = kk_function_alloc_as(struct kk_std_core__lift17082_map_indexed_peek_fun21258__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17082_map_indexed_peek_fun21258, kk_context());
  _self->x0_17971 = x0_17971;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17082_map_indexed_peek_fun21258(kk_function_t _fself, kk_std_core__list _y_174271, kk_context_t* _ctx) {
  struct kk_std_core__lift17082_map_indexed_peek_fun21258__t* _self = kk_function_as(struct kk_std_core__lift17082_map_indexed_peek_fun21258__t*, _fself);
  kk_box_t x0_17971 = _self->x0_17971; /* 14195 */
  kk_drop_match(_self, {kk_box_dup(x0_17971);}, {}, _ctx)
  return kk_std_core__mlift17566_op(x0_17971, _y_174271, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17082_map_indexed_peek_fun21260__t {
  struct kk_function_s _base;
  kk_function_t _b_19231_19228;
};
static kk_box_t kk_std_core__lift17082_map_indexed_peek_fun21260(kk_function_t _fself, kk_box_t _b_19229, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17082_map_indexed_peek_fun21260(kk_function_t _b_19231_19228, kk_context_t* _ctx) {
  struct kk_std_core__lift17082_map_indexed_peek_fun21260__t* _self = kk_function_alloc_as(struct kk_std_core__lift17082_map_indexed_peek_fun21260__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17082_map_indexed_peek_fun21260, kk_context());
  _self->_b_19231_19228 = _b_19231_19228;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17082_map_indexed_peek_fun21260(kk_function_t _fself, kk_box_t _b_19229, kk_context_t* _ctx) {
  struct kk_std_core__lift17082_map_indexed_peek_fun21260__t* _self = kk_function_as(struct kk_std_core__lift17082_map_indexed_peek_fun21260__t*, _fself);
  kk_function_t _b_19231_19228 = _self->_b_19231_19228; /* (list<14195>) -> 14196 list<14195> */
  kk_drop_match(_self, {kk_function_dup(_b_19231_19228);}, {}, _ctx)
  kk_std_core__list _x21261;
  kk_std_core__list _x21262 = kk_std_core__list_unbox(_b_19229, _ctx); /*list<14195>*/
  _x21261 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19231_19228, (_b_19231_19228, _x21262, _ctx)); /*list<14195>*/
  return kk_std_core__list_box(_x21261, _ctx);
}

kk_std_core__list kk_std_core__lift17082_map_indexed_peek(kk_function_t f0, kk_std_core__list ys, kk_integer_t i0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (idx : int, value : a, rest : list<a>) -> e b, ys : list<a>, i : int) -> e list<b> */ 
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21250 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con21250->head;
    kk_std_core__list yy0 = _con21250->tail;
    kk_reuse_t _ru_20078 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20078 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy0);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_box_t x0_17971;
    kk_function_t _x21253 = kk_function_dup(f0); /*(idx : int, value : 14194, rest : list<14194>) -> 14196 14195*/
    kk_integer_t _x21251 = kk_integer_dup(i0); /*int*/
    kk_std_core__list _x21252 = kk_std_core__list_dup(yy0); /*list<14194>*/
    x0_17971 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_box_t, kk_std_core__list, kk_context_t*), _x21253, (_x21253, _x21251, y, _x21252, _ctx)); /*14195*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20078);
      kk_box_drop(x0_17971, _ctx);
      kk_function_t _b_19230_19226 = kk_std_core__new_lift17082_map_indexed_peek_fun21254(f0, i0, yy0, _ctx); /*(14195) -> 14196 list<14195>*/;
      kk_box_t _x21255 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17082_map_indexed_peek_fun21256(_b_19230_19226, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21255, _ctx);
    }
    kk_integer_t i0_171310 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
    kk_std_core__list x1_17974 = kk_std_core__lift17082_map_indexed_peek(f0, yy0, i0_171310, _ctx); /*list<14195>*/;
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20078);
      kk_std_core__list_drop(x1_17974, _ctx);
      kk_function_t _b_19231_19228 = kk_std_core__new_lift17082_map_indexed_peek_fun21258(x0_17971, _ctx); /*(list<14195>) -> 14196 list<14195>*/;
      kk_box_t _x21259 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17082_map_indexed_peek_fun21260(_b_19231_19228, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21259, _ctx);
    }
    return kk_std_core__new_Cons(_ru_20078, x0_17971, x1_17974, _ctx);
  }
  kk_function_drop(f0, _ctx);
  kk_integer_drop(i0, _ctx);
  return kk_std_core__new_Nil(_ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17568_op(kk_std_core_types__ctail _acc, kk_function_t action, kk_std_core__list xx, kk_std_core_types__maybe _y_17430, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, action : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Just(_y_17430)) {
    kk_box_t y = _y_17430._cons.Just.value;
    kk_std_core__list _ctail_17200 = kk_std_core__list_hole(); /*list<14237>*/;
    kk_std_core__list _ctail_17201 = kk_std_core__new_Cons(kk_reuse_null, y, _ctail_17200, _ctx); /*list<14237>*/;
    kk_std_core_types__ctail _x21263;
    kk_box_t* field_17979 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17201)->tail)); /*cfield<list<14237>>*/;
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x19238 = _acc._cons.CTail._field1;
      kk_box_t* last0 = _acc._cons.CTail._field2;
      kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19238, NULL);
      kk_std_core__list_dup(head0);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __ = kk_Unit;
      *(last0) = kk_std_core__list_box(_ctail_17201, _ctx);
      kk_std_core__list _b_19252_19243 = head0; /*list<14237>*/;
      kk_box_t* _b_19253_19244 = field_17979; /*cfield<list<14237>>*/;
      _x21263 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19252_19243, _ctx), _b_19253_19244, _ctx); /*ctail<58>*/
      goto _match21264;
    }
    kk_std_core__list _b_19254_19245 = _ctail_17201; /*list<14237>*/;
    kk_box_t* _b_19255_19246 = field_17979; /*cfield<list<14237>>*/;
    _x21263 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19254_19245, _ctx), _b_19255_19246, _ctx); /*ctail<58>*/
    _match21264: ;
    return kk_std_core__ctail_map_while(xx, action, _x21263, _ctx);
  }
  kk_function_drop(action, _ctx);
  kk_std_core__list_drop(xx, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19247 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19247, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17569_op_fun21267__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t y0;
};
static kk_std_core__list kk_std_core__mlift17569_op_fun21267(kk_function_t _fself, kk_std_core__list _ctail_17203, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17569_op_fun21267(kk_function_t _accm, kk_box_t y0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17569_op_fun21267__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17569_op_fun21267__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17569_op_fun21267, kk_context());
  _self->_accm = _accm;
  _self->y0 = y0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17569_op_fun21267(kk_function_t _fself, kk_std_core__list _ctail_17203, kk_context_t* _ctx) {
  struct kk_std_core__mlift17569_op_fun21267__t* _self = kk_function_as(struct kk_std_core__mlift17569_op_fun21267__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<14237>) -> list<14237> */
  kk_box_t y0 = _self->y0; /* 14237 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(y0);}, {}, _ctx)
  kk_std_core__list _x21268 = kk_std_core__new_Cons(kk_reuse_null, y0, _ctail_17203, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x21268, _ctx));
}

kk_std_core__list kk_std_core__mlift17569_op(kk_function_t _accm, kk_function_t action0, kk_std_core__list xx0, kk_std_core_types__maybe _y_17434, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, action : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Just(_y_17434)) {
    kk_box_t y0 = _y_17434._cons.Just.value;
    return kk_std_core__ctailm_map_while(xx0, action0, kk_std_core__new_mlift17569_op_fun21267(_accm, y0, _ctx), _ctx);
  }
  kk_function_drop(action0, _ctx);
  kk_std_core__list_drop(xx0, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Invoke `action` on each element of a list while `action` returns `Just`


// lift anonymous function
struct kk_std_core__ctail_map_while_fun21272__t {
  struct kk_function_s _base;
  kk_function_t action1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_map_while_fun21272(kk_function_t _fself, kk_std_core_types__maybe _y_174300, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_map_while_fun21272(kk_function_t action1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_while_fun21272__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_map_while_fun21272__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_map_while_fun21272, kk_context());
  _self->action1 = action1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_map_while_fun21272(kk_function_t _fself, kk_std_core_types__maybe _y_174300, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_while_fun21272__t* _self = kk_function_as(struct kk_std_core__ctail_map_while_fun21272__t*, _fself);
  kk_function_t action1 = _self->action1; /* (14236) -> 14238 maybe<14237> */
  kk_std_core__list xx1 = _self->xx1; /* list<14236> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<14237>> */
  kk_drop_match(_self, {kk_function_dup(action1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17568_op(_acc0, action1, xx1, _y_174300, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_map_while_fun21274__t {
  struct kk_function_s _base;
  kk_function_t _b_19283_19261;
};
static kk_box_t kk_std_core__ctail_map_while_fun21274(kk_function_t _fself, kk_box_t _b_19262, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_map_while_fun21274(kk_function_t _b_19283_19261, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_while_fun21274__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_map_while_fun21274__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_map_while_fun21274, kk_context());
  _self->_b_19283_19261 = _b_19283_19261;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_map_while_fun21274(kk_function_t _fself, kk_box_t _b_19262, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_while_fun21274__t* _self = kk_function_as(struct kk_std_core__ctail_map_while_fun21274__t*, _fself);
  kk_function_t _b_19283_19261 = _self->_b_19283_19261; /* (maybe<14237>) -> 14238 list<14237> */
  kk_drop_match(_self, {kk_function_dup(_b_19283_19261);}, {}, _ctx)
  kk_std_core__list _x21275;
  kk_std_core_types__maybe _x21276 = kk_std_core_types__maybe_unbox(_b_19262, _ctx); /*maybe<14237>*/
  _x21275 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19283_19261, (_b_19283_19261, _x21276, _ctx)); /*list<14237>*/
  return kk_std_core__list_box(_x21275, _ctx);
}

kk_std_core__list kk_std_core__ctail_map_while(kk_std_core__list xs, kk_function_t action1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(action1, _ctx);
    if (kk_std_core_types__is_CTail(_acc0)) {
      kk_box_t _box_x19256 = _acc0._cons.CTail._field1;
      kk_box_t* last3 = _acc0._cons.CTail._field2;
      kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19256, NULL);
      kk_std_core__list_dup(head2);
      kk_std_core_types__ctail_drop(_acc0, _ctx);
      kk_Unit;
      kk_unit_t __1 = kk_Unit;
      *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
      return head2;
    }
    return kk_std_core__new_Nil(_ctx);
  }
  struct kk_std_core_Cons* _con21270 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21270->head;
  kk_std_core__list xx1 = _con21270->tail;
  kk_reuse_t _ru_20079 = kk_reuse_null; /*reuse*/;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    _ru_20079 = (kk_std_core__list_reuse(xs));
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx1);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core_types__maybe x0_17984;
  kk_function_t _x21271 = kk_function_dup(action1); /*(14236) -> 14238 maybe<14237>*/
  x0_17984 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21271, (_x21271, x, _ctx)); /*maybe<14237>*/
  if (kk_yielding(kk_context())) {
    kk_reuse_drop(_ru_20079);
    kk_std_core_types__maybe_drop(x0_17984, _ctx);
    kk_function_t _b_19283_19261 = kk_std_core__new_ctail_map_while_fun21272(action1, xx1, _acc0, _ctx); /*(maybe<14237>) -> 14238 list<14237>*/;
    kk_box_t _x21273 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_map_while_fun21274(_b_19283_19261, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21273, _ctx);
  }
  if (kk_std_core_types__is_Just(x0_17984)) {
    kk_box_t y1 = x0_17984._cons.Just.value;
    kk_std_core__list _ctail_172000 = kk_std_core__list_hole(); /*list<14237>*/;
    kk_std_core__list _ctail_172010 = kk_std_core__new_Cons(_ru_20079, y1, _ctail_172000, _ctx); /*list<14237>*/;
    { // tailcall
      kk_std_core_types__ctail _x21277;
      kk_box_t* field0_17989 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_172010)->tail)); /*cfield<list<14237>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x19269 = _acc0._cons.CTail._field1;
        kk_box_t* last4 = _acc0._cons.CTail._field2;
        kk_std_core__list head3 = kk_std_core__list_unbox(_box_x19269, NULL);
        kk_std_core__list_dup(head3);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __2 = kk_Unit;
        *(last4) = kk_std_core__list_box(_ctail_172010, _ctx);
        kk_std_core__list _b_19284_19274 = head3; /*list<14237>*/;
        kk_box_t* _b_19285_19275 = field0_17989; /*cfield<list<14237>>*/;
        _x21277 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19284_19274, _ctx), _b_19285_19275, _ctx); /*ctail<58>*/
        goto _match21278;
      }
      kk_std_core__list _b_19286_19276 = _ctail_172010; /*list<14237>*/;
      kk_box_t* _b_19287_19277 = field0_17989; /*cfield<list<14237>>*/;
      _x21277 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19286_19276, _ctx), _b_19287_19277, _ctx); /*ctail<58>*/
      _match21278: ;
      xs = xx1;
      _acc0 = _x21277;
      goto kk__tailcall;
    }
  }
  kk_reuse_drop(_ru_20079);
  kk_function_drop(action1, _ctx);
  kk_std_core__list_drop(xx1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x19278 = _acc0._cons.CTail._field1;
    kk_box_t* last5 = _acc0._cons.CTail._field2;
    kk_std_core__list head4 = kk_std_core__list_unbox(_box_x19278, NULL);
    kk_std_core__list_dup(head4);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __3 = kk_Unit;
    *(last5) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head4;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Invoke `action` on each element of a list while `action` returns `Just`


// lift anonymous function
struct kk_std_core__ctailm_map_while_fun21283__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t action2;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_map_while_fun21283(kk_function_t _fself, kk_std_core_types__maybe _y_174340, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_while_fun21283(kk_function_t _accm0, kk_function_t action2, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21283__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_while_fun21283__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_while_fun21283, kk_context());
  _self->_accm0 = _accm0;
  _self->action2 = action2;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_map_while_fun21283(kk_function_t _fself, kk_std_core_types__maybe _y_174340, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21283__t* _self = kk_function_as(struct kk_std_core__ctailm_map_while_fun21283__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<14237>) -> list<14237> */
  kk_function_t action2 = _self->action2; /* (14236) -> 14238 maybe<14237> */
  kk_std_core__list xx2 = _self->xx2; /* list<14236> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(action2);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17569_op(_accm0, action2, xx2, _y_174340, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_map_while_fun21285__t {
  struct kk_function_s _base;
  kk_function_t _b_19290_19288;
};
static kk_box_t kk_std_core__ctailm_map_while_fun21285(kk_function_t _fself, kk_box_t _b_19289, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_while_fun21285(kk_function_t _b_19290_19288, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21285__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_while_fun21285__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_while_fun21285, kk_context());
  _self->_b_19290_19288 = _b_19290_19288;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_map_while_fun21285(kk_function_t _fself, kk_box_t _b_19289, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21285__t* _self = kk_function_as(struct kk_std_core__ctailm_map_while_fun21285__t*, _fself);
  kk_function_t _b_19290_19288 = _self->_b_19290_19288; /* (maybe<14237>) -> 14238 list<14237> */
  kk_drop_match(_self, {kk_function_dup(_b_19290_19288);}, {}, _ctx)
  kk_std_core__list _x21286;
  kk_std_core_types__maybe _x21287 = kk_std_core_types__maybe_unbox(_b_19289, _ctx); /*maybe<14237>*/
  _x21286 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19290_19288, (_b_19290_19288, _x21287, _ctx)); /*list<14237>*/
  return kk_std_core__list_box(_x21286, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_map_while_fun21289__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t y2;
};
static kk_std_core__list kk_std_core__ctailm_map_while_fun21289(kk_function_t _fself, kk_std_core__list _ctail_172030, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_while_fun21289(kk_function_t _accm0, kk_box_t y2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21289__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_while_fun21289__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_while_fun21289, kk_context());
  _self->_accm0 = _accm0;
  _self->y2 = y2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_map_while_fun21289(kk_function_t _fself, kk_std_core__list _ctail_172030, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21289__t* _self = kk_function_as(struct kk_std_core__ctailm_map_while_fun21289__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<14237>) -> list<14237> */
  kk_box_t y2 = _self->y2; /* 14237 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(y2);}, {}, _ctx)
  kk_std_core__list _x21290 = kk_std_core__new_Cons(kk_reuse_null, y2, _ctail_172030, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21290, _ctx));
}

kk_std_core__list kk_std_core__ctailm_map_while(kk_std_core__list xs0, kk_function_t action2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs0)) {
    kk_function_drop(action2, _ctx);
    return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
  }
  struct kk_std_core_Cons* _con21281 = kk_std_core__as_Cons(xs0);
  kk_box_t x1 = _con21281->head;
  kk_std_core__list xx2 = _con21281->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs0))) {
    kk_std_core__list_free(xs0);
  }
  else {
    kk_box_dup(x1);
    kk_std_core__list_dup(xx2);
    kk_Unit;
    kk_std_core__list_decref(xs0, _ctx);
    kk_Unit;
  }
  kk_std_core_types__maybe x2_17992;
  kk_function_t _x21282 = kk_function_dup(action2); /*(14236) -> 14238 maybe<14237>*/
  x2_17992 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21282, (_x21282, x1, _ctx)); /*maybe<14237>*/
  if (kk_yielding(kk_context())) {
    kk_std_core_types__maybe_drop(x2_17992, _ctx);
    kk_function_t _b_19290_19288 = kk_std_core__new_ctailm_map_while_fun21283(_accm0, action2, xx2, _ctx); /*(maybe<14237>) -> 14238 list<14237>*/;
    kk_box_t _x21284 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_map_while_fun21285(_b_19290_19288, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21284, _ctx);
  }
  if (kk_std_core_types__is_Just(x2_17992)) {
    kk_box_t y2 = x2_17992._cons.Just.value;
    { // tailcall
      kk_function_t _x21288 = kk_std_core__new_ctailm_map_while_fun21289(_accm0, y2, _ctx); /*(list<14237>) -> list<14237>*/
      xs0 = xx2;
      _accm0 = _x21288;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action2, _ctx);
  kk_std_core__list_drop(xx2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Invoke `action` on each element of a list while `action` returns `Just`


// lift anonymous function
struct kk_std_core_map_while_fun21291__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_map_while_fun21291(kk_function_t _fself, kk_std_core__list _ctail_17202, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_while_fun21291(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_map_while_fun21291, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_map_while_fun21291(kk_function_t _fself, kk_std_core__list _ctail_17202, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17202;
}

kk_std_core__list kk_std_core_map_while(kk_std_core__list xs1, kk_function_t action3, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>) -> e list<b> */ 
  bool _match_19696 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19696) {
    return kk_std_core__ctail_map_while(xs1, action3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_map_while(xs1, action3, kk_std_core_new_map_while_fun21291(_ctx), _ctx);
}
 
// Returns the largest element of a list of integers (or `default` (=`0`) for the empty list)


// lift anonymous function
struct kk_std_core_maximum_fun21297__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_core_maximum_fun21297(kk_function_t _fself, kk_integer_t _x121295, kk_integer_t _x221296, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_maximum_fun21297(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_maximum_fun21297, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_core_maximum_fun21297(kk_function_t _fself, kk_integer_t _x121295, kk_integer_t _x221296, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_max(_x121295, _x221296, _ctx);
}


// lift anonymous function
struct kk_std_core_maximum_fun21299__t {
  struct kk_function_s _base;
  kk_function_t _b_19300_19295;
};
static kk_box_t kk_std_core_maximum_fun21299(kk_function_t _fself, kk_box_t _b_19296, kk_box_t _b_19297, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_maximum_fun21299(kk_function_t _b_19300_19295, kk_context_t* _ctx) {
  struct kk_std_core_maximum_fun21299__t* _self = kk_function_alloc_as(struct kk_std_core_maximum_fun21299__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_maximum_fun21299, kk_context());
  _self->_b_19300_19295 = _b_19300_19295;
  return &_self->_base;
}

static kk_box_t kk_std_core_maximum_fun21299(kk_function_t _fself, kk_box_t _b_19296, kk_box_t _b_19297, kk_context_t* _ctx) {
  struct kk_std_core_maximum_fun21299__t* _self = kk_function_as(struct kk_std_core_maximum_fun21299__t*, _fself);
  kk_function_t _b_19300_19295 = _self->_b_19300_19295; /* (i : int, j : int) -> int */
  kk_drop_match(_self, {kk_function_dup(_b_19300_19295);}, {}, _ctx)
  kk_integer_t _x21300;
  kk_integer_t _x21301 = kk_integer_unbox(_b_19296); /*int*/
  kk_integer_t _x21302 = kk_integer_unbox(_b_19297); /*int*/
  _x21300 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), _b_19300_19295, (_b_19300_19295, _x21301, _x21302, _ctx)); /*int*/
  return kk_integer_box(_x21300);
}

kk_integer_t kk_std_core_maximum(kk_std_core__list xs, kk_std_core_types__optional default0, kk_context_t* _ctx) { /* (xs : list<int>, default : optional<int>) -> int */ 
  if (kk_std_core__is_Nil(xs)) {
    if (kk_std_core_types__is_Optional(default0)) {
      kk_box_t _box_x19291 = default0._cons.Optional.value;
      kk_integer_t _default_14286 = kk_integer_unbox(_box_x19291);
      kk_integer_dup(_default_14286);
      kk_std_core_types__optional_drop(default0, _ctx);
      kk_Unit;
      return _default_14286;
    }
    return kk_integer_from_small(0);
  }
  struct kk_std_core_Cons* _con21293 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x19292 = _con21293->head;
  kk_std_core__list xx = _con21293->tail;
  kk_integer_t x = kk_integer_unbox(_box_x19292);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_integer_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core_types__optional_drop(default0, _ctx);
  kk_Unit;
  kk_std_core__list _b_19298_19293 = xx; /*list<int>*/;
  kk_integer_t _b_19299_19294 = x; /*int*/;
  kk_function_t _b_19300_19295 = kk_std_core_new_maximum_fun21297(_ctx); /*(i : int, j : int) -> int*/;
  kk_box_t _x21298 = kk_std_core_foldl(_b_19298_19293, kk_integer_box(_b_19299_19294), kk_std_core_new_maximum_fun21299(_b_19300_19295, _ctx), _ctx); /*12247*/
  return kk_integer_unbox(_x21298);
}
 
// Returns the largest element of a list of doubles (or `0` for the empty list)


// lift anonymous function
struct kk_std_core_maximum_fun21307__t_1 {
  struct kk_function_s _base;
};
static double kk_std_core_maximum_fun21307_1(kk_function_t _fself, double _x121305, double _x221306, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_maximum_fun21307_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_maximum_fun21307_1, _ctx)
  return kk_function_dup(_fself);
}

static double kk_std_core_maximum_fun21307_1(kk_function_t _fself, double _x121305, double _x221306, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_max_1(_x121305, _x221306, _ctx);
}


// lift anonymous function
struct kk_std_core_maximum_fun21309__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19309_19304;
};
static kk_box_t kk_std_core_maximum_fun21309_1(kk_function_t _fself, kk_box_t _b_19305, kk_box_t _b_19306, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_maximum_fun21309_1(kk_function_t _b_19309_19304, kk_context_t* _ctx) {
  struct kk_std_core_maximum_fun21309__t_1* _self = kk_function_alloc_as(struct kk_std_core_maximum_fun21309__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_maximum_fun21309_1, kk_context());
  _self->_b_19309_19304 = _b_19309_19304;
  return &_self->_base;
}

static kk_box_t kk_std_core_maximum_fun21309_1(kk_function_t _fself, kk_box_t _b_19305, kk_box_t _b_19306, kk_context_t* _ctx) {
  struct kk_std_core_maximum_fun21309__t_1* _self = kk_function_as(struct kk_std_core_maximum_fun21309__t_1*, _fself);
  kk_function_t _b_19309_19304 = _self->_b_19309_19304; /* (x : double, y : double) -> double */
  kk_drop_match(_self, {kk_function_dup(_b_19309_19304);}, {}, _ctx)
  double _x21310;
  double _x21311 = kk_double_unbox(_b_19305, _ctx); /*double*/
  double _x21312 = kk_double_unbox(_b_19306, _ctx); /*double*/
  _x21310 = kk_function_call(double, (kk_function_t, double, double, kk_context_t*), _b_19309_19304, (_b_19309_19304, _x21311, _x21312, _ctx)); /*double*/
  return kk_double_box(_x21310, _ctx);
}

double kk_std_core_maximum_1(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<double>) -> double */ 
  if (kk_std_core__is_Nil(xs)) {
    return 0.0;
  }
  struct kk_std_core_Cons* _con21303 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x19301 = _con21303->head;
  kk_std_core__list xx = _con21303->tail;
  double x = kk_double_unbox(_box_x19301, NULL);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_box_drop(_box_x19301, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core__list_free(xs);
  }
  else {
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core__list _b_19307_19302 = xx; /*list<double>*/;
  double _b_19308_19303 = x; /*double*/;
  kk_function_t _b_19309_19304 = kk_std_core_new_maximum_fun21307_1(_ctx); /*(x : double, y : double) -> double*/;
  kk_box_t _x21308 = kk_std_core_foldl(_b_19307_19302, kk_double_box(_b_19308_19303, _ctx), kk_std_core_new_maximum_fun21309_1(_b_19309_19304, _ctx), _ctx); /*12247*/
  return kk_double_unbox(_x21308, _ctx);
}
 
// Convert a list to a `:maybe` type, using `Nothing` for an empty list, and otherwise `Just` on the head element.
// Note: this is just `head`.

kk_std_core_types__maybe kk_std_core_maybe_3(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> maybe<a> */ 
  if (kk_std_core__is_Nil(xs)) {
    return kk_std_core_types__new_Nothing(_ctx);
  }
  struct kk_std_core_Cons* _con21313 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21313->head;
  kk_std_core__list _pat1 = _con21313->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_drop(_pat1, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core__list_free(xs);
  }
  else {
    kk_box_dup(x);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  return kk_std_core_types__new_Just(x, _ctx);
}
 
// Transform an integer to a maybe type, using `Nothing` for `0`

kk_std_core_types__maybe kk_std_core_maybe_5(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> maybe<int> */ 
  bool _match_19691;
  kk_integer_t _borrowedArg_19692 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19693 = kk_integer_eq_borrow(i,_borrowedArg_19692,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19692, _ctx);
  _match_19691 = _borrowApp_19693; /*bool*/
  if (_match_19691) {
    kk_integer_drop(i, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_integer_t _b_19313_19312 = i; /*int*/;
  return kk_std_core_types__new_Just(kk_integer_box(_b_19313_19312), _ctx);
}
 
// Transform a string to a maybe type, using `Nothing` for an empty string

kk_std_core_types__maybe kk_std_core_maybe_6(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> maybe<string> */ 
  bool _match_19690;
  kk_string_t _x21314 = kk_string_dup(s); /*string*/
  kk_string_t _x21315 = kk_string_empty(); /*string*/
  _match_19690 = kk_string_is_eq(_x21314,_x21315,kk_context()); /*bool*/
  if (_match_19690) {
    kk_string_drop(s, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_string_t _b_19315_19314 = s; /*string*/;
  return kk_std_core_types__new_Just(kk_string_box(_b_19315_19314), _ctx);
}
 
// Transform a `:null` type to a `:maybe` type. Note that it is not
// always the case that `id(x) == maybe(null(x))` (e.g. when `x = Just(Nothing)`).

kk_std_core_types__maybe kk_std_core_maybe_8(kk_std_core__null n, kk_context_t* _ctx) { /* forall<a> (n : null<a>) -> maybe<a> */ 
  return ((n).ptr == NULL ? kk_std_core_types__new_Nothing(kk_context()) : kk_std_core_types__new_Just(kk_datatype_box(n),kk_context()));
}
 
// Returns the smallest element of a list of integers (or `default` (=`0`) for the empty list)


// lift anonymous function
struct kk_std_core_minimum_fun21322__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_core_minimum_fun21322(kk_function_t _fself, kk_integer_t _x121320, kk_integer_t _x221321, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_minimum_fun21322(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_minimum_fun21322, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_core_minimum_fun21322(kk_function_t _fself, kk_integer_t _x121320, kk_integer_t _x221321, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_min(_x121320, _x221321, _ctx);
}


// lift anonymous function
struct kk_std_core_minimum_fun21324__t {
  struct kk_function_s _base;
  kk_function_t _b_19325_19320;
};
static kk_box_t kk_std_core_minimum_fun21324(kk_function_t _fself, kk_box_t _b_19321, kk_box_t _b_19322, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_minimum_fun21324(kk_function_t _b_19325_19320, kk_context_t* _ctx) {
  struct kk_std_core_minimum_fun21324__t* _self = kk_function_alloc_as(struct kk_std_core_minimum_fun21324__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_minimum_fun21324, kk_context());
  _self->_b_19325_19320 = _b_19325_19320;
  return &_self->_base;
}

static kk_box_t kk_std_core_minimum_fun21324(kk_function_t _fself, kk_box_t _b_19321, kk_box_t _b_19322, kk_context_t* _ctx) {
  struct kk_std_core_minimum_fun21324__t* _self = kk_function_as(struct kk_std_core_minimum_fun21324__t*, _fself);
  kk_function_t _b_19325_19320 = _self->_b_19325_19320; /* (i : int, j : int) -> int */
  kk_drop_match(_self, {kk_function_dup(_b_19325_19320);}, {}, _ctx)
  kk_integer_t _x21325;
  kk_integer_t _x21326 = kk_integer_unbox(_b_19321); /*int*/
  kk_integer_t _x21327 = kk_integer_unbox(_b_19322); /*int*/
  _x21325 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), _b_19325_19320, (_b_19325_19320, _x21326, _x21327, _ctx)); /*int*/
  return kk_integer_box(_x21325);
}

kk_integer_t kk_std_core_minimum(kk_std_core__list xs, kk_std_core_types__optional default0, kk_context_t* _ctx) { /* (xs : list<int>, default : optional<int>) -> int */ 
  if (kk_std_core__is_Nil(xs)) {
    if (kk_std_core_types__is_Optional(default0)) {
      kk_box_t _box_x19316 = default0._cons.Optional.value;
      kk_integer_t _default_14765 = kk_integer_unbox(_box_x19316);
      kk_integer_dup(_default_14765);
      kk_std_core_types__optional_drop(default0, _ctx);
      kk_Unit;
      return _default_14765;
    }
    return kk_integer_from_small(0);
  }
  struct kk_std_core_Cons* _con21318 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x19317 = _con21318->head;
  kk_std_core__list xx = _con21318->tail;
  kk_integer_t x = kk_integer_unbox(_box_x19317);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_integer_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core_types__optional_drop(default0, _ctx);
  kk_Unit;
  kk_std_core__list _b_19323_19318 = xx; /*list<int>*/;
  kk_integer_t _b_19324_19319 = x; /*int*/;
  kk_function_t _b_19325_19320 = kk_std_core_new_minimum_fun21322(_ctx); /*(i : int, j : int) -> int*/;
  kk_box_t _x21323 = kk_std_core_foldl(_b_19323_19318, kk_integer_box(_b_19324_19319), kk_std_core_new_minimum_fun21324(_b_19325_19320, _ctx), _ctx); /*12247*/
  return kk_integer_unbox(_x21323);
}
 
// Returns the smallest element of a list of doubles (or `0` for the empty list)


// lift anonymous function
struct kk_std_core_minimum_fun21332__t_1 {
  struct kk_function_s _base;
};
static double kk_std_core_minimum_fun21332_1(kk_function_t _fself, double _x121330, double _x221331, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_minimum_fun21332_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_minimum_fun21332_1, _ctx)
  return kk_function_dup(_fself);
}

static double kk_std_core_minimum_fun21332_1(kk_function_t _fself, double _x121330, double _x221331, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_min_1(_x121330, _x221331, _ctx);
}


// lift anonymous function
struct kk_std_core_minimum_fun21334__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19334_19329;
};
static kk_box_t kk_std_core_minimum_fun21334_1(kk_function_t _fself, kk_box_t _b_19330, kk_box_t _b_19331, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_minimum_fun21334_1(kk_function_t _b_19334_19329, kk_context_t* _ctx) {
  struct kk_std_core_minimum_fun21334__t_1* _self = kk_function_alloc_as(struct kk_std_core_minimum_fun21334__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_minimum_fun21334_1, kk_context());
  _self->_b_19334_19329 = _b_19334_19329;
  return &_self->_base;
}

static kk_box_t kk_std_core_minimum_fun21334_1(kk_function_t _fself, kk_box_t _b_19330, kk_box_t _b_19331, kk_context_t* _ctx) {
  struct kk_std_core_minimum_fun21334__t_1* _self = kk_function_as(struct kk_std_core_minimum_fun21334__t_1*, _fself);
  kk_function_t _b_19334_19329 = _self->_b_19334_19329; /* (x : double, y : double) -> double */
  kk_drop_match(_self, {kk_function_dup(_b_19334_19329);}, {}, _ctx)
  double _x21335;
  double _x21336 = kk_double_unbox(_b_19330, _ctx); /*double*/
  double _x21337 = kk_double_unbox(_b_19331, _ctx); /*double*/
  _x21335 = kk_function_call(double, (kk_function_t, double, double, kk_context_t*), _b_19334_19329, (_b_19334_19329, _x21336, _x21337, _ctx)); /*double*/
  return kk_double_box(_x21335, _ctx);
}

double kk_std_core_minimum_1(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<double>) -> double */ 
  if (kk_std_core__is_Nil(xs)) {
    return 0.0;
  }
  struct kk_std_core_Cons* _con21328 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x19326 = _con21328->head;
  kk_std_core__list xx = _con21328->tail;
  double x = kk_double_unbox(_box_x19326, NULL);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_box_drop(_box_x19326, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core__list_free(xs);
  }
  else {
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core__list _b_19332_19327 = xx; /*list<double>*/;
  double _b_19333_19328 = x; /*double*/;
  kk_function_t _b_19334_19329 = kk_std_core_new_minimum_fun21332_1(_ctx); /*(x : double, y : double) -> double*/;
  kk_box_t _x21333 = kk_std_core_foldl(_b_19332_19327, kk_double_box(_b_19333_19328, _ctx), kk_std_core_new_minimum_fun21334_1(_b_19334_19329, _ctx), _ctx); /*12247*/
  return kk_double_unbox(_x21333, _ctx);
}
 
// Disable tracing completely.

kk_unit_t kk_std_core_notrace(kk_context_t* _ctx) { /* () -> (st<global>) () */ 
  kk_ref_t _b_19337_19335 = kk_ref_dup(kk_std_core_trace_enabled); /*ref<global,bool>*/;
  bool _b_19338_19336 = false; /*bool*/;
  kk_unit_t _borrowApp_19687 = kk_Unit;
  kk_ref_set_borrow(_b_19337_19335,(kk_bool_box(_b_19338_19336)),kk_context());
  kk_ref_drop(_b_19337_19335, _ctx);
  _borrowApp_19687; return kk_Unit;
}
 
// Transform a `:maybe` type to a `:null` type (using `null` for `Nothing`).

kk_std_core__null kk_std_core_null(kk_std_core_types__maybe x, kk_context_t* _ctx) { /* forall<a> (x : maybe<a>) -> null<a> */ 
  return (kk_std_core_types__is_Nothing(x) ? kk_datatype_from_ptr(NULL) : kk_datatype_unbox((x)._cons.Just.value));
}
 
// Cast a integer that is zero to a null

kk_std_core__null kk_std_core_null_1(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> null<int> */ 
  kk_std_core_types__maybe _x21338;
  bool _match_19684;
  kk_integer_t _borrowedArg_19685 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19686 = kk_integer_eq_borrow(i,_borrowedArg_19685,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19685, _ctx);
  _match_19684 = _borrowApp_19686; /*bool*/
  if (_match_19684) {
    kk_integer_drop(i, _ctx);
    _x21338 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/
  }
  else {
    kk_integer_t _b_19340_19339 = i; /*int*/;
    _x21338 = kk_std_core_types__new_Just(kk_integer_box(_b_19340_19339), _ctx); /*forall<a> maybe<a>*/
  }
  return kk_std_core_null(_x21338, _ctx);
}
 
// Cast an empty string a null

kk_std_core__null kk_std_core_null_2(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> null<string> */ 
  kk_std_core_types__maybe _x21339;
  bool _match_19683;
  kk_string_t _x21340 = kk_string_dup(s); /*string*/
  kk_string_t _x21341 = kk_string_empty(); /*string*/
  _match_19683 = kk_string_is_eq(_x21340,_x21341,kk_context()); /*bool*/
  if (_match_19683) {
    kk_string_drop(s, _ctx);
    _x21339 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/
  }
  else {
    kk_string_t _b_19342_19341 = s; /*string*/;
    _x21339 = kk_std_core_types__new_Just(kk_string_box(_b_19342_19341), _ctx); /*forall<a> maybe<a>*/
  }
  return kk_std_core_null(_x21339, _ctx);
}
 
// Left-align a string to width `width`  using `fill`  (default is a space) to fill on the right.

kk_string_t kk_std_core_pad_right(kk_string_t s, kk_integer_t width, kk_std_core_types__optional fill, kk_context_t* _ctx) { /* (s : string, width : int, fill : optional<char>) -> string */ 
  kk_ssize_t w;
  kk_integer_t _x21344 = kk_integer_dup(width); /*int*/
  w = kk_std_core_ssize__t(_x21344, _ctx); /*ssize_t*/
  kk_ssize_t n;
  kk_string_t _x21345 = kk_string_dup(s); /*string*/
  n = kk_string_len(_x21345,kk_context()); /*ssize_t*/
  bool _match_19682 = (w <= n); /*bool*/;
  if (_match_19682) {
    kk_std_core_types__optional_drop(fill, _ctx);
    return s;
  }
  kk_string_t _x21346;
  kk_string_t _x21347;
  kk_char_t _x21348;
  if (kk_std_core_types__is_Optional(fill)) {
    kk_box_t _box_x19345 = fill._cons.Optional.value;
    kk_char_t _fill_15167 = kk_char_unbox(_box_x19345, NULL);
    kk_std_core_types__optional_drop(fill, _ctx);
    kk_Unit;
    _x21348 = _fill_15167; /*char*/
    goto _match21349;
  }
  _x21348 = ' '; /*char*/
  _match21349: ;
  _x21347 = kk_std_core_string(_x21348, _ctx); /*string*/
  kk_ssize_t _x21351 = (w - n); /*ssize_t*/
  _x21346 = kk_std_core_repeatz(_x21347, _x21351, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(s, _x21346, _ctx);
}
 
// Is `pre`  a prefix of `s`? If so, returns a slice
// of `s` following `pre` up to the end of `s`.

kk_std_core_types__maybe kk_std_core_starts_with(kk_string_t s, kk_string_t pre, kk_context_t* _ctx) { /* (s : string, pre : string) -> maybe<sslice> */ 
  bool _match_19681;
  kk_string_t _x21352 = kk_string_dup(s); /*string*/
  kk_string_t _x21353 = kk_string_dup(pre); /*string*/
  _match_19681 = kk_string_starts_with(_x21352,_x21353,kk_context()); /*bool*/
  if (_match_19681) {
    kk_std_core__sslice _b_19347_19346;
    kk_string_t _x21354 = kk_string_dup(s); /*string*/
    kk_ssize_t _x21355;
    kk_string_t _x21356 = kk_string_dup(pre); /*string*/
    _x21355 = kk_string_len(_x21356,kk_context()); /*ssize_t*/
    kk_ssize_t _x21357;
    kk_ssize_t _x21358 = kk_string_len(s,kk_context()); /*ssize_t*/
    kk_ssize_t _x21359 = kk_string_len(pre,kk_context()); /*ssize_t*/
    _x21357 = (_x21358 - _x21359); /*ssize_t*/
    _b_19347_19346 = kk_std_core__new_Sslice(_x21354, _x21355, _x21357, _ctx); /*sslice*/
    return kk_std_core_types__new_Just(kk_std_core__sslice_box(_b_19347_19346, _ctx), _ctx);
  }
  kk_string_drop(pre, _ctx);
  kk_string_drop(s, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Trim off a substring `sub` while `s` starts with that string.

kk_string_t kk_std_core_trim_left_1(kk_string_t s, kk_string_t sub, kk_context_t* _ctx) { /* (s : string, sub : string) -> string */ 
  kk__tailcall: ;
  bool _match_19679;
  kk_string_t _x21360 = kk_string_dup(sub); /*string*/
  kk_string_t _x21361 = kk_string_empty(); /*string*/
  _match_19679 = kk_string_is_eq(_x21360,_x21361,kk_context()); /*bool*/
  if (_match_19679) {
    kk_string_drop(sub, _ctx);
    return s;
  }
  kk_std_core_types__maybe _match_19680;
  kk_string_t _x21363 = kk_string_dup(s); /*string*/
  kk_string_t _x21364 = kk_string_dup(sub); /*string*/
  _match_19680 = kk_std_core_starts_with(_x21363, _x21364, _ctx); /*maybe<sslice>*/
  if (kk_std_core_types__is_Just(_match_19680)) {
    kk_box_t _box_x19348 = _match_19680._cons.Just.value;
    kk_std_core__sslice slice0 = kk_std_core__sslice_unbox(_box_x19348, NULL);
    kk_string_drop(s, _ctx);
    kk_std_core__sslice_dup(slice0);
    kk_std_core_types__maybe_drop(_match_19680, _ctx);
    kk_Unit;
    { // tailcall
      kk_string_t _x21366 = kk_std_core_string_3(slice0, _ctx); /*string*/
      s = _x21366;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sub, _ctx);
  return s;
}
 
// Trim off a substring `sub` while `s` ends with that string.

kk_string_t kk_std_core_trim_right_1(kk_string_t s, kk_string_t sub, kk_context_t* _ctx) { /* (s : string, sub : string) -> string */ 
  kk__tailcall: ;
  bool _match_19677;
  kk_string_t _x21367 = kk_string_dup(sub); /*string*/
  kk_string_t _x21368 = kk_string_empty(); /*string*/
  _match_19677 = kk_string_is_eq(_x21367,_x21368,kk_context()); /*bool*/
  if (_match_19677) {
    kk_string_drop(sub, _ctx);
    return s;
  }
  kk_std_core_types__maybe _match_19678;
  kk_string_t _x21370 = kk_string_dup(s); /*string*/
  kk_string_t _x21371 = kk_string_dup(sub); /*string*/
  _match_19678 = kk_std_core_ends_with(_x21370, _x21371, _ctx); /*maybe<sslice>*/
  if (kk_std_core_types__is_Just(_match_19678)) {
    kk_box_t _box_x19349 = _match_19678._cons.Just.value;
    kk_std_core__sslice slice0 = kk_std_core__sslice_unbox(_box_x19349, NULL);
    kk_string_drop(s, _ctx);
    kk_std_core__sslice_dup(slice0);
    kk_std_core_types__maybe_drop(_match_19678, _ctx);
    kk_Unit;
    { // tailcall
      kk_string_t _x21373 = kk_std_core_string_3(slice0, _ctx); /*string*/
      s = _x21373;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sub, _ctx);
  return s;
}
 
// Parse an integer using `parseInt`. If an illegal digit character is encountered the
// `default` value is returned. An empty string will also result in `default`.

kk_integer_t kk_std_core_parse_int_default(kk_string_t s, kk_std_core_types__optional default0, kk_std_core_types__optional hex, kk_context_t* _ctx) { /* (s : string, default : optional<int>, hex : optional<bool>) -> int */ 
  bool _match_19676;
  kk_string_t _x21380 = kk_string_dup(s); /*string*/
  kk_string_t _x21381 = kk_string_empty(); /*string*/
  _match_19676 = kk_string_is_eq(_x21380,_x21381,kk_context()); /*bool*/
  if (_match_19676) {
    kk_std_core_types__optional_drop(hex, _ctx);
    kk_string_drop(s, _ctx);
    if (kk_std_core_types__is_Optional(default0)) {
      kk_box_t _box_x19351 = default0._cons.Optional.value;
      kk_integer_t _default_15449 = kk_integer_unbox(_box_x19351);
      kk_integer_dup(_default_15449);
      kk_std_core_types__optional_drop(default0, _ctx);
      kk_Unit;
      return _default_15449;
    }
    return kk_integer_from_small(0);
  }
  kk_std_core_types__maybe m_17038;
  kk_string_t _x21384;
  kk_string_t _x21385 = kk_string_trim_left(s,kk_context()); /*string*/
  _x21384 = kk_string_trim_right(_x21385,kk_context()); /*string*/
  bool _x21386;
  if (kk_std_core_types__is_Optional(hex)) {
    kk_box_t _box_x19352 = hex._cons.Optional.value;
    bool _hex_15453 = kk_bool_unbox(_box_x19352);
    kk_std_core_types__optional_drop(hex, _ctx);
    kk_Unit;
    _x21386 = _hex_15453; /*bool*/
    goto _match21387;
  }
  _x21386 = false; /*bool*/
  _match21387: ;
  m_17038 = kk_std_core_xparse_int(_x21384, _x21386, _ctx); /*maybe<int>*/
  if (kk_std_core_types__is_Nothing(m_17038)) {
    if (kk_std_core_types__is_Optional(default0)) {
      kk_box_t _box_x19353 = default0._cons.Optional.value;
      kk_integer_t _default_154490 = kk_integer_unbox(_box_x19353);
      kk_integer_dup(_default_154490);
      kk_std_core_types__optional_drop(default0, _ctx);
      kk_Unit;
      return _default_154490;
    }
    return kk_integer_from_small(0);
  }
  kk_box_t _box_x19354 = m_17038._cons.Just.value;
  kk_integer_t x = kk_integer_unbox(_box_x19354);
  kk_integer_dup(x);
  kk_std_core_types__maybe_drop(m_17038, _ctx);
  kk_Unit;
  kk_std_core_types__optional_drop(default0, _ctx);
  kk_Unit;
  return x;
}
 
// monadic lift

kk_std_core_types__tuple2_ kk_std_core__mlift17570_partition_acc(kk_std_core__list acc1, kk_std_core__list acc2, kk_function_t pred, kk_box_t x, kk_std_core__list xx, bool _y_17443, kk_context_t* _ctx) { /* forall<a,e> (acc1 : list<a>, acc2 : list<a>, pred : (a) -> e bool, x : a, xx : list<a>, bool) -> e (list<a>, list<a>) */ 
  if (_y_17443) {
    kk_std_core__list _x21391 = kk_std_core__new_Cons(kk_reuse_null, x, acc1, _ctx); /*list<61>*/
    return kk_std_core_partition_acc(xx, pred, _x21391, acc2, _ctx);
  }
  kk_std_core__list _x21392 = kk_std_core__new_Cons(kk_reuse_null, x, acc2, _ctx); /*list<61>*/
  return kk_std_core_partition_acc(xx, pred, acc1, _x21392, _ctx);
}


// lift anonymous function
struct kk_std_core_partition_acc_fun21396__t {
  struct kk_function_s _base;
  kk_std_core__list acc10;
  kk_std_core__list acc20;
  kk_function_t pred0;
  kk_box_t x0;
  kk_std_core__list xx0;
};
static kk_std_core_types__tuple2_ kk_std_core_partition_acc_fun21396(kk_function_t _fself, bool _y_174430, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_partition_acc_fun21396(kk_std_core__list acc10, kk_std_core__list acc20, kk_function_t pred0, kk_box_t x0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_partition_acc_fun21396__t* _self = kk_function_alloc_as(struct kk_std_core_partition_acc_fun21396__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_partition_acc_fun21396, kk_context());
  _self->acc10 = acc10;
  _self->acc20 = acc20;
  _self->pred0 = pred0;
  _self->x0 = x0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_core_partition_acc_fun21396(kk_function_t _fself, bool _y_174430, kk_context_t* _ctx) {
  struct kk_std_core_partition_acc_fun21396__t* _self = kk_function_as(struct kk_std_core_partition_acc_fun21396__t*, _fself);
  kk_std_core__list acc10 = _self->acc10; /* list<15571> */
  kk_std_core__list acc20 = _self->acc20; /* list<15571> */
  kk_function_t pred0 = _self->pred0; /* (15571) -> 15572 bool */
  kk_box_t x0 = _self->x0; /* 15571 */
  kk_std_core__list xx0 = _self->xx0; /* list<15571> */
  kk_drop_match(_self, {kk_std_core__list_dup(acc10);kk_std_core__list_dup(acc20);kk_function_dup(pred0);kk_box_dup(x0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17570_partition_acc(acc10, acc20, pred0, x0, xx0, _y_174430, _ctx);
}


// lift anonymous function
struct kk_std_core_partition_acc_fun21398__t {
  struct kk_function_s _base;
  kk_function_t _b_19361_19357;
};
static kk_box_t kk_std_core_partition_acc_fun21398(kk_function_t _fself, kk_box_t _b_19358, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_partition_acc_fun21398(kk_function_t _b_19361_19357, kk_context_t* _ctx) {
  struct kk_std_core_partition_acc_fun21398__t* _self = kk_function_alloc_as(struct kk_std_core_partition_acc_fun21398__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_partition_acc_fun21398, kk_context());
  _self->_b_19361_19357 = _b_19361_19357;
  return &_self->_base;
}

static kk_box_t kk_std_core_partition_acc_fun21398(kk_function_t _fself, kk_box_t _b_19358, kk_context_t* _ctx) {
  struct kk_std_core_partition_acc_fun21398__t* _self = kk_function_as(struct kk_std_core_partition_acc_fun21398__t*, _fself);
  kk_function_t _b_19361_19357 = _self->_b_19361_19357; /* (bool) -> 15572 (list<15571>, list<15571>) */
  kk_drop_match(_self, {kk_function_dup(_b_19361_19357);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x21399;
  bool _x21400 = kk_bool_unbox(_b_19358); /*bool*/
  _x21399 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, bool, kk_context_t*), _b_19361_19357, (_b_19361_19357, _x21400, _ctx)); /*(list<15571>, list<15571>)*/
  return kk_std_core_types__tuple2__box(_x21399, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core_partition_acc(kk_std_core__list xs, kk_function_t pred0, kk_std_core__list acc10, kk_std_core__list acc20, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, pred : (a) -> e bool, acc1 : list<a>, acc2 : list<a>) -> e (list<a>, list<a>) */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(pred0, _ctx);
    kk_std_core__list _b_19359_19355 = kk_std_core__lift17078_reverse(kk_std_core__new_Nil(_ctx), acc10, _ctx); /*list<15571>*/;
    kk_std_core__list _b_19360_19356 = kk_std_core__lift17078_reverse(kk_std_core__new_Nil(_ctx), acc20, _ctx); /*list<15571>*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19359_19355, _ctx), kk_std_core__list_box(_b_19360_19356, _ctx), _ctx);
  }
  struct kk_std_core_Cons* _con21393 = kk_std_core__as_Cons(xs);
  kk_box_t x0 = _con21393->head;
  kk_std_core__list xx0 = _con21393->tail;
  kk_reuse_t _ru_20086 = kk_reuse_null; /*reuse*/;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    _ru_20086 = (kk_std_core__list_reuse(xs));
  }
  else {
    kk_box_dup(x0);
    kk_std_core__list_dup(xx0);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  bool x1_17997;
  kk_function_t _x21395 = kk_function_dup(pred0); /*(15571) -> 15572 bool*/
  kk_box_t _x21394 = kk_box_dup(x0); /*15571*/
  x1_17997 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21395, (_x21395, _x21394, _ctx)); /*bool*/
  if (kk_yielding(kk_context())) {
    kk_reuse_drop(_ru_20086);
    kk_function_t _b_19361_19357 = kk_std_core_new_partition_acc_fun21396(acc10, acc20, pred0, x0, xx0, _ctx); /*(bool) -> 15572 (list<15571>, list<15571>)*/;
    kk_box_t _x21397 = kk_std_core_hnd_yield_extend(kk_std_core_new_partition_acc_fun21398(_b_19361_19357, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple2__unbox(_x21397, _ctx);
  }
  if (x1_17997) {
    { // tailcall
      kk_std_core__list _x21401;
      if (kk_likely(_ru_20086!=NULL)) {
        struct kk_std_core_Cons* _con21402 = (struct kk_std_core_Cons*)_ru_20086;
        _con21402->tail = acc10;
        _x21401 = kk_std_core__base_Cons(_con21402); /*list<61>*/
      }
      else {
        _x21401 = kk_std_core__new_Cons(kk_reuse_null, x0, acc10, _ctx); /*list<61>*/
      }
      xs = xx0;
      acc10 = _x21401;
      goto kk__tailcall;
    }
  }
  { // tailcall
    kk_std_core__list _x21403;
    if (kk_likely(_ru_20086!=NULL)) {
      struct kk_std_core_Cons* _con21404 = (struct kk_std_core_Cons*)_ru_20086;
      _con21404->tail = acc20;
      _x21403 = kk_std_core__base_Cons(_con21404); /*list<61>*/
    }
    else {
      _x21403 = kk_std_core__new_Cons(kk_reuse_null, x0, acc20, _ctx); /*list<61>*/
    }
    xs = xx0;
    acc20 = _x21403;
    goto kk__tailcall;
  }
}
 
// redirect `print` and `println` calls to a specified function.


// lift anonymous function
struct kk_std_core_print_redirect_fun21405__t {
  struct kk_function_s _base;
  kk_function_t _b_19370_19362;
};
static kk_box_t kk_std_core_print_redirect_fun21405(kk_function_t _fself, kk_box_t _b_19365, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_print_redirect_fun21405(kk_function_t _b_19370_19362, kk_context_t* _ctx) {
  struct kk_std_core_print_redirect_fun21405__t* _self = kk_function_alloc_as(struct kk_std_core_print_redirect_fun21405__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_print_redirect_fun21405, kk_context());
  _self->_b_19370_19362 = _b_19370_19362;
  return &_self->_base;
}

static kk_box_t kk_std_core_print_redirect_fun21405(kk_function_t _fself, kk_box_t _b_19365, kk_context_t* _ctx) {
  struct kk_std_core_print_redirect_fun21405__t* _self = kk_function_as(struct kk_std_core_print_redirect_fun21405__t*, _fself);
  kk_function_t _b_19370_19362 = _self->_b_19370_19362; /* (msg : string) -> console () */
  kk_drop_match(_self, {kk_function_dup(_b_19370_19362);}, {}, _ctx)
  kk_unit_t _x21406 = kk_Unit;
  kk_string_t _x21407 = kk_string_unbox(_b_19365); /*string*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_string_t, kk_context_t*), _b_19370_19362, (_b_19370_19362, _x21407, _ctx));
  return kk_unit_box(_x21406);
}

kk_unit_t kk_std_core_print_redirect(kk_function_t print0, kk_context_t* _ctx) { /* (print : (msg : string) -> console ()) -> io () */ 
  kk_ref_t _b_19368_19366 = kk_ref_dup(kk_std_core_redirect); /*ref<global,maybe<(string) -> console ()>>*/;
  kk_std_core_types__maybe _b_19369_19367;
  kk_function_t _b_19370_19362 = print0; /*(msg : string) -> console ()*/;
  _b_19369_19367 = kk_std_core_types__new_Just(kk_function_box(kk_std_core_new_print_redirect_fun21405(_b_19370_19362, _ctx)), _ctx); /*maybe<(msg : string) -> console ()>*/
  kk_unit_t _borrowApp_19674 = kk_Unit;
  kk_ref_set_borrow(_b_19368_19366,(kk_std_core_types__maybe_box(_b_19369_19367, _ctx)),kk_context());
  kk_ref_drop(_b_19368_19366, _ctx);
  _borrowApp_19674; return kk_Unit;
}
extern bool kk_std_core_remove_fun21408(kk_function_t _fself, kk_box_t x, kk_context_t* _ctx) {
  struct kk_std_core_remove_fun21408__t* _self = kk_function_as(struct kk_std_core_remove_fun21408__t*, _fself);
  kk_function_t pred = _self->pred; /* (15640) -> bool */
  kk_drop_match(_self, {kk_function_dup(pred);}, {}, _ctx)
  bool b_17045 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), pred, (pred, x, _ctx)); /*bool*/;
  if (b_17045) {
    return false;
  }
  return true;
}
extern kk_unit_t kk_std_core_repeat_fun21411_1(kk_function_t _fself, kk_integer_t i, kk_context_t* _ctx) {
  struct kk_std_core_repeat_fun21411__t_1* _self = kk_function_as(struct kk_std_core_repeat_fun21411__t_1*, _fself);
  kk_function_t action = _self->action; /* () -> 15659 () */
  kk_drop_match(_self, {kk_function_dup(action);}, {}, _ctx)
  kk_integer_drop(i, _ctx);
  return kk_function_call(kk_unit_t, (kk_function_t, kk_context_t*), action, (action, _ctx));
}
 
// Create a list of `n`  repeated elementes `x`

kk_std_core__list kk_std_core__ctail_replicate(kk_box_t x, kk_integer_t n, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (x : a, n : int, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  bool _match_19671;
  kk_integer_t _borrowedArg_19672 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19673 = kk_integer_gt_borrow(n,_borrowedArg_19672,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19672, _ctx);
  _match_19671 = _borrowApp_19673; /*bool*/
  if (_match_19671) {
    kk_std_core__list _ctail_17204 = kk_std_core__list_hole(); /*list<15703>*/;
    kk_std_core__list _ctail_17205;
    kk_box_t _x21413 = kk_box_dup(x); /*15703*/
    _ctail_17205 = kk_std_core__new_Cons(kk_reuse_null, _x21413, _ctail_17204, _ctx); /*list<15703>*/
    { // tailcall
      kk_integer_t _x21414 = kk_integer_sub(n,(kk_integer_from_small(1)),kk_context()); /*int*/
      kk_std_core_types__ctail _x21415;
      kk_box_t* field_18005 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17205)->tail)); /*cfield<list<15703>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x19377 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19377, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17205, _ctx);
        kk_std_core__list _b_19391_19382 = head0; /*list<15703>*/;
        kk_box_t* _b_19392_19383 = field_18005; /*cfield<list<15703>>*/;
        _x21415 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19391_19382, _ctx), _b_19392_19383, _ctx); /*ctail<58>*/
        goto _match21416;
      }
      kk_std_core__list _b_19393_19384 = _ctail_17205; /*list<15703>*/;
      kk_box_t* _b_19394_19385 = field_18005; /*cfield<list<15703>>*/;
      _x21415 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19393_19384, _ctx), _b_19394_19385, _ctx); /*ctail<58>*/
      _match21416: ;
      n = _x21414;
      _acc = _x21415;
      goto kk__tailcall;
    }
  }
  kk_integer_drop(n, _ctx);
  kk_box_drop(x, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19386 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19386, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Create a list of `n`  repeated elementes `x`

kk_std_core__list kk_std_core_replicate(kk_box_t x0, kk_integer_t n0, kk_context_t* _ctx) { /* forall<a> (x : a, n : int) -> list<a> */ 
  return kk_std_core__ctail_replicate(x0, n0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17083_reverse_join(kk_std_core__list acc, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a> (acc : list<a>, ys : list<a>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21419 = kk_std_core__as_Cons(ys);
    kk_box_t x = _con21419->head;
    kk_std_core__list xx = _con21419->tail;
    kk_reuse_t _ru_20087 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20087 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x21420;
      if (kk_likely(_ru_20087!=NULL)) {
        struct kk_std_core_Cons* _con21421 = (struct kk_std_core_Cons*)_ru_20087;
        _con21421->tail = acc;
        _x21420 = kk_std_core__base_Cons(_con21421); /*list<61>*/
      }
      else {
        _x21420 = kk_std_core__new_Cons(kk_reuse_null, x, acc, _ctx); /*list<61>*/
      }
      acc = _x21420;
      ys = xx;
      goto kk__tailcall;
    }
  }
  return acc;
}
 
// lifted

kk_string_t kk_std_core__lift17084_reverse_join(kk_string_t sep_17048, kk_std_core__list ys0, kk_string_t acc0, kk_context_t* _ctx) { /* (sep.17048 : string, ys0 : list<string>, acc0 : string) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys0)) {
    struct kk_std_core_Cons* _con21422 = kk_std_core__as_Cons(ys0);
    kk_box_t _box_x19395 = _con21422->head;
    kk_std_core__list yy = _con21422->tail;
    kk_string_t y = kk_string_unbox(_box_x19395);
    if (kk_likely(kk_std_core__list_is_unique(ys0))) {
      kk_std_core__list_free(ys0);
    }
    else {
      kk_string_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys0, _ctx);
      kk_Unit;
    }
    kk_string_t acc00_17133;
    kk_string_t _x21424;
    kk_string_t _x21425 = kk_string_dup(sep_17048); /*string*/
    _x21424 = kk_std_core__lp__plus__plus__1_rp_(_x21425, y, _ctx); /*string*/
    acc00_17133 = kk_std_core__lp__plus__plus__1_rp_(acc0, _x21424, _ctx); /*string*/
    { // tailcall
      ys0 = yy;
      acc0 = acc00_17133;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sep_17048, _ctx);
  return acc0;
}
 
// Concatenate all strings in a list in reverse order

kk_string_t kk_std_core_reverse_join(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<string>) -> string */ 
  kk_std_core__list xs0_17047 = kk_std_core__lift17083_reverse_join(kk_std_core__new_Nil(_ctx), xs, _ctx); /*list<string>*/;
  if (kk_std_core__is_Nil(xs0_17047)) {
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con21427 = kk_std_core__as_Cons(xs0_17047);
  kk_box_t _box_x19396 = _con21427->head;
  kk_std_core__list xx0 = _con21427->tail;
  kk_string_t x0 = kk_string_unbox(_box_x19396);
  if (kk_likely(kk_std_core__list_is_unique(xs0_17047))) {
    kk_std_core__list_free(xs0_17047);
  }
  else {
    kk_string_dup(x0);
    kk_std_core__list_dup(xx0);
    kk_Unit;
    kk_std_core__list_decref(xs0_17047, _ctx);
    kk_Unit;
  }
  kk_string_t _x21429 = kk_string_empty(); /*string*/
  return kk_std_core__lift17084_reverse_join(_x21429, xx0, x0, _ctx);
}

kk_string_t kk_std_core_show_tuple(kk_std_core_types__tuple2_ x, kk_function_t showfst, kk_function_t showsnd, kk_context_t* _ctx) { /* forall<a,b> (x : (a, b), showfst : (a) -> string, showsnd : (b) -> string) -> string */ 
  kk_string_t _x21431;
  kk_define_string_literal(, _s21432, 1, "(")
  _x21431 = kk_string_dup(_s21432); /*string*/
  kk_string_t _x21433;
  kk_string_t _x21434;
  kk_box_t _x21435;
  kk_box_t _x = x.fst;
  kk_box_dup(_x);
  _x21435 = _x; /*16023*/
  _x21434 = kk_function_call(kk_string_t, (kk_function_t, kk_box_t, kk_context_t*), showfst, (showfst, _x21435, _ctx)); /*string*/
  kk_string_t _x21436;
  kk_string_t _x21437;
  kk_define_string_literal(, _s21438, 1, ",")
  _x21437 = kk_string_dup(_s21438); /*string*/
  kk_string_t _x21439;
  kk_string_t _x21440;
  kk_box_t _x21441;
  kk_box_t _x0 = x.snd;
  kk_box_dup(_x0);
  kk_std_core_types__tuple2__drop(x, _ctx);
  kk_Unit;
  _x21441 = _x0; /*16024*/
  _x21440 = kk_function_call(kk_string_t, (kk_function_t, kk_box_t, kk_context_t*), showsnd, (showsnd, _x21441, _ctx)); /*string*/
  kk_string_t _x21442;
  kk_define_string_literal(, _s21443, 1, ")")
  _x21442 = kk_string_dup(_s21443); /*string*/
  _x21439 = kk_std_core__lp__plus__plus__1_rp_(_x21440, _x21442, _ctx); /*string*/
  _x21436 = kk_std_core__lp__plus__plus__1_rp_(_x21437, _x21439, _ctx); /*string*/
  _x21433 = kk_std_core__lp__plus__plus__1_rp_(_x21434, _x21436, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x21431, _x21433, _ctx);
}
 
// monadic lift

kk_std_core_types__tuple2_ kk_std_core__mlift17571_op(kk_std_core__list acc, kk_function_t predicate, kk_box_t y, kk_std_core__list ys, kk_std_core__list yy, bool _y_17451, kk_context_t* _ctx) { /* forall<a,e> (acc : list<a>, predicate : (a) -> e bool, y : a, ys : list<a>, yy : list<a>, bool) -> e (list<a>, list<a>) */ 
  if (_y_17451) {
    kk_std_core__list_drop(ys, _ctx);
    kk_std_core__list _x21444 = kk_std_core__new_Cons(kk_reuse_null, y, acc, _ctx); /*list<61>*/
    return kk_std_core__lift17085_span(predicate, yy, _x21444, _ctx);
  }
  kk_function_drop(predicate, _ctx);
  kk_box_drop(y, _ctx);
  kk_std_core__list_drop(yy, _ctx);
  kk_std_core__list _b_19399_19397 = kk_std_core__lift17078_reverse(kk_std_core__new_Nil(_ctx), acc, _ctx); /*list<16082>*/;
  kk_std_core__list _b_19400_19398 = ys; /*list<16082>*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19399_19397, _ctx), kk_std_core__list_box(_b_19400_19398, _ctx), _ctx);
}
 
// lifted
// todo: implement TRMC with multiple results to avoid the reverse


// lift anonymous function
struct kk_std_core__lift17085_span_fun21448__t {
  struct kk_function_s _base;
  kk_std_core__list acc0;
  kk_function_t predicate0;
  kk_box_t y0;
  kk_std_core__list ys0;
  kk_std_core__list yy0;
};
static kk_std_core_types__tuple2_ kk_std_core__lift17085_span_fun21448(kk_function_t _fself, bool _y_174510, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17085_span_fun21448(kk_std_core__list acc0, kk_function_t predicate0, kk_box_t y0, kk_std_core__list ys0, kk_std_core__list yy0, kk_context_t* _ctx) {
  struct kk_std_core__lift17085_span_fun21448__t* _self = kk_function_alloc_as(struct kk_std_core__lift17085_span_fun21448__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17085_span_fun21448, kk_context());
  _self->acc0 = acc0;
  _self->predicate0 = predicate0;
  _self->y0 = y0;
  _self->ys0 = ys0;
  _self->yy0 = yy0;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_core__lift17085_span_fun21448(kk_function_t _fself, bool _y_174510, kk_context_t* _ctx) {
  struct kk_std_core__lift17085_span_fun21448__t* _self = kk_function_as(struct kk_std_core__lift17085_span_fun21448__t*, _fself);
  kk_std_core__list acc0 = _self->acc0; /* list<16082> */
  kk_function_t predicate0 = _self->predicate0; /* (16082) -> 16083 bool */
  kk_box_t y0 = _self->y0; /* 16082 */
  kk_std_core__list ys0 = _self->ys0; /* list<16082> */
  kk_std_core__list yy0 = _self->yy0; /* list<16082> */
  kk_drop_match(_self, {kk_std_core__list_dup(acc0);kk_function_dup(predicate0);kk_box_dup(y0);kk_std_core__list_dup(ys0);kk_std_core__list_dup(yy0);}, {}, _ctx)
  return kk_std_core__mlift17571_op(acc0, predicate0, y0, ys0, yy0, _y_174510, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17085_span_fun21450__t {
  struct kk_function_s _base;
  kk_function_t _b_19407_19401;
};
static kk_box_t kk_std_core__lift17085_span_fun21450(kk_function_t _fself, kk_box_t _b_19402, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17085_span_fun21450(kk_function_t _b_19407_19401, kk_context_t* _ctx) {
  struct kk_std_core__lift17085_span_fun21450__t* _self = kk_function_alloc_as(struct kk_std_core__lift17085_span_fun21450__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17085_span_fun21450, kk_context());
  _self->_b_19407_19401 = _b_19407_19401;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17085_span_fun21450(kk_function_t _fself, kk_box_t _b_19402, kk_context_t* _ctx) {
  struct kk_std_core__lift17085_span_fun21450__t* _self = kk_function_as(struct kk_std_core__lift17085_span_fun21450__t*, _fself);
  kk_function_t _b_19407_19401 = _self->_b_19407_19401; /* (bool) -> 16083 (list<16082>, list<16082>) */
  kk_drop_match(_self, {kk_function_dup(_b_19407_19401);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x21451;
  bool _x21452 = kk_bool_unbox(_b_19402); /*bool*/
  _x21451 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, bool, kk_context_t*), _b_19407_19401, (_b_19407_19401, _x21452, _ctx)); /*(list<16082>, list<16082>)*/
  return kk_std_core_types__tuple2__box(_x21451, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core__lift17085_span(kk_function_t predicate0, kk_std_core__list ys0, kk_std_core__list acc0, kk_context_t* _ctx) { /* forall<a,e> (predicate : (a) -> e bool, ys : list<a>, acc : list<a>) -> e (list<a>, list<a>) */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys0)) {
    struct kk_std_core_Cons* _con21445 = kk_std_core__as_Cons(ys0);
    kk_box_t y0 = _con21445->head;
    kk_std_core__list yy0 = _con21445->tail;
    kk_box_dup(y0);
    kk_std_core__list_dup(yy0);
    bool x_18009;
    kk_function_t _x21447 = kk_function_dup(predicate0); /*(16082) -> 16083 bool*/
    kk_box_t _x21446 = kk_box_dup(y0); /*16082*/
    x_18009 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21447, (_x21447, _x21446, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19407_19401 = kk_std_core__new_lift17085_span_fun21448(acc0, predicate0, y0, ys0, yy0, _ctx); /*(bool) -> 16083 (list<16082>, list<16082>)*/;
      kk_box_t _x21449 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17085_span_fun21450(_b_19407_19401, _ctx), _ctx); /*3858*/
      return kk_std_core_types__tuple2__unbox(_x21449, _ctx);
    }
    if (x_18009) {
      kk_std_core__list_dropn(ys0, ((int32_t)2), _ctx);
      { // tailcall
        kk_std_core__list _x21453 = kk_std_core__new_Cons(kk_reuse_null, y0, acc0, _ctx); /*list<61>*/
        ys0 = yy0;
        acc0 = _x21453;
        goto kk__tailcall;
      }
    }
    kk_function_drop(predicate0, _ctx);
    kk_box_drop(y0, _ctx);
    kk_std_core__list_drop(yy0, _ctx);
    kk_std_core__list _b_19408_19403 = kk_std_core__lift17078_reverse(kk_std_core__new_Nil(_ctx), acc0, _ctx); /*list<16082>*/;
    kk_std_core__list _b_19409_19404 = ys0; /*list<16082>*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19408_19403, _ctx), kk_std_core__list_box(_b_19409_19404, _ctx), _ctx);
  }
  kk_function_drop(predicate0, _ctx);
  kk_std_core__list _b_19410_19405 = kk_std_core__lift17078_reverse(kk_std_core__new_Nil(_ctx), acc0, _ctx); /*list<16082>*/;
  kk_std_core__list _b_19411_19406 = ys0; /*list<16082>*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19410_19405, _ctx), kk_std_core__list_box(_b_19411_19406, _ctx), _ctx);
}
 
// Return the sum of a list of integers


// lift anonymous function
struct kk_std_core_sum_fun21454__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_core_sum_fun21454(kk_function_t _fself, kk_integer_t x, kk_integer_t y, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_sum_fun21454(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_sum_fun21454, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_core_sum_fun21454(kk_function_t _fself, kk_integer_t x, kk_integer_t y, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_integer_add(x,y,kk_context());
}


// lift anonymous function
struct kk_std_core_sum_fun21456__t {
  struct kk_function_s _base;
  kk_function_t _b_19419_19414;
};
static kk_box_t kk_std_core_sum_fun21456(kk_function_t _fself, kk_box_t _b_19415, kk_box_t _b_19416, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_sum_fun21456(kk_function_t _b_19419_19414, kk_context_t* _ctx) {
  struct kk_std_core_sum_fun21456__t* _self = kk_function_alloc_as(struct kk_std_core_sum_fun21456__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_sum_fun21456, kk_context());
  _self->_b_19419_19414 = _b_19419_19414;
  return &_self->_base;
}

static kk_box_t kk_std_core_sum_fun21456(kk_function_t _fself, kk_box_t _b_19415, kk_box_t _b_19416, kk_context_t* _ctx) {
  struct kk_std_core_sum_fun21456__t* _self = kk_function_as(struct kk_std_core_sum_fun21456__t*, _fself);
  kk_function_t _b_19419_19414 = _self->_b_19419_19414; /* (x : int, y : int) -> int */
  kk_drop_match(_self, {kk_function_dup(_b_19419_19414);}, {}, _ctx)
  kk_integer_t _x21457;
  kk_integer_t _x21458 = kk_integer_unbox(_b_19415); /*int*/
  kk_integer_t _x21459 = kk_integer_unbox(_b_19416); /*int*/
  _x21457 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), _b_19419_19414, (_b_19419_19414, _x21458, _x21459, _ctx)); /*int*/
  return kk_integer_box(_x21457);
}

kk_integer_t kk_std_core_sum(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<int>) -> int */ 
  kk_std_core__list _b_19417_19412 = xs; /*list<int>*/;
  kk_integer_t _b_19418_19413 = kk_integer_from_small(0); /*int*/;
  kk_function_t _b_19419_19414 = kk_std_core_new_sum_fun21454(_ctx); /*(x : int, y : int) -> int*/;
  kk_box_t _x21455 = kk_std_core_foldl(_b_19417_19412, kk_integer_box(_b_19418_19413), kk_std_core_new_sum_fun21456(_b_19419_19414, _ctx), _ctx); /*12247*/
  return kk_integer_unbox(_x21455);
}
 
// Return the tail of list. Returns the empty list if `xs` is empty.

kk_std_core__list kk_std_core_tail_1(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> list<a> */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21460 = kk_std_core__as_Cons(xs);
    kk_box_t _pat0 = _con21460->head;
    kk_std_core__list xx = _con21460->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_box_drop(_pat0, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs);
    }
    else {
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    return xx;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Return the tail of a string (or the empty string)

kk_string_t kk_std_core_tail_2(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  kk_std_core__sslice _x21461;
  kk_std_core__sslice slice1 = kk_std_core_first1(s, _ctx); /*sslice*/;
  kk_std_core__sslice slice0_17051;
  bool _match_19662;
  kk_integer_t _borrowedArg_19667;
  kk_std_core_types__optional _match_19668 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_19668)) {
    kk_box_t _box_x19420 = _match_19668._cons.Optional.value;
    kk_integer_t _n_10065 = kk_integer_unbox(_box_x19420);
    kk_integer_dup(_n_10065);
    kk_std_core_types__optional_drop(_match_19668, _ctx);
    kk_Unit;
    _borrowedArg_19667 = _n_10065; /*int*/
    goto _match21462;
  }
  _borrowedArg_19667 = kk_integer_from_small(1); /*int*/
  _match21462: ;
  kk_integer_t _borrowedArg_19666 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19669 = kk_integer_eq_borrow(_borrowedArg_19667,_borrowedArg_19666,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19667, _ctx);
  kk_integer_drop(_borrowedArg_19666, _ctx);
  _match_19662 = _borrowApp_19669; /*bool*/
  if (_match_19662) {
    slice0_17051 = slice1; /*sslice*/
  }
  else {
    kk_integer_t _borrowedArg_19663;
    kk_integer_t _x21464;
    kk_std_core_types__optional _match_19664 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_19664)) {
      kk_box_t _box_x19421 = _match_19664._cons.Optional.value;
      kk_integer_t _n_100650 = kk_integer_unbox(_box_x19421);
      kk_integer_dup(_n_100650);
      kk_std_core_types__optional_drop(_match_19664, _ctx);
      kk_Unit;
      _x21464 = _n_100650; /*int*/
      goto _match21465;
    }
    _x21464 = kk_integer_from_small(1); /*int*/
    _match21465: ;
    _borrowedArg_19663 = kk_integer_sub(_x21464,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__sslice _borrowApp_19665 = kk_std_core_extend(slice1, _borrowedArg_19663, _ctx); /*sslice*/;
    kk_integer_drop(_borrowedArg_19663, _ctx);
    slice0_17051 = _borrowApp_19665; /*sslice*/
  }
  kk_string_t s0 = slice0_17051.str;
  kk_ssize_t start0 = slice0_17051.start;
  kk_ssize_t len0 = slice0_17051.len;
  kk_string_dup(s0);
  kk_std_core__sslice_drop(slice0_17051, _ctx);
  kk_Unit;
  kk_string_t _x21467 = kk_string_dup(s0); /*string*/
  kk_ssize_t _x21468 = (start0 + len0); /*ssize_t*/
  kk_ssize_t _x21469;
  kk_ssize_t _x21470 = kk_string_len(s0,kk_context()); /*ssize_t*/
  kk_ssize_t _x21471 = (start0 + len0); /*ssize_t*/
  _x21469 = (_x21470 - _x21471); /*ssize_t*/
  _x21461 = kk_std_core__new_Sslice(_x21467, _x21468, _x21469, _ctx); /*sslice*/
  return kk_std_core_string_3(_x21461, _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17572_op(kk_std_core_types__ctail _acc, kk_function_t predicate, kk_box_t x, kk_std_core__list xx, bool _y_17456, kk_context_t* _ctx) { /* forall<a,e> (ctail<list<a>>, predicate : (a) -> e bool, x : a, xx : list<a>, bool) -> e list<a> */ 
  if (_y_17456) {
    kk_std_core__list _ctail_17206 = kk_std_core__list_hole(); /*list<16193>*/;
    kk_std_core__list _ctail_17207 = kk_std_core__new_Cons(kk_reuse_null, x, _ctail_17206, _ctx); /*list<16193>*/;
    kk_std_core_types__ctail _x21472;
    kk_box_t* field_18016 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17207)->tail)); /*cfield<list<16193>>*/;
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x19428 = _acc._cons.CTail._field1;
      kk_box_t* last0 = _acc._cons.CTail._field2;
      kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19428, NULL);
      kk_std_core__list_dup(head0);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __ = kk_Unit;
      *(last0) = kk_std_core__list_box(_ctail_17207, _ctx);
      kk_std_core__list _b_19442_19433 = head0; /*list<16193>*/;
      kk_box_t* _b_19443_19434 = field_18016; /*cfield<list<16193>>*/;
      _x21472 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19442_19433, _ctx), _b_19443_19434, _ctx); /*ctail<58>*/
      goto _match21473;
    }
    kk_std_core__list _b_19444_19435 = _ctail_17207; /*list<16193>*/;
    kk_box_t* _b_19445_19436 = field_18016; /*cfield<list<16193>>*/;
    _x21472 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19444_19435, _ctx), _b_19445_19436, _ctx); /*ctail<58>*/
    _match21473: ;
    return kk_std_core__ctail_take_while(xx, predicate, _x21472, _ctx);
  }
  kk_function_drop(predicate, _ctx);
  kk_box_drop(x, _ctx);
  kk_std_core__list_drop(xx, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19437 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19437, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17573_op_fun21476__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t x0;
};
static kk_std_core__list kk_std_core__mlift17573_op_fun21476(kk_function_t _fself, kk_std_core__list _ctail_17209, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17573_op_fun21476(kk_function_t _accm, kk_box_t x0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17573_op_fun21476__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17573_op_fun21476__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17573_op_fun21476, kk_context());
  _self->_accm = _accm;
  _self->x0 = x0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17573_op_fun21476(kk_function_t _fself, kk_std_core__list _ctail_17209, kk_context_t* _ctx) {
  struct kk_std_core__mlift17573_op_fun21476__t* _self = kk_function_as(struct kk_std_core__mlift17573_op_fun21476__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<16193>) -> list<16193> */
  kk_box_t x0 = _self->x0; /* 16193 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(x0);}, {}, _ctx)
  kk_std_core__list _x21477 = kk_std_core__new_Cons(kk_reuse_null, x0, _ctail_17209, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x21477, _ctx));
}

kk_std_core__list kk_std_core__mlift17573_op(kk_function_t _accm, kk_function_t predicate0, kk_box_t x0, kk_std_core__list xx0, bool _y_17460, kk_context_t* _ctx) { /* forall<a,e> ((list<a>) -> list<a>, predicate : (a) -> e bool, x : a, xx : list<a>, bool) -> e list<a> */ 
  if (_y_17460) {
    return kk_std_core__ctailm_take_while(xx0, predicate0, kk_std_core__new_mlift17573_op_fun21476(_accm, x0, _ctx), _ctx);
  }
  kk_function_drop(predicate0, _ctx);
  kk_box_drop(x0, _ctx);
  kk_std_core__list_drop(xx0, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Keep only those initial elements that satisfy `predicate`


// lift anonymous function
struct kk_std_core__ctail_take_while_fun21481__t {
  struct kk_function_s _base;
  kk_function_t predicate1;
  kk_box_t x1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_take_while_fun21481(kk_function_t _fself, bool _y_174560, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_take_while_fun21481(kk_function_t predicate1, kk_box_t x1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_take_while_fun21481__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_take_while_fun21481__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_take_while_fun21481, kk_context());
  _self->predicate1 = predicate1;
  _self->x1 = x1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_take_while_fun21481(kk_function_t _fself, bool _y_174560, kk_context_t* _ctx) {
  struct kk_std_core__ctail_take_while_fun21481__t* _self = kk_function_as(struct kk_std_core__ctail_take_while_fun21481__t*, _fself);
  kk_function_t predicate1 = _self->predicate1; /* (16193) -> 16194 bool */
  kk_box_t x1 = _self->x1; /* 16193 */
  kk_std_core__list xx1 = _self->xx1; /* list<16193> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<16193>> */
  kk_drop_match(_self, {kk_function_dup(predicate1);kk_box_dup(x1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17572_op(_acc0, predicate1, x1, xx1, _y_174560, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_take_while_fun21483__t {
  struct kk_function_s _base;
  kk_function_t _b_19473_19446;
};
static kk_box_t kk_std_core__ctail_take_while_fun21483(kk_function_t _fself, kk_box_t _b_19447, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_take_while_fun21483(kk_function_t _b_19473_19446, kk_context_t* _ctx) {
  struct kk_std_core__ctail_take_while_fun21483__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_take_while_fun21483__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_take_while_fun21483, kk_context());
  _self->_b_19473_19446 = _b_19473_19446;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_take_while_fun21483(kk_function_t _fself, kk_box_t _b_19447, kk_context_t* _ctx) {
  struct kk_std_core__ctail_take_while_fun21483__t* _self = kk_function_as(struct kk_std_core__ctail_take_while_fun21483__t*, _fself);
  kk_function_t _b_19473_19446 = _self->_b_19473_19446; /* (bool) -> 16194 list<16193> */
  kk_drop_match(_self, {kk_function_dup(_b_19473_19446);}, {}, _ctx)
  kk_std_core__list _x21484;
  bool _x21485 = kk_bool_unbox(_b_19447); /*bool*/
  _x21484 = kk_function_call(kk_std_core__list, (kk_function_t, bool, kk_context_t*), _b_19473_19446, (_b_19473_19446, _x21485, _ctx)); /*list<16193>*/
  return kk_std_core__list_box(_x21484, _ctx);
}

kk_std_core__list kk_std_core__ctail_take_while(kk_std_core__list xs, kk_function_t predicate1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool, ctail<list<a>>) -> e list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21478 = kk_std_core__as_Cons(xs);
    kk_box_t x1 = _con21478->head;
    kk_std_core__list xx1 = _con21478->tail;
    kk_reuse_t _ru_20092 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20092 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    bool x2_18019;
    kk_function_t _x21480 = kk_function_dup(predicate1); /*(16193) -> 16194 bool*/
    kk_box_t _x21479 = kk_box_dup(x1); /*16193*/
    x2_18019 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21480, (_x21480, _x21479, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20092);
      kk_function_t _b_19473_19446 = kk_std_core__new_ctail_take_while_fun21481(predicate1, x1, xx1, _acc0, _ctx); /*(bool) -> 16194 list<16193>*/;
      kk_box_t _x21482 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_take_while_fun21483(_b_19473_19446, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21482, _ctx);
    }
    if (x2_18019) {
      kk_std_core__list _ctail_172060 = kk_std_core__list_hole(); /*list<16193>*/;
      kk_std_core__list _ctail_172070;
      if (kk_likely(_ru_20092!=NULL)) {
        struct kk_std_core_Cons* _con21486 = (struct kk_std_core_Cons*)_ru_20092;
        _con21486->tail = _ctail_172060;
        _ctail_172070 = kk_std_core__base_Cons(_con21486); /*list<16193>*/
      }
      else {
        _ctail_172070 = kk_std_core__new_Cons(kk_reuse_null, x1, _ctail_172060, _ctx); /*list<16193>*/
      }
      { // tailcall
        kk_std_core_types__ctail _x21487;
        kk_box_t* field0_18024 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_172070)->tail)); /*cfield<list<16193>>*/;
        if (kk_std_core_types__is_CTail(_acc0)) {
          kk_box_t _box_x19454 = _acc0._cons.CTail._field1;
          kk_box_t* last3 = _acc0._cons.CTail._field2;
          kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19454, NULL);
          kk_std_core__list_dup(head2);
          kk_std_core_types__ctail_drop(_acc0, _ctx);
          kk_Unit;
          kk_unit_t __1 = kk_Unit;
          *(last3) = kk_std_core__list_box(_ctail_172070, _ctx);
          kk_std_core__list _b_19474_19459 = head2; /*list<16193>*/;
          kk_box_t* _b_19475_19460 = field0_18024; /*cfield<list<16193>>*/;
          _x21487 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19474_19459, _ctx), _b_19475_19460, _ctx); /*ctail<58>*/
          goto _match21488;
        }
        kk_std_core__list _b_19476_19461 = _ctail_172070; /*list<16193>*/;
        kk_box_t* _b_19477_19462 = field0_18024; /*cfield<list<16193>>*/;
        _x21487 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19476_19461, _ctx), _b_19477_19462, _ctx); /*ctail<58>*/
        _match21488: ;
        xs = xx1;
        _acc0 = _x21487;
        goto kk__tailcall;
      }
    }
    kk_reuse_drop(_ru_20092);
    kk_function_drop(predicate1, _ctx);
    kk_box_drop(x1, _ctx);
    kk_std_core__list_drop(xx1, _ctx);
    if (kk_std_core_types__is_CTail(_acc0)) {
      kk_box_t _box_x19463 = _acc0._cons.CTail._field1;
      kk_box_t* last4 = _acc0._cons.CTail._field2;
      kk_std_core__list head3 = kk_std_core__list_unbox(_box_x19463, NULL);
      kk_std_core__list_dup(head3);
      kk_std_core_types__ctail_drop(_acc0, _ctx);
      kk_Unit;
      kk_unit_t __2 = kk_Unit;
      *(last4) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
      return head3;
    }
    return kk_std_core__new_Nil(_ctx);
  }
  kk_function_drop(predicate1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x19468 = _acc0._cons.CTail._field1;
    kk_box_t* last5 = _acc0._cons.CTail._field2;
    kk_std_core__list head4 = kk_std_core__list_unbox(_box_x19468, NULL);
    kk_std_core__list_dup(head4);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __3 = kk_Unit;
    *(last5) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head4;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Keep only those initial elements that satisfy `predicate`


// lift anonymous function
struct kk_std_core__ctailm_take_while_fun21495__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t predicate2;
  kk_box_t x3;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_take_while_fun21495(kk_function_t _fself, bool _y_174600, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_take_while_fun21495(kk_function_t _accm0, kk_function_t predicate2, kk_box_t x3, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21495__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_take_while_fun21495__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_take_while_fun21495, kk_context());
  _self->_accm0 = _accm0;
  _self->predicate2 = predicate2;
  _self->x3 = x3;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_take_while_fun21495(kk_function_t _fself, bool _y_174600, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21495__t* _self = kk_function_as(struct kk_std_core__ctailm_take_while_fun21495__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<16193>) -> list<16193> */
  kk_function_t predicate2 = _self->predicate2; /* (16193) -> 16194 bool */
  kk_box_t x3 = _self->x3; /* 16193 */
  kk_std_core__list xx2 = _self->xx2; /* list<16193> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(predicate2);kk_box_dup(x3);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17573_op(_accm0, predicate2, x3, xx2, _y_174600, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_take_while_fun21497__t {
  struct kk_function_s _base;
  kk_function_t _b_19480_19478;
};
static kk_box_t kk_std_core__ctailm_take_while_fun21497(kk_function_t _fself, kk_box_t _b_19479, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_take_while_fun21497(kk_function_t _b_19480_19478, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21497__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_take_while_fun21497__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_take_while_fun21497, kk_context());
  _self->_b_19480_19478 = _b_19480_19478;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_take_while_fun21497(kk_function_t _fself, kk_box_t _b_19479, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21497__t* _self = kk_function_as(struct kk_std_core__ctailm_take_while_fun21497__t*, _fself);
  kk_function_t _b_19480_19478 = _self->_b_19480_19478; /* (bool) -> 16194 list<16193> */
  kk_drop_match(_self, {kk_function_dup(_b_19480_19478);}, {}, _ctx)
  kk_std_core__list _x21498;
  bool _x21499 = kk_bool_unbox(_b_19479); /*bool*/
  _x21498 = kk_function_call(kk_std_core__list, (kk_function_t, bool, kk_context_t*), _b_19480_19478, (_b_19480_19478, _x21499, _ctx)); /*list<16193>*/
  return kk_std_core__list_box(_x21498, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_take_while_fun21501__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x3;
};
static kk_std_core__list kk_std_core__ctailm_take_while_fun21501(kk_function_t _fself, kk_std_core__list _ctail_172090, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_take_while_fun21501(kk_function_t _accm0, kk_box_t x3, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21501__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_take_while_fun21501__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_take_while_fun21501, kk_context());
  _self->_accm0 = _accm0;
  _self->x3 = x3;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_take_while_fun21501(kk_function_t _fself, kk_std_core__list _ctail_172090, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21501__t* _self = kk_function_as(struct kk_std_core__ctailm_take_while_fun21501__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<16193>) -> list<16193> */
  kk_box_t x3 = _self->x3; /* 16193 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x3);}, {}, _ctx)
  kk_std_core__list _x21502 = kk_std_core__new_Cons(kk_reuse_null, x3, _ctail_172090, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21502, _ctx));
}

kk_std_core__list kk_std_core__ctailm_take_while(kk_std_core__list xs0, kk_function_t predicate2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool, (list<a>) -> list<a>) -> e list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21492 = kk_std_core__as_Cons(xs0);
    kk_box_t x3 = _con21492->head;
    kk_std_core__list xx2 = _con21492->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x3);
      kk_std_core__list_dup(xx2);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    bool x4_18029;
    kk_function_t _x21494 = kk_function_dup(predicate2); /*(16193) -> 16194 bool*/
    kk_box_t _x21493 = kk_box_dup(x3); /*16193*/
    x4_18029 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21494, (_x21494, _x21493, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19480_19478 = kk_std_core__new_ctailm_take_while_fun21495(_accm0, predicate2, x3, xx2, _ctx); /*(bool) -> 16194 list<16193>*/;
      kk_box_t _x21496 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_take_while_fun21497(_b_19480_19478, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21496, _ctx);
    }
    if (x4_18029) {
      { // tailcall
        kk_function_t _x21500 = kk_std_core__new_ctailm_take_while_fun21501(_accm0, x3, _ctx); /*(list<16193>) -> list<16193>*/
        xs0 = xx2;
        _accm0 = _x21500;
        goto kk__tailcall;
      }
    }
    kk_function_drop(predicate2, _ctx);
    kk_box_drop(x3, _ctx);
    kk_std_core__list_drop(xx2, _ctx);
    return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
  }
  kk_function_drop(predicate2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Keep only those initial elements that satisfy `predicate`


// lift anonymous function
struct kk_std_core_take_while_fun21503__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_take_while_fun21503(kk_function_t _fself, kk_std_core__list _ctail_17208, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_take_while_fun21503(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_take_while_fun21503, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_take_while_fun21503(kk_function_t _fself, kk_std_core__list _ctail_17208, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17208;
}

kk_std_core__list kk_std_core_take_while(kk_std_core__list xs1, kk_function_t predicate3, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e list<a> */ 
  bool _match_19659 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19659) {
    return kk_std_core__ctail_take_while(xs1, predicate3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_take_while(xs1, predicate3, kk_std_core_new_take_while_fun21503(_ctx), _ctx);
}
 
// Trace a message used for debug purposes.
// The behaviour is system dependent. On a browser and node it uses
// `console.log`  by default.
// Disabled if `notrace` is called.


// lift anonymous function
struct kk_std_core_trace_fun21506__t {
  struct kk_function_s _base;
  kk_string_t message0;
};
static kk_unit_t kk_std_core_trace_fun21506(kk_function_t _fself, bool _y_17467, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_trace_fun21506(kk_string_t message0, kk_context_t* _ctx) {
  struct kk_std_core_trace_fun21506__t* _self = kk_function_alloc_as(struct kk_std_core_trace_fun21506__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_trace_fun21506, kk_context());
  _self->message0 = message0;
  return &_self->_base;
}

static kk_unit_t kk_std_core_trace_fun21506(kk_function_t _fself, bool _y_17467, kk_context_t* _ctx) {
  struct kk_std_core_trace_fun21506__t* _self = kk_function_as(struct kk_std_core_trace_fun21506__t*, _fself);
  kk_string_t message0 = _self->message0; /* string */
  kk_drop_match(_self, {kk_string_dup(message0);}, {}, _ctx)
  if (_y_17467) {
    return kk_std_core_xtrace(message0, _ctx);
  }
  kk_string_drop(message0, _ctx);
  return kk_Unit;
}


// lift anonymous function
struct kk_std_core_trace_fun21508__t {
  struct kk_function_s _base;
  kk_function_t _b_19485_19483;
};
static kk_box_t kk_std_core_trace_fun21508(kk_function_t _fself, kk_box_t _b_19484, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_trace_fun21508(kk_function_t _b_19485_19483, kk_context_t* _ctx) {
  struct kk_std_core_trace_fun21508__t* _self = kk_function_alloc_as(struct kk_std_core_trace_fun21508__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_trace_fun21508, kk_context());
  _self->_b_19485_19483 = _b_19485_19483;
  return &_self->_base;
}

static kk_box_t kk_std_core_trace_fun21508(kk_function_t _fself, kk_box_t _b_19484, kk_context_t* _ctx) {
  struct kk_std_core_trace_fun21508__t* _self = kk_function_as(struct kk_std_core_trace_fun21508__t*, _fself);
  kk_function_t _b_19485_19483 = _self->_b_19485_19483; /* (bool) -> <read<global>|_16227> () */
  kk_drop_match(_self, {kk_function_dup(_b_19485_19483);}, {}, _ctx)
  kk_unit_t _x21509 = kk_Unit;
  bool _x21510 = kk_bool_unbox(_b_19484); /*bool*/
  kk_function_call(kk_unit_t, (kk_function_t, bool, kk_context_t*), _b_19485_19483, (_b_19485_19483, _x21510, _ctx));
  return kk_unit_box(_x21509);
}

kk_unit_t kk_std_core_trace(kk_string_t message0, kk_context_t* _ctx) { /* (message : string) -> () */ 
  bool x_18032;
  kk_box_t _x21504;
  kk_ref_t _x21505 = kk_ref_dup(kk_std_core_trace_enabled); /*ref<global,bool>*/
  _x21504 = kk_ref_get(_x21505,kk_context()); /*182*/
  x_18032 = kk_bool_unbox(_x21504); /*bool*/
  kk_function_t next0_18033 = kk_std_core_new_trace_fun21506(message0, _ctx); /*(bool) -> <read<global>|_16227> ()*/;
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19485_19483 = next0_18033; /*(bool) -> <read<global>|_16227> ()*/;
    kk_box_t _x21507 = kk_std_core_hnd_yield_extend(kk_std_core_new_trace_fun21508(_b_19485_19483, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21507); return kk_Unit;
  }
  kk_function_call(kk_unit_t, (kk_function_t, bool, kk_context_t*), next0_18033, (next0_18033, x_18032, _ctx)); return kk_Unit;
}


// lift anonymous function
struct kk_std_core_trace_any_fun21513__t {
  struct kk_function_s _base;
  kk_string_t message0;
  kk_box_t x;
};
static kk_unit_t kk_std_core_trace_any_fun21513(kk_function_t _fself, bool _y_17469, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_trace_any_fun21513(kk_string_t message0, kk_box_t x, kk_context_t* _ctx) {
  struct kk_std_core_trace_any_fun21513__t* _self = kk_function_alloc_as(struct kk_std_core_trace_any_fun21513__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_trace_any_fun21513, kk_context());
  _self->message0 = message0;
  _self->x = x;
  return &_self->_base;
}

static kk_unit_t kk_std_core_trace_any_fun21513(kk_function_t _fself, bool _y_17469, kk_context_t* _ctx) {
  struct kk_std_core_trace_any_fun21513__t* _self = kk_function_as(struct kk_std_core_trace_any_fun21513__t*, _fself);
  kk_string_t message0 = _self->message0; /* string */
  kk_box_t x = _self->x; /* 16302 */
  kk_drop_match(_self, {kk_string_dup(message0);kk_box_dup(x);}, {}, _ctx)
  if (_y_17469) {
    return kk_std_core_xtrace_any(message0, x, _ctx);
  }
  kk_string_drop(message0, _ctx);
  kk_box_drop(x, _ctx);
  return kk_Unit;
}


// lift anonymous function
struct kk_std_core_trace_any_fun21515__t {
  struct kk_function_s _base;
  kk_function_t _b_19490_19488;
};
static kk_box_t kk_std_core_trace_any_fun21515(kk_function_t _fself, kk_box_t _b_19489, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_trace_any_fun21515(kk_function_t _b_19490_19488, kk_context_t* _ctx) {
  struct kk_std_core_trace_any_fun21515__t* _self = kk_function_alloc_as(struct kk_std_core_trace_any_fun21515__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_trace_any_fun21515, kk_context());
  _self->_b_19490_19488 = _b_19490_19488;
  return &_self->_base;
}

static kk_box_t kk_std_core_trace_any_fun21515(kk_function_t _fself, kk_box_t _b_19489, kk_context_t* _ctx) {
  struct kk_std_core_trace_any_fun21515__t* _self = kk_function_as(struct kk_std_core_trace_any_fun21515__t*, _fself);
  kk_function_t _b_19490_19488 = _self->_b_19490_19488; /* (bool) -> <read<global>|_16279> () */
  kk_drop_match(_self, {kk_function_dup(_b_19490_19488);}, {}, _ctx)
  kk_unit_t _x21516 = kk_Unit;
  bool _x21517 = kk_bool_unbox(_b_19489); /*bool*/
  kk_function_call(kk_unit_t, (kk_function_t, bool, kk_context_t*), _b_19490_19488, (_b_19490_19488, _x21517, _ctx));
  return kk_unit_box(_x21516);
}

kk_unit_t kk_std_core_trace_any(kk_string_t message0, kk_box_t x, kk_context_t* _ctx) { /* forall<a> (message : string, x : a) -> () */ 
  bool x0_18036;
  kk_box_t _x21511;
  kk_ref_t _x21512 = kk_ref_dup(kk_std_core_trace_enabled); /*ref<global,bool>*/
  _x21511 = kk_ref_get(_x21512,kk_context()); /*182*/
  x0_18036 = kk_bool_unbox(_x21511); /*bool*/
  kk_function_t next0_18037 = kk_std_core_new_trace_any_fun21513(message0, x, _ctx); /*(bool) -> <read<global>|_16279> ()*/;
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19490_19488 = next0_18037; /*(bool) -> <read<global>|_16279> ()*/;
    kk_box_t _x21514 = kk_std_core_hnd_yield_extend(kk_std_core_new_trace_any_fun21515(_b_19490_19488, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21514); return kk_Unit;
  }
  kk_function_call(kk_unit_t, (kk_function_t, bool, kk_context_t*), next0_18037, (next0_18037, x0_18036, _ctx)); return kk_Unit;
}
 
// Truncate a string to `count` characters.

kk_string_t kk_std_core_truncate(kk_string_t s, kk_integer_t count, kk_context_t* _ctx) { /* (s : string, count : int) -> string */ 
  kk_std_core__sslice _x21518;
  kk_std_core__sslice _ownedArg_19647;
  kk_std_core__sslice slice0 = kk_std_core_first1(s, _ctx); /*sslice*/;
  bool _match_19648;
  kk_integer_t _borrowedArg_19653;
  kk_std_core_types__optional _match_19654 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_19654)) {
    kk_box_t _box_x19491 = _match_19654._cons.Optional.value;
    kk_integer_t _n_10065 = kk_integer_unbox(_box_x19491);
    kk_integer_dup(_n_10065);
    kk_std_core_types__optional_drop(_match_19654, _ctx);
    kk_Unit;
    _borrowedArg_19653 = _n_10065; /*int*/
    goto _match21519;
  }
  _borrowedArg_19653 = kk_integer_from_small(1); /*int*/
  _match21519: ;
  kk_integer_t _borrowedArg_19652 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19655 = kk_integer_eq_borrow(_borrowedArg_19653,_borrowedArg_19652,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19653, _ctx);
  kk_integer_drop(_borrowedArg_19652, _ctx);
  _match_19648 = _borrowApp_19655; /*bool*/
  if (_match_19648) {
    _ownedArg_19647 = slice0; /*sslice*/
  }
  else {
    kk_integer_t _borrowedArg_19649;
    kk_integer_t _x21521;
    kk_std_core_types__optional _match_19650 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_19650)) {
      kk_box_t _box_x19492 = _match_19650._cons.Optional.value;
      kk_integer_t _n_100650 = kk_integer_unbox(_box_x19492);
      kk_integer_dup(_n_100650);
      kk_std_core_types__optional_drop(_match_19650, _ctx);
      kk_Unit;
      _x21521 = _n_100650; /*int*/
      goto _match21522;
    }
    _x21521 = kk_integer_from_small(1); /*int*/
    _match21522: ;
    _borrowedArg_19649 = kk_integer_sub(_x21521,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__sslice _borrowApp_19651 = kk_std_core_extend(slice0, _borrowedArg_19649, _ctx); /*sslice*/;
    kk_integer_drop(_borrowedArg_19649, _ctx);
    _ownedArg_19647 = _borrowApp_19651; /*sslice*/
  }
  kk_integer_t _borrowedArg_19646 = kk_integer_sub(count,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__sslice _borrowApp_19656 = kk_std_core_extend(_ownedArg_19647, _borrowedArg_19646, _ctx); /*sslice*/;
  kk_integer_drop(_borrowedArg_19646, _ctx);
  _x21518 = _borrowApp_19656; /*sslice*/
  return kk_std_core_string_3(_x21518, _ctx);
}
 
// Return a default value when an exception is raised


// lift anonymous function
struct kk_std_core_try_default_fun21526__t {
  struct kk_function_s _base;
  kk_box_t value;
};
static kk_box_t kk_std_core_try_default_fun21526(kk_function_t _fself, kk_std_core__exception ___wildcard__2564__19, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_default_fun21526(kk_box_t value, kk_context_t* _ctx) {
  struct kk_std_core_try_default_fun21526__t* _self = kk_function_alloc_as(struct kk_std_core_try_default_fun21526__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_default_fun21526, kk_context());
  _self->value = value;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_default_fun21526(kk_function_t _fself, kk_std_core__exception ___wildcard__2564__19, kk_context_t* _ctx) {
  struct kk_std_core_try_default_fun21526__t* _self = kk_function_as(struct kk_std_core_try_default_fun21526__t*, _fself);
  kk_box_t value = _self->value; /* 16378 */
  kk_drop_match(_self, {kk_box_dup(value);}, {}, _ctx)
  kk_std_core__exception_drop(___wildcard__2564__19, _ctx);
  return value;
}


// lift anonymous function
struct kk_std_core_try_default_fun21527__t {
  struct kk_function_s _base;
  kk_function_t _b_19495_19493;
};
static kk_box_t kk_std_core_try_default_fun21527(kk_function_t _fself, kk_box_t _b_19494, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_default_fun21527(kk_function_t _b_19495_19493, kk_context_t* _ctx) {
  struct kk_std_core_try_default_fun21527__t* _self = kk_function_alloc_as(struct kk_std_core_try_default_fun21527__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_default_fun21527, kk_context());
  _self->_b_19495_19493 = _b_19495_19493;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_default_fun21527(kk_function_t _fself, kk_box_t _b_19494, kk_context_t* _ctx) {
  struct kk_std_core_try_default_fun21527__t* _self = kk_function_as(struct kk_std_core_try_default_fun21527__t*, _fself);
  kk_function_t _b_19495_19493 = _self->_b_19495_19493; /* (exception) -> 16379 16378 */
  kk_drop_match(_self, {kk_function_dup(_b_19495_19493);}, {}, _ctx)
  kk_std_core__exception _x21528 = kk_std_core__exception_unbox(_b_19494, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core__exception, kk_context_t*), _b_19495_19493, (_b_19495_19493, _x21528, _ctx));
}


// lift anonymous function
struct kk_std_core_try_default_fun21529__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_try_default_fun21529(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_default_fun21529(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_try_default_fun21529, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_try_default_fun21529(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _x;
}

kk_box_t kk_std_core_try_default(kk_box_t value, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (value : a, action : () -> <exn|e> a) -> e a */ 
  kk_std_core__hnd_exn _x21524;
  kk_std_core_hnd__clause1 _x21525;
  kk_function_t _b_19495_19493 = kk_std_core_new_try_default_fun21526(value, _ctx); /*(exception) -> 16379 16378*/;
  _x21525 = kk_std_core_hnd_clause_never1(kk_std_core_new_try_default_fun21527(_b_19495_19493, _ctx), _ctx); /*std/core/hnd/clause1<8108,8109,8111,8110,8112>*/
  _x21524 = kk_std_core__new_Hnd_exn(kk_reuse_null, _x21525, _ctx); /*.hnd-exn<11,12>*/
  return kk_std_core__handle_exn(((int32_t)0), _x21524, kk_std_core_new_try_default_fun21529(_ctx), action, _ctx);
}
 
// Returns a unique integer (modulo 32-bits).


// lift anonymous function
struct kk_std_core_unique_fun21533__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_core_unique_fun21533(kk_function_t _fself, kk_integer_t u, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_unique_fun21533(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_unique_fun21533, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_core_unique_fun21533(kk_function_t _fself, kk_integer_t u, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_integer_t _b_19505_19503;
  kk_integer_t _x21534 = kk_integer_dup(u); /*int*/
  _b_19505_19503 = kk_integer_add(_x21534,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_unit_t __ = kk_Unit;
  kk_ref_set_borrow(kk_std_core_unique_count,(kk_integer_box(_b_19505_19503)),kk_context());
  return u;
}


// lift anonymous function
struct kk_std_core_unique_fun21536__t {
  struct kk_function_s _base;
  kk_function_t _b_19508_19506;
};
static kk_box_t kk_std_core_unique_fun21536(kk_function_t _fself, kk_box_t _b_19507, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_unique_fun21536(kk_function_t _b_19508_19506, kk_context_t* _ctx) {
  struct kk_std_core_unique_fun21536__t* _self = kk_function_alloc_as(struct kk_std_core_unique_fun21536__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_unique_fun21536, kk_context());
  _self->_b_19508_19506 = _b_19508_19506;
  return &_self->_base;
}

static kk_box_t kk_std_core_unique_fun21536(kk_function_t _fself, kk_box_t _b_19507, kk_context_t* _ctx) {
  struct kk_std_core_unique_fun21536__t* _self = kk_function_as(struct kk_std_core_unique_fun21536__t*, _fself);
  kk_function_t _b_19508_19506 = _self->_b_19508_19506; /* (int) -> <read<global>,write<global>|_16439> int */
  kk_drop_match(_self, {kk_function_dup(_b_19508_19506);}, {}, _ctx)
  kk_integer_t _x21537;
  kk_integer_t _x21538 = kk_integer_unbox(_b_19507); /*int*/
  _x21537 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), _b_19508_19506, (_b_19508_19506, _x21538, _ctx)); /*int*/
  return kk_integer_box(_x21537);
}

kk_integer_t kk_std_core_unique(kk_context_t* _ctx) { /* () -> ndet int */ 
  kk_integer_t x_18041;
  kk_box_t _x21531;
  kk_ref_t _x21532 = kk_ref_dup(kk_std_core_unique_count); /*ref<global,int>*/
  _x21531 = kk_ref_get(_x21532,kk_context()); /*182*/
  x_18041 = kk_integer_unbox(_x21531); /*int*/
  kk_function_t next0_18042 = kk_std_core_new_unique_fun21533(_ctx); /*(int) -> <read<global>,write<global>|_16439> int*/;
  if (kk_yielding(kk_context())) {
    kk_integer_drop(x_18041, _ctx);
    kk_function_t _b_19508_19506 = next0_18042; /*(int) -> <read<global>,write<global>|_16439> int*/;
    kk_box_t _x21535 = kk_std_core_hnd_yield_extend(kk_std_core_new_unique_fun21536(_b_19508_19506, _ctx), _ctx); /*3858*/
    return kk_integer_unbox(_x21535);
  }
  return kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), next0_18042, (next0_18042, x_18041, _ctx));
}
 
// Get the value of the `Just` constructor or raise an exception

kk_box_t kk_std_core_unjust(kk_std_core_types__maybe m, kk_context_t* _ctx) { /* forall<a> (m : maybe<a>) -> exn a */ 
  if (kk_std_core_types__is_Just(m)) {
    kk_box_t x = m._cons.Just.value;
    return x;
  }
  kk_std_core_hnd__ev ev_18044;
  kk_ssize_t _x21539 = ((kk_ssize_t)0); /*ssize_t*/
  ev_18044 = kk_evv_at(_x21539,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
  struct kk_std_core_hnd_Ev* _con21540 = kk_std_core_hnd__as_Ev(ev_18044);
  kk_std_core_hnd__marker m0 = _con21540->marker;
  kk_box_t _box_x19509 = _con21540->hnd;
  kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x19509, NULL);
  kk_std_core__hnd_exn_dup(h);
  kk_std_core_hnd__clause1 _match_19642;
  kk_std_core_hnd__clause1 _borrowApp_19644 = kk_std_core__select_throw_exn(h, _ctx); /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/;
  kk_std_core__hnd_exn_drop(h, _ctx);
  _match_19642 = _borrowApp_19644; /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/
  kk_function_t _fun_unbox_x19513 = _match_19642.clause;
  kk_std_core_hnd__marker _b_19514 = m0; /*std/core/hnd/marker<3573,3574>*/;
  kk_std_core_hnd__ev _b_19515 = ev_18044; /*std/core/hnd/ev<.hnd-exn>*/;
  kk_std_core__exception _b_19516;
  kk_string_t _x21542;
  kk_define_string_literal(, _s21543, 37, "unexpected Nothing in std/core/unjust")
  _x21542 = kk_string_dup(_s21543); /*string*/
  kk_std_core__exception_info _x21544;
  kk_std_core_types__optional _match_19643 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_19643)) {
    kk_box_t _box_x19517 = _match_19643._cons.Optional.value;
    kk_std_core__exception_info _info_11639 = kk_std_core__exception_info_unbox(_box_x19517, NULL);
    kk_std_core__exception_info_dup(_info_11639);
    kk_std_core_types__optional_drop(_match_19643, _ctx);
    kk_Unit;
    _x21544 = _info_11639; /*exception-info*/
    goto _match21545;
  }
  _x21544 = kk_std_core__new_ExnError(_ctx); /*exception-info*/
  _match21545: ;
  _b_19516 = kk_std_core__new_Exception(_x21542, _x21544, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x19513, (_fun_unbox_x19513, _b_19514, _b_19515, kk_std_core__exception_box(_b_19516, _ctx), _ctx));
}
 
// Join a list of strings with newlines

kk_string_t kk_std_core_unlines(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<string>) -> string */ 
  if (kk_std_core__is_Nil(xs)) {
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con21548 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x19518 = _con21548->head;
  kk_std_core__list xx = _con21548->tail;
  kk_string_t x = kk_string_unbox(_box_x19518);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_string_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_string_t _x21550;
  kk_define_string_literal(, _s21551, 1, "\n")
  _x21550 = kk_string_dup(_s21551); /*string*/
  return kk_std_core__lift17071_joinsep(_x21550, xx, x, _ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17087_unzip(kk_std_core__list acc, kk_std_core__list ys0, kk_context_t* _ctx) { /* forall<a> (acc : list<a>, ys0 : list<a>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys0)) {
    struct kk_std_core_Cons* _con21552 = kk_std_core__as_Cons(ys0);
    kk_box_t x0 = _con21552->head;
    kk_std_core__list xx0 = _con21552->tail;
    kk_reuse_t _ru_20096 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys0))) {
      _ru_20096 = (kk_std_core__list_reuse(ys0));
    }
    else {
      kk_box_dup(x0);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(ys0, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x21553;
      if (kk_likely(_ru_20096!=NULL)) {
        struct kk_std_core_Cons* _con21554 = (struct kk_std_core_Cons*)_ru_20096;
        _con21554->tail = acc;
        _x21553 = kk_std_core__base_Cons(_con21554); /*list<61>*/
      }
      else {
        _x21553 = kk_std_core__new_Cons(kk_reuse_null, x0, acc, _ctx); /*list<61>*/
      }
      acc = _x21553;
      ys0 = xx0;
      goto kk__tailcall;
    }
  }
  return acc;
}
 
// lifted

kk_std_core__list kk_std_core__lift17088_unzip(kk_std_core__list acc0, kk_std_core__list ys1, kk_context_t* _ctx) { /* forall<a> (acc0 : list<a>, ys1 : list<a>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys1)) {
    struct kk_std_core_Cons* _con21555 = kk_std_core__as_Cons(ys1);
    kk_box_t x1 = _con21555->head;
    kk_std_core__list xx1 = _con21555->tail;
    kk_reuse_t _ru_20097 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys1))) {
      _ru_20097 = (kk_std_core__list_reuse(ys1));
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(ys1, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x21556;
      if (kk_likely(_ru_20097!=NULL)) {
        struct kk_std_core_Cons* _con21557 = (struct kk_std_core_Cons*)_ru_20097;
        _con21557->tail = acc0;
        _x21556 = kk_std_core__base_Cons(_con21557); /*list<61>*/
      }
      else {
        _x21556 = kk_std_core__new_Cons(kk_reuse_null, x1, acc0, _ctx); /*list<61>*/
      }
      acc0 = _x21556;
      ys1 = xx1;
      goto kk__tailcall;
    }
  }
  return acc0;
}
 
// lifted
// todo: implement TRMC for multiple results

kk_std_core_types__tuple2_ kk_std_core__lift17086_unzip(kk_std_core__list ys, kk_std_core__list acc1, kk_std_core__list acc2, kk_context_t* _ctx) { /* forall<a,b> (ys : list<(a, b)>, acc1 : list<a>, acc2 : list<b>) -> (list<a>, list<b>) */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21558 = kk_std_core__as_Cons(ys);
    kk_box_t _box_x19519 = _con21558->head;
    kk_std_core__list xx = _con21558->tail;
    kk_std_core_types__tuple2_ _pat0 = kk_std_core_types__tuple2__unbox(_box_x19519, NULL);
    kk_box_t x = _pat0.fst;
    kk_box_t y = _pat0.snd;
    kk_reuse_t _ru_20098 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_box_dup(x);
      kk_box_dup(y);
      kk_box_drop(_box_x19519, _ctx);
      kk_Unit;
      kk_Unit;
      _ru_20098 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_box_dup(y);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x21560 = kk_std_core__new_Cons(_ru_20098, x, acc1, _ctx); /*list<61>*/
      kk_std_core__list _x21561 = kk_std_core__new_Cons(kk_reuse_null, y, acc2, _ctx); /*list<61>*/
      ys = xx;
      acc1 = _x21560;
      acc2 = _x21561;
      goto kk__tailcall;
    }
  }
  kk_std_core__list _b_19522_19520 = kk_std_core__lift17087_unzip(kk_std_core__new_Nil(_ctx), acc1, _ctx); /*list<16507>*/;
  kk_std_core__list _b_19523_19521 = kk_std_core__lift17088_unzip(kk_std_core__new_Nil(_ctx), acc2, _ctx); /*list<16508>*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19522_19520, _ctx), kk_std_core__list_box(_b_19523_19521, _ctx), _ctx);
}
 
// Convert a string to a vector of characters.

kk_vector_t kk_std_core_vector_1(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> vector<char> */ 
  return kk_string_to_chars(s,kk_context());
}
extern kk_box_t kk_std_core_vector_fun21564_2(kk_function_t _fself, kk_ssize_t ___wildcard__2076__30, kk_context_t* _ctx) {
  struct kk_std_core_vector_fun21564__t_2* _self = kk_function_as(struct kk_std_core_vector_fun21564__t_2*, _fself);
  kk_box_t default0 = _self->default0; /* 16584 */
  kk_drop_match(_self, {kk_box_dup(default0);}, {}, _ctx)
  return default0;
}
extern kk_box_t kk_std_core_vector_init_fun21567(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx) {
  struct kk_std_core_vector_init_fun21567__t* _self = kk_function_as(struct kk_std_core_vector_init_fun21567__t*, _fself);
  kk_function_t f = _self->f; /* (int) -> 16651 */
  kk_drop_match(_self, {kk_function_dup(f);}, {}, _ctx)
  kk_integer_t _x21568 = kk_integer_from_ssize_t(i,kk_context()); /*int*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_context_t*), f, (f, _x21568, _ctx));
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17577_while(kk_function_t action, kk_function_t predicate, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action : () -> <div|e> (), predicate : () -> <div|e> bool, wild_ : ()) -> <div|e> () */ 
  kk_std_core_while(predicate, action, _ctx); return kk_Unit;
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17578_while_fun21570__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_function_t predicate0;
};
static kk_unit_t kk_std_core__mlift17578_while_fun21570(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17578_while_fun21570(kk_function_t action0, kk_function_t predicate0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17578_while_fun21570__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17578_while_fun21570__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17578_while_fun21570, kk_context());
  _self->action0 = action0;
  _self->predicate0 = predicate0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__mlift17578_while_fun21570(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17578_while_fun21570__t* _self = kk_function_as(struct kk_std_core__mlift17578_while_fun21570__t*, _fself);
  kk_function_t action0 = _self->action0; /* () -> <div|16672> () */
  kk_function_t predicate0 = _self->predicate0; /* () -> <div|16672> bool */
  kk_drop_match(_self, {kk_function_dup(action0);kk_function_dup(predicate0);}, {}, _ctx)
  return kk_std_core__mlift17577_while(action0, predicate0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17578_while_fun21572__t {
  struct kk_function_s _base;
  kk_function_t _b_19526_19524;
};
static kk_box_t kk_std_core__mlift17578_while_fun21572(kk_function_t _fself, kk_box_t _b_19525, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17578_while_fun21572(kk_function_t _b_19526_19524, kk_context_t* _ctx) {
  struct kk_std_core__mlift17578_while_fun21572__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17578_while_fun21572__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17578_while_fun21572, kk_context());
  _self->_b_19526_19524 = _b_19526_19524;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17578_while_fun21572(kk_function_t _fself, kk_box_t _b_19525, kk_context_t* _ctx) {
  struct kk_std_core__mlift17578_while_fun21572__t* _self = kk_function_as(struct kk_std_core__mlift17578_while_fun21572__t*, _fself);
  kk_function_t _b_19526_19524 = _self->_b_19526_19524; /* (wild_0 : ()) -> <div|16672> () */
  kk_drop_match(_self, {kk_function_dup(_b_19526_19524);}, {}, _ctx)
  kk_unit_t _x21573 = kk_Unit;
  kk_unit_t _x21574 = kk_Unit;
  kk_unit_unbox(_b_19525);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19526_19524, (_b_19526_19524, _x21574, _ctx));
  return kk_unit_box(_x21573);
}

kk_unit_t kk_std_core__mlift17578_while(kk_function_t action0, kk_function_t predicate0, bool _y_17475, kk_context_t* _ctx) { /* forall<e> (action : () -> <div|e> (), predicate : () -> <div|e> bool, bool) -> <div|e> () */ 
  if (_y_17475) {
    kk_unit_t x_18049 = kk_Unit;
    kk_function_t _x21569 = kk_function_dup(action0); /*() -> <div|16672> ()*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_context_t*), _x21569, (_x21569, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19526_19524 = kk_std_core__new_mlift17578_while_fun21570(action0, predicate0, _ctx); /*(wild_0 : ()) -> <div|16672> ()*/;
      kk_box_t _x21571 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17578_while_fun21572(_b_19526_19524, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x21571); return kk_Unit;
    }
    kk_std_core__mlift17577_while(action0, predicate0, x_18049, _ctx); return kk_Unit;
  }
  kk_function_drop(action0, _ctx);
  kk_function_drop(predicate0, _ctx);
  kk_Unit; return kk_Unit;
}
 
// The `while` fun executes `action`  as long as `pred`  is `true`.


// lift anonymous function
struct kk_std_core_while_fun21576__t {
  struct kk_function_s _base;
  kk_function_t action1;
  kk_function_t predicate1;
};
static kk_unit_t kk_std_core_while_fun21576(kk_function_t _fself, bool _y_174750, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_while_fun21576(kk_function_t action1, kk_function_t predicate1, kk_context_t* _ctx) {
  struct kk_std_core_while_fun21576__t* _self = kk_function_alloc_as(struct kk_std_core_while_fun21576__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_while_fun21576, kk_context());
  _self->action1 = action1;
  _self->predicate1 = predicate1;
  return &_self->_base;
}

static kk_unit_t kk_std_core_while_fun21576(kk_function_t _fself, bool _y_174750, kk_context_t* _ctx) {
  struct kk_std_core_while_fun21576__t* _self = kk_function_as(struct kk_std_core_while_fun21576__t*, _fself);
  kk_function_t action1 = _self->action1; /* () -> <div|16672> () */
  kk_function_t predicate1 = _self->predicate1; /* () -> <div|16672> bool */
  kk_drop_match(_self, {kk_function_dup(action1);kk_function_dup(predicate1);}, {}, _ctx)
  return kk_std_core__mlift17578_while(action1, predicate1, _y_174750, _ctx);
}


// lift anonymous function
struct kk_std_core_while_fun21578__t {
  struct kk_function_s _base;
  kk_function_t _b_19531_19527;
};
static kk_box_t kk_std_core_while_fun21578(kk_function_t _fself, kk_box_t _b_19528, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_while_fun21578(kk_function_t _b_19531_19527, kk_context_t* _ctx) {
  struct kk_std_core_while_fun21578__t* _self = kk_function_alloc_as(struct kk_std_core_while_fun21578__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_while_fun21578, kk_context());
  _self->_b_19531_19527 = _b_19531_19527;
  return &_self->_base;
}

static kk_box_t kk_std_core_while_fun21578(kk_function_t _fself, kk_box_t _b_19528, kk_context_t* _ctx) {
  struct kk_std_core_while_fun21578__t* _self = kk_function_as(struct kk_std_core_while_fun21578__t*, _fself);
  kk_function_t _b_19531_19527 = _self->_b_19531_19527; /* (bool) -> <div|16672> () */
  kk_drop_match(_self, {kk_function_dup(_b_19531_19527);}, {}, _ctx)
  kk_unit_t _x21579 = kk_Unit;
  bool _x21580 = kk_bool_unbox(_b_19528); /*bool*/
  kk_function_call(kk_unit_t, (kk_function_t, bool, kk_context_t*), _b_19531_19527, (_b_19531_19527, _x21580, _ctx));
  return kk_unit_box(_x21579);
}


// lift anonymous function
struct kk_std_core_while_fun21582__t {
  struct kk_function_s _base;
  kk_function_t action1;
  kk_function_t predicate1;
};
static kk_unit_t kk_std_core_while_fun21582(kk_function_t _fself, kk_unit_t wild__1, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_while_fun21582(kk_function_t action1, kk_function_t predicate1, kk_context_t* _ctx) {
  struct kk_std_core_while_fun21582__t* _self = kk_function_alloc_as(struct kk_std_core_while_fun21582__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_while_fun21582, kk_context());
  _self->action1 = action1;
  _self->predicate1 = predicate1;
  return &_self->_base;
}

static kk_unit_t kk_std_core_while_fun21582(kk_function_t _fself, kk_unit_t wild__1, kk_context_t* _ctx) {
  struct kk_std_core_while_fun21582__t* _self = kk_function_as(struct kk_std_core_while_fun21582__t*, _fself);
  kk_function_t action1 = _self->action1; /* () -> <div|16672> () */
  kk_function_t predicate1 = _self->predicate1; /* () -> <div|16672> bool */
  kk_drop_match(_self, {kk_function_dup(action1);kk_function_dup(predicate1);}, {}, _ctx)
  return kk_std_core__mlift17577_while(action1, predicate1, wild__1, _ctx);
}


// lift anonymous function
struct kk_std_core_while_fun21584__t {
  struct kk_function_s _base;
  kk_function_t _b_19532_19529;
};
static kk_box_t kk_std_core_while_fun21584(kk_function_t _fself, kk_box_t _b_19530, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_while_fun21584(kk_function_t _b_19532_19529, kk_context_t* _ctx) {
  struct kk_std_core_while_fun21584__t* _self = kk_function_alloc_as(struct kk_std_core_while_fun21584__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_while_fun21584, kk_context());
  _self->_b_19532_19529 = _b_19532_19529;
  return &_self->_base;
}

static kk_box_t kk_std_core_while_fun21584(kk_function_t _fself, kk_box_t _b_19530, kk_context_t* _ctx) {
  struct kk_std_core_while_fun21584__t* _self = kk_function_as(struct kk_std_core_while_fun21584__t*, _fself);
  kk_function_t _b_19532_19529 = _self->_b_19532_19529; /* (wild_1 : ()) -> <div|16672> () */
  kk_drop_match(_self, {kk_function_dup(_b_19532_19529);}, {}, _ctx)
  kk_unit_t _x21585 = kk_Unit;
  kk_unit_t _x21586 = kk_Unit;
  kk_unit_unbox(_b_19530);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19532_19529, (_b_19532_19529, _x21586, _ctx));
  return kk_unit_box(_x21585);
}

kk_unit_t kk_std_core_while(kk_function_t predicate1, kk_function_t action1, kk_context_t* _ctx) { /* forall<e> (predicate : () -> <div|e> bool, action : () -> <div|e> ()) -> <div|e> () */ 
  kk__tailcall: ;
  bool x0_18051;
  kk_function_t _x21575 = kk_function_dup(predicate1); /*() -> <div|16672> bool*/
  x0_18051 = kk_function_call(bool, (kk_function_t, kk_context_t*), _x21575, (_x21575, _ctx)); /*bool*/
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19531_19527 = kk_std_core_new_while_fun21576(action1, predicate1, _ctx); /*(bool) -> <div|16672> ()*/;
    kk_box_t _x21577 = kk_std_core_hnd_yield_extend(kk_std_core_new_while_fun21578(_b_19531_19527, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21577); return kk_Unit;
  }
  if (x0_18051) {
    kk_unit_t x1_18054 = kk_Unit;
    kk_function_t _x21581 = kk_function_dup(action1); /*() -> <div|16672> ()*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_context_t*), _x21581, (_x21581, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19532_19529 = kk_std_core_new_while_fun21582(action1, predicate1, _ctx); /*(wild_1 : ()) -> <div|16672> ()*/;
      kk_box_t _x21583 = kk_std_core_hnd_yield_extend(kk_std_core_new_while_fun21584(_b_19532_19529, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x21583); return kk_Unit;
    }
    { // tailcall
      goto kk__tailcall;
    }
  }
  kk_function_drop(action1, _ctx);
  kk_function_drop(predicate1, _ctx);
  kk_Unit; return kk_Unit;
}
 
// Zip two lists together by pairing the corresponding elements.
// The returned list is only as long as the smallest input list.

kk_std_core__list kk_std_core__ctail_zip(kk_std_core__list xs, kk_std_core__list ys, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a,b> (xs : list<a>, ys : list<b>, ctail<list<(a, b)>>) -> list<(a, b)> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21587 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21587->head;
    kk_std_core__list xx = _con21587->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    if (kk_std_core__is_Cons(ys)) {
      struct kk_std_core_Cons* _con21588 = kk_std_core__as_Cons(ys);
      kk_box_t y = _con21588->head;
      kk_std_core__list yy = _con21588->tail;
      kk_reuse_t _ru_20100 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(ys))) {
        _ru_20100 = (kk_std_core__list_reuse(ys));
      }
      else {
        kk_box_dup(y);
        kk_std_core__list_dup(yy);
        kk_Unit;
        kk_std_core__list_decref(ys, _ctx);
        kk_Unit;
      }
      kk_std_core__list _ctail_17210 = kk_std_core__list_hole(); /*list<(16715, 16716)>*/;
      kk_std_core__list _ctail_17211;
      kk_box_t _x21589;
      kk_std_core_types__tuple2_ _x21590 = kk_std_core_types__new_dash__lp__comma__rp_(x, y, _ctx); /*(6, 7)*/
      _x21589 = kk_std_core_types__tuple2__box(_x21590, _ctx); /*61*/
      _ctail_17211 = kk_std_core__new_Cons(_ru_20100, _x21589, _ctail_17210, _ctx); /*list<(16715, 16716)>*/
      { // tailcall
        kk_std_core_types__ctail _x21591;
        kk_box_t* field_18059 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17211)->tail)); /*cfield<list<(16715, 16716)>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x19543 = _acc._cons.CTail._field1;
          kk_box_t* last0 = _acc._cons.CTail._field2;
          kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19543, NULL);
          kk_std_core__list_dup(head0);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last0) = kk_std_core__list_box(_ctail_17211, _ctx);
          kk_std_core__list _b_19562_19548 = head0; /*list<(16715, 16716)>*/;
          kk_box_t* _b_19563_19549 = field_18059; /*cfield<list<(16715, 16716)>>*/;
          _x21591 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19562_19548, _ctx), _b_19563_19549, _ctx); /*ctail<58>*/
          goto _match21592;
        }
        kk_std_core__list _b_19564_19550 = _ctail_17211; /*list<(16715, 16716)>*/;
        kk_box_t* _b_19565_19551 = field_18059; /*cfield<list<(16715, 16716)>>*/;
        _x21591 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19564_19550, _ctx), _b_19565_19551, _ctx); /*ctail<58>*/
        _match21592: ;
        xs = xx;
        ys = yy;
        _acc = _x21591;
        goto kk__tailcall;
      }
    }
    kk_box_drop(x, _ctx);
    kk_std_core__list_drop(xx, _ctx);
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x19552 = _acc._cons.CTail._field1;
      kk_box_t* last2 = _acc._cons.CTail._field2;
      kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19552, NULL);
      kk_std_core__list_dup(head1);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __0 = kk_Unit;
      *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
      return head1;
    }
    return kk_std_core__new_Nil(_ctx);
  }
  kk_std_core__list_drop(ys, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19557 = _acc._cons.CTail._field1;
    kk_box_t* last3 = _acc._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19557, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Zip two lists together by pairing the corresponding elements.
// The returned list is only as long as the smallest input list.

kk_std_core__list kk_std_core_zip(kk_std_core__list xs0, kk_std_core__list ys0, kk_context_t* _ctx) { /* forall<a,b> (xs : list<a>, ys : list<b>) -> list<(a, b)> */ 
  return kk_std_core__ctail_zip(xs0, ys0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17579_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_std_core__list xx, kk_std_core__list yy, kk_box_t _ctail_17212, kk_context_t* _ctx) { /* forall<a,b,c,e> (ctail<list<c>>, f : (a, b) -> e c, xx : list<a>, yy : list<b>, c) -> e list<c> */ 
  kk_std_core__list _ctail_17213 = kk_std_core__list_hole(); /*list<16763>*/;
  kk_std_core__list _ctail_17214 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17212, _ctail_17213, _ctx); /*list<16763>*/;
  kk_std_core_types__ctail _x21596;
  kk_box_t* field_18066 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17214)->tail)); /*cfield<list<16763>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19572 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19572, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17214, _ctx);
    kk_std_core__list _b_19581_19577 = head0; /*list<16763>*/;
    kk_box_t* _b_19582_19578 = field_18066; /*cfield<list<16763>>*/;
    _x21596 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19581_19577, _ctx), _b_19582_19578, _ctx); /*ctail<58>*/
    goto _match21597;
  }
  kk_std_core__list _b_19583_19579 = _ctail_17214; /*list<16763>*/;
  kk_box_t* _b_19584_19580 = field_18066; /*cfield<list<16763>>*/;
  _x21596 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19583_19579, _ctx), _b_19584_19580, _ctx); /*ctail<58>*/
  _match21597: ;
  return kk_std_core__ctail_zipwith(xx, yy, f, _x21596, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17580_op_fun21599__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t _ctail_17217;
};
static kk_std_core__list kk_std_core__mlift17580_op_fun21599(kk_function_t _fself, kk_std_core__list _ctail_17216, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17580_op_fun21599(kk_function_t _accm, kk_box_t _ctail_17217, kk_context_t* _ctx) {
  struct kk_std_core__mlift17580_op_fun21599__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17580_op_fun21599__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17580_op_fun21599, kk_context());
  _self->_accm = _accm;
  _self->_ctail_17217 = _ctail_17217;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17580_op_fun21599(kk_function_t _fself, kk_std_core__list _ctail_17216, kk_context_t* _ctx) {
  struct kk_std_core__mlift17580_op_fun21599__t* _self = kk_function_as(struct kk_std_core__mlift17580_op_fun21599__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<16763>) -> list<16763> */
  kk_box_t _ctail_17217 = _self->_ctail_17217; /* 16763 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(_ctail_17217);}, {}, _ctx)
  kk_std_core__list _x21600 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17217, _ctail_17216, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x21600, _ctx));
}

kk_std_core__list kk_std_core__mlift17580_op(kk_function_t _accm, kk_function_t f0, kk_std_core__list xx0, kk_std_core__list yy0, kk_box_t _ctail_17217, kk_context_t* _ctx) { /* forall<a,b,c,e> ((list<c>) -> list<c>, f : (a, b) -> e c, xx : list<a>, yy : list<b>, c) -> e list<c> */ 
  return kk_std_core__ctailm_zipwith(xx0, yy0, f0, kk_std_core__new_mlift17580_op_fun21599(_accm, _ctail_17217, _ctx), _ctx);
}
 
// Zip two lists together by apply a function `f` to all corresponding elements.
// The returned list is only as long as the smallest input list.


// lift anonymous function
struct kk_std_core__ctail_zipwith_fun21604__t {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_std_core__list xx1;
  kk_std_core__list yy1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_zipwith_fun21604(kk_function_t _fself, kk_box_t _ctail_172120, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_zipwith_fun21604(kk_function_t f1, kk_std_core__list xx1, kk_std_core__list yy1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_zipwith_fun21604__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_zipwith_fun21604__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_zipwith_fun21604, kk_context());
  _self->f1 = f1;
  _self->xx1 = xx1;
  _self->yy1 = yy1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_zipwith_fun21604(kk_function_t _fself, kk_box_t _ctail_172120, kk_context_t* _ctx) {
  struct kk_std_core__ctail_zipwith_fun21604__t* _self = kk_function_as(struct kk_std_core__ctail_zipwith_fun21604__t*, _fself);
  kk_function_t f1 = _self->f1; /* (16761, 16762) -> 16764 16763 */
  kk_std_core__list xx1 = _self->xx1; /* list<16761> */
  kk_std_core__list yy1 = _self->yy1; /* list<16762> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<16763>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_std_core__list_dup(xx1);kk_std_core__list_dup(yy1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17579_op(_acc0, f1, xx1, yy1, _ctail_172120, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_zipwith_fun21606__t {
  struct kk_function_s _base;
  kk_function_t _b_19612_19585;
};
static kk_box_t kk_std_core__ctail_zipwith_fun21606(kk_function_t _fself, kk_box_t _b_19586, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_zipwith_fun21606(kk_function_t _b_19612_19585, kk_context_t* _ctx) {
  struct kk_std_core__ctail_zipwith_fun21606__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_zipwith_fun21606__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_zipwith_fun21606, kk_context());
  _self->_b_19612_19585 = _b_19612_19585;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_zipwith_fun21606(kk_function_t _fself, kk_box_t _b_19586, kk_context_t* _ctx) {
  struct kk_std_core__ctail_zipwith_fun21606__t* _self = kk_function_as(struct kk_std_core__ctail_zipwith_fun21606__t*, _fself);
  kk_function_t _b_19612_19585 = _self->_b_19612_19585; /* (16763) -> 16764 list<16763> */
  kk_drop_match(_self, {kk_function_dup(_b_19612_19585);}, {}, _ctx)
  kk_std_core__list _x21607 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19612_19585, (_b_19612_19585, _b_19586, _ctx)); /*list<16763>*/
  return kk_std_core__list_box(_x21607, _ctx);
}

kk_std_core__list kk_std_core__ctail_zipwith(kk_std_core__list xs, kk_std_core__list ys, kk_function_t f1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,c,e> (xs : list<a>, ys : list<b>, f : (a, b) -> e c, ctail<list<c>>) -> e list<c> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21601 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21601->head;
    kk_std_core__list xx1 = _con21601->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    if (kk_std_core__is_Cons(ys)) {
      struct kk_std_core_Cons* _con21602 = kk_std_core__as_Cons(ys);
      kk_box_t y = _con21602->head;
      kk_std_core__list yy1 = _con21602->tail;
      kk_reuse_t _ru_20102 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(ys))) {
        _ru_20102 = (kk_std_core__list_reuse(ys));
      }
      else {
        kk_box_dup(y);
        kk_std_core__list_dup(yy1);
        kk_Unit;
        kk_std_core__list_decref(ys, _ctx);
        kk_Unit;
      }
      kk_box_t x0_18067;
      kk_function_t _x21603 = kk_function_dup(f1); /*(16761, 16762) -> 16764 16763*/
      x0_18067 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x21603, (_x21603, x, y, _ctx)); /*16763*/
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_20102);
        kk_box_drop(x0_18067, _ctx);
        kk_function_t _b_19612_19585 = kk_std_core__new_ctail_zipwith_fun21604(f1, xx1, yy1, _acc0, _ctx); /*(16763) -> 16764 list<16763>*/;
        kk_box_t _x21605 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_zipwith_fun21606(_b_19612_19585, _ctx), _ctx); /*3858*/
        return kk_std_core__list_unbox(_x21605, _ctx);
      }
      kk_std_core__list _ctail_172130 = kk_std_core__list_hole(); /*list<16763>*/;
      kk_std_core__list _ctail_172140 = kk_std_core__new_Cons(_ru_20102, x0_18067, _ctail_172130, _ctx); /*list<16763>*/;
      { // tailcall
        kk_std_core_types__ctail _x21608;
        kk_box_t* field0_18072 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_172140)->tail)); /*cfield<list<16763>>*/;
        if (kk_std_core_types__is_CTail(_acc0)) {
          kk_box_t _box_x19593 = _acc0._cons.CTail._field1;
          kk_box_t* last2 = _acc0._cons.CTail._field2;
          kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19593, NULL);
          kk_std_core__list_dup(head1);
          kk_std_core_types__ctail_drop(_acc0, _ctx);
          kk_Unit;
          kk_unit_t __0 = kk_Unit;
          *(last2) = kk_std_core__list_box(_ctail_172140, _ctx);
          kk_std_core__list _b_19613_19598 = head1; /*list<16763>*/;
          kk_box_t* _b_19614_19599 = field0_18072; /*cfield<list<16763>>*/;
          _x21608 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19613_19598, _ctx), _b_19614_19599, _ctx); /*ctail<58>*/
          goto _match21609;
        }
        kk_std_core__list _b_19615_19600 = _ctail_172140; /*list<16763>*/;
        kk_box_t* _b_19616_19601 = field0_18072; /*cfield<list<16763>>*/;
        _x21608 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19615_19600, _ctx), _b_19616_19601, _ctx); /*ctail<58>*/
        _match21609: ;
        xs = xx1;
        ys = yy1;
        _acc0 = _x21608;
        goto kk__tailcall;
      }
    }
    kk_function_drop(f1, _ctx);
    kk_box_drop(x, _ctx);
    kk_std_core__list_drop(xx1, _ctx);
    if (kk_std_core_types__is_CTail(_acc0)) {
      kk_box_t _box_x19602 = _acc0._cons.CTail._field1;
      kk_box_t* last3 = _acc0._cons.CTail._field2;
      kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19602, NULL);
      kk_std_core__list_dup(head2);
      kk_std_core_types__ctail_drop(_acc0, _ctx);
      kk_Unit;
      kk_unit_t __1 = kk_Unit;
      *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
      return head2;
    }
    return kk_std_core__new_Nil(_ctx);
  }
  kk_function_drop(f1, _ctx);
  kk_std_core__list_drop(ys, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x19607 = _acc0._cons.CTail._field1;
    kk_box_t* last4 = _acc0._cons.CTail._field2;
    kk_std_core__list head3 = kk_std_core__list_unbox(_box_x19607, NULL);
    kk_std_core__list_dup(head3);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __2 = kk_Unit;
    *(last4) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head3;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Zip two lists together by apply a function `f` to all corresponding elements.
// The returned list is only as long as the smallest input list.


// lift anonymous function
struct kk_std_core__ctailm_zipwith_fun21616__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_std_core__list xx2;
  kk_std_core__list yy2;
};
static kk_std_core__list kk_std_core__ctailm_zipwith_fun21616(kk_function_t _fself, kk_box_t _ctail_172170, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_zipwith_fun21616(kk_function_t _accm0, kk_function_t f2, kk_std_core__list xx2, kk_std_core__list yy2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun21616__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_zipwith_fun21616__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_zipwith_fun21616, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->xx2 = xx2;
  _self->yy2 = yy2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_zipwith_fun21616(kk_function_t _fself, kk_box_t _ctail_172170, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun21616__t* _self = kk_function_as(struct kk_std_core__ctailm_zipwith_fun21616__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<16763>) -> list<16763> */
  kk_function_t f2 = _self->f2; /* (16761, 16762) -> 16764 16763 */
  kk_std_core__list xx2 = _self->xx2; /* list<16761> */
  kk_std_core__list yy2 = _self->yy2; /* list<16762> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_std_core__list_dup(xx2);kk_std_core__list_dup(yy2);}, {}, _ctx)
  return kk_std_core__mlift17580_op(_accm0, f2, xx2, yy2, _ctail_172170, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_zipwith_fun21618__t {
  struct kk_function_s _base;
  kk_function_t _b_19619_19617;
};
static kk_box_t kk_std_core__ctailm_zipwith_fun21618(kk_function_t _fself, kk_box_t _b_19618, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_zipwith_fun21618(kk_function_t _b_19619_19617, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun21618__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_zipwith_fun21618__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_zipwith_fun21618, kk_context());
  _self->_b_19619_19617 = _b_19619_19617;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_zipwith_fun21618(kk_function_t _fself, kk_box_t _b_19618, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun21618__t* _self = kk_function_as(struct kk_std_core__ctailm_zipwith_fun21618__t*, _fself);
  kk_function_t _b_19619_19617 = _self->_b_19619_19617; /* (16763) -> 16764 list<16763> */
  kk_drop_match(_self, {kk_function_dup(_b_19619_19617);}, {}, _ctx)
  kk_std_core__list _x21619 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19619_19617, (_b_19619_19617, _b_19618, _ctx)); /*list<16763>*/
  return kk_std_core__list_box(_x21619, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_zipwith_fun21621__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x2_18077;
};
static kk_std_core__list kk_std_core__ctailm_zipwith_fun21621(kk_function_t _fself, kk_std_core__list _ctail_172160, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_zipwith_fun21621(kk_function_t _accm0, kk_box_t x2_18077, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun21621__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_zipwith_fun21621__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_zipwith_fun21621, kk_context());
  _self->_accm0 = _accm0;
  _self->x2_18077 = x2_18077;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_zipwith_fun21621(kk_function_t _fself, kk_std_core__list _ctail_172160, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun21621__t* _self = kk_function_as(struct kk_std_core__ctailm_zipwith_fun21621__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<16763>) -> list<16763> */
  kk_box_t x2_18077 = _self->x2_18077; /* 16763 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x2_18077);}, {}, _ctx)
  kk_std_core__list _x21622 = kk_std_core__new_Cons(kk_reuse_null, x2_18077, _ctail_172160, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21622, _ctx));
}

kk_std_core__list kk_std_core__ctailm_zipwith(kk_std_core__list xs0, kk_std_core__list ys0, kk_function_t f2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,c,e> (xs : list<a>, ys : list<b>, f : (a, b) -> e c, (list<c>) -> list<c>) -> e list<c> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21613 = kk_std_core__as_Cons(xs0);
    kk_box_t x1 = _con21613->head;
    kk_std_core__list xx2 = _con21613->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx2);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    if (kk_std_core__is_Cons(ys0)) {
      struct kk_std_core_Cons* _con21614 = kk_std_core__as_Cons(ys0);
      kk_box_t y0 = _con21614->head;
      kk_std_core__list yy2 = _con21614->tail;
      if (kk_likely(kk_std_core__list_is_unique(ys0))) {
        kk_std_core__list_free(ys0);
      }
      else {
        kk_box_dup(y0);
        kk_std_core__list_dup(yy2);
        kk_Unit;
        kk_std_core__list_decref(ys0, _ctx);
        kk_Unit;
      }
      kk_box_t x2_18077;
      kk_function_t _x21615 = kk_function_dup(f2); /*(16761, 16762) -> 16764 16763*/
      x2_18077 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x21615, (_x21615, x1, y0, _ctx)); /*16763*/
      if (kk_yielding(kk_context())) {
        kk_box_drop(x2_18077, _ctx);
        kk_function_t _b_19619_19617 = kk_std_core__new_ctailm_zipwith_fun21616(_accm0, f2, xx2, yy2, _ctx); /*(16763) -> 16764 list<16763>*/;
        kk_box_t _x21617 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_zipwith_fun21618(_b_19619_19617, _ctx), _ctx); /*3858*/
        return kk_std_core__list_unbox(_x21617, _ctx);
      }
      { // tailcall
        kk_function_t _x21620 = kk_std_core__new_ctailm_zipwith_fun21621(_accm0, x2_18077, _ctx); /*(list<16763>) -> list<16763>*/
        xs0 = xx2;
        ys0 = yy2;
        _accm0 = _x21620;
        goto kk__tailcall;
      }
    }
    kk_function_drop(f2, _ctx);
    kk_box_drop(x1, _ctx);
    kk_std_core__list_drop(xx2, _ctx);
    return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
  }
  kk_function_drop(f2, _ctx);
  kk_std_core__list_drop(ys0, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Zip two lists together by apply a function `f` to all corresponding elements.
// The returned list is only as long as the smallest input list.


// lift anonymous function
struct kk_std_core_zipwith_fun21623__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_zipwith_fun21623(kk_function_t _fself, kk_std_core__list _ctail_17215, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_zipwith_fun21623(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_zipwith_fun21623, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_zipwith_fun21623(kk_function_t _fself, kk_std_core__list _ctail_17215, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17215;
}

kk_std_core__list kk_std_core_zipwith(kk_std_core__list xs1, kk_std_core__list ys1, kk_function_t f3, kk_context_t* _ctx) { /* forall<a,b,c,e> (xs : list<a>, ys : list<b>, f : (a, b) -> e c) -> e list<c> */ 
  bool _match_19636 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19636) {
    return kk_std_core__ctail_zipwith(xs1, ys1, f3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_zipwith(xs1, ys1, f3, kk_std_core_new_zipwith_fun21623(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17581_zipwith_acc(kk_std_core__list acc, kk_function_t f, kk_integer_t i, kk_std_core__list xx, kk_std_core__list yy, kk_box_t _y_17490, kk_context_t* _ctx) { /* forall<e,a,b,c> (acc : list<b>, f : (int, a, c) -> e b, i : int, xx : list<a>, yy : list<c>, b) -> e list<b> */ 
  kk_integer_t _x21624 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_std_core__list _x21625 = kk_std_core__new_Cons(kk_reuse_null, _y_17490, acc, _ctx); /*list<61>*/
  return kk_std_core_zipwith_acc(f, _x21624, _x21625, xx, yy, _ctx);
}


// lift anonymous function
struct kk_std_core_zipwith_acc_fun21630__t {
  struct kk_function_s _base;
  kk_std_core__list acc0;
  kk_function_t f0;
  kk_integer_t i0;
  kk_std_core__list xx0;
  kk_std_core__list yy0;
};
static kk_std_core__list kk_std_core_zipwith_acc_fun21630(kk_function_t _fself, kk_box_t _y_174900, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_zipwith_acc_fun21630(kk_std_core__list acc0, kk_function_t f0, kk_integer_t i0, kk_std_core__list xx0, kk_std_core__list yy0, kk_context_t* _ctx) {
  struct kk_std_core_zipwith_acc_fun21630__t* _self = kk_function_alloc_as(struct kk_std_core_zipwith_acc_fun21630__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_zipwith_acc_fun21630, kk_context());
  _self->acc0 = acc0;
  _self->f0 = f0;
  _self->i0 = i0;
  _self->xx0 = xx0;
  _self->yy0 = yy0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core_zipwith_acc_fun21630(kk_function_t _fself, kk_box_t _y_174900, kk_context_t* _ctx) {
  struct kk_std_core_zipwith_acc_fun21630__t* _self = kk_function_as(struct kk_std_core_zipwith_acc_fun21630__t*, _fself);
  kk_std_core__list acc0 = _self->acc0; /* list<16776> */
  kk_function_t f0 = _self->f0; /* (int, 16774, 16784) -> 16771 16776 */
  kk_integer_t i0 = _self->i0; /* int */
  kk_std_core__list xx0 = _self->xx0; /* list<16774> */
  kk_std_core__list yy0 = _self->yy0; /* list<16784> */
  kk_drop_match(_self, {kk_std_core__list_dup(acc0);kk_function_dup(f0);kk_integer_dup(i0);kk_std_core__list_dup(xx0);kk_std_core__list_dup(yy0);}, {}, _ctx)
  return kk_std_core__mlift17581_zipwith_acc(acc0, f0, i0, xx0, yy0, _y_174900, _ctx);
}


// lift anonymous function
struct kk_std_core_zipwith_acc_fun21632__t {
  struct kk_function_s _base;
  kk_function_t _b_19622_19620;
};
static kk_box_t kk_std_core_zipwith_acc_fun21632(kk_function_t _fself, kk_box_t _b_19621, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_zipwith_acc_fun21632(kk_function_t _b_19622_19620, kk_context_t* _ctx) {
  struct kk_std_core_zipwith_acc_fun21632__t* _self = kk_function_alloc_as(struct kk_std_core_zipwith_acc_fun21632__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_zipwith_acc_fun21632, kk_context());
  _self->_b_19622_19620 = _b_19622_19620;
  return &_self->_base;
}

static kk_box_t kk_std_core_zipwith_acc_fun21632(kk_function_t _fself, kk_box_t _b_19621, kk_context_t* _ctx) {
  struct kk_std_core_zipwith_acc_fun21632__t* _self = kk_function_as(struct kk_std_core_zipwith_acc_fun21632__t*, _fself);
  kk_function_t _b_19622_19620 = _self->_b_19622_19620; /* (16776) -> 16771 list<16776> */
  kk_drop_match(_self, {kk_function_dup(_b_19622_19620);}, {}, _ctx)
  kk_std_core__list _x21633 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19622_19620, (_b_19622_19620, _b_19621, _ctx)); /*list<16776>*/
  return kk_std_core__list_box(_x21633, _ctx);
}

kk_std_core__list kk_std_core_zipwith_acc(kk_function_t f0, kk_integer_t i0, kk_std_core__list acc0, kk_std_core__list xs, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a,b,c,e> ((int, a, b) -> e c, int, list<c>, list<a>, list<b>) -> e list<c> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(f0, _ctx);
    kk_integer_drop(i0, _ctx);
    kk_std_core__list_drop(ys, _ctx);
    return kk_std_core__lift17078_reverse(kk_std_core__new_Nil(_ctx), acc0, _ctx);
  }
  struct kk_std_core_Cons* _con21626 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21626->head;
  kk_std_core__list xx0 = _con21626->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx0);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  if (kk_std_core__is_Nil(ys)) {
    kk_function_drop(f0, _ctx);
    kk_integer_drop(i0, _ctx);
    kk_box_drop(x, _ctx);
    kk_std_core__list_drop(xx0, _ctx);
    return kk_std_core__lift17078_reverse(kk_std_core__new_Nil(_ctx), acc0, _ctx);
  }
  struct kk_std_core_Cons* _con21627 = kk_std_core__as_Cons(ys);
  kk_box_t y = _con21627->head;
  kk_std_core__list yy0 = _con21627->tail;
  kk_reuse_t _ru_20106 = kk_reuse_null; /*reuse*/;
  if (kk_likely(kk_std_core__list_is_unique(ys))) {
    _ru_20106 = (kk_std_core__list_reuse(ys));
  }
  else {
    kk_box_dup(y);
    kk_std_core__list_dup(yy0);
    kk_Unit;
    kk_std_core__list_decref(ys, _ctx);
    kk_Unit;
  }
  kk_box_t x0_18082;
  kk_function_t _x21629 = kk_function_dup(f0); /*(int, 16774, 16784) -> 16771 16776*/
  kk_integer_t _x21628 = kk_integer_dup(i0); /*int*/
  x0_18082 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_box_t, kk_box_t, kk_context_t*), _x21629, (_x21629, _x21628, x, y, _ctx)); /*16776*/
  if (kk_yielding(kk_context())) {
    kk_reuse_drop(_ru_20106);
    kk_box_drop(x0_18082, _ctx);
    kk_function_t _b_19622_19620 = kk_std_core_new_zipwith_acc_fun21630(acc0, f0, i0, xx0, yy0, _ctx); /*(16776) -> 16771 list<16776>*/;
    kk_box_t _x21631 = kk_std_core_hnd_yield_extend(kk_std_core_new_zipwith_acc_fun21632(_b_19622_19620, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21631, _ctx);
  }
  { // tailcall
    kk_integer_t _x21634 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__list _x21635 = kk_std_core__new_Cons(_ru_20106, x0_18082, acc0, _ctx); /*list<61>*/
    i0 = _x21634;
    acc0 = _x21635;
    xs = xx0;
    ys = yy0;
    goto kk__tailcall;
  }
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17582_op(kk_box_t _y_17494, kk_std_core__list _y_17495, kk_context_t* _ctx) { /* forall<a,e> (a, list<a>) -> e list<a> */ 
  return kk_std_core__new_Cons(kk_reuse_null, _y_17494, _y_17495, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17583_op_fun21636__t {
  struct kk_function_s _base;
  kk_box_t _y_174940;
};
static kk_std_core__list kk_std_core__mlift17583_op_fun21636(kk_function_t _fself, kk_std_core__list _y_174950, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17583_op_fun21636(kk_box_t _y_174940, kk_context_t* _ctx) {
  struct kk_std_core__mlift17583_op_fun21636__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17583_op_fun21636__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17583_op_fun21636, kk_context());
  _self->_y_174940 = _y_174940;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17583_op_fun21636(kk_function_t _fself, kk_std_core__list _y_174950, kk_context_t* _ctx) {
  struct kk_std_core__mlift17583_op_fun21636__t* _self = kk_function_as(struct kk_std_core__mlift17583_op_fun21636__t*, _fself);
  kk_box_t _y_174940 = _self->_y_174940; /* 16905 */
  kk_drop_match(_self, {kk_box_dup(_y_174940);}, {}, _ctx)
  return kk_std_core__mlift17582_op(_y_174940, _y_174950, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17583_op_fun21638__t {
  struct kk_function_s _base;
  kk_function_t _b_19625_19623;
};
static kk_box_t kk_std_core__mlift17583_op_fun21638(kk_function_t _fself, kk_box_t _b_19624, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17583_op_fun21638(kk_function_t _b_19625_19623, kk_context_t* _ctx) {
  struct kk_std_core__mlift17583_op_fun21638__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17583_op_fun21638__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17583_op_fun21638, kk_context());
  _self->_b_19625_19623 = _b_19625_19623;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17583_op_fun21638(kk_function_t _fself, kk_box_t _b_19624, kk_context_t* _ctx) {
  struct kk_std_core__mlift17583_op_fun21638__t* _self = kk_function_as(struct kk_std_core__mlift17583_op_fun21638__t*, _fself);
  kk_function_t _b_19625_19623 = _self->_b_19625_19623; /* (list<16905>) -> 16906 list<16905> */
  kk_drop_match(_self, {kk_function_dup(_b_19625_19623);}, {}, _ctx)
  kk_std_core__list _x21639;
  kk_std_core__list _x21640 = kk_std_core__list_unbox(_b_19624, _ctx); /*list<16905>*/
  _x21639 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19625_19623, (_b_19625_19623, _x21640, _ctx)); /*list<16905>*/
  return kk_std_core__list_box(_x21639, _ctx);
}

kk_std_core__list kk_std_core__mlift17583_op(kk_function_t f, kk_integer_t i, kk_std_core__list xx, kk_std_core__list yy, kk_box_t _y_174940, kk_context_t* _ctx) { /* forall<a,b,c,e> (f : (int, a, b) -> e c, i : int, xx : list<a>, yy : list<b>, c) -> e list<c> */ 
  kk_integer_t i0_17141 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__list x_18085 = kk_std_core__lift17089_zipwith_indexed(f, i0_17141, xx, yy, _ctx); /*list<16905>*/;
  if (kk_yielding(kk_context())) {
    kk_std_core__list_drop(x_18085, _ctx);
    kk_function_t _b_19625_19623 = kk_std_core__new_mlift17583_op_fun21636(_y_174940, _ctx); /*(list<16905>) -> 16906 list<16905>*/;
    kk_box_t _x21637 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17583_op_fun21638(_b_19625_19623, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21637, _ctx);
  }
  return kk_std_core__mlift17582_op(_y_174940, x_18085, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17089_zipwith_indexed_fun21645__t {
  struct kk_function_s _base;
  kk_function_t f0;
  kk_integer_t i0;
  kk_std_core__list xx0;
  kk_std_core__list yy0;
};
static kk_std_core__list kk_std_core__lift17089_zipwith_indexed_fun21645(kk_function_t _fself, kk_box_t _y_174941, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17089_zipwith_indexed_fun21645(kk_function_t f0, kk_integer_t i0, kk_std_core__list xx0, kk_std_core__list yy0, kk_context_t* _ctx) {
  struct kk_std_core__lift17089_zipwith_indexed_fun21645__t* _self = kk_function_alloc_as(struct kk_std_core__lift17089_zipwith_indexed_fun21645__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17089_zipwith_indexed_fun21645, kk_context());
  _self->f0 = f0;
  _self->i0 = i0;
  _self->xx0 = xx0;
  _self->yy0 = yy0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17089_zipwith_indexed_fun21645(kk_function_t _fself, kk_box_t _y_174941, kk_context_t* _ctx) {
  struct kk_std_core__lift17089_zipwith_indexed_fun21645__t* _self = kk_function_as(struct kk_std_core__lift17089_zipwith_indexed_fun21645__t*, _fself);
  kk_function_t f0 = _self->f0; /* (int, 16903, 16904) -> 16906 16905 */
  kk_integer_t i0 = _self->i0; /* int */
  kk_std_core__list xx0 = _self->xx0; /* list<16903> */
  kk_std_core__list yy0 = _self->yy0; /* list<16904> */
  kk_drop_match(_self, {kk_function_dup(f0);kk_integer_dup(i0);kk_std_core__list_dup(xx0);kk_std_core__list_dup(yy0);}, {}, _ctx)
  return kk_std_core__mlift17583_op(f0, i0, xx0, yy0, _y_174941, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17089_zipwith_indexed_fun21647__t {
  struct kk_function_s _base;
  kk_function_t _b_19630_19626;
};
static kk_box_t kk_std_core__lift17089_zipwith_indexed_fun21647(kk_function_t _fself, kk_box_t _b_19627, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17089_zipwith_indexed_fun21647(kk_function_t _b_19630_19626, kk_context_t* _ctx) {
  struct kk_std_core__lift17089_zipwith_indexed_fun21647__t* _self = kk_function_alloc_as(struct kk_std_core__lift17089_zipwith_indexed_fun21647__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17089_zipwith_indexed_fun21647, kk_context());
  _self->_b_19630_19626 = _b_19630_19626;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17089_zipwith_indexed_fun21647(kk_function_t _fself, kk_box_t _b_19627, kk_context_t* _ctx) {
  struct kk_std_core__lift17089_zipwith_indexed_fun21647__t* _self = kk_function_as(struct kk_std_core__lift17089_zipwith_indexed_fun21647__t*, _fself);
  kk_function_t _b_19630_19626 = _self->_b_19630_19626; /* (16905) -> 16906 list<16905> */
  kk_drop_match(_self, {kk_function_dup(_b_19630_19626);}, {}, _ctx)
  kk_std_core__list _x21648 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19630_19626, (_b_19630_19626, _b_19627, _ctx)); /*list<16905>*/
  return kk_std_core__list_box(_x21648, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17089_zipwith_indexed_fun21649__t {
  struct kk_function_s _base;
  kk_box_t x1_18087;
};
static kk_std_core__list kk_std_core__lift17089_zipwith_indexed_fun21649(kk_function_t _fself, kk_std_core__list _y_174951, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17089_zipwith_indexed_fun21649(kk_box_t x1_18087, kk_context_t* _ctx) {
  struct kk_std_core__lift17089_zipwith_indexed_fun21649__t* _self = kk_function_alloc_as(struct kk_std_core__lift17089_zipwith_indexed_fun21649__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17089_zipwith_indexed_fun21649, kk_context());
  _self->x1_18087 = x1_18087;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17089_zipwith_indexed_fun21649(kk_function_t _fself, kk_std_core__list _y_174951, kk_context_t* _ctx) {
  struct kk_std_core__lift17089_zipwith_indexed_fun21649__t* _self = kk_function_as(struct kk_std_core__lift17089_zipwith_indexed_fun21649__t*, _fself);
  kk_box_t x1_18087 = _self->x1_18087; /* 16905 */
  kk_drop_match(_self, {kk_box_dup(x1_18087);}, {}, _ctx)
  return kk_std_core__mlift17582_op(x1_18087, _y_174951, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17089_zipwith_indexed_fun21651__t {
  struct kk_function_s _base;
  kk_function_t _b_19631_19628;
};
static kk_box_t kk_std_core__lift17089_zipwith_indexed_fun21651(kk_function_t _fself, kk_box_t _b_19629, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17089_zipwith_indexed_fun21651(kk_function_t _b_19631_19628, kk_context_t* _ctx) {
  struct kk_std_core__lift17089_zipwith_indexed_fun21651__t* _self = kk_function_alloc_as(struct kk_std_core__lift17089_zipwith_indexed_fun21651__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17089_zipwith_indexed_fun21651, kk_context());
  _self->_b_19631_19628 = _b_19631_19628;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17089_zipwith_indexed_fun21651(kk_function_t _fself, kk_box_t _b_19629, kk_context_t* _ctx) {
  struct kk_std_core__lift17089_zipwith_indexed_fun21651__t* _self = kk_function_as(struct kk_std_core__lift17089_zipwith_indexed_fun21651__t*, _fself);
  kk_function_t _b_19631_19628 = _self->_b_19631_19628; /* (list<16905>) -> 16906 list<16905> */
  kk_drop_match(_self, {kk_function_dup(_b_19631_19628);}, {}, _ctx)
  kk_std_core__list _x21652;
  kk_std_core__list _x21653 = kk_std_core__list_unbox(_b_19629, _ctx); /*list<16905>*/
  _x21652 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19631_19628, (_b_19631_19628, _x21653, _ctx)); /*list<16905>*/
  return kk_std_core__list_box(_x21652, _ctx);
}

kk_std_core__list kk_std_core__lift17089_zipwith_indexed(kk_function_t f0, kk_integer_t i0, kk_std_core__list xs, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a,b,c,e> (f : (int, a, b) -> e c, i : int, xs : list<a>, ys : list<b>) -> e list<c> */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21641 = kk_std_core__as_Cons(xs);
    kk_box_t x0 = _con21641->head;
    kk_std_core__list xx0 = _con21641->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x0);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    if (kk_std_core__is_Cons(ys)) {
      struct kk_std_core_Cons* _con21642 = kk_std_core__as_Cons(ys);
      kk_box_t y = _con21642->head;
      kk_std_core__list yy0 = _con21642->tail;
      kk_reuse_t _ru_20108 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(ys))) {
        _ru_20108 = (kk_std_core__list_reuse(ys));
      }
      else {
        kk_box_dup(y);
        kk_std_core__list_dup(yy0);
        kk_Unit;
        kk_std_core__list_decref(ys, _ctx);
        kk_Unit;
      }
      kk_box_t x1_18087;
      kk_function_t _x21644 = kk_function_dup(f0); /*(int, 16903, 16904) -> 16906 16905*/
      kk_integer_t _x21643 = kk_integer_dup(i0); /*int*/
      x1_18087 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_box_t, kk_box_t, kk_context_t*), _x21644, (_x21644, _x21643, x0, y, _ctx)); /*16905*/
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_20108);
        kk_box_drop(x1_18087, _ctx);
        kk_function_t _b_19630_19626 = kk_std_core__new_lift17089_zipwith_indexed_fun21645(f0, i0, xx0, yy0, _ctx); /*(16905) -> 16906 list<16905>*/;
        kk_box_t _x21646 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17089_zipwith_indexed_fun21647(_b_19630_19626, _ctx), _ctx); /*3858*/
        return kk_std_core__list_unbox(_x21646, _ctx);
      }
      kk_integer_t i0_171410 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
      kk_std_core__list x2_18090 = kk_std_core__lift17089_zipwith_indexed(f0, i0_171410, xx0, yy0, _ctx); /*list<16905>*/;
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_20108);
        kk_std_core__list_drop(x2_18090, _ctx);
        kk_function_t _b_19631_19628 = kk_std_core__new_lift17089_zipwith_indexed_fun21649(x1_18087, _ctx); /*(list<16905>) -> 16906 list<16905>*/;
        kk_box_t _x21650 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17089_zipwith_indexed_fun21651(_b_19631_19628, _ctx), _ctx); /*3858*/
        return kk_std_core__list_unbox(_x21650, _ctx);
      }
      return kk_std_core__new_Cons(_ru_20108, x1_18087, x2_18090, _ctx);
    }
    kk_function_drop(f0, _ctx);
    kk_integer_drop(i0, _ctx);
    kk_box_drop(x0, _ctx);
    kk_std_core__list_drop(xx0, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  kk_function_drop(f0, _ctx);
  kk_integer_drop(i0, _ctx);
  kk_std_core__list_drop(ys, _ctx);
  return kk_std_core__new_Nil(_ctx);
}

// initialization
void kk_std_core__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_string_t _x20146;
    kk_define_string_literal(, _s20147, 8, "exn.core")
    _x20146 = kk_string_dup(_s20147); /*string*/
    kk_std_core__tag_exn = kk_std_core_hnd__new_Htag(_x20146, _ctx); /*std/core/hnd/htag<.hnd-exn>*/
  }
  {
    kk_std_core_types__maybe _b_18193_18192 = kk_std_core_types__new_Nothing(_ctx); /*maybe<(string) -> console ()>*/;
    kk_std_core_redirect = kk_ref_alloc((kk_std_core_types__maybe_box(_b_18193_18192, _ctx)),kk_context()); /*ref<global,maybe<(string) -> console ()>>*/
  }
  {
    kk_std_core_maxListStack = kk_integer_from_small(50); /*int*/
  }
  {
    bool _b_18345_18344 = true; /*bool*/;
    kk_std_core_trace_enabled = kk_ref_alloc((kk_bool_box(_b_18345_18344)),kk_context()); /*ref<global,bool>*/
  }
  {
    kk_integer_t _b_18362_18361 = kk_integer_from_small(0); /*int*/;
    kk_std_core_unique_count = kk_ref_alloc((kk_integer_box(_b_18362_18361)),kk_context()); /*ref<global,int>*/
  }
  {
    kk_string_t _x20945 = kk_string_empty(); /*string*/
    kk_ssize_t _x20947 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x20948 = ((kk_ssize_t)0); /*ssize_t*/
    kk_std_core_empty = kk_std_core__new_Sslice(_x20945, _x20947, _x20948, _ctx); /*sslice*/
  }
  {
    kk_string_t _x21183 = kk_string_empty(); /*string*/
    kk_ssize_t _x21185 = ((kk_ssize_t)-1); /*ssize_t*/
    kk_ssize_t _x21186 = ((kk_ssize_t)0); /*ssize_t*/
    kk_std_core_invalid = kk_std_core__new_Sslice(_x21183, _x21185, _x21186, _ctx); /*sslice*/
  }
}

// termination
void kk_std_core__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_core__sslice_drop(kk_std_core_invalid, _ctx);
  kk_std_core__sslice_drop(kk_std_core_empty, _ctx);
  kk_ref_drop(kk_std_core_unique_count, _ctx);
  kk_ref_drop(kk_std_core_trace_enabled, _ctx);
  kk_integer_drop(kk_std_core_maxListStack, _ctx);
  kk_ref_drop(kk_std_core_redirect, _ctx);
  kk_std_core_hnd__htag_drop(kk_std_core__tag_exn, _ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
