// Koka generated module: "std/time/calendar", koka version: 2.2.1
#include "std_time_calendar.h"
/*---------------------------------------------------------------------------
  Copyright 2020-2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------------------------------
  Local time zone UTC delta
--------------------------------------------------------------------------------------------------*/

#if !defined(__STDC_WANT_LIB_EXT1__)
#define __STDC_WANT_LIB_EXT1__    // for gmtime_s
#endif
#if defined(__GLIBC__) && !defined(_GNU_SOURCE)
#define _GNU_SOURCE               // for tm_zone and tm_gmtoff
#endif
#include <time.h>

static long kk_local_utc_delta(double unix_secs, kk_string_t* ptzname, kk_context_t* ctx) {
  // get the UTC delta in a somewhat portable way...
  bool isdst = false;
  time_t t = (time_t)unix_secs;
  #if (_WIN32 && KK_INTPTR_SIZE==8)
  if (t < 0) { t += 3155673600; } // try to avoid errors for negative times on some platforms by adding 100 years..
  #endif
  time_t loct = t;    
  #if defined(_GNU_SOURCE)
    // GNU libc has the tm_zone and tm_gmtoff fields
    struct tm loctm;
    if (localtime_r(&t, &loctm) != NULL) {
      isdst = (loctm.tm_isdst != 0);
      loct  = t - loctm.tm_gmtoff + (isdst ? 3600 : 0);
    }
    else {
      loctm.tm_zone = "";
    }
  #elif defined(_WIN32) && !defined(__MINGW32__)
    struct tm gmtm;
    if (gmtime_s(&gmtm, &t) == 0) {      // switched parameters :-(      
      loct = mktime(&gmtm);              // interpret gmt as local time
      struct tm loctm;
      localtime_s(&loctm, &t);           // switched parameters :-(
      isdst = (loctm.tm_isdst != 0);
    }
  #elif defined(__STDC_LIB_EXT1__)
    struct tm gmtm;
    if (gmtime_s(&t, &gmtm) != NULL) {
      loct = mktime(&gmtm);              // interpret gmt as local time
      struct tm loctm;
      localtime_s(&t, &loctm);
      isdst = (loctm.tm_isdst != 0);
    }
  #else
    struct tm* pgmtm = gmtime(&t);
    if (pgmtm != NULL) {
      loct = mktime(pgmtm);                // interpret gmt as local time
      struct tm* ploctm = localtime(&t);
      isdst = (ploctm->tm_isdst != 0);
    }
  #endif
  const time_t utc_delta = t - loct + (isdst ? 3600 : 0);   // the difference is the utc offset at that time
  if (ptzname != NULL) {
    // getting the timezone name
    #if defined(_WIN32) && !defined(__MINGW32__)
      char tzonename[256];
      size_t tznamelen;
      _get_tzname(&tznamelen, tzonename, 255, isdst ? 1 : 0); tzonename[255] = 0;
      *ptzname = kk_string_alloc_from_qutf8(tzonename, ctx);
    #elif defined(_GNU_SOURCE)
      *ptzname = kk_string_alloc_from_qutf8(loctm.tm_zone, ctx);
    #elif (_POSIX_C_SOURCE >= 1) || _XOPEN_SOURCE || _POSIX_SOURCE || __MINGW32__ // tzname
      *ptzname = kk_string_alloc_from_qutf8(tzname[isdst ? 1 : 0], ctx);
    #else
      // give up :-(
      * ptzname = kk_string_empty();
    #endif
  }
  return (long)utc_delta;
}


static kk_std_time_calendar__local_timezone kk_local_get_timezone(kk_context_t* ctx) {
  return kk_datatype_from_tag((kk_tag_t)1); // dummy value; we cannot store the local timezone as it is a global :-(
}

static kk_std_core_types__tuple2_ kk_local_get_utc_delta_tuple(kk_std_time_calendar__local_timezone tz, double unix_secs, kk_context_t* ctx) {
  kk_string_t tzonename;
  long utc_delta = kk_local_utc_delta(unix_secs, &tzonename, ctx);
  return kk_std_core_types__new_dash__lp__comma__rp_( kk_double_box((double)utc_delta,ctx), kk_string_box(tzonename), ctx );
}



// lift anonymous function
struct kk_std_time_calendar__copy_fun5442__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4811;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5442(kk_function_t _fself, kk_std_time_instant__instant _b_4814, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5442(kk_box_t _fun_unbox_x4811, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5442__t* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5442__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5442, kk_context());
  _self->_fun_unbox_x4811 = _fun_unbox_x4811;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5442(kk_function_t _fself, kk_std_time_instant__instant _b_4814, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5442__t* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5442__t*, _fself);
  kk_box_t _fun_unbox_x4811 = _self->_fun_unbox_x4811; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4811);}, {}, _ctx)
  kk_box_t _x5443;
  kk_function_t _x5444 = kk_function_unbox(_fun_unbox_x4811); /*(4812) -> 4813*/
  _x5443 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5444, (_x5444, kk_std_time_instant__instant_box(_b_4814, _ctx), _ctx)); /*4813*/
  return kk_std_core_types__tuple2__unbox(_x5443, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5446__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4818;
};
static kk_std_core_types__maybe kk_std_time_calendar__copy_fun5446(kk_function_t _fself, kk_std_time_instant__instant _b_4821, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5446(kk_box_t _fun_unbox_x4818, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5446__t* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5446__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5446, kk_context());
  _self->_fun_unbox_x4818 = _fun_unbox_x4818;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_time_calendar__copy_fun5446(kk_function_t _fself, kk_std_time_instant__instant _b_4821, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5446__t* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5446__t*, _fself);
  kk_box_t _fun_unbox_x4818 = _self->_fun_unbox_x4818; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4818);}, {}, _ctx)
  kk_box_t _x5447;
  kk_function_t _x5448 = kk_function_unbox(_fun_unbox_x4818); /*(4819) -> 4820*/
  _x5447 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5448, (_x5448, kk_std_time_instant__instant_box(_b_4821, _ctx), _ctx)); /*4820*/
  return kk_std_core_types__maybe_unbox(_x5447, _ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar__copy(kk_std_time_calendar__timezone _this, kk_std_core_types__optional name0, kk_std_core_types__optional utc_delta0, kk_std_core_types__optional utc_inverse0, kk_context_t* _ctx) { /* (timezone, name : optional<string>, utc-delta : optional<(std/time/instant/instant) -> (std/time/duration/duration, string)>, utc-inverse : optional<(std/time/instant/instant) -> maybe<std/time/instant/instant>>) -> timezone */ 
  kk_string_t _x5438;
  if (kk_std_core_types__is_Optional(name0)) {
    kk_box_t _box_x4807 = name0._cons.Optional.value;
    kk_string_t _name_217 = kk_string_unbox(_box_x4807);
    kk_string_dup(_name_217);
    kk_std_core_types__optional_drop(name0, _ctx);
    kk_Unit;
    _x5438 = _name_217; /*string*/
    goto _match5439;
  }
  kk_string_t _x = _this.name;
  kk_string_dup(_x);
  _x5438 = _x; /*string*/
  _match5439: ;
  kk_function_t _x5441;
  if (kk_std_core_types__is_Optional(utc_delta0)) {
    kk_box_t _fun_unbox_x4811 = utc_delta0._cons.Optional.value;
    _x5441 = kk_std_time_calendar__new_copy_fun5442(_fun_unbox_x4811, _ctx); /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  }
  else {
    kk_function_t _x0 = _this.utc_delta;
    kk_function_dup(_x0);
    _x5441 = _x0; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  }
  kk_function_t _x5445;
  if (kk_std_core_types__is_Optional(utc_inverse0)) {
    kk_box_t _fun_unbox_x4818 = utc_inverse0._cons.Optional.value;
    kk_std_time_calendar__timezone_drop(_this, _ctx);
    kk_Unit;
    _x5445 = kk_std_time_calendar__new_copy_fun5446(_fun_unbox_x4818, _ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  else {
    kk_function_t _x1 = _this.utc_inverse;
    kk_function_dup(_x1);
    kk_std_time_calendar__timezone_drop(_this, _ctx);
    kk_Unit;
    _x5445 = _x1; /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  return kk_std_time_calendar__new_Timezone(_x5438, _x5441, _x5445, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5470__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4828;
};
static kk_string_t kk_std_time_calendar__copy_fun5470_1(kk_function_t _fself, kk_std_time_date__date _b_4831, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5470_1(kk_box_t _fun_unbox_x4828, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5470__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5470__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5470_1, kk_context());
  _self->_fun_unbox_x4828 = _fun_unbox_x4828;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar__copy_fun5470_1(kk_function_t _fself, kk_std_time_date__date _b_4831, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5470__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5470__t_1*, _fself);
  kk_box_t _fun_unbox_x4828 = _self->_fun_unbox_x4828; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4828);}, {}, _ctx)
  kk_box_t _x5471;
  kk_function_t _x5472 = kk_function_unbox(_fun_unbox_x4828); /*(4829) -> 4830*/
  _x5471 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5472, (_x5472, kk_std_time_date__date_box(_b_4831, _ctx), _ctx)); /*4830*/
  return kk_string_unbox(_x5471);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5475__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4837;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5475_1(kk_function_t _fself, kk_std_time_instant__instant _b_4841, kk_std_time_duration__duration _b_4842, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5475_1(kk_box_t _fun_unbox_x4837, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5475__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5475__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5475_1, kk_context());
  _self->_fun_unbox_x4837 = _fun_unbox_x4837;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5475_1(kk_function_t _fself, kk_std_time_instant__instant _b_4841, kk_std_time_duration__duration _b_4842, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5475__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5475__t_1*, _fself);
  kk_box_t _fun_unbox_x4837 = _self->_fun_unbox_x4837; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4837);}, {}, _ctx)
  kk_box_t _x5476;
  kk_function_t _x5477 = kk_function_unbox(_fun_unbox_x4837); /*(i : 4838, tzdelta : 4839) -> 4840*/
  _x5476 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x5477, (_x5477, kk_std_time_instant__instant_box(_b_4841, _ctx), kk_std_time_duration__duration_box(_b_4842, _ctx), _ctx)); /*4840*/
  return kk_std_core_types__tuple2__unbox(_x5476, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5480__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4852;
};
static kk_std_time_instant__instant kk_std_time_calendar__copy_fun5480_1(kk_function_t _fself, kk_std_time_date__date _b_4858, kk_std_time_date__clock _b_4859, kk_std_time_calendar__timezone _b_4860, kk_std_time_instant__timescale _b_4861, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5480_1(kk_box_t _fun_unbox_x4852, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5480__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5480__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5480_1, kk_context());
  _self->_fun_unbox_x4852 = _fun_unbox_x4852;
  return &_self->_base;
}

static kk_std_time_instant__instant kk_std_time_calendar__copy_fun5480_1(kk_function_t _fself, kk_std_time_date__date _b_4858, kk_std_time_date__clock _b_4859, kk_std_time_calendar__timezone _b_4860, kk_std_time_instant__timescale _b_4861, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5480__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5480__t_1*, _fself);
  kk_box_t _fun_unbox_x4852 = _self->_fun_unbox_x4852; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4852);}, {}, _ctx)
  kk_box_t _x5481;
  kk_function_t _x5482 = kk_function_unbox(_fun_unbox_x4852); /*(4853, 4854, 4855, 4856) -> 4857*/
  _x5481 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_box_t, kk_box_t, kk_context_t*), _x5482, (_x5482, kk_std_time_date__date_box(_b_4858, _ctx), kk_std_time_date__clock_box(_b_4859, _ctx), kk_std_time_calendar__timezone_box(_b_4860, _ctx), kk_std_time_instant__timescale_box(_b_4861, _ctx), _ctx)); /*4857*/
  return kk_std_time_instant__instant_unbox(_x5481, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5485__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4865;
};
static kk_std_time_date__date kk_std_time_calendar__copy_fun5485_1(kk_function_t _fself, kk_integer_t _b_4868, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5485_1(kk_box_t _fun_unbox_x4865, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5485__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5485__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5485_1, kk_context());
  _self->_fun_unbox_x4865 = _fun_unbox_x4865;
  return &_self->_base;
}

static kk_std_time_date__date kk_std_time_calendar__copy_fun5485_1(kk_function_t _fself, kk_integer_t _b_4868, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5485__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5485__t_1*, _fself);
  kk_box_t _fun_unbox_x4865 = _self->_fun_unbox_x4865; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4865);}, {}, _ctx)
  kk_box_t _x5486;
  kk_function_t _x5487 = kk_function_unbox(_fun_unbox_x4865); /*(days : 4866) -> 4867*/
  _x5486 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5487, (_x5487, kk_integer_box(_b_4868), _ctx)); /*4867*/
  return kk_std_time_date__date_unbox(_x5486, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5490__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4872;
};
static kk_integer_t kk_std_time_calendar__copy_fun5490_1(kk_function_t _fself, kk_std_time_date__date _b_4875, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5490_1(kk_box_t _fun_unbox_x4872, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5490__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5490__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5490_1, kk_context());
  _self->_fun_unbox_x4872 = _fun_unbox_x4872;
  return &_self->_base;
}

static kk_integer_t kk_std_time_calendar__copy_fun5490_1(kk_function_t _fself, kk_std_time_date__date _b_4875, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5490__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5490__t_1*, _fself);
  kk_box_t _fun_unbox_x4872 = _self->_fun_unbox_x4872; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4872);}, {}, _ctx)
  kk_box_t _x5491;
  kk_function_t _x5492 = kk_function_unbox(_fun_unbox_x4872); /*(date : 4873) -> 4874*/
  _x5491 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5492, (_x5492, kk_std_time_date__date_box(_b_4875, _ctx), _ctx)); /*4874*/
  return kk_integer_unbox(_x5491);
}

kk_std_time_calendar__calendar kk_std_time_calendar__copy_1(kk_std_time_calendar__calendar _this, kk_std_core_types__optional name0, kk_std_core_types__optional long_name0, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_std_core_types__optional instant_to_dc0, kk_std_core_types__optional dc_to_instant0, kk_std_core_types__optional days_to_date0, kk_std_core_types__optional date_to_days0, kk_context_t* _ctx) { /* (calendar, name : optional<string>, long-name : optional<string>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>, instant-to-dc : optional<(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)>, dc-to-instant : optional<(std/time/date/date, std/time/date/clock, timezone, std/time/instant/timescale) -> std/time/instant/instant>, days-to-date : optional<(days : int) -> std/time/date/date>, date-to-days : optional<(date : std/time/date/date) -> int>) -> calendar */ 
  kk_string_t _x5457;
  if (kk_std_core_types__is_Optional(name0)) {
    kk_box_t _box_x4822 = name0._cons.Optional.value;
    kk_string_t _name_314 = kk_string_unbox(_box_x4822);
    kk_string_dup(_name_314);
    kk_std_core_types__optional_drop(name0, _ctx);
    kk_Unit;
    _x5457 = _name_314; /*string*/
    goto _match5458;
  }
  struct kk_std_time_calendar_Calendar* _con5460 = kk_std_time_calendar__as_Calendar(_this);
  kk_string_t _x = _con5460->name;
  kk_string_dup(_x);
  _x5457 = _x; /*string*/
  _match5458: ;
  kk_string_t _x5461;
  if (kk_std_core_types__is_Optional(long_name0)) {
    kk_box_t _box_x4823 = long_name0._cons.Optional.value;
    kk_string_t _long_name_320 = kk_string_unbox(_box_x4823);
    kk_string_dup(_long_name_320);
    kk_std_core_types__optional_drop(long_name0, _ctx);
    kk_Unit;
    _x5461 = _long_name_320; /*string*/
    goto _match5462;
  }
  struct kk_std_time_calendar_Calendar* _con5464 = kk_std_time_calendar__as_Calendar(_this);
  kk_string_t _x0 = _con5464->long_name;
  kk_string_dup(_x0);
  _x5461 = _x0; /*string*/
  _match5462: ;
  kk_string_t _x5465;
  if (kk_std_core_types__is_Optional(month_prefix0)) {
    kk_box_t _box_x4824 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_326 = kk_string_unbox(_box_x4824);
    kk_string_dup(_month_prefix_326);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    kk_Unit;
    _x5465 = _month_prefix_326; /*string*/
    goto _match5466;
  }
  struct kk_std_time_calendar_Calendar* _con5468 = kk_std_time_calendar__as_Calendar(_this);
  kk_string_t _x1 = _con5468->month_prefix;
  kk_string_dup(_x1);
  _x5465 = _x1; /*string*/
  _match5466: ;
  kk_function_t _x5469;
  if (kk_std_core_types__is_Optional(show_era0)) {
    kk_box_t _fun_unbox_x4828 = show_era0._cons.Optional.value;
    _x5469 = kk_std_time_calendar__new_copy_fun5470_1(_fun_unbox_x4828, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    struct kk_std_time_calendar_Calendar* _con5473 = kk_std_time_calendar__as_Calendar(_this);
    kk_function_t _x2 = _con5473->show_era;
    kk_function_dup(_x2);
    _x5469 = _x2; /*(std/time/date/date) -> string*/
  }
  kk_function_t _x5474;
  if (kk_std_core_types__is_Optional(instant_to_dc0)) {
    kk_box_t _fun_unbox_x4837 = instant_to_dc0._cons.Optional.value;
    _x5474 = kk_std_time_calendar__new_copy_fun5475_1(_fun_unbox_x4837, _ctx); /*(std/time/instant/instant, std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)*/
  }
  else {
    struct kk_std_time_calendar_Calendar* _con5478 = kk_std_time_calendar__as_Calendar(_this);
    kk_function_t _x3 = _con5478->instant_to_dc;
    kk_function_dup(_x3);
    _x5474 = _x3; /*(std/time/instant/instant, std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)*/
  }
  kk_function_t _x5479;
  if (kk_std_core_types__is_Optional(dc_to_instant0)) {
    kk_box_t _fun_unbox_x4852 = dc_to_instant0._cons.Optional.value;
    _x5479 = kk_std_time_calendar__new_copy_fun5480_1(_fun_unbox_x4852, _ctx); /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  }
  else {
    struct kk_std_time_calendar_Calendar* _con5483 = kk_std_time_calendar__as_Calendar(_this);
    kk_function_t _x4 = _con5483->dc_to_instant;
    kk_function_dup(_x4);
    _x5479 = _x4; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  }
  kk_function_t _x5484;
  if (kk_std_core_types__is_Optional(days_to_date0)) {
    kk_box_t _fun_unbox_x4865 = days_to_date0._cons.Optional.value;
    _x5484 = kk_std_time_calendar__new_copy_fun5485_1(_fun_unbox_x4865, _ctx); /*(int) -> std/time/date/date*/
  }
  else {
    struct kk_std_time_calendar_Calendar* _con5488 = kk_std_time_calendar__as_Calendar(_this);
    kk_function_t _x5 = _con5488->days_to_date;
    kk_function_dup(_x5);
    _x5484 = _x5; /*(int) -> std/time/date/date*/
  }
  kk_function_t _x5489;
  if (kk_std_core_types__is_Optional(date_to_days0)) {
    kk_box_t _fun_unbox_x4872 = date_to_days0._cons.Optional.value;
    kk_std_time_calendar__calendar_dropn(_this, ((int32_t)8), _ctx);
    kk_Unit;
    _x5489 = kk_std_time_calendar__new_copy_fun5490_1(_fun_unbox_x4872, _ctx); /*(std/time/date/date) -> int*/
  }
  else {
    struct kk_std_time_calendar_Calendar* _con5493 = kk_std_time_calendar__as_Calendar(_this);
    kk_string_t _pat07 = _con5493->name;
    kk_string_t _pat112 = _con5493->long_name;
    kk_string_t _pat28 = _con5493->month_prefix;
    kk_function_t _pat37 = _con5493->show_era;
    kk_function_t _pat47 = _con5493->instant_to_dc;
    kk_function_t _pat57 = _con5493->dc_to_instant;
    kk_function_t _pat67 = _con5493->days_to_date;
    kk_function_t _x6 = _con5493->date_to_days;
    if (kk_likely(kk_std_time_calendar__calendar_is_unique(_this))) {
      kk_function_drop(_pat67, _ctx);
      kk_Unit;
      kk_function_drop(_pat57, _ctx);
      kk_Unit;
      kk_function_drop(_pat47, _ctx);
      kk_Unit;
      kk_function_drop(_pat37, _ctx);
      kk_Unit;
      kk_string_drop(_pat28, _ctx);
      kk_Unit;
      kk_string_drop(_pat112, _ctx);
      kk_Unit;
      kk_string_drop(_pat07, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_time_calendar__calendar_free(_this);
    }
    else {
      kk_function_dup(_x6);
      kk_Unit;
      kk_std_time_calendar__calendar_decref(_this, _ctx);
      kk_Unit;
    }
    _x5489 = _x6; /*(std/time/date/date) -> int*/
  }
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, _x5457, _x5461, _x5465, _x5469, _x5474, _x5479, _x5484, _x5489, _ctx);
}

kk_integer_t kk_std_time_calendar_iso_days_before_year(kk_integer_t year, kk_context_t* _ctx) { /* (year : int) -> int */ 
  kk_integer_t y = kk_integer_sub(year,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t leapdays;
  kk_integer_t _x5494;
  kk_integer_t _x5495;
  kk_integer_t _x5496 = kk_integer_dup(y); /*int*/
  _x5495 = kk_integer_div(_x5496,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t _x5497;
  kk_integer_t _x5498 = kk_integer_dup(y); /*int*/
  _x5497 = kk_integer_div(_x5498,(kk_integer_from_small(100)),kk_context()); /*int*/
  _x5494 = kk_integer_sub(_x5495,_x5497,kk_context()); /*int*/
  kk_integer_t _x5499;
  kk_integer_t _x5500 = kk_integer_dup(y); /*int*/
  _x5499 = kk_integer_div(_x5500,(kk_integer_from_small(400)),kk_context()); /*int*/
  leapdays = kk_integer_add(_x5494,_x5499,kk_context()); /*int*/
  kk_integer_t _x5501 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/
  return kk_integer_add(_x5501,leapdays,kk_context());
}

kk_std_core_types__tuple2_ kk_std_time_calendar_iso_estimate_year(kk_integer_t days, kk_context_t* _ctx) { /* (days : int) -> (int, maybe<int>) */ 
  kk_std_core_types__tuple2_ _match_5384 = kk_integer_div_mod_tuple(days,(kk_integer_from_int(146097, _ctx)),kk_context()); /*(int, int)*/;
  kk_box_t _box_x4876 = _match_5384.fst;
  kk_box_t _box_x4877 = _match_5384.snd;
  kk_integer_t era = kk_integer_unbox(_box_x4876);
  kk_integer_t yoe = kk_integer_unbox(_box_x4877);
  kk_integer_dup(era);
  kk_integer_dup(yoe);
  kk_std_core_types__tuple2__drop(_match_5384, _ctx);
  kk_Unit;
  kk_integer_t _b_4881_4879;
  kk_integer_t _x5504;
  kk_integer_t _x5505 = kk_integer_mul((kk_integer_from_small(400)),era,kk_context()); /*int*/
  _x5504 = kk_integer_add((kk_integer_from_small(1)),_x5505,kk_context()); /*int*/
  kk_integer_t _x5506;
  kk_integer_t _x5507 = kk_integer_mul((kk_integer_from_small(100)),yoe,kk_context()); /*int*/
  _x5506 = kk_integer_div(_x5507,(kk_integer_from_int(36525, _ctx)),kk_context()); /*int*/
  _b_4881_4879 = kk_integer_add(_x5504,_x5506,kk_context()); /*int*/
  kk_std_core_types__maybe _b_4882_4880;
  kk_integer_t _b_4883_4878 = kk_integer_from_small(363); /*int*/;
  _b_4882_4880 = kk_std_core_types__new_Just(kk_integer_box(_b_4883_4878), _ctx); /*maybe<int>*/
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_4881_4879), kk_std_core_types__maybe_box(_b_4882_4880, _ctx), _ctx);
}

kk_std_time_timestamp__timestamp kk_std_time_calendar_dc_timestamp(kk_std_time_date__date d, kk_std_time_date__clock c, kk_function_t date_to_days0, kk_std_core_types__optional has_leap_seconds, kk_context_t* _ctx) { /* (d : std/time/date/date, c : std/time/date/clock, date-to-days : (std/time/date/date) -> int, has-leap-seconds : optional<bool>) -> std/time/timestamp/timestamp */ 
  kk_std_core_types__tuple2_ _match_5382;
  if (kk_std_core_types__is_Optional(has_leap_seconds)) {
    kk_box_t _box_x4884 = has_leap_seconds._cons.Optional.value;
    bool _has_leap_seconds_652 = kk_bool_unbox(_box_x4884);
    kk_std_core_types__optional_drop(has_leap_seconds, _ctx);
    kk_Unit;
    if (_has_leap_seconds_652) {
      kk_std_num_ddouble__ddouble y_4614 = kk_std_num_ddouble_ddouble_int_exp(kk_integer_from_small(60), kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/;
      bool _match_5383;
      kk_std_core_types__order _x5510;
      kk_std_num_ddouble__ddouble _x5511;
      struct kk_std_time_date_Clock* _con5512 = kk_std_time_date__as_Clock(c);
      kk_std_num_ddouble__ddouble _x0 = _con5512->seconds;
      _x5511 = _x0; /*std/num/ddouble/ddouble*/
      _x5510 = kk_std_num_ddouble_compare(_x5511, y_4614, _ctx); /*order*/
      _match_5383 = kk_std_core__lp__eq__eq__4_rp_(_x5510, kk_std_core_types__new_Lt(_ctx), _ctx); /*bool*/
      if (_match_5383) {
        kk_std_num_ddouble__ddouble _b_4897_4885;
        struct kk_std_time_date_Clock* _con5513 = kk_std_time_date__as_Clock(c);
        kk_std_num_ddouble__ddouble _x1 = _con5513->seconds;
        _b_4897_4885 = _x1; /*std/num/ddouble/ddouble*/
        kk_integer_t _b_4898_4886 = kk_integer_from_small(0); /*int*/;
        _match_5382 = kk_std_core_types__new_dash__lp__comma__rp_(kk_std_num_ddouble__ddouble_box(_b_4897_4885, _ctx), kk_integer_box(_b_4898_4886), _ctx); /*(6, 7)*/
        goto _match5508;
      }
      kk_integer_t leap;
      kk_integer_t _x5514;
      kk_std_num_ddouble__ddouble _x5515;
      kk_std_num_ddouble__ddouble _x5516;
      struct kk_std_time_date_Clock* _con5517 = kk_std_time_date__as_Clock(c);
      kk_std_num_ddouble__ddouble _x2 = _con5517->seconds;
      _x5516 = _x2; /*std/num/ddouble/ddouble*/
      _x5515 = kk_std_num_ddouble_floor(_x5516, _ctx); /*std/num/ddouble/ddouble*/
      _x5514 = kk_std_num_ddouble_int(_x5515, kk_std_core_types__new_None(_ctx), _ctx); /*int*/
      leap = kk_integer_sub(_x5514,(kk_integer_from_small(59)),kk_context()); /*int*/
      kk_std_num_ddouble__ddouble _b_4899_4887;
      kk_std_num_ddouble__ddouble y0_4620;
      kk_integer_t _x5518 = kk_integer_dup(leap); /*int*/
      y0_4620 = kk_std_num_ddouble_ddouble_int_exp(_x5518, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
      kk_std_num_ddouble__ddouble _x5519;
      struct kk_std_time_date_Clock* _con5520 = kk_std_time_date__as_Clock(c);
      kk_std_num_ddouble__ddouble _x4 = _con5520->seconds;
      _x5519 = _x4; /*std/num/ddouble/ddouble*/
      kk_std_num_ddouble__ddouble _x5521;
      double _x5522;
      double _x5523;
      double _x3 = y0_4620.hi;
      _x5523 = _x3; /*double*/
      _x5522 = (-_x5523); /*double*/
      double _x5524;
      double _x5525;
      double _x00 = y0_4620.lo;
      _x5525 = _x00; /*double*/
      _x5524 = (-_x5525); /*double*/
      _x5521 = kk_std_num_ddouble__new_Ddouble(_x5522, _x5524, _ctx); /*std/num/ddouble/ddouble*/
      _b_4899_4887 = kk_std_num_ddouble__lp__plus__rp_(_x5519, _x5521, _ctx); /*std/num/ddouble/ddouble*/
      kk_integer_t _b_4900_4888 = leap; /*int*/;
      _match_5382 = kk_std_core_types__new_dash__lp__comma__rp_(kk_std_num_ddouble__ddouble_box(_b_4899_4887, _ctx), kk_integer_box(_b_4900_4888), _ctx); /*(6, 7)*/
      goto _match5508;
    }
    kk_std_num_ddouble__ddouble _b_4901_4889;
    struct kk_std_time_date_Clock* _con5526 = kk_std_time_date__as_Clock(c);
    kk_std_num_ddouble__ddouble _x = _con5526->seconds;
    _b_4901_4889 = _x; /*std/num/ddouble/ddouble*/
    kk_integer_t _b_4902_4890 = kk_integer_from_small(0); /*int*/;
    _match_5382 = kk_std_core_types__new_dash__lp__comma__rp_(kk_std_num_ddouble__ddouble_box(_b_4901_4889, _ctx), kk_integer_box(_b_4902_4890), _ctx); /*(6, 7)*/
    goto _match5508;
  }
  kk_std_num_ddouble__ddouble _b_4903_4891;
  struct kk_std_time_date_Clock* _con5527 = kk_std_time_date__as_Clock(c);
  kk_std_num_ddouble__ddouble _x5 = _con5527->seconds;
  _b_4903_4891 = _x5; /*std/num/ddouble/ddouble*/
  kk_integer_t _b_4904_4892 = kk_integer_from_small(0); /*int*/;
  _match_5382 = kk_std_core_types__new_dash__lp__comma__rp_(kk_std_num_ddouble__ddouble_box(_b_4903_4891, _ctx), kk_integer_box(_b_4904_4892), _ctx); /*(6, 7)*/
  _match5508: ;
  kk_box_t _box_x4893 = _match_5382.fst;
  kk_box_t _box_x4894 = _match_5382.snd;
  kk_std_num_ddouble__ddouble xsecs = kk_std_num_ddouble__ddouble_unbox(_box_x4893, NULL);
  kk_integer_t leap0 = kk_integer_unbox(_box_x4894);
  kk_integer_dup(leap0);
  kk_std_core_types__tuple2__drop(_match_5382, _ctx);
  kk_Unit;
  kk_std_num_ddouble__ddouble secs;
  kk_std_num_ddouble__ddouble _x5530;
  kk_integer_t i1_4623;
  kk_integer_t _x5531;
  kk_integer_t _x5532;
  kk_integer_t _x5533;
  struct kk_std_time_date_Clock* _con5534 = kk_std_time_date__as_Clock(c);
  kk_integer_t _x50 = _con5534->hours;
  kk_std_num_ddouble__ddouble _pat17 = _con5534->seconds;
  kk_integer_dup(_x50);
  _x5533 = _x50; /*int*/
  _x5532 = kk_integer_mul(_x5533,(kk_integer_from_small(60)),kk_context()); /*int*/
  kk_integer_t _x5535;
  struct kk_std_time_date_Clock* _con5536 = kk_std_time_date__as_Clock(c);
  kk_integer_t _pat08 = _con5536->hours;
  kk_integer_t _x6 = _con5536->minutes;
  kk_std_num_ddouble__ddouble _pat19 = _con5536->seconds;
  if (kk_likely(kk_std_time_date__clock_is_unique(c))) {
    kk_integer_drop(_pat08, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_date__clock_free(c);
  }
  else {
    kk_integer_dup(_x6);
    kk_Unit;
    kk_std_time_date__clock_decref(c, _ctx);
    kk_Unit;
  }
  _x5535 = _x6; /*int*/
  _x5531 = kk_integer_add(_x5532,_x5535,kk_context()); /*int*/
  i1_4623 = kk_integer_mul(_x5531,(kk_integer_from_small(60)),kk_context()); /*int*/
  _x5530 = kk_std_num_ddouble_ddouble_int_exp(i1_4623, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
  secs = kk_std_num_ddouble__lp__plus__rp_(_x5530, xsecs, _ctx); /*std/num/ddouble/ddouble*/
  kk_integer_t days = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), date_to_days0, (date_to_days0, d, _ctx)); /*int*/;
  kk_std_core_types__optional _x5537;
  kk_std_num_ddouble__ddouble _b_4905_4895 = secs; /*std/num/ddouble/ddouble*/;
  _x5537 = kk_std_core_types__new_Optional(kk_std_num_ddouble__ddouble_box(_b_4905_4895, _ctx), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5538;
  kk_integer_t _b_4906_4896 = leap0; /*int*/;
  _x5538 = kk_std_core_types__new_Optional(kk_integer_box(_b_4906_4896), _ctx); /*optional<106>*/
  return kk_std_time_timestamp_timestamp_days(days, _x5537, _x5538, _ctx);
}

kk_std_core_types__tuple2_ kk_std_time_calendar_earth_timestamp_to_dc(kk_std_time_timestamp__timestamp t, kk_std_num_ddouble__ddouble tzdelta, kk_function_t days_to_date0, kk_context_t* _ctx) { /* (t : std/time/timestamp/timestamp, tzdelta : std/time/timestamp/timespan, days-to-date : (int) -> std/time/date/date) -> (std/time/date/date, std/time/date/clock) */ 
  kk_std_time_timestamp__timestamp ts_4626;
  kk_std_num_ddouble__ddouble _x5539;
  kk_std_num_ddouble__ddouble _x5540;
  kk_std_num_ddouble__ddouble _x0 = t.since;
  _x5540 = _x0; /*std/time/timestamp/timespan*/
  _x5539 = kk_std_num_ddouble__lp__plus__rp_(_x5540, tzdelta, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x5541;
  kk_std_num_ddouble__ddouble _pat000 = t.since;
  int32_t _x00 = t.leap32;
  _x5541 = _x00; /*int32*/
  ts_4626 = kk_std_time_timestamp__new_Timestamp(_x5539, _x5541, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_core_types__tuple2_ _match_5380 = kk_std_time_timestamp_days_seconds(ts_4626, _ctx); /*(int, std/num/ddouble/ddouble)*/;
  kk_box_t _box_x4907 = _match_5380.fst;
  kk_box_t _box_x4908 = _match_5380.snd;
  kk_integer_t days0 = kk_integer_unbox(_box_x4907);
  kk_std_num_ddouble__ddouble secs = kk_std_num_ddouble__ddouble_unbox(_box_x4908, NULL);
  kk_integer_dup(days0);
  kk_std_core_types__tuple2__drop(_match_5380, _ctx);
  kk_Unit;
  kk_std_time_date__clock _match_5381;
  kk_integer_t _x5544;
  int32_t _x5545;
  kk_std_num_ddouble__ddouble _pat00 = ts_4626.since;
  int32_t _x = ts_4626.leap32;
  _x5545 = _x; /*int32*/
  _x5544 = kk_integer_from_int(_x5545,kk_context()); /*int*/
  _match_5381 = kk_std_time_date_clock_3(secs, _x5544, _ctx); /*std/time/date/clock*/
  struct kk_std_time_date_Clock* _con5546 = kk_std_time_date__as_Clock(_match_5381);
  kk_std_time_date__date _b_4911_4909 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0, (days_to_date0, days0, _ctx)); /*std/time/date/date*/;
  kk_std_time_date__clock _b_4912_4910 = _match_5381; /*std/time/date/clock*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_4911_4909, _ctx), kk_std_time_date__clock_box(_b_4912_4910, _ctx), _ctx);
}
 
// Return the days between two dates interpreted by calendar `cal`.

kk_integer_t kk_std_time_calendar_days_until(kk_std_time_calendar__calendar cal, kk_std_time_date__date d1, kk_std_time_date__date d2, kk_context_t* _ctx) { /* (cal : calendar, d1 : std/time/date/date, d2 : std/time/date/date) -> int */ 
  kk_integer_t _x5547;
  kk_function_t _x5548;
  struct kk_std_time_calendar_Calendar* _con5549 = kk_std_time_calendar__as_Calendar(cal);
  kk_function_t _x0 = _con5549->date_to_days;
  kk_function_dup(_x0);
  _x5548 = _x0; /*(date : std/time/date/date) -> int*/
  _x5547 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x5548, (_x5548, d2, _ctx)); /*int*/
  kk_integer_t _x5550;
  kk_function_t _x5551;
  struct kk_std_time_calendar_Calendar* _con5552 = kk_std_time_calendar__as_Calendar(cal);
  kk_string_t _pat0 = _con5552->name;
  kk_string_t _pat1 = _con5552->long_name;
  kk_string_t _pat2 = _con5552->month_prefix;
  kk_function_t _pat3 = _con5552->show_era;
  kk_function_t _pat4 = _con5552->instant_to_dc;
  kk_function_t _pat5 = _con5552->dc_to_instant;
  kk_function_t _pat6 = _con5552->days_to_date;
  kk_function_t _x = _con5552->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(cal))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(cal);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(cal, _ctx);
    kk_Unit;
  }
  _x5551 = _x; /*(date : std/time/date/date) -> int*/
  _x5550 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x5551, (_x5551, d1, _ctx)); /*int*/
  return kk_integer_sub(_x5547,_x5550,kk_context());
}
 
// Get the current local timezone structure.
// The timezone name is allowed to be the empty string.

kk_std_time_calendar__local_timezone kk_std_time_calendar_local_get_timezone(kk_context_t* _ctx) { /* () -> ndet local-timezone */ 
  return kk_local_get_timezone(kk_context());
}
 
// Return the utc delta in fractional seconds given a local timezone structure
// and time `i` in fractional seconds since the UNIX epoch (1970-01-01).
// Also returns time zone abbreviation.

kk_std_core_types__tuple2_ kk_std_time_calendar_local_utc_delta(kk_std_time_calendar__local_timezone tz, double i, kk_context_t* _ctx) { /* (tz : local-timezone, i : double) -> (double, string) */ 
  return kk_local_get_utc_delta_tuple(tz,i,kk_context());
}
extern kk_std_core_types__maybe kk_std_time_calendar__create_Timezone_fun5554(kk_function_t _fself, kk_std_time_instant__instant _b_4919, kk_context_t* _ctx) {
  struct kk_std_time_calendar__create_Timezone_fun5554__t* _self = kk_function_as(struct kk_std_time_calendar__create_Timezone_fun5554__t*, _fself);
  kk_box_t _fun_unbox_x4916 = _self->_fun_unbox_x4916; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4916);}, {}, _ctx)
  kk_box_t _x5555;
  kk_function_t _x5556 = kk_function_unbox(_fun_unbox_x4916); /*(4917) -> 4918*/
  _x5555 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5556, (_x5556, kk_std_time_instant__instant_box(_b_4919, _ctx), _ctx)); /*4918*/
  return kk_std_core_types__maybe_unbox(_x5555, _ctx);
}
extern kk_std_core_types__maybe kk_std_time_calendar__create_Timezone_fun5557(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Check if two calendars use the same date calculations. (Display of era names etc. may differ)

bool kk_std_time_calendar__lp__eq__eq__rp_(kk_std_time_calendar__calendar c1, kk_std_time_calendar__calendar c2, kk_context_t* _ctx) { /* (c1 : calendar, c2 : calendar) -> bool */ 
  kk_string_t _x5558;
  struct kk_std_time_calendar_Calendar* _con5559 = kk_std_time_calendar__as_Calendar(c1);
  kk_string_t _pat0 = _con5559->name;
  kk_string_t _x = _con5559->long_name;
  kk_string_t _pat1 = _con5559->month_prefix;
  kk_function_t _pat2 = _con5559->show_era;
  kk_function_t _pat3 = _con5559->instant_to_dc;
  kk_function_t _pat4 = _con5559->dc_to_instant;
  kk_function_t _pat5 = _con5559->days_to_date;
  kk_function_t _pat6 = _con5559->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(c1))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_function_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(c1);
  }
  else {
    kk_string_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(c1, _ctx);
    kk_Unit;
  }
  _x5558 = _x; /*string*/
  kk_string_t _x5560;
  struct kk_std_time_calendar_Calendar* _con5561 = kk_std_time_calendar__as_Calendar(c2);
  kk_string_t _pat00 = _con5561->name;
  kk_string_t _x0 = _con5561->long_name;
  kk_string_t _pat10 = _con5561->month_prefix;
  kk_function_t _pat20 = _con5561->show_era;
  kk_function_t _pat30 = _con5561->instant_to_dc;
  kk_function_t _pat40 = _con5561->dc_to_instant;
  kk_function_t _pat50 = _con5561->days_to_date;
  kk_function_t _pat60 = _con5561->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(c2))) {
    kk_function_drop(_pat60, _ctx);
    kk_Unit;
    kk_function_drop(_pat50, _ctx);
    kk_Unit;
    kk_function_drop(_pat40, _ctx);
    kk_Unit;
    kk_function_drop(_pat30, _ctx);
    kk_Unit;
    kk_function_drop(_pat20, _ctx);
    kk_Unit;
    kk_string_drop(_pat10, _ctx);
    kk_Unit;
    kk_string_drop(_pat00, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(c2);
  }
  else {
    kk_string_dup(_x0);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(c2, _ctx);
    kk_Unit;
  }
  _x5560 = _x0; /*string*/
  return kk_string_is_eq(_x5558,_x5560,kk_context());
}

bool kk_std_time_calendar_iso_is_leap(kk_integer_t year, kk_context_t* _ctx) { /* (year : int) -> bool */ 
  bool _match_5369;
  kk_integer_t _borrowedArg_5378;
  kk_integer_t _x5564 = kk_integer_dup(year); /*int*/
  _borrowedArg_5378 = kk_integer_mod(_x5564,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t _borrowedArg_5377 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_5379 = kk_integer_eq_borrow(_borrowedArg_5378,_borrowedArg_5377,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5378, _ctx);
  kk_integer_drop(_borrowedArg_5377, _ctx);
  _match_5369 = _borrowApp_5379; /*bool*/
  if (_match_5369) {
    bool _match_5370;
    kk_integer_t _borrowedArg_5375;
    kk_integer_t _x5565 = kk_integer_dup(year); /*int*/
    _borrowedArg_5375 = kk_integer_mod(_x5565,(kk_integer_from_small(100)),kk_context()); /*int*/
    kk_integer_t _borrowedArg_5374 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_5376 = kk_integer_neq_borrow(_borrowedArg_5375,_borrowedArg_5374,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_5375, _ctx);
    kk_integer_drop(_borrowedArg_5374, _ctx);
    _match_5370 = _borrowApp_5376; /*bool*/
    if (_match_5370) {
      kk_integer_drop(year, _ctx);
      return true;
    }
    kk_integer_t _borrowedArg_5372 = kk_integer_mod(year,(kk_integer_from_small(400)),kk_context()); /*int*/;
    kk_integer_t _borrowedArg_5371 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_5373 = kk_integer_eq_borrow(_borrowedArg_5372,_borrowedArg_5371,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_5372, _ctx);
    kk_integer_drop(_borrowedArg_5371, _ctx);
    return _borrowApp_5373;
  }
  kk_integer_drop(year, _ctx);
  return false;
}

kk_integer_t kk_std_time_calendar_iso_days_before_month(kk_integer_t year, kk_integer_t month, kk_context_t* _ctx) { /* (year : int, month : int) -> int */ 
  bool is_before_march_4635;
  kk_integer_t _borrowedArg_5366 = kk_integer_from_small(2); /*int*/;
  bool _borrowApp_5367 = kk_integer_lte_borrow(month,_borrowedArg_5366,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5366, _ctx);
  is_before_march_4635 = _borrowApp_5367; /*bool*/
  kk_integer_t adj;
  if (is_before_march_4635) {
    kk_integer_drop(year, _ctx);
    adj = kk_integer_from_small(0); /*int*/
  }
  else {
    bool _match_5365 = kk_std_time_calendar_iso_is_leap(year, _ctx); /*bool*/;
    if (_match_5365) {
      adj = kk_integer_from_small(1); /*int*/
    }
    else {
      adj = kk_integer_from_small(2); /*int*/
    }
  }
  kk_integer_t _x5566;
  kk_integer_t _x5567;
  kk_integer_t _x5568 = kk_integer_mul((kk_integer_from_small(367)),month,kk_context()); /*int*/
  _x5567 = kk_integer_sub(_x5568,(kk_integer_from_small(362)),kk_context()); /*int*/
  _x5566 = kk_integer_div(_x5567,(kk_integer_from_small(12)),kk_context()); /*int*/
  return kk_integer_sub(_x5566,adj,kk_context());
}

kk_integer_t kk_std_time_calendar_iso_doy_to_month(kk_integer_t year, kk_integer_t doy, kk_context_t* _ctx) { /* (year : int, doy : int) -> int */ 
  bool is_before_march_4637;
  kk_integer_t _borrowedArg_5363 = kk_integer_from_small(58); /*int*/;
  bool _borrowApp_5364 = kk_integer_lte_borrow(doy,_borrowedArg_5363,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5363, _ctx);
  is_before_march_4637 = _borrowApp_5364; /*bool*/
  kk_integer_t adj;
  if (is_before_march_4637) {
    kk_integer_drop(year, _ctx);
    adj = kk_integer_from_small(0); /*int*/
  }
  else {
    bool _match_5362 = kk_std_time_calendar_iso_is_leap(year, _ctx); /*bool*/;
    if (_match_5362) {
      adj = kk_integer_from_small(1); /*int*/
    }
    else {
      adj = kk_integer_from_small(2); /*int*/
    }
  }
  kk_integer_t _x5569;
  kk_integer_t _x5570;
  kk_integer_t _x5571 = kk_integer_add(doy,adj,kk_context()); /*int*/
  _x5570 = kk_integer_mul((kk_integer_from_small(12)),_x5571,kk_context()); /*int*/
  _x5569 = kk_integer_add(_x5570,(kk_integer_from_small(373)),kk_context()); /*int*/
  return kk_integer_div(_x5569,(kk_integer_from_small(367)),kk_context());
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5572__t_1 {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5572_1(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5572_1(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5572__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5572__t_1, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5572_1, kk_context());
  _self->abbrv = abbrv;
  _self->delta = delta;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5572_1(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5572__t_1* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5572__t_1*, _fself);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv);kk_std_time_duration__duration_dup(delta);}, {}, _ctx)
  kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_4924_4921 = delta; /*std/time/duration/duration*/;
  kk_string_t _b_4925_4922;
  if (kk_std_core_types__is_Optional(abbrv)) {
    kk_box_t _box_x4920 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_2653 = kk_string_unbox(_box_x4920);
    kk_string_dup(_abbrv_2653);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    kk_Unit;
    _b_4925_4922 = _abbrv_2653; /*string*/
    goto _match5573;
  }
  _b_4925_4922 = kk_string_empty(); /*string*/
  _match5573: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_4924_4921, _ctx), kk_string_box(_b_4925_4922), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5576__t_1 {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5576_1(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5576_1(kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5576__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5576__t_1, 1, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5576_1, kk_context());
  _self->delta = delta;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5576_1(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5576__t_1* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5576__t_1*, _fself);
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta);}, {}, _ctx)
  bool _match_5361;
  double _x5577;
  kk_std_num_ddouble__ddouble _x0 = delta.secs;
  double _x = _x0.hi;
  _x5577 = _x; /*double*/
  _match_5361 = (_x5577 == (0.0)); /*bool*/
  if (_match_5361) {
    kk_std_time_instant__instant _b_4926_4923 = i00; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_4926_4923, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i00, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_fixed_1(kk_string_t name0, kk_std_time_duration__duration delta, kk_std_core_types__optional abbrv, kk_context_t* _ctx) { /* (name : string, delta : std/time/duration/duration, abbrv : optional<string>) -> timezone */ 
  return kk_std_time_calendar__new_Timezone(name0, kk_std_time_calendar_new_tz_fixed_fun5572_1(abbrv, delta, _ctx), kk_std_time_calendar_new_tz_fixed_fun5576_1(delta, _ctx), _ctx);
}
 
// The standard UTC time zone with a 0 delta.


// lift anonymous function
struct kk_std_time_calendar_tz_utc_fun5580__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_utc_fun5580(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_utc_fun5580(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_utc_fun5580, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_utc_fun5580(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_4931_4928 = kk_std_time_duration_zero; /*std/time/duration/duration*/;
  kk_string_t _b_4932_4929;
  kk_std_core_types__optional _match_5360 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5360)) {
    kk_box_t _box_x4927 = _match_5360._cons.Optional.value;
    kk_string_t _abbrv_2653 = kk_string_unbox(_box_x4927);
    kk_string_dup(_abbrv_2653);
    kk_std_core_types__optional_drop(_match_5360, _ctx);
    kk_Unit;
    _b_4932_4929 = _abbrv_2653; /*string*/
    goto _match5581;
  }
  _b_4932_4929 = kk_string_empty(); /*string*/
  _match5581: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_4931_4928, _ctx), kk_string_box(_b_4932_4929), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_utc_fun5584__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_utc_fun5584(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_utc_fun5584(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_utc_fun5584, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_utc_fun5584(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  bool _match_5359;
  double _x5585;
  kk_std_time_duration__duration _x5586 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  kk_std_num_ddouble__ddouble _x0 = _x5586.secs;
  double _x = _x0.hi;
  _x5585 = _x; /*double*/
  _match_5359 = (_x5585 == (0.0)); /*bool*/
  if (_match_5359) {
    kk_std_time_instant__instant _b_4933_4930 = i00; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_4933_4930, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i00, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_utc;
 
// Create a time zone with a fixed number of hours and optional minutes (=`0`) difference from UTC/GMT.
// The time zone name and abbreviation is optional, and the default name
// will reflect the offset from UTC, for example `"UTC+1:20"`, or `"UTC-8"`.
// The default abbreviation is the empty string.
// The sign of the minutes will be matched to the hours (unless `hours` is `0`).
// For example, `tz-fixed(-1,30)` gives a time zone offset of -90 minutes, "UTC-1:30".
// Returns `tz-utc` if both `hours` and `mins` are zero.


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5591__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5591(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5591(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_fixed_fun5591, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5591(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_4967_4936 = kk_std_time_duration_zero; /*std/time/duration/duration*/;
  kk_string_t _b_4968_4937;
  kk_std_core_types__optional _match_5358 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5358)) {
    kk_box_t _box_x4935 = _match_5358._cons.Optional.value;
    kk_string_t _abbrv_2653 = kk_string_unbox(_box_x4935);
    kk_string_dup(_abbrv_2653);
    kk_std_core_types__optional_drop(_match_5358, _ctx);
    kk_Unit;
    _b_4968_4937 = _abbrv_2653; /*string*/
    goto _match5592;
  }
  _b_4968_4937 = kk_string_empty(); /*string*/
  _match5592: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_4967_4936, _ctx), kk_string_box(_b_4968_4937), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5595__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5595(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5595(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_fixed_fun5595, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5595(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  bool _match_5357;
  double _x5596;
  kk_std_time_duration__duration _x5597 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  kk_std_num_ddouble__ddouble _x0 = _x5597.secs;
  double _x = _x0.hi;
  _x5596 = _x; /*double*/
  _match_5357 = (_x5596 == (0.0)); /*bool*/
  if (_match_5357) {
    kk_std_time_instant__instant _b_4969_4938 = i00; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_4969_4938, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i00, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5640__t {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta0;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5640(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5640(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5640__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5640__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5640, kk_context());
  _self->abbrv = abbrv;
  _self->delta0 = delta0;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5640(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5640__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5640__t*, _fself);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta0 = _self->delta0; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv);kk_std_time_duration__duration_dup(delta0);}, {}, _ctx)
  kk_std_time_instant__instant_dropn(i01, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_4970_4950 = delta0; /*std/time/duration/duration*/;
  kk_string_t _b_4971_4951;
  if (kk_std_core_types__is_Optional(abbrv)) {
    kk_box_t _box_x4949 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_2271 = kk_string_unbox(_box_x4949);
    kk_string_dup(_abbrv_2271);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    kk_Unit;
    _b_4971_4951 = _abbrv_2271; /*string*/
    goto _match5641;
  }
  _b_4971_4951 = kk_string_empty(); /*string*/
  _match5641: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_4970_4950, _ctx), kk_string_box(_b_4971_4951), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5644__t {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta0;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5644(kk_function_t _fself, kk_std_time_instant__instant i000, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5644(kk_std_time_duration__duration delta0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5644__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5644__t, 1, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5644, kk_context());
  _self->delta0 = delta0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5644(kk_function_t _fself, kk_std_time_instant__instant i000, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5644__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5644__t*, _fself);
  kk_std_time_duration__duration delta0 = _self->delta0; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta0);}, {}, _ctx)
  bool _match_5344;
  double _x5645;
  kk_std_num_ddouble__ddouble _x00 = delta0.secs;
  double _x1 = _x00.hi;
  _x5645 = _x1; /*double*/
  _match_5344 = (_x5645 == (0.0)); /*bool*/
  if (_match_5344) {
    kk_std_time_instant__instant _b_4972_4952 = i000; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_4972_4952, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i000, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5688__t {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta00;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5688(kk_function_t _fself, kk_std_time_instant__instant i02, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5688(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5688__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5688__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5688, kk_context());
  _self->abbrv = abbrv;
  _self->delta00 = delta00;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5688(kk_function_t _fself, kk_std_time_instant__instant i02, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5688__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5688__t*, _fself);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta00 = _self->delta00; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv);kk_std_time_duration__duration_dup(delta00);}, {}, _ctx)
  kk_std_time_instant__instant_dropn(i02, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_4973_4964 = delta00; /*std/time/duration/duration*/;
  kk_string_t _b_4974_4965;
  if (kk_std_core_types__is_Optional(abbrv)) {
    kk_box_t _box_x4963 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_22710 = kk_string_unbox(_box_x4963);
    kk_string_dup(_abbrv_22710);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    kk_Unit;
    _b_4974_4965 = _abbrv_22710; /*string*/
    goto _match5689;
  }
  _b_4974_4965 = kk_string_empty(); /*string*/
  _match5689: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_4973_4964, _ctx), kk_string_box(_b_4974_4965), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5692__t {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta00;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5692(kk_function_t _fself, kk_std_time_instant__instant i001, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5692(kk_std_time_duration__duration delta00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5692__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5692__t, 1, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5692, kk_context());
  _self->delta00 = delta00;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5692(kk_function_t _fself, kk_std_time_instant__instant i001, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5692__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5692__t*, _fself);
  kk_std_time_duration__duration delta00 = _self->delta00; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta00);}, {}, _ctx)
  bool _match_5330;
  double _x5693;
  kk_std_num_ddouble__ddouble _x01 = delta00.secs;
  double _x2 = _x01.hi;
  _x5693 = _x2; /*double*/
  _match_5330 = (_x5693 == (0.0)); /*bool*/
  if (_match_5330) {
    kk_std_time_instant__instant _b_4975_4966 = i001; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_4975_4966, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i001, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_fixed(kk_integer_t hours, kk_std_core_types__optional mins, kk_std_core_types__optional name0, kk_std_core_types__optional abbrv, kk_std_core_types__optional hourwidth, kk_context_t* _ctx) { /* (hours : int, mins : optional<int>, name : optional<string>, abbrv : optional<string>, hourwidth : optional<int>) -> timezone */ 
  kk_integer_t _mins_2262;
  if (kk_std_core_types__is_Optional(mins)) {
    kk_box_t _box_x4934 = mins._cons.Optional.value;
    kk_integer_t _mins_2263 = kk_integer_unbox(_box_x4934);
    kk_integer_dup(_mins_2263);
    kk_std_core_types__optional_drop(mins, _ctx);
    kk_Unit;
    _mins_2262 = _mins_2263; /*int*/
    goto _match5587;
  }
  _mins_2262 = kk_integer_from_small(0); /*int*/
  _match5587: ;
  bool _match_5329 = kk_integer_is_zero_borrow(hours); /*bool*/;
  if (_match_5329) {
    bool _match_5343 = kk_integer_is_zero_borrow(_mins_2262); /*bool*/;
    if (_match_5343) {
      kk_integer_drop(_mins_2262, _ctx);
      kk_std_core_types__optional_drop(abbrv, _ctx);
      kk_integer_drop(hours, _ctx);
      kk_std_core_types__optional_drop(hourwidth, _ctx);
      kk_std_core_types__optional_drop(name0, _ctx);
      kk_string_t _x5589;
      kk_define_string_literal(, _s5590, 3, "UTC")
      _x5589 = kk_string_dup(_s5590); /*string*/
      return kk_std_time_calendar__new_Timezone(_x5589, kk_std_time_calendar_new_tz_fixed_fun5591(_ctx), kk_std_time_calendar_new_tz_fixed_fun5595(_ctx), _ctx);
    }
    kk_integer_t xmins;
    bool _match_5355;
    kk_integer_t _x5598 = kk_integer_dup(hours); /*int*/
    _match_5355 = kk_std_core_is_pos_2(_x5598, _ctx); /*bool*/
    if (_match_5355) {
      kk_integer_t _x5599 = kk_integer_dup(_mins_2262); /*int*/
      xmins = kk_integer_abs(_x5599,kk_context()); /*int*/
    }
    else {
      bool _match_5356;
      kk_integer_t _x5600 = kk_integer_dup(hours); /*int*/
      _match_5356 = kk_std_core_is_neg_2(_x5600, _ctx); /*bool*/
      if (_match_5356) {
        kk_integer_t i_4643;
        kk_integer_t _x5601 = kk_integer_dup(_mins_2262); /*int*/
        i_4643 = kk_integer_abs(_x5601,kk_context()); /*int*/
        xmins = kk_integer_neg(i_4643,kk_context()); /*int*/
      }
      else {
        xmins = kk_integer_dup(_mins_2262); /*int*/
      }
    }
    kk_integer_t secs;
    kk_integer_t _x5602;
    kk_integer_t _x5603 = kk_integer_dup(hours); /*int*/
    _x5602 = kk_integer_mul(_x5603,(kk_integer_from_int(3600, _ctx)),kk_context()); /*int*/
    kk_integer_t _x5604 = kk_integer_mul(xmins,(kk_integer_from_small(60)),kk_context()); /*int*/
    secs = kk_integer_add(_x5602,_x5604,kk_context()); /*int*/
    kk_string_t y_4645;
    kk_string_t _x5605;
    kk_define_string_literal(, _s5606, 3, "UTC")
    _x5605 = kk_string_dup(_s5606); /*string*/
    kk_string_t _x5607;
    kk_string_t _x5608;
    bool _match_5354;
    kk_integer_t _x5609 = kk_integer_dup(secs); /*int*/
    _match_5354 = kk_std_core_is_neg_2(_x5609, _ctx); /*bool*/
    if (_match_5354) {
      kk_define_string_literal(, _s5610, 1, "-")
      _x5608 = kk_string_dup(_s5610); /*string*/
    }
    else {
      kk_define_string_literal(, _s5611, 1, "+")
      _x5608 = kk_string_dup(_s5611); /*string*/
    }
    kk_string_t _x5612;
    kk_string_t _x5613;
    kk_string_t _ownedArg_5352;
    kk_integer_t _x5614 = kk_integer_abs(hours,kk_context()); /*int*/
    _ownedArg_5352 = kk_std_core_show(_x5614, _ctx); /*string*/
    kk_integer_t _borrowedArg_5351;
    if (kk_std_core_types__is_Optional(hourwidth)) {
      kk_box_t _box_x4939 = hourwidth._cons.Optional.value;
      kk_integer_t _hourwidth_2275 = kk_integer_unbox(_box_x4939);
      kk_integer_dup(_hourwidth_2275);
      kk_std_core_types__optional_drop(hourwidth, _ctx);
      kk_Unit;
      _borrowedArg_5351 = _hourwidth_2275; /*int*/
      goto _match5615;
    }
    _borrowedArg_5351 = kk_integer_from_small(1); /*int*/
    _match5615: ;
    kk_string_t _borrowApp_5353;
    kk_std_core_types__optional _x5617 = kk_std_core_types__new_Optional(kk_char_box('0', _ctx), _ctx); /*optional<106>*/
    _borrowApp_5353 = kk_std_core_pad_left(_ownedArg_5352, _borrowedArg_5351, _x5617, _ctx); /*string*/
    kk_integer_drop(_borrowedArg_5351, _ctx);
    _x5613 = _borrowApp_5353; /*string*/
    kk_string_t _x5618;
    bool _match_5347 = kk_integer_is_zero_borrow(_mins_2262); /*bool*/;
    if (_match_5347) {
      kk_integer_drop(_mins_2262, _ctx);
      _x5618 = kk_string_empty(); /*string*/
    }
    else {
      kk_string_t _x5620;
      kk_define_string_literal(, _s5621, 1, ":")
      _x5620 = kk_string_dup(_s5621); /*string*/
      kk_string_t _x5622;
      kk_string_t _ownedArg_5349;
      kk_integer_t _x5623 = kk_integer_abs(_mins_2262,kk_context()); /*int*/
      _ownedArg_5349 = kk_std_core_show(_x5623, _ctx); /*string*/
      kk_integer_t _borrowedArg_5348 = kk_integer_from_small(2); /*int*/;
      kk_string_t _borrowApp_5350;
      kk_std_core_types__optional _x5624 = kk_std_core_types__new_Optional(kk_char_box('0', _ctx), _ctx); /*optional<106>*/
      _borrowApp_5350 = kk_std_core_pad_left(_ownedArg_5349, _borrowedArg_5348, _x5624, _ctx); /*string*/
      kk_integer_drop(_borrowedArg_5348, _ctx);
      _x5622 = _borrowApp_5350; /*string*/
      _x5618 = kk_std_core__lp__plus__plus__1_rp_(_x5620, _x5622, _ctx); /*string*/
    }
    _x5612 = kk_std_core__lp__plus__plus__1_rp_(_x5613, _x5618, _ctx); /*string*/
    _x5607 = kk_std_core__lp__plus__plus__1_rp_(_x5608, _x5612, _ctx); /*string*/
    y_4645 = kk_std_core__lp__plus__plus__1_rp_(_x5605, _x5607, _ctx); /*string*/
    kk_string_t tzonename;
    bool _match_5346;
    kk_string_t _x5625;
    if (kk_std_core_types__is_Optional(name0)) {
      kk_box_t _box_x4944 = name0._cons.Optional.value;
      kk_string_t _name_2267 = kk_string_unbox(_box_x4944);
      kk_string_dup(_name_2267);
      _x5625 = _name_2267; /*string*/
      goto _match5626;
    }
    _x5625 = kk_string_empty(); /*string*/
    _match5626: ;
    kk_string_t _x5629 = kk_string_empty(); /*string*/
    _match_5346 = kk_string_is_eq(_x5625,_x5629,kk_context()); /*bool*/
    if (_match_5346) {
      kk_std_core_types__optional_drop(name0, _ctx);
      tzonename = y_4645; /*string*/
    }
    else {
      kk_string_drop(y_4645, _ctx);
      if (kk_std_core_types__is_Optional(name0)) {
        kk_box_t _box_x4945 = name0._cons.Optional.value;
        kk_string_t _name_22670 = kk_string_unbox(_box_x4945);
        kk_string_dup(_name_22670);
        kk_std_core_types__optional_drop(name0, _ctx);
        kk_Unit;
        tzonename = _name_22670; /*string*/
        goto _match5631;
      }
      tzonename = kk_string_empty(); /*string*/
      _match5631: ;
    }
    kk_std_time_duration__duration delta0;
    kk_std_num_ddouble__ddouble _x5634;
    kk_std_core_types__optional _x5635;
    kk_box_t _x5636;
    double _x5637;
    kk_std_core_types__optional _match_5345 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5345)) {
      kk_box_t _box_x4946 = _match_5345._cons.Optional.value;
      double _frac_162 = kk_double_unbox(_box_x4946, NULL);
      kk_std_core_types__optional_drop(_match_5345, _ctx);
      kk_Unit;
      _x5637 = _frac_162; /*double*/
      goto _match5638;
    }
    _x5637 = 0.0; /*double*/
    _match5638: ;
    _x5636 = kk_double_box(_x5637, _ctx); /*106*/
    _x5635 = kk_std_core_types__new_Optional(_x5636, _ctx); /*optional<106>*/
    _x5634 = kk_std_time_timestamp_timespan(secs, _x5635, _ctx); /*std/time/timestamp/timespan*/
    delta0 = kk_std_time_duration__new_Duration(_x5634, _ctx); /*std/time/duration/duration*/
    return kk_std_time_calendar__new_Timezone(tzonename, kk_std_time_calendar_new_tz_fixed_fun5640(abbrv, delta0, _ctx), kk_std_time_calendar_new_tz_fixed_fun5644(delta0, _ctx), _ctx);
  }
  kk_integer_t xmins0;
  bool _match_5341;
  kk_integer_t _x5646 = kk_integer_dup(hours); /*int*/
  _match_5341 = kk_std_core_is_pos_2(_x5646, _ctx); /*bool*/
  if (_match_5341) {
    kk_integer_t _x5647 = kk_integer_dup(_mins_2262); /*int*/
    xmins0 = kk_integer_abs(_x5647,kk_context()); /*int*/
  }
  else {
    bool _match_5342;
    kk_integer_t _x5648 = kk_integer_dup(hours); /*int*/
    _match_5342 = kk_std_core_is_neg_2(_x5648, _ctx); /*bool*/
    if (_match_5342) {
      kk_integer_t i0_4648;
      kk_integer_t _x5649 = kk_integer_dup(_mins_2262); /*int*/
      i0_4648 = kk_integer_abs(_x5649,kk_context()); /*int*/
      xmins0 = kk_integer_neg(i0_4648,kk_context()); /*int*/
    }
    else {
      xmins0 = kk_integer_dup(_mins_2262); /*int*/
    }
  }
  kk_integer_t secs1;
  kk_integer_t _x5650;
  kk_integer_t _x5651 = kk_integer_dup(hours); /*int*/
  _x5650 = kk_integer_mul(_x5651,(kk_integer_from_int(3600, _ctx)),kk_context()); /*int*/
  kk_integer_t _x5652 = kk_integer_mul(xmins0,(kk_integer_from_small(60)),kk_context()); /*int*/
  secs1 = kk_integer_add(_x5650,_x5652,kk_context()); /*int*/
  kk_string_t y0_4650;
  kk_string_t _x5653;
  kk_define_string_literal(, _s5654, 3, "UTC")
  _x5653 = kk_string_dup(_s5654); /*string*/
  kk_string_t _x5655;
  kk_string_t _x5656;
  bool _match_5340;
  kk_integer_t _x5657 = kk_integer_dup(secs1); /*int*/
  _match_5340 = kk_std_core_is_neg_2(_x5657, _ctx); /*bool*/
  if (_match_5340) {
    kk_define_string_literal(, _s5658, 1, "-")
    _x5656 = kk_string_dup(_s5658); /*string*/
  }
  else {
    kk_define_string_literal(, _s5659, 1, "+")
    _x5656 = kk_string_dup(_s5659); /*string*/
  }
  kk_string_t _x5660;
  kk_string_t _x5661;
  kk_string_t _ownedArg_5338;
  kk_integer_t _x5662 = kk_integer_abs(hours,kk_context()); /*int*/
  _ownedArg_5338 = kk_std_core_show(_x5662, _ctx); /*string*/
  kk_integer_t _borrowedArg_5337;
  if (kk_std_core_types__is_Optional(hourwidth)) {
    kk_box_t _box_x4953 = hourwidth._cons.Optional.value;
    kk_integer_t _hourwidth_22750 = kk_integer_unbox(_box_x4953);
    kk_integer_dup(_hourwidth_22750);
    kk_std_core_types__optional_drop(hourwidth, _ctx);
    kk_Unit;
    _borrowedArg_5337 = _hourwidth_22750; /*int*/
    goto _match5663;
  }
  _borrowedArg_5337 = kk_integer_from_small(1); /*int*/
  _match5663: ;
  kk_string_t _borrowApp_5339;
  kk_std_core_types__optional _x5665 = kk_std_core_types__new_Optional(kk_char_box('0', _ctx), _ctx); /*optional<106>*/
  _borrowApp_5339 = kk_std_core_pad_left(_ownedArg_5338, _borrowedArg_5337, _x5665, _ctx); /*string*/
  kk_integer_drop(_borrowedArg_5337, _ctx);
  _x5661 = _borrowApp_5339; /*string*/
  kk_string_t _x5666;
  bool _match_5333 = kk_integer_is_zero_borrow(_mins_2262); /*bool*/;
  if (_match_5333) {
    kk_integer_drop(_mins_2262, _ctx);
    _x5666 = kk_string_empty(); /*string*/
  }
  else {
    kk_string_t _x5668;
    kk_define_string_literal(, _s5669, 1, ":")
    _x5668 = kk_string_dup(_s5669); /*string*/
    kk_string_t _x5670;
    kk_string_t _ownedArg_5335;
    kk_integer_t _x5671 = kk_integer_abs(_mins_2262,kk_context()); /*int*/
    _ownedArg_5335 = kk_std_core_show(_x5671, _ctx); /*string*/
    kk_integer_t _borrowedArg_5334 = kk_integer_from_small(2); /*int*/;
    kk_string_t _borrowApp_5336;
    kk_std_core_types__optional _x5672 = kk_std_core_types__new_Optional(kk_char_box('0', _ctx), _ctx); /*optional<106>*/
    _borrowApp_5336 = kk_std_core_pad_left(_ownedArg_5335, _borrowedArg_5334, _x5672, _ctx); /*string*/
    kk_integer_drop(_borrowedArg_5334, _ctx);
    _x5670 = _borrowApp_5336; /*string*/
    _x5666 = kk_std_core__lp__plus__plus__1_rp_(_x5668, _x5670, _ctx); /*string*/
  }
  _x5660 = kk_std_core__lp__plus__plus__1_rp_(_x5661, _x5666, _ctx); /*string*/
  _x5655 = kk_std_core__lp__plus__plus__1_rp_(_x5656, _x5660, _ctx); /*string*/
  y0_4650 = kk_std_core__lp__plus__plus__1_rp_(_x5653, _x5655, _ctx); /*string*/
  kk_string_t tzonename0;
  bool _match_5332;
  kk_string_t _x5673;
  if (kk_std_core_types__is_Optional(name0)) {
    kk_box_t _box_x4958 = name0._cons.Optional.value;
    kk_string_t _name_226700 = kk_string_unbox(_box_x4958);
    kk_string_dup(_name_226700);
    _x5673 = _name_226700; /*string*/
    goto _match5674;
  }
  _x5673 = kk_string_empty(); /*string*/
  _match5674: ;
  kk_string_t _x5677 = kk_string_empty(); /*string*/
  _match_5332 = kk_string_is_eq(_x5673,_x5677,kk_context()); /*bool*/
  if (_match_5332) {
    kk_std_core_types__optional_drop(name0, _ctx);
    tzonename0 = y0_4650; /*string*/
  }
  else {
    kk_string_drop(y0_4650, _ctx);
    if (kk_std_core_types__is_Optional(name0)) {
      kk_box_t _box_x4959 = name0._cons.Optional.value;
      kk_string_t _name_226701 = kk_string_unbox(_box_x4959);
      kk_string_dup(_name_226701);
      kk_std_core_types__optional_drop(name0, _ctx);
      kk_Unit;
      tzonename0 = _name_226701; /*string*/
      goto _match5679;
    }
    tzonename0 = kk_string_empty(); /*string*/
    _match5679: ;
  }
  kk_std_time_duration__duration delta00;
  kk_std_num_ddouble__ddouble _x5682;
  kk_std_core_types__optional _x5683;
  kk_box_t _x5684;
  double _x5685;
  kk_std_core_types__optional _match_5331 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5331)) {
    kk_box_t _box_x4960 = _match_5331._cons.Optional.value;
    double _frac_1620 = kk_double_unbox(_box_x4960, NULL);
    kk_std_core_types__optional_drop(_match_5331, _ctx);
    kk_Unit;
    _x5685 = _frac_1620; /*double*/
    goto _match5686;
  }
  _x5685 = 0.0; /*double*/
  _match5686: ;
  _x5684 = kk_double_box(_x5685, _ctx); /*106*/
  _x5683 = kk_std_core_types__new_Optional(_x5684, _ctx); /*optional<106>*/
  _x5682 = kk_std_time_timestamp_timespan(secs1, _x5683, _ctx); /*std/time/timestamp/timespan*/
  delta00 = kk_std_time_duration__new_Duration(_x5682, _ctx); /*std/time/duration/duration*/
  return kk_std_time_calendar__new_Timezone(tzonename0, kk_std_time_calendar_new_tz_fixed_fun5688(abbrv, delta00, _ctx), kk_std_time_calendar_new_tz_fixed_fun5692(delta00, _ctx), _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_earth_timestamp_to_instant(kk_std_time_timestamp__timestamp t, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts, kk_context_t* _ctx) { /* (t : std/time/timestamp/timestamp, tz : timezone, ts : std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_std_time_instant__instant i;
  kk_std_time_instant__timescale _x5697 = kk_std_time_instant__timescale_dup(ts); /*std/time/instant/timescale*/
  i = kk_std_time_instant__new_Instant(kk_reuse_null, t, _x5697, _ctx); /*std/time/instant/instant*/
  bool _match_5326;
  kk_string_t _x5698;
  kk_string_t _x1 = tz.name;
  kk_string_dup(_x1);
  _x5698 = _x1; /*string*/
  kk_string_t _x5699;
  kk_define_string_literal(, _s5700, 3, "UTC")
  _x5699 = kk_string_dup(_s5700); /*string*/
  _match_5326 = kk_string_is_eq(_x5698,_x5699,kk_context()); /*bool*/
  if (_match_5326) {
    kk_std_time_instant__timescale_dropn(ts, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    return i;
  }
  kk_std_core_types__maybe _match_5327;
  kk_function_t _x5702;
  kk_function_t _x = tz.utc_inverse;
  kk_function_dup(_x);
  _x5702 = _x; /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  kk_std_time_instant__instant _x5701 = kk_std_time_instant__instant_dup(i); /*std/time/instant/instant*/
  _match_5327 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x5702, (_x5702, _x5701, _ctx)); /*maybe<std/time/instant/instant>*/
  if (kk_std_core_types__is_Just(_match_5327)) {
    kk_box_t _box_x4976 = _match_5327._cons.Just.value;
    kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x4976, NULL);
    struct kk_std_time_instant_Instant* _con5704 = kk_std_time_instant__as_Instant(inv);
    kk_std_time_instant__instant_dropn(i, ((int32_t)1), _ctx);
    kk_std_time_instant__timescale_dropn(ts, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    kk_std_time_instant__instant_dup(inv);
    kk_std_core_types__maybe_drop(_match_5327, _ctx);
    kk_Unit;
    return inv;
  }
  kk_std_core_types__tuple2_ _this_4660;
  kk_function_t _x5705;
  kk_function_t _x10 = tz.utc_delta;
  kk_function_dup(_x10);
  _x5705 = _x10; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  _this_4660 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x5705, (_x5705, i, _ctx)); /*(std/time/duration/duration, string)*/
  kk_std_time_timestamp__timestamp t1_4663;
  kk_std_num_ddouble__ddouble _x5706;
  kk_box_t _box_x4977 = _this_4660.fst;
  kk_box_t _box_x4978 = _this_4660.snd;
  kk_std_time_duration__duration _x0 = kk_std_time_duration__duration_unbox(_box_x4977, NULL);
  kk_std_num_ddouble__ddouble _x2 = _x0.secs;
  _x5706 = _x2; /*std/time/timestamp/timespan*/
  t1_4663 = kk_std_time_timestamp__lp__dash__rp_(t, _x5706, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant tzi1;
  kk_std_time_instant__timescale _x5709 = kk_std_time_instant__timescale_dup(ts); /*std/time/instant/timescale*/
  tzi1 = kk_std_time_instant__new_Instant(kk_reuse_null, t1_4663, _x5709, _ctx); /*std/time/instant/instant*/
  kk_std_core_types__tuple2_ _this0_4665;
  kk_function_t _x5711;
  kk_function_t _x4 = tz.utc_delta;
  kk_function_dup(_x4);
  kk_std_time_calendar__timezone_drop(tz, _ctx);
  kk_Unit;
  _x5711 = _x4; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  kk_std_time_instant__instant _x5710 = kk_std_time_instant__instant_dup(tzi1); /*std/time/instant/instant*/
  _this0_4665 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x5711, (_x5711, _x5710, _ctx)); /*(std/time/duration/duration, string)*/
  bool _match_5328;
  kk_std_core_types__order _x5712;
  kk_std_num_ddouble__ddouble _x5713;
  kk_box_t _box_x4979 = _this_4660.fst;
  kk_box_t _box_x4980 = _this_4660.snd;
  kk_std_time_duration__duration _x00 = kk_std_time_duration__duration_unbox(_box_x4979, NULL);
  kk_std_core_types__tuple2__drop(_this_4660, _ctx);
  kk_Unit;
  kk_std_num_ddouble__ddouble _x5 = _x00.secs;
  _x5713 = _x5; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x5716;
  kk_box_t _box_x4981 = _this0_4665.fst;
  kk_box_t _box_x4982 = _this0_4665.snd;
  kk_std_time_duration__duration _x3 = kk_std_time_duration__duration_unbox(_box_x4981, NULL);
  kk_std_num_ddouble__ddouble _x000 = _x3.secs;
  _x5716 = _x000; /*std/time/timestamp/timespan*/
  _x5712 = kk_std_num_ddouble_compare(_x5713, _x5716, _ctx); /*order*/
  _match_5328 = kk_std_core__lp__eq__eq__4_rp_(_x5712, kk_std_core_types__new_Eq(_ctx), _ctx); /*bool*/
  if (_match_5328) {
    kk_std_core_types__tuple2__drop(_this0_4665, _ctx);
    kk_std_time_instant__timescale_dropn(ts, ((int32_t)10), _ctx);
    return tzi1;
  }
  kk_std_time_instant__instant_dropn(tzi1, ((int32_t)1), _ctx);
  kk_std_time_timestamp__timestamp t2_4670;
  kk_std_num_ddouble__ddouble _x5719;
  kk_box_t _box_x4983 = _this0_4665.fst;
  kk_box_t _box_x4984 = _this0_4665.snd;
  kk_std_time_duration__duration _x30 = kk_std_time_duration__duration_unbox(_box_x4983, NULL);
  kk_std_core_types__tuple2__drop(_this0_4665, _ctx);
  kk_Unit;
  kk_std_num_ddouble__ddouble _x6 = _x30.secs;
  _x5719 = _x6; /*std/time/timestamp/timespan*/
  t2_4670 = kk_std_time_timestamp__lp__dash__rp_(t, _x5719, _ctx); /*std/time/timestamp/timestamp*/
  return kk_std_time_instant__new_Instant(kk_reuse_null, t2_4670, ts, _ctx);
}
 
// Create a calendar where we assume that each
// day has 24 hours with 60 minutes, and where minutes are 60 seconds
// (with possibly leap seconds). The function takes care of timezones,
// epoch shifts, and leap seconds, and only needs a calendar `name`,
// a function `days-to-date` to calculate a date given a number of days since the `epoch`,
// a function `date-to-days` to calculate days since the `epoch` for a given date,
// and an optional time scale to be used (=`ts-utc`).
// `month-prefix` (=`""`) is used when displaying numeric months and is set for
// the ISO week dates to `"W"` for example. The `show-era` function can be given
// to display the era of a date.


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun5727__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4989;
};
static kk_string_t kk_std_time_calendar_earth_calendar_fun5727(kk_function_t _fself, kk_std_time_date__date _b_4992, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun5727(kk_box_t _fun_unbox_x4989, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5727__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun5727__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun5727, kk_context());
  _self->_fun_unbox_x4989 = _fun_unbox_x4989;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_earth_calendar_fun5727(kk_function_t _fself, kk_std_time_date__date _b_4992, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5727__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun5727__t*, _fself);
  kk_box_t _fun_unbox_x4989 = _self->_fun_unbox_x4989; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4989);}, {}, _ctx)
  kk_box_t _x5728;
  kk_function_t _x5729 = kk_function_unbox(_fun_unbox_x4989); /*(4990) -> 4991*/
  _x5728 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5729, (_x5729, kk_std_time_date__date_box(_b_4992, _ctx), _ctx)); /*4991*/
  return kk_string_unbox(_x5728);
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun5730__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_earth_calendar_fun5730(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun5730(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_earth_calendar_fun5730, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_earth_calendar_fun5730(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun5733__t {
  struct kk_function_s _base;
  kk_function_t days_to_date0;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_earth_calendar_fun5733(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun5733(kk_function_t days_to_date0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5733__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun5733__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun5733, kk_context());
  _self->days_to_date0 = days_to_date0;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_earth_calendar_fun5733(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5733__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun5733__t*, _fself);
  kk_function_t days_to_date0 = _self->days_to_date0; /* (days : int) -> std/time/date/date */
  kk_drop_match(_self, {kk_function_dup(days_to_date0);}, {}, _ctx)
  kk_std_time_timestamp__timestamp ts_4626;
  kk_std_num_ddouble__ddouble _x5734;
  kk_std_num_ddouble__ddouble _x5735;
  struct kk_std_time_instant_Instant* _con5736 = kk_std_time_instant__as_Instant(i);
  kk_std_time_timestamp__timestamp _x1 = _con5736->since;
  kk_std_time_instant__timescale _pat202 = _con5736->ts;
  struct kk_std_time_instant_Timescale* _con5737 = kk_std_time_instant__as_Timescale(_pat202);
  kk_std_num_ddouble__ddouble _x0 = _x1.since;
  _x5735 = _x0; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x5738;
  kk_std_num_ddouble__ddouble _x01 = tzdelta.secs;
  _x5738 = _x01; /*std/time/timestamp/timespan*/
  _x5734 = kk_std_num_ddouble__lp__plus__rp_(_x5735, _x5738, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x5739;
  struct kk_std_time_instant_Instant* _con5740 = kk_std_time_instant__as_Instant(i);
  kk_std_time_timestamp__timestamp _x10 = _con5740->since;
  kk_std_time_instant__timescale _pat2020 = _con5740->ts;
  struct kk_std_time_instant_Timescale* _con5741 = kk_std_time_instant__as_Timescale(_pat2020);
  kk_string_t _pat3000 = _con5741->name;
  kk_string_t _pat410 = _con5741->unit;
  kk_function_t _pat50 = _con5741->from_tai;
  kk_function_t _pat60 = _con5741->to_tai;
  kk_std_core_types__maybe _pat70 = _con5741->mb_seconds_in_day;
  kk_std_core_types__maybe _pat80 = _con5741->mb_to_mjd2000;
  kk_std_core_types__maybe _pat90 = _con5741->mb_from_mjd2000;
  if (kk_likely(kk_std_time_instant__instant_is_unique(i))) {
    if (kk_likely(kk_std_time_instant__timescale_is_unique(_pat2020))) {
      kk_string_drop(_pat3000, _ctx);
      kk_string_drop(_pat410, _ctx);
      kk_function_drop(_pat50, _ctx);
      kk_function_drop(_pat60, _ctx);
      kk_std_core_types__maybe_drop(_pat70, _ctx);
      kk_std_core_types__maybe_drop(_pat80, _ctx);
      kk_std_core_types__maybe_drop(_pat90, _ctx);
      kk_Unit;
      kk_std_time_instant__timescale_free(_pat2020);
    }
    else {
      kk_std_time_instant__timescale_decref(_pat2020, _ctx);
      kk_Unit;
    }
    kk_Unit;
    kk_std_time_instant__instant_free(i);
  }
  else {
    kk_std_time_instant__instant_decref(i, _ctx);
    kk_Unit;
  }
  kk_std_num_ddouble__ddouble _pat000 = _x10.since;
  int32_t _x00 = _x10.leap32;
  _x5739 = _x00; /*int32*/
  ts_4626 = kk_std_time_timestamp__new_Timestamp(_x5734, _x5739, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_core_types__tuple2_ _match_5324 = kk_std_time_timestamp_days_seconds(ts_4626, _ctx); /*(int, std/num/ddouble/ddouble)*/;
  kk_box_t _box_x4993 = _match_5324.fst;
  kk_box_t _box_x4994 = _match_5324.snd;
  kk_integer_t days0 = kk_integer_unbox(_box_x4993);
  kk_std_num_ddouble__ddouble secs = kk_std_num_ddouble__ddouble_unbox(_box_x4994, NULL);
  kk_integer_dup(days0);
  kk_std_core_types__tuple2__drop(_match_5324, _ctx);
  kk_Unit;
  kk_std_time_date__clock _match_5325;
  kk_integer_t _x5744;
  int32_t _x5745;
  kk_std_num_ddouble__ddouble _pat001 = ts_4626.since;
  int32_t _x = ts_4626.leap32;
  _x5745 = _x; /*int32*/
  _x5744 = kk_integer_from_int(_x5745,kk_context()); /*int*/
  _match_5325 = kk_std_time_date_clock_3(secs, _x5744, _ctx); /*std/time/date/clock*/
  struct kk_std_time_date_Clock* _con5746 = kk_std_time_date__as_Clock(_match_5325);
  kk_std_time_date__date _b_4998_4995 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0, (days_to_date0, days0, _ctx)); /*std/time/date/date*/;
  kk_std_time_date__clock _b_4999_4996 = _match_5325; /*std/time/date/clock*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_4998_4995, _ctx), kk_std_time_date__clock_box(_b_4999_4996, _ctx), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun5748__t {
  struct kk_function_s _base;
  kk_function_t date_to_days0;
};
static kk_std_time_instant__instant kk_std_time_calendar_earth_calendar_fun5748(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun5748(kk_function_t date_to_days0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5748__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun5748__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun5748, kk_context());
  _self->date_to_days0 = date_to_days0;
  return &_self->_base;
}

static kk_std_time_instant__instant kk_std_time_calendar_earth_calendar_fun5748(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun5748__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun5748__t*, _fself);
  kk_function_t date_to_days0 = _self->date_to_days0; /* (std/time/date/date) -> int */
  kk_drop_match(_self, {kk_function_dup(date_to_days0);}, {}, _ctx)
  kk_std_time_timestamp__timestamp _x5749;
  kk_std_core_types__optional _x5750;
  bool _b_5000_4997;
  struct kk_std_time_instant_Timescale* _con5751 = kk_std_time_instant__as_Timescale(ts);
  kk_std_core_types__maybe _x100 = _con5751->mb_seconds_in_day;
  kk_std_core_types__maybe_dup(_x100);
  if (kk_std_core_types__is_Nothing(_x100)) {
    _b_5000_4997 = false; /*bool*/
  }
  else {
    kk_std_core_types__maybe_drop(_x100, _ctx);
    _b_5000_4997 = true; /*bool*/
  }
  _x5750 = kk_std_core_types__new_Optional(kk_bool_box(_b_5000_4997), _ctx); /*optional<106>*/
  _x5749 = kk_std_time_calendar_dc_timestamp(d00, c, date_to_days0, _x5750, _ctx); /*std/time/timestamp/timestamp*/
  return kk_std_time_calendar_earth_timestamp_to_instant(_x5749, tz, ts, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_earth_calendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_to_date0, kk_function_t date_to_days0, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-to-date : (days : int) -> std/time/date/date, date-to-days : (std/time/date/date) -> int, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_string_t _x5722;
  if (kk_std_core_types__is_Optional(month_prefix0)) {
    kk_box_t _box_x4985 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_3566 = kk_string_unbox(_box_x4985);
    kk_string_dup(_month_prefix_3566);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    kk_Unit;
    _x5722 = _month_prefix_3566; /*string*/
    goto _match5723;
  }
  _x5722 = kk_string_empty(); /*string*/
  _match5723: ;
  kk_function_t _x5726;
  if (kk_std_core_types__is_Optional(show_era0)) {
    kk_box_t _fun_unbox_x4989 = show_era0._cons.Optional.value;
    _x5726 = kk_std_time_calendar_new_earth_calendar_fun5727(_fun_unbox_x4989, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _x5726 = kk_std_time_calendar_new_earth_calendar_fun5730(_ctx); /*(std/time/date/date) -> string*/
  }
  kk_function_t _x5732;
  kk_function_dup(days_to_date0);
  _x5732 = kk_std_time_calendar_new_earth_calendar_fun5733(days_to_date0, _ctx); /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (6, 7)*/
  kk_function_t _x5747;
  kk_function_dup(date_to_days0);
  _x5747 = kk_std_time_calendar_new_earth_calendar_fun5748(date_to_days0, _ctx); /*(d00 : std/time/date/date, c : std/time/date/clock, tz : std/time/calendar/timezone, ts : std/time/instant/timescale) -> std/time/instant/instant*/
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, name0, long_name0, _x5722, _x5726, _x5732, _x5747, days_to_date0, date_to_days0, _ctx);
}
 
// An abstraction over solar calendars. Almost all calendars can be defined
// in terms of this function (and `solar-ecalendar`). Given functions to
// convert days to and from a year, and day-of-the-year to and from month
// and month day, this constructs a full calendar.
// Also needs an `epoch-shift` that gives the number of days from the
// calendar epoch to the date ``2000-01-01``. By default this is `730119`
// which corresponds to a ``0001-01-01`` ISO calendar epoch. The
// `has-year-zero` (=`True`) parameter is used when the calendar has a
// year zero. The `month-prefix` (=`""`) is used for displaying numeric
// months (and is `"W"` for `cal-iso-week` for example). The `show-era` function
// can be given to display an era for a given date. Finally the `ts` (=`ts-ti`)
// gives the time scale for the calendar.


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun5753__t {
  struct kk_function_s _base;
  kk_function_t days_before_month;
  kk_function_t days_to_month;
  kk_function_t days_to_yeardoy;
  kk_std_core_types__optional epoch_shift;
  kk_std_core_types__optional has_year_zero;
};
static kk_std_time_date__date kk_std_time_calendar_solar_calendar_fun5753(kk_function_t _fself, kk_integer_t days0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun5753(kk_function_t days_before_month, kk_function_t days_to_month, kk_function_t days_to_yeardoy, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5753__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun5753__t, 8, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun5753, kk_context());
  _self->days_before_month = days_before_month;
  _self->days_to_month = days_to_month;
  _self->days_to_yeardoy = days_to_yeardoy;
  _self->epoch_shift = epoch_shift;
  _self->has_year_zero = has_year_zero;
  return &_self->_base;
}

static kk_std_time_date__date kk_std_time_calendar_solar_calendar_fun5753(kk_function_t _fself, kk_integer_t days0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5753__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun5753__t*, _fself);
  kk_function_t days_before_month = _self->days_before_month; /* (year : int, month : int) -> int */
  kk_function_t days_to_month = _self->days_to_month; /* (year : int, doy : int) -> int */
  kk_function_t days_to_yeardoy = _self->days_to_yeardoy; /* (days : int) -> (int, int) */
  kk_std_core_types__optional epoch_shift = _self->epoch_shift; /* optional<int> */
  kk_std_core_types__optional has_year_zero = _self->has_year_zero; /* optional<bool> */
  kk_drop_match(_self, {kk_function_dup(days_before_month);kk_function_dup(days_to_month);kk_function_dup(days_to_yeardoy);kk_std_core_types__optional_dup(epoch_shift);kk_std_core_types__optional_dup(has_year_zero);}, {}, _ctx)
  kk_integer_t days;
  kk_integer_t _x5754;
  if (kk_std_core_types__is_Optional(epoch_shift)) {
    kk_box_t _box_x5001 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_3632 = kk_integer_unbox(_box_x5001);
    kk_integer_dup(_epoch_shift_3632);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    kk_Unit;
    _x5754 = _epoch_shift_3632; /*int*/
    goto _match5755;
  }
  _x5754 = kk_integer_from_int(730119, _ctx); /*int*/
  _match5755: ;
  days = kk_integer_add(days0,_x5754,kk_context()); /*int*/
  kk_std_core_types__tuple2_ _match_5322 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_integer_t, kk_context_t*), days_to_yeardoy, (days_to_yeardoy, days, _ctx)); /*(int, int)*/;
  kk_box_t _box_x5002 = _match_5322.fst;
  kk_box_t _box_x5003 = _match_5322.snd;
  kk_integer_t y0 = kk_integer_unbox(_box_x5002);
  kk_integer_t doy = kk_integer_unbox(_box_x5003);
  kk_integer_dup(doy);
  kk_integer_dup(y0);
  kk_std_core_types__tuple2__drop(_match_5322, _ctx);
  kk_Unit;
  kk_integer_t month;
  kk_integer_t _x5759 = kk_integer_dup(y0); /*int*/
  kk_integer_t _x5760 = kk_integer_dup(doy); /*int*/
  month = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_to_month, (days_to_month, _x5759, _x5760, _ctx)); /*int*/
  kk_integer_t day;
  kk_integer_t _x5761;
  kk_integer_t _x5762;
  kk_integer_t _x5763 = kk_integer_dup(y0); /*int*/
  kk_integer_t _x5764 = kk_integer_dup(month); /*int*/
  _x5762 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_before_month, (days_before_month, _x5763, _x5764, _ctx)); /*int*/
  _x5761 = kk_integer_sub(doy,_x5762,kk_context()); /*int*/
  day = kk_integer_add(_x5761,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_integer_t year;
  if (kk_std_core_types__is_Optional(has_year_zero)) {
    kk_box_t _box_x5004 = has_year_zero._cons.Optional.value;
    bool _has_year_zero_3636 = kk_bool_unbox(_box_x5004);
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    kk_Unit;
    if (_has_year_zero_3636) {
      year = y0; /*int*/
      goto _match5765;
    }
    bool _match_5323;
    kk_integer_t _x5767 = kk_integer_dup(y0); /*int*/
    _match_5323 = kk_std_core_is_pos_2(_x5767, _ctx); /*bool*/
    if (_match_5323) {
      year = y0; /*int*/
      goto _match5765;
    }
    year = kk_integer_sub(y0,(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match5765;
  }
  year = y0; /*int*/
  _match5765: ;
  return kk_std_time_date__new_Date(year, month, day, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun5768__t {
  struct kk_function_s _base;
  kk_function_t days_before_month;
  kk_function_t days_before_year;
  kk_std_core_types__optional epoch_shift;
  kk_std_core_types__optional has_year_zero;
};
static kk_integer_t kk_std_time_calendar_solar_calendar_fun5768(kk_function_t _fself, kk_std_time_date__date d0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun5768(kk_function_t days_before_month, kk_function_t days_before_year, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5768__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun5768__t, 7, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun5768, kk_context());
  _self->days_before_month = days_before_month;
  _self->days_before_year = days_before_year;
  _self->epoch_shift = epoch_shift;
  _self->has_year_zero = has_year_zero;
  return &_self->_base;
}

static kk_integer_t kk_std_time_calendar_solar_calendar_fun5768(kk_function_t _fself, kk_std_time_date__date d0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5768__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun5768__t*, _fself);
  kk_function_t days_before_month = _self->days_before_month; /* (year : int, month : int) -> int */
  kk_function_t days_before_year = _self->days_before_year; /* (year : int) -> int */
  kk_std_core_types__optional epoch_shift = _self->epoch_shift; /* optional<int> */
  kk_std_core_types__optional has_year_zero = _self->has_year_zero; /* optional<bool> */
  kk_drop_match(_self, {kk_function_dup(days_before_month);kk_function_dup(days_before_year);kk_std_core_types__optional_dup(epoch_shift);kk_std_core_types__optional_dup(has_year_zero);}, {}, _ctx)
  kk_integer_t y;
  if (kk_std_core_types__is_Optional(has_year_zero)) {
    kk_box_t _box_x5005 = has_year_zero._cons.Optional.value;
    bool _has_year_zero_36360 = kk_bool_unbox(_box_x5005);
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    kk_Unit;
    if (_has_year_zero_36360) {
      kk_integer_t _x = d0.year;
      kk_integer_dup(_x);
      y = _x; /*int*/
      goto _match5769;
    }
    bool _match_5321;
    kk_integer_t _x5771;
    kk_integer_t _x0 = d0.year;
    kk_integer_dup(_x0);
    _x5771 = _x0; /*int*/
    _match_5321 = kk_std_core_is_pos_2(_x5771, _ctx); /*bool*/
    if (_match_5321) {
      kk_integer_t _x1 = d0.year;
      kk_integer_dup(_x1);
      y = _x1; /*int*/
      goto _match5769;
    }
    kk_integer_t _x5772;
    kk_integer_t _x2 = d0.year;
    kk_integer_dup(_x2);
    _x5772 = _x2; /*int*/
    y = kk_integer_add(_x5772,(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match5769;
  }
  kk_integer_t _x3 = d0.year;
  kk_integer_dup(_x3);
  y = _x3; /*int*/
  _match5769: ;
  kk_integer_t _x5773;
  kk_integer_t _x5774;
  kk_integer_t _x5775;
  kk_integer_t _x5776 = kk_integer_dup(y); /*int*/
  _x5775 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), days_before_year, (days_before_year, _x5776, _ctx)); /*int*/
  kk_integer_t _x5777;
  kk_integer_t _x5778;
  kk_integer_t _x30 = d0.month;
  kk_integer_dup(_x30);
  _x5778 = _x30; /*int*/
  _x5777 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_before_month, (days_before_month, y, _x5778, _ctx)); /*int*/
  _x5774 = kk_integer_add(_x5775,_x5777,kk_context()); /*int*/
  kk_integer_t _x5779;
  kk_integer_t _x5780;
  kk_integer_t _x4 = d0.day;
  kk_integer_dup(_x4);
  kk_std_time_date__date_drop(d0, _ctx);
  kk_Unit;
  _x5780 = _x4; /*int*/
  _x5779 = kk_integer_sub(_x5780,(kk_integer_from_small(1)),kk_context()); /*int*/
  _x5773 = kk_integer_add(_x5774,_x5779,kk_context()); /*int*/
  kk_integer_t _x5781;
  if (kk_std_core_types__is_Optional(epoch_shift)) {
    kk_box_t _box_x5006 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_36320 = kk_integer_unbox(_box_x5006);
    kk_integer_dup(_epoch_shift_36320);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    kk_Unit;
    _x5781 = _epoch_shift_36320; /*int*/
    goto _match5782;
  }
  _x5781 = kk_integer_from_int(730119, _ctx); /*int*/
  _match5782: ;
  return kk_integer_sub(_x5773,_x5781,kk_context());
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun5789__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5012;
};
static kk_string_t kk_std_time_calendar_solar_calendar_fun5789(kk_function_t _fself, kk_std_time_date__date _b_5015, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun5789(kk_box_t _fun_unbox_x5012, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5789__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun5789__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun5789, kk_context());
  _self->_fun_unbox_x5012 = _fun_unbox_x5012;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_solar_calendar_fun5789(kk_function_t _fself, kk_std_time_date__date _b_5015, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5789__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun5789__t*, _fself);
  kk_box_t _fun_unbox_x5012 = _self->_fun_unbox_x5012; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5012);}, {}, _ctx)
  kk_box_t _x5790;
  kk_function_t _x5791 = kk_function_unbox(_fun_unbox_x5012); /*(5013) -> 5014*/
  _x5790 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5791, (_x5791, kk_std_time_date__date_box(_b_5015, _ctx), _ctx)); /*5014*/
  return kk_string_unbox(_x5790);
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun5792__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_solar_calendar_fun5792(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun5792(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_solar_calendar_fun5792, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_solar_calendar_fun5792(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun5794__t {
  struct kk_function_s _base;
  kk_function_t _b_5021_5016;
};
static kk_box_t kk_std_time_calendar_solar_calendar_fun5794(kk_function_t _fself, kk_box_t _b_5019, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun5794(kk_function_t _b_5021_5016, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5794__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun5794__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun5794, kk_context());
  _self->_b_5021_5016 = _b_5021_5016;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_solar_calendar_fun5794(kk_function_t _fself, kk_box_t _b_5019, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun5794__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun5794__t*, _fself);
  kk_function_t _b_5021_5016 = _self->_b_5021_5016; /* (std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5021_5016);}, {}, _ctx)
  kk_string_t _x5795;
  kk_std_time_date__date _x5796 = kk_std_time_date__date_unbox(_b_5019, _ctx); /*std/time/date/date*/
  _x5795 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5021_5016, (_b_5021_5016, _x5796, _ctx)); /*string*/
  return kk_string_box(_x5795);
}

kk_std_time_calendar__calendar kk_std_time_calendar_solar_calendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_before_year, kk_function_t days_to_yeardoy, kk_function_t days_before_month, kk_function_t days_to_month, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-before-year : (year : int) -> int, days-to-yeardoy : (days : int) -> (int, int), days-before-month : (year : int, month : int) -> int, days-to-month : (year : int, doy : int) -> int, epoch-shift : optional<int>, has-year-zero : optional<bool>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t _x5752;
  kk_function_dup(days_before_month);
  kk_std_core_types__optional_dup(epoch_shift);
  kk_std_core_types__optional_dup(has_year_zero);
  _x5752 = kk_std_time_calendar_new_solar_calendar_fun5753(days_before_month, days_to_month, days_to_yeardoy, epoch_shift, has_year_zero, _ctx); /*(days0 : int) -> std/time/date/date*/
  kk_std_core_types__optional _x5784;
  kk_string_t _b_5020_5008;
  if (kk_std_core_types__is_Optional(month_prefix0)) {
    kk_box_t _box_x5007 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_3640 = kk_string_unbox(_box_x5007);
    kk_string_dup(_month_prefix_3640);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    kk_Unit;
    _b_5020_5008 = _month_prefix_3640; /*string*/
    goto _match5785;
  }
  _b_5020_5008 = kk_string_empty(); /*string*/
  _match5785: ;
  _x5784 = kk_std_core_types__new_Optional(kk_string_box(_b_5020_5008), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5788;
  kk_function_t _b_5021_5016;
  if (kk_std_core_types__is_Optional(show_era0)) {
    kk_box_t _fun_unbox_x5012 = show_era0._cons.Optional.value;
    _b_5021_5016 = kk_std_time_calendar_new_solar_calendar_fun5789(_fun_unbox_x5012, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _b_5021_5016 = kk_std_time_calendar_new_solar_calendar_fun5792(_ctx); /*(std/time/date/date) -> string*/
  }
  _x5788 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_solar_calendar_fun5794(_b_5021_5016, _ctx)), _ctx); /*optional<106>*/
  return kk_std_time_calendar_earth_calendar(name0, long_name0, _x5752, kk_std_time_calendar_new_solar_calendar_fun5768(days_before_month, days_before_year, epoch_shift, has_year_zero, _ctx), _x5784, _x5788, _ctx);
}
 
// An abstraction over solar calendars. Almost all calendars can be defined
// in terms of this function (and `solar-calendar`). Given functions to
// convert days to and from a year, and day-of-the-year to and from month
// and month day, this constructs a full calendar. In contrast to `solar-calendar`
// this function just needs an estimation function from days to a year -- for most
// calendars this is much more convenient. The estimation function returns a
// conservative (i.e. lowest) estimate for the year and a `maybe<int>` that
// is the `safe-day`: if the day of the year is lower or equal to that the estimate
// is surely correct. Otherwise the wrapper will calculate the start day for the next
// year until the correct year is found.
// Also needs an `epoch-shift` that gives the number of days from the
// calendar epoch to the date ``2000-01-01``. By default this is `730119`
// which corresponds to a ``0001-01-01`` ISO calendar epoch. The
// `has-year-zero` (=`True`) parameter is used when the calendar has a
// year zero. The `month-prefix` (=`""`) is used for displaying numeric
// months (and is `"W"` for `cal-iso-week` for example). The `show-era` function
// can be given to display an era for a given date. Finally the `ts` (=`ts-utc`)
// gives the time scale for the calendar.


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun5798__t {
  struct kk_function_s _base;
  kk_function_t days_before_year;
  kk_function_t estimate_year;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_ecalendar_fun5798(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun5798(kk_function_t days_before_year, kk_function_t estimate_year, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun5798__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_ecalendar_fun5798__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_ecalendar_fun5798, kk_context());
  _self->days_before_year = days_before_year;
  _self->estimate_year = estimate_year;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_ecalendar_fun5798(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun5798__t* _self = kk_function_as(struct kk_std_time_calendar_solar_ecalendar_fun5798__t*, _fself);
  kk_function_t days_before_year = _self->days_before_year; /* (year : int) -> int */
  kk_function_t estimate_year = _self->estimate_year; /* (days : int) -> (int, maybe<int>) */
  kk_drop_match(_self, {kk_function_dup(days_before_year);kk_function_dup(estimate_year);}, {}, _ctx)
  kk_std_core_types__tuple2_ _match_5314;
  kk_integer_t _x5799 = kk_integer_dup(days); /*int*/
  _match_5314 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_integer_t, kk_context_t*), estimate_year, (estimate_year, _x5799, _ctx)); /*(int, maybe<int>)*/
  kk_box_t _box_x5022 = _match_5314.fst;
  kk_box_t _box_x5023 = _match_5314.snd;
  kk_integer_t approx = kk_integer_unbox(_box_x5022);
  kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5023, NULL);
  kk_integer_dup(approx);
  kk_std_core_types__maybe_dup(safe);
  kk_std_core_types__tuple2__drop(_match_5314, _ctx);
  kk_Unit;
  kk_integer_t doy1;
  kk_integer_t _x5802 = kk_integer_dup(days); /*int*/
  kk_integer_t _x5803;
  kk_function_t _x5805 = kk_function_dup(days_before_year); /*(year : int) -> int*/
  kk_integer_t _x5804 = kk_integer_dup(approx); /*int*/
  _x5803 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), _x5805, (_x5805, _x5804, _ctx)); /*int*/
  doy1 = kk_integer_sub(_x5802,_x5803,kk_context()); /*int*/
  bool _match_5315;
  kk_integer_t _borrowedArg_5319;
  if (kk_std_core_types__is_Nothing(safe)) {
    _borrowedArg_5319 = kk_integer_from_small(0); /*int*/
  }
  else {
    kk_box_t _box_x5024 = safe._cons.Just.value;
    kk_integer_t x = kk_integer_unbox(_box_x5024);
    kk_integer_dup(x);
    kk_std_core_types__maybe_drop(safe, _ctx);
    kk_Unit;
    _borrowedArg_5319 = x; /*int*/
  }
  bool _borrowApp_5320 = kk_integer_lte_borrow(doy1,_borrowedArg_5319,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5319, _ctx);
  _match_5315 = _borrowApp_5320; /*bool*/
  if (_match_5315) {
    kk_integer_drop(days, _ctx);
    kk_function_drop(days_before_year, _ctx);
    kk_integer_t _b_5048_5025 = approx; /*int*/;
    kk_integer_t _b_5049_5026 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5048_5025), kk_integer_box(_b_5049_5026), _ctx);
  }
  kk_integer_t doy2;
  kk_integer_t _x5807;
  kk_integer_t _x5808;
  kk_integer_t _x5809 = kk_integer_dup(approx); /*int*/
  _x5808 = kk_integer_add(_x5809,(kk_integer_from_small(1)),kk_context()); /*int*/
  _x5807 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), days_before_year, (days_before_year, _x5808, _ctx)); /*int*/
  doy2 = kk_integer_sub(days,_x5807,kk_context()); /*int*/
  bool _match_5316;
  kk_integer_t _borrowedArg_5317 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_5318 = kk_integer_lt_borrow(doy2,_borrowedArg_5317,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5317, _ctx);
  _match_5316 = _borrowApp_5318; /*bool*/
  if (_match_5316) {
    kk_integer_drop(doy2, _ctx);
    kk_integer_t _b_5050_5027 = approx; /*int*/;
    kk_integer_t _b_5051_5028 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5050_5027), kk_integer_box(_b_5051_5028), _ctx);
  }
  kk_integer_drop(doy1, _ctx);
  kk_integer_t _b_5052_5029 = kk_integer_add(approx,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t _b_5053_5030 = doy2; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5052_5029), kk_integer_box(_b_5053_5030), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun5821__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5040;
};
static kk_string_t kk_std_time_calendar_solar_ecalendar_fun5821(kk_function_t _fself, kk_std_time_date__date _b_5043, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun5821(kk_box_t _fun_unbox_x5040, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun5821__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_ecalendar_fun5821__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_ecalendar_fun5821, kk_context());
  _self->_fun_unbox_x5040 = _fun_unbox_x5040;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_solar_ecalendar_fun5821(kk_function_t _fself, kk_std_time_date__date _b_5043, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun5821__t* _self = kk_function_as(struct kk_std_time_calendar_solar_ecalendar_fun5821__t*, _fself);
  kk_box_t _fun_unbox_x5040 = _self->_fun_unbox_x5040; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5040);}, {}, _ctx)
  kk_box_t _x5822;
  kk_function_t _x5823 = kk_function_unbox(_fun_unbox_x5040); /*(5041) -> 5042*/
  _x5822 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5823, (_x5823, kk_std_time_date__date_box(_b_5043, _ctx), _ctx)); /*5042*/
  return kk_string_unbox(_x5822);
}


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun5824__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_solar_ecalendar_fun5824(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun5824(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_solar_ecalendar_fun5824, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_solar_ecalendar_fun5824(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun5826__t {
  struct kk_function_s _base;
  kk_function_t _b_5057_5044;
};
static kk_box_t kk_std_time_calendar_solar_ecalendar_fun5826(kk_function_t _fself, kk_box_t _b_5047, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun5826(kk_function_t _b_5057_5044, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun5826__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_ecalendar_fun5826__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_ecalendar_fun5826, kk_context());
  _self->_b_5057_5044 = _b_5057_5044;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_solar_ecalendar_fun5826(kk_function_t _fself, kk_box_t _b_5047, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun5826__t* _self = kk_function_as(struct kk_std_time_calendar_solar_ecalendar_fun5826__t*, _fself);
  kk_function_t _b_5057_5044 = _self->_b_5057_5044; /* (std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5057_5044);}, {}, _ctx)
  kk_string_t _x5827;
  kk_std_time_date__date _x5828 = kk_std_time_date__date_unbox(_b_5047, _ctx); /*std/time/date/date*/
  _x5827 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5057_5044, (_b_5057_5044, _x5828, _ctx)); /*string*/
  return kk_string_box(_x5827);
}

kk_std_time_calendar__calendar kk_std_time_calendar_solar_ecalendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_before_year, kk_function_t estimate_year, kk_function_t days_before_month, kk_function_t days_to_month, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-before-year : (year : int) -> int, estimate-year : (days : int) -> (int, maybe<int>), days-before-month : (year : int, month : int) -> int, days-to-month : (year : int, doy : int) -> int, epoch-shift : optional<int>, has-year-zero : optional<bool>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t _x5797 = kk_function_dup(days_before_year); /*(year : int) -> int*/
  kk_std_core_types__optional _x5810;
  kk_integer_t _b_5054_5032;
  if (kk_std_core_types__is_Optional(epoch_shift)) {
    kk_box_t _box_x5031 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_2726 = kk_integer_unbox(_box_x5031);
    kk_integer_dup(_epoch_shift_2726);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    kk_Unit;
    _b_5054_5032 = _epoch_shift_2726; /*int*/
    goto _match5811;
  }
  _b_5054_5032 = kk_integer_from_int(730119, _ctx); /*int*/
  _match5811: ;
  _x5810 = kk_std_core_types__new_Optional(kk_integer_box(_b_5054_5032), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5813;
  bool _b_5055_5034;
  if (kk_std_core_types__is_Optional(has_year_zero)) {
    kk_box_t _box_x5033 = has_year_zero._cons.Optional.value;
    bool _has_year_zero_2730 = kk_bool_unbox(_box_x5033);
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    kk_Unit;
    _b_5055_5034 = _has_year_zero_2730; /*bool*/
    goto _match5814;
  }
  _b_5055_5034 = true; /*bool*/
  _match5814: ;
  _x5813 = kk_std_core_types__new_Optional(kk_bool_box(_b_5055_5034), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5816;
  kk_string_t _b_5056_5036;
  if (kk_std_core_types__is_Optional(month_prefix0)) {
    kk_box_t _box_x5035 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_2734 = kk_string_unbox(_box_x5035);
    kk_string_dup(_month_prefix_2734);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    kk_Unit;
    _b_5056_5036 = _month_prefix_2734; /*string*/
    goto _match5817;
  }
  _b_5056_5036 = kk_string_empty(); /*string*/
  _match5817: ;
  _x5816 = kk_std_core_types__new_Optional(kk_string_box(_b_5056_5036), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5820;
  kk_function_t _b_5057_5044;
  if (kk_std_core_types__is_Optional(show_era0)) {
    kk_box_t _fun_unbox_x5040 = show_era0._cons.Optional.value;
    _b_5057_5044 = kk_std_time_calendar_new_solar_ecalendar_fun5821(_fun_unbox_x5040, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _b_5057_5044 = kk_std_time_calendar_new_solar_ecalendar_fun5824(_ctx); /*(std/time/date/date) -> string*/
  }
  _x5820 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_solar_ecalendar_fun5826(_b_5057_5044, _ctx)), _ctx); /*optional<106>*/
  return kk_std_time_calendar_solar_calendar(name0, long_name0, _x5797, kk_std_time_calendar_new_solar_ecalendar_fun5798(days_before_year, estimate_year, _ctx), days_before_month, days_to_month, _x5810, _x5813, _x5816, _x5820, _ctx);
}
 
// Create a standard ISO calendar using a particular time scale
// and calendar `name` (=`ts.name`).


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun5834__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun5834(kk_function_t _fself, kk_integer_t _x15833, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun5834(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun5834, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun5834(kk_function_t _fself, kk_integer_t _x15833, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x15833, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun5836__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_iso_calendar_fun5836(kk_function_t _fself, kk_integer_t _x15835, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun5836(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun5836, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_iso_calendar_fun5836(kk_function_t _fself, kk_integer_t _x15835, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x15835, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun5839__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun5839(kk_function_t _fself, kk_integer_t _x15837, kk_integer_t _x25838, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun5839(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun5839, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun5839(kk_function_t _fself, kk_integer_t _x15837, kk_integer_t _x25838, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x15837, _x25838, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun5842__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun5842(kk_function_t _fself, kk_integer_t _x15840, kk_integer_t _x25841, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun5842(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun5842, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun5842(kk_function_t _fself, kk_integer_t _x15840, kk_integer_t _x25841, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x15840, _x25841, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_iso_calendar(kk_std_time_instant__timescale ___wildcard__455__26, kk_string_t name0, kk_std_core_types__optional long_name0, kk_context_t* _ctx) { /* (std/time/instant/timescale, name : string, long-name : optional<string>) -> calendar */ 
  kk_std_time_instant__timescale_dropn(___wildcard__455__26, ((int32_t)10), _ctx);
  kk_string_t _x5829 = kk_string_dup(name0); /*string*/
  kk_string_t _x5830;
  if (kk_std_core_types__is_Optional(long_name0)) {
    kk_box_t _box_x5058 = long_name0._cons.Optional.value;
    kk_string_t _long_name_2918 = kk_string_unbox(_box_x5058);
    kk_string_drop(name0, _ctx);
    kk_string_dup(_long_name_2918);
    kk_std_core_types__optional_drop(long_name0, _ctx);
    kk_Unit;
    _x5830 = _long_name_2918; /*string*/
    goto _match5831;
  }
  _x5830 = name0; /*string*/
  _match5831: ;
  return kk_std_time_calendar_solar_ecalendar(_x5829, _x5830, kk_std_time_calendar_new_iso_calendar_fun5834(_ctx), kk_std_time_calendar_new_iso_calendar_fun5836(_ctx), kk_std_time_calendar_new_iso_calendar_fun5839(_ctx), kk_std_time_calendar_new_iso_calendar_fun5842(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), _ctx);
}
 
// The standard [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) calendar
// using TI time (`ts-ti`). This is a proleptic Gregorian
// calendar except that it uses the year 0 for 1 BCE, -1 for 2 BCE etc.
//
// This is the default calendar used in the library as it guarantees deterministic
// date calculations while still taking historical leap seconds into account.


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun5851__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun5851(kk_function_t _fself, kk_integer_t _x15850, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun5851(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun5851, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun5851(kk_function_t _fself, kk_integer_t _x15850, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x15850, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun5853__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_fun5853(kk_function_t _fself, kk_integer_t _x15852, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun5853(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun5853, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_fun5853(kk_function_t _fself, kk_integer_t _x15852, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x15852, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun5856__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun5856(kk_function_t _fself, kk_integer_t _x15854, kk_integer_t _x25855, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun5856(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun5856, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun5856(kk_function_t _fself, kk_integer_t _x15854, kk_integer_t _x25855, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x15854, _x25855, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun5859__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun5859(kk_function_t _fself, kk_integer_t _x15857, kk_integer_t _x25858, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun5859(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun5859, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun5859(kk_function_t _fself, kk_integer_t _x15857, kk_integer_t _x25858, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x15857, _x25858, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_iso;
 
// Return the instant in time for a given `:date` and `:clock` (= `clock0`) interpreted by
// calendar `cal` (=`cal-iso`) in a timezone `tz` (=`tz-utc` by default).


// lift anonymous function
struct kk_std_time_calendar_instant_fun5868__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_instant_fun5868(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5868(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_instant_fun5868, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_instant_fun5868(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5066_5062 = kk_std_time_duration_zero; /*std/time/duration/duration*/;
  kk_string_t _b_5067_5063;
  kk_std_core_types__optional _match_5312 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5312)) {
    kk_box_t _box_x5061 = _match_5312._cons.Optional.value;
    kk_string_t _abbrv_2653 = kk_string_unbox(_box_x5061);
    kk_string_dup(_abbrv_2653);
    kk_std_core_types__optional_drop(_match_5312, _ctx);
    kk_Unit;
    _b_5067_5063 = _abbrv_2653; /*string*/
    goto _match5869;
  }
  _b_5067_5063 = kk_string_empty(); /*string*/
  _match5869: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5066_5062, _ctx), kk_string_box(_b_5067_5063), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5872__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_instant_fun5872(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5872(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_instant_fun5872, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar_instant_fun5872(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  bool _match_5311;
  double _x5873;
  kk_std_time_duration__duration _x5874 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  kk_std_num_ddouble__ddouble _x0 = _x5874.secs;
  double _x1 = _x0.hi;
  _x5873 = _x1; /*double*/
  _match_5311 = (_x5873 == (0.0)); /*bool*/
  if (_match_5311) {
    kk_std_time_instant__instant _b_5068_5064 = i00; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_5068_5064, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i00, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_instant(kk_std_time_instant__timescale ts, kk_std_time_date__date d, kk_std_core_types__optional c, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (ts : std/time/instant/timescale, d : std/time/date/date, c : optional<std/time/date/clock>, tz : optional<timezone>, cal : optional<calendar>) -> std/time/instant/instant */ 
  kk_function_t _x5875;
  kk_std_time_calendar__calendar _match_5313;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5065 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_2960 = kk_std_time_calendar__calendar_unbox(_box_x5065, NULL);
    kk_std_time_calendar__calendar_dup(_cal_2960);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5313 = _cal_2960; /*std/time/calendar/calendar*/
    goto _match5876;
  }
  _match_5313 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match5876: ;
  struct kk_std_time_calendar_Calendar* _con5878 = kk_std_time_calendar__as_Calendar(_match_5313);
  kk_string_t _pat0 = _con5878->name;
  kk_string_t _pat1 = _con5878->long_name;
  kk_string_t _pat2 = _con5878->month_prefix;
  kk_function_t _pat3 = _con5878->show_era;
  kk_function_t _pat4 = _con5878->instant_to_dc;
  kk_function_t _x = _con5878->dc_to_instant;
  kk_function_t _pat5 = _con5878->days_to_date;
  kk_function_t _pat6 = _con5878->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5313))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5313);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5313, _ctx);
    kk_Unit;
  }
  _x5875 = _x; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  kk_std_time_date__clock _x5860;
  if (kk_std_core_types__is_Optional(c)) {
    kk_box_t _box_x5059 = c._cons.Optional.value;
    kk_std_time_date__clock _c_2952 = kk_std_time_date__clock_unbox(_box_x5059, NULL);
    kk_std_time_date__clock_dup(_c_2952);
    kk_std_core_types__optional_drop(c, _ctx);
    kk_Unit;
    _x5860 = _c_2952; /*std/time/date/clock*/
    goto _match5861;
  }
  _x5860 = kk_std_time_date__clock_dup(kk_std_time_date_clock0); /*std/time/date/clock*/
  _match5861: ;
  kk_std_time_calendar__timezone _x5863;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5060 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_2956 = kk_std_time_calendar__timezone_unbox(_box_x5060, NULL);
    kk_std_time_calendar__timezone_dup(_tz_2956);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _x5863 = _tz_2956; /*std/time/calendar/timezone*/
    goto _match5864;
  }
  kk_string_t _x5866;
  kk_define_string_literal(, _s5867, 3, "UTC")
  _x5866 = kk_string_dup(_s5867); /*string*/
  _x5863 = kk_std_time_calendar__new_Timezone(_x5866, kk_std_time_calendar_new_instant_fun5868(_ctx), kk_std_time_calendar_new_instant_fun5872(_ctx), _ctx); /*std/time/calendar/timezone*/
  _match5864: ;
  return kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x5875, (_x5875, d, _x5860, _x5863, ts, _ctx));
}
 
// monadic lift


// lift anonymous function
struct kk_std_time_calendar__mlift4792_instant_fun5879__t_1 {
  struct kk_function_s _base;
  kk_std_time_instant__timescale _c_4782;
  kk_std_core_types__optional c;
  kk_std_core_types__optional cal;
  kk_std_time_date__date d;
  kk_std_core_types__optional tz;
};
static kk_std_time_instant__instant kk_std_time_calendar__mlift4792_instant_fun5879_1(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_mlift4792_instant_fun5879_1(kk_std_time_instant__timescale _c_4782, kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4792_instant_fun5879__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__mlift4792_instant_fun5879__t_1, 11, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__mlift4792_instant_fun5879_1, kk_context());
  _self->_c_4782 = _c_4782;
  _self->c = c;
  _self->cal = cal;
  _self->d = d;
  _self->tz = tz;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_time_calendar__mlift4792_instant_fun5888__t_1 {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar__mlift4792_instant_fun5888_1(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_mlift4792_instant_fun5888_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar__mlift4792_instant_fun5888_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar__mlift4792_instant_fun5888_1(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i01, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5078_5072 = kk_std_time_duration_zero; /*std/time/duration/duration*/;
  kk_string_t _b_5079_5073;
  kk_std_core_types__optional _match_5309 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5309)) {
    kk_box_t _box_x5071 = _match_5309._cons.Optional.value;
    kk_string_t _abbrv_26530 = kk_string_unbox(_box_x5071);
    kk_string_dup(_abbrv_26530);
    kk_std_core_types__optional_drop(_match_5309, _ctx);
    kk_Unit;
    _b_5079_5073 = _abbrv_26530; /*string*/
    goto _match5889;
  }
  _b_5079_5073 = kk_string_empty(); /*string*/
  _match5889: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5078_5072, _ctx), kk_string_box(_b_5079_5073), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__mlift4792_instant_fun5892__t_1 {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar__mlift4792_instant_fun5892_1(kk_function_t _fself, kk_std_time_instant__instant i000, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_mlift4792_instant_fun5892_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar__mlift4792_instant_fun5892_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar__mlift4792_instant_fun5892_1(kk_function_t _fself, kk_std_time_instant__instant i000, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  bool _match_5308;
  double _x5893;
  kk_std_time_duration__duration _x5894 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  kk_std_num_ddouble__ddouble _x000 = _x5894.secs;
  double _x20 = _x000.hi;
  _x5893 = _x20; /*double*/
  _match_5308 = (_x5893 == (0.0)); /*bool*/
  if (_match_5308) {
    kk_std_time_instant__instant _b_5080_5074 = i000; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_5080_5074, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i000, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
static kk_std_time_instant__instant kk_std_time_calendar__mlift4792_instant_fun5879_1(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4792_instant_fun5879__t_1* _self = kk_function_as(struct kk_std_time_calendar__mlift4792_instant_fun5879__t_1*, _fself);
  kk_std_time_instant__timescale _c_4782 = _self->_c_4782; /* std/time/instant/timescale */
  kk_std_core_types__optional c = _self->c; /* optional<std/time/date/clock> */
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_time_date__date d = _self->d; /* std/time/date/date */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_drop_match(_self, {kk_std_time_instant__timescale_dup(_c_4782);kk_std_core_types__optional_dup(c);kk_std_core_types__optional_dup(cal);kk_std_time_date__date_dup(d);kk_std_core_types__optional_dup(tz);}, {}, _ctx)
  kk_function_t _x5895;
  kk_std_time_calendar__calendar _match_5310;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5075 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_2982 = kk_std_time_calendar__calendar_unbox(_box_x5075, NULL);
    kk_std_time_calendar__calendar_dup(_cal_2982);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5310 = _cal_2982; /*std/time/calendar/calendar*/
    goto _match5896;
  }
  _match_5310 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match5896: ;
  struct kk_std_time_calendar_Calendar* _con5898 = kk_std_time_calendar__as_Calendar(_match_5310);
  kk_string_t _pat0 = _con5898->name;
  kk_string_t _pat1 = _con5898->long_name;
  kk_string_t _pat2 = _con5898->month_prefix;
  kk_function_t _pat3 = _con5898->show_era;
  kk_function_t _pat4 = _con5898->instant_to_dc;
  kk_function_t _x = _con5898->dc_to_instant;
  kk_function_t _pat50 = _con5898->days_to_date;
  kk_function_t _pat60 = _con5898->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5310))) {
    kk_function_drop(_pat60, _ctx);
    kk_Unit;
    kk_function_drop(_pat50, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5310);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5310, _ctx);
    kk_Unit;
  }
  _x5895 = _x; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  kk_std_time_date__clock _x5880;
  if (kk_std_core_types__is_Optional(c)) {
    kk_box_t _box_x5069 = c._cons.Optional.value;
    kk_std_time_date__clock _c_2974 = kk_std_time_date__clock_unbox(_box_x5069, NULL);
    kk_std_time_date__clock_dup(_c_2974);
    kk_std_core_types__optional_drop(c, _ctx);
    kk_Unit;
    _x5880 = _c_2974; /*std/time/date/clock*/
    goto _match5881;
  }
  _x5880 = kk_std_time_date__clock_dup(kk_std_time_date_clock0); /*std/time/date/clock*/
  _match5881: ;
  kk_std_time_calendar__timezone _x5883;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5070 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_2978 = kk_std_time_calendar__timezone_unbox(_box_x5070, NULL);
    kk_std_time_calendar__timezone_dup(_tz_2978);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _x5883 = _tz_2978; /*std/time/calendar/timezone*/
    goto _match5884;
  }
  kk_string_t _x5886;
  kk_define_string_literal(, _s5887, 3, "UTC")
  _x5886 = kk_string_dup(_s5887); /*string*/
  _x5883 = kk_std_time_calendar__new_Timezone(_x5886, kk_std_time_calendar__new_mlift4792_instant_fun5888_1(_ctx), kk_std_time_calendar__new_mlift4792_instant_fun5892_1(_ctx), _ctx); /*std/time/calendar/timezone*/
  _match5884: ;
  return kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x5895, (_x5895, d, _x5880, _x5883, _c_4782, _ctx));
}


// lift anonymous function
struct kk_std_time_calendar__mlift4792_instant_fun5900__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_5077_5076;
};
static kk_box_t kk_std_time_calendar__mlift4792_instant_fun5900_1(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_mlift4792_instant_fun5900_1(kk_function_t _b_5077_5076, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4792_instant_fun5900__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__mlift4792_instant_fun5900__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__mlift4792_instant_fun5900_1, kk_context());
  _self->_b_5077_5076 = _b_5077_5076;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar__mlift4792_instant_fun5900_1(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4792_instant_fun5900__t_1* _self = kk_function_as(struct kk_std_time_calendar__mlift4792_instant_fun5900__t_1*, _fself);
  kk_function_t _b_5077_5076 = _self->_b_5077_5076; /* () -> std/time/utc/utc std/time/instant/instant */
  kk_drop_match(_self, {kk_function_dup(_b_5077_5076);}, {}, _ctx)
  kk_std_time_instant__instant _x5901 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_context_t*), _b_5077_5076, (_b_5077_5076, _ctx)); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x5901, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar__mlift4792_instant_1(kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_std_time_instant__timescale _c_4782, kk_context_t* _ctx) { /* (c : optional<std/time/date/clock>, cal : optional<calendar>, d : std/time/date/date, tz : optional<timezone>, std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_function_t _b_5077_5076 = kk_std_time_calendar__new_mlift4792_instant_fun5879_1(_c_4782, c, cal, d, tz, _ctx); /*() -> std/time/utc/utc std/time/instant/instant*/;
  kk_box_t _x5899 = kk_std_core_hnd__open_none0(kk_std_time_calendar__new_mlift4792_instant_fun5900_1(_b_5077_5076, _ctx), _ctx); /*3212*/
  return kk_std_time_instant__instant_unbox(_x5899, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5908__t_1 {
  struct kk_function_s _base;
  kk_std_core_types__optional c;
  kk_std_core_types__optional cal;
  kk_std_time_date__date d;
  kk_std_core_types__optional tz;
};
static kk_std_time_instant__instant kk_std_time_calendar_instant_fun5908_1(kk_function_t _fself, kk_std_time_instant__timescale _c_4782, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5908_1(kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5908__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun5908__t_1, 10, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun5908_1, kk_context());
  _self->c = c;
  _self->cal = cal;
  _self->d = d;
  _self->tz = tz;
  return &_self->_base;
}

static kk_std_time_instant__instant kk_std_time_calendar_instant_fun5908_1(kk_function_t _fself, kk_std_time_instant__timescale _c_4782, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5908__t_1* _self = kk_function_as(struct kk_std_time_calendar_instant_fun5908__t_1*, _fself);
  kk_std_core_types__optional c = _self->c; /* optional<std/time/date/clock> */
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_time_date__date d = _self->d; /* std/time/date/date */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(c);kk_std_core_types__optional_dup(cal);kk_std_time_date__date_dup(d);kk_std_core_types__optional_dup(tz);}, {}, _ctx)
  return kk_std_time_calendar__mlift4792_instant_1(c, cal, d, tz, _c_4782, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5910__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_5098_5088;
};
static kk_box_t kk_std_time_calendar_instant_fun5910_1(kk_function_t _fself, kk_box_t _b_5089, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5910_1(kk_function_t _b_5098_5088, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5910__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun5910__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun5910_1, kk_context());
  _self->_b_5098_5088 = _b_5098_5088;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_instant_fun5910_1(kk_function_t _fself, kk_box_t _b_5089, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5910__t_1* _self = kk_function_as(struct kk_std_time_calendar_instant_fun5910__t_1*, _fself);
  kk_function_t _b_5098_5088 = _self->_b_5098_5088; /* (std/time/instant/timescale) -> std/time/instant/instant */
  kk_drop_match(_self, {kk_function_dup(_b_5098_5088);}, {}, _ctx)
  kk_std_time_instant__instant _x5911;
  kk_std_time_instant__timescale _x5912 = kk_std_time_instant__timescale_unbox(_b_5089, _ctx); /*std/time/instant/timescale*/
  _x5911 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_instant__timescale, kk_context_t*), _b_5098_5088, (_b_5098_5088, _x5912, _ctx)); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x5911, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5913__t_1 {
  struct kk_function_s _base;
  kk_std_core_types__optional c;
  kk_std_core_types__optional cal;
  kk_std_time_date__date d;
  kk_std_core_types__optional tz;
  kk_std_time_instant__timescale x_4795;
};
static kk_std_time_instant__instant kk_std_time_calendar_instant_fun5913_1(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5913_1(kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_std_time_instant__timescale x_4795, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5913__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun5913__t_1, 11, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun5913_1, kk_context());
  _self->c = c;
  _self->cal = cal;
  _self->d = d;
  _self->tz = tz;
  _self->x_4795 = x_4795;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_time_calendar_instant_fun5922__t_1 {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_instant_fun5922_1(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5922_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_instant_fun5922_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_instant_fun5922_1(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i01, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5100_5093 = kk_std_time_duration_zero; /*std/time/duration/duration*/;
  kk_string_t _b_5101_5094;
  kk_std_core_types__optional _match_5304 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5304)) {
    kk_box_t _box_x5092 = _match_5304._cons.Optional.value;
    kk_string_t _abbrv_26530 = kk_string_unbox(_box_x5092);
    kk_string_dup(_abbrv_26530);
    kk_std_core_types__optional_drop(_match_5304, _ctx);
    kk_Unit;
    _b_5101_5094 = _abbrv_26530; /*string*/
    goto _match5923;
  }
  _b_5101_5094 = kk_string_empty(); /*string*/
  _match5923: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5100_5093, _ctx), kk_string_box(_b_5101_5094), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5926__t_1 {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_instant_fun5926_1(kk_function_t _fself, kk_std_time_instant__instant i000, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5926_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_instant_fun5926_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar_instant_fun5926_1(kk_function_t _fself, kk_std_time_instant__instant i000, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  bool _match_5303;
  double _x5927;
  kk_std_time_duration__duration _x5928 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  kk_std_num_ddouble__ddouble _x000 = _x5928.secs;
  double _x20 = _x000.hi;
  _x5927 = _x20; /*double*/
  _match_5303 = (_x5927 == (0.0)); /*bool*/
  if (_match_5303) {
    kk_std_time_instant__instant _b_5102_5095 = i000; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_5102_5095, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i000, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
static kk_std_time_instant__instant kk_std_time_calendar_instant_fun5913_1(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5913__t_1* _self = kk_function_as(struct kk_std_time_calendar_instant_fun5913__t_1*, _fself);
  kk_std_core_types__optional c = _self->c; /* optional<std/time/date/clock> */
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_time_date__date d = _self->d; /* std/time/date/date */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_std_time_instant__timescale x_4795 = _self->x_4795; /* std/time/instant/timescale */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(c);kk_std_core_types__optional_dup(cal);kk_std_time_date__date_dup(d);kk_std_core_types__optional_dup(tz);kk_std_time_instant__timescale_dup(x_4795);}, {}, _ctx)
  kk_function_t _x5929;
  kk_std_time_calendar__calendar _match_5305;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5096 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_2982 = kk_std_time_calendar__calendar_unbox(_box_x5096, NULL);
    kk_std_time_calendar__calendar_dup(_cal_2982);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5305 = _cal_2982; /*std/time/calendar/calendar*/
    goto _match5930;
  }
  _match_5305 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match5930: ;
  struct kk_std_time_calendar_Calendar* _con5932 = kk_std_time_calendar__as_Calendar(_match_5305);
  kk_string_t _pat01 = _con5932->name;
  kk_string_t _pat11 = _con5932->long_name;
  kk_string_t _pat20 = _con5932->month_prefix;
  kk_function_t _pat30 = _con5932->show_era;
  kk_function_t _pat40 = _con5932->instant_to_dc;
  kk_function_t _x = _con5932->dc_to_instant;
  kk_function_t _pat50 = _con5932->days_to_date;
  kk_function_t _pat60 = _con5932->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5305))) {
    kk_function_drop(_pat60, _ctx);
    kk_Unit;
    kk_function_drop(_pat50, _ctx);
    kk_Unit;
    kk_function_drop(_pat40, _ctx);
    kk_Unit;
    kk_function_drop(_pat30, _ctx);
    kk_Unit;
    kk_string_drop(_pat20, _ctx);
    kk_Unit;
    kk_string_drop(_pat11, _ctx);
    kk_Unit;
    kk_string_drop(_pat01, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5305);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5305, _ctx);
    kk_Unit;
  }
  _x5929 = _x; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  kk_std_time_date__clock _x5914;
  if (kk_std_core_types__is_Optional(c)) {
    kk_box_t _box_x5090 = c._cons.Optional.value;
    kk_std_time_date__clock _c_2974 = kk_std_time_date__clock_unbox(_box_x5090, NULL);
    kk_std_time_date__clock_dup(_c_2974);
    kk_std_core_types__optional_drop(c, _ctx);
    kk_Unit;
    _x5914 = _c_2974; /*std/time/date/clock*/
    goto _match5915;
  }
  _x5914 = kk_std_time_date__clock_dup(kk_std_time_date_clock0); /*std/time/date/clock*/
  _match5915: ;
  kk_std_time_calendar__timezone _x5917;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5091 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_2978 = kk_std_time_calendar__timezone_unbox(_box_x5091, NULL);
    kk_std_time_calendar__timezone_dup(_tz_2978);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _x5917 = _tz_2978; /*std/time/calendar/timezone*/
    goto _match5918;
  }
  kk_string_t _x5920;
  kk_define_string_literal(, _s5921, 3, "UTC")
  _x5920 = kk_string_dup(_s5921); /*string*/
  _x5917 = kk_std_time_calendar__new_Timezone(_x5920, kk_std_time_calendar_new_instant_fun5922_1(_ctx), kk_std_time_calendar_new_instant_fun5926_1(_ctx), _ctx); /*std/time/calendar/timezone*/
  _match5918: ;
  return kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x5929, (_x5929, d, _x5914, _x5917, x_4795, _ctx));
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5934__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_5099_5097;
};
static kk_box_t kk_std_time_calendar_instant_fun5934_1(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5934_1(kk_function_t _b_5099_5097, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5934__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun5934__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun5934_1, kk_context());
  _self->_b_5099_5097 = _b_5099_5097;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_instant_fun5934_1(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun5934__t_1* _self = kk_function_as(struct kk_std_time_calendar_instant_fun5934__t_1*, _fself);
  kk_function_t _b_5099_5097 = _self->_b_5099_5097; /* () -> std/time/utc/utc std/time/instant/instant */
  kk_drop_match(_self, {kk_function_dup(_b_5099_5097);}, {}, _ctx)
  kk_std_time_instant__instant _x5935 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_context_t*), _b_5099_5097, (_b_5099_5097, _ctx)); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x5935, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_instant_1(kk_std_time_date__date d, kk_std_core_types__optional c, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_std_core_types__optional ts, kk_context_t* _ctx) { /* (d : std/time/date/date, c : optional<std/time/date/clock>, tz : optional<timezone>, cal : optional<calendar>, ts : optional<std/time/instant/timescale>) -> std/time/utc/utc std/time/instant/instant */ 
  kk_std_time_instant__timescale x_4795;
  if (kk_std_core_types__is_Optional(ts)) {
    kk_box_t _box_x5081 = ts._cons.Optional.value;
    kk_std_time_instant__timescale _ts_2988 = kk_std_time_instant__timescale_unbox(_box_x5081, NULL);
    kk_std_time_instant__timescale_dup(_ts_2988);
    kk_std_core_types__optional_drop(ts, _ctx);
    kk_Unit;
    x_4795 = _ts_2988; /*std/time/instant/timescale*/
    goto _match5902;
  }
  kk_std_core_hnd__ev ev_4798;
  kk_ssize_t _x5904 = ((kk_ssize_t)0); /*ssize_t*/
  ev_4798 = kk_evv_at(_x5904,kk_context()); /*std/core/hnd/ev<std/time/utc/.hnd-utc>*/
  kk_box_t _x5905;
  struct kk_std_core_hnd_Ev* _con5906 = kk_std_core_hnd__as_Ev(ev_4798);
  kk_std_core_hnd__marker m0 = _con5906->marker;
  kk_box_t _box_x5082 = _con5906->hnd;
  kk_std_time_utc__hnd_utc h = kk_std_time_utc__hnd_utc_unbox(_box_x5082, NULL);
  kk_std_time_utc__hnd_utc_dup(h);
  kk_std_core_hnd__clause0 _match_5306;
  kk_std_core_hnd__clause0 _borrowApp_5307 = kk_std_time_utc__select_utc(h, _ctx); /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/;
  kk_std_time_utc__hnd_utc_drop(h, _ctx);
  _match_5306 = _borrowApp_5307; /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/
  kk_function_t _fun_unbox_x5085 = _match_5306.clause;
  _x5905 = kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_context_t*), _fun_unbox_x5085, (_fun_unbox_x5085, m0, ev_4798, _ctx)); /*37*/
  x_4795 = kk_std_time_instant__timescale_unbox(_x5905, _ctx); /*std/time/instant/timescale*/
  _match5902: ;
  if (kk_yielding(kk_context())) {
    kk_std_time_instant__timescale_dropn(x_4795, ((int32_t)10), _ctx);
    kk_function_t _b_5098_5088 = kk_std_time_calendar_new_instant_fun5908_1(c, cal, d, tz, _ctx); /*(std/time/instant/timescale) -> std/time/instant/instant*/;
    kk_box_t _x5909 = kk_std_core_hnd_yield_extend(kk_std_time_calendar_new_instant_fun5910_1(_b_5098_5088, _ctx), _ctx); /*3858*/
    return kk_std_time_instant__instant_unbox(_x5909, _ctx);
  }
  kk_function_t _b_5099_5097 = kk_std_time_calendar_new_instant_fun5913_1(c, cal, d, tz, x_4795, _ctx); /*() -> std/time/utc/utc std/time/instant/instant*/;
  kk_box_t _x5933 = kk_std_core_hnd__open_none0(kk_std_time_calendar_new_instant_fun5934_1(_b_5099_5097, _ctx), _ctx); /*3212*/
  return kk_std_time_instant__instant_unbox(_x5933, _ctx);
}
 
/* Return the instant in time for a given date and clock interpreted by
   calendar `cal` (=`cal-iso`) in a timezone `tz` (=`tz-utc` by default).
  The `month`, `day`, `hour`, `minutes` may be outside their usual ranges
  and will be normalized during the conversion. For example, January 33 converts to February 2.
  This makes it very easy to add- or subtract days or months to an existing time.
  When the `seconds` or fraction of seconds `frac` add up over 60 those extra seconds are
  interpreted as leap seconds.
  Due to timezone transitions, or leap seconds, it is possible to specify dates that never
  happened (as it was skipped by a timezone change), or ambigious times (as a timezone springs back).
  In such cases, the time is always interpreted in the earlier timezone.
*/


// lift anonymous function
struct kk_std_time_calendar_instant_fun5963__t_2 {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_instant_fun5963_2(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5963_2(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_instant_fun5963_2, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_instant_fun5963_2(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5115_5111 = kk_std_time_duration_zero; /*std/time/duration/duration*/;
  kk_string_t _b_5116_5112;
  kk_std_core_types__optional _match_5300 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5300)) {
    kk_box_t _box_x5110 = _match_5300._cons.Optional.value;
    kk_string_t _abbrv_2653 = kk_string_unbox(_box_x5110);
    kk_string_dup(_abbrv_2653);
    kk_std_core_types__optional_drop(_match_5300, _ctx);
    kk_Unit;
    _b_5116_5112 = _abbrv_2653; /*string*/
    goto _match5964;
  }
  _b_5116_5112 = kk_string_empty(); /*string*/
  _match5964: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5115_5111, _ctx), kk_string_box(_b_5116_5112), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun5967__t_2 {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_instant_fun5967_2(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun5967_2(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_instant_fun5967_2, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar_instant_fun5967_2(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  bool _match_5299;
  double _x5968;
  kk_std_time_duration__duration _x5969 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  kk_std_num_ddouble__ddouble _x0 = _x5969.secs;
  double _x = _x0.hi;
  _x5968 = _x; /*double*/
  _match_5299 = (_x5968 == (0.0)); /*bool*/
  if (_match_5299) {
    kk_std_time_instant__instant _b_5117_5113 = i00; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_5117_5113, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i00, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_instant_2(kk_std_time_instant__timescale ts, kk_integer_t year, kk_std_core_types__optional month, kk_std_core_types__optional day, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional seconds, kk_std_core_types__optional frac, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (ts : std/time/instant/timescale, year : int, month : optional<int>, day : optional<int>, hours : optional<int>, minutes : optional<int>, seconds : optional<int>, frac : optional<double>, tz : optional<timezone>, cal : optional<calendar>) -> std/time/instant/instant */ 
  kk_std_num_ddouble__ddouble fsecs;
  kk_std_num_ddouble__ddouble _x5936;
  kk_integer_t _x5937;
  if (kk_std_core_types__is_Optional(seconds)) {
    kk_box_t _box_x5103 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3041 = kk_integer_unbox(_box_x5103);
    kk_integer_dup(_seconds_3041);
    kk_std_core_types__optional_drop(seconds, _ctx);
    kk_Unit;
    _x5937 = _seconds_3041; /*int*/
    goto _match5938;
  }
  _x5937 = kk_integer_from_small(0); /*int*/
  _match5938: ;
  _x5936 = kk_std_num_ddouble_ddouble_int_exp(_x5937, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
  kk_std_num_ddouble__ddouble _x5940;
  double _x5941;
  if (kk_std_core_types__is_Optional(frac)) {
    kk_box_t _box_x5104 = frac._cons.Optional.value;
    double _frac_3045 = kk_double_unbox(_box_x5104, NULL);
    kk_std_core_types__optional_drop(frac, _ctx);
    kk_Unit;
    _x5941 = _frac_3045; /*double*/
    goto _match5942;
  }
  _x5941 = 0.0; /*double*/
  _match5942: ;
  _x5940 = kk_std_num_ddouble__new_Ddouble(_x5941, 0.0, _ctx); /*std/num/ddouble/ddouble*/
  fsecs = kk_std_num_ddouble__lp__plus__rp_(_x5936, _x5940, _ctx); /*std/num/ddouble/ddouble*/
  kk_function_t _x5970;
  kk_std_time_calendar__calendar _match_5301;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5114 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3053 = kk_std_time_calendar__calendar_unbox(_box_x5114, NULL);
    kk_std_time_calendar__calendar_dup(_cal_3053);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5301 = _cal_3053; /*std/time/calendar/calendar*/
    goto _match5971;
  }
  _match_5301 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match5971: ;
  struct kk_std_time_calendar_Calendar* _con5973 = kk_std_time_calendar__as_Calendar(_match_5301);
  kk_string_t _pat02 = _con5973->name;
  kk_string_t _pat12 = _con5973->long_name;
  kk_string_t _pat22 = _con5973->month_prefix;
  kk_function_t _pat32 = _con5973->show_era;
  kk_function_t _pat4 = _con5973->instant_to_dc;
  kk_function_t _x1 = _con5973->dc_to_instant;
  kk_function_t _pat5 = _con5973->days_to_date;
  kk_function_t _pat6 = _con5973->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5301))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat32, _ctx);
    kk_Unit;
    kk_string_drop(_pat22, _ctx);
    kk_Unit;
    kk_string_drop(_pat12, _ctx);
    kk_Unit;
    kk_string_drop(_pat02, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5301);
  }
  else {
    kk_function_dup(_x1);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5301, _ctx);
    kk_Unit;
  }
  _x5970 = _x1; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  kk_std_time_date__date _x5944;
  kk_integer_t _x5945;
  if (kk_std_core_types__is_Optional(month)) {
    kk_box_t _box_x5105 = month._cons.Optional.value;
    kk_integer_t _month_3025 = kk_integer_unbox(_box_x5105);
    kk_integer_dup(_month_3025);
    kk_std_core_types__optional_drop(month, _ctx);
    kk_Unit;
    _x5945 = _month_3025; /*int*/
    goto _match5946;
  }
  _x5945 = kk_integer_from_small(1); /*int*/
  _match5946: ;
  kk_integer_t _x5948;
  if (kk_std_core_types__is_Optional(day)) {
    kk_box_t _box_x5106 = day._cons.Optional.value;
    kk_integer_t _day_3029 = kk_integer_unbox(_box_x5106);
    kk_integer_dup(_day_3029);
    kk_std_core_types__optional_drop(day, _ctx);
    kk_Unit;
    _x5948 = _day_3029; /*int*/
    goto _match5949;
  }
  _x5948 = kk_integer_from_small(1); /*int*/
  _match5949: ;
  _x5944 = kk_std_time_date__new_Date(year, _x5945, _x5948, _ctx); /*std/time/date/date*/
  kk_std_time_date__clock _x5951;
  kk_integer_t _x5952;
  if (kk_std_core_types__is_Optional(hours)) {
    kk_box_t _box_x5107 = hours._cons.Optional.value;
    kk_integer_t _hours_3033 = kk_integer_unbox(_box_x5107);
    kk_integer_dup(_hours_3033);
    kk_std_core_types__optional_drop(hours, _ctx);
    kk_Unit;
    _x5952 = _hours_3033; /*int*/
    goto _match5953;
  }
  _x5952 = kk_integer_from_small(0); /*int*/
  _match5953: ;
  kk_integer_t _x5955;
  if (kk_std_core_types__is_Optional(minutes)) {
    kk_box_t _box_x5108 = minutes._cons.Optional.value;
    kk_integer_t _minutes_3037 = kk_integer_unbox(_box_x5108);
    kk_integer_dup(_minutes_3037);
    kk_std_core_types__optional_drop(minutes, _ctx);
    kk_Unit;
    _x5955 = _minutes_3037; /*int*/
    goto _match5956;
  }
  _x5955 = kk_integer_from_small(0); /*int*/
  _match5956: ;
  _x5951 = kk_std_time_date__new_Clock(kk_reuse_null, _x5952, _x5955, fsecs, _ctx); /*std/time/date/clock*/
  kk_std_time_calendar__timezone _x5958;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5109 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_3049 = kk_std_time_calendar__timezone_unbox(_box_x5109, NULL);
    kk_std_time_calendar__timezone_dup(_tz_3049);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _x5958 = _tz_3049; /*std/time/calendar/timezone*/
    goto _match5959;
  }
  kk_string_t _x5961;
  kk_define_string_literal(, _s5962, 3, "UTC")
  _x5961 = kk_string_dup(_s5962); /*string*/
  _x5958 = kk_std_time_calendar__new_Timezone(_x5961, kk_std_time_calendar_new_instant_fun5963_2(_ctx), kk_std_time_calendar_new_instant_fun5967_2(_ctx), _ctx); /*std/time/calendar/timezone*/
  _match5959: ;
  return kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x5970, (_x5970, _x5944, _x5951, _x5958, ts, _ctx));
}
 
// monadic lift


// lift anonymous function
struct kk_std_time_calendar__mlift4793_instant_fun5974__t_3 {
  struct kk_function_s _base;
  kk_std_time_instant__timescale _c_4784;
  kk_std_core_types__optional cal;
  kk_std_core_types__optional day;
  kk_std_core_types__optional frac;
  kk_std_core_types__optional hours;
  kk_std_core_types__optional minutes;
  kk_std_core_types__optional month;
  kk_std_core_types__optional seconds;
  kk_std_core_types__optional tz;
  kk_integer_t year;
};
static kk_std_time_instant__instant kk_std_time_calendar__mlift4793_instant_fun5974_3(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_mlift4793_instant_fun5974_3(kk_std_time_instant__timescale _c_4784, kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_integer_t year, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4793_instant_fun5974__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar__mlift4793_instant_fun5974__t_3, 19, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__mlift4793_instant_fun5974_3, kk_context());
  _self->_c_4784 = _c_4784;
  _self->cal = cal;
  _self->day = day;
  _self->frac = frac;
  _self->hours = hours;
  _self->minutes = minutes;
  _self->month = month;
  _self->seconds = seconds;
  _self->tz = tz;
  _self->year = year;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_time_calendar__mlift4793_instant_fun5998__t_3 {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar__mlift4793_instant_fun5998_3(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_mlift4793_instant_fun5998_3(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar__mlift4793_instant_fun5998_3, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar__mlift4793_instant_fun5998_3(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5147_5132 = kk_std_time_duration_zero; /*std/time/duration/duration*/;
  kk_string_t _b_5148_5133;
  kk_std_core_types__optional _match_5298 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5298)) {
    kk_box_t _box_x5131 = _match_5298._cons.Optional.value;
    kk_string_t _abbrv_2653 = kk_string_unbox(_box_x5131);
    kk_string_dup(_abbrv_2653);
    kk_std_core_types__optional_drop(_match_5298, _ctx);
    kk_Unit;
    _b_5148_5133 = _abbrv_2653; /*string*/
    goto _match5999;
  }
  _b_5148_5133 = kk_string_empty(); /*string*/
  _match5999: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5147_5132, _ctx), kk_string_box(_b_5148_5133), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__mlift4793_instant_fun6002__t_3 {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar__mlift4793_instant_fun6002_3(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_mlift4793_instant_fun6002_3(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar__mlift4793_instant_fun6002_3, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar__mlift4793_instant_fun6002_3(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  bool _match_5297;
  double _x6003;
  kk_std_time_duration__duration _x6004 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  kk_std_num_ddouble__ddouble _x00 = _x6004.secs;
  double _x = _x00.hi;
  _x6003 = _x; /*double*/
  _match_5297 = (_x6003 == (0.0)); /*bool*/
  if (_match_5297) {
    kk_std_time_instant__instant _b_5149_5134 = i00; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_5149_5134, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i00, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
static kk_std_time_instant__instant kk_std_time_calendar__mlift4793_instant_fun5974_3(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4793_instant_fun5974__t_3* _self = kk_function_as(struct kk_std_time_calendar__mlift4793_instant_fun5974__t_3*, _fself);
  kk_std_time_instant__timescale _c_4784 = _self->_c_4784; /* std/time/instant/timescale */
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_core_types__optional day = _self->day; /* optional<int> */
  kk_std_core_types__optional frac = _self->frac; /* optional<double> */
  kk_std_core_types__optional hours = _self->hours; /* optional<int> */
  kk_std_core_types__optional minutes = _self->minutes; /* optional<int> */
  kk_std_core_types__optional month = _self->month; /* optional<int> */
  kk_std_core_types__optional seconds = _self->seconds; /* optional<int> */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_integer_t year = _self->year; /* int */
  kk_drop_match(_self, {kk_std_time_instant__timescale_dup(_c_4784);kk_std_core_types__optional_dup(cal);kk_std_core_types__optional_dup(day);kk_std_core_types__optional_dup(frac);kk_std_core_types__optional_dup(hours);kk_std_core_types__optional_dup(minutes);kk_std_core_types__optional_dup(month);kk_std_core_types__optional_dup(seconds);kk_std_core_types__optional_dup(tz);kk_integer_dup(year);}, {}, _ctx)
  kk_std_core_types__optional _x5975;
  kk_integer_t _b_5140_5119;
  if (kk_std_core_types__is_Optional(month)) {
    kk_box_t _box_x5118 = month._cons.Optional.value;
    kk_integer_t _month_3139 = kk_integer_unbox(_box_x5118);
    kk_integer_dup(_month_3139);
    kk_std_core_types__optional_drop(month, _ctx);
    kk_Unit;
    _b_5140_5119 = _month_3139; /*int*/
    goto _match5976;
  }
  _b_5140_5119 = kk_integer_from_small(1); /*int*/
  _match5976: ;
  _x5975 = kk_std_core_types__new_Optional(kk_integer_box(_b_5140_5119), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5978;
  kk_integer_t _b_5141_5121;
  if (kk_std_core_types__is_Optional(day)) {
    kk_box_t _box_x5120 = day._cons.Optional.value;
    kk_integer_t _day_3143 = kk_integer_unbox(_box_x5120);
    kk_integer_dup(_day_3143);
    kk_std_core_types__optional_drop(day, _ctx);
    kk_Unit;
    _b_5141_5121 = _day_3143; /*int*/
    goto _match5979;
  }
  _b_5141_5121 = kk_integer_from_small(1); /*int*/
  _match5979: ;
  _x5978 = kk_std_core_types__new_Optional(kk_integer_box(_b_5141_5121), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5981;
  kk_integer_t _b_5142_5123;
  if (kk_std_core_types__is_Optional(hours)) {
    kk_box_t _box_x5122 = hours._cons.Optional.value;
    kk_integer_t _hours_3147 = kk_integer_unbox(_box_x5122);
    kk_integer_dup(_hours_3147);
    kk_std_core_types__optional_drop(hours, _ctx);
    kk_Unit;
    _b_5142_5123 = _hours_3147; /*int*/
    goto _match5982;
  }
  _b_5142_5123 = kk_integer_from_small(0); /*int*/
  _match5982: ;
  _x5981 = kk_std_core_types__new_Optional(kk_integer_box(_b_5142_5123), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5984;
  kk_integer_t _b_5143_5125;
  if (kk_std_core_types__is_Optional(minutes)) {
    kk_box_t _box_x5124 = minutes._cons.Optional.value;
    kk_integer_t _minutes_3151 = kk_integer_unbox(_box_x5124);
    kk_integer_dup(_minutes_3151);
    kk_std_core_types__optional_drop(minutes, _ctx);
    kk_Unit;
    _b_5143_5125 = _minutes_3151; /*int*/
    goto _match5985;
  }
  _b_5143_5125 = kk_integer_from_small(0); /*int*/
  _match5985: ;
  _x5984 = kk_std_core_types__new_Optional(kk_integer_box(_b_5143_5125), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5987;
  kk_integer_t _b_5144_5127;
  if (kk_std_core_types__is_Optional(seconds)) {
    kk_box_t _box_x5126 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3155 = kk_integer_unbox(_box_x5126);
    kk_integer_dup(_seconds_3155);
    kk_std_core_types__optional_drop(seconds, _ctx);
    kk_Unit;
    _b_5144_5127 = _seconds_3155; /*int*/
    goto _match5988;
  }
  _b_5144_5127 = kk_integer_from_small(0); /*int*/
  _match5988: ;
  _x5987 = kk_std_core_types__new_Optional(kk_integer_box(_b_5144_5127), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5990;
  double _b_5145_5129;
  if (kk_std_core_types__is_Optional(frac)) {
    kk_box_t _box_x5128 = frac._cons.Optional.value;
    double _frac_3159 = kk_double_unbox(_box_x5128, NULL);
    kk_std_core_types__optional_drop(frac, _ctx);
    kk_Unit;
    _b_5145_5129 = _frac_3159; /*double*/
    goto _match5991;
  }
  _b_5145_5129 = 0.0; /*double*/
  _match5991: ;
  _x5990 = kk_std_core_types__new_Optional(kk_double_box(_b_5145_5129, _ctx), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5993;
  kk_std_time_calendar__timezone _b_5146_5135;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5130 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_3163 = kk_std_time_calendar__timezone_unbox(_box_x5130, NULL);
    kk_std_time_calendar__timezone_dup(_tz_3163);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _b_5146_5135 = _tz_3163; /*std/time/calendar/timezone*/
    goto _match5994;
  }
  kk_string_t _x5996;
  kk_define_string_literal(, _s5997, 3, "UTC")
  _x5996 = kk_string_dup(_s5997); /*string*/
  _b_5146_5135 = kk_std_time_calendar__new_Timezone(_x5996, kk_std_time_calendar__new_mlift4793_instant_fun5998_3(_ctx), kk_std_time_calendar__new_mlift4793_instant_fun6002_3(_ctx), _ctx); /*std/time/calendar/timezone*/
  _match5994: ;
  _x5993 = kk_std_core_types__new_Optional(kk_std_time_calendar__timezone_box(_b_5146_5135, _ctx), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6005;
  kk_std_time_calendar__calendar _b_5150_5137;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5136 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3167 = kk_std_time_calendar__calendar_unbox(_box_x5136, NULL);
    kk_std_time_calendar__calendar_dup(_cal_3167);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _b_5150_5137 = _cal_3167; /*std/time/calendar/calendar*/
    goto _match6006;
  }
  _b_5150_5137 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match6006: ;
  _x6005 = kk_std_core_types__new_Optional(kk_std_time_calendar__calendar_box(_b_5150_5137, _ctx), _ctx); /*optional<106>*/
  return kk_std_time_calendar_instant_2(_c_4784, year, _x5975, _x5978, _x5981, _x5984, _x5987, _x5990, _x5993, _x6005, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__mlift4793_instant_fun6009__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_5139_5138;
};
static kk_box_t kk_std_time_calendar__mlift4793_instant_fun6009_3(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_mlift4793_instant_fun6009_3(kk_function_t _b_5139_5138, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4793_instant_fun6009__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar__mlift4793_instant_fun6009__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__mlift4793_instant_fun6009_3, kk_context());
  _self->_b_5139_5138 = _b_5139_5138;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar__mlift4793_instant_fun6009_3(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar__mlift4793_instant_fun6009__t_3* _self = kk_function_as(struct kk_std_time_calendar__mlift4793_instant_fun6009__t_3*, _fself);
  kk_function_t _b_5139_5138 = _self->_b_5139_5138; /* () -> std/time/utc/utc std/time/instant/instant */
  kk_drop_match(_self, {kk_function_dup(_b_5139_5138);}, {}, _ctx)
  kk_std_time_instant__instant _x6010 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_context_t*), _b_5139_5138, (_b_5139_5138, _ctx)); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x6010, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar__mlift4793_instant_3(kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_integer_t year, kk_std_time_instant__timescale _c_4784, kk_context_t* _ctx) { /* (cal : optional<calendar>, day : optional<int>, frac : optional<double>, hours : optional<int>, minutes : optional<int>, month : optional<int>, seconds : optional<int>, tz : optional<timezone>, year : int, std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_function_t _b_5139_5138 = kk_std_time_calendar__new_mlift4793_instant_fun5974_3(_c_4784, cal, day, frac, hours, minutes, month, seconds, tz, year, _ctx); /*() -> std/time/utc/utc std/time/instant/instant*/;
  kk_box_t _x6008 = kk_std_core_hnd__open_none0(kk_std_time_calendar__new_mlift4793_instant_fun6009_3(_b_5139_5138, _ctx), _ctx); /*3212*/
  return kk_std_time_instant__instant_unbox(_x6008, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6017__t_3 {
  struct kk_function_s _base;
  kk_std_core_types__optional cal;
  kk_std_core_types__optional day;
  kk_std_core_types__optional frac;
  kk_std_core_types__optional hours;
  kk_std_core_types__optional minutes;
  kk_std_core_types__optional month;
  kk_std_core_types__optional seconds;
  kk_std_core_types__optional tz;
  kk_integer_t year;
};
static kk_std_time_instant__instant kk_std_time_calendar_instant_fun6017_3(kk_function_t _fself, kk_std_time_instant__timescale _c_4784, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6017_3(kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_integer_t year, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6017__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun6017__t_3, 18, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun6017_3, kk_context());
  _self->cal = cal;
  _self->day = day;
  _self->frac = frac;
  _self->hours = hours;
  _self->minutes = minutes;
  _self->month = month;
  _self->seconds = seconds;
  _self->tz = tz;
  _self->year = year;
  return &_self->_base;
}

static kk_std_time_instant__instant kk_std_time_calendar_instant_fun6017_3(kk_function_t _fself, kk_std_time_instant__timescale _c_4784, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6017__t_3* _self = kk_function_as(struct kk_std_time_calendar_instant_fun6017__t_3*, _fself);
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_core_types__optional day = _self->day; /* optional<int> */
  kk_std_core_types__optional frac = _self->frac; /* optional<double> */
  kk_std_core_types__optional hours = _self->hours; /* optional<int> */
  kk_std_core_types__optional minutes = _self->minutes; /* optional<int> */
  kk_std_core_types__optional month = _self->month; /* optional<int> */
  kk_std_core_types__optional seconds = _self->seconds; /* optional<int> */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_integer_t year = _self->year; /* int */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(cal);kk_std_core_types__optional_dup(day);kk_std_core_types__optional_dup(frac);kk_std_core_types__optional_dup(hours);kk_std_core_types__optional_dup(minutes);kk_std_core_types__optional_dup(month);kk_std_core_types__optional_dup(seconds);kk_std_core_types__optional_dup(tz);kk_integer_dup(year);}, {}, _ctx)
  return kk_std_time_calendar__mlift4793_instant_3(cal, day, frac, hours, minutes, month, seconds, tz, year, _c_4784, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6019__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_5181_5158;
};
static kk_box_t kk_std_time_calendar_instant_fun6019_3(kk_function_t _fself, kk_box_t _b_5159, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6019_3(kk_function_t _b_5181_5158, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6019__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun6019__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun6019_3, kk_context());
  _self->_b_5181_5158 = _b_5181_5158;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_instant_fun6019_3(kk_function_t _fself, kk_box_t _b_5159, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6019__t_3* _self = kk_function_as(struct kk_std_time_calendar_instant_fun6019__t_3*, _fself);
  kk_function_t _b_5181_5158 = _self->_b_5181_5158; /* (std/time/instant/timescale) -> std/time/instant/instant */
  kk_drop_match(_self, {kk_function_dup(_b_5181_5158);}, {}, _ctx)
  kk_std_time_instant__instant _x6020;
  kk_std_time_instant__timescale _x6021 = kk_std_time_instant__timescale_unbox(_b_5159, _ctx); /*std/time/instant/timescale*/
  _x6020 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_instant__timescale, kk_context_t*), _b_5181_5158, (_b_5181_5158, _x6021, _ctx)); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x6020, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6022__t_3 {
  struct kk_function_s _base;
  kk_std_core_types__optional cal;
  kk_std_core_types__optional day;
  kk_std_core_types__optional frac;
  kk_std_core_types__optional hours;
  kk_std_core_types__optional minutes;
  kk_std_core_types__optional month;
  kk_std_core_types__optional seconds;
  kk_std_core_types__optional tz;
  kk_std_time_instant__timescale x_4800;
  kk_integer_t year;
};
static kk_std_time_instant__instant kk_std_time_calendar_instant_fun6022_3(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6022_3(kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_std_time_instant__timescale x_4800, kk_integer_t year, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6022__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun6022__t_3, 19, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun6022_3, kk_context());
  _self->cal = cal;
  _self->day = day;
  _self->frac = frac;
  _self->hours = hours;
  _self->minutes = minutes;
  _self->month = month;
  _self->seconds = seconds;
  _self->tz = tz;
  _self->x_4800 = x_4800;
  _self->year = year;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_time_calendar_instant_fun6046__t_3 {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_instant_fun6046_3(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6046_3(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_instant_fun6046_3, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_instant_fun6046_3(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5190_5174 = kk_std_time_duration_zero; /*std/time/duration/duration*/;
  kk_string_t _b_5191_5175;
  kk_std_core_types__optional _match_5294 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5294)) {
    kk_box_t _box_x5173 = _match_5294._cons.Optional.value;
    kk_string_t _abbrv_2653 = kk_string_unbox(_box_x5173);
    kk_string_dup(_abbrv_2653);
    kk_std_core_types__optional_drop(_match_5294, _ctx);
    kk_Unit;
    _b_5191_5175 = _abbrv_2653; /*string*/
    goto _match6047;
  }
  _b_5191_5175 = kk_string_empty(); /*string*/
  _match6047: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5190_5174, _ctx), kk_string_box(_b_5191_5175), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6050__t_3 {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_instant_fun6050_3(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6050_3(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_instant_fun6050_3, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar_instant_fun6050_3(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  bool _match_5293;
  double _x6051;
  kk_std_time_duration__duration _x6052 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  kk_std_num_ddouble__ddouble _x00 = _x6052.secs;
  double _x = _x00.hi;
  _x6051 = _x; /*double*/
  _match_5293 = (_x6051 == (0.0)); /*bool*/
  if (_match_5293) {
    kk_std_time_instant__instant _b_5192_5176 = i00; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_5192_5176, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i00, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
static kk_std_time_instant__instant kk_std_time_calendar_instant_fun6022_3(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6022__t_3* _self = kk_function_as(struct kk_std_time_calendar_instant_fun6022__t_3*, _fself);
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_core_types__optional day = _self->day; /* optional<int> */
  kk_std_core_types__optional frac = _self->frac; /* optional<double> */
  kk_std_core_types__optional hours = _self->hours; /* optional<int> */
  kk_std_core_types__optional minutes = _self->minutes; /* optional<int> */
  kk_std_core_types__optional month = _self->month; /* optional<int> */
  kk_std_core_types__optional seconds = _self->seconds; /* optional<int> */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_std_time_instant__timescale x_4800 = _self->x_4800; /* std/time/instant/timescale */
  kk_integer_t year = _self->year; /* int */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(cal);kk_std_core_types__optional_dup(day);kk_std_core_types__optional_dup(frac);kk_std_core_types__optional_dup(hours);kk_std_core_types__optional_dup(minutes);kk_std_core_types__optional_dup(month);kk_std_core_types__optional_dup(seconds);kk_std_core_types__optional_dup(tz);kk_std_time_instant__timescale_dup(x_4800);kk_integer_dup(year);}, {}, _ctx)
  kk_std_core_types__optional _x6023;
  kk_integer_t _b_5183_5161;
  if (kk_std_core_types__is_Optional(month)) {
    kk_box_t _box_x5160 = month._cons.Optional.value;
    kk_integer_t _month_3139 = kk_integer_unbox(_box_x5160);
    kk_integer_dup(_month_3139);
    kk_std_core_types__optional_drop(month, _ctx);
    kk_Unit;
    _b_5183_5161 = _month_3139; /*int*/
    goto _match6024;
  }
  _b_5183_5161 = kk_integer_from_small(1); /*int*/
  _match6024: ;
  _x6023 = kk_std_core_types__new_Optional(kk_integer_box(_b_5183_5161), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6026;
  kk_integer_t _b_5184_5163;
  if (kk_std_core_types__is_Optional(day)) {
    kk_box_t _box_x5162 = day._cons.Optional.value;
    kk_integer_t _day_3143 = kk_integer_unbox(_box_x5162);
    kk_integer_dup(_day_3143);
    kk_std_core_types__optional_drop(day, _ctx);
    kk_Unit;
    _b_5184_5163 = _day_3143; /*int*/
    goto _match6027;
  }
  _b_5184_5163 = kk_integer_from_small(1); /*int*/
  _match6027: ;
  _x6026 = kk_std_core_types__new_Optional(kk_integer_box(_b_5184_5163), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6029;
  kk_integer_t _b_5185_5165;
  if (kk_std_core_types__is_Optional(hours)) {
    kk_box_t _box_x5164 = hours._cons.Optional.value;
    kk_integer_t _hours_3147 = kk_integer_unbox(_box_x5164);
    kk_integer_dup(_hours_3147);
    kk_std_core_types__optional_drop(hours, _ctx);
    kk_Unit;
    _b_5185_5165 = _hours_3147; /*int*/
    goto _match6030;
  }
  _b_5185_5165 = kk_integer_from_small(0); /*int*/
  _match6030: ;
  _x6029 = kk_std_core_types__new_Optional(kk_integer_box(_b_5185_5165), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6032;
  kk_integer_t _b_5186_5167;
  if (kk_std_core_types__is_Optional(minutes)) {
    kk_box_t _box_x5166 = minutes._cons.Optional.value;
    kk_integer_t _minutes_3151 = kk_integer_unbox(_box_x5166);
    kk_integer_dup(_minutes_3151);
    kk_std_core_types__optional_drop(minutes, _ctx);
    kk_Unit;
    _b_5186_5167 = _minutes_3151; /*int*/
    goto _match6033;
  }
  _b_5186_5167 = kk_integer_from_small(0); /*int*/
  _match6033: ;
  _x6032 = kk_std_core_types__new_Optional(kk_integer_box(_b_5186_5167), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6035;
  kk_integer_t _b_5187_5169;
  if (kk_std_core_types__is_Optional(seconds)) {
    kk_box_t _box_x5168 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3155 = kk_integer_unbox(_box_x5168);
    kk_integer_dup(_seconds_3155);
    kk_std_core_types__optional_drop(seconds, _ctx);
    kk_Unit;
    _b_5187_5169 = _seconds_3155; /*int*/
    goto _match6036;
  }
  _b_5187_5169 = kk_integer_from_small(0); /*int*/
  _match6036: ;
  _x6035 = kk_std_core_types__new_Optional(kk_integer_box(_b_5187_5169), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6038;
  double _b_5188_5171;
  if (kk_std_core_types__is_Optional(frac)) {
    kk_box_t _box_x5170 = frac._cons.Optional.value;
    double _frac_3159 = kk_double_unbox(_box_x5170, NULL);
    kk_std_core_types__optional_drop(frac, _ctx);
    kk_Unit;
    _b_5188_5171 = _frac_3159; /*double*/
    goto _match6039;
  }
  _b_5188_5171 = 0.0; /*double*/
  _match6039: ;
  _x6038 = kk_std_core_types__new_Optional(kk_double_box(_b_5188_5171, _ctx), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6041;
  kk_std_time_calendar__timezone _b_5189_5177;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5172 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_3163 = kk_std_time_calendar__timezone_unbox(_box_x5172, NULL);
    kk_std_time_calendar__timezone_dup(_tz_3163);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _b_5189_5177 = _tz_3163; /*std/time/calendar/timezone*/
    goto _match6042;
  }
  kk_string_t _x6044;
  kk_define_string_literal(, _s6045, 3, "UTC")
  _x6044 = kk_string_dup(_s6045); /*string*/
  _b_5189_5177 = kk_std_time_calendar__new_Timezone(_x6044, kk_std_time_calendar_new_instant_fun6046_3(_ctx), kk_std_time_calendar_new_instant_fun6050_3(_ctx), _ctx); /*std/time/calendar/timezone*/
  _match6042: ;
  _x6041 = kk_std_core_types__new_Optional(kk_std_time_calendar__timezone_box(_b_5189_5177, _ctx), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6053;
  kk_std_time_calendar__calendar _b_5193_5179;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5178 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3167 = kk_std_time_calendar__calendar_unbox(_box_x5178, NULL);
    kk_std_time_calendar__calendar_dup(_cal_3167);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _b_5193_5179 = _cal_3167; /*std/time/calendar/calendar*/
    goto _match6054;
  }
  _b_5193_5179 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match6054: ;
  _x6053 = kk_std_core_types__new_Optional(kk_std_time_calendar__calendar_box(_b_5193_5179, _ctx), _ctx); /*optional<106>*/
  return kk_std_time_calendar_instant_2(x_4800, year, _x6023, _x6026, _x6029, _x6032, _x6035, _x6038, _x6041, _x6053, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6057__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_5182_5180;
};
static kk_box_t kk_std_time_calendar_instant_fun6057_3(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6057_3(kk_function_t _b_5182_5180, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6057__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun6057__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun6057_3, kk_context());
  _self->_b_5182_5180 = _b_5182_5180;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_instant_fun6057_3(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6057__t_3* _self = kk_function_as(struct kk_std_time_calendar_instant_fun6057__t_3*, _fself);
  kk_function_t _b_5182_5180 = _self->_b_5182_5180; /* () -> std/time/utc/utc std/time/instant/instant */
  kk_drop_match(_self, {kk_function_dup(_b_5182_5180);}, {}, _ctx)
  kk_std_time_instant__instant _x6058 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_context_t*), _b_5182_5180, (_b_5182_5180, _ctx)); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x6058, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_instant_3(kk_integer_t year, kk_std_core_types__optional month, kk_std_core_types__optional day, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional seconds, kk_std_core_types__optional frac, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_std_core_types__optional ts, kk_context_t* _ctx) { /* (year : int, month : optional<int>, day : optional<int>, hours : optional<int>, minutes : optional<int>, seconds : optional<int>, frac : optional<double>, tz : optional<timezone>, cal : optional<calendar>, ts : optional<std/time/instant/timescale>) -> std/time/utc/utc std/time/instant/instant */ 
  kk_std_time_instant__timescale x_4800;
  if (kk_std_core_types__is_Optional(ts)) {
    kk_box_t _box_x5151 = ts._cons.Optional.value;
    kk_std_time_instant__timescale _ts_3173 = kk_std_time_instant__timescale_unbox(_box_x5151, NULL);
    kk_std_time_instant__timescale_dup(_ts_3173);
    kk_std_core_types__optional_drop(ts, _ctx);
    kk_Unit;
    x_4800 = _ts_3173; /*std/time/instant/timescale*/
    goto _match6011;
  }
  kk_std_core_hnd__ev ev_4803;
  kk_ssize_t _x6013 = ((kk_ssize_t)0); /*ssize_t*/
  ev_4803 = kk_evv_at(_x6013,kk_context()); /*std/core/hnd/ev<std/time/utc/.hnd-utc>*/
  kk_box_t _x6014;
  struct kk_std_core_hnd_Ev* _con6015 = kk_std_core_hnd__as_Ev(ev_4803);
  kk_std_core_hnd__marker m0 = _con6015->marker;
  kk_box_t _box_x5152 = _con6015->hnd;
  kk_std_time_utc__hnd_utc h = kk_std_time_utc__hnd_utc_unbox(_box_x5152, NULL);
  kk_std_time_utc__hnd_utc_dup(h);
  kk_std_core_hnd__clause0 _match_5295;
  kk_std_core_hnd__clause0 _borrowApp_5296 = kk_std_time_utc__select_utc(h, _ctx); /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/;
  kk_std_time_utc__hnd_utc_drop(h, _ctx);
  _match_5295 = _borrowApp_5296; /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/
  kk_function_t _fun_unbox_x5155 = _match_5295.clause;
  _x6014 = kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_context_t*), _fun_unbox_x5155, (_fun_unbox_x5155, m0, ev_4803, _ctx)); /*37*/
  x_4800 = kk_std_time_instant__timescale_unbox(_x6014, _ctx); /*std/time/instant/timescale*/
  _match6011: ;
  if (kk_yielding(kk_context())) {
    kk_std_time_instant__timescale_dropn(x_4800, ((int32_t)10), _ctx);
    kk_function_t _b_5181_5158 = kk_std_time_calendar_new_instant_fun6017_3(cal, day, frac, hours, minutes, month, seconds, tz, year, _ctx); /*(std/time/instant/timescale) -> std/time/instant/instant*/;
    kk_box_t _x6018 = kk_std_core_hnd_yield_extend(kk_std_time_calendar_new_instant_fun6019_3(_b_5181_5158, _ctx), _ctx); /*3858*/
    return kk_std_time_instant__instant_unbox(_x6018, _ctx);
  }
  kk_function_t _b_5182_5180 = kk_std_time_calendar_new_instant_fun6022_3(cal, day, frac, hours, minutes, month, seconds, tz, x_4800, year, _ctx); /*() -> std/time/utc/utc std/time/instant/instant*/;
  kk_box_t _x6056 = kk_std_core_hnd__open_none0(kk_std_time_calendar_new_instant_fun6057_3(_b_5182_5180, _ctx), _ctx); /*3212*/
  return kk_std_time_instant__instant_unbox(_x6056, _ctx);
}
 
// The (proleptic) [Gregorian calendar](https://en.wikipedia.org/wiki/Gregorian_calendar).
// Just like the ISO calendar except that the year 0 does not exist, i.e. after 1 BCE (=`~1`)
// we have 1 CE (=`1`).
// The calendar short name is ``GC``.


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6064__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6064(kk_function_t _fself, kk_integer_t _x16063, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6064(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6064, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6064(kk_function_t _fself, kk_integer_t _x16063, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16063, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6066__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_gregorian_fun6066(kk_function_t _fself, kk_integer_t _x16065, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6066(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6066, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_gregorian_fun6066(kk_function_t _fself, kk_integer_t _x16065, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x16065, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6069__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6069(kk_function_t _fself, kk_integer_t _x16067, kk_integer_t _x26068, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6069(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6069, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6069(kk_function_t _fself, kk_integer_t _x16067, kk_integer_t _x26068, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16067, _x26068, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6072__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6072(kk_function_t _fself, kk_integer_t _x16070, kk_integer_t _x26071, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6072(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6072, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6072(kk_function_t _fself, kk_integer_t _x16070, kk_integer_t _x26071, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16070, _x26071, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6075__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_cal_gregorian_fun6075(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6075(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6075, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_cal_gregorian_fun6075(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  bool _match_5291;
  kk_integer_t _x6076;
  kk_integer_t _x = d.year;
  kk_integer_dup(_x);
  kk_std_time_date__date_drop(d, _ctx);
  kk_Unit;
  _x6076 = _x; /*int*/
  _match_5291 = kk_std_core_is_neg_2(_x6076, _ctx); /*bool*/
  if (_match_5291) {
    kk_define_string_literal(, _s6077, 3, "BCE")
    return kk_string_dup(_s6077);
  }
  kk_define_string_literal(, _s6078, 2, "CE")
  return kk_string_dup(_s6078);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6079__t {
  struct kk_function_s _base;
  kk_function_t _b_5200_5195;
};
static kk_box_t kk_std_time_calendar_cal_gregorian_fun6079(kk_function_t _fself, kk_box_t _b_5198, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6079(kk_function_t _b_5200_5195, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_gregorian_fun6079__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_cal_gregorian_fun6079__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_cal_gregorian_fun6079, kk_context());
  _self->_b_5200_5195 = _b_5200_5195;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_cal_gregorian_fun6079(kk_function_t _fself, kk_box_t _b_5198, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_gregorian_fun6079__t* _self = kk_function_as(struct kk_std_time_calendar_cal_gregorian_fun6079__t*, _fself);
  kk_function_t _b_5200_5195 = _self->_b_5200_5195; /* (d : std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5200_5195);}, {}, _ctx)
  kk_string_t _x6080;
  kk_std_time_date__date _x6081 = kk_std_time_date__date_unbox(_b_5198, _ctx); /*std/time/date/date*/
  _x6080 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5200_5195, (_b_5200_5195, _x6081, _ctx)); /*string*/
  return kk_string_box(_x6080);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_gregorian;
extern kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6087(kk_function_t _fself, kk_integer_t _x16086, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16086, _ctx);
}
extern kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_sls_fun6089(kk_function_t _fself, kk_integer_t _x16088, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x16088, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6092(kk_function_t _fself, kk_integer_t _x16090, kk_integer_t _x26091, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16090, _x26091, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6095(kk_function_t _fself, kk_integer_t _x16093, kk_integer_t _x26094, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16093, _x26094, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_iso_sls;

kk_std_time_timestamp__timestamp kk_std_time_calendar_cal_timestamp(kk_std_time_calendar__calendar cal, kk_std_time_instant__timescale ts, kk_std_time_date__date d, kk_std_time_date__clock c, kk_context_t* _ctx) { /* (cal : calendar, ts : std/time/instant/timescale, d : std/time/date/date, c : std/time/date/clock) -> std/time/timestamp/timestamp */ 
  kk_function_t _x6096;
  struct kk_std_time_calendar_Calendar* _con6097 = kk_std_time_calendar__as_Calendar(cal);
  kk_string_t _pat0 = _con6097->name;
  kk_string_t _pat1 = _con6097->long_name;
  kk_string_t _pat2 = _con6097->month_prefix;
  kk_function_t _pat3 = _con6097->show_era;
  kk_function_t _pat4 = _con6097->instant_to_dc;
  kk_function_t _pat5 = _con6097->dc_to_instant;
  kk_function_t _pat6 = _con6097->days_to_date;
  kk_function_t _x = _con6097->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(cal))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(cal);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(cal, _ctx);
    kk_Unit;
  }
  _x6096 = _x; /*(date : std/time/date/date) -> int*/
  kk_std_core_types__optional _x6098;
  bool _b_5202_5201;
  struct kk_std_time_instant_Timescale* _con6099 = kk_std_time_instant__as_Timescale(ts);
  kk_string_t _pat00 = _con6099->name;
  kk_string_t _pat100 = _con6099->unit;
  kk_function_t _pat20 = _con6099->from_tai;
  kk_function_t _pat30 = _con6099->to_tai;
  kk_std_core_types__maybe _x0 = _con6099->mb_seconds_in_day;
  kk_std_core_types__maybe _pat40 = _con6099->mb_to_mjd2000;
  kk_std_core_types__maybe _pat50 = _con6099->mb_from_mjd2000;
  if (kk_likely(kk_std_time_instant__timescale_is_unique(ts))) {
    kk_std_core_types__maybe_drop(_pat50, _ctx);
    kk_Unit;
    kk_std_core_types__maybe_drop(_pat40, _ctx);
    kk_Unit;
    kk_function_drop(_pat30, _ctx);
    kk_Unit;
    kk_function_drop(_pat20, _ctx);
    kk_Unit;
    kk_string_drop(_pat100, _ctx);
    kk_Unit;
    kk_string_drop(_pat00, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_instant__timescale_free(ts);
  }
  else {
    kk_std_core_types__maybe_dup(_x0);
    kk_Unit;
    kk_std_time_instant__timescale_decref(ts, _ctx);
    kk_Unit;
  }
  if (kk_std_core_types__is_Nothing(_x0)) {
    _b_5202_5201 = false; /*bool*/
  }
  else {
    kk_std_core_types__maybe_drop(_x0, _ctx);
    _b_5202_5201 = true; /*bool*/
  }
  _x6098 = kk_std_core_types__new_Optional(kk_bool_box(_b_5202_5201), _ctx); /*optional<106>*/
  return kk_std_time_calendar_dc_timestamp(d, c, _x6096, _x6098, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_create_fun6107(kk_function_t _fself, kk_integer_t _x16106, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16106, _ctx);
}
extern kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_create_fun6109(kk_function_t _fself, kk_integer_t _x16108, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x16108, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_create_fun6112(kk_function_t _fself, kk_integer_t _x16110, kk_integer_t _x26111, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16110, _x26111, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_create_fun6115(kk_function_t _fself, kk_integer_t _x16113, kk_integer_t _x26114, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16113, _x26114, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6121(kk_function_t _fself, kk_integer_t _x16120, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16120, _ctx);
}
extern kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_sls_create_fun6123(kk_function_t _fself, kk_integer_t _x16122, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_estimate_year(_x16122, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6126(kk_function_t _fself, kk_integer_t _x16124, kk_integer_t _x26125, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16124, _x26125, _ctx);
}
extern kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6129(kk_function_t _fself, kk_integer_t _x16127, kk_integer_t _x26128, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16127, _x26128, _ctx);
}
 
// Create a new calendar from by combining two other calendars. The `switch-date`
// is in terms of the second calendar (`cal2`) and time after (and including) the switch
// date is displayed in `cal2` while times before it in `cal1`. This function is used
// for example for the Julian Gregorian calendar.


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6131__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_time_date__date switch_date;
};
static kk_std_time_date__date kk_std_time_calendar_combine_earth_calendars_fun6131(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6131(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6131__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6131__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6131, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->switch_date = switch_date;
  return &_self->_base;
}

static kk_std_time_date__date kk_std_time_calendar_combine_earth_calendars_fun6131(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6131__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6131__t*, _fself);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1);kk_std_time_calendar__calendar_dup(cal2);kk_std_time_date__date_dup(switch_date);}, {}, _ctx)
  kk_function_t _x6132;
  kk_std_time_calendar__calendar calendar0_4714;
  bool _match_5288;
  kk_integer_t _borrowedArg_5289;
  kk_function_t _x6133;
  struct kk_std_time_calendar_Calendar* _con6134 = kk_std_time_calendar__as_Calendar(cal2);
  kk_function_t _x = _con6134->date_to_days;
  kk_function_dup(_x);
  _x6133 = _x; /*(date : std/time/date/date) -> int*/
  _borrowedArg_5289 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6133, (_x6133, switch_date, _ctx)); /*int*/
  bool _borrowApp_5290 = kk_integer_lt_borrow(days,_borrowedArg_5289,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5289, _ctx);
  _match_5288 = _borrowApp_5290; /*bool*/
  if (_match_5288) {
    kk_std_time_calendar__calendar_dropn(cal2, ((int32_t)8), _ctx);
    calendar0_4714 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_time_calendar__calendar_dropn(cal1, ((int32_t)8), _ctx);
    calendar0_4714 = cal2; /*std/time/calendar/calendar*/
  }
  struct kk_std_time_calendar_Calendar* _con6135 = kk_std_time_calendar__as_Calendar(calendar0_4714);
  kk_string_t _pat00 = _con6135->name;
  kk_string_t _pat10 = _con6135->long_name;
  kk_string_t _pat20 = _con6135->month_prefix;
  kk_function_t _pat30 = _con6135->show_era;
  kk_function_t _pat40 = _con6135->instant_to_dc;
  kk_function_t _pat50 = _con6135->dc_to_instant;
  kk_function_t _x0 = _con6135->days_to_date;
  kk_function_t _pat60 = _con6135->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(calendar0_4714))) {
    kk_function_drop(_pat60, _ctx);
    kk_Unit;
    kk_function_drop(_pat50, _ctx);
    kk_Unit;
    kk_function_drop(_pat40, _ctx);
    kk_Unit;
    kk_function_drop(_pat30, _ctx);
    kk_Unit;
    kk_string_drop(_pat20, _ctx);
    kk_Unit;
    kk_string_drop(_pat10, _ctx);
    kk_Unit;
    kk_string_drop(_pat00, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(calendar0_4714);
  }
  else {
    kk_function_dup(_x0);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(calendar0_4714, _ctx);
    kk_Unit;
  }
  _x6132 = _x0; /*(days : int) -> std/time/date/date*/
  return kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), _x6132, (_x6132, days, _ctx));
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6137__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_time_date__date switch_date;
};
static kk_integer_t kk_std_time_calendar_combine_earth_calendars_fun6137(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6137(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6137__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6137__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6137, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->switch_date = switch_date;
  return &_self->_base;
}

static kk_integer_t kk_std_time_calendar_combine_earth_calendars_fun6137(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6137__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6137__t*, _fself);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1);kk_std_time_calendar__calendar_dup(cal2);kk_std_time_date__date_dup(switch_date);}, {}, _ctx)
  kk_function_t _x6138;
  kk_std_time_calendar__calendar calendar1_4716;
  bool _match_5287;
  kk_std_core_types__order _x6139;
  kk_std_time_date__date _x6140 = kk_std_time_date__date_dup(d); /*std/time/date/date*/
  _x6139 = kk_std_time_date_compare(_x6140, switch_date, _ctx); /*order*/
  _match_5287 = kk_std_core__lp__eq__eq__4_rp_(_x6139, kk_std_core_types__new_Lt(_ctx), _ctx); /*bool*/
  if (_match_5287) {
    kk_std_time_calendar__calendar_dropn(cal2, ((int32_t)8), _ctx);
    calendar1_4716 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_time_calendar__calendar_dropn(cal1, ((int32_t)8), _ctx);
    calendar1_4716 = cal2; /*std/time/calendar/calendar*/
  }
  struct kk_std_time_calendar_Calendar* _con6141 = kk_std_time_calendar__as_Calendar(calendar1_4716);
  kk_string_t _pat01 = _con6141->name;
  kk_string_t _pat12 = _con6141->long_name;
  kk_string_t _pat22 = _con6141->month_prefix;
  kk_function_t _pat31 = _con6141->show_era;
  kk_function_t _pat41 = _con6141->instant_to_dc;
  kk_function_t _pat51 = _con6141->dc_to_instant;
  kk_function_t _pat61 = _con6141->days_to_date;
  kk_function_t _x1 = _con6141->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(calendar1_4716))) {
    kk_function_drop(_pat61, _ctx);
    kk_Unit;
    kk_function_drop(_pat51, _ctx);
    kk_Unit;
    kk_function_drop(_pat41, _ctx);
    kk_Unit;
    kk_function_drop(_pat31, _ctx);
    kk_Unit;
    kk_string_drop(_pat22, _ctx);
    kk_Unit;
    kk_string_drop(_pat12, _ctx);
    kk_Unit;
    kk_string_drop(_pat01, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(calendar1_4716);
  }
  else {
    kk_function_dup(_x1);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(calendar1_4716, _ctx);
    kk_Unit;
  }
  _x6138 = _x1; /*(date : std/time/date/date) -> int*/
  return kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6138, (_x6138, d, _ctx));
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6143__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_core_types__optional mb_show_era;
  kk_std_time_date__date switch_date;
};
static kk_string_t kk_std_time_calendar_combine_earth_calendars_fun6143(kk_function_t _fself, kk_std_time_date__date d0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6143(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_core_types__optional mb_show_era, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6143__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6143__t, 8, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6143, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->mb_show_era = mb_show_era;
  _self->switch_date = switch_date;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_combine_earth_calendars_fun6143(kk_function_t _fself, kk_std_time_date__date d0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6143__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6143__t*, _fself);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_core_types__optional mb_show_era = _self->mb_show_era; /* optional<maybe<(std/time/date/date) -> string>> */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1);kk_std_time_calendar__calendar_dup(cal2);kk_std_core_types__optional_dup(mb_show_era);kk_std_time_date__date_dup(switch_date);}, {}, _ctx)
  kk_std_core_types__maybe _match_5285;
  if (kk_std_core_types__is_Optional(mb_show_era)) {
    kk_box_t _box_x5203 = mb_show_era._cons.Optional.value;
    kk_std_core_types__maybe _mb_show_era_4136 = kk_std_core_types__maybe_unbox(_box_x5203, NULL);
    kk_std_core_types__maybe_dup(_mb_show_era_4136);
    kk_std_core_types__optional_drop(mb_show_era, _ctx);
    kk_Unit;
    _match_5285 = _mb_show_era_4136; /*maybe<(std/time/date/date) -> string>*/
    goto _match6144;
  }
  _match_5285 = kk_std_core_types__new_Nothing(_ctx); /*maybe<(std/time/date/date) -> string>*/
  _match6144: ;
  if (kk_std_core_types__is_Just(_match_5285)) {
    kk_box_t _fun_unbox_x5207 = _match_5285._cons.Just.value;
    kk_std_time_calendar__calendar_dropn(cal1, ((int32_t)8), _ctx);
    kk_std_time_calendar__calendar_dropn(cal2, ((int32_t)8), _ctx);
    kk_std_time_date__date_drop(switch_date, _ctx);
    kk_std_time_date__date _b_5210 = d0; /*std/time/date/date*/;
    kk_box_t _x6146;
    kk_function_t _x6147 = kk_function_unbox(_fun_unbox_x5207); /*(5208) -> 5209*/
    _x6146 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6147, (_x6147, kk_std_time_date__date_box(_b_5210, _ctx), _ctx)); /*5209*/
    return kk_string_unbox(_x6146);
  }
  kk_function_t _x6148;
  kk_std_time_calendar__calendar calendar2_4719;
  bool _match_5286;
  kk_std_core_types__order _x6149;
  kk_std_time_date__date _x6150 = kk_std_time_date__date_dup(d0); /*std/time/date/date*/
  _x6149 = kk_std_time_date_compare(_x6150, switch_date, _ctx); /*order*/
  _match_5286 = kk_std_core__lp__eq__eq__4_rp_(_x6149, kk_std_core_types__new_Lt(_ctx), _ctx); /*bool*/
  if (_match_5286) {
    kk_std_time_calendar__calendar_dropn(cal2, ((int32_t)8), _ctx);
    calendar2_4719 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_time_calendar__calendar_dropn(cal1, ((int32_t)8), _ctx);
    calendar2_4719 = cal2; /*std/time/calendar/calendar*/
  }
  struct kk_std_time_calendar_Calendar* _con6151 = kk_std_time_calendar__as_Calendar(calendar2_4719);
  kk_string_t _pat03 = _con6151->name;
  kk_string_t _pat14 = _con6151->long_name;
  kk_string_t _pat23 = _con6151->month_prefix;
  kk_function_t _x2 = _con6151->show_era;
  kk_function_t _pat33 = _con6151->instant_to_dc;
  kk_function_t _pat43 = _con6151->dc_to_instant;
  kk_function_t _pat53 = _con6151->days_to_date;
  kk_function_t _pat63 = _con6151->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(calendar2_4719))) {
    kk_function_drop(_pat63, _ctx);
    kk_Unit;
    kk_function_drop(_pat53, _ctx);
    kk_Unit;
    kk_function_drop(_pat43, _ctx);
    kk_Unit;
    kk_function_drop(_pat33, _ctx);
    kk_Unit;
    kk_string_drop(_pat23, _ctx);
    kk_Unit;
    kk_string_drop(_pat14, _ctx);
    kk_Unit;
    kk_string_drop(_pat03, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(calendar2_4719);
  }
  else {
    kk_function_dup(_x2);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(calendar2_4719, _ctx);
    kk_Unit;
  }
  _x6148 = _x2; /*(std/time/date/date) -> string*/
  return kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6148, (_x6148, d0, _ctx));
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6152__t {
  struct kk_function_s _base;
  kk_function_t _b_5215_5211;
};
static kk_box_t kk_std_time_calendar_combine_earth_calendars_fun6152(kk_function_t _fself, kk_box_t _b_5214, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6152(kk_function_t _b_5215_5211, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6152__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6152__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6152, kk_context());
  _self->_b_5215_5211 = _b_5215_5211;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_combine_earth_calendars_fun6152(kk_function_t _fself, kk_box_t _b_5214, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6152__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6152__t*, _fself);
  kk_function_t _b_5215_5211 = _self->_b_5215_5211; /* (d0 : std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5215_5211);}, {}, _ctx)
  kk_string_t _x6153;
  kk_std_time_date__date _x6154 = kk_std_time_date__date_unbox(_b_5214, _ctx); /*std/time/date/date*/
  _x6153 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5215_5211, (_b_5215_5211, _x6154, _ctx)); /*string*/
  return kk_string_box(_x6153);
}

kk_std_time_calendar__calendar kk_std_time_calendar_combine_earth_calendars(kk_string_t name0, kk_string_t long_name0, kk_std_time_date__date switch_date, kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_core_types__optional mb_show_era, kk_context_t* _ctx) { /* (name : string, long-name : string, switch-date : std/time/date/date, cal1 : calendar, cal2 : calendar, mb-show-era : optional<maybe<(std/time/date/date) -> string>>) -> calendar */ 
  kk_function_t _x6130;
  kk_std_time_calendar__calendar_dup(cal1);
  kk_std_time_calendar__calendar_dup(cal2);
  kk_std_time_date__date_dup(switch_date);
  _x6130 = kk_std_time_calendar_new_combine_earth_calendars_fun6131(cal1, cal2, switch_date, _ctx); /*(days : int) -> std/time/date/date*/
  kk_function_t _x6136;
  kk_std_time_calendar__calendar_dup(cal1);
  kk_std_time_calendar__calendar_dup(cal2);
  kk_std_time_date__date_dup(switch_date);
  _x6136 = kk_std_time_calendar_new_combine_earth_calendars_fun6137(cal1, cal2, switch_date, _ctx); /*(d : std/time/date/date) -> int*/
  kk_std_core_types__optional _x6142;
  kk_function_t _b_5215_5211 = kk_std_time_calendar_new_combine_earth_calendars_fun6143(cal1, cal2, mb_show_era, switch_date, _ctx); /*(d0 : std/time/date/date) -> string*/;
  _x6142 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_combine_earth_calendars_fun6152(_b_5215_5211, _ctx)), _ctx); /*optional<106>*/
  return kk_std_time_calendar_earth_calendar(name0, long_name0, _x6130, _x6136, kk_std_core_types__new_None(_ctx), _x6142, _ctx);
}
 
// Convert an `:instant` to a `:date`, `:clock`, timezone delta and abbreviation,
// for a given timezone `tz` (=`tz-utc` by default) and calendar (=`cal-iso` by default).


// lift anonymous function
struct kk_std_time_calendar_instant_dc_fun6159__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_instant_dc_fun6159(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_dc_fun6159(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_instant_dc_fun6159, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_instant_dc_fun6159(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5229_5218 = kk_std_time_duration_zero; /*std/time/duration/duration*/;
  kk_string_t _b_5230_5219;
  kk_std_core_types__optional _match_5284 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5284)) {
    kk_box_t _box_x5217 = _match_5284._cons.Optional.value;
    kk_string_t _abbrv_2653 = kk_string_unbox(_box_x5217);
    kk_string_dup(_abbrv_2653);
    kk_std_core_types__optional_drop(_match_5284, _ctx);
    kk_Unit;
    _b_5230_5219 = _abbrv_2653; /*string*/
    goto _match6160;
  }
  _b_5230_5219 = kk_string_empty(); /*string*/
  _match6160: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5229_5218, _ctx), kk_string_box(_b_5230_5219), _ctx);
}

kk_std_core_types__tuple4_ kk_std_time_calendar_instant_dc(kk_std_time_instant__instant i, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (i : std/time/instant/instant, tz : optional<timezone>, cal : optional<calendar>) -> (std/time/date/date, std/time/date/clock, std/time/duration/duration, string) */ 
  kk_std_core_types__tuple2_ _match_5281;
  kk_function_t _x6156;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5216 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_4272 = kk_std_time_calendar__timezone_unbox(_box_x5216, NULL);
    kk_std_time_calendar__timezone_dup(_tz_4272);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    kk_function_t _x1 = _tz_4272.utc_delta;
    kk_function_dup(_x1);
    kk_std_time_calendar__timezone_drop(_tz_4272, _ctx);
    kk_Unit;
    _x6156 = _x1; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
    goto _match6157;
  }
  _x6156 = kk_std_time_calendar_new_instant_dc_fun6159(_ctx); /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  _match6157: ;
  kk_std_time_instant__instant _x6155 = kk_std_time_instant__instant_dup(i); /*std/time/instant/instant*/
  _match_5281 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6156, (_x6156, _x6155, _ctx)); /*(std/time/duration/duration, string)*/
  kk_box_t _box_x5220 = _match_5281.fst;
  kk_box_t _box_x5221 = _match_5281.snd;
  kk_std_time_duration__duration tzdelta = kk_std_time_duration__duration_unbox(_box_x5220, NULL);
  kk_string_t tzabbrv = kk_string_unbox(_box_x5221);
  kk_string_dup(tzabbrv);
  kk_std_core_types__tuple2__drop(_match_5281, _ctx);
  kk_Unit;
  kk_std_core_types__tuple2_ _match_5282;
  kk_function_t _x6165;
  kk_std_time_calendar__calendar _match_5283;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5222 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_4276 = kk_std_time_calendar__calendar_unbox(_box_x5222, NULL);
    kk_std_time_calendar__calendar_dup(_cal_4276);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5283 = _cal_4276; /*std/time/calendar/calendar*/
    goto _match6166;
  }
  _match_5283 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match6166: ;
  struct kk_std_time_calendar_Calendar* _con6168 = kk_std_time_calendar__as_Calendar(_match_5283);
  kk_string_t _pat010 = _con6168->name;
  kk_string_t _pat101 = _con6168->long_name;
  kk_string_t _pat201 = _con6168->month_prefix;
  kk_function_t _pat300 = _con6168->show_era;
  kk_function_t _x00 = _con6168->instant_to_dc;
  kk_function_t _pat40 = _con6168->dc_to_instant;
  kk_function_t _pat50 = _con6168->days_to_date;
  kk_function_t _pat6 = _con6168->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5283))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat50, _ctx);
    kk_Unit;
    kk_function_drop(_pat40, _ctx);
    kk_Unit;
    kk_function_drop(_pat300, _ctx);
    kk_Unit;
    kk_string_drop(_pat201, _ctx);
    kk_Unit;
    kk_string_drop(_pat101, _ctx);
    kk_Unit;
    kk_string_drop(_pat010, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5283);
  }
  else {
    kk_function_dup(_x00);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5283, _ctx);
    kk_Unit;
  }
  _x6165 = _x00; /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)*/
  _match_5282 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_std_time_duration__duration, kk_context_t*), _x6165, (_x6165, i, tzdelta, _ctx)); /*(std/time/date/date, std/time/date/clock)*/
  kk_box_t _box_x5223 = _match_5282.fst;
  kk_box_t _box_x5224 = _match_5282.snd;
  kk_std_time_date__date d = kk_std_time_date__date_unbox(_box_x5223, NULL);
  kk_std_time_date__clock c = kk_std_time_date__clock_unbox(_box_x5224, NULL);
  struct kk_std_time_date_Clock* _con6171 = kk_std_time_date__as_Clock(c);
  kk_std_time_date__clock_dup(c);
  kk_std_time_date__date_dup(d);
  kk_std_core_types__tuple2__drop(_match_5282, _ctx);
  kk_Unit;
  kk_std_time_date__date _b_5231_5225 = d; /*std/time/date/date*/;
  kk_std_time_date__clock _b_5232_5226 = c; /*std/time/date/clock*/;
  kk_std_time_duration__duration _b_5233_5227 = tzdelta; /*std/time/duration/duration*/;
  kk_string_t _b_5234_5228 = tzabbrv; /*string*/;
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, kk_std_time_date__date_box(_b_5231_5225, _ctx), kk_std_time_date__clock_box(_b_5232_5226, _ctx), kk_std_time_duration__duration_box(_b_5233_5227, _ctx), kk_string_box(_b_5234_5228), _ctx);
}
 
// Return the local timezone on the current system.


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6174__t {
  struct kk_function_s _base;
  kk_std_time_calendar__local_timezone tz;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_local_fun6174(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6174(kk_std_time_calendar__local_timezone tz, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6174__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_local_fun6174__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_local_fun6174, kk_context());
  _self->tz = tz;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_local_fun6174(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6174__t* _self = kk_function_as(struct kk_std_time_calendar_tz_local_fun6174__t*, _fself);
  kk_std_time_calendar__local_timezone tz = _self->tz; /* std/time/calendar/local-timezone */
  kk_drop_match(_self, {kk_std_time_calendar__local_timezone_dup(tz);}, {}, _ctx)
  kk_std_core_types__tuple2_ _match_5280;
  double _x6175;
  kk_std_num_ddouble__ddouble x_4727 = kk_std_time_utc_unix_timestamp(i0, _ctx); /*std/num/ddouble/ddouble*/;
  double _x = x_4727.hi;
  _x6175 = _x; /*double*/
  _match_5280 = kk_std_time_calendar_local_utc_delta(tz, _x6175, _ctx); /*(double, string)*/
  kk_box_t _box_x5235 = _match_5280.fst;
  kk_box_t _box_x5236 = _match_5280.snd;
  double ofs = kk_double_unbox(_box_x5235, NULL);
  kk_string_t abbrv = kk_string_unbox(_box_x5236);
  kk_string_dup(abbrv);
  kk_std_core_types__tuple2__drop(_match_5280, _ctx);
  kk_Unit;
  kk_std_time_duration__duration _b_5246_5237;
  kk_std_num_ddouble__ddouble _x6178 = kk_std_num_ddouble__new_Ddouble(ofs, 0.0, _ctx); /*std/num/ddouble/ddouble*/
  _b_5246_5237 = kk_std_time_duration__new_Duration(_x6178, _ctx); /*std/time/duration/duration*/
  kk_string_t _b_5247_5238 = abbrv; /*string*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5246_5237, _ctx), kk_string_box(_b_5247_5238), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6181__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5242;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6181(kk_function_t _fself, kk_std_time_instant__instant _b_5245, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6181(kk_box_t _fun_unbox_x5242, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6181__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_local_fun6181__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_local_fun6181, kk_context());
  _self->_fun_unbox_x5242 = _fun_unbox_x5242;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6181(kk_function_t _fself, kk_std_time_instant__instant _b_5245, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6181__t* _self = kk_function_as(struct kk_std_time_calendar_tz_local_fun6181__t*, _fself);
  kk_box_t _fun_unbox_x5242 = _self->_fun_unbox_x5242; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5242);}, {}, _ctx)
  kk_box_t _x6182;
  kk_function_t _x6183 = kk_function_unbox(_fun_unbox_x5242); /*(5243) -> 5244*/
  _x6182 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6183, (_x6183, kk_std_time_instant__instant_box(_b_5245, _ctx), _ctx)); /*5244*/
  return kk_std_core_types__maybe_unbox(_x6182, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6184__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6184(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6184(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_local_fun6184, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6184(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_local(kk_context_t* _ctx) { /* () -> ndet timezone */ 
  kk_std_time_calendar__local_timezone tz = kk_std_time_calendar_local_get_timezone(_ctx); /*std/time/calendar/local-timezone*/;
  kk_string_t _x6172 = kk_string_empty(); /*string*/
  kk_function_t _x6179;
  kk_std_core_types__optional _x6180 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/
  if (kk_std_core_types__is_Optional(_x6180)) {
    kk_box_t _fun_unbox_x5242 = _x6180._cons.Optional.value;
    _x6179 = kk_std_time_calendar_new_tz_local_fun6181(_fun_unbox_x5242, _ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  else {
    _x6179 = kk_std_time_calendar_new_tz_local_fun6184(_ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  return kk_std_time_calendar__new_Timezone(_x6172, kk_std_time_calendar_new_tz_local_fun6174(tz, _ctx), _x6179, _ctx);
}
 
// Return the day of the week for a calendar `cal` (=`cal-iso`).

kk_std_time_date__weekday kk_std_time_calendar_weekday(kk_std_time_date__date d, kk_std_time_calendar__calendar cal, kk_context_t* _ctx) { /* (d : std/time/date/date, cal : calendar) -> std/time/date/weekday */ 
  kk_integer_t dow;
  kk_integer_t _x6185;
  kk_integer_t _x6186;
  kk_function_t _x6187;
  struct kk_std_time_calendar_Calendar* _con6188 = kk_std_time_calendar__as_Calendar(cal);
  kk_string_t _pat0 = _con6188->name;
  kk_string_t _pat1 = _con6188->long_name;
  kk_string_t _pat2 = _con6188->month_prefix;
  kk_function_t _pat3 = _con6188->show_era;
  kk_function_t _pat4 = _con6188->instant_to_dc;
  kk_function_t _pat5 = _con6188->dc_to_instant;
  kk_function_t _pat6 = _con6188->days_to_date;
  kk_function_t _x = _con6188->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(cal))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(cal);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(cal, _ctx);
    kk_Unit;
  }
  _x6187 = _x; /*(date : std/time/date/date) -> int*/
  _x6186 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6187, (_x6187, d, _ctx)); /*int*/
  _x6185 = kk_integer_add(_x6186,(kk_integer_from_small(6)),kk_context()); /*int*/
  dow = kk_integer_mod(_x6185,(kk_integer_from_small(7)),kk_context()); /*int*/
  return kk_std_time_date_weekday(dow, _ctx);
}
 
// Create a new calendar from anoter calendar `cal` by adding an offset to the
// years (`year-shift`). This is used for example to create the Ethiopian calendar
// from the Coptic calendar, by using:
// `year-shift-earth-calendar( "EC", "Ethiopian", ~276, cal-coptic )`


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6190__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal;
  kk_integer_t year_shift;
};
static kk_std_time_date__date kk_std_time_calendar_year_shift_earth_calendar_fun6190(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6190(kk_std_time_calendar__calendar cal, kk_integer_t year_shift, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6190__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6190__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6190, kk_context());
  _self->cal = cal;
  _self->year_shift = year_shift;
  return &_self->_base;
}

static kk_std_time_date__date kk_std_time_calendar_year_shift_earth_calendar_fun6190(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6190__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6190__t*, _fself);
  kk_std_time_calendar__calendar cal = _self->cal; /* std/time/calendar/calendar */
  kk_integer_t year_shift = _self->year_shift; /* int */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal);kk_integer_dup(year_shift);}, {}, _ctx)
  kk_std_time_date__date d0;
  kk_function_t _x6191;
  struct kk_std_time_calendar_Calendar* _con6192 = kk_std_time_calendar__as_Calendar(cal);
  kk_string_t _pat0 = _con6192->name;
  kk_string_t _pat1 = _con6192->long_name;
  kk_string_t _pat2 = _con6192->month_prefix;
  kk_function_t _pat3 = _con6192->show_era;
  kk_function_t _pat4 = _con6192->instant_to_dc;
  kk_function_t _pat5 = _con6192->dc_to_instant;
  kk_function_t _x = _con6192->days_to_date;
  kk_function_t _pat6 = _con6192->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(cal))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(cal);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(cal, _ctx);
    kk_Unit;
  }
  _x6191 = _x; /*(days : int) -> std/time/date/date*/
  d0 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), _x6191, (_x6191, days, _ctx)); /*std/time/date/date*/
  kk_integer_t _b_5249_5248;
  kk_integer_t _x6193;
  kk_integer_t _x2 = d0.year;
  kk_integer_dup(_x2);
  _x6193 = _x2; /*int*/
  _b_5249_5248 = kk_integer_sub(_x6193,year_shift,kk_context()); /*int*/
  kk_integer_t _x6194;
  kk_std_core_types__optional _match_5278 = kk_std_core_types__new_Optional(kk_integer_box(_b_5249_5248), _ctx); /*optional<106>*/;
  if (kk_std_core_types__is_Optional(_match_5278)) {
    kk_box_t _box_x5250 = _match_5278._cons.Optional.value;
    kk_integer_t _year_143 = kk_integer_unbox(_box_x5250);
    kk_integer_dup(_year_143);
    kk_std_core_types__optional_drop(_match_5278, _ctx);
    kk_Unit;
    _x6194 = _year_143; /*int*/
    goto _match6195;
  }
  kk_integer_t _x0 = d0.year;
  kk_integer_dup(_x0);
  _x6194 = _x0; /*int*/
  _match6195: ;
  kk_integer_t _x6197;
  kk_std_core_types__optional _match_5277 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5277)) {
    kk_box_t _box_x5251 = _match_5277._cons.Optional.value;
    kk_integer_t _month_149 = kk_integer_unbox(_box_x5251);
    kk_integer_dup(_month_149);
    kk_std_core_types__optional_drop(_match_5277, _ctx);
    kk_Unit;
    _x6197 = _month_149; /*int*/
    goto _match6198;
  }
  kk_integer_t _x00 = d0.month;
  kk_integer_dup(_x00);
  _x6197 = _x00; /*int*/
  _match6198: ;
  kk_integer_t _x6200;
  kk_std_core_types__optional _match_5276 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5276)) {
    kk_box_t _box_x5252 = _match_5276._cons.Optional.value;
    kk_integer_t _day_155 = kk_integer_unbox(_box_x5252);
    kk_std_time_date__date_drop(d0, _ctx);
    kk_integer_dup(_day_155);
    kk_std_core_types__optional_drop(_match_5276, _ctx);
    kk_Unit;
    _x6200 = _day_155; /*int*/
    goto _match6201;
  }
  kk_integer_t _x1 = d0.day;
  kk_integer_dup(_x1);
  kk_std_time_date__date_drop(d0, _ctx);
  kk_Unit;
  _x6200 = _x1; /*int*/
  _match6201: ;
  return kk_std_time_date__new_Date(_x6194, _x6197, _x6200, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6203__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal;
  kk_integer_t year_shift;
};
static kk_integer_t kk_std_time_calendar_year_shift_earth_calendar_fun6203(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6203(kk_std_time_calendar__calendar cal, kk_integer_t year_shift, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6203__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6203__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6203, kk_context());
  _self->cal = cal;
  _self->year_shift = year_shift;
  return &_self->_base;
}

static kk_integer_t kk_std_time_calendar_year_shift_earth_calendar_fun6203(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6203__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6203__t*, _fself);
  kk_std_time_calendar__calendar cal = _self->cal; /* std/time/calendar/calendar */
  kk_integer_t year_shift = _self->year_shift; /* int */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal);kk_integer_dup(year_shift);}, {}, _ctx)
  kk_function_t _x6215;
  struct kk_std_time_calendar_Calendar* _con6216 = kk_std_time_calendar__as_Calendar(cal);
  kk_string_t _pat04 = _con6216->name;
  kk_string_t _pat15 = _con6216->long_name;
  kk_string_t _pat21 = _con6216->month_prefix;
  kk_function_t _pat31 = _con6216->show_era;
  kk_function_t _pat41 = _con6216->instant_to_dc;
  kk_function_t _pat51 = _con6216->dc_to_instant;
  kk_function_t _pat60 = _con6216->days_to_date;
  kk_function_t _x3 = _con6216->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(cal))) {
    kk_function_drop(_pat60, _ctx);
    kk_Unit;
    kk_function_drop(_pat51, _ctx);
    kk_Unit;
    kk_function_drop(_pat41, _ctx);
    kk_Unit;
    kk_function_drop(_pat31, _ctx);
    kk_Unit;
    kk_string_drop(_pat21, _ctx);
    kk_Unit;
    kk_string_drop(_pat15, _ctx);
    kk_Unit;
    kk_string_drop(_pat04, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(cal);
  }
  else {
    kk_function_dup(_x3);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(cal, _ctx);
    kk_Unit;
  }
  _x6215 = _x3; /*(date : std/time/date/date) -> int*/
  kk_std_time_date__date _x6204;
  kk_integer_t _b_5254_5253;
  kk_integer_t _x6205;
  kk_integer_t _x5 = d1.year;
  kk_integer_dup(_x5);
  _x6205 = _x5; /*int*/
  _b_5254_5253 = kk_integer_add(_x6205,year_shift,kk_context()); /*int*/
  kk_integer_t _x6206;
  kk_std_core_types__optional _match_5275 = kk_std_core_types__new_Optional(kk_integer_box(_b_5254_5253), _ctx); /*optional<106>*/;
  if (kk_std_core_types__is_Optional(_match_5275)) {
    kk_box_t _box_x5255 = _match_5275._cons.Optional.value;
    kk_integer_t _year_1430 = kk_integer_unbox(_box_x5255);
    kk_integer_dup(_year_1430);
    kk_std_core_types__optional_drop(_match_5275, _ctx);
    kk_Unit;
    _x6206 = _year_1430; /*int*/
    goto _match6207;
  }
  kk_integer_t _x4 = d1.year;
  kk_integer_dup(_x4);
  _x6206 = _x4; /*int*/
  _match6207: ;
  kk_integer_t _x6209;
  kk_std_core_types__optional _match_5274 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5274)) {
    kk_box_t _box_x5256 = _match_5274._cons.Optional.value;
    kk_integer_t _month_1490 = kk_integer_unbox(_box_x5256);
    kk_integer_dup(_month_1490);
    kk_std_core_types__optional_drop(_match_5274, _ctx);
    kk_Unit;
    _x6209 = _month_1490; /*int*/
    goto _match6210;
  }
  kk_integer_t _x01 = d1.month;
  kk_integer_dup(_x01);
  _x6209 = _x01; /*int*/
  _match6210: ;
  kk_integer_t _x6212;
  kk_std_core_types__optional _match_5273 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5273)) {
    kk_box_t _box_x5257 = _match_5273._cons.Optional.value;
    kk_integer_t _day_1550 = kk_integer_unbox(_box_x5257);
    kk_std_time_date__date_drop(d1, _ctx);
    kk_integer_dup(_day_1550);
    kk_std_core_types__optional_drop(_match_5273, _ctx);
    kk_Unit;
    _x6212 = _day_1550; /*int*/
    goto _match6213;
  }
  kk_integer_t _x10 = d1.day;
  kk_integer_dup(_x10);
  kk_std_time_date__date_drop(d1, _ctx);
  kk_Unit;
  _x6212 = _x10; /*int*/
  _match6213: ;
  _x6204 = kk_std_time_date__new_Date(_x6206, _x6209, _x6212, _ctx); /*std/time/date/date*/
  return kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6215, (_x6215, _x6204, _ctx));
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6222__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5263;
};
static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6222(kk_function_t _fself, kk_std_time_date__date _b_5266, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6222(kk_box_t _fun_unbox_x5263, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6222__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6222__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6222, kk_context());
  _self->_fun_unbox_x5263 = _fun_unbox_x5263;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6222(kk_function_t _fself, kk_std_time_date__date _b_5266, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6222__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6222__t*, _fself);
  kk_box_t _fun_unbox_x5263 = _self->_fun_unbox_x5263; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5263);}, {}, _ctx)
  kk_box_t _x6223;
  kk_function_t _x6224 = kk_function_unbox(_fun_unbox_x5263); /*(5264) -> 5265*/
  _x6223 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6224, (_x6224, kk_std_time_date__date_box(_b_5266, _ctx), _ctx)); /*5265*/
  return kk_string_unbox(_x6223);
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6225__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6225(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6225(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_year_shift_earth_calendar_fun6225, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6225(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6227__t {
  struct kk_function_s _base;
  kk_function_t _b_5272_5267;
};
static kk_box_t kk_std_time_calendar_year_shift_earth_calendar_fun6227(kk_function_t _fself, kk_box_t _b_5270, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6227(kk_function_t _b_5272_5267, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6227__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6227__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6227, kk_context());
  _self->_b_5272_5267 = _b_5272_5267;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_year_shift_earth_calendar_fun6227(kk_function_t _fself, kk_box_t _b_5270, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6227__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6227__t*, _fself);
  kk_function_t _b_5272_5267 = _self->_b_5272_5267; /* (std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5272_5267);}, {}, _ctx)
  kk_string_t _x6228;
  kk_std_time_date__date _x6229 = kk_std_time_date__date_unbox(_b_5270, _ctx); /*std/time/date/date*/
  _x6228 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5272_5267, (_b_5272_5267, _x6229, _ctx)); /*string*/
  return kk_string_box(_x6228);
}

kk_std_time_calendar__calendar kk_std_time_calendar_year_shift_earth_calendar(kk_string_t name0, kk_string_t long_name0, kk_integer_t year_shift, kk_std_time_calendar__calendar cal, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, year-shift : int, cal : calendar, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t _x6189;
  kk_std_time_calendar__calendar_dup(cal);
  kk_integer_dup(year_shift);
  _x6189 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6190(cal, year_shift, _ctx); /*(days : int) -> std/time/date/date*/
  kk_std_core_types__optional _x6217;
  kk_string_t _b_5271_5259;
  if (kk_std_core_types__is_Optional(month_prefix0)) {
    kk_box_t _box_x5258 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_4415 = kk_string_unbox(_box_x5258);
    kk_string_dup(_month_prefix_4415);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    kk_Unit;
    _b_5271_5259 = _month_prefix_4415; /*string*/
    goto _match6218;
  }
  _b_5271_5259 = kk_string_empty(); /*string*/
  _match6218: ;
  _x6217 = kk_std_core_types__new_Optional(kk_string_box(_b_5271_5259), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6221;
  kk_function_t _b_5272_5267;
  if (kk_std_core_types__is_Optional(show_era0)) {
    kk_box_t _fun_unbox_x5263 = show_era0._cons.Optional.value;
    _b_5272_5267 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6222(_fun_unbox_x5263, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _b_5272_5267 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6225(_ctx); /*(std/time/date/date) -> string*/
  }
  _x6221 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_year_shift_earth_calendar_fun6227(_b_5272_5267, _ctx)), _ctx); /*optional<106>*/
  return kk_std_time_calendar_earth_calendar(name0, long_name0, _x6189, kk_std_time_calendar_new_year_shift_earth_calendar_fun6203(cal, year_shift, _ctx), _x6217, _x6221, _ctx);
}

// initialization
void kk_std_time_calendar__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  kk_std_core__init(_ctx);
  kk_std_num_ddouble__init(_ctx);
  kk_std_time_date__init(_ctx);
  kk_std_time_timestamp__init(_ctx);
  kk_std_time_duration__init(_ctx);
  kk_std_time_instant__init(_ctx);
  kk_std_time_utc__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_string_t _x5578;
    kk_define_string_literal(, _s5579, 3, "UTC")
    _x5578 = kk_string_dup(_s5579); /*string*/
    kk_std_time_calendar_tz_utc = kk_std_time_calendar__new_Timezone(_x5578, kk_std_time_calendar_new_tz_utc_fun5580(_ctx), kk_std_time_calendar_new_tz_utc_fun5584(_ctx), _ctx); /*std/time/calendar/timezone*/
  }
  {
    kk_std_time_instant__timescale ___wildcard__455__26_4688;
    kk_string_t _x5843 = kk_string_empty(); /*string*/
    kk_std_time_utc__leaps_table _x5845 = kk_std_time_utc__leaps_table_dup(kk_std_time_utc_leaps_table_ti); /*std/time/utc/leaps-table*/
    ___wildcard__455__26_4688 = kk_std_time_utc_utc_timescale(_x5843, _x5845, _ctx); /*std/time/instant/timescale*/
    kk_std_time_instant__timescale_dropn(___wildcard__455__26_4688, ((int32_t)10), _ctx);
    kk_string_t _x5846 = kk_string_empty(); /*string*/
    kk_string_t _x5848;
    kk_define_string_literal(, _s5849, 3, "ISO")
    _x5848 = kk_string_dup(_s5849); /*string*/
    kk_std_time_calendar_cal_iso = kk_std_time_calendar_solar_ecalendar(_x5846, _x5848, kk_std_time_calendar_new_cal_iso_fun5851(_ctx), kk_std_time_calendar_new_cal_iso_fun5853(_ctx), kk_std_time_calendar_new_cal_iso_fun5856(_ctx), kk_std_time_calendar_new_cal_iso_fun5859(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), _ctx); /*std/time/calendar/calendar*/
  }
  {
    kk_string_t _x6059;
    kk_define_string_literal(, _s6060, 2, "GC")
    _x6059 = kk_string_dup(_s6060); /*string*/
    kk_string_t _x6061;
    kk_define_string_literal(, _s6062, 9, "Gregorian")
    _x6061 = kk_string_dup(_s6062); /*string*/
    kk_std_core_types__optional _x6073;
    bool _b_5199_5194 = false; /*bool*/;
    _x6073 = kk_std_core_types__new_Optional(kk_bool_box(_b_5199_5194), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x6074;
    kk_function_t _b_5200_5195 = kk_std_time_calendar_new_cal_gregorian_fun6075(_ctx); /*(d : std/time/date/date) -> string*/;
    _x6074 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_gregorian_fun6079(_b_5200_5195, _ctx)), _ctx); /*optional<106>*/
    kk_std_time_calendar_cal_gregorian = kk_std_time_calendar_solar_ecalendar(_x6059, _x6061, kk_std_time_calendar_new_cal_gregorian_fun6064(_ctx), kk_std_time_calendar_new_cal_gregorian_fun6066(_ctx), kk_std_time_calendar_new_cal_gregorian_fun6069(_ctx), kk_std_time_calendar_new_cal_gregorian_fun6072(_ctx), kk_std_core_types__new_None(_ctx), _x6073, kk_std_core_types__new_None(_ctx), _x6074, _ctx); /*std/time/calendar/calendar*/
  }
  {
    kk_string_t _x6082;
    kk_define_string_literal(, _s6083, 3, "SLS")
    _x6082 = kk_string_dup(_s6083); /*string*/
    kk_string_t _x6084;
    kk_define_string_literal(, _s6085, 7, "ISO-SLS")
    _x6084 = kk_string_dup(_s6085); /*string*/
    kk_std_time_calendar_cal_iso_sls = kk_std_time_calendar_solar_ecalendar(_x6082, _x6084, kk_std_time_calendar_new_cal_iso_sls_fun6087(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun6089(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun6092(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun6095(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), kk_std_core_types__new_None(_ctx), _ctx); /*std/time/calendar/calendar*/
  }
}

// termination
void kk_std_time_calendar__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_iso_sls, _ctx);
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_gregorian, _ctx);
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_iso, _ctx);
  kk_std_time_calendar__timezone_drop(kk_std_time_calendar_tz_utc, _ctx);
  kk_std_time_utc__done(_ctx);
  kk_std_time_instant__done(_ctx);
  kk_std_time_duration__done(_ctx);
  kk_std_time_timestamp__done(_ctx);
  kk_std_time_date__done(_ctx);
  kk_std_num_ddouble__done(_ctx);
  kk_std_core__done(_ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
