// Koka generated module: "std/core", koka version: 2.2.1
#include "std_core.h"
/*---------------------------------------------------------------------------
  Copyright 2020-2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/

kk_std_core__list kk_vector_to_list(kk_vector_t v, kk_std_core__list tail, kk_context_t* ctx) {
  // todo: avoid boxed_dup if v is unique
  kk_ssize_t n;
  kk_box_t* p = kk_vector_buf_borrow(v, &n);
  if (n <= 0) {
    kk_vector_drop(v,ctx);
    return tail;
  }
  kk_std_core__list nil  = kk_std_core__new_Nil(ctx);
  struct kk_std_core_Cons* cons = NULL;
  kk_std_core__list list = kk_std_core__new_Nil(ctx);
  for( kk_ssize_t i = 0; i < n; i++ ) {
    kk_std_core__list hd = kk_std_core__new_Cons(kk_reuse_null,kk_box_dup(p[i]), nil, ctx);
    if (cons==NULL) {
      list = hd;
    }
    else {
      cons->tail = hd;
    }
    cons = kk_std_core__as_Cons(hd);
  }
  cons->tail = tail;
  kk_vector_drop(v,ctx);
  return list;
}

kk_vector_t kk_list_to_vector(kk_std_core__list xs, kk_context_t* ctx) {
  // todo: avoid boxed_dup if xs is unique
  // find the length
  kk_ssize_t len = 0;
  kk_std_core__list ys = xs;
  while (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* cons = kk_std_core__as_Cons(ys);
    len++;
    ys = cons->tail;
  }
  // alloc the vector and copy
  kk_box_t* p;
  kk_vector_t v = kk_vector_alloc_uninit(len, &p, ctx);  
  ys = xs;
  for( kk_ssize_t i = 0; i < len; i++) {
    struct kk_std_core_Cons* cons = kk_std_core__as_Cons(ys);
    ys = cons->tail;
    p[i] = kk_box_dup(cons->head);
  }
  kk_std_core__list_drop(xs,ctx);  // todo: drop while visiting?
  return v;
}

kk_vector_t kk_vector_init( kk_ssize_t n, kk_function_t init, kk_context_t* ctx) {
  kk_box_t* p;
  kk_vector_t v = kk_vector_alloc_uninit(n, &p, ctx);  
  for(kk_ssize_t i = 0; i < n; i++) {
    kk_function_dup(init);
    p[i] = kk_function_call(kk_box_t,(kk_function_t,kk_ssize_t,kk_context_t*),init,(init,i,ctx));
  }
  kk_function_drop(init,ctx);
  return v;
}

kk_box_t kk_main_console( kk_function_t action, kk_context_t* ctx ) {
  return kk_function_call(kk_box_t,(kk_function_t,kk_unit_t,kk_context_t*),action,(action,kk_Unit,ctx));
}


kk_std_core__list kk_string_to_list(kk_string_t s, kk_context_t* ctx) {
  kk_ssize_t len;
  const uint8_t* p = kk_string_buf_borrow(s,&len);
  const uint8_t* const end = p + len;
  kk_std_core__list nil  = kk_std_core__new_Nil(ctx);
  kk_std_core__list list = nil;
  struct kk_std_core_Cons* tl = NULL;
  kk_ssize_t count;
  while( p < end ) {
    kk_char_t c = kk_utf8_read(p,&count);
    p += count;
    kk_std_core__list cons = kk_std_core__new_Cons(kk_reuse_null,kk_char_box(c,ctx), nil, ctx);
    if (tl!=NULL) {
      tl->tail = cons;
    }
    else {
      list = cons;
    }
    tl = kk_std_core__as_Cons(cons);
  }
  kk_string_drop(s,ctx);
  return list;
}

kk_string_t kk_string_from_list(kk_std_core__list cs, kk_context_t* ctx) {
  // TODO: optimize for short strings to write directly into a local buffer?
  // find total UTF8 length
  kk_ssize_t len = 0;
  kk_std_core__list xs = cs;
  while (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* cons = kk_std_core__as_Cons(xs);
    len += kk_utf8_len(kk_char_unbox(cons->head,ctx));
    xs = cons->tail;
  }
  // allocate and copy the characters
  uint8_t* p;
  kk_string_t s = kk_unsafe_string_alloc_buf(len,&p,ctx);  // must be initialized
  xs = cs;
  while (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* cons = kk_std_core__as_Cons(xs);
    kk_ssize_t count;
    kk_utf8_write( kk_char_unbox(cons->head,ctx), p, &count );
    p += count;
    xs = cons->tail;
  }
  kk_assert_internal(*p == 0 && (p - kk_string_buf_borrow(s,NULL)) == len);
  kk_std_core__list_drop(cs,ctx);  // todo: drop while visiting?
  return s;
}

static inline void kk_sslice_start_end_borrowx( kk_std_core__sslice sslice, const uint8_t** start, const uint8_t** end, const uint8_t** sstart, const uint8_t** send) {
  kk_ssize_t slen;
  const uint8_t* s = kk_string_buf_borrow(sslice.str,&slen);
  *start = s + sslice.start;
  *end = s + sslice.start + sslice.len;
  if (sstart != NULL) *sstart = s;
  if (send != NULL) *send = s + slen;
  kk_assert_internal(*start >= s && *start <= *end);
  kk_assert_internal(*end >= *start && *end <= s + slen);
}

static inline void kk_sslice_start_end_borrow( kk_std_core__sslice sslice, const uint8_t** start, const uint8_t** end) {
  kk_sslice_start_end_borrowx(sslice,start,end,NULL,NULL);
}

kk_integer_t kk_slice_count( kk_std_core__sslice sslice, kk_context_t* ctx ) {
  // TODO: optimize this by extending kk_string_count
  const uint8_t* start;
  const uint8_t* end;
  kk_sslice_start_end_borrow(sslice, &start, &end);
  kk_ssize_t count = 0;
  while( start < end && *start != 0 ) {
    const uint8_t* next = kk_utf8_next(start);
    count++;
    start = next;
  }
  kk_std_core__sslice_drop(sslice,ctx);
  return kk_integer_from_ssize_t(count,ctx);
}

kk_string_t kk_slice_to_string( kk_std_core__sslice  sslice, kk_context_t* ctx ) {
  const uint8_t* start;
  const uint8_t* end;
  kk_sslice_start_end_borrow(sslice, &start, &end);
  // is it the full string?
  if (sslice.start == 0 && sslice.len == kk_string_len_borrow(sslice.str)) {
    // TODO: drop sslice and dup sslice.str?
    return sslice.str;
  }
  else {
    // if not, we copy len bytes
    kk_string_t s = kk_string_alloc_dupn_valid_utf8(sslice.len, start, ctx);
    kk_std_core__sslice_drop(sslice,ctx);
    return s;
  }
}

kk_std_core__sslice kk_slice_first( kk_string_t str, kk_context_t* ctx ) {
  kk_ssize_t slen;
  const uint8_t* s = kk_string_buf_borrow(str,&slen);
  const uint8_t* next = (slen > 0 ? kk_utf8_next(s) : s);
  return kk_std_core__new_Sslice(str, 0, (next - s), ctx);
}

kk_std_core__sslice kk_slice_last( kk_string_t str, kk_context_t* ctx ) {
  kk_ssize_t slen;
  const uint8_t* s = kk_string_buf_borrow(str,&slen);
  const uint8_t* end = s + slen;
  const uint8_t* prev = (s==end ? s : kk_utf8_prev(end));
  return kk_std_core__new_Sslice(str, (prev - s), (end - prev), ctx);
}

kk_std_core__sslice kk_slice_between( struct kk_std_core_Sslice slice1, struct kk_std_core_Sslice slice2, kk_context_t* ctx ) {
  const uint8_t* s1 = kk_string_buf_borrow( slice1.str, NULL );
  const uint8_t* s2 = kk_string_buf_borrow( slice2.str, NULL );
  if (s1 != s2) {
    kk_info_message("between: not equal slices: %p vs. %p\n", s1, s2);
    return kk_std_core__new_Sslice(kk_string_empty(), 0, -1, ctx); // invalid slice
  }
  kk_ssize_t start = (slice1.start <= slice2.start ? slice1.start : slice2.start);
  kk_ssize_t len   = (slice1.start <= slice2.start ? slice2.start - slice1.start : slice1.start - slice2.start);
  return kk_std_core__new_Sslice(slice1.str, start, len, ctx);
}

kk_std_core_types__maybe kk_slice_next( struct kk_std_core_Sslice slice, kk_context_t* ctx ) {
  if (slice.len <= 0) {
    kk_std_core__sslice_drop(slice,ctx);
    return kk_std_core_types__new_Nothing(ctx);
  }
  const uint8_t* start;
  const uint8_t* end;
  kk_sslice_start_end_borrow(slice, &start, &end);
  kk_ssize_t clen;
  const kk_char_t c = kk_utf8_read(start,&clen);
  kk_assert_internal(clen > 0 && clen <= slice.len);
  if (clen > slice.len) clen = slice.len;
  // TODO: specialize type to avoid boxing
  kk_std_core__sslice snext = kk_std_core__new_Sslice(slice.str, slice.start + clen, slice.len - clen, ctx);
  kk_std_core_types__tuple2_ res = kk_std_core_types__new_dash__lp__comma__rp_( kk_char_box(c,ctx), kk_std_core__sslice_box(snext,ctx), ctx);
  return kk_std_core_types__new_Just( kk_std_core_types__tuple2__box(res,ctx), ctx );
}

/* Borrow count */
struct kk_std_core_Sslice kk_slice_extend_borrow( struct kk_std_core_Sslice slice, kk_integer_t count, kk_context_t* ctx ) {
  kk_ssize_t cnt = kk_integer_clamp_borrow(count);
  if (cnt==0 || (slice.len <= 0 && cnt<0)) return slice;
  const uint8_t* s0;
  const uint8_t* s1;
  kk_sslice_start_end_borrow(slice,&s0,&s1);
  const uint8_t* t  = s1;
  if (cnt >= 0) {
    do {
      t = kk_utf8_next(t);
      cnt--;
    } while (cnt > 0 && *t != 0);
  }
  else {  // cnt < 0
    const uint8_t* sstart = s0 - slice.start;
    do {
      t = kk_utf8_prev(t);
      cnt++;
    } while (cnt < 0 && t > sstart);
  }
  if (t == s1) return slice;  // length is unchanged
  return kk_std_core__new_Sslice(slice.str, slice.start, (t < s0 ? 0 : (t - s0)), ctx);
}

/* Borrow count */
struct kk_std_core_Sslice kk_slice_advance_borrow( struct kk_std_core_Sslice slice, kk_integer_t count, kk_context_t* ctx ) {
  const kk_ssize_t cnt0 = kk_integer_clamp_borrow(count);
  kk_ssize_t cnt = cnt0;
  if (cnt==0 || (slice.start == 0 && cnt<0)) return slice;
  const uint8_t* sstart;
  const uint8_t* s0;
  const uint8_t* s1;
  const uint8_t* send;
  kk_sslice_start_end_borrowx(slice,&s0,&s1,&sstart,&send);
  // advance the start
  const uint8_t* t0  = s0;
  if (cnt >= 0) {
    do {
      t0 = kk_utf8_next(t0);
      cnt--;
    } while (cnt > 0 && t0 < send);
  }
  else {  // cnt < 0
    do {
      t0 = kk_utf8_prev(t0);
      cnt++;
    } while (cnt < 0 && t0 > sstart);
  }
  if (t0 == s0) return slice;  // start is unchanged
  // "t0" points to the new start, now advance the end by the same amount of codepoints
  const uint8_t* t1 = s1;
  cnt = cnt0;
  if (cnt >= 0) {
    do {
      t1 = kk_utf8_next(t1);
      cnt--;
    } while (cnt > 0 && t1 < send);
  }
  else {  // cnt < 0
    do {
      t1 = kk_utf8_prev(t1);
      cnt++;
    } while (cnt < 0 && t1 > sstart);
  }
  // t1 points to the new end
  kk_assert_internal(t1 >= t0);
  return kk_std_core__new_Sslice(slice.str, (t0 - sstart), (t1 - t0), ctx);
}

/* Borrow iupto */
struct kk_std_core_Sslice kk_slice_common_prefix_borrow( kk_string_t str1, kk_string_t str2, kk_integer_t iupto, kk_context_t* ctx ) {
  const uint8_t* s1 = kk_string_buf_borrow(str1,NULL);
  const uint8_t* s2 = kk_string_buf_borrow(str2,NULL);
  kk_ssize_t upto = kk_integer_clamp_ssize_t_borrow(iupto);
  kk_ssize_t count;
  for(count = 0; count < upto && *s1 != 0 && *s2 != 0; count++, s1++, s2++ ) {
    if (*s1 != *s2) break;
  }
  kk_string_drop(str2,ctx);
  return kk_std_core__new_Sslice(str1, 0, count, ctx);
}


kk_std_core__error kk_error_ok( kk_box_t result, kk_context_t* ctx ) {
  return kk_std_core__new_Ok( result, ctx );
}

kk_std_core__error kk_error_from_errno( int err, kk_context_t* ctx ) {  
  kk_string_t msg;
  #if defined(_GNU_SOURCE) && !defined(WIN32) && !defined(__APPLE__) && !defined(__FreeBSD__)
    // GNU version of strerror_r
    char buf[256];
    char* serr = strerror_r(err, buf, 255); buf[255] = 0;
    msg = kk_string_alloc_from_qutf8( serr, ctx );
  #elif (/* _POSIX_C_SOURCE >= 200112L ||*/ !defined(WIN32) && (_XOPEN_SOURCE >= 600 || defined(__APPLE__) || defined(__FreeBSD__)))
    // XSI version of strerror_r
    char buf[256];
    strerror_r(err, buf, 255); buf[255] = 0;
    msg = kk_string_alloc_from_qutf8( buf, ctx );
  #elif defined(_MSC_VER) || (__STDC_VERSION__ >= 201112L || __cplusplus >= 201103L)
    // MSVC, or C/C++ 11
    char buf[256];
    strerror_s(buf, 255, err); buf[255] = 0;
    msg = kk_string_alloc_from_qutf8( buf, ctx );
  #else
    // Old style
    msg = kk_string_alloc_from_qutf8( strerror(err), ctx );
  #endif
  return kk_std_core__new_Error( kk_std_core__new_Exception( msg, kk_std_core__new_ExnSystem(kk_reuse_null, kk_integer_from_int(err,ctx), ctx), ctx), ctx );  
}


kk_unit_t kk_assert_fail( kk_string_t msg, kk_context_t* ctx ) {
  kk_fatal_error(EINVAL, "assertion failed: %s\n", kk_string_cbuf_borrow(msg,NULL));
  kk_string_drop(msg,ctx);
  return kk_Unit;
}
kk_define_string_literal(, kk_std_core__tag_ExnError, 17, "std/core/ExnError")
kk_define_string_literal(, kk_std_core__tag_ExnAssert, 18, "std/core/ExnAssert")
kk_define_string_literal(, kk_std_core__tag_ExnTodo, 16, "std/core/ExnTodo")
kk_define_string_literal(, kk_std_core__tag_ExnRange, 17, "std/core/ExnRange")
kk_define_string_literal(, kk_std_core__tag_ExnPattern, 19, "std/core/ExnPattern")
kk_define_string_literal(, kk_std_core__tag_ExnSystem, 18, "std/core/ExnSystem")
kk_define_string_literal(, kk_std_core__tag_ExnInternal, 20, "std/core/ExnInternal")

kk_std_core__exception kk_std_core__copy(kk_std_core__exception _this, kk_std_core_types__optional message0, kk_std_core_types__optional info0, kk_context_t* _ctx) { /* (exception, message : optional<string>, info : optional<exception-info>) -> exception */ 
  kk_string_t _x20450;
  if (kk_std_core_types__is_Optional(message0)) {
    kk_box_t _box_x18413 = message0._cons.Optional.value;
    kk_string_t _message_1676 = kk_string_unbox(_box_x18413);
    kk_string_dup(_message_1676);
    kk_std_core_types__optional_drop(message0, _ctx);
    kk_Unit;
    _x20450 = _message_1676; /*string*/
    goto _match20451;
  }
  kk_string_t _x = _this.message;
  kk_string_dup(_x);
  _x20450 = _x; /*string*/
  _match20451: ;
  kk_std_core__exception_info _x20453;
  if (kk_std_core_types__is_Optional(info0)) {
    kk_box_t _box_x18414 = info0._cons.Optional.value;
    kk_std_core__exception_info _info_1682 = kk_std_core__exception_info_unbox(_box_x18414, NULL);
    kk_std_core__exception_info_dup(_info_1682);
    kk_std_core_types__optional_drop(info0, _ctx);
    kk_Unit;
    kk_std_core__exception_drop(_this, _ctx);
    kk_Unit;
    _x20453 = _info_1682; /*exception-info*/
    goto _match20454;
  }
  kk_std_core__exception_info _x0 = _this.info;
  kk_std_core__exception_info_dup(_x0);
  kk_std_core__exception_drop(_this, _ctx);
  kk_Unit;
  _x20453 = _x0; /*exception-info*/
  _match20454: ;
  return kk_std_core__new_Exception(_x20450, _x20453, _ctx);
}

kk_std_core__sslice kk_std_core__copy_2(kk_std_core__sslice _this, kk_std_core_types__optional str0, kk_std_core_types__optional start0, kk_std_core_types__optional len0, kk_context_t* _ctx) { /* (sslice, str : optional<string>, start : optional<ssize_t>, len : optional<ssize_t>) -> sslice */ 
  kk_string_t _x20460;
  if (kk_std_core_types__is_Optional(str0)) {
    kk_box_t _box_x18416 = str0._cons.Optional.value;
    kk_string_t _str_1814 = kk_string_unbox(_box_x18416);
    kk_string_dup(_str_1814);
    kk_std_core_types__optional_drop(str0, _ctx);
    kk_Unit;
    _x20460 = _str_1814; /*string*/
    goto _match20461;
  }
  kk_string_t _x = _this.str;
  kk_string_dup(_x);
  _x20460 = _x; /*string*/
  _match20461: ;
  kk_ssize_t _x20463;
  if (kk_std_core_types__is_Optional(start0)) {
    kk_box_t _box_x18417 = start0._cons.Optional.value;
    kk_ssize_t _start_1820 = kk_ssize_unbox(_box_x18417, NULL);
    kk_std_core_types__optional_drop(start0, _ctx);
    kk_Unit;
    _x20463 = _start_1820; /*ssize_t*/
    goto _match20464;
  }
  kk_ssize_t _x0 = _this.start;
  _x20463 = _x0; /*ssize_t*/
  _match20464: ;
  kk_ssize_t _x20466;
  if (kk_std_core_types__is_Optional(len0)) {
    kk_box_t _box_x18418 = len0._cons.Optional.value;
    kk_ssize_t _len_1826 = kk_ssize_unbox(_box_x18418, NULL);
    kk_std_core_types__optional_drop(len0, _ctx);
    kk_Unit;
    kk_std_core__sslice_drop(_this, _ctx);
    kk_Unit;
    _x20466 = _len_1826; /*ssize_t*/
    goto _match20467;
  }
  kk_ssize_t _x1 = _this.len;
  kk_std_core__sslice_drop(_this, _ctx);
  kk_Unit;
  _x20466 = _x1; /*ssize_t*/
  _match20467: ;
  return kk_std_core__new_Sslice(_x20460, _x20463, _x20466, _ctx);
}

kk_std_core__stream kk_std_core__copy_3(kk_std_core__stream _this, kk_std_core_types__optional head0, kk_std_core_types__optional tail0, kk_context_t* _ctx) { /* forall<a> (stream<a>, head : optional<a>, tail : optional<stream<a>>) -> stream<a> */ 
  kk_box_t _x20471;
  if (kk_std_core_types__is_Optional(head0)) {
    kk_box_t _head_1872 = head0._cons.Optional.value;
    _x20471 = _head_1872; /*1889*/
  }
  else {
    struct kk_std_core_Next* _con20472 = kk_std_core__as_Next(_this);
    kk_box_t _x = _con20472->head;
    kk_box_dup(_x);
    _x20471 = _x; /*1889*/
  }
  kk_std_core__stream _x20473;
  if (kk_std_core_types__is_Optional(tail0)) {
    kk_box_t _box_x18419 = tail0._cons.Optional.value;
    kk_std_core__stream _tail_1879 = kk_std_core__stream_unbox(_box_x18419, NULL);
    kk_std_core__stream_dup(_tail_1879);
    kk_std_core_types__optional_drop(tail0, _ctx);
    kk_Unit;
    kk_std_core__stream_drop(_this, _ctx);
    kk_Unit;
    _x20473 = _tail_1879; /*stream<1889>*/
    goto _match20474;
  }
  struct kk_std_core_Next* _con20476 = kk_std_core__as_Next(_this);
  kk_box_t _pat01 = _con20476->head;
  kk_std_core__stream _x0 = _con20476->tail;
  if (kk_likely(kk_std_core__stream_is_unique(_this))) {
    kk_box_drop(_pat01, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core__stream_free(_this);
  }
  else {
    kk_std_core__stream_dup(_x0);
    kk_Unit;
    kk_std_core__stream_decref(_this, _ctx);
    kk_Unit;
  }
  _x20473 = _x0; /*stream<1889>*/
  _match20474: ;
  return kk_std_core__new_Next(kk_reuse_null, _x20471, _x20473, _ctx);
}
 
// runtime tag for the `:exn` effect

kk_std_core_hnd__htag kk_std_core__tag_exn;
 
// handler for the `:exn` effect

kk_box_t kk_std_core__handle_exn(int32_t cfc, kk_std_core__hnd_exn hnd, kk_function_t ret, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e,b> (cfc : int32, hnd : .hnd-exn<e,b>, ret : (res : a) -> e b, action : () -> <exn|e> a) -> e b */ 
  kk_std_core_hnd__htag _b_18425_18420 = kk_std_core_hnd__htag_dup(kk_std_core__tag_exn); /*std/core/hnd/htag<.hnd-exn>*/;
  int32_t _b_18426_18421 = cfc; /*int32*/;
  kk_std_core__hnd_exn _b_18427_18422 = hnd; /*.hnd-exn<1956,1957>*/;
  kk_function_t _b_18428_18423 = ret; /*(res : 1955) -> 1956 1957*/;
  kk_function_t _b_18429_18424 = action; /*() -> <exn|1956> 1955*/;
  return kk_std_core_hnd__hhandle(_b_18425_18420, _b_18426_18421, kk_std_core__hnd_exn_box(_b_18427_18422, _ctx), _b_18428_18423, _b_18429_18424, _ctx);
}
 
// Unsafe: transform any type to a `null` type; used internally by the compiler.

kk_std_core__null kk_std_core__null_any(kk_box_t x, kk_context_t* _ctx) { /* forall<a> (x : a) -> null<a> */ 
  return ((x).box == kk_box_null.box ? kk_datatype_from_ptr(NULL) : kk_datatype_unbox(x));
}

kk_ref_t kk_std_core_redirect;
 
// Print a string to the console, including a final newline character.

kk_unit_t kk_std_core_xprintsln(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> console () */ 
  kk_println(s,kk_context()); return kk_Unit;
}

kk_integer_t kk_std_core_string_compare(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> int */ 
  return kk_string_cmp_int(x,y,kk_context());
}
 
// Convert an integer to an `:int32`. The number is _clamped_ to the maximal or minimum `:int32`
// value if it is outside the range of an `:int32`.

int32_t kk_std_core_int32(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int32 */ 
  return kk_integer_clamp32(i,kk_context());
}
 
// Convert an integer to an `:ssize_t`. The number is _clamped_ to the maximal or minimum `:ssize_t`
// value if it is outside the range of an `:ssize_t`.

kk_ssize_t kk_std_core_ssize__t(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> ssize_t */ 
  return kk_integer_clamp_ssize_t(i,kk_context());
}
 
// Convert a character to a string

kk_string_t kk_std_core_string(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> string */ 
  return kk_string_from_char(c,kk_context());
}
 
// Convert a vector of characters to a string.

kk_string_t kk_std_core_string_1(kk_vector_t _arg1, kk_context_t* _ctx) { /* (vector<char>) -> string */ 
  return kk_string_from_chars(_arg1,kk_context());
}
 
// Convert a list of characters to a string

kk_string_t kk_std_core_string_2(kk_std_core__list cs, kk_context_t* _ctx) { /* (cs : list<char>) -> total string */ 
  return kk_string_from_list(cs,kk_context());
}
 
// O(n). Copy the `slice` argument into a fresh string.
// Takes O(1) time if the slice covers the entire string.

kk_string_t kk_std_core_string_3(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> string */ 
  return kk_slice_to_string(slice0,kk_context());
}
 
// Convert a vector to a list with an optional tail.

kk_std_core__list kk_std_core_vlist(kk_vector_t v, kk_std_core_types__optional tail0, kk_context_t* _ctx) { /* forall<a> (v : vector<a>, tail : optional<list<a>>) -> list<a> */ 
  kk_std_core__list _x20482;
  if (kk_std_core_types__is_Optional(tail0)) {
    kk_box_t _box_x18433 = tail0._cons.Optional.value;
    kk_std_core__list _tail_2083 = kk_std_core__list_unbox(_box_x18433, NULL);
    kk_std_core__list_dup(_tail_2083);
    kk_std_core_types__optional_drop(tail0, _ctx);
    kk_Unit;
    _x20482 = _tail_2083; /*list<2093>*/
    goto _match20483;
  }
  _x20482 = kk_std_core__new_Nil(_ctx); /*list<2093>*/
  _match20483: ;
  return kk_vector_to_list(v,_x20482,kk_context());
}

kk_string_t kk_std_core_int_show_hex(kk_integer_t i, bool use_capitals, kk_context_t* _ctx) { /* (i : int, use-capitals : bool) -> string */ 
  return kk_integer_to_hex_string(i,use_capitals,kk_context());
}

kk_string_t kk_std_core_repeatz(kk_string_t s, kk_ssize_t n, kk_context_t* _ctx) { /* (s : string, n : ssize_t) -> string */ 
  return kk_string_repeat(s,n,kk_context());
}

kk_string_t kk_std_core_show_expx(double d, int32_t prec, kk_context_t* _ctx) { /* (d : double, prec : int32) -> string */ 
  return kk_double_show_exp(d,prec,kk_context());
}

kk_string_t kk_std_core_show_fixedx(double d, int32_t prec, kk_context_t* _ctx) { /* (d : double, prec : int32) -> string */ 
  return kk_double_show_fixed(d,prec,kk_context());
}
 
// Print a string to the console

kk_unit_t kk_std_core_xprints(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> console () */ 
  kk_print(s,kk_context()); return kk_Unit;
}
 
// Raise an integer `i` to the power of `exp`.

kk_integer_t kk_std_core_pow(kk_integer_t i, kk_integer_t exp, kk_context_t* _ctx) { /* (i : int, exp : int) -> int */ 
  return kk_integer_pow(i,exp,kk_context());
}
 
// O(`count`). Advance the start position of a string slice by `count` characters
// up to the end of the string.
// A negative `count` advances the start position backwards upto the first position
// in a string.
// Maintains the character count of the original slice upto the end of the string.
// For example:
//
// * `"abc".first.advance(1).string == "b"`,
// * `"abc".first.advance(3).string == ""`,
// * `"abc".last.advance(-1).string == "b"`.
//

kk_std_core__sslice kk_std_core_advance(kk_std_core__sslice slice0, kk_integer_t count, kk_context_t* _ctx) { /* (slice : sslice, count : int) -> sslice */ 
  return kk_slice_advance_borrow(slice0,count,kk_context());
}

kk_unit_t kk_std_core_unsafe_assert_fail(kk_string_t msg, kk_context_t* _ctx) { /* (msg : string) -> () */ 
  kk_assert_fail(msg,kk_context()); return kk_Unit;
}
 
// O(`count`). Extend a string slice by `count` characters up to the end of the string.
// A negative `count` shrinks the slice up to the empty slice.
// For example:
//
// * `"abc".first.extend(1).string == "ab"`
// * `"abc".last.extend(-1).string == ""`
//

kk_std_core__sslice kk_std_core_extend(kk_std_core__sslice slice0, kk_integer_t count, kk_context_t* _ctx) { /* (slice : sslice, count : int) -> sslice */ 
  return kk_slice_extend_borrow(slice0,count,kk_context());
}

kk_std_core__sslice kk_std_core_first1(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> sslice */ 
  return kk_slice_first(s,kk_context());
}
 
// Convert a string to upper-case

kk_string_t kk_std_core_to_upper(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  return kk_string_to_upper(s,kk_context());
}

kk_integer_t kk_std_core_cdiv_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx) { /* (i : int, n : int) -> int */ 
  return kk_integer_cdiv_pow10(i,n,kk_context());
}

kk_integer_t kk_std_core_mul_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx) { /* (i : int, n : int) -> int */ 
  return kk_integer_mul_pow10(i,n,kk_context());
}
 
// Return the common prefix of two strings (upto `upto` characters (default is minimum length of the two strings))

kk_std_core__sslice kk_std_core_common_prefix(kk_string_t s, kk_string_t t, kk_std_core_types__optional upto, kk_context_t* _ctx) { /* (s : string, t : string, upto : optional<int>) -> sslice */ 
  kk_integer_t _borrowedArg_20319;
  if (kk_std_core_types__is_Optional(upto)) {
    kk_box_t _box_x18434 = upto._cons.Optional.value;
    kk_integer_t _upto_2182 = kk_integer_unbox(_box_x18434);
    _borrowedArg_20319 = kk_integer_dup(_upto_2182); /*int*/
    goto _match20485;
  }
  _borrowedArg_20319 = kk_integer_from_small(-1); /*int*/
  _match20485: ;
  kk_std_core__sslice _borrowApp_20320 = kk_slice_common_prefix_borrow(s,t,_borrowedArg_20319,kk_context()); /*sslice*/;
  kk_integer_drop(_borrowedArg_20319, _ctx);
  return _borrowApp_20320;
}
 
// lifted

kk_std_core__list kk_std_core__ctail_lift17164_concat(kk_std_core__list ys, kk_std_core__list zss, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, zss : list<list<a>>, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20487 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con20487->head;
    kk_std_core__list yy = _con20487->tail;
    kk_reuse_t _ru_20330 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20330 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_std_core__list _ctail_17291 = kk_std_core__list_hole(); /*list<2230>*/;
    kk_std_core__list _ctail_17292;
    if (kk_likely(_ru_20330!=NULL)) {
      struct kk_std_core_Cons* _con20488 = (struct kk_std_core_Cons*)_ru_20330;
      _con20488->tail = _ctail_17291;
      _ctail_17292 = kk_std_core__base_Cons(_con20488); /*list<2230>*/
    }
    else {
      _ctail_17292 = kk_std_core__new_Cons(kk_reuse_null, y, _ctail_17291, _ctx); /*list<2230>*/
    }
    { // tailcall
      kk_std_core_types__ctail _x20489;
      kk_box_t* field_17780 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17292)->tail)); /*cfield<list<2230>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18441 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18441, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17292, _ctx);
        kk_std_core__list _b_18456_18446 = head0; /*list<2230>*/;
        kk_box_t* _b_18457_18447 = field_17780; /*cfield<list<2230>>*/;
        _x20489 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18456_18446, _ctx), _b_18457_18447, _ctx); /*ctail<58>*/
        goto _match20490;
      }
      kk_std_core__list _b_18458_18448 = _ctail_17292; /*list<2230>*/;
      kk_box_t* _b_18459_18449 = field_17780; /*cfield<list<2230>>*/;
      _x20489 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18458_18448, _ctx), _b_18459_18449, _ctx); /*ctail<58>*/
      _match20490: ;
      ys = yy;
      _acc = _x20489;
      goto kk__tailcall;
    }
  }
  if (kk_std_core__is_Cons(zss)) {
    struct kk_std_core_Cons* _con20492 = kk_std_core__as_Cons(zss);
    kk_box_t _box_x18450 = _con20492->head;
    kk_std_core__list zzs = _con20492->tail;
    kk_std_core__list zs = kk_std_core__list_unbox(_box_x18450, NULL);
    if (kk_likely(kk_std_core__list_is_unique(zss))) {
      kk_std_core__list_free(zss);
    }
    else {
      kk_std_core__list_dup(zs);
      kk_std_core__list_dup(zzs);
      kk_Unit;
      kk_std_core__list_decref(zss, _ctx);
      kk_Unit;
    }
    { // tailcall
      ys = zs;
      zss = zzs;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18451 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18451, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17164_concat(kk_std_core__list ys0, kk_std_core__list zss0, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, zss : list<list<a>>) -> list<a> */ 
  return kk_std_core__ctail_lift17164_concat(ys0, zss0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
extern kk_box_t kk_std_core_const_fun20495_1(kk_function_t _fself, kk_box_t ___wildcard__123__7, kk_context_t* _ctx) {
  struct kk_std_core_const_fun20495__t_1* _self = kk_function_as(struct kk_std_core_const_fun20495__t_1*, _fself);
  kk_box_t default0 = _self->default0; /* 2258 */
  kk_drop_match(_self, {kk_box_dup(default0);}, {}, _ctx)
  kk_box_drop(___wildcard__123__7, _ctx);
  return default0;
}
 
// If the slice is not empty,
// return the first character, and a new slice that is advanced by 1.

kk_std_core_types__maybe kk_std_core_next(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> maybe<(char, sslice)> */ 
  return kk_slice_next(slice0,kk_context());
}
 
// Return the number of decimal digits of `i`. Return `0` when `i==0`.

kk_integer_t kk_std_core_count_digits(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int */ 
  return kk_integer_count_digits(i,kk_context());
}
 
// Transform an `:error` type to an `:either` value.

kk_std_core_types__either kk_std_core_either(kk_std_core__error t, kk_context_t* _ctx) { /* forall<a> (t : error<a>) -> either<exception,a> */ 
  if (kk_std_core__is_Error(t)) {
    kk_std_core__exception exn0 = t._cons.Error.exception;
    kk_std_core__exception_dup(exn0);
    kk_std_core__error_drop(t, _ctx);
    kk_Unit;
    kk_std_core__exception _b_18461_18460 = exn0; /*exception*/;
    return kk_std_core_types__new_Left(kk_std_core__exception_box(_b_18461_18460, _ctx), _ctx);
  }
  kk_box_t x = t._cons.Ok.result;
  kk_box_dup(x);
  kk_std_core__error_drop(t, _ctx);
  kk_Unit;
  return kk_std_core_types__new_Right(x, _ctx);
}

bool kk_std_core_xends_with(kk_string_t s, kk_string_t post, kk_context_t* _ctx) { /* (s : string, post : string) -> bool */ 
  return kk_string_ends_with(s,post,kk_context());
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17696_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_std_core__list zz, kk_std_core__list ys1_17211, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, f : (a) -> e list<b>, zz : list<a>, ys1.17211 : list<b>) -> e list<b> */ 
  return kk_std_core__ctail_lift17165_flatmap(f, ys1_17211, zz, _acc, _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17697_op(kk_function_t _accm, kk_function_t f0, kk_std_core__list zz0, kk_std_core__list ys1_172110, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, f : (a) -> e list<b>, zz : list<a>, ys1.17211 : list<b>) -> e list<b> */ 
  return kk_std_core__ctailm_lift17165_flatmap(f0, ys1_172110, zz0, _accm, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__ctail_lift17165_flatmap_fun20503__t {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_std_core__list zz1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_lift17165_flatmap_fun20503(kk_function_t _fself, kk_std_core__list ys1_172111, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_lift17165_flatmap_fun20503(kk_function_t f1, kk_std_core__list zz1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17165_flatmap_fun20503__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_lift17165_flatmap_fun20503__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_lift17165_flatmap_fun20503, kk_context());
  _self->f1 = f1;
  _self->zz1 = zz1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_lift17165_flatmap_fun20503(kk_function_t _fself, kk_std_core__list ys1_172111, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17165_flatmap_fun20503__t* _self = kk_function_as(struct kk_std_core__ctail_lift17165_flatmap_fun20503__t*, _fself);
  kk_function_t f1 = _self->f1; /* (2407) -> 2409 list<2408> */
  kk_std_core__list zz1 = _self->zz1; /* list<2407> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<2408>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_std_core__list_dup(zz1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17696_op(_acc0, f1, zz1, ys1_172111, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_lift17165_flatmap_fun20505__t {
  struct kk_function_s _base;
  kk_function_t _b_18488_18477;
};
static kk_box_t kk_std_core__ctail_lift17165_flatmap_fun20505(kk_function_t _fself, kk_box_t _b_18478, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_lift17165_flatmap_fun20505(kk_function_t _b_18488_18477, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17165_flatmap_fun20505__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_lift17165_flatmap_fun20505__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_lift17165_flatmap_fun20505, kk_context());
  _self->_b_18488_18477 = _b_18488_18477;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_lift17165_flatmap_fun20505(kk_function_t _fself, kk_box_t _b_18478, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17165_flatmap_fun20505__t* _self = kk_function_as(struct kk_std_core__ctail_lift17165_flatmap_fun20505__t*, _fself);
  kk_function_t _b_18488_18477 = _self->_b_18488_18477; /* (ys1.172111 : list<2408>) -> 2409 list<2408> */
  kk_drop_match(_self, {kk_function_dup(_b_18488_18477);}, {}, _ctx)
  kk_std_core__list _x20506;
  kk_std_core__list _x20507 = kk_std_core__list_unbox(_b_18478, _ctx); /*list<2408>*/
  _x20506 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_18488_18477, (_b_18488_18477, _x20507, _ctx)); /*list<2408>*/
  return kk_std_core__list_box(_x20506, _ctx);
}

kk_std_core__list kk_std_core__ctail_lift17165_flatmap(kk_function_t f1, kk_std_core__list ys, kk_std_core__list zs, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e list<b>, ys : list<b>, zs : list<a>, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20496 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con20496->head;
    kk_std_core__list yy = _con20496->tail;
    kk_reuse_t _ru_20332 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20332 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_std_core__list _ctail_17293 = kk_std_core__list_hole(); /*list<2408>*/;
    kk_std_core__list _ctail_17294;
    if (kk_likely(_ru_20332!=NULL)) {
      struct kk_std_core_Cons* _con20497 = (struct kk_std_core_Cons*)_ru_20332;
      _con20497->tail = _ctail_17293;
      _ctail_17294 = kk_std_core__base_Cons(_con20497); /*list<2408>*/
    }
    else {
      _ctail_17294 = kk_std_core__new_Cons(kk_reuse_null, y, _ctail_17293, _ctx); /*list<2408>*/
    }
    { // tailcall
      kk_std_core_types__ctail _x20498;
      kk_box_t* field_17785 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17294)->tail)); /*cfield<list<2408>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x18468 = _acc0._cons.CTail._field1;
        kk_box_t* last0 = _acc0._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18468, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17294, _ctx);
        kk_std_core__list _b_18484_18473 = head0; /*list<2408>*/;
        kk_box_t* _b_18485_18474 = field_17785; /*cfield<list<2408>>*/;
        _x20498 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18484_18473, _ctx), _b_18485_18474, _ctx); /*ctail<58>*/
        goto _match20499;
      }
      kk_std_core__list _b_18486_18475 = _ctail_17294; /*list<2408>*/;
      kk_box_t* _b_18487_18476 = field_17785; /*cfield<list<2408>>*/;
      _x20498 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18486_18475, _ctx), _b_18487_18476, _ctx); /*ctail<58>*/
      _match20499: ;
      ys = yy;
      _acc0 = _x20498;
      goto kk__tailcall;
    }
  }
  if (kk_std_core__is_Cons(zs)) {
    struct kk_std_core_Cons* _con20501 = kk_std_core__as_Cons(zs);
    kk_box_t z = _con20501->head;
    kk_std_core__list zz1 = _con20501->tail;
    if (kk_likely(kk_std_core__list_is_unique(zs))) {
      kk_std_core__list_free(zs);
    }
    else {
      kk_box_dup(z);
      kk_std_core__list_dup(zz1);
      kk_Unit;
      kk_std_core__list_decref(zs, _ctx);
      kk_Unit;
    }
    kk_std_core__list x_17786;
    kk_function_t _x20502 = kk_function_dup(f1); /*(2407) -> 2409 list<2408>*/
    x_17786 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _x20502, (_x20502, z, _ctx)); /*list<2408>*/
    if (kk_yielding(kk_context())) {
      kk_std_core__list_drop(x_17786, _ctx);
      kk_function_t _b_18488_18477 = kk_std_core__new_ctail_lift17165_flatmap_fun20503(f1, zz1, _acc0, _ctx); /*(ys1.172111 : list<2408>) -> 2409 list<2408>*/;
      kk_box_t _x20504 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_lift17165_flatmap_fun20505(_b_18488_18477, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20504, _ctx);
    }
    { // tailcall
      ys = x_17786;
      zs = zz1;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18479 = _acc0._cons.CTail._field1;
    kk_box_t* last2 = _acc0._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18479, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__ctailm_lift17165_flatmap_fun20511__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t y0;
};
static kk_std_core__list kk_std_core__ctailm_lift17165_flatmap_fun20511(kk_function_t _fself, kk_std_core__list _ctail_17296, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17165_flatmap_fun20511(kk_function_t _accm0, kk_box_t y0, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17165_flatmap_fun20511__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17165_flatmap_fun20511__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17165_flatmap_fun20511, kk_context());
  _self->_accm0 = _accm0;
  _self->y0 = y0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_lift17165_flatmap_fun20511(kk_function_t _fself, kk_std_core__list _ctail_17296, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17165_flatmap_fun20511__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17165_flatmap_fun20511__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<2408>) -> list<2408> */
  kk_box_t y0 = _self->y0; /* 2408 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(y0);}, {}, _ctx)
  kk_std_core__list _x20512 = kk_std_core__new_Cons(kk_reuse_null, y0, _ctail_17296, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x20512, _ctx));
}


// lift anonymous function
struct kk_std_core__ctailm_lift17165_flatmap_fun20515__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_std_core__list zz2;
};
static kk_std_core__list kk_std_core__ctailm_lift17165_flatmap_fun20515(kk_function_t _fself, kk_std_core__list ys1_172113, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17165_flatmap_fun20515(kk_function_t _accm0, kk_function_t f2, kk_std_core__list zz2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17165_flatmap_fun20515__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17165_flatmap_fun20515__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17165_flatmap_fun20515, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->zz2 = zz2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_lift17165_flatmap_fun20515(kk_function_t _fself, kk_std_core__list ys1_172113, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17165_flatmap_fun20515__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17165_flatmap_fun20515__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<2408>) -> list<2408> */
  kk_function_t f2 = _self->f2; /* (2407) -> 2409 list<2408> */
  kk_std_core__list zz2 = _self->zz2; /* list<2407> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_std_core__list_dup(zz2);}, {}, _ctx)
  return kk_std_core__mlift17697_op(_accm0, f2, zz2, ys1_172113, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_lift17165_flatmap_fun20517__t {
  struct kk_function_s _base;
  kk_function_t _b_18491_18489;
};
static kk_box_t kk_std_core__ctailm_lift17165_flatmap_fun20517(kk_function_t _fself, kk_box_t _b_18490, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17165_flatmap_fun20517(kk_function_t _b_18491_18489, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17165_flatmap_fun20517__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17165_flatmap_fun20517__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17165_flatmap_fun20517, kk_context());
  _self->_b_18491_18489 = _b_18491_18489;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_lift17165_flatmap_fun20517(kk_function_t _fself, kk_box_t _b_18490, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17165_flatmap_fun20517__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17165_flatmap_fun20517__t*, _fself);
  kk_function_t _b_18491_18489 = _self->_b_18491_18489; /* (ys1.172113 : list<2408>) -> 2409 list<2408> */
  kk_drop_match(_self, {kk_function_dup(_b_18491_18489);}, {}, _ctx)
  kk_std_core__list _x20518;
  kk_std_core__list _x20519 = kk_std_core__list_unbox(_b_18490, _ctx); /*list<2408>*/
  _x20518 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_18491_18489, (_b_18491_18489, _x20519, _ctx)); /*list<2408>*/
  return kk_std_core__list_box(_x20518, _ctx);
}

kk_std_core__list kk_std_core__ctailm_lift17165_flatmap(kk_function_t f2, kk_std_core__list ys0, kk_std_core__list zs0, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e list<b>, ys : list<b>, zs : list<a>, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys0)) {
    struct kk_std_core_Cons* _con20509 = kk_std_core__as_Cons(ys0);
    kk_box_t y0 = _con20509->head;
    kk_std_core__list yy0 = _con20509->tail;
    if (kk_likely(kk_std_core__list_is_unique(ys0))) {
      kk_std_core__list_free(ys0);
    }
    else {
      kk_box_dup(y0);
      kk_std_core__list_dup(yy0);
      kk_Unit;
      kk_std_core__list_decref(ys0, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_function_t _x20510 = kk_std_core__new_ctailm_lift17165_flatmap_fun20511(_accm0, y0, _ctx); /*(list<2408>) -> list<2408>*/
      ys0 = yy0;
      _accm0 = _x20510;
      goto kk__tailcall;
    }
  }
  if (kk_std_core__is_Cons(zs0)) {
    struct kk_std_core_Cons* _con20513 = kk_std_core__as_Cons(zs0);
    kk_box_t z0 = _con20513->head;
    kk_std_core__list zz2 = _con20513->tail;
    if (kk_likely(kk_std_core__list_is_unique(zs0))) {
      kk_std_core__list_free(zs0);
    }
    else {
      kk_box_dup(z0);
      kk_std_core__list_dup(zz2);
      kk_Unit;
      kk_std_core__list_decref(zs0, _ctx);
      kk_Unit;
    }
    kk_std_core__list x0_17791;
    kk_function_t _x20514 = kk_function_dup(f2); /*(2407) -> 2409 list<2408>*/
    x0_17791 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _x20514, (_x20514, z0, _ctx)); /*list<2408>*/
    if (kk_yielding(kk_context())) {
      kk_std_core__list_drop(x0_17791, _ctx);
      kk_function_t _b_18491_18489 = kk_std_core__new_ctailm_lift17165_flatmap_fun20515(_accm0, f2, zz2, _ctx); /*(ys1.172113 : list<2408>) -> 2409 list<2408>*/;
      kk_box_t _x20516 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_lift17165_flatmap_fun20517(_b_18491_18489, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20516, _ctx);
    }
    { // tailcall
      ys0 = x0_17791;
      zs0 = zz2;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17165_flatmap_fun20520__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core__lift17165_flatmap_fun20520(kk_function_t _fself, kk_std_core__list _ctail_17295, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17165_flatmap_fun20520(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core__lift17165_flatmap_fun20520, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core__lift17165_flatmap_fun20520(kk_function_t _fself, kk_std_core__list _ctail_17295, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17295;
}

kk_std_core__list kk_std_core__lift17165_flatmap(kk_function_t f3, kk_std_core__list ys1, kk_std_core__list zs1, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e list<b>, ys : list<b>, zs : list<a>) -> e list<b> */ 
  bool _match_20316 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_20316) {
    return kk_std_core__ctail_lift17165_flatmap(f3, ys1, zs1, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_lift17165_flatmap(f3, ys1, zs1, kk_std_core__new_lift17165_flatmap_fun20520(_ctx), _ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17166_reverse_append(kk_std_core__list acc, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a> (acc : list<a>, ys : list<a>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20521 = kk_std_core__as_Cons(ys);
    kk_box_t x = _con20521->head;
    kk_std_core__list xx = _con20521->tail;
    kk_reuse_t _ru_20336 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20336 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x20522;
      if (kk_likely(_ru_20336!=NULL)) {
        struct kk_std_core_Cons* _con20523 = (struct kk_std_core_Cons*)_ru_20336;
        _con20523->tail = acc;
        _x20522 = kk_std_core__base_Cons(_con20523); /*list<61>*/
      }
      else {
        _x20522 = kk_std_core__new_Cons(kk_reuse_null, x, acc, _ctx); /*list<61>*/
      }
      acc = _x20522;
      ys = xx;
      goto kk__tailcall;
    }
  }
  return acc;
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17699_force_fun20528__t {
  struct kk_function_s _base;
  kk_ref_t r;
};
static kk_box_t kk_std_core__mlift17699_force_fun20528(kk_function_t _fself, kk_box_t x00, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17699_force_fun20528(kk_ref_t r, kk_context_t* _ctx) {
  struct kk_std_core__mlift17699_force_fun20528__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17699_force_fun20528__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17699_force_fun20528, kk_context());
  _self->r = r;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17699_force_fun20528(kk_function_t _fself, kk_box_t x00, kk_context_t* _ctx) {
  struct kk_std_core__mlift17699_force_fun20528__t* _self = kk_function_as(struct kk_std_core__mlift17699_force_fun20528__t*, _fself);
  kk_ref_t r = _self->r; /* ref<global,either<() -> 2544 2543,2543>> */
  kk_drop_match(_self, {kk_ref_dup(r);}, {}, _ctx)
  kk_unit_t __ = kk_Unit;
  kk_unit_t _borrowApp_20314 = kk_Unit;
  kk_box_t _x20529;
  kk_std_core_types__either _x20530;
  kk_box_t _x20531 = kk_box_dup(x00); /*2543*/
  _x20530 = kk_std_core_types__new_Right(_x20531, _ctx); /*either<70,71>*/
  _x20529 = kk_std_core_types__either_box(_x20530, _ctx); /*168*/
  kk_ref_set_borrow(r,_x20529,kk_context());
  kk_ref_drop(r, _ctx);
  _borrowApp_20314;
  return x00;
}

kk_box_t kk_std_core__mlift17699_force(kk_ref_t r, kk_std_core_types__either _y_17389, kk_context_t* _ctx) { /* forall<a,e> (r : ref<global,either<() -> e a,a>>, either<() -> e a,a>) -> <read<global>,div,alloc<global>,write<global>|e> a */ 
  if (kk_std_core_types__is_Right(_y_17389)) {
    kk_box_t x = _y_17389._cons.Right.right;
    kk_ref_drop(r, _ctx);
    return x;
  }
  kk_box_t _fun_unbox_x18497 = _y_17389._cons.Left.left;
  kk_box_t x0_17794;
  kk_function_t _x20527 = kk_function_unbox(_fun_unbox_x18497); /*() -> 2544 18498*/
  x0_17794 = kk_function_call(kk_box_t, (kk_function_t, kk_context_t*), _x20527, (_x20527, _ctx)); /*2543*/
  kk_function_t next0_17795 = kk_std_core__new_mlift17699_force_fun20528(r, _ctx); /*(2543) -> <st<global>,div|2544> 2543*/;
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17794, _ctx);
    return kk_std_core_hnd_yield_extend(next0_17795, _ctx);
  }
  return kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), next0_17795, (next0_17795, x0_17794, _ctx));
}
 
// Force a delayed value; the value is computed only on the first
// call to `force` and cached afterwards.


// lift anonymous function
struct kk_std_core_force_fun20534__t {
  struct kk_function_s _base;
  kk_std_core__delayed delayed;
};
static kk_box_t kk_std_core_force_fun20534(kk_function_t _fself, kk_std_core_types__either _y_17389, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_force_fun20534(kk_std_core__delayed delayed, kk_context_t* _ctx) {
  struct kk_std_core_force_fun20534__t* _self = kk_function_alloc_as(struct kk_std_core_force_fun20534__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_force_fun20534, kk_context());
  _self->delayed = delayed;
  return &_self->_base;
}

static kk_box_t kk_std_core_force_fun20534(kk_function_t _fself, kk_std_core_types__either _y_17389, kk_context_t* _ctx) {
  struct kk_std_core_force_fun20534__t* _self = kk_function_as(struct kk_std_core_force_fun20534__t*, _fself);
  kk_std_core__delayed delayed = _self->delayed; /* delayed<2544,2543> */
  kk_drop_match(_self, {kk_std_core__delayed_dup(delayed);}, {}, _ctx)
  kk_ref_t _x20535;
  kk_ref_t _x0 = delayed.dref;
  _x20535 = _x0; /*ref<global,either<() -> 2544 2543,2543>>*/
  return kk_std_core__mlift17699_force(_x20535, _y_17389, _ctx);
}


// lift anonymous function
struct kk_std_core_force_fun20536__t {
  struct kk_function_s _base;
  kk_function_t _b_18507_18505;
};
static kk_box_t kk_std_core_force_fun20536(kk_function_t _fself, kk_box_t _b_18506, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_force_fun20536(kk_function_t _b_18507_18505, kk_context_t* _ctx) {
  struct kk_std_core_force_fun20536__t* _self = kk_function_alloc_as(struct kk_std_core_force_fun20536__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_force_fun20536, kk_context());
  _self->_b_18507_18505 = _b_18507_18505;
  return &_self->_base;
}

static kk_box_t kk_std_core_force_fun20536(kk_function_t _fself, kk_box_t _b_18506, kk_context_t* _ctx) {
  struct kk_std_core_force_fun20536__t* _self = kk_function_as(struct kk_std_core_force_fun20536__t*, _fself);
  kk_function_t _b_18507_18505 = _self->_b_18507_18505; /* (either<() -> 2544 2543,2543>) -> <read<global>,div,alloc<global>,write<global>|2544> 2543 */
  kk_drop_match(_self, {kk_function_dup(_b_18507_18505);}, {}, _ctx)
  kk_std_core_types__either _x20537 = kk_std_core_types__either_unbox(_b_18506, _ctx); /*either<() -> 2544 2543,2543>*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_types__either, kk_context_t*), _b_18507_18505, (_b_18507_18505, _x20537, _ctx));
}

kk_box_t kk_std_core_force(kk_std_core__delayed delayed, kk_context_t* _ctx) { /* forall<a,e> (delayed : delayed<e,a>) -> e a */ 
  kk_std_core_types__either x_17801;
  kk_box_t _x20532;
  kk_ref_t _x20533;
  kk_ref_t _x = delayed.dref;
  kk_ref_dup(_x);
  _x20533 = _x; /*ref<global,either<() -> 2544 2543,2543>>*/
  _x20532 = kk_ref_get(_x20533,kk_context()); /*182*/
  x_17801 = kk_std_core_types__either_unbox(_x20532, _ctx); /*either<() -> 2544 2543,2543>*/
  kk_function_t next0_17802 = kk_std_core_new_force_fun20534(delayed, _ctx); /*(either<() -> 2544 2543,2543>) -> <read<global>,div,alloc<global>,write<global>|2544> 2543*/;
  if (kk_yielding(kk_context())) {
    kk_std_core_types__either_drop(x_17801, _ctx);
    kk_function_t _b_18507_18505 = next0_17802; /*(either<() -> 2544 2543,2543>) -> <read<global>,div,alloc<global>,write<global>|2544> 2543*/;
    return kk_std_core_hnd_yield_extend(kk_std_core_new_force_fun20536(_b_18507_18505, _ctx), _ctx);
  }
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_types__either, kk_context_t*), next0_17802, (next0_17802, x_17801, _ctx));
}
 
// Generic show: shows the internal representation of an object as a string
// Note: this breaks parametricity so it should not be public

kk_string_t kk_std_core_gshow(kk_box_t _arg1, kk_context_t* _ctx) { /* forall<a> (a) -> string */ 
  return kk_show_any(_arg1,kk_context());
}
 
// Return the host environment: `dotnet`, `browser`, `webworker`, `node`, or `libc`.

kk_string_t kk_std_core_host(kk_context_t* _ctx) { /* () -> ndet string */ 
  return kk_get_host(kk_context());
}

int64_t kk_std_core_int64(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int64 */ 
  return kk_integer_clamp64_borrow(i);
}
 
// lifted

kk_std_core__list kk_std_core__ctail_lift17167_intersperse(kk_std_core__list ys, kk_box_t s, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, s : a, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20538 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con20538->head;
    kk_std_core__list yy = _con20538->tail;
    kk_reuse_t _ru_20337 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20337 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_std_core__list _ctail_17297;
    kk_std_core__list _ru_20441 = kk_std_core__list_hole(); /*list<2575>*/;
    if (kk_likely(_ru_20337!=NULL)) {
      struct kk_std_core_Cons* _con20539 = (struct kk_std_core_Cons*)_ru_20337;
      _con20539->tail = _ru_20441;
      _ctail_17297 = kk_std_core__base_Cons(_con20539); /*list<2575>*/
    }
    else {
      _ctail_17297 = kk_std_core__new_Cons(kk_reuse_null, y, _ru_20441, _ctx); /*list<2575>*/
    }
    { // tailcall
      kk_box_t _x20540 = kk_box_dup(s); /*2575*/
      kk_std_core_types__ctail _x20541;
      kk_std_core__list res_17804;
      kk_std_core__list _x20542 = kk_std_core__list_dup(_ctail_17297); /*list<2575>*/
      res_17804 = kk_std_core__new_Cons(kk_reuse_null, s, _x20542, _ctx); /*list<2575>*/
      kk_box_t* field_17805;
      kk_std_core__list_drop(_ctail_17297, _ctx);
      field_17805 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17297)->tail)); /*cfield<list<2575>>*/
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18514 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18514, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(res_17804, _ctx);
        kk_std_core__list _b_18528_18519 = head0; /*list<2575>*/;
        kk_box_t* _b_18529_18520 = field_17805; /*cfield<list<2575>>*/;
        _x20541 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18528_18519, _ctx), _b_18529_18520, _ctx); /*ctail<58>*/
        goto _match20543;
      }
      kk_std_core__list _b_18530_18521 = res_17804; /*list<2575>*/;
      kk_box_t* _b_18531_18522 = field_17805; /*cfield<list<2575>>*/;
      _x20541 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18530_18521, _ctx), _b_18531_18522, _ctx); /*ctail<58>*/
      _match20543: ;
      ys = yy;
      s = _x20540;
      _acc = _x20541;
      goto kk__tailcall;
    }
  }
  kk_box_drop(s, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18523 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18523, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17167_intersperse(kk_std_core__list ys0, kk_box_t s0, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, s : a) -> list<a> */ 
  return kk_std_core__ctail_lift17167_intersperse(ys0, s0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Insert a separator `sep`  between all elements of a list `xs` .

kk_std_core__list kk_std_core_intersperse(kk_std_core__list xs, kk_box_t sep, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, sep : a) -> list<a> */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20546 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con20546->head;
    kk_std_core__list xx = _con20546->tail;
    kk_reuse_t _ru_20338 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20338 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_std_core__list _ru_20442 = kk_std_core__lift17167_intersperse(xx, sep, _ctx); /*list<2575>*/;
    if (kk_likely(_ru_20338!=NULL)) {
      struct kk_std_core_Cons* _con20547 = (struct kk_std_core_Cons*)_ru_20338;
      _con20547->tail = _ru_20442;
      return kk_std_core__base_Cons(_con20547);
    }
    return kk_std_core__new_Cons(kk_reuse_null, x, _ru_20442, _ctx);
  }
  kk_box_drop(sep, _ctx);
  return kk_std_core__new_Nil(_ctx);
}
 
// Return the number of ending `0` digits of `i`. Return `0` when `i==0`.

kk_integer_t kk_std_core_is_exp10(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> int */ 
  return kk_integer_ctz(i,kk_context());
}

kk_std_core__sslice kk_std_core_last1(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> sslice */ 
  return kk_slice_last(s,kk_context());
}
 
// Used by the compiler to wrap main console applications

kk_box_t kk_std_core_main_console(kk_function_t main, kk_context_t* _ctx) { /* forall<a,e> (main : () -> e a) -> e a */ 
  return kk_main_console(main,kk_context());
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17700_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_std_core__list yy, kk_box_t _ctail_17299, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, f : (value : a, rest : list<a>) -> e b, yy : list<a>, b) -> e list<b> */ 
  kk_std_core__list _ctail_17300 = kk_std_core__list_hole(); /*list<2711>*/;
  kk_std_core__list _ctail_17301 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17299, _ctail_17300, _ctx); /*list<2711>*/;
  kk_std_core_types__ctail _x20548;
  kk_box_t* field_17810 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17301)->tail)); /*cfield<list<2711>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18538 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18538, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17301, _ctx);
    kk_std_core__list _b_18547_18543 = head0; /*list<2711>*/;
    kk_box_t* _b_18548_18544 = field_17810; /*cfield<list<2711>>*/;
    _x20548 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18547_18543, _ctx), _b_18548_18544, _ctx); /*ctail<58>*/
    goto _match20549;
  }
  kk_std_core__list _b_18549_18545 = _ctail_17301; /*list<2711>*/;
  kk_box_t* _b_18550_18546 = field_17810; /*cfield<list<2711>>*/;
  _x20548 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18549_18545, _ctx), _b_18550_18546, _ctx); /*ctail<58>*/
  _match20549: ;
  return kk_std_core__ctail_lift17168_map_peek(f, yy, _x20548, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17701_op_fun20551__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t _ctail_17304;
};
static kk_std_core__list kk_std_core__mlift17701_op_fun20551(kk_function_t _fself, kk_std_core__list _ctail_17303, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17701_op_fun20551(kk_function_t _accm, kk_box_t _ctail_17304, kk_context_t* _ctx) {
  struct kk_std_core__mlift17701_op_fun20551__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17701_op_fun20551__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17701_op_fun20551, kk_context());
  _self->_accm = _accm;
  _self->_ctail_17304 = _ctail_17304;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17701_op_fun20551(kk_function_t _fself, kk_std_core__list _ctail_17303, kk_context_t* _ctx) {
  struct kk_std_core__mlift17701_op_fun20551__t* _self = kk_function_as(struct kk_std_core__mlift17701_op_fun20551__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<2711>) -> list<2711> */
  kk_box_t _ctail_17304 = _self->_ctail_17304; /* 2711 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(_ctail_17304);}, {}, _ctx)
  kk_std_core__list _x20552 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17304, _ctail_17303, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x20552, _ctx));
}

kk_std_core__list kk_std_core__mlift17701_op(kk_function_t _accm, kk_function_t f0, kk_std_core__list yy0, kk_box_t _ctail_17304, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, f : (value : a, rest : list<a>) -> e b, yy : list<a>, b) -> e list<b> */ 
  return kk_std_core__ctailm_lift17168_map_peek(f0, yy0, kk_std_core__new_mlift17701_op_fun20551(_accm, _ctail_17304, _ctx), _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__ctail_lift17168_map_peek_fun20556__t {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_std_core__list yy1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_lift17168_map_peek_fun20556(kk_function_t _fself, kk_box_t _ctail_172990, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_lift17168_map_peek_fun20556(kk_function_t f1, kk_std_core__list yy1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17168_map_peek_fun20556__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_lift17168_map_peek_fun20556__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_lift17168_map_peek_fun20556, kk_context());
  _self->f1 = f1;
  _self->yy1 = yy1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_lift17168_map_peek_fun20556(kk_function_t _fself, kk_box_t _ctail_172990, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17168_map_peek_fun20556__t* _self = kk_function_as(struct kk_std_core__ctail_lift17168_map_peek_fun20556__t*, _fself);
  kk_function_t f1 = _self->f1; /* (value : 2710, rest : list<2710>) -> 2712 2711 */
  kk_std_core__list yy1 = _self->yy1; /* list<2710> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<2711>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_std_core__list_dup(yy1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17700_op(_acc0, f1, yy1, _ctail_172990, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_lift17168_map_peek_fun20558__t {
  struct kk_function_s _base;
  kk_function_t _b_18573_18551;
};
static kk_box_t kk_std_core__ctail_lift17168_map_peek_fun20558(kk_function_t _fself, kk_box_t _b_18552, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_lift17168_map_peek_fun20558(kk_function_t _b_18573_18551, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17168_map_peek_fun20558__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_lift17168_map_peek_fun20558__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_lift17168_map_peek_fun20558, kk_context());
  _self->_b_18573_18551 = _b_18573_18551;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_lift17168_map_peek_fun20558(kk_function_t _fself, kk_box_t _b_18552, kk_context_t* _ctx) {
  struct kk_std_core__ctail_lift17168_map_peek_fun20558__t* _self = kk_function_as(struct kk_std_core__ctail_lift17168_map_peek_fun20558__t*, _fself);
  kk_function_t _b_18573_18551 = _self->_b_18573_18551; /* (2711) -> 2712 list<2711> */
  kk_drop_match(_self, {kk_function_dup(_b_18573_18551);}, {}, _ctx)
  kk_std_core__list _x20559 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18573_18551, (_b_18573_18551, _b_18552, _ctx)); /*list<2711>*/
  return kk_std_core__list_box(_x20559, _ctx);
}

kk_std_core__list kk_std_core__ctail_lift17168_map_peek(kk_function_t f1, kk_std_core__list ys, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (value : a, rest : list<a>) -> e b, ys : list<a>, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20553 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con20553->head;
    kk_std_core__list yy1 = _con20553->tail;
    kk_reuse_t _ru_20339 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20339 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy1);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_box_t x_17811;
    kk_function_t _x20555 = kk_function_dup(f1); /*(value : 2710, rest : list<2710>) -> 2712 2711*/
    kk_std_core__list _x20554 = kk_std_core__list_dup(yy1); /*list<2710>*/
    x_17811 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_std_core__list, kk_context_t*), _x20555, (_x20555, y, _x20554, _ctx)); /*2711*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20339);
      kk_box_drop(x_17811, _ctx);
      kk_function_t _b_18573_18551 = kk_std_core__new_ctail_lift17168_map_peek_fun20556(f1, yy1, _acc0, _ctx); /*(2711) -> 2712 list<2711>*/;
      kk_box_t _x20557 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_lift17168_map_peek_fun20558(_b_18573_18551, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20557, _ctx);
    }
    kk_std_core__list _ctail_173000 = kk_std_core__list_hole(); /*list<2711>*/;
    kk_std_core__list _ctail_173010 = kk_std_core__new_Cons(_ru_20339, x_17811, _ctail_173000, _ctx); /*list<2711>*/;
    { // tailcall
      kk_std_core_types__ctail _x20560;
      kk_box_t* field0_17816 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_173010)->tail)); /*cfield<list<2711>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x18559 = _acc0._cons.CTail._field1;
        kk_box_t* last2 = _acc0._cons.CTail._field2;
        kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18559, NULL);
        kk_std_core__list_dup(head1);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __0 = kk_Unit;
        *(last2) = kk_std_core__list_box(_ctail_173010, _ctx);
        kk_std_core__list _b_18574_18564 = head1; /*list<2711>*/;
        kk_box_t* _b_18575_18565 = field0_17816; /*cfield<list<2711>>*/;
        _x20560 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18574_18564, _ctx), _b_18575_18565, _ctx); /*ctail<58>*/
        goto _match20561;
      }
      kk_std_core__list _b_18576_18566 = _ctail_173010; /*list<2711>*/;
      kk_box_t* _b_18577_18567 = field0_17816; /*cfield<list<2711>>*/;
      _x20560 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18576_18566, _ctx), _b_18577_18567, _ctx); /*ctail<58>*/
      _match20561: ;
      ys = yy1;
      _acc0 = _x20560;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18568 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x18568, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__ctailm_lift17168_map_peek_fun20567__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_std_core__list yy2;
};
static kk_std_core__list kk_std_core__ctailm_lift17168_map_peek_fun20567(kk_function_t _fself, kk_box_t _ctail_173040, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17168_map_peek_fun20567(kk_function_t _accm0, kk_function_t f2, kk_std_core__list yy2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17168_map_peek_fun20567__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17168_map_peek_fun20567__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17168_map_peek_fun20567, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->yy2 = yy2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_lift17168_map_peek_fun20567(kk_function_t _fself, kk_box_t _ctail_173040, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17168_map_peek_fun20567__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17168_map_peek_fun20567__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<2711>) -> list<2711> */
  kk_function_t f2 = _self->f2; /* (value : 2710, rest : list<2710>) -> 2712 2711 */
  kk_std_core__list yy2 = _self->yy2; /* list<2710> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_std_core__list_dup(yy2);}, {}, _ctx)
  return kk_std_core__mlift17701_op(_accm0, f2, yy2, _ctail_173040, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_lift17168_map_peek_fun20569__t {
  struct kk_function_s _base;
  kk_function_t _b_18580_18578;
};
static kk_box_t kk_std_core__ctailm_lift17168_map_peek_fun20569(kk_function_t _fself, kk_box_t _b_18579, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17168_map_peek_fun20569(kk_function_t _b_18580_18578, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17168_map_peek_fun20569__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17168_map_peek_fun20569__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17168_map_peek_fun20569, kk_context());
  _self->_b_18580_18578 = _b_18580_18578;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_lift17168_map_peek_fun20569(kk_function_t _fself, kk_box_t _b_18579, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17168_map_peek_fun20569__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17168_map_peek_fun20569__t*, _fself);
  kk_function_t _b_18580_18578 = _self->_b_18580_18578; /* (2711) -> 2712 list<2711> */
  kk_drop_match(_self, {kk_function_dup(_b_18580_18578);}, {}, _ctx)
  kk_std_core__list _x20570 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18580_18578, (_b_18580_18578, _b_18579, _ctx)); /*list<2711>*/
  return kk_std_core__list_box(_x20570, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_lift17168_map_peek_fun20572__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x0_17819;
};
static kk_std_core__list kk_std_core__ctailm_lift17168_map_peek_fun20572(kk_function_t _fself, kk_std_core__list _ctail_173030, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_lift17168_map_peek_fun20572(kk_function_t _accm0, kk_box_t x0_17819, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17168_map_peek_fun20572__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_lift17168_map_peek_fun20572__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_lift17168_map_peek_fun20572, kk_context());
  _self->_accm0 = _accm0;
  _self->x0_17819 = x0_17819;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_lift17168_map_peek_fun20572(kk_function_t _fself, kk_std_core__list _ctail_173030, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_lift17168_map_peek_fun20572__t* _self = kk_function_as(struct kk_std_core__ctailm_lift17168_map_peek_fun20572__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<2711>) -> list<2711> */
  kk_box_t x0_17819 = _self->x0_17819; /* 2711 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x0_17819);}, {}, _ctx)
  kk_std_core__list _x20573 = kk_std_core__new_Cons(kk_reuse_null, x0_17819, _ctail_173030, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x20573, _ctx));
}

kk_std_core__list kk_std_core__ctailm_lift17168_map_peek(kk_function_t f2, kk_std_core__list ys0, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (value : a, rest : list<a>) -> e b, ys : list<a>, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys0)) {
    struct kk_std_core_Cons* _con20564 = kk_std_core__as_Cons(ys0);
    kk_box_t y0 = _con20564->head;
    kk_std_core__list yy2 = _con20564->tail;
    if (kk_likely(kk_std_core__list_is_unique(ys0))) {
      kk_std_core__list_free(ys0);
    }
    else {
      kk_box_dup(y0);
      kk_std_core__list_dup(yy2);
      kk_Unit;
      kk_std_core__list_decref(ys0, _ctx);
      kk_Unit;
    }
    kk_box_t x0_17819;
    kk_function_t _x20566 = kk_function_dup(f2); /*(value : 2710, rest : list<2710>) -> 2712 2711*/
    kk_std_core__list _x20565 = kk_std_core__list_dup(yy2); /*list<2710>*/
    x0_17819 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_std_core__list, kk_context_t*), _x20566, (_x20566, y0, _x20565, _ctx)); /*2711*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x0_17819, _ctx);
      kk_function_t _b_18580_18578 = kk_std_core__new_ctailm_lift17168_map_peek_fun20567(_accm0, f2, yy2, _ctx); /*(2711) -> 2712 list<2711>*/;
      kk_box_t _x20568 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_lift17168_map_peek_fun20569(_b_18580_18578, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20568, _ctx);
    }
    { // tailcall
      kk_function_t _x20571 = kk_std_core__new_ctailm_lift17168_map_peek_fun20572(_accm0, x0_17819, _ctx); /*(list<2711>) -> list<2711>*/
      ys0 = yy2;
      _accm0 = _x20571;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17168_map_peek_fun20574__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core__lift17168_map_peek_fun20574(kk_function_t _fself, kk_std_core__list _ctail_17302, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17168_map_peek_fun20574(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core__lift17168_map_peek_fun20574, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core__lift17168_map_peek_fun20574(kk_function_t _fself, kk_std_core__list _ctail_17302, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17302;
}

kk_std_core__list kk_std_core__lift17168_map_peek(kk_function_t f3, kk_std_core__list ys1, kk_context_t* _ctx) { /* forall<a,b,e> (f : (value : a, rest : list<a>) -> e b, ys : list<a>) -> e list<b> */ 
  bool _match_20309 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_20309) {
    return kk_std_core__ctail_lift17168_map_peek(f3, ys1, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_lift17168_map_peek(f3, ys1, kk_std_core__new_lift17168_map_peek_fun20574(_ctx), _ctx);
}

kk_integer_t kk_std_core_maxListStack;

kk_ref_t kk_std_core_trace_enabled;
 
// Compose two funs `f` and `g`.


// lift anonymous function
struct kk_std_core_o_fun20576__t {
  struct kk_function_s _base;
  kk_function_t f;
  kk_function_t g;
};
static kk_box_t kk_std_core_o_fun20576(kk_function_t _fself, kk_box_t x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_o_fun20576(kk_function_t f, kk_function_t g, kk_context_t* _ctx) {
  struct kk_std_core_o_fun20576__t* _self = kk_function_alloc_as(struct kk_std_core_o_fun20576__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_o_fun20576, kk_context());
  _self->f = f;
  _self->g = g;
  return &_self->_base;
}

static kk_box_t kk_std_core_o_fun20576(kk_function_t _fself, kk_box_t x, kk_context_t* _ctx) {
  struct kk_std_core_o_fun20576__t* _self = kk_function_as(struct kk_std_core_o_fun20576__t*, _fself);
  kk_function_t f = _self->f; /* (2754) -> 2751 2752 */
  kk_function_t g = _self->g; /* (2750) -> 2751 2754 */
  kk_drop_match(_self, {kk_function_dup(f);kk_function_dup(g);}, {}, _ctx)
  kk_box_t x0_17822 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), g, (g, x, _ctx)); /*2754*/;
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17822, _ctx);
    return kk_std_core_hnd_yield_extend(f, _ctx);
  }
  return kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x0_17822, _ctx));
}

kk_function_t kk_std_core_o(kk_function_t f, kk_function_t g, kk_context_t* _ctx) { /* forall<a,b,c,e> (f : (a) -> e b, g : (c) -> e a) -> ((x : c) -> e b) */ 
  return kk_std_core_new_o_fun20576(f, g, _ctx);
}
 
// monadic lift

kk_box_t kk_std_core__mlift17702_once(kk_function_t calc, kk_ref_t r, kk_std_core_types__maybe _y_17407, kk_context_t* _ctx) { /* forall<_h,_e,a> (calc : () -> a, r : ref<_h,maybe<a>>, maybe<a>) -> <read<_h>,write<_h>,div|_e> a */ 
  if (kk_std_core_types__is_Just(_y_17407)) {
    kk_box_t x = _y_17407._cons.Just.value;
    kk_function_drop(calc, _ctx);
    kk_ref_drop(r, _ctx);
    return x;
  }
  kk_box_t x0 = kk_function_call(kk_box_t, (kk_function_t, kk_context_t*), calc, (calc, _ctx)); /*2875*/;
  kk_unit_t __ = kk_Unit;
  kk_unit_t _borrowApp_20307 = kk_Unit;
  kk_box_t _x20578;
  kk_std_core_types__maybe _x20579;
  kk_box_t _x20580 = kk_box_dup(x0); /*2875*/
  _x20579 = kk_std_core_types__new_Just(_x20580, _ctx); /*maybe<101>*/
  _x20578 = kk_std_core_types__maybe_box(_x20579, _ctx); /*168*/
  kk_ref_set_borrow(r,_x20578,kk_context());
  kk_ref_drop(r, _ctx);
  _borrowApp_20307;
  return x0;
}
 
// Given a total function to calculate a value `:a`, return
// a total function that only calculates the value once and then
// returns the cached result.


// lift anonymous function
struct kk_std_core_once_fun20581__t {
  struct kk_function_s _base;
  kk_function_t calc;
  kk_ref_t r;
};
static kk_box_t kk_std_core_once_fun20581(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_once_fun20581(kk_function_t calc, kk_ref_t r, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20581__t* _self = kk_function_alloc_as(struct kk_std_core_once_fun20581__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_once_fun20581, kk_context());
  _self->calc = calc;
  _self->r = r;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_core_once_fun20584__t {
  struct kk_function_s _base;
  kk_function_t calc;
  kk_ref_t r;
};
static kk_box_t kk_std_core_once_fun20584(kk_function_t _fself, kk_std_core_types__maybe _y_17407, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_once_fun20584(kk_function_t calc, kk_ref_t r, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20584__t* _self = kk_function_alloc_as(struct kk_std_core_once_fun20584__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_once_fun20584, kk_context());
  _self->calc = calc;
  _self->r = r;
  return &_self->_base;
}

static kk_box_t kk_std_core_once_fun20584(kk_function_t _fself, kk_std_core_types__maybe _y_17407, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20584__t* _self = kk_function_as(struct kk_std_core_once_fun20584__t*, _fself);
  kk_function_t calc = _self->calc; /* () -> 2875 */
  kk_ref_t r = _self->r; /* ref<_2784,maybe<2875>> */
  kk_drop_match(_self, {kk_function_dup(calc);kk_ref_dup(r);}, {}, _ctx)
  if (kk_std_core_types__is_Just(_y_17407)) {
    kk_box_t x0 = _y_17407._cons.Just.value;
    kk_function_drop(calc, _ctx);
    kk_ref_drop(r, _ctx);
    return x0;
  }
  kk_box_t x00 = kk_function_call(kk_box_t, (kk_function_t, kk_context_t*), calc, (calc, _ctx)); /*2875*/;
  kk_unit_t __ = kk_Unit;
  kk_unit_t _borrowApp_20306 = kk_Unit;
  kk_box_t _x20585;
  kk_std_core_types__maybe _x20586;
  kk_box_t _x20587 = kk_box_dup(x00); /*2875*/
  _x20586 = kk_std_core_types__new_Just(_x20587, _ctx); /*maybe<101>*/
  _x20585 = kk_std_core_types__maybe_box(_x20586, _ctx); /*168*/
  kk_ref_set_borrow(r,_x20585,kk_context());
  kk_ref_drop(r, _ctx);
  _borrowApp_20306;
  return x00;
}


// lift anonymous function
struct kk_std_core_once_fun20588__t {
  struct kk_function_s _base;
  kk_function_t _b_18598_18596;
};
static kk_box_t kk_std_core_once_fun20588(kk_function_t _fself, kk_box_t _b_18597, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_once_fun20588(kk_function_t _b_18598_18596, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20588__t* _self = kk_function_alloc_as(struct kk_std_core_once_fun20588__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_once_fun20588, kk_context());
  _self->_b_18598_18596 = _b_18598_18596;
  return &_self->_base;
}

static kk_box_t kk_std_core_once_fun20588(kk_function_t _fself, kk_box_t _b_18597, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20588__t* _self = kk_function_as(struct kk_std_core_once_fun20588__t*, _fself);
  kk_function_t _b_18598_18596 = _self->_b_18598_18596; /* (maybe<2875>) -> <read<_2784>,write<_2784>,div|_2867> 2875 */
  kk_drop_match(_self, {kk_function_dup(_b_18598_18596);}, {}, _ctx)
  kk_std_core_types__maybe _x20589 = kk_std_core_types__maybe_unbox(_b_18597, _ctx); /*maybe<2875>*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_18598_18596, (_b_18598_18596, _x20589, _ctx));
}
static kk_box_t kk_std_core_once_fun20581(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_core_once_fun20581__t* _self = kk_function_as(struct kk_std_core_once_fun20581__t*, _fself);
  kk_function_t calc = _self->calc; /* () -> 2875 */
  kk_ref_t r = _self->r; /* ref<_2784,maybe<2875>> */
  kk_drop_match(_self, {kk_function_dup(calc);kk_ref_dup(r);}, {}, _ctx)
  kk_std_core_types__maybe x_17826;
  kk_box_t _x20582;
  kk_ref_t _x20583 = kk_ref_dup(r); /*ref<_2784,maybe<2875>>*/
  _x20582 = kk_ref_get(_x20583,kk_context()); /*182*/
  x_17826 = kk_std_core_types__maybe_unbox(_x20582, _ctx); /*maybe<2875>*/
  kk_function_t next0_17827 = kk_std_core_new_once_fun20584(calc, r, _ctx); /*(maybe<2875>) -> <read<_2784>,write<_2784>,div|_2867> 2875*/;
  if (kk_yielding(kk_context())) {
    kk_std_core_types__maybe_drop(x_17826, _ctx);
    kk_function_t _b_18598_18596 = next0_17827; /*(maybe<2875>) -> <read<_2784>,write<_2784>,div|_2867> 2875*/;
    return kk_std_core_hnd_yield_extend(kk_std_core_new_once_fun20588(_b_18598_18596, _ctx), _ctx);
  }
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), next0_17827, (next0_17827, x_17826, _ctx));
}

kk_function_t kk_std_core_once(kk_function_t calc, kk_context_t* _ctx) { /* forall<a> (calc : () -> a) -> (() -> a) */ 
  kk_ref_t r = kk_ref_alloc((kk_std_core_types__maybe_box(kk_std_core_types__new_Nothing(_ctx), _ctx)),kk_context()); /*ref<_2784,maybe<2875>>*/;
  return kk_std_core_new_once_fun20581(calc, r, _ctx);
}

kk_std_core_types__maybe kk_std_core_xparse_int(kk_string_t s, bool hex, kk_context_t* _ctx) { /* (s : string, hex : bool) -> maybe<int> */ 
  return kk_integer_xparse(s,hex,kk_context());
}

kk_box_t kk_std_core_phantom(kk_context_t* _ctx) { /* forall<a> () -> a */ 
  return kk_box_null;
}
 
// Convert a string to lower-case

kk_string_t kk_std_core_to_lower(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  return kk_string_to_lower(s,kk_context());
}

kk_unit_t kk_std_core_xtrace(kk_string_t message0, kk_context_t* _ctx) { /* (message : string) -> () */ 
  kk_trace(message0,kk_context()); return kk_Unit;
}

kk_unit_t kk_std_core_xtrace_any(kk_string_t message0, kk_box_t x, kk_context_t* _ctx) { /* forall<a> (message : string, x : a) -> () */ 
  kk_trace_any(message0,x,kk_context()); return kk_Unit;
}

kk_ref_t kk_std_core_unique_count;

kk_vector_t kk_std_core_unvlist(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> vector<a> */ 
  return kk_list_to_vector(xs,kk_context());
}
 
// Create a new vector of length `n`  with initial elements given by function `f` .

kk_vector_t kk_std_core_vector_initz(kk_ssize_t n, kk_function_t f, kk_context_t* _ctx) { /* forall<a> (n : ssize_t, f : (ssize_t) -> a) -> vector<a> */ 
  return kk_vector_init(n,f,kk_context());
}

bool kk_std_core__lp__excl__eq__4_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_20301;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_20301 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20592;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_20301 = kk_integer_from_small(0); /*int*/
    goto _match20592;
  }
  _borrowedArg_20301 = kk_integer_from_small(1); /*int*/
  _match20592: ;
  kk_integer_t _borrowedArg_20300;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_20300 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20593;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_20300 = kk_integer_from_small(0); /*int*/
    goto _match20593;
  }
  _borrowedArg_20300 = kk_integer_from_small(1); /*int*/
  _match20593: ;
  bool _borrowApp_20302 = kk_integer_neq_borrow(_borrowedArg_20301,_borrowedArg_20300,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20301, _ctx);
  kk_integer_drop(_borrowedArg_20300, _ctx);
  return _borrowApp_20302;
}
 
// Append two lists.

kk_std_core__list kk_std_core__ctail_append(kk_std_core__list xs, kk_std_core__list ys, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, ys : list<a>, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20596 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con20596->head;
    kk_std_core__list xx = _con20596->tail;
    kk_reuse_t _ru_20341 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20341 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_std_core__list _ctail_17305 = kk_std_core__list_hole(); /*list<3245>*/;
    kk_std_core__list _ctail_17306;
    if (kk_likely(_ru_20341!=NULL)) {
      struct kk_std_core_Cons* _con20597 = (struct kk_std_core_Cons*)_ru_20341;
      _con20597->tail = _ctail_17305;
      _ctail_17306 = kk_std_core__base_Cons(_con20597); /*list<3245>*/
    }
    else {
      _ctail_17306 = kk_std_core__new_Cons(kk_reuse_null, x, _ctail_17305, _ctx); /*list<3245>*/
    }
    { // tailcall
      kk_std_core_types__ctail _x20598;
      kk_box_t* field_17835 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17306)->tail)); /*cfield<list<3245>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18607 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18607, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17306, _ctx);
        kk_std_core__list _b_18621_18612 = head0; /*list<3245>*/;
        kk_box_t* _b_18622_18613 = field_17835; /*cfield<list<3245>>*/;
        _x20598 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18621_18612, _ctx), _b_18622_18613, _ctx); /*ctail<58>*/
        goto _match20599;
      }
      kk_std_core__list _b_18623_18614 = _ctail_17306; /*list<3245>*/;
      kk_box_t* _b_18624_18615 = field_17835; /*cfield<list<3245>>*/;
      _x20598 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18623_18614, _ctx), _b_18624_18615, _ctx); /*ctail<58>*/
      _match20599: ;
      xs = xx;
      _acc = _x20598;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18616 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18616, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(ys, _ctx);
    return head1;
  }
  return ys;
}
 
// Append two lists.

kk_std_core__list kk_std_core_append(kk_std_core__list xs0, kk_std_core__list ys0, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, ys : list<a>) -> list<a> */ 
  return kk_std_core__ctail_append(xs0, ys0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Append two strings.

kk_string_t kk_std_core__lp__plus__plus__1_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> string */ 
  return kk_string_cat(x,y,kk_context());
}

kk_unit_t kk_std_core_printsln(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> console () */ 
  kk_std_core_types__maybe _match_20297;
  kk_ref_t _b_18633_18625 = kk_ref_dup(kk_std_core_redirect); /*ref<global,maybe<(string) -> console ()>>*/;
  kk_box_t _x20602 = kk_ref_get(_b_18633_18625,kk_context()); /*182*/
  _match_20297 = kk_std_core_types__maybe_unbox(_x20602, _ctx); /*maybe<(string) -> console ()>*/
  if (kk_std_core_types__is_Nothing(_match_20297)) {
    kk_std_core_xprintsln(s, _ctx); return kk_Unit;
  }
  kk_box_t _fun_unbox_x18629 = _match_20297._cons.Just.value;
  kk_string_t _b_18632;
  kk_string_t _x20603;
  kk_define_string_literal(, _s20604, 1, "\n")
  _x20603 = kk_string_dup(_s20604); /*string*/
  _b_18632 = kk_std_core__lp__plus__plus__1_rp_(s, _x20603, _ctx); /*string*/
  kk_box_t _x20605;
  kk_function_t _x20606 = kk_function_unbox(_fun_unbox_x18629); /*(18630) -> console 18631*/
  _x20605 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20606, (_x20606, kk_string_box(_b_18632), _ctx)); /*18631*/
  kk_unit_unbox(_x20605); return kk_Unit;
}

bool kk_std_core__lp__eq__eq__4_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_20295;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_20295 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20607;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_20295 = kk_integer_from_small(0); /*int*/
    goto _match20607;
  }
  _borrowedArg_20295 = kk_integer_from_small(1); /*int*/
  _match20607: ;
  kk_integer_t _borrowedArg_20294;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_20294 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20608;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_20294 = kk_integer_from_small(0); /*int*/
    goto _match20608;
  }
  _borrowedArg_20294 = kk_integer_from_small(1); /*int*/
  _match20608: ;
  bool _borrowApp_20296 = kk_integer_eq_borrow(_borrowedArg_20295,_borrowedArg_20294,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20295, _ctx);
  kk_integer_drop(_borrowedArg_20294, _ctx);
  return _borrowApp_20296;
}

bool kk_std_core__lp__lt__5_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_20292;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_20292 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20609;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_20292 = kk_integer_from_small(0); /*int*/
    goto _match20609;
  }
  _borrowedArg_20292 = kk_integer_from_small(1); /*int*/
  _match20609: ;
  kk_integer_t _borrowedArg_20291;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_20291 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20610;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_20291 = kk_integer_from_small(0); /*int*/
    goto _match20610;
  }
  _borrowedArg_20291 = kk_integer_from_small(1); /*int*/
  _match20610: ;
  bool _borrowApp_20293 = kk_integer_lt_borrow(_borrowedArg_20292,_borrowedArg_20291,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20292, _ctx);
  kk_integer_drop(_borrowedArg_20291, _ctx);
  return _borrowApp_20293;
}

kk_std_core_types__order kk_std_core_order(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> order */ 
  bool _match_20285;
  kk_integer_t _borrowedArg_20289 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_20290 = kk_integer_lt_borrow(i,_borrowedArg_20289,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20289, _ctx);
  _match_20285 = _borrowApp_20290; /*bool*/
  if (_match_20285) {
    kk_integer_drop(i, _ctx);
    return kk_std_core_types__new_Lt(_ctx);
  }
  bool _match_20286;
  kk_integer_t _borrowedArg_20287 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_20288 = kk_integer_gt_borrow(i,_borrowedArg_20287,kk_context()); /*bool*/;
  kk_integer_drop(i, _ctx);
  kk_integer_drop(_borrowedArg_20287, _ctx);
  _match_20286 = _borrowApp_20288; /*bool*/
  if (_match_20286) {
    return kk_std_core_types__new_Gt(_ctx);
  }
  return kk_std_core_types__new_Eq(_ctx);
}
 
// Compare two strings.
// Uses the character codes directly for comparison

kk_std_core_types__order kk_std_core_compare_4(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> order */ 
  kk_integer_t i_16932 = kk_std_core_string_compare(x, y, _ctx); /*int*/;
  bool _match_20279;
  kk_integer_t _borrowedArg_20283 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_20284 = kk_integer_lt_borrow(i_16932,_borrowedArg_20283,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20283, _ctx);
  _match_20279 = _borrowApp_20284; /*bool*/
  if (_match_20279) {
    kk_integer_drop(i_16932, _ctx);
    return kk_std_core_types__new_Lt(_ctx);
  }
  bool _match_20280;
  kk_integer_t _borrowedArg_20281 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_20282 = kk_integer_gt_borrow(i_16932,_borrowedArg_20281,kk_context()); /*bool*/;
  kk_integer_drop(i_16932, _ctx);
  kk_integer_drop(_borrowedArg_20281, _ctx);
  _match_20280 = _borrowApp_20282; /*bool*/
  if (_match_20280) {
    return kk_std_core_types__new_Gt(_ctx);
  }
  return kk_std_core_types__new_Eq(_ctx);
}

bool kk_std_core__lp__lt__7_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_16933 = kk_std_core_compare_4(x, y, _ctx); /*order*/;
  kk_integer_t _borrowedArg_20277;
  if (kk_std_core_types__is_Lt(x0_16933)) {
    _borrowedArg_20277 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20611;
  }
  if (kk_std_core_types__is_Eq(x0_16933)) {
    _borrowedArg_20277 = kk_integer_from_small(0); /*int*/
    goto _match20611;
  }
  _borrowedArg_20277 = kk_integer_from_small(1); /*int*/
  _match20611: ;
  kk_integer_t _borrowedArg_20276 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/;
  bool _borrowApp_20278 = kk_integer_eq_borrow(_borrowedArg_20277,_borrowedArg_20276,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20277, _ctx);
  kk_integer_drop(_borrowedArg_20276, _ctx);
  return _borrowApp_20278;
}

bool kk_std_core__lp__gt__3_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_20274;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_20274 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20612;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_20274 = kk_integer_from_small(0); /*int*/
    goto _match20612;
  }
  _borrowedArg_20274 = kk_integer_from_small(1); /*int*/
  _match20612: ;
  kk_integer_t _borrowedArg_20273;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_20273 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20613;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_20273 = kk_integer_from_small(0); /*int*/
    goto _match20613;
  }
  _borrowedArg_20273 = kk_integer_from_small(1); /*int*/
  _match20613: ;
  bool _borrowApp_20275 = kk_integer_gt_borrow(_borrowedArg_20274,_borrowedArg_20273,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20274, _ctx);
  kk_integer_drop(_borrowedArg_20273, _ctx);
  return _borrowApp_20275;
}

bool kk_std_core__lp__gt__5_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_16940 = kk_std_core_compare_4(x, y, _ctx); /*order*/;
  kk_integer_t _borrowedArg_20271;
  if (kk_std_core_types__is_Lt(x0_16940)) {
    _borrowedArg_20271 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20614;
  }
  if (kk_std_core_types__is_Eq(x0_16940)) {
    _borrowedArg_20271 = kk_integer_from_small(0); /*int*/
    goto _match20614;
  }
  _borrowedArg_20271 = kk_integer_from_small(1); /*int*/
  _match20614: ;
  kk_integer_t _borrowedArg_20270 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_20272 = kk_integer_eq_borrow(_borrowedArg_20271,_borrowedArg_20270,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20271, _ctx);
  kk_integer_drop(_borrowedArg_20270, _ctx);
  return _borrowApp_20272;
}

kk_std_core_types__order kk_std_core_compare_2(bool x, bool y, kk_context_t* _ctx) { /* (x : bool, y : bool) -> order */ 
  if (x) {
    if (x) {
      if (y) {
        return kk_std_core_types__new_Eq(_ctx);
      }
      return kk_std_core_types__new_Gt(_ctx);
    }
    return kk_std_core_types__new_Eq(_ctx);
  }
  if (y) {
    return kk_std_core_types__new_Lt(_ctx);
  }
  if (x) {
    if (y) {
      return kk_std_core_types__new_Eq(_ctx);
    }
    return kk_std_core_types__new_Gt(_ctx);
  }
  return kk_std_core_types__new_Eq(_ctx);
}

bool kk_std_core__lp__gt__eq__4_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_20264;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_20264 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20615;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_20264 = kk_integer_from_small(0); /*int*/
    goto _match20615;
  }
  _borrowedArg_20264 = kk_integer_from_small(1); /*int*/
  _match20615: ;
  kk_integer_t _borrowedArg_20263;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_20263 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20616;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_20263 = kk_integer_from_small(0); /*int*/
    goto _match20616;
  }
  _borrowedArg_20263 = kk_integer_from_small(1); /*int*/
  _match20616: ;
  bool _borrowApp_20265 = kk_integer_gte_borrow(_borrowedArg_20264,_borrowedArg_20263,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20264, _ctx);
  kk_integer_drop(_borrowedArg_20263, _ctx);
  return _borrowApp_20265;
}

bool kk_std_core__lp__gt__eq__6_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_16956 = kk_std_core_compare_4(x, y, _ctx); /*order*/;
  kk_integer_t _borrowedArg_20261;
  if (kk_std_core_types__is_Lt(x0_16956)) {
    _borrowedArg_20261 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20617;
  }
  if (kk_std_core_types__is_Eq(x0_16956)) {
    _borrowedArg_20261 = kk_integer_from_small(0); /*int*/
    goto _match20617;
  }
  _borrowedArg_20261 = kk_integer_from_small(1); /*int*/
  _match20617: ;
  kk_integer_t _borrowedArg_20260 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/;
  bool _borrowApp_20262 = kk_integer_gt_borrow(_borrowedArg_20261,_borrowedArg_20260,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20261, _ctx);
  kk_integer_drop(_borrowedArg_20260, _ctx);
  return _borrowApp_20262;
}
 
// lifted

kk_string_t kk_std_core__lift17169_joinsep(kk_string_t sep, kk_std_core__list ys, kk_string_t acc, kk_context_t* _ctx) { /* (sep : string, ys : list<string>, acc : string) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20618 = kk_std_core__as_Cons(ys);
    kk_box_t _box_x18634 = _con20618->head;
    kk_std_core__list yy = _con20618->tail;
    kk_string_t y = kk_string_unbox(_box_x18634);
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_std_core__list_free(ys);
    }
    else {
      kk_string_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_string_t acc0_17215;
    kk_string_t _x20620;
    kk_string_t _x20621 = kk_string_dup(sep); /*string*/
    _x20620 = kk_std_core__lp__plus__plus__1_rp_(_x20621, y, _ctx); /*string*/
    acc0_17215 = kk_std_core__lp__plus__plus__1_rp_(acc, _x20620, _ctx); /*string*/
    { // tailcall
      ys = yy;
      acc = acc0_17215;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sep, _ctx);
  return acc;
}
 
// Concatenate all strings in a list

kk_string_t kk_std_core_joinsep(kk_std_core__list xs, kk_string_t sep, kk_context_t* _ctx) { /* (xs : list<string>, sep : string) -> string */ 
  if (kk_std_core__is_Nil(xs)) {
    kk_string_drop(sep, _ctx);
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con20623 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x18635 = _con20623->head;
  kk_std_core__list xx = _con20623->tail;
  kk_string_t x = kk_string_unbox(_box_x18635);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_string_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  return kk_std_core__lift17169_joinsep(sep, xx, x, _ctx);
}
 
// lifted

kk_string_t kk_std_core__lift17170_join_2(kk_string_t sep_16961, kk_std_core__list ys, kk_string_t acc, kk_context_t* _ctx) { /* (sep.16961 : string, ys : list<string>, acc : string) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20625 = kk_std_core__as_Cons(ys);
    kk_box_t _box_x18636 = _con20625->head;
    kk_std_core__list yy = _con20625->tail;
    kk_string_t y = kk_string_unbox(_box_x18636);
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_std_core__list_free(ys);
    }
    else {
      kk_string_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_string_t acc0_17219;
    kk_string_t _x20627;
    kk_string_t _x20628 = kk_string_dup(sep_16961); /*string*/
    _x20627 = kk_std_core__lp__plus__plus__1_rp_(_x20628, y, _ctx); /*string*/
    acc0_17219 = kk_std_core__lp__plus__plus__1_rp_(acc, _x20627, _ctx); /*string*/
    { // tailcall
      ys = yy;
      acc = acc0_17219;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sep_16961, _ctx);
  return acc;
}
 
// Concatenate all strings in a list

kk_string_t kk_std_core_join_2(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<string>) -> string */ 
  if (kk_std_core__is_Nil(xs)) {
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con20630 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x18637 = _con20630->head;
  kk_std_core__list xx = _con20630->tail;
  kk_string_t x = kk_string_unbox(_box_x18637);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_string_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_string_t _x20632 = kk_string_empty(); /*string*/
  return kk_std_core__lift17170_join_2(_x20632, xx, x, _ctx);
}
 
// Concatenate all strings in a list using a specific separator

kk_string_t kk_std_core_join_3(kk_std_core__list xs, kk_string_t sep, kk_context_t* _ctx) { /* (xs : list<string>, sep : string) -> string */ 
  if (kk_std_core__is_Nil(xs)) {
    kk_string_drop(sep, _ctx);
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con20635 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x18638 = _con20635->head;
  kk_std_core__list xx = _con20635->tail;
  kk_string_t x = kk_string_unbox(_box_x18638);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_string_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  return kk_std_core__lift17169_joinsep(sep, xx, x, _ctx);
}

bool kk_std_core__lp__lt__eq__5_rp_(kk_std_core_types__order x, kk_std_core_types__order y, kk_context_t* _ctx) { /* (x : order, y : order) -> bool */ 
  kk_integer_t _borrowedArg_20258;
  if (kk_std_core_types__is_Lt(x)) {
    _borrowedArg_20258 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20637;
  }
  if (kk_std_core_types__is_Eq(x)) {
    _borrowedArg_20258 = kk_integer_from_small(0); /*int*/
    goto _match20637;
  }
  _borrowedArg_20258 = kk_integer_from_small(1); /*int*/
  _match20637: ;
  kk_integer_t _borrowedArg_20257;
  if (kk_std_core_types__is_Lt(y)) {
    _borrowedArg_20257 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20638;
  }
  if (kk_std_core_types__is_Eq(y)) {
    _borrowedArg_20257 = kk_integer_from_small(0); /*int*/
    goto _match20638;
  }
  _borrowedArg_20257 = kk_integer_from_small(1); /*int*/
  _match20638: ;
  bool _borrowApp_20259 = kk_integer_lte_borrow(_borrowedArg_20258,_borrowedArg_20257,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20258, _ctx);
  kk_integer_drop(_borrowedArg_20257, _ctx);
  return _borrowApp_20259;
}

bool kk_std_core__lp__lt__eq__7_rp_(kk_string_t x, kk_string_t y, kk_context_t* _ctx) { /* (x : string, y : string) -> bool */ 
  kk_std_core_types__order x0_16968 = kk_std_core_compare_4(x, y, _ctx); /*order*/;
  kk_integer_t _borrowedArg_20255;
  if (kk_std_core_types__is_Lt(x0_16968)) {
    _borrowedArg_20255 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20639;
  }
  if (kk_std_core_types__is_Eq(x0_16968)) {
    _borrowedArg_20255 = kk_integer_from_small(0); /*int*/
    goto _match20639;
  }
  _borrowedArg_20255 = kk_integer_from_small(1); /*int*/
  _match20639: ;
  kk_integer_t _borrowedArg_20254 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_20256 = kk_integer_lt_borrow(_borrowedArg_20255,_borrowedArg_20254,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20255, _ctx);
  kk_integer_drop(_borrowedArg_20254, _ctx);
  return _borrowApp_20256;
}

kk_ssize_t kk_std_core_decr_1(kk_ssize_t i, kk_context_t* _ctx) { /* (i : ssize_t) -> ssize_t */ 
  return (i - 1);
}

kk_ssize_t kk_std_core_incr_1(kk_ssize_t i, kk_context_t* _ctx) { /* (i : ssize_t) -> ssize_t */ 
  return (i + 1);
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17703_op(kk_function_t action, kk_ssize_t end, kk_ssize_t i, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action : (ssize_t) -> e (), end : ssize_t, i : ssize_t, wild_ : ()) -> e () */ 
  kk_ssize_t i0_17222 = kk_std_core_incr_1(i, _ctx); /*ssize_t*/;
  kk_std_core__lift17171_forz(action, end, i0_17222, _ctx); return kk_Unit;
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17171_forz_fun20641__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_ssize_t end0;
  kk_ssize_t i0;
};
static kk_unit_t kk_std_core__lift17171_forz_fun20641(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17171_forz_fun20641(kk_function_t action0, kk_ssize_t end0, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17171_forz_fun20641__t* _self = kk_function_alloc_as(struct kk_std_core__lift17171_forz_fun20641__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17171_forz_fun20641, kk_context());
  _self->action0 = action0;
  _self->end0 = end0;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17171_forz_fun20641(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17171_forz_fun20641__t* _self = kk_function_as(struct kk_std_core__lift17171_forz_fun20641__t*, _fself);
  kk_function_t action0 = _self->action0; /* (ssize_t) -> 4353 () */
  kk_ssize_t end0 = _self->end0; /* ssize_t */
  kk_ssize_t i0 = _self->i0; /* ssize_t */
  kk_drop_match(_self, {kk_function_dup(action0);;;}, {}, _ctx)
  return kk_std_core__mlift17703_op(action0, end0, i0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17171_forz_fun20643__t {
  struct kk_function_s _base;
  kk_function_t _b_18641_18639;
};
static kk_box_t kk_std_core__lift17171_forz_fun20643(kk_function_t _fself, kk_box_t _b_18640, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17171_forz_fun20643(kk_function_t _b_18641_18639, kk_context_t* _ctx) {
  struct kk_std_core__lift17171_forz_fun20643__t* _self = kk_function_alloc_as(struct kk_std_core__lift17171_forz_fun20643__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17171_forz_fun20643, kk_context());
  _self->_b_18641_18639 = _b_18641_18639;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17171_forz_fun20643(kk_function_t _fself, kk_box_t _b_18640, kk_context_t* _ctx) {
  struct kk_std_core__lift17171_forz_fun20643__t* _self = kk_function_as(struct kk_std_core__lift17171_forz_fun20643__t*, _fself);
  kk_function_t _b_18641_18639 = _self->_b_18641_18639; /* (wild_0 : ()) -> 4353 () */
  kk_drop_match(_self, {kk_function_dup(_b_18641_18639);}, {}, _ctx)
  kk_unit_t _x20644 = kk_Unit;
  kk_unit_t _x20645 = kk_Unit;
  kk_unit_unbox(_b_18640);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_18641_18639, (_b_18641_18639, _x20645, _ctx));
  return kk_unit_box(_x20644);
}

kk_unit_t kk_std_core__lift17171_forz(kk_function_t action0, kk_ssize_t end0, kk_ssize_t i0, kk_context_t* _ctx) { /* forall<e> (action : (ssize_t) -> e (), end : ssize_t, i : ssize_t) -> e () */ 
  kk__tailcall: ;
  bool _match_20252 = (i0 <= end0); /*bool*/;
  if (_match_20252) {
    kk_unit_t x_17840 = kk_Unit;
    kk_function_t _x20640 = kk_function_dup(action0); /*(ssize_t) -> 4353 ()*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_ssize_t, kk_context_t*), _x20640, (_x20640, i0, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_18641_18639 = kk_std_core__new_lift17171_forz_fun20641(action0, end0, i0, _ctx); /*(wild_0 : ()) -> 4353 ()*/;
      kk_box_t _x20642 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17171_forz_fun20643(_b_18641_18639, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x20642); return kk_Unit;
    }
    kk_ssize_t i0_172220 = kk_std_core_incr_1(i0, _ctx); /*ssize_t*/;
    { // tailcall
      i0 = i0_172220;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action0, _ctx);
  kk_Unit; return kk_Unit;
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17704_op(kk_function_t action_16974, kk_ssize_t end_16973, kk_ssize_t i, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action.16974 : (ssize_t) -> e (), end.16973 : ssize_t, i : ssize_t, wild_ : ()) -> e () */ 
  kk_ssize_t i0_17223 = kk_std_core_incr_1(i, _ctx); /*ssize_t*/;
  kk_std_core__lift17172_foreach_indexedz(action_16974, end_16973, i0_17223, _ctx); return kk_Unit;
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17172_foreach_indexedz_fun20647__t {
  struct kk_function_s _base;
  kk_function_t action_169740;
  kk_ssize_t end_169730;
  kk_ssize_t i0;
};
static kk_unit_t kk_std_core__lift17172_foreach_indexedz_fun20647(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17172_foreach_indexedz_fun20647(kk_function_t action_169740, kk_ssize_t end_169730, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17172_foreach_indexedz_fun20647__t* _self = kk_function_alloc_as(struct kk_std_core__lift17172_foreach_indexedz_fun20647__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17172_foreach_indexedz_fun20647, kk_context());
  _self->action_169740 = action_169740;
  _self->end_169730 = end_169730;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17172_foreach_indexedz_fun20647(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17172_foreach_indexedz_fun20647__t* _self = kk_function_as(struct kk_std_core__lift17172_foreach_indexedz_fun20647__t*, _fself);
  kk_function_t action_169740 = _self->action_169740; /* (ssize_t) -> 4388 () */
  kk_ssize_t end_169730 = _self->end_169730; /* ssize_t */
  kk_ssize_t i0 = _self->i0; /* ssize_t */
  kk_drop_match(_self, {kk_function_dup(action_169740);;;}, {}, _ctx)
  return kk_std_core__mlift17704_op(action_169740, end_169730, i0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17172_foreach_indexedz_fun20649__t {
  struct kk_function_s _base;
  kk_function_t _b_18644_18642;
};
static kk_box_t kk_std_core__lift17172_foreach_indexedz_fun20649(kk_function_t _fself, kk_box_t _b_18643, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17172_foreach_indexedz_fun20649(kk_function_t _b_18644_18642, kk_context_t* _ctx) {
  struct kk_std_core__lift17172_foreach_indexedz_fun20649__t* _self = kk_function_alloc_as(struct kk_std_core__lift17172_foreach_indexedz_fun20649__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17172_foreach_indexedz_fun20649, kk_context());
  _self->_b_18644_18642 = _b_18644_18642;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17172_foreach_indexedz_fun20649(kk_function_t _fself, kk_box_t _b_18643, kk_context_t* _ctx) {
  struct kk_std_core__lift17172_foreach_indexedz_fun20649__t* _self = kk_function_as(struct kk_std_core__lift17172_foreach_indexedz_fun20649__t*, _fself);
  kk_function_t _b_18644_18642 = _self->_b_18644_18642; /* (wild_0 : ()) -> 4388 () */
  kk_drop_match(_self, {kk_function_dup(_b_18644_18642);}, {}, _ctx)
  kk_unit_t _x20650 = kk_Unit;
  kk_unit_t _x20651 = kk_Unit;
  kk_unit_unbox(_b_18643);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_18644_18642, (_b_18644_18642, _x20651, _ctx));
  return kk_unit_box(_x20650);
}

kk_unit_t kk_std_core__lift17172_foreach_indexedz(kk_function_t action_169740, kk_ssize_t end_169730, kk_ssize_t i0, kk_context_t* _ctx) { /* forall<e> (action.16974 : (ssize_t) -> e (), end.16973 : ssize_t, i : ssize_t) -> e () */ 
  kk__tailcall: ;
  bool _match_20250 = (i0 <= end_169730); /*bool*/;
  if (_match_20250) {
    kk_unit_t x_17843 = kk_Unit;
    kk_function_t _x20646 = kk_function_dup(action_169740); /*(ssize_t) -> 4388 ()*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_ssize_t, kk_context_t*), _x20646, (_x20646, i0, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_18644_18642 = kk_std_core__new_lift17172_foreach_indexedz_fun20647(action_169740, end_169730, i0, _ctx); /*(wild_0 : ()) -> 4388 ()*/;
      kk_box_t _x20648 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17172_foreach_indexedz_fun20649(_b_18644_18642, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x20648); return kk_Unit;
    }
    kk_ssize_t i0_172230 = kk_std_core_incr_1(i0, _ctx); /*ssize_t*/;
    { // tailcall
      i0 = i0_172230;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action_169740, _ctx);
  kk_Unit; return kk_Unit;
}


// lift anonymous function
struct kk_std_core_foreach_indexedz_fun20653__t {
  struct kk_function_s _base;
  kk_function_t f;
  kk_vector_t v;
};
static kk_unit_t kk_std_core_foreach_indexedz_fun20653(kk_function_t _fself, kk_ssize_t i0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_indexedz_fun20653(kk_function_t f, kk_vector_t v, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexedz_fun20653__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_indexedz_fun20653__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_indexedz_fun20653, kk_context());
  _self->f = f;
  _self->v = v;
  return &_self->_base;
}

static kk_unit_t kk_std_core_foreach_indexedz_fun20653(kk_function_t _fself, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexedz_fun20653__t* _self = kk_function_as(struct kk_std_core_foreach_indexedz_fun20653__t*, _fself);
  kk_function_t f = _self->f; /* (4387, ssize_t) -> 4388 () */
  kk_vector_t v = _self->v; /* vector<4387> */
  kk_drop_match(_self, {kk_function_dup(f);kk_vector_dup(v);}, {}, _ctx)
  kk_box_t _x20654;
  kk_box_t _borrowApp_20249 = kk_vector_at_borrow(v, i0); /*219*/;
  kk_vector_drop(v, _ctx);
  _x20654 = _borrowApp_20249; /*219*/
  return kk_function_call(kk_unit_t, (kk_function_t, kk_box_t, kk_ssize_t, kk_context_t*), f, (f, _x20654, i0, _ctx));
}

kk_unit_t kk_std_core_foreach_indexedz(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (v : vector<a>, f : (a, ssize_t) -> e ()) -> e () */ 
  kk_ssize_t start0_16972 = ((kk_ssize_t)0); /*ssize_t*/;
  kk_ssize_t end_16973;
  kk_ssize_t _x20652 = kk_vector_len_borrow(v); /*ssize_t*/
  end_16973 = kk_std_core_decr_1(_x20652, _ctx); /*ssize_t*/
  kk_std_core__lift17172_foreach_indexedz(kk_std_core_new_foreach_indexedz_fun20653(f, v, _ctx), end_16973, start0_16972, _ctx); return kk_Unit;
}
 
// lifted

kk_integer_t kk_std_core__lift17173_length_1(kk_std_core__list ys, kk_integer_t acc, kk_context_t* _ctx) { /* forall<a> (ys : list<a>, acc : int) -> int */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con20655 = kk_std_core__as_Cons(ys);
    kk_box_t _pat0 = _con20655->head;
    kk_std_core__list yy = _con20655->tail;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_box_drop(_pat0, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(ys);
    }
    else {
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_integer_t _x20656 = kk_integer_add(acc,(kk_integer_from_small(1)),kk_context()); /*int*/
      ys = yy;
      acc = _x20656;
      goto kk__tailcall;
    }
  }
  return acc;
}
 
// Returns an integer list of increasing elements from `lo`  to `hi`
// (including both `lo`  and `hi` ).
// If `lo > hi`  the function returns the empty list.

kk_std_core__list kk_std_core__ctail_list(kk_integer_t lo, kk_integer_t hi, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (lo : int, hi : int, ctail<list<int>>) -> total list<int> */ 
  kk__tailcall: ;
  bool _match_20248 = kk_integer_lte_borrow(lo,hi,kk_context()); /*bool*/;
  if (_match_20248) {
    kk_std_core__list _ctail_17307 = kk_std_core__list_hole(); /*list<int>*/;
    kk_std_core__list _ctail_17308;
    kk_box_t _x20658;
    kk_integer_t _x20659 = kk_integer_dup(lo); /*int*/
    _x20658 = kk_integer_box(_x20659); /*61*/
    _ctail_17308 = kk_std_core__new_Cons(kk_reuse_null, _x20658, _ctail_17307, _ctx); /*list<int>*/
    { // tailcall
      kk_integer_t _x20660 = kk_integer_add(lo,(kk_integer_from_small(1)),kk_context()); /*int*/
      kk_std_core_types__ctail _x20661;
      kk_box_t* field_17848 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17308)->tail)); /*cfield<list<int>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18655 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18655, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17308, _ctx);
        kk_std_core__list _b_18669_18660 = head0; /*list<int>*/;
        kk_box_t* _b_18670_18661 = field_17848; /*cfield<list<int>>*/;
        _x20661 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18669_18660, _ctx), _b_18670_18661, _ctx); /*ctail<58>*/
        goto _match20662;
      }
      kk_std_core__list _b_18671_18662 = _ctail_17308; /*list<int>*/;
      kk_box_t* _b_18672_18663 = field_17848; /*cfield<list<int>>*/;
      _x20661 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18671_18662, _ctx), _b_18672_18663, _ctx); /*ctail<58>*/
      _match20662: ;
      lo = _x20660;
      _acc = _x20661;
      goto kk__tailcall;
    }
  }
  kk_integer_drop(hi, _ctx);
  kk_integer_drop(lo, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18664 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18664, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Returns an integer list of increasing elements from `lo`  to `hi`
// (including both `lo`  and `hi` ).
// If `lo > hi`  the function returns the empty list.

kk_std_core__list kk_std_core_list(kk_integer_t lo0, kk_integer_t hi0, kk_context_t* _ctx) { /* (lo : int, hi : int) -> total list<int> */ 
  return kk_std_core__ctail_list(lo0, hi0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Returns an integer list of increasing elements from `lo`  to `hi` with stride `stride`.
// If `lo > hi`  the function returns the empty list.

kk_std_core__list kk_std_core__ctail_list_1(kk_integer_t lo, kk_integer_t hi, kk_integer_t stride, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (lo : int, hi : int, stride : int, ctail<list<int>>) -> total list<int> */ 
  kk__tailcall: ;
  bool _match_20247 = kk_integer_lte_borrow(lo,hi,kk_context()); /*bool*/;
  if (_match_20247) {
    kk_std_core__list _ctail_17309 = kk_std_core__list_hole(); /*list<int>*/;
    kk_std_core__list _ctail_17310;
    kk_box_t _x20665;
    kk_integer_t _x20666 = kk_integer_dup(lo); /*int*/
    _x20665 = kk_integer_box(_x20666); /*61*/
    _ctail_17310 = kk_std_core__new_Cons(kk_reuse_null, _x20665, _ctail_17309, _ctx); /*list<int>*/
    { // tailcall
      kk_integer_t _x20667;
      kk_integer_t _x20668 = kk_integer_dup(stride); /*int*/
      _x20667 = kk_integer_add(lo,_x20668,kk_context()); /*int*/
      kk_std_core_types__ctail _x20669;
      kk_box_t* field_17853 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17310)->tail)); /*cfield<list<int>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18683 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18683, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17310, _ctx);
        kk_std_core__list _b_18697_18688 = head0; /*list<int>*/;
        kk_box_t* _b_18698_18689 = field_17853; /*cfield<list<int>>*/;
        _x20669 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18697_18688, _ctx), _b_18698_18689, _ctx); /*ctail<58>*/
        goto _match20670;
      }
      kk_std_core__list _b_18699_18690 = _ctail_17310; /*list<int>*/;
      kk_box_t* _b_18700_18691 = field_17853; /*cfield<list<int>>*/;
      _x20669 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18699_18690, _ctx), _b_18700_18691, _ctx); /*ctail<58>*/
      _match20670: ;
      lo = _x20667;
      _acc = _x20669;
      goto kk__tailcall;
    }
  }
  kk_integer_drop(hi, _ctx);
  kk_integer_drop(lo, _ctx);
  kk_integer_drop(stride, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18692 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18692, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Returns an integer list of increasing elements from `lo`  to `hi` with stride `stride`.
// If `lo > hi`  the function returns the empty list.

kk_std_core__list kk_std_core_list_1(kk_integer_t lo0, kk_integer_t hi0, kk_integer_t stride0, kk_context_t* _ctx) { /* (lo : int, hi : int, stride : int) -> total list<int> */ 
  return kk_std_core__ctail_list_1(lo0, hi0, stride0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17705_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_integer_t hi, kk_integer_t lo, kk_box_t _ctail_17311, kk_context_t* _ctx) { /* forall<a,e> (ctail<list<a>>, f : (int) -> e a, hi : int, lo : int, a) -> e list<a> */ 
  kk_std_core__list _ctail_17312 = kk_std_core__list_hole(); /*list<4938>*/;
  kk_std_core__list _ctail_17313 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17311, _ctail_17312, _ctx); /*list<4938>*/;
  kk_integer_t _x20673 = kk_integer_add(lo,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_std_core_types__ctail _x20674;
  kk_box_t* field_17858 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17313)->tail)); /*cfield<list<4938>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18707 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18707, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17313, _ctx);
    kk_std_core__list _b_18716_18712 = head0; /*list<4938>*/;
    kk_box_t* _b_18717_18713 = field_17858; /*cfield<list<4938>>*/;
    _x20674 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18716_18712, _ctx), _b_18717_18713, _ctx); /*ctail<58>*/
    goto _match20675;
  }
  kk_std_core__list _b_18718_18714 = _ctail_17313; /*list<4938>*/;
  kk_box_t* _b_18719_18715 = field_17858; /*cfield<list<4938>>*/;
  _x20674 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18718_18714, _ctx), _b_18719_18715, _ctx); /*ctail<58>*/
  _match20675: ;
  return kk_std_core__ctail_list_2(_x20673, hi, f, _x20674, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17706_op_fun20678__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t _ctail_17316;
};
static kk_std_core__list kk_std_core__mlift17706_op_fun20678(kk_function_t _fself, kk_std_core__list _ctail_17315, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17706_op_fun20678(kk_function_t _accm, kk_box_t _ctail_17316, kk_context_t* _ctx) {
  struct kk_std_core__mlift17706_op_fun20678__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17706_op_fun20678__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17706_op_fun20678, kk_context());
  _self->_accm = _accm;
  _self->_ctail_17316 = _ctail_17316;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17706_op_fun20678(kk_function_t _fself, kk_std_core__list _ctail_17315, kk_context_t* _ctx) {
  struct kk_std_core__mlift17706_op_fun20678__t* _self = kk_function_as(struct kk_std_core__mlift17706_op_fun20678__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<4938>) -> list<4938> */
  kk_box_t _ctail_17316 = _self->_ctail_17316; /* 4938 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(_ctail_17316);}, {}, _ctx)
  kk_std_core__list _x20679 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17316, _ctail_17315, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x20679, _ctx));
}

kk_std_core__list kk_std_core__mlift17706_op(kk_function_t _accm, kk_function_t f0, kk_integer_t hi0, kk_integer_t lo0, kk_box_t _ctail_17316, kk_context_t* _ctx) { /* forall<a,e> ((list<a>) -> list<a>, f : (int) -> e a, hi : int, lo : int, a) -> e list<a> */ 
  kk_integer_t _x20677 = kk_integer_add(lo0,(kk_integer_from_small(1)),kk_context()); /*int*/
  return kk_std_core__ctailm_list_2(_x20677, hi0, f0, kk_std_core__new_mlift17706_op_fun20678(_accm, _ctail_17316, _ctx), _ctx);
}
 
// Applies a function `f` to list of increasing elements from `lo`  to `hi`
// (including both `lo`  and `hi` ).
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core__ctail_list_fun20682__t_2 {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_integer_t hi1;
  kk_integer_t lo1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_list_fun20682_2(kk_function_t _fself, kk_box_t _ctail_173110, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_list_fun20682_2(kk_function_t f1, kk_integer_t hi1, kk_integer_t lo1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20682__t_2* _self = kk_function_alloc_as(struct kk_std_core__ctail_list_fun20682__t_2, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_list_fun20682_2, kk_context());
  _self->f1 = f1;
  _self->hi1 = hi1;
  _self->lo1 = lo1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_list_fun20682_2(kk_function_t _fself, kk_box_t _ctail_173110, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20682__t_2* _self = kk_function_as(struct kk_std_core__ctail_list_fun20682__t_2*, _fself);
  kk_function_t f1 = _self->f1; /* (int) -> 4939 4938 */
  kk_integer_t hi1 = _self->hi1; /* int */
  kk_integer_t lo1 = _self->lo1; /* int */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<4938>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_integer_dup(hi1);kk_integer_dup(lo1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17705_op(_acc0, f1, hi1, lo1, _ctail_173110, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_list_fun20684__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18742_18720;
};
static kk_box_t kk_std_core__ctail_list_fun20684_2(kk_function_t _fself, kk_box_t _b_18721, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_list_fun20684_2(kk_function_t _b_18742_18720, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20684__t_2* _self = kk_function_alloc_as(struct kk_std_core__ctail_list_fun20684__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_list_fun20684_2, kk_context());
  _self->_b_18742_18720 = _b_18742_18720;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_list_fun20684_2(kk_function_t _fself, kk_box_t _b_18721, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20684__t_2* _self = kk_function_as(struct kk_std_core__ctail_list_fun20684__t_2*, _fself);
  kk_function_t _b_18742_18720 = _self->_b_18742_18720; /* (4938) -> 4939 list<4938> */
  kk_drop_match(_self, {kk_function_dup(_b_18742_18720);}, {}, _ctx)
  kk_std_core__list _x20685 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18742_18720, (_b_18742_18720, _b_18721, _ctx)); /*list<4938>*/
  return kk_std_core__list_box(_x20685, _ctx);
}

kk_std_core__list kk_std_core__ctail_list_2(kk_integer_t lo1, kk_integer_t hi1, kk_function_t f1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, f : (int) -> e a, ctail<list<a>>) -> e list<a> */ 
  kk__tailcall: ;
  bool _match_20245 = kk_integer_lte_borrow(lo1,hi1,kk_context()); /*bool*/;
  if (_match_20245) {
    kk_box_t x_17859;
    kk_function_t _x20681 = kk_function_dup(f1); /*(int) -> 4939 4938*/
    kk_integer_t _x20680 = kk_integer_dup(lo1); /*int*/
    x_17859 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_context_t*), _x20681, (_x20681, _x20680, _ctx)); /*4938*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x_17859, _ctx);
      kk_function_t _b_18742_18720 = kk_std_core__new_ctail_list_fun20682_2(f1, hi1, lo1, _acc0, _ctx); /*(4938) -> 4939 list<4938>*/;
      kk_box_t _x20683 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_list_fun20684_2(_b_18742_18720, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20683, _ctx);
    }
    kk_std_core__list _ctail_173120 = kk_std_core__list_hole(); /*list<4938>*/;
    kk_std_core__list _ctail_173130 = kk_std_core__new_Cons(kk_reuse_null, x_17859, _ctail_173120, _ctx); /*list<4938>*/;
    { // tailcall
      kk_integer_t _x20686 = kk_integer_add(lo1,(kk_integer_from_small(1)),kk_context()); /*int*/
      kk_std_core_types__ctail _x20687;
      kk_box_t* field0_17864 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_173130)->tail)); /*cfield<list<4938>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x18728 = _acc0._cons.CTail._field1;
        kk_box_t* last2 = _acc0._cons.CTail._field2;
        kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18728, NULL);
        kk_std_core__list_dup(head1);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __0 = kk_Unit;
        *(last2) = kk_std_core__list_box(_ctail_173130, _ctx);
        kk_std_core__list _b_18743_18733 = head1; /*list<4938>*/;
        kk_box_t* _b_18744_18734 = field0_17864; /*cfield<list<4938>>*/;
        _x20687 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18743_18733, _ctx), _b_18744_18734, _ctx); /*ctail<58>*/
        goto _match20688;
      }
      kk_std_core__list _b_18745_18735 = _ctail_173130; /*list<4938>*/;
      kk_box_t* _b_18746_18736 = field0_17864; /*cfield<list<4938>>*/;
      _x20687 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18745_18735, _ctx), _b_18746_18736, _ctx); /*ctail<58>*/
      _match20688: ;
      lo1 = _x20686;
      _acc0 = _x20687;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  kk_integer_drop(hi1, _ctx);
  kk_integer_drop(lo1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18737 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x18737, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Applies a function `f` to list of increasing elements from `lo`  to `hi`
// (including both `lo`  and `hi` ).
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core__ctailm_list_fun20693__t_2 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_integer_t hi2;
  kk_integer_t lo2;
};
static kk_std_core__list kk_std_core__ctailm_list_fun20693_2(kk_function_t _fself, kk_box_t _ctail_173160, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20693_2(kk_function_t _accm0, kk_function_t f2, kk_integer_t hi2, kk_integer_t lo2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20693__t_2* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20693__t_2, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20693_2, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->hi2 = hi2;
  _self->lo2 = lo2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_list_fun20693_2(kk_function_t _fself, kk_box_t _ctail_173160, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20693__t_2* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20693__t_2*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<4938>) -> list<4938> */
  kk_function_t f2 = _self->f2; /* (int) -> 4939 4938 */
  kk_integer_t hi2 = _self->hi2; /* int */
  kk_integer_t lo2 = _self->lo2; /* int */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_integer_dup(hi2);kk_integer_dup(lo2);}, {}, _ctx)
  return kk_std_core__mlift17706_op(_accm0, f2, hi2, lo2, _ctail_173160, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_list_fun20695__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18749_18747;
};
static kk_box_t kk_std_core__ctailm_list_fun20695_2(kk_function_t _fself, kk_box_t _b_18748, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20695_2(kk_function_t _b_18749_18747, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20695__t_2* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20695__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20695_2, kk_context());
  _self->_b_18749_18747 = _b_18749_18747;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_list_fun20695_2(kk_function_t _fself, kk_box_t _b_18748, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20695__t_2* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20695__t_2*, _fself);
  kk_function_t _b_18749_18747 = _self->_b_18749_18747; /* (4938) -> 4939 list<4938> */
  kk_drop_match(_self, {kk_function_dup(_b_18749_18747);}, {}, _ctx)
  kk_std_core__list _x20696 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18749_18747, (_b_18749_18747, _b_18748, _ctx)); /*list<4938>*/
  return kk_std_core__list_box(_x20696, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_list_fun20699__t_2 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x0_17867;
};
static kk_std_core__list kk_std_core__ctailm_list_fun20699_2(kk_function_t _fself, kk_std_core__list _ctail_173150, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20699_2(kk_function_t _accm0, kk_box_t x0_17867, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20699__t_2* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20699__t_2, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20699_2, kk_context());
  _self->_accm0 = _accm0;
  _self->x0_17867 = x0_17867;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_list_fun20699_2(kk_function_t _fself, kk_std_core__list _ctail_173150, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20699__t_2* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20699__t_2*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<4938>) -> list<4938> */
  kk_box_t x0_17867 = _self->x0_17867; /* 4938 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x0_17867);}, {}, _ctx)
  kk_std_core__list _x20700 = kk_std_core__new_Cons(kk_reuse_null, x0_17867, _ctail_173150, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x20700, _ctx));
}

kk_std_core__list kk_std_core__ctailm_list_2(kk_integer_t lo2, kk_integer_t hi2, kk_function_t f2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, f : (int) -> e a, (list<a>) -> list<a>) -> e list<a> */ 
  kk__tailcall: ;
  bool _match_20243 = kk_integer_lte_borrow(lo2,hi2,kk_context()); /*bool*/;
  if (_match_20243) {
    kk_box_t x0_17867;
    kk_function_t _x20692 = kk_function_dup(f2); /*(int) -> 4939 4938*/
    kk_integer_t _x20691 = kk_integer_dup(lo2); /*int*/
    x0_17867 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_context_t*), _x20692, (_x20692, _x20691, _ctx)); /*4938*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x0_17867, _ctx);
      kk_function_t _b_18749_18747 = kk_std_core__new_ctailm_list_fun20693_2(_accm0, f2, hi2, lo2, _ctx); /*(4938) -> 4939 list<4938>*/;
      kk_box_t _x20694 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_list_fun20695_2(_b_18749_18747, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20694, _ctx);
    }
    { // tailcall
      kk_integer_t _x20697 = kk_integer_add(lo2,(kk_integer_from_small(1)),kk_context()); /*int*/
      kk_function_t _x20698 = kk_std_core__new_ctailm_list_fun20699_2(_accm0, x0_17867, _ctx); /*(list<4938>) -> list<4938>*/
      lo2 = _x20697;
      _accm0 = _x20698;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  kk_integer_drop(hi2, _ctx);
  kk_integer_drop(lo2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Applies a function `f` to list of increasing elements from `lo`  to `hi`
// (including both `lo`  and `hi` ).
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core_list_fun20701__t_2 {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_list_fun20701_2(kk_function_t _fself, kk_std_core__list _ctail_17314, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_list_fun20701_2(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_list_fun20701_2, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_list_fun20701_2(kk_function_t _fself, kk_std_core__list _ctail_17314, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17314;
}

kk_std_core__list kk_std_core_list_2(kk_integer_t lo3, kk_integer_t hi3, kk_function_t f3, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, f : (int) -> e a) -> e list<a> */ 
  bool _match_20242 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_20242) {
    return kk_std_core__ctail_list_2(lo3, hi3, f3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_list_2(lo3, hi3, f3, kk_std_core_new_list_fun20701_2(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17707_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_integer_t hi, kk_integer_t lo, kk_integer_t stride, kk_box_t _ctail_17317, kk_context_t* _ctx) { /* forall<a,e> (ctail<list<a>>, f : (int) -> e a, hi : int, lo : int, stride : int, a) -> e list<a> */ 
  kk_std_core__list _ctail_17318 = kk_std_core__list_hole(); /*list<5029>*/;
  kk_std_core__list _ctail_17319 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17317, _ctail_17318, _ctx); /*list<5029>*/;
  kk_integer_t _x20702;
  kk_integer_t _x20703 = kk_integer_dup(stride); /*int*/
  _x20702 = kk_integer_add(lo,_x20703,kk_context()); /*int*/
  kk_std_core_types__ctail _x20704;
  kk_box_t* field_17872 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17319)->tail)); /*cfield<list<5029>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18756 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18756, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17319, _ctx);
    kk_std_core__list _b_18765_18761 = head0; /*list<5029>*/;
    kk_box_t* _b_18766_18762 = field_17872; /*cfield<list<5029>>*/;
    _x20704 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18765_18761, _ctx), _b_18766_18762, _ctx); /*ctail<58>*/
    goto _match20705;
  }
  kk_std_core__list _b_18767_18763 = _ctail_17319; /*list<5029>*/;
  kk_box_t* _b_18768_18764 = field_17872; /*cfield<list<5029>>*/;
  _x20704 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18767_18763, _ctx), _b_18768_18764, _ctx); /*ctail<58>*/
  _match20705: ;
  return kk_std_core__ctail_list_3(_x20702, hi, stride, f, _x20704, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17708_op_fun20709__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t _ctail_17322;
};
static kk_std_core__list kk_std_core__mlift17708_op_fun20709(kk_function_t _fself, kk_std_core__list _ctail_17321, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17708_op_fun20709(kk_function_t _accm, kk_box_t _ctail_17322, kk_context_t* _ctx) {
  struct kk_std_core__mlift17708_op_fun20709__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17708_op_fun20709__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17708_op_fun20709, kk_context());
  _self->_accm = _accm;
  _self->_ctail_17322 = _ctail_17322;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17708_op_fun20709(kk_function_t _fself, kk_std_core__list _ctail_17321, kk_context_t* _ctx) {
  struct kk_std_core__mlift17708_op_fun20709__t* _self = kk_function_as(struct kk_std_core__mlift17708_op_fun20709__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<5029>) -> list<5029> */
  kk_box_t _ctail_17322 = _self->_ctail_17322; /* 5029 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(_ctail_17322);}, {}, _ctx)
  kk_std_core__list _x20710 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17322, _ctail_17321, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x20710, _ctx));
}

kk_std_core__list kk_std_core__mlift17708_op(kk_function_t _accm, kk_function_t f0, kk_integer_t hi0, kk_integer_t lo0, kk_integer_t stride0, kk_box_t _ctail_17322, kk_context_t* _ctx) { /* forall<a,e> ((list<a>) -> list<a>, f : (int) -> e a, hi : int, lo : int, stride : int, a) -> e list<a> */ 
  kk_integer_t _x20707;
  kk_integer_t _x20708 = kk_integer_dup(stride0); /*int*/
  _x20707 = kk_integer_add(lo0,_x20708,kk_context()); /*int*/
  return kk_std_core__ctailm_list_3(_x20707, hi0, stride0, f0, kk_std_core__new_mlift17708_op_fun20709(_accm, _ctail_17322, _ctx), _ctx);
}
 
// Returns an integer list of increasing elements from `lo`  to `hi` with stride `stride`.
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core__ctail_list_fun20713__t_3 {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_integer_t hi1;
  kk_integer_t lo1;
  kk_integer_t stride1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_list_fun20713_3(kk_function_t _fself, kk_box_t _ctail_173170, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_list_fun20713_3(kk_function_t f1, kk_integer_t hi1, kk_integer_t lo1, kk_integer_t stride1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20713__t_3* _self = kk_function_alloc_as(struct kk_std_core__ctail_list_fun20713__t_3, 7, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_list_fun20713_3, kk_context());
  _self->f1 = f1;
  _self->hi1 = hi1;
  _self->lo1 = lo1;
  _self->stride1 = stride1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_list_fun20713_3(kk_function_t _fself, kk_box_t _ctail_173170, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20713__t_3* _self = kk_function_as(struct kk_std_core__ctail_list_fun20713__t_3*, _fself);
  kk_function_t f1 = _self->f1; /* (int) -> 5030 5029 */
  kk_integer_t hi1 = _self->hi1; /* int */
  kk_integer_t lo1 = _self->lo1; /* int */
  kk_integer_t stride1 = _self->stride1; /* int */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<5029>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_integer_dup(hi1);kk_integer_dup(lo1);kk_integer_dup(stride1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17707_op(_acc0, f1, hi1, lo1, stride1, _ctail_173170, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_list_fun20715__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18791_18769;
};
static kk_box_t kk_std_core__ctail_list_fun20715_3(kk_function_t _fself, kk_box_t _b_18770, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_list_fun20715_3(kk_function_t _b_18791_18769, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20715__t_3* _self = kk_function_alloc_as(struct kk_std_core__ctail_list_fun20715__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_list_fun20715_3, kk_context());
  _self->_b_18791_18769 = _b_18791_18769;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_list_fun20715_3(kk_function_t _fself, kk_box_t _b_18770, kk_context_t* _ctx) {
  struct kk_std_core__ctail_list_fun20715__t_3* _self = kk_function_as(struct kk_std_core__ctail_list_fun20715__t_3*, _fself);
  kk_function_t _b_18791_18769 = _self->_b_18791_18769; /* (5029) -> 5030 list<5029> */
  kk_drop_match(_self, {kk_function_dup(_b_18791_18769);}, {}, _ctx)
  kk_std_core__list _x20716 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18791_18769, (_b_18791_18769, _b_18770, _ctx)); /*list<5029>*/
  return kk_std_core__list_box(_x20716, _ctx);
}

kk_std_core__list kk_std_core__ctail_list_3(kk_integer_t lo1, kk_integer_t hi1, kk_integer_t stride1, kk_function_t f1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, stride : int, f : (int) -> e a, ctail<list<a>>) -> e list<a> */ 
  kk__tailcall: ;
  bool _match_20240 = kk_integer_lte_borrow(lo1,hi1,kk_context()); /*bool*/;
  if (_match_20240) {
    kk_box_t x_17873;
    kk_function_t _x20712 = kk_function_dup(f1); /*(int) -> 5030 5029*/
    kk_integer_t _x20711 = kk_integer_dup(lo1); /*int*/
    x_17873 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_context_t*), _x20712, (_x20712, _x20711, _ctx)); /*5029*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x_17873, _ctx);
      kk_function_t _b_18791_18769 = kk_std_core__new_ctail_list_fun20713_3(f1, hi1, lo1, stride1, _acc0, _ctx); /*(5029) -> 5030 list<5029>*/;
      kk_box_t _x20714 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_list_fun20715_3(_b_18791_18769, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20714, _ctx);
    }
    kk_std_core__list _ctail_173180 = kk_std_core__list_hole(); /*list<5029>*/;
    kk_std_core__list _ctail_173190 = kk_std_core__new_Cons(kk_reuse_null, x_17873, _ctail_173180, _ctx); /*list<5029>*/;
    { // tailcall
      kk_integer_t _x20717;
      kk_integer_t _x20718 = kk_integer_dup(stride1); /*int*/
      _x20717 = kk_integer_add(lo1,_x20718,kk_context()); /*int*/
      kk_std_core_types__ctail _x20719;
      kk_box_t* field0_17878 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_173190)->tail)); /*cfield<list<5029>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x18777 = _acc0._cons.CTail._field1;
        kk_box_t* last2 = _acc0._cons.CTail._field2;
        kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18777, NULL);
        kk_std_core__list_dup(head1);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __0 = kk_Unit;
        *(last2) = kk_std_core__list_box(_ctail_173190, _ctx);
        kk_std_core__list _b_18792_18782 = head1; /*list<5029>*/;
        kk_box_t* _b_18793_18783 = field0_17878; /*cfield<list<5029>>*/;
        _x20719 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18792_18782, _ctx), _b_18793_18783, _ctx); /*ctail<58>*/
        goto _match20720;
      }
      kk_std_core__list _b_18794_18784 = _ctail_173190; /*list<5029>*/;
      kk_box_t* _b_18795_18785 = field0_17878; /*cfield<list<5029>>*/;
      _x20719 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18794_18784, _ctx), _b_18795_18785, _ctx); /*ctail<58>*/
      _match20720: ;
      lo1 = _x20717;
      _acc0 = _x20719;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  kk_integer_drop(hi1, _ctx);
  kk_integer_drop(lo1, _ctx);
  kk_integer_drop(stride1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18786 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x18786, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Returns an integer list of increasing elements from `lo`  to `hi` with stride `stride`.
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core__ctailm_list_fun20725__t_3 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_integer_t hi2;
  kk_integer_t lo2;
  kk_integer_t stride2;
};
static kk_std_core__list kk_std_core__ctailm_list_fun20725_3(kk_function_t _fself, kk_box_t _ctail_173220, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20725_3(kk_function_t _accm0, kk_function_t f2, kk_integer_t hi2, kk_integer_t lo2, kk_integer_t stride2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20725__t_3* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20725__t_3, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20725_3, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->hi2 = hi2;
  _self->lo2 = lo2;
  _self->stride2 = stride2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_list_fun20725_3(kk_function_t _fself, kk_box_t _ctail_173220, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20725__t_3* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20725__t_3*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<5029>) -> list<5029> */
  kk_function_t f2 = _self->f2; /* (int) -> 5030 5029 */
  kk_integer_t hi2 = _self->hi2; /* int */
  kk_integer_t lo2 = _self->lo2; /* int */
  kk_integer_t stride2 = _self->stride2; /* int */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_integer_dup(hi2);kk_integer_dup(lo2);kk_integer_dup(stride2);}, {}, _ctx)
  return kk_std_core__mlift17708_op(_accm0, f2, hi2, lo2, stride2, _ctail_173220, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_list_fun20727__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18798_18796;
};
static kk_box_t kk_std_core__ctailm_list_fun20727_3(kk_function_t _fself, kk_box_t _b_18797, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20727_3(kk_function_t _b_18798_18796, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20727__t_3* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20727__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20727_3, kk_context());
  _self->_b_18798_18796 = _b_18798_18796;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_list_fun20727_3(kk_function_t _fself, kk_box_t _b_18797, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20727__t_3* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20727__t_3*, _fself);
  kk_function_t _b_18798_18796 = _self->_b_18798_18796; /* (5029) -> 5030 list<5029> */
  kk_drop_match(_self, {kk_function_dup(_b_18798_18796);}, {}, _ctx)
  kk_std_core__list _x20728 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18798_18796, (_b_18798_18796, _b_18797, _ctx)); /*list<5029>*/
  return kk_std_core__list_box(_x20728, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_list_fun20732__t_3 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x0_17881;
};
static kk_std_core__list kk_std_core__ctailm_list_fun20732_3(kk_function_t _fself, kk_std_core__list _ctail_173210, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_list_fun20732_3(kk_function_t _accm0, kk_box_t x0_17881, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20732__t_3* _self = kk_function_alloc_as(struct kk_std_core__ctailm_list_fun20732__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_list_fun20732_3, kk_context());
  _self->_accm0 = _accm0;
  _self->x0_17881 = x0_17881;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_list_fun20732_3(kk_function_t _fself, kk_std_core__list _ctail_173210, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_list_fun20732__t_3* _self = kk_function_as(struct kk_std_core__ctailm_list_fun20732__t_3*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<5029>) -> list<5029> */
  kk_box_t x0_17881 = _self->x0_17881; /* 5029 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x0_17881);}, {}, _ctx)
  kk_std_core__list _x20733 = kk_std_core__new_Cons(kk_reuse_null, x0_17881, _ctail_173210, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x20733, _ctx));
}

kk_std_core__list kk_std_core__ctailm_list_3(kk_integer_t lo2, kk_integer_t hi2, kk_integer_t stride2, kk_function_t f2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, stride : int, f : (int) -> e a, (list<a>) -> list<a>) -> e list<a> */ 
  kk__tailcall: ;
  bool _match_20238 = kk_integer_lte_borrow(lo2,hi2,kk_context()); /*bool*/;
  if (_match_20238) {
    kk_box_t x0_17881;
    kk_function_t _x20724 = kk_function_dup(f2); /*(int) -> 5030 5029*/
    kk_integer_t _x20723 = kk_integer_dup(lo2); /*int*/
    x0_17881 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_context_t*), _x20724, (_x20724, _x20723, _ctx)); /*5029*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x0_17881, _ctx);
      kk_function_t _b_18798_18796 = kk_std_core__new_ctailm_list_fun20725_3(_accm0, f2, hi2, lo2, stride2, _ctx); /*(5029) -> 5030 list<5029>*/;
      kk_box_t _x20726 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_list_fun20727_3(_b_18798_18796, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20726, _ctx);
    }
    { // tailcall
      kk_integer_t _x20729;
      kk_integer_t _x20730 = kk_integer_dup(stride2); /*int*/
      _x20729 = kk_integer_add(lo2,_x20730,kk_context()); /*int*/
      kk_function_t _x20731 = kk_std_core__new_ctailm_list_fun20732_3(_accm0, x0_17881, _ctx); /*(list<5029>) -> list<5029>*/
      lo2 = _x20729;
      _accm0 = _x20731;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  kk_integer_drop(hi2, _ctx);
  kk_integer_drop(lo2, _ctx);
  kk_integer_drop(stride2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Returns an integer list of increasing elements from `lo`  to `hi` with stride `stride`.
// If `lo > hi`  the function returns the empty list.


// lift anonymous function
struct kk_std_core_list_fun20734__t_3 {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_list_fun20734_3(kk_function_t _fself, kk_std_core__list _ctail_17320, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_list_fun20734_3(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_list_fun20734_3, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_list_fun20734_3(kk_function_t _fself, kk_std_core__list _ctail_17320, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17320;
}

kk_std_core__list kk_std_core_list_3(kk_integer_t lo3, kk_integer_t hi3, kk_integer_t stride3, kk_function_t f3, kk_context_t* _ctx) { /* forall<a,e> (lo : int, hi : int, stride : int, f : (int) -> e a) -> e list<a> */ 
  bool _match_20237 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_20237) {
    return kk_std_core__ctail_list_3(lo3, hi3, stride3, f3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_list_3(lo3, hi3, stride3, f3, kk_std_core_new_list_fun20734_3(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17709_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_std_core__list xx, kk_box_t _ctail_17323, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, f : (a) -> e b, xx : list<a>, b) -> e list<b> */ 
  kk_std_core__list _ctail_17324 = kk_std_core__list_hole(); /*list<6139>*/;
  kk_std_core__list _ctail_17325 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17323, _ctail_17324, _ctx); /*list<6139>*/;
  kk_std_core_types__ctail _x20735;
  kk_box_t* field_17886 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17325)->tail)); /*cfield<list<6139>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18805 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18805, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17325, _ctx);
    kk_std_core__list _b_18814_18810 = head0; /*list<6139>*/;
    kk_box_t* _b_18815_18811 = field_17886; /*cfield<list<6139>>*/;
    _x20735 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18814_18810, _ctx), _b_18815_18811, _ctx); /*ctail<58>*/
    goto _match20736;
  }
  kk_std_core__list _b_18816_18812 = _ctail_17325; /*list<6139>*/;
  kk_box_t* _b_18817_18813 = field_17886; /*cfield<list<6139>>*/;
  _x20735 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18816_18812, _ctx), _b_18817_18813, _ctx); /*ctail<58>*/
  _match20736: ;
  return kk_std_core__ctail_map_5(xx, f, _x20735, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17710_op_fun20738__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t _ctail_17328;
};
static kk_std_core__list kk_std_core__mlift17710_op_fun20738(kk_function_t _fself, kk_std_core__list _ctail_17327, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17710_op_fun20738(kk_function_t _accm, kk_box_t _ctail_17328, kk_context_t* _ctx) {
  struct kk_std_core__mlift17710_op_fun20738__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17710_op_fun20738__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17710_op_fun20738, kk_context());
  _self->_accm = _accm;
  _self->_ctail_17328 = _ctail_17328;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17710_op_fun20738(kk_function_t _fself, kk_std_core__list _ctail_17327, kk_context_t* _ctx) {
  struct kk_std_core__mlift17710_op_fun20738__t* _self = kk_function_as(struct kk_std_core__mlift17710_op_fun20738__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<6139>) -> list<6139> */
  kk_box_t _ctail_17328 = _self->_ctail_17328; /* 6139 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(_ctail_17328);}, {}, _ctx)
  kk_std_core__list _x20739 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17328, _ctail_17327, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x20739, _ctx));
}

kk_std_core__list kk_std_core__mlift17710_op(kk_function_t _accm, kk_function_t f0, kk_std_core__list xx0, kk_box_t _ctail_17328, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, f : (a) -> e b, xx : list<a>, b) -> e list<b> */ 
  return kk_std_core__ctailm_map_5(xx0, f0, kk_std_core__new_mlift17710_op_fun20738(_accm, _ctail_17328, _ctx), _ctx);
}
 
// Apply a function `f`  to each element of the input list in sequence.


// lift anonymous function
struct kk_std_core__ctail_map_fun20742__t_5 {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_map_fun20742_5(kk_function_t _fself, kk_box_t _ctail_173230, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_map_fun20742_5(kk_function_t f1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_fun20742__t_5* _self = kk_function_alloc_as(struct kk_std_core__ctail_map_fun20742__t_5, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_map_fun20742_5, kk_context());
  _self->f1 = f1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_map_fun20742_5(kk_function_t _fself, kk_box_t _ctail_173230, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_fun20742__t_5* _self = kk_function_as(struct kk_std_core__ctail_map_fun20742__t_5*, _fself);
  kk_function_t f1 = _self->f1; /* (6138) -> 6140 6139 */
  kk_std_core__list xx1 = _self->xx1; /* list<6138> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<6139>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17709_op(_acc0, f1, xx1, _ctail_173230, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_map_fun20744__t_5 {
  struct kk_function_s _base;
  kk_function_t _b_18840_18818;
};
static kk_box_t kk_std_core__ctail_map_fun20744_5(kk_function_t _fself, kk_box_t _b_18819, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_map_fun20744_5(kk_function_t _b_18840_18818, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_fun20744__t_5* _self = kk_function_alloc_as(struct kk_std_core__ctail_map_fun20744__t_5, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_map_fun20744_5, kk_context());
  _self->_b_18840_18818 = _b_18840_18818;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_map_fun20744_5(kk_function_t _fself, kk_box_t _b_18819, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_fun20744__t_5* _self = kk_function_as(struct kk_std_core__ctail_map_fun20744__t_5*, _fself);
  kk_function_t _b_18840_18818 = _self->_b_18840_18818; /* (6139) -> 6140 list<6139> */
  kk_drop_match(_self, {kk_function_dup(_b_18840_18818);}, {}, _ctx)
  kk_std_core__list _x20745 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18840_18818, (_b_18840_18818, _b_18819, _ctx)); /*list<6139>*/
  return kk_std_core__list_box(_x20745, _ctx);
}

kk_std_core__list kk_std_core__ctail_map_5(kk_std_core__list xs, kk_function_t f1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e b, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20740 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con20740->head;
    kk_std_core__list xx1 = _con20740->tail;
    kk_reuse_t _ru_20348 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20348 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_box_t x0_17887;
    kk_function_t _x20741 = kk_function_dup(f1); /*(6138) -> 6140 6139*/
    x0_17887 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20741, (_x20741, x, _ctx)); /*6139*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20348);
      kk_box_drop(x0_17887, _ctx);
      kk_function_t _b_18840_18818 = kk_std_core__new_ctail_map_fun20742_5(f1, xx1, _acc0, _ctx); /*(6139) -> 6140 list<6139>*/;
      kk_box_t _x20743 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_map_fun20744_5(_b_18840_18818, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20743, _ctx);
    }
    kk_std_core__list _ctail_173240 = kk_std_core__list_hole(); /*list<6139>*/;
    kk_std_core__list _ctail_173250 = kk_std_core__new_Cons(_ru_20348, x0_17887, _ctail_173240, _ctx); /*list<6139>*/;
    { // tailcall
      kk_std_core_types__ctail _x20746;
      kk_box_t* field0_17892 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_173250)->tail)); /*cfield<list<6139>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x18826 = _acc0._cons.CTail._field1;
        kk_box_t* last2 = _acc0._cons.CTail._field2;
        kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18826, NULL);
        kk_std_core__list_dup(head1);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __0 = kk_Unit;
        *(last2) = kk_std_core__list_box(_ctail_173250, _ctx);
        kk_std_core__list _b_18841_18831 = head1; /*list<6139>*/;
        kk_box_t* _b_18842_18832 = field0_17892; /*cfield<list<6139>>*/;
        _x20746 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18841_18831, _ctx), _b_18842_18832, _ctx); /*ctail<58>*/
        goto _match20747;
      }
      kk_std_core__list _b_18843_18833 = _ctail_173250; /*list<6139>*/;
      kk_box_t* _b_18844_18834 = field0_17892; /*cfield<list<6139>>*/;
      _x20746 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18843_18833, _ctx), _b_18844_18834, _ctx); /*ctail<58>*/
      _match20747: ;
      xs = xx1;
      _acc0 = _x20746;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x18835 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x18835, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Apply a function `f`  to each element of the input list in sequence.


// lift anonymous function
struct kk_std_core__ctailm_map_fun20752__t_5 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_map_fun20752_5(kk_function_t _fself, kk_box_t _ctail_173280, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_fun20752_5(kk_function_t _accm0, kk_function_t f2, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20752__t_5* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_fun20752__t_5, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_fun20752_5, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_map_fun20752_5(kk_function_t _fself, kk_box_t _ctail_173280, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20752__t_5* _self = kk_function_as(struct kk_std_core__ctailm_map_fun20752__t_5*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<6139>) -> list<6139> */
  kk_function_t f2 = _self->f2; /* (6138) -> 6140 6139 */
  kk_std_core__list xx2 = _self->xx2; /* list<6138> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17710_op(_accm0, f2, xx2, _ctail_173280, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_map_fun20754__t_5 {
  struct kk_function_s _base;
  kk_function_t _b_18847_18845;
};
static kk_box_t kk_std_core__ctailm_map_fun20754_5(kk_function_t _fself, kk_box_t _b_18846, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_fun20754_5(kk_function_t _b_18847_18845, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20754__t_5* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_fun20754__t_5, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_fun20754_5, kk_context());
  _self->_b_18847_18845 = _b_18847_18845;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_map_fun20754_5(kk_function_t _fself, kk_box_t _b_18846, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20754__t_5* _self = kk_function_as(struct kk_std_core__ctailm_map_fun20754__t_5*, _fself);
  kk_function_t _b_18847_18845 = _self->_b_18847_18845; /* (6139) -> 6140 list<6139> */
  kk_drop_match(_self, {kk_function_dup(_b_18847_18845);}, {}, _ctx)
  kk_std_core__list _x20755 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_18847_18845, (_b_18847_18845, _b_18846, _ctx)); /*list<6139>*/
  return kk_std_core__list_box(_x20755, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_map_fun20757__t_5 {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x2_17895;
};
static kk_std_core__list kk_std_core__ctailm_map_fun20757_5(kk_function_t _fself, kk_std_core__list _ctail_173270, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_fun20757_5(kk_function_t _accm0, kk_box_t x2_17895, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20757__t_5* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_fun20757__t_5, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_fun20757_5, kk_context());
  _self->_accm0 = _accm0;
  _self->x2_17895 = x2_17895;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_map_fun20757_5(kk_function_t _fself, kk_std_core__list _ctail_173270, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_fun20757__t_5* _self = kk_function_as(struct kk_std_core__ctailm_map_fun20757__t_5*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<6139>) -> list<6139> */
  kk_box_t x2_17895 = _self->x2_17895; /* 6139 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x2_17895);}, {}, _ctx)
  kk_std_core__list _x20758 = kk_std_core__new_Cons(kk_reuse_null, x2_17895, _ctail_173270, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x20758, _ctx));
}

kk_std_core__list kk_std_core__ctailm_map_5(kk_std_core__list xs0, kk_function_t f2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e b, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con20750 = kk_std_core__as_Cons(xs0);
    kk_box_t x1 = _con20750->head;
    kk_std_core__list xx2 = _con20750->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx2);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    kk_box_t x2_17895;
    kk_function_t _x20751 = kk_function_dup(f2); /*(6138) -> 6140 6139*/
    x2_17895 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20751, (_x20751, x1, _ctx)); /*6139*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x2_17895, _ctx);
      kk_function_t _b_18847_18845 = kk_std_core__new_ctailm_map_fun20752_5(_accm0, f2, xx2, _ctx); /*(6139) -> 6140 list<6139>*/;
      kk_box_t _x20753 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_map_fun20754_5(_b_18847_18845, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x20753, _ctx);
    }
    { // tailcall
      kk_function_t _x20756 = kk_std_core__new_ctailm_map_fun20757_5(_accm0, x2_17895, _ctx); /*(list<6139>) -> list<6139>*/
      xs0 = xx2;
      _accm0 = _x20756;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Apply a function `f`  to each element of the input list in sequence.


// lift anonymous function
struct kk_std_core_map_fun20759__t_5 {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_map_fun20759_5(kk_function_t _fself, kk_std_core__list _ctail_17326, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20759_5(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_map_fun20759_5, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_map_fun20759_5(kk_function_t _fself, kk_std_core__list _ctail_17326, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17326;
}

kk_std_core__list kk_std_core_map_5(kk_std_core__list xs1, kk_function_t f3, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e b) -> e list<b> */ 
  bool _match_20234 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_20234) {
    return kk_std_core__ctail_map_5(xs1, f3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_map_5(xs1, f3, kk_std_core_new_map_fun20759_5(_ctx), _ctx);
}
 
// lifted
// specialized std/core/map.5 to parameters [f]

kk_std_core__list kk_std_core__ctail_lift17174_list_4(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (xs : list<int>, ctail<list<char>>) -> total list<char> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con20760 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x18848 = _con20760->head;
    kk_std_core__list xx = _con20760->tail;
    kk_integer_t x = kk_integer_unbox(_box_x18848);
    kk_reuse_t _ru_20350 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20350 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_integer_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_char_t _ctail_17329;
    kk_char_t _borrowApp_20233 = kk_integer_clamp32_borrow(x); /*char*/;
    kk_integer_drop(x, _ctx);
    _ctail_17329 = _borrowApp_20233; /*char*/
    kk_std_core__list _ctail_17330 = kk_std_core__list_hole(); /*list<char>*/;
    kk_std_core__list _ctail_17331 = kk_std_core__new_Cons(_ru_20350, kk_char_box(_ctail_17329, _ctx), _ctail_17330, _ctx); /*list<char>*/;
    { // tailcall
      kk_std_core_types__ctail _x20762;
      kk_box_t* field_17900 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17331)->tail)); /*cfield<list<char>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18859 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18859, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17331, _ctx);
        kk_std_core__list _b_18873_18864 = head0; /*list<char>*/;
        kk_box_t* _b_18874_18865 = field_17900; /*cfield<list<char>>*/;
        _x20762 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18873_18864, _ctx), _b_18874_18865, _ctx); /*ctail<58>*/
        goto _match20763;
      }
      kk_std_core__list _b_18875_18866 = _ctail_17331; /*list<char>*/;
      kk_box_t* _b_18876_18867 = field_17900; /*cfield<list<char>>*/;
      _x20762 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_18875_18866, _ctx), _b_18876_18867, _ctx); /*ctail<58>*/
      _match20763: ;
      xs = xx;
      _acc = _x20762;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18868 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18868, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted
// specialized std/core/map.5 to parameters [f]

kk_std_core__list kk_std_core__lift17174_list_4(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<int>) -> total list<char> */ 
  return kk_std_core__ctail_lift17174_list_4(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// Convert a string to a list of characters

kk_std_core__list kk_std_core_list_6(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> total list<char> */ 
  return kk_string_to_list(s,kk_context());
}


// lift anonymous function
struct kk_std_core_map_fun20769__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_core_map_fun20769(kk_function_t _fself, kk_box_t _y_17445, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20769(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_map_fun20769, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_core_map_fun20769(kk_function_t _fself, kk_box_t _y_17445, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_types__new_Just(_y_17445, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20771__t {
  struct kk_function_s _base;
  kk_function_t _b_18879_18877;
};
static kk_box_t kk_std_core_map_fun20771(kk_function_t _fself, kk_box_t _b_18878, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20771(kk_function_t _b_18879_18877, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20771__t* _self = kk_function_alloc_as(struct kk_std_core_map_fun20771__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20771, kk_context());
  _self->_b_18879_18877 = _b_18879_18877;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20771(kk_function_t _fself, kk_box_t _b_18878, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20771__t* _self = kk_function_as(struct kk_std_core_map_fun20771__t*, _fself);
  kk_function_t _b_18879_18877 = _self->_b_18879_18877; /* (5319) -> 5320 maybe<5319> */
  kk_drop_match(_self, {kk_function_dup(_b_18879_18877);}, {}, _ctx)
  kk_std_core_types__maybe _x20772 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _b_18879_18877, (_b_18879_18877, _b_18878, _ctx)); /*maybe<5319>*/
  return kk_std_core_types__maybe_box(_x20772, _ctx);
}

kk_std_core_types__maybe kk_std_core_map(kk_std_core_types__maybe m, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (m : maybe<a>, f : (a) -> e b) -> e maybe<b> */ 
  if (kk_std_core_types__is_Nothing(m)) {
    kk_function_drop(f, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_box_t x = m._cons.Just.value;
  kk_box_t x0_17903 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x, _ctx)); /*5319*/;
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17903, _ctx);
    kk_function_t _b_18879_18877 = kk_std_core_new_map_fun20769(_ctx); /*(5319) -> 5320 maybe<5319>*/;
    kk_box_t _x20770 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20771(_b_18879_18877, _ctx), _ctx); /*3858*/
    return kk_std_core_types__maybe_unbox(_x20770, _ctx);
  }
  return kk_std_core_types__new_Just(x0_17903, _ctx);
}
 
// Map over the `Right` component of an `:either` type.


// lift anonymous function
struct kk_std_core_map_fun20773__t_1 {
  struct kk_function_s _base;
};
static kk_std_core_types__either kk_std_core_map_fun20773_1(kk_function_t _fself, kk_box_t _y_17447, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20773_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_map_fun20773_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__either kk_std_core_map_fun20773_1(kk_function_t _fself, kk_box_t _y_17447, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_types__new_Right(_y_17447, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20775__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_18882_18880;
};
static kk_box_t kk_std_core_map_fun20775_1(kk_function_t _fself, kk_box_t _b_18881, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20775_1(kk_function_t _b_18882_18880, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20775__t_1* _self = kk_function_alloc_as(struct kk_std_core_map_fun20775__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20775_1, kk_context());
  _self->_b_18882_18880 = _b_18882_18880;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20775_1(kk_function_t _fself, kk_box_t _b_18881, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20775__t_1* _self = kk_function_as(struct kk_std_core_map_fun20775__t_1*, _fself);
  kk_function_t _b_18882_18880 = _self->_b_18882_18880; /* (5357) -> 5358 either<5355,5357> */
  kk_drop_match(_self, {kk_function_dup(_b_18882_18880);}, {}, _ctx)
  kk_std_core_types__either _x20776 = kk_function_call(kk_std_core_types__either, (kk_function_t, kk_box_t, kk_context_t*), _b_18882_18880, (_b_18882_18880, _b_18881, _ctx)); /*either<5355,5357>*/
  return kk_std_core_types__either_box(_x20776, _ctx);
}

kk_std_core_types__either kk_std_core_map_1(kk_std_core_types__either e, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,c,e> (e : either<a,b>, f : (b) -> e c) -> e either<a,c> */ 
  if (kk_std_core_types__is_Right(e)) {
    kk_box_t x = e._cons.Right.right;
    kk_box_t x0_17907 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x, _ctx)); /*5357*/;
    if (kk_yielding(kk_context())) {
      kk_box_drop(x0_17907, _ctx);
      kk_function_t _b_18882_18880 = kk_std_core_new_map_fun20773_1(_ctx); /*(5357) -> 5358 either<5355,5357>*/;
      kk_box_t _x20774 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20775_1(_b_18882_18880, _ctx), _ctx); /*3858*/
      return kk_std_core_types__either_unbox(_x20774, _ctx);
    }
    return kk_std_core_types__new_Right(x0_17907, _ctx);
  }
  kk_box_t x00 = e._cons.Left.left;
  kk_function_drop(f, _ctx);
  return kk_std_core_types__new_Left(x00, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17714_map_fun20778__t_2 {
  struct kk_function_s _base;
  kk_box_t _y_17449;
};
static kk_std_core_types__tuple2_ kk_std_core__mlift17714_map_fun20778_2(kk_function_t _fself, kk_box_t _y_17450, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17714_map_fun20778_2(kk_box_t _y_17449, kk_context_t* _ctx) {
  struct kk_std_core__mlift17714_map_fun20778__t_2* _self = kk_function_alloc_as(struct kk_std_core__mlift17714_map_fun20778__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17714_map_fun20778_2, kk_context());
  _self->_y_17449 = _y_17449;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_core__mlift17714_map_fun20778_2(kk_function_t _fself, kk_box_t _y_17450, kk_context_t* _ctx) {
  struct kk_std_core__mlift17714_map_fun20778__t_2* _self = kk_function_as(struct kk_std_core__mlift17714_map_fun20778__t_2*, _fself);
  kk_box_t _y_17449 = _self->_y_17449; /* 5502 */
  kk_drop_match(_self, {kk_box_dup(_y_17449);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__rp_(_y_17449, _y_17450, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17714_map_fun20780__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18885_18883;
};
static kk_box_t kk_std_core__mlift17714_map_fun20780_2(kk_function_t _fself, kk_box_t _b_18884, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17714_map_fun20780_2(kk_function_t _b_18885_18883, kk_context_t* _ctx) {
  struct kk_std_core__mlift17714_map_fun20780__t_2* _self = kk_function_alloc_as(struct kk_std_core__mlift17714_map_fun20780__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17714_map_fun20780_2, kk_context());
  _self->_b_18885_18883 = _b_18885_18883;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17714_map_fun20780_2(kk_function_t _fself, kk_box_t _b_18884, kk_context_t* _ctx) {
  struct kk_std_core__mlift17714_map_fun20780__t_2* _self = kk_function_as(struct kk_std_core__mlift17714_map_fun20780__t_2*, _fself);
  kk_function_t _b_18885_18883 = _self->_b_18885_18883; /* (5502) -> 5503 (5502, 5502) */
  kk_drop_match(_self, {kk_function_dup(_b_18885_18883);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x20781 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_box_t, kk_context_t*), _b_18885_18883, (_b_18885_18883, _b_18884, _ctx)); /*(5502, 5502)*/
  return kk_std_core_types__tuple2__box(_x20781, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core__mlift17714_map_2(kk_function_t f, kk_std_core_types__tuple2_ t, kk_box_t _y_17449, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e b, t : (a, a), b) -> e (b, b) */ 
  kk_box_t x_17911;
  kk_box_t _x20777;
  kk_box_t _x = t.snd;
  kk_box_dup(_x);
  kk_std_core_types__tuple2__drop(t, _ctx);
  kk_Unit;
  _x20777 = _x; /*5501*/
  x_17911 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20777, _ctx)); /*5502*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17911, _ctx);
    kk_function_t _b_18885_18883 = kk_std_core__new_mlift17714_map_fun20778_2(_y_17449, _ctx); /*(5502) -> 5503 (5502, 5502)*/;
    kk_box_t _x20779 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17714_map_fun20780_2(_b_18885_18883, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple2__unbox(_x20779, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__rp_(_y_17449, x_17911, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20784__t_2 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple2_ t;
};
static kk_std_core_types__tuple2_ kk_std_core_map_fun20784_2(kk_function_t _fself, kk_box_t _y_17449, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20784_2(kk_function_t f, kk_std_core_types__tuple2_ t, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20784__t_2* _self = kk_function_alloc_as(struct kk_std_core_map_fun20784__t_2, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20784_2, kk_context());
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_core_map_fun20784_2(kk_function_t _fself, kk_box_t _y_17449, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20784__t_2* _self = kk_function_as(struct kk_std_core_map_fun20784__t_2*, _fself);
  kk_function_t f = _self->f; /* (5501) -> 5503 5502 */
  kk_std_core_types__tuple2_ t = _self->t; /* (5501, 5501) */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple2__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17714_map_2(f, t, _y_17449, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20786__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18890_18886;
};
static kk_box_t kk_std_core_map_fun20786_2(kk_function_t _fself, kk_box_t _b_18887, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20786_2(kk_function_t _b_18890_18886, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20786__t_2* _self = kk_function_alloc_as(struct kk_std_core_map_fun20786__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20786_2, kk_context());
  _self->_b_18890_18886 = _b_18890_18886;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20786_2(kk_function_t _fself, kk_box_t _b_18887, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20786__t_2* _self = kk_function_as(struct kk_std_core_map_fun20786__t_2*, _fself);
  kk_function_t _b_18890_18886 = _self->_b_18890_18886; /* (5502) -> 5503 (5502, 5502) */
  kk_drop_match(_self, {kk_function_dup(_b_18890_18886);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x20787 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_box_t, kk_context_t*), _b_18890_18886, (_b_18890_18886, _b_18887, _ctx)); /*(5502, 5502)*/
  return kk_std_core_types__tuple2__box(_x20787, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20789__t_2 {
  struct kk_function_s _base;
  kk_box_t x_17916;
};
static kk_std_core_types__tuple2_ kk_std_core_map_fun20789_2(kk_function_t _fself, kk_box_t _y_17450, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20789_2(kk_box_t x_17916, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20789__t_2* _self = kk_function_alloc_as(struct kk_std_core_map_fun20789__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20789_2, kk_context());
  _self->x_17916 = x_17916;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_core_map_fun20789_2(kk_function_t _fself, kk_box_t _y_17450, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20789__t_2* _self = kk_function_as(struct kk_std_core_map_fun20789__t_2*, _fself);
  kk_box_t x_17916 = _self->x_17916; /* 5502 */
  kk_drop_match(_self, {kk_box_dup(x_17916);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__rp_(x_17916, _y_17450, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20791__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_18891_18888;
};
static kk_box_t kk_std_core_map_fun20791_2(kk_function_t _fself, kk_box_t _b_18889, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20791_2(kk_function_t _b_18891_18888, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20791__t_2* _self = kk_function_alloc_as(struct kk_std_core_map_fun20791__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20791_2, kk_context());
  _self->_b_18891_18888 = _b_18891_18888;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20791_2(kk_function_t _fself, kk_box_t _b_18889, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20791__t_2* _self = kk_function_as(struct kk_std_core_map_fun20791__t_2*, _fself);
  kk_function_t _b_18891_18888 = _self->_b_18891_18888; /* (5502) -> 5503 (5502, 5502) */
  kk_drop_match(_self, {kk_function_dup(_b_18891_18888);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x20792 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_box_t, kk_context_t*), _b_18891_18888, (_b_18891_18888, _b_18889, _ctx)); /*(5502, 5502)*/
  return kk_std_core_types__tuple2__box(_x20792, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core_map_2(kk_std_core_types__tuple2_ t, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (t : (a, a), f : (a) -> e b) -> e (b, b) */ 
  kk_box_t x_17916;
  kk_function_t _x20783 = kk_function_dup(f); /*(5501) -> 5503 5502*/
  kk_box_t _x20782;
  kk_box_t _x = t.fst;
  kk_box_dup(_x);
  _x20782 = _x; /*5501*/
  x_17916 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20783, (_x20783, _x20782, _ctx)); /*5502*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17916, _ctx);
    kk_function_t _b_18890_18886 = kk_std_core_new_map_fun20784_2(f, t, _ctx); /*(5502) -> 5503 (5502, 5502)*/;
    kk_box_t _x20785 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20786_2(_b_18890_18886, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple2__unbox(_x20785, _ctx);
  }
  kk_box_t x0_17920;
  kk_box_t _x20788;
  kk_box_t _x0 = t.snd;
  kk_box_dup(_x0);
  kk_std_core_types__tuple2__drop(t, _ctx);
  kk_Unit;
  _x20788 = _x0; /*5501*/
  x0_17920 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20788, _ctx)); /*5502*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17920, _ctx);
    kk_function_t _b_18891_18888 = kk_std_core_new_map_fun20789_2(x_17916, _ctx); /*(5502) -> 5503 (5502, 5502)*/;
    kk_box_t _x20790 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20791_2(_b_18891_18888, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple2__unbox(_x20790, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__rp_(x_17916, x0_17920, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17716_map_fun20794__t_3 {
  struct kk_function_s _base;
  kk_box_t _y_17451;
  kk_box_t _y_17452;
};
static kk_std_core_types__tuple3_ kk_std_core__mlift17716_map_fun20794_3(kk_function_t _fself, kk_box_t _y_17453, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17716_map_fun20794_3(kk_box_t _y_17451, kk_box_t _y_17452, kk_context_t* _ctx) {
  struct kk_std_core__mlift17716_map_fun20794__t_3* _self = kk_function_alloc_as(struct kk_std_core__mlift17716_map_fun20794__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17716_map_fun20794_3, kk_context());
  _self->_y_17451 = _y_17451;
  _self->_y_17452 = _y_17452;
  return &_self->_base;
}

static kk_std_core_types__tuple3_ kk_std_core__mlift17716_map_fun20794_3(kk_function_t _fself, kk_box_t _y_17453, kk_context_t* _ctx) {
  struct kk_std_core__mlift17716_map_fun20794__t_3* _self = kk_function_as(struct kk_std_core__mlift17716_map_fun20794__t_3*, _fself);
  kk_box_t _y_17451 = _self->_y_17451; /* 5720 */
  kk_box_t _y_17452 = _self->_y_17452; /* 5720 */
  kk_drop_match(_self, {kk_box_dup(_y_17451);kk_box_dup(_y_17452);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__comma__rp_(_y_17451, _y_17452, _y_17453, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17716_map_fun20796__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18894_18892;
};
static kk_box_t kk_std_core__mlift17716_map_fun20796_3(kk_function_t _fself, kk_box_t _b_18893, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17716_map_fun20796_3(kk_function_t _b_18894_18892, kk_context_t* _ctx) {
  struct kk_std_core__mlift17716_map_fun20796__t_3* _self = kk_function_alloc_as(struct kk_std_core__mlift17716_map_fun20796__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17716_map_fun20796_3, kk_context());
  _self->_b_18894_18892 = _b_18894_18892;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17716_map_fun20796_3(kk_function_t _fself, kk_box_t _b_18893, kk_context_t* _ctx) {
  struct kk_std_core__mlift17716_map_fun20796__t_3* _self = kk_function_as(struct kk_std_core__mlift17716_map_fun20796__t_3*, _fself);
  kk_function_t _b_18894_18892 = _self->_b_18894_18892; /* (5720) -> 5721 (5720, 5720, 5720) */
  kk_drop_match(_self, {kk_function_dup(_b_18894_18892);}, {}, _ctx)
  kk_std_core_types__tuple3_ _x20797 = kk_function_call(kk_std_core_types__tuple3_, (kk_function_t, kk_box_t, kk_context_t*), _b_18894_18892, (_b_18894_18892, _b_18893, _ctx)); /*(5720, 5720, 5720)*/
  return kk_std_core_types__tuple3__box(_x20797, _ctx);
}

kk_std_core_types__tuple3_ kk_std_core__mlift17716_map_3(kk_box_t _y_17451, kk_function_t f, kk_std_core_types__tuple3_ t, kk_box_t _y_17452, kk_context_t* _ctx) { /* forall<a,b,e> (b, f : (a) -> e b, t : (a, a, a), b) -> e (b, b, b) */ 
  kk_box_t x_17926;
  kk_box_t _x20793;
  kk_box_t _x = t.thd;
  kk_box_dup(_x);
  kk_std_core_types__tuple3__drop(t, _ctx);
  kk_Unit;
  _x20793 = _x; /*5719*/
  x_17926 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20793, _ctx)); /*5720*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17926, _ctx);
    kk_function_t _b_18894_18892 = kk_std_core__new_mlift17716_map_fun20794_3(_y_17451, _y_17452, _ctx); /*(5720) -> 5721 (5720, 5720, 5720)*/;
    kk_box_t _x20795 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17716_map_fun20796_3(_b_18894_18892, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple3__unbox(_x20795, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__comma__rp_(_y_17451, _y_17452, x_17926, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17717_map_fun20800__t_3 {
  struct kk_function_s _base;
  kk_box_t _y_17451;
  kk_function_t f;
  kk_std_core_types__tuple3_ t;
};
static kk_std_core_types__tuple3_ kk_std_core__mlift17717_map_fun20800_3(kk_function_t _fself, kk_box_t _y_17452, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17717_map_fun20800_3(kk_box_t _y_17451, kk_function_t f, kk_std_core_types__tuple3_ t, kk_context_t* _ctx) {
  struct kk_std_core__mlift17717_map_fun20800__t_3* _self = kk_function_alloc_as(struct kk_std_core__mlift17717_map_fun20800__t_3, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17717_map_fun20800_3, kk_context());
  _self->_y_17451 = _y_17451;
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple3_ kk_std_core__mlift17717_map_fun20800_3(kk_function_t _fself, kk_box_t _y_17452, kk_context_t* _ctx) {
  struct kk_std_core__mlift17717_map_fun20800__t_3* _self = kk_function_as(struct kk_std_core__mlift17717_map_fun20800__t_3*, _fself);
  kk_box_t _y_17451 = _self->_y_17451; /* 5720 */
  kk_function_t f = _self->f; /* (5719) -> 5721 5720 */
  kk_std_core_types__tuple3_ t = _self->t; /* (5719, 5719, 5719) */
  kk_drop_match(_self, {kk_box_dup(_y_17451);kk_function_dup(f);kk_std_core_types__tuple3__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17716_map_3(_y_17451, f, t, _y_17452, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17717_map_fun20802__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18897_18895;
};
static kk_box_t kk_std_core__mlift17717_map_fun20802_3(kk_function_t _fself, kk_box_t _b_18896, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17717_map_fun20802_3(kk_function_t _b_18897_18895, kk_context_t* _ctx) {
  struct kk_std_core__mlift17717_map_fun20802__t_3* _self = kk_function_alloc_as(struct kk_std_core__mlift17717_map_fun20802__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17717_map_fun20802_3, kk_context());
  _self->_b_18897_18895 = _b_18897_18895;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17717_map_fun20802_3(kk_function_t _fself, kk_box_t _b_18896, kk_context_t* _ctx) {
  struct kk_std_core__mlift17717_map_fun20802__t_3* _self = kk_function_as(struct kk_std_core__mlift17717_map_fun20802__t_3*, _fself);
  kk_function_t _b_18897_18895 = _self->_b_18897_18895; /* (5720) -> 5721 (5720, 5720, 5720) */
  kk_drop_match(_self, {kk_function_dup(_b_18897_18895);}, {}, _ctx)
  kk_std_core_types__tuple3_ _x20803 = kk_function_call(kk_std_core_types__tuple3_, (kk_function_t, kk_box_t, kk_context_t*), _b_18897_18895, (_b_18897_18895, _b_18896, _ctx)); /*(5720, 5720, 5720)*/
  return kk_std_core_types__tuple3__box(_x20803, _ctx);
}

kk_std_core_types__tuple3_ kk_std_core__mlift17717_map_3(kk_function_t f, kk_std_core_types__tuple3_ t, kk_box_t _y_17451, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e b, t : (a, a, a), b) -> e (b, b, b) */ 
  kk_box_t x_17932;
  kk_function_t _x20799 = kk_function_dup(f); /*(5719) -> 5721 5720*/
  kk_box_t _x20798;
  kk_box_t _x = t.snd;
  kk_box_dup(_x);
  _x20798 = _x; /*5719*/
  x_17932 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20799, (_x20799, _x20798, _ctx)); /*5720*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17932, _ctx);
    kk_function_t _b_18897_18895 = kk_std_core__new_mlift17717_map_fun20800_3(_y_17451, f, t, _ctx); /*(5720) -> 5721 (5720, 5720, 5720)*/;
    kk_box_t _x20801 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17717_map_fun20802_3(_b_18897_18895, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple3__unbox(_x20801, _ctx);
  }
  return kk_std_core__mlift17716_map_3(_y_17451, f, t, x_17932, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20806__t_3 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple3_ t;
};
static kk_std_core_types__tuple3_ kk_std_core_map_fun20806_3(kk_function_t _fself, kk_box_t _y_17451, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20806_3(kk_function_t f, kk_std_core_types__tuple3_ t, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20806__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20806__t_3, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20806_3, kk_context());
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple3_ kk_std_core_map_fun20806_3(kk_function_t _fself, kk_box_t _y_17451, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20806__t_3* _self = kk_function_as(struct kk_std_core_map_fun20806__t_3*, _fself);
  kk_function_t f = _self->f; /* (5719) -> 5721 5720 */
  kk_std_core_types__tuple3_ t = _self->t; /* (5719, 5719, 5719) */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple3__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17717_map_3(f, t, _y_17451, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20808__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18904_18898;
};
static kk_box_t kk_std_core_map_fun20808_3(kk_function_t _fself, kk_box_t _b_18899, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20808_3(kk_function_t _b_18904_18898, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20808__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20808__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20808_3, kk_context());
  _self->_b_18904_18898 = _b_18904_18898;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20808_3(kk_function_t _fself, kk_box_t _b_18899, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20808__t_3* _self = kk_function_as(struct kk_std_core_map_fun20808__t_3*, _fself);
  kk_function_t _b_18904_18898 = _self->_b_18904_18898; /* (5720) -> 5721 (5720, 5720, 5720) */
  kk_drop_match(_self, {kk_function_dup(_b_18904_18898);}, {}, _ctx)
  kk_std_core_types__tuple3_ _x20809 = kk_function_call(kk_std_core_types__tuple3_, (kk_function_t, kk_box_t, kk_context_t*), _b_18904_18898, (_b_18904_18898, _b_18899, _ctx)); /*(5720, 5720, 5720)*/
  return kk_std_core_types__tuple3__box(_x20809, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20812__t_3 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple3_ t;
  kk_box_t x_17935;
};
static kk_std_core_types__tuple3_ kk_std_core_map_fun20812_3(kk_function_t _fself, kk_box_t _y_17452, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20812_3(kk_function_t f, kk_std_core_types__tuple3_ t, kk_box_t x_17935, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20812__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20812__t_3, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20812_3, kk_context());
  _self->f = f;
  _self->t = t;
  _self->x_17935 = x_17935;
  return &_self->_base;
}

static kk_std_core_types__tuple3_ kk_std_core_map_fun20812_3(kk_function_t _fself, kk_box_t _y_17452, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20812__t_3* _self = kk_function_as(struct kk_std_core_map_fun20812__t_3*, _fself);
  kk_function_t f = _self->f; /* (5719) -> 5721 5720 */
  kk_std_core_types__tuple3_ t = _self->t; /* (5719, 5719, 5719) */
  kk_box_t x_17935 = _self->x_17935; /* 5720 */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple3__dup(t);kk_box_dup(x_17935);}, {}, _ctx)
  return kk_std_core__mlift17716_map_3(x_17935, f, t, _y_17452, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20814__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18905_18900;
};
static kk_box_t kk_std_core_map_fun20814_3(kk_function_t _fself, kk_box_t _b_18901, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20814_3(kk_function_t _b_18905_18900, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20814__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20814__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20814_3, kk_context());
  _self->_b_18905_18900 = _b_18905_18900;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20814_3(kk_function_t _fself, kk_box_t _b_18901, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20814__t_3* _self = kk_function_as(struct kk_std_core_map_fun20814__t_3*, _fself);
  kk_function_t _b_18905_18900 = _self->_b_18905_18900; /* (5720) -> 5721 (5720, 5720, 5720) */
  kk_drop_match(_self, {kk_function_dup(_b_18905_18900);}, {}, _ctx)
  kk_std_core_types__tuple3_ _x20815 = kk_function_call(kk_std_core_types__tuple3_, (kk_function_t, kk_box_t, kk_context_t*), _b_18905_18900, (_b_18905_18900, _b_18901, _ctx)); /*(5720, 5720, 5720)*/
  return kk_std_core_types__tuple3__box(_x20815, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20817__t_3 {
  struct kk_function_s _base;
  kk_box_t x_17935;
  kk_box_t x0_17939;
};
static kk_std_core_types__tuple3_ kk_std_core_map_fun20817_3(kk_function_t _fself, kk_box_t _y_17453, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20817_3(kk_box_t x_17935, kk_box_t x0_17939, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20817__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20817__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20817_3, kk_context());
  _self->x_17935 = x_17935;
  _self->x0_17939 = x0_17939;
  return &_self->_base;
}

static kk_std_core_types__tuple3_ kk_std_core_map_fun20817_3(kk_function_t _fself, kk_box_t _y_17453, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20817__t_3* _self = kk_function_as(struct kk_std_core_map_fun20817__t_3*, _fself);
  kk_box_t x_17935 = _self->x_17935; /* 5720 */
  kk_box_t x0_17939 = _self->x0_17939; /* 5720 */
  kk_drop_match(_self, {kk_box_dup(x_17935);kk_box_dup(x0_17939);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__comma__rp_(x_17935, x0_17939, _y_17453, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20819__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_18906_18902;
};
static kk_box_t kk_std_core_map_fun20819_3(kk_function_t _fself, kk_box_t _b_18903, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20819_3(kk_function_t _b_18906_18902, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20819__t_3* _self = kk_function_alloc_as(struct kk_std_core_map_fun20819__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20819_3, kk_context());
  _self->_b_18906_18902 = _b_18906_18902;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20819_3(kk_function_t _fself, kk_box_t _b_18903, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20819__t_3* _self = kk_function_as(struct kk_std_core_map_fun20819__t_3*, _fself);
  kk_function_t _b_18906_18902 = _self->_b_18906_18902; /* (5720) -> 5721 (5720, 5720, 5720) */
  kk_drop_match(_self, {kk_function_dup(_b_18906_18902);}, {}, _ctx)
  kk_std_core_types__tuple3_ _x20820 = kk_function_call(kk_std_core_types__tuple3_, (kk_function_t, kk_box_t, kk_context_t*), _b_18906_18902, (_b_18906_18902, _b_18903, _ctx)); /*(5720, 5720, 5720)*/
  return kk_std_core_types__tuple3__box(_x20820, _ctx);
}

kk_std_core_types__tuple3_ kk_std_core_map_3(kk_std_core_types__tuple3_ t, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (t : (a, a, a), f : (a) -> e b) -> e (b, b, b) */ 
  kk_box_t x_17935;
  kk_function_t _x20805 = kk_function_dup(f); /*(5719) -> 5721 5720*/
  kk_box_t _x20804;
  kk_box_t _x = t.fst;
  kk_box_dup(_x);
  _x20804 = _x; /*5719*/
  x_17935 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20805, (_x20805, _x20804, _ctx)); /*5720*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17935, _ctx);
    kk_function_t _b_18904_18898 = kk_std_core_new_map_fun20806_3(f, t, _ctx); /*(5720) -> 5721 (5720, 5720, 5720)*/;
    kk_box_t _x20807 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20808_3(_b_18904_18898, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple3__unbox(_x20807, _ctx);
  }
  kk_box_t x0_17939;
  kk_function_t _x20811 = kk_function_dup(f); /*(5719) -> 5721 5720*/
  kk_box_t _x20810;
  kk_box_t _x0 = t.snd;
  kk_box_dup(_x0);
  _x20810 = _x0; /*5719*/
  x0_17939 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20811, (_x20811, _x20810, _ctx)); /*5720*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17939, _ctx);
    kk_function_t _b_18905_18900 = kk_std_core_new_map_fun20812_3(f, t, x_17935, _ctx); /*(5720) -> 5721 (5720, 5720, 5720)*/;
    kk_box_t _x20813 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20814_3(_b_18905_18900, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple3__unbox(_x20813, _ctx);
  }
  kk_box_t x1_17943;
  kk_box_t _x20816;
  kk_box_t _x1 = t.thd;
  kk_box_dup(_x1);
  kk_std_core_types__tuple3__drop(t, _ctx);
  kk_Unit;
  _x20816 = _x1; /*5719*/
  x1_17943 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20816, _ctx)); /*5720*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x1_17943, _ctx);
    kk_function_t _b_18906_18902 = kk_std_core_new_map_fun20817_3(x_17935, x0_17939, _ctx); /*(5720) -> 5721 (5720, 5720, 5720)*/;
    kk_box_t _x20818 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20819_3(_b_18906_18902, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple3__unbox(_x20818, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__comma__rp_(x_17935, x0_17939, x1_17943, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17719_map_fun20823__t_4 {
  struct kk_function_s _base;
  kk_box_t _y_17454;
  kk_box_t _y_17455;
  kk_box_t _y_17456;
};
static kk_std_core_types__tuple4_ kk_std_core__mlift17719_map_fun20823_4(kk_function_t _fself, kk_box_t _y_17457, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17719_map_fun20823_4(kk_box_t _y_17454, kk_box_t _y_17455, kk_box_t _y_17456, kk_context_t* _ctx) {
  struct kk_std_core__mlift17719_map_fun20823__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17719_map_fun20823__t_4, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17719_map_fun20823_4, kk_context());
  _self->_y_17454 = _y_17454;
  _self->_y_17455 = _y_17455;
  _self->_y_17456 = _y_17456;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core__mlift17719_map_fun20823_4(kk_function_t _fself, kk_box_t _y_17457, kk_context_t* _ctx) {
  struct kk_std_core__mlift17719_map_fun20823__t_4* _self = kk_function_as(struct kk_std_core__mlift17719_map_fun20823__t_4*, _fself);
  kk_box_t _y_17454 = _self->_y_17454; /* 6007 */
  kk_box_t _y_17455 = _self->_y_17455; /* 6007 */
  kk_box_t _y_17456 = _self->_y_17456; /* 6007 */
  kk_drop_match(_self, {kk_box_dup(_y_17454);kk_box_dup(_y_17455);kk_box_dup(_y_17456);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, _y_17454, _y_17455, _y_17456, _y_17457, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17719_map_fun20825__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18909_18907;
};
static kk_box_t kk_std_core__mlift17719_map_fun20825_4(kk_function_t _fself, kk_box_t _b_18908, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17719_map_fun20825_4(kk_function_t _b_18909_18907, kk_context_t* _ctx) {
  struct kk_std_core__mlift17719_map_fun20825__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17719_map_fun20825__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17719_map_fun20825_4, kk_context());
  _self->_b_18909_18907 = _b_18909_18907;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17719_map_fun20825_4(kk_function_t _fself, kk_box_t _b_18908, kk_context_t* _ctx) {
  struct kk_std_core__mlift17719_map_fun20825__t_4* _self = kk_function_as(struct kk_std_core__mlift17719_map_fun20825__t_4*, _fself);
  kk_function_t _b_18909_18907 = _self->_b_18909_18907; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18909_18907);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20826 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18909_18907, (_b_18909_18907, _b_18908, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20826, _ctx);
}

kk_std_core_types__tuple4_ kk_std_core__mlift17719_map_4(kk_box_t _y_17454, kk_box_t _y_17455, kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t _y_17456, kk_context_t* _ctx) { /* forall<a,b,e> (b, b, f : (a) -> e b, t : (a, a, a, a), b) -> e (b, b, b, b) */ 
  kk_box_t x_17950;
  kk_box_t _x20821;
  struct kk_std_core_types__Tuple4_* _con20822 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _pat00 = _con20822->fst;
  kk_box_t _pat10 = _con20822->snd;
  kk_box_t _pat2 = _con20822->thd;
  kk_box_t _x = _con20822->field4;
  if (kk_likely(kk_std_core_types__tuple4__is_unique(t))) {
    kk_box_drop(_pat2, _ctx);
    kk_Unit;
    kk_box_drop(_pat10, _ctx);
    kk_Unit;
    kk_box_drop(_pat00, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core_types__tuple4__free(t);
  }
  else {
    kk_box_dup(_x);
    kk_Unit;
    kk_std_core_types__tuple4__decref(t, _ctx);
    kk_Unit;
  }
  _x20821 = _x; /*6006*/
  x_17950 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20821, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17950, _ctx);
    kk_function_t _b_18909_18907 = kk_std_core__new_mlift17719_map_fun20823_4(_y_17454, _y_17455, _y_17456, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20824 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17719_map_fun20825_4(_b_18909_18907, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20824, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, _y_17454, _y_17455, _y_17456, x_17950, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17720_map_fun20830__t_4 {
  struct kk_function_s _base;
  kk_box_t _y_17454;
  kk_box_t _y_17455;
  kk_function_t f;
  kk_std_core_types__tuple4_ t;
};
static kk_std_core_types__tuple4_ kk_std_core__mlift17720_map_fun20830_4(kk_function_t _fself, kk_box_t _y_17456, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17720_map_fun20830_4(kk_box_t _y_17454, kk_box_t _y_17455, kk_function_t f, kk_std_core_types__tuple4_ t, kk_context_t* _ctx) {
  struct kk_std_core__mlift17720_map_fun20830__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17720_map_fun20830__t_4, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17720_map_fun20830_4, kk_context());
  _self->_y_17454 = _y_17454;
  _self->_y_17455 = _y_17455;
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core__mlift17720_map_fun20830_4(kk_function_t _fself, kk_box_t _y_17456, kk_context_t* _ctx) {
  struct kk_std_core__mlift17720_map_fun20830__t_4* _self = kk_function_as(struct kk_std_core__mlift17720_map_fun20830__t_4*, _fself);
  kk_box_t _y_17454 = _self->_y_17454; /* 6007 */
  kk_box_t _y_17455 = _self->_y_17455; /* 6007 */
  kk_function_t f = _self->f; /* (6006) -> 6008 6007 */
  kk_std_core_types__tuple4_ t = _self->t; /* (6006, 6006, 6006, 6006) */
  kk_drop_match(_self, {kk_box_dup(_y_17454);kk_box_dup(_y_17455);kk_function_dup(f);kk_std_core_types__tuple4__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17719_map_4(_y_17454, _y_17455, f, t, _y_17456, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17720_map_fun20832__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18912_18910;
};
static kk_box_t kk_std_core__mlift17720_map_fun20832_4(kk_function_t _fself, kk_box_t _b_18911, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17720_map_fun20832_4(kk_function_t _b_18912_18910, kk_context_t* _ctx) {
  struct kk_std_core__mlift17720_map_fun20832__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17720_map_fun20832__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17720_map_fun20832_4, kk_context());
  _self->_b_18912_18910 = _b_18912_18910;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17720_map_fun20832_4(kk_function_t _fself, kk_box_t _b_18911, kk_context_t* _ctx) {
  struct kk_std_core__mlift17720_map_fun20832__t_4* _self = kk_function_as(struct kk_std_core__mlift17720_map_fun20832__t_4*, _fself);
  kk_function_t _b_18912_18910 = _self->_b_18912_18910; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18912_18910);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20833 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18912_18910, (_b_18912_18910, _b_18911, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20833, _ctx);
}

kk_std_core_types__tuple4_ kk_std_core__mlift17720_map_4(kk_box_t _y_17454, kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t _y_17455, kk_context_t* _ctx) { /* forall<a,b,e> (b, f : (a) -> e b, t : (a, a, a, a), b) -> e (b, b, b, b) */ 
  kk_box_t x_17957;
  kk_function_t _x20829 = kk_function_dup(f); /*(6006) -> 6008 6007*/
  kk_box_t _x20827;
  struct kk_std_core_types__Tuple4_* _con20828 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _x = _con20828->thd;
  kk_box_dup(_x);
  _x20827 = _x; /*6006*/
  x_17957 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20829, (_x20829, _x20827, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17957, _ctx);
    kk_function_t _b_18912_18910 = kk_std_core__new_mlift17720_map_fun20830_4(_y_17454, _y_17455, f, t, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20831 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17720_map_fun20832_4(_b_18912_18910, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20831, _ctx);
  }
  return kk_std_core__mlift17719_map_4(_y_17454, _y_17455, f, t, x_17957, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17721_map_fun20837__t_4 {
  struct kk_function_s _base;
  kk_box_t _y_17454;
  kk_function_t f;
  kk_std_core_types__tuple4_ t;
};
static kk_std_core_types__tuple4_ kk_std_core__mlift17721_map_fun20837_4(kk_function_t _fself, kk_box_t _y_17455, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17721_map_fun20837_4(kk_box_t _y_17454, kk_function_t f, kk_std_core_types__tuple4_ t, kk_context_t* _ctx) {
  struct kk_std_core__mlift17721_map_fun20837__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17721_map_fun20837__t_4, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17721_map_fun20837_4, kk_context());
  _self->_y_17454 = _y_17454;
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core__mlift17721_map_fun20837_4(kk_function_t _fself, kk_box_t _y_17455, kk_context_t* _ctx) {
  struct kk_std_core__mlift17721_map_fun20837__t_4* _self = kk_function_as(struct kk_std_core__mlift17721_map_fun20837__t_4*, _fself);
  kk_box_t _y_17454 = _self->_y_17454; /* 6007 */
  kk_function_t f = _self->f; /* (6006) -> 6008 6007 */
  kk_std_core_types__tuple4_ t = _self->t; /* (6006, 6006, 6006, 6006) */
  kk_drop_match(_self, {kk_box_dup(_y_17454);kk_function_dup(f);kk_std_core_types__tuple4__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17720_map_4(_y_17454, f, t, _y_17455, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17721_map_fun20839__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18915_18913;
};
static kk_box_t kk_std_core__mlift17721_map_fun20839_4(kk_function_t _fself, kk_box_t _b_18914, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17721_map_fun20839_4(kk_function_t _b_18915_18913, kk_context_t* _ctx) {
  struct kk_std_core__mlift17721_map_fun20839__t_4* _self = kk_function_alloc_as(struct kk_std_core__mlift17721_map_fun20839__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17721_map_fun20839_4, kk_context());
  _self->_b_18915_18913 = _b_18915_18913;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17721_map_fun20839_4(kk_function_t _fself, kk_box_t _b_18914, kk_context_t* _ctx) {
  struct kk_std_core__mlift17721_map_fun20839__t_4* _self = kk_function_as(struct kk_std_core__mlift17721_map_fun20839__t_4*, _fself);
  kk_function_t _b_18915_18913 = _self->_b_18915_18913; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18915_18913);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20840 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18915_18913, (_b_18915_18913, _b_18914, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20840, _ctx);
}

kk_std_core_types__tuple4_ kk_std_core__mlift17721_map_4(kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t _y_17454, kk_context_t* _ctx) { /* forall<a,b,e> (f : (a) -> e b, t : (a, a, a, a), b) -> e (b, b, b, b) */ 
  kk_box_t x_17960;
  kk_function_t _x20836 = kk_function_dup(f); /*(6006) -> 6008 6007*/
  kk_box_t _x20834;
  struct kk_std_core_types__Tuple4_* _con20835 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _x = _con20835->snd;
  kk_box_dup(_x);
  _x20834 = _x; /*6006*/
  x_17960 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20836, (_x20836, _x20834, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17960, _ctx);
    kk_function_t _b_18915_18913 = kk_std_core__new_mlift17721_map_fun20837_4(_y_17454, f, t, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20838 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17721_map_fun20839_4(_b_18915_18913, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20838, _ctx);
  }
  return kk_std_core__mlift17720_map_4(_y_17454, f, t, x_17960, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20844__t_4 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple4_ t;
};
static kk_std_core_types__tuple4_ kk_std_core_map_fun20844_4(kk_function_t _fself, kk_box_t _y_17454, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20844_4(kk_function_t f, kk_std_core_types__tuple4_ t, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20844__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20844__t_4, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20844_4, kk_context());
  _self->f = f;
  _self->t = t;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core_map_fun20844_4(kk_function_t _fself, kk_box_t _y_17454, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20844__t_4* _self = kk_function_as(struct kk_std_core_map_fun20844__t_4*, _fself);
  kk_function_t f = _self->f; /* (6006) -> 6008 6007 */
  kk_std_core_types__tuple4_ t = _self->t; /* (6006, 6006, 6006, 6006) */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple4__dup(t);}, {}, _ctx)
  return kk_std_core__mlift17721_map_4(f, t, _y_17454, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20846__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18924_18916;
};
static kk_box_t kk_std_core_map_fun20846_4(kk_function_t _fself, kk_box_t _b_18917, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20846_4(kk_function_t _b_18924_18916, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20846__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20846__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20846_4, kk_context());
  _self->_b_18924_18916 = _b_18924_18916;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20846_4(kk_function_t _fself, kk_box_t _b_18917, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20846__t_4* _self = kk_function_as(struct kk_std_core_map_fun20846__t_4*, _fself);
  kk_function_t _b_18924_18916 = _self->_b_18924_18916; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18924_18916);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20847 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18924_18916, (_b_18924_18916, _b_18917, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20847, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20851__t_4 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple4_ t;
  kk_box_t x_17963;
};
static kk_std_core_types__tuple4_ kk_std_core_map_fun20851_4(kk_function_t _fself, kk_box_t _y_17455, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20851_4(kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t x_17963, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20851__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20851__t_4, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20851_4, kk_context());
  _self->f = f;
  _self->t = t;
  _self->x_17963 = x_17963;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core_map_fun20851_4(kk_function_t _fself, kk_box_t _y_17455, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20851__t_4* _self = kk_function_as(struct kk_std_core_map_fun20851__t_4*, _fself);
  kk_function_t f = _self->f; /* (6006) -> 6008 6007 */
  kk_std_core_types__tuple4_ t = _self->t; /* (6006, 6006, 6006, 6006) */
  kk_box_t x_17963 = _self->x_17963; /* 6007 */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple4__dup(t);kk_box_dup(x_17963);}, {}, _ctx)
  return kk_std_core__mlift17720_map_4(x_17963, f, t, _y_17455, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20853__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18925_18918;
};
static kk_box_t kk_std_core_map_fun20853_4(kk_function_t _fself, kk_box_t _b_18919, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20853_4(kk_function_t _b_18925_18918, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20853__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20853__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20853_4, kk_context());
  _self->_b_18925_18918 = _b_18925_18918;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20853_4(kk_function_t _fself, kk_box_t _b_18919, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20853__t_4* _self = kk_function_as(struct kk_std_core_map_fun20853__t_4*, _fself);
  kk_function_t _b_18925_18918 = _self->_b_18925_18918; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18925_18918);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20854 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18925_18918, (_b_18925_18918, _b_18919, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20854, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20858__t_4 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_std_core_types__tuple4_ t;
  kk_box_t x_17963;
  kk_box_t x0_17967;
};
static kk_std_core_types__tuple4_ kk_std_core_map_fun20858_4(kk_function_t _fself, kk_box_t _y_17456, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20858_4(kk_function_t f, kk_std_core_types__tuple4_ t, kk_box_t x_17963, kk_box_t x0_17967, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20858__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20858__t_4, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20858_4, kk_context());
  _self->f = f;
  _self->t = t;
  _self->x_17963 = x_17963;
  _self->x0_17967 = x0_17967;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core_map_fun20858_4(kk_function_t _fself, kk_box_t _y_17456, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20858__t_4* _self = kk_function_as(struct kk_std_core_map_fun20858__t_4*, _fself);
  kk_function_t f = _self->f; /* (6006) -> 6008 6007 */
  kk_std_core_types__tuple4_ t = _self->t; /* (6006, 6006, 6006, 6006) */
  kk_box_t x_17963 = _self->x_17963; /* 6007 */
  kk_box_t x0_17967 = _self->x0_17967; /* 6007 */
  kk_drop_match(_self, {kk_function_dup(f);kk_std_core_types__tuple4__dup(t);kk_box_dup(x_17963);kk_box_dup(x0_17967);}, {}, _ctx)
  return kk_std_core__mlift17719_map_4(x_17963, x0_17967, f, t, _y_17456, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20860__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18926_18920;
};
static kk_box_t kk_std_core_map_fun20860_4(kk_function_t _fself, kk_box_t _b_18921, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20860_4(kk_function_t _b_18926_18920, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20860__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20860__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20860_4, kk_context());
  _self->_b_18926_18920 = _b_18926_18920;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20860_4(kk_function_t _fself, kk_box_t _b_18921, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20860__t_4* _self = kk_function_as(struct kk_std_core_map_fun20860__t_4*, _fself);
  kk_function_t _b_18926_18920 = _self->_b_18926_18920; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18926_18920);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20861 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18926_18920, (_b_18926_18920, _b_18921, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20861, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20864__t_4 {
  struct kk_function_s _base;
  kk_box_t x_17963;
  kk_box_t x0_17967;
  kk_box_t x1_17971;
};
static kk_std_core_types__tuple4_ kk_std_core_map_fun20864_4(kk_function_t _fself, kk_box_t _y_17457, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20864_4(kk_box_t x_17963, kk_box_t x0_17967, kk_box_t x1_17971, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20864__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20864__t_4, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20864_4, kk_context());
  _self->x_17963 = x_17963;
  _self->x0_17967 = x0_17967;
  _self->x1_17971 = x1_17971;
  return &_self->_base;
}

static kk_std_core_types__tuple4_ kk_std_core_map_fun20864_4(kk_function_t _fself, kk_box_t _y_17457, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20864__t_4* _self = kk_function_as(struct kk_std_core_map_fun20864__t_4*, _fself);
  kk_box_t x_17963 = _self->x_17963; /* 6007 */
  kk_box_t x0_17967 = _self->x0_17967; /* 6007 */
  kk_box_t x1_17971 = _self->x1_17971; /* 6007 */
  kk_drop_match(_self, {kk_box_dup(x_17963);kk_box_dup(x0_17967);kk_box_dup(x1_17971);}, {}, _ctx)
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, x_17963, x0_17967, x1_17971, _y_17457, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20866__t_4 {
  struct kk_function_s _base;
  kk_function_t _b_18927_18922;
};
static kk_box_t kk_std_core_map_fun20866_4(kk_function_t _fself, kk_box_t _b_18923, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20866_4(kk_function_t _b_18927_18922, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20866__t_4* _self = kk_function_alloc_as(struct kk_std_core_map_fun20866__t_4, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20866_4, kk_context());
  _self->_b_18927_18922 = _b_18927_18922;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20866_4(kk_function_t _fself, kk_box_t _b_18923, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20866__t_4* _self = kk_function_as(struct kk_std_core_map_fun20866__t_4*, _fself);
  kk_function_t _b_18927_18922 = _self->_b_18927_18922; /* (6007) -> 6008 (6007, 6007, 6007, 6007) */
  kk_drop_match(_self, {kk_function_dup(_b_18927_18922);}, {}, _ctx)
  kk_std_core_types__tuple4_ _x20867 = kk_function_call(kk_std_core_types__tuple4_, (kk_function_t, kk_box_t, kk_context_t*), _b_18927_18922, (_b_18927_18922, _b_18923, _ctx)); /*(6007, 6007, 6007, 6007)*/
  return kk_std_core_types__tuple4__box(_x20867, _ctx);
}

kk_std_core_types__tuple4_ kk_std_core_map_4(kk_std_core_types__tuple4_ t, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (t : (a, a, a, a), f : (a) -> e b) -> e (b, b, b, b) */ 
  kk_box_t x_17963;
  kk_function_t _x20843 = kk_function_dup(f); /*(6006) -> 6008 6007*/
  kk_box_t _x20841;
  struct kk_std_core_types__Tuple4_* _con20842 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _x = _con20842->fst;
  kk_box_dup(_x);
  _x20841 = _x; /*6006*/
  x_17963 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20843, (_x20843, _x20841, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_17963, _ctx);
    kk_function_t _b_18924_18916 = kk_std_core_new_map_fun20844_4(f, t, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20845 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20846_4(_b_18924_18916, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20845, _ctx);
  }
  kk_box_t x0_17967;
  kk_function_t _x20850 = kk_function_dup(f); /*(6006) -> 6008 6007*/
  kk_box_t _x20848;
  struct kk_std_core_types__Tuple4_* _con20849 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _x0 = _con20849->snd;
  kk_box_dup(_x0);
  _x20848 = _x0; /*6006*/
  x0_17967 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20850, (_x20850, _x20848, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17967, _ctx);
    kk_function_t _b_18925_18918 = kk_std_core_new_map_fun20851_4(f, t, x_17963, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20852 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20853_4(_b_18925_18918, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20852, _ctx);
  }
  kk_box_t x1_17971;
  kk_function_t _x20857 = kk_function_dup(f); /*(6006) -> 6008 6007*/
  kk_box_t _x20855;
  struct kk_std_core_types__Tuple4_* _con20856 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _x1 = _con20856->thd;
  kk_box_dup(_x1);
  _x20855 = _x1; /*6006*/
  x1_17971 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x20857, (_x20857, _x20855, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x1_17971, _ctx);
    kk_function_t _b_18926_18920 = kk_std_core_new_map_fun20858_4(f, t, x_17963, x0_17967, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20859 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20860_4(_b_18926_18920, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20859, _ctx);
  }
  kk_box_t x2_17975;
  kk_box_t _x20862;
  struct kk_std_core_types__Tuple4_* _con20863 = kk_std_core_types__as_dash__lp__comma__comma__comma__rp_(t);
  kk_box_t _pat06 = _con20863->fst;
  kk_box_t _pat13 = _con20863->snd;
  kk_box_t _pat22 = _con20863->thd;
  kk_box_t _x2 = _con20863->field4;
  if (kk_likely(kk_std_core_types__tuple4__is_unique(t))) {
    kk_box_drop(_pat22, _ctx);
    kk_Unit;
    kk_box_drop(_pat13, _ctx);
    kk_Unit;
    kk_box_drop(_pat06, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core_types__tuple4__free(t);
  }
  else {
    kk_box_dup(_x2);
    kk_Unit;
    kk_std_core_types__tuple4__decref(t, _ctx);
    kk_Unit;
  }
  _x20862 = _x2; /*6006*/
  x2_17975 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x20862, _ctx)); /*6007*/
  if (kk_yielding(kk_context())) {
    kk_box_drop(x2_17975, _ctx);
    kk_function_t _b_18927_18922 = kk_std_core_new_map_fun20864_4(x_17963, x0_17967, x1_17971, _ctx); /*(6007) -> 6008 (6007, 6007, 6007, 6007)*/;
    kk_box_t _x20865 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20866_4(_b_18927_18922, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple4__unbox(_x20865, _ctx);
  }
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, x_17963, x0_17967, x1_17971, x2_17975, _ctx);
}
 
// Apply a function `f` to each character in a string


// lift anonymous function
struct kk_std_core_map_fun20868__t_6 {
  struct kk_function_s _base;
  kk_function_t f;
};
static kk_box_t kk_std_core_map_fun20868_6(kk_function_t _fself, kk_box_t _b_18930, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20868_6(kk_function_t f, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20868__t_6* _self = kk_function_alloc_as(struct kk_std_core_map_fun20868__t_6, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20868_6, kk_context());
  _self->f = f;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20868_6(kk_function_t _fself, kk_box_t _b_18930, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20868__t_6* _self = kk_function_as(struct kk_std_core_map_fun20868__t_6*, _fself);
  kk_function_t f = _self->f; /* (char) -> 6475 char */
  kk_drop_match(_self, {kk_function_dup(f);}, {}, _ctx)
  kk_char_t _x20869;
  kk_char_t _x20870 = kk_char_unbox(_b_18930, _ctx); /*char*/
  _x20869 = kk_function_call(kk_char_t, (kk_function_t, kk_char_t, kk_context_t*), f, (f, _x20870, _ctx)); /*char*/
  return kk_char_box(_x20869, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20872__t_6 {
  struct kk_function_s _base;
};
static kk_string_t kk_std_core_map_fun20872_6(kk_function_t _fself, kk_std_core__list _x120871, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20872_6(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_map_fun20872_6, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_core_map_fun20872_6(kk_function_t _fself, kk_std_core__list _x120871, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_string_2(_x120871, _ctx);
}


// lift anonymous function
struct kk_std_core_map_fun20874__t_6 {
  struct kk_function_s _base;
  kk_function_t _b_18935_18933;
};
static kk_box_t kk_std_core_map_fun20874_6(kk_function_t _fself, kk_box_t _b_18934, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20874_6(kk_function_t _b_18935_18933, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20874__t_6* _self = kk_function_alloc_as(struct kk_std_core_map_fun20874__t_6, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20874_6, kk_context());
  _self->_b_18935_18933 = _b_18935_18933;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20874_6(kk_function_t _fself, kk_box_t _b_18934, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20874__t_6* _self = kk_function_as(struct kk_std_core_map_fun20874__t_6*, _fself);
  kk_function_t _b_18935_18933 = _self->_b_18935_18933; /* (cs : list<char>) -> total string */
  kk_drop_match(_self, {kk_function_dup(_b_18935_18933);}, {}, _ctx)
  kk_string_t _x20875;
  kk_std_core__list _x20876 = kk_std_core__list_unbox(_b_18934, _ctx); /*list<char>*/
  _x20875 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _b_18935_18933, (_b_18935_18933, _x20876, _ctx)); /*string*/
  return kk_string_box(_x20875);
}

kk_string_t kk_std_core_map_6(kk_string_t s, kk_function_t f, kk_context_t* _ctx) { /* forall<e> (s : string, f : (char) -> e char) -> e string */ 
  kk_std_core__list _b_18931_18928 = kk_std_core_list_6(s, _ctx); /*list<char>*/;
  kk_std_core__list x_17983 = kk_std_core_map_5(_b_18931_18928, kk_std_core_new_map_fun20868_6(f, _ctx), _ctx); /*list<char>*/;
  if (kk_yielding(kk_context())) {
    kk_std_core__list_drop(x_17983, _ctx);
    kk_function_t _b_18935_18933 = kk_std_core_new_map_fun20872_6(_ctx); /*(cs : list<char>) -> total string*/;
    kk_box_t _x20873 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20874_6(_b_18935_18933, _ctx), _ctx); /*3858*/
    return kk_string_unbox(_x20873);
  }
  return kk_std_core_string_2(x_17983, _ctx);
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17722_op(kk_function_t action_16980, kk_ssize_t end_16979, kk_ssize_t i, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action.16980 : (ssize_t) -> e (), end.16979 : ssize_t, i : ssize_t, wild_ : ()) -> e () */ 
  kk_ssize_t i0_17225 = kk_std_core_incr_1(i, _ctx); /*ssize_t*/;
  kk_std_core__lift17175_map_7(action_16980, end_16979, i0_17225, _ctx); return kk_Unit;
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17175_map_fun20878__t_7 {
  struct kk_function_s _base;
  kk_function_t action_169800;
  kk_ssize_t end_169790;
  kk_ssize_t i0;
};
static kk_unit_t kk_std_core__lift17175_map_fun20878_7(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17175_map_fun20878_7(kk_function_t action_169800, kk_ssize_t end_169790, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17175_map_fun20878__t_7* _self = kk_function_alloc_as(struct kk_std_core__lift17175_map_fun20878__t_7, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17175_map_fun20878_7, kk_context());
  _self->action_169800 = action_169800;
  _self->end_169790 = end_169790;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17175_map_fun20878_7(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17175_map_fun20878__t_7* _self = kk_function_as(struct kk_std_core__lift17175_map_fun20878__t_7*, _fself);
  kk_function_t action_169800 = _self->action_169800; /* (ssize_t) -> 6525 () */
  kk_ssize_t end_169790 = _self->end_169790; /* ssize_t */
  kk_ssize_t i0 = _self->i0; /* ssize_t */
  kk_drop_match(_self, {kk_function_dup(action_169800);;;}, {}, _ctx)
  return kk_std_core__mlift17722_op(action_169800, end_169790, i0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17175_map_fun20880__t_7 {
  struct kk_function_s _base;
  kk_function_t _b_18938_18936;
};
static kk_box_t kk_std_core__lift17175_map_fun20880_7(kk_function_t _fself, kk_box_t _b_18937, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17175_map_fun20880_7(kk_function_t _b_18938_18936, kk_context_t* _ctx) {
  struct kk_std_core__lift17175_map_fun20880__t_7* _self = kk_function_alloc_as(struct kk_std_core__lift17175_map_fun20880__t_7, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17175_map_fun20880_7, kk_context());
  _self->_b_18938_18936 = _b_18938_18936;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17175_map_fun20880_7(kk_function_t _fself, kk_box_t _b_18937, kk_context_t* _ctx) {
  struct kk_std_core__lift17175_map_fun20880__t_7* _self = kk_function_as(struct kk_std_core__lift17175_map_fun20880__t_7*, _fself);
  kk_function_t _b_18938_18936 = _self->_b_18938_18936; /* (wild_0 : ()) -> 6525 () */
  kk_drop_match(_self, {kk_function_dup(_b_18938_18936);}, {}, _ctx)
  kk_unit_t _x20881 = kk_Unit;
  kk_unit_t _x20882 = kk_Unit;
  kk_unit_unbox(_b_18937);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_18938_18936, (_b_18938_18936, _x20882, _ctx));
  return kk_unit_box(_x20881);
}

kk_unit_t kk_std_core__lift17175_map_7(kk_function_t action_169800, kk_ssize_t end_169790, kk_ssize_t i0, kk_context_t* _ctx) { /* forall<e> (action.16980 : (ssize_t) -> e (), end.16979 : ssize_t, i : ssize_t) -> e () */ 
  kk__tailcall: ;
  bool _match_20213 = (i0 <= end_169790); /*bool*/;
  if (_match_20213) {
    kk_unit_t x_17985 = kk_Unit;
    kk_function_t _x20877 = kk_function_dup(action_169800); /*(ssize_t) -> 6525 ()*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_ssize_t, kk_context_t*), _x20877, (_x20877, i0, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_18938_18936 = kk_std_core__new_lift17175_map_fun20878_7(action_169800, end_169790, i0, _ctx); /*(wild_0 : ()) -> 6525 ()*/;
      kk_box_t _x20879 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17175_map_fun20880_7(_b_18938_18936, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x20879); return kk_Unit;
    }
    kk_ssize_t i0_172250 = kk_std_core_incr_1(i0, _ctx); /*ssize_t*/;
    { // tailcall
      i0 = i0_172250;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action_169800, _ctx);
  kk_Unit; return kk_Unit;
}
 
// Apply a total function `f` to each element in a vector `v`


// lift anonymous function
struct kk_std_core_map_fun20888__t_7 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_vector_t v;
  kk_vector_t w;
};
static kk_unit_t kk_std_core_map_fun20888_7(kk_function_t _fself, kk_ssize_t i0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20888_7(kk_function_t f, kk_vector_t v, kk_vector_t w, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20888__t_7* _self = kk_function_alloc_as(struct kk_std_core_map_fun20888__t_7, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20888_7, kk_context());
  _self->f = f;
  _self->v = v;
  _self->w = w;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_core_map_fun20890__t_7 {
  struct kk_function_s _base;
  kk_vector_t w;
  kk_ssize_t i0;
};
static kk_box_t kk_std_core_map_fun20890_7(kk_function_t _fself, kk_box_t _b_18940, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20890_7(kk_vector_t w, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20890__t_7* _self = kk_function_alloc_as(struct kk_std_core_map_fun20890__t_7, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20890_7, kk_context());
  _self->w = w;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20890_7(kk_function_t _fself, kk_box_t _b_18940, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20890__t_7* _self = kk_function_as(struct kk_std_core_map_fun20890__t_7*, _fself);
  kk_vector_t w = _self->w; /* vector<6524> */
  kk_ssize_t i0 = _self->i0; /* ssize_t */
  kk_drop_match(_self, {kk_vector_dup(w);;}, {}, _ctx)
  kk_unit_t _x20891 = kk_Unit;
  kk_box_t _y_18944_17462 = _b_18940; /*6524*/;
  kk_vector_unsafe_assign(w,i0,_y_18944_17462,kk_context());
  return kk_unit_box(_x20891);
}
static kk_unit_t kk_std_core_map_fun20888_7(kk_function_t _fself, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20888__t_7* _self = kk_function_as(struct kk_std_core_map_fun20888__t_7*, _fself);
  kk_function_t f = _self->f; /* (6523) -> 6525 6524 */
  kk_vector_t v = _self->v; /* vector<6523> */
  kk_vector_t w = _self->w; /* vector<6524> */
  kk_drop_match(_self, {kk_function_dup(f);kk_vector_dup(v);kk_vector_dup(w);}, {}, _ctx)
  kk_box_t x_17227;
  kk_box_t _borrowApp_20212 = kk_vector_at_borrow(v, i0); /*219*/;
  kk_vector_drop(v, _ctx);
  x_17227 = _borrowApp_20212; /*6523*/
  kk_box_t x0_17992 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x_17227, _ctx)); /*6524*/;
  if (kk_yielding(kk_context())) {
    kk_box_drop(x0_17992, _ctx);
    kk_box_t _x20889 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20890_7(w, i0, _ctx), _ctx); /*3858*/
    return kk_unit_unbox(_x20889);
  }
  return kk_vector_unsafe_assign(w,i0,x0_17992,kk_context());
}


// lift anonymous function
struct kk_std_core_map_fun20892__t_7 {
  struct kk_function_s _base;
  kk_vector_t w;
};
static kk_vector_t kk_std_core_map_fun20892_7(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20892_7(kk_vector_t w, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20892__t_7* _self = kk_function_alloc_as(struct kk_std_core_map_fun20892__t_7, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20892_7, kk_context());
  _self->w = w;
  return &_self->_base;
}

static kk_vector_t kk_std_core_map_fun20892_7(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20892__t_7* _self = kk_function_as(struct kk_std_core_map_fun20892__t_7*, _fself);
  kk_vector_t w = _self->w; /* vector<6524> */
  kk_drop_match(_self, {kk_vector_dup(w);}, {}, _ctx)
  return w;
}


// lift anonymous function
struct kk_std_core_map_fun20894__t_7 {
  struct kk_function_s _base;
  kk_function_t _b_18945_18942;
};
static kk_box_t kk_std_core_map_fun20894_7(kk_function_t _fself, kk_box_t _b_18943, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_fun20894_7(kk_function_t _b_18945_18942, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20894__t_7* _self = kk_function_alloc_as(struct kk_std_core_map_fun20894__t_7, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_map_fun20894_7, kk_context());
  _self->_b_18945_18942 = _b_18945_18942;
  return &_self->_base;
}

static kk_box_t kk_std_core_map_fun20894_7(kk_function_t _fself, kk_box_t _b_18943, kk_context_t* _ctx) {
  struct kk_std_core_map_fun20894__t_7* _self = kk_function_as(struct kk_std_core_map_fun20894__t_7*, _fself);
  kk_function_t _b_18945_18942 = _self->_b_18945_18942; /* (wild_0 : ()) -> 6525 vector<6524> */
  kk_drop_match(_self, {kk_function_dup(_b_18945_18942);}, {}, _ctx)
  kk_vector_t _x20895;
  kk_unit_t _x20896 = kk_Unit;
  kk_unit_unbox(_b_18943);
  _x20895 = kk_function_call(kk_vector_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_18945_18942, (_b_18945_18942, _x20896, _ctx)); /*vector<6524>*/
  return kk_vector_box(_x20895, _ctx);
}

kk_vector_t kk_std_core_map_7(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (v : vector<a>, f : (a) -> e b) -> e vector<b> */ 
  kk_vector_t w;
  kk_ssize_t _x20883;
  kk_integer_t _x20884;
  kk_ssize_t _x20885 = kk_vector_len_borrow(v); /*ssize_t*/
  _x20884 = kk_integer_from_ssize_t(_x20885,kk_context()); /*int*/
  _x20883 = kk_std_core_ssize__t(_x20884, _ctx); /*ssize_t*/
  w = kk_vector_alloc_uninit(_x20883,NULL,kk_context()); /*vector<6524>*/
  kk_ssize_t start0_16978 = ((kk_ssize_t)0); /*ssize_t*/;
  kk_ssize_t end_16979;
  kk_ssize_t _x20886 = kk_vector_len_borrow(v); /*ssize_t*/
  end_16979 = kk_std_core_decr_1(_x20886, _ctx); /*ssize_t*/
  kk_unit_t x_17989 = kk_Unit;
  kk_function_t _x20887;
  kk_vector_dup(w);
  _x20887 = kk_std_core_new_map_fun20888_7(f, v, w, _ctx); /*(i0 : ssize_t) -> 6525 ()*/
  kk_std_core__lift17175_map_7(_x20887, end_16979, start0_16978, _ctx);
  if (kk_yielding(kk_context())) {
    kk_function_t _b_18945_18942 = kk_std_core_new_map_fun20892_7(w, _ctx); /*(wild_0 : ()) -> 6525 vector<6524>*/;
    kk_box_t _x20893 = kk_std_core_hnd_yield_extend(kk_std_core_new_map_fun20894_7(_b_18945_18942, _ctx), _ctx); /*3858*/
    return kk_vector_unbox(_x20893, _ctx);
  }
  return w;
}
 
// Right-align a string to width `width`  using `fill`  (default is a space) to fill from the left.

kk_string_t kk_std_core_pad_left(kk_string_t s, kk_integer_t width, kk_std_core_types__optional fill, kk_context_t* _ctx) { /* (s : string, width : int, fill : optional<char>) -> string */ 
  kk_ssize_t w;
  kk_integer_t _x20897 = kk_integer_dup(width); /*int*/
  w = kk_std_core_ssize__t(_x20897, _ctx); /*ssize_t*/
  kk_ssize_t n;
  kk_string_t _x20898 = kk_string_dup(s); /*string*/
  n = kk_string_len(_x20898,kk_context()); /*ssize_t*/
  bool _match_20209 = (w <= n); /*bool*/;
  if (_match_20209) {
    kk_std_core_types__optional_drop(fill, _ctx);
    return s;
  }
  kk_string_t _x20899;
  kk_string_t _x20900;
  kk_char_t _x20901;
  if (kk_std_core_types__is_Optional(fill)) {
    kk_box_t _box_x18946 = fill._cons.Optional.value;
    kk_char_t _fill_7273 = kk_char_unbox(_box_x18946, NULL);
    kk_std_core_types__optional_drop(fill, _ctx);
    kk_Unit;
    _x20901 = _fill_7273; /*char*/
    goto _match20902;
  }
  _x20901 = ' '; /*char*/
  _match20902: ;
  _x20900 = kk_std_core_string(_x20901, _ctx); /*string*/
  kk_ssize_t _x20904 = (w - n); /*ssize_t*/
  _x20899 = kk_std_core_repeatz(_x20900, _x20904, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x20899, s, _ctx);
}
 
// Show an `:int` as a hexadecimal value.
// The `width`  parameter specifies how wide the hex value is where `"0"`  is used to align.
// The `use-capitals` parameter (= `True`) determines if captical letters should be used to display the hexadecimal digits.
// The `pre` (=`"0x"`) is an optional prefix for the number (goes between the sign and the number).

kk_string_t kk_std_core_show_hex(kk_integer_t i, kk_std_core_types__optional width, kk_std_core_types__optional use_capitals, kk_std_core_types__optional pre, kk_context_t* _ctx) { /* (i : int, width : optional<int>, use-capitals : optional<bool>, pre : optional<string>) -> string */ 
  kk_string_t _x20905;
  bool _match_20206;
  kk_integer_t _borrowedArg_20207 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_20208 = kk_integer_lt_borrow(i,_borrowedArg_20207,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20207, _ctx);
  _match_20206 = _borrowApp_20208; /*bool*/
  if (_match_20206) {
    kk_define_string_literal(, _s20906, 1, "-")
    _x20905 = kk_string_dup(_s20906); /*string*/
  }
  else {
    _x20905 = kk_string_empty(); /*string*/
  }
  kk_string_t _x20908;
  kk_string_t _x20909;
  if (kk_std_core_types__is_Optional(pre)) {
    kk_box_t _box_x18947 = pre._cons.Optional.value;
    kk_string_t _pre_7381 = kk_string_unbox(_box_x18947);
    kk_string_dup(_pre_7381);
    kk_std_core_types__optional_drop(pre, _ctx);
    kk_Unit;
    _x20909 = _pre_7381; /*string*/
    goto _match20910;
  }
  kk_define_string_literal(, _s20912, 2, "0x")
  _x20909 = kk_string_dup(_s20912); /*string*/
  _match20910: ;
  kk_string_t _x20913;
  kk_string_t s_16981;
  kk_integer_t _x20914 = kk_integer_abs(i,kk_context()); /*int*/
  bool _x20915;
  if (kk_std_core_types__is_Optional(use_capitals)) {
    kk_box_t _box_x18948 = use_capitals._cons.Optional.value;
    bool _use_capitals_7377 = kk_bool_unbox(_box_x18948);
    kk_std_core_types__optional_drop(use_capitals, _ctx);
    kk_Unit;
    _x20915 = _use_capitals_7377; /*bool*/
    goto _match20916;
  }
  _x20915 = true; /*bool*/
  _match20916: ;
  s_16981 = kk_std_core_int_show_hex(_x20914, _x20915, _ctx); /*string*/
  kk_ssize_t w;
  kk_integer_t _x20918;
  if (kk_std_core_types__is_Optional(width)) {
    kk_box_t _box_x18949 = width._cons.Optional.value;
    kk_integer_t _width_7373 = kk_integer_unbox(_box_x18949);
    kk_integer_dup(_width_7373);
    kk_std_core_types__optional_drop(width, _ctx);
    kk_Unit;
    _x20918 = _width_7373; /*int*/
    goto _match20919;
  }
  _x20918 = kk_integer_from_small(1); /*int*/
  _match20919: ;
  w = kk_std_core_ssize__t(_x20918, _ctx); /*ssize_t*/
  kk_ssize_t n;
  kk_string_t _x20921 = kk_string_dup(s_16981); /*string*/
  n = kk_string_len(_x20921,kk_context()); /*ssize_t*/
  bool _match_20205 = (w <= n); /*bool*/;
  if (_match_20205) {
    _x20913 = s_16981; /*string*/
  }
  else {
    kk_string_t _x20922;
    kk_string_t _x20923 = kk_std_core_string('0', _ctx); /*string*/
    kk_ssize_t _x20924 = (w - n); /*ssize_t*/
    _x20922 = kk_std_core_repeatz(_x20923, _x20924, _ctx); /*string*/
    _x20913 = kk_std_core__lp__plus__plus__1_rp_(_x20922, s_16981, _ctx); /*string*/
  }
  _x20908 = kk_std_core__lp__plus__plus__1_rp_(_x20909, _x20913, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x20905, _x20908, _ctx);
}
 
// Is the integer positive (stricly greater than zero)

bool kk_std_core_is_pos_2(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> bool */ 
  kk_std_core_types__order x_16984;
  kk_std_core_types__order _borrowApp_20202 = kk_int_as_order(kk_integer_signum_borrow(i),kk_context()); /*order*/;
  kk_integer_drop(i, _ctx);
  x_16984 = _borrowApp_20202; /*order*/
  kk_integer_t _borrowedArg_20200;
  if (kk_std_core_types__is_Lt(x_16984)) {
    _borrowedArg_20200 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match20925;
  }
  if (kk_std_core_types__is_Eq(x_16984)) {
    _borrowedArg_20200 = kk_integer_from_small(0); /*int*/
    goto _match20925;
  }
  _borrowedArg_20200 = kk_integer_from_small(1); /*int*/
  _match20925: ;
  kk_integer_t _borrowedArg_20199 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_20201 = kk_integer_eq_borrow(_borrowedArg_20200,_borrowedArg_20199,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20200, _ctx);
  kk_integer_drop(_borrowedArg_20199, _ctx);
  return _borrowApp_20201;
}
 
// Show a character as a string

kk_string_t kk_std_core_show_char(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> string */ 
  bool _match_20141 = (c < (' ')); /*bool*/;
  if (_match_20141) {
    bool _match_20172 = (c == 0x000A); /*bool*/;
    if (_match_20172) {
      kk_define_string_literal(, _s20932, 2, "\\n")
      return kk_string_dup(_s20932);
    }
    bool _match_20173 = (c == 0x000D); /*bool*/;
    if (_match_20173) {
      kk_define_string_literal(, _s20933, 2, "\\r")
      return kk_string_dup(_s20933);
    }
    bool _match_20174 = (c == 0x0009); /*bool*/;
    if (_match_20174) {
      kk_define_string_literal(, _s20934, 2, "\\t")
      return kk_string_dup(_s20934);
    }
    bool _match_20175;
    kk_integer_t _borrowedArg_20196 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_integer_t _borrowedArg_20195 = kk_integer_from_small(255); /*int*/;
    bool _borrowApp_20197 = kk_integer_lte_borrow(_borrowedArg_20196,_borrowedArg_20195,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_20196, _ctx);
    kk_integer_drop(_borrowedArg_20195, _ctx);
    _match_20175 = _borrowApp_20197; /*bool*/
    if (_match_20175) {
      kk_string_t _x20935;
      kk_define_string_literal(, _s20936, 2, "\\x")
      _x20935 = kk_string_dup(_s20936); /*string*/
      kk_string_t _x20937;
      kk_integer_t _arg_7841 = kk_integer_from_int(c,kk_context()); /*int*/;
      kk_string_t _x20938;
      bool _match_20192;
      kk_integer_t _borrowedArg_20193 = kk_integer_from_small(0); /*int*/;
      bool _borrowApp_20194 = kk_integer_lt_borrow(_arg_7841,_borrowedArg_20193,kk_context()); /*bool*/;
      kk_integer_drop(_borrowedArg_20193, _ctx);
      _match_20192 = _borrowApp_20194; /*bool*/
      if (_match_20192) {
        kk_define_string_literal(, _s20939, 1, "-")
        _x20938 = kk_string_dup(_s20939); /*string*/
      }
      else {
        _x20938 = kk_string_empty(); /*string*/
      }
      kk_string_t _x20941;
      kk_string_t _x20942 = kk_string_empty(); /*string*/
      kk_string_t _x20944;
      kk_string_t s_16995;
      kk_integer_t _x20945 = kk_integer_abs(_arg_7841,kk_context()); /*int*/
      bool _x20946;
      kk_std_core_types__optional _match_20191 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
      if (kk_std_core_types__is_Optional(_match_20191)) {
        kk_box_t _box_x18950 = _match_20191._cons.Optional.value;
        bool _use_capitals_7377 = kk_bool_unbox(_box_x18950);
        kk_std_core_types__optional_drop(_match_20191, _ctx);
        kk_Unit;
        _x20946 = _use_capitals_7377; /*bool*/
        goto _match20947;
      }
      _x20946 = true; /*bool*/
      _match20947: ;
      s_16995 = kk_std_core_int_show_hex(_x20945, _x20946, _ctx); /*string*/
      kk_ssize_t w = ((kk_ssize_t)2); /*ssize_t*/;
      kk_ssize_t n;
      kk_string_t _x20949 = kk_string_dup(s_16995); /*string*/
      n = kk_string_len(_x20949,kk_context()); /*ssize_t*/
      bool _match_20190 = (w <= n); /*bool*/;
      if (_match_20190) {
        _x20944 = s_16995; /*string*/
      }
      else {
        kk_string_t _x20950;
        kk_string_t _x20951 = kk_std_core_string('0', _ctx); /*string*/
        kk_ssize_t _x20952 = (w - n); /*ssize_t*/
        _x20950 = kk_std_core_repeatz(_x20951, _x20952, _ctx); /*string*/
        _x20944 = kk_std_core__lp__plus__plus__1_rp_(_x20950, s_16995, _ctx); /*string*/
      }
      _x20941 = kk_std_core__lp__plus__plus__1_rp_(_x20942, _x20944, _ctx); /*string*/
      _x20937 = kk_std_core__lp__plus__plus__1_rp_(_x20938, _x20941, _ctx); /*string*/
      return kk_std_core__lp__plus__plus__1_rp_(_x20935, _x20937, _ctx);
    }
    bool _match_20176;
    kk_integer_t _borrowedArg_20188 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_integer_t _borrowedArg_20187 = kk_integer_from_int(65535, _ctx); /*int*/;
    bool _borrowApp_20189 = kk_integer_lte_borrow(_borrowedArg_20188,_borrowedArg_20187,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_20188, _ctx);
    kk_integer_drop(_borrowedArg_20187, _ctx);
    _match_20176 = _borrowApp_20189; /*bool*/
    if (_match_20176) {
      kk_string_t _x20953;
      kk_define_string_literal(, _s20954, 2, "\\u")
      _x20953 = kk_string_dup(_s20954); /*string*/
      kk_string_t _x20955;
      kk_integer_t _arg_7920 = kk_integer_from_int(c,kk_context()); /*int*/;
      kk_string_t _x20956;
      bool _match_20184;
      kk_integer_t _borrowedArg_20185 = kk_integer_from_small(0); /*int*/;
      bool _borrowApp_20186 = kk_integer_lt_borrow(_arg_7920,_borrowedArg_20185,kk_context()); /*bool*/;
      kk_integer_drop(_borrowedArg_20185, _ctx);
      _match_20184 = _borrowApp_20186; /*bool*/
      if (_match_20184) {
        kk_define_string_literal(, _s20957, 1, "-")
        _x20956 = kk_string_dup(_s20957); /*string*/
      }
      else {
        _x20956 = kk_string_empty(); /*string*/
      }
      kk_string_t _x20959;
      kk_string_t _x20960 = kk_string_empty(); /*string*/
      kk_string_t _x20962;
      kk_string_t s0_17002;
      kk_integer_t _x20963 = kk_integer_abs(_arg_7920,kk_context()); /*int*/
      bool _x20964;
      kk_std_core_types__optional _match_20183 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
      if (kk_std_core_types__is_Optional(_match_20183)) {
        kk_box_t _box_x18951 = _match_20183._cons.Optional.value;
        bool _use_capitals_73770 = kk_bool_unbox(_box_x18951);
        kk_std_core_types__optional_drop(_match_20183, _ctx);
        kk_Unit;
        _x20964 = _use_capitals_73770; /*bool*/
        goto _match20965;
      }
      _x20964 = true; /*bool*/
      _match20965: ;
      s0_17002 = kk_std_core_int_show_hex(_x20963, _x20964, _ctx); /*string*/
      kk_ssize_t w0 = ((kk_ssize_t)4); /*ssize_t*/;
      kk_ssize_t n0;
      kk_string_t _x20967 = kk_string_dup(s0_17002); /*string*/
      n0 = kk_string_len(_x20967,kk_context()); /*ssize_t*/
      bool _match_20182 = (w0 <= n0); /*bool*/;
      if (_match_20182) {
        _x20962 = s0_17002; /*string*/
      }
      else {
        kk_string_t _x20968;
        kk_string_t _x20969 = kk_std_core_string('0', _ctx); /*string*/
        kk_ssize_t _x20970 = (w0 - n0); /*ssize_t*/
        _x20968 = kk_std_core_repeatz(_x20969, _x20970, _ctx); /*string*/
        _x20962 = kk_std_core__lp__plus__plus__1_rp_(_x20968, s0_17002, _ctx); /*string*/
      }
      _x20959 = kk_std_core__lp__plus__plus__1_rp_(_x20960, _x20962, _ctx); /*string*/
      _x20955 = kk_std_core__lp__plus__plus__1_rp_(_x20956, _x20959, _ctx); /*string*/
      return kk_std_core__lp__plus__plus__1_rp_(_x20953, _x20955, _ctx);
    }
    kk_string_t _x20971;
    kk_define_string_literal(, _s20972, 2, "\\U")
    _x20971 = kk_string_dup(_s20972); /*string*/
    kk_string_t _x20973;
    kk_integer_t _arg_7961 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_string_t _x20974;
    bool _match_20179;
    kk_integer_t _borrowedArg_20180 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_20181 = kk_integer_lt_borrow(_arg_7961,_borrowedArg_20180,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_20180, _ctx);
    _match_20179 = _borrowApp_20181; /*bool*/
    if (_match_20179) {
      kk_define_string_literal(, _s20975, 1, "-")
      _x20974 = kk_string_dup(_s20975); /*string*/
    }
    else {
      _x20974 = kk_string_empty(); /*string*/
    }
    kk_string_t _x20977;
    kk_string_t _x20978 = kk_string_empty(); /*string*/
    kk_string_t _x20980;
    kk_string_t s1_17009;
    kk_integer_t _x20981 = kk_integer_abs(_arg_7961,kk_context()); /*int*/
    bool _x20982;
    kk_std_core_types__optional _match_20178 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_20178)) {
      kk_box_t _box_x18952 = _match_20178._cons.Optional.value;
      bool _use_capitals_73771 = kk_bool_unbox(_box_x18952);
      kk_std_core_types__optional_drop(_match_20178, _ctx);
      kk_Unit;
      _x20982 = _use_capitals_73771; /*bool*/
      goto _match20983;
    }
    _x20982 = true; /*bool*/
    _match20983: ;
    s1_17009 = kk_std_core_int_show_hex(_x20981, _x20982, _ctx); /*string*/
    kk_ssize_t w1 = ((kk_ssize_t)6); /*ssize_t*/;
    kk_ssize_t n1;
    kk_string_t _x20985 = kk_string_dup(s1_17009); /*string*/
    n1 = kk_string_len(_x20985,kk_context()); /*ssize_t*/
    bool _match_20177 = (w1 <= n1); /*bool*/;
    if (_match_20177) {
      _x20980 = s1_17009; /*string*/
    }
    else {
      kk_string_t _x20986;
      kk_string_t _x20987 = kk_std_core_string('0', _ctx); /*string*/
      kk_ssize_t _x20988 = (w1 - n1); /*ssize_t*/
      _x20986 = kk_std_core_repeatz(_x20987, _x20988, _ctx); /*string*/
      _x20980 = kk_std_core__lp__plus__plus__1_rp_(_x20986, s1_17009, _ctx); /*string*/
    }
    _x20977 = kk_std_core__lp__plus__plus__1_rp_(_x20978, _x20980, _ctx); /*string*/
    _x20973 = kk_std_core__lp__plus__plus__1_rp_(_x20974, _x20977, _ctx); /*string*/
    return kk_std_core__lp__plus__plus__1_rp_(_x20971, _x20973, _ctx);
  }
  bool _match_20142 = (c > ('~')); /*bool*/;
  if (_match_20142) {
    bool _match_20146 = (c == 0x000A); /*bool*/;
    if (_match_20146) {
      kk_define_string_literal(, _s20989, 2, "\\n")
      return kk_string_dup(_s20989);
    }
    bool _match_20147 = (c == 0x000D); /*bool*/;
    if (_match_20147) {
      kk_define_string_literal(, _s20990, 2, "\\r")
      return kk_string_dup(_s20990);
    }
    bool _match_20148 = (c == 0x0009); /*bool*/;
    if (_match_20148) {
      kk_define_string_literal(, _s20991, 2, "\\t")
      return kk_string_dup(_s20991);
    }
    bool _match_20149;
    kk_integer_t _borrowedArg_20170 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_integer_t _borrowedArg_20169 = kk_integer_from_small(255); /*int*/;
    bool _borrowApp_20171 = kk_integer_lte_borrow(_borrowedArg_20170,_borrowedArg_20169,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_20170, _ctx);
    kk_integer_drop(_borrowedArg_20169, _ctx);
    _match_20149 = _borrowApp_20171; /*bool*/
    if (_match_20149) {
      kk_string_t _x20992;
      kk_define_string_literal(, _s20993, 2, "\\x")
      _x20992 = kk_string_dup(_s20993); /*string*/
      kk_string_t _x20994;
      kk_integer_t _arg_78410 = kk_integer_from_int(c,kk_context()); /*int*/;
      kk_string_t _x20995;
      bool _match_20166;
      kk_integer_t _borrowedArg_20167 = kk_integer_from_small(0); /*int*/;
      bool _borrowApp_20168 = kk_integer_lt_borrow(_arg_78410,_borrowedArg_20167,kk_context()); /*bool*/;
      kk_integer_drop(_borrowedArg_20167, _ctx);
      _match_20166 = _borrowApp_20168; /*bool*/
      if (_match_20166) {
        kk_define_string_literal(, _s20996, 1, "-")
        _x20995 = kk_string_dup(_s20996); /*string*/
      }
      else {
        _x20995 = kk_string_empty(); /*string*/
      }
      kk_string_t _x20998;
      kk_string_t _x20999 = kk_string_empty(); /*string*/
      kk_string_t _x21001;
      kk_string_t s2_17016;
      kk_integer_t _x21002 = kk_integer_abs(_arg_78410,kk_context()); /*int*/
      bool _x21003;
      kk_std_core_types__optional _match_20165 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
      if (kk_std_core_types__is_Optional(_match_20165)) {
        kk_box_t _box_x18953 = _match_20165._cons.Optional.value;
        bool _use_capitals_73772 = kk_bool_unbox(_box_x18953);
        kk_std_core_types__optional_drop(_match_20165, _ctx);
        kk_Unit;
        _x21003 = _use_capitals_73772; /*bool*/
        goto _match21004;
      }
      _x21003 = true; /*bool*/
      _match21004: ;
      s2_17016 = kk_std_core_int_show_hex(_x21002, _x21003, _ctx); /*string*/
      kk_ssize_t w2 = ((kk_ssize_t)2); /*ssize_t*/;
      kk_ssize_t n2;
      kk_string_t _x21006 = kk_string_dup(s2_17016); /*string*/
      n2 = kk_string_len(_x21006,kk_context()); /*ssize_t*/
      bool _match_20164 = (w2 <= n2); /*bool*/;
      if (_match_20164) {
        _x21001 = s2_17016; /*string*/
      }
      else {
        kk_string_t _x21007;
        kk_string_t _x21008 = kk_std_core_string('0', _ctx); /*string*/
        kk_ssize_t _x21009 = (w2 - n2); /*ssize_t*/
        _x21007 = kk_std_core_repeatz(_x21008, _x21009, _ctx); /*string*/
        _x21001 = kk_std_core__lp__plus__plus__1_rp_(_x21007, s2_17016, _ctx); /*string*/
      }
      _x20998 = kk_std_core__lp__plus__plus__1_rp_(_x20999, _x21001, _ctx); /*string*/
      _x20994 = kk_std_core__lp__plus__plus__1_rp_(_x20995, _x20998, _ctx); /*string*/
      return kk_std_core__lp__plus__plus__1_rp_(_x20992, _x20994, _ctx);
    }
    bool _match_20150;
    kk_integer_t _borrowedArg_20162 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_integer_t _borrowedArg_20161 = kk_integer_from_int(65535, _ctx); /*int*/;
    bool _borrowApp_20163 = kk_integer_lte_borrow(_borrowedArg_20162,_borrowedArg_20161,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_20162, _ctx);
    kk_integer_drop(_borrowedArg_20161, _ctx);
    _match_20150 = _borrowApp_20163; /*bool*/
    if (_match_20150) {
      kk_string_t _x21010;
      kk_define_string_literal(, _s21011, 2, "\\u")
      _x21010 = kk_string_dup(_s21011); /*string*/
      kk_string_t _x21012;
      kk_integer_t _arg_79200 = kk_integer_from_int(c,kk_context()); /*int*/;
      kk_string_t _x21013;
      bool _match_20158;
      kk_integer_t _borrowedArg_20159 = kk_integer_from_small(0); /*int*/;
      bool _borrowApp_20160 = kk_integer_lt_borrow(_arg_79200,_borrowedArg_20159,kk_context()); /*bool*/;
      kk_integer_drop(_borrowedArg_20159, _ctx);
      _match_20158 = _borrowApp_20160; /*bool*/
      if (_match_20158) {
        kk_define_string_literal(, _s21014, 1, "-")
        _x21013 = kk_string_dup(_s21014); /*string*/
      }
      else {
        _x21013 = kk_string_empty(); /*string*/
      }
      kk_string_t _x21016;
      kk_string_t _x21017 = kk_string_empty(); /*string*/
      kk_string_t _x21019;
      kk_string_t s3_17023;
      kk_integer_t _x21020 = kk_integer_abs(_arg_79200,kk_context()); /*int*/
      bool _x21021;
      kk_std_core_types__optional _match_20157 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
      if (kk_std_core_types__is_Optional(_match_20157)) {
        kk_box_t _box_x18954 = _match_20157._cons.Optional.value;
        bool _use_capitals_73773 = kk_bool_unbox(_box_x18954);
        kk_std_core_types__optional_drop(_match_20157, _ctx);
        kk_Unit;
        _x21021 = _use_capitals_73773; /*bool*/
        goto _match21022;
      }
      _x21021 = true; /*bool*/
      _match21022: ;
      s3_17023 = kk_std_core_int_show_hex(_x21020, _x21021, _ctx); /*string*/
      kk_ssize_t w3 = ((kk_ssize_t)4); /*ssize_t*/;
      kk_ssize_t n3;
      kk_string_t _x21024 = kk_string_dup(s3_17023); /*string*/
      n3 = kk_string_len(_x21024,kk_context()); /*ssize_t*/
      bool _match_20156 = (w3 <= n3); /*bool*/;
      if (_match_20156) {
        _x21019 = s3_17023; /*string*/
      }
      else {
        kk_string_t _x21025;
        kk_string_t _x21026 = kk_std_core_string('0', _ctx); /*string*/
        kk_ssize_t _x21027 = (w3 - n3); /*ssize_t*/
        _x21025 = kk_std_core_repeatz(_x21026, _x21027, _ctx); /*string*/
        _x21019 = kk_std_core__lp__plus__plus__1_rp_(_x21025, s3_17023, _ctx); /*string*/
      }
      _x21016 = kk_std_core__lp__plus__plus__1_rp_(_x21017, _x21019, _ctx); /*string*/
      _x21012 = kk_std_core__lp__plus__plus__1_rp_(_x21013, _x21016, _ctx); /*string*/
      return kk_std_core__lp__plus__plus__1_rp_(_x21010, _x21012, _ctx);
    }
    kk_string_t _x21028;
    kk_define_string_literal(, _s21029, 2, "\\U")
    _x21028 = kk_string_dup(_s21029); /*string*/
    kk_string_t _x21030;
    kk_integer_t _arg_79610 = kk_integer_from_int(c,kk_context()); /*int*/;
    kk_string_t _x21031;
    bool _match_20153;
    kk_integer_t _borrowedArg_20154 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_20155 = kk_integer_lt_borrow(_arg_79610,_borrowedArg_20154,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_20154, _ctx);
    _match_20153 = _borrowApp_20155; /*bool*/
    if (_match_20153) {
      kk_define_string_literal(, _s21032, 1, "-")
      _x21031 = kk_string_dup(_s21032); /*string*/
    }
    else {
      _x21031 = kk_string_empty(); /*string*/
    }
    kk_string_t _x21034;
    kk_string_t _x21035 = kk_string_empty(); /*string*/
    kk_string_t _x21037;
    kk_string_t s4_17030;
    kk_integer_t _x21038 = kk_integer_abs(_arg_79610,kk_context()); /*int*/
    bool _x21039;
    kk_std_core_types__optional _match_20152 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_20152)) {
      kk_box_t _box_x18955 = _match_20152._cons.Optional.value;
      bool _use_capitals_73774 = kk_bool_unbox(_box_x18955);
      kk_std_core_types__optional_drop(_match_20152, _ctx);
      kk_Unit;
      _x21039 = _use_capitals_73774; /*bool*/
      goto _match21040;
    }
    _x21039 = true; /*bool*/
    _match21040: ;
    s4_17030 = kk_std_core_int_show_hex(_x21038, _x21039, _ctx); /*string*/
    kk_ssize_t w4 = ((kk_ssize_t)6); /*ssize_t*/;
    kk_ssize_t n4;
    kk_string_t _x21042 = kk_string_dup(s4_17030); /*string*/
    n4 = kk_string_len(_x21042,kk_context()); /*ssize_t*/
    bool _match_20151 = (w4 <= n4); /*bool*/;
    if (_match_20151) {
      _x21037 = s4_17030; /*string*/
    }
    else {
      kk_string_t _x21043;
      kk_string_t _x21044 = kk_std_core_string('0', _ctx); /*string*/
      kk_ssize_t _x21045 = (w4 - n4); /*ssize_t*/
      _x21043 = kk_std_core_repeatz(_x21044, _x21045, _ctx); /*string*/
      _x21037 = kk_std_core__lp__plus__plus__1_rp_(_x21043, s4_17030, _ctx); /*string*/
    }
    _x21034 = kk_std_core__lp__plus__plus__1_rp_(_x21035, _x21037, _ctx); /*string*/
    _x21030 = kk_std_core__lp__plus__plus__1_rp_(_x21031, _x21034, _ctx); /*string*/
    return kk_std_core__lp__plus__plus__1_rp_(_x21028, _x21030, _ctx);
  }
  bool _match_20143 = (c == ('\'')); /*bool*/;
  if (_match_20143) {
    kk_define_string_literal(, _s21046, 2, "\\\'")
    return kk_string_dup(_s21046);
  }
  bool _match_20144 = (c == ('"')); /*bool*/;
  if (_match_20144) {
    kk_define_string_literal(, _s21047, 2, "\\\"")
    return kk_string_dup(_s21047);
  }
  bool _match_20145 = (c == ('\\')); /*bool*/;
  if (_match_20145) {
    kk_define_string_literal(, _s21048, 2, "\\\\")
    return kk_string_dup(_s21048);
  }
  return kk_std_core_string(c, _ctx);
}
 
// Show a `:double` fixed-point notation.
// The optional `precision` (= `-2`) specifies the maximum precision.
// If `>=0` it specifies the number of digits behind the dot (up to `20` max).
// If negative, then at most the absolute value of `precision` digits behind the dot are used.
// This may still show a number in exponential notation if the it is too small or large,
// in particular, for  a `d` where `d > 1.0e21` or `d < 1.0e-15`, or if
// `precision.abs > 17`, the `show-exp` routine is used.

kk_string_t kk_std_core_show_fixed(double d, kk_std_core_types__optional precision, kk_context_t* _ctx) { /* (d : double, precision : optional<int>) -> string */ 
  double dabs = kk_double_abs(d); /*double*/;
  bool _match_20139 = (dabs < (1.0e-15)); /*bool*/;
  if (_match_20139) {
    int32_t _x21053;
    kk_integer_t _x21054;
    if (kk_std_core_types__is_Optional(precision)) {
      kk_box_t _box_x18957 = precision._cons.Optional.value;
      kk_integer_t _precision_8050 = kk_integer_unbox(_box_x18957);
      kk_integer_dup(_precision_8050);
      kk_std_core_types__optional_drop(precision, _ctx);
      kk_Unit;
      _x21054 = _precision_8050; /*int*/
      goto _match21055;
    }
    _x21054 = kk_integer_from_small(-2); /*int*/
    _match21055: ;
    _x21053 = kk_std_core_int32(_x21054, _ctx); /*int32*/
    return kk_std_core_show_expx(d, _x21053, _ctx);
  }
  bool _match_20140 = (dabs > (1.0e21)); /*bool*/;
  if (_match_20140) {
    int32_t _x21057;
    kk_integer_t _x21058;
    if (kk_std_core_types__is_Optional(precision)) {
      kk_box_t _box_x18958 = precision._cons.Optional.value;
      kk_integer_t _precision_80500 = kk_integer_unbox(_box_x18958);
      kk_integer_dup(_precision_80500);
      kk_std_core_types__optional_drop(precision, _ctx);
      kk_Unit;
      _x21058 = _precision_80500; /*int*/
      goto _match21059;
    }
    _x21058 = kk_integer_from_small(-2); /*int*/
    _match21059: ;
    _x21057 = kk_std_core_int32(_x21058, _ctx); /*int32*/
    return kk_std_core_show_expx(d, _x21057, _ctx);
  }
  int32_t _x21061;
  kk_integer_t _x21062;
  if (kk_std_core_types__is_Optional(precision)) {
    kk_box_t _box_x18959 = precision._cons.Optional.value;
    kk_integer_t _precision_80501 = kk_integer_unbox(_box_x18959);
    kk_integer_dup(_precision_80501);
    kk_std_core_types__optional_drop(precision, _ctx);
    kk_Unit;
    _x21062 = _precision_80501; /*int*/
    goto _match21063;
  }
  _x21062 = kk_integer_from_small(-2); /*int*/
  _match21063: ;
  _x21061 = kk_std_core_int32(_x21062, _ctx); /*int32*/
  return kk_std_core_show_fixedx(d, _x21061, _ctx);
}
 
// lifted

kk_string_t kk_std_core__lift17176_show_list(kk_string_t sep, kk_std_core__list ys, kk_string_t acc, kk_context_t* _ctx) { /* (sep : string, ys : list<string>, acc : string) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21065 = kk_std_core__as_Cons(ys);
    kk_box_t _box_x18960 = _con21065->head;
    kk_std_core__list yy = _con21065->tail;
    kk_string_t y = kk_string_unbox(_box_x18960);
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_std_core__list_free(ys);
    }
    else {
      kk_string_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_string_t acc0_17230;
    kk_string_t _x21067;
    kk_string_t _x21068 = kk_string_dup(sep); /*string*/
    _x21067 = kk_std_core__lp__plus__plus__1_rp_(_x21068, y, _ctx); /*string*/
    acc0_17230 = kk_std_core__lp__plus__plus__1_rp_(acc, _x21067, _ctx); /*string*/
    { // tailcall
      ys = yy;
      acc = acc0_17230;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sep, _ctx);
  return acc;
}
 
// monadic lift

kk_string_t kk_std_core__mlift17725_show_list(kk_std_core__list _y_17464, kk_context_t* _ctx) { /* forall<e> (list<string>) -> e string */ 
  kk_string_t _x21069;
  kk_define_string_literal(, _s21070, 1, "[")
  _x21069 = kk_string_dup(_s21070); /*string*/
  kk_string_t _x21071;
  kk_string_t _x21072;
  if (kk_std_core__is_Nil(_y_17464)) {
    _x21072 = kk_string_empty(); /*string*/
  }
  else {
    struct kk_std_core_Cons* _con21074 = kk_std_core__as_Cons(_y_17464);
    kk_box_t _box_x18961 = _con21074->head;
    kk_std_core__list xx = _con21074->tail;
    kk_string_t x = kk_string_unbox(_box_x18961);
    if (kk_likely(kk_std_core__list_is_unique(_y_17464))) {
      kk_std_core__list_free(_y_17464);
    }
    else {
      kk_string_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(_y_17464, _ctx);
      kk_Unit;
    }
    kk_string_t _x21076;
    kk_define_string_literal(, _s21077, 1, ",")
    _x21076 = kk_string_dup(_s21077); /*string*/
    _x21072 = kk_std_core__lift17176_show_list(_x21076, xx, x, _ctx); /*string*/
  }
  kk_string_t _x21078;
  kk_define_string_literal(, _s21079, 1, "]")
  _x21078 = kk_string_dup(_s21079); /*string*/
  _x21071 = kk_std_core__lp__plus__plus__1_rp_(_x21072, _x21078, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x21069, _x21071, _ctx);
}
 
// Convert a list to a string


// lift anonymous function
struct kk_std_core_show_list_fun21080__t {
  struct kk_function_s _base;
  kk_function_t show_elem;
};
static kk_box_t kk_std_core_show_list_fun21080(kk_function_t _fself, kk_box_t _b_18964, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_show_list_fun21080(kk_function_t show_elem, kk_context_t* _ctx) {
  struct kk_std_core_show_list_fun21080__t* _self = kk_function_alloc_as(struct kk_std_core_show_list_fun21080__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_show_list_fun21080, kk_context());
  _self->show_elem = show_elem;
  return &_self->_base;
}

static kk_box_t kk_std_core_show_list_fun21080(kk_function_t _fself, kk_box_t _b_18964, kk_context_t* _ctx) {
  struct kk_std_core_show_list_fun21080__t* _self = kk_function_as(struct kk_std_core_show_list_fun21080__t*, _fself);
  kk_function_t show_elem = _self->show_elem; /* (8595) -> 8596 string */
  kk_drop_match(_self, {kk_function_dup(show_elem);}, {}, _ctx)
  kk_string_t _x21081 = kk_function_call(kk_string_t, (kk_function_t, kk_box_t, kk_context_t*), show_elem, (show_elem, _b_18964, _ctx)); /*string*/
  return kk_string_box(_x21081);
}


// lift anonymous function
struct kk_std_core_show_list_fun21082__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_core_show_list_fun21082(kk_function_t _fself, kk_std_core__list _y_17464, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_show_list_fun21082(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_show_list_fun21082, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_core_show_list_fun21082(kk_function_t _fself, kk_std_core__list _y_17464, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_string_t _x21083;
  kk_define_string_literal(, _s21084, 1, "[")
  _x21083 = kk_string_dup(_s21084); /*string*/
  kk_string_t _x21085;
  kk_string_t _x21086;
  if (kk_std_core__is_Nil(_y_17464)) {
    _x21086 = kk_string_empty(); /*string*/
  }
  else {
    struct kk_std_core_Cons* _con21088 = kk_std_core__as_Cons(_y_17464);
    kk_box_t _box_x18967 = _con21088->head;
    kk_std_core__list xx = _con21088->tail;
    kk_string_t x0 = kk_string_unbox(_box_x18967);
    if (kk_likely(kk_std_core__list_is_unique(_y_17464))) {
      kk_std_core__list_free(_y_17464);
    }
    else {
      kk_string_dup(x0);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(_y_17464, _ctx);
      kk_Unit;
    }
    kk_string_t _x21090;
    kk_define_string_literal(, _s21091, 1, ",")
    _x21090 = kk_string_dup(_s21091); /*string*/
    _x21086 = kk_std_core__lift17176_show_list(_x21090, xx, x0, _ctx); /*string*/
  }
  kk_string_t _x21092;
  kk_define_string_literal(, _s21093, 1, "]")
  _x21092 = kk_string_dup(_s21093); /*string*/
  _x21085 = kk_std_core__lp__plus__plus__1_rp_(_x21086, _x21092, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x21083, _x21085, _ctx);
}


// lift anonymous function
struct kk_std_core_show_list_fun21095__t {
  struct kk_function_s _base;
  kk_function_t _b_18971_18968;
};
static kk_box_t kk_std_core_show_list_fun21095(kk_function_t _fself, kk_box_t _b_18969, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_show_list_fun21095(kk_function_t _b_18971_18968, kk_context_t* _ctx) {
  struct kk_std_core_show_list_fun21095__t* _self = kk_function_alloc_as(struct kk_std_core_show_list_fun21095__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_show_list_fun21095, kk_context());
  _self->_b_18971_18968 = _b_18971_18968;
  return &_self->_base;
}

static kk_box_t kk_std_core_show_list_fun21095(kk_function_t _fself, kk_box_t _b_18969, kk_context_t* _ctx) {
  struct kk_std_core_show_list_fun21095__t* _self = kk_function_as(struct kk_std_core_show_list_fun21095__t*, _fself);
  kk_function_t _b_18971_18968 = _self->_b_18971_18968; /* (list<string>) -> 8596 string */
  kk_drop_match(_self, {kk_function_dup(_b_18971_18968);}, {}, _ctx)
  kk_string_t _x21096;
  kk_std_core__list _x21097 = kk_std_core__list_unbox(_b_18969, _ctx); /*list<string>*/
  _x21096 = kk_function_call(kk_string_t, (kk_function_t, kk_std_core__list, kk_context_t*), _b_18971_18968, (_b_18971_18968, _x21097, _ctx)); /*string*/
  return kk_string_box(_x21096);
}

kk_string_t kk_std_core_show_list(kk_std_core__list xs, kk_function_t show_elem, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, show-elem : (a) -> e string) -> e string */ 
  kk_std_core__list x_17999 = kk_std_core_map_5(xs, kk_std_core_new_show_list_fun21080(show_elem, _ctx), _ctx); /*list<string>*/;
  if (kk_yielding(kk_context())) {
    kk_std_core__list_drop(x_17999, _ctx);
    kk_function_t _b_18971_18968 = kk_std_core_new_show_list_fun21082(_ctx); /*(list<string>) -> 8596 string*/;
    kk_box_t _x21094 = kk_std_core_hnd_yield_extend(kk_std_core_new_show_list_fun21095(_b_18971_18968, _ctx), _ctx); /*3858*/
    return kk_string_unbox(_x21094);
  }
  kk_string_t _x21098;
  kk_define_string_literal(, _s21099, 1, "[")
  _x21098 = kk_string_dup(_s21099); /*string*/
  kk_string_t _x21100;
  kk_string_t _x21101;
  if (kk_std_core__is_Nil(x_17999)) {
    _x21101 = kk_string_empty(); /*string*/
  }
  else {
    struct kk_std_core_Cons* _con21103 = kk_std_core__as_Cons(x_17999);
    kk_box_t _box_x18970 = _con21103->head;
    kk_std_core__list xx0 = _con21103->tail;
    kk_string_t x1 = kk_string_unbox(_box_x18970);
    if (kk_likely(kk_std_core__list_is_unique(x_17999))) {
      kk_std_core__list_free(x_17999);
    }
    else {
      kk_string_dup(x1);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(x_17999, _ctx);
      kk_Unit;
    }
    kk_string_t _x21105;
    kk_define_string_literal(, _s21106, 1, ",")
    _x21105 = kk_string_dup(_s21106); /*string*/
    _x21101 = kk_std_core__lift17176_show_list(_x21105, xx0, x1, _ctx); /*string*/
  }
  kk_string_t _x21107;
  kk_define_string_literal(, _s21108, 1, "]")
  _x21107 = kk_string_dup(_s21108); /*string*/
  _x21100 = kk_std_core__lp__plus__plus__1_rp_(_x21101, _x21107, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x21098, _x21100, _ctx);
}
 
// Convert an `:int` to a string

kk_string_t kk_std_core_show(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> string */ 
  return kk_integer_to_string(i,kk_context());
}
 
// Show a `:double` as a string.
// If `d >= 1.0e-5` and `d < 1.0e+21`, `show-fixed` is used and otherwise `show-exp`.
// Default `precision` is `-17`.

kk_string_t kk_std_core_show_1(double d, kk_std_core_types__optional precision, kk_context_t* _ctx) { /* (d : double, precision : optional<int>) -> string */ 
  double dabs = kk_double_abs(d); /*double*/;
  bool _match_20134 = (dabs >= (1.0e-5)); /*bool*/;
  if (_match_20134) {
    bool _match_20135 = (dabs < (1.0e21)); /*bool*/;
    if (_match_20135) {
      double dabs0 = kk_double_abs(d); /*double*/;
      bool _match_20136 = (dabs0 < (1.0e-15)); /*bool*/;
      if (_match_20136) {
        int32_t _x21109;
        kk_integer_t _x21110;
        if (kk_std_core_types__is_Optional(precision)) {
          kk_box_t _box_x18972 = precision._cons.Optional.value;
          kk_integer_t _precision_8605 = kk_integer_unbox(_box_x18972);
          kk_integer_dup(_precision_8605);
          kk_std_core_types__optional_drop(precision, _ctx);
          kk_Unit;
          _x21110 = _precision_8605; /*int*/
          goto _match21111;
        }
        _x21110 = kk_integer_from_small(-17); /*int*/
        _match21111: ;
        _x21109 = kk_std_core_int32(_x21110, _ctx); /*int32*/
        return kk_std_core_show_expx(d, _x21109, _ctx);
      }
      bool _match_20137 = (dabs0 > (1.0e21)); /*bool*/;
      if (_match_20137) {
        int32_t _x21113;
        kk_integer_t _x21114;
        if (kk_std_core_types__is_Optional(precision)) {
          kk_box_t _box_x18973 = precision._cons.Optional.value;
          kk_integer_t _precision_86050 = kk_integer_unbox(_box_x18973);
          kk_integer_dup(_precision_86050);
          kk_std_core_types__optional_drop(precision, _ctx);
          kk_Unit;
          _x21114 = _precision_86050; /*int*/
          goto _match21115;
        }
        _x21114 = kk_integer_from_small(-17); /*int*/
        _match21115: ;
        _x21113 = kk_std_core_int32(_x21114, _ctx); /*int32*/
        return kk_std_core_show_expx(d, _x21113, _ctx);
      }
      int32_t _x21117;
      kk_integer_t _x21118;
      if (kk_std_core_types__is_Optional(precision)) {
        kk_box_t _box_x18974 = precision._cons.Optional.value;
        kk_integer_t _precision_86051 = kk_integer_unbox(_box_x18974);
        kk_integer_dup(_precision_86051);
        kk_std_core_types__optional_drop(precision, _ctx);
        kk_Unit;
        _x21118 = _precision_86051; /*int*/
        goto _match21119;
      }
      _x21118 = kk_integer_from_small(-17); /*int*/
      _match21119: ;
      _x21117 = kk_std_core_int32(_x21118, _ctx); /*int32*/
      return kk_std_core_show_fixedx(d, _x21117, _ctx);
    }
    int32_t _x21121;
    kk_integer_t _x21122;
    if (kk_std_core_types__is_Optional(precision)) {
      kk_box_t _box_x18975 = precision._cons.Optional.value;
      kk_integer_t _precision_86052 = kk_integer_unbox(_box_x18975);
      kk_integer_dup(_precision_86052);
      kk_std_core_types__optional_drop(precision, _ctx);
      kk_Unit;
      _x21122 = _precision_86052; /*int*/
      goto _match21123;
    }
    _x21122 = kk_integer_from_small(-17); /*int*/
    _match21123: ;
    _x21121 = kk_std_core_int32(_x21122, _ctx); /*int32*/
    return kk_std_core_show_expx(d, _x21121, _ctx);
  }
  int32_t _x21125;
  kk_integer_t _x21126;
  if (kk_std_core_types__is_Optional(precision)) {
    kk_box_t _box_x18976 = precision._cons.Optional.value;
    kk_integer_t _precision_86053 = kk_integer_unbox(_box_x18976);
    kk_integer_dup(_precision_86053);
    kk_std_core_types__optional_drop(precision, _ctx);
    kk_Unit;
    _x21126 = _precision_86053; /*int*/
    goto _match21127;
  }
  _x21126 = kk_integer_from_small(-17); /*int*/
  _match21127: ;
  _x21125 = kk_std_core_int32(_x21126, _ctx); /*int32*/
  return kk_std_core_show_expx(d, _x21125, _ctx);
}
 
// lifted
// specialized std/core/map.5 to parameters [f]

kk_std_core__list kk_std_core__ctail_lift17177_show_3(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* (xs : list<char>, ctail<list<string>>) -> list<string> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21135 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x18977 = _con21135->head;
    kk_std_core__list xx = _con21135->tail;
    kk_char_t x = kk_char_unbox(_box_x18977, NULL);
    kk_reuse_t _ru_20362 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20362 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_string_t _ctail_17332 = kk_std_core_show_char(x, _ctx); /*string*/;
    kk_std_core__list _ctail_17333 = kk_std_core__list_hole(); /*list<string>*/;
    kk_std_core__list _ctail_17334 = kk_std_core__new_Cons(_ru_20362, kk_string_box(_ctail_17332), _ctail_17333, _ctx); /*list<string>*/;
    { // tailcall
      kk_std_core_types__ctail _x21137;
      kk_box_t* field_18005 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17334)->tail)); /*cfield<list<string>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x18988 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x18988, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17334, _ctx);
        kk_std_core__list _b_19002_18993 = head0; /*list<string>*/;
        kk_box_t* _b_19003_18994 = field_18005; /*cfield<list<string>>*/;
        _x21137 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19002_18993, _ctx), _b_19003_18994, _ctx); /*ctail<58>*/
        goto _match21138;
      }
      kk_std_core__list _b_19004_18995 = _ctail_17334; /*list<string>*/;
      kk_box_t* _b_19005_18996 = field_18005; /*cfield<list<string>>*/;
      _x21137 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19004_18995, _ctx), _b_19005_18996, _ctx); /*ctail<58>*/
      _match21138: ;
      xs = xx;
      _acc = _x21137;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x18997 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x18997, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted
// specialized std/core/map.5 to parameters [f]

kk_std_core__list kk_std_core__lift17177_show_3(kk_std_core__list xs0, kk_context_t* _ctx) { /* (xs : list<char>) -> list<string> */ 
  return kk_std_core__ctail_lift17177_show_3(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// lifted

kk_string_t kk_std_core__lift17178_show_3(kk_string_t sep_17050, kk_std_core__list ys, kk_string_t acc, kk_context_t* _ctx) { /* (sep.17050 : string, ys : list<string>, acc : string) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21141 = kk_std_core__as_Cons(ys);
    kk_box_t _box_x19006 = _con21141->head;
    kk_std_core__list yy = _con21141->tail;
    kk_string_t y = kk_string_unbox(_box_x19006);
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_std_core__list_free(ys);
    }
    else {
      kk_string_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_string_t acc0_17234;
    kk_string_t _x21143;
    kk_string_t _x21144 = kk_string_dup(sep_17050); /*string*/
    _x21143 = kk_std_core__lp__plus__plus__1_rp_(_x21144, y, _ctx); /*string*/
    acc0_17234 = kk_std_core__lp__plus__plus__1_rp_(acc, _x21143, _ctx); /*string*/
    { // tailcall
      ys = yy;
      acc = acc0_17234;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sep_17050, _ctx);
  return acc;
}
 
// Show a string as a string literal

kk_string_t kk_std_core_show_3(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  kk_string_t _x21145;
  kk_define_string_literal(, _s21146, 1, "\"")
  _x21145 = kk_string_dup(_s21146); /*string*/
  kk_string_t _x21147;
  kk_string_t _x21148;
  kk_std_core__list xs_17048;
  kk_std_core__list _x21149 = kk_std_core_list_6(s, _ctx); /*list<char>*/
  xs_17048 = kk_std_core__lift17177_show_3(_x21149, _ctx); /*list<string>*/
  if (kk_std_core__is_Nil(xs_17048)) {
    _x21148 = kk_string_empty(); /*string*/
  }
  else {
    struct kk_std_core_Cons* _con21151 = kk_std_core__as_Cons(xs_17048);
    kk_box_t _box_x19007 = _con21151->head;
    kk_std_core__list xx0 = _con21151->tail;
    kk_string_t x0 = kk_string_unbox(_box_x19007);
    if (kk_likely(kk_std_core__list_is_unique(xs_17048))) {
      kk_std_core__list_free(xs_17048);
    }
    else {
      kk_string_dup(x0);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs_17048, _ctx);
      kk_Unit;
    }
    kk_string_t _x21153 = kk_string_empty(); /*string*/
    _x21148 = kk_std_core__lift17178_show_3(_x21153, xx0, x0, _ctx); /*string*/
  }
  kk_string_t _x21155;
  kk_define_string_literal(, _s21156, 1, "\"")
  _x21155 = kk_string_dup(_s21156); /*string*/
  _x21147 = kk_std_core__lp__plus__plus__1_rp_(_x21148, _x21155, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x21145, _x21147, _ctx);
}
extern kk_string_t kk_std_core_show_fun21162_7(kk_function_t _fself, kk_string_t _x121161, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_show_3(_x121161, _ctx);
}
extern kk_string_t kk_std_core_show_fun21163_7(kk_function_t _fself, kk_box_t _b_19010, kk_context_t* _ctx) {
  struct kk_std_core_show_fun21163__t_7* _self = kk_function_as(struct kk_std_core_show_fun21163__t_7*, _fself);
  kk_function_t _b_19012_19009 = _self->_b_19012_19009; /* (s : string) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_19012_19009);}, {}, _ctx)
  kk_string_t _x21164 = kk_string_unbox(_b_19010); /*string*/
  return kk_function_call(kk_string_t, (kk_function_t, kk_string_t, kk_context_t*), _b_19012_19009, (_b_19012_19009, _x21164, _ctx));
}
extern kk_string_t kk_std_core_show_fun21166_8(kk_function_t _fself, kk_integer_t _x121165, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_show(_x121165, _ctx);
}
extern kk_string_t kk_std_core_show_fun21167_8(kk_function_t _fself, kk_box_t _b_19015, kk_context_t* _ctx) {
  struct kk_std_core_show_fun21167__t_8* _self = kk_function_as(struct kk_std_core_show_fun21167__t_8*, _fself);
  kk_function_t _b_19017_19014 = _self->_b_19017_19014; /* (i : int) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_19017_19014);}, {}, _ctx)
  kk_integer_t _x21168 = kk_integer_unbox(_b_19015); /*int*/
  return kk_function_call(kk_string_t, (kk_function_t, kk_integer_t, kk_context_t*), _b_19017_19014, (_b_19017_19014, _x21168, _ctx));
}
extern kk_string_t kk_std_core_show_fun21170_9(kk_function_t _fself, bool _x121169, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_show_4(_x121169, _ctx);
}
extern kk_string_t kk_std_core_show_fun21171_9(kk_function_t _fself, kk_box_t _b_19020, kk_context_t* _ctx) {
  struct kk_std_core_show_fun21171__t_9* _self = kk_function_as(struct kk_std_core_show_fun21171__t_9*, _fself);
  kk_function_t _b_19022_19019 = _self->_b_19022_19019; /* (b : bool) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_19022_19019);}, {}, _ctx)
  bool _x21172 = kk_bool_unbox(_b_19020); /*bool*/
  return kk_function_call(kk_string_t, (kk_function_t, bool, kk_context_t*), _b_19022_19019, (_b_19022_19019, _x21172, _ctx));
}

kk_unit_t kk_std_core_prints(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> console () */ 
  kk_std_core_types__maybe _match_20133;
  kk_ref_t _b_19031_19023 = kk_ref_dup(kk_std_core_redirect); /*ref<global,maybe<(string) -> console ()>>*/;
  kk_box_t _x21181 = kk_ref_get(_b_19031_19023,kk_context()); /*182*/
  _match_20133 = kk_std_core_types__maybe_unbox(_x21181, _ctx); /*maybe<(string) -> console ()>*/
  if (kk_std_core_types__is_Nothing(_match_20133)) {
    kk_std_core_xprints(s, _ctx); return kk_Unit;
  }
  kk_box_t _fun_unbox_x19027 = _match_20133._cons.Just.value;
  kk_string_t _b_19030 = s; /*string*/;
  kk_box_t _x21182;
  kk_function_t _x21183 = kk_function_unbox(_fun_unbox_x19027); /*(19028) -> console 19029*/
  _x21182 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x21183, (_x21183, kk_string_box(_b_19030), _ctx)); /*19029*/
  kk_unit_unbox(_x21182); return kk_Unit;
}


// lift anonymous function
struct kk_std_core__default_exn_fun21188__t {
  struct kk_function_s _base;
};
static kk_unit_t kk_std_core__default_exn_fun21188(kk_function_t _fself, kk_std_core__exception exn0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_default_exn_fun21188(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core__default_exn_fun21188, _ctx)
  return kk_function_dup(_fself);
}

static kk_unit_t kk_std_core__default_exn_fun21188(kk_function_t _fself, kk_std_core__exception exn0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_unit_t __ = kk_Unit;
  kk_string_t _x21189;
  kk_define_string_literal(, _s21190, 20, "uncaught exception: ")
  _x21189 = kk_string_dup(_s21190); /*string*/
  kk_std_core_prints(_x21189, _ctx);
  kk_string_t _x21191;
  kk_string_t _x = exn0.message;
  kk_string_dup(_x);
  kk_std_core__exception_drop(exn0, _ctx);
  kk_Unit;
  _x21191 = _x; /*string*/
  return kk_std_core_printsln(_x21191, _ctx);
}


// lift anonymous function
struct kk_std_core__default_exn_fun21192__t {
  struct kk_function_s _base;
  kk_function_t _b_19051_19040;
};
static kk_box_t kk_std_core__default_exn_fun21192(kk_function_t _fself, kk_box_t _b_19041, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_default_exn_fun21192(kk_function_t _b_19051_19040, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun21192__t* _self = kk_function_alloc_as(struct kk_std_core__default_exn_fun21192__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__default_exn_fun21192, kk_context());
  _self->_b_19051_19040 = _b_19051_19040;
  return &_self->_base;
}

static kk_box_t kk_std_core__default_exn_fun21192(kk_function_t _fself, kk_box_t _b_19041, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun21192__t* _self = kk_function_as(struct kk_std_core__default_exn_fun21192__t*, _fself);
  kk_function_t _b_19051_19040 = _self->_b_19051_19040; /* (exn0 : exception) -> <console|9657> () */
  kk_drop_match(_self, {kk_function_dup(_b_19051_19040);}, {}, _ctx)
  kk_unit_t _x21193 = kk_Unit;
  kk_std_core__exception _x21194 = kk_std_core__exception_unbox(_b_19041, _ctx); /*exception*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_std_core__exception, kk_context_t*), _b_19051_19040, (_b_19051_19040, _x21194, _ctx));
  return kk_unit_box(_x21193);
}


// lift anonymous function
struct kk_std_core__default_exn_fun21195__t {
  struct kk_function_s _base;
};
static kk_unit_t kk_std_core__default_exn_fun21195(kk_function_t _fself, kk_unit_t _x0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_default_exn_fun21195(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core__default_exn_fun21195, _ctx)
  return kk_function_dup(_fself);
}

static kk_unit_t kk_std_core__default_exn_fun21195(kk_function_t _fself, kk_unit_t _x0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _x0;
}


// lift anonymous function
struct kk_std_core__default_exn_fun21197__t {
  struct kk_function_s _base;
  kk_function_t _b_19049_19044;
};
static kk_box_t kk_std_core__default_exn_fun21197(kk_function_t _fself, kk_box_t _b_19046, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_default_exn_fun21197(kk_function_t _b_19049_19044, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun21197__t* _self = kk_function_alloc_as(struct kk_std_core__default_exn_fun21197__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__default_exn_fun21197, kk_context());
  _self->_b_19049_19044 = _b_19049_19044;
  return &_self->_base;
}

static kk_box_t kk_std_core__default_exn_fun21197(kk_function_t _fself, kk_box_t _b_19046, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun21197__t* _self = kk_function_as(struct kk_std_core__default_exn_fun21197__t*, _fself);
  kk_function_t _b_19049_19044 = _self->_b_19049_19044; /* (()) -> <console|9657> () */
  kk_drop_match(_self, {kk_function_dup(_b_19049_19044);}, {}, _ctx)
  kk_unit_t _x21198 = kk_Unit;
  kk_unit_t _x21199 = kk_Unit;
  kk_unit_unbox(_b_19046);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19049_19044, (_b_19049_19044, _x21199, _ctx));
  return kk_unit_box(_x21198);
}


// lift anonymous function
struct kk_std_core__default_exn_fun21200__t {
  struct kk_function_s _base;
  kk_function_t _b_19050_19045;
};
static kk_box_t kk_std_core__default_exn_fun21200(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_default_exn_fun21200(kk_function_t _b_19050_19045, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun21200__t* _self = kk_function_alloc_as(struct kk_std_core__default_exn_fun21200__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__default_exn_fun21200, kk_context());
  _self->_b_19050_19045 = _b_19050_19045;
  return &_self->_base;
}

static kk_box_t kk_std_core__default_exn_fun21200(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_core__default_exn_fun21200__t* _self = kk_function_as(struct kk_std_core__default_exn_fun21200__t*, _fself);
  kk_function_t _b_19050_19045 = _self->_b_19050_19045; /* () -> <exn,console|9657> () */
  kk_drop_match(_self, {kk_function_dup(_b_19050_19045);}, {}, _ctx)
  kk_unit_t _x21201 = kk_Unit;
  kk_function_call(kk_unit_t, (kk_function_t, kk_context_t*), _b_19050_19045, (_b_19050_19045, _ctx));
  return kk_unit_box(_x21201);
}

kk_unit_t kk_std_core__default_exn(kk_function_t action, kk_context_t* _ctx) { /* forall<e> (action : () -> <console,exn|e> ()) -> <console|e> () */ 
  int32_t _b_19047_19042 = ((int32_t)0); /*int32*/;
  kk_std_core__hnd_exn _b_19048_19043;
  kk_std_core_hnd__clause1 _x21187;
  kk_function_t _b_19051_19040 = kk_std_core__new_default_exn_fun21188(_ctx); /*(exn0 : exception) -> <console|9657> ()*/;
  _x21187 = kk_std_core_hnd_clause_never1(kk_std_core__new_default_exn_fun21192(_b_19051_19040, _ctx), _ctx); /*std/core/hnd/clause1<8108,8109,8111,8110,8112>*/
  _b_19048_19043 = kk_std_core__new_Hnd_exn(kk_reuse_null, _x21187, _ctx); /*.hnd-exn<<console|9657>,()>*/
  kk_function_t _b_19049_19044 = kk_std_core__new_default_exn_fun21195(_ctx); /*(()) -> <console|9657> ()*/;
  kk_function_t _b_19050_19045 = action; /*() -> <exn,console|9657> ()*/;
  kk_box_t _x21196 = kk_std_core__handle_exn(_b_19047_19042, _b_19048_19043, kk_std_core__new_default_exn_fun21197(_b_19049_19044, _ctx), kk_std_core__new_default_exn_fun21200(_b_19050_19045, _ctx), _ctx); /*1957*/
  kk_unit_unbox(_x21196); return kk_Unit;
}
 
// Get (zero-based) element `n`  of a list. Return a `:maybe` type.

kk_std_core_types__maybe kk_std_core__lp__lb__rb__2_rp_(kk_std_core__list xs, kk_integer_t n, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int) -> maybe<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21202 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21202->head;
    kk_std_core__list xx = _con21202->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    bool _match_20125;
    kk_integer_t _borrowedArg_20129 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_20130 = kk_integer_gt_borrow(n,_borrowedArg_20129,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_20129, _ctx);
    _match_20125 = _borrowApp_20130; /*bool*/
    if (_match_20125) {
      kk_box_drop(x, _ctx);
      { // tailcall
        kk_integer_t _x21203 = kk_integer_sub(n,(kk_integer_from_small(1)),kk_context()); /*int*/
        xs = xx;
        n = _x21203;
        goto kk__tailcall;
      }
    }
    kk_std_core__list_drop(xx, _ctx);
    bool _match_20126;
    kk_integer_t _borrowedArg_20127 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_20128 = kk_integer_eq_borrow(n,_borrowedArg_20127,kk_context()); /*bool*/;
    kk_integer_drop(n, _ctx);
    kk_integer_drop(_borrowedArg_20127, _ctx);
    _match_20126 = _borrowApp_20128; /*bool*/
    if (_match_20126) {
      return kk_std_core_types__new_Just(x, _ctx);
    }
    kk_box_drop(x, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_integer_drop(n, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// O(1). Return the string slice from the end of `slice` argument
// to the end of the string.

kk_std_core__sslice kk_std_core_after(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> sslice */ 
  kk_string_t s = slice0.str;
  kk_ssize_t start0 = slice0.start;
  kk_ssize_t len0 = slice0.len;
  kk_string_dup(s);
  kk_std_core__sslice_drop(slice0, _ctx);
  kk_Unit;
  kk_string_t _x21204 = kk_string_dup(s); /*string*/
  kk_ssize_t _x21205 = (start0 + len0); /*ssize_t*/
  kk_ssize_t _x21206;
  kk_ssize_t _x21207 = kk_string_len(s,kk_context()); /*ssize_t*/
  kk_ssize_t _x21208 = (start0 + len0); /*ssize_t*/
  _x21206 = (_x21207 - _x21208); /*ssize_t*/
  return kk_std_core__new_Sslice(_x21204, _x21205, _x21206, _ctx);
}
 
// monadic lift

bool kk_std_core__mlift17726_all(kk_function_t predicate, kk_std_core__list xx, bool _y_17467, kk_context_t* _ctx) { /* forall<a,e> (predicate : (a) -> e bool, xx : list<a>, bool) -> e bool */ 
  if (_y_17467) {
    return kk_std_core_all(xx, predicate, _ctx);
  }
  kk_function_drop(predicate, _ctx);
  kk_std_core__list_drop(xx, _ctx);
  return false;
}
 
// Do all elements satisfy a predicate ?


// lift anonymous function
struct kk_std_core_all_fun21211__t {
  struct kk_function_s _base;
  kk_function_t predicate0;
  kk_std_core__list xx0;
};
static bool kk_std_core_all_fun21211(kk_function_t _fself, bool _y_174670, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_all_fun21211(kk_function_t predicate0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_all_fun21211__t* _self = kk_function_alloc_as(struct kk_std_core_all_fun21211__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_all_fun21211, kk_context());
  _self->predicate0 = predicate0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static bool kk_std_core_all_fun21211(kk_function_t _fself, bool _y_174670, kk_context_t* _ctx) {
  struct kk_std_core_all_fun21211__t* _self = kk_function_as(struct kk_std_core_all_fun21211__t*, _fself);
  kk_function_t predicate0 = _self->predicate0; /* (9894) -> 9895 bool */
  kk_std_core__list xx0 = _self->xx0; /* list<9894> */
  kk_drop_match(_self, {kk_function_dup(predicate0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17726_all(predicate0, xx0, _y_174670, _ctx);
}


// lift anonymous function
struct kk_std_core_all_fun21213__t {
  struct kk_function_s _base;
  kk_function_t _b_19054_19052;
};
static kk_box_t kk_std_core_all_fun21213(kk_function_t _fself, kk_box_t _b_19053, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_all_fun21213(kk_function_t _b_19054_19052, kk_context_t* _ctx) {
  struct kk_std_core_all_fun21213__t* _self = kk_function_alloc_as(struct kk_std_core_all_fun21213__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_all_fun21213, kk_context());
  _self->_b_19054_19052 = _b_19054_19052;
  return &_self->_base;
}

static kk_box_t kk_std_core_all_fun21213(kk_function_t _fself, kk_box_t _b_19053, kk_context_t* _ctx) {
  struct kk_std_core_all_fun21213__t* _self = kk_function_as(struct kk_std_core_all_fun21213__t*, _fself);
  kk_function_t _b_19054_19052 = _self->_b_19054_19052; /* (bool) -> 9895 bool */
  kk_drop_match(_self, {kk_function_dup(_b_19054_19052);}, {}, _ctx)
  bool _x21214;
  bool _x21215 = kk_bool_unbox(_b_19053); /*bool*/
  _x21214 = kk_function_call(bool, (kk_function_t, bool, kk_context_t*), _b_19054_19052, (_b_19054_19052, _x21215, _ctx)); /*bool*/
  return kk_bool_box(_x21214);
}

bool kk_std_core_all(kk_std_core__list xs, kk_function_t predicate0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e bool */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21209 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21209->head;
    kk_std_core__list xx0 = _con21209->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    bool x0_18011;
    kk_function_t _x21210 = kk_function_dup(predicate0); /*(9894) -> 9895 bool*/
    x0_18011 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21210, (_x21210, x, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19054_19052 = kk_std_core_new_all_fun21211(predicate0, xx0, _ctx); /*(bool) -> 9895 bool*/;
      kk_box_t _x21212 = kk_std_core_hnd_yield_extend(kk_std_core_new_all_fun21213(_b_19054_19052, _ctx), _ctx); /*3858*/
      return kk_bool_unbox(_x21212);
    }
    if (x0_18011) {
      { // tailcall
        xs = xx0;
        goto kk__tailcall;
      }
    }
    kk_function_drop(predicate0, _ctx);
    kk_std_core__list_drop(xx0, _ctx);
    return false;
  }
  kk_function_drop(predicate0, _ctx);
  return true;
}
 
// monadic lift

bool kk_std_core__mlift17727_any(kk_function_t predicate, kk_std_core__list xx, bool _y_17471, kk_context_t* _ctx) { /* forall<a,e> (predicate : (a) -> e bool, xx : list<a>, bool) -> e bool */ 
  if (_y_17471) {
    kk_function_drop(predicate, _ctx);
    kk_std_core__list_drop(xx, _ctx);
    return true;
  }
  return kk_std_core_any(xx, predicate, _ctx);
}
 
// Are there any elements in a list that satisfy a predicate ?


// lift anonymous function
struct kk_std_core_any_fun21218__t {
  struct kk_function_s _base;
  kk_function_t predicate0;
  kk_std_core__list xx0;
};
static bool kk_std_core_any_fun21218(kk_function_t _fself, bool _y_174710, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_any_fun21218(kk_function_t predicate0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_any_fun21218__t* _self = kk_function_alloc_as(struct kk_std_core_any_fun21218__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_any_fun21218, kk_context());
  _self->predicate0 = predicate0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static bool kk_std_core_any_fun21218(kk_function_t _fself, bool _y_174710, kk_context_t* _ctx) {
  struct kk_std_core_any_fun21218__t* _self = kk_function_as(struct kk_std_core_any_fun21218__t*, _fself);
  kk_function_t predicate0 = _self->predicate0; /* (9922) -> 9923 bool */
  kk_std_core__list xx0 = _self->xx0; /* list<9922> */
  kk_drop_match(_self, {kk_function_dup(predicate0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17727_any(predicate0, xx0, _y_174710, _ctx);
}


// lift anonymous function
struct kk_std_core_any_fun21220__t {
  struct kk_function_s _base;
  kk_function_t _b_19057_19055;
};
static kk_box_t kk_std_core_any_fun21220(kk_function_t _fself, kk_box_t _b_19056, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_any_fun21220(kk_function_t _b_19057_19055, kk_context_t* _ctx) {
  struct kk_std_core_any_fun21220__t* _self = kk_function_alloc_as(struct kk_std_core_any_fun21220__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_any_fun21220, kk_context());
  _self->_b_19057_19055 = _b_19057_19055;
  return &_self->_base;
}

static kk_box_t kk_std_core_any_fun21220(kk_function_t _fself, kk_box_t _b_19056, kk_context_t* _ctx) {
  struct kk_std_core_any_fun21220__t* _self = kk_function_as(struct kk_std_core_any_fun21220__t*, _fself);
  kk_function_t _b_19057_19055 = _self->_b_19057_19055; /* (bool) -> 9923 bool */
  kk_drop_match(_self, {kk_function_dup(_b_19057_19055);}, {}, _ctx)
  bool _x21221;
  bool _x21222 = kk_bool_unbox(_b_19056); /*bool*/
  _x21221 = kk_function_call(bool, (kk_function_t, bool, kk_context_t*), _b_19057_19055, (_b_19057_19055, _x21222, _ctx)); /*bool*/
  return kk_bool_box(_x21221);
}

bool kk_std_core_any(kk_std_core__list xs, kk_function_t predicate0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e bool */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21216 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21216->head;
    kk_std_core__list xx0 = _con21216->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    bool x0_18014;
    kk_function_t _x21217 = kk_function_dup(predicate0); /*(9922) -> 9923 bool*/
    x0_18014 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21217, (_x21217, x, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19057_19055 = kk_std_core_new_any_fun21218(predicate0, xx0, _ctx); /*(bool) -> 9923 bool*/;
      kk_box_t _x21219 = kk_std_core_hnd_yield_extend(kk_std_core_new_any_fun21220(_b_19057_19055, _ctx), _ctx); /*3858*/
      return kk_bool_unbox(_x21219);
    }
    if (x0_18014) {
      kk_function_drop(predicate0, _ctx);
      kk_std_core__list_drop(xx0, _ctx);
      return true;
    }
    { // tailcall
      xs = xx0;
      goto kk__tailcall;
    }
  }
  kk_function_drop(predicate0, _ctx);
  return false;
}
 
// Return the element at position `index` in vector `v`, or `Nothing` if out of bounds

kk_std_core_types__maybe kk_std_core_at(kk_vector_t v, kk_integer_t index, kk_context_t* _ctx) { /* forall<a> (v : vector<a>, index : int) -> maybe<a> */ 
  kk_ssize_t idx;
  kk_integer_t _x21223 = kk_integer_dup(index); /*int*/
  idx = kk_std_core_ssize__t(_x21223, _ctx); /*ssize_t*/
  bool _match_20122;
  kk_ssize_t _x21224 = kk_vector_len_borrow(v); /*ssize_t*/
  _match_20122 = (_x21224 <= idx); /*bool*/
  if (_match_20122) {
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_box_t _x21225 = kk_vector_at_borrow(v, idx); /*219*/
  return kk_std_core_types__new_Just(_x21225, _ctx);
}
 
// O(`n`). The first `n` (default = `1`) characters in a string.

kk_std_core__sslice kk_std_core_first(kk_string_t s, kk_std_core_types__optional n, kk_context_t* _ctx) { /* (s : string, n : optional<int>) -> sslice */ 
  kk_std_core__sslice slice0 = kk_std_core_first1(s, _ctx); /*sslice*/;
  bool _match_20114;
  kk_integer_t _borrowedArg_20118;
  if (kk_std_core_types__is_Optional(n)) {
    kk_box_t _box_x19058 = n._cons.Optional.value;
    kk_integer_t _n_10065 = kk_integer_unbox(_box_x19058);
    kk_integer_dup(_n_10065);
    _borrowedArg_20118 = _n_10065; /*int*/
    goto _match21229;
  }
  _borrowedArg_20118 = kk_integer_from_small(1); /*int*/
  _match21229: ;
  kk_integer_t _borrowedArg_20117 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_20119 = kk_integer_eq_borrow(_borrowedArg_20118,_borrowedArg_20117,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20118, _ctx);
  kk_integer_drop(_borrowedArg_20117, _ctx);
  _match_20114 = _borrowApp_20119; /*bool*/
  if (_match_20114) {
    kk_std_core_types__optional_drop(n, _ctx);
    return slice0;
  }
  kk_integer_t _borrowedArg_20115;
  kk_integer_t _x21231;
  if (kk_std_core_types__is_Optional(n)) {
    kk_box_t _box_x19059 = n._cons.Optional.value;
    kk_integer_t _n_100650 = kk_integer_unbox(_box_x19059);
    kk_integer_dup(_n_100650);
    kk_std_core_types__optional_drop(n, _ctx);
    kk_Unit;
    _x21231 = _n_100650; /*int*/
    goto _match21232;
  }
  _x21231 = kk_integer_from_small(1); /*int*/
  _match21232: ;
  _borrowedArg_20115 = kk_integer_sub(_x21231,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_std_core__sslice _borrowApp_20116 = kk_std_core_extend(slice0, _borrowedArg_20115, _ctx); /*sslice*/;
  kk_integer_drop(_borrowedArg_20115, _ctx);
  return _borrowApp_20116;
}
 
// Convert the first character of a string to uppercase.

kk_string_t kk_std_core_capitalize(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  kk_string_t _x21234;
  kk_string_t _x21235;
  kk_std_core__sslice _x21236;
  kk_std_core__sslice slice0;
  kk_string_t _x21237 = kk_string_dup(s); /*string*/
  slice0 = kk_std_core_first1(_x21237, _ctx); /*sslice*/
  bool _match_20106;
  kk_integer_t _borrowedArg_20111;
  kk_std_core_types__optional _match_20112 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_20112)) {
    kk_box_t _box_x19060 = _match_20112._cons.Optional.value;
    kk_integer_t _n_10065 = kk_integer_unbox(_box_x19060);
    kk_integer_dup(_n_10065);
    kk_std_core_types__optional_drop(_match_20112, _ctx);
    kk_Unit;
    _borrowedArg_20111 = _n_10065; /*int*/
    goto _match21238;
  }
  _borrowedArg_20111 = kk_integer_from_small(1); /*int*/
  _match21238: ;
  kk_integer_t _borrowedArg_20110 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_20113 = kk_integer_eq_borrow(_borrowedArg_20111,_borrowedArg_20110,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20111, _ctx);
  kk_integer_drop(_borrowedArg_20110, _ctx);
  _match_20106 = _borrowApp_20113; /*bool*/
  if (_match_20106) {
    _x21236 = slice0; /*sslice*/
  }
  else {
    kk_integer_t _borrowedArg_20107;
    kk_integer_t _x21240;
    kk_std_core_types__optional _match_20108 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_20108)) {
      kk_box_t _box_x19061 = _match_20108._cons.Optional.value;
      kk_integer_t _n_100650 = kk_integer_unbox(_box_x19061);
      kk_integer_dup(_n_100650);
      kk_std_core_types__optional_drop(_match_20108, _ctx);
      kk_Unit;
      _x21240 = _n_100650; /*int*/
      goto _match21241;
    }
    _x21240 = kk_integer_from_small(1); /*int*/
    _match21241: ;
    _borrowedArg_20107 = kk_integer_sub(_x21240,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__sslice _borrowApp_20109 = kk_std_core_extend(slice0, _borrowedArg_20107, _ctx); /*sslice*/;
    kk_integer_drop(_borrowedArg_20107, _ctx);
    _x21236 = _borrowApp_20109; /*sslice*/
  }
  _x21235 = kk_std_core_string_3(_x21236, _ctx); /*string*/
  _x21234 = kk_std_core_to_upper(_x21235, _ctx); /*string*/
  kk_string_t _x21243;
  kk_std_core__sslice _x21244;
  kk_std_core__sslice slice2 = kk_std_core_first1(s, _ctx); /*sslice*/;
  kk_std_core__sslice slice1_17057;
  bool _match_20098;
  kk_integer_t _borrowedArg_20103;
  kk_std_core_types__optional _match_20104 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_20104)) {
    kk_box_t _box_x19062 = _match_20104._cons.Optional.value;
    kk_integer_t _n_1006500 = kk_integer_unbox(_box_x19062);
    kk_integer_dup(_n_1006500);
    kk_std_core_types__optional_drop(_match_20104, _ctx);
    kk_Unit;
    _borrowedArg_20103 = _n_1006500; /*int*/
    goto _match21245;
  }
  _borrowedArg_20103 = kk_integer_from_small(1); /*int*/
  _match21245: ;
  kk_integer_t _borrowedArg_20102 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_20105 = kk_integer_eq_borrow(_borrowedArg_20103,_borrowedArg_20102,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20103, _ctx);
  kk_integer_drop(_borrowedArg_20102, _ctx);
  _match_20098 = _borrowApp_20105; /*bool*/
  if (_match_20098) {
    slice1_17057 = slice2; /*sslice*/
  }
  else {
    kk_integer_t _borrowedArg_20099;
    kk_integer_t _x21247;
    kk_std_core_types__optional _match_20100 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_20100)) {
      kk_box_t _box_x19063 = _match_20100._cons.Optional.value;
      kk_integer_t _n_1006501 = kk_integer_unbox(_box_x19063);
      kk_integer_dup(_n_1006501);
      kk_std_core_types__optional_drop(_match_20100, _ctx);
      kk_Unit;
      _x21247 = _n_1006501; /*int*/
      goto _match21248;
    }
    _x21247 = kk_integer_from_small(1); /*int*/
    _match21248: ;
    _borrowedArg_20099 = kk_integer_sub(_x21247,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__sslice _borrowApp_20101 = kk_std_core_extend(slice2, _borrowedArg_20099, _ctx); /*sslice*/;
    kk_integer_drop(_borrowedArg_20099, _ctx);
    slice1_17057 = _borrowApp_20101; /*sslice*/
  }
  kk_string_t s1 = slice1_17057.str;
  kk_ssize_t start0 = slice1_17057.start;
  kk_ssize_t len0 = slice1_17057.len;
  kk_string_dup(s1);
  kk_std_core__sslice_drop(slice1_17057, _ctx);
  kk_Unit;
  kk_string_t _x21250 = kk_string_dup(s1); /*string*/
  kk_ssize_t _x21251 = (start0 + len0); /*ssize_t*/
  kk_ssize_t _x21252;
  kk_ssize_t _x21253 = kk_string_len(s1,kk_context()); /*ssize_t*/
  kk_ssize_t _x21254 = (start0 + len0); /*ssize_t*/
  _x21252 = (_x21253 - _x21254); /*ssize_t*/
  _x21244 = kk_std_core__new_Sslice(_x21250, _x21251, _x21252, _ctx); /*sslice*/
  _x21243 = kk_std_core_string_3(_x21244, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x21234, _x21243, _ctx);
}
 
// Catch any exception raised in `action` and handle it.
// Use `on-exn` or `on-exit` when appropiate.


// lift anonymous function
struct kk_std_core_try_fun21257__t {
  struct kk_function_s _base;
  kk_function_t _b_19066_19064;
};
static kk_box_t kk_std_core_try_fun21257(kk_function_t _fself, kk_box_t _b_19065, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun21257(kk_function_t _b_19066_19064, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21257__t* _self = kk_function_alloc_as(struct kk_std_core_try_fun21257__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun21257, kk_context());
  _self->_b_19066_19064 = _b_19066_19064;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_fun21257(kk_function_t _fself, kk_box_t _b_19065, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21257__t* _self = kk_function_as(struct kk_std_core_try_fun21257__t*, _fself);
  kk_function_t _b_19066_19064 = _self->_b_19066_19064; /* (exception) -> 10232 10231 */
  kk_drop_match(_self, {kk_function_dup(_b_19066_19064);}, {}, _ctx)
  kk_std_core__exception _x21258 = kk_std_core__exception_unbox(_b_19065, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core__exception, kk_context_t*), _b_19066_19064, (_b_19066_19064, _x21258, _ctx));
}


// lift anonymous function
struct kk_std_core_try_fun21259__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_try_fun21259(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun21259(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_try_fun21259, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_try_fun21259(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _x;
}

kk_box_t kk_std_core_try(kk_function_t action, kk_function_t hndl, kk_context_t* _ctx) { /* forall<a,e> (action : () -> <exn|e> a, hndl : (exception) -> e a) -> e a */ 
  kk_std_core__hnd_exn _x21255;
  kk_std_core_hnd__clause1 _x21256;
  kk_function_t _b_19066_19064 = hndl; /*(exception) -> 10232 10231*/;
  _x21256 = kk_std_core_hnd_clause_never1(kk_std_core_new_try_fun21257(_b_19066_19064, _ctx), _ctx); /*std/core/hnd/clause1<8108,8109,8111,8110,8112>*/
  _x21255 = kk_std_core__new_Hnd_exn(kk_reuse_null, _x21256, _ctx); /*.hnd-exn<11,12>*/
  return kk_std_core__handle_exn(((int32_t)0), _x21255, kk_std_core_new_try_fun21259(_ctx), action, _ctx);
}
 
// Transform an exception effect to an  `:error` type.


// lift anonymous function
struct kk_std_core_try_fun21261__t_1 {
  struct kk_function_s _base;
};
static kk_std_core__error kk_std_core_try_fun21261_1(kk_function_t _fself, kk_std_core__exception exn0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun21261_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_try_fun21261_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__error kk_std_core_try_fun21261_1(kk_function_t _fself, kk_std_core__exception exn0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core__new_Error(exn0, _ctx);
}


// lift anonymous function
struct kk_std_core_try_fun21262__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19080_19067;
};
static kk_box_t kk_std_core_try_fun21262_1(kk_function_t _fself, kk_box_t _b_19068, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun21262_1(kk_function_t _b_19080_19067, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21262__t_1* _self = kk_function_alloc_as(struct kk_std_core_try_fun21262__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun21262_1, kk_context());
  _self->_b_19080_19067 = _b_19080_19067;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_fun21262_1(kk_function_t _fself, kk_box_t _b_19068, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21262__t_1* _self = kk_function_as(struct kk_std_core_try_fun21262__t_1*, _fself);
  kk_function_t _b_19080_19067 = _self->_b_19080_19067; /* (exn0 : exception) -> 10271 error<10270> */
  kk_drop_match(_self, {kk_function_dup(_b_19080_19067);}, {}, _ctx)
  kk_std_core__error _x21263;
  kk_std_core__exception _x21264 = kk_std_core__exception_unbox(_b_19068, _ctx); /*exception*/
  _x21263 = kk_function_call(kk_std_core__error, (kk_function_t, kk_std_core__exception, kk_context_t*), _b_19080_19067, (_b_19080_19067, _x21264, _ctx)); /*error<10270>*/
  return kk_std_core__error_box(_x21263, _ctx);
}


// lift anonymous function
struct kk_std_core_try_fun21265__t_1 {
  struct kk_function_s _base;
};
static kk_std_core__error kk_std_core_try_fun21265_1(kk_function_t _fself, kk_std_core__error _x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun21265_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_try_fun21265_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__error kk_std_core_try_fun21265_1(kk_function_t _fself, kk_std_core__error _x, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _x;
}


// lift anonymous function
struct kk_std_core_try_fun21266__t_1 {
  struct kk_function_s _base;
  kk_function_t action;
};
static kk_std_core__error kk_std_core_try_fun21266_1(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun21266_1(kk_function_t action, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21266__t_1* _self = kk_function_alloc_as(struct kk_std_core_try_fun21266__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun21266_1, kk_context());
  _self->action = action;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_core_try_fun21267__t_1 {
  struct kk_function_s _base;
};
static kk_std_core__error kk_std_core_try_fun21267_1(kk_function_t _fself, kk_box_t _y_17476, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun21267_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_try_fun21267_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__error kk_std_core_try_fun21267_1(kk_function_t _fself, kk_box_t _y_17476, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core__new_Ok(_y_17476, _ctx);
}


// lift anonymous function
struct kk_std_core_try_fun21269__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19081_19069;
};
static kk_box_t kk_std_core_try_fun21269_1(kk_function_t _fself, kk_box_t _b_19070, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun21269_1(kk_function_t _b_19081_19069, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21269__t_1* _self = kk_function_alloc_as(struct kk_std_core_try_fun21269__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun21269_1, kk_context());
  _self->_b_19081_19069 = _b_19081_19069;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_fun21269_1(kk_function_t _fself, kk_box_t _b_19070, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21269__t_1* _self = kk_function_as(struct kk_std_core_try_fun21269__t_1*, _fself);
  kk_function_t _b_19081_19069 = _self->_b_19081_19069; /* (10270) -> <exn|10271> error<10270> */
  kk_drop_match(_self, {kk_function_dup(_b_19081_19069);}, {}, _ctx)
  kk_std_core__error _x21270 = kk_function_call(kk_std_core__error, (kk_function_t, kk_box_t, kk_context_t*), _b_19081_19069, (_b_19081_19069, _b_19070, _ctx)); /*error<10270>*/
  return kk_std_core__error_box(_x21270, _ctx);
}
static kk_std_core__error kk_std_core_try_fun21266_1(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21266__t_1* _self = kk_function_as(struct kk_std_core_try_fun21266__t_1*, _fself);
  kk_function_t action = _self->action; /* () -> <exn|10271> 10270 */
  kk_drop_match(_self, {kk_function_dup(action);}, {}, _ctx)
  kk_box_t x_18017 = kk_function_call(kk_box_t, (kk_function_t, kk_context_t*), action, (action, _ctx)); /*10270*/;
  if (kk_yielding(kk_context())) {
    kk_box_drop(x_18017, _ctx);
    kk_function_t _b_19081_19069 = kk_std_core_new_try_fun21267_1(_ctx); /*(10270) -> <exn|10271> error<10270>*/;
    kk_box_t _x21268 = kk_std_core_hnd_yield_extend(kk_std_core_new_try_fun21269_1(_b_19081_19069, _ctx), _ctx); /*3858*/
    return kk_std_core__error_unbox(_x21268, _ctx);
  }
  return kk_std_core__new_Ok(x_18017, _ctx);
}


// lift anonymous function
struct kk_std_core_try_fun21272__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19078_19073;
};
static kk_box_t kk_std_core_try_fun21272_1(kk_function_t _fself, kk_box_t _b_19075, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun21272_1(kk_function_t _b_19078_19073, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21272__t_1* _self = kk_function_alloc_as(struct kk_std_core_try_fun21272__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun21272_1, kk_context());
  _self->_b_19078_19073 = _b_19078_19073;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_fun21272_1(kk_function_t _fself, kk_box_t _b_19075, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21272__t_1* _self = kk_function_as(struct kk_std_core_try_fun21272__t_1*, _fself);
  kk_function_t _b_19078_19073 = _self->_b_19078_19073; /* (error<10270>) -> 10271 error<10270> */
  kk_drop_match(_self, {kk_function_dup(_b_19078_19073);}, {}, _ctx)
  kk_std_core__error _x21273;
  kk_std_core__error _x21274 = kk_std_core__error_unbox(_b_19075, _ctx); /*error<10270>*/
  _x21273 = kk_function_call(kk_std_core__error, (kk_function_t, kk_std_core__error, kk_context_t*), _b_19078_19073, (_b_19078_19073, _x21274, _ctx)); /*error<10270>*/
  return kk_std_core__error_box(_x21273, _ctx);
}


// lift anonymous function
struct kk_std_core_try_fun21275__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19079_19074;
};
static kk_box_t kk_std_core_try_fun21275_1(kk_function_t _fself, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_fun21275_1(kk_function_t _b_19079_19074, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21275__t_1* _self = kk_function_alloc_as(struct kk_std_core_try_fun21275__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_fun21275_1, kk_context());
  _self->_b_19079_19074 = _b_19079_19074;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_fun21275_1(kk_function_t _fself, kk_context_t* _ctx) {
  struct kk_std_core_try_fun21275__t_1* _self = kk_function_as(struct kk_std_core_try_fun21275__t_1*, _fself);
  kk_function_t _b_19079_19074 = _self->_b_19079_19074; /* () -> <exn|10271> error<10270> */
  kk_drop_match(_self, {kk_function_dup(_b_19079_19074);}, {}, _ctx)
  kk_std_core__error _x21276 = kk_function_call(kk_std_core__error, (kk_function_t, kk_context_t*), _b_19079_19074, (_b_19079_19074, _ctx)); /*error<10270>*/
  return kk_std_core__error_box(_x21276, _ctx);
}

kk_std_core__error kk_std_core_try_1(kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (action : () -> <exn|e> a) -> e error<a> */ 
  int32_t _b_19076_19071 = ((int32_t)0); /*int32*/;
  kk_std_core__hnd_exn _b_19077_19072;
  kk_std_core_hnd__clause1 _x21260;
  kk_function_t _b_19080_19067 = kk_std_core_new_try_fun21261_1(_ctx); /*(exn0 : exception) -> 10271 error<10270>*/;
  _x21260 = kk_std_core_hnd_clause_never1(kk_std_core_new_try_fun21262_1(_b_19080_19067, _ctx), _ctx); /*std/core/hnd/clause1<8108,8109,8111,8110,8112>*/
  _b_19077_19072 = kk_std_core__new_Hnd_exn(kk_reuse_null, _x21260, _ctx); /*.hnd-exn<10271,error<10270>>*/
  kk_function_t _b_19078_19073 = kk_std_core_new_try_fun21265_1(_ctx); /*(error<10270>) -> 10271 error<10270>*/;
  kk_function_t _b_19079_19074 = kk_std_core_new_try_fun21266_1(action, _ctx); /*() -> <exn|10271> error<10270>*/;
  kk_box_t _x21271 = kk_std_core__handle_exn(_b_19076_19071, _b_19077_19072, kk_std_core_new_try_fun21272_1(_b_19078_19073, _ctx), kk_std_core_new_try_fun21275_1(_b_19079_19074, _ctx), _ctx); /*1957*/
  return kk_std_core__error_unbox(_x21271, _ctx);
}
 
// _Deprecated_; use `try` instead. Catch an exception raised by `throw` and handle it.
// Use `on-exn` or `on-exit` when appropiate.


// lift anonymous function
struct kk_std_core_catch_fun21279__t {
  struct kk_function_s _base;
  kk_function_t _b_19084_19082;
};
static kk_box_t kk_std_core_catch_fun21279(kk_function_t _fself, kk_box_t _b_19083, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_catch_fun21279(kk_function_t _b_19084_19082, kk_context_t* _ctx) {
  struct kk_std_core_catch_fun21279__t* _self = kk_function_alloc_as(struct kk_std_core_catch_fun21279__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_catch_fun21279, kk_context());
  _self->_b_19084_19082 = _b_19084_19082;
  return &_self->_base;
}

static kk_box_t kk_std_core_catch_fun21279(kk_function_t _fself, kk_box_t _b_19083, kk_context_t* _ctx) {
  struct kk_std_core_catch_fun21279__t* _self = kk_function_as(struct kk_std_core_catch_fun21279__t*, _fself);
  kk_function_t _b_19084_19082 = _self->_b_19084_19082; /* (exception) -> 10323 10322 */
  kk_drop_match(_self, {kk_function_dup(_b_19084_19082);}, {}, _ctx)
  kk_std_core__exception _x21280 = kk_std_core__exception_unbox(_b_19083, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core__exception, kk_context_t*), _b_19084_19082, (_b_19084_19082, _x21280, _ctx));
}


// lift anonymous function
struct kk_std_core_catch_fun21281__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_catch_fun21281(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_catch_fun21281(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_catch_fun21281, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_catch_fun21281(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _x;
}

kk_box_t kk_std_core_catch(kk_function_t action, kk_function_t hndl, kk_context_t* _ctx) { /* forall<a,e> (action : () -> <exn|e> a, hndl : (exception) -> e a) -> e a */ 
  kk_std_core__hnd_exn _x21277;
  kk_std_core_hnd__clause1 _x21278;
  kk_function_t _b_19084_19082 = hndl; /*(exception) -> 10323 10322*/;
  _x21278 = kk_std_core_hnd_clause_never1(kk_std_core_new_catch_fun21279(_b_19084_19082, _ctx), _ctx); /*std/core/hnd/clause1<8108,8109,8111,8110,8112>*/
  _x21277 = kk_std_core__new_Hnd_exn(kk_reuse_null, _x21278, _ctx); /*.hnd-exn<11,12>*/
  return kk_std_core__handle_exn(((int32_t)0), _x21277, kk_std_core_new_catch_fun21281(_ctx), action, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core_cdivmod_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx) { /* (i : int, n : int) -> (int, int) */ 
  bool _match_20094;
  kk_integer_t _borrowedArg_20095 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_20096 = kk_integer_lte_borrow(n,_borrowedArg_20095,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20095, _ctx);
  _match_20094 = _borrowApp_20096; /*bool*/
  if (_match_20094) {
    kk_integer_drop(n, _ctx);
    kk_integer_t _b_19089_19085 = i; /*int*/;
    kk_integer_t _b_19090_19086 = kk_integer_from_small(0); /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_19089_19085), kk_integer_box(_b_19090_19086), _ctx);
  }
  kk_integer_t cq;
  kk_integer_t _x21282 = kk_integer_dup(i); /*int*/
  kk_integer_t _x21283 = kk_integer_dup(n); /*int*/
  cq = kk_std_core_cdiv_exp10(_x21282, _x21283, _ctx); /*int*/
  kk_integer_t cr;
  kk_integer_t _x21284;
  kk_integer_t _x21285 = kk_integer_dup(cq); /*int*/
  _x21284 = kk_std_core_mul_exp10(_x21285, n, _ctx); /*int*/
  cr = kk_integer_sub(i,_x21284,kk_context()); /*int*/
  kk_integer_t _b_19091_19087 = cq; /*int*/;
  kk_integer_t _b_19092_19088 = cr; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_19091_19087), kk_integer_box(_b_19092_19088), _ctx);
}
 
// Concatenate a list of `:maybe` values

kk_std_core__list kk_std_core__ctail_concat_maybe(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (xs : list<maybe<a>>, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21286 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x19093 = _con21286->head;
    kk_std_core__list xx = _con21286->tail;
    kk_std_core_types__maybe x = kk_std_core_types__maybe_unbox(_box_x19093, NULL);
    kk_reuse_t _ru_20369 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core_types__maybe_dup(x);
      kk_box_drop(_box_x19093, _ctx);
      kk_Unit;
      kk_Unit;
      _ru_20369 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_std_core_types__maybe_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    if (kk_std_core_types__is_Just(x)) {
      kk_box_t y = x._cons.Just.value;
      kk_std_core__list _ctail_17335 = kk_std_core__list_hole(); /*list<10410>*/;
      kk_std_core__list _ctail_17336 = kk_std_core__new_Cons(_ru_20369, y, _ctail_17335, _ctx); /*list<10410>*/;
      { // tailcall
        kk_std_core_types__ctail _x21288;
        kk_box_t* field_18024 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17336)->tail)); /*cfield<list<10410>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x19100 = _acc._cons.CTail._field1;
          kk_box_t* last0 = _acc._cons.CTail._field2;
          kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19100, NULL);
          kk_std_core__list_dup(head0);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last0) = kk_std_core__list_box(_ctail_17336, _ctx);
          kk_std_core__list _b_19114_19105 = head0; /*list<10410>*/;
          kk_box_t* _b_19115_19106 = field_18024; /*cfield<list<10410>>*/;
          _x21288 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19114_19105, _ctx), _b_19115_19106, _ctx); /*ctail<58>*/
          goto _match21289;
        }
        kk_std_core__list _b_19116_19107 = _ctail_17336; /*list<10410>*/;
        kk_box_t* _b_19117_19108 = field_18024; /*cfield<list<10410>>*/;
        _x21288 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19116_19107, _ctx), _b_19117_19108, _ctx); /*ctail<58>*/
        _match21289: ;
        xs = xx;
        _acc = _x21288;
        goto kk__tailcall;
      }
    }
    kk_reuse_drop(_ru_20369);
    { // tailcall
      xs = xx;
      goto kk__tailcall;
    }
  }
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19109 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19109, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Concatenate a list of `:maybe` values

kk_std_core__list kk_std_core_concat_maybe(kk_std_core__list xs0, kk_context_t* _ctx) { /* forall<a> (xs : list<maybe<a>>) -> list<a> */ 
  return kk_std_core__ctail_concat_maybe(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17729_op(kk_function_t action, kk_ssize_t end, kk_ssize_t i, kk_std_core_types__maybe _y_17479, kk_context_t* _ctx) { /* forall<a,e> (action : (ssize_t) -> e maybe<a>, end : ssize_t, i : ssize_t, maybe<a>) -> e maybe<a> */ 
  if (kk_std_core_types__is_Nothing(_y_17479)) {
    kk_ssize_t i0_17237 = kk_std_core_incr_1(i, _ctx); /*ssize_t*/;
    return kk_std_core__lift17179_for_whilez(action, end, i0_17237, _ctx);
  }
  kk_box_t x = _y_17479._cons.Just.value;
  kk_function_drop(action, _ctx);
  return kk_std_core_types__new_Just(x, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17179_for_whilez_fun21293__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_ssize_t end0;
  kk_ssize_t i0;
};
static kk_std_core_types__maybe kk_std_core__lift17179_for_whilez_fun21293(kk_function_t _fself, kk_std_core_types__maybe _y_174790, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17179_for_whilez_fun21293(kk_function_t action0, kk_ssize_t end0, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17179_for_whilez_fun21293__t* _self = kk_function_alloc_as(struct kk_std_core__lift17179_for_whilez_fun21293__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17179_for_whilez_fun21293, kk_context());
  _self->action0 = action0;
  _self->end0 = end0;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core__lift17179_for_whilez_fun21293(kk_function_t _fself, kk_std_core_types__maybe _y_174790, kk_context_t* _ctx) {
  struct kk_std_core__lift17179_for_whilez_fun21293__t* _self = kk_function_as(struct kk_std_core__lift17179_for_whilez_fun21293__t*, _fself);
  kk_function_t action0 = _self->action0; /* (ssize_t) -> 10465 maybe<10464> */
  kk_ssize_t end0 = _self->end0; /* ssize_t */
  kk_ssize_t i0 = _self->i0; /* ssize_t */
  kk_drop_match(_self, {kk_function_dup(action0);;;}, {}, _ctx)
  return kk_std_core__mlift17729_op(action0, end0, i0, _y_174790, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17179_for_whilez_fun21295__t {
  struct kk_function_s _base;
  kk_function_t _b_19120_19118;
};
static kk_box_t kk_std_core__lift17179_for_whilez_fun21295(kk_function_t _fself, kk_box_t _b_19119, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17179_for_whilez_fun21295(kk_function_t _b_19120_19118, kk_context_t* _ctx) {
  struct kk_std_core__lift17179_for_whilez_fun21295__t* _self = kk_function_alloc_as(struct kk_std_core__lift17179_for_whilez_fun21295__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17179_for_whilez_fun21295, kk_context());
  _self->_b_19120_19118 = _b_19120_19118;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17179_for_whilez_fun21295(kk_function_t _fself, kk_box_t _b_19119, kk_context_t* _ctx) {
  struct kk_std_core__lift17179_for_whilez_fun21295__t* _self = kk_function_as(struct kk_std_core__lift17179_for_whilez_fun21295__t*, _fself);
  kk_function_t _b_19120_19118 = _self->_b_19120_19118; /* (maybe<10464>) -> 10465 maybe<10464> */
  kk_drop_match(_self, {kk_function_dup(_b_19120_19118);}, {}, _ctx)
  kk_std_core_types__maybe _x21296;
  kk_std_core_types__maybe _x21297 = kk_std_core_types__maybe_unbox(_b_19119, _ctx); /*maybe<10464>*/
  _x21296 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19120_19118, (_b_19120_19118, _x21297, _ctx)); /*maybe<10464>*/
  return kk_std_core_types__maybe_box(_x21296, _ctx);
}

kk_std_core_types__maybe kk_std_core__lift17179_for_whilez(kk_function_t action0, kk_ssize_t end0, kk_ssize_t i0, kk_context_t* _ctx) { /* forall<a,e> (action : (ssize_t) -> e maybe<a>, end : ssize_t, i : ssize_t) -> e maybe<a> */ 
  kk__tailcall: ;
  bool _match_20092 = (i0 <= end0); /*bool*/;
  if (_match_20092) {
    kk_std_core_types__maybe x0_18027;
    kk_function_t _x21292 = kk_function_dup(action0); /*(ssize_t) -> 10465 maybe<10464>*/
    x0_18027 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_ssize_t, kk_context_t*), _x21292, (_x21292, i0, _ctx)); /*maybe<10464>*/
    if (kk_yielding(kk_context())) {
      kk_std_core_types__maybe_drop(x0_18027, _ctx);
      kk_function_t _b_19120_19118 = kk_std_core__new_lift17179_for_whilez_fun21293(action0, end0, i0, _ctx); /*(maybe<10464>) -> 10465 maybe<10464>*/;
      kk_box_t _x21294 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17179_for_whilez_fun21295(_b_19120_19118, _ctx), _ctx); /*3858*/
      return kk_std_core_types__maybe_unbox(_x21294, _ctx);
    }
    if (kk_std_core_types__is_Nothing(x0_18027)) {
      kk_ssize_t i0_172370 = kk_std_core_incr_1(i0, _ctx); /*ssize_t*/;
      { // tailcall
        i0 = i0_172370;
        goto kk__tailcall;
      }
    }
    kk_box_t x1 = x0_18027._cons.Just.value;
    kk_function_drop(action0, _ctx);
    kk_Unit;
    return kk_std_core_types__new_Just(x1, _ctx);
  }
  kk_function_drop(action0, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17730_foreach_while(kk_function_t action, kk_std_core__list xx, kk_std_core_types__maybe _y_17484, kk_context_t* _ctx) { /* forall<a,b,e> (action : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e maybe<b> */ 
  if (kk_std_core_types__is_Nothing(_y_17484)) {
    return kk_std_core_foreach_while(xx, action, _ctx);
  }
  kk_function_drop(action, _ctx);
  kk_std_core__list_drop(xx, _ctx);
  return _y_17484;
}
 
// Invoke `action` for each element of a list while `action` return `Nothing`


// lift anonymous function
struct kk_std_core_foreach_while_fun21303__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_std_core__list xx0;
};
static kk_std_core_types__maybe kk_std_core_foreach_while_fun21303(kk_function_t _fself, kk_std_core_types__maybe _y_174840, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_while_fun21303(kk_function_t action0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun21303__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_while_fun21303__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_while_fun21303, kk_context());
  _self->action0 = action0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core_foreach_while_fun21303(kk_function_t _fself, kk_std_core_types__maybe _y_174840, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun21303__t* _self = kk_function_as(struct kk_std_core_foreach_while_fun21303__t*, _fself);
  kk_function_t action0 = _self->action0; /* (10598) -> 10600 maybe<10599> */
  kk_std_core__list xx0 = _self->xx0; /* list<10598> */
  kk_drop_match(_self, {kk_function_dup(action0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17730_foreach_while(action0, xx0, _y_174840, _ctx);
}


// lift anonymous function
struct kk_std_core_foreach_while_fun21305__t {
  struct kk_function_s _base;
  kk_function_t _b_19123_19121;
};
static kk_box_t kk_std_core_foreach_while_fun21305(kk_function_t _fself, kk_box_t _b_19122, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_while_fun21305(kk_function_t _b_19123_19121, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun21305__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_while_fun21305__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_while_fun21305, kk_context());
  _self->_b_19123_19121 = _b_19123_19121;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_while_fun21305(kk_function_t _fself, kk_box_t _b_19122, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun21305__t* _self = kk_function_as(struct kk_std_core_foreach_while_fun21305__t*, _fself);
  kk_function_t _b_19123_19121 = _self->_b_19123_19121; /* (maybe<10599>) -> 10600 maybe<10599> */
  kk_drop_match(_self, {kk_function_dup(_b_19123_19121);}, {}, _ctx)
  kk_std_core_types__maybe _x21306;
  kk_std_core_types__maybe _x21307 = kk_std_core_types__maybe_unbox(_b_19122, _ctx); /*maybe<10599>*/
  _x21306 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19123_19121, (_b_19123_19121, _x21307, _ctx)); /*maybe<10599>*/
  return kk_std_core_types__maybe_box(_x21306, _ctx);
}

kk_std_core_types__maybe kk_std_core_foreach_while(kk_std_core__list xs, kk_function_t action0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>) -> e maybe<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(action0, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  struct kk_std_core_Cons* _con21301 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21301->head;
  kk_std_core__list xx0 = _con21301->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx0);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core_types__maybe x0_18030;
  kk_function_t _x21302 = kk_function_dup(action0); /*(10598) -> 10600 maybe<10599>*/
  x0_18030 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21302, (_x21302, x, _ctx)); /*maybe<10599>*/
  if (kk_yielding(kk_context())) {
    kk_std_core_types__maybe_drop(x0_18030, _ctx);
    kk_function_t _b_19123_19121 = kk_std_core_new_foreach_while_fun21303(action0, xx0, _ctx); /*(maybe<10599>) -> 10600 maybe<10599>*/;
    kk_box_t _x21304 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_while_fun21305(_b_19123_19121, _ctx), _ctx); /*3858*/
    return kk_std_core_types__maybe_unbox(_x21304, _ctx);
  }
  if (kk_std_core_types__is_Nothing(x0_18030)) {
    { // tailcall
      xs = xx0;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action0, _ctx);
  kk_std_core__list_drop(xx0, _ctx);
  return x0_18030;
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17731_foreach_while_1(kk_function_t action, kk_std_core__sslice rest, kk_std_core_types__maybe _y_17488, kk_context_t* _ctx) { /* forall<a,e> (action : (c : char) -> e maybe<a>, rest : sslice, maybe<a>) -> e maybe<a> */ 
  if (kk_std_core_types__is_Nothing(_y_17488)) {
    return kk_std_core_foreach_while_1(rest, action, _ctx);
  }
  kk_function_drop(action, _ctx);
  kk_std_core__sslice_drop(rest, _ctx);
  return _y_17488;
}
 
// Apply a function for each character in a string slice.
// If `action` returns `Just`, the function returns immediately with that result.


// lift anonymous function
struct kk_std_core_foreach_while_fun21312__t_1 {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_std_core__sslice rest0;
};
static kk_std_core_types__maybe kk_std_core_foreach_while_fun21312_1(kk_function_t _fself, kk_std_core_types__maybe _y_174880, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_while_fun21312_1(kk_function_t action0, kk_std_core__sslice rest0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun21312__t_1* _self = kk_function_alloc_as(struct kk_std_core_foreach_while_fun21312__t_1, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_while_fun21312_1, kk_context());
  _self->action0 = action0;
  _self->rest0 = rest0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core_foreach_while_fun21312_1(kk_function_t _fself, kk_std_core_types__maybe _y_174880, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun21312__t_1* _self = kk_function_as(struct kk_std_core_foreach_while_fun21312__t_1*, _fself);
  kk_function_t action0 = _self->action0; /* (c : char) -> 10685 maybe<10684> */
  kk_std_core__sslice rest0 = _self->rest0; /* sslice */
  kk_drop_match(_self, {kk_function_dup(action0);kk_std_core__sslice_dup(rest0);}, {}, _ctx)
  return kk_std_core__mlift17731_foreach_while_1(action0, rest0, _y_174880, _ctx);
}


// lift anonymous function
struct kk_std_core_foreach_while_fun21314__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19129_19127;
};
static kk_box_t kk_std_core_foreach_while_fun21314_1(kk_function_t _fself, kk_box_t _b_19128, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_while_fun21314_1(kk_function_t _b_19129_19127, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun21314__t_1* _self = kk_function_alloc_as(struct kk_std_core_foreach_while_fun21314__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_while_fun21314_1, kk_context());
  _self->_b_19129_19127 = _b_19129_19127;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_while_fun21314_1(kk_function_t _fself, kk_box_t _b_19128, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun21314__t_1* _self = kk_function_as(struct kk_std_core_foreach_while_fun21314__t_1*, _fself);
  kk_function_t _b_19129_19127 = _self->_b_19129_19127; /* (maybe<10684>) -> 10685 maybe<10684> */
  kk_drop_match(_self, {kk_function_dup(_b_19129_19127);}, {}, _ctx)
  kk_std_core_types__maybe _x21315;
  kk_std_core_types__maybe _x21316 = kk_std_core_types__maybe_unbox(_b_19128, _ctx); /*maybe<10684>*/
  _x21315 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19129_19127, (_b_19129_19127, _x21316, _ctx)); /*maybe<10684>*/
  return kk_std_core_types__maybe_box(_x21315, _ctx);
}

kk_std_core_types__maybe kk_std_core_foreach_while_1(kk_std_core__sslice slice0, kk_function_t action0, kk_context_t* _ctx) { /* forall<a,e> (slice : sslice, action : (c : char) -> e maybe<a>) -> e maybe<a> */ 
  kk__tailcall: ;
  kk_std_core_types__maybe _match_20089 = kk_std_core_next(slice0, _ctx); /*maybe<(char, sslice)>*/;
  if (kk_std_core_types__is_Nothing(_match_20089)) {
    kk_function_drop(action0, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_box_t _box_x19124 = _match_20089._cons.Just.value;
  kk_std_core_types__tuple2_ _pat1 = kk_std_core_types__tuple2__unbox(_box_x19124, NULL);
  kk_box_t _box_x19125 = _pat1.fst;
  kk_box_t _box_x19126 = _pat1.snd;
  kk_char_t c = kk_char_unbox(_box_x19125, NULL);
  kk_std_core__sslice rest0 = kk_std_core__sslice_unbox(_box_x19126, NULL);
  kk_std_core__sslice_dup(rest0);
  kk_std_core_types__maybe_drop(_match_20089, _ctx);
  kk_Unit;
  kk_std_core_types__maybe x_18033;
  kk_function_t _x21311 = kk_function_dup(action0); /*(c : char) -> 10685 maybe<10684>*/
  x_18033 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_char_t, kk_context_t*), _x21311, (_x21311, c, _ctx)); /*maybe<10684>*/
  if (kk_yielding(kk_context())) {
    kk_std_core_types__maybe_drop(x_18033, _ctx);
    kk_function_t _b_19129_19127 = kk_std_core_new_foreach_while_fun21312_1(action0, rest0, _ctx); /*(maybe<10684>) -> 10685 maybe<10684>*/;
    kk_box_t _x21313 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_while_fun21314_1(_b_19129_19127, _ctx), _ctx); /*3858*/
    return kk_std_core_types__maybe_unbox(_x21313, _ctx);
  }
  if (kk_std_core_types__is_Nothing(x_18033)) {
    { // tailcall
      slice0 = rest0;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action0, _ctx);
  kk_std_core__sslice_drop(rest0, _ctx);
  return x_18033;
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17732_op(kk_function_t action_17064, kk_ssize_t end_17063, kk_ssize_t i, kk_std_core_types__maybe _y_17493, kk_context_t* _ctx) { /* forall<a,e> (action.17064 : (ssize_t) -> e maybe<a>, end.17063 : ssize_t, i : ssize_t, maybe<a>) -> e maybe<a> */ 
  if (kk_std_core_types__is_Nothing(_y_17493)) {
    kk_ssize_t i0_17238 = kk_std_core_incr_1(i, _ctx); /*ssize_t*/;
    return kk_std_core__lift17180_foreach_while_3(action_17064, end_17063, i0_17238, _ctx);
  }
  kk_box_t x = _y_17493._cons.Just.value;
  kk_function_drop(action_17064, _ctx);
  return kk_std_core_types__new_Just(x, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17180_foreach_while_fun21322__t_3 {
  struct kk_function_s _base;
  kk_function_t action_170640;
  kk_ssize_t end_170630;
  kk_ssize_t i0;
};
static kk_std_core_types__maybe kk_std_core__lift17180_foreach_while_fun21322_3(kk_function_t _fself, kk_std_core_types__maybe _y_174930, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17180_foreach_while_fun21322_3(kk_function_t action_170640, kk_ssize_t end_170630, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17180_foreach_while_fun21322__t_3* _self = kk_function_alloc_as(struct kk_std_core__lift17180_foreach_while_fun21322__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17180_foreach_while_fun21322_3, kk_context());
  _self->action_170640 = action_170640;
  _self->end_170630 = end_170630;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core__lift17180_foreach_while_fun21322_3(kk_function_t _fself, kk_std_core_types__maybe _y_174930, kk_context_t* _ctx) {
  struct kk_std_core__lift17180_foreach_while_fun21322__t_3* _self = kk_function_as(struct kk_std_core__lift17180_foreach_while_fun21322__t_3*, _fself);
  kk_function_t action_170640 = _self->action_170640; /* (ssize_t) -> 10785 maybe<10784> */
  kk_ssize_t end_170630 = _self->end_170630; /* ssize_t */
  kk_ssize_t i0 = _self->i0; /* ssize_t */
  kk_drop_match(_self, {kk_function_dup(action_170640);;;}, {}, _ctx)
  return kk_std_core__mlift17732_op(action_170640, end_170630, i0, _y_174930, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17180_foreach_while_fun21324__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_19132_19130;
};
static kk_box_t kk_std_core__lift17180_foreach_while_fun21324_3(kk_function_t _fself, kk_box_t _b_19131, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17180_foreach_while_fun21324_3(kk_function_t _b_19132_19130, kk_context_t* _ctx) {
  struct kk_std_core__lift17180_foreach_while_fun21324__t_3* _self = kk_function_alloc_as(struct kk_std_core__lift17180_foreach_while_fun21324__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17180_foreach_while_fun21324_3, kk_context());
  _self->_b_19132_19130 = _b_19132_19130;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17180_foreach_while_fun21324_3(kk_function_t _fself, kk_box_t _b_19131, kk_context_t* _ctx) {
  struct kk_std_core__lift17180_foreach_while_fun21324__t_3* _self = kk_function_as(struct kk_std_core__lift17180_foreach_while_fun21324__t_3*, _fself);
  kk_function_t _b_19132_19130 = _self->_b_19132_19130; /* (maybe<10784>) -> 10785 maybe<10784> */
  kk_drop_match(_self, {kk_function_dup(_b_19132_19130);}, {}, _ctx)
  kk_std_core_types__maybe _x21325;
  kk_std_core_types__maybe _x21326 = kk_std_core_types__maybe_unbox(_b_19131, _ctx); /*maybe<10784>*/
  _x21325 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19132_19130, (_b_19132_19130, _x21326, _ctx)); /*maybe<10784>*/
  return kk_std_core_types__maybe_box(_x21325, _ctx);
}

kk_std_core_types__maybe kk_std_core__lift17180_foreach_while_3(kk_function_t action_170640, kk_ssize_t end_170630, kk_ssize_t i0, kk_context_t* _ctx) { /* forall<a,e> (action.17064 : (ssize_t) -> e maybe<a>, end.17063 : ssize_t, i : ssize_t) -> e maybe<a> */ 
  kk__tailcall: ;
  bool _match_20087 = (i0 <= end_170630); /*bool*/;
  if (_match_20087) {
    kk_std_core_types__maybe x0_18036;
    kk_function_t _x21321 = kk_function_dup(action_170640); /*(ssize_t) -> 10785 maybe<10784>*/
    x0_18036 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_ssize_t, kk_context_t*), _x21321, (_x21321, i0, _ctx)); /*maybe<10784>*/
    if (kk_yielding(kk_context())) {
      kk_std_core_types__maybe_drop(x0_18036, _ctx);
      kk_function_t _b_19132_19130 = kk_std_core__new_lift17180_foreach_while_fun21322_3(action_170640, end_170630, i0, _ctx); /*(maybe<10784>) -> 10785 maybe<10784>*/;
      kk_box_t _x21323 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17180_foreach_while_fun21324_3(_b_19132_19130, _ctx), _ctx); /*3858*/
      return kk_std_core_types__maybe_unbox(_x21323, _ctx);
    }
    if (kk_std_core_types__is_Nothing(x0_18036)) {
      kk_ssize_t i0_172380 = kk_std_core_incr_1(i0, _ctx); /*ssize_t*/;
      { // tailcall
        i0 = i0_172380;
        goto kk__tailcall;
      }
    }
    kk_box_t x1 = x0_18036._cons.Just.value;
    kk_function_drop(action_170640, _ctx);
    kk_Unit;
    return kk_std_core_types__new_Just(x1, _ctx);
  }
  kk_function_drop(action_170640, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Invoke a function `f` for each element in a vector `v`.
// If `f` returns `Just`, the iteration is stopped early and the result is returned.


// lift anonymous function
struct kk_std_core_foreach_while_fun21328__t_3 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_vector_t v;
};
static kk_std_core_types__maybe kk_std_core_foreach_while_fun21328_3(kk_function_t _fself, kk_ssize_t i0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_while_fun21328_3(kk_function_t f, kk_vector_t v, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun21328__t_3* _self = kk_function_alloc_as(struct kk_std_core_foreach_while_fun21328__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_while_fun21328_3, kk_context());
  _self->f = f;
  _self->v = v;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core_foreach_while_fun21328_3(kk_function_t _fself, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_while_fun21328__t_3* _self = kk_function_as(struct kk_std_core_foreach_while_fun21328__t_3*, _fself);
  kk_function_t f = _self->f; /* (10783) -> 10785 maybe<10784> */
  kk_vector_t v = _self->v; /* vector<10783> */
  kk_drop_match(_self, {kk_function_dup(f);kk_vector_dup(v);}, {}, _ctx)
  kk_box_t _x21329;
  kk_box_t _borrowApp_20086 = kk_vector_at_borrow(v, i0); /*219*/;
  kk_vector_drop(v, _ctx);
  _x21329 = _borrowApp_20086; /*219*/
  return kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), f, (f, _x21329, _ctx));
}

kk_std_core_types__maybe kk_std_core_foreach_while_3(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,b,e> (v : vector<a>, f : (a) -> e maybe<b>) -> e maybe<b> */ 
  kk_ssize_t start0_17062 = ((kk_ssize_t)0); /*ssize_t*/;
  kk_ssize_t end_17063;
  kk_ssize_t _x21327 = kk_vector_len_borrow(v); /*ssize_t*/
  end_17063 = kk_std_core_decr_1(_x21327, _ctx); /*ssize_t*/
  return kk_std_core__lift17180_foreach_while_3(kk_std_core_new_foreach_while_fun21328_3(f, v, _ctx), end_17063, start0_17062, _ctx);
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17733_foreach(kk_function_t action, kk_std_core__list xx, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<a,e> (action : (a) -> e (), xx : list<a>, wild_ : ()) -> e () */ 
  kk_std_core_foreach(xx, action, _ctx); return kk_Unit;
}
 
// Invoke `action` for each element of a list


// lift anonymous function
struct kk_std_core_foreach_fun21332__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_std_core__list xx0;
};
static kk_unit_t kk_std_core_foreach_fun21332(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun21332(kk_function_t action0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun21332__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun21332__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun21332, kk_context());
  _self->action0 = action0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_unit_t kk_std_core_foreach_fun21332(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun21332__t* _self = kk_function_as(struct kk_std_core_foreach_fun21332__t*, _fself);
  kk_function_t action0 = _self->action0; /* (10978) -> 10979 () */
  kk_std_core__list xx0 = _self->xx0; /* list<10978> */
  kk_drop_match(_self, {kk_function_dup(action0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17733_foreach(action0, xx0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core_foreach_fun21334__t {
  struct kk_function_s _base;
  kk_function_t _b_19135_19133;
};
static kk_box_t kk_std_core_foreach_fun21334(kk_function_t _fself, kk_box_t _b_19134, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun21334(kk_function_t _b_19135_19133, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun21334__t* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun21334__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun21334, kk_context());
  _self->_b_19135_19133 = _b_19135_19133;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_fun21334(kk_function_t _fself, kk_box_t _b_19134, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun21334__t* _self = kk_function_as(struct kk_std_core_foreach_fun21334__t*, _fself);
  kk_function_t _b_19135_19133 = _self->_b_19135_19133; /* (wild_0 : ()) -> 10979 () */
  kk_drop_match(_self, {kk_function_dup(_b_19135_19133);}, {}, _ctx)
  kk_unit_t _x21335 = kk_Unit;
  kk_unit_t _x21336 = kk_Unit;
  kk_unit_unbox(_b_19134);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19135_19133, (_b_19135_19133, _x21336, _ctx));
  return kk_unit_box(_x21335);
}

kk_unit_t kk_std_core_foreach(kk_std_core__list xs, kk_function_t action0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, action : (a) -> e ()) -> e () */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(action0, _ctx);
    kk_Unit; return kk_Unit;
  }
  struct kk_std_core_Cons* _con21330 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21330->head;
  kk_std_core__list xx0 = _con21330->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx0);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_unit_t x0_18039 = kk_Unit;
  kk_function_t _x21331 = kk_function_dup(action0); /*(10978) -> 10979 ()*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_box_t, kk_context_t*), _x21331, (_x21331, x, _ctx));
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19135_19133 = kk_std_core_new_foreach_fun21332(action0, xx0, _ctx); /*(wild_0 : ()) -> 10979 ()*/;
    kk_box_t _x21333 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_fun21334(_b_19135_19133, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21333); return kk_Unit;
  }
  { // tailcall
    xs = xx0;
    goto kk__tailcall;
  }
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17734_op(kk_function_t action, kk_std_core__sslice rest, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<_a,e> (action : (c : char) -> e (), rest : sslice, wild_ : ()) -> e maybe<_a> */ 
  kk_std_core_types__maybe _pat5 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/;
  kk_std_core_types__maybe _scrut = _pat5; /*maybe<_11024>*/;
  kk_std_core_types__maybe_drop(_scrut, _ctx);
  return kk_std_core__lift17181_foreach_1(action, rest, _ctx);
}
 
// lifted
// specialized std/core/foreach-while.1 to parameters [action]


// lift anonymous function
struct kk_std_core__lift17181_foreach_fun21341__t_1 {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_std_core__sslice rest0;
};
static kk_std_core_types__maybe kk_std_core__lift17181_foreach_fun21341_1(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17181_foreach_fun21341_1(kk_function_t action0, kk_std_core__sslice rest0, kk_context_t* _ctx) {
  struct kk_std_core__lift17181_foreach_fun21341__t_1* _self = kk_function_alloc_as(struct kk_std_core__lift17181_foreach_fun21341__t_1, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17181_foreach_fun21341_1, kk_context());
  _self->action0 = action0;
  _self->rest0 = rest0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core__lift17181_foreach_fun21341_1(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17181_foreach_fun21341__t_1* _self = kk_function_as(struct kk_std_core__lift17181_foreach_fun21341__t_1*, _fself);
  kk_function_t action0 = _self->action0; /* (c : char) -> 11037 () */
  kk_std_core__sslice rest0 = _self->rest0; /* sslice */
  kk_drop_match(_self, {kk_function_dup(action0);kk_std_core__sslice_dup(rest0);}, {}, _ctx)
  return kk_std_core__mlift17734_op(action0, rest0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17181_foreach_fun21343__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19141_19139;
};
static kk_box_t kk_std_core__lift17181_foreach_fun21343_1(kk_function_t _fself, kk_box_t _b_19140, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17181_foreach_fun21343_1(kk_function_t _b_19141_19139, kk_context_t* _ctx) {
  struct kk_std_core__lift17181_foreach_fun21343__t_1* _self = kk_function_alloc_as(struct kk_std_core__lift17181_foreach_fun21343__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17181_foreach_fun21343_1, kk_context());
  _self->_b_19141_19139 = _b_19141_19139;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17181_foreach_fun21343_1(kk_function_t _fself, kk_box_t _b_19140, kk_context_t* _ctx) {
  struct kk_std_core__lift17181_foreach_fun21343__t_1* _self = kk_function_as(struct kk_std_core__lift17181_foreach_fun21343__t_1*, _fself);
  kk_function_t _b_19141_19139 = _self->_b_19141_19139; /* (wild_0 : ()) -> 11037 maybe<_11024> */
  kk_drop_match(_self, {kk_function_dup(_b_19141_19139);}, {}, _ctx)
  kk_std_core_types__maybe _x21344;
  kk_unit_t _x21345 = kk_Unit;
  kk_unit_unbox(_b_19140);
  _x21344 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_unit_t, kk_context_t*), _b_19141_19139, (_b_19141_19139, _x21345, _ctx)); /*maybe<_11024>*/
  return kk_std_core_types__maybe_box(_x21344, _ctx);
}

kk_std_core_types__maybe kk_std_core__lift17181_foreach_1(kk_function_t action0, kk_std_core__sslice slice00, kk_context_t* _ctx) { /* forall<_a,e> (action : (c : char) -> e (), slice00 : sslice) -> e maybe<_a> */ 
  kk__tailcall: ;
  kk_std_core_types__maybe _match_20083 = kk_std_core_next(slice00, _ctx); /*maybe<(char, sslice)>*/;
  if (kk_std_core_types__is_Nothing(_match_20083)) {
    kk_function_drop(action0, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_box_t _box_x19136 = _match_20083._cons.Just.value;
  kk_std_core_types__tuple2_ _pat1 = kk_std_core_types__tuple2__unbox(_box_x19136, NULL);
  kk_box_t _box_x19137 = _pat1.fst;
  kk_box_t _box_x19138 = _pat1.snd;
  kk_char_t c0 = kk_char_unbox(_box_x19137, NULL);
  kk_std_core__sslice rest0 = kk_std_core__sslice_unbox(_box_x19138, NULL);
  kk_std_core__sslice_dup(rest0);
  kk_std_core_types__maybe_drop(_match_20083, _ctx);
  kk_Unit;
  kk_unit_t x_18042 = kk_Unit;
  kk_function_t _x21340 = kk_function_dup(action0); /*(c : char) -> 11037 ()*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_char_t, kk_context_t*), _x21340, (_x21340, c0, _ctx));
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19141_19139 = kk_std_core__new_lift17181_foreach_fun21341_1(action0, rest0, _ctx); /*(wild_0 : ()) -> 11037 maybe<_11024>*/;
    kk_box_t _x21342 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17181_foreach_fun21343_1(_b_19141_19139, _ctx), _ctx); /*3858*/
    return kk_std_core_types__maybe_unbox(_x21342, _ctx);
  }
  kk_std_core_types__maybe _pat50 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/;
  kk_std_core_types__maybe _scrut = _pat50; /*maybe<_11024>*/;
  kk_std_core_types__maybe_drop(_scrut, _ctx);
  { // tailcall
    slice00 = rest0;
    goto kk__tailcall;
  }
}
 
// Apply a function for each character in a string slice.


// lift anonymous function
struct kk_std_core_foreach_fun21346__t_1 {
  struct kk_function_s _base;
};
static kk_unit_t kk_std_core_foreach_fun21346_1(kk_function_t _fself, kk_std_core_types__maybe wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun21346_1(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_foreach_fun21346_1, _ctx)
  return kk_function_dup(_fself);
}

static kk_unit_t kk_std_core_foreach_fun21346_1(kk_function_t _fself, kk_std_core_types__maybe wild__0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__maybe_drop(wild__0, _ctx);
  return kk_Unit;
}


// lift anonymous function
struct kk_std_core_foreach_fun21348__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19144_19142;
};
static kk_box_t kk_std_core_foreach_fun21348_1(kk_function_t _fself, kk_box_t _b_19143, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun21348_1(kk_function_t _b_19144_19142, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun21348__t_1* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun21348__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun21348_1, kk_context());
  _self->_b_19144_19142 = _b_19144_19142;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_fun21348_1(kk_function_t _fself, kk_box_t _b_19143, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun21348__t_1* _self = kk_function_as(struct kk_std_core_foreach_fun21348__t_1*, _fself);
  kk_function_t _b_19144_19142 = _self->_b_19144_19142; /* (wild_0 : maybe<_11024>) -> 11037 () */
  kk_drop_match(_self, {kk_function_dup(_b_19144_19142);}, {}, _ctx)
  kk_unit_t _x21349 = kk_Unit;
  kk_std_core_types__maybe _x21350 = kk_std_core_types__maybe_unbox(_b_19143, _ctx); /*maybe<_11024>*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19144_19142, (_b_19144_19142, _x21350, _ctx));
  return kk_unit_box(_x21349);
}

kk_unit_t kk_std_core_foreach_1(kk_std_core__sslice slice0, kk_function_t action, kk_context_t* _ctx) { /* forall<e> (slice : sslice, action : (c : char) -> e ()) -> e () */ 
  kk_std_core_types__maybe x_18045 = kk_std_core__lift17181_foreach_1(action, slice0, _ctx); /*maybe<_11024>*/;
  kk_std_core_types__maybe_drop(x_18045, _ctx);
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19144_19142 = kk_std_core_new_foreach_fun21346_1(_ctx); /*(wild_0 : maybe<_11024>) -> 11037 ()*/;
    kk_box_t _x21347 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_fun21348_1(_b_19144_19142, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21347); return kk_Unit;
  }
  kk_Unit; return kk_Unit;
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17736_op(kk_function_t action0_17066, kk_std_core__sslice rest, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<_a,e> (action0.17066 : (c : char) -> e (), rest : sslice, wild_ : ()) -> e maybe<_a> */ 
  kk_std_core_types__maybe _pat5 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/;
  kk_std_core_types__maybe _scrut = _pat5; /*maybe<_11024>*/;
  kk_std_core_types__maybe_drop(_scrut, _ctx);
  return kk_std_core__lift17182_foreach_2(action0_17066, rest, _ctx);
}
 
// lifted
// specialized std/core/foreach-while.1 to parameters [action]


// lift anonymous function
struct kk_std_core__lift17182_foreach_fun21355__t_2 {
  struct kk_function_s _base;
  kk_function_t action0_170660;
  kk_std_core__sslice rest0;
};
static kk_std_core_types__maybe kk_std_core__lift17182_foreach_fun21355_2(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17182_foreach_fun21355_2(kk_function_t action0_170660, kk_std_core__sslice rest0, kk_context_t* _ctx) {
  struct kk_std_core__lift17182_foreach_fun21355__t_2* _self = kk_function_alloc_as(struct kk_std_core__lift17182_foreach_fun21355__t_2, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17182_foreach_fun21355_2, kk_context());
  _self->action0_170660 = action0_170660;
  _self->rest0 = rest0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core__lift17182_foreach_fun21355_2(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17182_foreach_fun21355__t_2* _self = kk_function_as(struct kk_std_core__lift17182_foreach_fun21355__t_2*, _fself);
  kk_function_t action0_170660 = _self->action0_170660; /* (c : char) -> 11076 () */
  kk_std_core__sslice rest0 = _self->rest0; /* sslice */
  kk_drop_match(_self, {kk_function_dup(action0_170660);kk_std_core__sslice_dup(rest0);}, {}, _ctx)
  return kk_std_core__mlift17736_op(action0_170660, rest0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17182_foreach_fun21357__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_19150_19148;
};
static kk_box_t kk_std_core__lift17182_foreach_fun21357_2(kk_function_t _fself, kk_box_t _b_19149, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17182_foreach_fun21357_2(kk_function_t _b_19150_19148, kk_context_t* _ctx) {
  struct kk_std_core__lift17182_foreach_fun21357__t_2* _self = kk_function_alloc_as(struct kk_std_core__lift17182_foreach_fun21357__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17182_foreach_fun21357_2, kk_context());
  _self->_b_19150_19148 = _b_19150_19148;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17182_foreach_fun21357_2(kk_function_t _fself, kk_box_t _b_19149, kk_context_t* _ctx) {
  struct kk_std_core__lift17182_foreach_fun21357__t_2* _self = kk_function_as(struct kk_std_core__lift17182_foreach_fun21357__t_2*, _fself);
  kk_function_t _b_19150_19148 = _self->_b_19150_19148; /* (wild_0 : ()) -> 11076 maybe<_11024> */
  kk_drop_match(_self, {kk_function_dup(_b_19150_19148);}, {}, _ctx)
  kk_std_core_types__maybe _x21358;
  kk_unit_t _x21359 = kk_Unit;
  kk_unit_unbox(_b_19149);
  _x21358 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_unit_t, kk_context_t*), _b_19150_19148, (_b_19150_19148, _x21359, _ctx)); /*maybe<_11024>*/
  return kk_std_core_types__maybe_box(_x21358, _ctx);
}

kk_std_core_types__maybe kk_std_core__lift17182_foreach_2(kk_function_t action0_170660, kk_std_core__sslice slice0, kk_context_t* _ctx) { /* forall<_a,e> (action0.17066 : (c : char) -> e (), slice0 : sslice) -> e maybe<_a> */ 
  kk__tailcall: ;
  kk_std_core_types__maybe _match_20080 = kk_std_core_next(slice0, _ctx); /*maybe<(char, sslice)>*/;
  if (kk_std_core_types__is_Nothing(_match_20080)) {
    kk_function_drop(action0_170660, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_box_t _box_x19145 = _match_20080._cons.Just.value;
  kk_std_core_types__tuple2_ _pat1 = kk_std_core_types__tuple2__unbox(_box_x19145, NULL);
  kk_box_t _box_x19146 = _pat1.fst;
  kk_box_t _box_x19147 = _pat1.snd;
  kk_char_t c0 = kk_char_unbox(_box_x19146, NULL);
  kk_std_core__sslice rest0 = kk_std_core__sslice_unbox(_box_x19147, NULL);
  kk_std_core__sslice_dup(rest0);
  kk_std_core_types__maybe_drop(_match_20080, _ctx);
  kk_Unit;
  kk_unit_t x_18049 = kk_Unit;
  kk_function_t _x21354 = kk_function_dup(action0_170660); /*(c : char) -> 11076 ()*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_char_t, kk_context_t*), _x21354, (_x21354, c0, _ctx));
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19150_19148 = kk_std_core__new_lift17182_foreach_fun21355_2(action0_170660, rest0, _ctx); /*(wild_0 : ()) -> 11076 maybe<_11024>*/;
    kk_box_t _x21356 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17182_foreach_fun21357_2(_b_19150_19148, _ctx), _ctx); /*3858*/
    return kk_std_core_types__maybe_unbox(_x21356, _ctx);
  }
  kk_std_core_types__maybe _pat50 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/;
  kk_std_core_types__maybe _scrut = _pat50; /*maybe<_11024>*/;
  kk_std_core_types__maybe_drop(_scrut, _ctx);
  { // tailcall
    slice0 = rest0;
    goto kk__tailcall;
  }
}
 
// Invoke a function for each character in a string


// lift anonymous function
struct kk_std_core_foreach_fun21363__t_2 {
  struct kk_function_s _base;
};
static kk_unit_t kk_std_core_foreach_fun21363_2(kk_function_t _fself, kk_std_core_types__maybe wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun21363_2(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_foreach_fun21363_2, _ctx)
  return kk_function_dup(_fself);
}

static kk_unit_t kk_std_core_foreach_fun21363_2(kk_function_t _fself, kk_std_core_types__maybe wild__0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__maybe_drop(wild__0, _ctx);
  return kk_Unit;
}


// lift anonymous function
struct kk_std_core_foreach_fun21365__t_2 {
  struct kk_function_s _base;
  kk_function_t _b_19153_19151;
};
static kk_box_t kk_std_core_foreach_fun21365_2(kk_function_t _fself, kk_box_t _b_19152, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun21365_2(kk_function_t _b_19153_19151, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun21365__t_2* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun21365__t_2, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun21365_2, kk_context());
  _self->_b_19153_19151 = _b_19153_19151;
  return &_self->_base;
}

static kk_box_t kk_std_core_foreach_fun21365_2(kk_function_t _fself, kk_box_t _b_19152, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun21365__t_2* _self = kk_function_as(struct kk_std_core_foreach_fun21365__t_2*, _fself);
  kk_function_t _b_19153_19151 = _self->_b_19153_19151; /* (wild_0 : maybe<_11024>) -> 11076 () */
  kk_drop_match(_self, {kk_function_dup(_b_19153_19151);}, {}, _ctx)
  kk_unit_t _x21366 = kk_Unit;
  kk_std_core_types__maybe _x21367 = kk_std_core_types__maybe_unbox(_b_19152, _ctx); /*maybe<_11024>*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19153_19151, (_b_19153_19151, _x21367, _ctx));
  return kk_unit_box(_x21366);
}

kk_unit_t kk_std_core_foreach_2(kk_string_t s, kk_function_t action, kk_context_t* _ctx) { /* forall<e> (s : string, action : (c : char) -> e ()) -> e () */ 
  kk_std_core__sslice slice0_17065;
  kk_string_t _x21360 = kk_string_dup(s); /*string*/
  kk_ssize_t _x21361 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x21362 = kk_string_len(s,kk_context()); /*ssize_t*/
  slice0_17065 = kk_std_core__new_Sslice(_x21360, _x21361, _x21362, _ctx); /*sslice*/
  kk_std_core_types__maybe x_18052 = kk_std_core__lift17182_foreach_2(action, slice0_17065, _ctx); /*maybe<_11024>*/;
  kk_std_core_types__maybe_drop(x_18052, _ctx);
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19153_19151 = kk_std_core_new_foreach_fun21363_2(_ctx); /*(wild_0 : maybe<_11024>) -> 11076 ()*/;
    kk_box_t _x21364 = kk_std_core_hnd_yield_extend(kk_std_core_new_foreach_fun21365_2(_b_19153_19151, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21364); return kk_Unit;
  }
  kk_Unit; return kk_Unit;
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17738_op(kk_function_t action_17071, kk_ssize_t end_17070, kk_ssize_t i, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action.17071 : (ssize_t) -> e (), end.17070 : ssize_t, i : ssize_t, wild_ : ()) -> e () */ 
  kk_ssize_t i0_17246 = kk_std_core_incr_1(i, _ctx); /*ssize_t*/;
  kk_std_core__lift17183_foreach_3(action_17071, end_17070, i0_17246, _ctx); return kk_Unit;
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17183_foreach_fun21369__t_3 {
  struct kk_function_s _base;
  kk_function_t action_170710;
  kk_ssize_t end_170700;
  kk_ssize_t i0;
};
static kk_unit_t kk_std_core__lift17183_foreach_fun21369_3(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17183_foreach_fun21369_3(kk_function_t action_170710, kk_ssize_t end_170700, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17183_foreach_fun21369__t_3* _self = kk_function_alloc_as(struct kk_std_core__lift17183_foreach_fun21369__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17183_foreach_fun21369_3, kk_context());
  _self->action_170710 = action_170710;
  _self->end_170700 = end_170700;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17183_foreach_fun21369_3(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17183_foreach_fun21369__t_3* _self = kk_function_as(struct kk_std_core__lift17183_foreach_fun21369__t_3*, _fself);
  kk_function_t action_170710 = _self->action_170710; /* (ssize_t) -> 11092 () */
  kk_ssize_t end_170700 = _self->end_170700; /* ssize_t */
  kk_ssize_t i0 = _self->i0; /* ssize_t */
  kk_drop_match(_self, {kk_function_dup(action_170710);;;}, {}, _ctx)
  return kk_std_core__mlift17738_op(action_170710, end_170700, i0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17183_foreach_fun21371__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_19156_19154;
};
static kk_box_t kk_std_core__lift17183_foreach_fun21371_3(kk_function_t _fself, kk_box_t _b_19155, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17183_foreach_fun21371_3(kk_function_t _b_19156_19154, kk_context_t* _ctx) {
  struct kk_std_core__lift17183_foreach_fun21371__t_3* _self = kk_function_alloc_as(struct kk_std_core__lift17183_foreach_fun21371__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17183_foreach_fun21371_3, kk_context());
  _self->_b_19156_19154 = _b_19156_19154;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17183_foreach_fun21371_3(kk_function_t _fself, kk_box_t _b_19155, kk_context_t* _ctx) {
  struct kk_std_core__lift17183_foreach_fun21371__t_3* _self = kk_function_as(struct kk_std_core__lift17183_foreach_fun21371__t_3*, _fself);
  kk_function_t _b_19156_19154 = _self->_b_19156_19154; /* (wild_0 : ()) -> 11092 () */
  kk_drop_match(_self, {kk_function_dup(_b_19156_19154);}, {}, _ctx)
  kk_unit_t _x21372 = kk_Unit;
  kk_unit_t _x21373 = kk_Unit;
  kk_unit_unbox(_b_19155);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19156_19154, (_b_19156_19154, _x21373, _ctx));
  return kk_unit_box(_x21372);
}

kk_unit_t kk_std_core__lift17183_foreach_3(kk_function_t action_170710, kk_ssize_t end_170700, kk_ssize_t i0, kk_context_t* _ctx) { /* forall<e> (action.17071 : (ssize_t) -> e (), end.17070 : ssize_t, i : ssize_t) -> e () */ 
  kk__tailcall: ;
  bool _match_20077 = (i0 <= end_170700); /*bool*/;
  if (_match_20077) {
    kk_unit_t x_18056 = kk_Unit;
    kk_function_t _x21368 = kk_function_dup(action_170710); /*(ssize_t) -> 11092 ()*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_ssize_t, kk_context_t*), _x21368, (_x21368, i0, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19156_19154 = kk_std_core__new_lift17183_foreach_fun21369_3(action_170710, end_170700, i0, _ctx); /*(wild_0 : ()) -> 11092 ()*/;
      kk_box_t _x21370 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17183_foreach_fun21371_3(_b_19156_19154, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x21370); return kk_Unit;
    }
    kk_ssize_t i0_172460 = kk_std_core_incr_1(i0, _ctx); /*ssize_t*/;
    { // tailcall
      i0 = i0_172460;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action_170710, _ctx);
  kk_Unit; return kk_Unit;
}
 
// Invoke a function `f` for each element in a vector `v`


// lift anonymous function
struct kk_std_core_foreach_fun21375__t_3 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_vector_t v;
};
static kk_unit_t kk_std_core_foreach_fun21375_3(kk_function_t _fself, kk_ssize_t i0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_fun21375_3(kk_function_t f, kk_vector_t v, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun21375__t_3* _self = kk_function_alloc_as(struct kk_std_core_foreach_fun21375__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_fun21375_3, kk_context());
  _self->f = f;
  _self->v = v;
  return &_self->_base;
}

static kk_unit_t kk_std_core_foreach_fun21375_3(kk_function_t _fself, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_fun21375__t_3* _self = kk_function_as(struct kk_std_core_foreach_fun21375__t_3*, _fself);
  kk_function_t f = _self->f; /* (11091) -> 11092 () */
  kk_vector_t v = _self->v; /* vector<11091> */
  kk_drop_match(_self, {kk_function_dup(f);kk_vector_dup(v);}, {}, _ctx)
  kk_box_t x_17248;
  kk_box_t _borrowApp_20076 = kk_vector_at_borrow(v, i0); /*219*/;
  kk_vector_drop(v, _ctx);
  x_17248 = _borrowApp_20076; /*11091*/
  return kk_function_call(kk_unit_t, (kk_function_t, kk_box_t, kk_context_t*), f, (f, x_17248, _ctx));
}

kk_unit_t kk_std_core_foreach_3(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (v : vector<a>, f : (a) -> e ()) -> e () */ 
  kk_ssize_t start0_17069 = ((kk_ssize_t)0); /*ssize_t*/;
  kk_ssize_t end_17070;
  kk_ssize_t _x21374 = kk_vector_len_borrow(v); /*ssize_t*/
  end_17070 = kk_std_core_decr_1(_x21374, _ctx); /*ssize_t*/
  kk_std_core__lift17183_foreach_3(kk_std_core_new_foreach_fun21375_3(f, v, _ctx), end_17070, start0_17069, _ctx); return kk_Unit;
}
 
// O(n). Return the number of characters in a string.

kk_integer_t kk_std_core_count_1(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> int */ 
  return kk_string_count_int(s,kk_context());
}
 
// O(n). Return the number of characters in a string slice

kk_integer_t kk_std_core_count_2(kk_std_core__sslice slice0, kk_context_t* _ctx) { /* (slice : sslice) -> int */ 
  return kk_slice_count(slice0,kk_context());
}
 
// lifted
// specialized std/core/foreach-while.1 to parameters [action]

kk_std_core_types__maybe kk_std_core__lift17184_count_3(kk_function_t action1_17076, kk_std_core__sslice slice0, kk_context_t* _ctx) { /* forall<_a,h> (action1.17076 : (c : char) -> (local<h>) (), slice0 : sslice) -> (local<h>) maybe<_a> */ 
  kk__tailcall: ;
  kk_std_core_types__maybe _match_20075 = kk_std_core_next(slice0, _ctx); /*maybe<(char, sslice)>*/;
  if (kk_std_core_types__is_Nothing(_match_20075)) {
    kk_function_drop(action1_17076, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_box_t _box_x19157 = _match_20075._cons.Just.value;
  kk_std_core_types__tuple2_ _pat10 = kk_std_core_types__tuple2__unbox(_box_x19157, NULL);
  kk_box_t _box_x19158 = _pat10.fst;
  kk_box_t _box_x19159 = _pat10.snd;
  kk_char_t c1 = kk_char_unbox(_box_x19158, NULL);
  kk_std_core__sslice rest = kk_std_core__sslice_unbox(_box_x19159, NULL);
  kk_std_core__sslice_dup(rest);
  kk_std_core_types__maybe_drop(_match_20075, _ctx);
  kk_Unit;
  kk_unit_t __ = kk_Unit;
  kk_function_t _x21379 = kk_function_dup(action1_17076); /*(c : char) -> (local<11285>) ()*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_char_t, kk_context_t*), _x21379, (_x21379, c1, _ctx));
  kk_std_core_types__maybe _pat5 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/;
  kk_std_core_types__maybe _scrut = _pat5; /*maybe<_11024>*/;
  kk_std_core_types__maybe_drop(_scrut, _ctx);
  { // tailcall
    slice0 = rest;
    goto kk__tailcall;
  }
}
 
// Count the number of times a predicate is true for each character in a string


// lift anonymous function
struct kk_std_core_count_fun21384__t_3 {
  struct kk_function_s _base;
  kk_ref_t loc;
  kk_function_t pred;
};
static kk_unit_t kk_std_core_count_fun21384_3(kk_function_t _fself, kk_char_t c0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_count_fun21384_3(kk_ref_t loc, kk_function_t pred, kk_context_t* _ctx) {
  struct kk_std_core_count_fun21384__t_3* _self = kk_function_alloc_as(struct kk_std_core_count_fun21384__t_3, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_count_fun21384_3, kk_context());
  _self->loc = loc;
  _self->pred = pred;
  return &_self->_base;
}

static kk_unit_t kk_std_core_count_fun21384_3(kk_function_t _fself, kk_char_t c0, kk_context_t* _ctx) {
  struct kk_std_core_count_fun21384__t_3* _self = kk_function_as(struct kk_std_core_count_fun21384__t_3*, _fself);
  kk_ref_t loc = _self->loc; /* local-var<global,int> */
  kk_function_t pred = _self->pred; /* (char) -> bool */
  kk_drop_match(_self, {kk_ref_dup(loc);kk_function_dup(pred);}, {}, _ctx)
  bool _match_20073 = kk_function_call(bool, (kk_function_t, kk_char_t, kk_context_t*), pred, (pred, c0, _ctx)); /*bool*/;
  if (_match_20073) {
    kk_integer_t _b_19166_19164;
    kk_integer_t _x21385;
    kk_box_t _x21386;
    kk_ref_t _x21387 = kk_ref_dup(loc); /*local-var<global,int>*/
    _x21386 = (kk_ref_get(_x21387,kk_context())); /*256*/
    _x21385 = kk_integer_unbox(_x21386); /*int*/
    _b_19166_19164 = kk_integer_add(_x21385,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_unit_t _borrowApp_20074 = kk_Unit;
    (kk_ref_set_borrow(loc,(kk_integer_box(_b_19166_19164)),kk_context()));
    kk_ref_drop(loc, _ctx);
    return _borrowApp_20074;
  }
  kk_ref_drop(loc, _ctx);
  return kk_Unit;
}

kk_integer_t kk_std_core_count_3(kk_string_t s, kk_function_t pred, kk_context_t* _ctx) { /* (s : string, pred : (char) -> bool) -> int */ 
  kk_ref_t loc = kk_ref_alloc((kk_integer_box(kk_integer_from_small(0))),kk_context()); /*local-var<global,int>*/;
  kk_std_core__sslice slice0_17075;
  kk_string_t _x21380 = kk_string_dup(s); /*string*/
  kk_ssize_t _x21381 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x21382 = kk_string_len(s,kk_context()); /*ssize_t*/
  slice0_17075 = kk_std_core__new_Sslice(_x21380, _x21381, _x21382, _ctx); /*sslice*/
  kk_std_core_types__maybe __0;
  kk_function_t _x21383;
  kk_ref_dup(loc);
  _x21383 = kk_std_core_new_count_fun21384_3(loc, pred, _ctx); /*(c0 : char) -> (local<global>) ()*/
  __0 = kk_std_core__lift17184_count_3(_x21383, slice0_17075, _ctx); /*maybe<_11024>*/
  kk_std_core_types__maybe_drop(__0, _ctx);
  kk_integer_t res;
  kk_box_t _x21388;
  kk_ref_t _x21389 = kk_ref_dup(loc); /*local-var<global,int>*/
  _x21388 = (kk_ref_get(_x21389,kk_context())); /*256*/
  res = kk_integer_unbox(_x21388); /*int*/
  kk_ref_t _b_19172_19170 = loc; /*local-var<global,int>*/;
  kk_integer_t _b_19173_19171 = res; /*int*/;
  kk_box_t _x21390 = kk_std_core_hnd_prompt_local_var(_b_19172_19170, kk_integer_box(_b_19173_19171), _ctx); /*9895*/
  return kk_integer_unbox(_x21390);
}
 
// Is the integer negative (stricly smaller than zero)

bool kk_std_core_is_neg_2(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> bool */ 
  kk_std_core_types__order x_17078;
  kk_std_core_types__order _borrowApp_20072 = kk_int_as_order(kk_integer_signum_borrow(i),kk_context()); /*order*/;
  kk_integer_drop(i, _ctx);
  x_17078 = _borrowApp_20072; /*order*/
  kk_integer_t _borrowedArg_20070;
  if (kk_std_core_types__is_Lt(x_17078)) {
    _borrowedArg_20070 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match21392;
  }
  if (kk_std_core_types__is_Eq(x_17078)) {
    _borrowedArg_20070 = kk_integer_from_small(0); /*int*/
    goto _match21392;
  }
  _borrowedArg_20070 = kk_integer_from_small(1); /*int*/
  _match21392: ;
  kk_integer_t _borrowedArg_20069 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/;
  bool _borrowApp_20071 = kk_integer_eq_borrow(_borrowedArg_20070,_borrowedArg_20069,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20070, _ctx);
  kk_integer_drop(_borrowedArg_20069, _ctx);
  return _borrowApp_20071;
}

kk_std_core_types__tuple2_ kk_std_core_divmod_exp10(kk_integer_t i, kk_integer_t n, kk_context_t* _ctx) { /* (i : int, n : int) -> (int, int) */ 
  kk_std_core_types__tuple2_ _match_20068;
  kk_integer_t _x21393 = kk_integer_dup(n); /*int*/
  _match_20068 = kk_std_core_cdivmod_exp10(i, _x21393, _ctx); /*(int, int)*/
  kk_box_t _box_x19179 = _match_20068.fst;
  kk_box_t _box_x19180 = _match_20068.snd;
  kk_integer_t cq = kk_integer_unbox(_box_x19179);
  kk_integer_t cr = kk_integer_unbox(_box_x19180);
  kk_integer_dup(cq);
  kk_integer_dup(cr);
  kk_std_core_types__tuple2__drop(_match_20068, _ctx);
  kk_Unit;
  bool b_17082;
  kk_integer_t _x21396 = kk_integer_dup(cr); /*int*/
  b_17082 = kk_std_core_is_neg_2(_x21396, _ctx); /*bool*/
  if (b_17082) {
    kk_integer_t _b_19185_19181 = kk_integer_sub(cq,(kk_integer_from_small(1)),kk_context()); /*int*/;
    kk_integer_t _b_19186_19182;
    kk_integer_t _x21397 = kk_std_core_mul_exp10(kk_integer_from_small(1), n, _ctx); /*int*/
    _b_19186_19182 = kk_integer_add(cr,_x21397,kk_context()); /*int*/
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_19185_19181), kk_integer_box(_b_19186_19182), _ctx);
  }
  kk_integer_drop(n, _ctx);
  kk_integer_t _b_19187_19183 = cq; /*int*/;
  kk_integer_t _b_19188_19184 = cr; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_19187_19183), kk_integer_box(_b_19188_19184), _ctx);
}
 
// Drop the first `n` elements of a list (or fewer if the list is shorter than `n`)

kk_std_core__list kk_std_core_drop(kk_std_core__list xs, kk_integer_t n, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21398 = kk_std_core__as_Cons(xs);
    kk_box_t _pat0 = _con21398->head;
    kk_std_core__list xx = _con21398->tail;
    kk_integer_t _borrowedArg_20066 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_20067 = kk_integer_gt_borrow(n,_borrowedArg_20066,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_20066, _ctx);
    if (_borrowApp_20067) {
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        kk_box_drop(_pat0, _ctx);
        kk_Unit;
        kk_Unit;
        kk_std_core__list_free(xs);
      }
      else {
        kk_std_core__list_dup(xx);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      { // tailcall
        kk_integer_t _x21399 = kk_integer_sub(n,(kk_integer_from_small(1)),kk_context()); /*int*/
        xs = xx;
        n = _x21399;
        goto kk__tailcall;
      }
    }
  }
  kk_integer_drop(n, _ctx);
  return xs;
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17739_drop_while(kk_function_t predicate, kk_std_core__list xs, kk_std_core__list xx, bool _y_17517, kk_context_t* _ctx) { /* forall<a,e> (predicate : (a) -> e bool, xs : list<a>, xx : list<a>, bool) -> e list<a> */ 
  if (_y_17517) {
    kk_std_core__list_drop(xs, _ctx);
    return kk_std_core_drop_while(xx, predicate, _ctx);
  }
  kk_function_drop(predicate, _ctx);
  kk_std_core__list_drop(xx, _ctx);
  return xs;
}
 
// Drop all initial elements that satisfy `predicate`


// lift anonymous function
struct kk_std_core_drop_while_fun21402__t {
  struct kk_function_s _base;
  kk_function_t predicate0;
  kk_std_core__list xs0;
  kk_std_core__list xx0;
};
static kk_std_core__list kk_std_core_drop_while_fun21402(kk_function_t _fself, bool _y_175170, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_drop_while_fun21402(kk_function_t predicate0, kk_std_core__list xs0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_drop_while_fun21402__t* _self = kk_function_alloc_as(struct kk_std_core_drop_while_fun21402__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_drop_while_fun21402, kk_context());
  _self->predicate0 = predicate0;
  _self->xs0 = xs0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core_drop_while_fun21402(kk_function_t _fself, bool _y_175170, kk_context_t* _ctx) {
  struct kk_std_core_drop_while_fun21402__t* _self = kk_function_as(struct kk_std_core_drop_while_fun21402__t*, _fself);
  kk_function_t predicate0 = _self->predicate0; /* (11553) -> 11554 bool */
  kk_std_core__list xs0 = _self->xs0; /* list<11553> */
  kk_std_core__list xx0 = _self->xx0; /* list<11553> */
  kk_drop_match(_self, {kk_function_dup(predicate0);kk_std_core__list_dup(xs0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17739_drop_while(predicate0, xs0, xx0, _y_175170, _ctx);
}


// lift anonymous function
struct kk_std_core_drop_while_fun21404__t {
  struct kk_function_s _base;
  kk_function_t _b_19191_19189;
};
static kk_box_t kk_std_core_drop_while_fun21404(kk_function_t _fself, kk_box_t _b_19190, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_drop_while_fun21404(kk_function_t _b_19191_19189, kk_context_t* _ctx) {
  struct kk_std_core_drop_while_fun21404__t* _self = kk_function_alloc_as(struct kk_std_core_drop_while_fun21404__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_drop_while_fun21404, kk_context());
  _self->_b_19191_19189 = _b_19191_19189;
  return &_self->_base;
}

static kk_box_t kk_std_core_drop_while_fun21404(kk_function_t _fself, kk_box_t _b_19190, kk_context_t* _ctx) {
  struct kk_std_core_drop_while_fun21404__t* _self = kk_function_as(struct kk_std_core_drop_while_fun21404__t*, _fself);
  kk_function_t _b_19191_19189 = _self->_b_19191_19189; /* (bool) -> 11554 list<11553> */
  kk_drop_match(_self, {kk_function_dup(_b_19191_19189);}, {}, _ctx)
  kk_std_core__list _x21405;
  bool _x21406 = kk_bool_unbox(_b_19190); /*bool*/
  _x21405 = kk_function_call(kk_std_core__list, (kk_function_t, bool, kk_context_t*), _b_19191_19189, (_b_19191_19189, _x21406, _ctx)); /*list<11553>*/
  return kk_std_core__list_box(_x21405, _ctx);
}

kk_std_core__list kk_std_core_drop_while(kk_std_core__list xs0, kk_function_t predicate0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21400 = kk_std_core__as_Cons(xs0);
    kk_box_t x = _con21400->head;
    kk_std_core__list xx0 = _con21400->tail;
    kk_box_dup(x);
    kk_std_core__list_dup(xx0);
    bool x0_18061;
    kk_function_t _x21401 = kk_function_dup(predicate0); /*(11553) -> 11554 bool*/
    x0_18061 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21401, (_x21401, x, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19191_19189 = kk_std_core_new_drop_while_fun21402(predicate0, xs0, xx0, _ctx); /*(bool) -> 11554 list<11553>*/;
      kk_box_t _x21403 = kk_std_core_hnd_yield_extend(kk_std_core_new_drop_while_fun21404(_b_19191_19189, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21403, _ctx);
    }
    if (x0_18061) {
      kk_std_core__list_dropn(xs0, ((int32_t)2), _ctx);
      { // tailcall
        xs0 = xx0;
        goto kk__tailcall;
      }
    }
    kk_function_drop(predicate0, _ctx);
    kk_std_core__list_drop(xx0, _ctx);
    return xs0;
  }
  kk_function_drop(predicate0, _ctx);
  return kk_std_core__new_Nil(_ctx);
}
 
// An empty slice

kk_std_core__sslice kk_std_core_empty;
 
// Does string `s`  end with `post`?
// If so, returns a slice of `s` from the start up to the `post` string at the end.

kk_std_core_types__maybe kk_std_core_ends_with(kk_string_t s, kk_string_t post, kk_context_t* _ctx) { /* (s : string, post : string) -> maybe<sslice> */ 
  bool _match_20064;
  kk_string_t _x21411 = kk_string_dup(s); /*string*/
  kk_string_t _x21412 = kk_string_dup(post); /*string*/
  _match_20064 = kk_std_core_xends_with(_x21411, _x21412, _ctx); /*bool*/
  if (_match_20064) {
    kk_std_core__sslice _b_19193_19192;
    kk_string_t _x21413 = kk_string_dup(s); /*string*/
    kk_ssize_t _x21414 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x21415;
    kk_ssize_t _x21416 = kk_string_len(s,kk_context()); /*ssize_t*/
    kk_ssize_t _x21417 = kk_string_len(post,kk_context()); /*ssize_t*/
    _x21415 = (_x21416 - _x21417); /*ssize_t*/
    _b_19193_19192 = kk_std_core__new_Sslice(_x21413, _x21414, _x21415, _ctx); /*sslice*/
    return kk_std_core_types__new_Just(kk_std_core__sslice_box(_b_19193_19192, _ctx), _ctx);
  }
  kk_string_drop(post, _ctx);
  kk_string_drop(s, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Throw an exception with a specified message.

kk_box_t kk_std_core_throw(kk_string_t message0, kk_std_core_types__optional info0, kk_context_t* _ctx) { /* forall<a> (message : string, info : optional<exception-info>) -> exn a */ 
  kk_std_core_hnd__ev ev_18064;
  kk_ssize_t _x21418 = ((kk_ssize_t)0); /*ssize_t*/
  ev_18064 = kk_evv_at(_x21418,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
  struct kk_std_core_hnd_Ev* _con21419 = kk_std_core_hnd__as_Ev(ev_18064);
  kk_std_core_hnd__marker m0 = _con21419->marker;
  kk_box_t _box_x19194 = _con21419->hnd;
  kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x19194, NULL);
  kk_std_core__hnd_exn_dup(h);
  kk_std_core_hnd__clause1 _match_20062;
  kk_std_core_hnd__clause1 _borrowApp_20063 = kk_std_core__select_throw_exn(h, _ctx); /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/;
  kk_std_core__hnd_exn_drop(h, _ctx);
  _match_20062 = _borrowApp_20063; /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/
  kk_function_t _fun_unbox_x19198 = _match_20062.clause;
  kk_std_core_hnd__marker _b_19199 = m0; /*std/core/hnd/marker<3573,3574>*/;
  kk_std_core_hnd__ev _b_19200 = ev_18064; /*std/core/hnd/ev<.hnd-exn>*/;
  kk_std_core__exception _b_19201;
  kk_std_core__exception_info _x21421;
  if (kk_std_core_types__is_Optional(info0)) {
    kk_box_t _box_x19202 = info0._cons.Optional.value;
    kk_std_core__exception_info _info_11639 = kk_std_core__exception_info_unbox(_box_x19202, NULL);
    kk_std_core__exception_info_dup(_info_11639);
    kk_std_core_types__optional_drop(info0, _ctx);
    kk_Unit;
    _x21421 = _info_11639; /*exception-info*/
    goto _match21422;
  }
  _x21421 = kk_std_core__new_ExnError(_ctx); /*exception-info*/
  _match21422: ;
  _b_19201 = kk_std_core__new_Exception(message0, _x21421, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x19198, (_fun_unbox_x19198, _b_19199, _b_19200, kk_std_core__exception_box(_b_19201, _ctx), _ctx));
}
 
// Transform an `:error` type back to an `exn` effect.

kk_box_t kk_std_core_throw_1(kk_std_core__error err, kk_context_t* _ctx) { /* forall<a> (err : error<a>) -> exn a */ 
  if (kk_std_core__is_Error(err)) {
    kk_std_core__exception exn0 = err._cons.Error.exception;
    kk_std_core__exception_dup(exn0);
    kk_std_core__error_drop(err, _ctx);
    kk_Unit;
    kk_std_core_hnd__ev ev_18067;
    kk_ssize_t _x21424 = ((kk_ssize_t)0); /*ssize_t*/
    ev_18067 = kk_evv_at(_x21424,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
    struct kk_std_core_hnd_Ev* _con21425 = kk_std_core_hnd__as_Ev(ev_18067);
    kk_std_core_hnd__marker m0 = _con21425->marker;
    kk_box_t _box_x19203 = _con21425->hnd;
    kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x19203, NULL);
    kk_std_core__hnd_exn_dup(h);
    kk_std_core_hnd__clause1 _match_20060;
    kk_std_core_hnd__clause1 _borrowApp_20061 = kk_std_core__select_throw_exn(h, _ctx); /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/;
    kk_std_core__hnd_exn_drop(h, _ctx);
    _match_20060 = _borrowApp_20061; /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/
    kk_function_t _fun_unbox_x19207 = _match_20060.clause;
    kk_std_core_hnd__marker _b_19208 = m0; /*std/core/hnd/marker<3573,3574>*/;
    kk_std_core_hnd__ev _b_19209 = ev_18067; /*std/core/hnd/ev<.hnd-exn>*/;
    kk_std_core__exception _b_19210 = exn0; /*exception*/;
    return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x19207, (_fun_unbox_x19207, _b_19208, _b_19209, kk_std_core__exception_box(_b_19210, _ctx), _ctx));
  }
  kk_box_t x0 = err._cons.Ok.result;
  kk_box_dup(x0);
  kk_std_core__error_drop(err, _ctx);
  kk_Unit;
  return x0;
}
 
// monadic lift

kk_box_t kk_std_core__mlift17740_error_pattern(kk_string_t definition, kk_string_t location, kk_string_t _c_17524, kk_context_t* _ctx) { /* forall<a> (definition : string, location : string, string) -> a */ 
  kk_string_t message0_17087;
  kk_string_t _x21427 = kk_string_dup(location); /*string*/
  kk_string_t _x21428;
  kk_string_t _x21429;
  kk_define_string_literal(, _s21430, 23, ": pattern match failure")
  _x21429 = kk_string_dup(_s21430); /*string*/
  _x21428 = kk_std_core__lp__plus__plus__1_rp_(_c_17524, _x21429, _ctx); /*string*/
  message0_17087 = kk_std_core__lp__plus__plus__1_rp_(_x21427, _x21428, _ctx); /*string*/
  kk_std_core_hnd__ev ev_18070;
  kk_ssize_t _x21431 = ((kk_ssize_t)0); /*ssize_t*/
  ev_18070 = kk_evv_at(_x21431,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
  struct kk_std_core_hnd_Ev* _con21432 = kk_std_core_hnd__as_Ev(ev_18070);
  kk_std_core_hnd__marker m0 = _con21432->marker;
  kk_box_t _box_x19211 = _con21432->hnd;
  kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x19211, NULL);
  kk_std_core__hnd_exn_dup(h);
  kk_std_core_hnd__clause1 _match_20058;
  kk_std_core_hnd__clause1 _borrowApp_20059 = kk_std_core__select_throw_exn(h, _ctx); /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/;
  kk_std_core__hnd_exn_drop(h, _ctx);
  _match_20058 = _borrowApp_20059; /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/
  kk_function_t _fun_unbox_x19215 = _match_20058.clause;
  kk_std_core_hnd__marker _b_19216 = m0; /*std/core/hnd/marker<3573,3574>*/;
  kk_std_core_hnd__ev _b_19217 = ev_18070; /*std/core/hnd/ev<.hnd-exn>*/;
  kk_std_core__exception _b_19218;
  kk_std_core__exception_info _x21434 = kk_std_core__new_ExnPattern(kk_reuse_null, location, definition, _ctx); /*exception-info*/
  _b_19218 = kk_std_core__new_Exception(message0_17087, _x21434, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x19215, (_fun_unbox_x19215, _b_19216, _b_19217, kk_std_core__exception_box(_b_19218, _ctx), _ctx));
}
 
// Raise a pattern match exception. This is function is used internally by the
// compiler to generate error messages on pattern match failures.


// lift anonymous function
struct kk_std_core_error_pattern_fun21442__t {
  struct kk_function_s _base;
  kk_string_t definition;
  kk_string_t location;
};
static kk_box_t kk_std_core_error_pattern_fun21442(kk_function_t _fself, kk_string_t _c_17524, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_error_pattern_fun21442(kk_string_t definition, kk_string_t location, kk_context_t* _ctx) {
  struct kk_std_core_error_pattern_fun21442__t* _self = kk_function_alloc_as(struct kk_std_core_error_pattern_fun21442__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_error_pattern_fun21442, kk_context());
  _self->definition = definition;
  _self->location = location;
  return &_self->_base;
}

static kk_box_t kk_std_core_error_pattern_fun21442(kk_function_t _fself, kk_string_t _c_17524, kk_context_t* _ctx) {
  struct kk_std_core_error_pattern_fun21442__t* _self = kk_function_as(struct kk_std_core_error_pattern_fun21442__t*, _fself);
  kk_string_t definition = _self->definition; /* string */
  kk_string_t location = _self->location; /* string */
  kk_drop_match(_self, {kk_string_dup(definition);kk_string_dup(location);}, {}, _ctx)
  return kk_std_core__mlift17740_error_pattern(definition, location, _c_17524, _ctx);
}


// lift anonymous function
struct kk_std_core_error_pattern_fun21443__t {
  struct kk_function_s _base;
  kk_function_t _b_19229_19219;
};
static kk_box_t kk_std_core_error_pattern_fun21443(kk_function_t _fself, kk_box_t _b_19220, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_error_pattern_fun21443(kk_function_t _b_19229_19219, kk_context_t* _ctx) {
  struct kk_std_core_error_pattern_fun21443__t* _self = kk_function_alloc_as(struct kk_std_core_error_pattern_fun21443__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_error_pattern_fun21443, kk_context());
  _self->_b_19229_19219 = _b_19229_19219;
  return &_self->_base;
}

static kk_box_t kk_std_core_error_pattern_fun21443(kk_function_t _fself, kk_box_t _b_19220, kk_context_t* _ctx) {
  struct kk_std_core_error_pattern_fun21443__t* _self = kk_function_as(struct kk_std_core_error_pattern_fun21443__t*, _fself);
  kk_function_t _b_19229_19219 = _self->_b_19229_19219; /* (string) -> 11784 */
  kk_drop_match(_self, {kk_function_dup(_b_19229_19219);}, {}, _ctx)
  kk_string_t _x21444 = kk_string_unbox(_b_19220); /*string*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_string_t, kk_context_t*), _b_19229_19219, (_b_19229_19219, _x21444, _ctx));
}

kk_box_t kk_std_core_error_pattern(kk_string_t location, kk_string_t definition, kk_context_t* _ctx) { /* forall<a> (location : string, definition : string) -> exn a */ 
  kk_string_t x_18073;
  bool _match_20057;
  kk_string_t _x21435 = kk_string_dup(definition); /*string*/
  kk_string_t _x21436 = kk_string_empty(); /*string*/
  _match_20057 = kk_string_is_eq(_x21435,_x21436,kk_context()); /*bool*/
  if (_match_20057) {
    x_18073 = kk_string_empty(); /*string*/
  }
  else {
    kk_string_t _x21439;
    kk_define_string_literal(, _s21440, 2, ": ")
    _x21439 = kk_string_dup(_s21440); /*string*/
    kk_string_t _x21441 = kk_string_dup(definition); /*string*/
    x_18073 = kk_std_core__lp__plus__plus__1_rp_(_x21439, _x21441, _ctx); /*string*/
  }
  if (kk_yielding(kk_context())) {
    kk_string_drop(x_18073, _ctx);
    kk_function_t _b_19229_19219 = kk_std_core_new_error_pattern_fun21442(definition, location, _ctx); /*(string) -> 11784*/;
    return kk_std_core_hnd_yield_extend(kk_std_core_new_error_pattern_fun21443(_b_19229_19219, _ctx), _ctx);
  }
  kk_string_t message0_17087;
  kk_string_t _x21445 = kk_string_dup(location); /*string*/
  kk_string_t _x21446;
  kk_string_t _x21447;
  kk_define_string_literal(, _s21448, 23, ": pattern match failure")
  _x21447 = kk_string_dup(_s21448); /*string*/
  _x21446 = kk_std_core__lp__plus__plus__1_rp_(x_18073, _x21447, _ctx); /*string*/
  message0_17087 = kk_std_core__lp__plus__plus__1_rp_(_x21445, _x21446, _ctx); /*string*/
  kk_std_core_hnd__ev ev_18076;
  kk_ssize_t _x21449 = ((kk_ssize_t)0); /*ssize_t*/
  ev_18076 = kk_evv_at(_x21449,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
  struct kk_std_core_hnd_Ev* _con21450 = kk_std_core_hnd__as_Ev(ev_18076);
  kk_std_core_hnd__marker m0 = _con21450->marker;
  kk_box_t _box_x19221 = _con21450->hnd;
  kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x19221, NULL);
  kk_std_core__hnd_exn_dup(h);
  kk_std_core_hnd__clause1 _match_20055;
  kk_std_core_hnd__clause1 _borrowApp_20056 = kk_std_core__select_throw_exn(h, _ctx); /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/;
  kk_std_core__hnd_exn_drop(h, _ctx);
  _match_20055 = _borrowApp_20056; /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/
  kk_function_t _fun_unbox_x19225 = _match_20055.clause;
  kk_std_core_hnd__marker _b_19226 = m0; /*std/core/hnd/marker<3573,3574>*/;
  kk_std_core_hnd__ev _b_19227 = ev_18076; /*std/core/hnd/ev<.hnd-exn>*/;
  kk_std_core__exception _b_19228;
  kk_std_core__exception_info _x21452 = kk_std_core__new_ExnPattern(kk_reuse_null, location, definition, _ctx); /*exception-info*/
  _b_19228 = kk_std_core__new_Exception(message0_17087, _x21452, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x19225, (_fun_unbox_x19225, _b_19226, _b_19227, kk_std_core__exception_box(_b_19228, _ctx), _ctx));
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17741_op(kk_std_core_types__ctail _acc, kk_function_t pred, kk_box_t x, kk_std_core__list xx, bool _y_17527, kk_context_t* _ctx) { /* forall<a,e> (ctail<list<a>>, pred : (a) -> e bool, x : a, xx : list<a>, bool) -> e list<a> */ 
  if (_y_17527) {
    kk_std_core__list _ctail_17337 = kk_std_core__list_hole(); /*list<11850>*/;
    kk_std_core__list _ctail_17338 = kk_std_core__new_Cons(kk_reuse_null, x, _ctail_17337, _ctx); /*list<11850>*/;
    kk_std_core_types__ctail _x21453;
    kk_box_t* field_18081 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17338)->tail)); /*cfield<list<11850>>*/;
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x19236 = _acc._cons.CTail._field1;
      kk_box_t* last0 = _acc._cons.CTail._field2;
      kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19236, NULL);
      kk_std_core__list_dup(head0);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __ = kk_Unit;
      *(last0) = kk_std_core__list_box(_ctail_17338, _ctx);
      kk_std_core__list _b_19245_19241 = head0; /*list<11850>*/;
      kk_box_t* _b_19246_19242 = field_18081; /*cfield<list<11850>>*/;
      _x21453 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19245_19241, _ctx), _b_19246_19242, _ctx); /*ctail<58>*/
      goto _match21454;
    }
    kk_std_core__list _b_19247_19243 = _ctail_17338; /*list<11850>*/;
    kk_box_t* _b_19248_19244 = field_18081; /*cfield<list<11850>>*/;
    _x21453 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19247_19243, _ctx), _b_19248_19244, _ctx); /*ctail<58>*/
    _match21454: ;
    return kk_std_core__ctail_filter(xx, pred, _x21453, _ctx);
  }
  kk_box_drop(x, _ctx);
  return kk_std_core__ctail_filter(xx, pred, _acc, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17742_op_fun21456__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t x0;
};
static kk_std_core__list kk_std_core__mlift17742_op_fun21456(kk_function_t _fself, kk_std_core__list _ctail_17340, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17742_op_fun21456(kk_function_t _accm, kk_box_t x0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17742_op_fun21456__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17742_op_fun21456__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17742_op_fun21456, kk_context());
  _self->_accm = _accm;
  _self->x0 = x0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17742_op_fun21456(kk_function_t _fself, kk_std_core__list _ctail_17340, kk_context_t* _ctx) {
  struct kk_std_core__mlift17742_op_fun21456__t* _self = kk_function_as(struct kk_std_core__mlift17742_op_fun21456__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<11850>) -> list<11850> */
  kk_box_t x0 = _self->x0; /* 11850 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(x0);}, {}, _ctx)
  kk_std_core__list _x21457 = kk_std_core__new_Cons(kk_reuse_null, x0, _ctail_17340, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x21457, _ctx));
}

kk_std_core__list kk_std_core__mlift17742_op(kk_function_t _accm, kk_function_t pred0, kk_box_t x0, kk_std_core__list xx0, bool _y_17532, kk_context_t* _ctx) { /* forall<a,e> ((list<a>) -> list<a>, pred : (a) -> e bool, x : a, xx : list<a>, bool) -> e list<a> */ 
  if (_y_17532) {
    return kk_std_core__ctailm_filter(xx0, pred0, kk_std_core__new_mlift17742_op_fun21456(_accm, x0, _ctx), _ctx);
  }
  kk_box_drop(x0, _ctx);
  return kk_std_core__ctailm_filter(xx0, pred0, _accm, _ctx);
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filter([1,2,3],odd?) == [1,3]`


// lift anonymous function
struct kk_std_core__ctail_filter_fun21461__t {
  struct kk_function_s _base;
  kk_function_t pred1;
  kk_box_t x1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_filter_fun21461(kk_function_t _fself, bool _y_175270, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_filter_fun21461(kk_function_t pred1, kk_box_t x1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_fun21461__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_filter_fun21461__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_filter_fun21461, kk_context());
  _self->pred1 = pred1;
  _self->x1 = x1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_filter_fun21461(kk_function_t _fself, bool _y_175270, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_fun21461__t* _self = kk_function_as(struct kk_std_core__ctail_filter_fun21461__t*, _fself);
  kk_function_t pred1 = _self->pred1; /* (11850) -> 11851 bool */
  kk_box_t x1 = _self->x1; /* 11850 */
  kk_std_core__list xx1 = _self->xx1; /* list<11850> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<11850>> */
  kk_drop_match(_self, {kk_function_dup(pred1);kk_box_dup(x1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17741_op(_acc0, pred1, x1, xx1, _y_175270, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_filter_fun21463__t {
  struct kk_function_s _base;
  kk_function_t _b_19271_19249;
};
static kk_box_t kk_std_core__ctail_filter_fun21463(kk_function_t _fself, kk_box_t _b_19250, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_filter_fun21463(kk_function_t _b_19271_19249, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_fun21463__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_filter_fun21463__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_filter_fun21463, kk_context());
  _self->_b_19271_19249 = _b_19271_19249;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_filter_fun21463(kk_function_t _fself, kk_box_t _b_19250, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_fun21463__t* _self = kk_function_as(struct kk_std_core__ctail_filter_fun21463__t*, _fself);
  kk_function_t _b_19271_19249 = _self->_b_19271_19249; /* (bool) -> 11851 list<11850> */
  kk_drop_match(_self, {kk_function_dup(_b_19271_19249);}, {}, _ctx)
  kk_std_core__list _x21464;
  bool _x21465 = kk_bool_unbox(_b_19250); /*bool*/
  _x21464 = kk_function_call(kk_std_core__list, (kk_function_t, bool, kk_context_t*), _b_19271_19249, (_b_19271_19249, _x21465, _ctx)); /*list<11850>*/
  return kk_std_core__list_box(_x21464, _ctx);
}

kk_std_core__list kk_std_core__ctail_filter(kk_std_core__list xs, kk_function_t pred1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, pred : (a) -> e bool, ctail<list<a>>) -> e list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21458 = kk_std_core__as_Cons(xs);
    kk_box_t x1 = _con21458->head;
    kk_std_core__list xx1 = _con21458->tail;
    kk_reuse_t _ru_20378 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20378 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    bool x2_18082;
    kk_function_t _x21460 = kk_function_dup(pred1); /*(11850) -> 11851 bool*/
    kk_box_t _x21459 = kk_box_dup(x1); /*11850*/
    x2_18082 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21460, (_x21460, _x21459, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20378);
      kk_function_t _b_19271_19249 = kk_std_core__new_ctail_filter_fun21461(pred1, x1, xx1, _acc0, _ctx); /*(bool) -> 11851 list<11850>*/;
      kk_box_t _x21462 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_filter_fun21463(_b_19271_19249, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21462, _ctx);
    }
    if (x2_18082) {
      kk_std_core__list _ctail_173370 = kk_std_core__list_hole(); /*list<11850>*/;
      kk_std_core__list _ctail_173380;
      if (kk_likely(_ru_20378!=NULL)) {
        struct kk_std_core_Cons* _con21466 = (struct kk_std_core_Cons*)_ru_20378;
        _con21466->tail = _ctail_173370;
        _ctail_173380 = kk_std_core__base_Cons(_con21466); /*list<11850>*/
      }
      else {
        _ctail_173380 = kk_std_core__new_Cons(kk_reuse_null, x1, _ctail_173370, _ctx); /*list<11850>*/
      }
      { // tailcall
        kk_std_core_types__ctail _x21467;
        kk_box_t* field0_18087 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_173380)->tail)); /*cfield<list<11850>>*/;
        if (kk_std_core_types__is_CTail(_acc0)) {
          kk_box_t _box_x19257 = _acc0._cons.CTail._field1;
          kk_box_t* last2 = _acc0._cons.CTail._field2;
          kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19257, NULL);
          kk_std_core__list_dup(head1);
          kk_std_core_types__ctail_drop(_acc0, _ctx);
          kk_Unit;
          kk_unit_t __0 = kk_Unit;
          *(last2) = kk_std_core__list_box(_ctail_173380, _ctx);
          kk_std_core__list _b_19272_19262 = head1; /*list<11850>*/;
          kk_box_t* _b_19273_19263 = field0_18087; /*cfield<list<11850>>*/;
          _x21467 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19272_19262, _ctx), _b_19273_19263, _ctx); /*ctail<58>*/
          goto _match21468;
        }
        kk_std_core__list _b_19274_19264 = _ctail_173380; /*list<11850>*/;
        kk_box_t* _b_19275_19265 = field0_18087; /*cfield<list<11850>>*/;
        _x21467 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19274_19264, _ctx), _b_19275_19265, _ctx); /*ctail<58>*/
        _match21468: ;
        xs = xx1;
        _acc0 = _x21467;
        goto kk__tailcall;
      }
    }
    kk_reuse_drop(_ru_20378);
    kk_box_drop(x1, _ctx);
    { // tailcall
      xs = xx1;
      goto kk__tailcall;
    }
  }
  kk_function_drop(pred1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x19266 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19266, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filter([1,2,3],odd?) == [1,3]`


// lift anonymous function
struct kk_std_core__ctailm_filter_fun21474__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t pred2;
  kk_box_t x3;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_filter_fun21474(kk_function_t _fself, bool _y_175320, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_fun21474(kk_function_t _accm0, kk_function_t pred2, kk_box_t x3, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21474__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_fun21474__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_fun21474, kk_context());
  _self->_accm0 = _accm0;
  _self->pred2 = pred2;
  _self->x3 = x3;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_filter_fun21474(kk_function_t _fself, bool _y_175320, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21474__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_fun21474__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<11850>) -> list<11850> */
  kk_function_t pred2 = _self->pred2; /* (11850) -> 11851 bool */
  kk_box_t x3 = _self->x3; /* 11850 */
  kk_std_core__list xx2 = _self->xx2; /* list<11850> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(pred2);kk_box_dup(x3);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17742_op(_accm0, pred2, x3, xx2, _y_175320, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_filter_fun21476__t {
  struct kk_function_s _base;
  kk_function_t _b_19278_19276;
};
static kk_box_t kk_std_core__ctailm_filter_fun21476(kk_function_t _fself, kk_box_t _b_19277, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_fun21476(kk_function_t _b_19278_19276, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21476__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_fun21476__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_fun21476, kk_context());
  _self->_b_19278_19276 = _b_19278_19276;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_filter_fun21476(kk_function_t _fself, kk_box_t _b_19277, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21476__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_fun21476__t*, _fself);
  kk_function_t _b_19278_19276 = _self->_b_19278_19276; /* (bool) -> 11851 list<11850> */
  kk_drop_match(_self, {kk_function_dup(_b_19278_19276);}, {}, _ctx)
  kk_std_core__list _x21477;
  bool _x21478 = kk_bool_unbox(_b_19277); /*bool*/
  _x21477 = kk_function_call(kk_std_core__list, (kk_function_t, bool, kk_context_t*), _b_19278_19276, (_b_19278_19276, _x21478, _ctx)); /*list<11850>*/
  return kk_std_core__list_box(_x21477, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_filter_fun21480__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x3;
};
static kk_std_core__list kk_std_core__ctailm_filter_fun21480(kk_function_t _fself, kk_std_core__list _ctail_173400, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_fun21480(kk_function_t _accm0, kk_box_t x3, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21480__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_fun21480__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_fun21480, kk_context());
  _self->_accm0 = _accm0;
  _self->x3 = x3;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_filter_fun21480(kk_function_t _fself, kk_std_core__list _ctail_173400, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_fun21480__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_fun21480__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<11850>) -> list<11850> */
  kk_box_t x3 = _self->x3; /* 11850 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x3);}, {}, _ctx)
  kk_std_core__list _x21481 = kk_std_core__new_Cons(kk_reuse_null, x3, _ctail_173400, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21481, _ctx));
}

kk_std_core__list kk_std_core__ctailm_filter(kk_std_core__list xs0, kk_function_t pred2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, pred : (a) -> e bool, (list<a>) -> list<a>) -> e list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21471 = kk_std_core__as_Cons(xs0);
    kk_box_t x3 = _con21471->head;
    kk_std_core__list xx2 = _con21471->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x3);
      kk_std_core__list_dup(xx2);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    bool x4_18090;
    kk_function_t _x21473 = kk_function_dup(pred2); /*(11850) -> 11851 bool*/
    kk_box_t _x21472 = kk_box_dup(x3); /*11850*/
    x4_18090 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21473, (_x21473, _x21472, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19278_19276 = kk_std_core__new_ctailm_filter_fun21474(_accm0, pred2, x3, xx2, _ctx); /*(bool) -> 11851 list<11850>*/;
      kk_box_t _x21475 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_filter_fun21476(_b_19278_19276, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21475, _ctx);
    }
    if (x4_18090) {
      { // tailcall
        kk_function_t _x21479 = kk_std_core__new_ctailm_filter_fun21480(_accm0, x3, _ctx); /*(list<11850>) -> list<11850>*/
        xs0 = xx2;
        _accm0 = _x21479;
        goto kk__tailcall;
      }
    }
    kk_box_drop(x3, _ctx);
    { // tailcall
      xs0 = xx2;
      goto kk__tailcall;
    }
  }
  kk_function_drop(pred2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filter([1,2,3],odd?) == [1,3]`


// lift anonymous function
struct kk_std_core_filter_fun21482__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_filter_fun21482(kk_function_t _fself, kk_std_core__list _ctail_17339, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_filter_fun21482(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_filter_fun21482, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_filter_fun21482(kk_function_t _fself, kk_std_core__list _ctail_17339, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17339;
}

kk_std_core__list kk_std_core_filter(kk_std_core__list xs1, kk_function_t pred3, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, pred : (a) -> e bool) -> e list<a> */ 
  bool _match_20051 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_20051) {
    return kk_std_core__ctail_filter(xs1, pred3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_filter(xs1, pred3, kk_std_core_new_filter_fun21482(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17743_op(kk_std_core_types__ctail _acc, kk_function_t pred, kk_std_core__list xx, kk_std_core_types__maybe _y_17540, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, pred : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Nothing(_y_17540)) {
    return kk_std_core__ctail_filter_map(xx, pred, _acc, _ctx);
  }
  kk_box_t y = _y_17540._cons.Just.value;
  kk_std_core__list _ctail_17341 = kk_std_core__list_hole(); /*list<11896>*/;
  kk_std_core__list _ctail_17342 = kk_std_core__new_Cons(kk_reuse_null, y, _ctail_17341, _ctx); /*list<11896>*/;
  kk_std_core_types__ctail _x21483;
  kk_box_t* field_18095 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17342)->tail)); /*cfield<list<11896>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19285 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19285, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17342, _ctx);
    kk_std_core__list _b_19294_19290 = head0; /*list<11896>*/;
    kk_box_t* _b_19295_19291 = field_18095; /*cfield<list<11896>>*/;
    _x21483 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19294_19290, _ctx), _b_19295_19291, _ctx); /*ctail<58>*/
    goto _match21484;
  }
  kk_std_core__list _b_19296_19292 = _ctail_17342; /*list<11896>*/;
  kk_box_t* _b_19297_19293 = field_18095; /*cfield<list<11896>>*/;
  _x21483 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19296_19292, _ctx), _b_19297_19293, _ctx); /*ctail<58>*/
  _match21484: ;
  return kk_std_core__ctail_filter_map(xx, pred, _x21483, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17744_op_fun21486__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t y0;
};
static kk_std_core__list kk_std_core__mlift17744_op_fun21486(kk_function_t _fself, kk_std_core__list _ctail_17344, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17744_op_fun21486(kk_function_t _accm, kk_box_t y0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17744_op_fun21486__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17744_op_fun21486__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17744_op_fun21486, kk_context());
  _self->_accm = _accm;
  _self->y0 = y0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17744_op_fun21486(kk_function_t _fself, kk_std_core__list _ctail_17344, kk_context_t* _ctx) {
  struct kk_std_core__mlift17744_op_fun21486__t* _self = kk_function_as(struct kk_std_core__mlift17744_op_fun21486__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<11896>) -> list<11896> */
  kk_box_t y0 = _self->y0; /* 11896 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(y0);}, {}, _ctx)
  kk_std_core__list _x21487 = kk_std_core__new_Cons(kk_reuse_null, y0, _ctail_17344, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x21487, _ctx));
}

kk_std_core__list kk_std_core__mlift17744_op(kk_function_t _accm, kk_function_t pred0, kk_std_core__list xx0, kk_std_core_types__maybe _y_17545, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, pred : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Nothing(_y_17545)) {
    return kk_std_core__ctailm_filter_map(xx0, pred0, _accm, _ctx);
  }
  kk_box_t y0 = _y_17545._cons.Just.value;
  return kk_std_core__ctailm_filter_map(xx0, pred0, kk_std_core__new_mlift17744_op_fun21486(_accm, y0, _ctx), _ctx);
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filterMap([1,2,3],fn(i) { if (i.odd?) then Nothing else Just(i*i) }) == [4]`


// lift anonymous function
struct kk_std_core__ctail_filter_map_fun21491__t {
  struct kk_function_s _base;
  kk_function_t pred1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_filter_map_fun21491(kk_function_t _fself, kk_std_core_types__maybe _y_175400, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_filter_map_fun21491(kk_function_t pred1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_map_fun21491__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_filter_map_fun21491__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_filter_map_fun21491, kk_context());
  _self->pred1 = pred1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_filter_map_fun21491(kk_function_t _fself, kk_std_core_types__maybe _y_175400, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_map_fun21491__t* _self = kk_function_as(struct kk_std_core__ctail_filter_map_fun21491__t*, _fself);
  kk_function_t pred1 = _self->pred1; /* (11895) -> 11897 maybe<11896> */
  kk_std_core__list xx1 = _self->xx1; /* list<11895> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<11896>> */
  kk_drop_match(_self, {kk_function_dup(pred1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17743_op(_acc0, pred1, xx1, _y_175400, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_filter_map_fun21493__t {
  struct kk_function_s _base;
  kk_function_t _b_19320_19303;
};
static kk_box_t kk_std_core__ctail_filter_map_fun21493(kk_function_t _fself, kk_box_t _b_19304, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_filter_map_fun21493(kk_function_t _b_19320_19303, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_map_fun21493__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_filter_map_fun21493__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_filter_map_fun21493, kk_context());
  _self->_b_19320_19303 = _b_19320_19303;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_filter_map_fun21493(kk_function_t _fself, kk_box_t _b_19304, kk_context_t* _ctx) {
  struct kk_std_core__ctail_filter_map_fun21493__t* _self = kk_function_as(struct kk_std_core__ctail_filter_map_fun21493__t*, _fself);
  kk_function_t _b_19320_19303 = _self->_b_19320_19303; /* (maybe<11896>) -> 11897 list<11896> */
  kk_drop_match(_self, {kk_function_dup(_b_19320_19303);}, {}, _ctx)
  kk_std_core__list _x21494;
  kk_std_core_types__maybe _x21495 = kk_std_core_types__maybe_unbox(_b_19304, _ctx); /*maybe<11896>*/
  _x21494 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19320_19303, (_b_19320_19303, _x21495, _ctx)); /*list<11896>*/
  return kk_std_core__list_box(_x21494, _ctx);
}

kk_std_core__list kk_std_core__ctail_filter_map(kk_std_core__list xs, kk_function_t pred1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, pred : (a) -> e maybe<b>, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(pred1, _ctx);
    if (kk_std_core_types__is_CTail(_acc0)) {
      kk_box_t _box_x19298 = _acc0._cons.CTail._field1;
      kk_box_t* last2 = _acc0._cons.CTail._field2;
      kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19298, NULL);
      kk_std_core__list_dup(head1);
      kk_std_core_types__ctail_drop(_acc0, _ctx);
      kk_Unit;
      kk_unit_t __0 = kk_Unit;
      *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
      return head1;
    }
    return kk_std_core__new_Nil(_ctx);
  }
  struct kk_std_core_Cons* _con21489 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21489->head;
  kk_std_core__list xx1 = _con21489->tail;
  kk_reuse_t _ru_20380 = kk_reuse_null; /*reuse*/;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    _ru_20380 = (kk_std_core__list_reuse(xs));
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx1);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core_types__maybe x0_18098;
  kk_function_t _x21490 = kk_function_dup(pred1); /*(11895) -> 11897 maybe<11896>*/
  x0_18098 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21490, (_x21490, x, _ctx)); /*maybe<11896>*/
  if (kk_yielding(kk_context())) {
    kk_reuse_drop(_ru_20380);
    kk_std_core_types__maybe_drop(x0_18098, _ctx);
    kk_function_t _b_19320_19303 = kk_std_core__new_ctail_filter_map_fun21491(pred1, xx1, _acc0, _ctx); /*(maybe<11896>) -> 11897 list<11896>*/;
    kk_box_t _x21492 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_filter_map_fun21493(_b_19320_19303, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21492, _ctx);
  }
  if (kk_std_core_types__is_Nothing(x0_18098)) {
    kk_reuse_drop(_ru_20380);
    { // tailcall
      xs = xx1;
      goto kk__tailcall;
    }
  }
  kk_box_t y1 = x0_18098._cons.Just.value;
  kk_std_core__list _ctail_173410 = kk_std_core__list_hole(); /*list<11896>*/;
  kk_std_core__list _ctail_173420 = kk_std_core__new_Cons(_ru_20380, y1, _ctail_173410, _ctx); /*list<11896>*/;
  { // tailcall
    kk_std_core_types__ctail _x21496;
    kk_box_t* field0_18103 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_173420)->tail)); /*cfield<list<11896>>*/;
    if (kk_std_core_types__is_CTail(_acc0)) {
      kk_box_t _box_x19311 = _acc0._cons.CTail._field1;
      kk_box_t* last3 = _acc0._cons.CTail._field2;
      kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19311, NULL);
      kk_std_core__list_dup(head2);
      kk_std_core_types__ctail_drop(_acc0, _ctx);
      kk_Unit;
      kk_unit_t __1 = kk_Unit;
      *(last3) = kk_std_core__list_box(_ctail_173420, _ctx);
      kk_std_core__list _b_19321_19316 = head2; /*list<11896>*/;
      kk_box_t* _b_19322_19317 = field0_18103; /*cfield<list<11896>>*/;
      _x21496 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19321_19316, _ctx), _b_19322_19317, _ctx); /*ctail<58>*/
      goto _match21497;
    }
    kk_std_core__list _b_19323_19318 = _ctail_173420; /*list<11896>*/;
    kk_box_t* _b_19324_19319 = field0_18103; /*cfield<list<11896>>*/;
    _x21496 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19323_19318, _ctx), _b_19324_19319, _ctx); /*ctail<58>*/
    _match21497: ;
    xs = xx1;
    _acc0 = _x21496;
    goto kk__tailcall;
  }
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filterMap([1,2,3],fn(i) { if (i.odd?) then Nothing else Just(i*i) }) == [4]`


// lift anonymous function
struct kk_std_core__ctailm_filter_map_fun21501__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t pred2;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_filter_map_fun21501(kk_function_t _fself, kk_std_core_types__maybe _y_175450, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_map_fun21501(kk_function_t _accm0, kk_function_t pred2, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21501__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_map_fun21501__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_map_fun21501, kk_context());
  _self->_accm0 = _accm0;
  _self->pred2 = pred2;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_filter_map_fun21501(kk_function_t _fself, kk_std_core_types__maybe _y_175450, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21501__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_map_fun21501__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<11896>) -> list<11896> */
  kk_function_t pred2 = _self->pred2; /* (11895) -> 11897 maybe<11896> */
  kk_std_core__list xx2 = _self->xx2; /* list<11895> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(pred2);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17744_op(_accm0, pred2, xx2, _y_175450, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_filter_map_fun21503__t {
  struct kk_function_s _base;
  kk_function_t _b_19327_19325;
};
static kk_box_t kk_std_core__ctailm_filter_map_fun21503(kk_function_t _fself, kk_box_t _b_19326, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_map_fun21503(kk_function_t _b_19327_19325, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21503__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_map_fun21503__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_map_fun21503, kk_context());
  _self->_b_19327_19325 = _b_19327_19325;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_filter_map_fun21503(kk_function_t _fself, kk_box_t _b_19326, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21503__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_map_fun21503__t*, _fself);
  kk_function_t _b_19327_19325 = _self->_b_19327_19325; /* (maybe<11896>) -> 11897 list<11896> */
  kk_drop_match(_self, {kk_function_dup(_b_19327_19325);}, {}, _ctx)
  kk_std_core__list _x21504;
  kk_std_core_types__maybe _x21505 = kk_std_core_types__maybe_unbox(_b_19326, _ctx); /*maybe<11896>*/
  _x21504 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19327_19325, (_b_19327_19325, _x21505, _ctx)); /*list<11896>*/
  return kk_std_core__list_box(_x21504, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_filter_map_fun21507__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t y2;
};
static kk_std_core__list kk_std_core__ctailm_filter_map_fun21507(kk_function_t _fself, kk_std_core__list _ctail_173440, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_filter_map_fun21507(kk_function_t _accm0, kk_box_t y2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21507__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_filter_map_fun21507__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_filter_map_fun21507, kk_context());
  _self->_accm0 = _accm0;
  _self->y2 = y2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_filter_map_fun21507(kk_function_t _fself, kk_std_core__list _ctail_173440, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_filter_map_fun21507__t* _self = kk_function_as(struct kk_std_core__ctailm_filter_map_fun21507__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<11896>) -> list<11896> */
  kk_box_t y2 = _self->y2; /* 11896 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(y2);}, {}, _ctx)
  kk_std_core__list _x21508 = kk_std_core__new_Cons(kk_reuse_null, y2, _ctail_173440, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21508, _ctx));
}

kk_std_core__list kk_std_core__ctailm_filter_map(kk_std_core__list xs0, kk_function_t pred2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, pred : (a) -> e maybe<b>, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs0)) {
    kk_function_drop(pred2, _ctx);
    return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
  }
  struct kk_std_core_Cons* _con21499 = kk_std_core__as_Cons(xs0);
  kk_box_t x1 = _con21499->head;
  kk_std_core__list xx2 = _con21499->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs0))) {
    kk_std_core__list_free(xs0);
  }
  else {
    kk_box_dup(x1);
    kk_std_core__list_dup(xx2);
    kk_Unit;
    kk_std_core__list_decref(xs0, _ctx);
    kk_Unit;
  }
  kk_std_core_types__maybe x2_18104;
  kk_function_t _x21500 = kk_function_dup(pred2); /*(11895) -> 11897 maybe<11896>*/
  x2_18104 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21500, (_x21500, x1, _ctx)); /*maybe<11896>*/
  if (kk_yielding(kk_context())) {
    kk_std_core_types__maybe_drop(x2_18104, _ctx);
    kk_function_t _b_19327_19325 = kk_std_core__new_ctailm_filter_map_fun21501(_accm0, pred2, xx2, _ctx); /*(maybe<11896>) -> 11897 list<11896>*/;
    kk_box_t _x21502 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_filter_map_fun21503(_b_19327_19325, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21502, _ctx);
  }
  if (kk_std_core_types__is_Nothing(x2_18104)) {
    { // tailcall
      xs0 = xx2;
      goto kk__tailcall;
    }
  }
  kk_box_t y2 = x2_18104._cons.Just.value;
  { // tailcall
    kk_function_t _x21506 = kk_std_core__new_ctailm_filter_map_fun21507(_accm0, y2, _ctx); /*(list<11896>) -> list<11896>*/
    xs0 = xx2;
    _accm0 = _x21506;
    goto kk__tailcall;
  }
}
 
// Retain only those elements of a list that satisfy the given predicate `pred`.
// For example: `filterMap([1,2,3],fn(i) { if (i.odd?) then Nothing else Just(i*i) }) == [4]`


// lift anonymous function
struct kk_std_core_filter_map_fun21509__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_filter_map_fun21509(kk_function_t _fself, kk_std_core__list _ctail_17343, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_filter_map_fun21509(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_filter_map_fun21509, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_filter_map_fun21509(kk_function_t _fself, kk_std_core__list _ctail_17343, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17343;
}

kk_std_core__list kk_std_core_filter_map(kk_std_core__list xs1, kk_function_t pred3, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, pred : (a) -> e maybe<b>) -> e list<b> */ 
  bool _match_20048 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_20048) {
    return kk_std_core__ctail_filter_map(xs1, pred3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_filter_map(xs1, pred3, kk_std_core_new_filter_map_fun21509(_ctx), _ctx);
}

bool kk_std_core_is_zero_1(kk_ssize_t i, kk_context_t* _ctx) { /* (i : ssize_t) -> bool */ 
  return (i == 0);
}
 
// lifted
// specialized std/core/foreach-while to parameters [action]

kk_std_core_types__maybe kk_std_core__lift17185_find(kk_function_t pred, kk_std_core__list xs0, kk_context_t* _ctx) { /* forall<a> (pred : (a) -> bool, xs0 : list<a>) -> maybe<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs0)) {
    kk_function_drop(pred, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  struct kk_std_core_Cons* _con21510 = kk_std_core__as_Cons(xs0);
  kk_box_t x0 = _con21510->head;
  kk_std_core__list xx = _con21510->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs0))) {
    kk_std_core__list_free(xs0);
  }
  else {
    kk_box_dup(x0);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs0, _ctx);
    kk_Unit;
  }
  bool _match_20047;
  kk_function_t _x21512 = kk_function_dup(pred); /*(11994) -> bool*/
  kk_box_t _x21511 = kk_box_dup(x0); /*11994*/
  _match_20047 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21512, (_x21512, _x21511, _ctx)); /*bool*/
  if (_match_20047) {
    kk_function_drop(pred, _ctx);
    kk_std_core__list_drop(xx, _ctx);
    return kk_std_core_types__new_Just(x0, _ctx);
  }
  kk_box_drop(x0, _ctx);
  kk_std_core_types__maybe _pat10 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/;
  kk_std_core_types__maybe _scrut = _pat10; /*maybe<11994>*/;
  kk_std_core_types__maybe_drop(_scrut, _ctx);
  { // tailcall
    xs0 = xx;
    goto kk__tailcall;
  }
}
 
// O(n). If it occurs, return the position of substring `sub` in `s`, tupled with
// the position just following the substring `sub`.

kk_std_core_types__maybe kk_std_core_find_1(kk_string_t s, kk_string_t sub, kk_context_t* _ctx) { /* (s : string, sub : string) -> maybe<sslice> */ 
  kk_ssize_t i;
  kk_string_t _x21513 = kk_string_dup(s); /*string*/
  kk_string_t _x21514 = kk_string_dup(sub); /*string*/
  i = kk_string_index_of1(_x21513,_x21514,kk_context()); /*ssize_t*/
  bool _match_20046 = kk_std_core_is_zero_1(i, _ctx); /*bool*/;
  if (_match_20046) {
    kk_string_drop(s, _ctx);
    kk_string_drop(sub, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_std_core__sslice _b_19329_19328;
  kk_ssize_t _x21515 = kk_std_core_decr_1(i, _ctx); /*ssize_t*/
  kk_ssize_t _x21516 = kk_string_len(sub,kk_context()); /*ssize_t*/
  _b_19329_19328 = kk_std_core__new_Sslice(s, _x21515, _x21516, _ctx); /*sslice*/
  return kk_std_core_types__new_Just(kk_std_core__sslice_box(_b_19329_19328, _ctx), _ctx);
}
 
// Return the last index of substring `sub` in `s` if it occurs.

kk_std_core_types__maybe kk_std_core_find_last(kk_string_t s, kk_string_t sub, kk_context_t* _ctx) { /* (s : string, sub : string) -> maybe<sslice> */ 
  kk_ssize_t i;
  kk_string_t _x21517 = kk_string_dup(s); /*string*/
  kk_string_t _x21518 = kk_string_dup(sub); /*string*/
  i = kk_string_last_index_of1(_x21517,_x21518,kk_context()); /*ssize_t*/
  bool _match_20045 = kk_std_core_is_zero_1(i, _ctx); /*bool*/;
  if (_match_20045) {
    kk_string_drop(s, _ctx);
    kk_string_drop(sub, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_std_core__sslice _b_19331_19330;
  kk_ssize_t _x21519 = kk_std_core_decr_1(i, _ctx); /*ssize_t*/
  kk_ssize_t _x21520 = kk_string_len(sub,kk_context()); /*ssize_t*/
  _b_19331_19330 = kk_std_core__new_Sslice(s, _x21519, _x21520, _ctx); /*sslice*/
  return kk_std_core_types__new_Just(kk_std_core__sslice_box(_b_19331_19330, _ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17745_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_std_core__list xx, kk_std_core_types__maybe _y_17553, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, f : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Just(_y_17553)) {
    kk_box_t y = _y_17553._cons.Just.value;
    kk_std_core__list _ctail_17345 = kk_std_core__list_hole(); /*list<12200>*/;
    kk_std_core__list _ctail_17346 = kk_std_core__new_Cons(kk_reuse_null, y, _ctail_17345, _ctx); /*list<12200>*/;
    kk_std_core_types__ctail _x21521;
    kk_box_t* field_18109 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17346)->tail)); /*cfield<list<12200>>*/;
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x19338 = _acc._cons.CTail._field1;
      kk_box_t* last0 = _acc._cons.CTail._field2;
      kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19338, NULL);
      kk_std_core__list_dup(head0);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __ = kk_Unit;
      *(last0) = kk_std_core__list_box(_ctail_17346, _ctx);
      kk_std_core__list _b_19347_19343 = head0; /*list<12200>*/;
      kk_box_t* _b_19348_19344 = field_18109; /*cfield<list<12200>>*/;
      _x21521 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19347_19343, _ctx), _b_19348_19344, _ctx); /*ctail<58>*/
      goto _match21522;
    }
    kk_std_core__list _b_19349_19345 = _ctail_17346; /*list<12200>*/;
    kk_box_t* _b_19350_19346 = field_18109; /*cfield<list<12200>>*/;
    _x21521 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19349_19345, _ctx), _b_19350_19346, _ctx); /*ctail<58>*/
    _match21522: ;
    return kk_std_core__ctail_flatmap_maybe(xx, f, _x21521, _ctx);
  }
  return kk_std_core__ctail_flatmap_maybe(xx, f, _acc, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17746_op_fun21524__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t y0;
};
static kk_std_core__list kk_std_core__mlift17746_op_fun21524(kk_function_t _fself, kk_std_core__list _ctail_17348, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17746_op_fun21524(kk_function_t _accm, kk_box_t y0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17746_op_fun21524__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17746_op_fun21524__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17746_op_fun21524, kk_context());
  _self->_accm = _accm;
  _self->y0 = y0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17746_op_fun21524(kk_function_t _fself, kk_std_core__list _ctail_17348, kk_context_t* _ctx) {
  struct kk_std_core__mlift17746_op_fun21524__t* _self = kk_function_as(struct kk_std_core__mlift17746_op_fun21524__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<12200>) -> list<12200> */
  kk_box_t y0 = _self->y0; /* 12200 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(y0);}, {}, _ctx)
  kk_std_core__list _x21525 = kk_std_core__new_Cons(kk_reuse_null, y0, _ctail_17348, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x21525, _ctx));
}

kk_std_core__list kk_std_core__mlift17746_op(kk_function_t _accm, kk_function_t f0, kk_std_core__list xx0, kk_std_core_types__maybe _y_17558, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, f : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Just(_y_17558)) {
    kk_box_t y0 = _y_17558._cons.Just.value;
    return kk_std_core__ctailm_flatmap_maybe(xx0, f0, kk_std_core__new_mlift17746_op_fun21524(_accm, y0, _ctx), _ctx);
  }
  return kk_std_core__ctailm_flatmap_maybe(xx0, f0, _accm, _ctx);
}
 
// Concatenate the `Just` result elements from applying a function to all elements.


// lift anonymous function
struct kk_std_core__ctail_flatmap_maybe_fun21528__t {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_flatmap_maybe_fun21528(kk_function_t _fself, kk_std_core_types__maybe _y_175530, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_flatmap_maybe_fun21528(kk_function_t f1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_flatmap_maybe_fun21528__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_flatmap_maybe_fun21528__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_flatmap_maybe_fun21528, kk_context());
  _self->f1 = f1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_flatmap_maybe_fun21528(kk_function_t _fself, kk_std_core_types__maybe _y_175530, kk_context_t* _ctx) {
  struct kk_std_core__ctail_flatmap_maybe_fun21528__t* _self = kk_function_as(struct kk_std_core__ctail_flatmap_maybe_fun21528__t*, _fself);
  kk_function_t f1 = _self->f1; /* (12199) -> 12201 maybe<12200> */
  kk_std_core__list xx1 = _self->xx1; /* list<12199> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<12200>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17745_op(_acc0, f1, xx1, _y_175530, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_flatmap_maybe_fun21530__t {
  struct kk_function_s _base;
  kk_function_t _b_19373_19351;
};
static kk_box_t kk_std_core__ctail_flatmap_maybe_fun21530(kk_function_t _fself, kk_box_t _b_19352, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_flatmap_maybe_fun21530(kk_function_t _b_19373_19351, kk_context_t* _ctx) {
  struct kk_std_core__ctail_flatmap_maybe_fun21530__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_flatmap_maybe_fun21530__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_flatmap_maybe_fun21530, kk_context());
  _self->_b_19373_19351 = _b_19373_19351;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_flatmap_maybe_fun21530(kk_function_t _fself, kk_box_t _b_19352, kk_context_t* _ctx) {
  struct kk_std_core__ctail_flatmap_maybe_fun21530__t* _self = kk_function_as(struct kk_std_core__ctail_flatmap_maybe_fun21530__t*, _fself);
  kk_function_t _b_19373_19351 = _self->_b_19373_19351; /* (maybe<12200>) -> 12201 list<12200> */
  kk_drop_match(_self, {kk_function_dup(_b_19373_19351);}, {}, _ctx)
  kk_std_core__list _x21531;
  kk_std_core_types__maybe _x21532 = kk_std_core_types__maybe_unbox(_b_19352, _ctx); /*maybe<12200>*/
  _x21531 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19373_19351, (_b_19373_19351, _x21532, _ctx)); /*list<12200>*/
  return kk_std_core__list_box(_x21531, _ctx);
}

kk_std_core__list kk_std_core__ctail_flatmap_maybe(kk_std_core__list xs, kk_function_t f1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e maybe<b>, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21526 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21526->head;
    kk_std_core__list xx1 = _con21526->tail;
    kk_reuse_t _ru_20383 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20383 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_std_core_types__maybe x0_18110;
    kk_function_t _x21527 = kk_function_dup(f1); /*(12199) -> 12201 maybe<12200>*/
    x0_18110 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21527, (_x21527, x, _ctx)); /*maybe<12200>*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20383);
      kk_std_core_types__maybe_drop(x0_18110, _ctx);
      kk_function_t _b_19373_19351 = kk_std_core__new_ctail_flatmap_maybe_fun21528(f1, xx1, _acc0, _ctx); /*(maybe<12200>) -> 12201 list<12200>*/;
      kk_box_t _x21529 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_flatmap_maybe_fun21530(_b_19373_19351, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21529, _ctx);
    }
    if (kk_std_core_types__is_Just(x0_18110)) {
      kk_box_t y1 = x0_18110._cons.Just.value;
      kk_std_core__list _ctail_173450 = kk_std_core__list_hole(); /*list<12200>*/;
      kk_std_core__list _ctail_173460 = kk_std_core__new_Cons(_ru_20383, y1, _ctail_173450, _ctx); /*list<12200>*/;
      { // tailcall
        kk_std_core_types__ctail _x21533;
        kk_box_t* field0_18115 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_173460)->tail)); /*cfield<list<12200>>*/;
        if (kk_std_core_types__is_CTail(_acc0)) {
          kk_box_t _box_x19359 = _acc0._cons.CTail._field1;
          kk_box_t* last2 = _acc0._cons.CTail._field2;
          kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19359, NULL);
          kk_std_core__list_dup(head1);
          kk_std_core_types__ctail_drop(_acc0, _ctx);
          kk_Unit;
          kk_unit_t __0 = kk_Unit;
          *(last2) = kk_std_core__list_box(_ctail_173460, _ctx);
          kk_std_core__list _b_19374_19364 = head1; /*list<12200>*/;
          kk_box_t* _b_19375_19365 = field0_18115; /*cfield<list<12200>>*/;
          _x21533 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19374_19364, _ctx), _b_19375_19365, _ctx); /*ctail<58>*/
          goto _match21534;
        }
        kk_std_core__list _b_19376_19366 = _ctail_173460; /*list<12200>*/;
        kk_box_t* _b_19377_19367 = field0_18115; /*cfield<list<12200>>*/;
        _x21533 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19376_19366, _ctx), _b_19377_19367, _ctx); /*ctail<58>*/
        _match21534: ;
        xs = xx1;
        _acc0 = _x21533;
        goto kk__tailcall;
      }
    }
    kk_reuse_drop(_ru_20383);
    { // tailcall
      xs = xx1;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x19368 = _acc0._cons.CTail._field1;
    kk_box_t* last3 = _acc0._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19368, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Concatenate the `Just` result elements from applying a function to all elements.


// lift anonymous function
struct kk_std_core__ctailm_flatmap_maybe_fun21539__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_flatmap_maybe_fun21539(kk_function_t _fself, kk_std_core_types__maybe _y_175580, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_flatmap_maybe_fun21539(kk_function_t _accm0, kk_function_t f2, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21539__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_flatmap_maybe_fun21539__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_flatmap_maybe_fun21539, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_flatmap_maybe_fun21539(kk_function_t _fself, kk_std_core_types__maybe _y_175580, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21539__t* _self = kk_function_as(struct kk_std_core__ctailm_flatmap_maybe_fun21539__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<12200>) -> list<12200> */
  kk_function_t f2 = _self->f2; /* (12199) -> 12201 maybe<12200> */
  kk_std_core__list xx2 = _self->xx2; /* list<12199> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17746_op(_accm0, f2, xx2, _y_175580, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_flatmap_maybe_fun21541__t {
  struct kk_function_s _base;
  kk_function_t _b_19380_19378;
};
static kk_box_t kk_std_core__ctailm_flatmap_maybe_fun21541(kk_function_t _fself, kk_box_t _b_19379, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_flatmap_maybe_fun21541(kk_function_t _b_19380_19378, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21541__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_flatmap_maybe_fun21541__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_flatmap_maybe_fun21541, kk_context());
  _self->_b_19380_19378 = _b_19380_19378;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_flatmap_maybe_fun21541(kk_function_t _fself, kk_box_t _b_19379, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21541__t* _self = kk_function_as(struct kk_std_core__ctailm_flatmap_maybe_fun21541__t*, _fself);
  kk_function_t _b_19380_19378 = _self->_b_19380_19378; /* (maybe<12200>) -> 12201 list<12200> */
  kk_drop_match(_self, {kk_function_dup(_b_19380_19378);}, {}, _ctx)
  kk_std_core__list _x21542;
  kk_std_core_types__maybe _x21543 = kk_std_core_types__maybe_unbox(_b_19379, _ctx); /*maybe<12200>*/
  _x21542 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19380_19378, (_b_19380_19378, _x21543, _ctx)); /*list<12200>*/
  return kk_std_core__list_box(_x21542, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_flatmap_maybe_fun21545__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t y2;
};
static kk_std_core__list kk_std_core__ctailm_flatmap_maybe_fun21545(kk_function_t _fself, kk_std_core__list _ctail_173480, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_flatmap_maybe_fun21545(kk_function_t _accm0, kk_box_t y2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21545__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_flatmap_maybe_fun21545__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_flatmap_maybe_fun21545, kk_context());
  _self->_accm0 = _accm0;
  _self->y2 = y2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_flatmap_maybe_fun21545(kk_function_t _fself, kk_std_core__list _ctail_173480, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_flatmap_maybe_fun21545__t* _self = kk_function_as(struct kk_std_core__ctailm_flatmap_maybe_fun21545__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<12200>) -> list<12200> */
  kk_box_t y2 = _self->y2; /* 12200 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(y2);}, {}, _ctx)
  kk_std_core__list _x21546 = kk_std_core__new_Cons(kk_reuse_null, y2, _ctail_173480, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21546, _ctx));
}

kk_std_core__list kk_std_core__ctailm_flatmap_maybe(kk_std_core__list xs0, kk_function_t f2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e maybe<b>, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21537 = kk_std_core__as_Cons(xs0);
    kk_box_t x1 = _con21537->head;
    kk_std_core__list xx2 = _con21537->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx2);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    kk_std_core_types__maybe x2_18118;
    kk_function_t _x21538 = kk_function_dup(f2); /*(12199) -> 12201 maybe<12200>*/
    x2_18118 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21538, (_x21538, x1, _ctx)); /*maybe<12200>*/
    if (kk_yielding(kk_context())) {
      kk_std_core_types__maybe_drop(x2_18118, _ctx);
      kk_function_t _b_19380_19378 = kk_std_core__new_ctailm_flatmap_maybe_fun21539(_accm0, f2, xx2, _ctx); /*(maybe<12200>) -> 12201 list<12200>*/;
      kk_box_t _x21540 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_flatmap_maybe_fun21541(_b_19380_19378, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21540, _ctx);
    }
    if (kk_std_core_types__is_Just(x2_18118)) {
      kk_box_t y2 = x2_18118._cons.Just.value;
      { // tailcall
        kk_function_t _x21544 = kk_std_core__new_ctailm_flatmap_maybe_fun21545(_accm0, y2, _ctx); /*(list<12200>) -> list<12200>*/
        xs0 = xx2;
        _accm0 = _x21544;
        goto kk__tailcall;
      }
    }
    { // tailcall
      xs0 = xx2;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Concatenate the `Just` result elements from applying a function to all elements.


// lift anonymous function
struct kk_std_core_flatmap_maybe_fun21547__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_flatmap_maybe_fun21547(kk_function_t _fself, kk_std_core__list _ctail_17347, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_flatmap_maybe_fun21547(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_flatmap_maybe_fun21547, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_flatmap_maybe_fun21547(kk_function_t _fself, kk_std_core__list _ctail_17347, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17347;
}

kk_std_core__list kk_std_core_flatmap_maybe(kk_std_core__list xs1, kk_function_t f3, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, f : (a) -> e maybe<b>) -> e list<b> */ 
  bool _match_20042 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_20042) {
    return kk_std_core__ctail_flatmap_maybe(xs1, f3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_flatmap_maybe(xs1, f3, kk_std_core_new_flatmap_maybe_fun21547(_ctx), _ctx);
}

kk_box_t kk_std_core_fold_int(kk_integer_t start0, kk_integer_t end, kk_box_t init0, kk_function_t f, kk_context_t* _ctx) { /* forall<a> (start : int, end : int, init : a, f : (int, a) -> a) -> a */ 
  kk__tailcall: ;
  bool _match_20041 = kk_integer_gte_borrow(start0,end,kk_context()); /*bool*/;
  if (_match_20041) {
    kk_integer_drop(end, _ctx);
    kk_function_drop(f, _ctx);
    kk_integer_drop(start0, _ctx);
    return init0;
  }
  kk_box_t x;
  kk_function_t _x21549 = kk_function_dup(f); /*(int, 12244) -> 12244*/
  kk_integer_t _x21548 = kk_integer_dup(start0); /*int*/
  x = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_box_t, kk_context_t*), _x21549, (_x21549, _x21548, init0, _ctx)); /*12244*/
  { // tailcall
    kk_integer_t _x21550 = kk_integer_add(start0,(kk_integer_from_small(1)),kk_context()); /*int*/
    start0 = _x21550;
    init0 = x;
    goto kk__tailcall;
  }
}
 
// monadic lift

kk_box_t kk_std_core__mlift17747_foldl(kk_function_t f, kk_std_core__list xx, kk_box_t _y_17566, kk_context_t* _ctx) { /* forall<a,e,b> (f : (a, b) -> e a, xx : list<b>, a) -> e a */ 
  return kk_std_core_foldl(xx, _y_17566, f, _ctx);
}
 
// Fold a list from the left, i.e. `foldl([1,2],0,(+)) == (0+1)+2`
// Since `foldl` is tail recursive, it is preferred over `foldr` when using an associative function `f`


// lift anonymous function
struct kk_std_core_foldl_fun21553__t {
  struct kk_function_s _base;
  kk_function_t f0;
  kk_std_core__list xx0;
};
static kk_box_t kk_std_core_foldl_fun21553(kk_function_t _fself, kk_box_t _y_175660, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foldl_fun21553(kk_function_t f0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_foldl_fun21553__t* _self = kk_function_alloc_as(struct kk_std_core_foldl_fun21553__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foldl_fun21553, kk_context());
  _self->f0 = f0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_box_t kk_std_core_foldl_fun21553(kk_function_t _fself, kk_box_t _y_175660, kk_context_t* _ctx) {
  struct kk_std_core_foldl_fun21553__t* _self = kk_function_as(struct kk_std_core_foldl_fun21553__t*, _fself);
  kk_function_t f0 = _self->f0; /* (12247, 12252) -> 12249 12247 */
  kk_std_core__list xx0 = _self->xx0; /* list<12252> */
  kk_drop_match(_self, {kk_function_dup(f0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17747_foldl(f0, xx0, _y_175660, _ctx);
}

kk_box_t kk_std_core_foldl(kk_std_core__list xs, kk_box_t z, kk_function_t f0, kk_context_t* _ctx) { /* forall<a,b,e> (list<a>, b, (b, a) -> e b) -> e b */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21551 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21551->head;
    kk_std_core__list xx0 = _con21551->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_box_t x0_18121;
    kk_function_t _x21552 = kk_function_dup(f0); /*(12247, 12252) -> 12249 12247*/
    x0_18121 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x21552, (_x21552, z, x, _ctx)); /*12247*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x0_18121, _ctx);
      return kk_std_core_hnd_yield_extend(kk_std_core_new_foldl_fun21553(f0, xx0, _ctx), _ctx);
    }
    { // tailcall
      xs = xx0;
      z = x0_18121;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f0, _ctx);
  return z;
}


// lift anonymous function
struct kk_std_core_foldl1_fun21558__t {
  struct kk_function_s _base;
  kk_ssize_t i_18124;
};
static kk_box_t kk_std_core_foldl1_fun21558(kk_function_t _fself, kk_function_t cont, kk_box_t res, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foldl1_fun21558(kk_ssize_t i_18124, kk_context_t* _ctx) {
  struct kk_std_core_foldl1_fun21558__t* _self = kk_function_alloc_as(struct kk_std_core_foldl1_fun21558__t, 1, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foldl1_fun21558, kk_context());
  _self->i_18124 = i_18124;
  return &_self->_base;
}

static kk_box_t kk_std_core_foldl1_fun21558(kk_function_t _fself, kk_function_t cont, kk_box_t res, kk_context_t* _ctx) {
  struct kk_std_core_foldl1_fun21558__t* _self = kk_function_as(struct kk_std_core_foldl1_fun21558__t*, _fself);
  kk_ssize_t i_18124 = _self->i_18124; /* std/core/hnd/ev-index */
  kk_drop_match(_self, {;}, {}, _ctx)
  return kk_std_core_hnd_open_at1(i_18124, cont, res, _ctx);
}

kk_box_t kk_std_core_foldl1(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, f : (a, a) -> <exn|e> a) -> <exn|e> a */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21554 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21554->head;
    kk_std_core__list xx = _con21554->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    return kk_std_core_foldl(xx, x, f, _ctx);
  }
  kk_function_drop(f, _ctx);
  kk_ssize_t i_18124;
  kk_std_core_hnd__htag _x21555 = kk_std_core_hnd__htag_dup(kk_std_core__tag_exn); /*std/core/hnd/htag<.hnd-exn>*/
  i_18124 = kk_std_core_hnd__evv_index(_x21555, _ctx); /*std/core/hnd/ev-index*/
  kk_evv_t w = kk_evv_swap_create1(i_18124,kk_context()); /*std/core/hnd/evv<<exn|12320>>*/;
  kk_box_t y;
  kk_string_t _x21556;
  kk_define_string_literal(, _s21557, 33, "unexpected Nil in std/core/foldl1")
  _x21556 = kk_string_dup(_s21557); /*string*/
  y = kk_std_core_throw(_x21556, kk_std_core_types__new_None(_ctx), _ctx); /*12319*/
  kk_unit_t __ = kk_Unit;
  kk_evv_set(w,kk_context());
  if (kk_yielding(kk_context())) {
    kk_box_drop(y, _ctx);
    return kk_std_core_hnd_yield_cont(kk_std_core_new_foldl1_fun21558(i_18124, _ctx), _ctx);
  }
  return y;
}
 
// lifted

kk_std_core__list kk_std_core__lift17186_reverse(kk_std_core__list acc, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a> (acc : list<a>, ys : list<a>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21559 = kk_std_core__as_Cons(ys);
    kk_box_t x = _con21559->head;
    kk_std_core__list xx = _con21559->tail;
    kk_reuse_t _ru_20387 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20387 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x21560;
      if (kk_likely(_ru_20387!=NULL)) {
        struct kk_std_core_Cons* _con21561 = (struct kk_std_core_Cons*)_ru_20387;
        _con21561->tail = acc;
        _x21560 = kk_std_core__base_Cons(_con21561); /*list<61>*/
      }
      else {
        _x21560 = kk_std_core__new_Cons(kk_reuse_null, x, acc, _ctx); /*list<61>*/
      }
      acc = _x21560;
      ys = xx;
      goto kk__tailcall;
    }
  }
  return acc;
}
 
// monadic lift

kk_box_t kk_std_core__mlift17748_op(kk_function_t f, kk_std_core__list xx, kk_box_t z00_17256, kk_context_t* _ctx) { /* forall<a,e,b> (f : (b, a) -> e a, xx : list<b>, z00.17256 : a) -> e a */ 
  return kk_std_core__lift17187_foldr(f, xx, z00_17256, _ctx);
}
 
// lifted
// specialized std/core/foldl to parameters [f]


// lift anonymous function
struct kk_std_core__lift17187_foldr_fun21564__t {
  struct kk_function_s _base;
  kk_function_t f0;
  kk_std_core__list xx0;
};
static kk_box_t kk_std_core__lift17187_foldr_fun21564(kk_function_t _fself, kk_box_t z00_172560, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17187_foldr_fun21564(kk_function_t f0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core__lift17187_foldr_fun21564__t* _self = kk_function_alloc_as(struct kk_std_core__lift17187_foldr_fun21564__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17187_foldr_fun21564, kk_context());
  _self->f0 = f0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17187_foldr_fun21564(kk_function_t _fself, kk_box_t z00_172560, kk_context_t* _ctx) {
  struct kk_std_core__lift17187_foldr_fun21564__t* _self = kk_function_as(struct kk_std_core__lift17187_foldr_fun21564__t*, _fself);
  kk_function_t f0 = _self->f0; /* (12338, 12334) -> 12336 12334 */
  kk_std_core__list xx0 = _self->xx0; /* list<12338> */
  kk_drop_match(_self, {kk_function_dup(f0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17748_op(f0, xx0, z00_172560, _ctx);
}

kk_box_t kk_std_core__lift17187_foldr(kk_function_t f0, kk_std_core__list xs0, kk_box_t z0, kk_context_t* _ctx) { /* forall<a,e,b> (f : (b, a) -> e a, xs0 : list<b>, z0 : a) -> e a */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21562 = kk_std_core__as_Cons(xs0);
    kk_box_t x0 = _con21562->head;
    kk_std_core__list xx0 = _con21562->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x0);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    kk_box_t x_18128;
    kk_function_t _x21563 = kk_function_dup(f0); /*(12338, 12334) -> 12336 12334*/
    x_18128 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x21563, (_x21563, x0, z0, _ctx)); /*12334*/
    if (kk_yielding(kk_context())) {
      kk_box_drop(x_18128, _ctx);
      return kk_std_core_hnd_yield_extend(kk_std_core__new_lift17187_foldr_fun21564(f0, xx0, _ctx), _ctx);
    }
    { // tailcall
      xs0 = xx0;
      z0 = x_18128;
      goto kk__tailcall;
    }
  }
  kk_function_drop(f0, _ctx);
  return z0;
}


// lift anonymous function
struct kk_std_core_foldr1_fun21569__t {
  struct kk_function_s _base;
  kk_ssize_t i_18133;
};
static kk_box_t kk_std_core_foldr1_fun21569(kk_function_t _fself, kk_function_t cont, kk_box_t res, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foldr1_fun21569(kk_ssize_t i_18133, kk_context_t* _ctx) {
  struct kk_std_core_foldr1_fun21569__t* _self = kk_function_alloc_as(struct kk_std_core_foldr1_fun21569__t, 1, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foldr1_fun21569, kk_context());
  _self->i_18133 = i_18133;
  return &_self->_base;
}

static kk_box_t kk_std_core_foldr1_fun21569(kk_function_t _fself, kk_function_t cont, kk_box_t res, kk_context_t* _ctx) {
  struct kk_std_core_foldr1_fun21569__t* _self = kk_function_as(struct kk_std_core_foldr1_fun21569__t*, _fself);
  kk_ssize_t i_18133 = _self->i_18133; /* std/core/hnd/ev-index */
  kk_drop_match(_self, {;}, {}, _ctx)
  return kk_std_core_hnd_open_at1(i_18133, cont, res, _ctx);
}

kk_box_t kk_std_core_foldr1(kk_std_core__list xs, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, f : (a, a) -> <exn|e> a) -> <exn|e> a */ 
  kk_std_core__list xs0_17093 = kk_std_core__lift17186_reverse(kk_std_core__new_Nil(_ctx), xs, _ctx); /*list<12372>*/;
  if (kk_std_core__is_Cons(xs0_17093)) {
    struct kk_std_core_Cons* _con21565 = kk_std_core__as_Cons(xs0_17093);
    kk_box_t x = _con21565->head;
    kk_std_core__list xx = _con21565->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0_17093))) {
      kk_std_core__list_free(xs0_17093);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs0_17093, _ctx);
      kk_Unit;
    }
    return kk_std_core_foldl(xx, x, f, _ctx);
  }
  kk_function_drop(f, _ctx);
  kk_ssize_t i_18133;
  kk_std_core_hnd__htag _x21566 = kk_std_core_hnd__htag_dup(kk_std_core__tag_exn); /*std/core/hnd/htag<.hnd-exn>*/
  i_18133 = kk_std_core_hnd__evv_index(_x21566, _ctx); /*std/core/hnd/ev-index*/
  kk_evv_t w = kk_evv_swap_create1(i_18133,kk_context()); /*std/core/hnd/evv<<exn|12373>>*/;
  kk_box_t y;
  kk_string_t _x21567;
  kk_define_string_literal(, _s21568, 33, "unexpected Nil in std/core/foldl1")
  _x21567 = kk_string_dup(_s21568); /*string*/
  y = kk_std_core_throw(_x21567, kk_std_core_types__new_None(_ctx), _ctx); /*12372*/
  kk_unit_t __ = kk_Unit;
  kk_evv_set(w,kk_context());
  if (kk_yielding(kk_context())) {
    kk_box_drop(y, _ctx);
    return kk_std_core_hnd_yield_cont(kk_std_core_new_foldr1_fun21569(i_18133, _ctx), _ctx);
  }
  return y;
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17749_op(kk_function_t action, kk_integer_t end, kk_integer_t i, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action : (int) -> e (), end : int, i : int, wild_ : ()) -> e () */ 
  kk_integer_t i0_17259 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__lift17188_for(action, end, i0_17259, _ctx); return kk_Unit;
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17188_for_fun21572__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_integer_t end0;
  kk_integer_t i0;
};
static kk_unit_t kk_std_core__lift17188_for_fun21572(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17188_for_fun21572(kk_function_t action0, kk_integer_t end0, kk_integer_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17188_for_fun21572__t* _self = kk_function_alloc_as(struct kk_std_core__lift17188_for_fun21572__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17188_for_fun21572, kk_context());
  _self->action0 = action0;
  _self->end0 = end0;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17188_for_fun21572(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17188_for_fun21572__t* _self = kk_function_as(struct kk_std_core__lift17188_for_fun21572__t*, _fself);
  kk_function_t action0 = _self->action0; /* (int) -> 12409 () */
  kk_integer_t end0 = _self->end0; /* int */
  kk_integer_t i0 = _self->i0; /* int */
  kk_drop_match(_self, {kk_function_dup(action0);kk_integer_dup(end0);kk_integer_dup(i0);}, {}, _ctx)
  return kk_std_core__mlift17749_op(action0, end0, i0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17188_for_fun21574__t {
  struct kk_function_s _base;
  kk_function_t _b_19383_19381;
};
static kk_box_t kk_std_core__lift17188_for_fun21574(kk_function_t _fself, kk_box_t _b_19382, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17188_for_fun21574(kk_function_t _b_19383_19381, kk_context_t* _ctx) {
  struct kk_std_core__lift17188_for_fun21574__t* _self = kk_function_alloc_as(struct kk_std_core__lift17188_for_fun21574__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17188_for_fun21574, kk_context());
  _self->_b_19383_19381 = _b_19383_19381;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17188_for_fun21574(kk_function_t _fself, kk_box_t _b_19382, kk_context_t* _ctx) {
  struct kk_std_core__lift17188_for_fun21574__t* _self = kk_function_as(struct kk_std_core__lift17188_for_fun21574__t*, _fself);
  kk_function_t _b_19383_19381 = _self->_b_19383_19381; /* (wild_0 : ()) -> 12409 () */
  kk_drop_match(_self, {kk_function_dup(_b_19383_19381);}, {}, _ctx)
  kk_unit_t _x21575 = kk_Unit;
  kk_unit_t _x21576 = kk_Unit;
  kk_unit_unbox(_b_19382);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19383_19381, (_b_19383_19381, _x21576, _ctx));
  return kk_unit_box(_x21575);
}

kk_unit_t kk_std_core__lift17188_for(kk_function_t action0, kk_integer_t end0, kk_integer_t i0, kk_context_t* _ctx) { /* forall<e> (action : (int) -> e (), end : int, i : int) -> e () */ 
  kk__tailcall: ;
  bool _match_20035 = kk_integer_lte_borrow(i0,end0,kk_context()); /*bool*/;
  if (_match_20035) {
    kk_unit_t x_18137 = kk_Unit;
    kk_function_t _x21571 = kk_function_dup(action0); /*(int) -> 12409 ()*/
    kk_integer_t _x21570 = kk_integer_dup(i0); /*int*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_context_t*), _x21571, (_x21571, _x21570, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19383_19381 = kk_std_core__new_lift17188_for_fun21572(action0, end0, i0, _ctx); /*(wild_0 : ()) -> 12409 ()*/;
      kk_box_t _x21573 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17188_for_fun21574(_b_19383_19381, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x21573); return kk_Unit;
    }
    kk_integer_t i0_172590 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
    { // tailcall
      i0 = i0_172590;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action0, _ctx);
  kk_integer_drop(end0, _ctx);
  kk_integer_drop(i0, _ctx);
  kk_Unit; return kk_Unit;
}
 
// monadic lift

kk_std_core_types__maybe kk_std_core__mlift17750_op(kk_function_t action, kk_integer_t end, kk_integer_t i, kk_std_core_types__maybe _y_17583, kk_context_t* _ctx) { /* forall<a,e> (action : (int) -> e maybe<a>, end : int, i : int, maybe<a>) -> e maybe<a> */ 
  if (kk_std_core_types__is_Nothing(_y_17583)) {
    kk_integer_t i0_17260 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
    return kk_std_core__lift17189_for_while(action, end, i0_17260, _ctx);
  }
  kk_box_t x = _y_17583._cons.Just.value;
  kk_function_drop(action, _ctx);
  kk_integer_drop(end, _ctx);
  kk_integer_drop(i, _ctx);
  return kk_std_core_types__new_Just(x, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17189_for_while_fun21580__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_integer_t end0;
  kk_integer_t i0;
};
static kk_std_core_types__maybe kk_std_core__lift17189_for_while_fun21580(kk_function_t _fself, kk_std_core_types__maybe _y_175830, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17189_for_while_fun21580(kk_function_t action0, kk_integer_t end0, kk_integer_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17189_for_while_fun21580__t* _self = kk_function_alloc_as(struct kk_std_core__lift17189_for_while_fun21580__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17189_for_while_fun21580, kk_context());
  _self->action0 = action0;
  _self->end0 = end0;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_core__lift17189_for_while_fun21580(kk_function_t _fself, kk_std_core_types__maybe _y_175830, kk_context_t* _ctx) {
  struct kk_std_core__lift17189_for_while_fun21580__t* _self = kk_function_as(struct kk_std_core__lift17189_for_while_fun21580__t*, _fself);
  kk_function_t action0 = _self->action0; /* (int) -> 12460 maybe<12459> */
  kk_integer_t end0 = _self->end0; /* int */
  kk_integer_t i0 = _self->i0; /* int */
  kk_drop_match(_self, {kk_function_dup(action0);kk_integer_dup(end0);kk_integer_dup(i0);}, {}, _ctx)
  return kk_std_core__mlift17750_op(action0, end0, i0, _y_175830, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17189_for_while_fun21582__t {
  struct kk_function_s _base;
  kk_function_t _b_19386_19384;
};
static kk_box_t kk_std_core__lift17189_for_while_fun21582(kk_function_t _fself, kk_box_t _b_19385, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17189_for_while_fun21582(kk_function_t _b_19386_19384, kk_context_t* _ctx) {
  struct kk_std_core__lift17189_for_while_fun21582__t* _self = kk_function_alloc_as(struct kk_std_core__lift17189_for_while_fun21582__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17189_for_while_fun21582, kk_context());
  _self->_b_19386_19384 = _b_19386_19384;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17189_for_while_fun21582(kk_function_t _fself, kk_box_t _b_19385, kk_context_t* _ctx) {
  struct kk_std_core__lift17189_for_while_fun21582__t* _self = kk_function_as(struct kk_std_core__lift17189_for_while_fun21582__t*, _fself);
  kk_function_t _b_19386_19384 = _self->_b_19386_19384; /* (maybe<12459>) -> 12460 maybe<12459> */
  kk_drop_match(_self, {kk_function_dup(_b_19386_19384);}, {}, _ctx)
  kk_std_core_types__maybe _x21583;
  kk_std_core_types__maybe _x21584 = kk_std_core_types__maybe_unbox(_b_19385, _ctx); /*maybe<12459>*/
  _x21583 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19386_19384, (_b_19386_19384, _x21584, _ctx)); /*maybe<12459>*/
  return kk_std_core_types__maybe_box(_x21583, _ctx);
}

kk_std_core_types__maybe kk_std_core__lift17189_for_while(kk_function_t action0, kk_integer_t end0, kk_integer_t i0, kk_context_t* _ctx) { /* forall<a,e> (action : (int) -> e maybe<a>, end : int, i : int) -> e maybe<a> */ 
  kk__tailcall: ;
  bool _match_20033 = kk_integer_lte_borrow(i0,end0,kk_context()); /*bool*/;
  if (_match_20033) {
    kk_std_core_types__maybe x0_18140;
    kk_function_t _x21579 = kk_function_dup(action0); /*(int) -> 12460 maybe<12459>*/
    kk_integer_t _x21578 = kk_integer_dup(i0); /*int*/
    x0_18140 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_integer_t, kk_context_t*), _x21579, (_x21579, _x21578, _ctx)); /*maybe<12459>*/
    if (kk_yielding(kk_context())) {
      kk_std_core_types__maybe_drop(x0_18140, _ctx);
      kk_function_t _b_19386_19384 = kk_std_core__new_lift17189_for_while_fun21580(action0, end0, i0, _ctx); /*(maybe<12459>) -> 12460 maybe<12459>*/;
      kk_box_t _x21581 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17189_for_while_fun21582(_b_19386_19384, _ctx), _ctx); /*3858*/
      return kk_std_core_types__maybe_unbox(_x21581, _ctx);
    }
    if (kk_std_core_types__is_Nothing(x0_18140)) {
      kk_integer_t i0_172600 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
      { // tailcall
        i0 = i0_172600;
        goto kk__tailcall;
      }
    }
    kk_box_t x1 = x0_18140._cons.Just.value;
    kk_integer_drop(i0, _ctx);
    kk_Unit;
    kk_integer_drop(end0, _ctx);
    kk_Unit;
    kk_function_drop(action0, _ctx);
    kk_Unit;
    return kk_std_core_types__new_Just(x1, _ctx);
  }
  kk_function_drop(action0, _ctx);
  kk_integer_drop(end0, _ctx);
  kk_integer_drop(i0, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17751_op(kk_function_t action, kk_ref_t i, kk_std_core__list xx, kk_unit_t wild__0, kk_context_t* _ctx) { /* forall<h,a,e> (action : (int, a) -> e (), i : local-var<h,int>, xx : list<a>, wild_0 : ()) -> <local<h>|e> () */ 
  kk_std_core__lift17190_foreach_indexed(action, i, xx, _ctx); return kk_Unit;
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17752_op_fun21585__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_ref_t i0;
  kk_std_core__list xx0;
};
static kk_unit_t kk_std_core__mlift17752_op_fun21585(kk_function_t _fself, kk_unit_t wild__00, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17752_op_fun21585(kk_function_t action0, kk_ref_t i0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17752_op_fun21585__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17752_op_fun21585__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17752_op_fun21585, kk_context());
  _self->action0 = action0;
  _self->i0 = i0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__mlift17752_op_fun21585(kk_function_t _fself, kk_unit_t wild__00, kk_context_t* _ctx) {
  struct kk_std_core__mlift17752_op_fun21585__t* _self = kk_function_as(struct kk_std_core__mlift17752_op_fun21585__t*, _fself);
  kk_function_t action0 = _self->action0; /* (int, 12570) -> 12571 () */
  kk_ref_t i0 = _self->i0; /* local-var<12564,int> */
  kk_std_core__list xx0 = _self->xx0; /* list<12570> */
  kk_drop_match(_self, {kk_function_dup(action0);kk_ref_dup(i0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17751_op(action0, i0, xx0, wild__00, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17752_op_fun21587__t {
  struct kk_function_s _base;
  kk_function_t _b_19393_19391;
};
static kk_box_t kk_std_core__mlift17752_op_fun21587(kk_function_t _fself, kk_box_t _b_19392, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17752_op_fun21587(kk_function_t _b_19393_19391, kk_context_t* _ctx) {
  struct kk_std_core__mlift17752_op_fun21587__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17752_op_fun21587__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17752_op_fun21587, kk_context());
  _self->_b_19393_19391 = _b_19393_19391;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17752_op_fun21587(kk_function_t _fself, kk_box_t _b_19392, kk_context_t* _ctx) {
  struct kk_std_core__mlift17752_op_fun21587__t* _self = kk_function_as(struct kk_std_core__mlift17752_op_fun21587__t*, _fself);
  kk_function_t _b_19393_19391 = _self->_b_19393_19391; /* (wild_00 : ()) -> <local<12564>|12571> () */
  kk_drop_match(_self, {kk_function_dup(_b_19393_19391);}, {}, _ctx)
  kk_unit_t _x21588 = kk_Unit;
  kk_unit_t _x21589 = kk_Unit;
  kk_unit_unbox(_b_19392);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19393_19391, (_b_19393_19391, _x21589, _ctx));
  return kk_unit_box(_x21588);
}

kk_unit_t kk_std_core__mlift17752_op(kk_function_t action0, kk_ref_t i0, kk_std_core__list xx0, kk_integer_t _y_17591, kk_context_t* _ctx) { /* forall<h,a,e> (action : (int, a) -> e (), i : local-var<h,int>, xx : list<a>, int) -> <local<h>|e> () */ 
  kk_integer_t _b_19390_19388 = kk_integer_add(_y_17591,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_unit_t x_18143 = kk_Unit;
  (kk_ref_set_borrow(i0,(kk_integer_box(_b_19390_19388)),kk_context()));
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19393_19391 = kk_std_core__new_mlift17752_op_fun21585(action0, i0, xx0, _ctx); /*(wild_00 : ()) -> <local<12564>|12571> ()*/;
    kk_box_t _x21586 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17752_op_fun21587(_b_19393_19391, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21586); return kk_Unit;
  }
  kk_std_core__mlift17751_op(action0, i0, xx0, x_18143, _ctx); return kk_Unit;
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17753_op_fun21592__t {
  struct kk_function_s _base;
  kk_function_t action1;
  kk_ref_t i1;
  kk_std_core__list xx1;
};
static kk_unit_t kk_std_core__mlift17753_op_fun21592(kk_function_t _fself, kk_integer_t _y_175910, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17753_op_fun21592(kk_function_t action1, kk_ref_t i1, kk_std_core__list xx1, kk_context_t* _ctx) {
  struct kk_std_core__mlift17753_op_fun21592__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17753_op_fun21592__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17753_op_fun21592, kk_context());
  _self->action1 = action1;
  _self->i1 = i1;
  _self->xx1 = xx1;
  return &_self->_base;
}

static kk_unit_t kk_std_core__mlift17753_op_fun21592(kk_function_t _fself, kk_integer_t _y_175910, kk_context_t* _ctx) {
  struct kk_std_core__mlift17753_op_fun21592__t* _self = kk_function_as(struct kk_std_core__mlift17753_op_fun21592__t*, _fself);
  kk_function_t action1 = _self->action1; /* (int, 12570) -> 12571 () */
  kk_ref_t i1 = _self->i1; /* local-var<12564,int> */
  kk_std_core__list xx1 = _self->xx1; /* list<12570> */
  kk_drop_match(_self, {kk_function_dup(action1);kk_ref_dup(i1);kk_std_core__list_dup(xx1);}, {}, _ctx)
  return kk_std_core__mlift17752_op(action1, i1, xx1, _y_175910, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17753_op_fun21594__t {
  struct kk_function_s _base;
  kk_function_t _b_19398_19396;
};
static kk_box_t kk_std_core__mlift17753_op_fun21594(kk_function_t _fself, kk_box_t _b_19397, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17753_op_fun21594(kk_function_t _b_19398_19396, kk_context_t* _ctx) {
  struct kk_std_core__mlift17753_op_fun21594__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17753_op_fun21594__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17753_op_fun21594, kk_context());
  _self->_b_19398_19396 = _b_19398_19396;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17753_op_fun21594(kk_function_t _fself, kk_box_t _b_19397, kk_context_t* _ctx) {
  struct kk_std_core__mlift17753_op_fun21594__t* _self = kk_function_as(struct kk_std_core__mlift17753_op_fun21594__t*, _fself);
  kk_function_t _b_19398_19396 = _self->_b_19398_19396; /* (int) -> <local<12564>|12571> () */
  kk_drop_match(_self, {kk_function_dup(_b_19398_19396);}, {}, _ctx)
  kk_unit_t _x21595 = kk_Unit;
  kk_integer_t _x21596 = kk_integer_unbox(_b_19397); /*int*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_context_t*), _b_19398_19396, (_b_19398_19396, _x21596, _ctx));
  return kk_unit_box(_x21595);
}

kk_unit_t kk_std_core__mlift17753_op(kk_function_t action1, kk_ref_t i1, kk_std_core__list xx1, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<h,a,e> (action : (int, a) -> e (), i : local-var<h,int>, xx : list<a>, wild_ : ()) -> <local<h>|e> () */ 
  kk_integer_t x0_18145;
  kk_box_t _x21590;
  kk_ref_t _x21591 = kk_ref_dup(i1); /*local-var<12564,int>*/
  _x21590 = (kk_ref_get(_x21591,kk_context())); /*256*/
  x0_18145 = kk_integer_unbox(_x21590); /*int*/
  if (kk_yielding(kk_context())) {
    kk_integer_drop(x0_18145, _ctx);
    kk_function_t _b_19398_19396 = kk_std_core__new_mlift17753_op_fun21592(action1, i1, xx1, _ctx); /*(int) -> <local<12564>|12571> ()*/;
    kk_box_t _x21593 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17753_op_fun21594(_b_19398_19396, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21593); return kk_Unit;
  }
  kk_std_core__mlift17752_op(action1, i1, xx1, x0_18145, _ctx); return kk_Unit;
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17754_op_fun21598__t {
  struct kk_function_s _base;
  kk_function_t action2;
  kk_ref_t i2;
  kk_std_core__list xx2;
};
static kk_unit_t kk_std_core__mlift17754_op_fun21598(kk_function_t _fself, kk_unit_t wild__1, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17754_op_fun21598(kk_function_t action2, kk_ref_t i2, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__mlift17754_op_fun21598__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17754_op_fun21598__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17754_op_fun21598, kk_context());
  _self->action2 = action2;
  _self->i2 = i2;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_unit_t kk_std_core__mlift17754_op_fun21598(kk_function_t _fself, kk_unit_t wild__1, kk_context_t* _ctx) {
  struct kk_std_core__mlift17754_op_fun21598__t* _self = kk_function_as(struct kk_std_core__mlift17754_op_fun21598__t*, _fself);
  kk_function_t action2 = _self->action2; /* (int, 12570) -> 12571 () */
  kk_ref_t i2 = _self->i2; /* local-var<12564,int> */
  kk_std_core__list xx2 = _self->xx2; /* list<12570> */
  kk_drop_match(_self, {kk_function_dup(action2);kk_ref_dup(i2);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17753_op(action2, i2, xx2, wild__1, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17754_op_fun21600__t {
  struct kk_function_s _base;
  kk_function_t _b_19401_19399;
};
static kk_box_t kk_std_core__mlift17754_op_fun21600(kk_function_t _fself, kk_box_t _b_19400, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17754_op_fun21600(kk_function_t _b_19401_19399, kk_context_t* _ctx) {
  struct kk_std_core__mlift17754_op_fun21600__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17754_op_fun21600__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17754_op_fun21600, kk_context());
  _self->_b_19401_19399 = _b_19401_19399;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17754_op_fun21600(kk_function_t _fself, kk_box_t _b_19400, kk_context_t* _ctx) {
  struct kk_std_core__mlift17754_op_fun21600__t* _self = kk_function_as(struct kk_std_core__mlift17754_op_fun21600__t*, _fself);
  kk_function_t _b_19401_19399 = _self->_b_19401_19399; /* (wild_1 : ()) -> <local<12564>|12571> () */
  kk_drop_match(_self, {kk_function_dup(_b_19401_19399);}, {}, _ctx)
  kk_unit_t _x21601 = kk_Unit;
  kk_unit_t _x21602 = kk_Unit;
  kk_unit_unbox(_b_19400);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19401_19399, (_b_19401_19399, _x21602, _ctx));
  return kk_unit_box(_x21601);
}

kk_unit_t kk_std_core__mlift17754_op(kk_function_t action2, kk_ref_t i2, kk_box_t x00, kk_std_core__list xx2, kk_integer_t j, kk_context_t* _ctx) { /* forall<h,a,e> (action : (int, a) -> e (), i : local-var<h,int>, x0 : a, xx : list<a>, j : int) -> <local<h>|e> () */ 
  kk_unit_t x1_18147 = kk_Unit;
  kk_function_t _x21597 = kk_function_dup(action2); /*(int, 12570) -> 12571 ()*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_box_t, kk_context_t*), _x21597, (_x21597, j, x00, _ctx));
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19401_19399 = kk_std_core__new_mlift17754_op_fun21598(action2, i2, xx2, _ctx); /*(wild_1 : ()) -> <local<12564>|12571> ()*/;
    kk_box_t _x21599 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17754_op_fun21600(_b_19401_19399, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21599); return kk_Unit;
  }
  kk_std_core__mlift17753_op(action2, i2, xx2, x1_18147, _ctx); return kk_Unit;
}
 
// lifted
// specialized std/core/foreach to parameters [action]


// lift anonymous function
struct kk_std_core__lift17190_foreach_indexed_fun21606__t {
  struct kk_function_s _base;
  kk_function_t action4;
  kk_ref_t i3;
  kk_box_t x01;
  kk_std_core__list xx3;
};
static kk_unit_t kk_std_core__lift17190_foreach_indexed_fun21606(kk_function_t _fself, kk_integer_t j0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17190_foreach_indexed_fun21606(kk_function_t action4, kk_ref_t i3, kk_box_t x01, kk_std_core__list xx3, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21606__t* _self = kk_function_alloc_as(struct kk_std_core__lift17190_foreach_indexed_fun21606__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17190_foreach_indexed_fun21606, kk_context());
  _self->action4 = action4;
  _self->i3 = i3;
  _self->x01 = x01;
  _self->xx3 = xx3;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17190_foreach_indexed_fun21606(kk_function_t _fself, kk_integer_t j0, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21606__t* _self = kk_function_as(struct kk_std_core__lift17190_foreach_indexed_fun21606__t*, _fself);
  kk_function_t action4 = _self->action4; /* (int, 12570) -> 12571 () */
  kk_ref_t i3 = _self->i3; /* local-var<12564,int> */
  kk_box_t x01 = _self->x01; /* 12570 */
  kk_std_core__list xx3 = _self->xx3; /* list<12570> */
  kk_drop_match(_self, {kk_function_dup(action4);kk_ref_dup(i3);kk_box_dup(x01);kk_std_core__list_dup(xx3);}, {}, _ctx)
  return kk_std_core__mlift17754_op(action4, i3, x01, xx3, j0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17190_foreach_indexed_fun21608__t {
  struct kk_function_s _base;
  kk_function_t _b_19418_19404;
};
static kk_box_t kk_std_core__lift17190_foreach_indexed_fun21608(kk_function_t _fself, kk_box_t _b_19405, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17190_foreach_indexed_fun21608(kk_function_t _b_19418_19404, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21608__t* _self = kk_function_alloc_as(struct kk_std_core__lift17190_foreach_indexed_fun21608__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17190_foreach_indexed_fun21608, kk_context());
  _self->_b_19418_19404 = _b_19418_19404;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17190_foreach_indexed_fun21608(kk_function_t _fself, kk_box_t _b_19405, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21608__t* _self = kk_function_as(struct kk_std_core__lift17190_foreach_indexed_fun21608__t*, _fself);
  kk_function_t _b_19418_19404 = _self->_b_19418_19404; /* (j0 : int) -> <local<12564>|12571> () */
  kk_drop_match(_self, {kk_function_dup(_b_19418_19404);}, {}, _ctx)
  kk_unit_t _x21609 = kk_Unit;
  kk_integer_t _x21610 = kk_integer_unbox(_b_19405); /*int*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_context_t*), _b_19418_19404, (_b_19418_19404, _x21610, _ctx));
  return kk_unit_box(_x21609);
}


// lift anonymous function
struct kk_std_core__lift17190_foreach_indexed_fun21612__t {
  struct kk_function_s _base;
  kk_function_t action4;
  kk_ref_t i3;
  kk_std_core__list xx3;
};
static kk_unit_t kk_std_core__lift17190_foreach_indexed_fun21612(kk_function_t _fself, kk_unit_t wild__2, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17190_foreach_indexed_fun21612(kk_function_t action4, kk_ref_t i3, kk_std_core__list xx3, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21612__t* _self = kk_function_alloc_as(struct kk_std_core__lift17190_foreach_indexed_fun21612__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17190_foreach_indexed_fun21612, kk_context());
  _self->action4 = action4;
  _self->i3 = i3;
  _self->xx3 = xx3;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17190_foreach_indexed_fun21612(kk_function_t _fself, kk_unit_t wild__2, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21612__t* _self = kk_function_as(struct kk_std_core__lift17190_foreach_indexed_fun21612__t*, _fself);
  kk_function_t action4 = _self->action4; /* (int, 12570) -> 12571 () */
  kk_ref_t i3 = _self->i3; /* local-var<12564,int> */
  kk_std_core__list xx3 = _self->xx3; /* list<12570> */
  kk_drop_match(_self, {kk_function_dup(action4);kk_ref_dup(i3);kk_std_core__list_dup(xx3);}, {}, _ctx)
  return kk_std_core__mlift17753_op(action4, i3, xx3, wild__2, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17190_foreach_indexed_fun21614__t {
  struct kk_function_s _base;
  kk_function_t _b_19419_19406;
};
static kk_box_t kk_std_core__lift17190_foreach_indexed_fun21614(kk_function_t _fself, kk_box_t _b_19407, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17190_foreach_indexed_fun21614(kk_function_t _b_19419_19406, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21614__t* _self = kk_function_alloc_as(struct kk_std_core__lift17190_foreach_indexed_fun21614__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17190_foreach_indexed_fun21614, kk_context());
  _self->_b_19419_19406 = _b_19419_19406;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17190_foreach_indexed_fun21614(kk_function_t _fself, kk_box_t _b_19407, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21614__t* _self = kk_function_as(struct kk_std_core__lift17190_foreach_indexed_fun21614__t*, _fself);
  kk_function_t _b_19419_19406 = _self->_b_19419_19406; /* (wild_2 : ()) -> <local<12564>|12571> () */
  kk_drop_match(_self, {kk_function_dup(_b_19419_19406);}, {}, _ctx)
  kk_unit_t _x21615 = kk_Unit;
  kk_unit_t _x21616 = kk_Unit;
  kk_unit_unbox(_b_19407);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19419_19406, (_b_19419_19406, _x21616, _ctx));
  return kk_unit_box(_x21615);
}


// lift anonymous function
struct kk_std_core__lift17190_foreach_indexed_fun21619__t {
  struct kk_function_s _base;
  kk_function_t action4;
  kk_ref_t i3;
  kk_std_core__list xx3;
};
static kk_unit_t kk_std_core__lift17190_foreach_indexed_fun21619(kk_function_t _fself, kk_integer_t _y_175911, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17190_foreach_indexed_fun21619(kk_function_t action4, kk_ref_t i3, kk_std_core__list xx3, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21619__t* _self = kk_function_alloc_as(struct kk_std_core__lift17190_foreach_indexed_fun21619__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17190_foreach_indexed_fun21619, kk_context());
  _self->action4 = action4;
  _self->i3 = i3;
  _self->xx3 = xx3;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17190_foreach_indexed_fun21619(kk_function_t _fself, kk_integer_t _y_175911, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21619__t* _self = kk_function_as(struct kk_std_core__lift17190_foreach_indexed_fun21619__t*, _fself);
  kk_function_t action4 = _self->action4; /* (int, 12570) -> 12571 () */
  kk_ref_t i3 = _self->i3; /* local-var<12564,int> */
  kk_std_core__list xx3 = _self->xx3; /* list<12570> */
  kk_drop_match(_self, {kk_function_dup(action4);kk_ref_dup(i3);kk_std_core__list_dup(xx3);}, {}, _ctx)
  return kk_std_core__mlift17752_op(action4, i3, xx3, _y_175911, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17190_foreach_indexed_fun21621__t {
  struct kk_function_s _base;
  kk_function_t _b_19420_19410;
};
static kk_box_t kk_std_core__lift17190_foreach_indexed_fun21621(kk_function_t _fself, kk_box_t _b_19411, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17190_foreach_indexed_fun21621(kk_function_t _b_19420_19410, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21621__t* _self = kk_function_alloc_as(struct kk_std_core__lift17190_foreach_indexed_fun21621__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17190_foreach_indexed_fun21621, kk_context());
  _self->_b_19420_19410 = _b_19420_19410;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17190_foreach_indexed_fun21621(kk_function_t _fself, kk_box_t _b_19411, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21621__t* _self = kk_function_as(struct kk_std_core__lift17190_foreach_indexed_fun21621__t*, _fself);
  kk_function_t _b_19420_19410 = _self->_b_19420_19410; /* (int) -> <local<12564>|12571> () */
  kk_drop_match(_self, {kk_function_dup(_b_19420_19410);}, {}, _ctx)
  kk_unit_t _x21622 = kk_Unit;
  kk_integer_t _x21623 = kk_integer_unbox(_b_19411); /*int*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_context_t*), _b_19420_19410, (_b_19420_19410, _x21623, _ctx));
  return kk_unit_box(_x21622);
}


// lift anonymous function
struct kk_std_core__lift17190_foreach_indexed_fun21624__t {
  struct kk_function_s _base;
  kk_function_t action4;
  kk_ref_t i3;
  kk_std_core__list xx3;
};
static kk_unit_t kk_std_core__lift17190_foreach_indexed_fun21624(kk_function_t _fself, kk_unit_t wild__01, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17190_foreach_indexed_fun21624(kk_function_t action4, kk_ref_t i3, kk_std_core__list xx3, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21624__t* _self = kk_function_alloc_as(struct kk_std_core__lift17190_foreach_indexed_fun21624__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17190_foreach_indexed_fun21624, kk_context());
  _self->action4 = action4;
  _self->i3 = i3;
  _self->xx3 = xx3;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17190_foreach_indexed_fun21624(kk_function_t _fself, kk_unit_t wild__01, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21624__t* _self = kk_function_as(struct kk_std_core__lift17190_foreach_indexed_fun21624__t*, _fself);
  kk_function_t action4 = _self->action4; /* (int, 12570) -> 12571 () */
  kk_ref_t i3 = _self->i3; /* local-var<12564,int> */
  kk_std_core__list xx3 = _self->xx3; /* list<12570> */
  kk_drop_match(_self, {kk_function_dup(action4);kk_ref_dup(i3);kk_std_core__list_dup(xx3);}, {}, _ctx)
  return kk_std_core__mlift17751_op(action4, i3, xx3, wild__01, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17190_foreach_indexed_fun21626__t {
  struct kk_function_s _base;
  kk_function_t _b_19421_19416;
};
static kk_box_t kk_std_core__lift17190_foreach_indexed_fun21626(kk_function_t _fself, kk_box_t _b_19417, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17190_foreach_indexed_fun21626(kk_function_t _b_19421_19416, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21626__t* _self = kk_function_alloc_as(struct kk_std_core__lift17190_foreach_indexed_fun21626__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17190_foreach_indexed_fun21626, kk_context());
  _self->_b_19421_19416 = _b_19421_19416;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17190_foreach_indexed_fun21626(kk_function_t _fself, kk_box_t _b_19417, kk_context_t* _ctx) {
  struct kk_std_core__lift17190_foreach_indexed_fun21626__t* _self = kk_function_as(struct kk_std_core__lift17190_foreach_indexed_fun21626__t*, _fself);
  kk_function_t _b_19421_19416 = _self->_b_19421_19416; /* (wild_01 : ()) -> <local<12564>|12571> () */
  kk_drop_match(_self, {kk_function_dup(_b_19421_19416);}, {}, _ctx)
  kk_unit_t _x21627 = kk_Unit;
  kk_unit_t _x21628 = kk_Unit;
  kk_unit_unbox(_b_19417);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19421_19416, (_b_19421_19416, _x21628, _ctx));
  return kk_unit_box(_x21627);
}

kk_unit_t kk_std_core__lift17190_foreach_indexed(kk_function_t action4, kk_ref_t i3, kk_std_core__list xs0, kk_context_t* _ctx) { /* forall<h,a,e> (action : (int, a) -> e (), i : local-var<h,int>, xs0 : list<a>) -> <local<h>|e> () */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs0)) {
    kk_function_drop(action4, _ctx);
    kk_ref_drop(i3, _ctx);
    kk_Unit; return kk_Unit;
  }
  struct kk_std_core_Cons* _con21603 = kk_std_core__as_Cons(xs0);
  kk_box_t x01 = _con21603->head;
  kk_std_core__list xx3 = _con21603->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs0))) {
    kk_std_core__list_free(xs0);
  }
  else {
    kk_box_dup(x01);
    kk_std_core__list_dup(xx3);
    kk_Unit;
    kk_std_core__list_decref(xs0, _ctx);
    kk_Unit;
  }
  kk_integer_t x2_18150;
  kk_box_t _x21604;
  kk_ref_t _x21605 = kk_ref_dup(i3); /*local-var<12564,int>*/
  _x21604 = (kk_ref_get(_x21605,kk_context())); /*256*/
  x2_18150 = kk_integer_unbox(_x21604); /*int*/
  if (kk_yielding(kk_context())) {
    kk_integer_drop(x2_18150, _ctx);
    kk_function_t _b_19418_19404 = kk_std_core__new_lift17190_foreach_indexed_fun21606(action4, i3, x01, xx3, _ctx); /*(j0 : int) -> <local<12564>|12571> ()*/;
    kk_box_t _x21607 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17190_foreach_indexed_fun21608(_b_19418_19404, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21607); return kk_Unit;
  }
  kk_unit_t x3_18153 = kk_Unit;
  kk_function_t _x21611 = kk_function_dup(action4); /*(int, 12570) -> 12571 ()*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_box_t, kk_context_t*), _x21611, (_x21611, x2_18150, x01, _ctx));
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19419_19406 = kk_std_core__new_lift17190_foreach_indexed_fun21612(action4, i3, xx3, _ctx); /*(wild_2 : ()) -> <local<12564>|12571> ()*/;
    kk_box_t _x21613 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17190_foreach_indexed_fun21614(_b_19419_19406, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21613); return kk_Unit;
  }
  kk_integer_t x4_18157;
  kk_box_t _x21617;
  kk_ref_t _x21618 = kk_ref_dup(i3); /*local-var<12564,int>*/
  _x21617 = (kk_ref_get(_x21618,kk_context())); /*256*/
  x4_18157 = kk_integer_unbox(_x21617); /*int*/
  if (kk_yielding(kk_context())) {
    kk_integer_drop(x4_18157, _ctx);
    kk_function_t _b_19420_19410 = kk_std_core__new_lift17190_foreach_indexed_fun21619(action4, i3, xx3, _ctx); /*(int) -> <local<12564>|12571> ()*/;
    kk_box_t _x21620 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17190_foreach_indexed_fun21621(_b_19420_19410, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21620); return kk_Unit;
  }
  kk_integer_t _b_19415_19413 = kk_integer_add(x4_18157,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_unit_t x5_18160 = kk_Unit;
  (kk_ref_set_borrow(i3,(kk_integer_box(_b_19415_19413)),kk_context()));
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19421_19416 = kk_std_core__new_lift17190_foreach_indexed_fun21624(action4, i3, xx3, _ctx); /*(wild_01 : ()) -> <local<12564>|12571> ()*/;
    kk_box_t _x21625 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17190_foreach_indexed_fun21626(_b_19421_19416, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21625); return kk_Unit;
  }
  { // tailcall
    xs0 = xx3;
    goto kk__tailcall;
  }
}
 
// Invoke `action` for each element of a list, passing also the position of the element.

kk_unit_t kk_std_core_foreach_indexed(kk_std_core__list xs, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, action : (int, a) -> e ()) -> e () */ 
  kk_ref_t loc = kk_ref_alloc((kk_integer_box(kk_integer_from_small(0))),kk_context()); /*local-var<global,int>*/;
  kk_unit_t res = kk_Unit;
  kk_ref_t _x21629 = kk_ref_dup(loc); /*local-var<global,int>*/
  kk_std_core__lift17190_foreach_indexed(action, _x21629, xs, _ctx);
  kk_ref_t _b_19426_19424 = loc; /*local-var<global,int>*/;
  kk_unit_t _b_19427_19425 = kk_Unit;
  res;
  kk_box_t _x21630 = kk_std_core_hnd_prompt_local_var(_b_19426_19424, kk_unit_box(_b_19427_19425), _ctx); /*9895*/
  kk_unit_unbox(_x21630); return kk_Unit;
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17755_op(kk_function_t action_17100, kk_ssize_t end_17099, kk_ssize_t i, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action.17100 : (ssize_t) -> e (), end.17099 : ssize_t, i : ssize_t, wild_ : ()) -> e () */ 
  kk_ssize_t i0_17263 = kk_std_core_incr_1(i, _ctx); /*ssize_t*/;
  kk_std_core__lift17191_foreach_indexed_1(action_17100, end_17099, i0_17263, _ctx); return kk_Unit;
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17191_foreach_indexed_fun21632__t_1 {
  struct kk_function_s _base;
  kk_function_t action_171000;
  kk_ssize_t end_170990;
  kk_ssize_t i0;
};
static kk_unit_t kk_std_core__lift17191_foreach_indexed_fun21632_1(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17191_foreach_indexed_fun21632_1(kk_function_t action_171000, kk_ssize_t end_170990, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17191_foreach_indexed_fun21632__t_1* _self = kk_function_alloc_as(struct kk_std_core__lift17191_foreach_indexed_fun21632__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17191_foreach_indexed_fun21632_1, kk_context());
  _self->action_171000 = action_171000;
  _self->end_170990 = end_170990;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17191_foreach_indexed_fun21632_1(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17191_foreach_indexed_fun21632__t_1* _self = kk_function_as(struct kk_std_core__lift17191_foreach_indexed_fun21632__t_1*, _fself);
  kk_function_t action_171000 = _self->action_171000; /* (ssize_t) -> 12599 () */
  kk_ssize_t end_170990 = _self->end_170990; /* ssize_t */
  kk_ssize_t i0 = _self->i0; /* ssize_t */
  kk_drop_match(_self, {kk_function_dup(action_171000);;;}, {}, _ctx)
  return kk_std_core__mlift17755_op(action_171000, end_170990, i0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17191_foreach_indexed_fun21634__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19430_19428;
};
static kk_box_t kk_std_core__lift17191_foreach_indexed_fun21634_1(kk_function_t _fself, kk_box_t _b_19429, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17191_foreach_indexed_fun21634_1(kk_function_t _b_19430_19428, kk_context_t* _ctx) {
  struct kk_std_core__lift17191_foreach_indexed_fun21634__t_1* _self = kk_function_alloc_as(struct kk_std_core__lift17191_foreach_indexed_fun21634__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17191_foreach_indexed_fun21634_1, kk_context());
  _self->_b_19430_19428 = _b_19430_19428;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17191_foreach_indexed_fun21634_1(kk_function_t _fself, kk_box_t _b_19429, kk_context_t* _ctx) {
  struct kk_std_core__lift17191_foreach_indexed_fun21634__t_1* _self = kk_function_as(struct kk_std_core__lift17191_foreach_indexed_fun21634__t_1*, _fself);
  kk_function_t _b_19430_19428 = _self->_b_19430_19428; /* (wild_0 : ()) -> 12599 () */
  kk_drop_match(_self, {kk_function_dup(_b_19430_19428);}, {}, _ctx)
  kk_unit_t _x21635 = kk_Unit;
  kk_unit_t _x21636 = kk_Unit;
  kk_unit_unbox(_b_19429);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19430_19428, (_b_19430_19428, _x21636, _ctx));
  return kk_unit_box(_x21635);
}

kk_unit_t kk_std_core__lift17191_foreach_indexed_1(kk_function_t action_171000, kk_ssize_t end_170990, kk_ssize_t i0, kk_context_t* _ctx) { /* forall<e> (action.17100 : (ssize_t) -> e (), end.17099 : ssize_t, i : ssize_t) -> e () */ 
  kk__tailcall: ;
  bool _match_20024 = (i0 <= end_170990); /*bool*/;
  if (_match_20024) {
    kk_unit_t x_18165 = kk_Unit;
    kk_function_t _x21631 = kk_function_dup(action_171000); /*(ssize_t) -> 12599 ()*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_ssize_t, kk_context_t*), _x21631, (_x21631, i0, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19430_19428 = kk_std_core__new_lift17191_foreach_indexed_fun21632_1(action_171000, end_170990, i0, _ctx); /*(wild_0 : ()) -> 12599 ()*/;
      kk_box_t _x21633 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17191_foreach_indexed_fun21634_1(_b_19430_19428, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x21633); return kk_Unit;
    }
    kk_ssize_t i0_172630 = kk_std_core_incr_1(i0, _ctx); /*ssize_t*/;
    { // tailcall
      i0 = i0_172630;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action_171000, _ctx);
  kk_Unit; return kk_Unit;
}
 
// Invoke a function `f` for each element in a vector `v`


// lift anonymous function
struct kk_std_core_foreach_indexed_fun21638__t_1 {
  struct kk_function_s _base;
  kk_function_t f;
  kk_vector_t v;
};
static kk_unit_t kk_std_core_foreach_indexed_fun21638_1(kk_function_t _fself, kk_ssize_t i0, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_foreach_indexed_fun21638_1(kk_function_t f, kk_vector_t v, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexed_fun21638__t_1* _self = kk_function_alloc_as(struct kk_std_core_foreach_indexed_fun21638__t_1, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_foreach_indexed_fun21638_1, kk_context());
  _self->f = f;
  _self->v = v;
  return &_self->_base;
}

static kk_unit_t kk_std_core_foreach_indexed_fun21638_1(kk_function_t _fself, kk_ssize_t i0, kk_context_t* _ctx) {
  struct kk_std_core_foreach_indexed_fun21638__t_1* _self = kk_function_as(struct kk_std_core_foreach_indexed_fun21638__t_1*, _fself);
  kk_function_t f = _self->f; /* (12598, int) -> 12599 () */
  kk_vector_t v = _self->v; /* vector<12598> */
  kk_drop_match(_self, {kk_function_dup(f);kk_vector_dup(v);}, {}, _ctx)
  kk_box_t x_17265;
  kk_box_t _borrowApp_20023 = kk_vector_at_borrow(v, i0); /*219*/;
  kk_vector_drop(v, _ctx);
  x_17265 = _borrowApp_20023; /*12598*/
  kk_integer_t _x21639 = kk_integer_from_ssize_t(i0,kk_context()); /*int*/
  return kk_function_call(kk_unit_t, (kk_function_t, kk_box_t, kk_integer_t, kk_context_t*), f, (f, x_17265, _x21639, _ctx));
}

kk_unit_t kk_std_core_foreach_indexed_1(kk_vector_t v, kk_function_t f, kk_context_t* _ctx) { /* forall<a,e> (v : vector<a>, f : (a, int) -> e ()) -> e () */ 
  kk_ssize_t start0_17098 = ((kk_ssize_t)0); /*ssize_t*/;
  kk_ssize_t end_17099;
  kk_ssize_t _x21637 = kk_vector_len_borrow(v); /*ssize_t*/
  end_17099 = kk_std_core_decr_1(_x21637, _ctx); /*ssize_t*/
  kk_std_core__lift17191_foreach_indexed_1(kk_std_core_new_foreach_indexed_fun21638_1(f, v, _ctx), end_17099, start0_17098, _ctx); return kk_Unit;
}
 
// Return the head of list if the list is not empty.

kk_std_core_types__maybe kk_std_core_head_1(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> maybe<a> */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21648 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21648->head;
    kk_std_core__list _pat0 = _con21648->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_drop(_pat0, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    return kk_std_core_types__new_Just(x, _ctx);
  }
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Return the head of list if the list is not empty, or use `default` otherwise

kk_box_t kk_std_core_head_2(kk_std_core__list xs, kk_box_t default0, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, default : a) -> a */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21649 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21649->head;
    kk_std_core__list _pat0 = _con21649->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_drop(_pat0, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_box_drop(default0, _ctx);
    kk_Unit;
    return x;
  }
  return default0;
}
 
// Return the first character of a string as a string (or the empty string)

kk_string_t kk_std_core_head_3(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  kk_std_core__sslice _x21650;
  kk_std_core__sslice slice0 = kk_std_core_first1(s, _ctx); /*sslice*/;
  bool _match_20015;
  kk_integer_t _borrowedArg_20020;
  kk_std_core_types__optional _match_20021 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_20021)) {
    kk_box_t _box_x19431 = _match_20021._cons.Optional.value;
    kk_integer_t _n_10065 = kk_integer_unbox(_box_x19431);
    kk_integer_dup(_n_10065);
    kk_std_core_types__optional_drop(_match_20021, _ctx);
    kk_Unit;
    _borrowedArg_20020 = _n_10065; /*int*/
    goto _match21651;
  }
  _borrowedArg_20020 = kk_integer_from_small(1); /*int*/
  _match21651: ;
  kk_integer_t _borrowedArg_20019 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_20022 = kk_integer_eq_borrow(_borrowedArg_20020,_borrowedArg_20019,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_20020, _ctx);
  kk_integer_drop(_borrowedArg_20019, _ctx);
  _match_20015 = _borrowApp_20022; /*bool*/
  if (_match_20015) {
    _x21650 = slice0; /*sslice*/
  }
  else {
    kk_integer_t _borrowedArg_20016;
    kk_integer_t _x21653;
    kk_std_core_types__optional _match_20017 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_20017)) {
      kk_box_t _box_x19432 = _match_20017._cons.Optional.value;
      kk_integer_t _n_100650 = kk_integer_unbox(_box_x19432);
      kk_integer_dup(_n_100650);
      kk_std_core_types__optional_drop(_match_20017, _ctx);
      kk_Unit;
      _x21653 = _n_100650; /*int*/
      goto _match21654;
    }
    _x21653 = kk_integer_from_small(1); /*int*/
    _match21654: ;
    _borrowedArg_20016 = kk_integer_sub(_x21653,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__sslice _borrowApp_20018 = kk_std_core_extend(slice0, _borrowedArg_20016, _ctx); /*sslice*/;
    kk_integer_drop(_borrowedArg_20016, _ctx);
    _x21650 = _borrowApp_20018; /*sslice*/
  }
  return kk_std_core_string_3(_x21650, _ctx);
}
 
// Return the first character of a string (or `Nothing` for the empty string).


// lift anonymous function
struct kk_std_core_head_char_fun21660__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_core_head_char_fun21660(kk_function_t _fself, kk_char_t _b_19433, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_head_char_fun21660(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_head_char_fun21660, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_core_head_char_fun21660(kk_function_t _fself, kk_char_t _b_19433, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_core_types__new_Just(kk_char_box(_b_19433, _ctx), _ctx);
}

kk_std_core_types__maybe kk_std_core_head_char(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> maybe<char> */ 
  kk_std_core__sslice _x21656;
  kk_string_t _x21657 = kk_string_dup(s); /*string*/
  kk_ssize_t _x21658 = ((kk_ssize_t)0); /*ssize_t*/
  kk_ssize_t _x21659 = kk_string_len(s,kk_context()); /*ssize_t*/
  _x21656 = kk_std_core__new_Sslice(_x21657, _x21658, _x21659, _ctx); /*sslice*/
  return kk_std_core_foreach_while_1(_x21656, kk_std_core_new_head_char_fun21660(_ctx), _ctx);
}

kk_integer_t kk_std_core_index_of_acc(kk_std_core__list xs, kk_function_t pred, kk_integer_t idx, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, pred : (a) -> bool, idx : int) -> int */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_integer_drop(idx, _ctx);
    kk_function_drop(pred, _ctx);
    return kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context());
  }
  struct kk_std_core_Cons* _con21661 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21661->head;
  kk_std_core__list xx = _con21661->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  bool _match_20014;
  kk_function_t _x21662 = kk_function_dup(pred); /*(12975) -> bool*/
  _match_20014 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21662, (_x21662, x, _ctx)); /*bool*/
  if (_match_20014) {
    kk_function_drop(pred, _ctx);
    kk_std_core__list_drop(xx, _ctx);
    return idx;
  }
  { // tailcall
    kk_integer_t _x21663 = kk_integer_add(idx,(kk_integer_from_small(1)),kk_context()); /*int*/
    xs = xx;
    idx = _x21663;
    goto kk__tailcall;
  }
}
 
// Return the list without its last element.
// Return an empty list for an empty list.

kk_std_core__list kk_std_core__ctail_init(kk_std_core__list xs, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21664 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21664->head;
    kk_std_core__list xx = _con21664->tail;
    if (kk_std_core__is_Cons(xx)) {
      struct kk_std_core_Cons* _con21665 = kk_std_core__as_Cons(xx);
      kk_reuse_t _ru_20394 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        _ru_20394 = (kk_std_core__list_reuse(xs));
      }
      else {
        kk_box_dup(x);
        kk_std_core__list_dup(xx);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      kk_std_core__list _ctail_17349 = kk_std_core__list_hole(); /*list<13012>*/;
      kk_std_core__list _ctail_17350;
      if (kk_likely(_ru_20394!=NULL)) {
        struct kk_std_core_Cons* _con21666 = (struct kk_std_core_Cons*)_ru_20394;
        _con21666->tail = _ctail_17349;
        _ctail_17350 = kk_std_core__base_Cons(_con21666); /*list<13012>*/
      }
      else {
        _ctail_17350 = kk_std_core__new_Cons(kk_reuse_null, x, _ctail_17349, _ctx); /*list<13012>*/
      }
      { // tailcall
        kk_std_core_types__ctail _x21667;
        kk_box_t* field_18170 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17350)->tail)); /*cfield<list<13012>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x19440 = _acc._cons.CTail._field1;
          kk_box_t* last0 = _acc._cons.CTail._field2;
          kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19440, NULL);
          kk_std_core__list_dup(head0);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last0) = kk_std_core__list_box(_ctail_17350, _ctx);
          kk_std_core__list _b_19454_19445 = head0; /*list<13012>*/;
          kk_box_t* _b_19455_19446 = field_18170; /*cfield<list<13012>>*/;
          _x21667 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19454_19445, _ctx), _b_19455_19446, _ctx); /*ctail<58>*/
          goto _match21668;
        }
        kk_std_core__list _b_19456_19447 = _ctail_17350; /*list<13012>*/;
        kk_box_t* _b_19457_19448 = field_18170; /*cfield<list<13012>>*/;
        _x21667 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19456_19447, _ctx), _b_19457_19448, _ctx); /*ctail<58>*/
        _match21668: ;
        xs = xx;
        _acc = _x21667;
        goto kk__tailcall;
      }
    }
  }
  kk_std_core__list_drop(xs, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19449 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19449, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Return the list without its last element.
// Return an empty list for an empty list.

kk_std_core__list kk_std_core_init(kk_std_core__list xs0, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> list<a> */ 
  return kk_std_core__ctail_init(xs0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// An invalid slice

kk_std_core__sslice kk_std_core_invalid;
 
// Is the character an ASCII letter is-

bool kk_std_core_is_alpha(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  bool _match_20008 = (c >= ('a')); /*bool*/;
  if (_match_20008) {
    bool _match_20010 = (c <= ('z')); /*bool*/;
    if (_match_20010) {
      return true;
    }
    bool _match_20011 = (c >= ('A')); /*bool*/;
    if (_match_20011) {
      return (c <= ('Z'));
    }
    return false;
  }
  bool _match_20009 = (c >= ('A')); /*bool*/;
  if (_match_20009) {
    return (c <= ('Z'));
  }
  return false;
}
 
// Is the character an ASCII hexa-decimal digit ?

bool kk_std_core_is_hex_digit(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  bool _match_19995 = (c >= ('0')); /*bool*/;
  if (_match_19995) {
    bool _match_20000 = (c <= ('9')); /*bool*/;
    if (_match_20000) {
      return true;
    }
    bool _match_20001 = (c >= ('a')); /*bool*/;
    if (_match_20001) {
      bool _match_20003 = (c <= ('f')); /*bool*/;
      if (_match_20003) {
        return true;
      }
      bool _match_20004 = (c >= ('A')); /*bool*/;
      if (_match_20004) {
        return (c <= ('F'));
      }
      return false;
    }
    bool _match_20002 = (c >= ('A')); /*bool*/;
    if (_match_20002) {
      return (c <= ('F'));
    }
    return false;
  }
  bool _match_19996 = (c >= ('a')); /*bool*/;
  if (_match_19996) {
    bool _match_19998 = (c <= ('f')); /*bool*/;
    if (_match_19998) {
      return true;
    }
    bool _match_19999 = (c >= ('A')); /*bool*/;
    if (_match_19999) {
      return (c <= ('F'));
    }
    return false;
  }
  bool _match_19997 = (c >= ('A')); /*bool*/;
  if (_match_19997) {
    return (c <= ('F'));
  }
  return false;
}
 
// Tests if a character is an element of `" \t\n\r"`

bool kk_std_core_is_white(kk_char_t c, kk_context_t* _ctx) { /* (c : char) -> bool */ 
  bool _match_19992 = (c == (' ')); /*bool*/;
  if (_match_19992) {
    return true;
  }
  bool _match_19993 = (c == 0x0009); /*bool*/;
  if (_match_19993) {
    return true;
  }
  bool _match_19994 = (c == 0x000A); /*bool*/;
  if (_match_19994) {
    return true;
  }
  return (c == 0x000D);
}
 
// Append `end` to each string in the list `xs` and join them all together.
// `join-end([],end) === ""`
// `join-end(["a","b"],"/") === "a/b/"`

kk_string_t kk_std_core_join_end(kk_std_core__list xs, kk_string_t end, kk_context_t* _ctx) { /* (xs : list<string>, end : string) -> string */ 
  if (kk_std_core__is_Nil(xs)) {
    kk_string_drop(end, _ctx);
    return kk_string_empty();
  }
  kk_string_t _x21681;
  if (kk_std_core__is_Nil(xs)) {
    _x21681 = kk_string_empty(); /*string*/
  }
  else {
    struct kk_std_core_Cons* _con21683 = kk_std_core__as_Cons(xs);
    kk_box_t _box_x19458 = _con21683->head;
    kk_std_core__list xx = _con21683->tail;
    kk_string_t x = kk_string_unbox(_box_x19458);
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_string_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    kk_string_t _x21685 = kk_string_dup(end); /*string*/
    _x21681 = kk_std_core__lift17169_joinsep(_x21685, xx, x, _ctx); /*string*/
  }
  return kk_std_core__lp__plus__plus__1_rp_(_x21681, end, _ctx);
}
 
// Return the last element of a list (or `Nothing` for the empty list)

kk_std_core_types__maybe kk_std_core_last(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> maybe<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21686 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21686->head;
    kk_std_core__list _pat0 = _con21686->tail;
    if (kk_std_core__is_Nil(_pat0)) {
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        if (kk_likely(kk_std_core__list_is_unique(_pat0))) {
          kk_std_core__list_free(_pat0);
        }
        else {
          kk_std_core__list_decref(_pat0, _ctx);
          kk_Unit;
        }
        kk_Unit;
        kk_std_core__list_free(xs);
      }
      else {
        kk_box_dup(x);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      return kk_std_core_types__new_Just(x, _ctx);
    }
  }
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21687 = kk_std_core__as_Cons(xs);
    kk_box_t _pat2 = _con21687->head;
    kk_std_core__list xx = _con21687->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_box_drop(_pat2, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs);
    }
    else {
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    { // tailcall
      xs = xx;
      goto kk__tailcall;
    }
  }
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Return the last element of a list (or `default` for the empty list)

kk_box_t kk_std_core_last_1(kk_std_core__list xs, kk_box_t default0, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, default : a) -> a */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21688 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21688->head;
    kk_std_core__list _pat0 = _con21688->tail;
    if (kk_std_core__is_Nil(_pat0)) {
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        if (kk_likely(kk_std_core__list_is_unique(_pat0))) {
          kk_std_core__list_free(_pat0);
        }
        else {
          kk_std_core__list_decref(_pat0, _ctx);
          kk_Unit;
        }
        kk_Unit;
        kk_std_core__list_free(xs);
      }
      else {
        kk_box_dup(x);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      kk_box_drop(default0, _ctx);
      kk_Unit;
      return x;
    }
  }
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21689 = kk_std_core__as_Cons(xs);
    kk_box_t _pat2 = _con21689->head;
    kk_std_core__list xx = _con21689->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_box_drop(_pat2, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs);
    }
    else {
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    { // tailcall
      xs = xx;
      goto kk__tailcall;
    }
  }
  return default0;
}
 
// O(`n`). The last `n` (default = `1`) characters in a string

kk_std_core__sslice kk_std_core_last_2(kk_string_t s, kk_std_core_types__optional n, kk_context_t* _ctx) { /* (s : string, n : optional<int>) -> sslice */ 
  kk_std_core__sslice slice0 = kk_std_core_last1(s, _ctx); /*sslice*/;
  bool _match_19983;
  kk_integer_t _borrowedArg_19990;
  if (kk_std_core_types__is_Optional(n)) {
    kk_box_t _box_x19459 = n._cons.Optional.value;
    kk_integer_t _n_13606 = kk_integer_unbox(_box_x19459);
    kk_integer_dup(_n_13606);
    _borrowedArg_19990 = _n_13606; /*int*/
    goto _match21690;
  }
  _borrowedArg_19990 = kk_integer_from_small(1); /*int*/
  _match21690: ;
  kk_integer_t _borrowedArg_19989 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19991 = kk_integer_eq_borrow(_borrowedArg_19990,_borrowedArg_19989,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19990, _ctx);
  kk_integer_drop(_borrowedArg_19989, _ctx);
  _match_19983 = _borrowApp_19991; /*bool*/
  if (_match_19983) {
    kk_std_core_types__optional_drop(n, _ctx);
    return slice0;
  }
  kk_std_core__sslice _ownedArg_19985;
  kk_integer_t _borrowedArg_19986;
  kk_integer_t _x21692;
  if (kk_std_core_types__is_Optional(n)) {
    kk_box_t _box_x19460 = n._cons.Optional.value;
    kk_integer_t _n_136060 = kk_integer_unbox(_box_x19460);
    kk_integer_dup(_n_136060);
    _x21692 = _n_136060; /*int*/
    goto _match21693;
  }
  _x21692 = kk_integer_from_small(1); /*int*/
  _match21693: ;
  _borrowedArg_19986 = kk_integer_sub((kk_integer_from_small(1)),_x21692,kk_context()); /*int*/
  kk_std_core__sslice _borrowApp_19987 = kk_std_core_advance(slice0, _borrowedArg_19986, _ctx); /*sslice*/;
  kk_integer_drop(_borrowedArg_19986, _ctx);
  _ownedArg_19985 = _borrowApp_19987; /*sslice*/
  kk_integer_t _borrowedArg_19984;
  kk_integer_t _x21695;
  if (kk_std_core_types__is_Optional(n)) {
    kk_box_t _box_x19461 = n._cons.Optional.value;
    kk_integer_t _n_136061 = kk_integer_unbox(_box_x19461);
    kk_integer_dup(_n_136061);
    kk_std_core_types__optional_drop(n, _ctx);
    kk_Unit;
    _x21695 = _n_136061; /*int*/
    goto _match21696;
  }
  _x21695 = kk_integer_from_small(1); /*int*/
  _match21696: ;
  _borrowedArg_19984 = kk_integer_sub(_x21695,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_std_core__sslice _borrowApp_19988 = kk_std_core_extend(_ownedArg_19985, _borrowedArg_19984, _ctx); /*sslice*/;
  kk_integer_drop(_borrowedArg_19984, _ctx);
  return _borrowApp_19988;
}
 
// Take the first `n` elements of a list (or fewer if the list is shorter than `n`)

kk_std_core__list kk_std_core__ctail_take(kk_std_core__list xs, kk_integer_t n, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21698 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con21698->head;
    kk_std_core__list xx = _con21698->tail;
    kk_integer_t _borrowedArg_19981 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_19982 = kk_integer_gt_borrow(n,_borrowedArg_19981,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_19981, _ctx);
    if (_borrowApp_19982) {
      kk_reuse_t _ru_20401 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(xs))) {
        _ru_20401 = (kk_std_core__list_reuse(xs));
      }
      else {
        kk_box_dup(x);
        kk_std_core__list_dup(xx);
        kk_Unit;
        kk_std_core__list_decref(xs, _ctx);
        kk_Unit;
      }
      kk_std_core__list _ctail_17351 = kk_std_core__list_hole(); /*list<13730>*/;
      kk_std_core__list _ctail_17352;
      if (kk_likely(_ru_20401!=NULL)) {
        struct kk_std_core_Cons* _con21699 = (struct kk_std_core_Cons*)_ru_20401;
        _con21699->tail = _ctail_17351;
        _ctail_17352 = kk_std_core__base_Cons(_con21699); /*list<13730>*/
      }
      else {
        _ctail_17352 = kk_std_core__new_Cons(kk_reuse_null, x, _ctail_17351, _ctx); /*list<13730>*/
      }
      { // tailcall
        kk_integer_t _x21700 = kk_integer_sub(n,(kk_integer_from_small(1)),kk_context()); /*int*/
        kk_std_core_types__ctail _x21701;
        kk_box_t* field_18177 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17352)->tail)); /*cfield<list<13730>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x19468 = _acc._cons.CTail._field1;
          kk_box_t* last0 = _acc._cons.CTail._field2;
          kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19468, NULL);
          kk_std_core__list_dup(head0);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last0) = kk_std_core__list_box(_ctail_17352, _ctx);
          kk_std_core__list _b_19482_19473 = head0; /*list<13730>*/;
          kk_box_t* _b_19483_19474 = field_18177; /*cfield<list<13730>>*/;
          _x21701 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19482_19473, _ctx), _b_19483_19474, _ctx); /*ctail<58>*/
          goto _match21702;
        }
        kk_std_core__list _b_19484_19475 = _ctail_17352; /*list<13730>*/;
        kk_box_t* _b_19485_19476 = field_18177; /*cfield<list<13730>>*/;
        _x21701 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19484_19475, _ctx), _b_19485_19476, _ctx); /*ctail<58>*/
        _match21702: ;
        xs = xx;
        n = _x21700;
        _acc = _x21701;
        goto kk__tailcall;
      }
    }
  }
  kk_integer_drop(n, _ctx);
  kk_std_core__list_drop(xs, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19477 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19477, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Take the first `n` elements of a list (or fewer if the list is shorter than `n`)

kk_std_core__list kk_std_core_take(kk_std_core__list xs0, kk_integer_t n0, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int) -> list<a> */ 
  return kk_std_core__ctail_take(xs0, n0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// split a list at position `n`

kk_std_core_types__tuple2_ kk_std_core_split(kk_std_core__list xs, kk_integer_t n, kk_context_t* _ctx) { /* forall<a> (xs : list<a>, n : int) -> (list<a>, list<a>) */ 
  kk_std_core__list _b_19488_19486;
  kk_std_core__list _x21705 = kk_std_core__list_dup(xs); /*list<13750>*/
  kk_integer_t _x21706 = kk_integer_dup(n); /*int*/
  _b_19488_19486 = kk_std_core_take(_x21705, _x21706, _ctx); /*list<13750>*/
  kk_std_core__list _b_19489_19487 = kk_std_core_drop(xs, n, _ctx); /*list<13750>*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19488_19486, _ctx), kk_std_core__list_box(_b_19489_19487, _ctx), _ctx);
}
 
// lifted
// specialized std/core/foreach-while to parameters [action]

kk_std_core_types__maybe kk_std_core__lift17192_lookup(kk_function_t pred, kk_std_core__list xs0, kk_context_t* _ctx) { /* forall<a,b> (pred : (a) -> bool, xs0 : list<(a, b)>) -> maybe<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs0)) {
    kk_function_drop(pred, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  struct kk_std_core_Cons* _con21711 = kk_std_core__as_Cons(xs0);
  kk_box_t _box_x19490 = _con21711->head;
  kk_std_core__list xx = _con21711->tail;
  kk_std_core_types__tuple2_ x = kk_std_core_types__tuple2__unbox(_box_x19490, NULL);
  if (kk_likely(kk_std_core__list_is_unique(xs0))) {
    kk_std_core_types__tuple2__dup(x);
    kk_box_drop(_box_x19490, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core__list_free(xs0);
  }
  else {
    kk_std_core_types__tuple2__dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs0, _ctx);
    kk_Unit;
  }
  bool _match_19980;
  kk_function_t _x21714 = kk_function_dup(pred); /*(14082) -> bool*/
  kk_box_t _x21713;
  kk_box_t _x = x.fst;
  kk_box_dup(_x);
  _x21713 = _x; /*14082*/
  _match_19980 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21714, (_x21714, _x21713, _ctx)); /*bool*/
  if (_match_19980) {
    kk_function_drop(pred, _ctx);
    kk_std_core__list_drop(xx, _ctx);
    kk_box_t _x21715;
    kk_box_t _x0 = x.snd;
    kk_box_dup(_x0);
    kk_std_core_types__tuple2__drop(x, _ctx);
    kk_Unit;
    _x21715 = _x0; /*14083*/
    return kk_std_core_types__new_Just(_x21715, _ctx);
  }
  kk_std_core_types__tuple2__drop(x, _ctx);
  kk_std_core_types__maybe _pat10 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/;
  kk_std_core_types__maybe _scrut = _pat10; /*maybe<14083>*/;
  kk_std_core_types__maybe_drop(_scrut, _ctx);
  { // tailcall
    xs0 = xx;
    goto kk__tailcall;
  }
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17756_op(kk_box_t _y_17603, kk_std_core__list _y_17604, kk_context_t* _ctx) { /* forall<a,e> (a, list<a>) -> e list<a> */ 
  return kk_std_core__new_Cons(kk_reuse_null, _y_17603, _y_17604, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17757_op_fun21716__t {
  struct kk_function_s _base;
  kk_box_t _y_176030;
};
static kk_std_core__list kk_std_core__mlift17757_op_fun21716(kk_function_t _fself, kk_std_core__list _y_176040, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17757_op_fun21716(kk_box_t _y_176030, kk_context_t* _ctx) {
  struct kk_std_core__mlift17757_op_fun21716__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17757_op_fun21716__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17757_op_fun21716, kk_context());
  _self->_y_176030 = _y_176030;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17757_op_fun21716(kk_function_t _fself, kk_std_core__list _y_176040, kk_context_t* _ctx) {
  struct kk_std_core__mlift17757_op_fun21716__t* _self = kk_function_as(struct kk_std_core__mlift17757_op_fun21716__t*, _fself);
  kk_box_t _y_176030 = _self->_y_176030; /* 14138 */
  kk_drop_match(_self, {kk_box_dup(_y_176030);}, {}, _ctx)
  return kk_std_core__mlift17756_op(_y_176030, _y_176040, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17757_op_fun21718__t {
  struct kk_function_s _base;
  kk_function_t _b_19493_19491;
};
static kk_box_t kk_std_core__mlift17757_op_fun21718(kk_function_t _fself, kk_box_t _b_19492, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17757_op_fun21718(kk_function_t _b_19493_19491, kk_context_t* _ctx) {
  struct kk_std_core__mlift17757_op_fun21718__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17757_op_fun21718__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17757_op_fun21718, kk_context());
  _self->_b_19493_19491 = _b_19493_19491;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17757_op_fun21718(kk_function_t _fself, kk_box_t _b_19492, kk_context_t* _ctx) {
  struct kk_std_core__mlift17757_op_fun21718__t* _self = kk_function_as(struct kk_std_core__mlift17757_op_fun21718__t*, _fself);
  kk_function_t _b_19493_19491 = _self->_b_19493_19491; /* (list<14138>) -> 14139 list<14138> */
  kk_drop_match(_self, {kk_function_dup(_b_19493_19491);}, {}, _ctx)
  kk_std_core__list _x21719;
  kk_std_core__list _x21720 = kk_std_core__list_unbox(_b_19492, _ctx); /*list<14138>*/
  _x21719 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19493_19491, (_b_19493_19491, _x21720, _ctx)); /*list<14138>*/
  return kk_std_core__list_box(_x21719, _ctx);
}

kk_std_core__list kk_std_core__mlift17757_op(kk_function_t f, kk_integer_t i, kk_std_core__list yy, kk_box_t _y_176030, kk_context_t* _ctx) { /* forall<a,b,e> (f : (idx : int, value : a) -> e b, i : int, yy : list<a>, b) -> e list<b> */ 
  kk_integer_t i0_17270 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__list x_18180 = kk_std_core__lift17193_map_indexed(f, yy, i0_17270, _ctx); /*list<14138>*/;
  if (kk_yielding(kk_context())) {
    kk_std_core__list_drop(x_18180, _ctx);
    kk_function_t _b_19493_19491 = kk_std_core__new_mlift17757_op_fun21716(_y_176030, _ctx); /*(list<14138>) -> 14139 list<14138>*/;
    kk_box_t _x21717 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17757_op_fun21718(_b_19493_19491, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21717, _ctx);
  }
  return kk_std_core__mlift17756_op(_y_176030, x_18180, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17193_map_indexed_fun21724__t {
  struct kk_function_s _base;
  kk_function_t f0;
  kk_integer_t i0;
  kk_std_core__list yy0;
};
static kk_std_core__list kk_std_core__lift17193_map_indexed_fun21724(kk_function_t _fself, kk_box_t _y_176031, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17193_map_indexed_fun21724(kk_function_t f0, kk_integer_t i0, kk_std_core__list yy0, kk_context_t* _ctx) {
  struct kk_std_core__lift17193_map_indexed_fun21724__t* _self = kk_function_alloc_as(struct kk_std_core__lift17193_map_indexed_fun21724__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17193_map_indexed_fun21724, kk_context());
  _self->f0 = f0;
  _self->i0 = i0;
  _self->yy0 = yy0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17193_map_indexed_fun21724(kk_function_t _fself, kk_box_t _y_176031, kk_context_t* _ctx) {
  struct kk_std_core__lift17193_map_indexed_fun21724__t* _self = kk_function_as(struct kk_std_core__lift17193_map_indexed_fun21724__t*, _fself);
  kk_function_t f0 = _self->f0; /* (idx : int, value : 14137) -> 14139 14138 */
  kk_integer_t i0 = _self->i0; /* int */
  kk_std_core__list yy0 = _self->yy0; /* list<14137> */
  kk_drop_match(_self, {kk_function_dup(f0);kk_integer_dup(i0);kk_std_core__list_dup(yy0);}, {}, _ctx)
  return kk_std_core__mlift17757_op(f0, i0, yy0, _y_176031, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17193_map_indexed_fun21726__t {
  struct kk_function_s _base;
  kk_function_t _b_19498_19494;
};
static kk_box_t kk_std_core__lift17193_map_indexed_fun21726(kk_function_t _fself, kk_box_t _b_19495, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17193_map_indexed_fun21726(kk_function_t _b_19498_19494, kk_context_t* _ctx) {
  struct kk_std_core__lift17193_map_indexed_fun21726__t* _self = kk_function_alloc_as(struct kk_std_core__lift17193_map_indexed_fun21726__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17193_map_indexed_fun21726, kk_context());
  _self->_b_19498_19494 = _b_19498_19494;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17193_map_indexed_fun21726(kk_function_t _fself, kk_box_t _b_19495, kk_context_t* _ctx) {
  struct kk_std_core__lift17193_map_indexed_fun21726__t* _self = kk_function_as(struct kk_std_core__lift17193_map_indexed_fun21726__t*, _fself);
  kk_function_t _b_19498_19494 = _self->_b_19498_19494; /* (14138) -> 14139 list<14138> */
  kk_drop_match(_self, {kk_function_dup(_b_19498_19494);}, {}, _ctx)
  kk_std_core__list _x21727 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19498_19494, (_b_19498_19494, _b_19495, _ctx)); /*list<14138>*/
  return kk_std_core__list_box(_x21727, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17193_map_indexed_fun21728__t {
  struct kk_function_s _base;
  kk_box_t x0_18182;
};
static kk_std_core__list kk_std_core__lift17193_map_indexed_fun21728(kk_function_t _fself, kk_std_core__list _y_176041, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17193_map_indexed_fun21728(kk_box_t x0_18182, kk_context_t* _ctx) {
  struct kk_std_core__lift17193_map_indexed_fun21728__t* _self = kk_function_alloc_as(struct kk_std_core__lift17193_map_indexed_fun21728__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17193_map_indexed_fun21728, kk_context());
  _self->x0_18182 = x0_18182;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17193_map_indexed_fun21728(kk_function_t _fself, kk_std_core__list _y_176041, kk_context_t* _ctx) {
  struct kk_std_core__lift17193_map_indexed_fun21728__t* _self = kk_function_as(struct kk_std_core__lift17193_map_indexed_fun21728__t*, _fself);
  kk_box_t x0_18182 = _self->x0_18182; /* 14138 */
  kk_drop_match(_self, {kk_box_dup(x0_18182);}, {}, _ctx)
  return kk_std_core__mlift17756_op(x0_18182, _y_176041, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17193_map_indexed_fun21730__t {
  struct kk_function_s _base;
  kk_function_t _b_19499_19496;
};
static kk_box_t kk_std_core__lift17193_map_indexed_fun21730(kk_function_t _fself, kk_box_t _b_19497, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17193_map_indexed_fun21730(kk_function_t _b_19499_19496, kk_context_t* _ctx) {
  struct kk_std_core__lift17193_map_indexed_fun21730__t* _self = kk_function_alloc_as(struct kk_std_core__lift17193_map_indexed_fun21730__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17193_map_indexed_fun21730, kk_context());
  _self->_b_19499_19496 = _b_19499_19496;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17193_map_indexed_fun21730(kk_function_t _fself, kk_box_t _b_19497, kk_context_t* _ctx) {
  struct kk_std_core__lift17193_map_indexed_fun21730__t* _self = kk_function_as(struct kk_std_core__lift17193_map_indexed_fun21730__t*, _fself);
  kk_function_t _b_19499_19496 = _self->_b_19499_19496; /* (list<14138>) -> 14139 list<14138> */
  kk_drop_match(_self, {kk_function_dup(_b_19499_19496);}, {}, _ctx)
  kk_std_core__list _x21731;
  kk_std_core__list _x21732 = kk_std_core__list_unbox(_b_19497, _ctx); /*list<14138>*/
  _x21731 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19499_19496, (_b_19499_19496, _x21732, _ctx)); /*list<14138>*/
  return kk_std_core__list_box(_x21731, _ctx);
}

kk_std_core__list kk_std_core__lift17193_map_indexed(kk_function_t f0, kk_std_core__list ys, kk_integer_t i0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (idx : int, value : a) -> e b, ys : list<a>, i : int) -> e list<b> */ 
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21721 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con21721->head;
    kk_std_core__list yy0 = _con21721->tail;
    kk_reuse_t _ru_20403 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20403 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy0);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_box_t x0_18182;
    kk_function_t _x21723 = kk_function_dup(f0); /*(idx : int, value : 14137) -> 14139 14138*/
    kk_integer_t _x21722 = kk_integer_dup(i0); /*int*/
    x0_18182 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_box_t, kk_context_t*), _x21723, (_x21723, _x21722, y, _ctx)); /*14138*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20403);
      kk_box_drop(x0_18182, _ctx);
      kk_function_t _b_19498_19494 = kk_std_core__new_lift17193_map_indexed_fun21724(f0, i0, yy0, _ctx); /*(14138) -> 14139 list<14138>*/;
      kk_box_t _x21725 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17193_map_indexed_fun21726(_b_19498_19494, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21725, _ctx);
    }
    kk_integer_t i0_172700 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
    kk_std_core__list x1_18185 = kk_std_core__lift17193_map_indexed(f0, yy0, i0_172700, _ctx); /*list<14138>*/;
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20403);
      kk_std_core__list_drop(x1_18185, _ctx);
      kk_function_t _b_19499_19496 = kk_std_core__new_lift17193_map_indexed_fun21728(x0_18182, _ctx); /*(list<14138>) -> 14139 list<14138>*/;
      kk_box_t _x21729 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17193_map_indexed_fun21730(_b_19499_19496, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21729, _ctx);
    }
    return kk_std_core__new_Cons(_ru_20403, x0_18182, x1_18185, _ctx);
  }
  kk_function_drop(f0, _ctx);
  kk_integer_drop(i0, _ctx);
  return kk_std_core__new_Nil(_ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17758_op(kk_box_t _y_17607, kk_std_core__list _y_17608, kk_context_t* _ctx) { /* forall<a,e> (a, list<a>) -> e list<a> */ 
  return kk_std_core__new_Cons(kk_reuse_null, _y_17607, _y_17608, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17759_op_fun21733__t {
  struct kk_function_s _base;
  kk_box_t _y_176070;
};
static kk_std_core__list kk_std_core__mlift17759_op_fun21733(kk_function_t _fself, kk_std_core__list _y_176080, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17759_op_fun21733(kk_box_t _y_176070, kk_context_t* _ctx) {
  struct kk_std_core__mlift17759_op_fun21733__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17759_op_fun21733__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17759_op_fun21733, kk_context());
  _self->_y_176070 = _y_176070;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17759_op_fun21733(kk_function_t _fself, kk_std_core__list _y_176080, kk_context_t* _ctx) {
  struct kk_std_core__mlift17759_op_fun21733__t* _self = kk_function_as(struct kk_std_core__mlift17759_op_fun21733__t*, _fself);
  kk_box_t _y_176070 = _self->_y_176070; /* 14195 */
  kk_drop_match(_self, {kk_box_dup(_y_176070);}, {}, _ctx)
  return kk_std_core__mlift17758_op(_y_176070, _y_176080, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17759_op_fun21735__t {
  struct kk_function_s _base;
  kk_function_t _b_19502_19500;
};
static kk_box_t kk_std_core__mlift17759_op_fun21735(kk_function_t _fself, kk_box_t _b_19501, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17759_op_fun21735(kk_function_t _b_19502_19500, kk_context_t* _ctx) {
  struct kk_std_core__mlift17759_op_fun21735__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17759_op_fun21735__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17759_op_fun21735, kk_context());
  _self->_b_19502_19500 = _b_19502_19500;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17759_op_fun21735(kk_function_t _fself, kk_box_t _b_19501, kk_context_t* _ctx) {
  struct kk_std_core__mlift17759_op_fun21735__t* _self = kk_function_as(struct kk_std_core__mlift17759_op_fun21735__t*, _fself);
  kk_function_t _b_19502_19500 = _self->_b_19502_19500; /* (list<14195>) -> 14196 list<14195> */
  kk_drop_match(_self, {kk_function_dup(_b_19502_19500);}, {}, _ctx)
  kk_std_core__list _x21736;
  kk_std_core__list _x21737 = kk_std_core__list_unbox(_b_19501, _ctx); /*list<14195>*/
  _x21736 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19502_19500, (_b_19502_19500, _x21737, _ctx)); /*list<14195>*/
  return kk_std_core__list_box(_x21736, _ctx);
}

kk_std_core__list kk_std_core__mlift17759_op(kk_function_t f, kk_integer_t i, kk_std_core__list yy, kk_box_t _y_176070, kk_context_t* _ctx) { /* forall<a,b,e> (f : (idx : int, value : a, rest : list<a>) -> e b, i : int, yy : list<a>, b) -> e list<b> */ 
  kk_integer_t i0_17272 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__list x_18188 = kk_std_core__lift17194_map_indexed_peek(f, yy, i0_17272, _ctx); /*list<14195>*/;
  if (kk_yielding(kk_context())) {
    kk_std_core__list_drop(x_18188, _ctx);
    kk_function_t _b_19502_19500 = kk_std_core__new_mlift17759_op_fun21733(_y_176070, _ctx); /*(list<14195>) -> 14196 list<14195>*/;
    kk_box_t _x21734 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17759_op_fun21735(_b_19502_19500, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21734, _ctx);
  }
  return kk_std_core__mlift17758_op(_y_176070, x_18188, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17194_map_indexed_peek_fun21742__t {
  struct kk_function_s _base;
  kk_function_t f0;
  kk_integer_t i0;
  kk_std_core__list yy0;
};
static kk_std_core__list kk_std_core__lift17194_map_indexed_peek_fun21742(kk_function_t _fself, kk_box_t _y_176071, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17194_map_indexed_peek_fun21742(kk_function_t f0, kk_integer_t i0, kk_std_core__list yy0, kk_context_t* _ctx) {
  struct kk_std_core__lift17194_map_indexed_peek_fun21742__t* _self = kk_function_alloc_as(struct kk_std_core__lift17194_map_indexed_peek_fun21742__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17194_map_indexed_peek_fun21742, kk_context());
  _self->f0 = f0;
  _self->i0 = i0;
  _self->yy0 = yy0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17194_map_indexed_peek_fun21742(kk_function_t _fself, kk_box_t _y_176071, kk_context_t* _ctx) {
  struct kk_std_core__lift17194_map_indexed_peek_fun21742__t* _self = kk_function_as(struct kk_std_core__lift17194_map_indexed_peek_fun21742__t*, _fself);
  kk_function_t f0 = _self->f0; /* (idx : int, value : 14194, rest : list<14194>) -> 14196 14195 */
  kk_integer_t i0 = _self->i0; /* int */
  kk_std_core__list yy0 = _self->yy0; /* list<14194> */
  kk_drop_match(_self, {kk_function_dup(f0);kk_integer_dup(i0);kk_std_core__list_dup(yy0);}, {}, _ctx)
  return kk_std_core__mlift17759_op(f0, i0, yy0, _y_176071, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17194_map_indexed_peek_fun21744__t {
  struct kk_function_s _base;
  kk_function_t _b_19507_19503;
};
static kk_box_t kk_std_core__lift17194_map_indexed_peek_fun21744(kk_function_t _fself, kk_box_t _b_19504, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17194_map_indexed_peek_fun21744(kk_function_t _b_19507_19503, kk_context_t* _ctx) {
  struct kk_std_core__lift17194_map_indexed_peek_fun21744__t* _self = kk_function_alloc_as(struct kk_std_core__lift17194_map_indexed_peek_fun21744__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17194_map_indexed_peek_fun21744, kk_context());
  _self->_b_19507_19503 = _b_19507_19503;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17194_map_indexed_peek_fun21744(kk_function_t _fself, kk_box_t _b_19504, kk_context_t* _ctx) {
  struct kk_std_core__lift17194_map_indexed_peek_fun21744__t* _self = kk_function_as(struct kk_std_core__lift17194_map_indexed_peek_fun21744__t*, _fself);
  kk_function_t _b_19507_19503 = _self->_b_19507_19503; /* (14195) -> 14196 list<14195> */
  kk_drop_match(_self, {kk_function_dup(_b_19507_19503);}, {}, _ctx)
  kk_std_core__list _x21745 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19507_19503, (_b_19507_19503, _b_19504, _ctx)); /*list<14195>*/
  return kk_std_core__list_box(_x21745, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17194_map_indexed_peek_fun21746__t {
  struct kk_function_s _base;
  kk_box_t x0_18190;
};
static kk_std_core__list kk_std_core__lift17194_map_indexed_peek_fun21746(kk_function_t _fself, kk_std_core__list _y_176081, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17194_map_indexed_peek_fun21746(kk_box_t x0_18190, kk_context_t* _ctx) {
  struct kk_std_core__lift17194_map_indexed_peek_fun21746__t* _self = kk_function_alloc_as(struct kk_std_core__lift17194_map_indexed_peek_fun21746__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17194_map_indexed_peek_fun21746, kk_context());
  _self->x0_18190 = x0_18190;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17194_map_indexed_peek_fun21746(kk_function_t _fself, kk_std_core__list _y_176081, kk_context_t* _ctx) {
  struct kk_std_core__lift17194_map_indexed_peek_fun21746__t* _self = kk_function_as(struct kk_std_core__lift17194_map_indexed_peek_fun21746__t*, _fself);
  kk_box_t x0_18190 = _self->x0_18190; /* 14195 */
  kk_drop_match(_self, {kk_box_dup(x0_18190);}, {}, _ctx)
  return kk_std_core__mlift17758_op(x0_18190, _y_176081, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17194_map_indexed_peek_fun21748__t {
  struct kk_function_s _base;
  kk_function_t _b_19508_19505;
};
static kk_box_t kk_std_core__lift17194_map_indexed_peek_fun21748(kk_function_t _fself, kk_box_t _b_19506, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17194_map_indexed_peek_fun21748(kk_function_t _b_19508_19505, kk_context_t* _ctx) {
  struct kk_std_core__lift17194_map_indexed_peek_fun21748__t* _self = kk_function_alloc_as(struct kk_std_core__lift17194_map_indexed_peek_fun21748__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17194_map_indexed_peek_fun21748, kk_context());
  _self->_b_19508_19505 = _b_19508_19505;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17194_map_indexed_peek_fun21748(kk_function_t _fself, kk_box_t _b_19506, kk_context_t* _ctx) {
  struct kk_std_core__lift17194_map_indexed_peek_fun21748__t* _self = kk_function_as(struct kk_std_core__lift17194_map_indexed_peek_fun21748__t*, _fself);
  kk_function_t _b_19508_19505 = _self->_b_19508_19505; /* (list<14195>) -> 14196 list<14195> */
  kk_drop_match(_self, {kk_function_dup(_b_19508_19505);}, {}, _ctx)
  kk_std_core__list _x21749;
  kk_std_core__list _x21750 = kk_std_core__list_unbox(_b_19506, _ctx); /*list<14195>*/
  _x21749 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19508_19505, (_b_19508_19505, _x21750, _ctx)); /*list<14195>*/
  return kk_std_core__list_box(_x21749, _ctx);
}

kk_std_core__list kk_std_core__lift17194_map_indexed_peek(kk_function_t f0, kk_std_core__list ys, kk_integer_t i0, kk_context_t* _ctx) { /* forall<a,b,e> (f : (idx : int, value : a, rest : list<a>) -> e b, ys : list<a>, i : int) -> e list<b> */ 
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21738 = kk_std_core__as_Cons(ys);
    kk_box_t y = _con21738->head;
    kk_std_core__list yy0 = _con21738->tail;
    kk_reuse_t _ru_20404 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20404 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(y);
      kk_std_core__list_dup(yy0);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    kk_box_t x0_18190;
    kk_function_t _x21741 = kk_function_dup(f0); /*(idx : int, value : 14194, rest : list<14194>) -> 14196 14195*/
    kk_integer_t _x21739 = kk_integer_dup(i0); /*int*/
    kk_std_core__list _x21740 = kk_std_core__list_dup(yy0); /*list<14194>*/
    x0_18190 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_box_t, kk_std_core__list, kk_context_t*), _x21741, (_x21741, _x21739, y, _x21740, _ctx)); /*14195*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20404);
      kk_box_drop(x0_18190, _ctx);
      kk_function_t _b_19507_19503 = kk_std_core__new_lift17194_map_indexed_peek_fun21742(f0, i0, yy0, _ctx); /*(14195) -> 14196 list<14195>*/;
      kk_box_t _x21743 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17194_map_indexed_peek_fun21744(_b_19507_19503, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21743, _ctx);
    }
    kk_integer_t i0_172720 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
    kk_std_core__list x1_18193 = kk_std_core__lift17194_map_indexed_peek(f0, yy0, i0_172720, _ctx); /*list<14195>*/;
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20404);
      kk_std_core__list_drop(x1_18193, _ctx);
      kk_function_t _b_19508_19505 = kk_std_core__new_lift17194_map_indexed_peek_fun21746(x0_18190, _ctx); /*(list<14195>) -> 14196 list<14195>*/;
      kk_box_t _x21747 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17194_map_indexed_peek_fun21748(_b_19508_19505, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21747, _ctx);
    }
    return kk_std_core__new_Cons(_ru_20404, x0_18190, x1_18193, _ctx);
  }
  kk_function_drop(f0, _ctx);
  kk_integer_drop(i0, _ctx);
  return kk_std_core__new_Nil(_ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17760_op(kk_std_core_types__ctail _acc, kk_function_t action, kk_std_core__list xx, kk_std_core_types__maybe _y_17611, kk_context_t* _ctx) { /* forall<a,b,e> (ctail<list<b>>, action : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Just(_y_17611)) {
    kk_box_t y = _y_17611._cons.Just.value;
    kk_std_core__list _ctail_17353 = kk_std_core__list_hole(); /*list<14237>*/;
    kk_std_core__list _ctail_17354 = kk_std_core__new_Cons(kk_reuse_null, y, _ctail_17353, _ctx); /*list<14237>*/;
    kk_std_core_types__ctail _x21751;
    kk_box_t* field_18198 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17354)->tail)); /*cfield<list<14237>>*/;
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x19515 = _acc._cons.CTail._field1;
      kk_box_t* last0 = _acc._cons.CTail._field2;
      kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19515, NULL);
      kk_std_core__list_dup(head0);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __ = kk_Unit;
      *(last0) = kk_std_core__list_box(_ctail_17354, _ctx);
      kk_std_core__list _b_19529_19520 = head0; /*list<14237>*/;
      kk_box_t* _b_19530_19521 = field_18198; /*cfield<list<14237>>*/;
      _x21751 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19529_19520, _ctx), _b_19530_19521, _ctx); /*ctail<58>*/
      goto _match21752;
    }
    kk_std_core__list _b_19531_19522 = _ctail_17354; /*list<14237>*/;
    kk_box_t* _b_19532_19523 = field_18198; /*cfield<list<14237>>*/;
    _x21751 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19531_19522, _ctx), _b_19532_19523, _ctx); /*ctail<58>*/
    _match21752: ;
    return kk_std_core__ctail_map_while(xx, action, _x21751, _ctx);
  }
  kk_function_drop(action, _ctx);
  kk_std_core__list_drop(xx, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19524 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19524, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17761_op_fun21755__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t y0;
};
static kk_std_core__list kk_std_core__mlift17761_op_fun21755(kk_function_t _fself, kk_std_core__list _ctail_17356, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17761_op_fun21755(kk_function_t _accm, kk_box_t y0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17761_op_fun21755__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17761_op_fun21755__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17761_op_fun21755, kk_context());
  _self->_accm = _accm;
  _self->y0 = y0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17761_op_fun21755(kk_function_t _fself, kk_std_core__list _ctail_17356, kk_context_t* _ctx) {
  struct kk_std_core__mlift17761_op_fun21755__t* _self = kk_function_as(struct kk_std_core__mlift17761_op_fun21755__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<14237>) -> list<14237> */
  kk_box_t y0 = _self->y0; /* 14237 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(y0);}, {}, _ctx)
  kk_std_core__list _x21756 = kk_std_core__new_Cons(kk_reuse_null, y0, _ctail_17356, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x21756, _ctx));
}

kk_std_core__list kk_std_core__mlift17761_op(kk_function_t _accm, kk_function_t action0, kk_std_core__list xx0, kk_std_core_types__maybe _y_17615, kk_context_t* _ctx) { /* forall<a,b,e> ((list<b>) -> list<b>, action : (a) -> e maybe<b>, xx : list<a>, maybe<b>) -> e list<b> */ 
  if (kk_std_core_types__is_Just(_y_17615)) {
    kk_box_t y0 = _y_17615._cons.Just.value;
    return kk_std_core__ctailm_map_while(xx0, action0, kk_std_core__new_mlift17761_op_fun21755(_accm, y0, _ctx), _ctx);
  }
  kk_function_drop(action0, _ctx);
  kk_std_core__list_drop(xx0, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Invoke `action` on each element of a list while `action` returns `Just`


// lift anonymous function
struct kk_std_core__ctail_map_while_fun21760__t {
  struct kk_function_s _base;
  kk_function_t action1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_map_while_fun21760(kk_function_t _fself, kk_std_core_types__maybe _y_176110, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_map_while_fun21760(kk_function_t action1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_while_fun21760__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_map_while_fun21760__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_map_while_fun21760, kk_context());
  _self->action1 = action1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_map_while_fun21760(kk_function_t _fself, kk_std_core_types__maybe _y_176110, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_while_fun21760__t* _self = kk_function_as(struct kk_std_core__ctail_map_while_fun21760__t*, _fself);
  kk_function_t action1 = _self->action1; /* (14236) -> 14238 maybe<14237> */
  kk_std_core__list xx1 = _self->xx1; /* list<14236> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<14237>> */
  kk_drop_match(_self, {kk_function_dup(action1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17760_op(_acc0, action1, xx1, _y_176110, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_map_while_fun21762__t {
  struct kk_function_s _base;
  kk_function_t _b_19560_19538;
};
static kk_box_t kk_std_core__ctail_map_while_fun21762(kk_function_t _fself, kk_box_t _b_19539, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_map_while_fun21762(kk_function_t _b_19560_19538, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_while_fun21762__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_map_while_fun21762__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_map_while_fun21762, kk_context());
  _self->_b_19560_19538 = _b_19560_19538;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_map_while_fun21762(kk_function_t _fself, kk_box_t _b_19539, kk_context_t* _ctx) {
  struct kk_std_core__ctail_map_while_fun21762__t* _self = kk_function_as(struct kk_std_core__ctail_map_while_fun21762__t*, _fself);
  kk_function_t _b_19560_19538 = _self->_b_19560_19538; /* (maybe<14237>) -> 14238 list<14237> */
  kk_drop_match(_self, {kk_function_dup(_b_19560_19538);}, {}, _ctx)
  kk_std_core__list _x21763;
  kk_std_core_types__maybe _x21764 = kk_std_core_types__maybe_unbox(_b_19539, _ctx); /*maybe<14237>*/
  _x21763 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19560_19538, (_b_19560_19538, _x21764, _ctx)); /*list<14237>*/
  return kk_std_core__list_box(_x21763, _ctx);
}

kk_std_core__list kk_std_core__ctail_map_while(kk_std_core__list xs, kk_function_t action1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>, ctail<list<b>>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(action1, _ctx);
    if (kk_std_core_types__is_CTail(_acc0)) {
      kk_box_t _box_x19533 = _acc0._cons.CTail._field1;
      kk_box_t* last3 = _acc0._cons.CTail._field2;
      kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19533, NULL);
      kk_std_core__list_dup(head2);
      kk_std_core_types__ctail_drop(_acc0, _ctx);
      kk_Unit;
      kk_unit_t __1 = kk_Unit;
      *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
      return head2;
    }
    return kk_std_core__new_Nil(_ctx);
  }
  struct kk_std_core_Cons* _con21758 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21758->head;
  kk_std_core__list xx1 = _con21758->tail;
  kk_reuse_t _ru_20405 = kk_reuse_null; /*reuse*/;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    _ru_20405 = (kk_std_core__list_reuse(xs));
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx1);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core_types__maybe x0_18203;
  kk_function_t _x21759 = kk_function_dup(action1); /*(14236) -> 14238 maybe<14237>*/
  x0_18203 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21759, (_x21759, x, _ctx)); /*maybe<14237>*/
  if (kk_yielding(kk_context())) {
    kk_reuse_drop(_ru_20405);
    kk_std_core_types__maybe_drop(x0_18203, _ctx);
    kk_function_t _b_19560_19538 = kk_std_core__new_ctail_map_while_fun21760(action1, xx1, _acc0, _ctx); /*(maybe<14237>) -> 14238 list<14237>*/;
    kk_box_t _x21761 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_map_while_fun21762(_b_19560_19538, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21761, _ctx);
  }
  if (kk_std_core_types__is_Just(x0_18203)) {
    kk_box_t y1 = x0_18203._cons.Just.value;
    kk_std_core__list _ctail_173530 = kk_std_core__list_hole(); /*list<14237>*/;
    kk_std_core__list _ctail_173540 = kk_std_core__new_Cons(_ru_20405, y1, _ctail_173530, _ctx); /*list<14237>*/;
    { // tailcall
      kk_std_core_types__ctail _x21765;
      kk_box_t* field0_18208 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_173540)->tail)); /*cfield<list<14237>>*/;
      if (kk_std_core_types__is_CTail(_acc0)) {
        kk_box_t _box_x19546 = _acc0._cons.CTail._field1;
        kk_box_t* last4 = _acc0._cons.CTail._field2;
        kk_std_core__list head3 = kk_std_core__list_unbox(_box_x19546, NULL);
        kk_std_core__list_dup(head3);
        kk_std_core_types__ctail_drop(_acc0, _ctx);
        kk_Unit;
        kk_unit_t __2 = kk_Unit;
        *(last4) = kk_std_core__list_box(_ctail_173540, _ctx);
        kk_std_core__list _b_19561_19551 = head3; /*list<14237>*/;
        kk_box_t* _b_19562_19552 = field0_18208; /*cfield<list<14237>>*/;
        _x21765 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19561_19551, _ctx), _b_19562_19552, _ctx); /*ctail<58>*/
        goto _match21766;
      }
      kk_std_core__list _b_19563_19553 = _ctail_173540; /*list<14237>*/;
      kk_box_t* _b_19564_19554 = field0_18208; /*cfield<list<14237>>*/;
      _x21765 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19563_19553, _ctx), _b_19564_19554, _ctx); /*ctail<58>*/
      _match21766: ;
      xs = xx1;
      _acc0 = _x21765;
      goto kk__tailcall;
    }
  }
  kk_reuse_drop(_ru_20405);
  kk_function_drop(action1, _ctx);
  kk_std_core__list_drop(xx1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x19555 = _acc0._cons.CTail._field1;
    kk_box_t* last5 = _acc0._cons.CTail._field2;
    kk_std_core__list head4 = kk_std_core__list_unbox(_box_x19555, NULL);
    kk_std_core__list_dup(head4);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __3 = kk_Unit;
    *(last5) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head4;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Invoke `action` on each element of a list while `action` returns `Just`


// lift anonymous function
struct kk_std_core__ctailm_map_while_fun21771__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t action2;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_map_while_fun21771(kk_function_t _fself, kk_std_core_types__maybe _y_176150, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_while_fun21771(kk_function_t _accm0, kk_function_t action2, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21771__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_while_fun21771__t, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_while_fun21771, kk_context());
  _self->_accm0 = _accm0;
  _self->action2 = action2;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_map_while_fun21771(kk_function_t _fself, kk_std_core_types__maybe _y_176150, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21771__t* _self = kk_function_as(struct kk_std_core__ctailm_map_while_fun21771__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<14237>) -> list<14237> */
  kk_function_t action2 = _self->action2; /* (14236) -> 14238 maybe<14237> */
  kk_std_core__list xx2 = _self->xx2; /* list<14236> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(action2);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17761_op(_accm0, action2, xx2, _y_176150, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_map_while_fun21773__t {
  struct kk_function_s _base;
  kk_function_t _b_19567_19565;
};
static kk_box_t kk_std_core__ctailm_map_while_fun21773(kk_function_t _fself, kk_box_t _b_19566, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_while_fun21773(kk_function_t _b_19567_19565, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21773__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_while_fun21773__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_while_fun21773, kk_context());
  _self->_b_19567_19565 = _b_19567_19565;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_map_while_fun21773(kk_function_t _fself, kk_box_t _b_19566, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21773__t* _self = kk_function_as(struct kk_std_core__ctailm_map_while_fun21773__t*, _fself);
  kk_function_t _b_19567_19565 = _self->_b_19567_19565; /* (maybe<14237>) -> 14238 list<14237> */
  kk_drop_match(_self, {kk_function_dup(_b_19567_19565);}, {}, _ctx)
  kk_std_core__list _x21774;
  kk_std_core_types__maybe _x21775 = kk_std_core_types__maybe_unbox(_b_19566, _ctx); /*maybe<14237>*/
  _x21774 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core_types__maybe, kk_context_t*), _b_19567_19565, (_b_19567_19565, _x21775, _ctx)); /*list<14237>*/
  return kk_std_core__list_box(_x21774, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_map_while_fun21777__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t y2;
};
static kk_std_core__list kk_std_core__ctailm_map_while_fun21777(kk_function_t _fself, kk_std_core__list _ctail_173560, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_map_while_fun21777(kk_function_t _accm0, kk_box_t y2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21777__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_map_while_fun21777__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_map_while_fun21777, kk_context());
  _self->_accm0 = _accm0;
  _self->y2 = y2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_map_while_fun21777(kk_function_t _fself, kk_std_core__list _ctail_173560, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_map_while_fun21777__t* _self = kk_function_as(struct kk_std_core__ctailm_map_while_fun21777__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<14237>) -> list<14237> */
  kk_box_t y2 = _self->y2; /* 14237 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(y2);}, {}, _ctx)
  kk_std_core__list _x21778 = kk_std_core__new_Cons(kk_reuse_null, y2, _ctail_173560, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21778, _ctx));
}

kk_std_core__list kk_std_core__ctailm_map_while(kk_std_core__list xs0, kk_function_t action2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>, (list<b>) -> list<b>) -> e list<b> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs0)) {
    kk_function_drop(action2, _ctx);
    return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
  }
  struct kk_std_core_Cons* _con21769 = kk_std_core__as_Cons(xs0);
  kk_box_t x1 = _con21769->head;
  kk_std_core__list xx2 = _con21769->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs0))) {
    kk_std_core__list_free(xs0);
  }
  else {
    kk_box_dup(x1);
    kk_std_core__list_dup(xx2);
    kk_Unit;
    kk_std_core__list_decref(xs0, _ctx);
    kk_Unit;
  }
  kk_std_core_types__maybe x2_18211;
  kk_function_t _x21770 = kk_function_dup(action2); /*(14236) -> 14238 maybe<14237>*/
  x2_18211 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_box_t, kk_context_t*), _x21770, (_x21770, x1, _ctx)); /*maybe<14237>*/
  if (kk_yielding(kk_context())) {
    kk_std_core_types__maybe_drop(x2_18211, _ctx);
    kk_function_t _b_19567_19565 = kk_std_core__new_ctailm_map_while_fun21771(_accm0, action2, xx2, _ctx); /*(maybe<14237>) -> 14238 list<14237>*/;
    kk_box_t _x21772 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_map_while_fun21773(_b_19567_19565, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x21772, _ctx);
  }
  if (kk_std_core_types__is_Just(x2_18211)) {
    kk_box_t y2 = x2_18211._cons.Just.value;
    { // tailcall
      kk_function_t _x21776 = kk_std_core__new_ctailm_map_while_fun21777(_accm0, y2, _ctx); /*(list<14237>) -> list<14237>*/
      xs0 = xx2;
      _accm0 = _x21776;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action2, _ctx);
  kk_std_core__list_drop(xx2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Invoke `action` on each element of a list while `action` returns `Just`


// lift anonymous function
struct kk_std_core_map_while_fun21779__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_map_while_fun21779(kk_function_t _fself, kk_std_core__list _ctail_17355, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_map_while_fun21779(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_map_while_fun21779, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_map_while_fun21779(kk_function_t _fself, kk_std_core__list _ctail_17355, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17355;
}

kk_std_core__list kk_std_core_map_while(kk_std_core__list xs1, kk_function_t action3, kk_context_t* _ctx) { /* forall<a,b,e> (xs : list<a>, action : (a) -> e maybe<b>) -> e list<b> */ 
  bool _match_19971 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19971) {
    return kk_std_core__ctail_map_while(xs1, action3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_map_while(xs1, action3, kk_std_core_new_map_while_fun21779(_ctx), _ctx);
}
 
// lifted
// specialized std/core/foldl to parameters [f]

kk_integer_t kk_std_core__lift17195_maximum(kk_std_core__list xs0, kk_integer_t z, kk_context_t* _ctx) { /* (xs0 : list<int>, z : int) -> int */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21780 = kk_std_core__as_Cons(xs0);
    kk_box_t _box_x19568 = _con21780->head;
    kk_std_core__list xx0 = _con21780->tail;
    kk_integer_t x0 = kk_integer_unbox(_box_x19568);
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_integer_dup(x0);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_integer_t _x21782;
      bool _match_19968 = kk_integer_gte_borrow(z,x0,kk_context()); /*bool*/;
      if (_match_19968) {
        kk_integer_drop(x0, _ctx);
        _x21782 = z; /*int*/
      }
      else {
        kk_integer_drop(z, _ctx);
        _x21782 = x0; /*int*/
      }
      xs0 = xx0;
      z = _x21782;
      goto kk__tailcall;
    }
  }
  return z;
}
 
// Returns the largest element of a list of integers (or `default` (=`0`) for the empty list)

kk_integer_t kk_std_core_maximum(kk_std_core__list xs, kk_std_core_types__optional default0, kk_context_t* _ctx) { /* (xs : list<int>, default : optional<int>) -> int */ 
  if (kk_std_core__is_Nil(xs)) {
    if (kk_std_core_types__is_Optional(default0)) {
      kk_box_t _box_x19569 = default0._cons.Optional.value;
      kk_integer_t _default_14286 = kk_integer_unbox(_box_x19569);
      kk_integer_dup(_default_14286);
      kk_std_core_types__optional_drop(default0, _ctx);
      kk_Unit;
      return _default_14286;
    }
    return kk_integer_from_small(0);
  }
  struct kk_std_core_Cons* _con21784 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x19570 = _con21784->head;
  kk_std_core__list xx = _con21784->tail;
  kk_integer_t x = kk_integer_unbox(_box_x19570);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_integer_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core_types__optional_drop(default0, _ctx);
  kk_Unit;
  return kk_std_core__lift17195_maximum(xx, x, _ctx);
}
 
// lifted
// specialized std/core/foldl to parameters [f]

double kk_std_core__lift17196_maximum_1(kk_std_core__list xs0, double z, kk_context_t* _ctx) { /* (xs0 : list<double>, z : double) -> double */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21786 = kk_std_core__as_Cons(xs0);
    kk_box_t _box_x19571 = _con21786->head;
    kk_std_core__list xx0 = _con21786->tail;
    double x0 = kk_double_unbox(_box_x19571, NULL);
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_box_drop(_box_x19571, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs0);
    }
    else {
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    { // tailcall
      double _x21788;
      bool _match_19967 = (z >= x0); /*bool*/;
      if (_match_19967) {
        _x21788 = z; /*double*/
      }
      else {
        _x21788 = x0; /*double*/
      }
      xs0 = xx0;
      z = _x21788;
      goto kk__tailcall;
    }
  }
  return z;
}
 
// Returns the largest element of a list of doubles (or `0` for the empty list)

double kk_std_core_maximum_1(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<double>) -> double */ 
  if (kk_std_core__is_Nil(xs)) {
    return 0.0;
  }
  struct kk_std_core_Cons* _con21789 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x19572 = _con21789->head;
  kk_std_core__list xx = _con21789->tail;
  double x = kk_double_unbox(_box_x19572, NULL);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_box_drop(_box_x19572, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core__list_free(xs);
  }
  else {
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  return kk_std_core__lift17196_maximum_1(xx, x, _ctx);
}
 
// Convert a list to a `:maybe` type, using `Nothing` for an empty list, and otherwise `Just` on the head element.
// Note: this is just `head`.

kk_std_core_types__maybe kk_std_core_maybe_3(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> maybe<a> */ 
  if (kk_std_core__is_Nil(xs)) {
    return kk_std_core_types__new_Nothing(_ctx);
  }
  struct kk_std_core_Cons* _con21791 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con21791->head;
  kk_std_core__list _pat1 = _con21791->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_drop(_pat1, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core__list_free(xs);
  }
  else {
    kk_box_dup(x);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  return kk_std_core_types__new_Just(x, _ctx);
}
 
// Transform an integer to a maybe type, using `Nothing` for `0`

kk_std_core_types__maybe kk_std_core_maybe_5(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> maybe<int> */ 
  bool _match_19964;
  kk_integer_t _borrowedArg_19965 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19966 = kk_integer_eq_borrow(i,_borrowedArg_19965,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19965, _ctx);
  _match_19964 = _borrowApp_19966; /*bool*/
  if (_match_19964) {
    kk_integer_drop(i, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_integer_t _b_19576_19575 = i; /*int*/;
  return kk_std_core_types__new_Just(kk_integer_box(_b_19576_19575), _ctx);
}
 
// Transform a string to a maybe type, using `Nothing` for an empty string

kk_std_core_types__maybe kk_std_core_maybe_6(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> maybe<string> */ 
  bool _match_19963;
  kk_string_t _x21792 = kk_string_dup(s); /*string*/
  kk_string_t _x21793 = kk_string_empty(); /*string*/
  _match_19963 = kk_string_is_eq(_x21792,_x21793,kk_context()); /*bool*/
  if (_match_19963) {
    kk_string_drop(s, _ctx);
    return kk_std_core_types__new_Nothing(_ctx);
  }
  kk_string_t _b_19578_19577 = s; /*string*/;
  return kk_std_core_types__new_Just(kk_string_box(_b_19578_19577), _ctx);
}
 
// Transform a `:null` type to a `:maybe` type. Note that it is not
// always the case that `id(x) == maybe(null(x))` (e.g. when `x = Just(Nothing)`).

kk_std_core_types__maybe kk_std_core_maybe_8(kk_std_core__null n, kk_context_t* _ctx) { /* forall<a> (n : null<a>) -> maybe<a> */ 
  return ((n).ptr == NULL ? kk_std_core_types__new_Nothing(kk_context()) : kk_std_core_types__new_Just(kk_datatype_box(n),kk_context()));
}
 
// lifted
// specialized std/core/foldl to parameters [f]

kk_integer_t kk_std_core__lift17197_minimum(kk_std_core__list xs0, kk_integer_t z, kk_context_t* _ctx) { /* (xs0 : list<int>, z : int) -> int */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21795 = kk_std_core__as_Cons(xs0);
    kk_box_t _box_x19579 = _con21795->head;
    kk_std_core__list xx0 = _con21795->tail;
    kk_integer_t x0 = kk_integer_unbox(_box_x19579);
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_integer_dup(x0);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_integer_t _x21797;
      bool _match_19960 = kk_integer_lte_borrow(z,x0,kk_context()); /*bool*/;
      if (_match_19960) {
        kk_integer_drop(x0, _ctx);
        _x21797 = z; /*int*/
      }
      else {
        kk_integer_drop(z, _ctx);
        _x21797 = x0; /*int*/
      }
      xs0 = xx0;
      z = _x21797;
      goto kk__tailcall;
    }
  }
  return z;
}
 
// Returns the smallest element of a list of integers (or `default` (=`0`) for the empty list)

kk_integer_t kk_std_core_minimum(kk_std_core__list xs, kk_std_core_types__optional default0, kk_context_t* _ctx) { /* (xs : list<int>, default : optional<int>) -> int */ 
  if (kk_std_core__is_Nil(xs)) {
    if (kk_std_core_types__is_Optional(default0)) {
      kk_box_t _box_x19580 = default0._cons.Optional.value;
      kk_integer_t _default_14765 = kk_integer_unbox(_box_x19580);
      kk_integer_dup(_default_14765);
      kk_std_core_types__optional_drop(default0, _ctx);
      kk_Unit;
      return _default_14765;
    }
    return kk_integer_from_small(0);
  }
  struct kk_std_core_Cons* _con21799 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x19581 = _con21799->head;
  kk_std_core__list xx = _con21799->tail;
  kk_integer_t x = kk_integer_unbox(_box_x19581);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_integer_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_std_core_types__optional_drop(default0, _ctx);
  kk_Unit;
  return kk_std_core__lift17197_minimum(xx, x, _ctx);
}
 
// lifted
// specialized std/core/foldl to parameters [f]

double kk_std_core__lift17198_minimum_1(kk_std_core__list xs0, double z, kk_context_t* _ctx) { /* (xs0 : list<double>, z : double) -> double */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21801 = kk_std_core__as_Cons(xs0);
    kk_box_t _box_x19582 = _con21801->head;
    kk_std_core__list xx0 = _con21801->tail;
    double x0 = kk_double_unbox(_box_x19582, NULL);
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_box_drop(_box_x19582, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs0);
    }
    else {
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    { // tailcall
      double _x21803;
      bool _match_19959 = (z <= x0); /*bool*/;
      if (_match_19959) {
        _x21803 = z; /*double*/
      }
      else {
        _x21803 = x0; /*double*/
      }
      xs0 = xx0;
      z = _x21803;
      goto kk__tailcall;
    }
  }
  return z;
}
 
// Returns the smallest element of a list of doubles (or `0` for the empty list)

double kk_std_core_minimum_1(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<double>) -> double */ 
  if (kk_std_core__is_Nil(xs)) {
    return 0.0;
  }
  struct kk_std_core_Cons* _con21804 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x19583 = _con21804->head;
  kk_std_core__list xx = _con21804->tail;
  double x = kk_double_unbox(_box_x19583, NULL);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_box_drop(_box_x19583, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_core__list_free(xs);
  }
  else {
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  return kk_std_core__lift17198_minimum_1(xx, x, _ctx);
}
 
// Disable tracing completely.

kk_unit_t kk_std_core_notrace(kk_context_t* _ctx) { /* () -> (st<global>) () */ 
  kk_ref_t _b_19586_19584 = kk_ref_dup(kk_std_core_trace_enabled); /*ref<global,bool>*/;
  bool _b_19587_19585 = false; /*bool*/;
  kk_unit_t _borrowApp_19958 = kk_Unit;
  kk_ref_set_borrow(_b_19586_19584,(kk_bool_box(_b_19587_19585)),kk_context());
  kk_ref_drop(_b_19586_19584, _ctx);
  _borrowApp_19958; return kk_Unit;
}
 
// Transform a `:maybe` type to a `:null` type (using `null` for `Nothing`).

kk_std_core__null kk_std_core_null(kk_std_core_types__maybe x, kk_context_t* _ctx) { /* forall<a> (x : maybe<a>) -> null<a> */ 
  return (kk_std_core_types__is_Nothing(x) ? kk_datatype_from_ptr(NULL) : kk_datatype_unbox((x)._cons.Just.value));
}
 
// Cast a integer that is zero to a null

kk_std_core__null kk_std_core_null_1(kk_integer_t i, kk_context_t* _ctx) { /* (i : int) -> null<int> */ 
  kk_std_core_types__maybe _x21806;
  bool _match_19955;
  kk_integer_t _borrowedArg_19956 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19957 = kk_integer_eq_borrow(i,_borrowedArg_19956,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19956, _ctx);
  _match_19955 = _borrowApp_19957; /*bool*/
  if (_match_19955) {
    kk_integer_drop(i, _ctx);
    _x21806 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/
  }
  else {
    kk_integer_t _b_19589_19588 = i; /*int*/;
    _x21806 = kk_std_core_types__new_Just(kk_integer_box(_b_19589_19588), _ctx); /*forall<a> maybe<a>*/
  }
  return kk_std_core_null(_x21806, _ctx);
}
 
// Cast an empty string a null

kk_std_core__null kk_std_core_null_2(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> null<string> */ 
  kk_std_core_types__maybe _x21807;
  bool _match_19954;
  kk_string_t _x21808 = kk_string_dup(s); /*string*/
  kk_string_t _x21809 = kk_string_empty(); /*string*/
  _match_19954 = kk_string_is_eq(_x21808,_x21809,kk_context()); /*bool*/
  if (_match_19954) {
    kk_string_drop(s, _ctx);
    _x21807 = kk_std_core_types__new_Nothing(_ctx); /*forall<a> maybe<a>*/
  }
  else {
    kk_string_t _b_19591_19590 = s; /*string*/;
    _x21807 = kk_std_core_types__new_Just(kk_string_box(_b_19591_19590), _ctx); /*forall<a> maybe<a>*/
  }
  return kk_std_core_null(_x21807, _ctx);
}
 
// Left-align a string to width `width`  using `fill`  (default is a space) to fill on the right.

kk_string_t kk_std_core_pad_right(kk_string_t s, kk_integer_t width, kk_std_core_types__optional fill, kk_context_t* _ctx) { /* (s : string, width : int, fill : optional<char>) -> string */ 
  kk_ssize_t w;
  kk_integer_t _x21812 = kk_integer_dup(width); /*int*/
  w = kk_std_core_ssize__t(_x21812, _ctx); /*ssize_t*/
  kk_ssize_t n;
  kk_string_t _x21813 = kk_string_dup(s); /*string*/
  n = kk_string_len(_x21813,kk_context()); /*ssize_t*/
  bool _match_19953 = (w <= n); /*bool*/;
  if (_match_19953) {
    kk_std_core_types__optional_drop(fill, _ctx);
    return s;
  }
  kk_string_t _x21814;
  kk_string_t _x21815;
  kk_char_t _x21816;
  if (kk_std_core_types__is_Optional(fill)) {
    kk_box_t _box_x19594 = fill._cons.Optional.value;
    kk_char_t _fill_15167 = kk_char_unbox(_box_x19594, NULL);
    kk_std_core_types__optional_drop(fill, _ctx);
    kk_Unit;
    _x21816 = _fill_15167; /*char*/
    goto _match21817;
  }
  _x21816 = ' '; /*char*/
  _match21817: ;
  _x21815 = kk_std_core_string(_x21816, _ctx); /*string*/
  kk_ssize_t _x21819 = (w - n); /*ssize_t*/
  _x21814 = kk_std_core_repeatz(_x21815, _x21819, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(s, _x21814, _ctx);
}
 
// Is `pre`  a prefix of `s`? If so, returns a slice
// of `s` following `pre` up to the end of `s`.

kk_std_core_types__maybe kk_std_core_starts_with(kk_string_t s, kk_string_t pre, kk_context_t* _ctx) { /* (s : string, pre : string) -> maybe<sslice> */ 
  bool _match_19952;
  kk_string_t _x21820 = kk_string_dup(s); /*string*/
  kk_string_t _x21821 = kk_string_dup(pre); /*string*/
  _match_19952 = kk_string_starts_with(_x21820,_x21821,kk_context()); /*bool*/
  if (_match_19952) {
    kk_std_core__sslice _b_19596_19595;
    kk_string_t _x21822 = kk_string_dup(s); /*string*/
    kk_ssize_t _x21823;
    kk_string_t _x21824 = kk_string_dup(pre); /*string*/
    _x21823 = kk_string_len(_x21824,kk_context()); /*ssize_t*/
    kk_ssize_t _x21825;
    kk_ssize_t _x21826 = kk_string_len(s,kk_context()); /*ssize_t*/
    kk_ssize_t _x21827 = kk_string_len(pre,kk_context()); /*ssize_t*/
    _x21825 = (_x21826 - _x21827); /*ssize_t*/
    _b_19596_19595 = kk_std_core__new_Sslice(_x21822, _x21823, _x21825, _ctx); /*sslice*/
    return kk_std_core_types__new_Just(kk_std_core__sslice_box(_b_19596_19595, _ctx), _ctx);
  }
  kk_string_drop(pre, _ctx);
  kk_string_drop(s, _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Trim off a substring `sub` while `s` starts with that string.

kk_string_t kk_std_core_trim_left_1(kk_string_t s, kk_string_t sub, kk_context_t* _ctx) { /* (s : string, sub : string) -> string */ 
  kk__tailcall: ;
  bool _match_19950;
  kk_string_t _x21828 = kk_string_dup(sub); /*string*/
  kk_string_t _x21829 = kk_string_empty(); /*string*/
  _match_19950 = kk_string_is_eq(_x21828,_x21829,kk_context()); /*bool*/
  if (_match_19950) {
    kk_string_drop(sub, _ctx);
    return s;
  }
  kk_std_core_types__maybe _match_19951;
  kk_string_t _x21831 = kk_string_dup(s); /*string*/
  kk_string_t _x21832 = kk_string_dup(sub); /*string*/
  _match_19951 = kk_std_core_starts_with(_x21831, _x21832, _ctx); /*maybe<sslice>*/
  if (kk_std_core_types__is_Just(_match_19951)) {
    kk_box_t _box_x19597 = _match_19951._cons.Just.value;
    kk_std_core__sslice slice0 = kk_std_core__sslice_unbox(_box_x19597, NULL);
    kk_string_drop(s, _ctx);
    kk_std_core__sslice_dup(slice0);
    kk_std_core_types__maybe_drop(_match_19951, _ctx);
    kk_Unit;
    { // tailcall
      kk_string_t _x21834 = kk_std_core_string_3(slice0, _ctx); /*string*/
      s = _x21834;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sub, _ctx);
  return s;
}
 
// Trim off a substring `sub` while `s` ends with that string.

kk_string_t kk_std_core_trim_right_1(kk_string_t s, kk_string_t sub, kk_context_t* _ctx) { /* (s : string, sub : string) -> string */ 
  kk__tailcall: ;
  bool _match_19948;
  kk_string_t _x21835 = kk_string_dup(sub); /*string*/
  kk_string_t _x21836 = kk_string_empty(); /*string*/
  _match_19948 = kk_string_is_eq(_x21835,_x21836,kk_context()); /*bool*/
  if (_match_19948) {
    kk_string_drop(sub, _ctx);
    return s;
  }
  kk_std_core_types__maybe _match_19949;
  kk_string_t _x21838 = kk_string_dup(s); /*string*/
  kk_string_t _x21839 = kk_string_dup(sub); /*string*/
  _match_19949 = kk_std_core_ends_with(_x21838, _x21839, _ctx); /*maybe<sslice>*/
  if (kk_std_core_types__is_Just(_match_19949)) {
    kk_box_t _box_x19598 = _match_19949._cons.Just.value;
    kk_std_core__sslice slice0 = kk_std_core__sslice_unbox(_box_x19598, NULL);
    kk_string_drop(s, _ctx);
    kk_std_core__sslice_dup(slice0);
    kk_std_core_types__maybe_drop(_match_19949, _ctx);
    kk_Unit;
    { // tailcall
      kk_string_t _x21841 = kk_std_core_string_3(slice0, _ctx); /*string*/
      s = _x21841;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sub, _ctx);
  return s;
}
 
// Parse an integer using `parseInt`. If an illegal digit character is encountered the
// `default` value is returned. An empty string will also result in `default`.

kk_integer_t kk_std_core_parse_int_default(kk_string_t s, kk_std_core_types__optional default0, kk_std_core_types__optional hex, kk_context_t* _ctx) { /* (s : string, default : optional<int>, hex : optional<bool>) -> int */ 
  bool _match_19947;
  kk_string_t _x21848 = kk_string_dup(s); /*string*/
  kk_string_t _x21849 = kk_string_empty(); /*string*/
  _match_19947 = kk_string_is_eq(_x21848,_x21849,kk_context()); /*bool*/
  if (_match_19947) {
    kk_std_core_types__optional_drop(hex, _ctx);
    kk_string_drop(s, _ctx);
    if (kk_std_core_types__is_Optional(default0)) {
      kk_box_t _box_x19600 = default0._cons.Optional.value;
      kk_integer_t _default_15449 = kk_integer_unbox(_box_x19600);
      kk_integer_dup(_default_15449);
      kk_std_core_types__optional_drop(default0, _ctx);
      kk_Unit;
      return _default_15449;
    }
    return kk_integer_from_small(0);
  }
  kk_std_core_types__maybe m_17133;
  kk_string_t _x21852;
  kk_string_t _x21853 = kk_string_trim_left(s,kk_context()); /*string*/
  _x21852 = kk_string_trim_right(_x21853,kk_context()); /*string*/
  bool _x21854;
  if (kk_std_core_types__is_Optional(hex)) {
    kk_box_t _box_x19601 = hex._cons.Optional.value;
    bool _hex_15453 = kk_bool_unbox(_box_x19601);
    kk_std_core_types__optional_drop(hex, _ctx);
    kk_Unit;
    _x21854 = _hex_15453; /*bool*/
    goto _match21855;
  }
  _x21854 = false; /*bool*/
  _match21855: ;
  m_17133 = kk_std_core_xparse_int(_x21852, _x21854, _ctx); /*maybe<int>*/
  if (kk_std_core_types__is_Nothing(m_17133)) {
    if (kk_std_core_types__is_Optional(default0)) {
      kk_box_t _box_x19602 = default0._cons.Optional.value;
      kk_integer_t _default_154490 = kk_integer_unbox(_box_x19602);
      kk_integer_dup(_default_154490);
      kk_std_core_types__optional_drop(default0, _ctx);
      kk_Unit;
      return _default_154490;
    }
    return kk_integer_from_small(0);
  }
  kk_box_t _box_x19603 = m_17133._cons.Just.value;
  kk_integer_t x = kk_integer_unbox(_box_x19603);
  kk_integer_dup(x);
  kk_std_core_types__maybe_drop(m_17133, _ctx);
  kk_Unit;
  kk_std_core_types__optional_drop(default0, _ctx);
  kk_Unit;
  return x;
}
 
// monadic lift

kk_std_core_types__tuple2_ kk_std_core__mlift17762_partition_acc(kk_std_core__list acc1, kk_std_core__list acc2, kk_function_t pred, kk_box_t x, kk_std_core__list xx, bool _y_17624, kk_context_t* _ctx) { /* forall<a,e> (acc1 : list<a>, acc2 : list<a>, pred : (a) -> e bool, x : a, xx : list<a>, bool) -> e (list<a>, list<a>) */ 
  if (_y_17624) {
    kk_std_core__list _x21859 = kk_std_core__new_Cons(kk_reuse_null, x, acc1, _ctx); /*list<61>*/
    return kk_std_core_partition_acc(xx, pred, _x21859, acc2, _ctx);
  }
  kk_std_core__list _x21860 = kk_std_core__new_Cons(kk_reuse_null, x, acc2, _ctx); /*list<61>*/
  return kk_std_core_partition_acc(xx, pred, acc1, _x21860, _ctx);
}


// lift anonymous function
struct kk_std_core_partition_acc_fun21864__t {
  struct kk_function_s _base;
  kk_std_core__list acc10;
  kk_std_core__list acc20;
  kk_function_t pred0;
  kk_box_t x0;
  kk_std_core__list xx0;
};
static kk_std_core_types__tuple2_ kk_std_core_partition_acc_fun21864(kk_function_t _fself, bool _y_176240, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_partition_acc_fun21864(kk_std_core__list acc10, kk_std_core__list acc20, kk_function_t pred0, kk_box_t x0, kk_std_core__list xx0, kk_context_t* _ctx) {
  struct kk_std_core_partition_acc_fun21864__t* _self = kk_function_alloc_as(struct kk_std_core_partition_acc_fun21864__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_partition_acc_fun21864, kk_context());
  _self->acc10 = acc10;
  _self->acc20 = acc20;
  _self->pred0 = pred0;
  _self->x0 = x0;
  _self->xx0 = xx0;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_core_partition_acc_fun21864(kk_function_t _fself, bool _y_176240, kk_context_t* _ctx) {
  struct kk_std_core_partition_acc_fun21864__t* _self = kk_function_as(struct kk_std_core_partition_acc_fun21864__t*, _fself);
  kk_std_core__list acc10 = _self->acc10; /* list<15571> */
  kk_std_core__list acc20 = _self->acc20; /* list<15571> */
  kk_function_t pred0 = _self->pred0; /* (15571) -> 15572 bool */
  kk_box_t x0 = _self->x0; /* 15571 */
  kk_std_core__list xx0 = _self->xx0; /* list<15571> */
  kk_drop_match(_self, {kk_std_core__list_dup(acc10);kk_std_core__list_dup(acc20);kk_function_dup(pred0);kk_box_dup(x0);kk_std_core__list_dup(xx0);}, {}, _ctx)
  return kk_std_core__mlift17762_partition_acc(acc10, acc20, pred0, x0, xx0, _y_176240, _ctx);
}


// lift anonymous function
struct kk_std_core_partition_acc_fun21866__t {
  struct kk_function_s _base;
  kk_function_t _b_19610_19606;
};
static kk_box_t kk_std_core_partition_acc_fun21866(kk_function_t _fself, kk_box_t _b_19607, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_partition_acc_fun21866(kk_function_t _b_19610_19606, kk_context_t* _ctx) {
  struct kk_std_core_partition_acc_fun21866__t* _self = kk_function_alloc_as(struct kk_std_core_partition_acc_fun21866__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_partition_acc_fun21866, kk_context());
  _self->_b_19610_19606 = _b_19610_19606;
  return &_self->_base;
}

static kk_box_t kk_std_core_partition_acc_fun21866(kk_function_t _fself, kk_box_t _b_19607, kk_context_t* _ctx) {
  struct kk_std_core_partition_acc_fun21866__t* _self = kk_function_as(struct kk_std_core_partition_acc_fun21866__t*, _fself);
  kk_function_t _b_19610_19606 = _self->_b_19610_19606; /* (bool) -> 15572 (list<15571>, list<15571>) */
  kk_drop_match(_self, {kk_function_dup(_b_19610_19606);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x21867;
  bool _x21868 = kk_bool_unbox(_b_19607); /*bool*/
  _x21867 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, bool, kk_context_t*), _b_19610_19606, (_b_19610_19606, _x21868, _ctx)); /*(list<15571>, list<15571>)*/
  return kk_std_core_types__tuple2__box(_x21867, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core_partition_acc(kk_std_core__list xs, kk_function_t pred0, kk_std_core__list acc10, kk_std_core__list acc20, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, pred : (a) -> e bool, acc1 : list<a>, acc2 : list<a>) -> e (list<a>, list<a>) */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(pred0, _ctx);
    kk_std_core__list _b_19608_19604 = kk_std_core__lift17186_reverse(kk_std_core__new_Nil(_ctx), acc10, _ctx); /*list<15571>*/;
    kk_std_core__list _b_19609_19605 = kk_std_core__lift17186_reverse(kk_std_core__new_Nil(_ctx), acc20, _ctx); /*list<15571>*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19608_19604, _ctx), kk_std_core__list_box(_b_19609_19605, _ctx), _ctx);
  }
  struct kk_std_core_Cons* _con21861 = kk_std_core__as_Cons(xs);
  kk_box_t x0 = _con21861->head;
  kk_std_core__list xx0 = _con21861->tail;
  kk_reuse_t _ru_20416 = kk_reuse_null; /*reuse*/;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    _ru_20416 = (kk_std_core__list_reuse(xs));
  }
  else {
    kk_box_dup(x0);
    kk_std_core__list_dup(xx0);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  bool x1_18224;
  kk_function_t _x21863 = kk_function_dup(pred0); /*(15571) -> 15572 bool*/
  kk_box_t _x21862 = kk_box_dup(x0); /*15571*/
  x1_18224 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21863, (_x21863, _x21862, _ctx)); /*bool*/
  if (kk_yielding(kk_context())) {
    kk_reuse_drop(_ru_20416);
    kk_function_t _b_19610_19606 = kk_std_core_new_partition_acc_fun21864(acc10, acc20, pred0, x0, xx0, _ctx); /*(bool) -> 15572 (list<15571>, list<15571>)*/;
    kk_box_t _x21865 = kk_std_core_hnd_yield_extend(kk_std_core_new_partition_acc_fun21866(_b_19610_19606, _ctx), _ctx); /*3858*/
    return kk_std_core_types__tuple2__unbox(_x21865, _ctx);
  }
  if (x1_18224) {
    { // tailcall
      kk_std_core__list _x21869;
      if (kk_likely(_ru_20416!=NULL)) {
        struct kk_std_core_Cons* _con21870 = (struct kk_std_core_Cons*)_ru_20416;
        _con21870->tail = acc10;
        _x21869 = kk_std_core__base_Cons(_con21870); /*list<61>*/
      }
      else {
        _x21869 = kk_std_core__new_Cons(kk_reuse_null, x0, acc10, _ctx); /*list<61>*/
      }
      xs = xx0;
      acc10 = _x21869;
      goto kk__tailcall;
    }
  }
  { // tailcall
    kk_std_core__list _x21871;
    if (kk_likely(_ru_20416!=NULL)) {
      struct kk_std_core_Cons* _con21872 = (struct kk_std_core_Cons*)_ru_20416;
      _con21872->tail = acc20;
      _x21871 = kk_std_core__base_Cons(_con21872); /*list<61>*/
    }
    else {
      _x21871 = kk_std_core__new_Cons(kk_reuse_null, x0, acc20, _ctx); /*list<61>*/
    }
    xs = xx0;
    acc20 = _x21871;
    goto kk__tailcall;
  }
}
 
// redirect `print` and `println` calls to a specified function.


// lift anonymous function
struct kk_std_core_print_redirect_fun21873__t {
  struct kk_function_s _base;
  kk_function_t _b_19619_19611;
};
static kk_box_t kk_std_core_print_redirect_fun21873(kk_function_t _fself, kk_box_t _b_19614, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_print_redirect_fun21873(kk_function_t _b_19619_19611, kk_context_t* _ctx) {
  struct kk_std_core_print_redirect_fun21873__t* _self = kk_function_alloc_as(struct kk_std_core_print_redirect_fun21873__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_print_redirect_fun21873, kk_context());
  _self->_b_19619_19611 = _b_19619_19611;
  return &_self->_base;
}

static kk_box_t kk_std_core_print_redirect_fun21873(kk_function_t _fself, kk_box_t _b_19614, kk_context_t* _ctx) {
  struct kk_std_core_print_redirect_fun21873__t* _self = kk_function_as(struct kk_std_core_print_redirect_fun21873__t*, _fself);
  kk_function_t _b_19619_19611 = _self->_b_19619_19611; /* (msg : string) -> console () */
  kk_drop_match(_self, {kk_function_dup(_b_19619_19611);}, {}, _ctx)
  kk_unit_t _x21874 = kk_Unit;
  kk_string_t _x21875 = kk_string_unbox(_b_19614); /*string*/
  kk_function_call(kk_unit_t, (kk_function_t, kk_string_t, kk_context_t*), _b_19619_19611, (_b_19619_19611, _x21875, _ctx));
  return kk_unit_box(_x21874);
}

kk_unit_t kk_std_core_print_redirect(kk_function_t print0, kk_context_t* _ctx) { /* (print : (msg : string) -> console ()) -> io () */ 
  kk_ref_t _b_19617_19615 = kk_ref_dup(kk_std_core_redirect); /*ref<global,maybe<(string) -> console ()>>*/;
  kk_std_core_types__maybe _b_19618_19616;
  kk_function_t _b_19619_19611 = print0; /*(msg : string) -> console ()*/;
  _b_19618_19616 = kk_std_core_types__new_Just(kk_function_box(kk_std_core_new_print_redirect_fun21873(_b_19619_19611, _ctx)), _ctx); /*maybe<(msg : string) -> console ()>*/
  kk_unit_t _borrowApp_19945 = kk_Unit;
  kk_ref_set_borrow(_b_19617_19615,(kk_std_core_types__maybe_box(_b_19618_19616, _ctx)),kk_context());
  kk_ref_drop(_b_19617_19615, _ctx);
  _borrowApp_19945; return kk_Unit;
}
 
// lifted
// specialized std/core/filter to parameters [pred]

kk_std_core__list kk_std_core__ctail_lift17199_remove(kk_function_t pred, kk_std_core__list xs0, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (pred : (a) -> bool, xs0 : list<a>, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21876 = kk_std_core__as_Cons(xs0);
    kk_box_t x0 = _con21876->head;
    kk_std_core__list xx = _con21876->tail;
    kk_reuse_t _ru_20417 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      _ru_20417 = (kk_std_core__list_reuse(xs0));
    }
    else {
      kk_box_dup(x0);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    bool b_17140;
    kk_function_t _x21878 = kk_function_dup(pred); /*(15640) -> bool*/
    kk_box_t _x21877 = kk_box_dup(x0); /*15640*/
    b_17140 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21878, (_x21878, _x21877, _ctx)); /*bool*/
    if (b_17140) {
      kk_reuse_drop(_ru_20417);
      kk_box_drop(x0, _ctx);
      { // tailcall
        xs0 = xx;
        goto kk__tailcall;
      }
    }
    kk_std_core__list _ctail_17357 = kk_std_core__list_hole(); /*list<15640>*/;
    kk_std_core__list _ctail_17358;
    if (kk_likely(_ru_20417!=NULL)) {
      struct kk_std_core_Cons* _con21879 = (struct kk_std_core_Cons*)_ru_20417;
      _con21879->tail = _ctail_17357;
      _ctail_17358 = kk_std_core__base_Cons(_con21879); /*list<15640>*/
    }
    else {
      _ctail_17358 = kk_std_core__new_Cons(kk_reuse_null, x0, _ctail_17357, _ctx); /*list<15640>*/
    }
    { // tailcall
      kk_std_core_types__ctail _x21880;
      kk_box_t* field_18229 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17358)->tail)); /*cfield<list<15640>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x19626 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19626, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17358, _ctx);
        kk_std_core__list _b_19640_19631 = head0; /*list<15640>*/;
        kk_box_t* _b_19641_19632 = field_18229; /*cfield<list<15640>>*/;
        _x21880 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19640_19631, _ctx), _b_19641_19632, _ctx); /*ctail<58>*/
        goto _match21881;
      }
      kk_std_core__list _b_19642_19633 = _ctail_17358; /*list<15640>*/;
      kk_box_t* _b_19643_19634 = field_18229; /*cfield<list<15640>>*/;
      _x21880 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19642_19633, _ctx), _b_19643_19634, _ctx); /*ctail<58>*/
      _match21881: ;
      xs0 = xx;
      _acc = _x21880;
      goto kk__tailcall;
    }
  }
  kk_function_drop(pred, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19635 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19635, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// lifted
// specialized std/core/filter to parameters [pred]

kk_std_core__list kk_std_core__lift17199_remove(kk_function_t pred0, kk_std_core__list xs00, kk_context_t* _ctx) { /* forall<a> (pred : (a) -> bool, xs0 : list<a>) -> list<a> */ 
  return kk_std_core__ctail_lift17199_remove(pred0, xs00, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17763_op(kk_function_t action0_17143, kk_integer_t end_17142, kk_integer_t i, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action0.17143 : (int) -> e (), end.17142 : int, i : int, wild_ : ()) -> e () */ 
  kk_integer_t i0_17276 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__lift17200_repeat_1(action0_17143, end_17142, i0_17276, _ctx); return kk_Unit;
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17200_repeat_fun21888__t_1 {
  struct kk_function_s _base;
  kk_function_t action0_171430;
  kk_integer_t end_171420;
  kk_integer_t i0;
};
static kk_unit_t kk_std_core__lift17200_repeat_fun21888_1(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17200_repeat_fun21888_1(kk_function_t action0_171430, kk_integer_t end_171420, kk_integer_t i0, kk_context_t* _ctx) {
  struct kk_std_core__lift17200_repeat_fun21888__t_1* _self = kk_function_alloc_as(struct kk_std_core__lift17200_repeat_fun21888__t_1, 4, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17200_repeat_fun21888_1, kk_context());
  _self->action0_171430 = action0_171430;
  _self->end_171420 = end_171420;
  _self->i0 = i0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__lift17200_repeat_fun21888_1(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__lift17200_repeat_fun21888__t_1* _self = kk_function_as(struct kk_std_core__lift17200_repeat_fun21888__t_1*, _fself);
  kk_function_t action0_171430 = _self->action0_171430; /* (int) -> 15659 () */
  kk_integer_t end_171420 = _self->end_171420; /* int */
  kk_integer_t i0 = _self->i0; /* int */
  kk_drop_match(_self, {kk_function_dup(action0_171430);kk_integer_dup(end_171420);kk_integer_dup(i0);}, {}, _ctx)
  return kk_std_core__mlift17763_op(action0_171430, end_171420, i0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17200_repeat_fun21890__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_19646_19644;
};
static kk_box_t kk_std_core__lift17200_repeat_fun21890_1(kk_function_t _fself, kk_box_t _b_19645, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17200_repeat_fun21890_1(kk_function_t _b_19646_19644, kk_context_t* _ctx) {
  struct kk_std_core__lift17200_repeat_fun21890__t_1* _self = kk_function_alloc_as(struct kk_std_core__lift17200_repeat_fun21890__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17200_repeat_fun21890_1, kk_context());
  _self->_b_19646_19644 = _b_19646_19644;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17200_repeat_fun21890_1(kk_function_t _fself, kk_box_t _b_19645, kk_context_t* _ctx) {
  struct kk_std_core__lift17200_repeat_fun21890__t_1* _self = kk_function_as(struct kk_std_core__lift17200_repeat_fun21890__t_1*, _fself);
  kk_function_t _b_19646_19644 = _self->_b_19646_19644; /* (wild_0 : ()) -> 15659 () */
  kk_drop_match(_self, {kk_function_dup(_b_19646_19644);}, {}, _ctx)
  kk_unit_t _x21891 = kk_Unit;
  kk_unit_t _x21892 = kk_Unit;
  kk_unit_unbox(_b_19645);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19646_19644, (_b_19646_19644, _x21892, _ctx));
  return kk_unit_box(_x21891);
}

kk_unit_t kk_std_core__lift17200_repeat_1(kk_function_t action0_171430, kk_integer_t end_171420, kk_integer_t i0, kk_context_t* _ctx) { /* forall<e> (action0.17143 : (int) -> e (), end.17142 : int, i : int) -> e () */ 
  kk__tailcall: ;
  bool _match_19943 = kk_integer_lte_borrow(i0,end_171420,kk_context()); /*bool*/;
  if (_match_19943) {
    kk_unit_t x_18232 = kk_Unit;
    kk_function_t _x21887 = kk_function_dup(action0_171430); /*(int) -> 15659 ()*/
    kk_integer_t _x21886 = kk_integer_dup(i0); /*int*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_integer_t, kk_context_t*), _x21887, (_x21887, _x21886, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19646_19644 = kk_std_core__new_lift17200_repeat_fun21888_1(action0_171430, end_171420, i0, _ctx); /*(wild_0 : ()) -> 15659 ()*/;
      kk_box_t _x21889 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17200_repeat_fun21890_1(_b_19646_19644, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x21889); return kk_Unit;
    }
    kk_integer_t i0_172760 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
    { // tailcall
      i0 = i0_172760;
      goto kk__tailcall;
    }
  }
  kk_function_drop(action0_171430, _ctx);
  kk_integer_drop(end_171420, _ctx);
  kk_integer_drop(i0, _ctx);
  kk_Unit; return kk_Unit;
}
extern kk_unit_t kk_std_core_repeat_fun21893_1(kk_function_t _fself, kk_integer_t i1, kk_context_t* _ctx) {
  struct kk_std_core_repeat_fun21893__t_1* _self = kk_function_as(struct kk_std_core_repeat_fun21893__t_1*, _fself);
  kk_function_t action = _self->action; /* () -> 15659 () */
  kk_drop_match(_self, {kk_function_dup(action);}, {}, _ctx)
  kk_integer_drop(i1, _ctx);
  return kk_function_call(kk_unit_t, (kk_function_t, kk_context_t*), action, (action, _ctx));
}
 
// Create a list of `n`  repeated elementes `x`

kk_std_core__list kk_std_core__ctail_replicate(kk_box_t x, kk_integer_t n, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a> (x : a, n : int, ctail<list<a>>) -> list<a> */ 
  kk__tailcall: ;
  bool _match_19940;
  kk_integer_t _borrowedArg_19941 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_19942 = kk_integer_gt_borrow(n,_borrowedArg_19941,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19941, _ctx);
  _match_19940 = _borrowApp_19942; /*bool*/
  if (_match_19940) {
    kk_std_core__list _ctail_17359 = kk_std_core__list_hole(); /*list<15703>*/;
    kk_std_core__list _ctail_17360;
    kk_box_t _x21895 = kk_box_dup(x); /*15703*/
    _ctail_17360 = kk_std_core__new_Cons(kk_reuse_null, _x21895, _ctail_17359, _ctx); /*list<15703>*/
    { // tailcall
      kk_integer_t _x21896 = kk_integer_sub(n,(kk_integer_from_small(1)),kk_context()); /*int*/
      kk_std_core_types__ctail _x21897;
      kk_box_t* field_18237 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17360)->tail)); /*cfield<list<15703>>*/;
      if (kk_std_core_types__is_CTail(_acc)) {
        kk_box_t _box_x19653 = _acc._cons.CTail._field1;
        kk_box_t* last0 = _acc._cons.CTail._field2;
        kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19653, NULL);
        kk_std_core__list_dup(head0);
        kk_std_core_types__ctail_drop(_acc, _ctx);
        kk_Unit;
        kk_unit_t __ = kk_Unit;
        *(last0) = kk_std_core__list_box(_ctail_17360, _ctx);
        kk_std_core__list _b_19667_19658 = head0; /*list<15703>*/;
        kk_box_t* _b_19668_19659 = field_18237; /*cfield<list<15703>>*/;
        _x21897 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19667_19658, _ctx), _b_19668_19659, _ctx); /*ctail<58>*/
        goto _match21898;
      }
      kk_std_core__list _b_19669_19660 = _ctail_17360; /*list<15703>*/;
      kk_box_t* _b_19670_19661 = field_18237; /*cfield<list<15703>>*/;
      _x21897 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19669_19660, _ctx), _b_19670_19661, _ctx); /*ctail<58>*/
      _match21898: ;
      n = _x21896;
      _acc = _x21897;
      goto kk__tailcall;
    }
  }
  kk_integer_drop(n, _ctx);
  kk_box_drop(x, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19662 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19662, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Create a list of `n`  repeated elementes `x`

kk_std_core__list kk_std_core_replicate(kk_box_t x0, kk_integer_t n0, kk_context_t* _ctx) { /* forall<a> (x : a, n : int) -> list<a> */ 
  return kk_std_core__ctail_replicate(x0, n0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17201_reverse_join(kk_std_core__list acc, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a> (acc : list<a>, ys : list<a>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con21901 = kk_std_core__as_Cons(ys);
    kk_box_t x = _con21901->head;
    kk_std_core__list xx = _con21901->tail;
    kk_reuse_t _ru_20418 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      _ru_20418 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x21902;
      if (kk_likely(_ru_20418!=NULL)) {
        struct kk_std_core_Cons* _con21903 = (struct kk_std_core_Cons*)_ru_20418;
        _con21903->tail = acc;
        _x21902 = kk_std_core__base_Cons(_con21903); /*list<61>*/
      }
      else {
        _x21902 = kk_std_core__new_Cons(kk_reuse_null, x, acc, _ctx); /*list<61>*/
      }
      acc = _x21902;
      ys = xx;
      goto kk__tailcall;
    }
  }
  return acc;
}
 
// lifted

kk_string_t kk_std_core__lift17202_reverse_join(kk_string_t sep_17146, kk_std_core__list ys0, kk_string_t acc0, kk_context_t* _ctx) { /* (sep.17146 : string, ys0 : list<string>, acc0 : string) -> string */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys0)) {
    struct kk_std_core_Cons* _con21904 = kk_std_core__as_Cons(ys0);
    kk_box_t _box_x19671 = _con21904->head;
    kk_std_core__list yy = _con21904->tail;
    kk_string_t y = kk_string_unbox(_box_x19671);
    if (kk_likely(kk_std_core__list_is_unique(ys0))) {
      kk_std_core__list_free(ys0);
    }
    else {
      kk_string_dup(y);
      kk_std_core__list_dup(yy);
      kk_Unit;
      kk_std_core__list_decref(ys0, _ctx);
      kk_Unit;
    }
    kk_string_t acc00_17278;
    kk_string_t _x21906;
    kk_string_t _x21907 = kk_string_dup(sep_17146); /*string*/
    _x21906 = kk_std_core__lp__plus__plus__1_rp_(_x21907, y, _ctx); /*string*/
    acc00_17278 = kk_std_core__lp__plus__plus__1_rp_(acc0, _x21906, _ctx); /*string*/
    { // tailcall
      ys0 = yy;
      acc0 = acc00_17278;
      goto kk__tailcall;
    }
  }
  kk_string_drop(sep_17146, _ctx);
  return acc0;
}
 
// Concatenate all strings in a list in reverse order

kk_string_t kk_std_core_reverse_join(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<string>) -> string */ 
  kk_std_core__list xs0_17145 = kk_std_core__lift17201_reverse_join(kk_std_core__new_Nil(_ctx), xs, _ctx); /*list<string>*/;
  if (kk_std_core__is_Nil(xs0_17145)) {
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con21909 = kk_std_core__as_Cons(xs0_17145);
  kk_box_t _box_x19672 = _con21909->head;
  kk_std_core__list xx0 = _con21909->tail;
  kk_string_t x0 = kk_string_unbox(_box_x19672);
  if (kk_likely(kk_std_core__list_is_unique(xs0_17145))) {
    kk_std_core__list_free(xs0_17145);
  }
  else {
    kk_string_dup(x0);
    kk_std_core__list_dup(xx0);
    kk_Unit;
    kk_std_core__list_decref(xs0_17145, _ctx);
    kk_Unit;
  }
  kk_string_t _x21911 = kk_string_empty(); /*string*/
  return kk_std_core__lift17202_reverse_join(_x21911, xx0, x0, _ctx);
}

kk_string_t kk_std_core_show_tuple(kk_std_core_types__tuple2_ x, kk_function_t showfst, kk_function_t showsnd, kk_context_t* _ctx) { /* forall<a,b> (x : (a, b), showfst : (a) -> string, showsnd : (b) -> string) -> string */ 
  kk_string_t _x21913;
  kk_define_string_literal(, _s21914, 1, "(")
  _x21913 = kk_string_dup(_s21914); /*string*/
  kk_string_t _x21915;
  kk_string_t _x21916;
  kk_box_t _x21917;
  kk_box_t _x = x.fst;
  kk_box_dup(_x);
  _x21917 = _x; /*16023*/
  _x21916 = kk_function_call(kk_string_t, (kk_function_t, kk_box_t, kk_context_t*), showfst, (showfst, _x21917, _ctx)); /*string*/
  kk_string_t _x21918;
  kk_string_t _x21919;
  kk_define_string_literal(, _s21920, 1, ",")
  _x21919 = kk_string_dup(_s21920); /*string*/
  kk_string_t _x21921;
  kk_string_t _x21922;
  kk_box_t _x21923;
  kk_box_t _x0 = x.snd;
  kk_box_dup(_x0);
  kk_std_core_types__tuple2__drop(x, _ctx);
  kk_Unit;
  _x21923 = _x0; /*16024*/
  _x21922 = kk_function_call(kk_string_t, (kk_function_t, kk_box_t, kk_context_t*), showsnd, (showsnd, _x21923, _ctx)); /*string*/
  kk_string_t _x21924;
  kk_define_string_literal(, _s21925, 1, ")")
  _x21924 = kk_string_dup(_s21925); /*string*/
  _x21921 = kk_std_core__lp__plus__plus__1_rp_(_x21922, _x21924, _ctx); /*string*/
  _x21918 = kk_std_core__lp__plus__plus__1_rp_(_x21919, _x21921, _ctx); /*string*/
  _x21915 = kk_std_core__lp__plus__plus__1_rp_(_x21916, _x21918, _ctx); /*string*/
  return kk_std_core__lp__plus__plus__1_rp_(_x21913, _x21915, _ctx);
}
 
// monadic lift

kk_std_core_types__tuple2_ kk_std_core__mlift17764_op(kk_std_core__list acc, kk_function_t predicate, kk_box_t y, kk_std_core__list ys, kk_std_core__list yy, bool _y_17635, kk_context_t* _ctx) { /* forall<a,e> (acc : list<a>, predicate : (a) -> e bool, y : a, ys : list<a>, yy : list<a>, bool) -> e (list<a>, list<a>) */ 
  if (_y_17635) {
    kk_std_core__list_drop(ys, _ctx);
    kk_std_core__list _x21926 = kk_std_core__new_Cons(kk_reuse_null, y, acc, _ctx); /*list<61>*/
    return kk_std_core__lift17203_span(predicate, yy, _x21926, _ctx);
  }
  kk_function_drop(predicate, _ctx);
  kk_box_drop(y, _ctx);
  kk_std_core__list_drop(yy, _ctx);
  kk_std_core__list _b_19675_19673 = kk_std_core__lift17186_reverse(kk_std_core__new_Nil(_ctx), acc, _ctx); /*list<16082>*/;
  kk_std_core__list _b_19676_19674 = ys; /*list<16082>*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19675_19673, _ctx), kk_std_core__list_box(_b_19676_19674, _ctx), _ctx);
}
 
// lifted
// todo: implement TRMC with multiple results to avoid the reverse


// lift anonymous function
struct kk_std_core__lift17203_span_fun21930__t {
  struct kk_function_s _base;
  kk_std_core__list acc0;
  kk_function_t predicate0;
  kk_box_t y0;
  kk_std_core__list ys0;
  kk_std_core__list yy0;
};
static kk_std_core_types__tuple2_ kk_std_core__lift17203_span_fun21930(kk_function_t _fself, bool _y_176350, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17203_span_fun21930(kk_std_core__list acc0, kk_function_t predicate0, kk_box_t y0, kk_std_core__list ys0, kk_std_core__list yy0, kk_context_t* _ctx) {
  struct kk_std_core__lift17203_span_fun21930__t* _self = kk_function_alloc_as(struct kk_std_core__lift17203_span_fun21930__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17203_span_fun21930, kk_context());
  _self->acc0 = acc0;
  _self->predicate0 = predicate0;
  _self->y0 = y0;
  _self->ys0 = ys0;
  _self->yy0 = yy0;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_core__lift17203_span_fun21930(kk_function_t _fself, bool _y_176350, kk_context_t* _ctx) {
  struct kk_std_core__lift17203_span_fun21930__t* _self = kk_function_as(struct kk_std_core__lift17203_span_fun21930__t*, _fself);
  kk_std_core__list acc0 = _self->acc0; /* list<16082> */
  kk_function_t predicate0 = _self->predicate0; /* (16082) -> 16083 bool */
  kk_box_t y0 = _self->y0; /* 16082 */
  kk_std_core__list ys0 = _self->ys0; /* list<16082> */
  kk_std_core__list yy0 = _self->yy0; /* list<16082> */
  kk_drop_match(_self, {kk_std_core__list_dup(acc0);kk_function_dup(predicate0);kk_box_dup(y0);kk_std_core__list_dup(ys0);kk_std_core__list_dup(yy0);}, {}, _ctx)
  return kk_std_core__mlift17764_op(acc0, predicate0, y0, ys0, yy0, _y_176350, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17203_span_fun21932__t {
  struct kk_function_s _base;
  kk_function_t _b_19683_19677;
};
static kk_box_t kk_std_core__lift17203_span_fun21932(kk_function_t _fself, kk_box_t _b_19678, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17203_span_fun21932(kk_function_t _b_19683_19677, kk_context_t* _ctx) {
  struct kk_std_core__lift17203_span_fun21932__t* _self = kk_function_alloc_as(struct kk_std_core__lift17203_span_fun21932__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17203_span_fun21932, kk_context());
  _self->_b_19683_19677 = _b_19683_19677;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17203_span_fun21932(kk_function_t _fself, kk_box_t _b_19678, kk_context_t* _ctx) {
  struct kk_std_core__lift17203_span_fun21932__t* _self = kk_function_as(struct kk_std_core__lift17203_span_fun21932__t*, _fself);
  kk_function_t _b_19683_19677 = _self->_b_19683_19677; /* (bool) -> 16083 (list<16082>, list<16082>) */
  kk_drop_match(_self, {kk_function_dup(_b_19683_19677);}, {}, _ctx)
  kk_std_core_types__tuple2_ _x21933;
  bool _x21934 = kk_bool_unbox(_b_19678); /*bool*/
  _x21933 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, bool, kk_context_t*), _b_19683_19677, (_b_19683_19677, _x21934, _ctx)); /*(list<16082>, list<16082>)*/
  return kk_std_core_types__tuple2__box(_x21933, _ctx);
}

kk_std_core_types__tuple2_ kk_std_core__lift17203_span(kk_function_t predicate0, kk_std_core__list ys0, kk_std_core__list acc0, kk_context_t* _ctx) { /* forall<a,e> (predicate : (a) -> e bool, ys : list<a>, acc : list<a>) -> e (list<a>, list<a>) */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys0)) {
    struct kk_std_core_Cons* _con21927 = kk_std_core__as_Cons(ys0);
    kk_box_t y0 = _con21927->head;
    kk_std_core__list yy0 = _con21927->tail;
    kk_box_dup(y0);
    kk_std_core__list_dup(yy0);
    bool x_18241;
    kk_function_t _x21929 = kk_function_dup(predicate0); /*(16082) -> 16083 bool*/
    kk_box_t _x21928 = kk_box_dup(y0); /*16082*/
    x_18241 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21929, (_x21929, _x21928, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19683_19677 = kk_std_core__new_lift17203_span_fun21930(acc0, predicate0, y0, ys0, yy0, _ctx); /*(bool) -> 16083 (list<16082>, list<16082>)*/;
      kk_box_t _x21931 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17203_span_fun21932(_b_19683_19677, _ctx), _ctx); /*3858*/
      return kk_std_core_types__tuple2__unbox(_x21931, _ctx);
    }
    if (x_18241) {
      kk_std_core__list_dropn(ys0, ((int32_t)2), _ctx);
      { // tailcall
        kk_std_core__list _x21935 = kk_std_core__new_Cons(kk_reuse_null, y0, acc0, _ctx); /*list<61>*/
        ys0 = yy0;
        acc0 = _x21935;
        goto kk__tailcall;
      }
    }
    kk_function_drop(predicate0, _ctx);
    kk_box_drop(y0, _ctx);
    kk_std_core__list_drop(yy0, _ctx);
    kk_std_core__list _b_19684_19679 = kk_std_core__lift17186_reverse(kk_std_core__new_Nil(_ctx), acc0, _ctx); /*list<16082>*/;
    kk_std_core__list _b_19685_19680 = ys0; /*list<16082>*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19684_19679, _ctx), kk_std_core__list_box(_b_19685_19680, _ctx), _ctx);
  }
  kk_function_drop(predicate0, _ctx);
  kk_std_core__list _b_19686_19681 = kk_std_core__lift17186_reverse(kk_std_core__new_Nil(_ctx), acc0, _ctx); /*list<16082>*/;
  kk_std_core__list _b_19687_19682 = ys0; /*list<16082>*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19686_19681, _ctx), kk_std_core__list_box(_b_19687_19682, _ctx), _ctx);
}
 
// lifted
// specialized std/core/foldl to parameters [f]

kk_integer_t kk_std_core__lift17204_sum(kk_std_core__list xs0, kk_integer_t z, kk_context_t* _ctx) { /* (xs0 : list<int>, z : int) -> int */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21936 = kk_std_core__as_Cons(xs0);
    kk_box_t _box_x19688 = _con21936->head;
    kk_std_core__list xx = _con21936->tail;
    kk_integer_t x0 = kk_integer_unbox(_box_x19688);
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_integer_dup(x0);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_integer_t _x21938 = kk_integer_add(z,x0,kk_context()); /*int*/
      xs0 = xx;
      z = _x21938;
      goto kk__tailcall;
    }
  }
  return z;
}
 
// Return the tail of list. Returns the empty list if `xs` is empty.

kk_std_core__list kk_std_core_tail_1(kk_std_core__list xs, kk_context_t* _ctx) { /* forall<a> (xs : list<a>) -> list<a> */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21939 = kk_std_core__as_Cons(xs);
    kk_box_t _pat0 = _con21939->head;
    kk_std_core__list xx = _con21939->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_box_drop(_pat0, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_core__list_free(xs);
    }
    else {
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    return xx;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Return the tail of a string (or the empty string)

kk_string_t kk_std_core_tail_2(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> string */ 
  kk_std_core__sslice _x21940;
  kk_std_core__sslice slice1 = kk_std_core_first1(s, _ctx); /*sslice*/;
  kk_std_core__sslice slice0_17149;
  bool _match_19931;
  kk_integer_t _borrowedArg_19936;
  kk_std_core_types__optional _match_19937 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_19937)) {
    kk_box_t _box_x19689 = _match_19937._cons.Optional.value;
    kk_integer_t _n_10065 = kk_integer_unbox(_box_x19689);
    kk_integer_dup(_n_10065);
    kk_std_core_types__optional_drop(_match_19937, _ctx);
    kk_Unit;
    _borrowedArg_19936 = _n_10065; /*int*/
    goto _match21941;
  }
  _borrowedArg_19936 = kk_integer_from_small(1); /*int*/
  _match21941: ;
  kk_integer_t _borrowedArg_19935 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19938 = kk_integer_eq_borrow(_borrowedArg_19936,_borrowedArg_19935,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19936, _ctx);
  kk_integer_drop(_borrowedArg_19935, _ctx);
  _match_19931 = _borrowApp_19938; /*bool*/
  if (_match_19931) {
    slice0_17149 = slice1; /*sslice*/
  }
  else {
    kk_integer_t _borrowedArg_19932;
    kk_integer_t _x21943;
    kk_std_core_types__optional _match_19933 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_19933)) {
      kk_box_t _box_x19690 = _match_19933._cons.Optional.value;
      kk_integer_t _n_100650 = kk_integer_unbox(_box_x19690);
      kk_integer_dup(_n_100650);
      kk_std_core_types__optional_drop(_match_19933, _ctx);
      kk_Unit;
      _x21943 = _n_100650; /*int*/
      goto _match21944;
    }
    _x21943 = kk_integer_from_small(1); /*int*/
    _match21944: ;
    _borrowedArg_19932 = kk_integer_sub(_x21943,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__sslice _borrowApp_19934 = kk_std_core_extend(slice1, _borrowedArg_19932, _ctx); /*sslice*/;
    kk_integer_drop(_borrowedArg_19932, _ctx);
    slice0_17149 = _borrowApp_19934; /*sslice*/
  }
  kk_string_t s0 = slice0_17149.str;
  kk_ssize_t start0 = slice0_17149.start;
  kk_ssize_t len0 = slice0_17149.len;
  kk_string_dup(s0);
  kk_std_core__sslice_drop(slice0_17149, _ctx);
  kk_Unit;
  kk_string_t _x21946 = kk_string_dup(s0); /*string*/
  kk_ssize_t _x21947 = (start0 + len0); /*ssize_t*/
  kk_ssize_t _x21948;
  kk_ssize_t _x21949 = kk_string_len(s0,kk_context()); /*ssize_t*/
  kk_ssize_t _x21950 = (start0 + len0); /*ssize_t*/
  _x21948 = (_x21949 - _x21950); /*ssize_t*/
  _x21940 = kk_std_core__new_Sslice(_x21946, _x21947, _x21948, _ctx); /*sslice*/
  return kk_std_core_string_3(_x21940, _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17765_op(kk_std_core_types__ctail _acc, kk_function_t predicate, kk_box_t x, kk_std_core__list xx, bool _y_17640, kk_context_t* _ctx) { /* forall<a,e> (ctail<list<a>>, predicate : (a) -> e bool, x : a, xx : list<a>, bool) -> e list<a> */ 
  if (_y_17640) {
    kk_std_core__list _ctail_17361 = kk_std_core__list_hole(); /*list<16193>*/;
    kk_std_core__list _ctail_17362 = kk_std_core__new_Cons(kk_reuse_null, x, _ctail_17361, _ctx); /*list<16193>*/;
    kk_std_core_types__ctail _x21951;
    kk_box_t* field_18248 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17362)->tail)); /*cfield<list<16193>>*/;
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x19697 = _acc._cons.CTail._field1;
      kk_box_t* last0 = _acc._cons.CTail._field2;
      kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19697, NULL);
      kk_std_core__list_dup(head0);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __ = kk_Unit;
      *(last0) = kk_std_core__list_box(_ctail_17362, _ctx);
      kk_std_core__list _b_19711_19702 = head0; /*list<16193>*/;
      kk_box_t* _b_19712_19703 = field_18248; /*cfield<list<16193>>*/;
      _x21951 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19711_19702, _ctx), _b_19712_19703, _ctx); /*ctail<58>*/
      goto _match21952;
    }
    kk_std_core__list _b_19713_19704 = _ctail_17362; /*list<16193>*/;
    kk_box_t* _b_19714_19705 = field_18248; /*cfield<list<16193>>*/;
    _x21951 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19713_19704, _ctx), _b_19714_19705, _ctx); /*ctail<58>*/
    _match21952: ;
    return kk_std_core__ctail_take_while(xx, predicate, _x21951, _ctx);
  }
  kk_function_drop(predicate, _ctx);
  kk_box_drop(x, _ctx);
  kk_std_core__list_drop(xx, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19706 = _acc._cons.CTail._field1;
    kk_box_t* last2 = _acc._cons.CTail._field2;
    kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19706, NULL);
    kk_std_core__list_dup(head1);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __0 = kk_Unit;
    *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head1;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17766_op_fun21955__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t x0;
};
static kk_std_core__list kk_std_core__mlift17766_op_fun21955(kk_function_t _fself, kk_std_core__list _ctail_17364, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17766_op_fun21955(kk_function_t _accm, kk_box_t x0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17766_op_fun21955__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17766_op_fun21955__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17766_op_fun21955, kk_context());
  _self->_accm = _accm;
  _self->x0 = x0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17766_op_fun21955(kk_function_t _fself, kk_std_core__list _ctail_17364, kk_context_t* _ctx) {
  struct kk_std_core__mlift17766_op_fun21955__t* _self = kk_function_as(struct kk_std_core__mlift17766_op_fun21955__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<16193>) -> list<16193> */
  kk_box_t x0 = _self->x0; /* 16193 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(x0);}, {}, _ctx)
  kk_std_core__list _x21956 = kk_std_core__new_Cons(kk_reuse_null, x0, _ctail_17364, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x21956, _ctx));
}

kk_std_core__list kk_std_core__mlift17766_op(kk_function_t _accm, kk_function_t predicate0, kk_box_t x0, kk_std_core__list xx0, bool _y_17644, kk_context_t* _ctx) { /* forall<a,e> ((list<a>) -> list<a>, predicate : (a) -> e bool, x : a, xx : list<a>, bool) -> e list<a> */ 
  if (_y_17644) {
    return kk_std_core__ctailm_take_while(xx0, predicate0, kk_std_core__new_mlift17766_op_fun21955(_accm, x0, _ctx), _ctx);
  }
  kk_function_drop(predicate0, _ctx);
  kk_box_drop(x0, _ctx);
  kk_std_core__list_drop(xx0, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Keep only those initial elements that satisfy `predicate`


// lift anonymous function
struct kk_std_core__ctail_take_while_fun21960__t {
  struct kk_function_s _base;
  kk_function_t predicate1;
  kk_box_t x1;
  kk_std_core__list xx1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_take_while_fun21960(kk_function_t _fself, bool _y_176400, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_take_while_fun21960(kk_function_t predicate1, kk_box_t x1, kk_std_core__list xx1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_take_while_fun21960__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_take_while_fun21960__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_take_while_fun21960, kk_context());
  _self->predicate1 = predicate1;
  _self->x1 = x1;
  _self->xx1 = xx1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_take_while_fun21960(kk_function_t _fself, bool _y_176400, kk_context_t* _ctx) {
  struct kk_std_core__ctail_take_while_fun21960__t* _self = kk_function_as(struct kk_std_core__ctail_take_while_fun21960__t*, _fself);
  kk_function_t predicate1 = _self->predicate1; /* (16193) -> 16194 bool */
  kk_box_t x1 = _self->x1; /* 16193 */
  kk_std_core__list xx1 = _self->xx1; /* list<16193> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<16193>> */
  kk_drop_match(_self, {kk_function_dup(predicate1);kk_box_dup(x1);kk_std_core__list_dup(xx1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17765_op(_acc0, predicate1, x1, xx1, _y_176400, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_take_while_fun21962__t {
  struct kk_function_s _base;
  kk_function_t _b_19742_19715;
};
static kk_box_t kk_std_core__ctail_take_while_fun21962(kk_function_t _fself, kk_box_t _b_19716, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_take_while_fun21962(kk_function_t _b_19742_19715, kk_context_t* _ctx) {
  struct kk_std_core__ctail_take_while_fun21962__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_take_while_fun21962__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_take_while_fun21962, kk_context());
  _self->_b_19742_19715 = _b_19742_19715;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_take_while_fun21962(kk_function_t _fself, kk_box_t _b_19716, kk_context_t* _ctx) {
  struct kk_std_core__ctail_take_while_fun21962__t* _self = kk_function_as(struct kk_std_core__ctail_take_while_fun21962__t*, _fself);
  kk_function_t _b_19742_19715 = _self->_b_19742_19715; /* (bool) -> 16194 list<16193> */
  kk_drop_match(_self, {kk_function_dup(_b_19742_19715);}, {}, _ctx)
  kk_std_core__list _x21963;
  bool _x21964 = kk_bool_unbox(_b_19716); /*bool*/
  _x21963 = kk_function_call(kk_std_core__list, (kk_function_t, bool, kk_context_t*), _b_19742_19715, (_b_19742_19715, _x21964, _ctx)); /*list<16193>*/
  return kk_std_core__list_box(_x21963, _ctx);
}

kk_std_core__list kk_std_core__ctail_take_while(kk_std_core__list xs, kk_function_t predicate1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool, ctail<list<a>>) -> e list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con21957 = kk_std_core__as_Cons(xs);
    kk_box_t x1 = _con21957->head;
    kk_std_core__list xx1 = _con21957->tail;
    kk_reuse_t _ru_20424 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      _ru_20424 = (kk_std_core__list_reuse(xs));
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    bool x2_18251;
    kk_function_t _x21959 = kk_function_dup(predicate1); /*(16193) -> 16194 bool*/
    kk_box_t _x21958 = kk_box_dup(x1); /*16193*/
    x2_18251 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21959, (_x21959, _x21958, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_reuse_drop(_ru_20424);
      kk_function_t _b_19742_19715 = kk_std_core__new_ctail_take_while_fun21960(predicate1, x1, xx1, _acc0, _ctx); /*(bool) -> 16194 list<16193>*/;
      kk_box_t _x21961 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_take_while_fun21962(_b_19742_19715, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21961, _ctx);
    }
    if (x2_18251) {
      kk_std_core__list _ctail_173610 = kk_std_core__list_hole(); /*list<16193>*/;
      kk_std_core__list _ctail_173620;
      if (kk_likely(_ru_20424!=NULL)) {
        struct kk_std_core_Cons* _con21965 = (struct kk_std_core_Cons*)_ru_20424;
        _con21965->tail = _ctail_173610;
        _ctail_173620 = kk_std_core__base_Cons(_con21965); /*list<16193>*/
      }
      else {
        _ctail_173620 = kk_std_core__new_Cons(kk_reuse_null, x1, _ctail_173610, _ctx); /*list<16193>*/
      }
      { // tailcall
        kk_std_core_types__ctail _x21966;
        kk_box_t* field0_18256 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_173620)->tail)); /*cfield<list<16193>>*/;
        if (kk_std_core_types__is_CTail(_acc0)) {
          kk_box_t _box_x19723 = _acc0._cons.CTail._field1;
          kk_box_t* last3 = _acc0._cons.CTail._field2;
          kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19723, NULL);
          kk_std_core__list_dup(head2);
          kk_std_core_types__ctail_drop(_acc0, _ctx);
          kk_Unit;
          kk_unit_t __1 = kk_Unit;
          *(last3) = kk_std_core__list_box(_ctail_173620, _ctx);
          kk_std_core__list _b_19743_19728 = head2; /*list<16193>*/;
          kk_box_t* _b_19744_19729 = field0_18256; /*cfield<list<16193>>*/;
          _x21966 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19743_19728, _ctx), _b_19744_19729, _ctx); /*ctail<58>*/
          goto _match21967;
        }
        kk_std_core__list _b_19745_19730 = _ctail_173620; /*list<16193>*/;
        kk_box_t* _b_19746_19731 = field0_18256; /*cfield<list<16193>>*/;
        _x21966 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19745_19730, _ctx), _b_19746_19731, _ctx); /*ctail<58>*/
        _match21967: ;
        xs = xx1;
        _acc0 = _x21966;
        goto kk__tailcall;
      }
    }
    kk_reuse_drop(_ru_20424);
    kk_function_drop(predicate1, _ctx);
    kk_box_drop(x1, _ctx);
    kk_std_core__list_drop(xx1, _ctx);
    if (kk_std_core_types__is_CTail(_acc0)) {
      kk_box_t _box_x19732 = _acc0._cons.CTail._field1;
      kk_box_t* last4 = _acc0._cons.CTail._field2;
      kk_std_core__list head3 = kk_std_core__list_unbox(_box_x19732, NULL);
      kk_std_core__list_dup(head3);
      kk_std_core_types__ctail_drop(_acc0, _ctx);
      kk_Unit;
      kk_unit_t __2 = kk_Unit;
      *(last4) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
      return head3;
    }
    return kk_std_core__new_Nil(_ctx);
  }
  kk_function_drop(predicate1, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x19737 = _acc0._cons.CTail._field1;
    kk_box_t* last5 = _acc0._cons.CTail._field2;
    kk_std_core__list head4 = kk_std_core__list_unbox(_box_x19737, NULL);
    kk_std_core__list_dup(head4);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __3 = kk_Unit;
    *(last5) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head4;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Keep only those initial elements that satisfy `predicate`


// lift anonymous function
struct kk_std_core__ctailm_take_while_fun21974__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t predicate2;
  kk_box_t x3;
  kk_std_core__list xx2;
};
static kk_std_core__list kk_std_core__ctailm_take_while_fun21974(kk_function_t _fself, bool _y_176440, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_take_while_fun21974(kk_function_t _accm0, kk_function_t predicate2, kk_box_t x3, kk_std_core__list xx2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21974__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_take_while_fun21974__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_take_while_fun21974, kk_context());
  _self->_accm0 = _accm0;
  _self->predicate2 = predicate2;
  _self->x3 = x3;
  _self->xx2 = xx2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_take_while_fun21974(kk_function_t _fself, bool _y_176440, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21974__t* _self = kk_function_as(struct kk_std_core__ctailm_take_while_fun21974__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<16193>) -> list<16193> */
  kk_function_t predicate2 = _self->predicate2; /* (16193) -> 16194 bool */
  kk_box_t x3 = _self->x3; /* 16193 */
  kk_std_core__list xx2 = _self->xx2; /* list<16193> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(predicate2);kk_box_dup(x3);kk_std_core__list_dup(xx2);}, {}, _ctx)
  return kk_std_core__mlift17766_op(_accm0, predicate2, x3, xx2, _y_176440, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_take_while_fun21976__t {
  struct kk_function_s _base;
  kk_function_t _b_19749_19747;
};
static kk_box_t kk_std_core__ctailm_take_while_fun21976(kk_function_t _fself, kk_box_t _b_19748, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_take_while_fun21976(kk_function_t _b_19749_19747, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21976__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_take_while_fun21976__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_take_while_fun21976, kk_context());
  _self->_b_19749_19747 = _b_19749_19747;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_take_while_fun21976(kk_function_t _fself, kk_box_t _b_19748, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21976__t* _self = kk_function_as(struct kk_std_core__ctailm_take_while_fun21976__t*, _fself);
  kk_function_t _b_19749_19747 = _self->_b_19749_19747; /* (bool) -> 16194 list<16193> */
  kk_drop_match(_self, {kk_function_dup(_b_19749_19747);}, {}, _ctx)
  kk_std_core__list _x21977;
  bool _x21978 = kk_bool_unbox(_b_19748); /*bool*/
  _x21977 = kk_function_call(kk_std_core__list, (kk_function_t, bool, kk_context_t*), _b_19749_19747, (_b_19749_19747, _x21978, _ctx)); /*list<16193>*/
  return kk_std_core__list_box(_x21977, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_take_while_fun21980__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x3;
};
static kk_std_core__list kk_std_core__ctailm_take_while_fun21980(kk_function_t _fself, kk_std_core__list _ctail_173640, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_take_while_fun21980(kk_function_t _accm0, kk_box_t x3, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21980__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_take_while_fun21980__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_take_while_fun21980, kk_context());
  _self->_accm0 = _accm0;
  _self->x3 = x3;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_take_while_fun21980(kk_function_t _fself, kk_std_core__list _ctail_173640, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_take_while_fun21980__t* _self = kk_function_as(struct kk_std_core__ctailm_take_while_fun21980__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<16193>) -> list<16193> */
  kk_box_t x3 = _self->x3; /* 16193 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x3);}, {}, _ctx)
  kk_std_core__list _x21981 = kk_std_core__new_Cons(kk_reuse_null, x3, _ctail_173640, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x21981, _ctx));
}

kk_std_core__list kk_std_core__ctailm_take_while(kk_std_core__list xs0, kk_function_t predicate2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool, (list<a>) -> list<a>) -> e list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con21971 = kk_std_core__as_Cons(xs0);
    kk_box_t x3 = _con21971->head;
    kk_std_core__list xx2 = _con21971->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x3);
      kk_std_core__list_dup(xx2);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    bool x4_18261;
    kk_function_t _x21973 = kk_function_dup(predicate2); /*(16193) -> 16194 bool*/
    kk_box_t _x21972 = kk_box_dup(x3); /*16193*/
    x4_18261 = kk_function_call(bool, (kk_function_t, kk_box_t, kk_context_t*), _x21973, (_x21973, _x21972, _ctx)); /*bool*/
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19749_19747 = kk_std_core__new_ctailm_take_while_fun21974(_accm0, predicate2, x3, xx2, _ctx); /*(bool) -> 16194 list<16193>*/;
      kk_box_t _x21975 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_take_while_fun21976(_b_19749_19747, _ctx), _ctx); /*3858*/
      return kk_std_core__list_unbox(_x21975, _ctx);
    }
    if (x4_18261) {
      { // tailcall
        kk_function_t _x21979 = kk_std_core__new_ctailm_take_while_fun21980(_accm0, x3, _ctx); /*(list<16193>) -> list<16193>*/
        xs0 = xx2;
        _accm0 = _x21979;
        goto kk__tailcall;
      }
    }
    kk_function_drop(predicate2, _ctx);
    kk_box_drop(x3, _ctx);
    kk_std_core__list_drop(xx2, _ctx);
    return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
  }
  kk_function_drop(predicate2, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Keep only those initial elements that satisfy `predicate`


// lift anonymous function
struct kk_std_core_take_while_fun21982__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_take_while_fun21982(kk_function_t _fself, kk_std_core__list _ctail_17363, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_take_while_fun21982(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_take_while_fun21982, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_take_while_fun21982(kk_function_t _fself, kk_std_core__list _ctail_17363, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17363;
}

kk_std_core__list kk_std_core_take_while(kk_std_core__list xs1, kk_function_t predicate3, kk_context_t* _ctx) { /* forall<a,e> (xs : list<a>, predicate : (a) -> e bool) -> e list<a> */ 
  bool _match_19928 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19928) {
    return kk_std_core__ctail_take_while(xs1, predicate3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_take_while(xs1, predicate3, kk_std_core_new_take_while_fun21982(_ctx), _ctx);
}
 
// Trace a message used for debug purposes.
// The behaviour is system dependent. On a browser and node it uses
// `console.log`  by default.
// Disabled if `notrace` is called.


// lift anonymous function
struct kk_std_core_trace_fun21985__t {
  struct kk_function_s _base;
  kk_string_t message0;
};
static kk_unit_t kk_std_core_trace_fun21985(kk_function_t _fself, bool _y_17651, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_trace_fun21985(kk_string_t message0, kk_context_t* _ctx) {
  struct kk_std_core_trace_fun21985__t* _self = kk_function_alloc_as(struct kk_std_core_trace_fun21985__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_trace_fun21985, kk_context());
  _self->message0 = message0;
  return &_self->_base;
}

static kk_unit_t kk_std_core_trace_fun21985(kk_function_t _fself, bool _y_17651, kk_context_t* _ctx) {
  struct kk_std_core_trace_fun21985__t* _self = kk_function_as(struct kk_std_core_trace_fun21985__t*, _fself);
  kk_string_t message0 = _self->message0; /* string */
  kk_drop_match(_self, {kk_string_dup(message0);}, {}, _ctx)
  if (_y_17651) {
    return kk_std_core_xtrace(message0, _ctx);
  }
  kk_string_drop(message0, _ctx);
  return kk_Unit;
}


// lift anonymous function
struct kk_std_core_trace_fun21987__t {
  struct kk_function_s _base;
  kk_function_t _b_19754_19752;
};
static kk_box_t kk_std_core_trace_fun21987(kk_function_t _fself, kk_box_t _b_19753, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_trace_fun21987(kk_function_t _b_19754_19752, kk_context_t* _ctx) {
  struct kk_std_core_trace_fun21987__t* _self = kk_function_alloc_as(struct kk_std_core_trace_fun21987__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_trace_fun21987, kk_context());
  _self->_b_19754_19752 = _b_19754_19752;
  return &_self->_base;
}

static kk_box_t kk_std_core_trace_fun21987(kk_function_t _fself, kk_box_t _b_19753, kk_context_t* _ctx) {
  struct kk_std_core_trace_fun21987__t* _self = kk_function_as(struct kk_std_core_trace_fun21987__t*, _fself);
  kk_function_t _b_19754_19752 = _self->_b_19754_19752; /* (bool) -> <read<global>|_16227> () */
  kk_drop_match(_self, {kk_function_dup(_b_19754_19752);}, {}, _ctx)
  kk_unit_t _x21988 = kk_Unit;
  bool _x21989 = kk_bool_unbox(_b_19753); /*bool*/
  kk_function_call(kk_unit_t, (kk_function_t, bool, kk_context_t*), _b_19754_19752, (_b_19754_19752, _x21989, _ctx));
  return kk_unit_box(_x21988);
}

kk_unit_t kk_std_core_trace(kk_string_t message0, kk_context_t* _ctx) { /* (message : string) -> () */ 
  bool x_18264;
  kk_box_t _x21983;
  kk_ref_t _x21984 = kk_ref_dup(kk_std_core_trace_enabled); /*ref<global,bool>*/
  _x21983 = kk_ref_get(_x21984,kk_context()); /*182*/
  x_18264 = kk_bool_unbox(_x21983); /*bool*/
  kk_function_t next0_18265 = kk_std_core_new_trace_fun21985(message0, _ctx); /*(bool) -> <read<global>|_16227> ()*/;
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19754_19752 = next0_18265; /*(bool) -> <read<global>|_16227> ()*/;
    kk_box_t _x21986 = kk_std_core_hnd_yield_extend(kk_std_core_new_trace_fun21987(_b_19754_19752, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21986); return kk_Unit;
  }
  kk_function_call(kk_unit_t, (kk_function_t, bool, kk_context_t*), next0_18265, (next0_18265, x_18264, _ctx)); return kk_Unit;
}


// lift anonymous function
struct kk_std_core_trace_any_fun21992__t {
  struct kk_function_s _base;
  kk_string_t message0;
  kk_box_t x;
};
static kk_unit_t kk_std_core_trace_any_fun21992(kk_function_t _fself, bool _y_17653, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_trace_any_fun21992(kk_string_t message0, kk_box_t x, kk_context_t* _ctx) {
  struct kk_std_core_trace_any_fun21992__t* _self = kk_function_alloc_as(struct kk_std_core_trace_any_fun21992__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_trace_any_fun21992, kk_context());
  _self->message0 = message0;
  _self->x = x;
  return &_self->_base;
}

static kk_unit_t kk_std_core_trace_any_fun21992(kk_function_t _fself, bool _y_17653, kk_context_t* _ctx) {
  struct kk_std_core_trace_any_fun21992__t* _self = kk_function_as(struct kk_std_core_trace_any_fun21992__t*, _fself);
  kk_string_t message0 = _self->message0; /* string */
  kk_box_t x = _self->x; /* 16302 */
  kk_drop_match(_self, {kk_string_dup(message0);kk_box_dup(x);}, {}, _ctx)
  if (_y_17653) {
    return kk_std_core_xtrace_any(message0, x, _ctx);
  }
  kk_string_drop(message0, _ctx);
  kk_box_drop(x, _ctx);
  return kk_Unit;
}


// lift anonymous function
struct kk_std_core_trace_any_fun21994__t {
  struct kk_function_s _base;
  kk_function_t _b_19759_19757;
};
static kk_box_t kk_std_core_trace_any_fun21994(kk_function_t _fself, kk_box_t _b_19758, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_trace_any_fun21994(kk_function_t _b_19759_19757, kk_context_t* _ctx) {
  struct kk_std_core_trace_any_fun21994__t* _self = kk_function_alloc_as(struct kk_std_core_trace_any_fun21994__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_trace_any_fun21994, kk_context());
  _self->_b_19759_19757 = _b_19759_19757;
  return &_self->_base;
}

static kk_box_t kk_std_core_trace_any_fun21994(kk_function_t _fself, kk_box_t _b_19758, kk_context_t* _ctx) {
  struct kk_std_core_trace_any_fun21994__t* _self = kk_function_as(struct kk_std_core_trace_any_fun21994__t*, _fself);
  kk_function_t _b_19759_19757 = _self->_b_19759_19757; /* (bool) -> <read<global>|_16279> () */
  kk_drop_match(_self, {kk_function_dup(_b_19759_19757);}, {}, _ctx)
  kk_unit_t _x21995 = kk_Unit;
  bool _x21996 = kk_bool_unbox(_b_19758); /*bool*/
  kk_function_call(kk_unit_t, (kk_function_t, bool, kk_context_t*), _b_19759_19757, (_b_19759_19757, _x21996, _ctx));
  return kk_unit_box(_x21995);
}

kk_unit_t kk_std_core_trace_any(kk_string_t message0, kk_box_t x, kk_context_t* _ctx) { /* forall<a> (message : string, x : a) -> () */ 
  bool x0_18268;
  kk_box_t _x21990;
  kk_ref_t _x21991 = kk_ref_dup(kk_std_core_trace_enabled); /*ref<global,bool>*/
  _x21990 = kk_ref_get(_x21991,kk_context()); /*182*/
  x0_18268 = kk_bool_unbox(_x21990); /*bool*/
  kk_function_t next0_18269 = kk_std_core_new_trace_any_fun21992(message0, x, _ctx); /*(bool) -> <read<global>|_16279> ()*/;
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19759_19757 = next0_18269; /*(bool) -> <read<global>|_16279> ()*/;
    kk_box_t _x21993 = kk_std_core_hnd_yield_extend(kk_std_core_new_trace_any_fun21994(_b_19759_19757, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x21993); return kk_Unit;
  }
  kk_function_call(kk_unit_t, (kk_function_t, bool, kk_context_t*), next0_18269, (next0_18269, x0_18268, _ctx)); return kk_Unit;
}
 
// Truncate a string to `count` characters.

kk_string_t kk_std_core_truncate(kk_string_t s, kk_integer_t count, kk_context_t* _ctx) { /* (s : string, count : int) -> string */ 
  kk_std_core__sslice _x21997;
  kk_std_core__sslice _ownedArg_19916;
  kk_std_core__sslice slice0 = kk_std_core_first1(s, _ctx); /*sslice*/;
  bool _match_19917;
  kk_integer_t _borrowedArg_19922;
  kk_std_core_types__optional _match_19923 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_19923)) {
    kk_box_t _box_x19760 = _match_19923._cons.Optional.value;
    kk_integer_t _n_10065 = kk_integer_unbox(_box_x19760);
    kk_integer_dup(_n_10065);
    kk_std_core_types__optional_drop(_match_19923, _ctx);
    kk_Unit;
    _borrowedArg_19922 = _n_10065; /*int*/
    goto _match21998;
  }
  _borrowedArg_19922 = kk_integer_from_small(1); /*int*/
  _match21998: ;
  kk_integer_t _borrowedArg_19921 = kk_integer_from_small(1); /*int*/;
  bool _borrowApp_19924 = kk_integer_eq_borrow(_borrowedArg_19922,_borrowedArg_19921,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_19922, _ctx);
  kk_integer_drop(_borrowedArg_19921, _ctx);
  _match_19917 = _borrowApp_19924; /*bool*/
  if (_match_19917) {
    _ownedArg_19916 = slice0; /*sslice*/
  }
  else {
    kk_integer_t _borrowedArg_19918;
    kk_integer_t _x22000;
    kk_std_core_types__optional _match_19919 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_19919)) {
      kk_box_t _box_x19761 = _match_19919._cons.Optional.value;
      kk_integer_t _n_100650 = kk_integer_unbox(_box_x19761);
      kk_integer_dup(_n_100650);
      kk_std_core_types__optional_drop(_match_19919, _ctx);
      kk_Unit;
      _x22000 = _n_100650; /*int*/
      goto _match22001;
    }
    _x22000 = kk_integer_from_small(1); /*int*/
    _match22001: ;
    _borrowedArg_19918 = kk_integer_sub(_x22000,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__sslice _borrowApp_19920 = kk_std_core_extend(slice0, _borrowedArg_19918, _ctx); /*sslice*/;
    kk_integer_drop(_borrowedArg_19918, _ctx);
    _ownedArg_19916 = _borrowApp_19920; /*sslice*/
  }
  kk_integer_t _borrowedArg_19915 = kk_integer_sub(count,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__sslice _borrowApp_19925 = kk_std_core_extend(_ownedArg_19916, _borrowedArg_19915, _ctx); /*sslice*/;
  kk_integer_drop(_borrowedArg_19915, _ctx);
  _x21997 = _borrowApp_19925; /*sslice*/
  return kk_std_core_string_3(_x21997, _ctx);
}
 
// Return a default value when an exception is raised


// lift anonymous function
struct kk_std_core_try_default_fun22005__t {
  struct kk_function_s _base;
  kk_box_t value;
};
static kk_box_t kk_std_core_try_default_fun22005(kk_function_t _fself, kk_std_core__exception ___wildcard__2564__19, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_default_fun22005(kk_box_t value, kk_context_t* _ctx) {
  struct kk_std_core_try_default_fun22005__t* _self = kk_function_alloc_as(struct kk_std_core_try_default_fun22005__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_default_fun22005, kk_context());
  _self->value = value;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_default_fun22005(kk_function_t _fself, kk_std_core__exception ___wildcard__2564__19, kk_context_t* _ctx) {
  struct kk_std_core_try_default_fun22005__t* _self = kk_function_as(struct kk_std_core_try_default_fun22005__t*, _fself);
  kk_box_t value = _self->value; /* 16378 */
  kk_drop_match(_self, {kk_box_dup(value);}, {}, _ctx)
  kk_std_core__exception_drop(___wildcard__2564__19, _ctx);
  return value;
}


// lift anonymous function
struct kk_std_core_try_default_fun22006__t {
  struct kk_function_s _base;
  kk_function_t _b_19764_19762;
};
static kk_box_t kk_std_core_try_default_fun22006(kk_function_t _fself, kk_box_t _b_19763, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_default_fun22006(kk_function_t _b_19764_19762, kk_context_t* _ctx) {
  struct kk_std_core_try_default_fun22006__t* _self = kk_function_alloc_as(struct kk_std_core_try_default_fun22006__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_try_default_fun22006, kk_context());
  _self->_b_19764_19762 = _b_19764_19762;
  return &_self->_base;
}

static kk_box_t kk_std_core_try_default_fun22006(kk_function_t _fself, kk_box_t _b_19763, kk_context_t* _ctx) {
  struct kk_std_core_try_default_fun22006__t* _self = kk_function_as(struct kk_std_core_try_default_fun22006__t*, _fself);
  kk_function_t _b_19764_19762 = _self->_b_19764_19762; /* (exception) -> 16379 16378 */
  kk_drop_match(_self, {kk_function_dup(_b_19764_19762);}, {}, _ctx)
  kk_std_core__exception _x22007 = kk_std_core__exception_unbox(_b_19763, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core__exception, kk_context_t*), _b_19764_19762, (_b_19764_19762, _x22007, _ctx));
}


// lift anonymous function
struct kk_std_core_try_default_fun22008__t {
  struct kk_function_s _base;
};
static kk_box_t kk_std_core_try_default_fun22008(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_try_default_fun22008(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_try_default_fun22008, _ctx)
  return kk_function_dup(_fself);
}

static kk_box_t kk_std_core_try_default_fun22008(kk_function_t _fself, kk_box_t _x, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _x;
}

kk_box_t kk_std_core_try_default(kk_box_t value, kk_function_t action, kk_context_t* _ctx) { /* forall<a,e> (value : a, action : () -> <exn|e> a) -> e a */ 
  kk_std_core__hnd_exn _x22003;
  kk_std_core_hnd__clause1 _x22004;
  kk_function_t _b_19764_19762 = kk_std_core_new_try_default_fun22005(value, _ctx); /*(exception) -> 16379 16378*/;
  _x22004 = kk_std_core_hnd_clause_never1(kk_std_core_new_try_default_fun22006(_b_19764_19762, _ctx), _ctx); /*std/core/hnd/clause1<8108,8109,8111,8110,8112>*/
  _x22003 = kk_std_core__new_Hnd_exn(kk_reuse_null, _x22004, _ctx); /*.hnd-exn<11,12>*/
  return kk_std_core__handle_exn(((int32_t)0), _x22003, kk_std_core_new_try_default_fun22008(_ctx), action, _ctx);
}
 
// Returns a unique integer (modulo 32-bits).


// lift anonymous function
struct kk_std_core_unique_fun22012__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_core_unique_fun22012(kk_function_t _fself, kk_integer_t u, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_unique_fun22012(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_unique_fun22012, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_core_unique_fun22012(kk_function_t _fself, kk_integer_t u, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_integer_t _b_19774_19772;
  kk_integer_t _x22013 = kk_integer_dup(u); /*int*/
  _b_19774_19772 = kk_integer_add(_x22013,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_unit_t __ = kk_Unit;
  kk_ref_set_borrow(kk_std_core_unique_count,(kk_integer_box(_b_19774_19772)),kk_context());
  return u;
}


// lift anonymous function
struct kk_std_core_unique_fun22015__t {
  struct kk_function_s _base;
  kk_function_t _b_19777_19775;
};
static kk_box_t kk_std_core_unique_fun22015(kk_function_t _fself, kk_box_t _b_19776, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_unique_fun22015(kk_function_t _b_19777_19775, kk_context_t* _ctx) {
  struct kk_std_core_unique_fun22015__t* _self = kk_function_alloc_as(struct kk_std_core_unique_fun22015__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_unique_fun22015, kk_context());
  _self->_b_19777_19775 = _b_19777_19775;
  return &_self->_base;
}

static kk_box_t kk_std_core_unique_fun22015(kk_function_t _fself, kk_box_t _b_19776, kk_context_t* _ctx) {
  struct kk_std_core_unique_fun22015__t* _self = kk_function_as(struct kk_std_core_unique_fun22015__t*, _fself);
  kk_function_t _b_19777_19775 = _self->_b_19777_19775; /* (int) -> <read<global>,write<global>|_16439> int */
  kk_drop_match(_self, {kk_function_dup(_b_19777_19775);}, {}, _ctx)
  kk_integer_t _x22016;
  kk_integer_t _x22017 = kk_integer_unbox(_b_19776); /*int*/
  _x22016 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), _b_19777_19775, (_b_19777_19775, _x22017, _ctx)); /*int*/
  return kk_integer_box(_x22016);
}

kk_integer_t kk_std_core_unique(kk_context_t* _ctx) { /* () -> ndet int */ 
  kk_integer_t x_18273;
  kk_box_t _x22010;
  kk_ref_t _x22011 = kk_ref_dup(kk_std_core_unique_count); /*ref<global,int>*/
  _x22010 = kk_ref_get(_x22011,kk_context()); /*182*/
  x_18273 = kk_integer_unbox(_x22010); /*int*/
  kk_function_t next0_18274 = kk_std_core_new_unique_fun22012(_ctx); /*(int) -> <read<global>,write<global>|_16439> int*/;
  if (kk_yielding(kk_context())) {
    kk_integer_drop(x_18273, _ctx);
    kk_function_t _b_19777_19775 = next0_18274; /*(int) -> <read<global>,write<global>|_16439> int*/;
    kk_box_t _x22014 = kk_std_core_hnd_yield_extend(kk_std_core_new_unique_fun22015(_b_19777_19775, _ctx), _ctx); /*3858*/
    return kk_integer_unbox(_x22014);
  }
  return kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), next0_18274, (next0_18274, x_18273, _ctx));
}
 
// Get the value of the `Just` constructor or raise an exception

kk_box_t kk_std_core_unjust(kk_std_core_types__maybe m, kk_context_t* _ctx) { /* forall<a> (m : maybe<a>) -> exn a */ 
  if (kk_std_core_types__is_Just(m)) {
    kk_box_t x = m._cons.Just.value;
    return x;
  }
  kk_std_core_hnd__ev ev_18276;
  kk_ssize_t _x22018 = ((kk_ssize_t)0); /*ssize_t*/
  ev_18276 = kk_evv_at(_x22018,kk_context()); /*std/core/hnd/ev<.hnd-exn>*/
  struct kk_std_core_hnd_Ev* _con22019 = kk_std_core_hnd__as_Ev(ev_18276);
  kk_std_core_hnd__marker m0 = _con22019->marker;
  kk_box_t _box_x19778 = _con22019->hnd;
  kk_std_core__hnd_exn h = kk_std_core__hnd_exn_unbox(_box_x19778, NULL);
  kk_std_core__hnd_exn_dup(h);
  kk_std_core_hnd__clause1 _match_19911;
  kk_std_core_hnd__clause1 _borrowApp_19913 = kk_std_core__select_throw_exn(h, _ctx); /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/;
  kk_std_core__hnd_exn_drop(h, _ctx);
  _match_19911 = _borrowApp_19913; /*std/core/hnd/clause1<exception,1926,.hnd-exn,1927,1928>*/
  kk_function_t _fun_unbox_x19782 = _match_19911.clause;
  kk_std_core_hnd__marker _b_19783 = m0; /*std/core/hnd/marker<3573,3574>*/;
  kk_std_core_hnd__ev _b_19784 = ev_18276; /*std/core/hnd/ev<.hnd-exn>*/;
  kk_std_core__exception _b_19785;
  kk_string_t _x22021;
  kk_define_string_literal(, _s22022, 37, "unexpected Nothing in std/core/unjust")
  _x22021 = kk_string_dup(_s22022); /*string*/
  kk_std_core__exception_info _x22023;
  kk_std_core_types__optional _match_19912 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_19912)) {
    kk_box_t _box_x19786 = _match_19912._cons.Optional.value;
    kk_std_core__exception_info _info_11639 = kk_std_core__exception_info_unbox(_box_x19786, NULL);
    kk_std_core__exception_info_dup(_info_11639);
    kk_std_core_types__optional_drop(_match_19912, _ctx);
    kk_Unit;
    _x22023 = _info_11639; /*exception-info*/
    goto _match22024;
  }
  _x22023 = kk_std_core__new_ExnError(_ctx); /*exception-info*/
  _match22024: ;
  _b_19785 = kk_std_core__new_Exception(_x22021, _x22023, _ctx); /*exception*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_box_t, kk_context_t*), _fun_unbox_x19782, (_fun_unbox_x19782, _b_19783, _b_19784, kk_std_core__exception_box(_b_19785, _ctx), _ctx));
}
 
// Join a list of strings with newlines

kk_string_t kk_std_core_unlines(kk_std_core__list xs, kk_context_t* _ctx) { /* (xs : list<string>) -> string */ 
  if (kk_std_core__is_Nil(xs)) {
    return kk_string_empty();
  }
  struct kk_std_core_Cons* _con22027 = kk_std_core__as_Cons(xs);
  kk_box_t _box_x19787 = _con22027->head;
  kk_std_core__list xx = _con22027->tail;
  kk_string_t x = kk_string_unbox(_box_x19787);
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_string_dup(x);
    kk_std_core__list_dup(xx);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  kk_string_t _x22029;
  kk_define_string_literal(, _s22030, 1, "\n")
  _x22029 = kk_string_dup(_s22030); /*string*/
  return kk_std_core__lift17169_joinsep(_x22029, xx, x, _ctx);
}
 
// lifted

kk_std_core__list kk_std_core__lift17206_unzip(kk_std_core__list acc, kk_std_core__list ys0, kk_context_t* _ctx) { /* forall<a> (acc : list<a>, ys0 : list<a>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys0)) {
    struct kk_std_core_Cons* _con22031 = kk_std_core__as_Cons(ys0);
    kk_box_t x0 = _con22031->head;
    kk_std_core__list xx0 = _con22031->tail;
    kk_reuse_t _ru_20428 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys0))) {
      _ru_20428 = (kk_std_core__list_reuse(ys0));
    }
    else {
      kk_box_dup(x0);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(ys0, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x22032;
      if (kk_likely(_ru_20428!=NULL)) {
        struct kk_std_core_Cons* _con22033 = (struct kk_std_core_Cons*)_ru_20428;
        _con22033->tail = acc;
        _x22032 = kk_std_core__base_Cons(_con22033); /*list<61>*/
      }
      else {
        _x22032 = kk_std_core__new_Cons(kk_reuse_null, x0, acc, _ctx); /*list<61>*/
      }
      acc = _x22032;
      ys0 = xx0;
      goto kk__tailcall;
    }
  }
  return acc;
}
 
// lifted

kk_std_core__list kk_std_core__lift17207_unzip(kk_std_core__list acc0, kk_std_core__list ys1, kk_context_t* _ctx) { /* forall<a> (acc0 : list<a>, ys1 : list<a>) -> list<a> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys1)) {
    struct kk_std_core_Cons* _con22034 = kk_std_core__as_Cons(ys1);
    kk_box_t x1 = _con22034->head;
    kk_std_core__list xx1 = _con22034->tail;
    kk_reuse_t _ru_20429 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys1))) {
      _ru_20429 = (kk_std_core__list_reuse(ys1));
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(ys1, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x22035;
      if (kk_likely(_ru_20429!=NULL)) {
        struct kk_std_core_Cons* _con22036 = (struct kk_std_core_Cons*)_ru_20429;
        _con22036->tail = acc0;
        _x22035 = kk_std_core__base_Cons(_con22036); /*list<61>*/
      }
      else {
        _x22035 = kk_std_core__new_Cons(kk_reuse_null, x1, acc0, _ctx); /*list<61>*/
      }
      acc0 = _x22035;
      ys1 = xx1;
      goto kk__tailcall;
    }
  }
  return acc0;
}
 
// lifted
// todo: implement TRMC for multiple results

kk_std_core_types__tuple2_ kk_std_core__lift17205_unzip(kk_std_core__list ys, kk_std_core__list acc1, kk_std_core__list acc2, kk_context_t* _ctx) { /* forall<a,b> (ys : list<(a, b)>, acc1 : list<a>, acc2 : list<b>) -> (list<a>, list<b>) */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(ys)) {
    struct kk_std_core_Cons* _con22037 = kk_std_core__as_Cons(ys);
    kk_box_t _box_x19788 = _con22037->head;
    kk_std_core__list xx = _con22037->tail;
    kk_std_core_types__tuple2_ _pat0 = kk_std_core_types__tuple2__unbox(_box_x19788, NULL);
    kk_box_t x = _pat0.fst;
    kk_box_t y = _pat0.snd;
    kk_reuse_t _ru_20430 = kk_reuse_null; /*reuse*/;
    if (kk_likely(kk_std_core__list_is_unique(ys))) {
      kk_box_dup(x);
      kk_box_dup(y);
      kk_box_drop(_box_x19788, _ctx);
      kk_Unit;
      kk_Unit;
      _ru_20430 = (kk_std_core__list_reuse(ys));
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_box_dup(y);
      kk_Unit;
      kk_std_core__list_decref(ys, _ctx);
      kk_Unit;
    }
    { // tailcall
      kk_std_core__list _x22039 = kk_std_core__new_Cons(_ru_20430, x, acc1, _ctx); /*list<61>*/
      kk_std_core__list _x22040 = kk_std_core__new_Cons(kk_reuse_null, y, acc2, _ctx); /*list<61>*/
      ys = xx;
      acc1 = _x22039;
      acc2 = _x22040;
      goto kk__tailcall;
    }
  }
  kk_std_core__list _b_19791_19789 = kk_std_core__lift17206_unzip(kk_std_core__new_Nil(_ctx), acc1, _ctx); /*list<16507>*/;
  kk_std_core__list _b_19792_19790 = kk_std_core__lift17207_unzip(kk_std_core__new_Nil(_ctx), acc2, _ctx); /*list<16508>*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_core__list_box(_b_19791_19789, _ctx), kk_std_core__list_box(_b_19792_19790, _ctx), _ctx);
}
 
// Convert a string to a vector of characters.

kk_vector_t kk_std_core_vector_1(kk_string_t s, kk_context_t* _ctx) { /* (s : string) -> vector<char> */ 
  return kk_string_to_chars(s,kk_context());
}
extern kk_box_t kk_std_core_vector_fun22043_2(kk_function_t _fself, kk_ssize_t ___wildcard__2076__30, kk_context_t* _ctx) {
  struct kk_std_core_vector_fun22043__t_2* _self = kk_function_as(struct kk_std_core_vector_fun22043__t_2*, _fself);
  kk_box_t default0 = _self->default0; /* 16584 */
  kk_drop_match(_self, {kk_box_dup(default0);}, {}, _ctx)
  return default0;
}
extern kk_box_t kk_std_core_vector_init_fun22046(kk_function_t _fself, kk_ssize_t i, kk_context_t* _ctx) {
  struct kk_std_core_vector_init_fun22046__t* _self = kk_function_as(struct kk_std_core_vector_init_fun22046__t*, _fself);
  kk_function_t f = _self->f; /* (int) -> 16651 */
  kk_drop_match(_self, {kk_function_dup(f);}, {}, _ctx)
  kk_integer_t _x22047 = kk_integer_from_ssize_t(i,kk_context()); /*int*/
  return kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_context_t*), f, (f, _x22047, _ctx));
}
 
// monadic lift

kk_unit_t kk_std_core__mlift17770_while(kk_function_t action, kk_function_t predicate, kk_unit_t wild__, kk_context_t* _ctx) { /* forall<e> (action : () -> <div|e> (), predicate : () -> <div|e> bool, wild_ : ()) -> <div|e> () */ 
  kk_std_core_while(predicate, action, _ctx); return kk_Unit;
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17771_while_fun22049__t {
  struct kk_function_s _base;
  kk_function_t action0;
  kk_function_t predicate0;
};
static kk_unit_t kk_std_core__mlift17771_while_fun22049(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17771_while_fun22049(kk_function_t action0, kk_function_t predicate0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17771_while_fun22049__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17771_while_fun22049__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17771_while_fun22049, kk_context());
  _self->action0 = action0;
  _self->predicate0 = predicate0;
  return &_self->_base;
}

static kk_unit_t kk_std_core__mlift17771_while_fun22049(kk_function_t _fself, kk_unit_t wild__0, kk_context_t* _ctx) {
  struct kk_std_core__mlift17771_while_fun22049__t* _self = kk_function_as(struct kk_std_core__mlift17771_while_fun22049__t*, _fself);
  kk_function_t action0 = _self->action0; /* () -> <div|16672> () */
  kk_function_t predicate0 = _self->predicate0; /* () -> <div|16672> bool */
  kk_drop_match(_self, {kk_function_dup(action0);kk_function_dup(predicate0);}, {}, _ctx)
  return kk_std_core__mlift17770_while(action0, predicate0, wild__0, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17771_while_fun22051__t {
  struct kk_function_s _base;
  kk_function_t _b_19795_19793;
};
static kk_box_t kk_std_core__mlift17771_while_fun22051(kk_function_t _fself, kk_box_t _b_19794, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17771_while_fun22051(kk_function_t _b_19795_19793, kk_context_t* _ctx) {
  struct kk_std_core__mlift17771_while_fun22051__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17771_while_fun22051__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17771_while_fun22051, kk_context());
  _self->_b_19795_19793 = _b_19795_19793;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17771_while_fun22051(kk_function_t _fself, kk_box_t _b_19794, kk_context_t* _ctx) {
  struct kk_std_core__mlift17771_while_fun22051__t* _self = kk_function_as(struct kk_std_core__mlift17771_while_fun22051__t*, _fself);
  kk_function_t _b_19795_19793 = _self->_b_19795_19793; /* (wild_0 : ()) -> <div|16672> () */
  kk_drop_match(_self, {kk_function_dup(_b_19795_19793);}, {}, _ctx)
  kk_unit_t _x22052 = kk_Unit;
  kk_unit_t _x22053 = kk_Unit;
  kk_unit_unbox(_b_19794);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19795_19793, (_b_19795_19793, _x22053, _ctx));
  return kk_unit_box(_x22052);
}

kk_unit_t kk_std_core__mlift17771_while(kk_function_t action0, kk_function_t predicate0, bool _y_17659, kk_context_t* _ctx) { /* forall<e> (action : () -> <div|e> (), predicate : () -> <div|e> bool, bool) -> <div|e> () */ 
  if (_y_17659) {
    kk_unit_t x_18281 = kk_Unit;
    kk_function_t _x22048 = kk_function_dup(action0); /*() -> <div|16672> ()*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_context_t*), _x22048, (_x22048, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19795_19793 = kk_std_core__new_mlift17771_while_fun22049(action0, predicate0, _ctx); /*(wild_0 : ()) -> <div|16672> ()*/;
      kk_box_t _x22050 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17771_while_fun22051(_b_19795_19793, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x22050); return kk_Unit;
    }
    kk_std_core__mlift17770_while(action0, predicate0, x_18281, _ctx); return kk_Unit;
  }
  kk_function_drop(action0, _ctx);
  kk_function_drop(predicate0, _ctx);
  kk_Unit; return kk_Unit;
}
 
// The `while` fun executes `action`  as long as `pred`  is `true`.


// lift anonymous function
struct kk_std_core_while_fun22055__t {
  struct kk_function_s _base;
  kk_function_t action1;
  kk_function_t predicate1;
};
static kk_unit_t kk_std_core_while_fun22055(kk_function_t _fself, bool _y_176590, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_while_fun22055(kk_function_t action1, kk_function_t predicate1, kk_context_t* _ctx) {
  struct kk_std_core_while_fun22055__t* _self = kk_function_alloc_as(struct kk_std_core_while_fun22055__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_while_fun22055, kk_context());
  _self->action1 = action1;
  _self->predicate1 = predicate1;
  return &_self->_base;
}

static kk_unit_t kk_std_core_while_fun22055(kk_function_t _fself, bool _y_176590, kk_context_t* _ctx) {
  struct kk_std_core_while_fun22055__t* _self = kk_function_as(struct kk_std_core_while_fun22055__t*, _fself);
  kk_function_t action1 = _self->action1; /* () -> <div|16672> () */
  kk_function_t predicate1 = _self->predicate1; /* () -> <div|16672> bool */
  kk_drop_match(_self, {kk_function_dup(action1);kk_function_dup(predicate1);}, {}, _ctx)
  return kk_std_core__mlift17771_while(action1, predicate1, _y_176590, _ctx);
}


// lift anonymous function
struct kk_std_core_while_fun22057__t {
  struct kk_function_s _base;
  kk_function_t _b_19800_19796;
};
static kk_box_t kk_std_core_while_fun22057(kk_function_t _fself, kk_box_t _b_19797, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_while_fun22057(kk_function_t _b_19800_19796, kk_context_t* _ctx) {
  struct kk_std_core_while_fun22057__t* _self = kk_function_alloc_as(struct kk_std_core_while_fun22057__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_while_fun22057, kk_context());
  _self->_b_19800_19796 = _b_19800_19796;
  return &_self->_base;
}

static kk_box_t kk_std_core_while_fun22057(kk_function_t _fself, kk_box_t _b_19797, kk_context_t* _ctx) {
  struct kk_std_core_while_fun22057__t* _self = kk_function_as(struct kk_std_core_while_fun22057__t*, _fself);
  kk_function_t _b_19800_19796 = _self->_b_19800_19796; /* (bool) -> <div|16672> () */
  kk_drop_match(_self, {kk_function_dup(_b_19800_19796);}, {}, _ctx)
  kk_unit_t _x22058 = kk_Unit;
  bool _x22059 = kk_bool_unbox(_b_19797); /*bool*/
  kk_function_call(kk_unit_t, (kk_function_t, bool, kk_context_t*), _b_19800_19796, (_b_19800_19796, _x22059, _ctx));
  return kk_unit_box(_x22058);
}


// lift anonymous function
struct kk_std_core_while_fun22061__t {
  struct kk_function_s _base;
  kk_function_t action1;
  kk_function_t predicate1;
};
static kk_unit_t kk_std_core_while_fun22061(kk_function_t _fself, kk_unit_t wild__1, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_while_fun22061(kk_function_t action1, kk_function_t predicate1, kk_context_t* _ctx) {
  struct kk_std_core_while_fun22061__t* _self = kk_function_alloc_as(struct kk_std_core_while_fun22061__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_while_fun22061, kk_context());
  _self->action1 = action1;
  _self->predicate1 = predicate1;
  return &_self->_base;
}

static kk_unit_t kk_std_core_while_fun22061(kk_function_t _fself, kk_unit_t wild__1, kk_context_t* _ctx) {
  struct kk_std_core_while_fun22061__t* _self = kk_function_as(struct kk_std_core_while_fun22061__t*, _fself);
  kk_function_t action1 = _self->action1; /* () -> <div|16672> () */
  kk_function_t predicate1 = _self->predicate1; /* () -> <div|16672> bool */
  kk_drop_match(_self, {kk_function_dup(action1);kk_function_dup(predicate1);}, {}, _ctx)
  return kk_std_core__mlift17770_while(action1, predicate1, wild__1, _ctx);
}


// lift anonymous function
struct kk_std_core_while_fun22063__t {
  struct kk_function_s _base;
  kk_function_t _b_19801_19798;
};
static kk_box_t kk_std_core_while_fun22063(kk_function_t _fself, kk_box_t _b_19799, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_while_fun22063(kk_function_t _b_19801_19798, kk_context_t* _ctx) {
  struct kk_std_core_while_fun22063__t* _self = kk_function_alloc_as(struct kk_std_core_while_fun22063__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_while_fun22063, kk_context());
  _self->_b_19801_19798 = _b_19801_19798;
  return &_self->_base;
}

static kk_box_t kk_std_core_while_fun22063(kk_function_t _fself, kk_box_t _b_19799, kk_context_t* _ctx) {
  struct kk_std_core_while_fun22063__t* _self = kk_function_as(struct kk_std_core_while_fun22063__t*, _fself);
  kk_function_t _b_19801_19798 = _self->_b_19801_19798; /* (wild_1 : ()) -> <div|16672> () */
  kk_drop_match(_self, {kk_function_dup(_b_19801_19798);}, {}, _ctx)
  kk_unit_t _x22064 = kk_Unit;
  kk_unit_t _x22065 = kk_Unit;
  kk_unit_unbox(_b_19799);
  kk_function_call(kk_unit_t, (kk_function_t, kk_unit_t, kk_context_t*), _b_19801_19798, (_b_19801_19798, _x22065, _ctx));
  return kk_unit_box(_x22064);
}

kk_unit_t kk_std_core_while(kk_function_t predicate1, kk_function_t action1, kk_context_t* _ctx) { /* forall<e> (predicate : () -> <div|e> bool, action : () -> <div|e> ()) -> <div|e> () */ 
  kk__tailcall: ;
  bool x0_18283;
  kk_function_t _x22054 = kk_function_dup(predicate1); /*() -> <div|16672> bool*/
  x0_18283 = kk_function_call(bool, (kk_function_t, kk_context_t*), _x22054, (_x22054, _ctx)); /*bool*/
  if (kk_yielding(kk_context())) {
    kk_function_t _b_19800_19796 = kk_std_core_new_while_fun22055(action1, predicate1, _ctx); /*(bool) -> <div|16672> ()*/;
    kk_box_t _x22056 = kk_std_core_hnd_yield_extend(kk_std_core_new_while_fun22057(_b_19800_19796, _ctx), _ctx); /*3858*/
    kk_unit_unbox(_x22056); return kk_Unit;
  }
  if (x0_18283) {
    kk_unit_t x1_18286 = kk_Unit;
    kk_function_t _x22060 = kk_function_dup(action1); /*() -> <div|16672> ()*/
    kk_function_call(kk_unit_t, (kk_function_t, kk_context_t*), _x22060, (_x22060, _ctx));
    if (kk_yielding(kk_context())) {
      kk_function_t _b_19801_19798 = kk_std_core_new_while_fun22061(action1, predicate1, _ctx); /*(wild_1 : ()) -> <div|16672> ()*/;
      kk_box_t _x22062 = kk_std_core_hnd_yield_extend(kk_std_core_new_while_fun22063(_b_19801_19798, _ctx), _ctx); /*3858*/
      kk_unit_unbox(_x22062); return kk_Unit;
    }
    { // tailcall
      goto kk__tailcall;
    }
  }
  kk_function_drop(action1, _ctx);
  kk_function_drop(predicate1, _ctx);
  kk_Unit; return kk_Unit;
}
 
// Zip two lists together by pairing the corresponding elements.
// The returned list is only as long as the smallest input list.

kk_std_core__list kk_std_core__ctail_zip(kk_std_core__list xs, kk_std_core__list ys, kk_std_core_types__ctail _acc, kk_context_t* _ctx) { /* forall<a,b> (xs : list<a>, ys : list<b>, ctail<list<(a, b)>>) -> list<(a, b)> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con22066 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con22066->head;
    kk_std_core__list xx = _con22066->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    if (kk_std_core__is_Cons(ys)) {
      struct kk_std_core_Cons* _con22067 = kk_std_core__as_Cons(ys);
      kk_box_t y = _con22067->head;
      kk_std_core__list yy = _con22067->tail;
      kk_reuse_t _ru_20432 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(ys))) {
        _ru_20432 = (kk_std_core__list_reuse(ys));
      }
      else {
        kk_box_dup(y);
        kk_std_core__list_dup(yy);
        kk_Unit;
        kk_std_core__list_decref(ys, _ctx);
        kk_Unit;
      }
      kk_std_core__list _ctail_17365 = kk_std_core__list_hole(); /*list<(16715, 16716)>*/;
      kk_std_core__list _ctail_17366;
      kk_box_t _x22068;
      kk_std_core_types__tuple2_ _x22069 = kk_std_core_types__new_dash__lp__comma__rp_(x, y, _ctx); /*(6, 7)*/
      _x22068 = kk_std_core_types__tuple2__box(_x22069, _ctx); /*61*/
      _ctail_17366 = kk_std_core__new_Cons(_ru_20432, _x22068, _ctail_17365, _ctx); /*list<(16715, 16716)>*/
      { // tailcall
        kk_std_core_types__ctail _x22070;
        kk_box_t* field_18291 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17366)->tail)); /*cfield<list<(16715, 16716)>>*/;
        if (kk_std_core_types__is_CTail(_acc)) {
          kk_box_t _box_x19812 = _acc._cons.CTail._field1;
          kk_box_t* last0 = _acc._cons.CTail._field2;
          kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19812, NULL);
          kk_std_core__list_dup(head0);
          kk_std_core_types__ctail_drop(_acc, _ctx);
          kk_Unit;
          kk_unit_t __ = kk_Unit;
          *(last0) = kk_std_core__list_box(_ctail_17366, _ctx);
          kk_std_core__list _b_19831_19817 = head0; /*list<(16715, 16716)>*/;
          kk_box_t* _b_19832_19818 = field_18291; /*cfield<list<(16715, 16716)>>*/;
          _x22070 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19831_19817, _ctx), _b_19832_19818, _ctx); /*ctail<58>*/
          goto _match22071;
        }
        kk_std_core__list _b_19833_19819 = _ctail_17366; /*list<(16715, 16716)>*/;
        kk_box_t* _b_19834_19820 = field_18291; /*cfield<list<(16715, 16716)>>*/;
        _x22070 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19833_19819, _ctx), _b_19834_19820, _ctx); /*ctail<58>*/
        _match22071: ;
        xs = xx;
        ys = yy;
        _acc = _x22070;
        goto kk__tailcall;
      }
    }
    kk_box_drop(x, _ctx);
    kk_std_core__list_drop(xx, _ctx);
    if (kk_std_core_types__is_CTail(_acc)) {
      kk_box_t _box_x19821 = _acc._cons.CTail._field1;
      kk_box_t* last2 = _acc._cons.CTail._field2;
      kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19821, NULL);
      kk_std_core__list_dup(head1);
      kk_std_core_types__ctail_drop(_acc, _ctx);
      kk_Unit;
      kk_unit_t __0 = kk_Unit;
      *(last2) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
      return head1;
    }
    return kk_std_core__new_Nil(_ctx);
  }
  kk_std_core__list_drop(ys, _ctx);
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19826 = _acc._cons.CTail._field1;
    kk_box_t* last3 = _acc._cons.CTail._field2;
    kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19826, NULL);
    kk_std_core__list_dup(head2);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __1 = kk_Unit;
    *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head2;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Zip two lists together by pairing the corresponding elements.
// The returned list is only as long as the smallest input list.

kk_std_core__list kk_std_core_zip(kk_std_core__list xs0, kk_std_core__list ys0, kk_context_t* _ctx) { /* forall<a,b> (xs : list<a>, ys : list<b>) -> list<(a, b)> */ 
  return kk_std_core__ctail_zip(xs0, ys0, kk_std_core_types__new_CTailNil(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17772_op(kk_std_core_types__ctail _acc, kk_function_t f, kk_std_core__list xx, kk_std_core__list yy, kk_box_t _ctail_17367, kk_context_t* _ctx) { /* forall<a,b,c,e> (ctail<list<c>>, f : (a, b) -> e c, xx : list<a>, yy : list<b>, c) -> e list<c> */ 
  kk_std_core__list _ctail_17368 = kk_std_core__list_hole(); /*list<16763>*/;
  kk_std_core__list _ctail_17369 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17367, _ctail_17368, _ctx); /*list<16763>*/;
  kk_std_core_types__ctail _x22075;
  kk_box_t* field_18298 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_17369)->tail)); /*cfield<list<16763>>*/;
  if (kk_std_core_types__is_CTail(_acc)) {
    kk_box_t _box_x19841 = _acc._cons.CTail._field1;
    kk_box_t* last0 = _acc._cons.CTail._field2;
    kk_std_core__list head0 = kk_std_core__list_unbox(_box_x19841, NULL);
    kk_std_core__list_dup(head0);
    kk_std_core_types__ctail_drop(_acc, _ctx);
    kk_Unit;
    kk_unit_t __ = kk_Unit;
    *(last0) = kk_std_core__list_box(_ctail_17369, _ctx);
    kk_std_core__list _b_19850_19846 = head0; /*list<16763>*/;
    kk_box_t* _b_19851_19847 = field_18298; /*cfield<list<16763>>*/;
    _x22075 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19850_19846, _ctx), _b_19851_19847, _ctx); /*ctail<58>*/
    goto _match22076;
  }
  kk_std_core__list _b_19852_19848 = _ctail_17369; /*list<16763>*/;
  kk_box_t* _b_19853_19849 = field_18298; /*cfield<list<16763>>*/;
  _x22075 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19852_19848, _ctx), _b_19853_19849, _ctx); /*ctail<58>*/
  _match22076: ;
  return kk_std_core__ctail_zipwith(xx, yy, f, _x22075, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17773_op_fun22078__t {
  struct kk_function_s _base;
  kk_function_t _accm;
  kk_box_t _ctail_17372;
};
static kk_std_core__list kk_std_core__mlift17773_op_fun22078(kk_function_t _fself, kk_std_core__list _ctail_17371, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17773_op_fun22078(kk_function_t _accm, kk_box_t _ctail_17372, kk_context_t* _ctx) {
  struct kk_std_core__mlift17773_op_fun22078__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17773_op_fun22078__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17773_op_fun22078, kk_context());
  _self->_accm = _accm;
  _self->_ctail_17372 = _ctail_17372;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17773_op_fun22078(kk_function_t _fself, kk_std_core__list _ctail_17371, kk_context_t* _ctx) {
  struct kk_std_core__mlift17773_op_fun22078__t* _self = kk_function_as(struct kk_std_core__mlift17773_op_fun22078__t*, _fself);
  kk_function_t _accm = _self->_accm; /* (list<16763>) -> list<16763> */
  kk_box_t _ctail_17372 = _self->_ctail_17372; /* 16763 */
  kk_drop_match(_self, {kk_function_dup(_accm);kk_box_dup(_ctail_17372);}, {}, _ctx)
  kk_std_core__list _x22079 = kk_std_core__new_Cons(kk_reuse_null, _ctail_17372, _ctail_17371, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm, (_accm, _x22079, _ctx));
}

kk_std_core__list kk_std_core__mlift17773_op(kk_function_t _accm, kk_function_t f0, kk_std_core__list xx0, kk_std_core__list yy0, kk_box_t _ctail_17372, kk_context_t* _ctx) { /* forall<a,b,c,e> ((list<c>) -> list<c>, f : (a, b) -> e c, xx : list<a>, yy : list<b>, c) -> e list<c> */ 
  return kk_std_core__ctailm_zipwith(xx0, yy0, f0, kk_std_core__new_mlift17773_op_fun22078(_accm, _ctail_17372, _ctx), _ctx);
}
 
// Zip two lists together by apply a function `f` to all corresponding elements.
// The returned list is only as long as the smallest input list.


// lift anonymous function
struct kk_std_core__ctail_zipwith_fun22083__t {
  struct kk_function_s _base;
  kk_function_t f1;
  kk_std_core__list xx1;
  kk_std_core__list yy1;
  kk_std_core_types__ctail _acc0;
};
static kk_std_core__list kk_std_core__ctail_zipwith_fun22083(kk_function_t _fself, kk_box_t _ctail_173670, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_zipwith_fun22083(kk_function_t f1, kk_std_core__list xx1, kk_std_core__list yy1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) {
  struct kk_std_core__ctail_zipwith_fun22083__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_zipwith_fun22083__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_zipwith_fun22083, kk_context());
  _self->f1 = f1;
  _self->xx1 = xx1;
  _self->yy1 = yy1;
  _self->_acc0 = _acc0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctail_zipwith_fun22083(kk_function_t _fself, kk_box_t _ctail_173670, kk_context_t* _ctx) {
  struct kk_std_core__ctail_zipwith_fun22083__t* _self = kk_function_as(struct kk_std_core__ctail_zipwith_fun22083__t*, _fself);
  kk_function_t f1 = _self->f1; /* (16761, 16762) -> 16764 16763 */
  kk_std_core__list xx1 = _self->xx1; /* list<16761> */
  kk_std_core__list yy1 = _self->yy1; /* list<16762> */
  kk_std_core_types__ctail _acc0 = _self->_acc0; /* ctail<list<16763>> */
  kk_drop_match(_self, {kk_function_dup(f1);kk_std_core__list_dup(xx1);kk_std_core__list_dup(yy1);kk_std_core_types__ctail_dup(_acc0);}, {}, _ctx)
  return kk_std_core__mlift17772_op(_acc0, f1, xx1, yy1, _ctail_173670, _ctx);
}


// lift anonymous function
struct kk_std_core__ctail_zipwith_fun22085__t {
  struct kk_function_s _base;
  kk_function_t _b_19881_19854;
};
static kk_box_t kk_std_core__ctail_zipwith_fun22085(kk_function_t _fself, kk_box_t _b_19855, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctail_zipwith_fun22085(kk_function_t _b_19881_19854, kk_context_t* _ctx) {
  struct kk_std_core__ctail_zipwith_fun22085__t* _self = kk_function_alloc_as(struct kk_std_core__ctail_zipwith_fun22085__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctail_zipwith_fun22085, kk_context());
  _self->_b_19881_19854 = _b_19881_19854;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctail_zipwith_fun22085(kk_function_t _fself, kk_box_t _b_19855, kk_context_t* _ctx) {
  struct kk_std_core__ctail_zipwith_fun22085__t* _self = kk_function_as(struct kk_std_core__ctail_zipwith_fun22085__t*, _fself);
  kk_function_t _b_19881_19854 = _self->_b_19881_19854; /* (16763) -> 16764 list<16763> */
  kk_drop_match(_self, {kk_function_dup(_b_19881_19854);}, {}, _ctx)
  kk_std_core__list _x22086 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19881_19854, (_b_19881_19854, _b_19855, _ctx)); /*list<16763>*/
  return kk_std_core__list_box(_x22086, _ctx);
}

kk_std_core__list kk_std_core__ctail_zipwith(kk_std_core__list xs, kk_std_core__list ys, kk_function_t f1, kk_std_core_types__ctail _acc0, kk_context_t* _ctx) { /* forall<a,b,c,e> (xs : list<a>, ys : list<b>, f : (a, b) -> e c, ctail<list<c>>) -> e list<c> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con22080 = kk_std_core__as_Cons(xs);
    kk_box_t x = _con22080->head;
    kk_std_core__list xx1 = _con22080->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x);
      kk_std_core__list_dup(xx1);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    if (kk_std_core__is_Cons(ys)) {
      struct kk_std_core_Cons* _con22081 = kk_std_core__as_Cons(ys);
      kk_box_t y = _con22081->head;
      kk_std_core__list yy1 = _con22081->tail;
      kk_reuse_t _ru_20434 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(ys))) {
        _ru_20434 = (kk_std_core__list_reuse(ys));
      }
      else {
        kk_box_dup(y);
        kk_std_core__list_dup(yy1);
        kk_Unit;
        kk_std_core__list_decref(ys, _ctx);
        kk_Unit;
      }
      kk_box_t x0_18299;
      kk_function_t _x22082 = kk_function_dup(f1); /*(16761, 16762) -> 16764 16763*/
      x0_18299 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x22082, (_x22082, x, y, _ctx)); /*16763*/
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_20434);
        kk_box_drop(x0_18299, _ctx);
        kk_function_t _b_19881_19854 = kk_std_core__new_ctail_zipwith_fun22083(f1, xx1, yy1, _acc0, _ctx); /*(16763) -> 16764 list<16763>*/;
        kk_box_t _x22084 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctail_zipwith_fun22085(_b_19881_19854, _ctx), _ctx); /*3858*/
        return kk_std_core__list_unbox(_x22084, _ctx);
      }
      kk_std_core__list _ctail_173680 = kk_std_core__list_hole(); /*list<16763>*/;
      kk_std_core__list _ctail_173690 = kk_std_core__new_Cons(_ru_20434, x0_18299, _ctail_173680, _ctx); /*list<16763>*/;
      { // tailcall
        kk_std_core_types__ctail _x22087;
        kk_box_t* field0_18304 = (kk_box_t*)((&kk_std_core__as_Cons(_ctail_173690)->tail)); /*cfield<list<16763>>*/;
        if (kk_std_core_types__is_CTail(_acc0)) {
          kk_box_t _box_x19862 = _acc0._cons.CTail._field1;
          kk_box_t* last2 = _acc0._cons.CTail._field2;
          kk_std_core__list head1 = kk_std_core__list_unbox(_box_x19862, NULL);
          kk_std_core__list_dup(head1);
          kk_std_core_types__ctail_drop(_acc0, _ctx);
          kk_Unit;
          kk_unit_t __0 = kk_Unit;
          *(last2) = kk_std_core__list_box(_ctail_173690, _ctx);
          kk_std_core__list _b_19882_19867 = head1; /*list<16763>*/;
          kk_box_t* _b_19883_19868 = field0_18304; /*cfield<list<16763>>*/;
          _x22087 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19882_19867, _ctx), _b_19883_19868, _ctx); /*ctail<58>*/
          goto _match22088;
        }
        kk_std_core__list _b_19884_19869 = _ctail_173690; /*list<16763>*/;
        kk_box_t* _b_19885_19870 = field0_18304; /*cfield<list<16763>>*/;
        _x22087 = kk_std_core_types__new_CTail(kk_std_core__list_box(_b_19884_19869, _ctx), _b_19885_19870, _ctx); /*ctail<58>*/
        _match22088: ;
        xs = xx1;
        ys = yy1;
        _acc0 = _x22087;
        goto kk__tailcall;
      }
    }
    kk_function_drop(f1, _ctx);
    kk_box_drop(x, _ctx);
    kk_std_core__list_drop(xx1, _ctx);
    if (kk_std_core_types__is_CTail(_acc0)) {
      kk_box_t _box_x19871 = _acc0._cons.CTail._field1;
      kk_box_t* last3 = _acc0._cons.CTail._field2;
      kk_std_core__list head2 = kk_std_core__list_unbox(_box_x19871, NULL);
      kk_std_core__list_dup(head2);
      kk_std_core_types__ctail_drop(_acc0, _ctx);
      kk_Unit;
      kk_unit_t __1 = kk_Unit;
      *(last3) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
      return head2;
    }
    return kk_std_core__new_Nil(_ctx);
  }
  kk_function_drop(f1, _ctx);
  kk_std_core__list_drop(ys, _ctx);
  if (kk_std_core_types__is_CTail(_acc0)) {
    kk_box_t _box_x19876 = _acc0._cons.CTail._field1;
    kk_box_t* last4 = _acc0._cons.CTail._field2;
    kk_std_core__list head3 = kk_std_core__list_unbox(_box_x19876, NULL);
    kk_std_core__list_dup(head3);
    kk_std_core_types__ctail_drop(_acc0, _ctx);
    kk_Unit;
    kk_unit_t __2 = kk_Unit;
    *(last4) = kk_std_core__list_box(kk_std_core__new_Nil(_ctx), _ctx);
    return head3;
  }
  return kk_std_core__new_Nil(_ctx);
}
 
// Zip two lists together by apply a function `f` to all corresponding elements.
// The returned list is only as long as the smallest input list.


// lift anonymous function
struct kk_std_core__ctailm_zipwith_fun22095__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_function_t f2;
  kk_std_core__list xx2;
  kk_std_core__list yy2;
};
static kk_std_core__list kk_std_core__ctailm_zipwith_fun22095(kk_function_t _fself, kk_box_t _ctail_173720, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_zipwith_fun22095(kk_function_t _accm0, kk_function_t f2, kk_std_core__list xx2, kk_std_core__list yy2, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun22095__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_zipwith_fun22095__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_zipwith_fun22095, kk_context());
  _self->_accm0 = _accm0;
  _self->f2 = f2;
  _self->xx2 = xx2;
  _self->yy2 = yy2;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_zipwith_fun22095(kk_function_t _fself, kk_box_t _ctail_173720, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun22095__t* _self = kk_function_as(struct kk_std_core__ctailm_zipwith_fun22095__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<16763>) -> list<16763> */
  kk_function_t f2 = _self->f2; /* (16761, 16762) -> 16764 16763 */
  kk_std_core__list xx2 = _self->xx2; /* list<16761> */
  kk_std_core__list yy2 = _self->yy2; /* list<16762> */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_function_dup(f2);kk_std_core__list_dup(xx2);kk_std_core__list_dup(yy2);}, {}, _ctx)
  return kk_std_core__mlift17773_op(_accm0, f2, xx2, yy2, _ctail_173720, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_zipwith_fun22097__t {
  struct kk_function_s _base;
  kk_function_t _b_19888_19886;
};
static kk_box_t kk_std_core__ctailm_zipwith_fun22097(kk_function_t _fself, kk_box_t _b_19887, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_zipwith_fun22097(kk_function_t _b_19888_19886, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun22097__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_zipwith_fun22097__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_zipwith_fun22097, kk_context());
  _self->_b_19888_19886 = _b_19888_19886;
  return &_self->_base;
}

static kk_box_t kk_std_core__ctailm_zipwith_fun22097(kk_function_t _fself, kk_box_t _b_19887, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun22097__t* _self = kk_function_as(struct kk_std_core__ctailm_zipwith_fun22097__t*, _fself);
  kk_function_t _b_19888_19886 = _self->_b_19888_19886; /* (16763) -> 16764 list<16763> */
  kk_drop_match(_self, {kk_function_dup(_b_19888_19886);}, {}, _ctx)
  kk_std_core__list _x22098 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19888_19886, (_b_19888_19886, _b_19887, _ctx)); /*list<16763>*/
  return kk_std_core__list_box(_x22098, _ctx);
}


// lift anonymous function
struct kk_std_core__ctailm_zipwith_fun22100__t {
  struct kk_function_s _base;
  kk_function_t _accm0;
  kk_box_t x2_18309;
};
static kk_std_core__list kk_std_core__ctailm_zipwith_fun22100(kk_function_t _fself, kk_std_core__list _ctail_173710, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_ctailm_zipwith_fun22100(kk_function_t _accm0, kk_box_t x2_18309, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun22100__t* _self = kk_function_alloc_as(struct kk_std_core__ctailm_zipwith_fun22100__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__ctailm_zipwith_fun22100, kk_context());
  _self->_accm0 = _accm0;
  _self->x2_18309 = x2_18309;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__ctailm_zipwith_fun22100(kk_function_t _fself, kk_std_core__list _ctail_173710, kk_context_t* _ctx) {
  struct kk_std_core__ctailm_zipwith_fun22100__t* _self = kk_function_as(struct kk_std_core__ctailm_zipwith_fun22100__t*, _fself);
  kk_function_t _accm0 = _self->_accm0; /* (list<16763>) -> list<16763> */
  kk_box_t x2_18309 = _self->x2_18309; /* 16763 */
  kk_drop_match(_self, {kk_function_dup(_accm0);kk_box_dup(x2_18309);}, {}, _ctx)
  kk_std_core__list _x22101 = kk_std_core__new_Cons(kk_reuse_null, x2_18309, _ctail_173710, _ctx); /*list<61>*/
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, _x22101, _ctx));
}

kk_std_core__list kk_std_core__ctailm_zipwith(kk_std_core__list xs0, kk_std_core__list ys0, kk_function_t f2, kk_function_t _accm0, kk_context_t* _ctx) { /* forall<a,b,c,e> (xs : list<a>, ys : list<b>, f : (a, b) -> e c, (list<c>) -> list<c>) -> e list<c> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Cons(xs0)) {
    struct kk_std_core_Cons* _con22092 = kk_std_core__as_Cons(xs0);
    kk_box_t x1 = _con22092->head;
    kk_std_core__list xx2 = _con22092->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs0))) {
      kk_std_core__list_free(xs0);
    }
    else {
      kk_box_dup(x1);
      kk_std_core__list_dup(xx2);
      kk_Unit;
      kk_std_core__list_decref(xs0, _ctx);
      kk_Unit;
    }
    if (kk_std_core__is_Cons(ys0)) {
      struct kk_std_core_Cons* _con22093 = kk_std_core__as_Cons(ys0);
      kk_box_t y0 = _con22093->head;
      kk_std_core__list yy2 = _con22093->tail;
      if (kk_likely(kk_std_core__list_is_unique(ys0))) {
        kk_std_core__list_free(ys0);
      }
      else {
        kk_box_dup(y0);
        kk_std_core__list_dup(yy2);
        kk_Unit;
        kk_std_core__list_decref(ys0, _ctx);
        kk_Unit;
      }
      kk_box_t x2_18309;
      kk_function_t _x22094 = kk_function_dup(f2); /*(16761, 16762) -> 16764 16763*/
      x2_18309 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x22094, (_x22094, x1, y0, _ctx)); /*16763*/
      if (kk_yielding(kk_context())) {
        kk_box_drop(x2_18309, _ctx);
        kk_function_t _b_19888_19886 = kk_std_core__new_ctailm_zipwith_fun22095(_accm0, f2, xx2, yy2, _ctx); /*(16763) -> 16764 list<16763>*/;
        kk_box_t _x22096 = kk_std_core_hnd_yield_extend(kk_std_core__new_ctailm_zipwith_fun22097(_b_19888_19886, _ctx), _ctx); /*3858*/
        return kk_std_core__list_unbox(_x22096, _ctx);
      }
      { // tailcall
        kk_function_t _x22099 = kk_std_core__new_ctailm_zipwith_fun22100(_accm0, x2_18309, _ctx); /*(list<16763>) -> list<16763>*/
        xs0 = xx2;
        ys0 = yy2;
        _accm0 = _x22099;
        goto kk__tailcall;
      }
    }
    kk_function_drop(f2, _ctx);
    kk_box_drop(x1, _ctx);
    kk_std_core__list_drop(xx2, _ctx);
    return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
  }
  kk_function_drop(f2, _ctx);
  kk_std_core__list_drop(ys0, _ctx);
  return kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _accm0, (_accm0, kk_std_core__new_Nil(_ctx), _ctx));
}
 
// Zip two lists together by apply a function `f` to all corresponding elements.
// The returned list is only as long as the smallest input list.


// lift anonymous function
struct kk_std_core_zipwith_fun22102__t {
  struct kk_function_s _base;
};
static kk_std_core__list kk_std_core_zipwith_fun22102(kk_function_t _fself, kk_std_core__list _ctail_17370, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_zipwith_fun22102(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_core_zipwith_fun22102, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core__list kk_std_core_zipwith_fun22102(kk_function_t _fself, kk_std_core__list _ctail_17370, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return _ctail_17370;
}

kk_std_core__list kk_std_core_zipwith(kk_std_core__list xs1, kk_std_core__list ys1, kk_function_t f3, kk_context_t* _ctx) { /* forall<a,b,c,e> (xs : list<a>, ys : list<b>, f : (a, b) -> e c) -> e list<c> */ 
  bool _match_19905 = kk_std_core_hnd__evv_is_affine(_ctx); /*bool*/;
  if (_match_19905) {
    return kk_std_core__ctail_zipwith(xs1, ys1, f3, kk_std_core_types__new_CTailNil(_ctx), _ctx);
  }
  return kk_std_core__ctailm_zipwith(xs1, ys1, f3, kk_std_core_new_zipwith_fun22102(_ctx), _ctx);
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17774_zipwith_acc(kk_std_core__list acc, kk_function_t f, kk_integer_t i, kk_std_core__list xx, kk_std_core__list yy, kk_box_t _y_17674, kk_context_t* _ctx) { /* forall<e,a,b,c> (acc : list<b>, f : (int, a, c) -> e b, i : int, xx : list<a>, yy : list<c>, b) -> e list<b> */ 
  kk_integer_t _x22103 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_std_core__list _x22104 = kk_std_core__new_Cons(kk_reuse_null, _y_17674, acc, _ctx); /*list<61>*/
  return kk_std_core_zipwith_acc(f, _x22103, _x22104, xx, yy, _ctx);
}


// lift anonymous function
struct kk_std_core_zipwith_acc_fun22109__t {
  struct kk_function_s _base;
  kk_std_core__list acc0;
  kk_function_t f0;
  kk_integer_t i0;
  kk_std_core__list xx0;
  kk_std_core__list yy0;
};
static kk_std_core__list kk_std_core_zipwith_acc_fun22109(kk_function_t _fself, kk_box_t _y_176740, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_zipwith_acc_fun22109(kk_std_core__list acc0, kk_function_t f0, kk_integer_t i0, kk_std_core__list xx0, kk_std_core__list yy0, kk_context_t* _ctx) {
  struct kk_std_core_zipwith_acc_fun22109__t* _self = kk_function_alloc_as(struct kk_std_core_zipwith_acc_fun22109__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_zipwith_acc_fun22109, kk_context());
  _self->acc0 = acc0;
  _self->f0 = f0;
  _self->i0 = i0;
  _self->xx0 = xx0;
  _self->yy0 = yy0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core_zipwith_acc_fun22109(kk_function_t _fself, kk_box_t _y_176740, kk_context_t* _ctx) {
  struct kk_std_core_zipwith_acc_fun22109__t* _self = kk_function_as(struct kk_std_core_zipwith_acc_fun22109__t*, _fself);
  kk_std_core__list acc0 = _self->acc0; /* list<16776> */
  kk_function_t f0 = _self->f0; /* (int, 16774, 16784) -> 16771 16776 */
  kk_integer_t i0 = _self->i0; /* int */
  kk_std_core__list xx0 = _self->xx0; /* list<16774> */
  kk_std_core__list yy0 = _self->yy0; /* list<16784> */
  kk_drop_match(_self, {kk_std_core__list_dup(acc0);kk_function_dup(f0);kk_integer_dup(i0);kk_std_core__list_dup(xx0);kk_std_core__list_dup(yy0);}, {}, _ctx)
  return kk_std_core__mlift17774_zipwith_acc(acc0, f0, i0, xx0, yy0, _y_176740, _ctx);
}


// lift anonymous function
struct kk_std_core_zipwith_acc_fun22111__t {
  struct kk_function_s _base;
  kk_function_t _b_19891_19889;
};
static kk_box_t kk_std_core_zipwith_acc_fun22111(kk_function_t _fself, kk_box_t _b_19890, kk_context_t* _ctx);
static kk_function_t kk_std_core_new_zipwith_acc_fun22111(kk_function_t _b_19891_19889, kk_context_t* _ctx) {
  struct kk_std_core_zipwith_acc_fun22111__t* _self = kk_function_alloc_as(struct kk_std_core_zipwith_acc_fun22111__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core_zipwith_acc_fun22111, kk_context());
  _self->_b_19891_19889 = _b_19891_19889;
  return &_self->_base;
}

static kk_box_t kk_std_core_zipwith_acc_fun22111(kk_function_t _fself, kk_box_t _b_19890, kk_context_t* _ctx) {
  struct kk_std_core_zipwith_acc_fun22111__t* _self = kk_function_as(struct kk_std_core_zipwith_acc_fun22111__t*, _fself);
  kk_function_t _b_19891_19889 = _self->_b_19891_19889; /* (16776) -> 16771 list<16776> */
  kk_drop_match(_self, {kk_function_dup(_b_19891_19889);}, {}, _ctx)
  kk_std_core__list _x22112 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19891_19889, (_b_19891_19889, _b_19890, _ctx)); /*list<16776>*/
  return kk_std_core__list_box(_x22112, _ctx);
}

kk_std_core__list kk_std_core_zipwith_acc(kk_function_t f0, kk_integer_t i0, kk_std_core__list acc0, kk_std_core__list xs, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a,b,c,e> ((int, a, b) -> e c, int, list<c>, list<a>, list<b>) -> e list<c> */ 
  kk__tailcall: ;
  if (kk_std_core__is_Nil(xs)) {
    kk_function_drop(f0, _ctx);
    kk_integer_drop(i0, _ctx);
    kk_std_core__list_drop(ys, _ctx);
    return kk_std_core__lift17186_reverse(kk_std_core__new_Nil(_ctx), acc0, _ctx);
  }
  struct kk_std_core_Cons* _con22105 = kk_std_core__as_Cons(xs);
  kk_box_t x = _con22105->head;
  kk_std_core__list xx0 = _con22105->tail;
  if (kk_likely(kk_std_core__list_is_unique(xs))) {
    kk_std_core__list_free(xs);
  }
  else {
    kk_box_dup(x);
    kk_std_core__list_dup(xx0);
    kk_Unit;
    kk_std_core__list_decref(xs, _ctx);
    kk_Unit;
  }
  if (kk_std_core__is_Nil(ys)) {
    kk_function_drop(f0, _ctx);
    kk_integer_drop(i0, _ctx);
    kk_box_drop(x, _ctx);
    kk_std_core__list_drop(xx0, _ctx);
    return kk_std_core__lift17186_reverse(kk_std_core__new_Nil(_ctx), acc0, _ctx);
  }
  struct kk_std_core_Cons* _con22106 = kk_std_core__as_Cons(ys);
  kk_box_t y = _con22106->head;
  kk_std_core__list yy0 = _con22106->tail;
  kk_reuse_t _ru_20438 = kk_reuse_null; /*reuse*/;
  if (kk_likely(kk_std_core__list_is_unique(ys))) {
    _ru_20438 = (kk_std_core__list_reuse(ys));
  }
  else {
    kk_box_dup(y);
    kk_std_core__list_dup(yy0);
    kk_Unit;
    kk_std_core__list_decref(ys, _ctx);
    kk_Unit;
  }
  kk_box_t x0_18314;
  kk_function_t _x22108 = kk_function_dup(f0); /*(int, 16774, 16784) -> 16771 16776*/
  kk_integer_t _x22107 = kk_integer_dup(i0); /*int*/
  x0_18314 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_box_t, kk_box_t, kk_context_t*), _x22108, (_x22108, _x22107, x, y, _ctx)); /*16776*/
  if (kk_yielding(kk_context())) {
    kk_reuse_drop(_ru_20438);
    kk_box_drop(x0_18314, _ctx);
    kk_function_t _b_19891_19889 = kk_std_core_new_zipwith_acc_fun22109(acc0, f0, i0, xx0, yy0, _ctx); /*(16776) -> 16771 list<16776>*/;
    kk_box_t _x22110 = kk_std_core_hnd_yield_extend(kk_std_core_new_zipwith_acc_fun22111(_b_19891_19889, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x22110, _ctx);
  }
  { // tailcall
    kk_integer_t _x22113 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/
    kk_std_core__list _x22114 = kk_std_core__new_Cons(_ru_20438, x0_18314, acc0, _ctx); /*list<61>*/
    i0 = _x22113;
    acc0 = _x22114;
    xs = xx0;
    ys = yy0;
    goto kk__tailcall;
  }
}
 
// monadic lift

kk_std_core__list kk_std_core__mlift17775_op(kk_box_t _y_17678, kk_std_core__list _y_17679, kk_context_t* _ctx) { /* forall<a,e> (a, list<a>) -> e list<a> */ 
  return kk_std_core__new_Cons(kk_reuse_null, _y_17678, _y_17679, _ctx);
}
 
// monadic lift


// lift anonymous function
struct kk_std_core__mlift17776_op_fun22115__t {
  struct kk_function_s _base;
  kk_box_t _y_176780;
};
static kk_std_core__list kk_std_core__mlift17776_op_fun22115(kk_function_t _fself, kk_std_core__list _y_176790, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17776_op_fun22115(kk_box_t _y_176780, kk_context_t* _ctx) {
  struct kk_std_core__mlift17776_op_fun22115__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17776_op_fun22115__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17776_op_fun22115, kk_context());
  _self->_y_176780 = _y_176780;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__mlift17776_op_fun22115(kk_function_t _fself, kk_std_core__list _y_176790, kk_context_t* _ctx) {
  struct kk_std_core__mlift17776_op_fun22115__t* _self = kk_function_as(struct kk_std_core__mlift17776_op_fun22115__t*, _fself);
  kk_box_t _y_176780 = _self->_y_176780; /* 16905 */
  kk_drop_match(_self, {kk_box_dup(_y_176780);}, {}, _ctx)
  return kk_std_core__mlift17775_op(_y_176780, _y_176790, _ctx);
}


// lift anonymous function
struct kk_std_core__mlift17776_op_fun22117__t {
  struct kk_function_s _base;
  kk_function_t _b_19894_19892;
};
static kk_box_t kk_std_core__mlift17776_op_fun22117(kk_function_t _fself, kk_box_t _b_19893, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_mlift17776_op_fun22117(kk_function_t _b_19894_19892, kk_context_t* _ctx) {
  struct kk_std_core__mlift17776_op_fun22117__t* _self = kk_function_alloc_as(struct kk_std_core__mlift17776_op_fun22117__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__mlift17776_op_fun22117, kk_context());
  _self->_b_19894_19892 = _b_19894_19892;
  return &_self->_base;
}

static kk_box_t kk_std_core__mlift17776_op_fun22117(kk_function_t _fself, kk_box_t _b_19893, kk_context_t* _ctx) {
  struct kk_std_core__mlift17776_op_fun22117__t* _self = kk_function_as(struct kk_std_core__mlift17776_op_fun22117__t*, _fself);
  kk_function_t _b_19894_19892 = _self->_b_19894_19892; /* (list<16905>) -> 16906 list<16905> */
  kk_drop_match(_self, {kk_function_dup(_b_19894_19892);}, {}, _ctx)
  kk_std_core__list _x22118;
  kk_std_core__list _x22119 = kk_std_core__list_unbox(_b_19893, _ctx); /*list<16905>*/
  _x22118 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19894_19892, (_b_19894_19892, _x22119, _ctx)); /*list<16905>*/
  return kk_std_core__list_box(_x22118, _ctx);
}

kk_std_core__list kk_std_core__mlift17776_op(kk_function_t f, kk_integer_t i, kk_std_core__list xx, kk_std_core__list yy, kk_box_t _y_176780, kk_context_t* _ctx) { /* forall<a,b,c,e> (f : (int, a, b) -> e c, i : int, xx : list<a>, yy : list<b>, c) -> e list<c> */ 
  kk_integer_t i0_17288 = kk_integer_add(i,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_std_core__list x_18317 = kk_std_core__lift17208_zipwith_indexed(f, i0_17288, xx, yy, _ctx); /*list<16905>*/;
  if (kk_yielding(kk_context())) {
    kk_std_core__list_drop(x_18317, _ctx);
    kk_function_t _b_19894_19892 = kk_std_core__new_mlift17776_op_fun22115(_y_176780, _ctx); /*(list<16905>) -> 16906 list<16905>*/;
    kk_box_t _x22116 = kk_std_core_hnd_yield_extend(kk_std_core__new_mlift17776_op_fun22117(_b_19894_19892, _ctx), _ctx); /*3858*/
    return kk_std_core__list_unbox(_x22116, _ctx);
  }
  return kk_std_core__mlift17775_op(_y_176780, x_18317, _ctx);
}
 
// lifted


// lift anonymous function
struct kk_std_core__lift17208_zipwith_indexed_fun22124__t {
  struct kk_function_s _base;
  kk_function_t f0;
  kk_integer_t i0;
  kk_std_core__list xx0;
  kk_std_core__list yy0;
};
static kk_std_core__list kk_std_core__lift17208_zipwith_indexed_fun22124(kk_function_t _fself, kk_box_t _y_176781, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17208_zipwith_indexed_fun22124(kk_function_t f0, kk_integer_t i0, kk_std_core__list xx0, kk_std_core__list yy0, kk_context_t* _ctx) {
  struct kk_std_core__lift17208_zipwith_indexed_fun22124__t* _self = kk_function_alloc_as(struct kk_std_core__lift17208_zipwith_indexed_fun22124__t, 5, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17208_zipwith_indexed_fun22124, kk_context());
  _self->f0 = f0;
  _self->i0 = i0;
  _self->xx0 = xx0;
  _self->yy0 = yy0;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17208_zipwith_indexed_fun22124(kk_function_t _fself, kk_box_t _y_176781, kk_context_t* _ctx) {
  struct kk_std_core__lift17208_zipwith_indexed_fun22124__t* _self = kk_function_as(struct kk_std_core__lift17208_zipwith_indexed_fun22124__t*, _fself);
  kk_function_t f0 = _self->f0; /* (int, 16903, 16904) -> 16906 16905 */
  kk_integer_t i0 = _self->i0; /* int */
  kk_std_core__list xx0 = _self->xx0; /* list<16903> */
  kk_std_core__list yy0 = _self->yy0; /* list<16904> */
  kk_drop_match(_self, {kk_function_dup(f0);kk_integer_dup(i0);kk_std_core__list_dup(xx0);kk_std_core__list_dup(yy0);}, {}, _ctx)
  return kk_std_core__mlift17776_op(f0, i0, xx0, yy0, _y_176781, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17208_zipwith_indexed_fun22126__t {
  struct kk_function_s _base;
  kk_function_t _b_19899_19895;
};
static kk_box_t kk_std_core__lift17208_zipwith_indexed_fun22126(kk_function_t _fself, kk_box_t _b_19896, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17208_zipwith_indexed_fun22126(kk_function_t _b_19899_19895, kk_context_t* _ctx) {
  struct kk_std_core__lift17208_zipwith_indexed_fun22126__t* _self = kk_function_alloc_as(struct kk_std_core__lift17208_zipwith_indexed_fun22126__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17208_zipwith_indexed_fun22126, kk_context());
  _self->_b_19899_19895 = _b_19899_19895;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17208_zipwith_indexed_fun22126(kk_function_t _fself, kk_box_t _b_19896, kk_context_t* _ctx) {
  struct kk_std_core__lift17208_zipwith_indexed_fun22126__t* _self = kk_function_as(struct kk_std_core__lift17208_zipwith_indexed_fun22126__t*, _fself);
  kk_function_t _b_19899_19895 = _self->_b_19899_19895; /* (16905) -> 16906 list<16905> */
  kk_drop_match(_self, {kk_function_dup(_b_19899_19895);}, {}, _ctx)
  kk_std_core__list _x22127 = kk_function_call(kk_std_core__list, (kk_function_t, kk_box_t, kk_context_t*), _b_19899_19895, (_b_19899_19895, _b_19896, _ctx)); /*list<16905>*/
  return kk_std_core__list_box(_x22127, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17208_zipwith_indexed_fun22128__t {
  struct kk_function_s _base;
  kk_box_t x1_18319;
};
static kk_std_core__list kk_std_core__lift17208_zipwith_indexed_fun22128(kk_function_t _fself, kk_std_core__list _y_176791, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17208_zipwith_indexed_fun22128(kk_box_t x1_18319, kk_context_t* _ctx) {
  struct kk_std_core__lift17208_zipwith_indexed_fun22128__t* _self = kk_function_alloc_as(struct kk_std_core__lift17208_zipwith_indexed_fun22128__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17208_zipwith_indexed_fun22128, kk_context());
  _self->x1_18319 = x1_18319;
  return &_self->_base;
}

static kk_std_core__list kk_std_core__lift17208_zipwith_indexed_fun22128(kk_function_t _fself, kk_std_core__list _y_176791, kk_context_t* _ctx) {
  struct kk_std_core__lift17208_zipwith_indexed_fun22128__t* _self = kk_function_as(struct kk_std_core__lift17208_zipwith_indexed_fun22128__t*, _fself);
  kk_box_t x1_18319 = _self->x1_18319; /* 16905 */
  kk_drop_match(_self, {kk_box_dup(x1_18319);}, {}, _ctx)
  return kk_std_core__mlift17775_op(x1_18319, _y_176791, _ctx);
}


// lift anonymous function
struct kk_std_core__lift17208_zipwith_indexed_fun22130__t {
  struct kk_function_s _base;
  kk_function_t _b_19900_19897;
};
static kk_box_t kk_std_core__lift17208_zipwith_indexed_fun22130(kk_function_t _fself, kk_box_t _b_19898, kk_context_t* _ctx);
static kk_function_t kk_std_core__new_lift17208_zipwith_indexed_fun22130(kk_function_t _b_19900_19897, kk_context_t* _ctx) {
  struct kk_std_core__lift17208_zipwith_indexed_fun22130__t* _self = kk_function_alloc_as(struct kk_std_core__lift17208_zipwith_indexed_fun22130__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_core__lift17208_zipwith_indexed_fun22130, kk_context());
  _self->_b_19900_19897 = _b_19900_19897;
  return &_self->_base;
}

static kk_box_t kk_std_core__lift17208_zipwith_indexed_fun22130(kk_function_t _fself, kk_box_t _b_19898, kk_context_t* _ctx) {
  struct kk_std_core__lift17208_zipwith_indexed_fun22130__t* _self = kk_function_as(struct kk_std_core__lift17208_zipwith_indexed_fun22130__t*, _fself);
  kk_function_t _b_19900_19897 = _self->_b_19900_19897; /* (list<16905>) -> 16906 list<16905> */
  kk_drop_match(_self, {kk_function_dup(_b_19900_19897);}, {}, _ctx)
  kk_std_core__list _x22131;
  kk_std_core__list _x22132 = kk_std_core__list_unbox(_b_19898, _ctx); /*list<16905>*/
  _x22131 = kk_function_call(kk_std_core__list, (kk_function_t, kk_std_core__list, kk_context_t*), _b_19900_19897, (_b_19900_19897, _x22132, _ctx)); /*list<16905>*/
  return kk_std_core__list_box(_x22131, _ctx);
}

kk_std_core__list kk_std_core__lift17208_zipwith_indexed(kk_function_t f0, kk_integer_t i0, kk_std_core__list xs, kk_std_core__list ys, kk_context_t* _ctx) { /* forall<a,b,c,e> (f : (int, a, b) -> e c, i : int, xs : list<a>, ys : list<b>) -> e list<c> */ 
  if (kk_std_core__is_Cons(xs)) {
    struct kk_std_core_Cons* _con22120 = kk_std_core__as_Cons(xs);
    kk_box_t x0 = _con22120->head;
    kk_std_core__list xx0 = _con22120->tail;
    if (kk_likely(kk_std_core__list_is_unique(xs))) {
      kk_std_core__list_free(xs);
    }
    else {
      kk_box_dup(x0);
      kk_std_core__list_dup(xx0);
      kk_Unit;
      kk_std_core__list_decref(xs, _ctx);
      kk_Unit;
    }
    if (kk_std_core__is_Cons(ys)) {
      struct kk_std_core_Cons* _con22121 = kk_std_core__as_Cons(ys);
      kk_box_t y = _con22121->head;
      kk_std_core__list yy0 = _con22121->tail;
      kk_reuse_t _ru_20440 = kk_reuse_null; /*reuse*/;
      if (kk_likely(kk_std_core__list_is_unique(ys))) {
        _ru_20440 = (kk_std_core__list_reuse(ys));
      }
      else {
        kk_box_dup(y);
        kk_std_core__list_dup(yy0);
        kk_Unit;
        kk_std_core__list_decref(ys, _ctx);
        kk_Unit;
      }
      kk_box_t x1_18319;
      kk_function_t _x22123 = kk_function_dup(f0); /*(int, 16903, 16904) -> 16906 16905*/
      kk_integer_t _x22122 = kk_integer_dup(i0); /*int*/
      x1_18319 = kk_function_call(kk_box_t, (kk_function_t, kk_integer_t, kk_box_t, kk_box_t, kk_context_t*), _x22123, (_x22123, _x22122, x0, y, _ctx)); /*16905*/
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_20440);
        kk_box_drop(x1_18319, _ctx);
        kk_function_t _b_19899_19895 = kk_std_core__new_lift17208_zipwith_indexed_fun22124(f0, i0, xx0, yy0, _ctx); /*(16905) -> 16906 list<16905>*/;
        kk_box_t _x22125 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17208_zipwith_indexed_fun22126(_b_19899_19895, _ctx), _ctx); /*3858*/
        return kk_std_core__list_unbox(_x22125, _ctx);
      }
      kk_integer_t i0_172880 = kk_integer_add(i0,(kk_integer_from_small(1)),kk_context()); /*int*/;
      kk_std_core__list x2_18322 = kk_std_core__lift17208_zipwith_indexed(f0, i0_172880, xx0, yy0, _ctx); /*list<16905>*/;
      if (kk_yielding(kk_context())) {
        kk_reuse_drop(_ru_20440);
        kk_std_core__list_drop(x2_18322, _ctx);
        kk_function_t _b_19900_19897 = kk_std_core__new_lift17208_zipwith_indexed_fun22128(x1_18319, _ctx); /*(list<16905>) -> 16906 list<16905>*/;
        kk_box_t _x22129 = kk_std_core_hnd_yield_extend(kk_std_core__new_lift17208_zipwith_indexed_fun22130(_b_19900_19897, _ctx), _ctx); /*3858*/
        return kk_std_core__list_unbox(_x22129, _ctx);
      }
      return kk_std_core__new_Cons(_ru_20440, x1_18319, x2_18322, _ctx);
    }
    kk_function_drop(f0, _ctx);
    kk_integer_drop(i0, _ctx);
    kk_box_drop(x0, _ctx);
    kk_std_core__list_drop(xx0, _ctx);
    return kk_std_core__new_Nil(_ctx);
  }
  kk_function_drop(f0, _ctx);
  kk_integer_drop(i0, _ctx);
  kk_std_core__list_drop(ys, _ctx);
  return kk_std_core__new_Nil(_ctx);
}

// initialization
void kk_std_core__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_string_t _x20478;
    kk_define_string_literal(, _s20479, 8, "exn.core")
    _x20478 = kk_string_dup(_s20479); /*string*/
    kk_std_core__tag_exn = kk_std_core_hnd__new_Htag(_x20478, _ctx); /*std/core/hnd/htag<.hnd-exn>*/
  }
  {
    kk_std_core_types__maybe _b_18431_18430 = kk_std_core_types__new_Nothing(_ctx); /*maybe<(string) -> console ()>*/;
    kk_std_core_redirect = kk_ref_alloc((kk_std_core_types__maybe_box(_b_18431_18430, _ctx)),kk_context()); /*ref<global,maybe<(string) -> console ()>>*/
  }
  {
    kk_std_core_maxListStack = kk_integer_from_small(50); /*int*/
  }
  {
    bool _b_18583_18582 = true; /*bool*/;
    kk_std_core_trace_enabled = kk_ref_alloc((kk_bool_box(_b_18583_18582)),kk_context()); /*ref<global,bool>*/
  }
  {
    kk_integer_t _b_18600_18599 = kk_integer_from_small(0); /*int*/;
    kk_std_core_unique_count = kk_ref_alloc((kk_integer_box(_b_18600_18599)),kk_context()); /*ref<global,int>*/
  }
  {
    kk_string_t _x21407 = kk_string_empty(); /*string*/
    kk_ssize_t _x21409 = ((kk_ssize_t)0); /*ssize_t*/
    kk_ssize_t _x21410 = ((kk_ssize_t)0); /*ssize_t*/
    kk_std_core_empty = kk_std_core__new_Sslice(_x21407, _x21409, _x21410, _ctx); /*sslice*/
  }
  {
    kk_string_t _x21671 = kk_string_empty(); /*string*/
    kk_ssize_t _x21673 = ((kk_ssize_t)-1); /*ssize_t*/
    kk_ssize_t _x21674 = ((kk_ssize_t)0); /*ssize_t*/
    kk_std_core_invalid = kk_std_core__new_Sslice(_x21671, _x21673, _x21674, _ctx); /*sslice*/
  }
}

// termination
void kk_std_core__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_core__sslice_drop(kk_std_core_invalid, _ctx);
  kk_std_core__sslice_drop(kk_std_core_empty, _ctx);
  kk_ref_drop(kk_std_core_unique_count, _ctx);
  kk_ref_drop(kk_std_core_trace_enabled, _ctx);
  kk_integer_drop(kk_std_core_maxListStack, _ctx);
  kk_ref_drop(kk_std_core_redirect, _ctx);
  kk_std_core_hnd__htag_drop(kk_std_core__tag_exn, _ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
