// Koka generated module: "std/time/calendar", koka version: 2.2.1
#include "std_time_calendar.h"
/*---------------------------------------------------------------------------
  Copyright 2020-2021, Microsoft Research, Daan Leijen.

  This is free software; you can redistribute it and/or modify it under the
  terms of the Apache License, Version 2.0. A copy of the License can be
  found in the LICENSE file at the root of this distribution.
---------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------------------------------
  Local time zone UTC delta
--------------------------------------------------------------------------------------------------*/

#if !defined(__STDC_WANT_LIB_EXT1__)
#define __STDC_WANT_LIB_EXT1__    // for gmtime_s
#endif
#if defined(__GLIBC__) && !defined(_GNU_SOURCE)
#define _GNU_SOURCE               // for tm_zone and tm_gmtoff
#endif
#include <time.h>

static long kk_local_utc_delta(double unix_secs, kk_string_t* ptzname, kk_context_t* ctx) {
  // get the UTC delta in a somewhat portable way...
  bool isdst = false;
  time_t t = (time_t)unix_secs;
  #if (_WIN32 && KK_INTPTR_SIZE==8)
  if (t < 0) { t += 3155673600; } // try to avoid errors for negative times on some platforms by adding 100 years..
  #endif
  time_t loct = t;    
  #if defined(_GNU_SOURCE)
    // GNU libc has the tm_zone and tm_gmtoff fields
    struct tm loctm;
    if (localtime_r(&t, &loctm) != NULL) {
      isdst = (loctm.tm_isdst != 0);
      loct  = t - loctm.tm_gmtoff + (isdst ? 3600 : 0);
    }
    else {
      loctm.tm_zone = "";
    }
  #elif defined(_WIN32) && !defined(__MINGW32__)
    struct tm gmtm;
    if (gmtime_s(&gmtm, &t) == 0) {      // switched parameters :-(      
      loct = mktime(&gmtm);              // interpret gmt as local time
      struct tm loctm;
      localtime_s(&loctm, &t);           // switched parameters :-(
      isdst = (loctm.tm_isdst != 0);
    }
  #elif defined(__STDC_LIB_EXT1__)
    struct tm gmtm;
    if (gmtime_s(&t, &gmtm) != NULL) {
      loct = mktime(&gmtm);              // interpret gmt as local time
      struct tm loctm;
      localtime_s(&t, &loctm);
      isdst = (loctm.tm_isdst != 0);
    }
  #else
    struct tm* pgmtm = gmtime(&t);
    if (pgmtm != NULL) {
      loct = mktime(pgmtm);                // interpret gmt as local time
      struct tm* ploctm = localtime(&t);
      isdst = (ploctm->tm_isdst != 0);
    }
  #endif
  const time_t utc_delta = t - loct + (isdst ? 3600 : 0);   // the difference is the utc offset at that time
  if (ptzname != NULL) {
    // getting the timezone name
    #if defined(_WIN32) && !defined(__MINGW32__)
      char tzonename[256];
      size_t tznamelen;
      _get_tzname(&tznamelen, tzonename, 255, isdst ? 1 : 0); tzonename[255] = 0;
      *ptzname = kk_string_alloc_from_qutf8(tzonename, ctx);
    #elif defined(_GNU_SOURCE)
      *ptzname = kk_string_alloc_from_qutf8(loctm.tm_zone, ctx);
    #elif (_POSIX_C_SOURCE >= 1) || _XOPEN_SOURCE || _POSIX_SOURCE || __MINGW32__ // tzname
      *ptzname = kk_string_alloc_from_qutf8(tzname[isdst ? 1 : 0], ctx);
    #else
      // give up :-(
      * ptzname = kk_string_empty();
    #endif
  }
  return (long)utc_delta;
}


static kk_std_time_calendar__local_timezone kk_local_get_timezone(kk_context_t* ctx) {
  return kk_datatype_from_tag((kk_tag_t)1); // dummy value; we cannot store the local timezone as it is a global :-(
}

static kk_std_core_types__tuple2_ kk_local_get_utc_delta_tuple(kk_std_time_calendar__local_timezone tz, double unix_secs, kk_context_t* ctx) {
  kk_string_t tzonename;
  long utc_delta = kk_local_utc_delta(unix_secs, &tzonename, ctx);
  return kk_std_core_types__new_dash__lp__comma__rp_( kk_double_box((double)utc_delta,ctx), kk_string_box(tzonename), ctx );
}



// lift anonymous function
struct kk_std_time_calendar__copy_fun5833__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4933;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5833(kk_function_t _fself, kk_std_time_instant__instant _b_4936, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5833(kk_box_t _fun_unbox_x4933, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5833__t* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5833__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5833, kk_context());
  _self->_fun_unbox_x4933 = _fun_unbox_x4933;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5833(kk_function_t _fself, kk_std_time_instant__instant _b_4936, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5833__t* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5833__t*, _fself);
  kk_box_t _fun_unbox_x4933 = _self->_fun_unbox_x4933; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4933);}, {}, _ctx)
  kk_box_t _x5834;
  kk_function_t _x5835 = kk_function_unbox(_fun_unbox_x4933); /*(4934) -> 4935*/
  _x5834 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5835, (_x5835, kk_std_time_instant__instant_box(_b_4936, _ctx), _ctx)); /*4935*/
  return kk_std_core_types__tuple2__unbox(_x5834, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5837__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4940;
};
static kk_std_core_types__maybe kk_std_time_calendar__copy_fun5837(kk_function_t _fself, kk_std_time_instant__instant _b_4943, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5837(kk_box_t _fun_unbox_x4940, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5837__t* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5837__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5837, kk_context());
  _self->_fun_unbox_x4940 = _fun_unbox_x4940;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_time_calendar__copy_fun5837(kk_function_t _fself, kk_std_time_instant__instant _b_4943, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5837__t* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5837__t*, _fself);
  kk_box_t _fun_unbox_x4940 = _self->_fun_unbox_x4940; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4940);}, {}, _ctx)
  kk_box_t _x5838;
  kk_function_t _x5839 = kk_function_unbox(_fun_unbox_x4940); /*(4941) -> 4942*/
  _x5838 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5839, (_x5839, kk_std_time_instant__instant_box(_b_4943, _ctx), _ctx)); /*4942*/
  return kk_std_core_types__maybe_unbox(_x5838, _ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar__copy(kk_std_time_calendar__timezone _this, kk_std_core_types__optional name0, kk_std_core_types__optional utc_delta0, kk_std_core_types__optional utc_inverse0, kk_context_t* _ctx) { /* (timezone, name : optional<string>, utc-delta : optional<(std/time/instant/instant) -> (std/time/duration/duration, string)>, utc-inverse : optional<(std/time/instant/instant) -> maybe<std/time/instant/instant>>) -> timezone */ 
  kk_string_t _x5829;
  if (kk_std_core_types__is_Optional(name0)) {
    kk_box_t _box_x4929 = name0._cons.Optional.value;
    kk_string_t _name_217 = kk_string_unbox(_box_x4929);
    kk_string_dup(_name_217);
    kk_std_core_types__optional_drop(name0, _ctx);
    kk_Unit;
    _x5829 = _name_217; /*string*/
    goto _match5830;
  }
  kk_string_t _x = _this.name;
  kk_string_dup(_x);
  _x5829 = _x; /*string*/
  _match5830: ;
  kk_function_t _x5832;
  if (kk_std_core_types__is_Optional(utc_delta0)) {
    kk_box_t _fun_unbox_x4933 = utc_delta0._cons.Optional.value;
    _x5832 = kk_std_time_calendar__new_copy_fun5833(_fun_unbox_x4933, _ctx); /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  }
  else {
    kk_function_t _x0 = _this.utc_delta;
    kk_function_dup(_x0);
    _x5832 = _x0; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  }
  kk_function_t _x5836;
  if (kk_std_core_types__is_Optional(utc_inverse0)) {
    kk_box_t _fun_unbox_x4940 = utc_inverse0._cons.Optional.value;
    kk_std_time_calendar__timezone_drop(_this, _ctx);
    kk_Unit;
    _x5836 = kk_std_time_calendar__new_copy_fun5837(_fun_unbox_x4940, _ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  else {
    kk_function_t _x1 = _this.utc_inverse;
    kk_function_dup(_x1);
    kk_std_time_calendar__timezone_drop(_this, _ctx);
    kk_Unit;
    _x5836 = _x1; /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  return kk_std_time_calendar__new_Timezone(_x5829, _x5832, _x5836, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5861__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4950;
};
static kk_string_t kk_std_time_calendar__copy_fun5861_1(kk_function_t _fself, kk_std_time_date__date _b_4953, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5861_1(kk_box_t _fun_unbox_x4950, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5861__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5861__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5861_1, kk_context());
  _self->_fun_unbox_x4950 = _fun_unbox_x4950;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar__copy_fun5861_1(kk_function_t _fself, kk_std_time_date__date _b_4953, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5861__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5861__t_1*, _fself);
  kk_box_t _fun_unbox_x4950 = _self->_fun_unbox_x4950; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4950);}, {}, _ctx)
  kk_box_t _x5862;
  kk_function_t _x5863 = kk_function_unbox(_fun_unbox_x4950); /*(4951) -> 4952*/
  _x5862 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5863, (_x5863, kk_std_time_date__date_box(_b_4953, _ctx), _ctx)); /*4952*/
  return kk_string_unbox(_x5862);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5866__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4959;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5866_1(kk_function_t _fself, kk_std_time_instant__instant _b_4963, kk_std_time_duration__duration _b_4964, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5866_1(kk_box_t _fun_unbox_x4959, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5866__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5866__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5866_1, kk_context());
  _self->_fun_unbox_x4959 = _fun_unbox_x4959;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar__copy_fun5866_1(kk_function_t _fself, kk_std_time_instant__instant _b_4963, kk_std_time_duration__duration _b_4964, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5866__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5866__t_1*, _fself);
  kk_box_t _fun_unbox_x4959 = _self->_fun_unbox_x4959; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4959);}, {}, _ctx)
  kk_box_t _x5867;
  kk_function_t _x5868 = kk_function_unbox(_fun_unbox_x4959); /*(i : 4960, tzdelta : 4961) -> 4962*/
  _x5867 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_context_t*), _x5868, (_x5868, kk_std_time_instant__instant_box(_b_4963, _ctx), kk_std_time_duration__duration_box(_b_4964, _ctx), _ctx)); /*4962*/
  return kk_std_core_types__tuple2__unbox(_x5867, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5871__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4974;
};
static kk_std_time_instant__instant kk_std_time_calendar__copy_fun5871_1(kk_function_t _fself, kk_std_time_date__date _b_4980, kk_std_time_date__clock _b_4981, kk_std_time_calendar__timezone _b_4982, kk_std_time_instant__timescale _b_4983, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5871_1(kk_box_t _fun_unbox_x4974, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5871__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5871__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5871_1, kk_context());
  _self->_fun_unbox_x4974 = _fun_unbox_x4974;
  return &_self->_base;
}

static kk_std_time_instant__instant kk_std_time_calendar__copy_fun5871_1(kk_function_t _fself, kk_std_time_date__date _b_4980, kk_std_time_date__clock _b_4981, kk_std_time_calendar__timezone _b_4982, kk_std_time_instant__timescale _b_4983, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5871__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5871__t_1*, _fself);
  kk_box_t _fun_unbox_x4974 = _self->_fun_unbox_x4974; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4974);}, {}, _ctx)
  kk_box_t _x5872;
  kk_function_t _x5873 = kk_function_unbox(_fun_unbox_x4974); /*(4975, 4976, 4977, 4978) -> 4979*/
  _x5872 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_box_t, kk_box_t, kk_box_t, kk_context_t*), _x5873, (_x5873, kk_std_time_date__date_box(_b_4980, _ctx), kk_std_time_date__clock_box(_b_4981, _ctx), kk_std_time_calendar__timezone_box(_b_4982, _ctx), kk_std_time_instant__timescale_box(_b_4983, _ctx), _ctx)); /*4979*/
  return kk_std_time_instant__instant_unbox(_x5872, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5876__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4987;
};
static kk_std_time_date__date kk_std_time_calendar__copy_fun5876_1(kk_function_t _fself, kk_integer_t _b_4990, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5876_1(kk_box_t _fun_unbox_x4987, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5876__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5876__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5876_1, kk_context());
  _self->_fun_unbox_x4987 = _fun_unbox_x4987;
  return &_self->_base;
}

static kk_std_time_date__date kk_std_time_calendar__copy_fun5876_1(kk_function_t _fself, kk_integer_t _b_4990, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5876__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5876__t_1*, _fself);
  kk_box_t _fun_unbox_x4987 = _self->_fun_unbox_x4987; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4987);}, {}, _ctx)
  kk_box_t _x5877;
  kk_function_t _x5878 = kk_function_unbox(_fun_unbox_x4987); /*(days : 4988) -> 4989*/
  _x5877 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5878, (_x5878, kk_integer_box(_b_4990), _ctx)); /*4989*/
  return kk_std_time_date__date_unbox(_x5877, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar__copy_fun5881__t_1 {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x4994;
};
static kk_integer_t kk_std_time_calendar__copy_fun5881_1(kk_function_t _fself, kk_std_time_date__date _b_4997, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar__new_copy_fun5881_1(kk_box_t _fun_unbox_x4994, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5881__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar__copy_fun5881__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar__copy_fun5881_1, kk_context());
  _self->_fun_unbox_x4994 = _fun_unbox_x4994;
  return &_self->_base;
}

static kk_integer_t kk_std_time_calendar__copy_fun5881_1(kk_function_t _fself, kk_std_time_date__date _b_4997, kk_context_t* _ctx) {
  struct kk_std_time_calendar__copy_fun5881__t_1* _self = kk_function_as(struct kk_std_time_calendar__copy_fun5881__t_1*, _fself);
  kk_box_t _fun_unbox_x4994 = _self->_fun_unbox_x4994; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x4994);}, {}, _ctx)
  kk_box_t _x5882;
  kk_function_t _x5883 = kk_function_unbox(_fun_unbox_x4994); /*(date : 4995) -> 4996*/
  _x5882 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5883, (_x5883, kk_std_time_date__date_box(_b_4997, _ctx), _ctx)); /*4996*/
  return kk_integer_unbox(_x5882);
}

kk_std_time_calendar__calendar kk_std_time_calendar__copy_1(kk_std_time_calendar__calendar _this, kk_std_core_types__optional name0, kk_std_core_types__optional long_name0, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_std_core_types__optional instant_to_dc0, kk_std_core_types__optional dc_to_instant0, kk_std_core_types__optional days_to_date0, kk_std_core_types__optional date_to_days0, kk_context_t* _ctx) { /* (calendar, name : optional<string>, long-name : optional<string>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>, instant-to-dc : optional<(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)>, dc-to-instant : optional<(std/time/date/date, std/time/date/clock, timezone, std/time/instant/timescale) -> std/time/instant/instant>, days-to-date : optional<(days : int) -> std/time/date/date>, date-to-days : optional<(date : std/time/date/date) -> int>) -> calendar */ 
  kk_string_t _x5848;
  if (kk_std_core_types__is_Optional(name0)) {
    kk_box_t _box_x4944 = name0._cons.Optional.value;
    kk_string_t _name_314 = kk_string_unbox(_box_x4944);
    kk_string_dup(_name_314);
    kk_std_core_types__optional_drop(name0, _ctx);
    kk_Unit;
    _x5848 = _name_314; /*string*/
    goto _match5849;
  }
  struct kk_std_time_calendar_Calendar* _con5851 = kk_std_time_calendar__as_Calendar(_this);
  kk_string_t _x = _con5851->name;
  kk_string_dup(_x);
  _x5848 = _x; /*string*/
  _match5849: ;
  kk_string_t _x5852;
  if (kk_std_core_types__is_Optional(long_name0)) {
    kk_box_t _box_x4945 = long_name0._cons.Optional.value;
    kk_string_t _long_name_320 = kk_string_unbox(_box_x4945);
    kk_string_dup(_long_name_320);
    kk_std_core_types__optional_drop(long_name0, _ctx);
    kk_Unit;
    _x5852 = _long_name_320; /*string*/
    goto _match5853;
  }
  struct kk_std_time_calendar_Calendar* _con5855 = kk_std_time_calendar__as_Calendar(_this);
  kk_string_t _x0 = _con5855->long_name;
  kk_string_dup(_x0);
  _x5852 = _x0; /*string*/
  _match5853: ;
  kk_string_t _x5856;
  if (kk_std_core_types__is_Optional(month_prefix0)) {
    kk_box_t _box_x4946 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_326 = kk_string_unbox(_box_x4946);
    kk_string_dup(_month_prefix_326);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    kk_Unit;
    _x5856 = _month_prefix_326; /*string*/
    goto _match5857;
  }
  struct kk_std_time_calendar_Calendar* _con5859 = kk_std_time_calendar__as_Calendar(_this);
  kk_string_t _x1 = _con5859->month_prefix;
  kk_string_dup(_x1);
  _x5856 = _x1; /*string*/
  _match5857: ;
  kk_function_t _x5860;
  if (kk_std_core_types__is_Optional(show_era0)) {
    kk_box_t _fun_unbox_x4950 = show_era0._cons.Optional.value;
    _x5860 = kk_std_time_calendar__new_copy_fun5861_1(_fun_unbox_x4950, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    struct kk_std_time_calendar_Calendar* _con5864 = kk_std_time_calendar__as_Calendar(_this);
    kk_function_t _x2 = _con5864->show_era;
    kk_function_dup(_x2);
    _x5860 = _x2; /*(std/time/date/date) -> string*/
  }
  kk_function_t _x5865;
  if (kk_std_core_types__is_Optional(instant_to_dc0)) {
    kk_box_t _fun_unbox_x4959 = instant_to_dc0._cons.Optional.value;
    _x5865 = kk_std_time_calendar__new_copy_fun5866_1(_fun_unbox_x4959, _ctx); /*(std/time/instant/instant, std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)*/
  }
  else {
    struct kk_std_time_calendar_Calendar* _con5869 = kk_std_time_calendar__as_Calendar(_this);
    kk_function_t _x3 = _con5869->instant_to_dc;
    kk_function_dup(_x3);
    _x5865 = _x3; /*(std/time/instant/instant, std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)*/
  }
  kk_function_t _x5870;
  if (kk_std_core_types__is_Optional(dc_to_instant0)) {
    kk_box_t _fun_unbox_x4974 = dc_to_instant0._cons.Optional.value;
    _x5870 = kk_std_time_calendar__new_copy_fun5871_1(_fun_unbox_x4974, _ctx); /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  }
  else {
    struct kk_std_time_calendar_Calendar* _con5874 = kk_std_time_calendar__as_Calendar(_this);
    kk_function_t _x4 = _con5874->dc_to_instant;
    kk_function_dup(_x4);
    _x5870 = _x4; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  }
  kk_function_t _x5875;
  if (kk_std_core_types__is_Optional(days_to_date0)) {
    kk_box_t _fun_unbox_x4987 = days_to_date0._cons.Optional.value;
    _x5875 = kk_std_time_calendar__new_copy_fun5876_1(_fun_unbox_x4987, _ctx); /*(int) -> std/time/date/date*/
  }
  else {
    struct kk_std_time_calendar_Calendar* _con5879 = kk_std_time_calendar__as_Calendar(_this);
    kk_function_t _x5 = _con5879->days_to_date;
    kk_function_dup(_x5);
    _x5875 = _x5; /*(int) -> std/time/date/date*/
  }
  kk_function_t _x5880;
  if (kk_std_core_types__is_Optional(date_to_days0)) {
    kk_box_t _fun_unbox_x4994 = date_to_days0._cons.Optional.value;
    kk_std_time_calendar__calendar_dropn(_this, ((int32_t)8), _ctx);
    kk_Unit;
    _x5880 = kk_std_time_calendar__new_copy_fun5881_1(_fun_unbox_x4994, _ctx); /*(std/time/date/date) -> int*/
  }
  else {
    struct kk_std_time_calendar_Calendar* _con5884 = kk_std_time_calendar__as_Calendar(_this);
    kk_string_t _pat07 = _con5884->name;
    kk_string_t _pat112 = _con5884->long_name;
    kk_string_t _pat28 = _con5884->month_prefix;
    kk_function_t _pat37 = _con5884->show_era;
    kk_function_t _pat47 = _con5884->instant_to_dc;
    kk_function_t _pat57 = _con5884->dc_to_instant;
    kk_function_t _pat67 = _con5884->days_to_date;
    kk_function_t _x6 = _con5884->date_to_days;
    if (kk_likely(kk_std_time_calendar__calendar_is_unique(_this))) {
      kk_function_drop(_pat67, _ctx);
      kk_Unit;
      kk_function_drop(_pat57, _ctx);
      kk_Unit;
      kk_function_drop(_pat47, _ctx);
      kk_Unit;
      kk_function_drop(_pat37, _ctx);
      kk_Unit;
      kk_string_drop(_pat28, _ctx);
      kk_Unit;
      kk_string_drop(_pat112, _ctx);
      kk_Unit;
      kk_string_drop(_pat07, _ctx);
      kk_Unit;
      kk_Unit;
      kk_std_time_calendar__calendar_free(_this);
    }
    else {
      kk_function_dup(_x6);
      kk_Unit;
      kk_std_time_calendar__calendar_decref(_this, _ctx);
      kk_Unit;
    }
    _x5880 = _x6; /*(std/time/date/date) -> int*/
  }
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, _x5848, _x5852, _x5856, _x5860, _x5865, _x5870, _x5875, _x5880, _ctx);
}

kk_integer_t kk_std_time_calendar_iso_days_before_year(kk_integer_t year, kk_context_t* _ctx) { /* (year : int) -> int */ 
  kk_integer_t y = kk_integer_sub(year,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t leapdays;
  kk_integer_t _x5885;
  kk_integer_t _x5886;
  kk_integer_t _x5887 = kk_integer_dup(y); /*int*/
  _x5886 = kk_integer_div(_x5887,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t _x5888;
  kk_integer_t _x5889 = kk_integer_dup(y); /*int*/
  _x5888 = kk_integer_div(_x5889,(kk_integer_from_small(100)),kk_context()); /*int*/
  _x5885 = kk_integer_sub(_x5886,_x5888,kk_context()); /*int*/
  kk_integer_t _x5890;
  kk_integer_t _x5891 = kk_integer_dup(y); /*int*/
  _x5890 = kk_integer_div(_x5891,(kk_integer_from_small(400)),kk_context()); /*int*/
  leapdays = kk_integer_add(_x5885,_x5890,kk_context()); /*int*/
  kk_integer_t _x5892 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/
  return kk_integer_add(_x5892,leapdays,kk_context());
}

kk_std_core_types__tuple2_ kk_std_time_calendar_iso_estimate_year(kk_integer_t days, kk_context_t* _ctx) { /* (days : int) -> (int, maybe<int>) */ 
  kk_std_core_types__tuple2_ _match_5750 = kk_integer_div_mod_tuple(days,(kk_integer_from_int(146097, _ctx)),kk_context()); /*(int, int)*/;
  kk_box_t _box_x4998 = _match_5750.fst;
  kk_box_t _box_x4999 = _match_5750.snd;
  kk_integer_t era = kk_integer_unbox(_box_x4998);
  kk_integer_t yoe = kk_integer_unbox(_box_x4999);
  kk_integer_dup(era);
  kk_integer_dup(yoe);
  kk_std_core_types__tuple2__drop(_match_5750, _ctx);
  kk_Unit;
  kk_integer_t _b_5003_5001;
  kk_integer_t _x5895;
  kk_integer_t _x5896 = kk_integer_mul((kk_integer_from_small(400)),era,kk_context()); /*int*/
  _x5895 = kk_integer_add((kk_integer_from_small(1)),_x5896,kk_context()); /*int*/
  kk_integer_t _x5897;
  kk_integer_t _x5898 = kk_integer_mul((kk_integer_from_small(100)),yoe,kk_context()); /*int*/
  _x5897 = kk_integer_div(_x5898,(kk_integer_from_int(36525, _ctx)),kk_context()); /*int*/
  _b_5003_5001 = kk_integer_add(_x5895,_x5897,kk_context()); /*int*/
  kk_std_core_types__maybe _b_5004_5002;
  kk_integer_t _b_5005_5000 = kk_integer_from_small(363); /*int*/;
  _b_5004_5002 = kk_std_core_types__new_Just(kk_integer_box(_b_5005_5000), _ctx); /*maybe<int>*/
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5003_5001), kk_std_core_types__maybe_box(_b_5004_5002, _ctx), _ctx);
}

kk_std_time_timestamp__timestamp kk_std_time_calendar_dc_timestamp(kk_std_time_date__date d, kk_std_time_date__clock c, kk_function_t date_to_days0, kk_std_core_types__optional has_leap_seconds, kk_context_t* _ctx) { /* (d : std/time/date/date, c : std/time/date/clock, date-to-days : (std/time/date/date) -> int, has-leap-seconds : optional<bool>) -> std/time/timestamp/timestamp */ 
  kk_std_core_types__tuple2_ _match_5745;
  if (kk_std_core_types__is_Optional(has_leap_seconds)) {
    kk_box_t _box_x5006 = has_leap_seconds._cons.Optional.value;
    bool _has_leap_seconds_652 = kk_bool_unbox(_box_x5006);
    kk_std_core_types__optional_drop(has_leap_seconds, _ctx);
    kk_Unit;
    if (_has_leap_seconds_652) {
      kk_std_num_ddouble__ddouble y_4614 = kk_std_num_ddouble_ddouble_int_exp(kk_integer_from_small(60), kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/;
      kk_std_core_types__order x0_12793;
      kk_std_num_ddouble__ddouble _x5901;
      struct kk_std_time_date_Clock* _con5902 = kk_std_time_date__as_Clock(c);
      kk_std_num_ddouble__ddouble _x0 = _con5902->seconds;
      _x5901 = _x0; /*std/num/ddouble/ddouble*/
      x0_12793 = kk_std_num_ddouble_compare(_x5901, y_4614, _ctx); /*order*/
      bool _match_5746;
      kk_integer_t _borrowedArg_5748;
      if (kk_std_core_types__is_Lt(x0_12793)) {
        _borrowedArg_5748 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
        goto _match5903;
      }
      if (kk_std_core_types__is_Eq(x0_12793)) {
        _borrowedArg_5748 = kk_integer_from_small(0); /*int*/
        goto _match5903;
      }
      _borrowedArg_5748 = kk_integer_from_small(1); /*int*/
      _match5903: ;
      kk_integer_t _borrowedArg_5747 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/;
      bool _borrowApp_5749 = kk_integer_eq_borrow(_borrowedArg_5748,_borrowedArg_5747,kk_context()); /*bool*/;
      kk_integer_drop(_borrowedArg_5748, _ctx);
      kk_integer_drop(_borrowedArg_5747, _ctx);
      _match_5746 = _borrowApp_5749; /*bool*/
      if (_match_5746) {
        kk_std_num_ddouble__ddouble _b_5019_5007;
        struct kk_std_time_date_Clock* _con5904 = kk_std_time_date__as_Clock(c);
        kk_std_num_ddouble__ddouble _x1 = _con5904->seconds;
        _b_5019_5007 = _x1; /*std/num/ddouble/ddouble*/
        kk_integer_t _b_5020_5008 = kk_integer_from_small(0); /*int*/;
        _match_5745 = kk_std_core_types__new_dash__lp__comma__rp_(kk_std_num_ddouble__ddouble_box(_b_5019_5007, _ctx), kk_integer_box(_b_5020_5008), _ctx); /*(6, 7)*/
        goto _match5899;
      }
      kk_integer_t leap;
      kk_integer_t _x5905;
      kk_std_num_ddouble__ddouble _x5906;
      kk_std_num_ddouble__ddouble _x5907;
      struct kk_std_time_date_Clock* _con5908 = kk_std_time_date__as_Clock(c);
      kk_std_num_ddouble__ddouble _x2 = _con5908->seconds;
      _x5907 = _x2; /*std/num/ddouble/ddouble*/
      _x5906 = kk_std_num_ddouble_floor(_x5907, _ctx); /*std/num/ddouble/ddouble*/
      _x5905 = kk_std_num_ddouble_int(_x5906, kk_std_core_types__new_None(_ctx), _ctx); /*int*/
      leap = kk_integer_sub(_x5905,(kk_integer_from_small(59)),kk_context()); /*int*/
      kk_std_num_ddouble__ddouble _b_5021_5009;
      kk_std_num_ddouble__ddouble y0_4620;
      kk_integer_t _x5909 = kk_integer_dup(leap); /*int*/
      y0_4620 = kk_std_num_ddouble_ddouble_int_exp(_x5909, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
      kk_std_num_ddouble__ddouble _x5910;
      struct kk_std_time_date_Clock* _con5911 = kk_std_time_date__as_Clock(c);
      kk_std_num_ddouble__ddouble _x4 = _con5911->seconds;
      _x5910 = _x4; /*std/num/ddouble/ddouble*/
      kk_std_num_ddouble__ddouble _x5912;
      double _x5913;
      double _x5914;
      double _x3 = y0_4620.hi;
      _x5914 = _x3; /*double*/
      _x5913 = (-_x5914); /*double*/
      double _x5915;
      double _x5916;
      double _x00 = y0_4620.lo;
      _x5916 = _x00; /*double*/
      _x5915 = (-_x5916); /*double*/
      _x5912 = kk_std_num_ddouble__new_Ddouble(_x5913, _x5915, _ctx); /*std/num/ddouble/ddouble*/
      _b_5021_5009 = kk_std_num_ddouble__lp__plus__rp_(_x5910, _x5912, _ctx); /*std/num/ddouble/ddouble*/
      kk_integer_t _b_5022_5010 = leap; /*int*/;
      _match_5745 = kk_std_core_types__new_dash__lp__comma__rp_(kk_std_num_ddouble__ddouble_box(_b_5021_5009, _ctx), kk_integer_box(_b_5022_5010), _ctx); /*(6, 7)*/
      goto _match5899;
    }
    kk_std_num_ddouble__ddouble _b_5023_5011;
    struct kk_std_time_date_Clock* _con5917 = kk_std_time_date__as_Clock(c);
    kk_std_num_ddouble__ddouble _x = _con5917->seconds;
    _b_5023_5011 = _x; /*std/num/ddouble/ddouble*/
    kk_integer_t _b_5024_5012 = kk_integer_from_small(0); /*int*/;
    _match_5745 = kk_std_core_types__new_dash__lp__comma__rp_(kk_std_num_ddouble__ddouble_box(_b_5023_5011, _ctx), kk_integer_box(_b_5024_5012), _ctx); /*(6, 7)*/
    goto _match5899;
  }
  kk_std_num_ddouble__ddouble _b_5025_5013;
  struct kk_std_time_date_Clock* _con5918 = kk_std_time_date__as_Clock(c);
  kk_std_num_ddouble__ddouble _x5 = _con5918->seconds;
  _b_5025_5013 = _x5; /*std/num/ddouble/ddouble*/
  kk_integer_t _b_5026_5014 = kk_integer_from_small(0); /*int*/;
  _match_5745 = kk_std_core_types__new_dash__lp__comma__rp_(kk_std_num_ddouble__ddouble_box(_b_5025_5013, _ctx), kk_integer_box(_b_5026_5014), _ctx); /*(6, 7)*/
  _match5899: ;
  kk_box_t _box_x5015 = _match_5745.fst;
  kk_box_t _box_x5016 = _match_5745.snd;
  kk_std_num_ddouble__ddouble xsecs = kk_std_num_ddouble__ddouble_unbox(_box_x5015, NULL);
  kk_integer_t leap0 = kk_integer_unbox(_box_x5016);
  kk_integer_dup(leap0);
  kk_std_core_types__tuple2__drop(_match_5745, _ctx);
  kk_Unit;
  kk_std_num_ddouble__ddouble secs;
  kk_std_num_ddouble__ddouble _x5921;
  kk_integer_t i1_4623;
  kk_integer_t _x5922;
  kk_integer_t _x5923;
  kk_integer_t _x5924;
  struct kk_std_time_date_Clock* _con5925 = kk_std_time_date__as_Clock(c);
  kk_integer_t _x50 = _con5925->hours;
  kk_std_num_ddouble__ddouble _pat19 = _con5925->seconds;
  kk_integer_dup(_x50);
  _x5924 = _x50; /*int*/
  _x5923 = kk_integer_mul(_x5924,(kk_integer_from_small(60)),kk_context()); /*int*/
  kk_integer_t _x5926;
  struct kk_std_time_date_Clock* _con5927 = kk_std_time_date__as_Clock(c);
  kk_integer_t _pat09 = _con5927->hours;
  kk_integer_t _x6 = _con5927->minutes;
  kk_std_num_ddouble__ddouble _pat1100 = _con5927->seconds;
  if (kk_likely(kk_std_time_date__clock_is_unique(c))) {
    kk_integer_drop(_pat09, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_date__clock_free(c);
  }
  else {
    kk_integer_dup(_x6);
    kk_Unit;
    kk_std_time_date__clock_decref(c, _ctx);
    kk_Unit;
  }
  _x5926 = _x6; /*int*/
  _x5922 = kk_integer_add(_x5923,_x5926,kk_context()); /*int*/
  i1_4623 = kk_integer_mul(_x5922,(kk_integer_from_small(60)),kk_context()); /*int*/
  _x5921 = kk_std_num_ddouble_ddouble_int_exp(i1_4623, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
  secs = kk_std_num_ddouble__lp__plus__rp_(_x5921, xsecs, _ctx); /*std/num/ddouble/ddouble*/
  kk_integer_t days = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), date_to_days0, (date_to_days0, d, _ctx)); /*int*/;
  kk_std_core_types__optional _x5928;
  kk_std_num_ddouble__ddouble _b_5027_5017 = secs; /*std/num/ddouble/ddouble*/;
  _x5928 = kk_std_core_types__new_Optional(kk_std_num_ddouble__ddouble_box(_b_5027_5017, _ctx), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x5929;
  kk_integer_t _b_5028_5018 = leap0; /*int*/;
  _x5929 = kk_std_core_types__new_Optional(kk_integer_box(_b_5028_5018), _ctx); /*optional<106>*/
  return kk_std_time_timestamp_timestamp_days(days, _x5928, _x5929, _ctx);
}

kk_std_core_types__tuple2_ kk_std_time_calendar_earth_timestamp_to_dc(kk_std_time_timestamp__timestamp t, kk_std_num_ddouble__ddouble tzdelta, kk_function_t days_to_date0, kk_context_t* _ctx) { /* (t : std/time/timestamp/timestamp, tzdelta : std/time/timestamp/timespan, days-to-date : (int) -> std/time/date/date) -> (std/time/date/date, std/time/date/clock) */ 
  kk_std_time_timestamp__timestamp ts_4626;
  kk_std_num_ddouble__ddouble _x5930;
  kk_std_num_ddouble__ddouble _x5931;
  kk_std_num_ddouble__ddouble _x0 = t.since;
  _x5931 = _x0; /*std/time/timestamp/timespan*/
  _x5930 = kk_std_num_ddouble__lp__plus__rp_(_x5931, tzdelta, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x5932;
  kk_std_num_ddouble__ddouble _pat000 = t.since;
  int32_t _x00 = t.leap32;
  _x5932 = _x00; /*int32*/
  ts_4626 = kk_std_time_timestamp__new_Timestamp(_x5930, _x5932, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_core_types__tuple2_ _match_5743 = kk_std_time_timestamp_days_seconds(ts_4626, _ctx); /*(int, std/num/ddouble/ddouble)*/;
  kk_box_t _box_x5029 = _match_5743.fst;
  kk_box_t _box_x5030 = _match_5743.snd;
  kk_integer_t days0 = kk_integer_unbox(_box_x5029);
  kk_std_num_ddouble__ddouble secs = kk_std_num_ddouble__ddouble_unbox(_box_x5030, NULL);
  kk_integer_dup(days0);
  kk_std_core_types__tuple2__drop(_match_5743, _ctx);
  kk_Unit;
  kk_std_time_date__clock _match_5744;
  kk_integer_t _x5935;
  int32_t _x5936;
  kk_std_num_ddouble__ddouble _pat00 = ts_4626.since;
  int32_t _x = ts_4626.leap32;
  _x5936 = _x; /*int32*/
  _x5935 = kk_integer_from_int(_x5936,kk_context()); /*int*/
  _match_5744 = kk_std_time_date_clock_3(secs, _x5935, _ctx); /*std/time/date/clock*/
  struct kk_std_time_date_Clock* _con5937 = kk_std_time_date__as_Clock(_match_5744);
  kk_std_time_date__date _b_5033_5031 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0, (days_to_date0, days0, _ctx)); /*std/time/date/date*/;
  kk_std_time_date__clock _b_5034_5032 = _match_5744; /*std/time/date/clock*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5033_5031, _ctx), kk_std_time_date__clock_box(_b_5034_5032, _ctx), _ctx);
}
 
// Return the days between two dates interpreted by calendar `cal`.

kk_integer_t kk_std_time_calendar_days_until(kk_std_time_calendar__calendar cal, kk_std_time_date__date d1, kk_std_time_date__date d2, kk_context_t* _ctx) { /* (cal : calendar, d1 : std/time/date/date, d2 : std/time/date/date) -> int */ 
  kk_integer_t _x5938;
  kk_function_t _x5939;
  struct kk_std_time_calendar_Calendar* _con5940 = kk_std_time_calendar__as_Calendar(cal);
  kk_function_t _x0 = _con5940->date_to_days;
  kk_function_dup(_x0);
  _x5939 = _x0; /*(date : std/time/date/date) -> int*/
  _x5938 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x5939, (_x5939, d2, _ctx)); /*int*/
  kk_integer_t _x5941;
  kk_function_t _x5942;
  struct kk_std_time_calendar_Calendar* _con5943 = kk_std_time_calendar__as_Calendar(cal);
  kk_string_t _pat0 = _con5943->name;
  kk_string_t _pat1 = _con5943->long_name;
  kk_string_t _pat2 = _con5943->month_prefix;
  kk_function_t _pat3 = _con5943->show_era;
  kk_function_t _pat4 = _con5943->instant_to_dc;
  kk_function_t _pat5 = _con5943->dc_to_instant;
  kk_function_t _pat6 = _con5943->days_to_date;
  kk_function_t _x = _con5943->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(cal))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(cal);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(cal, _ctx);
    kk_Unit;
  }
  _x5942 = _x; /*(date : std/time/date/date) -> int*/
  _x5941 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x5942, (_x5942, d1, _ctx)); /*int*/
  return kk_integer_sub(_x5938,_x5941,kk_context());
}
 
// Get the current local timezone structure.
// The timezone name is allowed to be the empty string.

kk_std_time_calendar__local_timezone kk_std_time_calendar_local_get_timezone(kk_context_t* _ctx) { /* () -> ndet local-timezone */ 
  return kk_local_get_timezone(kk_context());
}
 
// Return the utc delta in fractional seconds given a local timezone structure
// and time `i` in fractional seconds since the UNIX epoch (1970-01-01).
// Also returns time zone abbreviation.

kk_std_core_types__tuple2_ kk_std_time_calendar_local_utc_delta(kk_std_time_calendar__local_timezone tz, double i, kk_context_t* _ctx) { /* (tz : local-timezone, i : double) -> (double, string) */ 
  return kk_local_get_utc_delta_tuple(tz,i,kk_context());
}
extern kk_std_core_types__maybe kk_std_time_calendar__create_Timezone_fun5945(kk_function_t _fself, kk_std_time_instant__instant _b_5041, kk_context_t* _ctx) {
  struct kk_std_time_calendar__create_Timezone_fun5945__t* _self = kk_function_as(struct kk_std_time_calendar__create_Timezone_fun5945__t*, _fself);
  kk_box_t _fun_unbox_x5038 = _self->_fun_unbox_x5038; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5038);}, {}, _ctx)
  kk_box_t _x5946;
  kk_function_t _x5947 = kk_function_unbox(_fun_unbox_x5038); /*(5039) -> 5040*/
  _x5946 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x5947, (_x5947, kk_std_time_instant__instant_box(_b_5041, _ctx), _ctx)); /*5040*/
  return kk_std_core_types__maybe_unbox(_x5946, _ctx);
}
extern kk_std_core_types__maybe kk_std_time_calendar__create_Timezone_fun5948(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}
 
// Check if two calendars use the same date calculations. (Display of era names etc. may differ)

bool kk_std_time_calendar__lp__eq__eq__rp_(kk_std_time_calendar__calendar c1, kk_std_time_calendar__calendar c2, kk_context_t* _ctx) { /* (c1 : calendar, c2 : calendar) -> bool */ 
  kk_string_t _x5949;
  struct kk_std_time_calendar_Calendar* _con5950 = kk_std_time_calendar__as_Calendar(c1);
  kk_string_t _pat0 = _con5950->name;
  kk_string_t _x = _con5950->long_name;
  kk_string_t _pat1 = _con5950->month_prefix;
  kk_function_t _pat2 = _con5950->show_era;
  kk_function_t _pat3 = _con5950->instant_to_dc;
  kk_function_t _pat4 = _con5950->dc_to_instant;
  kk_function_t _pat5 = _con5950->days_to_date;
  kk_function_t _pat6 = _con5950->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(c1))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_function_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(c1);
  }
  else {
    kk_string_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(c1, _ctx);
    kk_Unit;
  }
  _x5949 = _x; /*string*/
  kk_string_t _x5951;
  struct kk_std_time_calendar_Calendar* _con5952 = kk_std_time_calendar__as_Calendar(c2);
  kk_string_t _pat00 = _con5952->name;
  kk_string_t _x0 = _con5952->long_name;
  kk_string_t _pat10 = _con5952->month_prefix;
  kk_function_t _pat20 = _con5952->show_era;
  kk_function_t _pat30 = _con5952->instant_to_dc;
  kk_function_t _pat40 = _con5952->dc_to_instant;
  kk_function_t _pat50 = _con5952->days_to_date;
  kk_function_t _pat60 = _con5952->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(c2))) {
    kk_function_drop(_pat60, _ctx);
    kk_Unit;
    kk_function_drop(_pat50, _ctx);
    kk_Unit;
    kk_function_drop(_pat40, _ctx);
    kk_Unit;
    kk_function_drop(_pat30, _ctx);
    kk_Unit;
    kk_function_drop(_pat20, _ctx);
    kk_Unit;
    kk_string_drop(_pat10, _ctx);
    kk_Unit;
    kk_string_drop(_pat00, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(c2);
  }
  else {
    kk_string_dup(_x0);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(c2, _ctx);
    kk_Unit;
  }
  _x5951 = _x0; /*string*/
  return kk_string_is_eq(_x5949,_x5951,kk_context());
}

bool kk_std_time_calendar_iso_is_leap(kk_integer_t year, kk_context_t* _ctx) { /* (year : int) -> bool */ 
  bool _match_5732;
  kk_integer_t _borrowedArg_5741;
  kk_integer_t _x5955 = kk_integer_dup(year); /*int*/
  _borrowedArg_5741 = kk_integer_mod(_x5955,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t _borrowedArg_5740 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_5742 = kk_integer_eq_borrow(_borrowedArg_5741,_borrowedArg_5740,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5741, _ctx);
  kk_integer_drop(_borrowedArg_5740, _ctx);
  _match_5732 = _borrowApp_5742; /*bool*/
  if (_match_5732) {
    bool _match_5733;
    kk_integer_t _borrowedArg_5738;
    kk_integer_t _x5956 = kk_integer_dup(year); /*int*/
    _borrowedArg_5738 = kk_integer_mod(_x5956,(kk_integer_from_small(100)),kk_context()); /*int*/
    kk_integer_t _borrowedArg_5737 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_5739 = kk_integer_neq_borrow(_borrowedArg_5738,_borrowedArg_5737,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_5738, _ctx);
    kk_integer_drop(_borrowedArg_5737, _ctx);
    _match_5733 = _borrowApp_5739; /*bool*/
    if (_match_5733) {
      kk_integer_drop(year, _ctx);
      return true;
    }
    kk_integer_t _borrowedArg_5735 = kk_integer_mod(year,(kk_integer_from_small(400)),kk_context()); /*int*/;
    kk_integer_t _borrowedArg_5734 = kk_integer_from_small(0); /*int*/;
    bool _borrowApp_5736 = kk_integer_eq_borrow(_borrowedArg_5735,_borrowedArg_5734,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_5735, _ctx);
    kk_integer_drop(_borrowedArg_5734, _ctx);
    return _borrowApp_5736;
  }
  kk_integer_drop(year, _ctx);
  return false;
}

kk_integer_t kk_std_time_calendar_iso_days_before_month(kk_integer_t year, kk_integer_t month, kk_context_t* _ctx) { /* (year : int, month : int) -> int */ 
  bool is_before_march_4635;
  kk_integer_t _borrowedArg_5729 = kk_integer_from_small(2); /*int*/;
  bool _borrowApp_5730 = kk_integer_lte_borrow(month,_borrowedArg_5729,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5729, _ctx);
  is_before_march_4635 = _borrowApp_5730; /*bool*/
  kk_integer_t adj;
  if (is_before_march_4635) {
    kk_integer_drop(year, _ctx);
    adj = kk_integer_from_small(0); /*int*/
  }
  else {
    bool _match_5728 = kk_std_time_calendar_iso_is_leap(year, _ctx); /*bool*/;
    if (_match_5728) {
      adj = kk_integer_from_small(1); /*int*/
    }
    else {
      adj = kk_integer_from_small(2); /*int*/
    }
  }
  kk_integer_t _x5957;
  kk_integer_t _x5958;
  kk_integer_t _x5959 = kk_integer_mul((kk_integer_from_small(367)),month,kk_context()); /*int*/
  _x5958 = kk_integer_sub(_x5959,(kk_integer_from_small(362)),kk_context()); /*int*/
  _x5957 = kk_integer_div(_x5958,(kk_integer_from_small(12)),kk_context()); /*int*/
  return kk_integer_sub(_x5957,adj,kk_context());
}

kk_integer_t kk_std_time_calendar_iso_doy_to_month(kk_integer_t year, kk_integer_t doy, kk_context_t* _ctx) { /* (year : int, doy : int) -> int */ 
  bool is_before_march_4637;
  kk_integer_t _borrowedArg_5726 = kk_integer_from_small(58); /*int*/;
  bool _borrowApp_5727 = kk_integer_lte_borrow(doy,_borrowedArg_5726,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5726, _ctx);
  is_before_march_4637 = _borrowApp_5727; /*bool*/
  kk_integer_t adj;
  if (is_before_march_4637) {
    kk_integer_drop(year, _ctx);
    adj = kk_integer_from_small(0); /*int*/
  }
  else {
    bool _match_5725 = kk_std_time_calendar_iso_is_leap(year, _ctx); /*bool*/;
    if (_match_5725) {
      adj = kk_integer_from_small(1); /*int*/
    }
    else {
      adj = kk_integer_from_small(2); /*int*/
    }
  }
  kk_integer_t _x5960;
  kk_integer_t _x5961;
  kk_integer_t _x5962 = kk_integer_add(doy,adj,kk_context()); /*int*/
  _x5961 = kk_integer_mul((kk_integer_from_small(12)),_x5962,kk_context()); /*int*/
  _x5960 = kk_integer_add(_x5961,(kk_integer_from_small(373)),kk_context()); /*int*/
  return kk_integer_div(_x5960,(kk_integer_from_small(367)),kk_context());
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5963__t_1 {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5963_1(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5963_1(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5963__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5963__t_1, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5963_1, kk_context());
  _self->abbrv = abbrv;
  _self->delta = delta;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun5963_1(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5963__t_1* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5963__t_1*, _fself);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv);kk_std_time_duration__duration_dup(delta);}, {}, _ctx)
  kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5046_5043 = delta; /*std/time/duration/duration*/;
  kk_string_t _b_5047_5044;
  if (kk_std_core_types__is_Optional(abbrv)) {
    kk_box_t _box_x5042 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_2653 = kk_string_unbox(_box_x5042);
    kk_string_dup(_abbrv_2653);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    kk_Unit;
    _b_5047_5044 = _abbrv_2653; /*string*/
    goto _match5964;
  }
  _b_5047_5044 = kk_string_empty(); /*string*/
  _match5964: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5046_5043, _ctx), kk_string_box(_b_5047_5044), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun5967__t_1 {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5967_1(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun5967_1(kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5967__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun5967__t_1, 1, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun5967_1, kk_context());
  _self->delta = delta;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun5967_1(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun5967__t_1* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun5967__t_1*, _fself);
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta);}, {}, _ctx)
  bool _match_5724;
  double _x5968;
  kk_std_num_ddouble__ddouble _x0 = delta.secs;
  double _x = _x0.hi;
  _x5968 = _x; /*double*/
  _match_5724 = (_x5968 == (0.0)); /*bool*/
  if (_match_5724) {
    kk_std_time_instant__instant _b_5048_5045 = i00; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_5048_5045, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i00, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_fixed_1(kk_string_t name0, kk_std_time_duration__duration delta, kk_std_core_types__optional abbrv, kk_context_t* _ctx) { /* (name : string, delta : std/time/duration/duration, abbrv : optional<string>) -> timezone */ 
  return kk_std_time_calendar__new_Timezone(name0, kk_std_time_calendar_new_tz_fixed_fun5963_1(abbrv, delta, _ctx), kk_std_time_calendar_new_tz_fixed_fun5967_1(delta, _ctx), _ctx);
}
 
// The standard UTC time zone with a 0 delta.


// lift anonymous function
struct kk_std_time_calendar_tz_utc_fun5971__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_utc_fun5971(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_utc_fun5971(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_utc_fun5971, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_utc_fun5971(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5053_5050 = kk_std_time_duration_zero; /*std/time/duration/duration*/;
  kk_string_t _b_5054_5051;
  kk_std_core_types__optional _match_5723 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5723)) {
    kk_box_t _box_x5049 = _match_5723._cons.Optional.value;
    kk_string_t _abbrv_2653 = kk_string_unbox(_box_x5049);
    kk_string_dup(_abbrv_2653);
    kk_std_core_types__optional_drop(_match_5723, _ctx);
    kk_Unit;
    _b_5054_5051 = _abbrv_2653; /*string*/
    goto _match5972;
  }
  _b_5054_5051 = kk_string_empty(); /*string*/
  _match5972: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5053_5050, _ctx), kk_string_box(_b_5054_5051), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_utc_fun5975__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_utc_fun5975(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_utc_fun5975(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_utc_fun5975, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_utc_fun5975(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  bool _match_5722;
  double _x5976;
  kk_std_time_duration__duration _x5977 = kk_std_time_duration_zero; /*std/time/duration/duration*/
  kk_std_num_ddouble__ddouble _x0 = _x5977.secs;
  double _x = _x0.hi;
  _x5976 = _x; /*double*/
  _match_5722 = (_x5976 == (0.0)); /*bool*/
  if (_match_5722) {
    kk_std_time_instant__instant _b_5055_5052 = i00; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_5055_5052, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i00, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_utc;
 
// Create a time zone with a fixed number of hours and optional minutes (=`0`) difference from UTC/GMT.
// The time zone name and abbreviation is optional, and the default name
// will reflect the offset from UTC, for example `"UTC+1:20"`, or `"UTC-8"`.
// The default abbreviation is the empty string.
// The sign of the minutes will be matched to the hours (unless `hours` is `0`).
// For example, `tz-fixed(-1,30)` gives a time zone offset of -90 minutes, "UTC-1:30".
// Returns `tz-utc` if both `hours` and `mins` are zero.


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun6036__t {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun6036(kk_function_t _fself, kk_std_time_instant__instant i1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun6036(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6036__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun6036__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun6036, kk_context());
  _self->abbrv = abbrv;
  _self->delta = delta;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun6036(kk_function_t _fself, kk_std_time_instant__instant i1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6036__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun6036__t*, _fself);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv);kk_std_time_duration__duration_dup(delta);}, {}, _ctx)
  kk_std_time_instant__instant_dropn(i1, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5081_5066 = delta; /*std/time/duration/duration*/;
  kk_string_t _b_5082_5067;
  if (kk_std_core_types__is_Optional(abbrv)) {
    kk_box_t _box_x5065 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_2271 = kk_string_unbox(_box_x5065);
    kk_string_dup(_abbrv_2271);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    kk_Unit;
    _b_5082_5067 = _abbrv_2271; /*string*/
    goto _match6037;
  }
  _b_5082_5067 = kk_string_empty(); /*string*/
  _match6037: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5081_5066, _ctx), kk_string_box(_b_5082_5067), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun6040__t {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun6040(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun6040(kk_std_time_duration__duration delta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6040__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun6040__t, 1, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun6040, kk_context());
  _self->delta = delta;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun6040(kk_function_t _fself, kk_std_time_instant__instant i00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6040__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun6040__t*, _fself);
  kk_std_time_duration__duration delta = _self->delta; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta);}, {}, _ctx)
  bool _match_5712;
  double _x6041;
  kk_std_num_ddouble__ddouble _x0 = delta.secs;
  double _x = _x0.hi;
  _x6041 = _x; /*double*/
  _match_5712 = (_x6041 == (0.0)); /*bool*/
  if (_match_5712) {
    kk_std_time_instant__instant _b_5083_5068 = i00; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_5083_5068, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i00, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun6098__t {
  struct kk_function_s _base;
  kk_std_core_types__optional abbrv;
  kk_std_time_duration__duration delta1;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun6098(kk_function_t _fself, kk_std_time_instant__instant i4, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun6098(kk_std_core_types__optional abbrv, kk_std_time_duration__duration delta1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6098__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun6098__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun6098, kk_context());
  _self->abbrv = abbrv;
  _self->delta1 = delta1;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_fixed_fun6098(kk_function_t _fself, kk_std_time_instant__instant i4, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6098__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun6098__t*, _fself);
  kk_std_core_types__optional abbrv = _self->abbrv; /* optional<string> */
  kk_std_time_duration__duration delta1 = _self->delta1; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(abbrv);kk_std_time_duration__duration_dup(delta1);}, {}, _ctx)
  kk_std_time_instant__instant_dropn(i4, ((int32_t)1), _ctx);
  kk_std_time_duration__duration _b_5084_5078 = delta1; /*std/time/duration/duration*/;
  kk_string_t _b_5085_5079;
  if (kk_std_core_types__is_Optional(abbrv)) {
    kk_box_t _box_x5077 = abbrv._cons.Optional.value;
    kk_string_t _abbrv_22710 = kk_string_unbox(_box_x5077);
    kk_string_dup(_abbrv_22710);
    kk_std_core_types__optional_drop(abbrv, _ctx);
    kk_Unit;
    _b_5085_5079 = _abbrv_22710; /*string*/
    goto _match6099;
  }
  _b_5085_5079 = kk_string_empty(); /*string*/
  _match6099: ;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5084_5078, _ctx), kk_string_box(_b_5085_5079), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_fixed_fun6102__t {
  struct kk_function_s _base;
  kk_std_time_duration__duration delta1;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun6102(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_fixed_fun6102(kk_std_time_duration__duration delta1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6102__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_fixed_fun6102__t, 1, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_fixed_fun6102, kk_context());
  _self->delta1 = delta1;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_fixed_fun6102(kk_function_t _fself, kk_std_time_instant__instant i01, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_fixed_fun6102__t* _self = kk_function_as(struct kk_std_time_calendar_tz_fixed_fun6102__t*, _fself);
  kk_std_time_duration__duration delta1 = _self->delta1; /* std/time/duration/duration */
  kk_drop_match(_self, {kk_std_time_duration__duration_dup(delta1);}, {}, _ctx)
  bool _match_5701;
  double _x6103;
  kk_std_num_ddouble__ddouble _x00 = delta1.secs;
  double _x1 = _x00.hi;
  _x6103 = _x1; /*double*/
  _match_5701 = (_x6103 == (0.0)); /*bool*/
  if (_match_5701) {
    kk_std_time_instant__instant _b_5086_5080 = i01; /*std/time/instant/instant*/;
    return kk_std_core_types__new_Just(kk_std_time_instant__instant_box(_b_5086_5080, _ctx), _ctx);
  }
  kk_std_time_instant__instant_dropn(i01, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_fixed(kk_integer_t hours, kk_std_core_types__optional mins, kk_std_core_types__optional name0, kk_std_core_types__optional abbrv, kk_std_core_types__optional hourwidth, kk_context_t* _ctx) { /* (hours : int, mins : optional<int>, name : optional<string>, abbrv : optional<string>, hourwidth : optional<int>) -> timezone */ 
  kk_integer_t _mins_2262;
  if (kk_std_core_types__is_Optional(mins)) {
    kk_box_t _box_x5056 = mins._cons.Optional.value;
    kk_integer_t _mins_2263 = kk_integer_unbox(_box_x5056);
    kk_integer_dup(_mins_2263);
    kk_std_core_types__optional_drop(mins, _ctx);
    kk_Unit;
    _mins_2262 = _mins_2263; /*int*/
    goto _match5978;
  }
  _mins_2262 = kk_integer_from_small(0); /*int*/
  _match5978: ;
  bool _match_5700 = kk_integer_is_zero_borrow(hours); /*bool*/;
  if (_match_5700) {
    bool _match_5711 = kk_integer_is_zero_borrow(_mins_2262); /*bool*/;
    if (_match_5711) {
      kk_integer_drop(_mins_2262, _ctx);
      kk_std_core_types__optional_drop(abbrv, _ctx);
      kk_integer_drop(hours, _ctx);
      kk_std_core_types__optional_drop(hourwidth, _ctx);
      kk_std_core_types__optional_drop(name0, _ctx);
      return kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc);
    }
    kk_integer_t xmins;
    bool _match_5720;
    kk_integer_t _x5980 = kk_integer_dup(hours); /*int*/
    _match_5720 = kk_std_core_is_pos_2(_x5980, _ctx); /*bool*/
    if (_match_5720) {
      kk_integer_t _x5981 = kk_integer_dup(_mins_2262); /*int*/
      xmins = kk_integer_abs(_x5981,kk_context()); /*int*/
    }
    else {
      bool _match_5721;
      kk_integer_t _x5982 = kk_integer_dup(hours); /*int*/
      _match_5721 = kk_std_core_is_neg_2(_x5982, _ctx); /*bool*/
      if (_match_5721) {
        kk_integer_t i_4650;
        kk_integer_t _x5983 = kk_integer_dup(_mins_2262); /*int*/
        i_4650 = kk_integer_abs(_x5983,kk_context()); /*int*/
        xmins = kk_integer_neg(i_4650,kk_context()); /*int*/
      }
      else {
        xmins = kk_integer_dup(_mins_2262); /*int*/
      }
    }
    kk_integer_t secs;
    kk_integer_t _x5984;
    kk_integer_t _x5985 = kk_integer_dup(hours); /*int*/
    _x5984 = kk_integer_mul(_x5985,(kk_integer_from_int(3600, _ctx)),kk_context()); /*int*/
    kk_integer_t _x5986 = kk_integer_mul(xmins,(kk_integer_from_small(60)),kk_context()); /*int*/
    secs = kk_integer_add(_x5984,_x5986,kk_context()); /*int*/
    kk_string_t y_4652;
    kk_string_t _x5987;
    kk_define_string_literal(, _s5988, 3, "UTC")
    _x5987 = kk_string_dup(_s5988); /*string*/
    kk_string_t _x5989;
    kk_string_t _x5990;
    bool _match_5719;
    kk_integer_t _x5991 = kk_integer_dup(secs); /*int*/
    _match_5719 = kk_std_core_is_neg_2(_x5991, _ctx); /*bool*/
    if (_match_5719) {
      kk_define_string_literal(, _s5992, 1, "-")
      _x5990 = kk_string_dup(_s5992); /*string*/
    }
    else {
      kk_define_string_literal(, _s5993, 1, "+")
      _x5990 = kk_string_dup(_s5993); /*string*/
    }
    kk_string_t _x5994;
    kk_string_t _x5995;
    kk_string_t s_4653;
    kk_integer_t _x5996 = kk_integer_abs(hours,kk_context()); /*int*/
    s_4653 = kk_std_core_show(_x5996, _ctx); /*string*/
    kk_ssize_t w;
    kk_integer_t _x5997;
    if (kk_std_core_types__is_Optional(hourwidth)) {
      kk_box_t _box_x5057 = hourwidth._cons.Optional.value;
      kk_integer_t _hourwidth_2275 = kk_integer_unbox(_box_x5057);
      kk_integer_dup(_hourwidth_2275);
      kk_std_core_types__optional_drop(hourwidth, _ctx);
      kk_Unit;
      _x5997 = _hourwidth_2275; /*int*/
      goto _match5998;
    }
    _x5997 = kk_integer_from_small(1); /*int*/
    _match5998: ;
    w = kk_std_core_ssize__t(_x5997, _ctx); /*ssize_t*/
    kk_ssize_t n;
    kk_string_t _x6000 = kk_string_dup(s_4653); /*string*/
    n = kk_string_len(_x6000,kk_context()); /*ssize_t*/
    bool _match_5718 = (w <= n); /*bool*/;
    if (_match_5718) {
      _x5995 = s_4653; /*string*/
    }
    else {
      kk_string_t _x6001;
      kk_string_t _x6002 = kk_std_core_string('0', _ctx); /*string*/
      kk_ssize_t _x6003 = (w - n); /*ssize_t*/
      _x6001 = kk_std_core_repeatz(_x6002, _x6003, _ctx); /*string*/
      _x5995 = kk_std_core__lp__plus__plus__1_rp_(_x6001, s_4653, _ctx); /*string*/
    }
    kk_string_t _x6004;
    bool _match_5716 = kk_integer_is_zero_borrow(_mins_2262); /*bool*/;
    if (_match_5716) {
      kk_integer_drop(_mins_2262, _ctx);
      _x6004 = kk_string_empty(); /*string*/
    }
    else {
      kk_string_t _x6006;
      kk_define_string_literal(, _s6007, 1, ":")
      _x6006 = kk_string_dup(_s6007); /*string*/
      kk_string_t _x6008;
      kk_string_t s0_4656;
      kk_integer_t _x6009 = kk_integer_abs(_mins_2262,kk_context()); /*int*/
      s0_4656 = kk_std_core_show(_x6009, _ctx); /*string*/
      kk_ssize_t w0 = ((kk_ssize_t)2); /*ssize_t*/;
      kk_ssize_t n0;
      kk_string_t _x6010 = kk_string_dup(s0_4656); /*string*/
      n0 = kk_string_len(_x6010,kk_context()); /*ssize_t*/
      bool _match_5717 = (w0 <= n0); /*bool*/;
      if (_match_5717) {
        _x6008 = s0_4656; /*string*/
      }
      else {
        kk_string_t _x6011;
        kk_string_t _x6012 = kk_std_core_string('0', _ctx); /*string*/
        kk_ssize_t _x6013 = (w0 - n0); /*ssize_t*/
        _x6011 = kk_std_core_repeatz(_x6012, _x6013, _ctx); /*string*/
        _x6008 = kk_std_core__lp__plus__plus__1_rp_(_x6011, s0_4656, _ctx); /*string*/
      }
      _x6004 = kk_std_core__lp__plus__plus__1_rp_(_x6006, _x6008, _ctx); /*string*/
    }
    _x5994 = kk_std_core__lp__plus__plus__1_rp_(_x5995, _x6004, _ctx); /*string*/
    _x5989 = kk_std_core__lp__plus__plus__1_rp_(_x5990, _x5994, _ctx); /*string*/
    y_4652 = kk_std_core__lp__plus__plus__1_rp_(_x5987, _x5989, _ctx); /*string*/
    kk_string_t tzonename;
    bool _match_5715;
    kk_string_t _x6014;
    if (kk_std_core_types__is_Optional(name0)) {
      kk_box_t _box_x5058 = name0._cons.Optional.value;
      kk_string_t _name_2267 = kk_string_unbox(_box_x5058);
      kk_string_dup(_name_2267);
      _x6014 = _name_2267; /*string*/
      goto _match6015;
    }
    _x6014 = kk_string_empty(); /*string*/
    _match6015: ;
    kk_string_t _x6018 = kk_string_empty(); /*string*/
    _match_5715 = kk_string_is_eq(_x6014,_x6018,kk_context()); /*bool*/
    if (_match_5715) {
      kk_std_core_types__optional_drop(name0, _ctx);
      tzonename = y_4652; /*string*/
    }
    else {
      kk_string_drop(y_4652, _ctx);
      if (kk_std_core_types__is_Optional(name0)) {
        kk_box_t _box_x5059 = name0._cons.Optional.value;
        kk_string_t _name_22670 = kk_string_unbox(_box_x5059);
        kk_string_dup(_name_22670);
        kk_std_core_types__optional_drop(name0, _ctx);
        kk_Unit;
        tzonename = _name_22670; /*string*/
        goto _match6020;
      }
      tzonename = kk_string_empty(); /*string*/
      _match6020: ;
    }
    kk_std_time_duration__duration delta;
    kk_std_num_ddouble__ddouble _x6023;
    kk_std_core_types__optional frac0_1361;
    kk_box_t _x6024;
    double _x6025;
    kk_std_core_types__optional _match_5714 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5714)) {
      kk_box_t _box_x5060 = _match_5714._cons.Optional.value;
      double _frac_162 = kk_double_unbox(_box_x5060, NULL);
      kk_std_core_types__optional_drop(_match_5714, _ctx);
      kk_Unit;
      _x6025 = _frac_162; /*double*/
      goto _match6026;
    }
    _x6025 = 0.0; /*double*/
    _match6026: ;
    _x6024 = kk_double_box(_x6025, _ctx); /*106*/
    frac0_1361 = kk_std_core_types__new_Optional(_x6024, _ctx); /*optional<double>*/
    bool _match_5713;
    double _x6028;
    if (kk_std_core_types__is_Optional(frac0_1361)) {
      kk_box_t _box_x5063 = frac0_1361._cons.Optional.value;
      double _frac_788 = kk_double_unbox(_box_x5063, NULL);
      _x6028 = _frac_788; /*double*/
      goto _match6029;
    }
    _x6028 = 0.0; /*double*/
    _match6029: ;
    _match_5713 = (_x6028 == (0.0)); /*bool*/
    if (_match_5713) {
      kk_std_core_types__optional_drop(frac0_1361, _ctx);
      _x6023 = kk_std_num_ddouble_ddouble_int_exp(secs, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
    }
    else {
      kk_std_num_ddouble__ddouble _x6031 = kk_std_num_ddouble_ddouble_int_exp(secs, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
      kk_std_num_ddouble__ddouble _x6032;
      double _x6033;
      if (kk_std_core_types__is_Optional(frac0_1361)) {
        kk_box_t _box_x5064 = frac0_1361._cons.Optional.value;
        double _frac_7880 = kk_double_unbox(_box_x5064, NULL);
        kk_std_core_types__optional_drop(frac0_1361, _ctx);
        kk_Unit;
        _x6033 = _frac_7880; /*double*/
        goto _match6034;
      }
      _x6033 = 0.0; /*double*/
      _match6034: ;
      _x6032 = kk_std_num_ddouble__new_Ddouble(_x6033, 0.0, _ctx); /*std/num/ddouble/ddouble*/
      _x6023 = kk_std_num_ddouble__lp__plus__rp_(_x6031, _x6032, _ctx); /*std/num/ddouble/ddouble*/
    }
    delta = kk_std_time_duration__new_Duration(_x6023, _ctx); /*std/time/duration/duration*/
    return kk_std_time_calendar__new_Timezone(tzonename, kk_std_time_calendar_new_tz_fixed_fun6036(abbrv, delta, _ctx), kk_std_time_calendar_new_tz_fixed_fun6040(delta, _ctx), _ctx);
  }
  kk_integer_t xmins0;
  bool _match_5709;
  kk_integer_t _x6042 = kk_integer_dup(hours); /*int*/
  _match_5709 = kk_std_core_is_pos_2(_x6042, _ctx); /*bool*/
  if (_match_5709) {
    kk_integer_t _x6043 = kk_integer_dup(_mins_2262); /*int*/
    xmins0 = kk_integer_abs(_x6043,kk_context()); /*int*/
  }
  else {
    bool _match_5710;
    kk_integer_t _x6044 = kk_integer_dup(hours); /*int*/
    _match_5710 = kk_std_core_is_neg_2(_x6044, _ctx); /*bool*/
    if (_match_5710) {
      kk_integer_t i2_4668;
      kk_integer_t _x6045 = kk_integer_dup(_mins_2262); /*int*/
      i2_4668 = kk_integer_abs(_x6045,kk_context()); /*int*/
      xmins0 = kk_integer_neg(i2_4668,kk_context()); /*int*/
    }
    else {
      xmins0 = kk_integer_dup(_mins_2262); /*int*/
    }
  }
  kk_integer_t secs1;
  kk_integer_t _x6046;
  kk_integer_t _x6047 = kk_integer_dup(hours); /*int*/
  _x6046 = kk_integer_mul(_x6047,(kk_integer_from_int(3600, _ctx)),kk_context()); /*int*/
  kk_integer_t _x6048 = kk_integer_mul(xmins0,(kk_integer_from_small(60)),kk_context()); /*int*/
  secs1 = kk_integer_add(_x6046,_x6048,kk_context()); /*int*/
  kk_string_t y0_4670;
  kk_string_t _x6049;
  kk_define_string_literal(, _s6050, 3, "UTC")
  _x6049 = kk_string_dup(_s6050); /*string*/
  kk_string_t _x6051;
  kk_string_t _x6052;
  bool _match_5708;
  kk_integer_t _x6053 = kk_integer_dup(secs1); /*int*/
  _match_5708 = kk_std_core_is_neg_2(_x6053, _ctx); /*bool*/
  if (_match_5708) {
    kk_define_string_literal(, _s6054, 1, "-")
    _x6052 = kk_string_dup(_s6054); /*string*/
  }
  else {
    kk_define_string_literal(, _s6055, 1, "+")
    _x6052 = kk_string_dup(_s6055); /*string*/
  }
  kk_string_t _x6056;
  kk_string_t _x6057;
  kk_string_t s1_4671;
  kk_integer_t _x6058 = kk_integer_abs(hours,kk_context()); /*int*/
  s1_4671 = kk_std_core_show(_x6058, _ctx); /*string*/
  kk_ssize_t w1;
  kk_integer_t _x6059;
  if (kk_std_core_types__is_Optional(hourwidth)) {
    kk_box_t _box_x5069 = hourwidth._cons.Optional.value;
    kk_integer_t _hourwidth_22750 = kk_integer_unbox(_box_x5069);
    kk_integer_dup(_hourwidth_22750);
    kk_std_core_types__optional_drop(hourwidth, _ctx);
    kk_Unit;
    _x6059 = _hourwidth_22750; /*int*/
    goto _match6060;
  }
  _x6059 = kk_integer_from_small(1); /*int*/
  _match6060: ;
  w1 = kk_std_core_ssize__t(_x6059, _ctx); /*ssize_t*/
  kk_ssize_t n1;
  kk_string_t _x6062 = kk_string_dup(s1_4671); /*string*/
  n1 = kk_string_len(_x6062,kk_context()); /*ssize_t*/
  bool _match_5707 = (w1 <= n1); /*bool*/;
  if (_match_5707) {
    _x6057 = s1_4671; /*string*/
  }
  else {
    kk_string_t _x6063;
    kk_string_t _x6064 = kk_std_core_string('0', _ctx); /*string*/
    kk_ssize_t _x6065 = (w1 - n1); /*ssize_t*/
    _x6063 = kk_std_core_repeatz(_x6064, _x6065, _ctx); /*string*/
    _x6057 = kk_std_core__lp__plus__plus__1_rp_(_x6063, s1_4671, _ctx); /*string*/
  }
  kk_string_t _x6066;
  bool _match_5705 = kk_integer_is_zero_borrow(_mins_2262); /*bool*/;
  if (_match_5705) {
    kk_integer_drop(_mins_2262, _ctx);
    _x6066 = kk_string_empty(); /*string*/
  }
  else {
    kk_string_t _x6068;
    kk_define_string_literal(, _s6069, 1, ":")
    _x6068 = kk_string_dup(_s6069); /*string*/
    kk_string_t _x6070;
    kk_string_t s2_4674;
    kk_integer_t _x6071 = kk_integer_abs(_mins_2262,kk_context()); /*int*/
    s2_4674 = kk_std_core_show(_x6071, _ctx); /*string*/
    kk_ssize_t w2 = ((kk_ssize_t)2); /*ssize_t*/;
    kk_ssize_t n2;
    kk_string_t _x6072 = kk_string_dup(s2_4674); /*string*/
    n2 = kk_string_len(_x6072,kk_context()); /*ssize_t*/
    bool _match_5706 = (w2 <= n2); /*bool*/;
    if (_match_5706) {
      _x6070 = s2_4674; /*string*/
    }
    else {
      kk_string_t _x6073;
      kk_string_t _x6074 = kk_std_core_string('0', _ctx); /*string*/
      kk_ssize_t _x6075 = (w2 - n2); /*ssize_t*/
      _x6073 = kk_std_core_repeatz(_x6074, _x6075, _ctx); /*string*/
      _x6070 = kk_std_core__lp__plus__plus__1_rp_(_x6073, s2_4674, _ctx); /*string*/
    }
    _x6066 = kk_std_core__lp__plus__plus__1_rp_(_x6068, _x6070, _ctx); /*string*/
  }
  _x6056 = kk_std_core__lp__plus__plus__1_rp_(_x6057, _x6066, _ctx); /*string*/
  _x6051 = kk_std_core__lp__plus__plus__1_rp_(_x6052, _x6056, _ctx); /*string*/
  y0_4670 = kk_std_core__lp__plus__plus__1_rp_(_x6049, _x6051, _ctx); /*string*/
  kk_string_t tzonename0;
  bool _match_5704;
  kk_string_t _x6076;
  if (kk_std_core_types__is_Optional(name0)) {
    kk_box_t _box_x5070 = name0._cons.Optional.value;
    kk_string_t _name_226700 = kk_string_unbox(_box_x5070);
    kk_string_dup(_name_226700);
    _x6076 = _name_226700; /*string*/
    goto _match6077;
  }
  _x6076 = kk_string_empty(); /*string*/
  _match6077: ;
  kk_string_t _x6080 = kk_string_empty(); /*string*/
  _match_5704 = kk_string_is_eq(_x6076,_x6080,kk_context()); /*bool*/
  if (_match_5704) {
    kk_std_core_types__optional_drop(name0, _ctx);
    tzonename0 = y0_4670; /*string*/
  }
  else {
    kk_string_drop(y0_4670, _ctx);
    if (kk_std_core_types__is_Optional(name0)) {
      kk_box_t _box_x5071 = name0._cons.Optional.value;
      kk_string_t _name_226701 = kk_string_unbox(_box_x5071);
      kk_string_dup(_name_226701);
      kk_std_core_types__optional_drop(name0, _ctx);
      kk_Unit;
      tzonename0 = _name_226701; /*string*/
      goto _match6082;
    }
    tzonename0 = kk_string_empty(); /*string*/
    _match6082: ;
  }
  kk_std_time_duration__duration delta1;
  kk_std_num_ddouble__ddouble _x6085;
  kk_std_core_types__optional frac0_13610;
  kk_box_t _x6086;
  double _x6087;
  kk_std_core_types__optional _match_5703 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5703)) {
    kk_box_t _box_x5072 = _match_5703._cons.Optional.value;
    double _frac_1620 = kk_double_unbox(_box_x5072, NULL);
    kk_std_core_types__optional_drop(_match_5703, _ctx);
    kk_Unit;
    _x6087 = _frac_1620; /*double*/
    goto _match6088;
  }
  _x6087 = 0.0; /*double*/
  _match6088: ;
  _x6086 = kk_double_box(_x6087, _ctx); /*106*/
  frac0_13610 = kk_std_core_types__new_Optional(_x6086, _ctx); /*optional<double>*/
  bool _match_5702;
  double _x6090;
  if (kk_std_core_types__is_Optional(frac0_13610)) {
    kk_box_t _box_x5075 = frac0_13610._cons.Optional.value;
    double _frac_7881 = kk_double_unbox(_box_x5075, NULL);
    _x6090 = _frac_7881; /*double*/
    goto _match6091;
  }
  _x6090 = 0.0; /*double*/
  _match6091: ;
  _match_5702 = (_x6090 == (0.0)); /*bool*/
  if (_match_5702) {
    kk_std_core_types__optional_drop(frac0_13610, _ctx);
    _x6085 = kk_std_num_ddouble_ddouble_int_exp(secs1, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
  }
  else {
    kk_std_num_ddouble__ddouble _x6093 = kk_std_num_ddouble_ddouble_int_exp(secs1, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
    kk_std_num_ddouble__ddouble _x6094;
    double _x6095;
    if (kk_std_core_types__is_Optional(frac0_13610)) {
      kk_box_t _box_x5076 = frac0_13610._cons.Optional.value;
      double _frac_78800 = kk_double_unbox(_box_x5076, NULL);
      kk_std_core_types__optional_drop(frac0_13610, _ctx);
      kk_Unit;
      _x6095 = _frac_78800; /*double*/
      goto _match6096;
    }
    _x6095 = 0.0; /*double*/
    _match6096: ;
    _x6094 = kk_std_num_ddouble__new_Ddouble(_x6095, 0.0, _ctx); /*std/num/ddouble/ddouble*/
    _x6085 = kk_std_num_ddouble__lp__plus__rp_(_x6093, _x6094, _ctx); /*std/num/ddouble/ddouble*/
  }
  delta1 = kk_std_time_duration__new_Duration(_x6085, _ctx); /*std/time/duration/duration*/
  return kk_std_time_calendar__new_Timezone(tzonename0, kk_std_time_calendar_new_tz_fixed_fun6098(abbrv, delta1, _ctx), kk_std_time_calendar_new_tz_fixed_fun6102(delta1, _ctx), _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_earth_timestamp_to_instant(kk_std_time_timestamp__timestamp t, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts, kk_context_t* _ctx) { /* (t : std/time/timestamp/timestamp, tz : timezone, ts : std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_std_time_instant__instant i;
  kk_std_time_instant__timescale _x6107 = kk_std_time_instant__timescale_dup(ts); /*std/time/instant/timescale*/
  i = kk_std_time_instant__new_Instant(kk_reuse_null, t, _x6107, _ctx); /*std/time/instant/instant*/
  bool _match_5697;
  kk_string_t _x6108;
  kk_string_t _x = tz.name;
  kk_string_dup(_x);
  _x6108 = _x; /*string*/
  kk_string_t _x6109;
  kk_std_time_calendar__timezone _x6110 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  kk_string_t _x0 = _x6110.name;
  _x6109 = kk_string_dup(_x0); /*string*/
  _match_5697 = kk_string_is_eq(_x6108,_x6109,kk_context()); /*bool*/
  if (_match_5697) {
    kk_std_time_instant__timescale_dropn(ts, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    return i;
  }
  kk_std_core_types__maybe _match_5698;
  kk_function_t _x6112;
  kk_function_t _x1 = tz.utc_inverse;
  kk_function_dup(_x1);
  _x6112 = _x1; /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  kk_std_time_instant__instant _x6111 = kk_std_time_instant__instant_dup(i); /*std/time/instant/instant*/
  _match_5698 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6112, (_x6112, _x6111, _ctx)); /*maybe<std/time/instant/instant>*/
  if (kk_std_core_types__is_Just(_match_5698)) {
    kk_box_t _box_x5087 = _match_5698._cons.Just.value;
    kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5087, NULL);
    struct kk_std_time_instant_Instant* _con6114 = kk_std_time_instant__as_Instant(inv);
    kk_std_time_instant__instant_dropn(i, ((int32_t)1), _ctx);
    kk_std_time_instant__timescale_dropn(ts, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    kk_std_time_instant__instant_dup(inv);
    kk_std_core_types__maybe_drop(_match_5698, _ctx);
    kk_Unit;
    return inv;
  }
  kk_std_core_types__tuple2_ _this_4698;
  kk_function_t _x6115;
  kk_function_t _x3 = tz.utc_delta;
  kk_function_dup(_x3);
  _x6115 = _x3; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  _this_4698 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6115, (_x6115, i, _ctx)); /*(std/time/duration/duration, string)*/
  kk_std_time_timestamp__timestamp t1_4701;
  kk_std_num_ddouble__ddouble _x6116;
  kk_std_num_ddouble__ddouble _x6117;
  kk_std_num_ddouble__ddouble _x10 = t.since;
  _x6117 = _x10; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6118;
  double _x6119;
  double _x6120;
  kk_box_t _box_x5088 = _this_4698.fst;
  kk_box_t _box_x5089 = _this_4698.snd;
  kk_std_time_duration__duration _x2 = kk_std_time_duration__duration_unbox(_box_x5088, NULL);
  kk_std_num_ddouble__ddouble _x5 = _x2.secs;
  double _x4 = _x5.hi;
  _x6120 = _x4; /*double*/
  _x6119 = (-_x6120); /*double*/
  double _x6123;
  double _x6124;
  kk_box_t _box_x5090 = _this_4698.fst;
  kk_box_t _box_x5091 = _this_4698.snd;
  kk_std_time_duration__duration _x20 = kk_std_time_duration__duration_unbox(_box_x5090, NULL);
  kk_std_num_ddouble__ddouble _x50 = _x20.secs;
  double _x00 = _x50.lo;
  _x6124 = _x00; /*double*/
  _x6123 = (-_x6124); /*double*/
  _x6118 = kk_std_num_ddouble__new_Ddouble(_x6119, _x6123, _ctx); /*std/num/ddouble/ddouble*/
  _x6116 = kk_std_num_ddouble__lp__plus__rp_(_x6117, _x6118, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6127;
  kk_std_num_ddouble__ddouble _pat020 = t.since;
  int32_t _x200 = t.leap32;
  _x6127 = _x200; /*int32*/
  t1_4701 = kk_std_time_timestamp__new_Timestamp(_x6116, _x6127, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant tzi1;
  kk_std_time_instant__timescale _x6128 = kk_std_time_instant__timescale_dup(ts); /*std/time/instant/timescale*/
  tzi1 = kk_std_time_instant__new_Instant(kk_reuse_null, t1_4701, _x6128, _ctx); /*std/time/instant/instant*/
  kk_std_core_types__tuple2_ _this0_4705;
  kk_function_t _x6130;
  kk_function_t _x7 = tz.utc_delta;
  kk_function_dup(_x7);
  kk_std_time_calendar__timezone_drop(tz, _ctx);
  kk_Unit;
  _x6130 = _x7; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  kk_std_time_instant__instant _x6129 = kk_std_time_instant__instant_dup(tzi1); /*std/time/instant/instant*/
  _this0_4705 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6130, (_x6130, _x6129, _ctx)); /*(std/time/duration/duration, string)*/
  bool _match_5699;
  kk_std_time_duration__duration _x6131;
  kk_box_t _box_x5092 = _this_4698.fst;
  kk_box_t _box_x5093 = _this_4698.snd;
  kk_std_time_duration__duration _x21 = kk_std_time_duration__duration_unbox(_box_x5092, NULL);
  kk_std_core_types__tuple2__drop(_this_4698, _ctx);
  kk_Unit;
  _x6131 = _x21; /*std/time/duration/duration*/
  kk_std_time_duration__duration _x6134;
  kk_box_t _box_x5094 = _this0_4705.fst;
  kk_box_t _box_x5095 = _this0_4705.snd;
  kk_std_time_duration__duration _x6 = kk_std_time_duration__duration_unbox(_box_x5094, NULL);
  _x6134 = _x6; /*std/time/duration/duration*/
  _match_5699 = kk_std_time_duration__lp__eq__eq__rp_(_x6131, _x6134, _ctx); /*bool*/
  if (_match_5699) {
    kk_std_core_types__tuple2__drop(_this0_4705, _ctx);
    kk_std_time_instant__timescale_dropn(ts, ((int32_t)10), _ctx);
    return tzi1;
  }
  kk_std_time_instant__instant_dropn(tzi1, ((int32_t)1), _ctx);
  kk_std_time_timestamp__timestamp t3_4708;
  kk_std_num_ddouble__ddouble _x6137;
  kk_std_num_ddouble__ddouble _x6138;
  kk_std_num_ddouble__ddouble _x11 = t.since;
  _x6138 = _x11; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6139;
  double _x6140;
  double _x6141;
  kk_box_t _box_x5096 = _this0_4705.fst;
  kk_box_t _box_x5097 = _this0_4705.snd;
  kk_std_time_duration__duration _x60 = kk_std_time_duration__duration_unbox(_box_x5096, NULL);
  kk_std_num_ddouble__ddouble _x9 = _x60.secs;
  double _x8 = _x9.hi;
  _x6141 = _x8; /*double*/
  _x6140 = (-_x6141); /*double*/
  double _x6144;
  double _x6145;
  kk_box_t _box_x5098 = _this0_4705.fst;
  kk_box_t _box_x5099 = _this0_4705.snd;
  kk_std_time_duration__duration _x61 = kk_std_time_duration__duration_unbox(_box_x5098, NULL);
  kk_std_core_types__tuple2__drop(_this0_4705, _ctx);
  kk_Unit;
  kk_std_num_ddouble__ddouble _x90 = _x61.secs;
  double _x01 = _x90.lo;
  _x6145 = _x01; /*double*/
  _x6144 = (-_x6145); /*double*/
  _x6139 = kk_std_num_ddouble__new_Ddouble(_x6140, _x6144, _ctx); /*std/num/ddouble/ddouble*/
  _x6137 = kk_std_num_ddouble__lp__plus__rp_(_x6138, _x6139, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6148;
  kk_std_num_ddouble__ddouble _pat021 = t.since;
  int32_t _x210 = t.leap32;
  _x6148 = _x210; /*int32*/
  t3_4708 = kk_std_time_timestamp__new_Timestamp(_x6137, _x6148, _ctx); /*std/time/timestamp/timestamp*/
  return kk_std_time_instant__new_Instant(kk_reuse_null, t3_4708, ts, _ctx);
}
 
// Create a calendar where we assume that each
// day has 24 hours with 60 minutes, and where minutes are 60 seconds
// (with possibly leap seconds). The function takes care of timezones,
// epoch shifts, and leap seconds, and only needs a calendar `name`,
// a function `days-to-date` to calculate a date given a number of days since the `epoch`,
// a function `date-to-days` to calculate days since the `epoch` for a given date,
// and an optional time scale to be used (=`ts-utc`).
// `month-prefix` (=`""`) is used when displaying numeric months and is set for
// the ISO week dates to `"W"` for example. The `show-era` function can be given
// to display the era of a date.


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun6154__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5104;
};
static kk_string_t kk_std_time_calendar_earth_calendar_fun6154(kk_function_t _fself, kk_std_time_date__date _b_5107, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun6154(kk_box_t _fun_unbox_x5104, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6154__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun6154__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun6154, kk_context());
  _self->_fun_unbox_x5104 = _fun_unbox_x5104;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_earth_calendar_fun6154(kk_function_t _fself, kk_std_time_date__date _b_5107, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6154__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun6154__t*, _fself);
  kk_box_t _fun_unbox_x5104 = _self->_fun_unbox_x5104; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5104);}, {}, _ctx)
  kk_box_t _x6155;
  kk_function_t _x6156 = kk_function_unbox(_fun_unbox_x5104); /*(5105) -> 5106*/
  _x6155 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6156, (_x6156, kk_std_time_date__date_box(_b_5107, _ctx), _ctx)); /*5106*/
  return kk_string_unbox(_x6155);
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun6157__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_earth_calendar_fun6157(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun6157(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_earth_calendar_fun6157, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_earth_calendar_fun6157(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun6160__t {
  struct kk_function_s _base;
  kk_function_t days_to_date0;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_earth_calendar_fun6160(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun6160(kk_function_t days_to_date0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6160__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun6160__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun6160, kk_context());
  _self->days_to_date0 = days_to_date0;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_earth_calendar_fun6160(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6160__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun6160__t*, _fself);
  kk_function_t days_to_date0 = _self->days_to_date0; /* (days : int) -> std/time/date/date */
  kk_drop_match(_self, {kk_function_dup(days_to_date0);}, {}, _ctx)
  kk_std_time_timestamp__timestamp ts_4717;
  kk_std_num_ddouble__ddouble _x6161;
  kk_std_num_ddouble__ddouble _x6162;
  struct kk_std_time_instant_Instant* _con6163 = kk_std_time_instant__as_Instant(i);
  kk_std_time_timestamp__timestamp _x1 = _con6163->since;
  kk_std_time_instant__timescale _pat23 = _con6163->ts;
  struct kk_std_time_instant_Timescale* _con6164 = kk_std_time_instant__as_Timescale(_pat23);
  kk_std_num_ddouble__ddouble _x0 = _x1.since;
  _x6162 = _x0; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6165;
  kk_std_num_ddouble__ddouble _x2 = tzdelta.secs;
  _x6165 = _x2; /*std/time/timestamp/timespan*/
  _x6161 = kk_std_num_ddouble__lp__plus__rp_(_x6162, _x6165, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6166;
  struct kk_std_time_instant_Instant* _con6167 = kk_std_time_instant__as_Instant(i);
  kk_std_time_timestamp__timestamp _x10 = _con6167->since;
  kk_std_time_instant__timescale _pat230 = _con6167->ts;
  struct kk_std_time_instant_Timescale* _con6168 = kk_std_time_instant__as_Timescale(_pat230);
  kk_string_t _pat310 = _con6168->name;
  kk_string_t _pat400 = _con6168->unit;
  kk_function_t _pat500 = _con6168->from_tai;
  kk_function_t _pat600 = _con6168->to_tai;
  kk_std_core_types__maybe _pat70 = _con6168->mb_seconds_in_day;
  kk_std_core_types__maybe _pat80 = _con6168->mb_to_mjd2000;
  kk_std_core_types__maybe _pat90 = _con6168->mb_from_mjd2000;
  if (kk_likely(kk_std_time_instant__instant_is_unique(i))) {
    if (kk_likely(kk_std_time_instant__timescale_is_unique(_pat230))) {
      kk_string_drop(_pat310, _ctx);
      kk_string_drop(_pat400, _ctx);
      kk_function_drop(_pat500, _ctx);
      kk_function_drop(_pat600, _ctx);
      kk_std_core_types__maybe_drop(_pat70, _ctx);
      kk_std_core_types__maybe_drop(_pat80, _ctx);
      kk_std_core_types__maybe_drop(_pat90, _ctx);
      kk_Unit;
      kk_std_time_instant__timescale_free(_pat230);
    }
    else {
      kk_std_time_instant__timescale_decref(_pat230, _ctx);
      kk_Unit;
    }
    kk_Unit;
    kk_std_time_instant__instant_free(i);
  }
  else {
    kk_std_time_instant__instant_decref(i, _ctx);
    kk_Unit;
  }
  kk_std_num_ddouble__ddouble _pat001 = _x10.since;
  int32_t _x00 = _x10.leap32;
  _x6166 = _x00; /*int32*/
  ts_4717 = kk_std_time_timestamp__new_Timestamp(_x6161, _x6166, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_core_types__tuple2_ _match_5695 = kk_std_time_timestamp_days_seconds(ts_4717, _ctx); /*(int, std/num/ddouble/ddouble)*/;
  kk_box_t _box_x5108 = _match_5695.fst;
  kk_box_t _box_x5109 = _match_5695.snd;
  kk_integer_t days0 = kk_integer_unbox(_box_x5108);
  kk_std_num_ddouble__ddouble secs = kk_std_num_ddouble__ddouble_unbox(_box_x5109, NULL);
  kk_integer_dup(days0);
  kk_std_core_types__tuple2__drop(_match_5695, _ctx);
  kk_Unit;
  kk_std_time_date__clock _match_5696;
  kk_integer_t _x6171;
  int32_t _x6172;
  kk_std_num_ddouble__ddouble _pat000 = ts_4717.since;
  int32_t _x = ts_4717.leap32;
  _x6172 = _x; /*int32*/
  _x6171 = kk_integer_from_int(_x6172,kk_context()); /*int*/
  _match_5696 = kk_std_time_date_clock_3(secs, _x6171, _ctx); /*std/time/date/clock*/
  struct kk_std_time_date_Clock* _con6173 = kk_std_time_date__as_Clock(_match_5696);
  kk_std_time_date__date _b_5127_5110 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0, (days_to_date0, days0, _ctx)); /*std/time/date/date*/;
  kk_std_time_date__clock _b_5128_5111 = _match_5696; /*std/time/date/clock*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5127_5110, _ctx), kk_std_time_date__clock_box(_b_5128_5111, _ctx), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_earth_calendar_fun6175__t {
  struct kk_function_s _base;
  kk_function_t date_to_days0;
};
static kk_std_time_instant__instant kk_std_time_calendar_earth_calendar_fun6175(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_earth_calendar_fun6175(kk_function_t date_to_days0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6175__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_earth_calendar_fun6175__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_earth_calendar_fun6175, kk_context());
  _self->date_to_days0 = date_to_days0;
  return &_self->_base;
}

static kk_std_time_instant__instant kk_std_time_calendar_earth_calendar_fun6175(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_earth_calendar_fun6175__t* _self = kk_function_as(struct kk_std_time_calendar_earth_calendar_fun6175__t*, _fself);
  kk_function_t date_to_days0 = _self->date_to_days0; /* (std/time/date/date) -> int */
  kk_drop_match(_self, {kk_function_dup(date_to_days0);}, {}, _ctx)
  kk_std_time_timestamp__timestamp t_4832;
  kk_std_core_types__optional _x6176;
  kk_box_t _x6177;
  bool _x6178;
  struct kk_std_time_instant_Timescale* _con6179 = kk_std_time_instant__as_Timescale(ts1);
  kk_std_core_types__maybe _x31 = _con6179->mb_seconds_in_day;
  kk_std_core_types__maybe_dup(_x31);
  if (kk_std_core_types__is_Nothing(_x31)) {
    _x6178 = false; /*bool*/
  }
  else {
    kk_std_core_types__maybe_drop(_x31, _ctx);
    _x6178 = true; /*bool*/
  }
  _x6177 = kk_bool_box(_x6178); /*106*/
  _x6176 = kk_std_core_types__new_Optional(_x6177, _ctx); /*optional<106>*/
  t_4832 = kk_std_time_calendar_dc_timestamp(d00, c, date_to_days0, _x6176, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant i0;
  kk_std_time_instant__timescale _x6180 = kk_std_time_instant__timescale_dup(ts1); /*std/time/instant/timescale*/
  i0 = kk_std_time_instant__new_Instant(kk_reuse_null, t_4832, _x6180, _ctx); /*std/time/instant/instant*/
  bool _match_5692;
  kk_string_t _x6181;
  kk_string_t _x3 = tz.name;
  kk_string_dup(_x3);
  _x6181 = _x3; /*string*/
  kk_string_t _x6182;
  kk_std_time_calendar__timezone _x6183 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  kk_string_t _x01 = _x6183.name;
  _x6182 = kk_string_dup(_x01); /*string*/
  _match_5692 = kk_string_is_eq(_x6181,_x6182,kk_context()); /*bool*/
  if (_match_5692) {
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    return i0;
  }
  kk_std_core_types__maybe _match_5693;
  kk_function_t _x6185;
  kk_function_t _x100 = tz.utc_inverse;
  kk_function_dup(_x100);
  _x6185 = _x100; /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  kk_std_time_instant__instant _x6184 = kk_std_time_instant__instant_dup(i0); /*std/time/instant/instant*/
  _match_5693 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6185, (_x6185, _x6184, _ctx)); /*maybe<std/time/instant/instant>*/
  if (kk_std_core_types__is_Just(_match_5693)) {
    kk_box_t _box_x5114 = _match_5693._cons.Just.value;
    kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5114, NULL);
    struct kk_std_time_instant_Instant* _con6187 = kk_std_time_instant__as_Instant(inv);
    kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    kk_std_time_instant__instant_dup(inv);
    kk_std_core_types__maybe_drop(_match_5693, _ctx);
    kk_Unit;
    return inv;
  }
  kk_std_core_types__tuple2_ _this_4698;
  kk_function_t _x6188;
  kk_function_t _x30 = tz.utc_delta;
  kk_function_dup(_x30);
  _x6188 = _x30; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  _this_4698 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6188, (_x6188, i0, _ctx)); /*(std/time/duration/duration, string)*/
  kk_std_time_timestamp__timestamp t1_4701;
  kk_std_num_ddouble__ddouble _x6189;
  kk_std_num_ddouble__ddouble _x6190;
  kk_std_num_ddouble__ddouble _x1000 = t_4832.since;
  _x6190 = _x1000; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6191;
  double _x6192;
  double _x6193;
  kk_box_t _box_x5115 = _this_4698.fst;
  kk_box_t _box_x5116 = _this_4698.snd;
  kk_std_time_duration__duration _x20 = kk_std_time_duration__duration_unbox(_box_x5115, NULL);
  kk_std_num_ddouble__ddouble _x5 = _x20.secs;
  double _x4 = _x5.hi;
  _x6193 = _x4; /*double*/
  _x6192 = (-_x6193); /*double*/
  double _x6196;
  double _x6197;
  kk_box_t _box_x5117 = _this_4698.fst;
  kk_box_t _box_x5118 = _this_4698.snd;
  kk_std_time_duration__duration _x200 = kk_std_time_duration__duration_unbox(_box_x5117, NULL);
  kk_std_num_ddouble__ddouble _x50 = _x200.secs;
  double _x000 = _x50.lo;
  _x6197 = _x000; /*double*/
  _x6196 = (-_x6197); /*double*/
  _x6191 = kk_std_num_ddouble__new_Ddouble(_x6192, _x6196, _ctx); /*std/num/ddouble/ddouble*/
  _x6189 = kk_std_num_ddouble__lp__plus__rp_(_x6190, _x6191, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6200;
  kk_std_num_ddouble__ddouble _pat0200 = t_4832.since;
  int32_t _x2000 = t_4832.leap32;
  _x6200 = _x2000; /*int32*/
  t1_4701 = kk_std_time_timestamp__new_Timestamp(_x6189, _x6200, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant tzi1;
  kk_std_time_instant__timescale _x6201 = kk_std_time_instant__timescale_dup(ts1); /*std/time/instant/timescale*/
  tzi1 = kk_std_time_instant__new_Instant(kk_reuse_null, t1_4701, _x6201, _ctx); /*std/time/instant/instant*/
  kk_std_core_types__tuple2_ _this0_4705;
  kk_function_t _x6203;
  kk_function_t _x7 = tz.utc_delta;
  kk_function_dup(_x7);
  kk_std_time_calendar__timezone_drop(tz, _ctx);
  kk_Unit;
  _x6203 = _x7; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  kk_std_time_instant__instant _x6202 = kk_std_time_instant__instant_dup(tzi1); /*std/time/instant/instant*/
  _this0_4705 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6203, (_x6203, _x6202, _ctx)); /*(std/time/duration/duration, string)*/
  bool _match_5694;
  kk_std_time_duration__duration _x6204;
  kk_box_t _box_x5119 = _this_4698.fst;
  kk_box_t _box_x5120 = _this_4698.snd;
  kk_std_time_duration__duration _x201 = kk_std_time_duration__duration_unbox(_box_x5119, NULL);
  kk_std_core_types__tuple2__drop(_this_4698, _ctx);
  kk_Unit;
  _x6204 = _x201; /*std/time/duration/duration*/
  kk_std_time_duration__duration _x6207;
  kk_box_t _box_x5121 = _this0_4705.fst;
  kk_box_t _box_x5122 = _this0_4705.snd;
  kk_std_time_duration__duration _x6 = kk_std_time_duration__duration_unbox(_box_x5121, NULL);
  _x6207 = _x6; /*std/time/duration/duration*/
  _match_5694 = kk_std_time_duration__lp__eq__eq__rp_(_x6204, _x6207, _ctx); /*bool*/
  if (_match_5694) {
    kk_std_core_types__tuple2__drop(_this0_4705, _ctx);
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    return tzi1;
  }
  kk_std_time_instant__instant_dropn(tzi1, ((int32_t)1), _ctx);
  kk_std_time_timestamp__timestamp t3_4708;
  kk_std_num_ddouble__ddouble _x6210;
  kk_std_num_ddouble__ddouble _x6211;
  kk_std_num_ddouble__ddouble _x11 = t_4832.since;
  _x6211 = _x11; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6212;
  double _x6213;
  double _x6214;
  kk_box_t _box_x5123 = _this0_4705.fst;
  kk_box_t _box_x5124 = _this0_4705.snd;
  kk_std_time_duration__duration _x60 = kk_std_time_duration__duration_unbox(_box_x5123, NULL);
  kk_std_num_ddouble__ddouble _x9 = _x60.secs;
  double _x8 = _x9.hi;
  _x6214 = _x8; /*double*/
  _x6213 = (-_x6214); /*double*/
  double _x6217;
  double _x6218;
  kk_box_t _box_x5125 = _this0_4705.fst;
  kk_box_t _box_x5126 = _this0_4705.snd;
  kk_std_time_duration__duration _x61 = kk_std_time_duration__duration_unbox(_box_x5125, NULL);
  kk_std_core_types__tuple2__drop(_this0_4705, _ctx);
  kk_Unit;
  kk_std_num_ddouble__ddouble _x90 = _x61.secs;
  double _x010 = _x90.lo;
  _x6218 = _x010; /*double*/
  _x6217 = (-_x6218); /*double*/
  _x6212 = kk_std_num_ddouble__new_Ddouble(_x6213, _x6217, _ctx); /*std/num/ddouble/ddouble*/
  _x6210 = kk_std_num_ddouble__lp__plus__rp_(_x6211, _x6212, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6221;
  kk_std_num_ddouble__ddouble _pat021 = t_4832.since;
  int32_t _x21 = t_4832.leap32;
  _x6221 = _x21; /*int32*/
  t3_4708 = kk_std_time_timestamp__new_Timestamp(_x6210, _x6221, _ctx); /*std/time/timestamp/timestamp*/
  return kk_std_time_instant__new_Instant(kk_reuse_null, t3_4708, ts1, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_earth_calendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_to_date0, kk_function_t date_to_days0, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-to-date : (days : int) -> std/time/date/date, date-to-days : (std/time/date/date) -> int, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_string_t _x6149;
  if (kk_std_core_types__is_Optional(month_prefix0)) {
    kk_box_t _box_x5100 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_3566 = kk_string_unbox(_box_x5100);
    kk_string_dup(_month_prefix_3566);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    kk_Unit;
    _x6149 = _month_prefix_3566; /*string*/
    goto _match6150;
  }
  _x6149 = kk_string_empty(); /*string*/
  _match6150: ;
  kk_function_t _x6153;
  if (kk_std_core_types__is_Optional(show_era0)) {
    kk_box_t _fun_unbox_x5104 = show_era0._cons.Optional.value;
    _x6153 = kk_std_time_calendar_new_earth_calendar_fun6154(_fun_unbox_x5104, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _x6153 = kk_std_time_calendar_new_earth_calendar_fun6157(_ctx); /*(std/time/date/date) -> string*/
  }
  kk_function_t _x6159;
  kk_function_dup(days_to_date0);
  _x6159 = kk_std_time_calendar_new_earth_calendar_fun6160(days_to_date0, _ctx); /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (6, 7)*/
  kk_function_t _x6174;
  kk_function_dup(date_to_days0);
  _x6174 = kk_std_time_calendar_new_earth_calendar_fun6175(date_to_days0, _ctx); /*(d00 : std/time/date/date, c : std/time/date/clock, tz : std/time/calendar/timezone, ts1 : std/time/instant/timescale) -> std/time/instant/instant*/
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, name0, long_name0, _x6149, _x6153, _x6159, _x6174, days_to_date0, date_to_days0, _ctx);
}
 
// An abstraction over solar calendars. Almost all calendars can be defined
// in terms of this function (and `solar-ecalendar`). Given functions to
// convert days to and from a year, and day-of-the-year to and from month
// and month day, this constructs a full calendar.
// Also needs an `epoch-shift` that gives the number of days from the
// calendar epoch to the date ``2000-01-01``. By default this is `730119`
// which corresponds to a ``0001-01-01`` ISO calendar epoch. The
// `has-year-zero` (=`True`) parameter is used when the calendar has a
// year zero. The `month-prefix` (=`""`) is used for displaying numeric
// months (and is `"W"` for `cal-iso-week` for example). The `show-era` function
// can be given to display an era for a given date. Finally the `ts` (=`ts-ti`)
// gives the time scale for the calendar.


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6222__t {
  struct kk_function_s _base;
  kk_function_t days_before_month;
  kk_function_t days_to_month;
  kk_function_t days_to_yeardoy;
  kk_std_core_types__optional epoch_shift;
  kk_std_core_types__optional has_year_zero;
};
static kk_std_time_date__date kk_std_time_calendar_solar_calendar_fun6222(kk_function_t _fself, kk_integer_t days00, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6222(kk_function_t days_before_month, kk_function_t days_to_month, kk_function_t days_to_yeardoy, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6222__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun6222__t, 8, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun6222, kk_context());
  _self->days_before_month = days_before_month;
  _self->days_to_month = days_to_month;
  _self->days_to_yeardoy = days_to_yeardoy;
  _self->epoch_shift = epoch_shift;
  _self->has_year_zero = has_year_zero;
  return &_self->_base;
}

static kk_std_time_date__date kk_std_time_calendar_solar_calendar_fun6222(kk_function_t _fself, kk_integer_t days00, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6222__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun6222__t*, _fself);
  kk_function_t days_before_month = _self->days_before_month; /* (year : int, month : int) -> int */
  kk_function_t days_to_month = _self->days_to_month; /* (year : int, doy : int) -> int */
  kk_function_t days_to_yeardoy = _self->days_to_yeardoy; /* (days : int) -> (int, int) */
  kk_std_core_types__optional epoch_shift = _self->epoch_shift; /* optional<int> */
  kk_std_core_types__optional has_year_zero = _self->has_year_zero; /* optional<bool> */
  kk_drop_match(_self, {kk_function_dup(days_before_month);kk_function_dup(days_to_month);kk_function_dup(days_to_yeardoy);kk_std_core_types__optional_dup(epoch_shift);kk_std_core_types__optional_dup(has_year_zero);}, {}, _ctx)
  kk_integer_t days1;
  kk_integer_t _x6223;
  if (kk_std_core_types__is_Optional(epoch_shift)) {
    kk_box_t _box_x5129 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_3632 = kk_integer_unbox(_box_x5129);
    kk_integer_dup(_epoch_shift_3632);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    kk_Unit;
    _x6223 = _epoch_shift_3632; /*int*/
    goto _match6224;
  }
  _x6223 = kk_integer_from_int(730119, _ctx); /*int*/
  _match6224: ;
  days1 = kk_integer_add(days00,_x6223,kk_context()); /*int*/
  kk_std_core_types__tuple2_ _match_5690 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_integer_t, kk_context_t*), days_to_yeardoy, (days_to_yeardoy, days1, _ctx)); /*(int, int)*/;
  kk_box_t _box_x5130 = _match_5690.fst;
  kk_box_t _box_x5131 = _match_5690.snd;
  kk_integer_t y0 = kk_integer_unbox(_box_x5130);
  kk_integer_t doy = kk_integer_unbox(_box_x5131);
  kk_integer_dup(doy);
  kk_integer_dup(y0);
  kk_std_core_types__tuple2__drop(_match_5690, _ctx);
  kk_Unit;
  kk_integer_t month;
  kk_integer_t _x6228 = kk_integer_dup(y0); /*int*/
  kk_integer_t _x6229 = kk_integer_dup(doy); /*int*/
  month = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_to_month, (days_to_month, _x6228, _x6229, _ctx)); /*int*/
  kk_integer_t day;
  kk_integer_t _x6230;
  kk_integer_t _x6231;
  kk_integer_t _x6232 = kk_integer_dup(y0); /*int*/
  kk_integer_t _x6233 = kk_integer_dup(month); /*int*/
  _x6231 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_before_month, (days_before_month, _x6232, _x6233, _ctx)); /*int*/
  _x6230 = kk_integer_sub(doy,_x6231,kk_context()); /*int*/
  day = kk_integer_add(_x6230,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_integer_t year;
  if (kk_std_core_types__is_Optional(has_year_zero)) {
    kk_box_t _box_x5132 = has_year_zero._cons.Optional.value;
    bool _has_year_zero_3636 = kk_bool_unbox(_box_x5132);
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    kk_Unit;
    if (_has_year_zero_3636) {
      year = y0; /*int*/
      goto _match6234;
    }
    bool _match_5691;
    kk_integer_t _x6236 = kk_integer_dup(y0); /*int*/
    _match_5691 = kk_std_core_is_pos_2(_x6236, _ctx); /*bool*/
    if (_match_5691) {
      year = y0; /*int*/
      goto _match6234;
    }
    year = kk_integer_sub(y0,(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match6234;
  }
  year = y0; /*int*/
  _match6234: ;
  return kk_std_time_date__new_Date(year, month, day, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6237__t {
  struct kk_function_s _base;
  kk_function_t days_before_month;
  kk_function_t days_before_year;
  kk_std_core_types__optional epoch_shift;
  kk_std_core_types__optional has_year_zero;
};
static kk_integer_t kk_std_time_calendar_solar_calendar_fun6237(kk_function_t _fself, kk_std_time_date__date d01, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6237(kk_function_t days_before_month, kk_function_t days_before_year, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6237__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun6237__t, 7, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun6237, kk_context());
  _self->days_before_month = days_before_month;
  _self->days_before_year = days_before_year;
  _self->epoch_shift = epoch_shift;
  _self->has_year_zero = has_year_zero;
  return &_self->_base;
}

static kk_integer_t kk_std_time_calendar_solar_calendar_fun6237(kk_function_t _fself, kk_std_time_date__date d01, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6237__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun6237__t*, _fself);
  kk_function_t days_before_month = _self->days_before_month; /* (year : int, month : int) -> int */
  kk_function_t days_before_year = _self->days_before_year; /* (year : int) -> int */
  kk_std_core_types__optional epoch_shift = _self->epoch_shift; /* optional<int> */
  kk_std_core_types__optional has_year_zero = _self->has_year_zero; /* optional<bool> */
  kk_drop_match(_self, {kk_function_dup(days_before_month);kk_function_dup(days_before_year);kk_std_core_types__optional_dup(epoch_shift);kk_std_core_types__optional_dup(has_year_zero);}, {}, _ctx)
  kk_integer_t y;
  if (kk_std_core_types__is_Optional(has_year_zero)) {
    kk_box_t _box_x5133 = has_year_zero._cons.Optional.value;
    bool _has_year_zero_36360 = kk_bool_unbox(_box_x5133);
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    kk_Unit;
    if (_has_year_zero_36360) {
      kk_integer_t _x4 = d01.year;
      kk_integer_dup(_x4);
      y = _x4; /*int*/
      goto _match6238;
    }
    kk_std_core_types__order x_16984;
    kk_integer_t _borrowedArg_5688;
    kk_integer_t _x5 = d01.year;
    kk_integer_dup(_x5);
    _borrowedArg_5688 = _x5; /*int*/
    kk_std_core_types__order _borrowApp_5689 = kk_int_as_order(kk_integer_signum_borrow(_borrowedArg_5688),kk_context()); /*order*/;
    kk_integer_drop(_borrowedArg_5688, _ctx);
    x_16984 = _borrowApp_5689; /*order*/
    bool _match_5684;
    kk_integer_t _borrowedArg_5686;
    if (kk_std_core_types__is_Lt(x_16984)) {
      _borrowedArg_5686 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
      goto _match6240;
    }
    if (kk_std_core_types__is_Eq(x_16984)) {
      _borrowedArg_5686 = kk_integer_from_small(0); /*int*/
      goto _match6240;
    }
    _borrowedArg_5686 = kk_integer_from_small(1); /*int*/
    _match6240: ;
    kk_integer_t _borrowedArg_5685 = kk_integer_from_small(1); /*int*/;
    bool _borrowApp_5687 = kk_integer_eq_borrow(_borrowedArg_5686,_borrowedArg_5685,kk_context()); /*bool*/;
    kk_integer_drop(_borrowedArg_5686, _ctx);
    kk_integer_drop(_borrowedArg_5685, _ctx);
    _match_5684 = _borrowApp_5687; /*bool*/
    if (_match_5684) {
      kk_integer_t _x6 = d01.year;
      kk_integer_dup(_x6);
      y = _x6; /*int*/
      goto _match6238;
    }
    kk_integer_t _x6241;
    kk_integer_t _x7 = d01.year;
    kk_integer_dup(_x7);
    _x6241 = _x7; /*int*/
    y = kk_integer_add(_x6241,(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match6238;
  }
  kk_integer_t _x40 = d01.year;
  kk_integer_dup(_x40);
  y = _x40; /*int*/
  _match6238: ;
  kk_integer_t _x6242;
  kk_integer_t _x6243;
  kk_integer_t _x6244;
  kk_integer_t _x6245 = kk_integer_dup(y); /*int*/
  _x6244 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), days_before_year, (days_before_year, _x6245, _ctx)); /*int*/
  kk_integer_t _x6246;
  kk_integer_t _x6247;
  kk_integer_t _x8 = d01.month;
  kk_integer_dup(_x8);
  _x6247 = _x8; /*int*/
  _x6246 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_integer_t, kk_context_t*), days_before_month, (days_before_month, y, _x6247, _ctx)); /*int*/
  _x6243 = kk_integer_add(_x6244,_x6246,kk_context()); /*int*/
  kk_integer_t _x6248;
  kk_integer_t _x6249;
  kk_integer_t _x9 = d01.day;
  kk_integer_dup(_x9);
  kk_std_time_date__date_drop(d01, _ctx);
  kk_Unit;
  _x6249 = _x9; /*int*/
  _x6248 = kk_integer_sub(_x6249,(kk_integer_from_small(1)),kk_context()); /*int*/
  _x6242 = kk_integer_add(_x6243,_x6248,kk_context()); /*int*/
  kk_integer_t _x6250;
  if (kk_std_core_types__is_Optional(epoch_shift)) {
    kk_box_t _box_x5134 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_36320 = kk_integer_unbox(_box_x5134);
    kk_integer_dup(_epoch_shift_36320);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    kk_Unit;
    _x6250 = _epoch_shift_36320; /*int*/
    goto _match6251;
  }
  _x6250 = kk_integer_from_int(730119, _ctx); /*int*/
  _match6251: ;
  return kk_integer_sub(_x6242,_x6250,kk_context());
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6258__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5139;
};
static kk_string_t kk_std_time_calendar_solar_calendar_fun6258(kk_function_t _fself, kk_std_time_date__date _b_5142, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6258(kk_box_t _fun_unbox_x5139, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6258__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun6258__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun6258, kk_context());
  _self->_fun_unbox_x5139 = _fun_unbox_x5139;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_solar_calendar_fun6258(kk_function_t _fself, kk_std_time_date__date _b_5142, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6258__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun6258__t*, _fself);
  kk_box_t _fun_unbox_x5139 = _self->_fun_unbox_x5139; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5139);}, {}, _ctx)
  kk_box_t _x6259;
  kk_function_t _x6260 = kk_function_unbox(_fun_unbox_x5139); /*(5140) -> 5141*/
  _x6259 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6260, (_x6260, kk_std_time_date__date_box(_b_5142, _ctx), _ctx)); /*5141*/
  return kk_string_unbox(_x6259);
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6261__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_solar_calendar_fun6261(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6261(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_solar_calendar_fun6261, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_solar_calendar_fun6261(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d1, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6264__t {
  struct kk_function_s _base;
  kk_function_t days_to_date0_4723;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_calendar_fun6264(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6264(kk_function_t days_to_date0_4723, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6264__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun6264__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun6264, kk_context());
  _self->days_to_date0_4723 = days_to_date0_4723;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_calendar_fun6264(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6264__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun6264__t*, _fself);
  kk_function_t days_to_date0_4723 = _self->days_to_date0_4723; /* (days : int) -> std/time/date/date */
  kk_drop_match(_self, {kk_function_dup(days_to_date0_4723);}, {}, _ctx)
  kk_std_time_timestamp__timestamp ts_4742;
  kk_std_num_ddouble__ddouble _x6265;
  kk_std_num_ddouble__ddouble _x6266;
  struct kk_std_time_instant_Instant* _con6267 = kk_std_time_instant__as_Instant(i);
  kk_std_time_timestamp__timestamp _x1 = _con6267->since;
  kk_std_time_instant__timescale _pat24 = _con6267->ts;
  struct kk_std_time_instant_Timescale* _con6268 = kk_std_time_instant__as_Timescale(_pat24);
  kk_std_num_ddouble__ddouble _x0 = _x1.since;
  _x6266 = _x0; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6269;
  kk_std_num_ddouble__ddouble _x2 = tzdelta.secs;
  _x6269 = _x2; /*std/time/timestamp/timespan*/
  _x6265 = kk_std_num_ddouble__lp__plus__rp_(_x6266, _x6269, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6270;
  struct kk_std_time_instant_Instant* _con6271 = kk_std_time_instant__as_Instant(i);
  kk_std_time_timestamp__timestamp _x10 = _con6271->since;
  kk_std_time_instant__timescale _pat240 = _con6271->ts;
  struct kk_std_time_instant_Timescale* _con6272 = kk_std_time_instant__as_Timescale(_pat240);
  kk_string_t _pat310 = _con6272->name;
  kk_string_t _pat400 = _con6272->unit;
  kk_function_t _pat500 = _con6272->from_tai;
  kk_function_t _pat600 = _con6272->to_tai;
  kk_std_core_types__maybe _pat700 = _con6272->mb_seconds_in_day;
  kk_std_core_types__maybe _pat81 = _con6272->mb_to_mjd2000;
  kk_std_core_types__maybe _pat91 = _con6272->mb_from_mjd2000;
  if (kk_likely(kk_std_time_instant__instant_is_unique(i))) {
    if (kk_likely(kk_std_time_instant__timescale_is_unique(_pat240))) {
      kk_string_drop(_pat310, _ctx);
      kk_string_drop(_pat400, _ctx);
      kk_function_drop(_pat500, _ctx);
      kk_function_drop(_pat600, _ctx);
      kk_std_core_types__maybe_drop(_pat700, _ctx);
      kk_std_core_types__maybe_drop(_pat81, _ctx);
      kk_std_core_types__maybe_drop(_pat91, _ctx);
      kk_Unit;
      kk_std_time_instant__timescale_free(_pat240);
    }
    else {
      kk_std_time_instant__timescale_decref(_pat240, _ctx);
      kk_Unit;
    }
    kk_Unit;
    kk_std_time_instant__instant_free(i);
  }
  else {
    kk_std_time_instant__instant_decref(i, _ctx);
    kk_Unit;
  }
  kk_std_num_ddouble__ddouble _pat001 = _x10.since;
  int32_t _x00 = _x10.leap32;
  _x6270 = _x00; /*int32*/
  ts_4742 = kk_std_time_timestamp__new_Timestamp(_x6265, _x6270, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_core_types__tuple2_ _match_5682 = kk_std_time_timestamp_days_seconds(ts_4742, _ctx); /*(int, std/num/ddouble/ddouble)*/;
  kk_box_t _box_x5143 = _match_5682.fst;
  kk_box_t _box_x5144 = _match_5682.snd;
  kk_integer_t days0 = kk_integer_unbox(_box_x5143);
  kk_std_num_ddouble__ddouble secs = kk_std_num_ddouble__ddouble_unbox(_box_x5144, NULL);
  kk_integer_dup(days0);
  kk_std_core_types__tuple2__drop(_match_5682, _ctx);
  kk_Unit;
  kk_std_time_date__clock _match_5683;
  kk_integer_t _x6275;
  int32_t _x6276;
  kk_std_num_ddouble__ddouble _pat000 = ts_4742.since;
  int32_t _x = ts_4742.leap32;
  _x6276 = _x; /*int32*/
  _x6275 = kk_integer_from_int(_x6276,kk_context()); /*int*/
  _match_5683 = kk_std_time_date_clock_3(secs, _x6275, _ctx); /*std/time/date/clock*/
  struct kk_std_time_date_Clock* _con6277 = kk_std_time_date__as_Clock(_match_5683);
  kk_std_time_date__date _b_5162_5145 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0_4723, (days_to_date0_4723, days0, _ctx)); /*std/time/date/date*/;
  kk_std_time_date__clock _b_5163_5146 = _match_5683; /*std/time/date/clock*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5162_5145, _ctx), kk_std_time_date__clock_box(_b_5163_5146, _ctx), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_solar_calendar_fun6279__t {
  struct kk_function_s _base;
  kk_function_t date_to_days0_4724;
};
static kk_std_time_instant__instant kk_std_time_calendar_solar_calendar_fun6279(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_calendar_fun6279(kk_function_t date_to_days0_4724, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6279__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_calendar_fun6279__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_calendar_fun6279, kk_context());
  _self->date_to_days0_4724 = date_to_days0_4724;
  return &_self->_base;
}

static kk_std_time_instant__instant kk_std_time_calendar_solar_calendar_fun6279(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_calendar_fun6279__t* _self = kk_function_as(struct kk_std_time_calendar_solar_calendar_fun6279__t*, _fself);
  kk_function_t date_to_days0_4724 = _self->date_to_days0_4724; /* (std/time/date/date) -> int */
  kk_drop_match(_self, {kk_function_dup(date_to_days0_4724);}, {}, _ctx)
  kk_std_time_timestamp__timestamp t_4835;
  kk_std_core_types__optional _x6280;
  kk_box_t _x6281;
  bool _x6282;
  struct kk_std_time_instant_Timescale* _con6283 = kk_std_time_instant__as_Timescale(ts1);
  kk_std_core_types__maybe _x31 = _con6283->mb_seconds_in_day;
  kk_std_core_types__maybe_dup(_x31);
  if (kk_std_core_types__is_Nothing(_x31)) {
    _x6282 = false; /*bool*/
  }
  else {
    kk_std_core_types__maybe_drop(_x31, _ctx);
    _x6282 = true; /*bool*/
  }
  _x6281 = kk_bool_box(_x6282); /*106*/
  _x6280 = kk_std_core_types__new_Optional(_x6281, _ctx); /*optional<106>*/
  t_4835 = kk_std_time_calendar_dc_timestamp(d00, c, date_to_days0_4724, _x6280, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant i0;
  kk_std_time_instant__timescale _x6284 = kk_std_time_instant__timescale_dup(ts1); /*std/time/instant/timescale*/
  i0 = kk_std_time_instant__new_Instant(kk_reuse_null, t_4835, _x6284, _ctx); /*std/time/instant/instant*/
  bool _match_5679;
  kk_string_t _x6285;
  kk_string_t _x3 = tz.name;
  kk_string_dup(_x3);
  _x6285 = _x3; /*string*/
  kk_string_t _x6286;
  kk_std_time_calendar__timezone _x6287 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  kk_string_t _x01 = _x6287.name;
  _x6286 = kk_string_dup(_x01); /*string*/
  _match_5679 = kk_string_is_eq(_x6285,_x6286,kk_context()); /*bool*/
  if (_match_5679) {
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    return i0;
  }
  kk_std_core_types__maybe _match_5680;
  kk_function_t _x6289;
  kk_function_t _x100 = tz.utc_inverse;
  kk_function_dup(_x100);
  _x6289 = _x100; /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  kk_std_time_instant__instant _x6288 = kk_std_time_instant__instant_dup(i0); /*std/time/instant/instant*/
  _match_5680 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6289, (_x6289, _x6288, _ctx)); /*maybe<std/time/instant/instant>*/
  if (kk_std_core_types__is_Just(_match_5680)) {
    kk_box_t _box_x5149 = _match_5680._cons.Just.value;
    kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5149, NULL);
    struct kk_std_time_instant_Instant* _con6291 = kk_std_time_instant__as_Instant(inv);
    kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    kk_std_time_instant__instant_dup(inv);
    kk_std_core_types__maybe_drop(_match_5680, _ctx);
    kk_Unit;
    return inv;
  }
  kk_std_core_types__tuple2_ _this_4698;
  kk_function_t _x6292;
  kk_function_t _x30 = tz.utc_delta;
  kk_function_dup(_x30);
  _x6292 = _x30; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  _this_4698 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6292, (_x6292, i0, _ctx)); /*(std/time/duration/duration, string)*/
  kk_std_time_timestamp__timestamp t1_4701;
  kk_std_num_ddouble__ddouble _x6293;
  kk_std_num_ddouble__ddouble _x6294;
  kk_std_num_ddouble__ddouble _x1000 = t_4835.since;
  _x6294 = _x1000; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6295;
  double _x6296;
  double _x6297;
  kk_box_t _box_x5150 = _this_4698.fst;
  kk_box_t _box_x5151 = _this_4698.snd;
  kk_std_time_duration__duration _x20 = kk_std_time_duration__duration_unbox(_box_x5150, NULL);
  kk_std_num_ddouble__ddouble _x50 = _x20.secs;
  double _x400 = _x50.hi;
  _x6297 = _x400; /*double*/
  _x6296 = (-_x6297); /*double*/
  double _x6300;
  double _x6301;
  kk_box_t _box_x5152 = _this_4698.fst;
  kk_box_t _box_x5153 = _this_4698.snd;
  kk_std_time_duration__duration _x200 = kk_std_time_duration__duration_unbox(_box_x5152, NULL);
  kk_std_num_ddouble__ddouble _x500 = _x200.secs;
  double _x000 = _x500.lo;
  _x6301 = _x000; /*double*/
  _x6300 = (-_x6301); /*double*/
  _x6295 = kk_std_num_ddouble__new_Ddouble(_x6296, _x6300, _ctx); /*std/num/ddouble/ddouble*/
  _x6293 = kk_std_num_ddouble__lp__plus__rp_(_x6294, _x6295, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6304;
  kk_std_num_ddouble__ddouble _pat0200 = t_4835.since;
  int32_t _x2000 = t_4835.leap32;
  _x6304 = _x2000; /*int32*/
  t1_4701 = kk_std_time_timestamp__new_Timestamp(_x6293, _x6304, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant tzi1;
  kk_std_time_instant__timescale _x6305 = kk_std_time_instant__timescale_dup(ts1); /*std/time/instant/timescale*/
  tzi1 = kk_std_time_instant__new_Instant(kk_reuse_null, t1_4701, _x6305, _ctx); /*std/time/instant/instant*/
  kk_std_core_types__tuple2_ _this0_4705;
  kk_function_t _x6307;
  kk_function_t _x70 = tz.utc_delta;
  kk_function_dup(_x70);
  kk_std_time_calendar__timezone_drop(tz, _ctx);
  kk_Unit;
  _x6307 = _x70; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  kk_std_time_instant__instant _x6306 = kk_std_time_instant__instant_dup(tzi1); /*std/time/instant/instant*/
  _this0_4705 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6307, (_x6307, _x6306, _ctx)); /*(std/time/duration/duration, string)*/
  bool _match_5681;
  kk_std_time_duration__duration _x6308;
  kk_box_t _box_x5154 = _this_4698.fst;
  kk_box_t _box_x5155 = _this_4698.snd;
  kk_std_time_duration__duration _x201 = kk_std_time_duration__duration_unbox(_box_x5154, NULL);
  kk_std_core_types__tuple2__drop(_this_4698, _ctx);
  kk_Unit;
  _x6308 = _x201; /*std/time/duration/duration*/
  kk_std_time_duration__duration _x6311;
  kk_box_t _box_x5156 = _this0_4705.fst;
  kk_box_t _box_x5157 = _this0_4705.snd;
  kk_std_time_duration__duration _x60 = kk_std_time_duration__duration_unbox(_box_x5156, NULL);
  _x6311 = _x60; /*std/time/duration/duration*/
  _match_5681 = kk_std_time_duration__lp__eq__eq__rp_(_x6308, _x6311, _ctx); /*bool*/
  if (_match_5681) {
    kk_std_core_types__tuple2__drop(_this0_4705, _ctx);
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    return tzi1;
  }
  kk_std_time_instant__instant_dropn(tzi1, ((int32_t)1), _ctx);
  kk_std_time_timestamp__timestamp t3_4708;
  kk_std_num_ddouble__ddouble _x6314;
  kk_std_num_ddouble__ddouble _x6315;
  kk_std_num_ddouble__ddouble _x11 = t_4835.since;
  _x6315 = _x11; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6316;
  double _x6317;
  double _x6318;
  kk_box_t _box_x5158 = _this0_4705.fst;
  kk_box_t _box_x5159 = _this0_4705.snd;
  kk_std_time_duration__duration _x600 = kk_std_time_duration__duration_unbox(_box_x5158, NULL);
  kk_std_num_ddouble__ddouble _x90 = _x600.secs;
  double _x80 = _x90.hi;
  _x6318 = _x80; /*double*/
  _x6317 = (-_x6318); /*double*/
  double _x6321;
  double _x6322;
  kk_box_t _box_x5160 = _this0_4705.fst;
  kk_box_t _box_x5161 = _this0_4705.snd;
  kk_std_time_duration__duration _x601 = kk_std_time_duration__duration_unbox(_box_x5160, NULL);
  kk_std_core_types__tuple2__drop(_this0_4705, _ctx);
  kk_Unit;
  kk_std_num_ddouble__ddouble _x900 = _x601.secs;
  double _x010 = _x900.lo;
  _x6322 = _x010; /*double*/
  _x6321 = (-_x6322); /*double*/
  _x6316 = kk_std_num_ddouble__new_Ddouble(_x6317, _x6321, _ctx); /*std/num/ddouble/ddouble*/
  _x6314 = kk_std_num_ddouble__lp__plus__rp_(_x6315, _x6316, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6325;
  kk_std_num_ddouble__ddouble _pat021 = t_4835.since;
  int32_t _x21 = t_4835.leap32;
  _x6325 = _x21; /*int32*/
  t3_4708 = kk_std_time_timestamp__new_Timestamp(_x6314, _x6325, _ctx); /*std/time/timestamp/timestamp*/
  return kk_std_time_instant__new_Instant(kk_reuse_null, t3_4708, ts1, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_solar_calendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_before_year, kk_function_t days_to_yeardoy, kk_function_t days_before_month, kk_function_t days_to_month, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-before-year : (year : int) -> int, days-to-yeardoy : (days : int) -> (int, int), days-before-month : (year : int, month : int) -> int, days-to-month : (year : int, doy : int) -> int, epoch-shift : optional<int>, has-year-zero : optional<bool>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t days_to_date0_4723;
  kk_function_dup(days_before_month);
  kk_std_core_types__optional_dup(epoch_shift);
  kk_std_core_types__optional_dup(has_year_zero);
  days_to_date0_4723 = kk_std_time_calendar_new_solar_calendar_fun6222(days_before_month, days_to_month, days_to_yeardoy, epoch_shift, has_year_zero, _ctx); /*(days00 : int) -> std/time/date/date*/
  kk_function_t date_to_days0_4724 = kk_std_time_calendar_new_solar_calendar_fun6237(days_before_month, days_before_year, epoch_shift, has_year_zero, _ctx); /*(d01 : std/time/date/date) -> int*/;
  kk_string_t _x6253;
  if (kk_std_core_types__is_Optional(month_prefix0)) {
    kk_box_t _box_x5135 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_3640 = kk_string_unbox(_box_x5135);
    kk_string_dup(_month_prefix_3640);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    kk_Unit;
    _x6253 = _month_prefix_3640; /*string*/
    goto _match6254;
  }
  _x6253 = kk_string_empty(); /*string*/
  _match6254: ;
  kk_function_t _x6257;
  if (kk_std_core_types__is_Optional(show_era0)) {
    kk_box_t _fun_unbox_x5139 = show_era0._cons.Optional.value;
    _x6257 = kk_std_time_calendar_new_solar_calendar_fun6258(_fun_unbox_x5139, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _x6257 = kk_std_time_calendar_new_solar_calendar_fun6261(_ctx); /*(std/time/date/date) -> string*/
  }
  kk_function_t _x6263;
  kk_function_dup(days_to_date0_4723);
  _x6263 = kk_std_time_calendar_new_solar_calendar_fun6264(days_to_date0_4723, _ctx); /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (6, 7)*/
  kk_function_t _x6278;
  kk_function_dup(date_to_days0_4724);
  _x6278 = kk_std_time_calendar_new_solar_calendar_fun6279(date_to_days0_4724, _ctx); /*(d00 : std/time/date/date, c : std/time/date/clock, tz : std/time/calendar/timezone, ts1 : std/time/instant/timescale) -> std/time/instant/instant*/
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, name0, long_name0, _x6253, _x6257, _x6263, _x6278, days_to_date0_4723, date_to_days0_4724, _ctx);
}
 
// An abstraction over solar calendars. Almost all calendars can be defined
// in terms of this function (and `solar-calendar`). Given functions to
// convert days to and from a year, and day-of-the-year to and from month
// and month day, this constructs a full calendar. In contrast to `solar-calendar`
// this function just needs an estimation function from days to a year -- for most
// calendars this is much more convenient. The estimation function returns a
// conservative (i.e. lowest) estimate for the year and a `maybe<int>` that
// is the `safe-day`: if the day of the year is lower or equal to that the estimate
// is surely correct. Otherwise the wrapper will calculate the start day for the next
// year until the correct year is found.
// Also needs an `epoch-shift` that gives the number of days from the
// calendar epoch to the date ``2000-01-01``. By default this is `730119`
// which corresponds to a ``0001-01-01`` ISO calendar epoch. The
// `has-year-zero` (=`True`) parameter is used when the calendar has a
// year zero. The `month-prefix` (=`""`) is used for displaying numeric
// months (and is `"W"` for `cal-iso-week` for example). The `show-era` function
// can be given to display an era for a given date. Finally the `ts` (=`ts-utc`)
// gives the time scale for the calendar.


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun6327__t {
  struct kk_function_s _base;
  kk_function_t days_before_year;
  kk_function_t estimate_year;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_ecalendar_fun6327(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun6327(kk_function_t days_before_year, kk_function_t estimate_year, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun6327__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_ecalendar_fun6327__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_ecalendar_fun6327, kk_context());
  _self->days_before_year = days_before_year;
  _self->estimate_year = estimate_year;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_solar_ecalendar_fun6327(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun6327__t* _self = kk_function_as(struct kk_std_time_calendar_solar_ecalendar_fun6327__t*, _fself);
  kk_function_t days_before_year = _self->days_before_year; /* (year : int) -> int */
  kk_function_t estimate_year = _self->estimate_year; /* (days : int) -> (int, maybe<int>) */
  kk_drop_match(_self, {kk_function_dup(days_before_year);kk_function_dup(estimate_year);}, {}, _ctx)
  kk_std_core_types__tuple2_ _match_5672;
  kk_integer_t _x6328 = kk_integer_dup(days); /*int*/
  _match_5672 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_integer_t, kk_context_t*), estimate_year, (estimate_year, _x6328, _ctx)); /*(int, maybe<int>)*/
  kk_box_t _box_x5164 = _match_5672.fst;
  kk_box_t _box_x5165 = _match_5672.snd;
  kk_integer_t approx = kk_integer_unbox(_box_x5164);
  kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5165, NULL);
  kk_integer_dup(approx);
  kk_std_core_types__maybe_dup(safe);
  kk_std_core_types__tuple2__drop(_match_5672, _ctx);
  kk_Unit;
  kk_integer_t doy1;
  kk_integer_t _x6331 = kk_integer_dup(days); /*int*/
  kk_integer_t _x6332;
  kk_function_t _x6334 = kk_function_dup(days_before_year); /*(year : int) -> int*/
  kk_integer_t _x6333 = kk_integer_dup(approx); /*int*/
  _x6332 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), _x6334, (_x6334, _x6333, _ctx)); /*int*/
  doy1 = kk_integer_sub(_x6331,_x6332,kk_context()); /*int*/
  bool _match_5673;
  kk_integer_t _borrowedArg_5677;
  if (kk_std_core_types__is_Nothing(safe)) {
    _borrowedArg_5677 = kk_integer_from_small(0); /*int*/
  }
  else {
    kk_box_t _box_x5166 = safe._cons.Just.value;
    kk_integer_t x = kk_integer_unbox(_box_x5166);
    kk_integer_dup(x);
    kk_std_core_types__maybe_drop(safe, _ctx);
    kk_Unit;
    _borrowedArg_5677 = x; /*int*/
  }
  bool _borrowApp_5678 = kk_integer_lte_borrow(doy1,_borrowedArg_5677,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5677, _ctx);
  _match_5673 = _borrowApp_5678; /*bool*/
  if (_match_5673) {
    kk_integer_drop(days, _ctx);
    kk_function_drop(days_before_year, _ctx);
    kk_integer_t _b_5190_5167 = approx; /*int*/;
    kk_integer_t _b_5191_5168 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5190_5167), kk_integer_box(_b_5191_5168), _ctx);
  }
  kk_integer_t doy2;
  kk_integer_t _x6336;
  kk_integer_t _x6337;
  kk_integer_t _x6338 = kk_integer_dup(approx); /*int*/
  _x6337 = kk_integer_add(_x6338,(kk_integer_from_small(1)),kk_context()); /*int*/
  _x6336 = kk_function_call(kk_integer_t, (kk_function_t, kk_integer_t, kk_context_t*), days_before_year, (days_before_year, _x6337, _ctx)); /*int*/
  doy2 = kk_integer_sub(days,_x6336,kk_context()); /*int*/
  bool _match_5674;
  kk_integer_t _borrowedArg_5675 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_5676 = kk_integer_lt_borrow(doy2,_borrowedArg_5675,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5675, _ctx);
  _match_5674 = _borrowApp_5676; /*bool*/
  if (_match_5674) {
    kk_integer_drop(doy2, _ctx);
    kk_integer_t _b_5192_5169 = approx; /*int*/;
    kk_integer_t _b_5193_5170 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5192_5169), kk_integer_box(_b_5193_5170), _ctx);
  }
  kk_integer_drop(doy1, _ctx);
  kk_integer_t _b_5194_5171 = kk_integer_add(approx,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t _b_5195_5172 = doy2; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5194_5171), kk_integer_box(_b_5195_5172), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun6350__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5182;
};
static kk_string_t kk_std_time_calendar_solar_ecalendar_fun6350(kk_function_t _fself, kk_std_time_date__date _b_5185, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun6350(kk_box_t _fun_unbox_x5182, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun6350__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_ecalendar_fun6350__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_ecalendar_fun6350, kk_context());
  _self->_fun_unbox_x5182 = _fun_unbox_x5182;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_solar_ecalendar_fun6350(kk_function_t _fself, kk_std_time_date__date _b_5185, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun6350__t* _self = kk_function_as(struct kk_std_time_calendar_solar_ecalendar_fun6350__t*, _fself);
  kk_box_t _fun_unbox_x5182 = _self->_fun_unbox_x5182; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5182);}, {}, _ctx)
  kk_box_t _x6351;
  kk_function_t _x6352 = kk_function_unbox(_fun_unbox_x5182); /*(5183) -> 5184*/
  _x6351 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6352, (_x6352, kk_std_time_date__date_box(_b_5185, _ctx), _ctx)); /*5184*/
  return kk_string_unbox(_x6351);
}


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun6353__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_solar_ecalendar_fun6353(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun6353(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_solar_ecalendar_fun6353, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_solar_ecalendar_fun6353(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_solar_ecalendar_fun6355__t {
  struct kk_function_s _base;
  kk_function_t _b_5199_5186;
};
static kk_box_t kk_std_time_calendar_solar_ecalendar_fun6355(kk_function_t _fself, kk_box_t _b_5189, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_solar_ecalendar_fun6355(kk_function_t _b_5199_5186, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun6355__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_solar_ecalendar_fun6355__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_solar_ecalendar_fun6355, kk_context());
  _self->_b_5199_5186 = _b_5199_5186;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_solar_ecalendar_fun6355(kk_function_t _fself, kk_box_t _b_5189, kk_context_t* _ctx) {
  struct kk_std_time_calendar_solar_ecalendar_fun6355__t* _self = kk_function_as(struct kk_std_time_calendar_solar_ecalendar_fun6355__t*, _fself);
  kk_function_t _b_5199_5186 = _self->_b_5199_5186; /* (std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5199_5186);}, {}, _ctx)
  kk_string_t _x6356;
  kk_std_time_date__date _x6357 = kk_std_time_date__date_unbox(_b_5189, _ctx); /*std/time/date/date*/
  _x6356 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5199_5186, (_b_5199_5186, _x6357, _ctx)); /*string*/
  return kk_string_box(_x6356);
}

kk_std_time_calendar__calendar kk_std_time_calendar_solar_ecalendar(kk_string_t name0, kk_string_t long_name0, kk_function_t days_before_year, kk_function_t estimate_year, kk_function_t days_before_month, kk_function_t days_to_month, kk_std_core_types__optional epoch_shift, kk_std_core_types__optional has_year_zero, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, days-before-year : (year : int) -> int, estimate-year : (days : int) -> (int, maybe<int>), days-before-month : (year : int, month : int) -> int, days-to-month : (year : int, doy : int) -> int, epoch-shift : optional<int>, has-year-zero : optional<bool>, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t _x6326 = kk_function_dup(days_before_year); /*(year : int) -> int*/
  kk_std_core_types__optional _x6339;
  kk_integer_t _b_5196_5174;
  if (kk_std_core_types__is_Optional(epoch_shift)) {
    kk_box_t _box_x5173 = epoch_shift._cons.Optional.value;
    kk_integer_t _epoch_shift_2726 = kk_integer_unbox(_box_x5173);
    kk_integer_dup(_epoch_shift_2726);
    kk_std_core_types__optional_drop(epoch_shift, _ctx);
    kk_Unit;
    _b_5196_5174 = _epoch_shift_2726; /*int*/
    goto _match6340;
  }
  _b_5196_5174 = kk_integer_from_int(730119, _ctx); /*int*/
  _match6340: ;
  _x6339 = kk_std_core_types__new_Optional(kk_integer_box(_b_5196_5174), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6342;
  bool _b_5197_5176;
  if (kk_std_core_types__is_Optional(has_year_zero)) {
    kk_box_t _box_x5175 = has_year_zero._cons.Optional.value;
    bool _has_year_zero_2730 = kk_bool_unbox(_box_x5175);
    kk_std_core_types__optional_drop(has_year_zero, _ctx);
    kk_Unit;
    _b_5197_5176 = _has_year_zero_2730; /*bool*/
    goto _match6343;
  }
  _b_5197_5176 = true; /*bool*/
  _match6343: ;
  _x6342 = kk_std_core_types__new_Optional(kk_bool_box(_b_5197_5176), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6345;
  kk_string_t _b_5198_5178;
  if (kk_std_core_types__is_Optional(month_prefix0)) {
    kk_box_t _box_x5177 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_2734 = kk_string_unbox(_box_x5177);
    kk_string_dup(_month_prefix_2734);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    kk_Unit;
    _b_5198_5178 = _month_prefix_2734; /*string*/
    goto _match6346;
  }
  _b_5198_5178 = kk_string_empty(); /*string*/
  _match6346: ;
  _x6345 = kk_std_core_types__new_Optional(kk_string_box(_b_5198_5178), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6349;
  kk_function_t _b_5199_5186;
  if (kk_std_core_types__is_Optional(show_era0)) {
    kk_box_t _fun_unbox_x5182 = show_era0._cons.Optional.value;
    _b_5199_5186 = kk_std_time_calendar_new_solar_ecalendar_fun6350(_fun_unbox_x5182, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _b_5199_5186 = kk_std_time_calendar_new_solar_ecalendar_fun6353(_ctx); /*(std/time/date/date) -> string*/
  }
  _x6349 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_solar_ecalendar_fun6355(_b_5199_5186, _ctx)), _ctx); /*optional<106>*/
  return kk_std_time_calendar_solar_calendar(name0, long_name0, _x6326, kk_std_time_calendar_new_solar_ecalendar_fun6327(days_before_year, estimate_year, _ctx), days_before_month, days_to_month, _x6339, _x6342, _x6345, _x6349, _ctx);
}
 
// Create a standard ISO calendar using a particular time scale
// and calendar `name` (=`ts.name`).


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6363__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun6363(kk_function_t _fself, kk_integer_t _x16362, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6363(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun6363, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun6363(kk_function_t _fself, kk_integer_t _x16362, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16362, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6364__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_iso_calendar_fun6364(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6364(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun6364, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_iso_calendar_fun6364(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__tuple2_ _match_5665;
  kk_integer_t _x6365 = kk_integer_dup(days); /*int*/
  _match_5665 = kk_std_time_calendar_iso_estimate_year(_x6365, _ctx); /*(int, maybe<int>)*/
  kk_box_t _box_x5201 = _match_5665.fst;
  kk_box_t _box_x5202 = _match_5665.snd;
  kk_integer_t approx = kk_integer_unbox(_box_x5201);
  kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5202, NULL);
  kk_integer_dup(approx);
  kk_std_core_types__maybe_dup(safe);
  kk_std_core_types__tuple2__drop(_match_5665, _ctx);
  kk_Unit;
  kk_integer_t doy1;
  kk_integer_t _x6368 = kk_integer_dup(days); /*int*/
  kk_integer_t _x6369;
  kk_integer_t _x6370 = kk_integer_dup(approx); /*int*/
  _x6369 = kk_std_time_calendar_iso_days_before_year(_x6370, _ctx); /*int*/
  doy1 = kk_integer_sub(_x6368,_x6369,kk_context()); /*int*/
  bool _match_5666;
  kk_integer_t _borrowedArg_5670;
  if (kk_std_core_types__is_Nothing(safe)) {
    _borrowedArg_5670 = kk_integer_from_small(0); /*int*/
  }
  else {
    kk_box_t _box_x5203 = safe._cons.Just.value;
    kk_integer_t x = kk_integer_unbox(_box_x5203);
    kk_integer_dup(x);
    kk_std_core_types__maybe_drop(safe, _ctx);
    kk_Unit;
    _borrowedArg_5670 = x; /*int*/
  }
  bool _borrowApp_5671 = kk_integer_lte_borrow(doy1,_borrowedArg_5670,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5670, _ctx);
  _match_5666 = _borrowApp_5671; /*bool*/
  if (_match_5666) {
    kk_integer_drop(days, _ctx);
    kk_integer_t _b_5227_5204 = approx; /*int*/;
    kk_integer_t _b_5228_5205 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5227_5204), kk_integer_box(_b_5228_5205), _ctx);
  }
  kk_integer_t doy2;
  kk_integer_t _x6372;
  kk_integer_t year_4907;
  kk_integer_t _x6373 = kk_integer_dup(approx); /*int*/
  year_4907 = kk_integer_add(_x6373,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_integer_t y = kk_integer_sub(year_4907,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t leapdays;
  kk_integer_t _x6374;
  kk_integer_t _x6375;
  kk_integer_t _x6376 = kk_integer_dup(y); /*int*/
  _x6375 = kk_integer_div(_x6376,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t _x6377;
  kk_integer_t _x6378 = kk_integer_dup(y); /*int*/
  _x6377 = kk_integer_div(_x6378,(kk_integer_from_small(100)),kk_context()); /*int*/
  _x6374 = kk_integer_sub(_x6375,_x6377,kk_context()); /*int*/
  kk_integer_t _x6379;
  kk_integer_t _x6380 = kk_integer_dup(y); /*int*/
  _x6379 = kk_integer_div(_x6380,(kk_integer_from_small(400)),kk_context()); /*int*/
  leapdays = kk_integer_add(_x6374,_x6379,kk_context()); /*int*/
  kk_integer_t _x6381 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/
  _x6372 = kk_integer_add(_x6381,leapdays,kk_context()); /*int*/
  doy2 = kk_integer_sub(days,_x6372,kk_context()); /*int*/
  bool _match_5667;
  kk_integer_t _borrowedArg_5668 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_5669 = kk_integer_lt_borrow(doy2,_borrowedArg_5668,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5668, _ctx);
  _match_5667 = _borrowApp_5669; /*bool*/
  if (_match_5667) {
    kk_integer_drop(doy2, _ctx);
    kk_integer_t _b_5229_5206 = approx; /*int*/;
    kk_integer_t _b_5230_5207 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5229_5206), kk_integer_box(_b_5230_5207), _ctx);
  }
  kk_integer_drop(doy1, _ctx);
  kk_integer_t _b_5231_5208 = kk_integer_add(approx,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t _b_5232_5209 = doy2; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5231_5208), kk_integer_box(_b_5232_5209), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6384__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun6384(kk_function_t _fself, kk_integer_t _x16382, kk_integer_t _x26383, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6384(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun6384, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun6384(kk_function_t _fself, kk_integer_t _x16382, kk_integer_t _x26383, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16382, _x26383, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6387__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_iso_calendar_fun6387(kk_function_t _fself, kk_integer_t _x16385, kk_integer_t _x26386, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6387(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun6387, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_iso_calendar_fun6387(kk_function_t _fself, kk_integer_t _x16385, kk_integer_t _x26386, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16385, _x26386, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6400__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5219;
};
static kk_string_t kk_std_time_calendar_iso_calendar_fun6400(kk_function_t _fself, kk_std_time_date__date _b_5222, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6400(kk_box_t _fun_unbox_x5219, kk_context_t* _ctx) {
  struct kk_std_time_calendar_iso_calendar_fun6400__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_iso_calendar_fun6400__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_iso_calendar_fun6400, kk_context());
  _self->_fun_unbox_x5219 = _fun_unbox_x5219;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_iso_calendar_fun6400(kk_function_t _fself, kk_std_time_date__date _b_5222, kk_context_t* _ctx) {
  struct kk_std_time_calendar_iso_calendar_fun6400__t* _self = kk_function_as(struct kk_std_time_calendar_iso_calendar_fun6400__t*, _fself);
  kk_box_t _fun_unbox_x5219 = _self->_fun_unbox_x5219; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5219);}, {}, _ctx)
  kk_box_t _x6401;
  kk_function_t _x6402 = kk_function_unbox(_fun_unbox_x5219); /*(5220) -> 5221*/
  _x6401 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6402, (_x6402, kk_std_time_date__date_box(_b_5222, _ctx), _ctx)); /*5221*/
  return kk_string_unbox(_x6401);
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6403__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_iso_calendar_fun6403(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6403(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_iso_calendar_fun6403, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_iso_calendar_fun6403(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_iso_calendar_fun6405__t {
  struct kk_function_s _base;
  kk_function_t _b_5236_5223;
};
static kk_box_t kk_std_time_calendar_iso_calendar_fun6405(kk_function_t _fself, kk_box_t _b_5226, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_iso_calendar_fun6405(kk_function_t _b_5236_5223, kk_context_t* _ctx) {
  struct kk_std_time_calendar_iso_calendar_fun6405__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_iso_calendar_fun6405__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_iso_calendar_fun6405, kk_context());
  _self->_b_5236_5223 = _b_5236_5223;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_iso_calendar_fun6405(kk_function_t _fself, kk_box_t _b_5226, kk_context_t* _ctx) {
  struct kk_std_time_calendar_iso_calendar_fun6405__t* _self = kk_function_as(struct kk_std_time_calendar_iso_calendar_fun6405__t*, _fself);
  kk_function_t _b_5236_5223 = _self->_b_5236_5223; /* (std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5236_5223);}, {}, _ctx)
  kk_string_t _x6406;
  kk_std_time_date__date _x6407 = kk_std_time_date__date_unbox(_b_5226, _ctx); /*std/time/date/date*/
  _x6406 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5236_5223, (_b_5236_5223, _x6407, _ctx)); /*string*/
  return kk_string_box(_x6406);
}

kk_std_time_calendar__calendar kk_std_time_calendar_iso_calendar(kk_std_time_instant__timescale ___wildcard__455__26, kk_string_t name0, kk_std_core_types__optional long_name0, kk_context_t* _ctx) { /* (std/time/instant/timescale, name : string, long-name : optional<string>) -> calendar */ 
  kk_std_time_instant__timescale_dropn(___wildcard__455__26, ((int32_t)10), _ctx);
  kk_string_t _x6358 = kk_string_dup(name0); /*string*/
  kk_string_t _x6359;
  if (kk_std_core_types__is_Optional(long_name0)) {
    kk_box_t _box_x5200 = long_name0._cons.Optional.value;
    kk_string_t _long_name_2918 = kk_string_unbox(_box_x5200);
    kk_string_drop(name0, _ctx);
    kk_string_dup(_long_name_2918);
    kk_std_core_types__optional_drop(long_name0, _ctx);
    kk_Unit;
    _x6359 = _long_name_2918; /*string*/
    goto _match6360;
  }
  _x6359 = name0; /*string*/
  _match6360: ;
  kk_std_core_types__optional _x6388;
  kk_integer_t _b_5233_5211;
  kk_std_core_types__optional _match_5664 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5664)) {
    kk_box_t _box_x5210 = _match_5664._cons.Optional.value;
    kk_integer_t _epoch_shift_2726 = kk_integer_unbox(_box_x5210);
    kk_integer_dup(_epoch_shift_2726);
    kk_std_core_types__optional_drop(_match_5664, _ctx);
    kk_Unit;
    _b_5233_5211 = _epoch_shift_2726; /*int*/
    goto _match6389;
  }
  _b_5233_5211 = kk_integer_from_int(730119, _ctx); /*int*/
  _match6389: ;
  _x6388 = kk_std_core_types__new_Optional(kk_integer_box(_b_5233_5211), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6391;
  bool _b_5234_5213;
  kk_std_core_types__optional _match_5663 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5663)) {
    kk_box_t _box_x5212 = _match_5663._cons.Optional.value;
    bool _has_year_zero_2730 = kk_bool_unbox(_box_x5212);
    kk_std_core_types__optional_drop(_match_5663, _ctx);
    kk_Unit;
    _b_5234_5213 = _has_year_zero_2730; /*bool*/
    goto _match6392;
  }
  _b_5234_5213 = true; /*bool*/
  _match6392: ;
  _x6391 = kk_std_core_types__new_Optional(kk_bool_box(_b_5234_5213), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6394;
  kk_string_t _b_5235_5215;
  kk_std_core_types__optional _match_5662 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5662)) {
    kk_box_t _box_x5214 = _match_5662._cons.Optional.value;
    kk_string_t _month_prefix_2734 = kk_string_unbox(_box_x5214);
    kk_string_dup(_month_prefix_2734);
    kk_std_core_types__optional_drop(_match_5662, _ctx);
    kk_Unit;
    _b_5235_5215 = _month_prefix_2734; /*string*/
    goto _match6395;
  }
  _b_5235_5215 = kk_string_empty(); /*string*/
  _match6395: ;
  _x6394 = kk_std_core_types__new_Optional(kk_string_box(_b_5235_5215), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6398;
  kk_function_t _b_5236_5223;
  kk_std_core_types__optional _x6399 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/
  if (kk_std_core_types__is_Optional(_x6399)) {
    kk_box_t _fun_unbox_x5219 = _x6399._cons.Optional.value;
    _b_5236_5223 = kk_std_time_calendar_new_iso_calendar_fun6400(_fun_unbox_x5219, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _b_5236_5223 = kk_std_time_calendar_new_iso_calendar_fun6403(_ctx); /*(std/time/date/date) -> string*/
  }
  _x6398 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_iso_calendar_fun6405(_b_5236_5223, _ctx)), _ctx); /*optional<106>*/
  return kk_std_time_calendar_solar_calendar(_x6358, _x6359, kk_std_time_calendar_new_iso_calendar_fun6363(_ctx), kk_std_time_calendar_new_iso_calendar_fun6364(_ctx), kk_std_time_calendar_new_iso_calendar_fun6384(_ctx), kk_std_time_calendar_new_iso_calendar_fun6387(_ctx), _x6388, _x6391, _x6394, _x6398, _ctx);
}
 
// The standard [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) calendar
// using TI time (`ts-ti`). This is a proleptic Gregorian
// calendar except that it uses the year 0 for 1 BCE, -1 for 2 BCE etc.
//
// This is the default calendar used in the library as it guarantees deterministic
// date calculations while still taking historical leap seconds into account.


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6413__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun6413(kk_function_t _fself, kk_integer_t _x16412, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6413(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun6413, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun6413(kk_function_t _fself, kk_integer_t _x16412, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16412, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6414__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_fun6414(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6414(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun6414, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_fun6414(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__tuple2_ _match_5654;
  kk_integer_t _x6415 = kk_integer_dup(days); /*int*/
  _match_5654 = kk_std_time_calendar_iso_estimate_year(_x6415, _ctx); /*(int, maybe<int>)*/
  kk_box_t _box_x5237 = _match_5654.fst;
  kk_box_t _box_x5238 = _match_5654.snd;
  kk_integer_t approx = kk_integer_unbox(_box_x5237);
  kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5238, NULL);
  kk_integer_dup(approx);
  kk_std_core_types__maybe_dup(safe);
  kk_std_core_types__tuple2__drop(_match_5654, _ctx);
  kk_Unit;
  kk_integer_t doy1;
  kk_integer_t _x6418 = kk_integer_dup(days); /*int*/
  kk_integer_t _x6419;
  kk_integer_t _x6420 = kk_integer_dup(approx); /*int*/
  _x6419 = kk_std_time_calendar_iso_days_before_year(_x6420, _ctx); /*int*/
  doy1 = kk_integer_sub(_x6418,_x6419,kk_context()); /*int*/
  bool _match_5655;
  kk_integer_t _borrowedArg_5659;
  if (kk_std_core_types__is_Nothing(safe)) {
    _borrowedArg_5659 = kk_integer_from_small(0); /*int*/
  }
  else {
    kk_box_t _box_x5239 = safe._cons.Just.value;
    kk_integer_t x = kk_integer_unbox(_box_x5239);
    kk_integer_dup(x);
    kk_std_core_types__maybe_drop(safe, _ctx);
    kk_Unit;
    _borrowedArg_5659 = x; /*int*/
  }
  bool _borrowApp_5660 = kk_integer_lte_borrow(doy1,_borrowedArg_5659,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5659, _ctx);
  _match_5655 = _borrowApp_5660; /*bool*/
  if (_match_5655) {
    kk_integer_drop(days, _ctx);
    kk_integer_t _b_5263_5240 = approx; /*int*/;
    kk_integer_t _b_5264_5241 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5263_5240), kk_integer_box(_b_5264_5241), _ctx);
  }
  kk_integer_t doy2;
  kk_integer_t _x6422;
  kk_integer_t year_4908;
  kk_integer_t _x6423 = kk_integer_dup(approx); /*int*/
  year_4908 = kk_integer_add(_x6423,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_integer_t y = kk_integer_sub(year_4908,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t leapdays;
  kk_integer_t _x6424;
  kk_integer_t _x6425;
  kk_integer_t _x6426 = kk_integer_dup(y); /*int*/
  _x6425 = kk_integer_div(_x6426,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t _x6427;
  kk_integer_t _x6428 = kk_integer_dup(y); /*int*/
  _x6427 = kk_integer_div(_x6428,(kk_integer_from_small(100)),kk_context()); /*int*/
  _x6424 = kk_integer_sub(_x6425,_x6427,kk_context()); /*int*/
  kk_integer_t _x6429;
  kk_integer_t _x6430 = kk_integer_dup(y); /*int*/
  _x6429 = kk_integer_div(_x6430,(kk_integer_from_small(400)),kk_context()); /*int*/
  leapdays = kk_integer_add(_x6424,_x6429,kk_context()); /*int*/
  kk_integer_t _x6431 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/
  _x6422 = kk_integer_add(_x6431,leapdays,kk_context()); /*int*/
  doy2 = kk_integer_sub(days,_x6422,kk_context()); /*int*/
  bool _match_5656;
  kk_integer_t _borrowedArg_5657 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_5658 = kk_integer_lt_borrow(doy2,_borrowedArg_5657,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5657, _ctx);
  _match_5656 = _borrowApp_5658; /*bool*/
  if (_match_5656) {
    kk_integer_drop(doy2, _ctx);
    kk_integer_t _b_5265_5242 = approx; /*int*/;
    kk_integer_t _b_5266_5243 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5265_5242), kk_integer_box(_b_5266_5243), _ctx);
  }
  kk_integer_drop(doy1, _ctx);
  kk_integer_t _b_5267_5244 = kk_integer_add(approx,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t _b_5268_5245 = doy2; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5267_5244), kk_integer_box(_b_5268_5245), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6434__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun6434(kk_function_t _fself, kk_integer_t _x16432, kk_integer_t _x26433, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6434(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun6434, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun6434(kk_function_t _fself, kk_integer_t _x16432, kk_integer_t _x26433, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16432, _x26433, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6437__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_fun6437(kk_function_t _fself, kk_integer_t _x16435, kk_integer_t _x26436, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6437(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun6437, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_iso_fun6437(kk_function_t _fself, kk_integer_t _x16435, kk_integer_t _x26436, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16435, _x26436, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6450__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5255;
};
static kk_string_t kk_std_time_calendar_cal_iso_fun6450(kk_function_t _fself, kk_std_time_date__date _b_5258, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6450(kk_box_t _fun_unbox_x5255, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_iso_fun6450__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_cal_iso_fun6450__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_cal_iso_fun6450, kk_context());
  _self->_fun_unbox_x5255 = _fun_unbox_x5255;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_cal_iso_fun6450(kk_function_t _fself, kk_std_time_date__date _b_5258, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_iso_fun6450__t* _self = kk_function_as(struct kk_std_time_calendar_cal_iso_fun6450__t*, _fself);
  kk_box_t _fun_unbox_x5255 = _self->_fun_unbox_x5255; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5255);}, {}, _ctx)
  kk_box_t _x6451;
  kk_function_t _x6452 = kk_function_unbox(_fun_unbox_x5255); /*(5256) -> 5257*/
  _x6451 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6452, (_x6452, kk_std_time_date__date_box(_b_5258, _ctx), _ctx)); /*5257*/
  return kk_string_unbox(_x6451);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6453__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_cal_iso_fun6453(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6453(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_fun6453, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_cal_iso_fun6453(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_fun6455__t {
  struct kk_function_s _base;
  kk_function_t _b_5272_5259;
};
static kk_box_t kk_std_time_calendar_cal_iso_fun6455(kk_function_t _fself, kk_box_t _b_5262, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_fun6455(kk_function_t _b_5272_5259, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_iso_fun6455__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_cal_iso_fun6455__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_cal_iso_fun6455, kk_context());
  _self->_b_5272_5259 = _b_5272_5259;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_cal_iso_fun6455(kk_function_t _fself, kk_box_t _b_5262, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_iso_fun6455__t* _self = kk_function_as(struct kk_std_time_calendar_cal_iso_fun6455__t*, _fself);
  kk_function_t _b_5272_5259 = _self->_b_5272_5259; /* (std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5272_5259);}, {}, _ctx)
  kk_string_t _x6456;
  kk_std_time_date__date _x6457 = kk_std_time_date__date_unbox(_b_5262, _ctx); /*std/time/date/date*/
  _x6456 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5272_5259, (_b_5272_5259, _x6457, _ctx)); /*string*/
  return kk_string_box(_x6456);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_iso;
 
// Return the instant in time for a given `:date` and `:clock` (= `clock0`) interpreted by
// calendar `cal` (=`cal-iso`) in a timezone `tz` (=`tz-utc` by default).

kk_std_time_instant__instant kk_std_time_calendar_instant(kk_std_time_instant__timescale ts, kk_std_time_date__date d, kk_std_core_types__optional c, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (ts : std/time/instant/timescale, d : std/time/date/date, c : optional<std/time/date/clock>, tz : optional<timezone>, cal : optional<calendar>) -> std/time/instant/instant */ 
  kk_function_t _x6464;
  kk_std_time_calendar__calendar _match_5649;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5275 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_2960 = kk_std_time_calendar__calendar_unbox(_box_x5275, NULL);
    kk_std_time_calendar__calendar_dup(_cal_2960);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5649 = _cal_2960; /*std/time/calendar/calendar*/
    goto _match6465;
  }
  _match_5649 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match6465: ;
  struct kk_std_time_calendar_Calendar* _con6467 = kk_std_time_calendar__as_Calendar(_match_5649);
  kk_string_t _pat0 = _con6467->name;
  kk_string_t _pat1 = _con6467->long_name;
  kk_string_t _pat2 = _con6467->month_prefix;
  kk_function_t _pat3 = _con6467->show_era;
  kk_function_t _pat4 = _con6467->instant_to_dc;
  kk_function_t _x = _con6467->dc_to_instant;
  kk_function_t _pat5 = _con6467->days_to_date;
  kk_function_t _pat6 = _con6467->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5649))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5649);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5649, _ctx);
    kk_Unit;
  }
  _x6464 = _x; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  kk_std_time_date__clock _x6458;
  if (kk_std_core_types__is_Optional(c)) {
    kk_box_t _box_x5273 = c._cons.Optional.value;
    kk_std_time_date__clock _c_2952 = kk_std_time_date__clock_unbox(_box_x5273, NULL);
    kk_std_time_date__clock_dup(_c_2952);
    kk_std_core_types__optional_drop(c, _ctx);
    kk_Unit;
    _x6458 = _c_2952; /*std/time/date/clock*/
    goto _match6459;
  }
  _x6458 = kk_std_time_date__clock_dup(kk_std_time_date_clock0); /*std/time/date/clock*/
  _match6459: ;
  kk_std_time_calendar__timezone _x6461;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5274 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_2956 = kk_std_time_calendar__timezone_unbox(_box_x5274, NULL);
    kk_std_time_calendar__timezone_dup(_tz_2956);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _x6461 = _tz_2956; /*std/time/calendar/timezone*/
    goto _match6462;
  }
  _x6461 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc); /*std/time/calendar/timezone*/
  _match6462: ;
  return kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x6464, (_x6464, d, _x6458, _x6461, ts, _ctx));
}
 
// monadic lift

kk_std_time_instant__instant kk_std_time_calendar__mlift4905_instant_1(kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_std_time_instant__timescale _c_4885, kk_context_t* _ctx) { /* (c : optional<std/time/date/clock>, cal : optional<calendar>, d : std/time/date/date, tz : optional<timezone>, std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_evv_t w = kk_evv_swap_create0(kk_context()); /*std/core/hnd/evv<(<>)>*/;
  kk_std_time_instant__instant x;
  kk_function_t _x6474;
  kk_std_time_calendar__calendar _match_5648;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5278 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_2982 = kk_std_time_calendar__calendar_unbox(_box_x5278, NULL);
    kk_std_time_calendar__calendar_dup(_cal_2982);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5648 = _cal_2982; /*std/time/calendar/calendar*/
    goto _match6475;
  }
  _match_5648 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match6475: ;
  struct kk_std_time_calendar_Calendar* _con6477 = kk_std_time_calendar__as_Calendar(_match_5648);
  kk_string_t _pat0 = _con6477->name;
  kk_string_t _pat1 = _con6477->long_name;
  kk_string_t _pat2 = _con6477->month_prefix;
  kk_function_t _pat3 = _con6477->show_era;
  kk_function_t _pat4 = _con6477->instant_to_dc;
  kk_function_t _x = _con6477->dc_to_instant;
  kk_function_t _pat50 = _con6477->days_to_date;
  kk_function_t _pat60 = _con6477->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5648))) {
    kk_function_drop(_pat60, _ctx);
    kk_Unit;
    kk_function_drop(_pat50, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5648);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5648, _ctx);
    kk_Unit;
  }
  _x6474 = _x; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  kk_std_time_date__clock _x6468;
  if (kk_std_core_types__is_Optional(c)) {
    kk_box_t _box_x5276 = c._cons.Optional.value;
    kk_std_time_date__clock _c_2974 = kk_std_time_date__clock_unbox(_box_x5276, NULL);
    kk_std_time_date__clock_dup(_c_2974);
    kk_std_core_types__optional_drop(c, _ctx);
    kk_Unit;
    _x6468 = _c_2974; /*std/time/date/clock*/
    goto _match6469;
  }
  _x6468 = kk_std_time_date__clock_dup(kk_std_time_date_clock0); /*std/time/date/clock*/
  _match6469: ;
  kk_std_time_calendar__timezone _x6471;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5277 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_2978 = kk_std_time_calendar__timezone_unbox(_box_x5277, NULL);
    kk_std_time_calendar__timezone_dup(_tz_2978);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _x6471 = _tz_2978; /*std/time/calendar/timezone*/
    goto _match6472;
  }
  _x6471 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc); /*std/time/calendar/timezone*/
  _match6472: ;
  x = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x6474, (_x6474, d, _x6468, _x6471, _c_4885, _ctx)); /*std/time/instant/instant*/
  kk_unit_t keep = kk_Unit;
  kk_evv_set(w,kk_context());
  return x;
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6484__t_1 {
  struct kk_function_s _base;
  kk_std_core_types__optional c;
  kk_std_core_types__optional cal;
  kk_std_time_date__date d;
  kk_std_core_types__optional tz;
};
static kk_std_time_instant__instant kk_std_time_calendar_instant_fun6484_1(kk_function_t _fself, kk_std_time_instant__timescale _c_4885, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6484_1(kk_std_core_types__optional c, kk_std_core_types__optional cal, kk_std_time_date__date d, kk_std_core_types__optional tz, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6484__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun6484__t_1, 10, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun6484_1, kk_context());
  _self->c = c;
  _self->cal = cal;
  _self->d = d;
  _self->tz = tz;
  return &_self->_base;
}

static kk_std_time_instant__instant kk_std_time_calendar_instant_fun6484_1(kk_function_t _fself, kk_std_time_instant__timescale _c_4885, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6484__t_1* _self = kk_function_as(struct kk_std_time_calendar_instant_fun6484__t_1*, _fself);
  kk_std_core_types__optional c = _self->c; /* optional<std/time/date/clock> */
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_time_date__date d = _self->d; /* std/time/date/date */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(c);kk_std_core_types__optional_dup(cal);kk_std_time_date__date_dup(d);kk_std_core_types__optional_dup(tz);}, {}, _ctx)
  return kk_std_time_calendar__mlift4905_instant_1(c, cal, d, tz, _c_4885, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6486__t_1 {
  struct kk_function_s _base;
  kk_function_t _b_5291_5286;
};
static kk_box_t kk_std_time_calendar_instant_fun6486_1(kk_function_t _fself, kk_box_t _b_5287, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6486_1(kk_function_t _b_5291_5286, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6486__t_1* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun6486__t_1, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun6486_1, kk_context());
  _self->_b_5291_5286 = _b_5291_5286;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_instant_fun6486_1(kk_function_t _fself, kk_box_t _b_5287, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6486__t_1* _self = kk_function_as(struct kk_std_time_calendar_instant_fun6486__t_1*, _fself);
  kk_function_t _b_5291_5286 = _self->_b_5291_5286; /* (std/time/instant/timescale) -> std/time/instant/instant */
  kk_drop_match(_self, {kk_function_dup(_b_5291_5286);}, {}, _ctx)
  kk_std_time_instant__instant _x6487;
  kk_std_time_instant__timescale _x6488 = kk_std_time_instant__timescale_unbox(_b_5287, _ctx); /*std/time/instant/timescale*/
  _x6487 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_instant__timescale, kk_context_t*), _b_5291_5286, (_b_5291_5286, _x6488, _ctx)); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x6487, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_instant_1(kk_std_time_date__date d, kk_std_core_types__optional c, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_std_core_types__optional ts, kk_context_t* _ctx) { /* (d : std/time/date/date, c : optional<std/time/date/clock>, tz : optional<timezone>, cal : optional<calendar>, ts : optional<std/time/instant/timescale>) -> std/time/utc/utc std/time/instant/instant */ 
  kk_std_time_instant__timescale x_4910;
  if (kk_std_core_types__is_Optional(ts)) {
    kk_box_t _box_x5279 = ts._cons.Optional.value;
    kk_std_time_instant__timescale _ts_2988 = kk_std_time_instant__timescale_unbox(_box_x5279, NULL);
    kk_std_time_instant__timescale_dup(_ts_2988);
    kk_std_core_types__optional_drop(ts, _ctx);
    kk_Unit;
    x_4910 = _ts_2988; /*std/time/instant/timescale*/
    goto _match6478;
  }
  kk_std_core_hnd__ev ev_4913;
  kk_ssize_t _x6480 = ((kk_ssize_t)0); /*ssize_t*/
  ev_4913 = kk_evv_at(_x6480,kk_context()); /*std/core/hnd/ev<std/time/utc/.hnd-utc>*/
  kk_box_t _x6481;
  struct kk_std_core_hnd_Ev* _con6482 = kk_std_core_hnd__as_Ev(ev_4913);
  kk_std_core_hnd__marker m0 = _con6482->marker;
  kk_box_t _box_x5280 = _con6482->hnd;
  kk_std_time_utc__hnd_utc h = kk_std_time_utc__hnd_utc_unbox(_box_x5280, NULL);
  kk_std_time_utc__hnd_utc_dup(h);
  kk_std_core_hnd__clause0 _match_5646;
  kk_std_core_hnd__clause0 _borrowApp_5647 = kk_std_time_utc__select_utc(h, _ctx); /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/;
  kk_std_time_utc__hnd_utc_drop(h, _ctx);
  _match_5646 = _borrowApp_5647; /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/
  kk_function_t _fun_unbox_x5283 = _match_5646.clause;
  _x6481 = kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_context_t*), _fun_unbox_x5283, (_fun_unbox_x5283, m0, ev_4913, _ctx)); /*37*/
  x_4910 = kk_std_time_instant__timescale_unbox(_x6481, _ctx); /*std/time/instant/timescale*/
  _match6478: ;
  if (kk_yielding(kk_context())) {
    kk_std_time_instant__timescale_dropn(x_4910, ((int32_t)10), _ctx);
    kk_function_t _b_5291_5286 = kk_std_time_calendar_new_instant_fun6484_1(c, cal, d, tz, _ctx); /*(std/time/instant/timescale) -> std/time/instant/instant*/;
    kk_box_t _x6485 = kk_std_core_hnd_yield_extend(kk_std_time_calendar_new_instant_fun6486_1(_b_5291_5286, _ctx), _ctx); /*3858*/
    return kk_std_time_instant__instant_unbox(_x6485, _ctx);
  }
  kk_evv_t w = kk_evv_swap_create0(kk_context()); /*std/core/hnd/evv<(<>)>*/;
  kk_std_time_instant__instant x0;
  kk_function_t _x6495;
  kk_std_time_calendar__calendar _match_5645;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5290 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_2982 = kk_std_time_calendar__calendar_unbox(_box_x5290, NULL);
    kk_std_time_calendar__calendar_dup(_cal_2982);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5645 = _cal_2982; /*std/time/calendar/calendar*/
    goto _match6496;
  }
  _match_5645 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match6496: ;
  struct kk_std_time_calendar_Calendar* _con6498 = kk_std_time_calendar__as_Calendar(_match_5645);
  kk_string_t _pat01 = _con6498->name;
  kk_string_t _pat11 = _con6498->long_name;
  kk_string_t _pat20 = _con6498->month_prefix;
  kk_function_t _pat30 = _con6498->show_era;
  kk_function_t _pat40 = _con6498->instant_to_dc;
  kk_function_t _x = _con6498->dc_to_instant;
  kk_function_t _pat50 = _con6498->days_to_date;
  kk_function_t _pat60 = _con6498->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5645))) {
    kk_function_drop(_pat60, _ctx);
    kk_Unit;
    kk_function_drop(_pat50, _ctx);
    kk_Unit;
    kk_function_drop(_pat40, _ctx);
    kk_Unit;
    kk_function_drop(_pat30, _ctx);
    kk_Unit;
    kk_string_drop(_pat20, _ctx);
    kk_Unit;
    kk_string_drop(_pat11, _ctx);
    kk_Unit;
    kk_string_drop(_pat01, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5645);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5645, _ctx);
    kk_Unit;
  }
  _x6495 = _x; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  kk_std_time_date__clock _x6489;
  if (kk_std_core_types__is_Optional(c)) {
    kk_box_t _box_x5288 = c._cons.Optional.value;
    kk_std_time_date__clock _c_2974 = kk_std_time_date__clock_unbox(_box_x5288, NULL);
    kk_std_time_date__clock_dup(_c_2974);
    kk_std_core_types__optional_drop(c, _ctx);
    kk_Unit;
    _x6489 = _c_2974; /*std/time/date/clock*/
    goto _match6490;
  }
  _x6489 = kk_std_time_date__clock_dup(kk_std_time_date_clock0); /*std/time/date/clock*/
  _match6490: ;
  kk_std_time_calendar__timezone _x6492;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5289 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_2978 = kk_std_time_calendar__timezone_unbox(_box_x5289, NULL);
    kk_std_time_calendar__timezone_dup(_tz_2978);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _x6492 = _tz_2978; /*std/time/calendar/timezone*/
    goto _match6493;
  }
  _x6492 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc); /*std/time/calendar/timezone*/
  _match6493: ;
  x0 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x6495, (_x6495, d, _x6489, _x6492, x_4910, _ctx)); /*std/time/instant/instant*/
  kk_unit_t keep = kk_Unit;
  kk_evv_set(w,kk_context());
  return x0;
}
 
/* Return the instant in time for a given date and clock interpreted by
   calendar `cal` (=`cal-iso`) in a timezone `tz` (=`tz-utc` by default).
  The `month`, `day`, `hour`, `minutes` may be outside their usual ranges
  and will be normalized during the conversion. For example, January 33 converts to February 2.
  This makes it very easy to add- or subtract days or months to an existing time.
  When the `seconds` or fraction of seconds `frac` add up over 60 those extra seconds are
  interpreted as leap seconds.
  Due to timezone transitions, or leap seconds, it is possible to specify dates that never
  happened (as it was skipped by a timezone change), or ambigious times (as a timezone springs back).
  In such cases, the time is always interpreted in the earlier timezone.
*/

kk_std_time_instant__instant kk_std_time_calendar_instant_2(kk_std_time_instant__timescale ts, kk_integer_t year, kk_std_core_types__optional month, kk_std_core_types__optional day, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional seconds, kk_std_core_types__optional frac, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (ts : std/time/instant/timescale, year : int, month : optional<int>, day : optional<int>, hours : optional<int>, minutes : optional<int>, seconds : optional<int>, frac : optional<double>, tz : optional<timezone>, cal : optional<calendar>) -> std/time/instant/instant */ 
  kk_std_num_ddouble__ddouble fsecs;
  kk_std_num_ddouble__ddouble _x6499;
  kk_integer_t _x6500;
  if (kk_std_core_types__is_Optional(seconds)) {
    kk_box_t _box_x5292 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3041 = kk_integer_unbox(_box_x5292);
    kk_integer_dup(_seconds_3041);
    kk_std_core_types__optional_drop(seconds, _ctx);
    kk_Unit;
    _x6500 = _seconds_3041; /*int*/
    goto _match6501;
  }
  _x6500 = kk_integer_from_small(0); /*int*/
  _match6501: ;
  _x6499 = kk_std_num_ddouble_ddouble_int_exp(_x6500, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
  kk_std_num_ddouble__ddouble _x6503;
  double _x6504;
  if (kk_std_core_types__is_Optional(frac)) {
    kk_box_t _box_x5293 = frac._cons.Optional.value;
    double _frac_3045 = kk_double_unbox(_box_x5293, NULL);
    kk_std_core_types__optional_drop(frac, _ctx);
    kk_Unit;
    _x6504 = _frac_3045; /*double*/
    goto _match6505;
  }
  _x6504 = 0.0; /*double*/
  _match6505: ;
  _x6503 = kk_std_num_ddouble__new_Ddouble(_x6504, 0.0, _ctx); /*std/num/ddouble/ddouble*/
  fsecs = kk_std_num_ddouble__lp__plus__rp_(_x6499, _x6503, _ctx); /*std/num/ddouble/ddouble*/
  kk_function_t _x6524;
  kk_std_time_calendar__calendar _match_5643;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5299 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3053 = kk_std_time_calendar__calendar_unbox(_box_x5299, NULL);
    kk_std_time_calendar__calendar_dup(_cal_3053);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5643 = _cal_3053; /*std/time/calendar/calendar*/
    goto _match6525;
  }
  _match_5643 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match6525: ;
  struct kk_std_time_calendar_Calendar* _con6527 = kk_std_time_calendar__as_Calendar(_match_5643);
  kk_string_t _pat0 = _con6527->name;
  kk_string_t _pat1 = _con6527->long_name;
  kk_string_t _pat2 = _con6527->month_prefix;
  kk_function_t _pat3 = _con6527->show_era;
  kk_function_t _pat4 = _con6527->instant_to_dc;
  kk_function_t _x = _con6527->dc_to_instant;
  kk_function_t _pat5 = _con6527->days_to_date;
  kk_function_t _pat6 = _con6527->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5643))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5643);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5643, _ctx);
    kk_Unit;
  }
  _x6524 = _x; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  kk_std_time_date__date _x6507;
  kk_integer_t _x6508;
  if (kk_std_core_types__is_Optional(month)) {
    kk_box_t _box_x5294 = month._cons.Optional.value;
    kk_integer_t _month_3025 = kk_integer_unbox(_box_x5294);
    kk_integer_dup(_month_3025);
    kk_std_core_types__optional_drop(month, _ctx);
    kk_Unit;
    _x6508 = _month_3025; /*int*/
    goto _match6509;
  }
  _x6508 = kk_integer_from_small(1); /*int*/
  _match6509: ;
  kk_integer_t _x6511;
  if (kk_std_core_types__is_Optional(day)) {
    kk_box_t _box_x5295 = day._cons.Optional.value;
    kk_integer_t _day_3029 = kk_integer_unbox(_box_x5295);
    kk_integer_dup(_day_3029);
    kk_std_core_types__optional_drop(day, _ctx);
    kk_Unit;
    _x6511 = _day_3029; /*int*/
    goto _match6512;
  }
  _x6511 = kk_integer_from_small(1); /*int*/
  _match6512: ;
  _x6507 = kk_std_time_date__new_Date(year, _x6508, _x6511, _ctx); /*std/time/date/date*/
  kk_std_time_date__clock _x6514;
  kk_integer_t _x6515;
  if (kk_std_core_types__is_Optional(hours)) {
    kk_box_t _box_x5296 = hours._cons.Optional.value;
    kk_integer_t _hours_3033 = kk_integer_unbox(_box_x5296);
    kk_integer_dup(_hours_3033);
    kk_std_core_types__optional_drop(hours, _ctx);
    kk_Unit;
    _x6515 = _hours_3033; /*int*/
    goto _match6516;
  }
  _x6515 = kk_integer_from_small(0); /*int*/
  _match6516: ;
  kk_integer_t _x6518;
  if (kk_std_core_types__is_Optional(minutes)) {
    kk_box_t _box_x5297 = minutes._cons.Optional.value;
    kk_integer_t _minutes_3037 = kk_integer_unbox(_box_x5297);
    kk_integer_dup(_minutes_3037);
    kk_std_core_types__optional_drop(minutes, _ctx);
    kk_Unit;
    _x6518 = _minutes_3037; /*int*/
    goto _match6519;
  }
  _x6518 = kk_integer_from_small(0); /*int*/
  _match6519: ;
  _x6514 = kk_std_time_date__new_Clock(kk_reuse_null, _x6515, _x6518, fsecs, _ctx); /*std/time/date/clock*/
  kk_std_time_calendar__timezone _x6521;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5298 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_3049 = kk_std_time_calendar__timezone_unbox(_box_x5298, NULL);
    kk_std_time_calendar__timezone_dup(_tz_3049);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _x6521 = _tz_3049; /*std/time/calendar/timezone*/
    goto _match6522;
  }
  _x6521 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc); /*std/time/calendar/timezone*/
  _match6522: ;
  return kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x6524, (_x6524, _x6507, _x6514, _x6521, ts, _ctx));
}
 
// monadic lift

kk_std_time_instant__instant kk_std_time_calendar__mlift4906_instant_3(kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_integer_t year, kk_std_time_instant__timescale _c_4887, kk_context_t* _ctx) { /* (cal : optional<calendar>, day : optional<int>, frac : optional<double>, hours : optional<int>, minutes : optional<int>, month : optional<int>, seconds : optional<int>, tz : optional<timezone>, year : int, std/time/instant/timescale) -> std/time/instant/instant */ 
  kk_evv_t w = kk_evv_swap_create0(kk_context()); /*std/core/hnd/evv<(<>)>*/;
  kk_std_num_ddouble__ddouble fsecs;
  kk_std_num_ddouble__ddouble _x6528;
  kk_integer_t _x6529;
  if (kk_std_core_types__is_Optional(seconds)) {
    kk_box_t _box_x5300 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3155 = kk_integer_unbox(_box_x5300);
    kk_integer_dup(_seconds_3155);
    kk_std_core_types__optional_drop(seconds, _ctx);
    kk_Unit;
    _x6529 = _seconds_3155; /*int*/
    goto _match6530;
  }
  _x6529 = kk_integer_from_small(0); /*int*/
  _match6530: ;
  _x6528 = kk_std_num_ddouble_ddouble_int_exp(_x6529, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
  kk_std_num_ddouble__ddouble _x6532;
  double _x6533;
  if (kk_std_core_types__is_Optional(frac)) {
    kk_box_t _box_x5301 = frac._cons.Optional.value;
    double _frac_3159 = kk_double_unbox(_box_x5301, NULL);
    kk_std_core_types__optional_drop(frac, _ctx);
    kk_Unit;
    _x6533 = _frac_3159; /*double*/
    goto _match6534;
  }
  _x6533 = 0.0; /*double*/
  _match6534: ;
  _x6532 = kk_std_num_ddouble__new_Ddouble(_x6533, 0.0, _ctx); /*std/num/ddouble/ddouble*/
  fsecs = kk_std_num_ddouble__lp__plus__rp_(_x6528, _x6532, _ctx); /*std/num/ddouble/ddouble*/
  kk_std_time_instant__instant x;
  kk_function_t _x6553;
  kk_std_time_calendar__calendar _match_5642;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5307 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3167 = kk_std_time_calendar__calendar_unbox(_box_x5307, NULL);
    kk_std_time_calendar__calendar_dup(_cal_3167);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5642 = _cal_3167; /*std/time/calendar/calendar*/
    goto _match6554;
  }
  _match_5642 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match6554: ;
  struct kk_std_time_calendar_Calendar* _con6556 = kk_std_time_calendar__as_Calendar(_match_5642);
  kk_string_t _pat0 = _con6556->name;
  kk_string_t _pat1 = _con6556->long_name;
  kk_string_t _pat2 = _con6556->month_prefix;
  kk_function_t _pat3 = _con6556->show_era;
  kk_function_t _pat4 = _con6556->instant_to_dc;
  kk_function_t _x = _con6556->dc_to_instant;
  kk_function_t _pat5 = _con6556->days_to_date;
  kk_function_t _pat6 = _con6556->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5642))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5642);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5642, _ctx);
    kk_Unit;
  }
  _x6553 = _x; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  kk_std_time_date__date _x6536;
  kk_integer_t _x6537;
  if (kk_std_core_types__is_Optional(month)) {
    kk_box_t _box_x5302 = month._cons.Optional.value;
    kk_integer_t _month_3139 = kk_integer_unbox(_box_x5302);
    kk_integer_dup(_month_3139);
    kk_std_core_types__optional_drop(month, _ctx);
    kk_Unit;
    _x6537 = _month_3139; /*int*/
    goto _match6538;
  }
  _x6537 = kk_integer_from_small(1); /*int*/
  _match6538: ;
  kk_integer_t _x6540;
  if (kk_std_core_types__is_Optional(day)) {
    kk_box_t _box_x5303 = day._cons.Optional.value;
    kk_integer_t _day_3143 = kk_integer_unbox(_box_x5303);
    kk_integer_dup(_day_3143);
    kk_std_core_types__optional_drop(day, _ctx);
    kk_Unit;
    _x6540 = _day_3143; /*int*/
    goto _match6541;
  }
  _x6540 = kk_integer_from_small(1); /*int*/
  _match6541: ;
  _x6536 = kk_std_time_date__new_Date(year, _x6537, _x6540, _ctx); /*std/time/date/date*/
  kk_std_time_date__clock _x6543;
  kk_integer_t _x6544;
  if (kk_std_core_types__is_Optional(hours)) {
    kk_box_t _box_x5304 = hours._cons.Optional.value;
    kk_integer_t _hours_3147 = kk_integer_unbox(_box_x5304);
    kk_integer_dup(_hours_3147);
    kk_std_core_types__optional_drop(hours, _ctx);
    kk_Unit;
    _x6544 = _hours_3147; /*int*/
    goto _match6545;
  }
  _x6544 = kk_integer_from_small(0); /*int*/
  _match6545: ;
  kk_integer_t _x6547;
  if (kk_std_core_types__is_Optional(minutes)) {
    kk_box_t _box_x5305 = minutes._cons.Optional.value;
    kk_integer_t _minutes_3151 = kk_integer_unbox(_box_x5305);
    kk_integer_dup(_minutes_3151);
    kk_std_core_types__optional_drop(minutes, _ctx);
    kk_Unit;
    _x6547 = _minutes_3151; /*int*/
    goto _match6548;
  }
  _x6547 = kk_integer_from_small(0); /*int*/
  _match6548: ;
  _x6543 = kk_std_time_date__new_Clock(kk_reuse_null, _x6544, _x6547, fsecs, _ctx); /*std/time/date/clock*/
  kk_std_time_calendar__timezone _x6550;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5306 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_3163 = kk_std_time_calendar__timezone_unbox(_box_x5306, NULL);
    kk_std_time_calendar__timezone_dup(_tz_3163);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _x6550 = _tz_3163; /*std/time/calendar/timezone*/
    goto _match6551;
  }
  _x6550 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc); /*std/time/calendar/timezone*/
  _match6551: ;
  x = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x6553, (_x6553, _x6536, _x6543, _x6550, _c_4887, _ctx)); /*std/time/instant/instant*/
  kk_unit_t keep = kk_Unit;
  kk_evv_set(w,kk_context());
  return x;
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6563__t_3 {
  struct kk_function_s _base;
  kk_std_core_types__optional cal;
  kk_std_core_types__optional day;
  kk_std_core_types__optional frac;
  kk_std_core_types__optional hours;
  kk_std_core_types__optional minutes;
  kk_std_core_types__optional month;
  kk_std_core_types__optional seconds;
  kk_std_core_types__optional tz;
  kk_integer_t year;
};
static kk_std_time_instant__instant kk_std_time_calendar_instant_fun6563_3(kk_function_t _fself, kk_std_time_instant__timescale _c_4887, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6563_3(kk_std_core_types__optional cal, kk_std_core_types__optional day, kk_std_core_types__optional frac, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional month, kk_std_core_types__optional seconds, kk_std_core_types__optional tz, kk_integer_t year, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6563__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun6563__t_3, 18, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun6563_3, kk_context());
  _self->cal = cal;
  _self->day = day;
  _self->frac = frac;
  _self->hours = hours;
  _self->minutes = minutes;
  _self->month = month;
  _self->seconds = seconds;
  _self->tz = tz;
  _self->year = year;
  return &_self->_base;
}

static kk_std_time_instant__instant kk_std_time_calendar_instant_fun6563_3(kk_function_t _fself, kk_std_time_instant__timescale _c_4887, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6563__t_3* _self = kk_function_as(struct kk_std_time_calendar_instant_fun6563__t_3*, _fself);
  kk_std_core_types__optional cal = _self->cal; /* optional<std/time/calendar/calendar> */
  kk_std_core_types__optional day = _self->day; /* optional<int> */
  kk_std_core_types__optional frac = _self->frac; /* optional<double> */
  kk_std_core_types__optional hours = _self->hours; /* optional<int> */
  kk_std_core_types__optional minutes = _self->minutes; /* optional<int> */
  kk_std_core_types__optional month = _self->month; /* optional<int> */
  kk_std_core_types__optional seconds = _self->seconds; /* optional<int> */
  kk_std_core_types__optional tz = _self->tz; /* optional<std/time/calendar/timezone> */
  kk_integer_t year = _self->year; /* int */
  kk_drop_match(_self, {kk_std_core_types__optional_dup(cal);kk_std_core_types__optional_dup(day);kk_std_core_types__optional_dup(frac);kk_std_core_types__optional_dup(hours);kk_std_core_types__optional_dup(minutes);kk_std_core_types__optional_dup(month);kk_std_core_types__optional_dup(seconds);kk_std_core_types__optional_dup(tz);kk_integer_dup(year);}, {}, _ctx)
  return kk_std_time_calendar__mlift4906_instant_3(cal, day, frac, hours, minutes, month, seconds, tz, year, _c_4887, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_instant_fun6565__t_3 {
  struct kk_function_s _base;
  kk_function_t _b_5325_5315;
};
static kk_box_t kk_std_time_calendar_instant_fun6565_3(kk_function_t _fself, kk_box_t _b_5316, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_instant_fun6565_3(kk_function_t _b_5325_5315, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6565__t_3* _self = kk_function_alloc_as(struct kk_std_time_calendar_instant_fun6565__t_3, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_instant_fun6565_3, kk_context());
  _self->_b_5325_5315 = _b_5325_5315;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_instant_fun6565_3(kk_function_t _fself, kk_box_t _b_5316, kk_context_t* _ctx) {
  struct kk_std_time_calendar_instant_fun6565__t_3* _self = kk_function_as(struct kk_std_time_calendar_instant_fun6565__t_3*, _fself);
  kk_function_t _b_5325_5315 = _self->_b_5325_5315; /* (std/time/instant/timescale) -> std/time/instant/instant */
  kk_drop_match(_self, {kk_function_dup(_b_5325_5315);}, {}, _ctx)
  kk_std_time_instant__instant _x6566;
  kk_std_time_instant__timescale _x6567 = kk_std_time_instant__timescale_unbox(_b_5316, _ctx); /*std/time/instant/timescale*/
  _x6566 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_instant__timescale, kk_context_t*), _b_5325_5315, (_b_5325_5315, _x6567, _ctx)); /*std/time/instant/instant*/
  return kk_std_time_instant__instant_box(_x6566, _ctx);
}

kk_std_time_instant__instant kk_std_time_calendar_instant_3(kk_integer_t year, kk_std_core_types__optional month, kk_std_core_types__optional day, kk_std_core_types__optional hours, kk_std_core_types__optional minutes, kk_std_core_types__optional seconds, kk_std_core_types__optional frac, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_std_core_types__optional ts, kk_context_t* _ctx) { /* (year : int, month : optional<int>, day : optional<int>, hours : optional<int>, minutes : optional<int>, seconds : optional<int>, frac : optional<double>, tz : optional<timezone>, cal : optional<calendar>, ts : optional<std/time/instant/timescale>) -> std/time/utc/utc std/time/instant/instant */ 
  kk_std_time_instant__timescale x_4917;
  if (kk_std_core_types__is_Optional(ts)) {
    kk_box_t _box_x5308 = ts._cons.Optional.value;
    kk_std_time_instant__timescale _ts_3173 = kk_std_time_instant__timescale_unbox(_box_x5308, NULL);
    kk_std_time_instant__timescale_dup(_ts_3173);
    kk_std_core_types__optional_drop(ts, _ctx);
    kk_Unit;
    x_4917 = _ts_3173; /*std/time/instant/timescale*/
    goto _match6557;
  }
  kk_std_core_hnd__ev ev_4920;
  kk_ssize_t _x6559 = ((kk_ssize_t)0); /*ssize_t*/
  ev_4920 = kk_evv_at(_x6559,kk_context()); /*std/core/hnd/ev<std/time/utc/.hnd-utc>*/
  kk_box_t _x6560;
  struct kk_std_core_hnd_Ev* _con6561 = kk_std_core_hnd__as_Ev(ev_4920);
  kk_std_core_hnd__marker m0 = _con6561->marker;
  kk_box_t _box_x5309 = _con6561->hnd;
  kk_std_time_utc__hnd_utc h = kk_std_time_utc__hnd_utc_unbox(_box_x5309, NULL);
  kk_std_time_utc__hnd_utc_dup(h);
  kk_std_core_hnd__clause0 _match_5640;
  kk_std_core_hnd__clause0 _borrowApp_5641 = kk_std_time_utc__select_utc(h, _ctx); /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/;
  kk_std_time_utc__hnd_utc_drop(h, _ctx);
  _match_5640 = _borrowApp_5641; /*std/core/hnd/clause0<std/time/instant/timescale,std/time/utc/.hnd-utc,277,278>*/
  kk_function_t _fun_unbox_x5312 = _match_5640.clause;
  _x6560 = kk_function_call(kk_box_t, (kk_function_t, kk_std_core_hnd__marker, kk_std_core_hnd__ev, kk_context_t*), _fun_unbox_x5312, (_fun_unbox_x5312, m0, ev_4920, _ctx)); /*37*/
  x_4917 = kk_std_time_instant__timescale_unbox(_x6560, _ctx); /*std/time/instant/timescale*/
  _match6557: ;
  if (kk_yielding(kk_context())) {
    kk_std_time_instant__timescale_dropn(x_4917, ((int32_t)10), _ctx);
    kk_function_t _b_5325_5315 = kk_std_time_calendar_new_instant_fun6563_3(cal, day, frac, hours, minutes, month, seconds, tz, year, _ctx); /*(std/time/instant/timescale) -> std/time/instant/instant*/;
    kk_box_t _x6564 = kk_std_core_hnd_yield_extend(kk_std_time_calendar_new_instant_fun6565_3(_b_5325_5315, _ctx), _ctx); /*3858*/
    return kk_std_time_instant__instant_unbox(_x6564, _ctx);
  }
  kk_evv_t w = kk_evv_swap_create0(kk_context()); /*std/core/hnd/evv<(<>)>*/;
  kk_std_num_ddouble__ddouble fsecs;
  kk_std_num_ddouble__ddouble _x6568;
  kk_integer_t _x6569;
  if (kk_std_core_types__is_Optional(seconds)) {
    kk_box_t _box_x5317 = seconds._cons.Optional.value;
    kk_integer_t _seconds_3155 = kk_integer_unbox(_box_x5317);
    kk_integer_dup(_seconds_3155);
    kk_std_core_types__optional_drop(seconds, _ctx);
    kk_Unit;
    _x6569 = _seconds_3155; /*int*/
    goto _match6570;
  }
  _x6569 = kk_integer_from_small(0); /*int*/
  _match6570: ;
  _x6568 = kk_std_num_ddouble_ddouble_int_exp(_x6569, kk_integer_from_small(0), _ctx); /*std/num/ddouble/ddouble*/
  kk_std_num_ddouble__ddouble _x6572;
  double _x6573;
  if (kk_std_core_types__is_Optional(frac)) {
    kk_box_t _box_x5318 = frac._cons.Optional.value;
    double _frac_3159 = kk_double_unbox(_box_x5318, NULL);
    kk_std_core_types__optional_drop(frac, _ctx);
    kk_Unit;
    _x6573 = _frac_3159; /*double*/
    goto _match6574;
  }
  _x6573 = 0.0; /*double*/
  _match6574: ;
  _x6572 = kk_std_num_ddouble__new_Ddouble(_x6573, 0.0, _ctx); /*std/num/ddouble/ddouble*/
  fsecs = kk_std_num_ddouble__lp__plus__rp_(_x6568, _x6572, _ctx); /*std/num/ddouble/ddouble*/
  kk_std_time_instant__instant x0;
  kk_function_t _x6593;
  kk_std_time_calendar__calendar _match_5639;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5324 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_3167 = kk_std_time_calendar__calendar_unbox(_box_x5324, NULL);
    kk_std_time_calendar__calendar_dup(_cal_3167);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5639 = _cal_3167; /*std/time/calendar/calendar*/
    goto _match6594;
  }
  _match_5639 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match6594: ;
  struct kk_std_time_calendar_Calendar* _con6596 = kk_std_time_calendar__as_Calendar(_match_5639);
  kk_string_t _pat01 = _con6596->name;
  kk_string_t _pat11 = _con6596->long_name;
  kk_string_t _pat20 = _con6596->month_prefix;
  kk_function_t _pat30 = _con6596->show_era;
  kk_function_t _pat40 = _con6596->instant_to_dc;
  kk_function_t _x = _con6596->dc_to_instant;
  kk_function_t _pat5 = _con6596->days_to_date;
  kk_function_t _pat6 = _con6596->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5639))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat40, _ctx);
    kk_Unit;
    kk_function_drop(_pat30, _ctx);
    kk_Unit;
    kk_string_drop(_pat20, _ctx);
    kk_Unit;
    kk_string_drop(_pat11, _ctx);
    kk_Unit;
    kk_string_drop(_pat01, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5639);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5639, _ctx);
    kk_Unit;
  }
  _x6593 = _x; /*(std/time/date/date, std/time/date/clock, std/time/calendar/timezone, std/time/instant/timescale) -> std/time/instant/instant*/
  kk_std_time_date__date _x6576;
  kk_integer_t _x6577;
  if (kk_std_core_types__is_Optional(month)) {
    kk_box_t _box_x5319 = month._cons.Optional.value;
    kk_integer_t _month_3139 = kk_integer_unbox(_box_x5319);
    kk_integer_dup(_month_3139);
    kk_std_core_types__optional_drop(month, _ctx);
    kk_Unit;
    _x6577 = _month_3139; /*int*/
    goto _match6578;
  }
  _x6577 = kk_integer_from_small(1); /*int*/
  _match6578: ;
  kk_integer_t _x6580;
  if (kk_std_core_types__is_Optional(day)) {
    kk_box_t _box_x5320 = day._cons.Optional.value;
    kk_integer_t _day_3143 = kk_integer_unbox(_box_x5320);
    kk_integer_dup(_day_3143);
    kk_std_core_types__optional_drop(day, _ctx);
    kk_Unit;
    _x6580 = _day_3143; /*int*/
    goto _match6581;
  }
  _x6580 = kk_integer_from_small(1); /*int*/
  _match6581: ;
  _x6576 = kk_std_time_date__new_Date(year, _x6577, _x6580, _ctx); /*std/time/date/date*/
  kk_std_time_date__clock _x6583;
  kk_integer_t _x6584;
  if (kk_std_core_types__is_Optional(hours)) {
    kk_box_t _box_x5321 = hours._cons.Optional.value;
    kk_integer_t _hours_3147 = kk_integer_unbox(_box_x5321);
    kk_integer_dup(_hours_3147);
    kk_std_core_types__optional_drop(hours, _ctx);
    kk_Unit;
    _x6584 = _hours_3147; /*int*/
    goto _match6585;
  }
  _x6584 = kk_integer_from_small(0); /*int*/
  _match6585: ;
  kk_integer_t _x6587;
  if (kk_std_core_types__is_Optional(minutes)) {
    kk_box_t _box_x5322 = minutes._cons.Optional.value;
    kk_integer_t _minutes_3151 = kk_integer_unbox(_box_x5322);
    kk_integer_dup(_minutes_3151);
    kk_std_core_types__optional_drop(minutes, _ctx);
    kk_Unit;
    _x6587 = _minutes_3151; /*int*/
    goto _match6588;
  }
  _x6587 = kk_integer_from_small(0); /*int*/
  _match6588: ;
  _x6583 = kk_std_time_date__new_Clock(kk_reuse_null, _x6584, _x6587, fsecs, _ctx); /*std/time/date/clock*/
  kk_std_time_calendar__timezone _x6590;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5323 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_3163 = kk_std_time_calendar__timezone_unbox(_box_x5323, NULL);
    kk_std_time_calendar__timezone_dup(_tz_3163);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _x6590 = _tz_3163; /*std/time/calendar/timezone*/
    goto _match6591;
  }
  _x6590 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc); /*std/time/calendar/timezone*/
  _match6591: ;
  x0 = kk_function_call(kk_std_time_instant__instant, (kk_function_t, kk_std_time_date__date, kk_std_time_date__clock, kk_std_time_calendar__timezone, kk_std_time_instant__timescale, kk_context_t*), _x6593, (_x6593, _x6576, _x6583, _x6590, x_4917, _ctx)); /*std/time/instant/instant*/
  kk_unit_t keep = kk_Unit;
  kk_evv_set(w,kk_context());
  return x0;
}
 
// The (proleptic) [Gregorian calendar](https://en.wikipedia.org/wiki/Gregorian_calendar).
// Just like the ISO calendar except that the year 0 does not exist, i.e. after 1 BCE (=`~1`)
// we have 1 CE (=`1`).
// The calendar short name is ``GC``.


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6602__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6602(kk_function_t _fself, kk_integer_t _x16601, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6602(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6602, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6602(kk_function_t _fself, kk_integer_t _x16601, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16601, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6603__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_gregorian_fun6603(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6603(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6603, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_gregorian_fun6603(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__tuple2_ _match_5631;
  kk_integer_t _x6604 = kk_integer_dup(days); /*int*/
  _match_5631 = kk_std_time_calendar_iso_estimate_year(_x6604, _ctx); /*(int, maybe<int>)*/
  kk_box_t _box_x5326 = _match_5631.fst;
  kk_box_t _box_x5327 = _match_5631.snd;
  kk_integer_t approx = kk_integer_unbox(_box_x5326);
  kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5327, NULL);
  kk_integer_dup(approx);
  kk_std_core_types__maybe_dup(safe);
  kk_std_core_types__tuple2__drop(_match_5631, _ctx);
  kk_Unit;
  kk_integer_t doy1;
  kk_integer_t _x6607 = kk_integer_dup(days); /*int*/
  kk_integer_t _x6608;
  kk_integer_t _x6609 = kk_integer_dup(approx); /*int*/
  _x6608 = kk_std_time_calendar_iso_days_before_year(_x6609, _ctx); /*int*/
  doy1 = kk_integer_sub(_x6607,_x6608,kk_context()); /*int*/
  bool _match_5632;
  kk_integer_t _borrowedArg_5636;
  if (kk_std_core_types__is_Nothing(safe)) {
    _borrowedArg_5636 = kk_integer_from_small(0); /*int*/
  }
  else {
    kk_box_t _box_x5328 = safe._cons.Just.value;
    kk_integer_t x = kk_integer_unbox(_box_x5328);
    kk_integer_dup(x);
    kk_std_core_types__maybe_drop(safe, _ctx);
    kk_Unit;
    _borrowedArg_5636 = x; /*int*/
  }
  bool _borrowApp_5637 = kk_integer_lte_borrow(doy1,_borrowedArg_5636,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5636, _ctx);
  _match_5632 = _borrowApp_5637; /*bool*/
  if (_match_5632) {
    kk_integer_drop(days, _ctx);
    kk_integer_t _b_5344_5329 = approx; /*int*/;
    kk_integer_t _b_5345_5330 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5344_5329), kk_integer_box(_b_5345_5330), _ctx);
  }
  kk_integer_t doy2;
  kk_integer_t _x6611;
  kk_integer_t year_4923;
  kk_integer_t _x6612 = kk_integer_dup(approx); /*int*/
  year_4923 = kk_integer_add(_x6612,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_integer_t y = kk_integer_sub(year_4923,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t leapdays;
  kk_integer_t _x6613;
  kk_integer_t _x6614;
  kk_integer_t _x6615 = kk_integer_dup(y); /*int*/
  _x6614 = kk_integer_div(_x6615,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t _x6616;
  kk_integer_t _x6617 = kk_integer_dup(y); /*int*/
  _x6616 = kk_integer_div(_x6617,(kk_integer_from_small(100)),kk_context()); /*int*/
  _x6613 = kk_integer_sub(_x6614,_x6616,kk_context()); /*int*/
  kk_integer_t _x6618;
  kk_integer_t _x6619 = kk_integer_dup(y); /*int*/
  _x6618 = kk_integer_div(_x6619,(kk_integer_from_small(400)),kk_context()); /*int*/
  leapdays = kk_integer_add(_x6613,_x6618,kk_context()); /*int*/
  kk_integer_t _x6620 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/
  _x6611 = kk_integer_add(_x6620,leapdays,kk_context()); /*int*/
  doy2 = kk_integer_sub(days,_x6611,kk_context()); /*int*/
  bool _match_5633;
  kk_integer_t _borrowedArg_5634 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_5635 = kk_integer_lt_borrow(doy2,_borrowedArg_5634,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5634, _ctx);
  _match_5633 = _borrowApp_5635; /*bool*/
  if (_match_5633) {
    kk_integer_drop(doy2, _ctx);
    kk_integer_t _b_5346_5331 = approx; /*int*/;
    kk_integer_t _b_5347_5332 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5346_5331), kk_integer_box(_b_5347_5332), _ctx);
  }
  kk_integer_drop(doy1, _ctx);
  kk_integer_t _b_5348_5333 = kk_integer_add(approx,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t _b_5349_5334 = doy2; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5348_5333), kk_integer_box(_b_5349_5334), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6623__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6623(kk_function_t _fself, kk_integer_t _x16621, kk_integer_t _x26622, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6623(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6623, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6623(kk_function_t _fself, kk_integer_t _x16621, kk_integer_t _x26622, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16621, _x26622, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6626__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6626(kk_function_t _fself, kk_integer_t _x16624, kk_integer_t _x26625, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6626(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6626, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_gregorian_fun6626(kk_function_t _fself, kk_integer_t _x16624, kk_integer_t _x26625, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16624, _x26625, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6636__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_cal_gregorian_fun6636(kk_function_t _fself, kk_std_time_date__date d0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6636(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_gregorian_fun6636, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_cal_gregorian_fun6636(kk_function_t _fself, kk_std_time_date__date d0, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__order x_17078;
  kk_integer_t _borrowedArg_5627;
  kk_integer_t _x = d0.year;
  kk_integer_dup(_x);
  kk_std_time_date__date_drop(d0, _ctx);
  kk_Unit;
  _borrowedArg_5627 = _x; /*int*/
  kk_std_core_types__order _borrowApp_5628 = kk_int_as_order(kk_integer_signum_borrow(_borrowedArg_5627),kk_context()); /*order*/;
  kk_integer_drop(_borrowedArg_5627, _ctx);
  x_17078 = _borrowApp_5628; /*order*/
  bool _match_5623;
  kk_integer_t _borrowedArg_5625;
  if (kk_std_core_types__is_Lt(x_17078)) {
    _borrowedArg_5625 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/
    goto _match6637;
  }
  if (kk_std_core_types__is_Eq(x_17078)) {
    _borrowedArg_5625 = kk_integer_from_small(0); /*int*/
    goto _match6637;
  }
  _borrowedArg_5625 = kk_integer_from_small(1); /*int*/
  _match6637: ;
  kk_integer_t _borrowedArg_5624 = kk_integer_sub((kk_integer_from_small(0)),(kk_integer_from_small(1)),kk_context()); /*int*/;
  bool _borrowApp_5626 = kk_integer_eq_borrow(_borrowedArg_5625,_borrowedArg_5624,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5625, _ctx);
  kk_integer_drop(_borrowedArg_5624, _ctx);
  _match_5623 = _borrowApp_5626; /*bool*/
  if (_match_5623) {
    kk_define_string_literal(, _s6638, 3, "BCE")
    return kk_string_dup(_s6638);
  }
  kk_define_string_literal(, _s6639, 2, "CE")
  return kk_string_dup(_s6639);
}


// lift anonymous function
struct kk_std_time_calendar_cal_gregorian_fun6640__t {
  struct kk_function_s _base;
  kk_function_t _b_5353_5340;
};
static kk_box_t kk_std_time_calendar_cal_gregorian_fun6640(kk_function_t _fself, kk_box_t _b_5343, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_gregorian_fun6640(kk_function_t _b_5353_5340, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_gregorian_fun6640__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_cal_gregorian_fun6640__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_cal_gregorian_fun6640, kk_context());
  _self->_b_5353_5340 = _b_5353_5340;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_cal_gregorian_fun6640(kk_function_t _fself, kk_box_t _b_5343, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_gregorian_fun6640__t* _self = kk_function_as(struct kk_std_time_calendar_cal_gregorian_fun6640__t*, _fself);
  kk_function_t _b_5353_5340 = _self->_b_5353_5340; /* (d0 : std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5353_5340);}, {}, _ctx)
  kk_string_t _x6641;
  kk_std_time_date__date _x6642 = kk_std_time_date__date_unbox(_b_5343, _ctx); /*std/time/date/date*/
  _x6641 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5353_5340, (_b_5353_5340, _x6642, _ctx)); /*string*/
  return kk_string_box(_x6641);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_gregorian;
 
/* The standard [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) calendar
but using TI-SLS time (`ts-ti-sls`), i.e. TI with
[smoothed leap seconds](https://www.cl.cam.ac.uk/~mgk25/time/utc-sls/).
This calendar is equivalent to the ISO calendar except for the last 1000 seconds of a day where
a leap second occurs. On such day, the leap second time step (positive or negative)
is distributed over the last 1000 seconds of the day. On the full hour, ISO and ISO-SLS are equal again.
This is a recommended calendar to use for
time stamps or communication with other services since it avoids any potential trouble
with leap seconds while still being quite precise.
Other good properties include:
- All days have 86400 seconds, and the time 23:59:60 never appears
- The time never differs more than one second from UTC
- Time as always equal to UTC on the full and half hour
*/


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6648__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6648(kk_function_t _fself, kk_integer_t _x16647, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6648(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun6648, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6648(kk_function_t _fself, kk_integer_t _x16647, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16647, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6649__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_sls_fun6649(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6649(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun6649, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_iso_sls_fun6649(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__tuple2_ _match_5616;
  kk_integer_t _x6650 = kk_integer_dup(days); /*int*/
  _match_5616 = kk_std_time_calendar_iso_estimate_year(_x6650, _ctx); /*(int, maybe<int>)*/
  kk_box_t _box_x5354 = _match_5616.fst;
  kk_box_t _box_x5355 = _match_5616.snd;
  kk_integer_t approx = kk_integer_unbox(_box_x5354);
  kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5355, NULL);
  kk_integer_dup(approx);
  kk_std_core_types__maybe_dup(safe);
  kk_std_core_types__tuple2__drop(_match_5616, _ctx);
  kk_Unit;
  kk_integer_t doy1;
  kk_integer_t _x6653 = kk_integer_dup(days); /*int*/
  kk_integer_t _x6654;
  kk_integer_t _x6655 = kk_integer_dup(approx); /*int*/
  _x6654 = kk_std_time_calendar_iso_days_before_year(_x6655, _ctx); /*int*/
  doy1 = kk_integer_sub(_x6653,_x6654,kk_context()); /*int*/
  bool _match_5617;
  kk_integer_t _borrowedArg_5621;
  if (kk_std_core_types__is_Nothing(safe)) {
    _borrowedArg_5621 = kk_integer_from_small(0); /*int*/
  }
  else {
    kk_box_t _box_x5356 = safe._cons.Just.value;
    kk_integer_t x = kk_integer_unbox(_box_x5356);
    kk_integer_dup(x);
    kk_std_core_types__maybe_drop(safe, _ctx);
    kk_Unit;
    _borrowedArg_5621 = x; /*int*/
  }
  bool _borrowApp_5622 = kk_integer_lte_borrow(doy1,_borrowedArg_5621,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5621, _ctx);
  _match_5617 = _borrowApp_5622; /*bool*/
  if (_match_5617) {
    kk_integer_drop(days, _ctx);
    kk_integer_t _b_5380_5357 = approx; /*int*/;
    kk_integer_t _b_5381_5358 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5380_5357), kk_integer_box(_b_5381_5358), _ctx);
  }
  kk_integer_t doy2;
  kk_integer_t _x6657;
  kk_integer_t year_4924;
  kk_integer_t _x6658 = kk_integer_dup(approx); /*int*/
  year_4924 = kk_integer_add(_x6658,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_integer_t y = kk_integer_sub(year_4924,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t leapdays;
  kk_integer_t _x6659;
  kk_integer_t _x6660;
  kk_integer_t _x6661 = kk_integer_dup(y); /*int*/
  _x6660 = kk_integer_div(_x6661,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t _x6662;
  kk_integer_t _x6663 = kk_integer_dup(y); /*int*/
  _x6662 = kk_integer_div(_x6663,(kk_integer_from_small(100)),kk_context()); /*int*/
  _x6659 = kk_integer_sub(_x6660,_x6662,kk_context()); /*int*/
  kk_integer_t _x6664;
  kk_integer_t _x6665 = kk_integer_dup(y); /*int*/
  _x6664 = kk_integer_div(_x6665,(kk_integer_from_small(400)),kk_context()); /*int*/
  leapdays = kk_integer_add(_x6659,_x6664,kk_context()); /*int*/
  kk_integer_t _x6666 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/
  _x6657 = kk_integer_add(_x6666,leapdays,kk_context()); /*int*/
  doy2 = kk_integer_sub(days,_x6657,kk_context()); /*int*/
  bool _match_5618;
  kk_integer_t _borrowedArg_5619 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_5620 = kk_integer_lt_borrow(doy2,_borrowedArg_5619,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5619, _ctx);
  _match_5618 = _borrowApp_5620; /*bool*/
  if (_match_5618) {
    kk_integer_drop(doy2, _ctx);
    kk_integer_t _b_5382_5359 = approx; /*int*/;
    kk_integer_t _b_5383_5360 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5382_5359), kk_integer_box(_b_5383_5360), _ctx);
  }
  kk_integer_drop(doy1, _ctx);
  kk_integer_t _b_5384_5361 = kk_integer_add(approx,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t _b_5385_5362 = doy2; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5384_5361), kk_integer_box(_b_5385_5362), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6669__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6669(kk_function_t _fself, kk_integer_t _x16667, kk_integer_t _x26668, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6669(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun6669, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6669(kk_function_t _fself, kk_integer_t _x16667, kk_integer_t _x26668, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16667, _x26668, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6672__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6672(kk_function_t _fself, kk_integer_t _x16670, kk_integer_t _x26671, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6672(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun6672, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_iso_sls_fun6672(kk_function_t _fself, kk_integer_t _x16670, kk_integer_t _x26671, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16670, _x26671, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6685__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5372;
};
static kk_string_t kk_std_time_calendar_cal_iso_sls_fun6685(kk_function_t _fself, kk_std_time_date__date _b_5375, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6685(kk_box_t _fun_unbox_x5372, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_iso_sls_fun6685__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_cal_iso_sls_fun6685__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_cal_iso_sls_fun6685, kk_context());
  _self->_fun_unbox_x5372 = _fun_unbox_x5372;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_cal_iso_sls_fun6685(kk_function_t _fself, kk_std_time_date__date _b_5375, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_iso_sls_fun6685__t* _self = kk_function_as(struct kk_std_time_calendar_cal_iso_sls_fun6685__t*, _fself);
  kk_box_t _fun_unbox_x5372 = _self->_fun_unbox_x5372; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5372);}, {}, _ctx)
  kk_box_t _x6686;
  kk_function_t _x6687 = kk_function_unbox(_fun_unbox_x5372); /*(5373) -> 5374*/
  _x6686 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6687, (_x6687, kk_std_time_date__date_box(_b_5375, _ctx), _ctx)); /*5374*/
  return kk_string_unbox(_x6686);
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6688__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_cal_iso_sls_fun6688(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6688(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_iso_sls_fun6688, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_cal_iso_sls_fun6688(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_cal_iso_sls_fun6690__t {
  struct kk_function_s _base;
  kk_function_t _b_5389_5376;
};
static kk_box_t kk_std_time_calendar_cal_iso_sls_fun6690(kk_function_t _fself, kk_box_t _b_5379, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_iso_sls_fun6690(kk_function_t _b_5389_5376, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_iso_sls_fun6690__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_cal_iso_sls_fun6690__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_cal_iso_sls_fun6690, kk_context());
  _self->_b_5389_5376 = _b_5389_5376;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_cal_iso_sls_fun6690(kk_function_t _fself, kk_box_t _b_5379, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_iso_sls_fun6690__t* _self = kk_function_as(struct kk_std_time_calendar_cal_iso_sls_fun6690__t*, _fself);
  kk_function_t _b_5389_5376 = _self->_b_5389_5376; /* (std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5389_5376);}, {}, _ctx)
  kk_string_t _x6691;
  kk_std_time_date__date _x6692 = kk_std_time_date__date_unbox(_b_5379, _ctx); /*std/time/date/date*/
  _x6691 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5389_5376, (_b_5389_5376, _x6692, _ctx)); /*string*/
  return kk_string_box(_x6691);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_iso_sls;

kk_std_time_timestamp__timestamp kk_std_time_calendar_cal_timestamp(kk_std_time_calendar__calendar cal, kk_std_time_instant__timescale ts, kk_std_time_date__date d, kk_std_time_date__clock c, kk_context_t* _ctx) { /* (cal : calendar, ts : std/time/instant/timescale, d : std/time/date/date, c : std/time/date/clock) -> std/time/timestamp/timestamp */ 
  kk_function_t _x6693;
  struct kk_std_time_calendar_Calendar* _con6694 = kk_std_time_calendar__as_Calendar(cal);
  kk_string_t _pat0 = _con6694->name;
  kk_string_t _pat1 = _con6694->long_name;
  kk_string_t _pat2 = _con6694->month_prefix;
  kk_function_t _pat3 = _con6694->show_era;
  kk_function_t _pat4 = _con6694->instant_to_dc;
  kk_function_t _pat5 = _con6694->dc_to_instant;
  kk_function_t _pat6 = _con6694->days_to_date;
  kk_function_t _x = _con6694->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(cal))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(cal);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(cal, _ctx);
    kk_Unit;
  }
  _x6693 = _x; /*(date : std/time/date/date) -> int*/
  kk_std_core_types__optional _x6695;
  bool _b_5391_5390;
  struct kk_std_time_instant_Timescale* _con6696 = kk_std_time_instant__as_Timescale(ts);
  kk_string_t _pat00 = _con6696->name;
  kk_string_t _pat100 = _con6696->unit;
  kk_function_t _pat20 = _con6696->from_tai;
  kk_function_t _pat30 = _con6696->to_tai;
  kk_std_core_types__maybe _x0 = _con6696->mb_seconds_in_day;
  kk_std_core_types__maybe _pat40 = _con6696->mb_to_mjd2000;
  kk_std_core_types__maybe _pat50 = _con6696->mb_from_mjd2000;
  if (kk_likely(kk_std_time_instant__timescale_is_unique(ts))) {
    kk_std_core_types__maybe_drop(_pat50, _ctx);
    kk_Unit;
    kk_std_core_types__maybe_drop(_pat40, _ctx);
    kk_Unit;
    kk_function_drop(_pat30, _ctx);
    kk_Unit;
    kk_function_drop(_pat20, _ctx);
    kk_Unit;
    kk_string_drop(_pat100, _ctx);
    kk_Unit;
    kk_string_drop(_pat00, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_instant__timescale_free(ts);
  }
  else {
    kk_std_core_types__maybe_dup(_x0);
    kk_Unit;
    kk_std_time_instant__timescale_decref(ts, _ctx);
    kk_Unit;
  }
  if (kk_std_core_types__is_Nothing(_x0)) {
    _b_5391_5390 = false; /*bool*/
  }
  else {
    kk_std_core_types__maybe_drop(_x0, _ctx);
    _b_5391_5390 = true; /*bool*/
  }
  _x6695 = kk_std_core_types__new_Optional(kk_bool_box(_b_5391_5390), _ctx); /*optional<106>*/
  return kk_std_time_calendar_dc_timestamp(d, c, _x6693, _x6695, _ctx);
}
 
// Create a standard [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) calendar
// using UTC time (`ts-utc-create`) given a provide leap second table (`:leaps-table`).
// This is a proleptic Gregorian
// calendar except that it uses the year 0 for 1 BCE, -1 for 2 BCE etc.
//
// You can create an ISO 8601 UTC calendar using the latest IETF leap
// second data using [`cal-utc-load`](std_time_download.html#cal_utc_load).


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6702__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6702(kk_function_t _fself, kk_integer_t _x16701, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6702(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_create_fun6702, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6702(kk_function_t _fself, kk_integer_t _x16701, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16701, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6703__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_create_fun6703(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6703(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_create_fun6703, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_create_fun6703(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__tuple2_ _match_5605;
  kk_integer_t _x6704 = kk_integer_dup(days); /*int*/
  _match_5605 = kk_std_time_calendar_iso_estimate_year(_x6704, _ctx); /*(int, maybe<int>)*/
  kk_box_t _box_x5392 = _match_5605.fst;
  kk_box_t _box_x5393 = _match_5605.snd;
  kk_integer_t approx = kk_integer_unbox(_box_x5392);
  kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5393, NULL);
  kk_integer_dup(approx);
  kk_std_core_types__maybe_dup(safe);
  kk_std_core_types__tuple2__drop(_match_5605, _ctx);
  kk_Unit;
  kk_integer_t doy1;
  kk_integer_t _x6707 = kk_integer_dup(days); /*int*/
  kk_integer_t _x6708;
  kk_integer_t _x6709 = kk_integer_dup(approx); /*int*/
  _x6708 = kk_std_time_calendar_iso_days_before_year(_x6709, _ctx); /*int*/
  doy1 = kk_integer_sub(_x6707,_x6708,kk_context()); /*int*/
  bool _match_5606;
  kk_integer_t _borrowedArg_5610;
  if (kk_std_core_types__is_Nothing(safe)) {
    _borrowedArg_5610 = kk_integer_from_small(0); /*int*/
  }
  else {
    kk_box_t _box_x5394 = safe._cons.Just.value;
    kk_integer_t x = kk_integer_unbox(_box_x5394);
    kk_integer_dup(x);
    kk_std_core_types__maybe_drop(safe, _ctx);
    kk_Unit;
    _borrowedArg_5610 = x; /*int*/
  }
  bool _borrowApp_5611 = kk_integer_lte_borrow(doy1,_borrowedArg_5610,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5610, _ctx);
  _match_5606 = _borrowApp_5611; /*bool*/
  if (_match_5606) {
    kk_integer_drop(days, _ctx);
    kk_integer_t _b_5418_5395 = approx; /*int*/;
    kk_integer_t _b_5419_5396 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5418_5395), kk_integer_box(_b_5419_5396), _ctx);
  }
  kk_integer_t doy2;
  kk_integer_t _x6711;
  kk_integer_t year_4925;
  kk_integer_t _x6712 = kk_integer_dup(approx); /*int*/
  year_4925 = kk_integer_add(_x6712,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_integer_t y = kk_integer_sub(year_4925,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t leapdays;
  kk_integer_t _x6713;
  kk_integer_t _x6714;
  kk_integer_t _x6715 = kk_integer_dup(y); /*int*/
  _x6714 = kk_integer_div(_x6715,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t _x6716;
  kk_integer_t _x6717 = kk_integer_dup(y); /*int*/
  _x6716 = kk_integer_div(_x6717,(kk_integer_from_small(100)),kk_context()); /*int*/
  _x6713 = kk_integer_sub(_x6714,_x6716,kk_context()); /*int*/
  kk_integer_t _x6718;
  kk_integer_t _x6719 = kk_integer_dup(y); /*int*/
  _x6718 = kk_integer_div(_x6719,(kk_integer_from_small(400)),kk_context()); /*int*/
  leapdays = kk_integer_add(_x6713,_x6718,kk_context()); /*int*/
  kk_integer_t _x6720 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/
  _x6711 = kk_integer_add(_x6720,leapdays,kk_context()); /*int*/
  doy2 = kk_integer_sub(days,_x6711,kk_context()); /*int*/
  bool _match_5607;
  kk_integer_t _borrowedArg_5608 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_5609 = kk_integer_lt_borrow(doy2,_borrowedArg_5608,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5608, _ctx);
  _match_5607 = _borrowApp_5609; /*bool*/
  if (_match_5607) {
    kk_integer_drop(doy2, _ctx);
    kk_integer_t _b_5420_5397 = approx; /*int*/;
    kk_integer_t _b_5421_5398 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5420_5397), kk_integer_box(_b_5421_5398), _ctx);
  }
  kk_integer_drop(doy1, _ctx);
  kk_integer_t _b_5422_5399 = kk_integer_add(approx,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t _b_5423_5400 = doy2; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5422_5399), kk_integer_box(_b_5423_5400), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6723__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6723(kk_function_t _fself, kk_integer_t _x16721, kk_integer_t _x26722, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6723(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_create_fun6723, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6723(kk_function_t _fself, kk_integer_t _x16721, kk_integer_t _x26722, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16721, _x26722, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6726__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6726(kk_function_t _fself, kk_integer_t _x16724, kk_integer_t _x26725, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6726(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_create_fun6726, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_utc_create_fun6726(kk_function_t _fself, kk_integer_t _x16724, kk_integer_t _x26725, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16724, _x26725, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6739__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5410;
};
static kk_string_t kk_std_time_calendar_cal_utc_create_fun6739(kk_function_t _fself, kk_std_time_date__date _b_5413, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6739(kk_box_t _fun_unbox_x5410, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_utc_create_fun6739__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_cal_utc_create_fun6739__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_cal_utc_create_fun6739, kk_context());
  _self->_fun_unbox_x5410 = _fun_unbox_x5410;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_cal_utc_create_fun6739(kk_function_t _fself, kk_std_time_date__date _b_5413, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_utc_create_fun6739__t* _self = kk_function_as(struct kk_std_time_calendar_cal_utc_create_fun6739__t*, _fself);
  kk_box_t _fun_unbox_x5410 = _self->_fun_unbox_x5410; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5410);}, {}, _ctx)
  kk_box_t _x6740;
  kk_function_t _x6741 = kk_function_unbox(_fun_unbox_x5410); /*(5411) -> 5412*/
  _x6740 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6741, (_x6741, kk_std_time_date__date_box(_b_5413, _ctx), _ctx)); /*5412*/
  return kk_string_unbox(_x6740);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6742__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_cal_utc_create_fun6742(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6742(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_create_fun6742, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_cal_utc_create_fun6742(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_create_fun6744__t {
  struct kk_function_s _base;
  kk_function_t _b_5427_5414;
};
static kk_box_t kk_std_time_calendar_cal_utc_create_fun6744(kk_function_t _fself, kk_box_t _b_5417, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_create_fun6744(kk_function_t _b_5427_5414, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_utc_create_fun6744__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_cal_utc_create_fun6744__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_cal_utc_create_fun6744, kk_context());
  _self->_b_5427_5414 = _b_5427_5414;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_cal_utc_create_fun6744(kk_function_t _fself, kk_box_t _b_5417, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_utc_create_fun6744__t* _self = kk_function_as(struct kk_std_time_calendar_cal_utc_create_fun6744__t*, _fself);
  kk_function_t _b_5427_5414 = _self->_b_5427_5414; /* (std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5427_5414);}, {}, _ctx)
  kk_string_t _x6745;
  kk_std_time_date__date _x6746 = kk_std_time_date__date_unbox(_b_5417, _ctx); /*std/time/date/date*/
  _x6745 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5427_5414, (_b_5427_5414, _x6746, _ctx)); /*string*/
  return kk_string_box(_x6745);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_utc_create(kk_std_time_utc__leaps_table leaps, kk_context_t* _ctx) { /* (leaps : std/time/utc/leaps-table) -> calendar */ 
  kk_std_time_instant__timescale ts_utc = kk_std_time_utc_ts_utc_create(leaps, _ctx); /*std/time/instant/timescale*/;
  kk_std_time_instant__timescale_dropn(ts_utc, ((int32_t)10), _ctx);
  kk_string_t _x6697 = kk_string_empty(); /*string*/
  kk_string_t _x6699;
  kk_define_string_literal(, _s6700, 3, "ISO")
  _x6699 = kk_string_dup(_s6700); /*string*/
  kk_std_core_types__optional _x6727;
  kk_integer_t _b_5424_5402;
  kk_std_core_types__optional _match_5604 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5604)) {
    kk_box_t _box_x5401 = _match_5604._cons.Optional.value;
    kk_integer_t _epoch_shift_2726 = kk_integer_unbox(_box_x5401);
    kk_integer_dup(_epoch_shift_2726);
    kk_std_core_types__optional_drop(_match_5604, _ctx);
    kk_Unit;
    _b_5424_5402 = _epoch_shift_2726; /*int*/
    goto _match6728;
  }
  _b_5424_5402 = kk_integer_from_int(730119, _ctx); /*int*/
  _match6728: ;
  _x6727 = kk_std_core_types__new_Optional(kk_integer_box(_b_5424_5402), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6730;
  bool _b_5425_5404;
  kk_std_core_types__optional _match_5603 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5603)) {
    kk_box_t _box_x5403 = _match_5603._cons.Optional.value;
    bool _has_year_zero_2730 = kk_bool_unbox(_box_x5403);
    kk_std_core_types__optional_drop(_match_5603, _ctx);
    kk_Unit;
    _b_5425_5404 = _has_year_zero_2730; /*bool*/
    goto _match6731;
  }
  _b_5425_5404 = true; /*bool*/
  _match6731: ;
  _x6730 = kk_std_core_types__new_Optional(kk_bool_box(_b_5425_5404), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6733;
  kk_string_t _b_5426_5406;
  kk_std_core_types__optional _match_5602 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5602)) {
    kk_box_t _box_x5405 = _match_5602._cons.Optional.value;
    kk_string_t _month_prefix_2734 = kk_string_unbox(_box_x5405);
    kk_string_dup(_month_prefix_2734);
    kk_std_core_types__optional_drop(_match_5602, _ctx);
    kk_Unit;
    _b_5426_5406 = _month_prefix_2734; /*string*/
    goto _match6734;
  }
  _b_5426_5406 = kk_string_empty(); /*string*/
  _match6734: ;
  _x6733 = kk_std_core_types__new_Optional(kk_string_box(_b_5426_5406), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6737;
  kk_function_t _b_5427_5414;
  kk_std_core_types__optional _x6738 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/
  if (kk_std_core_types__is_Optional(_x6738)) {
    kk_box_t _fun_unbox_x5410 = _x6738._cons.Optional.value;
    _b_5427_5414 = kk_std_time_calendar_new_cal_utc_create_fun6739(_fun_unbox_x5410, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _b_5427_5414 = kk_std_time_calendar_new_cal_utc_create_fun6742(_ctx); /*(std/time/date/date) -> string*/
  }
  _x6737 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_utc_create_fun6744(_b_5427_5414, _ctx)), _ctx); /*optional<106>*/
  return kk_std_time_calendar_solar_calendar(_x6697, _x6699, kk_std_time_calendar_new_cal_utc_create_fun6702(_ctx), kk_std_time_calendar_new_cal_utc_create_fun6703(_ctx), kk_std_time_calendar_new_cal_utc_create_fun6723(_ctx), kk_std_time_calendar_new_cal_utc_create_fun6726(_ctx), _x6727, _x6730, _x6733, _x6737, _ctx);
}
 
// Create a new ISO 8601 calendar based on UTC-SLS time, i.e. UTC with smoothed leap
// seconds.


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6752__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6752(kk_function_t _fself, kk_integer_t _x16751, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6752(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_sls_create_fun6752, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6752(kk_function_t _fself, kk_integer_t _x16751, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_year(_x16751, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6753__t {
  struct kk_function_s _base;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_sls_create_fun6753(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6753(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_sls_create_fun6753, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_cal_utc_sls_create_fun6753(kk_function_t _fself, kk_integer_t days, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_core_types__tuple2_ _match_5594;
  kk_integer_t _x6754 = kk_integer_dup(days); /*int*/
  _match_5594 = kk_std_time_calendar_iso_estimate_year(_x6754, _ctx); /*(int, maybe<int>)*/
  kk_box_t _box_x5428 = _match_5594.fst;
  kk_box_t _box_x5429 = _match_5594.snd;
  kk_integer_t approx = kk_integer_unbox(_box_x5428);
  kk_std_core_types__maybe safe = kk_std_core_types__maybe_unbox(_box_x5429, NULL);
  kk_integer_dup(approx);
  kk_std_core_types__maybe_dup(safe);
  kk_std_core_types__tuple2__drop(_match_5594, _ctx);
  kk_Unit;
  kk_integer_t doy1;
  kk_integer_t _x6757 = kk_integer_dup(days); /*int*/
  kk_integer_t _x6758;
  kk_integer_t _x6759 = kk_integer_dup(approx); /*int*/
  _x6758 = kk_std_time_calendar_iso_days_before_year(_x6759, _ctx); /*int*/
  doy1 = kk_integer_sub(_x6757,_x6758,kk_context()); /*int*/
  bool _match_5595;
  kk_integer_t _borrowedArg_5599;
  if (kk_std_core_types__is_Nothing(safe)) {
    _borrowedArg_5599 = kk_integer_from_small(0); /*int*/
  }
  else {
    kk_box_t _box_x5430 = safe._cons.Just.value;
    kk_integer_t x = kk_integer_unbox(_box_x5430);
    kk_integer_dup(x);
    kk_std_core_types__maybe_drop(safe, _ctx);
    kk_Unit;
    _borrowedArg_5599 = x; /*int*/
  }
  bool _borrowApp_5600 = kk_integer_lte_borrow(doy1,_borrowedArg_5599,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5599, _ctx);
  _match_5595 = _borrowApp_5600; /*bool*/
  if (_match_5595) {
    kk_integer_drop(days, _ctx);
    kk_integer_t _b_5454_5431 = approx; /*int*/;
    kk_integer_t _b_5455_5432 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5454_5431), kk_integer_box(_b_5455_5432), _ctx);
  }
  kk_integer_t doy2;
  kk_integer_t _x6761;
  kk_integer_t year_4926;
  kk_integer_t _x6762 = kk_integer_dup(approx); /*int*/
  year_4926 = kk_integer_add(_x6762,(kk_integer_from_small(1)),kk_context()); /*int*/
  kk_integer_t y = kk_integer_sub(year_4926,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t leapdays;
  kk_integer_t _x6763;
  kk_integer_t _x6764;
  kk_integer_t _x6765 = kk_integer_dup(y); /*int*/
  _x6764 = kk_integer_div(_x6765,(kk_integer_from_small(4)),kk_context()); /*int*/
  kk_integer_t _x6766;
  kk_integer_t _x6767 = kk_integer_dup(y); /*int*/
  _x6766 = kk_integer_div(_x6767,(kk_integer_from_small(100)),kk_context()); /*int*/
  _x6763 = kk_integer_sub(_x6764,_x6766,kk_context()); /*int*/
  kk_integer_t _x6768;
  kk_integer_t _x6769 = kk_integer_dup(y); /*int*/
  _x6768 = kk_integer_div(_x6769,(kk_integer_from_small(400)),kk_context()); /*int*/
  leapdays = kk_integer_add(_x6763,_x6768,kk_context()); /*int*/
  kk_integer_t _x6770 = kk_integer_mul((kk_integer_from_small(365)),y,kk_context()); /*int*/
  _x6761 = kk_integer_add(_x6770,leapdays,kk_context()); /*int*/
  doy2 = kk_integer_sub(days,_x6761,kk_context()); /*int*/
  bool _match_5596;
  kk_integer_t _borrowedArg_5597 = kk_integer_from_small(0); /*int*/;
  bool _borrowApp_5598 = kk_integer_lt_borrow(doy2,_borrowedArg_5597,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5597, _ctx);
  _match_5596 = _borrowApp_5598; /*bool*/
  if (_match_5596) {
    kk_integer_drop(doy2, _ctx);
    kk_integer_t _b_5456_5433 = approx; /*int*/;
    kk_integer_t _b_5457_5434 = doy1; /*int*/;
    return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5456_5433), kk_integer_box(_b_5457_5434), _ctx);
  }
  kk_integer_drop(doy1, _ctx);
  kk_integer_t _b_5458_5435 = kk_integer_add(approx,(kk_integer_from_small(1)),kk_context()); /*int*/;
  kk_integer_t _b_5459_5436 = doy2; /*int*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_integer_box(_b_5458_5435), kk_integer_box(_b_5459_5436), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6773__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6773(kk_function_t _fself, kk_integer_t _x16771, kk_integer_t _x26772, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6773(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_sls_create_fun6773, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6773(kk_function_t _fself, kk_integer_t _x16771, kk_integer_t _x26772, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_days_before_month(_x16771, _x26772, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6776__t {
  struct kk_function_s _base;
};
static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6776(kk_function_t _fself, kk_integer_t _x16774, kk_integer_t _x26775, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6776(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_sls_create_fun6776, _ctx)
  return kk_function_dup(_fself);
}

static kk_integer_t kk_std_time_calendar_cal_utc_sls_create_fun6776(kk_function_t _fself, kk_integer_t _x16774, kk_integer_t _x26775, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  return kk_std_time_calendar_iso_doy_to_month(_x16774, _x26775, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6789__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5446;
};
static kk_string_t kk_std_time_calendar_cal_utc_sls_create_fun6789(kk_function_t _fself, kk_std_time_date__date _b_5449, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6789(kk_box_t _fun_unbox_x5446, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_utc_sls_create_fun6789__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_cal_utc_sls_create_fun6789__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_cal_utc_sls_create_fun6789, kk_context());
  _self->_fun_unbox_x5446 = _fun_unbox_x5446;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_cal_utc_sls_create_fun6789(kk_function_t _fself, kk_std_time_date__date _b_5449, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_utc_sls_create_fun6789__t* _self = kk_function_as(struct kk_std_time_calendar_cal_utc_sls_create_fun6789__t*, _fself);
  kk_box_t _fun_unbox_x5446 = _self->_fun_unbox_x5446; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5446);}, {}, _ctx)
  kk_box_t _x6790;
  kk_function_t _x6791 = kk_function_unbox(_fun_unbox_x5446); /*(5447) -> 5448*/
  _x6790 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6791, (_x6791, kk_std_time_date__date_box(_b_5449, _ctx), _ctx)); /*5448*/
  return kk_string_unbox(_x6790);
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6792__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_cal_utc_sls_create_fun6792(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6792(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_cal_utc_sls_create_fun6792, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_cal_utc_sls_create_fun6792(kk_function_t _fself, kk_std_time_date__date d, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_cal_utc_sls_create_fun6794__t {
  struct kk_function_s _base;
  kk_function_t _b_5463_5450;
};
static kk_box_t kk_std_time_calendar_cal_utc_sls_create_fun6794(kk_function_t _fself, kk_box_t _b_5453, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_cal_utc_sls_create_fun6794(kk_function_t _b_5463_5450, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_utc_sls_create_fun6794__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_cal_utc_sls_create_fun6794__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_cal_utc_sls_create_fun6794, kk_context());
  _self->_b_5463_5450 = _b_5463_5450;
  return &_self->_base;
}

static kk_box_t kk_std_time_calendar_cal_utc_sls_create_fun6794(kk_function_t _fself, kk_box_t _b_5453, kk_context_t* _ctx) {
  struct kk_std_time_calendar_cal_utc_sls_create_fun6794__t* _self = kk_function_as(struct kk_std_time_calendar_cal_utc_sls_create_fun6794__t*, _fself);
  kk_function_t _b_5463_5450 = _self->_b_5463_5450; /* (std/time/date/date) -> string */
  kk_drop_match(_self, {kk_function_dup(_b_5463_5450);}, {}, _ctx)
  kk_string_t _x6795;
  kk_std_time_date__date _x6796 = kk_std_time_date__date_unbox(_b_5453, _ctx); /*std/time/date/date*/
  _x6795 = kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _b_5463_5450, (_b_5463_5450, _x6796, _ctx)); /*string*/
  return kk_string_box(_x6795);
}

kk_std_time_calendar__calendar kk_std_time_calendar_cal_utc_sls_create(kk_std_time_utc__leaps_table leaps, kk_context_t* _ctx) { /* (leaps : std/time/utc/leaps-table) -> calendar */ 
  kk_std_time_instant__timescale ts_utc_sls = kk_std_time_utc_ts_utc_sls_create(leaps, _ctx); /*std/time/instant/timescale*/;
  kk_std_time_instant__timescale_dropn(ts_utc_sls, ((int32_t)10), _ctx);
  kk_string_t _x6747;
  kk_define_string_literal(, _s6748, 3, "SLS")
  _x6747 = kk_string_dup(_s6748); /*string*/
  kk_string_t _x6749;
  kk_define_string_literal(, _s6750, 7, "ISO-SLS")
  _x6749 = kk_string_dup(_s6750); /*string*/
  kk_std_core_types__optional _x6777;
  kk_integer_t _b_5460_5438;
  kk_std_core_types__optional _match_5593 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5593)) {
    kk_box_t _box_x5437 = _match_5593._cons.Optional.value;
    kk_integer_t _epoch_shift_2726 = kk_integer_unbox(_box_x5437);
    kk_integer_dup(_epoch_shift_2726);
    kk_std_core_types__optional_drop(_match_5593, _ctx);
    kk_Unit;
    _b_5460_5438 = _epoch_shift_2726; /*int*/
    goto _match6778;
  }
  _b_5460_5438 = kk_integer_from_int(730119, _ctx); /*int*/
  _match6778: ;
  _x6777 = kk_std_core_types__new_Optional(kk_integer_box(_b_5460_5438), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6780;
  bool _b_5461_5440;
  kk_std_core_types__optional _match_5592 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5592)) {
    kk_box_t _box_x5439 = _match_5592._cons.Optional.value;
    bool _has_year_zero_2730 = kk_bool_unbox(_box_x5439);
    kk_std_core_types__optional_drop(_match_5592, _ctx);
    kk_Unit;
    _b_5461_5440 = _has_year_zero_2730; /*bool*/
    goto _match6781;
  }
  _b_5461_5440 = true; /*bool*/
  _match6781: ;
  _x6780 = kk_std_core_types__new_Optional(kk_bool_box(_b_5461_5440), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6783;
  kk_string_t _b_5462_5442;
  kk_std_core_types__optional _match_5591 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5591)) {
    kk_box_t _box_x5441 = _match_5591._cons.Optional.value;
    kk_string_t _month_prefix_2734 = kk_string_unbox(_box_x5441);
    kk_string_dup(_month_prefix_2734);
    kk_std_core_types__optional_drop(_match_5591, _ctx);
    kk_Unit;
    _b_5462_5442 = _month_prefix_2734; /*string*/
    goto _match6784;
  }
  _b_5462_5442 = kk_string_empty(); /*string*/
  _match6784: ;
  _x6783 = kk_std_core_types__new_Optional(kk_string_box(_b_5462_5442), _ctx); /*optional<106>*/
  kk_std_core_types__optional _x6787;
  kk_function_t _b_5463_5450;
  kk_std_core_types__optional _x6788 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/
  if (kk_std_core_types__is_Optional(_x6788)) {
    kk_box_t _fun_unbox_x5446 = _x6788._cons.Optional.value;
    _b_5463_5450 = kk_std_time_calendar_new_cal_utc_sls_create_fun6789(_fun_unbox_x5446, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _b_5463_5450 = kk_std_time_calendar_new_cal_utc_sls_create_fun6792(_ctx); /*(std/time/date/date) -> string*/
  }
  _x6787 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_utc_sls_create_fun6794(_b_5463_5450, _ctx)), _ctx); /*optional<106>*/
  return kk_std_time_calendar_solar_calendar(_x6747, _x6749, kk_std_time_calendar_new_cal_utc_sls_create_fun6752(_ctx), kk_std_time_calendar_new_cal_utc_sls_create_fun6753(_ctx), kk_std_time_calendar_new_cal_utc_sls_create_fun6773(_ctx), kk_std_time_calendar_new_cal_utc_sls_create_fun6776(_ctx), _x6777, _x6780, _x6783, _x6787, _ctx);
}
 
// Create a new calendar from by combining two other calendars. The `switch-date`
// is in terms of the second calendar (`cal2`) and time after (and including) the switch
// date is displayed in `cal2` while times before it in `cal1`. This function is used
// for example for the Julian Gregorian calendar.


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6797__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_time_date__date switch_date;
};
static kk_std_time_date__date kk_std_time_calendar_combine_earth_calendars_fun6797(kk_function_t _fself, kk_integer_t days1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6797(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6797__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6797__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6797, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->switch_date = switch_date;
  return &_self->_base;
}

static kk_std_time_date__date kk_std_time_calendar_combine_earth_calendars_fun6797(kk_function_t _fself, kk_integer_t days1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6797__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6797__t*, _fself);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1);kk_std_time_calendar__calendar_dup(cal2);kk_std_time_date__date_dup(switch_date);}, {}, _ctx)
  kk_function_t _x6798;
  kk_std_time_calendar__calendar calendar0_4793;
  bool _match_5587;
  kk_integer_t _borrowedArg_5588;
  kk_function_t _x6799;
  struct kk_std_time_calendar_Calendar* _con6800 = kk_std_time_calendar__as_Calendar(cal2);
  kk_function_t _x = _con6800->date_to_days;
  kk_function_dup(_x);
  _x6799 = _x; /*(date : std/time/date/date) -> int*/
  _borrowedArg_5588 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6799, (_x6799, switch_date, _ctx)); /*int*/
  bool _borrowApp_5589 = kk_integer_lt_borrow(days1,_borrowedArg_5588,kk_context()); /*bool*/;
  kk_integer_drop(_borrowedArg_5588, _ctx);
  _match_5587 = _borrowApp_5589; /*bool*/
  if (_match_5587) {
    kk_std_time_calendar__calendar_dropn(cal2, ((int32_t)8), _ctx);
    calendar0_4793 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_time_calendar__calendar_dropn(cal1, ((int32_t)8), _ctx);
    calendar0_4793 = cal2; /*std/time/calendar/calendar*/
  }
  struct kk_std_time_calendar_Calendar* _con6801 = kk_std_time_calendar__as_Calendar(calendar0_4793);
  kk_string_t _pat07 = _con6801->name;
  kk_string_t _pat110 = _con6801->long_name;
  kk_string_t _pat28 = _con6801->month_prefix;
  kk_function_t _pat33 = _con6801->show_era;
  kk_function_t _pat42 = _con6801->instant_to_dc;
  kk_function_t _pat52 = _con6801->dc_to_instant;
  kk_function_t _x5 = _con6801->days_to_date;
  kk_function_t _pat61 = _con6801->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(calendar0_4793))) {
    kk_function_drop(_pat61, _ctx);
    kk_Unit;
    kk_function_drop(_pat52, _ctx);
    kk_Unit;
    kk_function_drop(_pat42, _ctx);
    kk_Unit;
    kk_function_drop(_pat33, _ctx);
    kk_Unit;
    kk_string_drop(_pat28, _ctx);
    kk_Unit;
    kk_string_drop(_pat110, _ctx);
    kk_Unit;
    kk_string_drop(_pat07, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(calendar0_4793);
  }
  else {
    kk_function_dup(_x5);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(calendar0_4793, _ctx);
    kk_Unit;
  }
  _x6798 = _x5; /*(days : int) -> std/time/date/date*/
  return kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), _x6798, (_x6798, days1, _ctx));
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6807__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_core_types__optional mb_show_era;
  kk_std_time_date__date switch_date;
};
static kk_string_t kk_std_time_calendar_combine_earth_calendars_fun6807(kk_function_t _fself, kk_std_time_date__date d01, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6807(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_core_types__optional mb_show_era, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6807__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6807__t, 8, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6807, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->mb_show_era = mb_show_era;
  _self->switch_date = switch_date;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_combine_earth_calendars_fun6807(kk_function_t _fself, kk_std_time_date__date d01, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6807__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6807__t*, _fself);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_core_types__optional mb_show_era = _self->mb_show_era; /* optional<maybe<(std/time/date/date) -> string>> */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1);kk_std_time_calendar__calendar_dup(cal2);kk_std_core_types__optional_dup(mb_show_era);kk_std_time_date__date_dup(switch_date);}, {}, _ctx)
  kk_std_core_types__maybe _match_5584;
  if (kk_std_core_types__is_Optional(mb_show_era)) {
    kk_box_t _box_x5465 = mb_show_era._cons.Optional.value;
    kk_std_core_types__maybe _mb_show_era_4136 = kk_std_core_types__maybe_unbox(_box_x5465, NULL);
    kk_std_core_types__maybe_dup(_mb_show_era_4136);
    kk_std_core_types__optional_drop(mb_show_era, _ctx);
    kk_Unit;
    _match_5584 = _mb_show_era_4136; /*maybe<(std/time/date/date) -> string>*/
    goto _match6808;
  }
  _match_5584 = kk_std_core_types__new_Nothing(_ctx); /*maybe<(std/time/date/date) -> string>*/
  _match6808: ;
  if (kk_std_core_types__is_Just(_match_5584)) {
    kk_box_t _fun_unbox_x5469 = _match_5584._cons.Just.value;
    kk_std_time_calendar__calendar_dropn(cal1, ((int32_t)8), _ctx);
    kk_std_time_calendar__calendar_dropn(cal2, ((int32_t)8), _ctx);
    kk_std_time_date__date_drop(switch_date, _ctx);
    kk_std_time_date__date _b_5472 = d01; /*std/time/date/date*/;
    kk_box_t _x6810;
    kk_function_t _x6811 = kk_function_unbox(_fun_unbox_x5469); /*(5470) -> 5471*/
    _x6810 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6811, (_x6811, kk_std_time_date__date_box(_b_5472, _ctx), _ctx)); /*5471*/
    return kk_string_unbox(_x6810);
  }
  kk_function_t _x6812;
  kk_std_time_calendar__calendar calendar2_4796;
  bool _match_5585;
  kk_std_time_date__date _x6813 = kk_std_time_date__date_dup(d01); /*std/time/date/date*/
  _match_5585 = kk_std_time_date__lp__lt__rp_(_x6813, switch_date, _ctx); /*bool*/
  if (_match_5585) {
    kk_std_time_calendar__calendar_dropn(cal2, ((int32_t)8), _ctx);
    calendar2_4796 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_time_calendar__calendar_dropn(cal1, ((int32_t)8), _ctx);
    calendar2_4796 = cal2; /*std/time/calendar/calendar*/
  }
  struct kk_std_time_calendar_Calendar* _con6814 = kk_std_time_calendar__as_Calendar(calendar2_4796);
  kk_string_t _pat010 = _con6814->name;
  kk_string_t _pat113 = _con6814->long_name;
  kk_string_t _pat211 = _con6814->month_prefix;
  kk_function_t _x7 = _con6814->show_era;
  kk_function_t _pat39 = _con6814->instant_to_dc;
  kk_function_t _pat45 = _con6814->dc_to_instant;
  kk_function_t _pat55 = _con6814->days_to_date;
  kk_function_t _pat64 = _con6814->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(calendar2_4796))) {
    kk_function_drop(_pat64, _ctx);
    kk_Unit;
    kk_function_drop(_pat55, _ctx);
    kk_Unit;
    kk_function_drop(_pat45, _ctx);
    kk_Unit;
    kk_function_drop(_pat39, _ctx);
    kk_Unit;
    kk_string_drop(_pat211, _ctx);
    kk_Unit;
    kk_string_drop(_pat113, _ctx);
    kk_Unit;
    kk_string_drop(_pat010, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(calendar2_4796);
  }
  else {
    kk_function_dup(_x7);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(calendar2_4796, _ctx);
    kk_Unit;
  }
  _x6812 = _x7; /*(std/time/date/date) -> string*/
  return kk_function_call(kk_string_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6812, (_x6812, d01, _ctx));
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6816__t {
  struct kk_function_s _base;
  kk_function_t days_to_date0;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_combine_earth_calendars_fun6816(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6816(kk_function_t days_to_date0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6816__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6816__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6816, kk_context());
  _self->days_to_date0 = days_to_date0;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_combine_earth_calendars_fun6816(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6816__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6816__t*, _fself);
  kk_function_t days_to_date0 = _self->days_to_date0; /* (days : int) -> std/time/date/date */
  kk_drop_match(_self, {kk_function_dup(days_to_date0);}, {}, _ctx)
  kk_std_time_timestamp__timestamp ts_4789;
  kk_std_num_ddouble__ddouble _x6817;
  kk_std_num_ddouble__ddouble _x6818;
  struct kk_std_time_instant_Instant* _con6819 = kk_std_time_instant__as_Instant(i);
  kk_std_time_timestamp__timestamp _x2 = _con6819->since;
  kk_std_time_instant__timescale _pat24 = _con6819->ts;
  struct kk_std_time_instant_Timescale* _con6820 = kk_std_time_instant__as_Timescale(_pat24);
  kk_std_num_ddouble__ddouble _x1 = _x2.since;
  _x6818 = _x1; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6821;
  kk_std_num_ddouble__ddouble _x3 = tzdelta.secs;
  _x6821 = _x3; /*std/time/timestamp/timespan*/
  _x6817 = kk_std_num_ddouble__lp__plus__rp_(_x6818, _x6821, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6822;
  struct kk_std_time_instant_Instant* _con6823 = kk_std_time_instant__as_Instant(i);
  kk_std_time_timestamp__timestamp _x20 = _con6823->since;
  kk_std_time_instant__timescale _pat240 = _con6823->ts;
  struct kk_std_time_instant_Timescale* _con6824 = kk_std_time_instant__as_Timescale(_pat240);
  kk_string_t _pat310 = _con6824->name;
  kk_string_t _pat400 = _con6824->unit;
  kk_function_t _pat500 = _con6824->from_tai;
  kk_function_t _pat600 = _con6824->to_tai;
  kk_std_core_types__maybe _pat700 = _con6824->mb_seconds_in_day;
  kk_std_core_types__maybe _pat800 = _con6824->mb_to_mjd2000;
  kk_std_core_types__maybe _pat900 = _con6824->mb_from_mjd2000;
  if (kk_likely(kk_std_time_instant__instant_is_unique(i))) {
    if (kk_likely(kk_std_time_instant__timescale_is_unique(_pat240))) {
      kk_string_drop(_pat310, _ctx);
      kk_string_drop(_pat400, _ctx);
      kk_function_drop(_pat500, _ctx);
      kk_function_drop(_pat600, _ctx);
      kk_std_core_types__maybe_drop(_pat700, _ctx);
      kk_std_core_types__maybe_drop(_pat800, _ctx);
      kk_std_core_types__maybe_drop(_pat900, _ctx);
      kk_Unit;
      kk_std_time_instant__timescale_free(_pat240);
    }
    else {
      kk_std_time_instant__timescale_decref(_pat240, _ctx);
      kk_Unit;
    }
    kk_Unit;
    kk_std_time_instant__instant_free(i);
  }
  else {
    kk_std_time_instant__instant_decref(i, _ctx);
    kk_Unit;
  }
  kk_std_num_ddouble__ddouble _pat001 = _x20.since;
  int32_t _x00 = _x20.leap32;
  _x6822 = _x00; /*int32*/
  ts_4789 = kk_std_time_timestamp__new_Timestamp(_x6817, _x6822, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_core_types__tuple2_ _match_5582 = kk_std_time_timestamp_days_seconds(ts_4789, _ctx); /*(int, std/num/ddouble/ddouble)*/;
  kk_box_t _box_x5473 = _match_5582.fst;
  kk_box_t _box_x5474 = _match_5582.snd;
  kk_integer_t days0 = kk_integer_unbox(_box_x5473);
  kk_std_num_ddouble__ddouble secs = kk_std_num_ddouble__ddouble_unbox(_box_x5474, NULL);
  kk_integer_dup(days0);
  kk_std_core_types__tuple2__drop(_match_5582, _ctx);
  kk_Unit;
  kk_std_time_date__clock _match_5583;
  kk_integer_t _x6827;
  int32_t _x6828;
  kk_std_num_ddouble__ddouble _pat000 = ts_4789.since;
  int32_t _x0 = ts_4789.leap32;
  _x6828 = _x0; /*int32*/
  _x6827 = kk_integer_from_int(_x6828,kk_context()); /*int*/
  _match_5583 = kk_std_time_date_clock_3(secs, _x6827, _ctx); /*std/time/date/clock*/
  struct kk_std_time_date_Clock* _con6829 = kk_std_time_date__as_Clock(_match_5583);
  kk_std_time_date__date _b_5492_5475 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0, (days_to_date0, days0, _ctx)); /*std/time/date/date*/;
  kk_std_time_date__clock _b_5493_5476 = _match_5583; /*std/time/date/clock*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5492_5475, _ctx), kk_std_time_date__clock_box(_b_5493_5476, _ctx), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6831__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_time_date__date switch_date;
};
static kk_std_time_instant__instant kk_std_time_calendar_combine_earth_calendars_fun6831(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6831(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6831__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6831__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6831, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->switch_date = switch_date;
  return &_self->_base;
}



// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6832__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_time_date__date switch_date;
};
static kk_integer_t kk_std_time_calendar_combine_earth_calendars_fun6832(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6832(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6832__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6832__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6832, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->switch_date = switch_date;
  return &_self->_base;
}

static kk_integer_t kk_std_time_calendar_combine_earth_calendars_fun6832(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6832__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6832__t*, _fself);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1);kk_std_time_calendar__calendar_dup(cal2);kk_std_time_date__date_dup(switch_date);}, {}, _ctx)
  kk_function_t _x6833;
  kk_std_time_calendar__calendar calendar1_4795;
  bool _match_5581;
  kk_std_time_date__date _x6834 = kk_std_time_date__date_dup(d1); /*std/time/date/date*/
  _match_5581 = kk_std_time_date__lp__lt__rp_(_x6834, switch_date, _ctx); /*bool*/
  if (_match_5581) {
    kk_std_time_calendar__calendar_dropn(cal2, ((int32_t)8), _ctx);
    calendar1_4795 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_time_calendar__calendar_dropn(cal1, ((int32_t)8), _ctx);
    calendar1_4795 = cal2; /*std/time/calendar/calendar*/
  }
  struct kk_std_time_calendar_Calendar* _con6835 = kk_std_time_calendar__as_Calendar(calendar1_4795);
  kk_string_t _pat08 = _con6835->name;
  kk_string_t _pat112 = _con6835->long_name;
  kk_string_t _pat210 = _con6835->month_prefix;
  kk_function_t _pat35 = _con6835->show_era;
  kk_function_t _pat43 = _con6835->instant_to_dc;
  kk_function_t _pat53 = _con6835->dc_to_instant;
  kk_function_t _pat62 = _con6835->days_to_date;
  kk_function_t _x6 = _con6835->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(calendar1_4795))) {
    kk_function_drop(_pat62, _ctx);
    kk_Unit;
    kk_function_drop(_pat53, _ctx);
    kk_Unit;
    kk_function_drop(_pat43, _ctx);
    kk_Unit;
    kk_function_drop(_pat35, _ctx);
    kk_Unit;
    kk_string_drop(_pat210, _ctx);
    kk_Unit;
    kk_string_drop(_pat112, _ctx);
    kk_Unit;
    kk_string_drop(_pat08, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(calendar1_4795);
  }
  else {
    kk_function_dup(_x6);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(calendar1_4795, _ctx);
    kk_Unit;
  }
  _x6833 = _x6; /*(date : std/time/date/date) -> int*/
  return kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6833, (_x6833, d1, _ctx));
}
static kk_std_time_instant__instant kk_std_time_calendar_combine_earth_calendars_fun6831(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6831__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6831__t*, _fself);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1);kk_std_time_calendar__calendar_dup(cal2);kk_std_time_date__date_dup(switch_date);}, {}, _ctx)
  kk_std_time_timestamp__timestamp t_4878;
  kk_std_core_types__optional _x6836;
  kk_box_t _x6837;
  bool _x6838;
  struct kk_std_time_instant_Timescale* _con6839 = kk_std_time_instant__as_Timescale(ts1);
  kk_std_core_types__maybe _x41 = _con6839->mb_seconds_in_day;
  kk_std_core_types__maybe_dup(_x41);
  if (kk_std_core_types__is_Nothing(_x41)) {
    _x6838 = false; /*bool*/
  }
  else {
    kk_std_core_types__maybe_drop(_x41, _ctx);
    _x6838 = true; /*bool*/
  }
  _x6837 = kk_bool_box(_x6838); /*106*/
  _x6836 = kk_std_core_types__new_Optional(_x6837, _ctx); /*optional<106>*/
  t_4878 = kk_std_time_calendar_dc_timestamp(d00, c, kk_std_time_calendar_new_combine_earth_calendars_fun6832(cal1, cal2, switch_date, _ctx), _x6836, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant i0;
  kk_std_time_instant__timescale _x6840 = kk_std_time_instant__timescale_dup(ts1); /*std/time/instant/timescale*/
  i0 = kk_std_time_instant__new_Instant(kk_reuse_null, t_4878, _x6840, _ctx); /*std/time/instant/instant*/
  bool _match_5578;
  kk_string_t _x6841;
  kk_string_t _x4 = tz.name;
  kk_string_dup(_x4);
  _x6841 = _x4; /*string*/
  kk_string_t _x6842;
  kk_std_time_calendar__timezone _x6843 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  kk_string_t _x01 = _x6843.name;
  _x6842 = kk_string_dup(_x01); /*string*/
  _match_5578 = kk_string_is_eq(_x6841,_x6842,kk_context()); /*bool*/
  if (_match_5578) {
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    return i0;
  }
  kk_std_core_types__maybe _match_5579;
  kk_function_t _x6845;
  kk_function_t _x10 = tz.utc_inverse;
  kk_function_dup(_x10);
  _x6845 = _x10; /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  kk_std_time_instant__instant _x6844 = kk_std_time_instant__instant_dup(i0); /*std/time/instant/instant*/
  _match_5579 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6845, (_x6845, _x6844, _ctx)); /*maybe<std/time/instant/instant>*/
  if (kk_std_core_types__is_Just(_match_5579)) {
    kk_box_t _box_x5479 = _match_5579._cons.Just.value;
    kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5479, NULL);
    struct kk_std_time_instant_Instant* _con6847 = kk_std_time_instant__as_Instant(inv);
    kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    kk_std_time_instant__instant_dup(inv);
    kk_std_core_types__maybe_drop(_match_5579, _ctx);
    kk_Unit;
    return inv;
  }
  kk_std_core_types__tuple2_ _this_4698;
  kk_function_t _x6848;
  kk_function_t _x30 = tz.utc_delta;
  kk_function_dup(_x30);
  _x6848 = _x30; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  _this_4698 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6848, (_x6848, i0, _ctx)); /*(std/time/duration/duration, string)*/
  kk_std_time_timestamp__timestamp t1_4701;
  kk_std_num_ddouble__ddouble _x6849;
  kk_std_num_ddouble__ddouble _x6850;
  kk_std_num_ddouble__ddouble _x100 = t_4878.since;
  _x6850 = _x100; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6851;
  double _x6852;
  double _x6853;
  kk_box_t _box_x5480 = _this_4698.fst;
  kk_box_t _box_x5481 = _this_4698.snd;
  kk_std_time_duration__duration _x200 = kk_std_time_duration__duration_unbox(_box_x5480, NULL);
  kk_std_num_ddouble__ddouble _x50 = _x200.secs;
  double _x40 = _x50.hi;
  _x6853 = _x40; /*double*/
  _x6852 = (-_x6853); /*double*/
  double _x6856;
  double _x6857;
  kk_box_t _box_x5482 = _this_4698.fst;
  kk_box_t _box_x5483 = _this_4698.snd;
  kk_std_time_duration__duration _x201 = kk_std_time_duration__duration_unbox(_box_x5482, NULL);
  kk_std_num_ddouble__ddouble _x500 = _x201.secs;
  double _x000 = _x500.lo;
  _x6857 = _x000; /*double*/
  _x6856 = (-_x6857); /*double*/
  _x6851 = kk_std_num_ddouble__new_Ddouble(_x6852, _x6856, _ctx); /*std/num/ddouble/ddouble*/
  _x6849 = kk_std_num_ddouble__lp__plus__rp_(_x6850, _x6851, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6860;
  kk_std_num_ddouble__ddouble _pat0200 = t_4878.since;
  int32_t _x2000 = t_4878.leap32;
  _x6860 = _x2000; /*int32*/
  t1_4701 = kk_std_time_timestamp__new_Timestamp(_x6849, _x6860, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant tzi1;
  kk_std_time_instant__timescale _x6861 = kk_std_time_instant__timescale_dup(ts1); /*std/time/instant/timescale*/
  tzi1 = kk_std_time_instant__new_Instant(kk_reuse_null, t1_4701, _x6861, _ctx); /*std/time/instant/instant*/
  kk_std_core_types__tuple2_ _this0_4705;
  kk_function_t _x6863;
  kk_function_t _x70 = tz.utc_delta;
  kk_function_dup(_x70);
  kk_std_time_calendar__timezone_drop(tz, _ctx);
  kk_Unit;
  _x6863 = _x70; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  kk_std_time_instant__instant _x6862 = kk_std_time_instant__instant_dup(tzi1); /*std/time/instant/instant*/
  _this0_4705 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6863, (_x6863, _x6862, _ctx)); /*(std/time/duration/duration, string)*/
  bool _match_5580;
  kk_std_time_duration__duration _x6864;
  kk_box_t _box_x5484 = _this_4698.fst;
  kk_box_t _box_x5485 = _this_4698.snd;
  kk_std_time_duration__duration _x202 = kk_std_time_duration__duration_unbox(_box_x5484, NULL);
  kk_std_core_types__tuple2__drop(_this_4698, _ctx);
  kk_Unit;
  _x6864 = _x202; /*std/time/duration/duration*/
  kk_std_time_duration__duration _x6867;
  kk_box_t _box_x5486 = _this0_4705.fst;
  kk_box_t _box_x5487 = _this0_4705.snd;
  kk_std_time_duration__duration _x60 = kk_std_time_duration__duration_unbox(_box_x5486, NULL);
  _x6867 = _x60; /*std/time/duration/duration*/
  _match_5580 = kk_std_time_duration__lp__eq__eq__rp_(_x6864, _x6867, _ctx); /*bool*/
  if (_match_5580) {
    kk_std_core_types__tuple2__drop(_this0_4705, _ctx);
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    return tzi1;
  }
  kk_std_time_instant__instant_dropn(tzi1, ((int32_t)1), _ctx);
  kk_std_time_timestamp__timestamp t3_4708;
  kk_std_num_ddouble__ddouble _x6870;
  kk_std_num_ddouble__ddouble _x6871;
  kk_std_num_ddouble__ddouble _x11 = t_4878.since;
  _x6871 = _x11; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6872;
  double _x6873;
  double _x6874;
  kk_box_t _box_x5488 = _this0_4705.fst;
  kk_box_t _box_x5489 = _this0_4705.snd;
  kk_std_time_duration__duration _x600 = kk_std_time_duration__duration_unbox(_box_x5488, NULL);
  kk_std_num_ddouble__ddouble _x9 = _x600.secs;
  double _x8 = _x9.hi;
  _x6874 = _x8; /*double*/
  _x6873 = (-_x6874); /*double*/
  double _x6877;
  double _x6878;
  kk_box_t _box_x5490 = _this0_4705.fst;
  kk_box_t _box_x5491 = _this0_4705.snd;
  kk_std_time_duration__duration _x601 = kk_std_time_duration__duration_unbox(_box_x5490, NULL);
  kk_std_core_types__tuple2__drop(_this0_4705, _ctx);
  kk_Unit;
  kk_std_num_ddouble__ddouble _x90 = _x601.secs;
  double _x010 = _x90.lo;
  _x6878 = _x010; /*double*/
  _x6877 = (-_x6878); /*double*/
  _x6872 = kk_std_num_ddouble__new_Ddouble(_x6873, _x6877, _ctx); /*std/num/ddouble/ddouble*/
  _x6870 = kk_std_num_ddouble__lp__plus__rp_(_x6871, _x6872, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6881;
  kk_std_num_ddouble__ddouble _pat021 = t_4878.since;
  int32_t _x21 = t_4878.leap32;
  _x6881 = _x21; /*int32*/
  t3_4708 = kk_std_time_timestamp__new_Timestamp(_x6870, _x6881, _ctx); /*std/time/timestamp/timestamp*/
  return kk_std_time_instant__new_Instant(kk_reuse_null, t3_4708, ts1, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_combine_earth_calendars_fun6882__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal1;
  kk_std_time_calendar__calendar cal2;
  kk_std_time_date__date switch_date;
};
static kk_integer_t kk_std_time_calendar_combine_earth_calendars_fun6882(kk_function_t _fself, kk_std_time_date__date d10, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_combine_earth_calendars_fun6882(kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_time_date__date switch_date, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6882__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_combine_earth_calendars_fun6882__t, 6, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_combine_earth_calendars_fun6882, kk_context());
  _self->cal1 = cal1;
  _self->cal2 = cal2;
  _self->switch_date = switch_date;
  return &_self->_base;
}

static kk_integer_t kk_std_time_calendar_combine_earth_calendars_fun6882(kk_function_t _fself, kk_std_time_date__date d10, kk_context_t* _ctx) {
  struct kk_std_time_calendar_combine_earth_calendars_fun6882__t* _self = kk_function_as(struct kk_std_time_calendar_combine_earth_calendars_fun6882__t*, _fself);
  kk_std_time_calendar__calendar cal1 = _self->cal1; /* std/time/calendar/calendar */
  kk_std_time_calendar__calendar cal2 = _self->cal2; /* std/time/calendar/calendar */
  kk_std_time_date__date switch_date = _self->switch_date; /* std/time/date/date */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal1);kk_std_time_calendar__calendar_dup(cal2);kk_std_time_date__date_dup(switch_date);}, {}, _ctx)
  kk_function_t _x6883;
  kk_std_time_calendar__calendar calendar1_47950;
  bool _match_5577;
  kk_std_time_date__date _x6884 = kk_std_time_date__date_dup(d10); /*std/time/date/date*/
  _match_5577 = kk_std_time_date__lp__lt__rp_(_x6884, switch_date, _ctx); /*bool*/
  if (_match_5577) {
    kk_std_time_calendar__calendar_dropn(cal2, ((int32_t)8), _ctx);
    calendar1_47950 = cal1; /*std/time/calendar/calendar*/
  }
  else {
    kk_std_time_calendar__calendar_dropn(cal1, ((int32_t)8), _ctx);
    calendar1_47950 = cal2; /*std/time/calendar/calendar*/
  }
  struct kk_std_time_calendar_Calendar* _con6885 = kk_std_time_calendar__as_Calendar(calendar1_47950);
  kk_string_t _pat081 = _con6885->name;
  kk_string_t _pat1121 = _con6885->long_name;
  kk_string_t _pat2101 = _con6885->month_prefix;
  kk_function_t _pat350 = _con6885->show_era;
  kk_function_t _pat430 = _con6885->instant_to_dc;
  kk_function_t _pat530 = _con6885->dc_to_instant;
  kk_function_t _pat620 = _con6885->days_to_date;
  kk_function_t _x61 = _con6885->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(calendar1_47950))) {
    kk_function_drop(_pat620, _ctx);
    kk_Unit;
    kk_function_drop(_pat530, _ctx);
    kk_Unit;
    kk_function_drop(_pat430, _ctx);
    kk_Unit;
    kk_function_drop(_pat350, _ctx);
    kk_Unit;
    kk_string_drop(_pat2101, _ctx);
    kk_Unit;
    kk_string_drop(_pat1121, _ctx);
    kk_Unit;
    kk_string_drop(_pat081, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(calendar1_47950);
  }
  else {
    kk_function_dup(_x61);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(calendar1_47950, _ctx);
    kk_Unit;
  }
  _x6883 = _x61; /*(date : std/time/date/date) -> int*/
  return kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6883, (_x6883, d10, _ctx));
}

kk_std_time_calendar__calendar kk_std_time_calendar_combine_earth_calendars(kk_string_t name0, kk_string_t long_name0, kk_std_time_date__date switch_date, kk_std_time_calendar__calendar cal1, kk_std_time_calendar__calendar cal2, kk_std_core_types__optional mb_show_era, kk_context_t* _ctx) { /* (name : string, long-name : string, switch-date : std/time/date/date, cal1 : calendar, cal2 : calendar, mb-show-era : optional<maybe<(std/time/date/date) -> string>>) -> calendar */ 
  kk_function_t days_to_date0;
  kk_std_time_calendar__calendar_dup(cal1);
  kk_std_time_calendar__calendar_dup(cal2);
  kk_std_time_date__date_dup(switch_date);
  days_to_date0 = kk_std_time_calendar_new_combine_earth_calendars_fun6797(cal1, cal2, switch_date, _ctx); /*(days1 : int) -> std/time/date/date*/
  kk_string_t _x6802;
  kk_std_core_types__optional _match_5586 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5586)) {
    kk_box_t _box_x5464 = _match_5586._cons.Optional.value;
    kk_string_t _month_prefix_3566 = kk_string_unbox(_box_x5464);
    kk_string_dup(_month_prefix_3566);
    kk_std_core_types__optional_drop(_match_5586, _ctx);
    kk_Unit;
    _x6802 = _month_prefix_3566; /*string*/
    goto _match6803;
  }
  _x6802 = kk_string_empty(); /*string*/
  _match6803: ;
  kk_function_t _x6806;
  kk_std_time_calendar__calendar_dup(cal1);
  kk_std_time_calendar__calendar_dup(cal2);
  kk_std_time_date__date_dup(switch_date);
  _x6806 = kk_std_time_calendar_new_combine_earth_calendars_fun6807(cal1, cal2, mb_show_era, switch_date, _ctx); /*(d01 : std/time/date/date) -> string*/
  kk_function_t _x6815;
  kk_function_dup(days_to_date0);
  _x6815 = kk_std_time_calendar_new_combine_earth_calendars_fun6816(days_to_date0, _ctx); /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (6, 7)*/
  kk_function_t _x6830;
  kk_std_time_calendar__calendar_dup(cal1);
  kk_std_time_calendar__calendar_dup(cal2);
  kk_std_time_date__date_dup(switch_date);
  _x6830 = kk_std_time_calendar_new_combine_earth_calendars_fun6831(cal1, cal2, switch_date, _ctx); /*(d00 : std/time/date/date, c : std/time/date/clock, tz : std/time/calendar/timezone, ts1 : std/time/instant/timescale) -> std/time/instant/instant*/
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, name0, long_name0, _x6802, _x6806, _x6815, _x6830, days_to_date0, kk_std_time_calendar_new_combine_earth_calendars_fun6882(cal1, cal2, switch_date, _ctx), _ctx);
}
 
// Convert an `:instant` to a `:date`, `:clock`, timezone delta and abbreviation,
// for a given timezone `tz` (=`tz-utc` by default) and calendar (=`cal-iso` by default).

kk_std_core_types__tuple4_ kk_std_time_calendar_instant_dc(kk_std_time_instant__instant i, kk_std_core_types__optional tz, kk_std_core_types__optional cal, kk_context_t* _ctx) { /* (i : std/time/instant/instant, tz : optional<timezone>, cal : optional<calendar>) -> (std/time/date/date, std/time/date/clock, std/time/duration/duration, string) */ 
  kk_std_core_types__tuple2_ _match_5573;
  kk_function_t _x6887;
  kk_std_time_calendar__timezone _match_5576;
  if (kk_std_core_types__is_Optional(tz)) {
    kk_box_t _box_x5494 = tz._cons.Optional.value;
    kk_std_time_calendar__timezone _tz_4272 = kk_std_time_calendar__timezone_unbox(_box_x5494, NULL);
    kk_std_time_calendar__timezone_dup(_tz_4272);
    kk_std_core_types__optional_drop(tz, _ctx);
    kk_Unit;
    _match_5576 = _tz_4272; /*std/time/calendar/timezone*/
    goto _match6888;
  }
  _match_5576 = kk_std_time_calendar__timezone_dup(kk_std_time_calendar_tz_utc); /*std/time/calendar/timezone*/
  _match6888: ;
  kk_function_t _x = _match_5576.utc_delta;
  kk_function_dup(_x);
  kk_std_time_calendar__timezone_drop(_match_5576, _ctx);
  kk_Unit;
  _x6887 = _x; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  kk_std_time_instant__instant _x6886 = kk_std_time_instant__instant_dup(i); /*std/time/instant/instant*/
  _match_5573 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6887, (_x6887, _x6886, _ctx)); /*(std/time/duration/duration, string)*/
  kk_box_t _box_x5495 = _match_5573.fst;
  kk_box_t _box_x5496 = _match_5573.snd;
  kk_std_time_duration__duration tzdelta = kk_std_time_duration__duration_unbox(_box_x5495, NULL);
  kk_string_t tzabbrv = kk_string_unbox(_box_x5496);
  kk_string_dup(tzabbrv);
  kk_std_core_types__tuple2__drop(_match_5573, _ctx);
  kk_Unit;
  kk_std_core_types__tuple2_ _match_5574;
  kk_function_t _x6892;
  kk_std_time_calendar__calendar _match_5575;
  if (kk_std_core_types__is_Optional(cal)) {
    kk_box_t _box_x5497 = cal._cons.Optional.value;
    kk_std_time_calendar__calendar _cal_4276 = kk_std_time_calendar__calendar_unbox(_box_x5497, NULL);
    kk_std_time_calendar__calendar_dup(_cal_4276);
    kk_std_core_types__optional_drop(cal, _ctx);
    kk_Unit;
    _match_5575 = _cal_4276; /*std/time/calendar/calendar*/
    goto _match6893;
  }
  _match_5575 = kk_std_time_calendar__calendar_dup(kk_std_time_calendar_cal_iso); /*std/time/calendar/calendar*/
  _match6893: ;
  struct kk_std_time_calendar_Calendar* _con6895 = kk_std_time_calendar__as_Calendar(_match_5575);
  kk_string_t _pat01 = _con6895->name;
  kk_string_t _pat10 = _con6895->long_name;
  kk_string_t _pat20 = _con6895->month_prefix;
  kk_function_t _pat30 = _con6895->show_era;
  kk_function_t _x0 = _con6895->instant_to_dc;
  kk_function_t _pat40 = _con6895->dc_to_instant;
  kk_function_t _pat50 = _con6895->days_to_date;
  kk_function_t _pat6 = _con6895->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(_match_5575))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat50, _ctx);
    kk_Unit;
    kk_function_drop(_pat40, _ctx);
    kk_Unit;
    kk_function_drop(_pat30, _ctx);
    kk_Unit;
    kk_string_drop(_pat20, _ctx);
    kk_Unit;
    kk_string_drop(_pat10, _ctx);
    kk_Unit;
    kk_string_drop(_pat01, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(_match_5575);
  }
  else {
    kk_function_dup(_x0);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(_match_5575, _ctx);
    kk_Unit;
  }
  _x6892 = _x0; /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (std/time/date/date, std/time/date/clock)*/
  _match_5574 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_std_time_duration__duration, kk_context_t*), _x6892, (_x6892, i, tzdelta, _ctx)); /*(std/time/date/date, std/time/date/clock)*/
  kk_box_t _box_x5498 = _match_5574.fst;
  kk_box_t _box_x5499 = _match_5574.snd;
  kk_std_time_date__date d = kk_std_time_date__date_unbox(_box_x5498, NULL);
  kk_std_time_date__clock c = kk_std_time_date__clock_unbox(_box_x5499, NULL);
  struct kk_std_time_date_Clock* _con6898 = kk_std_time_date__as_Clock(c);
  kk_std_time_date__clock_dup(c);
  kk_std_time_date__date_dup(d);
  kk_std_core_types__tuple2__drop(_match_5574, _ctx);
  kk_Unit;
  kk_std_time_date__date _b_5504_5500 = d; /*std/time/date/date*/;
  kk_std_time_date__clock _b_5505_5501 = c; /*std/time/date/clock*/;
  kk_std_time_duration__duration _b_5506_5502 = tzdelta; /*std/time/duration/duration*/;
  kk_string_t _b_5507_5503 = tzabbrv; /*string*/;
  return kk_std_core_types__new_dash__lp__comma__comma__comma__rp_(kk_reuse_null, kk_std_time_date__date_box(_b_5504_5500, _ctx), kk_std_time_date__clock_box(_b_5505_5501, _ctx), kk_std_time_duration__duration_box(_b_5506_5502, _ctx), kk_string_box(_b_5507_5503), _ctx);
}
 
// Return the local timezone on the current system.


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6901__t {
  struct kk_function_s _base;
  kk_std_time_calendar__local_timezone tz;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_local_fun6901(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6901(kk_std_time_calendar__local_timezone tz, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6901__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_local_fun6901__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_local_fun6901, kk_context());
  _self->tz = tz;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_tz_local_fun6901(kk_function_t _fself, kk_std_time_instant__instant i0, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6901__t* _self = kk_function_as(struct kk_std_time_calendar_tz_local_fun6901__t*, _fself);
  kk_std_time_calendar__local_timezone tz = _self->tz; /* std/time/calendar/local-timezone */
  kk_drop_match(_self, {kk_std_time_calendar__local_timezone_dup(tz);}, {}, _ctx)
  kk_std_core_types__tuple2_ _match_5572;
  double _x6902;
  kk_std_num_ddouble__ddouble x_4802 = kk_std_time_utc_unix_timestamp(i0, _ctx); /*std/num/ddouble/ddouble*/;
  double _x = x_4802.hi;
  _x6902 = _x; /*double*/
  _match_5572 = kk_std_time_calendar_local_utc_delta(tz, _x6902, _ctx); /*(double, string)*/
  kk_box_t _box_x5508 = _match_5572.fst;
  kk_box_t _box_x5509 = _match_5572.snd;
  double ofs = kk_double_unbox(_box_x5508, NULL);
  kk_string_t abbrv = kk_string_unbox(_box_x5509);
  kk_string_dup(abbrv);
  kk_std_core_types__tuple2__drop(_match_5572, _ctx);
  kk_Unit;
  kk_std_time_duration__duration _b_5519_5510;
  kk_std_num_ddouble__ddouble _x6905 = kk_std_num_ddouble__new_Ddouble(ofs, 0.0, _ctx); /*std/num/ddouble/ddouble*/
  _b_5519_5510 = kk_std_time_duration__new_Duration(_x6905, _ctx); /*std/time/duration/duration*/
  kk_string_t _b_5520_5511 = abbrv; /*string*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_duration__duration_box(_b_5519_5510, _ctx), kk_string_box(_b_5520_5511), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6908__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5515;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6908(kk_function_t _fself, kk_std_time_instant__instant _b_5518, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6908(kk_box_t _fun_unbox_x5515, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6908__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_tz_local_fun6908__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_tz_local_fun6908, kk_context());
  _self->_fun_unbox_x5515 = _fun_unbox_x5515;
  return &_self->_base;
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6908(kk_function_t _fself, kk_std_time_instant__instant _b_5518, kk_context_t* _ctx) {
  struct kk_std_time_calendar_tz_local_fun6908__t* _self = kk_function_as(struct kk_std_time_calendar_tz_local_fun6908__t*, _fself);
  kk_box_t _fun_unbox_x5515 = _self->_fun_unbox_x5515; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5515);}, {}, _ctx)
  kk_box_t _x6909;
  kk_function_t _x6910 = kk_function_unbox(_fun_unbox_x5515); /*(5516) -> 5517*/
  _x6909 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6910, (_x6910, kk_std_time_instant__instant_box(_b_5518, _ctx), _ctx)); /*5517*/
  return kk_std_core_types__maybe_unbox(_x6909, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_tz_local_fun6911__t {
  struct kk_function_s _base;
};
static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6911(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_tz_local_fun6911(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_tz_local_fun6911, _ctx)
  return kk_function_dup(_fself);
}

static kk_std_core_types__maybe kk_std_time_calendar_tz_local_fun6911(kk_function_t _fself, kk_std_time_instant__instant i, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_instant__instant_dropn(i, ((int32_t)1), _ctx);
  return kk_std_core_types__new_Nothing(_ctx);
}

kk_std_time_calendar__timezone kk_std_time_calendar_tz_local(kk_context_t* _ctx) { /* () -> ndet timezone */ 
  kk_std_time_calendar__local_timezone tz = kk_std_time_calendar_local_get_timezone(_ctx); /*std/time/calendar/local-timezone*/;
  kk_string_t _x6899 = kk_string_empty(); /*string*/
  kk_function_t _x6906;
  kk_std_core_types__optional _x6907 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/
  if (kk_std_core_types__is_Optional(_x6907)) {
    kk_box_t _fun_unbox_x5515 = _x6907._cons.Optional.value;
    _x6906 = kk_std_time_calendar_new_tz_local_fun6908(_fun_unbox_x5515, _ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  else {
    _x6906 = kk_std_time_calendar_new_tz_local_fun6911(_ctx); /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  }
  return kk_std_time_calendar__new_Timezone(_x6899, kk_std_time_calendar_new_tz_local_fun6901(tz, _ctx), _x6906, _ctx);
}
 
// Return the day of the week for a calendar `cal` (=`cal-iso`).

kk_std_time_date__weekday kk_std_time_calendar_weekday(kk_std_time_date__date d, kk_std_time_calendar__calendar cal, kk_context_t* _ctx) { /* (d : std/time/date/date, cal : calendar) -> std/time/date/weekday */ 
  kk_integer_t dow;
  kk_integer_t _x6912;
  kk_integer_t _x6913;
  kk_function_t _x6914;
  struct kk_std_time_calendar_Calendar* _con6915 = kk_std_time_calendar__as_Calendar(cal);
  kk_string_t _pat0 = _con6915->name;
  kk_string_t _pat1 = _con6915->long_name;
  kk_string_t _pat2 = _con6915->month_prefix;
  kk_function_t _pat3 = _con6915->show_era;
  kk_function_t _pat4 = _con6915->instant_to_dc;
  kk_function_t _pat5 = _con6915->dc_to_instant;
  kk_function_t _pat6 = _con6915->days_to_date;
  kk_function_t _x = _con6915->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(cal))) {
    kk_function_drop(_pat6, _ctx);
    kk_Unit;
    kk_function_drop(_pat5, _ctx);
    kk_Unit;
    kk_function_drop(_pat4, _ctx);
    kk_Unit;
    kk_function_drop(_pat3, _ctx);
    kk_Unit;
    kk_string_drop(_pat2, _ctx);
    kk_Unit;
    kk_string_drop(_pat1, _ctx);
    kk_Unit;
    kk_string_drop(_pat0, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(cal);
  }
  else {
    kk_function_dup(_x);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(cal, _ctx);
    kk_Unit;
  }
  _x6914 = _x; /*(date : std/time/date/date) -> int*/
  _x6913 = kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6914, (_x6914, d, _ctx)); /*int*/
  _x6912 = kk_integer_add(_x6913,(kk_integer_from_small(6)),kk_context()); /*int*/
  dow = kk_integer_mod(_x6912,(kk_integer_from_small(7)),kk_context()); /*int*/
  return kk_std_time_date_weekday(dow, _ctx);
}
 
// Create a new calendar from anoter calendar `cal` by adding an offset to the
// years (`year-shift`). This is used for example to create the Ethiopian calendar
// from the Coptic calendar, by using:
// `year-shift-earth-calendar( "EC", "Ethiopian", ~276, cal-coptic )`


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6916__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal;
  kk_integer_t year_shift;
};
static kk_std_time_date__date kk_std_time_calendar_year_shift_earth_calendar_fun6916(kk_function_t _fself, kk_integer_t days1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6916(kk_std_time_calendar__calendar cal, kk_integer_t year_shift, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6916__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6916__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6916, kk_context());
  _self->cal = cal;
  _self->year_shift = year_shift;
  return &_self->_base;
}

static kk_std_time_date__date kk_std_time_calendar_year_shift_earth_calendar_fun6916(kk_function_t _fself, kk_integer_t days1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6916__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6916__t*, _fself);
  kk_std_time_calendar__calendar cal = _self->cal; /* std/time/calendar/calendar */
  kk_integer_t year_shift = _self->year_shift; /* int */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal);kk_integer_dup(year_shift);}, {}, _ctx)
  kk_std_time_date__date d01;
  kk_function_t _x6917;
  struct kk_std_time_calendar_Calendar* _con6918 = kk_std_time_calendar__as_Calendar(cal);
  kk_string_t _pat06 = _con6918->name;
  kk_string_t _pat19 = _con6918->long_name;
  kk_string_t _pat25 = _con6918->month_prefix;
  kk_function_t _pat33 = _con6918->show_era;
  kk_function_t _pat42 = _con6918->instant_to_dc;
  kk_function_t _pat52 = _con6918->dc_to_instant;
  kk_function_t _x4 = _con6918->days_to_date;
  kk_function_t _pat61 = _con6918->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(cal))) {
    kk_function_drop(_pat61, _ctx);
    kk_Unit;
    kk_function_drop(_pat52, _ctx);
    kk_Unit;
    kk_function_drop(_pat42, _ctx);
    kk_Unit;
    kk_function_drop(_pat33, _ctx);
    kk_Unit;
    kk_string_drop(_pat25, _ctx);
    kk_Unit;
    kk_string_drop(_pat19, _ctx);
    kk_Unit;
    kk_string_drop(_pat06, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(cal);
  }
  else {
    kk_function_dup(_x4);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(cal, _ctx);
    kk_Unit;
  }
  _x6917 = _x4; /*(days : int) -> std/time/date/date*/
  d01 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), _x6917, (_x6917, days1, _ctx)); /*std/time/date/date*/
  kk_integer_t _b_5522_5521;
  kk_integer_t _x6919;
  kk_integer_t _x6 = d01.year;
  kk_integer_dup(_x6);
  _x6919 = _x6; /*int*/
  _b_5522_5521 = kk_integer_sub(_x6919,year_shift,kk_context()); /*int*/
  kk_integer_t _x6920;
  kk_std_core_types__optional _match_5570 = kk_std_core_types__new_Optional(kk_integer_box(_b_5522_5521), _ctx); /*optional<106>*/;
  if (kk_std_core_types__is_Optional(_match_5570)) {
    kk_box_t _box_x5523 = _match_5570._cons.Optional.value;
    kk_integer_t _year_143 = kk_integer_unbox(_box_x5523);
    kk_integer_dup(_year_143);
    kk_std_core_types__optional_drop(_match_5570, _ctx);
    kk_Unit;
    _x6920 = _year_143; /*int*/
    goto _match6921;
  }
  kk_integer_t _x5 = d01.year;
  kk_integer_dup(_x5);
  _x6920 = _x5; /*int*/
  _match6921: ;
  kk_integer_t _x6923;
  kk_std_core_types__optional _match_5569 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5569)) {
    kk_box_t _box_x5524 = _match_5569._cons.Optional.value;
    kk_integer_t _month_149 = kk_integer_unbox(_box_x5524);
    kk_integer_dup(_month_149);
    kk_std_core_types__optional_drop(_match_5569, _ctx);
    kk_Unit;
    _x6923 = _month_149; /*int*/
    goto _match6924;
  }
  kk_integer_t _x01 = d01.month;
  kk_integer_dup(_x01);
  _x6923 = _x01; /*int*/
  _match6924: ;
  kk_integer_t _x6926;
  kk_std_core_types__optional _match_5568 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5568)) {
    kk_box_t _box_x5525 = _match_5568._cons.Optional.value;
    kk_integer_t _day_155 = kk_integer_unbox(_box_x5525);
    kk_std_time_date__date_drop(d01, _ctx);
    kk_integer_dup(_day_155);
    kk_std_core_types__optional_drop(_match_5568, _ctx);
    kk_Unit;
    _x6926 = _day_155; /*int*/
    goto _match6927;
  }
  kk_integer_t _x10 = d01.day;
  kk_integer_dup(_x10);
  kk_std_time_date__date_drop(d01, _ctx);
  kk_Unit;
  _x6926 = _x10; /*int*/
  _match6927: ;
  return kk_std_time_date__new_Date(_x6920, _x6923, _x6926, _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6929__t {
  struct kk_function_s _base;
  kk_std_time_calendar__calendar cal;
  kk_integer_t year_shift;
};
static kk_integer_t kk_std_time_calendar_year_shift_earth_calendar_fun6929(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6929(kk_std_time_calendar__calendar cal, kk_integer_t year_shift, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6929__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6929__t, 3, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6929, kk_context());
  _self->cal = cal;
  _self->year_shift = year_shift;
  return &_self->_base;
}

static kk_integer_t kk_std_time_calendar_year_shift_earth_calendar_fun6929(kk_function_t _fself, kk_std_time_date__date d1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6929__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6929__t*, _fself);
  kk_std_time_calendar__calendar cal = _self->cal; /* std/time/calendar/calendar */
  kk_integer_t year_shift = _self->year_shift; /* int */
  kk_drop_match(_self, {kk_std_time_calendar__calendar_dup(cal);kk_integer_dup(year_shift);}, {}, _ctx)
  kk_function_t _x6941;
  struct kk_std_time_calendar_Calendar* _con6942 = kk_std_time_calendar__as_Calendar(cal);
  kk_string_t _pat09 = _con6942->name;
  kk_string_t _pat113 = _con6942->long_name;
  kk_string_t _pat210 = _con6942->month_prefix;
  kk_function_t _pat35 = _con6942->show_era;
  kk_function_t _pat44 = _con6942->instant_to_dc;
  kk_function_t _pat54 = _con6942->dc_to_instant;
  kk_function_t _pat62 = _con6942->days_to_date;
  kk_function_t _x7 = _con6942->date_to_days;
  if (kk_likely(kk_std_time_calendar__calendar_is_unique(cal))) {
    kk_function_drop(_pat62, _ctx);
    kk_Unit;
    kk_function_drop(_pat54, _ctx);
    kk_Unit;
    kk_function_drop(_pat44, _ctx);
    kk_Unit;
    kk_function_drop(_pat35, _ctx);
    kk_Unit;
    kk_string_drop(_pat210, _ctx);
    kk_Unit;
    kk_string_drop(_pat113, _ctx);
    kk_Unit;
    kk_string_drop(_pat09, _ctx);
    kk_Unit;
    kk_Unit;
    kk_std_time_calendar__calendar_free(cal);
  }
  else {
    kk_function_dup(_x7);
    kk_Unit;
    kk_std_time_calendar__calendar_decref(cal, _ctx);
    kk_Unit;
  }
  _x6941 = _x7; /*(date : std/time/date/date) -> int*/
  kk_std_time_date__date _x6930;
  kk_integer_t _b_5527_5526;
  kk_integer_t _x6931;
  kk_integer_t _x9 = d1.year;
  kk_integer_dup(_x9);
  _x6931 = _x9; /*int*/
  _b_5527_5526 = kk_integer_add(_x6931,year_shift,kk_context()); /*int*/
  kk_integer_t _x6932;
  kk_std_core_types__optional _match_5567 = kk_std_core_types__new_Optional(kk_integer_box(_b_5527_5526), _ctx); /*optional<106>*/;
  if (kk_std_core_types__is_Optional(_match_5567)) {
    kk_box_t _box_x5528 = _match_5567._cons.Optional.value;
    kk_integer_t _year_1430 = kk_integer_unbox(_box_x5528);
    kk_integer_dup(_year_1430);
    kk_std_core_types__optional_drop(_match_5567, _ctx);
    kk_Unit;
    _x6932 = _year_1430; /*int*/
    goto _match6933;
  }
  kk_integer_t _x8 = d1.year;
  kk_integer_dup(_x8);
  _x6932 = _x8; /*int*/
  _match6933: ;
  kk_integer_t _x6935;
  kk_std_core_types__optional _match_5566 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5566)) {
    kk_box_t _box_x5529 = _match_5566._cons.Optional.value;
    kk_integer_t _month_1490 = kk_integer_unbox(_box_x5529);
    kk_integer_dup(_month_1490);
    kk_std_core_types__optional_drop(_match_5566, _ctx);
    kk_Unit;
    _x6935 = _month_1490; /*int*/
    goto _match6936;
  }
  kk_integer_t _x02 = d1.month;
  kk_integer_dup(_x02);
  _x6935 = _x02; /*int*/
  _match6936: ;
  kk_integer_t _x6938;
  kk_std_core_types__optional _match_5565 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
  if (kk_std_core_types__is_Optional(_match_5565)) {
    kk_box_t _box_x5530 = _match_5565._cons.Optional.value;
    kk_integer_t _day_1550 = kk_integer_unbox(_box_x5530);
    kk_std_time_date__date_drop(d1, _ctx);
    kk_integer_dup(_day_1550);
    kk_std_core_types__optional_drop(_match_5565, _ctx);
    kk_Unit;
    _x6938 = _day_1550; /*int*/
    goto _match6939;
  }
  kk_integer_t _x11 = d1.day;
  kk_integer_dup(_x11);
  kk_std_time_date__date_drop(d1, _ctx);
  kk_Unit;
  _x6938 = _x11; /*int*/
  _match6939: ;
  _x6930 = kk_std_time_date__new_Date(_x6932, _x6935, _x6938, _ctx); /*std/time/date/date*/
  return kk_function_call(kk_integer_t, (kk_function_t, kk_std_time_date__date, kk_context_t*), _x6941, (_x6941, _x6930, _ctx));
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6948__t {
  struct kk_function_s _base;
  kk_box_t _fun_unbox_x5535;
};
static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6948(kk_function_t _fself, kk_std_time_date__date _b_5538, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6948(kk_box_t _fun_unbox_x5535, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6948__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6948__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6948, kk_context());
  _self->_fun_unbox_x5535 = _fun_unbox_x5535;
  return &_self->_base;
}

static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6948(kk_function_t _fself, kk_std_time_date__date _b_5538, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6948__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6948__t*, _fself);
  kk_box_t _fun_unbox_x5535 = _self->_fun_unbox_x5535; /* 106 */
  kk_drop_match(_self, {kk_box_dup(_fun_unbox_x5535);}, {}, _ctx)
  kk_box_t _x6949;
  kk_function_t _x6950 = kk_function_unbox(_fun_unbox_x5535); /*(5536) -> 5537*/
  _x6949 = kk_function_call(kk_box_t, (kk_function_t, kk_box_t, kk_context_t*), _x6950, (_x6950, kk_std_time_date__date_box(_b_5538, _ctx), _ctx)); /*5537*/
  return kk_string_unbox(_x6949);
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6951__t {
  struct kk_function_s _base;
};
static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6951(kk_function_t _fself, kk_std_time_date__date d2, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6951(kk_context_t* _ctx) {
  kk_define_static_function(_fself, kk_std_time_calendar_year_shift_earth_calendar_fun6951, _ctx)
  return kk_function_dup(_fself);
}

static kk_string_t kk_std_time_calendar_year_shift_earth_calendar_fun6951(kk_function_t _fself, kk_std_time_date__date d2, kk_context_t* _ctx) {
  KK_UNUSED(_fself);
  kk_std_time_date__date_drop(d2, _ctx);
  return kk_string_empty();
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6954__t {
  struct kk_function_s _base;
  kk_function_t days_to_date0_4807;
};
static kk_std_core_types__tuple2_ kk_std_time_calendar_year_shift_earth_calendar_fun6954(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6954(kk_function_t days_to_date0_4807, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6954__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6954__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6954, kk_context());
  _self->days_to_date0_4807 = days_to_date0_4807;
  return &_self->_base;
}

static kk_std_core_types__tuple2_ kk_std_time_calendar_year_shift_earth_calendar_fun6954(kk_function_t _fself, kk_std_time_instant__instant i, kk_std_time_duration__duration tzdelta, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6954__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6954__t*, _fself);
  kk_function_t days_to_date0_4807 = _self->days_to_date0_4807; /* (days : int) -> std/time/date/date */
  kk_drop_match(_self, {kk_function_dup(days_to_date0_4807);}, {}, _ctx)
  kk_std_time_timestamp__timestamp ts_4828;
  kk_std_num_ddouble__ddouble _x6955;
  kk_std_num_ddouble__ddouble _x6956;
  struct kk_std_time_instant_Instant* _con6957 = kk_std_time_instant__as_Instant(i);
  kk_std_time_timestamp__timestamp _x1 = _con6957->since;
  kk_std_time_instant__timescale _pat23 = _con6957->ts;
  struct kk_std_time_instant_Timescale* _con6958 = kk_std_time_instant__as_Timescale(_pat23);
  kk_std_num_ddouble__ddouble _x0 = _x1.since;
  _x6956 = _x0; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6959;
  kk_std_num_ddouble__ddouble _x2 = tzdelta.secs;
  _x6959 = _x2; /*std/time/timestamp/timespan*/
  _x6955 = kk_std_num_ddouble__lp__plus__rp_(_x6956, _x6959, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6960;
  struct kk_std_time_instant_Instant* _con6961 = kk_std_time_instant__as_Instant(i);
  kk_std_time_timestamp__timestamp _x12 = _con6961->since;
  kk_std_time_instant__timescale _pat230 = _con6961->ts;
  struct kk_std_time_instant_Timescale* _con6962 = kk_std_time_instant__as_Timescale(_pat230);
  kk_string_t _pat310 = _con6962->name;
  kk_string_t _pat400 = _con6962->unit;
  kk_function_t _pat500 = _con6962->from_tai;
  kk_function_t _pat600 = _con6962->to_tai;
  kk_std_core_types__maybe _pat70 = _con6962->mb_seconds_in_day;
  kk_std_core_types__maybe _pat80 = _con6962->mb_to_mjd2000;
  kk_std_core_types__maybe _pat90 = _con6962->mb_from_mjd2000;
  if (kk_likely(kk_std_time_instant__instant_is_unique(i))) {
    if (kk_likely(kk_std_time_instant__timescale_is_unique(_pat230))) {
      kk_string_drop(_pat310, _ctx);
      kk_string_drop(_pat400, _ctx);
      kk_function_drop(_pat500, _ctx);
      kk_function_drop(_pat600, _ctx);
      kk_std_core_types__maybe_drop(_pat70, _ctx);
      kk_std_core_types__maybe_drop(_pat80, _ctx);
      kk_std_core_types__maybe_drop(_pat90, _ctx);
      kk_Unit;
      kk_std_time_instant__timescale_free(_pat230);
    }
    else {
      kk_std_time_instant__timescale_decref(_pat230, _ctx);
      kk_Unit;
    }
    kk_Unit;
    kk_std_time_instant__instant_free(i);
  }
  else {
    kk_std_time_instant__instant_decref(i, _ctx);
    kk_Unit;
  }
  kk_std_num_ddouble__ddouble _pat001 = _x12.since;
  int32_t _x00 = _x12.leap32;
  _x6960 = _x00; /*int32*/
  ts_4828 = kk_std_time_timestamp__new_Timestamp(_x6955, _x6960, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_core_types__tuple2_ _match_5563 = kk_std_time_timestamp_days_seconds(ts_4828, _ctx); /*(int, std/num/ddouble/ddouble)*/;
  kk_box_t _box_x5539 = _match_5563.fst;
  kk_box_t _box_x5540 = _match_5563.snd;
  kk_integer_t days0 = kk_integer_unbox(_box_x5539);
  kk_std_num_ddouble__ddouble secs = kk_std_num_ddouble__ddouble_unbox(_box_x5540, NULL);
  kk_integer_dup(days0);
  kk_std_core_types__tuple2__drop(_match_5563, _ctx);
  kk_Unit;
  kk_std_time_date__clock _match_5564;
  kk_integer_t _x6965;
  int32_t _x6966;
  kk_std_num_ddouble__ddouble _pat000 = ts_4828.since;
  int32_t _x = ts_4828.leap32;
  _x6966 = _x; /*int32*/
  _x6965 = kk_integer_from_int(_x6966,kk_context()); /*int*/
  _match_5564 = kk_std_time_date_clock_3(secs, _x6965, _ctx); /*std/time/date/clock*/
  struct kk_std_time_date_Clock* _con6967 = kk_std_time_date__as_Clock(_match_5564);
  kk_std_time_date__date _b_5558_5541 = kk_function_call(kk_std_time_date__date, (kk_function_t, kk_integer_t, kk_context_t*), days_to_date0_4807, (days_to_date0_4807, days0, _ctx)); /*std/time/date/date*/;
  kk_std_time_date__clock _b_5559_5542 = _match_5564; /*std/time/date/clock*/;
  return kk_std_core_types__new_dash__lp__comma__rp_(kk_std_time_date__date_box(_b_5558_5541, _ctx), kk_std_time_date__clock_box(_b_5559_5542, _ctx), _ctx);
}


// lift anonymous function
struct kk_std_time_calendar_year_shift_earth_calendar_fun6969__t {
  struct kk_function_s _base;
  kk_function_t date_to_days0_4808;
};
static kk_std_time_instant__instant kk_std_time_calendar_year_shift_earth_calendar_fun6969(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts1, kk_context_t* _ctx);
static kk_function_t kk_std_time_calendar_new_year_shift_earth_calendar_fun6969(kk_function_t date_to_days0_4808, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6969__t* _self = kk_function_alloc_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6969__t, 2, _ctx);
  _self->_base.fun = kk_cfun_ptr_box(&kk_std_time_calendar_year_shift_earth_calendar_fun6969, kk_context());
  _self->date_to_days0_4808 = date_to_days0_4808;
  return &_self->_base;
}

static kk_std_time_instant__instant kk_std_time_calendar_year_shift_earth_calendar_fun6969(kk_function_t _fself, kk_std_time_date__date d00, kk_std_time_date__clock c, kk_std_time_calendar__timezone tz, kk_std_time_instant__timescale ts1, kk_context_t* _ctx) {
  struct kk_std_time_calendar_year_shift_earth_calendar_fun6969__t* _self = kk_function_as(struct kk_std_time_calendar_year_shift_earth_calendar_fun6969__t*, _fself);
  kk_function_t date_to_days0_4808 = _self->date_to_days0_4808; /* (std/time/date/date) -> int */
  kk_drop_match(_self, {kk_function_dup(date_to_days0_4808);}, {}, _ctx)
  kk_std_time_timestamp__timestamp t_4881;
  kk_std_core_types__optional _x6970;
  kk_box_t _x6971;
  bool _x6972;
  struct kk_std_time_instant_Timescale* _con6973 = kk_std_time_instant__as_Timescale(ts1);
  kk_std_core_types__maybe _x31 = _con6973->mb_seconds_in_day;
  kk_std_core_types__maybe_dup(_x31);
  if (kk_std_core_types__is_Nothing(_x31)) {
    _x6972 = false; /*bool*/
  }
  else {
    kk_std_core_types__maybe_drop(_x31, _ctx);
    _x6972 = true; /*bool*/
  }
  _x6971 = kk_bool_box(_x6972); /*106*/
  _x6970 = kk_std_core_types__new_Optional(_x6971, _ctx); /*optional<106>*/
  t_4881 = kk_std_time_calendar_dc_timestamp(d00, c, date_to_days0_4808, _x6970, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant i0;
  kk_std_time_instant__timescale _x6974 = kk_std_time_instant__timescale_dup(ts1); /*std/time/instant/timescale*/
  i0 = kk_std_time_instant__new_Instant(kk_reuse_null, t_4881, _x6974, _ctx); /*std/time/instant/instant*/
  bool _match_5560;
  kk_string_t _x6975;
  kk_string_t _x3 = tz.name;
  kk_string_dup(_x3);
  _x6975 = _x3; /*string*/
  kk_string_t _x6976;
  kk_std_time_calendar__timezone _x6977 = kk_std_time_calendar_tz_utc; /*std/time/calendar/timezone*/
  kk_string_t _x03 = _x6977.name;
  _x6976 = kk_string_dup(_x03); /*string*/
  _match_5560 = kk_string_is_eq(_x6975,_x6976,kk_context()); /*bool*/
  if (_match_5560) {
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    return i0;
  }
  kk_std_core_types__maybe _match_5561;
  kk_function_t _x6979;
  kk_function_t _x120 = tz.utc_inverse;
  kk_function_dup(_x120);
  _x6979 = _x120; /*(std/time/instant/instant) -> maybe<std/time/instant/instant>*/
  kk_std_time_instant__instant _x6978 = kk_std_time_instant__instant_dup(i0); /*std/time/instant/instant*/
  _match_5561 = kk_function_call(kk_std_core_types__maybe, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6979, (_x6979, _x6978, _ctx)); /*maybe<std/time/instant/instant>*/
  if (kk_std_core_types__is_Just(_match_5561)) {
    kk_box_t _box_x5545 = _match_5561._cons.Just.value;
    kk_std_time_instant__instant inv = kk_std_time_instant__instant_unbox(_box_x5545, NULL);
    struct kk_std_time_instant_Instant* _con6981 = kk_std_time_instant__as_Instant(inv);
    kk_std_time_instant__instant_dropn(i0, ((int32_t)1), _ctx);
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    kk_std_time_calendar__timezone_drop(tz, _ctx);
    kk_std_time_instant__instant_dup(inv);
    kk_std_core_types__maybe_drop(_match_5561, _ctx);
    kk_Unit;
    return inv;
  }
  kk_std_core_types__tuple2_ _this_4698;
  kk_function_t _x6982;
  kk_function_t _x30 = tz.utc_delta;
  kk_function_dup(_x30);
  _x6982 = _x30; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  _this_4698 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6982, (_x6982, i0, _ctx)); /*(std/time/duration/duration, string)*/
  kk_std_time_timestamp__timestamp t1_4701;
  kk_std_num_ddouble__ddouble _x6983;
  kk_std_num_ddouble__ddouble _x6984;
  kk_std_num_ddouble__ddouble _x100 = t_4881.since;
  _x6984 = _x100; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x6985;
  double _x6986;
  double _x6987;
  kk_box_t _box_x5546 = _this_4698.fst;
  kk_box_t _box_x5547 = _this_4698.snd;
  kk_std_time_duration__duration _x20 = kk_std_time_duration__duration_unbox(_box_x5546, NULL);
  kk_std_num_ddouble__ddouble _x50 = _x20.secs;
  double _x40 = _x50.hi;
  _x6987 = _x40; /*double*/
  _x6986 = (-_x6987); /*double*/
  double _x6990;
  double _x6991;
  kk_box_t _box_x5548 = _this_4698.fst;
  kk_box_t _box_x5549 = _this_4698.snd;
  kk_std_time_duration__duration _x200 = kk_std_time_duration__duration_unbox(_box_x5548, NULL);
  kk_std_num_ddouble__ddouble _x500 = _x200.secs;
  double _x000 = _x500.lo;
  _x6991 = _x000; /*double*/
  _x6990 = (-_x6991); /*double*/
  _x6985 = kk_std_num_ddouble__new_Ddouble(_x6986, _x6990, _ctx); /*std/num/ddouble/ddouble*/
  _x6983 = kk_std_num_ddouble__lp__plus__rp_(_x6984, _x6985, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x6994;
  kk_std_num_ddouble__ddouble _pat0200 = t_4881.since;
  int32_t _x2000 = t_4881.leap32;
  _x6994 = _x2000; /*int32*/
  t1_4701 = kk_std_time_timestamp__new_Timestamp(_x6983, _x6994, _ctx); /*std/time/timestamp/timestamp*/
  kk_std_time_instant__instant tzi1;
  kk_std_time_instant__timescale _x6995 = kk_std_time_instant__timescale_dup(ts1); /*std/time/instant/timescale*/
  tzi1 = kk_std_time_instant__new_Instant(kk_reuse_null, t1_4701, _x6995, _ctx); /*std/time/instant/instant*/
  kk_std_core_types__tuple2_ _this0_4705;
  kk_function_t _x6997;
  kk_function_t _x70 = tz.utc_delta;
  kk_function_dup(_x70);
  kk_std_time_calendar__timezone_drop(tz, _ctx);
  kk_Unit;
  _x6997 = _x70; /*(std/time/instant/instant) -> (std/time/duration/duration, string)*/
  kk_std_time_instant__instant _x6996 = kk_std_time_instant__instant_dup(tzi1); /*std/time/instant/instant*/
  _this0_4705 = kk_function_call(kk_std_core_types__tuple2_, (kk_function_t, kk_std_time_instant__instant, kk_context_t*), _x6997, (_x6997, _x6996, _ctx)); /*(std/time/duration/duration, string)*/
  bool _match_5562;
  kk_std_time_duration__duration _x6998;
  kk_box_t _box_x5550 = _this_4698.fst;
  kk_box_t _box_x5551 = _this_4698.snd;
  kk_std_time_duration__duration _x201 = kk_std_time_duration__duration_unbox(_box_x5550, NULL);
  kk_std_core_types__tuple2__drop(_this_4698, _ctx);
  kk_Unit;
  _x6998 = _x201; /*std/time/duration/duration*/
  kk_std_time_duration__duration _x7001;
  kk_box_t _box_x5552 = _this0_4705.fst;
  kk_box_t _box_x5553 = _this0_4705.snd;
  kk_std_time_duration__duration _x60 = kk_std_time_duration__duration_unbox(_box_x5552, NULL);
  _x7001 = _x60; /*std/time/duration/duration*/
  _match_5562 = kk_std_time_duration__lp__eq__eq__rp_(_x6998, _x7001, _ctx); /*bool*/
  if (_match_5562) {
    kk_std_core_types__tuple2__drop(_this0_4705, _ctx);
    kk_std_time_instant__timescale_dropn(ts1, ((int32_t)10), _ctx);
    return tzi1;
  }
  kk_std_time_instant__instant_dropn(tzi1, ((int32_t)1), _ctx);
  kk_std_time_timestamp__timestamp t3_4708;
  kk_std_num_ddouble__ddouble _x7004;
  kk_std_num_ddouble__ddouble _x7005;
  kk_std_num_ddouble__ddouble _x110 = t_4881.since;
  _x7005 = _x110; /*std/time/timestamp/timespan*/
  kk_std_num_ddouble__ddouble _x7006;
  double _x7007;
  double _x7008;
  kk_box_t _box_x5554 = _this0_4705.fst;
  kk_box_t _box_x5555 = _this0_4705.snd;
  kk_std_time_duration__duration _x600 = kk_std_time_duration__duration_unbox(_box_x5554, NULL);
  kk_std_num_ddouble__ddouble _x90 = _x600.secs;
  double _x80 = _x90.hi;
  _x7008 = _x80; /*double*/
  _x7007 = (-_x7008); /*double*/
  double _x7011;
  double _x7012;
  kk_box_t _box_x5556 = _this0_4705.fst;
  kk_box_t _box_x5557 = _this0_4705.snd;
  kk_std_time_duration__duration _x601 = kk_std_time_duration__duration_unbox(_box_x5556, NULL);
  kk_std_core_types__tuple2__drop(_this0_4705, _ctx);
  kk_Unit;
  kk_std_num_ddouble__ddouble _x900 = _x601.secs;
  double _x010 = _x900.lo;
  _x7012 = _x010; /*double*/
  _x7011 = (-_x7012); /*double*/
  _x7006 = kk_std_num_ddouble__new_Ddouble(_x7007, _x7011, _ctx); /*std/num/ddouble/ddouble*/
  _x7004 = kk_std_num_ddouble__lp__plus__rp_(_x7005, _x7006, _ctx); /*std/num/ddouble/ddouble*/
  int32_t _x7015;
  kk_std_num_ddouble__ddouble _pat0210 = t_4881.since;
  int32_t _x21 = t_4881.leap32;
  _x7015 = _x21; /*int32*/
  t3_4708 = kk_std_time_timestamp__new_Timestamp(_x7004, _x7015, _ctx); /*std/time/timestamp/timestamp*/
  return kk_std_time_instant__new_Instant(kk_reuse_null, t3_4708, ts1, _ctx);
}

kk_std_time_calendar__calendar kk_std_time_calendar_year_shift_earth_calendar(kk_string_t name0, kk_string_t long_name0, kk_integer_t year_shift, kk_std_time_calendar__calendar cal, kk_std_core_types__optional month_prefix0, kk_std_core_types__optional show_era0, kk_context_t* _ctx) { /* (name : string, long-name : string, year-shift : int, cal : calendar, month-prefix : optional<string>, show-era : optional<(std/time/date/date) -> string>) -> calendar */ 
  kk_function_t days_to_date0_4807;
  kk_std_time_calendar__calendar_dup(cal);
  kk_integer_dup(year_shift);
  days_to_date0_4807 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6916(cal, year_shift, _ctx); /*(days1 : int) -> std/time/date/date*/
  kk_function_t date_to_days0_4808 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6929(cal, year_shift, _ctx); /*(d1 : std/time/date/date) -> int*/;
  kk_string_t _x6943;
  if (kk_std_core_types__is_Optional(month_prefix0)) {
    kk_box_t _box_x5531 = month_prefix0._cons.Optional.value;
    kk_string_t _month_prefix_4415 = kk_string_unbox(_box_x5531);
    kk_string_dup(_month_prefix_4415);
    kk_std_core_types__optional_drop(month_prefix0, _ctx);
    kk_Unit;
    _x6943 = _month_prefix_4415; /*string*/
    goto _match6944;
  }
  _x6943 = kk_string_empty(); /*string*/
  _match6944: ;
  kk_function_t _x6947;
  if (kk_std_core_types__is_Optional(show_era0)) {
    kk_box_t _fun_unbox_x5535 = show_era0._cons.Optional.value;
    _x6947 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6948(_fun_unbox_x5535, _ctx); /*(std/time/date/date) -> string*/
  }
  else {
    _x6947 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6951(_ctx); /*(std/time/date/date) -> string*/
  }
  kk_function_t _x6953;
  kk_function_dup(days_to_date0_4807);
  _x6953 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6954(days_to_date0_4807, _ctx); /*(i : std/time/instant/instant, tzdelta : std/time/duration/duration) -> (6, 7)*/
  kk_function_t _x6968;
  kk_function_dup(date_to_days0_4808);
  _x6968 = kk_std_time_calendar_new_year_shift_earth_calendar_fun6969(date_to_days0_4808, _ctx); /*(d00 : std/time/date/date, c : std/time/date/clock, tz : std/time/calendar/timezone, ts1 : std/time/instant/timescale) -> std/time/instant/instant*/
  return kk_std_time_calendar__new_Calendar(kk_reuse_null, name0, long_name0, _x6943, _x6947, _x6953, _x6968, days_to_date0_4807, date_to_days0_4808, _ctx);
}

// initialization
void kk_std_time_calendar__init(kk_context_t* _ctx){
  static bool _kk_initialized = false;
  if (_kk_initialized) return;
  _kk_initialized = true;
  kk_std_core_types__init(_ctx);
  kk_std_core_hnd__init(_ctx);
  kk_std_core__init(_ctx);
  kk_std_num_ddouble__init(_ctx);
  kk_std_time_date__init(_ctx);
  kk_std_time_timestamp__init(_ctx);
  kk_std_time_duration__init(_ctx);
  kk_std_time_instant__init(_ctx);
  kk_std_time_utc__init(_ctx);
  #if defined(KK_CUSTOM_INIT)
    KK_CUSTOM_INIT (_ctx);
  #endif
  {
    kk_string_t _x5969;
    kk_define_string_literal(, _s5970, 3, "UTC")
    _x5969 = kk_string_dup(_s5970); /*string*/
    kk_std_time_calendar_tz_utc = kk_std_time_calendar__new_Timezone(_x5969, kk_std_time_calendar_new_tz_utc_fun5971(_ctx), kk_std_time_calendar_new_tz_utc_fun5975(_ctx), _ctx); /*std/time/calendar/timezone*/
  }
  {
    kk_string_t _x6408 = kk_string_empty(); /*string*/
    kk_string_t _x6410;
    kk_define_string_literal(, _s6411, 3, "ISO")
    _x6410 = kk_string_dup(_s6411); /*string*/
    kk_std_core_types__optional _x6438;
    kk_integer_t _b_5269_5247;
    kk_std_core_types__optional _match_5653 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5653)) {
      kk_box_t _box_x5246 = _match_5653._cons.Optional.value;
      kk_integer_t _epoch_shift_2726 = kk_integer_unbox(_box_x5246);
      kk_integer_dup(_epoch_shift_2726);
      kk_std_core_types__optional_drop(_match_5653, _ctx);
      kk_Unit;
      _b_5269_5247 = _epoch_shift_2726; /*int*/
      goto _match6439;
    }
    _b_5269_5247 = kk_integer_from_int(730119, _ctx); /*int*/
    _match6439: ;
    _x6438 = kk_std_core_types__new_Optional(kk_integer_box(_b_5269_5247), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x6441;
    bool _b_5270_5249;
    kk_std_core_types__optional _match_5652 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5652)) {
      kk_box_t _box_x5248 = _match_5652._cons.Optional.value;
      bool _has_year_zero_2730 = kk_bool_unbox(_box_x5248);
      kk_std_core_types__optional_drop(_match_5652, _ctx);
      kk_Unit;
      _b_5270_5249 = _has_year_zero_2730; /*bool*/
      goto _match6442;
    }
    _b_5270_5249 = true; /*bool*/
    _match6442: ;
    _x6441 = kk_std_core_types__new_Optional(kk_bool_box(_b_5270_5249), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x6444;
    kk_string_t _b_5271_5251;
    kk_std_core_types__optional _match_5651 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5651)) {
      kk_box_t _box_x5250 = _match_5651._cons.Optional.value;
      kk_string_t _month_prefix_2734 = kk_string_unbox(_box_x5250);
      kk_string_dup(_month_prefix_2734);
      kk_std_core_types__optional_drop(_match_5651, _ctx);
      kk_Unit;
      _b_5271_5251 = _month_prefix_2734; /*string*/
      goto _match6445;
    }
    _b_5271_5251 = kk_string_empty(); /*string*/
    _match6445: ;
    _x6444 = kk_std_core_types__new_Optional(kk_string_box(_b_5271_5251), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x6448;
    kk_function_t _b_5272_5259;
    kk_std_core_types__optional _x6449 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/
    if (kk_std_core_types__is_Optional(_x6449)) {
      kk_box_t _fun_unbox_x5255 = _x6449._cons.Optional.value;
      _b_5272_5259 = kk_std_time_calendar_new_cal_iso_fun6450(_fun_unbox_x5255, _ctx); /*(std/time/date/date) -> string*/
    }
    else {
      _b_5272_5259 = kk_std_time_calendar_new_cal_iso_fun6453(_ctx); /*(std/time/date/date) -> string*/
    }
    _x6448 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_iso_fun6455(_b_5272_5259, _ctx)), _ctx); /*optional<106>*/
    kk_std_time_calendar_cal_iso = kk_std_time_calendar_solar_calendar(_x6408, _x6410, kk_std_time_calendar_new_cal_iso_fun6413(_ctx), kk_std_time_calendar_new_cal_iso_fun6414(_ctx), kk_std_time_calendar_new_cal_iso_fun6434(_ctx), kk_std_time_calendar_new_cal_iso_fun6437(_ctx), _x6438, _x6441, _x6444, _x6448, _ctx); /*std/time/calendar/calendar*/
  }
  {
    kk_string_t _x6597;
    kk_define_string_literal(, _s6598, 2, "GC")
    _x6597 = kk_string_dup(_s6598); /*string*/
    kk_string_t _x6599;
    kk_define_string_literal(, _s6600, 9, "Gregorian")
    _x6599 = kk_string_dup(_s6600); /*string*/
    kk_std_core_types__optional _x6627;
    kk_integer_t _b_5350_5336;
    kk_std_core_types__optional _match_5630 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5630)) {
      kk_box_t _box_x5335 = _match_5630._cons.Optional.value;
      kk_integer_t _epoch_shift_2726 = kk_integer_unbox(_box_x5335);
      kk_integer_dup(_epoch_shift_2726);
      kk_std_core_types__optional_drop(_match_5630, _ctx);
      kk_Unit;
      _b_5350_5336 = _epoch_shift_2726; /*int*/
      goto _match6628;
    }
    _b_5350_5336 = kk_integer_from_int(730119, _ctx); /*int*/
    _match6628: ;
    _x6627 = kk_std_core_types__new_Optional(kk_integer_box(_b_5350_5336), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x6630;
    bool _b_5351_5337 = false; /*bool*/;
    _x6630 = kk_std_core_types__new_Optional(kk_bool_box(_b_5351_5337), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x6631;
    kk_string_t _b_5352_5339;
    kk_std_core_types__optional _match_5629 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5629)) {
      kk_box_t _box_x5338 = _match_5629._cons.Optional.value;
      kk_string_t _month_prefix_2734 = kk_string_unbox(_box_x5338);
      kk_string_dup(_month_prefix_2734);
      kk_std_core_types__optional_drop(_match_5629, _ctx);
      kk_Unit;
      _b_5352_5339 = _month_prefix_2734; /*string*/
      goto _match6632;
    }
    _b_5352_5339 = kk_string_empty(); /*string*/
    _match6632: ;
    _x6631 = kk_std_core_types__new_Optional(kk_string_box(_b_5352_5339), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x6635;
    kk_function_t _b_5353_5340 = kk_std_time_calendar_new_cal_gregorian_fun6636(_ctx); /*(d0 : std/time/date/date) -> string*/;
    _x6635 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_gregorian_fun6640(_b_5353_5340, _ctx)), _ctx); /*optional<106>*/
    kk_std_time_calendar_cal_gregorian = kk_std_time_calendar_solar_calendar(_x6597, _x6599, kk_std_time_calendar_new_cal_gregorian_fun6602(_ctx), kk_std_time_calendar_new_cal_gregorian_fun6603(_ctx), kk_std_time_calendar_new_cal_gregorian_fun6623(_ctx), kk_std_time_calendar_new_cal_gregorian_fun6626(_ctx), _x6627, _x6630, _x6631, _x6635, _ctx); /*std/time/calendar/calendar*/
  }
  {
    kk_string_t _x6643;
    kk_define_string_literal(, _s6644, 3, "SLS")
    _x6643 = kk_string_dup(_s6644); /*string*/
    kk_string_t _x6645;
    kk_define_string_literal(, _s6646, 7, "ISO-SLS")
    _x6645 = kk_string_dup(_s6646); /*string*/
    kk_std_core_types__optional _x6673;
    kk_integer_t _b_5386_5364;
    kk_std_core_types__optional _match_5615 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5615)) {
      kk_box_t _box_x5363 = _match_5615._cons.Optional.value;
      kk_integer_t _epoch_shift_2726 = kk_integer_unbox(_box_x5363);
      kk_integer_dup(_epoch_shift_2726);
      kk_std_core_types__optional_drop(_match_5615, _ctx);
      kk_Unit;
      _b_5386_5364 = _epoch_shift_2726; /*int*/
      goto _match6674;
    }
    _b_5386_5364 = kk_integer_from_int(730119, _ctx); /*int*/
    _match6674: ;
    _x6673 = kk_std_core_types__new_Optional(kk_integer_box(_b_5386_5364), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x6676;
    bool _b_5387_5366;
    kk_std_core_types__optional _match_5614 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5614)) {
      kk_box_t _box_x5365 = _match_5614._cons.Optional.value;
      bool _has_year_zero_2730 = kk_bool_unbox(_box_x5365);
      kk_std_core_types__optional_drop(_match_5614, _ctx);
      kk_Unit;
      _b_5387_5366 = _has_year_zero_2730; /*bool*/
      goto _match6677;
    }
    _b_5387_5366 = true; /*bool*/
    _match6677: ;
    _x6676 = kk_std_core_types__new_Optional(kk_bool_box(_b_5387_5366), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x6679;
    kk_string_t _b_5388_5368;
    kk_std_core_types__optional _match_5613 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/;
    if (kk_std_core_types__is_Optional(_match_5613)) {
      kk_box_t _box_x5367 = _match_5613._cons.Optional.value;
      kk_string_t _month_prefix_2734 = kk_string_unbox(_box_x5367);
      kk_string_dup(_month_prefix_2734);
      kk_std_core_types__optional_drop(_match_5613, _ctx);
      kk_Unit;
      _b_5388_5368 = _month_prefix_2734; /*string*/
      goto _match6680;
    }
    _b_5388_5368 = kk_string_empty(); /*string*/
    _match6680: ;
    _x6679 = kk_std_core_types__new_Optional(kk_string_box(_b_5388_5368), _ctx); /*optional<106>*/
    kk_std_core_types__optional _x6683;
    kk_function_t _b_5389_5376;
    kk_std_core_types__optional _x6684 = kk_std_core_types__new_None(_ctx); /*forall<a> optional<a>*/
    if (kk_std_core_types__is_Optional(_x6684)) {
      kk_box_t _fun_unbox_x5372 = _x6684._cons.Optional.value;
      _b_5389_5376 = kk_std_time_calendar_new_cal_iso_sls_fun6685(_fun_unbox_x5372, _ctx); /*(std/time/date/date) -> string*/
    }
    else {
      _b_5389_5376 = kk_std_time_calendar_new_cal_iso_sls_fun6688(_ctx); /*(std/time/date/date) -> string*/
    }
    _x6683 = kk_std_core_types__new_Optional(kk_function_box(kk_std_time_calendar_new_cal_iso_sls_fun6690(_b_5389_5376, _ctx)), _ctx); /*optional<106>*/
    kk_std_time_calendar_cal_iso_sls = kk_std_time_calendar_solar_calendar(_x6643, _x6645, kk_std_time_calendar_new_cal_iso_sls_fun6648(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun6649(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun6669(_ctx), kk_std_time_calendar_new_cal_iso_sls_fun6672(_ctx), _x6673, _x6676, _x6679, _x6683, _ctx); /*std/time/calendar/calendar*/
  }
}

// termination
void kk_std_time_calendar__done(kk_context_t* _ctx){
  static bool _kk_done = false;
  if (_kk_done) return;
  _kk_done = true;
  #if defined(KK_CUSTOM_DONE)
    KK_CUSTOM_DONE (_ctx);
  #endif
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_iso_sls, _ctx);
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_gregorian, _ctx);
  kk_std_time_calendar__calendar_drop(kk_std_time_calendar_cal_iso, _ctx);
  kk_std_time_calendar__timezone_drop(kk_std_time_calendar_tz_utc, _ctx);
  kk_std_time_utc__done(_ctx);
  kk_std_time_instant__done(_ctx);
  kk_std_time_duration__done(_ctx);
  kk_std_time_timestamp__done(_ctx);
  kk_std_time_date__done(_ctx);
  kk_std_num_ddouble__done(_ctx);
  kk_std_core__done(_ctx);
  kk_std_core_hnd__done(_ctx);
  kk_std_core_types__done(_ctx);
}
